@@ -14,10 +14,10 @@ local battle_calcs = {}
 function battle_calcs.unit_attack_info(unit, cache)
     -- Return a table containing information about attack-related properties of @unit
     -- The result can be cached if variable @cache is given
-    -- This is done in order to avoid duplication of slow processes, such as access to unit.__cfg
+    -- This is done in order to avoid duplication of slow processes
 
     -- Return table has fields:
-    --  - attacks: the attack tables from unit.__cfg
+    --  - attacks: the attack tables
     --  - resist_mod: resistance modifiers (multiplicative factors) index by attack type
     --  - alignment: just that
 
@@ -32,51 +32,47 @@ function battle_calcs.unit_attack_info(unit, cache)
     end
 
     -- Otherwise collect the information
-    local unit_cfg = unit.__cfg
     local unit_info = {
         attacks = {},
         resist_mod = {},
-        alignment = unit_cfg.alignment
+        alignment = unit.alignment
     }
-    for attack in wml.child_range(unit_cfg, 'attack') do
+    local attacks = unit.attacks
+    for i_a = 1,#attacks do
+        local attack = attacks[i_a]
         -- Extract information for specials; we do this first because some
         -- custom special might have the same name as one of the default scalar fields
         local a = {}
-        for special in wml.child_range(attack, 'specials') do
-            for _,sp in ipairs(special) do
-                if (sp[1] == 'damage') then  -- this is 'backstab'
-                    if (sp[2].id == 'backstab') then
-                        a.backstab = true
-                    else
-                        if (sp[2].id == 'charge') then a.charge = true end
-                    end
+        for _,sp in ipairs(attack.specials) do
+            if (sp[1] == 'damage') then  -- this is 'backstab'
+                if (sp[2].id == 'backstab') then
+                    a.backstab = true
                 else
-                    -- magical, marksman
-                    if (sp[1] == 'chance_to_hit') then
-                        a[sp[2].id] = true
-                    else
-                        a[sp[1]] = true
-                    end
+                    if (sp[2].id == 'charge') then a.charge = true end
+                end
+            else
+                -- magical, marksman
+                if (sp[1] == 'chance_to_hit') then
+                    a[sp[2].id or 'no_id'] = true
+                else
+                    a[sp[1]] = true
                 end
             end
         end
 
         -- Now extract the scalar (string and number) values from attack
-        for k,v in pairs(attack) do
-            if (type(v) == 'number') or (type(v) == 'string') then
-                a[k] = v
-            end
-        end
-
-        -- [attack]number= defaults to zero; must be defined for battle_calcs.best_weapons()
-        a.number = a.number or 0
+        a.damage = attack.damage
+        a.type = attack.type
+        a.range = attack.range
+        -- number must be defined for battle_calcs.best_weapons()
+        a.number = attack.number or 0
 
         table.insert(unit_info.attacks, a)
     end
 
     local attack_types = { "arcane", "blade", "cold", "fire", "impact", "pierce" }
     for _,attack_type in ipairs(attack_types) do
-        unit_info.resist_mod[attack_type] = wesnoth.unit_resistance(unit, attack_type) / 100.
+        unit_info.resist_mod[attack_type] = 1 - unit:resistance_against(attack_type) / 100.
     end
 
     if cache then cache[cind] = unit_info end
@@ -99,8 +95,8 @@ function battle_calcs.strike_damage(attacker, defender, att_weapon, def_weapon,
     -- Set up a cache index. We use id+max_hitpoints+side for each unit, since the
     -- unit can level up.
     -- Also need to add the weapons and lawful_bonus values for each unit
-    local att_lawful_bonus = wesnoth.get_time_of_day({ dst[1], dst[2], true }).lawful_bonus
-    local def_lawful_bonus = wesnoth.get_time_of_day({ defender.x, defender.y, true }).lawful_bonus
+    local att_lawful_bonus = wesnoth.schedule.get_illumination(dst).lawful_bonus
+    local def_lawful_bonus = wesnoth.schedule.get_illumination(defender).lawful_bonus
 
     local cind = 'SD-' .. attacker.id .. attacker.max_hitpoints .. attacker.side
     cind = cind .. 'x' .. defender.id .. defender.max_hitpoints .. defender.side
@@ -121,6 +117,7 @@ function battle_calcs.strike_damage(attacker, defender, att_weapon, def_weapon,
     -- Opponent resistance modifier
     local att_multiplier = defender_info.resist_mod[attacker_info.attacks[att_weapon].type] or 1
 
+
     -- TOD modifier
     att_multiplier = att_multiplier * AH.get_unit_time_of_day_bonus(attacker_info.alignment, att_lawful_bonus)
 
@@ -146,16 +143,16 @@ function battle_calcs.strike_damage(attacker, defender, att_weapon, def_weapon,
     -- Rounding of .5 values is done differently depending on whether the
     -- multiplier is greater or smaller than 1
     if (att_multiplier > 1) then
-        att_damage = H.round(att_damage * att_multiplier - 0.001)
+        att_damage = mathx.round(att_damage * att_multiplier - 0.001)
     else
-        att_damage = H.round(att_damage * att_multiplier + 0.001)
+        att_damage = mathx.round(att_damage * att_multiplier + 0.001)
     end
 
     if (def_weapon ~= 0) then
         if (def_multiplier > 1) then
-            def_damage = H.round(def_damage * def_multiplier - 0.001)
+            def_damage = mathx.round(def_damage * def_multiplier - 0.001)
         else
-            def_damage = H.round(def_damage * def_multiplier + 0.001)
+            def_damage = mathx.round(def_damage * def_multiplier + 0.001)
         end
     end
 
@@ -186,8 +183,8 @@ function battle_calcs.best_weapons(attacker, defender, dst, cache)
     -- Set up a cache index. We use id+max_hitpoints+side for each unit, since the
     -- unit can level up.
     -- Also need to add the weapons and lawful_bonus values for each unit
-    local att_lawful_bonus = wesnoth.get_time_of_day({ dst[1], dst[2], true }).lawful_bonus
-    local def_lawful_bonus = wesnoth.get_time_of_day({ defender.x, defender.y, true }).lawful_bonus
+    local att_lawful_bonus = wesnoth.schedule.get_illumination(dst).lawful_bonus
+    local def_lawful_bonus = wesnoth.schedule.get_illumination(defender).lawful_bonus
 
     local cind = 'BW-' .. attacker.id .. attacker.max_hitpoints .. attacker.side
     cind = cind .. 'x' .. defender.id .. defender.max_hitpoints .. defender.side
@@ -202,10 +199,10 @@ function battle_calcs.best_weapons(attacker, defender, dst, cache)
     local defender_info = battle_calcs.unit_attack_info(defender, cache)
 
     -- Best attacker weapon
-    local max_rating, best_att_weapon, best_def_weapon = -9e99, 0, 0
+    local max_rating, best_att_weapon, best_def_weapon = - math.huge, 0, 0
     for att_weapon_number,att_weapon in ipairs(attacker_info.attacks) do
         local att_damage = battle_calcs.strike_damage(attacker, defender, att_weapon_number, 0, { dst[1], dst[2] }, cache)
-        local max_def_rating, tmp_best_def_weapon = -9e99, 0
+        local max_def_rating, tmp_best_def_weapon = - math.huge, 0
         for def_weapon_number,def_weapon in ipairs(defender_info.attacks) do
             if (def_weapon.range == att_weapon.range) then
                 local def_damage = battle_calcs.strike_damage(defender, attacker, def_weapon_number, 0, { defender.x, defender.y }, cache)
@@ -217,7 +214,7 @@ function battle_calcs.best_weapons(attacker, defender, dst, cache)
         end
 
         local rating = att_damage * att_weapon.number
-        if (max_def_rating > -9e99) then rating = rating - max_def_rating / 2. end
+        if (max_def_rating > - math.huge) then rating = rating - max_def_rating / 2. end
 
         if (rating > max_rating) then
             max_rating, best_att_weapon, best_def_weapon = rating, att_weapon_number, tmp_best_def_weapon
@@ -532,10 +529,10 @@ function battle_calcs.print_coefficients()
                 dummy, coeffs = battle_calcs.battle_outcome_coefficients(cfg)
             end
 
-            print()
-            print('Attacker: ' .. cfg.att.strikes .. ' strikes, can survive ' .. cfg.att.max_hits .. ' hits')
-            print('Defender: ' .. cfg.def.strikes .. ' strikes, can survive ' .. cfg.def.max_hits .. ' hits')
-            print('Chance of hits on defender: ')
+            std_print()
+            std_print('Attacker: ' .. cfg.att.strikes .. ' strikes, can survive ' .. cfg.att.max_hits .. ' hits')
+            std_print('Defender: ' .. cfg.def.strikes .. ' strikes, can survive ' .. cfg.def.max_hits .. ' hits')
+            std_print('Chance of hits on defender: ')
 
             -- The first indices of coeffs are the possible number of hits the attacker can land on the defender
             for hits = 0,#coeffs do
@@ -570,8 +567,8 @@ function battle_calcs.print_coefficients()
                 local skip_str = ''
                 if combs.skip then skip_str = ' (skip)' end
 
-                print(hits .. skip_str .. ':  ' .. str)
-                print('      = ' .. hit_prob)
+                std_print(hits .. skip_str .. ':  ' .. str)
+                std_print('      = ' .. hit_prob)
             end
         end
     end
@@ -680,8 +677,8 @@ function battle_calcs.battle_outcome(attacker, defender, cfg, cache)
     if (def_max_hits > att_strikes) then def_max_hits = att_strikes end
 
     -- Probability of landing a hit
-    local att_hit_prob = wesnoth.unit_defense(defender, wesnoth.get_terrain(defender.x, defender.y)) / 100.
-    local def_hit_prob = wesnoth.unit_defense(attacker, wesnoth.get_terrain(dst[1], dst[2])) / 100.
+    local att_hit_prob = 1 - defender:defense_on(wesnoth.current.map[defender]) / 100.
+    local def_hit_prob = 1 - attacker:defense_on(wesnoth.current.map[dst]) / 100.
 
     -- Magical: attack and defense, and under all circumstances
     if att_attack.magical then att_hit_prob = 0.7 end
@@ -731,7 +728,7 @@ function battle_calcs.simulate_combat_loc(attacker, dst, defender, weapon)
     -- when on terrain of same type as that at @dst, which is of form { x, y }
     -- If @weapon is set, use that weapon (Lua index starting at 1), otherwise use best weapon
 
-    local attacker_dst = wesnoth.copy_unit(attacker)
+    local attacker_dst = attacker:clone()
     attacker_dst.x, attacker_dst.y = dst[1], dst[2]
 
     if weapon then
@@ -794,7 +791,7 @@ function battle_calcs.attack_rating(attacker, defender, dst, cfg, cache)
 
     -- We also need the leader (well, the location at least)
     -- because if there's no other difference, prefer location _between_ the leader and the defender
-    local leader = wesnoth.get_units { side = attacker.side, canrecruit = 'yes' }[1]
+    local leader = wesnoth.units.find_on_map { side = attacker.side, canrecruit = 'yes' }[1]
 
     ------ All the attacker contributions: ------
     -- Add up rating for the attacking unit
@@ -803,26 +800,25 @@ function battle_calcs.attack_rating(attacker, defender, dst, cfg, cache)
 
     -- Average damage to unit is negative rating
     local damage = attacker.hitpoints - att_stats.average_hp
-    -- Count poisoned as additional 8 HP damage times probability of being poisoned
+    -- Count poisoned as additional damage done by poison times probability of being poisoned
     if (att_stats.poisoned ~= 0) then
-        damage = damage + 8 * (att_stats.poisoned - att_stats.hp_chance[0])
+        damage = damage + wesnoth.game_config.poison_amount * (att_stats.poisoned - att_stats.hp_chance[0])
     end
     -- Count slowed as additional 6 HP damage times probability of being slowed
     if (att_stats.slowed ~= 0) then
         damage = damage + 6 * (att_stats.slowed - att_stats.hp_chance[0])
     end
 
-    -- If attack is from a village, we count that as a 10 HP bonus
-    local is_village = wesnoth.get_terrain_info(wesnoth.get_terrain(dst[1], dst[2])).village
-    if is_village then
-        damage = damage - 10.
-    end
+    local map = wesnoth.current.map
 
-    -- If attack is adjacent to an unoccupied village, that's bad
+    -- If attack is from a healing location, count that as slightly more than the healing amount
+    damage = damage - 1.25 * wesnoth.terrain_types[map[dst]].healing
+
+    -- Equivalently, if attack is adjacent to an unoccupied healing location, that's bad
     for xa,ya in H.adjacent_tiles(dst[1], dst[2]) do
-        local is_adjacent_village = wesnoth.get_terrain_info(wesnoth.get_terrain(xa, ya)).village
-        if is_adjacent_village and (not wesnoth.get_unit(xa, ya)) then
-            damage = damage + 10
+        local healing = wesnoth.terrain_types[map[{xa, ya}]].healing
+        if (healing > 0) and (not wesnoth.units.get(xa, ya)) then
+            damage = damage + 1.25 * healing
         end
     end
 
@@ -837,11 +833,12 @@ function battle_calcs.attack_rating(attacker, defender, dst, cfg, cache)
     -- In addition, potentially leveling up in this attack is a huge bonus,
     -- proportional to the chance of it happening and the chance of not dying itself
     local level_bonus = 0.
-    local defender_level = wesnoth.unit_types[defender.type].level
-    if (attacker.max_experience - attacker.experience <= defender_level) then
+    local defender_level = defender.level
+    if (attacker.max_experience - attacker.experience <= defender_level * wesnoth.game_config.combat_experience) then
         level_bonus = 1. - att_stats.hp_chance[0]
     else
-        if (attacker.max_experience - attacker.experience <= defender_level * 8) then
+        if (defender_level == 0) then defender_level = 0.5 end
+        if (attacker.max_experience - attacker.experience <= defender_level * wesnoth.game_config.kill_experience) then
             level_bonus = (1. - att_stats.hp_chance[0]) * def_stats.hp_chance[0]
         end
     end
@@ -849,7 +846,7 @@ function battle_calcs.attack_rating(attacker, defender, dst, cfg, cache)
 
 
     -- Now convert this into gold-equivalent value
-    local attacker_value = wesnoth.unit_types[attacker.type].cost
+    local attacker_value = attacker.cost
 
     -- Being closer to leveling is good (this makes AI prefer units with lots of XP)
     local xp_bonus = attacker.experience / attacker.max_experience
@@ -860,20 +857,17 @@ function battle_calcs.attack_rating(attacker, defender, dst, cfg, cache)
     ------ Now (most of) the same for the defender ------
     -- Average damage to defender is positive rating
     local damage = defender.hitpoints - def_stats.average_hp
-    -- Count poisoned as additional 8 HP damage times probability of being poisoned
+    -- Count poisoned as additional damage done by poison times probability of being poisoned
     if (def_stats.poisoned ~= 0) then
-        damage = damage + 8 * (def_stats.poisoned - def_stats.hp_chance[0])
+        damage = damage + wesnoth.game_config.poison_amount * (def_stats.poisoned - def_stats.hp_chance[0])
     end
     -- Count slowed as additional 6 HP damage times probability of being slowed
     if (def_stats.slowed ~= 0) then
         damage = damage + 6 * (def_stats.slowed - def_stats.hp_chance[0])
     end
 
-    -- If defender is on a village, we count that as a 10 HP bonus
-    local is_village = wesnoth.get_terrain_info(wesnoth.get_terrain(defender.x, defender.y)).village
-    if is_village then
-        damage = damage - 10.
-    end
+    -- If defender is on a healing location, count that as slightly more than the healing amount
+    damage = damage - 1.25 * wesnoth.terrain_types[map[defender]].healing
 
     if (damage < 0) then damage = 0. end
 
@@ -886,18 +880,19 @@ function battle_calcs.attack_rating(attacker, defender, dst, cfg, cache)
     -- In addition, the defender potentially leveling up in this attack is a huge penalty,
     -- proportional to the chance of it happening and the chance of not dying itself
     local defender_level_penalty = 0.
-    local attacker_level = wesnoth.unit_types[attacker.type].level
-    if (defender.max_experience - defender.experience <= attacker_level) then
+    local attacker_level = attacker.level
+    if (defender.max_experience - defender.experience <= attacker_level * wesnoth.game_config.combat_experience) then
         defender_level_penalty = 1. - def_stats.hp_chance[0]
     else
-        if (defender.max_experience - defender.experience <= attacker_level * 8) then
+        if (attacker_level == 0) then attacker_level = 0.5 end
+        if (defender.max_experience - defender.experience <= attacker_level * wesnoth.game_config.kill_experience) then
             defender_level_penalty = (1. - def_stats.hp_chance[0]) * att_stats.hp_chance[0]
         end
     end
     value_fraction = value_fraction - defender_level_penalty * defender_level_weight
 
     -- Now convert this into gold-equivalent value
-    local defender_value = wesnoth.unit_types[defender.type].cost
+    local defender_value = defender.cost
 
     -- If this is the enemy leader, make damage to it much more important
     if defender.canrecruit then
@@ -914,9 +909,8 @@ function battle_calcs.attack_rating(attacker, defender, dst, cfg, cache)
 
     -- If defender is on a village, add a bonus rating (we want to get rid of those preferentially)
     -- So yes, this is positive, even though it's a plus for the defender
-    -- Defenders on villages also got a negative damage rating above (these don't exactly cancel each other though)
-    local is_village = wesnoth.get_terrain_info(wesnoth.get_terrain(defender.x, defender.y)).village
-    if is_village then
+    -- Note: defenders on healing locations also got a negative damage rating above (these don't exactly cancel each other though)
+    if wesnoth.terrain_types[map[defender]].village then
         defender_value = defender_value * (1. + 10. / attacker.max_hitpoints)
     end
 
@@ -932,7 +926,7 @@ function battle_calcs.attack_rating(attacker, defender, dst, cfg, cache)
     -- We don't need a bonus for good terrain for the attacker, as that is covered in the damage calculation
     -- However, we add a small bonus for good terrain defense of the _defender_ on the _attack_ hex
     -- This is in order to take good terrain away from defender on next move, all else being equal
-    local defender_defense = - wesnoth.unit_defense(defender, wesnoth.get_terrain(dst[1], dst[2])) / 100.
+    local defender_defense = - (100 - defender:defense_on(map[dst])) / 100.
     defender_value = defender_value + defender_defense * defense_weight
 
     -- Get a very small bonus for hexes in between defender and AI leader
@@ -947,7 +941,7 @@ function battle_calcs.attack_rating(attacker, defender, dst, cfg, cache)
     -- Add a very small penalty for attack hexes occupied by other units
     -- Note: it must be checked previously that the unit on the hex can move away
     if (dst[1] ~= attacker.x) or (dst[2] ~= attacker.y) then
-        if wesnoth.get_unit(dst[1], dst[2]) then
+        if wesnoth.units.get(dst[1], dst[2]) then
             defender_value = defender_value + occupied_hex_penalty
         end
     end
@@ -1011,15 +1005,13 @@ function battle_calcs.attack_combo_stats(tmp_attackers, tmp_dsts, defender, cach
 
             --for hp,p in pairs(tmp_def_stats[i].hp_chance) do
             --    if (p > 0) then
-            --        local dhp_norm = (hp - av) / defender.max_hitpoints * wesnoth.unit_types[defender.type].cost
+            --        local dhp_norm = (hp - av) / defender.max_hitpoints * defender.cost
             --        local dvar = p * dhp_norm^2
-                    --print(hp,p,av, dvar)
             --        outcome_variance = outcome_variance + dvar
             --        n_outcomes = n_outcomes + 1
             --    end
             --end
             --outcome_variance = outcome_variance / n_outcomes
-            --print('outcome_variance', outcome_variance)
 
             -- Note that this is a variance, not a standard deviations (as in, it's squared),
             -- so it does not matter much for low-variance attacks, but takes on large values for
@@ -1029,8 +1021,8 @@ function battle_calcs.attack_combo_stats(tmp_attackers, tmp_dsts, defender, cach
             -- If attacker has attack with 'slow' special, it should always go first
             -- Almost, bonus should not be quite as high as a really high CTK
             -- This isn't quite true in reality, but can be refined later
-            if AH.has_weapon_special(attacker, "slow") then
-                rating = rating + wesnoth.unit_types[defender.type].cost / 2.
+            if attacker:find_attack { special_id = "slow" } then
+                rating = rating + defender.cost / 2.
             end
 
             ratings[i] = { i, rating, base_rating, def_rating, att_rating }
@@ -1169,21 +1161,21 @@ function battle_calcs.get_attack_map_unit(unit, cfg)
     -- MP left off the map (for enemy pathfinding)
     local units_MP = {}
     if (unit.side ~= wesnoth.current.side) then
-        local all_units = wesnoth.get_units { side = wesnoth.current.side }
+        local all_units = wesnoth.units.find_on_map { side = wesnoth.current.side }
         for _,unit in ipairs(all_units) do
             if (unit.moves > 0) then
                 table.insert(units_MP, unit)
-                wesnoth.extract_unit(unit)
+                unit:extract()
             end
         end
     end
 
     -- Find hexes the unit can reach
-    local initial_reach = wesnoth.find_reach(unit, cfg)
+    local initial_reach = wesnoth.paths.find_reach(unit, cfg)
 
     -- Put the units back out there
     if (unit.side ~= wesnoth.current.side) then
-        for _,uMP in ipairs(units_MP) do wesnoth.put_unit(uMP) end
+        for _,uMP in ipairs(units_MP) do uMP:to_map() end
     end
 
     for _,loc in ipairs(initial_reach) do
@@ -1239,10 +1231,10 @@ function battle_calcs.relative_damage_map(units, enemies, cache)
     -- Get the attack maps for each unit in 'units' and 'enemies'
     local my_attack_maps, enemy_attack_maps = {}, {}
     for i,unit in ipairs(units) do
-        my_attack_maps[i] = battle_calcs.get_attack_map_unit(unit, cfg)
+        my_attack_maps[i] = battle_calcs.get_attack_map_unit(unit)
     end
     for i,e in ipairs(enemies) do
-        enemy_attack_maps[i] = battle_calcs.get_attack_map_unit(e, cfg)
+        enemy_attack_maps[i] = battle_calcs.get_attack_map_unit(e)
     end
 
     -- Get the damage rating for each unit in 'units'. It is the maximum
@@ -1250,7 +1242,7 @@ function battle_calcs.relative_damage_map(units, enemies, cache)
     -- against any of the enemy units
     local unit_ratings = {}
     for i,unit in ipairs(units) do
-        local max_rating, best_enemy = -9e99, {}
+        local max_rating, best_enemy = - math.huge, {}
         for _,enemy in ipairs(enemies) do
             local rating, defender_rating, attacker_rating =
                 battle_calcs.attack_rating(unit, enemy, { unit.x, unit.y }, { enemy_leader_weight = 1 }, cache)
@@ -1261,13 +1253,13 @@ function battle_calcs.relative_damage_map(units, enemies, cache)
                 best_enemy = enemy
             end
         end
-        unit_ratings[i] = { rating = max_rating, unit_id = u.id, enemy_id = best_enemy.id }
+        unit_ratings[i] = { rating = max_rating, unit_id = unit.id, enemy_id = best_enemy.id }
     end
 
     -- Then we want the same thing for all of the enemy units (for the counter attack on enemy turn)
     local enemy_ratings = {}
     for i,enemy in ipairs(enemies) do
-        local max_rating, best_unit = -9e99, {}
+        local max_rating, best_unit = - math.huge, {}
         for _,unit in ipairs(units) do
             local rating, defender_rating, attacker_rating =
                 battle_calcs.attack_rating(enemy, unit, { enemy.x, enemy.y }, { enemy_leader_weight = 1 }, cache)
@@ -1312,7 +1304,7 @@ function battle_calcs.best_defense_map(units, cfg)
     local defense_map = LS.create()
 
     if cfg.ignore_these_units then
-        for _,unit in ipairs(cfg.ignore_these_units) do wesnoth.extract_unit(unit) end
+        for _,unit in ipairs(cfg.ignore_these_units) do unit:extract() end
     end
 
     for _,unit in ipairs(units) do
@@ -1322,20 +1314,20 @@ function battle_calcs.best_defense_map(units, cfg)
         if (unit.side ~= wesnoth.current.side) then max_moves = true end
         local old_moves = unit.moves
         if max_moves then unit.moves = unit.max_moves end
-        local reach = wesnoth.find_reach(unit, cfg)
+        local reach = wesnoth.paths.find_reach(unit, cfg)
         if max_moves then unit.moves = old_moves end
 
         for _,loc in ipairs(reach) do
-            local defense = 100 - wesnoth.unit_defense(unit, wesnoth.get_terrain(loc[1], loc[2]))
+            local defense = unit:defense_on(wesnoth.current.map[loc])
 
-            if (defense > (defense_map:get(loc[1], loc[2]) or -9e99)) then
+            if (defense > (defense_map:get(loc[1], loc[2]) or - math.huge)) then
                 defense_map:insert(loc[1], loc[2], defense)
             end
         end
     end
 
     if cfg.ignore_these_units then
-        for _,unit in ipairs(cfg.ignore_these_units) do wesnoth.put_unit(unit) end
+        for _,unit in ipairs(cfg.ignore_these_units) do unit:to_map() end
     end
 
     return defense_map
@@ -1378,7 +1370,7 @@ function battle_calcs.get_attack_combos_subset(units, enemy, cfg)
 
     cfg = cfg or {}
     cfg.order_matters = cfg.order_matters or false
-    cfg.max_combos = cfg.max_combos or 9e99
+    cfg.max_combos = cfg.max_combos or math.huge
     cfg.max_time = cfg.max_time or false
     cfg.skip_presort = cfg.skip_presort or 5
 
@@ -1388,7 +1380,7 @@ function battle_calcs.get_attack_combos_subset(units, enemy, cfg)
     local function add_attack(attacks, reachable_hexes, n_reach, attack_combos, combos_str, current_combo, hexes_used, cfg)
 
         local time_up = false
-        if cfg.max_time and (wesnoth.get_time_stamp() / 1000. - cfg.start_time >= cfg.max_time) then
+        if cfg.max_time and (wesnoth.ms_since_init() / 1000. - cfg.start_time >= cfg.max_time) then
             time_up = true
         end
 
@@ -1476,11 +1468,11 @@ function battle_calcs.get_attack_combos_subset(units, enemy, cfg)
     local blocked_hexes = LS.create()
     if units[1] and (units[1].side == wesnoth.current.side) then
         for xa,ya in H.adjacent_tiles(enemy.x, enemy.y) do
-            local unit_in_way = wesnoth.get_unit(xa, ya)
+            local unit_in_way = wesnoth.units.get(xa, ya)
             if unit_in_way then
                 -- Units on the same side are blockers if they cannot move away
                 if (unit_in_way.side == wesnoth.current.side) then
-                    local reach = wesnoth.find_reach(unit_in_way)
+                    local reach = wesnoth.paths.find_reach(unit_in_way)
                     if (#reach <= 1) then
                         blocked_hexes:insert(unit_in_way.x, unit_in_way.y)
                     end
@@ -1518,7 +1510,7 @@ function battle_calcs.get_attack_combos_subset(units, enemy, cfg)
             if (not blocked_hexes:get(xa, ya) or ((xa == unit.x) and (ya == unit.y))) then
 
                 -- Check whether the unit can get to the hex
-                -- wesnoth.map.distance_between() is much faster than wesnoth.find_path()
+                -- wesnoth.map.distance_between() is much faster than wesnoth.paths.find_path()
                 --> pre-filter using the former
                 local cost = M.distance_between(unit.x, unit.y, xa, ya)
 
@@ -1534,7 +1526,7 @@ function battle_calcs.get_attack_combos_subset(units, enemy, cfg)
                     -- Store information about it in 'loc' and add this to 'locs'
                     -- Want coordinates (dst) and terrain defense (for sorting)
                     loc.dst = xa * 1000 + ya
-                    loc.hit_prob = wesnoth.unit_defense(unit, wesnoth.get_terrain(xa, ya))
+                    loc.hit_prob = 100 - unit:defense_on(wesnoth.current.map[{xa, ya}])
                     table.insert(locs, loc)
 
                     -- Also mark this hex as usable
@@ -1591,7 +1583,7 @@ function battle_calcs.get_attack_combos_subset(units, enemy, cfg)
     -- If cfg.max_time is set, record the start time
     -- For convenience, we store this in cfg
     if cfg.max_time then
-        cfg.start_time = wesnoth.get_time_stamp() / 1000.
+        cfg.start_time = wesnoth.ms_since_init() / 1000.
     end
 
 
