@@ -1,13 +1,11 @@
 
-local _ = wesnoth.textdomain "wesnoth-ai"
-
-wesnoth.deprecated_message('data/ai/lua/patrol.lua', 1, nil, _"Use the Patrols Micro AI instead of patrol.lua.")
+wesnoth.deprecated_message('data/ai/lua/patrol.lua', 3, '1.17.0', "Use the Patrols Micro AI instead of patrol.lua.")
 
 function patrol_gen(n, wp)
     -- n is the name of the unit, like Kiressh
     -- wp - a table of waypoint tables of form {x,y}
 
-    local unit = wesnoth.get_units({name=n})[1]
+    local unit = wesnoth.units.find_on_map({name=n})[1]
 
     local x, y = unit.x, unit.y
     local wpn = 1 --WayPoint Number - we have to remember which waypoint we are heading to
