@@ -10,14 +10,13 @@ function ca_recruit_random:evaluation(cfg)
     -- Random recruiting from all the units the side has
 
     -- Check if leader is on keep
-    local leader = wesnoth.get_units { side = wesnoth.current.side, canrecruit = 'yes' }[1]
-    if (not leader) or (not wesnoth.get_terrain_info(wesnoth.get_terrain(leader.x, leader.y)).keep) then
+    local leader = wesnoth.units.find_on_map { side = wesnoth.current.side, canrecruit = 'yes' }[1]
+    if (not leader) or (not wesnoth.terrain_types[wesnoth.current.map[leader]].keep) then
         return 0
     end
 
     -- Find all connected castle hexes
     local castle_map = LS.of_pairs({ { leader.x, leader.y } })
-    local width, height, border = wesnoth.get_map_size()
     local new_castle_hex_found = true
 
     while new_castle_hex_found do
@@ -26,11 +25,8 @@ function ca_recruit_random:evaluation(cfg)
 
         castle_map:iter(function(x, y)
             for xa,ya in H.adjacent_tiles(x, y) do
-                if (not castle_map:get(xa, ya))
-                    and (xa >= 1) and (xa <= width)
-                    and (ya >= 1) and (ya <= height)
-                then
-                    local is_castle = wesnoth.get_terrain_info(wesnoth.get_terrain(xa, ya)).castle
+                if (not castle_map:get(xa, ya)) and wesnoth.current.map:on_board(xa, ya) then
+                    local is_castle = wesnoth.terrain_types[wesnoth.current.map[{xa, ya}]].castle
 
                     if is_castle then
                         table.insert(new_hexes, { xa, ya })
@@ -48,7 +44,7 @@ function ca_recruit_random:evaluation(cfg)
     -- Check if there is space left for recruiting
     local no_space = true
     castle_map:iter(function(x, y)
-        local unit = wesnoth.get_unit(x, y)
+        local unit = wesnoth.units.get(x, y)
         if (not unit) then
             no_space = false
         end
@@ -60,7 +56,7 @@ function ca_recruit_random:evaluation(cfg)
 
     -- Go through all the types listed in [probability] tags (which can be comma-separated lists)
     for prob in wml.child_range(cfg, "probability") do
-        types = AH.split(prob.type, ",")
+        local types = AH.split(prob.type, ",")
         for _,typ in ipairs(types) do  -- 'type' is a reserved keyword in Lua
             -- If this type is in the recruit list, add it
             for _,recruit in ipairs(wesnoth.sides[wesnoth.current.side].recruit) do
