@@ -11,8 +11,8 @@ on_event("die", function()
 		return
 	end
 
-	local u_killer = wesnoth.get_unit(ec.x2, ec.y2)
-	local u_victim = wesnoth.get_unit(ec.x1, ec.y1)
+	local u_killer = wesnoth.units.get(ec.x2, ec.y2)
+	local u_victim = wesnoth.units.get(ec.x1, ec.y1)
 
 	if not u_killer or not u_killer:matches { ability = "feeding" } then
 		return
@@ -27,8 +27,8 @@ on_event("die", function()
 			effect.increase_total = effect.increase_total + 1
 			u_killer_cfg.max_hitpoints = u_killer_cfg.max_hitpoints + 1
 			u_killer_cfg.hitpoints = u_killer_cfg.hitpoints + 1
-			wesnoth.create_unit(u_killer_cfg):to_map()
-			wesnoth.float_label(ec.x2, ec.y2, _ "+1 max HP", "0,255,0")
+			wesnoth.units.to_map(u_killer_cfg)
+			wesnoth.interface.float_label(ec.x2, ec.y2, _ "+1 max HP", "0,255,0")
 			return
 		end
 	end
@@ -41,5 +41,5 @@ on_event("die", function()
 		},
 	})
 	u_killer.hitpoints = u_killer.hitpoints + 1
-	wesnoth.float_label(ec.x2, ec.y2, _ "+1 max HP", "0,255,0")
+	wesnoth.interface.float_label(ec.x2, ec.y2, _ "+1 max HP", "0,255,0")
 end)
