@@ -4,7 +4,7 @@ import re
 
 
 def autoscan(pathdir):
-    filelist = None
+    filelist = []
     parentdir = os.path.realpath(os.path.join(pathdir, '..'))
     for root, dirs, files in os.walk(pathdir, topdown=False):
         for name in files:
@@ -17,10 +17,7 @@ def autoscan(pathdir):
                     value = re.sub(r'^\/', '', value)
                 else:
                     value = re.sub(r'^(?:[A-Za-z]\:)?\\', '', value)
-                if filelist is None:
-                    filelist = [ value ]
-                else:
-                    filelist.append(value)
+                filelist.append(value)
                 # end if m
             # end for name
         # end for root
