@@ -27,6 +27,8 @@ dumpi = 0
 class CampaignClient:
     # First port listed will be used as default.
     portmap = (
+        ("15016", "1.16.x"),
+        ("15015", "1.15.x"),
         ("15014", "1.14.x"),
         ("15008", "1.13.x"),
         ("15007", "1.12.x"),
@@ -57,7 +59,7 @@ class CampaignClient:
         self.verbose = False
         self.quiet = quiet
 
-        if address != None:
+        if address is not None:
             self.canceled = False
             self.error = False
             s = address.split(":")
