@@ -67,7 +67,9 @@ def CheckBoost(context, boost_lib, require_version = None, header_only = False):
                       "unit_test_framework" : "test/unit_test.hpp",
                       "filesystem" : "filesystem/operations.hpp",
                       "random" : "random/random_number_generator.hpp",
-                      "system" : "system/error_code.hpp"}
+                      "system" : "system/error_code.hpp",
+                      "context" : "context/continuation.hpp",
+                      "coroutine" : "coroutine/coroutine.hpp" }
 
     header_name = boost_headers.get(boost_lib, boost_lib + ".hpp")
     libname = "boost_" + boost_lib + env.get("boost_suffix", "")
@@ -78,7 +80,7 @@ def CheckBoost(context, boost_lib, require_version = None, header_only = False):
         else:
             env.AppendUnique(CPPPATH = [boostdir], LIBPATH = [boostlibdir])
     if not header_only:
-        env.AppendUnique(LIBS = [libname])
+        env.PrependUnique(LIBS = [libname])
     if boost_lib == "thread" and env["PLATFORM"] == "posix":
         env.AppendUnique(CCFLAGS = ["-pthread"], LINKFLAGS = ["-pthread"])
 
