@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -23,6 +24,9 @@
 #include "actions/vision.hpp"
 
 #include "ai/lua/aspect_advancements.hpp"
+#include "formula/callable_objects.hpp"
+#include "formula/formula.hpp"
+#include "game_classification.hpp"
 #include "game_config.hpp"
 #include "game_data.hpp"
 #include "game_events/pump.hpp"
@@ -43,9 +47,12 @@
 #include "units/abilities.hpp"
 #include "units/animation_component.hpp"
 #include "units/helper.hpp"
+#include "units/filter.hpp"
 #include "units/map.hpp"
 #include "units/udisplay.hpp"
 #include "units/unit.hpp"
+#include "units/types.hpp"
+#include <optional>
 #include "whiteboard/manager.hpp"
 #include "wml_exception.hpp"
 
@@ -68,19 +75,19 @@ static lg::log_domain log_config("config");
 // BATTLE CONTEXT UNIT STATS
 // ==================================================================================
 
-battle_context_unit_stats::battle_context_unit_stats(const unit& u,
+battle_context_unit_stats::battle_context_unit_stats(nonempty_unit_const_ptr up,
 		const map_location& u_loc,
 		int u_attack_num,
 		bool attacking,
-		const unit& opp,
+		nonempty_unit_const_ptr oppp,
 		const map_location& opp_loc,
 		const_attack_ptr opp_weapon,
 		const unit_map& units)
 	: weapon(nullptr)
 	, attack_num(u_attack_num)
 	, is_attacker(attacking)
-	, is_poisoned(u.get_state(unit::STATE_POISONED))
-	, is_slowed(u.get_state(unit::STATE_SLOWED))
+	, is_poisoned(up->get_state(unit::STATE_POISONED))
+	, is_slowed(up->get_state(unit::STATE_SLOWED))
 	, slows(false)
 	, drains(false)
 	, petrifies(false)
@@ -90,12 +97,12 @@ battle_context_unit_stats::battle_context_unit_stats(const unit& u,
 	, swarm(false)
 	, firststrike(false)
 	, disable(false)
-	, experience(u.experience())
-	, max_experience(u.max_experience())
-	, level(u.level())
+	, experience(up->experience())
+	, max_experience(up->max_experience())
+	, level(up->level())
 	, rounds(1)
 	, hp(0)
-	, max_hp(u.max_hitpoints())
+	, max_hp(up->max_hitpoints())
 	, chance_to_hit(0)
 	, damage(0)
 	, slow_damage(0)
@@ -106,6 +113,8 @@ battle_context_unit_stats::battle_context_unit_stats(const unit& u,
 	, swarm_max(0)
 	, plague_type()
 {
+	const unit& u = *up;
+	const unit& opp = *oppp;
 	// Get the current state of the unit.
 	if(attack_num >= 0) {
 		weapon = u.attacks()[attack_num].shared_from_this();
@@ -128,37 +137,38 @@ battle_context_unit_stats::battle_context_unit_stats(const unit& u,
 	}
 
 	// Get the weapon characteristics as appropriate.
-	auto ctx = weapon->specials_context(&u, &opp, u_loc, opp_loc, attacking, opp_weapon);
-	boost::optional<decltype(ctx)> opp_ctx;
+	auto ctx = weapon->specials_context(up, oppp, u_loc, opp_loc, attacking, opp_weapon);
+	std::optional<decltype(ctx)> opp_ctx;
 
 	if(opp_weapon) {
-		opp_ctx = opp_weapon->specials_context(&opp, &u, opp_loc, u_loc, !attacking, weapon);
+		opp_ctx.emplace(opp_weapon->specials_context(oppp, up, opp_loc, u_loc, !attacking, weapon));
 	}
 
-	slows = weapon->get_special_bool("slow");
-	drains = !opp.get_state("undrainable") && weapon->get_special_bool("drains");
-	petrifies = weapon->get_special_bool("petrifies");
-	poisons = !opp.get_state("unpoisonable") && weapon->get_special_bool("poison") && !opp.get_state(unit::STATE_POISONED);
+	slows = weapon->has_special_or_ability("slow") && !opp.get_state("unslowable") ;
+	drains = !opp.get_state("undrainable") && weapon->has_special_or_ability("drains");
+	petrifies = !opp.get_state("unpetrifiable") && weapon->has_special_or_ability("petrifies");
+	poisons = !opp.get_state("unpoisonable") && weapon->has_special_or_ability("poison") && !opp.get_state(unit::STATE_POISONED);
 	backstab_pos = is_attacker && backstab_check(u_loc, opp_loc, units, resources::gameboard->teams());
-	rounds = weapon->get_specials("berserk").highest("value", 1).first;
-	firststrike = weapon->get_special_bool("firststrike");
+	rounds = weapon->get_specials_and_abilities("berserk").highest("value", 1).first;
+
+	firststrike = weapon->has_special_or_ability("firststrike");
 
 	{
 		const int distance = distance_between(u_loc, opp_loc);
 		const bool out_of_range = distance > weapon->max_range() || distance < weapon->min_range();
-		disable = weapon->get_special_bool("disable") || out_of_range;
+		disable = weapon->has_special("disable") || out_of_range;
 	}
 
 	// Handle plague.
-	unit_ability_list plague_specials = weapon->get_specials("plague");
+	unit_ability_list plague_specials = weapon->get_specials_and_abilities("plague");
 	plagues = !opp.get_state("unplagueable") && !plague_specials.empty() &&
 		opp.undead_variation() != "null" && !resources::gameboard->map().is_village(opp_loc);
 
 	if(plagues) {
-		plague_type = (*plague_specials.front().first)["type"].str();
+		plague_type = (*plague_specials.front().ability_cfg)["type"].str();
 
 		if(plague_type.empty()) {
-			plague_type = u.type().base_id();
+			plague_type = u.type().parent_id();
 		}
 	}
 
@@ -166,17 +176,18 @@ battle_context_unit_stats::battle_context_unit_stats(const unit& u,
 	signed int cth = opp.defense_modifier(resources::gameboard->map().get_terrain(opp_loc)) + weapon->accuracy()
 		- (opp_weapon ? opp_weapon->parry() : 0);
 
-	cth = utils::clamp(cth, 0, 100);
+	cth = std::clamp(cth, 0, 100);
 
-	unit_ability_list cth_specials = weapon->get_specials("chance_to_hit");
-	unit_abilities::effect cth_effects(cth_specials, cth, backstab_pos);
+	unit_ability_list cth_specials = weapon->get_specials_and_abilities("chance_to_hit");
+	unit_abilities::effect cth_effects(cth_specials, cth, backstab_pos, weapon);
 	cth = cth_effects.get_composite_value();
 
+
 	if(opp.get_state("invulnerable")) {
 		cth = 0;
 	}
 
-	chance_to_hit = utils::clamp(cth, 0, 100);
+	chance_to_hit = std::clamp(cth, 0, 100);
 
 	// Compute base damage done with the weapon.
 	int base_damage = weapon->modified_damage(backstab_pos);
@@ -189,13 +200,13 @@ battle_context_unit_stats::battle_context_unit_stats(const unit& u,
 			resources::gameboard->units(), resources::gameboard->map(), u_loc, u.alignment(), u.is_fearless());
 
 	// Leadership bonus.
-	int leader_bonus = under_leadership(u, u_loc);
+	int leader_bonus = under_leadership(u, u_loc, weapon, opp_weapon);
 	if(leader_bonus != 0) {
 		damage_multiplier += leader_bonus;
 	}
 
 	// Resistance modifier.
-	damage_multiplier *= opp.damage_from(*weapon, !attacking, opp_loc);
+	damage_multiplier *= opp.damage_from(*weapon, !attacking, opp_loc, opp_weapon);
 
 	// Compute both the normal and slowed damage.
 	damage = round_damage(base_damage, damage_multiplier, 10000);
@@ -207,16 +218,15 @@ battle_context_unit_stats::battle_context_unit_stats(const unit& u,
 
 	// Compute drain amounts only if draining is possible.
 	if(drains) {
-		unit_ability_list drain_specials = weapon->get_specials("drains");
-
+		unit_ability_list drain_specials = weapon->get_specials_and_abilities("drains");
 		// Compute the drain percent (with 50% as the base for backward compatibility)
-		unit_abilities::effect drain_percent_effects(drain_specials, 50, backstab_pos);
+		unit_abilities::effect drain_percent_effects(drain_specials, 50, backstab_pos, weapon);
 		drain_percent = drain_percent_effects.get_composite_value();
 	}
 
 	// Add heal_on_hit (the drain constant)
-	unit_ability_list heal_on_hit_specials = weapon->get_specials("heal_on_hit");
-	unit_abilities::effect heal_on_hit_effects(heal_on_hit_specials, 0, backstab_pos);
+	unit_ability_list heal_on_hit_specials = weapon->get_specials_and_abilities("heal_on_hit");
+	unit_abilities::effect heal_on_hit_effects(heal_on_hit_specials, 0, backstab_pos, weapon);
 	drain_constant += heal_on_hit_effects.get_composite_value();
 
 	drains = drain_constant || drain_percent;
@@ -286,44 +296,44 @@ battle_context_unit_stats::battle_context_unit_stats(const unit_type* u_type,
 
 	// Get the weapon characteristics as appropriate.
 	auto ctx = weapon->specials_context(*u_type, map_location::null_location(), attacking);
-	boost::optional<decltype(ctx)> opp_ctx;
+	std::optional<decltype(ctx)> opp_ctx;
 
 	if(opp_weapon) {
-		opp_ctx = opp_weapon->specials_context(*opp_type, map_location::null_location(), !attacking);
+		opp_ctx.emplace(opp_weapon->specials_context(*opp_type, map_location::null_location(), !attacking));
 	}
 
-	slows = weapon->get_special_bool("slow");
-	drains = !opp_type->musthave_status("undrainable") && weapon->get_special_bool("drains");
-	petrifies = weapon->get_special_bool("petrifies");
-	poisons = !opp_type->musthave_status("unpoisonable") && weapon->get_special_bool("poison");
+	slows = weapon->has_special("slow");
+	drains = !opp_type->musthave_status("undrainable") && weapon->has_special("drains");
+	petrifies = weapon->has_special("petrifies");
+	poisons = !opp_type->musthave_status("unpoisonable") && weapon->has_special("poison");
 	rounds = weapon->get_specials("berserk").highest("value", 1).first;
-	firststrike = weapon->get_special_bool("firststrike");
-	disable = weapon->get_special_bool("disable");
+	firststrike = weapon->has_special("firststrike");
+	disable = weapon->has_special("disable");
 
 	unit_ability_list plague_specials = weapon->get_specials("plague");
 	plagues = !opp_type->musthave_status("unplagueable") && !plague_specials.empty() &&
 		opp_type->undead_variation() != "null";
 
 	if(plagues) {
-		plague_type = (*plague_specials.front().first)["type"].str();
+		plague_type = (*plague_specials.front().ability_cfg)["type"].str();
 		if(plague_type.empty()) {
-			plague_type = u_type->base_id();
+			plague_type = u_type->parent_id();
 		}
 	}
 
 	signed int cth = 100 - opp_terrain_defense + weapon->accuracy() - (opp_weapon ? opp_weapon->parry() : 0);
-	cth = utils::clamp(cth, 0, 100);
+	cth = std::clamp(cth, 0, 100);
 
 	unit_ability_list cth_specials = weapon->get_specials("chance_to_hit");
-	unit_abilities::effect cth_effects(cth_specials, cth, backstab_pos);
+	unit_abilities::effect cth_effects(cth_specials, cth, backstab_pos, weapon);
 	cth = cth_effects.get_composite_value();
 
-	chance_to_hit = utils::clamp(cth, 0, 100);
+	chance_to_hit = std::clamp(cth, 0, 100);
 
 	int base_damage = weapon->modified_damage(backstab_pos);
 	int damage_multiplier = 100;
 	damage_multiplier
-			+= generic_combat_modifier(lawful_bonus, u_type->alignment(), u_type->musthave_status("fearless"));
+			+= generic_combat_modifier(lawful_bonus, u_type->alignment(), u_type->musthave_status("fearless"), 0);
 	damage_multiplier *= opp_type->resistance_against(weapon->type(), !attacking);
 
 	damage = round_damage(base_damage, damage_multiplier, 10000);
@@ -333,13 +343,13 @@ battle_context_unit_stats::battle_context_unit_stats(const unit_type* u_type,
 		unit_ability_list drain_specials = weapon->get_specials("drains");
 
 		// Compute the drain percent (with 50% as the base for backward compatibility)
-		unit_abilities::effect drain_percent_effects(drain_specials, 50, backstab_pos);
+		unit_abilities::effect drain_percent_effects(drain_specials, 50, backstab_pos, weapon);
 		drain_percent = drain_percent_effects.get_composite_value();
 	}
 
 	// Add heal_on_hit (the drain constant)
 	unit_ability_list heal_on_hit_specials = weapon->get_specials("heal_on_hit");
-	unit_abilities::effect heal_on_hit_effects(heal_on_hit_specials, 0, backstab_pos);
+	unit_abilities::effect heal_on_hit_effects(heal_on_hit_specials, 0, backstab_pos, weapon);
 	drain_constant += heal_on_hit_effects.get_composite_value();
 
 	drains = drain_constant || drain_percent;
@@ -356,10 +366,10 @@ battle_context_unit_stats::battle_context_unit_stats(const unit_type* u_type,
 // ==================================================================================
 
 battle_context::battle_context(
-		const unit& attacker,
+		nonempty_unit_const_ptr attacker,
 		const map_location& a_loc,
 		int a_wep_index,
-		const unit& defender,
+		nonempty_unit_const_ptr defender,
 		const map_location& d_loc,
 		int d_wep_index,
 		const unit_map& units)
@@ -371,31 +381,13 @@ battle_context::battle_context(
 	size_t a_wep_uindex = static_cast<size_t>(a_wep_index);
 	size_t d_wep_uindex = static_cast<size_t>(d_wep_index);
 
-	const_attack_ptr a_wep(a_wep_uindex < attacker.attacks().size() ? attacker.attacks()[a_wep_index].shared_from_this() : nullptr);
-	const_attack_ptr d_wep(d_wep_uindex < defender.attacks().size() ? defender.attacks()[d_wep_index].shared_from_this() : nullptr);
+	const_attack_ptr a_wep(a_wep_uindex < attacker->attacks().size() ? attacker->attacks()[a_wep_index].shared_from_this() : nullptr);
+	const_attack_ptr d_wep(d_wep_uindex < defender->attacks().size() ? defender->attacks()[d_wep_index].shared_from_this() : nullptr);
 
 	attacker_stats_.reset(new battle_context_unit_stats(attacker, a_loc, a_wep_index, true , defender, d_loc, d_wep, units));
 	defender_stats_.reset(new battle_context_unit_stats(defender, d_loc, d_wep_index, false, attacker, a_loc, a_wep, units));
 }
 
-battle_context::battle_context(battle_context&& other)
-	: attacker_stats_(std::move(other.attacker_stats_))
-	, defender_stats_(std::move(other.defender_stats_))
-	, attacker_combatant_(std::move(other.attacker_combatant_))
-	, defender_combatant_(std::move(other.defender_combatant_))
-{
-
-}
-
-battle_context& battle_context::operator=(battle_context&& other)
-{
-	attacker_stats_ = std::move(other.attacker_stats_);
-	defender_stats_ = std::move(other.defender_stats_);
-	attacker_combatant_ = std::move(other.attacker_combatant_);
-	defender_combatant_ = std::move(other.defender_combatant_);
-	return *this;
-}
-
 void battle_context::simulate(const combatant* prev_def)
 {
 	assert((attacker_combatant_.get() != nullptr) == (defender_combatant_.get() != nullptr));
@@ -416,28 +408,37 @@ battle_context::battle_context(const unit_map& units,
 		int defender_weapon,
 		double aggression,
 		const combatant* prev_def,
-		const unit* attacker_ptr)
+		unit_const_ptr attacker,
+		unit_const_ptr defender)
 	: attacker_stats_(nullptr)
 	, defender_stats_(nullptr)
 	, attacker_combatant_(nullptr)
 	, defender_combatant_(nullptr)
 {
-	const unit& attacker = attacker_ptr ? *attacker_ptr : *units.find(attacker_loc);
-	const unit& defender = *units.find(defender_loc);
+	//TODO: maybe check before dereferencing units.find(attacker_loc),units.find(defender_loc) ?
+	if(!attacker) {
+		attacker = units.find(attacker_loc).get_shared_ptr();
+	}
+	if(!defender) {
+		defender = units.find(defender_loc).get_shared_ptr();
+	}
+	nonempty_unit_const_ptr n_attacker { attacker };
+	nonempty_unit_const_ptr n_defender { defender };
+
 	const double harm_weight = 1.0 - aggression;
 
 	if(attacker_weapon == -1) {
 		*this = choose_attacker_weapon(
-			attacker, defender, units, attacker_loc, defender_loc, harm_weight, prev_def
+			n_attacker, n_defender, units, attacker_loc, defender_loc, harm_weight, prev_def
 		);
 	}
 	else if(defender_weapon == -1) {
 		*this = choose_defender_weapon(
-			attacker, defender, attacker_weapon, units, attacker_loc, defender_loc, prev_def
+			n_attacker, n_defender, attacker_weapon, units, attacker_loc, defender_loc, prev_def
 		);
 	}
 	else {
-		*this = battle_context(attacker, attacker_loc, attacker_weapon, defender, defender_loc, defender_weapon, units);
+		*this = battle_context(n_attacker, attacker_loc, attacker_weapon, n_defender, defender_loc, defender_weapon, units);
 	}
 
 	assert(attacker_stats_);
@@ -536,8 +537,8 @@ bool battle_context::better_combat(const combatant& us_a,
 	return them_a.average_hp() < them_b.average_hp();
 }
 
-battle_context battle_context::choose_attacker_weapon(const unit& attacker,
-		const unit& defender,
+battle_context battle_context::choose_attacker_weapon(nonempty_unit_const_ptr attacker,
+		nonempty_unit_const_ptr defender,
 		const unit_map& units,
 		const map_location& attacker_loc,
 		const map_location& defender_loc,
@@ -548,8 +549,8 @@ battle_context battle_context::choose_attacker_weapon(const unit& attacker,
 	std::vector<battle_context> choices;
 
 	// What options does attacker have?
-	for(size_t i = 0; i < attacker.attacks().size(); ++i) {
-		const attack_type& att = attacker.attacks()[i];
+	for(size_t i = 0; i < attacker->attacks().size(); ++i) {
+		const attack_type& att = attacker->attacks()[i];
 
 		if(att.attack_weight() <= 0) {
 			continue;
@@ -590,8 +591,8 @@ battle_context battle_context::choose_attacker_weapon(const unit& attacker,
 }
 
 /** @todo FIXME: Hand previous defender unit in here. */
-battle_context battle_context::choose_defender_weapon(const unit& attacker,
-		const unit& defender,
+battle_context battle_context::choose_defender_weapon(nonempty_unit_const_ptr attacker,
+		nonempty_unit_const_ptr defender,
 		unsigned attacker_weapon,
 		const unit_map& units,
 		const map_location& attacker_loc,
@@ -599,15 +600,15 @@ battle_context battle_context::choose_defender_weapon(const unit& attacker,
 		const combatant* prev_def)
 {
 	log_scope2(log_attack, "choose_defender_weapon");
-	VALIDATE(attacker_weapon < attacker.attacks().size(), _("An invalid attacker weapon got selected."));
+	VALIDATE(attacker_weapon < attacker->attacks().size(), _("An invalid attacker weapon got selected."));
 
-	const attack_type& att = attacker.attacks()[attacker_weapon];
+	const attack_type& att = attacker->attacks()[attacker_weapon];
 	auto no_weapon = [&]() { return battle_context(attacker, attacker_loc, attacker_weapon, defender, defender_loc, -1, units); };
 	std::vector<battle_context> choices;
 
 	// What options does defender have?
-	for(size_t i = 0; i < defender.attacks().size(); ++i) {
-		const attack_type& def = defender.attacks()[i];
+	for(size_t i = 0; i < defender->attacks().size(); ++i) {
+		const attack_type& def = defender->attacks()[i];
 		if(def.range() != att.range() || def.defense_weight() <= 0) {
 			//no need to calculate the battle_context here.
 			continue;
@@ -643,7 +644,7 @@ battle_context battle_context::choose_defender_weapon(const unit& attacker,
 		double max_weight = 0.0;
 
 		for(const auto& choice : choices) {
-			const attack_type& def = defender.attacks()[choice.defender_stats_->attack_num];
+			const attack_type& def = defender->attacks()[choice.defender_stats_->attack_num];
 
 			if(def.defense_weight() >= max_weight) {
 				const battle_context_unit_stats& def_stats = *choice.defender_stats_;
@@ -662,10 +663,11 @@ battle_context battle_context::choose_defender_weapon(const unit& attacker,
 	battle_context* best_choice = nullptr;
 	// Multiple options: simulate them, save best.
 	for(auto& choice : choices) {
-		const attack_type& def = defender.attacks()[choice.defender_stats_->attack_num];
+		const attack_type& def = defender->attacks()[choice.defender_stats_->attack_num];
 
 		choice.simulate(prev_def);
 
+
 		int simple_rating = static_cast<int>(
 				choice.defender_stats_->num_blows * choice.defender_stats_->damage * choice.defender_stats_->chance_to_hit * def.defense_weight());
 
@@ -740,7 +742,7 @@ private:
 		const map_location loc_;
 		int weapon_;
 		unit_map& units_;
-		size_t id_; /**< unit.underlying_id() */
+		std::size_t id_; /**< unit.underlying_id() */
 		std::string weap_id_;
 		int orig_attacks_;
 		int n_attacks_; /**< Number of attacks left. */
@@ -750,6 +752,7 @@ private:
 
 		unit_info(const map_location& loc, int weapon, unit_map& units);
 		unit& get_unit();
+		unit_ptr get_unit_ptr();
 		bool valid();
 
 		std::string dump();
@@ -813,6 +816,15 @@ unit& attack::unit_info::get_unit()
 	return *i;
 }
 
+unit_ptr attack::unit_info::get_unit_ptr()
+{
+	unit_map::iterator i = units_.find(loc_);
+	if(i.valid() && i->underlying_id() == id_) {
+		return i.get_shared_ptr();
+	}
+	return unit_ptr();
+}
+
 bool attack::unit_info::valid()
 {
 	unit_map::iterator i = units_.find(loc_);
@@ -847,10 +859,12 @@ attack::attack(const map_location& attacker,
 	, OOS_error_(false)
 
 	//new experimental prng mode.
-	, use_prng_(preferences::get("use_prng") == "yes" && randomness::generator->is_networked() == false)
+	, use_prng_(resources::classification->random_mode == "biased" && randomness::generator->is_networked() == false)
+	, prng_attacker_()
+	, prng_defender_()
 {
 	if(use_prng_) {
-		std::cerr << "Using experimental PRNG for combat\n";
+		LOG_NG << "Using experimental PRNG for combat\n";
 	}
 }
 
@@ -864,22 +878,22 @@ void attack::fire_event(const std::string& n)
 	config& d_weapon_cfg = ev_data.add_child("second");
 
 	// Need these to ensure weapon filters work correctly
-	boost::optional<attack_type::specials_context_t> a_ctx, d_ctx;
+	std::optional<attack_type::specials_context_t> a_ctx, d_ctx;
 
 	if(a_stats_->weapon != nullptr && a_.valid()) {
 		if(d_stats_->weapon != nullptr && d_.valid()) {
-			a_ctx = a_stats_->weapon->specials_context(nullptr, nullptr, a_.loc_, d_.loc_, true, d_stats_->weapon);
+			a_ctx.emplace(a_stats_->weapon->specials_context(nullptr, nullptr, a_.loc_, d_.loc_, true, d_stats_->weapon));
 		} else {
-			a_ctx = a_stats_->weapon->specials_context(nullptr, a_.loc_, true);
+			a_ctx.emplace(a_stats_->weapon->specials_context(nullptr, a_.loc_, true));
 		}
 		a_stats_->weapon->write(a_weapon_cfg);
 	}
 
 	if(d_stats_->weapon != nullptr && d_.valid()) {
 		if(a_stats_->weapon != nullptr && a_.valid()) {
-			d_ctx = d_stats_->weapon->specials_context(nullptr, nullptr, d_.loc_, a_.loc_, false, a_stats_->weapon);
+			d_ctx.emplace(d_stats_->weapon->specials_context(nullptr, nullptr, d_.loc_, a_.loc_, false, a_stats_->weapon));
 		} else {
-			d_ctx = d_stats_->weapon->specials_context(nullptr, d_.loc_, false);
+			d_ctx.emplace(d_stats_->weapon->specials_context(nullptr, d_.loc_, false));
 		}
 		d_stats_->weapon->write(d_weapon_cfg);
 	}
@@ -1085,7 +1099,7 @@ bool attack::perform_hit(bool attacker_turn, statistics::attack_context& stats)
 			damage,
 			*attacker_stats->weapon, defender_stats->weapon,
 			abs_n, float_text.str(), drains_damage, "",
-			&extra_hit_sounds
+			&extra_hit_sounds, attacker_turn
 		);
 	}
 
@@ -1097,14 +1111,16 @@ bool attack::perform_hit(bool attacker_turn, statistics::attack_context& stats)
 			? (dies
 				? statistics::attack_context::KILLS
 				: statistics::attack_context::HITS)
-			: statistics::attack_context::MISSES, damage_done, drains_damage
+			: statistics::attack_context::MISSES,
+			attacker.cth_, damage_done, drains_damage
 		);
 	} else {
 		stats.defend_result(hits
 			? (dies
 				? statistics::attack_context::KILLS
 				: statistics::attack_context::HITS)
-			: statistics::attack_context::MISSES, damage_done, drains_damage
+			: statistics::attack_context::MISSES,
+			attacker.cth_, damage_done, drains_damage
 		);
 	}
 
@@ -1265,7 +1281,7 @@ void attack::unit_killed(unit_info& attacker,
 			attacker_stats->weapon,
 			defender_stats->weapon,
 			attacker.loc_,
-			&attacker.get_unit()
+			attacker.get_unit_ptr()
 		);
 	}
 
@@ -1390,8 +1406,8 @@ void attack::perform()
 	d_.orig_attacks_ = d_stats_->num_blows;
 	a_.n_attacks_ = a_.orig_attacks_;
 	d_.n_attacks_ = d_.orig_attacks_;
-	a_.xp_ = d_.get_unit().level();
-	d_.xp_ = a_.get_unit().level();
+	a_.xp_ = game_config::combat_xp(d_.get_unit().level());
+	d_.xp_ = game_config::combat_xp(a_.get_unit().level());
 
 	bool defender_strikes_first = (d_stats_->firststrike && !a_stats_->firststrike);
 	unsigned int rounds = std::max<unsigned int>(a_stats_->rounds, d_stats_->rounds) - 1;
@@ -1404,9 +1420,9 @@ void attack::perform()
 		   << (defender_strikes_first ? " defender first-strike" : "") << "\n";
 
 	// Play the pre-fight animation
-	unit_display::unit_draw_weapon(a_.loc_, a_.get_unit(), a_stats_->weapon, d_stats_->weapon, d_.loc_, &d_.get_unit());
+	unit_display::unit_draw_weapon(a_.loc_, a_.get_unit(), a_stats_->weapon, d_stats_->weapon, d_.loc_, d_.get_unit_ptr());
 
-	for(;;) {
+	while(true) {
 		DBG_NG << "start of attack loop...\n";
 		++abs_n_attack_;
 
@@ -1468,8 +1484,8 @@ void attack::perform()
 		u.set_experience(u.experience() + d_.xp_);
 	}
 
-	unit_display::unit_sheath_weapon(a_.loc_, a_.valid() ? &a_.get_unit() : nullptr, a_stats_->weapon, d_stats_->weapon,
-			d_.loc_, d_.valid() ? &d_.get_unit() : nullptr);
+	unit_display::unit_sheath_weapon(a_.loc_, a_.get_unit_ptr(), a_stats_->weapon, d_stats_->weapon,
+			d_.loc_, d_.get_unit_ptr());
 
 	if(update_display_) {
 		game_display::get_singleton()->invalidate_unit();
@@ -1551,26 +1567,26 @@ void attack_unit_and_advance(const map_location& attacker,
 		const map_location& defender,
 		int attack_with,
 		int defend_with,
-		bool update_display,
-		const ai::unit_advancements_aspect& ai_advancement)
+		bool update_display)
 {
 	attack_unit(attacker, defender, attack_with, defend_with, update_display);
 
 	unit_map::const_iterator atku = resources::gameboard->units().find(attacker);
 	if(atku != resources::gameboard->units().end()) {
-		advance_unit_at(advance_unit_params(attacker).ai_advancements(ai_advancement));
+		advance_unit_at(advance_unit_params(attacker));
 	}
 
 	unit_map::const_iterator defu = resources::gameboard->units().find(defender);
 	if(defu != resources::gameboard->units().end()) {
-		advance_unit_at(advance_unit_params(defender).ai_advancements(ai_advancement));
+		advance_unit_at(advance_unit_params(defender));
 	}
 }
 
-int under_leadership(const unit &u, const map_location& loc)
+int under_leadership(const unit &u, const map_location& loc, const_attack_ptr weapon, const_attack_ptr opp_weapon)
 {
-	unit_ability_list abil = u.get_abilities("leadership", loc);
-	return abil.highest("value").first;
+	unit_ability_list abil = u.get_abilities_weapons("leadership", loc, weapon, opp_weapon);
+	unit_abilities::effect leader_effect(abil, 0, false);
+	return leader_effect.get_composite_value();
 }
 
 int combat_modifier(const unit_map& units,
@@ -1588,11 +1604,12 @@ int combat_modifier(const time_of_day& effective_tod,
 		unit_type::ALIGNMENT alignment,
 		bool is_fearless)
 {
+	const tod_manager& tod_m = *resources::tod_manager;
 	const int lawful_bonus = effective_tod.lawful_bonus;
-	return generic_combat_modifier(lawful_bonus, alignment, is_fearless);
+	return generic_combat_modifier(lawful_bonus, alignment, is_fearless, tod_m.get_max_liminal_bonus());
 }
 
-int generic_combat_modifier(int lawful_bonus, unit_type::ALIGNMENT alignment, bool is_fearless)
+int generic_combat_modifier(int lawful_bonus, unit_type::ALIGNMENT alignment, bool is_fearless, int max_liminal_bonus)
 {
 	int bonus;
 
@@ -1607,7 +1624,7 @@ int generic_combat_modifier(int lawful_bonus, unit_type::ALIGNMENT alignment, bo
 		bonus = -lawful_bonus;
 		break;
 	case unit_type::ALIGNMENT::LIMINAL:
-		bonus = -std::abs(lawful_bonus);
+		bonus = std::max(0, max_liminal_bonus-std::abs(lawful_bonus));
 		break;
 	default:
 		bonus = 0;
@@ -1630,9 +1647,7 @@ bool backstab_check(const map_location& attacker_loc,
 		return false; // No defender
 	}
 
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(defender_loc, adj.data());
-
+	const auto adj = get_adjacent_tiles(defender_loc);
 	unsigned i;
 
 	for(i = 0; i < adj.size(); ++i) {
@@ -1657,7 +1672,7 @@ bool backstab_check(const map_location& attacker_loc,
 	}
 
 	// If sides aren't valid teams, then they are enemies.
-	if(size_t(defender->side() - 1) >= teams.size() || size_t(opp->side() - 1) >= teams.size()) {
+	if(std::size_t(defender->side() - 1) >= teams.size() || std::size_t(opp->side() - 1) >= teams.size()) {
 		return true;
 	}
 
