@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -23,13 +24,15 @@
 
 #include "ai/lua/aspect_advancements.hpp"
 #include "attack_prediction.hpp"
-#include "units/types.hpp"
+#include "units/alignment.hpp"
+#include "units/ptr.hpp"
 
 #include <vector>
 
 struct map_location;
 class team;
 struct time_of_day;
+class unit_type;
 class unit;
 class unit_map;
 class gamemap;
@@ -81,16 +84,16 @@ struct battle_context_unit_stats
 
 	std::string plague_type; /**< The plague type used by the attack, if any. */
 
-	battle_context_unit_stats(const unit& u,
+	battle_context_unit_stats(nonempty_unit_const_ptr u,
 			const map_location& u_loc,
 			int u_attack_num,
 			bool attacking,
-			const unit& opp,
+			nonempty_unit_const_ptr opp,
 			const map_location& opp_loc,
 			const_attack_ptr opp_weapon,
 			const unit_map& units);
 
-	/** Used by AI for combat analysis */
+	/** Used by AI for combat analysis, and by statistics_dialog */
 	battle_context_unit_stats(const unit_type* u_type,
 			const_attack_ptr att_weapon,
 			bool attacking,
@@ -103,8 +106,7 @@ struct battle_context_unit_stats
 	{
 	}
 
-	/// Calculates the number of blows we would have if we had @a new_hp
-	// instead of the recorded hp.
+	/** Calculates the number of blows we would have if we had @a new_hp instead of the recorded hp. */
 	unsigned int calc_blows(unsigned new_hp) const
 	{
 		return swarm_blows(swarm_min, swarm_max, new_hp, max_hp);
@@ -184,16 +186,15 @@ public:
 			int defender_weapon = -1,
 			double aggression = 0.0,
 			const combatant* prev_def = nullptr,
-			const unit* attacker_ptr = nullptr);
+			unit_const_ptr attacker_ptr = unit_const_ptr(),
+			unit_const_ptr defender_ptr = unit_const_ptr());
 
 	/** Used by the AI which caches battle_context_unit_stats */
 	battle_context(const battle_context_unit_stats& att, const battle_context_unit_stats& def);
 
-	battle_context(const battle_context& other) = delete;
-	battle_context(battle_context&& other);
+	battle_context(battle_context&& other) = default;
 
-	battle_context& operator=(const battle_context& other) = delete;
-	battle_context& operator=(battle_context&& other);
+	battle_context& operator=(battle_context&& other) = default;
 
 	/** This method returns the statistics of the attacker. */
 	const battle_context_unit_stats& get_attacker_stats() const
@@ -225,24 +226,24 @@ public:
 	void simulate(const combatant* prev_def);
 private:
 	battle_context(
-			const unit& attacker,
+			nonempty_unit_const_ptr attacker,
 			const map_location& attacker_loc,
 			int attacker_weapon,
-			const unit& defender,
+			nonempty_unit_const_ptr defender,
 			const map_location& defender_loc,
 			int defender_weapon,
 			const unit_map& units);
 
-	static battle_context choose_attacker_weapon(const unit& attacker,
-			const unit& defender,
+	static battle_context choose_attacker_weapon(nonempty_unit_const_ptr attacker,
+			nonempty_unit_const_ptr defender,
 			const unit_map& units,
 			const map_location& attacker_loc,
 			const map_location& defender_loc,
 			double harm_weight,
 			const combatant* prev_def);
 
-	static battle_context choose_defender_weapon(const unit& attacker,
-			const unit& defender,
+	static battle_context choose_defender_weapon(nonempty_unit_const_ptr attacker,
+			nonempty_unit_const_ptr defender,
 			unsigned attacker_weapon,
 			const unit_map& units,
 			const map_location& attacker_loc,
@@ -270,8 +271,7 @@ void attack_unit_and_advance(const map_location& attacker,
 		const map_location& defender,
 		int attack_with,
 		int defend_with,
-		bool update_display = true,
-		const ai::unit_advancements_aspect& ai_advancement = ai::unit_advancements_aspect());
+		bool update_display = true);
 
 /**
  * Tests if the unit at loc is currently affected by leadership.
@@ -279,7 +279,7 @@ void attack_unit_and_advance(const map_location& attacker,
  *
  * Returns the bonus percentage (possibly 0 if there's no leader adjacent).
  */
-int under_leadership(const unit &u, const map_location& loc);
+int under_leadership(const unit &u, const map_location& loc, const_attack_ptr weapon = nullptr, const_attack_ptr opp_weapon = nullptr);
 
 /**
  * Returns the amount that a unit's damage should be multiplied by
@@ -288,7 +288,7 @@ int under_leadership(const unit &u, const map_location& loc);
 int combat_modifier(const unit_map& units,
 		const gamemap& map,
 		const map_location& loc,
-		unit_type::ALIGNMENT alignment,
+		UNIT_ALIGNMENT alignment,
 		bool is_fearless);
 
 /**
@@ -296,14 +296,14 @@ int combat_modifier(const unit_map& units,
  * due to the current time of day.
  */
 int combat_modifier(const time_of_day& effective_tod,
-		unit_type::ALIGNMENT alignment,
+		UNIT_ALIGNMENT alignment,
 		bool is_fearless);
 
 /**
  * Returns the amount that a unit's damage should be multiplied by
  * due to a given lawful_bonus.
  */
-int generic_combat_modifier(int lawful_bonus, unit_type::ALIGNMENT alignment, bool is_fearless);
+int generic_combat_modifier(int lawful_bonus, UNIT_ALIGNMENT alignment, bool is_fearless, int max_liminal_bonus);
 /**
  * Function to check if an attack will satisfy the requirements for backstab.
  * Input:
