@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -45,6 +46,8 @@
 #include "units/unit.hpp"
 #include "units/udisplay.hpp"
 #include "units/filter.hpp"
+#include "units/types.hpp"
+#include "utils/general.hpp"
 #include "variable.hpp"
 #include "whiteboard/manager.hpp"
 
@@ -83,7 +86,7 @@ const std::set<std::string> get_recruits(int side, const map_location &recruit_l
 			local_result.insert(find_it->recruits().begin(),
 			                    find_it->recruits().end());
 		}
-		else if ( find_it->is_visible_to_team(current_team, *resources::gameboard, false) )
+		else if ( find_it->is_visible_to_team(current_team, false) )
 		{
 			// This hex is visibly occupied, so we cannot recruit here.
 			allow_local = false;
@@ -127,18 +130,18 @@ namespace { // Helpers for get_recalls()
 	 */
 	void add_leader_filtered_recalls(const unit_const_ptr leader,
 	                                 std::vector< unit_const_ptr > & result,
-	                                 std::set<size_t> * already_added = nullptr)
+	                                 std::set<std::size_t> * already_added = nullptr)
 	{
 		const team& leader_team = resources::gameboard->get_team(leader->side());
-		std::string save_id = leader_team.save_id_or_number();
+		const std::string& save_id = leader_team.save_id_or_number();
 
 		const unit_filter ufilt(vconfig(leader->recall_filter()));
 
-		for (const unit_const_ptr & recall_unit_ptr : leader_team.recall_list())
+		for (const unit_const_ptr recall_unit_ptr : leader_team.recall_list())
 		{
 			const unit & recall_unit = *recall_unit_ptr;
 			// Do not add a unit twice.
-			size_t underlying_id = recall_unit.underlying_id();
+			std::size_t underlying_id = recall_unit.underlying_id();
 			if ( !already_added  ||  already_added->count(underlying_id) == 0 )
 			{
 				// Only units that match the leader's recall filter are valid.
@@ -184,7 +187,7 @@ std::vector<unit_const_ptr > get_recalls(int side, const map_location &recall_lo
 			add_leader_filtered_recalls(find_it.get_shared_ptr(), result);
 			return result;
 		}
-		else if ( find_it->is_visible_to_team(resources::gameboard->get_team(side), *resources::gameboard, false) )
+		else if ( find_it->is_visible_to_team(resources::gameboard->get_team(side), false) )
 		{
 			// This hex is visibly occupied, so we cannot recall here.
 			allow_local = false;
@@ -195,7 +198,7 @@ std::vector<unit_const_ptr > get_recalls(int side, const map_location &recall_lo
 	{
 		unit_map::const_iterator u = resources::gameboard->units().begin(),
 				u_end = resources::gameboard->units().end();
-		std::set<size_t> valid_local_recalls;
+		std::set<std::size_t> valid_local_recalls;
 
 		for(; u != u_end; ++u) {
 			//We only consider leaders on our side.
@@ -213,7 +216,7 @@ std::vector<unit_const_ptr > get_recalls(int side, const map_location &recall_lo
 
 	if ( !leader_in_place )
 	{
-		std::set<size_t> valid_local_recalls;
+		std::set<std::size_t> valid_local_recalls;
 
 		for(auto u = resources::gameboard->units().begin(); u != resources::gameboard->units().end(); ++u) {
 			//We only consider leaders on our side.
@@ -272,7 +275,7 @@ namespace { // Helpers for check_recall_location()
 	}
 }//anonymous namespace
 
-/// Checks if there is a location on which to recall @a unit_recall.
+/** Checks if there is a location on which to recall @a unit_recall. */
 RECRUIT_CHECK check_recall_location(const int side, map_location& recall_location,
                                     map_location& recall_from,
                                     const unit &unit_recall)
@@ -402,7 +405,7 @@ namespace { // Helpers for check_recruit_location()
 	}
 }//anonymous namespace
 
-/// Checks if there is a location on which to place a recruited unit.
+/** Checks if there is a location on which to place a recruited unit. */
 RECRUIT_CHECK check_recruit_location(const int side, map_location &recruit_location,
                                      map_location& recruited_from,
                                      const std::string& unit_type)
@@ -585,7 +588,7 @@ namespace { // Helpers for place_recruit()
 
 		for ( unit_itor = units.begin(); unit_itor != units.end(); ++unit_itor ) {
 			if (resources::gameboard->get_team(unit_itor->side()).is_enemy(new_unit.side()) &&
-				unit_itor->is_visible_to_team(resources::gameboard->get_team(new_unit.side()), *resources::gameboard, false)) {
+				unit_itor->is_visible_to_team(resources::gameboard->get_team(new_unit.side()), false)) {
 				int dist = distance_between(unit_itor->get_location(),recruit_loc) - unit_itor->level();
 				if (dist < min_dist) {
 					min_dist = dist;
@@ -619,7 +622,9 @@ place_recruit_result place_recruit(unit_ptr u, const map_location &recruit_locat
 		u->set_movement(0, true);
 		u->set_attacks(0);
 	}
-	u->heal_fully();
+	if(!is_recall) {
+		u->heal_fully();
+	}
 	u->set_hidden(true);
 
 	// Get the leader location before adding the unit to the board.
@@ -627,11 +632,8 @@ place_recruit_result place_recruit(unit_ptr u, const map_location &recruit_locat
 			find_recruit_leader(u->side(), recruit_location, recruited_from);
 	u->set_location(recruit_location);
 
-	unit_map::unit_iterator new_unit_itor;
-	bool success = false;
-
 	// Add the unit to the board.
-	std::tie(new_unit_itor, success) = resources::gameboard->units().insert(u);
+	auto [new_unit_itor, success] = resources::gameboard->units().insert(u);
 	assert(success);
 
 	map_location current_loc = recruit_location;
@@ -659,7 +661,7 @@ place_recruit_result place_recruit(unit_ptr u, const map_location &recruit_locat
 			std::get<0>(res) |= std::get<0>(resources::game_events->pump().fire(event_name, current_loc, recruited_from));
 		}
 		if ( !validate_recruit_iterator(new_unit_itor, current_loc) )
-			return std::make_tuple(true, 0, false);
+			return std::tuple(true, 0, false);
 		new_unit_itor->set_hidden(true);
 	}
 	preferences::encountered_units().insert(new_unit_itor->type_id());
@@ -677,10 +679,10 @@ place_recruit_result place_recruit(unit_ptr u, const map_location &recruit_locat
 
 	// Village capturing.
 	if ( resources::gameboard->map().is_village(current_loc) ) {
-		std::get<1>(res) = resources::gameboard->village_owner(current_loc) + 1;
+		std::get<1>(res) = resources::gameboard->village_owner(current_loc);
 		std::get<0>(res) |= std::get<0>(actions::get_village(current_loc, new_unit_itor->side(), &std::get<2>(res)));
 		if ( !validate_recruit_iterator(new_unit_itor, current_loc) )
-			return std::make_tuple(true, 0, false);
+			return std::tuple(true, 0, false);
 	}
 
 	// Fog clearing.
@@ -704,10 +706,6 @@ place_recruit_result place_recruit(unit_ptr u, const map_location &recruit_locat
 	return res;
 }
 
-
-/**
- * Recruits a unit of the given type for the given side.
- */
 void recruit_unit(const unit_type & u_type, int side_num, const map_location & loc,
                   const map_location & from, bool show, bool use_undo)
 {
@@ -736,10 +734,6 @@ void recruit_unit(const unit_type & u_type, int side_num, const map_location & l
 		// Other updates were done by place_recruit().
 }
 
-
-/**
- * Recalls the unit with the indicated ID for the provided team.
- */
 bool recall_unit(const std::string & id, team & current_team,
                  const map_location & loc, const map_location & from,
                  map_location::DIRECTION facing, bool show, bool use_undo)
