@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -97,7 +98,7 @@ namespace {
 			// Regeneration?
 			for (const unit_ability & regen : patient.get_abilities("regenerate"))
 			{
-				curing = std::max(curing, poison_status((*regen.first)["poison"]));
+				curing = std::max(curing, poison_status((*regen.ability_cfg)["poison"]));
 				if ( curing == POISON_CURE )
 					// This is as good as it gets.
 					return POISON_CURE;
@@ -109,14 +110,14 @@ namespace {
 		// Assumed: curing is not POISON_CURE at the start of any iteration.
 		for (const unit_ability & heal : patient.get_abilities("heals"))
 		{
-			POISON_STATUS this_cure = poison_status((*heal.first)["poison"]);
+			POISON_STATUS this_cure = poison_status((*heal.ability_cfg)["poison"]);
 			if ( this_cure <= curing )
 				// We already recorded this level of curing.
 				continue;
 
 			// NOTE: At this point, this_cure will be *_SLOW or *_CURE.
 
-			unit_map::iterator cure_it = units.find(heal.second);
+			unit_map::iterator cure_it = units.find(heal.teacher_loc);
 			assert(cure_it != units.end());
 			const int cure_side = cure_it->side();
 
@@ -201,7 +202,7 @@ namespace {
 		// Remove all healers not on this side (since they do not heal now).
 		unit_ability_list::iterator heal_it = heal_list.begin();
 		while ( heal_it != heal_list.end() ) {
-			unit_map::iterator healer = units.find(heal_it->second);
+			unit_map::iterator healer = units.find(heal_it->teacher_loc);
 			assert(healer != units.end());
 
 			if ( healer->side() != side )
@@ -299,8 +300,6 @@ void calculate_healing(int side, bool update_display)
 	for (unit &patient : resources::gameboard->units()) {
 
 		if ( patient.get_state("unhealable") || patient.incapacitated() ) {
-			if ( patient.side() == side )
-				patient.set_resting(true);
 			continue;
 		}
 
@@ -315,7 +314,6 @@ void calculate_healing(int side, bool update_display)
 		if ( patient.side() == side ) {
 			if ( patient.resting() || patient.is_healthy() )
 				healing += game_config::rest_heal_amount;
-			patient.set_resting(true);
 		}
 
 		// Main healing.
@@ -346,10 +344,7 @@ void calculate_healing(int side, bool update_display)
 			DBG_NG << "Just before healing animations, unit has " << healers.size() << " potential healers.\n";
 		}
 
-		const team & viewing_team =
-			resources::gameboard->teams()[display::get_singleton()->viewing_team()];
-		if (!resources::controller->is_skipping_replay() && update_display &&
-		    patient.is_visible_to_team(viewing_team, *resources::gameboard, false) )
+		if (!resources::controller->is_skipping_replay() && update_display)
 		{
 			unit_list.emplace_front(patient, healers, healing, curing == POISON_CURE);
 		}
