@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -186,7 +187,7 @@ game_events::pump_result_t get_village(const map_location& loc, int side, bool *
 
 namespace { // Private helpers for move_unit()
 
-	/// Helper class for move_unit().
+	/** Helper class for move_unit(). */
 	class unit_mover {
 		typedef std::vector<map_location>::const_iterator route_iterator;
 
@@ -199,28 +200,30 @@ namespace { // Private helpers for move_unit()
 		           bool skip_sightings, bool skip_ally_sightings);
 		~unit_mover();
 
-		/// Determines how far along the route the unit can expect to move this turn.
+		/** Determines how far along the route the unit can expect to move this turn. */
 		bool check_expected_movement();
-		/// Attempts to move the unit along the expected path.
+		/** Attempts to move the unit along the expected path. */
 		void try_actual_movement(bool show);
-		/// Does some bookkeeping and event firing, for use after movement.
+		/** Does some bookkeeping and event firing, for use after movement. */
 		void post_move(undo_list *undo_stack);
-		/// Shows the various on-screen messages, for use after movement.
+		/** Shows the various on-screen messages, for use after movement. */
 		void feedback() const;
 
-		/// After checking expected movement, this is the expected path.
+		/** After checking expected movement, this is the expected path. */
 		std::vector<map_location> expected_path() const
 		{ return std::vector<map_location>(begin_, expected_end_); }
-		/// After moving, this is the final hex reached.
+		/** After moving, this is the final hex reached. */
 		const map_location & final_hex() const
 		{ return *move_loc_; }
-		/// The number of hexes actually entered.
-		size_t steps_travelled() const
+		/** The number of hexes actually entered. */
+		std::size_t steps_travelled() const
 		{ return move_loc_ - begin_; }
-		/// After moving, use this to detect if movement was less than expected.
+		/** After moving, use this to detect if movement was less than expected. */
 		bool stopped_early() const  { return expected_end_ != real_end_; }
-		/// After moving, use this to detect if something happened that would
-		/// interrupt movement (even if movement ended for a different reason).
+		/**
+		 * After moving, use this to detect if something happened that would
+		 * interrupt movement (even if movement ended for a different reason).
+		 */
 		bool interrupted(bool include_end_of_move_events=true) const
 		{
 			return ambushed_ || blocked() || sighted_ || teleport_failed_ ||
@@ -229,31 +232,31 @@ namespace { // Private helpers for move_unit()
 		}
 
 	private: // functions
-		/// Returns whether or not movement was blocked by a non-ambushing enemy.
+		/** Returns whether or not movement was blocked by a non-ambushing enemy. */
 		bool blocked() const { return blocked_loc_ != map_location::null_location(); }
-		/// Checks the expected route for hidden units.
+		/** Checks the expected route for hidden units. */
 		void cache_hidden_units(const route_iterator & start,
 		                        const route_iterator & stop);
-		/// Fires the enter_hex or exit_hex event and updates our data as needed.
+		/** Fires the enter_hex or exit_hex event and updates our data as needed. */
 		void fire_hex_event(const std::string & event_name,
 		                    const route_iterator & current,
 	                        const route_iterator & other);
-		/// AI moves are supposed to not change the "goto" order.
+		/** AI moves are supposed to not change the "goto" order. */
 		bool is_ai_move() const	{ return spectator_ != nullptr; }
-		/// Checks how far it appears we can move this turn.
+		/** Checks how far it appears we can move this turn. */
 		route_iterator plot_turn(const route_iterator & start,
 		                         const route_iterator & stop);
-		/// Updates our stored info after a WML event might have changed something.
+		/** Updates our stored info after a WML event might have changed something. */
 		void post_wml(game_events::pump_result_t pump_res, const route_iterator & step);
 		void post_wml(game_events::pump_result_t pump_res) { return post_wml(pump_res, full_end_); }
-		/// Fires the sighted events that were raised earlier.
+		/** Fires the sighted events that were raised earlier. */
 		void pump_sighted(const route_iterator & from);
-		/// Returns the ambush alert (if any) for the given unit.
+		/** Returns the ambush alert (if any) for the given unit. */
 		static std::string read_ambush_string(const unit & ambusher);
-		/// Reveals the unit at the indicated location.
+		/** Reveals the unit at the indicated location. */
 		void reveal_ambusher(const map_location & hex, bool update_alert=true);
 
-		/// Returns whether or not undoing this move should be blocked.
+		/** Returns whether or not undoing this move should be blocked. */
 		bool undo_blocked() const
 		{ return ambushed_ || blocked() || wml_removed_unit_ || wml_undo_disabled_ || fog_changed_ ||
 		         teleport_failed_; }
@@ -262,21 +265,21 @@ namespace { // Private helpers for move_unit()
 		// each is used in only one place. (They are separate functions to ease
 		// code reading.)
 
-		/// Checks for ambushers around @a hex, setting flags as appropriate.
+		/** Checks for ambushers around @a hex, setting flags as appropriate. */
 		inline void check_for_ambushers(const map_location & hex);
-		/// Makes sure the path is not obstructed by a unit.
+		/** Makes sure the path is not obstructed by a unit. */
 		inline bool check_for_obstructing_unit(const map_location & hex,
 		                                       const map_location & prev_hex);
-		/// Moves the unit the next step.
+		/** Moves the unit the next step. */
 		inline bool do_move(const route_iterator & step_from,
 		                    const route_iterator & step_to,
 		                    unit_display::unit_mover & animator);
-		/// Clears fog/shroud and handles units being sighted.
+		/** Clears fog/shroud and handles units being sighted. */
 		inline void handle_fog(const map_location & hex, bool new_animation);
 		inline bool is_reasonable_stop(const map_location & hex) const;
-		/// Reveals the units stored in ambushers_ (and blocked_loc_).
+		/** Reveals the units stored in ambushers_ (and blocked_loc_). */
 		inline void reveal_ambushers();
-		/// Makes sure the units in ambushers_ still exist.
+		/** Makes sure the units in ambushers_ still exist. */
 		inline void validate_ambushers();
 
 	private: // data
@@ -327,8 +330,8 @@ namespace { // Private helpers for move_unit()
 		bool sighted_;	// Records if sightings were made that could interrupt movement.
 		bool sighted_stop_;	// Records if sightings were made that did interrupt movement (the same as sighted_ unless movement ended for another reason).
 		bool teleport_failed_;
-		size_t enemy_count_;
-		size_t friend_count_;
+		std::size_t enemy_count_;
+		std::size_t friend_count_;
 		std::string ambush_string_;
 		std::vector<map_location> ambushers_;
 		std::deque<int> moves_left_;	// The front value is what the moving unit's remaining moves should be set to after the next step through the route.
@@ -337,11 +340,13 @@ namespace { // Private helpers for move_unit()
 	};
 
 
-	/// This constructor assumes @a route is not empty, and it will assert() that
-	/// there is a unit at route.front().
-	/// Iterators into @a route must remain valid for the life of this object.
-	/// It is assumed that move_spectator is only supplied for AI moves (only
-	/// affects whether or not gotos are changed).
+	/**
+	 * This constructor assumes @a route is not empty, and it will assert() that
+	 * there is a unit at route.front().
+	 * Iterators into @a route must remain valid for the life of this object.
+	 * It is assumed that move_spectator is only supplied for AI moves (only
+	 * affects whether or not gotos are changed).
+	 */
 	unit_mover::unit_mover(const std::vector<map_location> & route,
 	                       move_unit_spectator *move_spectator,
 	                       bool skip_sightings, bool skip_ally_sightings)
@@ -420,20 +425,17 @@ namespace { // Private helpers for move_unit()
 		const unit_map &units = resources::gameboard->units();
 
 		// Need to check each adjacent hex for hidden enemies.
-		adjacent_loc_array_t adjacent;
-		get_adjacent_tiles(hex, adjacent.data());
-		for (unsigned i = 0; i < adjacent.size(); ++i )
-		{
-			const unit_map::const_iterator neighbor_it = units.find(adjacent[i]);
+		for(const map_location& loc : get_adjacent_tiles(hex)) {
+			const unit_map::const_iterator neighbor_it = units.find(loc);
 
 			if ( neighbor_it != units.end()  &&
 			     current_team_->is_enemy(neighbor_it->side())  &&
-			     neighbor_it->invisible(adjacent[i], *resources::gameboard) )
+			     neighbor_it->invisible(loc) )
 			{
 				// Ambushed!
 				ambushed_ = true;
 				ambush_stop_ = hex;
-				ambushers_.push_back(adjacent[i]);
+				ambushers_.push_back(loc);
 			}
 		}
 	}
@@ -468,19 +470,9 @@ namespace { // Private helpers for move_unit()
 				// block the tunnel if pass_allied_units=true. Whether the teleport is possible
 				// is checked by getting the teleport map with the see_all flag set to true.
 				const pathfind::teleport_map teleports = pathfind::get_teleport_locations(*move_it_, *current_team_, true, false, false);
-				std::set<map_location> allowed_teleports;
-				teleports.get_adjacents(allowed_teleports, prev_hex);
+				const auto allowed_teleports = teleports.get_adjacents(prev_hex);
 
-				bool found_valid_teleport = false;
-				std::set<map_location>::const_iterator it = allowed_teleports.begin();
-				while( it!=allowed_teleports.end() && !found_valid_teleport ) {
-					if ( *it == hex ) {
-						found_valid_teleport = true;
-					}
-					++it;
-				}
-
-				if (!found_valid_teleport) {
+				if(allowed_teleports.count(hex) == 0) {
 					teleport_failed_ = true;
 					return true;
 				}
@@ -523,10 +515,7 @@ namespace { // Private helpers for move_unit()
 		move_it_->anim_comp().invalidate(disp);
 
 		// Attempt actually moving. Fails if *step_to is occupied.
-		unit_map::unit_iterator unit_it;
-		bool success = false;
-
-		std::tie(unit_it, success) = resources::gameboard->units().move(*move_loc_, *step_to);
+		auto [unit_it, success] = resources::gameboard->units().move(*move_loc_, *step_to);
 
 		if(success) {
 			// Update the moving unit.
@@ -632,7 +621,7 @@ namespace { // Private helpers for move_unit()
 		const unit_map &units = resources::gameboard->units();
 
 		// Loop through the previously-detected ambushers.
-		size_t i = 0;
+		std::size_t i = 0;
 		while ( i != ambushers_.size() ) {
 			if (units.count(ambushers_[i]) == 0) {
 				// Ambusher is gone.
@@ -751,7 +740,7 @@ namespace { // Private helpers for move_unit()
 
 		if ( start != begin_ ) {
 			// Check for being unable to leave the current hex.
-			if ( !move_it_->get_ability_bool("skirmisher", *start, *resources::gameboard) &&
+			if ( !move_it_->get_ability_bool("skirmisher", *start) &&
 			      pathfind::enemy_zoc(*current_team_, *start, *current_team_) )
 				zoc_stop_ = *start;
 		}
@@ -774,7 +763,7 @@ namespace { // Private helpers for move_unit()
 			moves_left_.push_back(remaining_moves);
 
 			// Check for being unable to leave this hex.
-			if (!move_it_->get_ability_bool("skirmisher", *end, *resources::gameboard) &&
+			if (!move_it_->get_ability_bool("skirmisher", *end) &&
 				pathfind::enemy_zoc(*current_team_, *end, *current_team_))
 			{
 				zoc_stop_ = *end;
@@ -850,7 +839,7 @@ namespace { // Private helpers for move_unit()
 	{
 		for(const unit_ability &hide : ambusher.get_abilities("hides"))
 		{
-			const std::string & ambush_string = (*hide.first)["alert"].str();
+			const std::string & ambush_string = (*hide.ability_cfg)["alert"].str();
 			if (!ambush_string.empty()) {
 				return ambush_string;
 			}
@@ -900,11 +889,7 @@ namespace { // Private helpers for move_unit()
 			// Make sure this hex is drawn correctly.
 			disp.invalidate(hex);
 			// Fire sighted events.
-
-			bool wml_undo_blocked = false;
-			bool wml_move_aborted = false;
-
-			std::tie(wml_undo_blocked, wml_move_aborted) = actor_sighted(*ambusher, &sight_cache);
+			auto [wml_undo_blocked, wml_move_aborted] = actor_sighted(*ambusher, &sight_cache);
 			// TODO: should we call post_wml ?
 			wml_move_aborted_ |= wml_move_aborted;
 			wml_undo_disabled_ |= wml_undo_blocked;
@@ -1009,11 +994,7 @@ namespace { // Private helpers for move_unit()
 				// Finish animating.
 				animator.finish(move_it_.get_shared_ptr());
 				// Check for the moving unit being seen.
-
-				bool wml_undo_blocked = false;
-				bool wml_move_aborted = false;
-
-				std::tie(wml_undo_blocked, wml_move_aborted) = actor_sighted(*move_it_, &not_seeing);
+				auto [wml_undo_blocked, wml_move_aborted] = actor_sighted(*move_it_, &not_seeing);
 				// TODO: should we call post_wml ?
 				wml_move_aborted_ |= wml_move_aborted;
 				wml_undo_disabled_ |= wml_undo_blocked;
@@ -1066,7 +1047,7 @@ namespace { // Private helpers for move_unit()
 			// Village capturing.
 			if ( resources::gameboard->map().is_village(final_loc) ) {
 				// Is this a capture?
-				orig_village_owner = resources::gameboard->village_owner(final_loc) + 1;
+				orig_village_owner = resources::gameboard->village_owner(final_loc);
 				if ( orig_village_owner != current_side_) {
 					// Captured. Zap movement and take over the village.
 					move_it_->set_movement(0, true);
@@ -1142,17 +1123,22 @@ namespace { // Private helpers for move_unit()
 			std::string message;
 			color_t msg_color;
 			if ( friend_count_ != 0  &&  enemy_count_ != 0 ) {
-				// Both friends and enemies sighted -- neutral message.
+				// TRANSLATORS: This becomes the "friendphrase" in "Units sighted! ($friendphrase, $enemyphrase)"
 				symbols["friendphrase"] = VNGETTEXT("Part of 'Units sighted! (...)' sentence^1 friendly", "$friends friendly", friend_count_, symbols);
+				// TRANSLATORS: This becomes the "enemyphrase" in "Units sighted! ($friendphrase, $enemyphrase)"
 				symbols["enemyphrase"] = VNGETTEXT("Part of 'Units sighted! (...)' sentence^1 enemy", "$enemies enemy", enemy_count_, symbols);
+				// TRANSLATORS: Both friends and enemies sighted -- neutral message.
+				// This is shown when a move is interrupted because units were revealed from the fog of war.
 				message = VGETTEXT("Units sighted! ($friendphrase, $enemyphrase)", symbols);
 				msg_color = font::NORMAL_COLOR;
 			} else if ( enemy_count_ != 0 ) {
-				// Only enemies sighted -- bad message.
+				// TRANSLATORS: Only enemies sighted -- bad message.
+				// This is shown when a move is interrupted because units were revealed from the fog of war.
 				message = VNGETTEXT("Enemy unit sighted!", "$enemies enemy units sighted!", enemy_count_, symbols);
 				msg_color = font::BAD_COLOR;
 			} else if ( friend_count_ != 0 ) {
-				// Only friends sighted -- good message.
+				// TRANSLATORS: Only friends sighted -- good message.
+				// This is shown when a move is interrupted because units were revealed from the fog of war.
 				message = VNGETTEXT("Friendly unit sighted", "$friends friendly units sighted", friend_count_, symbols);
 				msg_color = font::GOOD_COLOR;
 			}
@@ -1178,7 +1164,7 @@ namespace { // Private helpers for move_unit()
 }//end anonymous namespace
 
 
-static size_t move_unit_internal(undo_list* undo_stack,
+static std::size_t move_unit_internal(undo_list* undo_stack,
                  bool show_move,
                  bool* interrupted,
 				 unit_mover& mover)
@@ -1243,7 +1229,7 @@ static size_t move_unit_internal(undo_list* undo_stack,
  *          @a steps is not empty (the return value is guaranteed to be less
  *          than steps.size() ).
  */
-size_t move_unit_and_record(const std::vector<map_location> &steps,
+std::size_t move_unit_and_record(const std::vector<map_location> &steps,
                  undo_list* undo_stack,
                  bool continued_move, bool show_move,
                  bool* interrupted,
@@ -1276,7 +1262,7 @@ size_t move_unit_and_record(const std::vector<map_location> &steps,
 		*/
 		resources::recorder->add_synced_command("move",replay_helper::get_movement(steps, continued_move, skip_ally_sighted));
 		set_scontext_synced sync;
-		size_t r =  move_unit_internal(undo_stack, show_move, interrupted, mover);
+		std::size_t r =  move_unit_internal(undo_stack, show_move, interrupted, mover);
 		resources::controller->check_victory();
 		resources::controller->maybe_throw_return_to_play_side();
 		sync.do_final_checkup();
@@ -1289,7 +1275,7 @@ size_t move_unit_and_record(const std::vector<map_location> &steps,
 	}
 }
 
-size_t move_unit_from_replay(const std::vector<map_location> &steps,
+std::size_t move_unit_from_replay(const std::vector<map_location> &steps,
                  undo_list* undo_stack,
                  bool continued_move,bool skip_ally_sighted, bool show_move)
 {
