@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -99,18 +100,22 @@ private:
  */
 game_events::pump_result_t get_village(const map_location& loc, int side, bool *time_bonus = nullptr, bool fire_event = true);
 
-/// Moves a unit across the board.
-/// And enters the synced context.
-size_t move_unit_and_record(const std::vector<map_location> &steps,
+/**
+ * Moves a unit across the board.
+ * And enters the synced context.
+ */
+std::size_t move_unit_and_record(const std::vector<map_location> &steps,
                  undo_list* undo_stack,
                  bool continued_move = false,
 				 bool show_move = true,
                  bool* interrupted = nullptr,
                  move_unit_spectator* move_spectator = nullptr);
 
-/// Moves a unit across the board.
-/// to be called from replay when we are already in the synced context.
-size_t move_unit_from_replay(const std::vector<map_location> &steps,
+/**
+ * Moves a unit across the board.
+ * to be called from replay when we are already in the synced context.
+ */
+std::size_t move_unit_from_replay(const std::vector<map_location> &steps,
                  undo_list* undo_stack,
                  bool continued_move, bool skip_ally_sighted,
 				 bool show_move = true);
