@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -39,8 +40,7 @@ struct addon_info_translation
 		: supported(true)
 		, title()
 		, description()
-	{
-	}
+	{}
 
 	addon_info_translation(bool sup, std::string titl, std::string desc)
 		: supported(sup)
@@ -59,15 +59,7 @@ struct addon_info_translation
 
 	addon_info_translation(const addon_info_translation&) = default;
 
-	addon_info_translation& operator=(const addon_info_translation& o)
-	{
-		if(this != &o) {
-			this->supported = o.supported;
-			this->title = o.title;
-			this->description = o.description;
-		}
-		return *this;
-	}
+	addon_info_translation& operator=(const addon_info_translation& o) = default;
 
 	void read(const config& cfg);
 
@@ -87,7 +79,8 @@ struct addon_info
 
 	std::string icon;
 
-	version_info version;
+	version_info current_version;
+	std::set<version_info, std::greater<version_info>> versions;
 
 	std::string author;
 
@@ -107,13 +100,8 @@ struct addon_info
 
 	std::string feedback_url;
 
-	time_t updated;
-	time_t created;
-
-	// Artificial upload order index used to preserve add-ons upload order
-	// until we have actual first-upload timestamps implemented. This index
-	// is not serialized anywhere.
-	unsigned order;
+	std::time_t updated;
+	std::time_t created;
 
 	// Flag to indicate whether this object was generaled from pbl info for an addon
 	// not previously published.
@@ -123,59 +111,35 @@ struct addon_info
 
 	addon_info()
 		: id(), title(), description(), icon()
-		, version(), author(), size(), downloads()
-		, uploads(), type(), tags(), locales()
+		, current_version(), versions(), author(), size()
+		, downloads(), uploads(), type(), tags(), locales()
 		, core()
 		, depends()
 		, feedback_url()
 		, updated()
 		, created()
-		, order()
 		, local_only(false)
 		, info_translations()
 	{}
 
 	explicit addon_info(const config& cfg)
 		: id(), title(), description(), icon()
-		, version(), author(), size(), downloads()
-		, uploads(), type(), tags(), locales()
+		, current_version(), versions(), author(), size()
+		, downloads(), uploads(), type(), tags(), locales()
 		, core()
 		, depends()
 		, feedback_url()
 		, updated()
 		, created()
-		, order()
 		, local_only(false)
 		, info_translations()
 	{
 		this->read(cfg);
 	}
 
-	addon_info& operator=(const addon_info& o) {
-		if(this != &o) {
-			this->id = o.id;
-			this->title = o.title;
-			this->description = o.description;
-			this->icon = o.icon;
-			this->version = o.version;
-			this->author = o.author;
-			this->size = o.size;
-			this->downloads = o.downloads;
-			this->uploads = o.uploads;
-			this->type = o.type;
-			this->tags = o.tags;
-			this->locales = o.locales;
-			this->core = o.core;
-			this->depends = o.depends;
-			this->feedback_url = o.feedback_url;
-			this->updated = o.updated;
-			this->created = o.created;
-			this->order = o.order;
-			this->local_only = o.local_only;
-			this->info_translations = o.info_translations;
-		}
-		return *this;
-	}
+	addon_info(const addon_info&) = default;
+
+	addon_info& operator=(const addon_info& o) = default;
 
 	void read(const config& cfg);
 
