@@ -1,37 +1,25 @@
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
-                 2008 - 2015 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "addon/manager.hpp"
-#include "addon/manager_ui.hpp"
+
 #include "filesystem.hpp"
-#include "preferences/game.hpp"
-#include "gettext.hpp"
-#include "gui/dialogs/addon/connect.hpp"
-#include "gui/dialogs/addon/manager.hpp"
-#include "gui/dialogs/addon/uninstall_list.hpp"
-#include "gui/dialogs/message.hpp"
-#include "gui/dialogs/simple_item_selector.hpp"
-#include "gui/dialogs/transient_message.hpp"
-#include "gui/widgets/settings.hpp"
-#include "gui/widgets/retval.hpp"
 #include "log.hpp"
 #include "serialization/parser.hpp"
 #include "game_version.hpp"
-#include "serialization/string_utils.hpp"
-#include "addon/client.hpp"
-#include "game_config_manager.hpp"
 
 #include <boost/algorithm/string.hpp>
 
@@ -107,7 +95,14 @@ void get_addon_install_info(const std::string& addon_name, config& cfg)
 	const std::string& info_path = get_info_file_path(addon_name);
 	filesystem::scoped_istream stream = filesystem::istream_file(info_path);
 	try {
-		read(cfg, *stream);
+		// The parser's read() API would normally do this at the start. This
+		// is a safeguard in case read() throws later
+		cfg.clear();
+		config envelope;
+		read(envelope, *stream);
+		if(config& info = envelope.child("info")) {
+			cfg = std::move(info);
+		}
 	} catch(const config::error& e) {
 		ERR_CFG << "Failed to read add-on installation information for '"
 				<< addon_name << "' from " << info_path << ":\n"
@@ -115,6 +110,23 @@ void get_addon_install_info(const std::string& addon_name, config& cfg)
 	}
 }
 
+void write_addon_install_info(const std::string& addon_name, const config& cfg)
+{
+	LOG_CFG << "Writing version info for add-on '" << addon_name << "'\n";
+
+	const auto& info_path = get_info_file_path(addon_name);
+	auto out = filesystem::ostream_file(info_path);
+
+	*out << "#\n"
+		 << "# File automatically generated by Wesnoth to keep track\n"
+		 << "# of version information on installed add-ons. DO NOT EDIT!\n"
+		 << "#\n";
+
+	config envelope;
+	envelope.add_child("info", cfg);
+	write(*out, envelope);
+}
+
 bool remove_local_addon(const std::string& addon)
 {
 	const std::string addon_dir = filesystem::get_addons_dir() + "/" + addon;
@@ -129,36 +141,43 @@ bool remove_local_addon(const std::string& addon)
 	return true;
 }
 
-std::vector<std::string> available_addons()
+namespace {
+
+enum ADDON_ENUM_CRITERIA
+{
+	ADDON_ANY,
+	ADDON_HAS_PBL,
+};
+
+std::vector<std::string> enumerate_addons_internal(ADDON_ENUM_CRITERIA filter)
 {
 	std::vector<std::string> res;
-	std::vector<std::string> files, dirs;
-	const std::string parentd = filesystem::get_addons_dir();
-	filesystem::get_files_in_dir(parentd,&files,&dirs);
+	std::vector<std::string> addon_dirnames;
 
-	for(std::vector<std::string>::const_iterator i = dirs.begin(); i != dirs.end(); ++i) {
-		if (filesystem::file_exists(parentd + "/" + *i + "/_main.cfg") && have_addon_pbl_info(*i)) {
-			res.push_back(*i);
+	const auto& addons_root = filesystem::get_addons_dir();
+	filesystem::get_files_in_dir(addons_root, nullptr, &addon_dirnames);
+
+	for(const auto& addon_name : addon_dirnames) {
+		if(filesystem::file_exists(addons_root + "/" + addon_name + "/_main.cfg") &&
+		   (filter != ADDON_HAS_PBL || have_addon_pbl_info(addon_name)))
+		{
+			res.emplace_back(addon_name);
 		}
 	}
 
 	return res;
 }
 
+}
+
+std::vector<std::string> available_addons()
+{
+	return enumerate_addons_internal(ADDON_HAS_PBL);
+}
+
 std::vector<std::string> installed_addons()
 {
-	std::vector<std::string> res;
-	const std::string parentd = filesystem::get_addons_dir();
-	std::vector<std::string> files, dirs;
-	filesystem::get_files_in_dir(parentd,&files,&dirs);
-
-	for(std::vector<std::string>::const_iterator i = dirs.begin(); i != dirs.end(); ++i) {
-		if(filesystem::file_exists(parentd + "/" + *i + "/_main.cfg")) {
-			res.push_back(*i);
-		}
-	}
-
-	return res;
+	return enumerate_addons_internal(ADDON_ANY);
 }
 
 std::map<std::string, std::string> installed_addons_and_versions()
@@ -169,13 +188,13 @@ std::map<std::string, std::string> installed_addons_and_versions()
 		if(have_addon_pbl_info(addon_id)) {
 			try {
 				addons[addon_id] = get_addon_pbl_info(addon_id)["version"].str();
-			} catch(const invalid_pbl_exception& e) {
+			} catch(const invalid_pbl_exception&) {
 				addons[addon_id] = "Invalid pbl file, version unknown";
 			}
 		} else if(filesystem::file_exists(get_info_file_path(addon_id))) {
-			config temp;
-			get_addon_install_info(addon_id, temp);
-			addons[addon_id] = !temp.empty() && temp.has_child("info") ? temp.child("info")["version"].str() : "Unknown";
+			config info_cfg;
+			get_addon_install_info(addon_id, info_cfg);
+			addons[addon_id] = !info_cfg.empty() ? info_cfg["version"].str() : "Unknown";
 		} else {
 			addons[addon_id] = "Unknown";
 		}
@@ -220,7 +239,7 @@ static filesystem::blacklist_pattern_list read_ignore_patterns(const std::string
 	std::string line;
 	while (std::getline(*stream, line)) {
 		boost::trim(line);
-		const size_t l = line.size();
+		const std::size_t l = line.size();
 		// .gitignore & WML like comments
 		if (l == 0 || !line.compare(0,2,"# ")) continue;
 		if (line[l - 1] == '/') { // directory; we strip the last /
@@ -299,6 +318,37 @@ void unarchive_addon(const config& cfg)
 	unarchive_dir(parentd, cfg);
 }
 
+static void purge_dir(const std::string& path, const config& removelist)
+{
+	std::string dir;
+	if(removelist["name"].empty())
+		dir = path;
+	else
+		dir = path + '/' + removelist["name"].str();
+
+	if(!filesystem::is_directory(dir)) {
+		return;
+	}
+
+	for(const config& d : removelist.child_range("dir")) {
+		purge_dir(dir, d);
+	}
+
+	for(const config& f : removelist.child_range("file")) {
+		filesystem::delete_file(dir + '/' + f["name"].str());
+	}
+
+	if(filesystem::dir_size(dir) < 1) {
+		filesystem::delete_directory(dir);
+	}
+}
+
+void purge_addon(const config& removelist)
+{
+	const std::string parentd = filesystem::get_addons_dir();
+	purge_dir(parentd, removelist);
+}
+
 namespace {
 	std::map< std::string, version_info > version_info_cache;
 } // end unnamed namespace 5
@@ -319,18 +369,17 @@ void refresh_addon_version_info_cache()
 	std::transform(addons.begin(), addons.end(),
 	               addon_info_files.begin(), get_info_file_path);
 
-	for(size_t i = 0; i < addon_info_files.size(); ++i) {
+	for(std::size_t i = 0; i < addon_info_files.size(); ++i) {
 		assert(i < addons.size());
 
 		const std::string& addon = addons[i];
 		const std::string& info_file = addon_info_files[i];
 
 		if(filesystem::file_exists(info_file)) {
-			config cfg;
-			get_addon_install_info(addon, cfg);
+			config info_cfg;
+			get_addon_install_info(addon, info_cfg);
 
-			const config& info_cfg = cfg.child("info");
-			if(!info_cfg) {
+			if(info_cfg.empty()) {
 				continue;
 			}
 
