@@ -1,25 +1,37 @@
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
-                 2008 - 2015 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
+/**
+ * @file
+ * Local add-on content management interface.
+ *
+ * This API only concerns local functionality dealing with enumerating and
+ * manipulating installed add-ons, as well as extracting add-ons in WML pack
+ * form, usually but not necessarily obtained through the networked client.
+ *
+ * It also includes functions for the handling of add-on versioning
+ * information (_info.cfg) and publishing information (_server.pbl).
+ */
+
 #pragma once
 
 class config;
 class version_info;
 
 #include "addon/validation.hpp"
-#include "global.hpp"
 
 #include <string>
 #include <vector>
@@ -50,27 +62,34 @@ struct invalid_pbl_exception : public std::exception
 	/** Destructor.
 	 * Virtual to allow for subclassing.
 	 */
-	virtual ~invalid_pbl_exception() NOEXCEPT {}
+	virtual ~invalid_pbl_exception() noexcept {}
 
 	/** Returns a pointer to the (constant) error description.
 	 *  @return A pointer to a const char*. The underlying memory
 	 *          is in posession of the Exception object. Callers must
 	 *          not attempt to free the memory.
 	 */
-	virtual const char* what() const NOEXCEPT {
+	virtual const char* what() const noexcept {
 		return message.c_str();
 	}
 };
 
+/**
+ * Removes the specified add-on, deleting its full directory structure.
+ */
 bool remove_local_addon(const std::string& addon);
 
 /**
- * Returns true if there's a local .pbl file stored for the specified add-on.
+ * Returns whether a .pbl file is present for the specified add-on or not.
  */
 bool have_addon_pbl_info(const std::string& addon_name);
 
 /**
- * Returns true if the specified add-ons appear to be managed by a 'supported' VCS.
+ * Returns whether the specified add-on appears to be managed by a VCS or not.
+ *
+ * This is used by the add-ons client to prompt the user before overwriting
+ * add-ons that may currently be managed externally instead of through the
+ * built-in campaignd client.
  *
  * Currently supported VCSes are: Subversion, Git, Mercurial.
  */
@@ -87,7 +106,7 @@ bool have_addon_in_vcs_tree(const std::string& addon_name);
 config get_addon_pbl_info(const std::string& addon_name);
 
 /**
- * Sets the publish information for an add-on
+ * Sets the publish information for an add-on.
  *
  * @param addon_name              The add-on's main directory/file name.
  * @param cfg                     A config object from which the add-on's
@@ -109,6 +128,15 @@ bool have_addon_install_info(const std::string& addon_name);
  */
 void get_addon_install_info(const std::string& addon_name, class config& cfg);
 
+/**
+ * Writes the installation info (_info.cfg) for an add-on to disk.
+ *
+ * @param addon_name              The add-on's main directory/file name.
+ * @param cfg                     A config object containing the add-on's
+ *                                properties.
+ */
+void write_addon_install_info(const std::string& addon_name, const class config& cfg);
+
 /** Returns a list of local add-ons that can be published. */
 std::vector<std::string> available_addons();
 
@@ -127,6 +155,9 @@ void archive_addon(const std::string& addon_name, class config& cfg);
 /** Unarchives an add-on from campaignd's retrieved config object. */
 void unarchive_addon(const class config& cfg);
 
+/** Removes the listed files from the addon. */
+void purge_addon(const config& removelist);
+
 /** Refreshes the per-session cache of add-on's version information structs. */
 void refresh_addon_version_info_cache();
 
