@@ -1,16 +1,16 @@
-
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -29,7 +29,7 @@
 #include "actions/attack.hpp"
 #include "log.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 namespace ai {
 
@@ -70,16 +70,16 @@ void ai_composite::on_create()
 	}
 
 	std::function<void(std::vector<engine_ptr>&, const config&)> factory_engines =
-		std::bind(&ai::ai_composite::create_engine,*this,_1,_2);
+		std::bind(&ai::ai_composite::create_engine, *this, std::placeholders::_1, std::placeholders::_2);
 
 	std::function<void(std::vector<goal_ptr>&, const config&)> factory_goals =
-		std::bind(&ai::ai_composite::create_goal,*this,_1,_2);
+		std::bind(&ai::ai_composite::create_goal, *this, std::placeholders::_1, std::placeholders::_2);
 
 	std::function<void(std::vector<stage_ptr>&, const config&)> factory_stages =
-		std::bind(&ai::ai_composite::create_stage,*this,_1,_2);
+		std::bind(&ai::ai_composite::create_stage, *this, std::placeholders::_1, std::placeholders::_2);
 
 	std::function<void(std::map<std::string,aspect_ptr>&, const config&, std::string)> factory_aspects =
-		std::bind(&ai::ai_composite::replace_aspect,*this,_1,_2,_3);
+		std::bind(&ai::ai_composite::replace_aspect,*this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3);
 
 	register_vector_property(property_handlers(),"engine",get_engines(), factory_engines);
 	register_vector_property(property_handlers(),"goal",get_goals(), factory_goals);
@@ -88,25 +88,21 @@ void ai_composite::on_create()
 
 }
 
-
 void ai_composite::create_stage(std::vector<stage_ptr> &stages, const config &cfg)
 {
 	engine::parse_stage_from_config(*this,cfg,std::back_inserter(stages));
 }
 
-
 void ai_composite::create_goal(std::vector<goal_ptr> &goals, const config &cfg)
 {
 	engine::parse_goal_from_config(*this,cfg,std::back_inserter(goals));
 }
 
-
 void ai_composite::create_engine(std::vector<engine_ptr> &engines, const config &cfg)
 {
 	engine::parse_engine_from_config(*this,cfg,std::back_inserter(engines));
 }
 
-
 void ai_composite::replace_aspect(std::map<std::string,aspect_ptr> &aspects, const config &cfg, std::string id)
 {
 	std::vector<aspect_ptr> temp_aspects;
@@ -118,7 +114,6 @@ ai_composite::~ai_composite()
 {
 }
 
-
 bool ai_composite::add_stage(const config &cfg)
 {
 	std::vector< stage_ptr > stages;
@@ -131,7 +126,6 @@ bool ai_composite::add_stage(const config &cfg)
 	return (j>0);
 }
 
-
 bool ai_composite::add_goal(const config &cfg)
 {
 	std::vector< goal_ptr > goals;
@@ -144,37 +138,32 @@ bool ai_composite::add_goal(const config &cfg)
 	return (j>0);
 }
 
-
 void ai_composite::play_turn(){
 	for (stage_ptr &s : stages_) {
 		s->play_stage();
 	}
 }
 
-
 std::string ai_composite::get_id() const
 {
 	return cfg_["id"];
 }
 
 
-
 std::string ai_composite::get_name() const
 {
 	return cfg_["name"];
 }
 
-
 std::string ai_composite::get_engine() const
 {
 	return cfg_["engine"];
 }
 
-
 std::string ai_composite::evaluate(const std::string& str)
 {
 	config cfg;
-	cfg["engine"] = "fai";///@todo 1.9 : consider allowing other engines to evaluate
+	cfg["engine"] = "fai";
 	engine_ptr e_ptr = get_engine_by_cfg(cfg);
 	if (!e_ptr) {
 		// This should be unreachable, but not entirely sure...
@@ -183,10 +172,8 @@ std::string ai_composite::evaluate(const std::string& str)
 	return e_ptr->evaluate(str);
 }
 
-
 void ai_composite::new_turn()
 {
-	///@todo 1.9 replace with event system
 	recalculate_move_maps();
 	invalidate_defensive_position_cache();
 	invalidate_keeps_cache();
@@ -194,7 +181,6 @@ void ai_composite::new_turn()
 	unit_stats_cache().clear();
 }
 
-
 int ai_composite::get_recursion_count() const
 {
 	return recursion_counter_.get_count();
@@ -210,7 +196,6 @@ ai_context& ai_composite::get_ai_context()
 	return *this;
 }
 
-
 config ai_composite::to_config() const
 {
 	config cfg;
