@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -34,7 +35,6 @@ aspect::aspect(readonly_context &context, const config &cfg, const std::string &
 	invalidate_on_turn_start_(cfg["invalidate_on_turn_start"].to_bool(true)),
 	invalidate_on_tod_change_(cfg["invalidate_on_tod_change"].to_bool(true)),
 	invalidate_on_gamestate_change_(cfg["invalidate_on_gamestate_change"].to_bool()),
-	invalidate_on_minor_gamestate_change_(cfg["invalidate_on_minor_gamestate_change"].to_bool()),
 	engine_(cfg["engine"]), name_(cfg["name"]), id_(id)
 	{
 		DBG_AI_ASPECT << "creating new aspect: engine=["<<engine_<<"], name=["<<name_<<"], id=["<<id_<<"]"<< std::endl;
@@ -43,7 +43,6 @@ aspect::aspect(readonly_context &context, const config &cfg, const std::string &
 		DBG_AI_ASPECT << "aspect has time_of_day=["<<time_of_day_<<"], turns=["<<turns_<<"]" << std::endl;
 	}
 
-
 aspect::~aspect()
 	{
 		manager& manager = manager::get_singleton();
@@ -56,10 +55,6 @@ aspect::~aspect()
 		if (invalidate_on_gamestate_change_) {
 			manager.remove_gamestate_observer(this);
 		}
-		if (invalidate_on_minor_gamestate_change_) {
-			///@todo 1.9 add minor_gamestate_change_observer
-			//manager::remove_minor_gamestate_observer(this);
-		}
 	}
 
 lg::log_domain& aspect::log()
@@ -84,10 +79,6 @@ bool aspect::redeploy(const config &cfg, const std::string& /*id*/)
 	if (invalidate_on_gamestate_change_) {
 		manager.remove_gamestate_observer(this);
 	}
-	if (invalidate_on_minor_gamestate_change_) {
-		///@todo 1.9 add minor_gamestate_change_observer
-		//manager::remove_minor_gamestate_observer(this);
-	}
 
 	valid_ = false;
 	valid_variant_ =false;
@@ -96,7 +87,6 @@ bool aspect::redeploy(const config &cfg, const std::string& /*id*/)
 	invalidate_on_turn_start_ = cfg["invalidate_on_turn_start"].to_bool(true);
 	invalidate_on_tod_change_ = cfg["invalidate_on_tod_change"].to_bool(true);
 	invalidate_on_gamestate_change_ = cfg["invalidate_on_gamestate_change"].to_bool();
-	invalidate_on_minor_gamestate_change_ = cfg["invalidate_on_minor_gamestate_change"].to_bool();
 	engine_ = cfg["engine"].str();
 	name_ = cfg["name"].str();
 	id_ = cfg["id"].str();
@@ -110,10 +100,6 @@ bool aspect::redeploy(const config &cfg, const std::string& /*id*/)
 	if (invalidate_on_gamestate_change_) {
 		manager.add_gamestate_observer(this);
 	}
-	if (invalidate_on_minor_gamestate_change_) {
-		///@todo 1.9 add minor_gamestate_change_observer
-		//manager::add_minor_gamestate_observer(this);
-	}
 	return true;
 }
 
@@ -123,7 +109,6 @@ config aspect::to_config() const
 	cfg["invalidate_on_turn_start"] = invalidate_on_turn_start_;
 	cfg["invalidate_on_tod_change"] = invalidate_on_tod_change_;
 	cfg["invalidate_on_gamestate_change"] = invalidate_on_gamestate_change_;
-	cfg["invalidate_on_minor_gamestate_change"] = invalidate_on_minor_gamestate_change_;
 	if (!time_of_day_.empty()) {
 		cfg["time_of_day"] = time_of_day_;
 	}
