@@ -1,16 +1,16 @@
-
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -50,8 +50,6 @@ goal::goal(readonly_context &context, const config &cfg)
 	init_readonly_context_proxy(context);
 }
 
-
-
 void goal::on_create()
 {
 	LOG_AI_GOAL << "side " << get_side() << " : " << " created goal with name=[" << cfg_["name"] << "]" << std::endl;
@@ -68,18 +66,14 @@ void goal::unrecognized()
 	ok_ = false;
 }
 
-
 goal::~goal()
 {
 }
 
-
 void goal::add_targets(std::back_insert_iterator< std::vector< target >> /*target_list*/)
 {
 }
 
-
-
 config goal::to_config() const
 {
 	return cfg_;
@@ -100,7 +94,6 @@ std::string goal::get_engine() const
 	return cfg_["engine"];
 }
 
-
 bool goal::redeploy(const config &cfg)
 {
 	cfg_ = cfg;
@@ -108,19 +101,16 @@ bool goal::redeploy(const config &cfg)
 	return true;
 }
 
-
 bool goal::ok() const
 {
 	return ok_;
 }
 
-
 bool goal::active() const
 {
 	return is_active(cfg_["time_of_day"],cfg_["turns"]);
 }
 
-
 void target_unit_goal::on_create()
 {
 	goal::on_create();
@@ -152,17 +142,14 @@ void target_unit_goal::add_targets(std::back_insert_iterator< std::vector< targe
 		}
 	}
 
-
 }
 
-
 target_unit_goal::target_unit_goal(readonly_context &context, const config &cfg)
 	: goal(context,cfg)
 	, value_(0.0)
 {
 }
 
-
 void target_location_goal::on_create()
 {
 	goal::on_create();
@@ -176,7 +163,7 @@ void target_location_goal::on_create()
 	}
 	const config &criteria = cfg_.child("criteria");
 	if (criteria) {
-		filter_ptr_.reset(new terrain_filter(vconfig(criteria),resources::filter_con));
+		filter_ptr_.reset(new terrain_filter(vconfig(criteria),resources::filter_con, false));
 	}
 }
 
@@ -205,8 +192,6 @@ target_location_goal::target_location_goal(readonly_context &context, const conf
 {
 }
 
-
-
 void protect_goal::on_create()
 {
 	goal::on_create();
@@ -227,13 +212,11 @@ void protect_goal::on_create()
 	}
 	const config &criteria = cfg_.child("criteria");
 	if (criteria) {
-		filter_ptr_.reset(new terrain_filter(vconfig(criteria),resources::filter_con));
+		filter_ptr_.reset(new terrain_filter(vconfig(criteria), resources::filter_con, false));
 	}
 
-
 }
 
-
 void protect_goal::add_targets(std::back_insert_iterator< std::vector< target >> target_list)
 {
 	std::string goal_type;
@@ -263,8 +246,12 @@ void protect_goal::add_targets(std::back_insert_iterator< std::vector< target >>
 		const unit_filter ufilt{ vconfig(criteria) };
 		for (const unit &u : units)
 		{
-			//TODO: we will protect hidden units, by not testing for invisibility to current side
-			if (ufilt(u)) {
+			// 'protect_unit' can be set to any unit of any side -> exclude hidden units
+			// unless they are visible to the AI side (e.g. allies with shared vision).
+			// As is done in other parts of the AI, units under fog/shroud count as visible to the AI.
+			if (ufilt(u)
+				&& (!u.invisible(u.get_location()) || u.is_visible_to_team(current_team(), false)))
+			{
 				DBG_AI_GOAL << "side " << get_side() << ": in " << goal_type << ": " << u.get_location() << " should be protected\n";
 				items.insert(u.get_location());
 			}
@@ -280,20 +267,18 @@ void protect_goal::add_targets(std::back_insert_iterator< std::vector< target >>
 		{
 			int distance = distance_between(u.get_location(), loc);
 			if (current_team().is_enemy(u.side()) && distance < radius_ &&
-			    !u.invisible(u.get_location(), *resources::gameboard))
+			    !u.invisible(u.get_location()))
 			{
 				DBG_AI_GOAL << "side " << get_side() << ": in " << goal_type << ": found threat target. " << u.get_location() << " is a threat to "<< loc << '\n';
 				*target_list = target(u.get_location(),
-					value_ * double(radius_ - distance) /
+					value_ * static_cast<double>(radius_ - distance) /
 					radius_, target::TYPE::THREAT);
 			}
 		}
 	}
 
-
 }
 
-
 protect_goal::protect_goal(readonly_context &context, const config &cfg, bool protect_unit)
 	: goal(context,cfg)
 	, filter_ptr_()
@@ -326,7 +311,8 @@ void lua_goal::add_targets(std::back_insert_iterator< std::vector< target >> tar
 {
 	std::shared_ptr<lua_object<std::vector<target>>> l_obj = std::make_shared<lua_object<std::vector<target>>>();
 	config c(cfg_.child_or_empty("args"));
-	handler_->handle(c, true, l_obj);
+	const config empty_cfg;
+	handler_->handle(c, empty_cfg, true, l_obj);
 	try {
 		std::vector < target > targets = *(l_obj->get());
 
@@ -340,7 +326,6 @@ void lua_goal::add_targets(std::back_insert_iterator< std::vector< target >> tar
 
 }
 
-
 // This is defined in the source file so that it can easily access the logger
 bool goal_factory::is_duplicate(const std::string& name)
 {
@@ -351,5 +336,4 @@ bool goal_factory::is_duplicate(const std::string& name)
 	return false;
 }
 
-
 } //end of namespace ai
