@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -19,7 +20,7 @@
 
 #pragma once
 
-#include "utils/functional.hpp"
+#include <functional>
 #include "config.hpp"
 #include "ai/composite/component.hpp"
 
@@ -53,7 +54,6 @@ private:
 	path_element element_;
 };
 
-
 class component;
 
 class base_property_handler {
@@ -79,7 +79,6 @@ public:
 	vector_property_handler(const std::string &property, ptr_vector &values, std::function<void(ptr_vector&, const config&)> &construction_factory)
 		: factory_(construction_factory), property_(property), values_(values){}
 
-
         component* handle_get(const path_element &child)
 	{
 			typename ptr_vector::iterator i = std::find_if(values_.begin(),values_.end(),path_element_matches<ptr>(child));
@@ -130,7 +129,6 @@ public:
 		return false;
 	}
 
-
 	std::vector<component*> handle_get_children()
 	{
 		std::vector<component*> children;
@@ -167,8 +165,6 @@ private:
 
 };
 
-
-
 template<typename T>
 class facets_property_handler : public vector_property_handler<T> {
 	typedef typename vector_property_handler<T>::ptr ptr;
@@ -213,8 +209,6 @@ private:
 	ptr& default_;
 };
 
-
-
 template<typename T>
 class aspect_property_handler : public base_property_handler {
 public:
@@ -226,7 +220,6 @@ public:
 	{
 	}
 
-
         component* handle_get(const path_element &child)
 	{
 		typename aspect_map::const_iterator a = aspects_.find(child.id);
@@ -267,7 +260,6 @@ public:
 		return false;
 	}
 
-
 	std::vector<component*> handle_get_children()
 	{
 		std::vector<component*> children;
@@ -285,7 +277,6 @@ private:
 
 };
 
-
 template<typename X>
 static inline void register_vector_property(property_handler_map& property_handlers, const std::string& property,
 	std::vector<std::shared_ptr<X>>& values, std::function<void(std::vector<std::shared_ptr<X>>&, const config&)> construction_factory)
