@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -39,6 +40,11 @@ candidate_action::candidate_action(rca_context &context, const config &cfg):
 	max_score_(cfg["max_score"].to_double(HIGH_SCORE)),
 	id_(cfg["id"]), name_(cfg["name"]), type_(cfg["type"]), to_be_removed_(false)
 {
+	if (const config &filter_own = cfg.child("filter_own")) {
+		vconfig vcfg(filter_own);
+		vcfg.make_safe();
+		filter_own_.reset(new unit_filter(vcfg));
+	}
 	init_rca_context_proxy(context);
 }
 
@@ -46,13 +52,11 @@ candidate_action::~candidate_action()
 {
 }
 
-
 bool candidate_action::is_enabled() const
 {
 	return enabled_;
 }
 
-
 void candidate_action::enable()
 {
 	enabled_ = true;
@@ -68,18 +72,29 @@ void candidate_action::disable()
 	enabled_ = false;
 }
 
-
 double candidate_action::get_score() const
 {
 	return score_;
 }
 
-
 double candidate_action::get_max_score() const
 {
 	return max_score_;
 }
 
+std::shared_ptr<unit_filter> candidate_action::get_filter_own() const
+{
+	return filter_own_;
+}
+
+bool candidate_action::is_allowed_unit(const unit& u) const
+{
+	if (filter_own_) {
+		return (*filter_own_)(u);
+	}
+	return true;
+}
+
 const std::string& candidate_action::get_type() const
 {
 	return type_;
@@ -94,6 +109,9 @@ config candidate_action::to_config() const
 	cfg["name"] = name_;
 	cfg["score"] = score_;
 	cfg["max_score"] = max_score_;
+	if (filter_own_ && !filter_own_->empty()) {
+		cfg.add_child("filter_own", filter_own_->to_config());
+	}
 	cfg["type"] = type_;
 	return cfg;
 }
