@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -21,6 +22,7 @@
 
 #include "ai/composite/component.hpp"
 #include "ai/composite/contexts.hpp"
+#include "units/filter.hpp"
 
 //============================================================================
 namespace ai {
@@ -40,7 +42,6 @@ public:
 	 */
 	virtual ~candidate_action();
 
-
 	/**
 	 * Evaluate the candidate action, resetting the internal state of the action
 	 * @return the score
@@ -74,12 +75,21 @@ public:
 	 */
 	double get_score() const;
 
-
 	/**
 	 * Get the upper bound of the score of the candidate action without re-evaluation
 	 */
 	double get_max_score() const;
 
+	/**
+	 * Get the unit filter for allowed units for this candidate action
+	 */
+	std::shared_ptr<unit_filter> get_filter_own() const;
+
+	/**
+	 * Flag indicating whether unit may be used by this candidate action
+	 */
+	bool is_allowed_unit(const unit& u) const;
+
 	/**
 	 * Get the name of the candidate action (useful for debug purposes)
 	 */
@@ -99,7 +109,6 @@ public:
 
 	int get_recursion_count() const;
 
-
 	/**
 	 * serialize
 	 */
@@ -115,25 +124,20 @@ private:
 
 	bool enabled_;
 
-
 	std::string engine_;
 
-
 	double score_;
 
-
 	double max_score_;
 
+	std::shared_ptr<unit_filter> filter_own_;
 
 	std::string id_;
 
-
 	std::string name_;
 
-
 	std::string type_;
 
-
 	bool to_be_removed_;
 
 };
@@ -171,7 +175,6 @@ public:
 	virtual ~candidate_action_factory() {}
 };
 
-
 template<class CANDIDATE_ACTION>
 class register_candidate_action_factory : public candidate_action_factory {
 public:
