@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -20,13 +21,13 @@
 
 #include "ai/composite/engine.hpp"
 #include "ai/composite/stage.hpp"
-
-#include "ai/manager.hpp"
-#include "terrain/filter.hpp"
-#include "lexical_cast.hpp"
-#include "serialization/string_utils.hpp"
-#include "resources.hpp"
 #include "ai/lua/aspect_advancements.hpp"
+#include "ai/manager.hpp"
+#include "lexical_cast.hpp"
+#include "resources.hpp"
+#include "serialization/string_utils.hpp"
+#include "terrain/filter.hpp"
+#include "utils/variant.hpp"
 
 namespace ai {
 
@@ -57,6 +58,33 @@ public:
 	}
 };
 
+template<>
+class config_value_translator<std::string> {
+public:
+
+	static std::string cfg_to_value(const config &cfg)
+	{
+		return cfg["value"].str();
+	}
+
+	static void cfg_to_value(const config &cfg, std::string &value)
+	{
+		value = cfg_to_value(cfg);
+	}
+
+	static void value_to_cfg(const std::string &value, config &cfg)
+	{
+		cfg["value"] = value;
+	}
+
+	static config value_to_cfg(const std::string &value)
+	{
+		config cfg;
+		value_to_cfg(value,cfg);
+		return cfg;
+	}
+
+};
 
 template<>
 class config_value_translator<bool> {
@@ -83,7 +111,52 @@ public:
 		value_to_cfg(value,cfg);
 		return cfg;
 	}
+};
 
+class leader_aspects_visitor
+#ifdef USING_BOOST_VARIANT
+	: public boost::static_visitor<std::string>
+#endif
+{
+public:
+	std::string operator()(const bool b) const {
+		if (b) {
+			return "yes";
+		} else {
+			return "no";
+		}
+	}
+	std::string operator()(const std::vector<std::string> s) const { return utils::join(s); }
+};
+
+template<>
+class config_value_translator<utils::variant<bool, std::vector<std::string>>> {
+public:
+
+	static utils::variant<bool, std::vector<std::string>> cfg_to_value(const config &cfg)
+	{
+		if (cfg["value"].to_bool(true) == cfg["value"].to_bool(false)) {
+			return cfg["value"].to_bool();
+		}
+		return utils::split(cfg["value"]);
+	}
+
+	static void cfg_to_value(const config &cfg, utils::variant<bool, std::vector<std::string>> &value)
+	{
+		value = cfg_to_value(cfg);
+	}
+
+	static void value_to_cfg(const utils::variant<bool, std::vector<std::string>> &value, config &cfg)
+	{
+		cfg["value"] = utils::visit(leader_aspects_visitor(), value);
+	}
+
+	static config value_to_cfg(const utils::variant<bool, std::vector<std::string>> &value)
+	{
+		config cfg;
+		value_to_cfg(value,cfg);
+		return cfg;
+	}
 };
 
 template<>
@@ -151,10 +224,10 @@ public:
 	static terrain_filter cfg_to_value(const config &cfg)
 	{
 		if (const config &v = cfg.child("value")) {
-			return terrain_filter(vconfig(v), resources::filter_con);
+			return terrain_filter(vconfig(v), resources::filter_con, false);
 		}
 		static config c("not");
-		return terrain_filter(vconfig(c),resources::filter_con);
+		return terrain_filter(vconfig(c),resources::filter_con, false);
 	}
 
 	static void cfg_to_value(const config &cfg, terrain_filter &value)
@@ -203,8 +276,6 @@ public:
 	}
 };
 
-
-
 // variant value translator
 
 template<typename T>
@@ -265,7 +336,6 @@ public:
 	}
 };
 
-
 template<>
 class variant_value_translator<bool> {
 public:
@@ -295,8 +365,6 @@ public:
 	}
 };
 
-
-
 template<>
 class variant_value_translator<std::string> {
 public:
@@ -326,8 +394,6 @@ public:
 	}
 };
 
-
-
 template<>
 class variant_value_translator<attacks_vector> {
 public:
@@ -361,7 +427,6 @@ public:
 	}
 };
 
-
 template<>
 class variant_value_translator<terrain_filter> {
 public:
@@ -386,7 +451,7 @@ public:
 	static terrain_filter variant_to_value(const wfl::variant &var)
 	{
 		static config c("not");
-		terrain_filter value(vconfig(c),resources::filter_con);
+		terrain_filter value(vconfig(c), resources::filter_con, false);
 		variant_to_value(var,value);
 		return value;
 	}
