@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -21,6 +22,7 @@
 
 #include "config.hpp"
 #include "ai/game_info.hpp"
+#include "game_config_view.hpp"
 
 namespace ai {
 
@@ -64,7 +66,7 @@ public:
 	 * Init the parameters of ai configuration parser
 	 * @param game_config game config
 	 */
-	static void init(const config &game_config);
+	static void init(const game_config_view &game_config);
 	static void add_era_ai_from_config(const config &game_config);
 	static void add_mod_ai_from_config(config::const_child_itors configs);
 
@@ -101,6 +103,7 @@ public:
 
 
 	/**
+	 * @param[in] side the side to be parsed
 	 * @param[in] original_cfg the config to be read
 	 * @param[out] cfg parsed config
 	 * @return was all ok?
