@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -27,7 +28,7 @@
 #include "ai/composite/engine.hpp"         // for engine, engine_factory, etc
 #include "ai/composite/goal.hpp"           // for goal
 #include "ai/composite/stage.hpp"       // for ministage
-#include "ai/game_info.hpp"             // for aspect_type<>::typesafe_ptr, etc
+#include "ai/game_info.hpp"             // for typesafe_aspect_ptr, etc
 #include "ai/lua/aspect_advancements.hpp"
 #include "ai/manager.hpp"                  // for manager
 
@@ -40,7 +41,6 @@
 #include "log.hpp"                   // for LOG_STREAM, logger, etc
 #include "map/map.hpp"                   // for gamemap
 #include "pathfind/pathfind.hpp"        // for paths::dest_vect, paths, etc
-#include "recall_list_manager.hpp"   // for recall_list_manager
 #include "resources.hpp"             // for units, gameboard, etc
 #include "serialization/string_utils.hpp"  // for split, etc
 #include "team.hpp"                     // for team
@@ -77,31 +77,26 @@ int side_context_impl::get_recursion_count() const
 	return recursion_counter_.get_count();
 }
 
-
 int readonly_context_impl::get_recursion_count() const
 {
 	return recursion_counter_.get_count();
 }
 
-
 int readwrite_context_impl::get_recursion_count() const
 {
 	return recursion_counter_.get_count();
 }
 
-
 void readonly_context_impl::raise_user_interact() const
 {
 	manager::get_singleton().raise_user_interact();
 }
 
-
 void readwrite_context_impl::raise_gamestate_changed() const
 {
 	manager::get_singleton().raise_gamestate_changed();
 }
 
-
 team& readwrite_context_impl::current_team_w()
 {
 	return resources::gameboard->get_team(get_side());
@@ -110,152 +105,138 @@ team& readwrite_context_impl::current_team_w()
 attack_result_ptr readwrite_context_impl::execute_attack_action(const map_location& attacker_loc, const map_location& defender_loc, int attacker_weapon){
 	unit_map::iterator i = resources::gameboard->units().find(attacker_loc);
 	double m_aggression = i.valid() && i->can_recruit() ? get_leader_aggression() : get_aggression();
-	const unit_advancements_aspect& m_advancements = get_advancements();
-	return actions::execute_attack_action(get_side(),true,attacker_loc,defender_loc,attacker_weapon, m_aggression, m_advancements);
+	return actions::execute_attack_action(get_side(),true,attacker_loc,defender_loc,attacker_weapon, m_aggression);
 }
 
-
 attack_result_ptr readonly_context_impl::check_attack_action(const map_location& attacker_loc, const map_location& defender_loc, int attacker_weapon){
 	unit_map::iterator i = resources::gameboard->units().find(attacker_loc);
 	double m_aggression = i.valid() && i->can_recruit() ? get_leader_aggression() : get_aggression();
-	const unit_advancements_aspect& m_advancements = get_advancements();
-	return actions::execute_attack_action(get_side(),false,attacker_loc,defender_loc,attacker_weapon, m_aggression, m_advancements);
+	return actions::execute_attack_action(get_side(),false,attacker_loc,defender_loc,attacker_weapon, m_aggression);
 }
 
-
 move_result_ptr readwrite_context_impl::execute_move_action(const map_location& from, const map_location& to, bool remove_movement, bool unreach_is_ok){
 	return actions::execute_move_action(get_side(),true,from,to,remove_movement,unreach_is_ok);
 }
 
-
 move_result_ptr readonly_context_impl::check_move_action(const map_location& from, const map_location& to, bool remove_movement, bool unreach_is_ok){
 	return actions::execute_move_action(get_side(),false,from,to,remove_movement,unreach_is_ok);
 }
 
-
 recall_result_ptr readwrite_context_impl::execute_recall_action(const std::string& id, const map_location &where, const map_location &from){
 	return actions::execute_recall_action(get_side(),true,id,where,from);
 }
 
-
 recruit_result_ptr readwrite_context_impl::execute_recruit_action(const std::string& unit_name, const map_location &where, const map_location &from){
 	return actions::execute_recruit_action(get_side(),true,unit_name,where,from);
 }
 
-
 recall_result_ptr readonly_context_impl::check_recall_action(const std::string& id, const map_location &where, const map_location &from){
 	return actions::execute_recall_action(get_side(),false,id,where,from);
 }
 
-
 recruit_result_ptr readonly_context_impl::check_recruit_action(const std::string& unit_name, const map_location &where, const map_location &from){
 	return actions::execute_recruit_action(get_side(),false,unit_name,where,from);
 }
 
-
 stopunit_result_ptr readwrite_context_impl::execute_stopunit_action(const map_location& unit_location, bool remove_movement, bool remove_attacks){
 	return actions::execute_stopunit_action(get_side(),true,unit_location,remove_movement,remove_attacks);
 }
 
-
 stopunit_result_ptr readonly_context_impl::check_stopunit_action(const map_location& unit_location, bool remove_movement, bool remove_attacks){
 	return actions::execute_stopunit_action(get_side(),false,unit_location,remove_movement,remove_attacks);
 }
 
-
 synced_command_result_ptr readwrite_context_impl::execute_synced_command_action(const std::string& lua_code, const map_location& location){
 	return actions::execute_synced_command_action(get_side(),true,lua_code,location);
 }
 
-
 synced_command_result_ptr readonly_context_impl::check_synced_command_action(const std::string& lua_code, const map_location& location){
 	return actions::execute_synced_command_action(get_side(),false,lua_code,location);
 }
 
-
 template<typename T>
-void readonly_context_impl::add_known_aspect(const std::string &name, std::shared_ptr< typesafe_aspect <T>> &where)
+void readonly_context_impl::add_known_aspect(const std::string& name, typesafe_aspect_ptr<T>& where)
 {
-	std::shared_ptr< typesafe_known_aspect <T>> ka_ptr(new typesafe_known_aspect<T>(name,where,aspects_));
-	known_aspects_.emplace(name,ka_ptr);
+	known_aspects_.emplace(name, std::make_shared<typesafe_known_aspect<T>>(name, where, aspects_));
 }
 
 readonly_context_impl::readonly_context_impl(side_context &context, const config &cfg)
-		: cfg_(cfg),
-		engines_(),
-		known_aspects_(),
-		advancements_(),
-		aggression_(),
-		attack_depth_(),
-		aspects_(),
-		attacks_(),
-		avoid_(),
-		caution_(),
-		defensive_position_cache_(),
-		dstsrc_(),enemy_dstsrc_(),
-		enemy_possible_moves_(),
-		enemy_srcdst_(),
-		grouping_(),
-		goals_(),
-		keeps_(),
-		leader_aggression_(),
-		leader_goal_(),
-		leader_ignores_keep_(),
-		leader_value_(),
-		move_maps_enemy_valid_(false),
-		move_maps_valid_(false),
-		dst_src_valid_lua_(false),
-		dst_src_enemy_valid_lua_(false),
-		src_dst_valid_lua_(false),
-		src_dst_enemy_valid_lua_(false),
-		passive_leader_(),
-		passive_leader_shares_keep_(),
-		possible_moves_(),
-		recruitment_diversity_(),
-		recruitment_instructions_(),
-		recruitment_more_(),
-		recruitment_pattern_(),
-		recruitment_randomness_(),
-		recruitment_save_gold_(),
-		recursion_counter_(context.get_recursion_count()),
-		scout_village_targeting_(),
-		simple_targeting_(),
-		srcdst_(),
-		support_villages_(),
-		unit_stats_cache_(),
-		village_value_(),
-		villages_per_scout_()
-	{
-		init_side_context_proxy(context);
-		manager::get_singleton().add_gamestate_observer(this);
+	: cfg_(cfg),
+	engines_(),
+	known_aspects_(),
+	advancements_(),
+	aggression_(),
+	aspects_(),
+	attacks_(),
+	avoid_(),
+	caution_(),
+	defensive_position_cache_(),
+	dstsrc_(),enemy_dstsrc_(),
+	enemy_possible_moves_(),
+	enemy_srcdst_(),
+	grouping_(),
+	goals_(),
+	keeps_(),
+	leader_aggression_(),
+	leader_goal_(),
+	leader_ignores_keep_(),
+	leader_value_(),
+	move_maps_enemy_valid_(false),
+	move_maps_valid_(false),
+	dst_src_valid_lua_(false),
+	dst_src_enemy_valid_lua_(false),
+	src_dst_valid_lua_(false),
+	src_dst_enemy_valid_lua_(false),
+	passive_leader_(),
+	passive_leader_shares_keep_(),
+	possible_moves_(),
+	recruitment_diversity_(),
+	recruitment_instructions_(),
+	recruitment_more_(),
+	recruitment_pattern_(),
+	recruitment_randomness_(),
+	recruitment_save_gold_(),
+	recursion_counter_(context.get_recursion_count()),
+	retreat_enemy_weight_(),
+	retreat_factor_(),
+	scout_village_targeting_(),
+	simple_targeting_(),
+	srcdst_(),
+	support_villages_(),
+	unit_stats_cache_(),
+	village_value_(),
+	villages_per_scout_()
+{
+	init_side_context_proxy(context);
+	manager::get_singleton().add_gamestate_observer(this);
 
-		add_known_aspect("advancements", advancements_);
-		add_known_aspect("aggression",aggression_);
-		add_known_aspect("attack_depth",attack_depth_);
-		add_known_aspect("attacks",attacks_);
-		add_known_aspect("avoid",avoid_);
-		add_known_aspect("caution",caution_);
-		add_known_aspect("grouping",grouping_);
-		add_known_aspect("leader_aggression",leader_aggression_);
-		add_known_aspect("leader_goal",leader_goal_);
-		add_known_aspect("leader_ignores_keep",leader_ignores_keep_);
-		add_known_aspect("leader_value",leader_value_);
-		add_known_aspect("passive_leader",passive_leader_);
-		add_known_aspect("passive_leader_shares_keep",passive_leader_shares_keep_);
-		add_known_aspect("recruitment_diversity",recruitment_diversity_);
-		add_known_aspect("recruitment_instructions",recruitment_instructions_);
-		add_known_aspect("recruitment_more",recruitment_more_);
-		add_known_aspect("recruitment_pattern",recruitment_pattern_);
-		add_known_aspect("recruitment_randomness",recruitment_randomness_);
-		add_known_aspect("recruitment_save_gold",recruitment_save_gold_);
-		add_known_aspect("scout_village_targeting",scout_village_targeting_);
-		add_known_aspect("simple_targeting",simple_targeting_);
-		add_known_aspect("support_villages",support_villages_);
-		add_known_aspect("village_value",village_value_);
-		add_known_aspect("villages_per_scout",villages_per_scout_);
-		keeps_.init(resources::gameboard->map());
-
-	}
+	add_known_aspect("advancements", advancements_);
+	add_known_aspect("aggression",aggression_);
+	add_known_aspect("attacks",attacks_);
+	add_known_aspect("avoid",avoid_);
+	add_known_aspect("caution",caution_);
+	add_known_aspect("grouping",grouping_);
+	add_known_aspect("leader_aggression",leader_aggression_);
+	add_known_aspect("leader_goal",leader_goal_);
+	add_known_aspect("leader_ignores_keep",leader_ignores_keep_);
+	add_known_aspect("leader_value",leader_value_);
+	add_known_aspect("passive_leader",passive_leader_);
+	add_known_aspect("passive_leader_shares_keep",passive_leader_shares_keep_);
+	add_known_aspect("recruitment_diversity",recruitment_diversity_);
+	add_known_aspect("recruitment_instructions",recruitment_instructions_);
+	add_known_aspect("recruitment_more",recruitment_more_);
+	add_known_aspect("recruitment_pattern",recruitment_pattern_);
+	add_known_aspect("recruitment_randomness",recruitment_randomness_);
+	add_known_aspect("recruitment_save_gold",recruitment_save_gold_);
+	add_known_aspect("retreat_enemy_weight",retreat_enemy_weight_);
+	add_known_aspect("retreat_factor",retreat_factor_);
+	add_known_aspect("scout_village_targeting",scout_village_targeting_);
+	add_known_aspect("simple_targeting",simple_targeting_);
+	add_known_aspect("support_villages",support_villages_);
+	add_known_aspect("village_value",village_value_);
+	add_known_aspect("villages_per_scout",villages_per_scout_);
+	keeps_.init(resources::gameboard->map());
+}
 
 void readonly_context_impl::on_readonly_context_create() {
 	//init the composite ai engines
@@ -276,7 +257,6 @@ void readonly_context_impl::on_readonly_context_create() {
 	}
 }
 
-
 config side_context_impl::to_side_context_config() const
 {
 	return config();
@@ -287,17 +267,16 @@ config readwrite_context_impl::to_readwrite_context_config() const
 	return config();
 }
 
-
 config readonly_context_impl::to_readonly_context_config() const
 {
 	config cfg;
-	for(const engine_ptr e : engines_) {
+	for(const engine_ptr &e : engines_) {
 		cfg.add_child("engine",e->to_config());
 	}
-	for(const aspect_map::value_type a : aspects_) {
+	for(const aspect_map::value_type &a : aspects_) {
 		cfg.add_child("aspect",a.second->to_config());
 	}
-	for(const goal_ptr g : goals_) {
+	for(const goal_ptr &g : goals_) {
 		cfg.add_child("goal",g->to_config());
 	}
 	return cfg;
@@ -313,12 +292,10 @@ void readonly_context_impl::handle_generic_event(const std::string& /*event_name
 	invalidate_move_maps();
 }
 
-
 const game_info& readonly_context_impl::get_info() const{
 	return manager::get_singleton().get_active_ai_info_for_side(get_side());
 }
 
-
 game_info& readwrite_context_impl::get_info_w(){
 	return manager::get_singleton().get_active_ai_info_for_side(get_side());
 }
@@ -330,22 +307,19 @@ void readonly_context_impl::diagnostic(const std::string& msg)
 	}
 }
 
-
 const team& readonly_context_impl::current_team() const
 {
 	return resources::gameboard->get_team(get_side());
 }
 
-
 void readonly_context_impl::log_message(const std::string& msg)
 {
 	if(game_config::debug) {
-		game_display::get_singleton()->get_chat_manager().add_chat_message(time(nullptr), "ai", get_side(), msg,
-				events::chat_handler::MESSAGE_PUBLIC, false);
+		game_display::get_singleton()->get_chat_manager().add_chat_message(
+			std::time(nullptr), "ai", get_side(), msg, events::chat_handler::MESSAGE_PUBLIC, false);
 	}
 }
 
-
 void readonly_context_impl::calculate_possible_moves(std::map<map_location,pathfind::paths>& res, move_map& srcdst,
 		move_map& dstsrc, bool enemy, bool assume_full_movement,
 		const terrain_filter* remove_destinations) const
@@ -376,7 +350,7 @@ void readonly_context_impl::calculate_moves(const unit_map& units, std::map<map_
 		}
 
 		// We can't see where invisible enemy units might move.
-		if (enemy && un_it->invisible(un_it->get_location(), *resources::gameboard) && !see_all) {
+		if (enemy && un_it->invisible(un_it->get_location()) && !see_all) {
 			continue;
 		}
 		// If it's an enemy unit, reset its moves while we do the calculations.
@@ -389,7 +363,7 @@ void readonly_context_impl::calculate_moves(const unit_map& units, std::map<map_
 			dstsrc.insert(trivial_mv);
 		}
 		/**
-		 * @todo This is where support for a speculative unit map is incomplete.
+		 * TODO: This is where support for a speculative unit map is incomplete.
 		 *       There are several places (deep) within the paths constructor
 		 *       where resources::gameboard->units() is assumed to be the unit map. Rather
 		 *       than introduce a new parameter to numerous functions, a better
@@ -423,7 +397,7 @@ void readonly_context_impl::calculate_moves(const unit_map& units, std::map<map_
 
 			// Don't take friendly villages
 			if(!enemy && resources::gameboard->map().is_village(dst)) {
-				for(size_t n = 0; n != resources::gameboard->teams().size(); ++n) {
+				for(std::size_t n = 0; n != resources::gameboard->teams().size(); ++n) {
 					if(resources::gameboard->teams()[n].owns_village(dst)) {
 						int side = n + 1;
 						if (get_side() != side && !current_team().is_enemy(side)) {
@@ -447,7 +421,6 @@ void readonly_context_impl::calculate_moves(const unit_map& units, std::map<map_
 	}
 }
 
-
 void readonly_context_impl::add_aspects(std::vector< aspect_ptr > &aspects )
 {
 	for(aspect_ptr a : aspects) {
@@ -517,13 +490,11 @@ const defensive_position& readonly_context_impl::best_defensive_position(const m
 	return defensive_position_cache_[loc];
 }
 
-
 std::map<map_location,defensive_position>& readonly_context_impl::defensive_position_cache() const
 {
 	return defensive_position_cache_;
 }
 
-
 const unit_advancements_aspect& readonly_context_impl::get_advancements() const
 {
 	if (advancements_) {
@@ -534,7 +505,6 @@ const unit_advancements_aspect& readonly_context_impl::get_advancements() const
 	return uaa;
 }
 
-
 double readonly_context_impl::get_aggression() const
 {
 	if (aggression_) {
@@ -543,28 +513,16 @@ double readonly_context_impl::get_aggression() const
 	return 0;
 }
 
-
-int readonly_context_impl::get_attack_depth() const
-{
-	if (attack_depth_) {
-		return std::max<int>(1,attack_depth_->get()); ///@todo 1.9: add validators, such as minmax filters to aspects
-	}
-	return 1;
-}
-
-
 const aspect_map& readonly_context_impl::get_aspects() const
 {
 	return aspects_;
 }
 
-
 aspect_map& readonly_context_impl::get_aspects()
 {
 	return aspects_;
 }
 
-
 const attacks_vector& readonly_context_impl::get_attacks() const
 {
 	if (attacks_) {
@@ -574,13 +532,12 @@ const attacks_vector& readonly_context_impl::get_attacks() const
 	return av;
 }
 
-
 const wfl::variant& readonly_context_impl::get_attacks_as_variant() const
 {
 	if (attacks_) {
 		return attacks_->get_variant();
 	}
-	static wfl::variant v;///@todo 1.9: replace with variant::null_variant;
+	static wfl::variant v;
 	return v;
 }
 
@@ -591,11 +548,10 @@ const terrain_filter& readonly_context_impl::get_avoid() const
 	}
 	config cfg;
 	cfg.add_child("not");
-	static terrain_filter tf(vconfig(cfg, true), resources::filter_con);
+	static terrain_filter tf(vconfig(cfg, true), resources::filter_con, false);
 	return tf;
 }
 
-
 double readonly_context_impl::get_caution() const
 {
 	if (caution_) {
@@ -612,7 +568,6 @@ const move_map& readonly_context_impl::get_dstsrc() const
 	return dstsrc_;
 }
 
-
 const move_map& readonly_context_impl::get_enemy_dstsrc() const
 {
 	if (!move_maps_enemy_valid_) {
@@ -621,7 +576,6 @@ const move_map& readonly_context_impl::get_enemy_dstsrc() const
 	return enemy_dstsrc_;
 }
 
-
 const moves_map& readonly_context_impl::get_enemy_possible_moves() const
 {
 	if (!move_maps_enemy_valid_) {
@@ -630,7 +584,6 @@ const moves_map& readonly_context_impl::get_enemy_possible_moves() const
 	return enemy_possible_moves_;
 }
 
-
 const move_map& readonly_context_impl::get_enemy_srcdst() const
 {
 	if (!move_maps_enemy_valid_) {
@@ -639,7 +592,6 @@ const move_map& readonly_context_impl::get_enemy_srcdst() const
 	return enemy_srcdst_;
 }
 
-
 engine_ptr readonly_context_impl::get_engine_by_cfg(const config& cfg)
 {
 	std::string engine_name = cfg["engine"];
@@ -656,7 +608,6 @@ engine_ptr readonly_context_impl::get_engine_by_cfg(const config& cfg)
 		return *en;
 	}
 
-	//TODO: fix, removing some code duplication
 	engine_factory::factory_map::iterator eng = engine_factory::get_list().find(engine_name);
 	if (eng == engine_factory::get_list().end()){
 		ERR_AI << "side "<<get_side()<<" : UNABLE TO FIND engine["<<
@@ -676,19 +627,16 @@ engine_ptr readonly_context_impl::get_engine_by_cfg(const config& cfg)
 	return engines_.back();
 }
 
-
 const std::vector<engine_ptr>& readonly_context_impl::get_engines() const
 {
 	return engines_;
 }
 
-
 std::vector<engine_ptr>& readonly_context_impl::get_engines()
 {
 	return engines_;
 }
 
-
 std::string readonly_context_impl::get_grouping() const
 {
 	if (grouping_) {
@@ -697,20 +645,16 @@ std::string readonly_context_impl::get_grouping() const
 	return std::string();
 }
 
-
 const std::vector<goal_ptr>& readonly_context_impl::get_goals() const
 {
 	return goals_;
 }
 
-
 std::vector<goal_ptr>& readonly_context_impl::get_goals()
 {
 	return goals_;
 }
 
-
-
 double readonly_context_impl::get_leader_aggression() const
 {
 	if (leader_aggression_) {
@@ -719,7 +663,6 @@ double readonly_context_impl::get_leader_aggression() const
 	return 0;
 }
 
-
 config readonly_context_impl::get_leader_goal() const
 {
 	if (leader_goal_) {
@@ -728,16 +671,14 @@ config readonly_context_impl::get_leader_goal() const
 	return config();
 }
 
-
-bool readonly_context_impl::get_leader_ignores_keep() const
+utils::variant<bool, std::vector<std::string>> readonly_context_impl::get_leader_ignores_keep() const
 {
 	if (leader_ignores_keep_) {
 		return leader_ignores_keep_->get();
 	}
-	return false;
+	return {};
 }
 
-
 double readonly_context_impl::get_leader_value() const
 {
 	if (leader_value_) {
@@ -746,25 +687,22 @@ double readonly_context_impl::get_leader_value() const
 	return 0;
 }
 
-
-bool readonly_context_impl::get_passive_leader() const
+utils::variant<bool, std::vector<std::string>> readonly_context_impl::get_passive_leader() const
 {
 	if (passive_leader_) {
 		return passive_leader_->get();
 	}
-	return false;
+	return {};
 }
 
-
-bool readonly_context_impl::get_passive_leader_shares_keep() const
+utils::variant<bool, std::vector<std::string>> readonly_context_impl::get_passive_leader_shares_keep() const
 {
 	if (passive_leader_shares_keep_) {
 		return passive_leader_shares_keep_->get();
 	}
-	return false;
+	return {};
 }
 
-
 const moves_map& readonly_context_impl::get_possible_moves() const
 {
 	if (!move_maps_valid_) {
@@ -773,14 +711,6 @@ const moves_map& readonly_context_impl::get_possible_moves() const
 	return possible_moves_;
 }
 
-
-const std::vector<unit_ptr>& readonly_context_impl::get_recall_list() const
-{
-	///@todo 1.9: check for (level_["disallow_recall"]))
-	return current_team().recall_list().recall_list_; //TODO: Refactor ai so that friend of ai context is not required of recall_list_manager at this line
-}
-
-
 double readonly_context_impl::get_recruitment_diversity() const
 {
 	if (recruitment_diversity_) {
@@ -789,7 +719,6 @@ double readonly_context_impl::get_recruitment_diversity() const
 	return 0.;
 }
 
-
 const config readonly_context_impl::get_recruitment_instructions() const
 {
 	if (recruitment_instructions_) {
@@ -798,7 +727,6 @@ const config readonly_context_impl::get_recruitment_instructions() const
 	return config();
 }
 
-
 const std::vector<std::string> readonly_context_impl::get_recruitment_more() const
 {
 	if (recruitment_more_) {
@@ -807,7 +735,6 @@ const std::vector<std::string> readonly_context_impl::get_recruitment_more() con
 	return std::vector<std::string>();
 }
 
-
 const std::vector<std::string> readonly_context_impl::get_recruitment_pattern() const
 {
 	if (recruitment_pattern_) {
@@ -816,7 +743,6 @@ const std::vector<std::string> readonly_context_impl::get_recruitment_pattern()
 	return std::vector<std::string>();
 }
 
-
 int readonly_context_impl::get_recruitment_randomness() const
 {
 	if (recruitment_randomness_) {
@@ -825,7 +751,6 @@ int readonly_context_impl::get_recruitment_randomness() const
 	return 0;
 }
 
-
 const config readonly_context_impl::get_recruitment_save_gold() const
 {
 	if (recruitment_save_gold_) {
@@ -834,6 +759,21 @@ const config readonly_context_impl::get_recruitment_save_gold() const
 	return config();
 }
 
+double readonly_context_impl::get_retreat_enemy_weight() const
+{
+	if (retreat_enemy_weight_) {
+		return retreat_enemy_weight_->get();
+	}
+	return 1;
+}
+
+double readonly_context_impl::get_retreat_factor() const
+{
+	if (retreat_factor_) {
+		return retreat_factor_->get();
+	}
+	return 1;
+}
 
 double readonly_context_impl::get_scout_village_targeting() const
 {
@@ -843,7 +783,6 @@ double readonly_context_impl::get_scout_village_targeting() const
 	return 1;
 }
 
-
 bool readonly_context_impl::get_simple_targeting() const
 {
 	if (simple_targeting_) {
@@ -852,7 +791,6 @@ bool readonly_context_impl::get_simple_targeting() const
 	return false;
 }
 
-
 const move_map& readonly_context_impl::get_srcdst() const
 {
 	if (!move_maps_valid_) {
@@ -861,7 +799,6 @@ const move_map& readonly_context_impl::get_srcdst() const
 	return srcdst_;
 }
 
-
 bool readonly_context_impl::get_support_villages() const
 {
 	if (support_villages_) {
@@ -870,7 +807,6 @@ bool readonly_context_impl::get_support_villages() const
 	return false;
 }
 
-
 double readonly_context_impl::get_village_value() const
 {
 	if (village_value_) {
@@ -879,7 +815,6 @@ double readonly_context_impl::get_village_value() const
 	return 0;
 }
 
-
 int readonly_context_impl::get_villages_per_scout() const
 {
 	if (villages_per_scout_) {
@@ -888,7 +823,6 @@ int readonly_context_impl::get_villages_per_scout() const
 	return 0;
 }
 
-
 bool readonly_context_impl::is_dst_src_valid_lua() const
 {
 	return dst_src_valid_lua_;
@@ -914,19 +848,16 @@ void readonly_context_impl::invalidate_defensive_position_cache() const
 	defensive_position_cache_.clear();
 }
 
-
 void readonly_context_impl::invalidate_keeps_cache() const
 {
 	keeps_.clear();
 }
 
-
 void keeps_cache::handle_generic_event(const std::string &/*event_name*/)
 {
 	clear();
 }
 
-
 void readonly_context_impl::invalidate_move_maps() const
 {
 	move_maps_valid_ = false;
@@ -939,13 +870,11 @@ void readonly_context_impl::invalidate_move_maps() const
 	src_dst_enemy_valid_lua_ = false;
 }
 
-
 const std::set<map_location>& readonly_context_impl::keeps() const
 {
 	return keeps_.get();
 }
 
-
 keeps_cache::keeps_cache()
 	: map_(nullptr)
 	, keeps_()
@@ -954,7 +883,6 @@ keeps_cache::keeps_cache()
 	ai::manager::get_singleton().add_map_changed_observer(this);
 }
 
-
 keeps_cache::~keeps_cache()
 {
 	ai::manager::get_singleton().remove_turn_started_observer(this);
@@ -966,7 +894,6 @@ void keeps_cache::clear()
 	keeps_.clear();
 }
 
-
 void keeps_cache::init(const gamemap &map)
 {
 	map_ = &map;
@@ -981,10 +908,8 @@ const std::set<map_location>& keeps_cache::get()
 			for(int y = 0; y != map_->h(); ++y) {
 				const map_location loc(x,y);
 				if(map_->is_keep(loc)) {
-					adjacent_loc_array_t adj;
-					get_adjacent_tiles(loc,adj.data());
-					for(size_t n = 0; n < adj.size(); ++n) {
-						if(map_->is_castle(adj[n])) {
+					for(const map_location& adj : get_adjacent_tiles(loc)) {
+						if(map_->is_castle(adj)) {
 							keeps_.insert(loc);
 							break;
 						}
@@ -997,7 +922,6 @@ const std::set<map_location>& keeps_cache::get()
 	return keeps_;
 }
 
-
 bool readonly_context_impl::leader_can_reach_keep() const
 {
 	const unit_map::iterator leader = resources::gameboard->units().find_leader(get_side());
@@ -1020,7 +944,6 @@ bool readonly_context_impl::leader_can_reach_keep() const
 	return leader_paths.destinations.contains(start_pos);
 }
 
-
 const map_location& readonly_context_impl::nearest_keep(const map_location& loc) const
 {
 	std::set<map_location> avoided_locations;
@@ -1050,7 +973,6 @@ const map_location& readonly_context_impl::nearest_keep(const map_location& loc)
 	}
 }
 
-
 double readonly_context_impl::power_projection(const map_location& loc, const move_map& dstsrc) const
 {
 	map_location used_locs[6];
@@ -1058,8 +980,7 @@ double readonly_context_impl::power_projection(const map_location& loc, const mo
 	std::fill_n(ratings, 0, 6);
 	int num_used_locs = 0;
 
-	adjacent_loc_array_t locs;
-	get_adjacent_tiles(loc,locs.data());
+	const auto locs = get_adjacent_tiles(loc);
 
 	const gamemap& map_ = resources::gameboard->map();
 	unit_map& units_ = resources::gameboard->units();
@@ -1119,7 +1040,7 @@ double readonly_context_impl::power_projection(const map_location& loc, const mo
 			}
 
 			// The 0.5 power avoids underestimating too much the damage of a wounded unit.
-			int64_t hp = int(sqrt(double(un.hitpoints()) / un.max_hitpoints()) * 1000);
+			int64_t hp = static_cast<int>(std::sqrt(static_cast<double>(un.hitpoints()) / un.max_hitpoints()) * 1000);
 			int64_t most_damage = 0;
 			for(const attack_type &att : un.attacks())
 			{
@@ -1173,7 +1094,7 @@ void readonly_context_impl::recalculate_move_maps() const
 	possible_moves_ = moves_map();
 	srcdst_ = move_map();
 	calculate_possible_moves(possible_moves_,srcdst_,dstsrc_,false,false,&get_avoid());
-	if (get_passive_leader()||get_passive_leader_shares_keep()) {
+	if (is_passive_leader("") && !is_passive_keep_sharing_leader("")) {
 		unit_map::iterator i = resources::gameboard->units().find_leader(get_side());
 		if (i.valid()) {
 			map_location loc = i->get_location();
@@ -1185,7 +1106,6 @@ void readonly_context_impl::recalculate_move_maps() const
 					++it;
 				}
 			}
-		///@todo: shall possible moves be modified as well ?
 		}
 	}
 	move_maps_valid_ = true;
@@ -1195,7 +1115,6 @@ void readonly_context_impl::recalculate_move_maps() const
 	src_dst_valid_lua_ = false;
 }
 
-
 void readonly_context_impl::recalculate_move_maps_enemy() const
 {
 	enemy_dstsrc_ = move_map();
@@ -1271,35 +1190,62 @@ const map_location& readonly_context_impl::suitable_keep(const map_location& lea
 	return nearest_keep(leader_location); // return nearest keep
 }
 
-
 	/** Weapon choice cache, to speed simulations. */
 readonly_context::unit_stats_cache_t & readonly_context_impl::unit_stats_cache() const
 {
 	return unit_stats_cache_;
 }
 
-
 bool readonly_context_impl::is_active(const std::string &time_of_day, const std::string &turns) const
 {
-		if(time_of_day.empty() == false) {
-			const std::vector<std::string>& times = utils::split(time_of_day);
-			if(std::count(times.begin(),times.end(),resources::tod_manager->get_time_of_day().id) == 0) {
-				return false;
-			}
-		}
-
-		if(turns.empty() == false) {
-			int turn = resources::tod_manager->turn();
-			const std::vector<std::string>& turns_list = utils::split(turns);
-			for(std::vector<std::string>::const_iterator j = turns_list.begin(); j != turns_list.end() ; ++j ) {
-				const std::pair<int,int> range = utils::parse_range(*j);
-				if(turn >= range.first && turn <= range.second) {
-				      return true;
-				}
-			}
+	if(time_of_day.empty() == false) {
+		const std::vector<std::string>& times = utils::split(time_of_day);
+		if(std::count(times.begin(),times.end(),resources::tod_manager->get_time_of_day().id) == 0) {
 			return false;
 		}
-		return true;
+	}
+
+	if(turns.empty() == false) {
+		int turn = resources::tod_manager->turn();
+		const std::vector<std::string>& turns_list = utils::split(turns);
+		for(std::vector<std::string>::const_iterator j = turns_list.begin(); j != turns_list.end() ; ++j ) {
+			const std::pair<int,int> range = utils::parse_range(*j);
+			if(turn >= range.first && turn <= range.second) {
+					return true;
+			}
+		}
+		return false;
+	}
+	return true;
+}
+
+bool readonly_context_impl::applies_to_leader(
+	const utils::variant<bool, std::vector<std::string>>& aspect_value, const std::string& id) const
+{
+	return utils::visit(
+		[&id](const auto& v) {
+			if constexpr(utils::decayed_is_same<bool, decltype(v)>) {
+				return v;
+			} else {
+				return utils::contains(v, id);
+			}
+		},
+		aspect_value);
+}
+
+bool readonly_context_impl::is_keep_ignoring_leader(const std::string &id) const
+{
+	return applies_to_leader(leader_ignores_keep_->get(), id);
+}
+
+bool readonly_context_impl::is_passive_leader(const std::string &id) const
+{
+	return applies_to_leader(passive_leader_->get(), id);
+}
+
+bool readonly_context_impl::is_passive_keep_sharing_leader(const std::string &id) const
+{
+	return applies_to_leader(passive_leader_shares_keep_->get(), id);
 }
 
 } //of namespace ai
