@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -51,9 +52,8 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 	assert(defend_it != units.end());
 
 	// See if the target is a threat to our leader or an ally's leader.
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(target,adj.data());
-	size_t tile;
+	const auto adj = get_adjacent_tiles(target);
+	std::size_t tile;
 	for(tile = 0; tile < adj.size(); ++tile) {
 		const unit_map::const_iterator leader = units.find(adj[tile]);
 		if(leader != units.end() && leader->can_recruit() && !ai_obj.current_team().is_enemy(leader->side())) {
@@ -65,8 +65,8 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 	uses_leader = false;
 
 	target_value = defend_it->cost();
-	target_value += (double(defend_it->experience())/
-	                 double(defend_it->max_experience()))*target_value;
+	target_value += (static_cast<double>(defend_it->experience())/
+	                 static_cast<double>(defend_it->max_experience()))*target_value;
 	target_starting_damage = defend_it->max_hitpoints() -
 	                         defend_it->hitpoints();
 
@@ -76,7 +76,7 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 	// making itself.
 	alternative_terrain_quality = 0.0;
 	double cost_sum = 0.0;
-	for(size_t i = 0; i != movements.size(); ++i) {
+	for(std::size_t i = 0; i != movements.size(); ++i) {
 		const unit_map::const_iterator att = units.find(movements[i].first);
 		const double cost = att->cost();
 		cost_sum += cost;
@@ -126,7 +126,7 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 
 		// This cache is only about 99% correct, but speeds up evaluation by about 1000 times.
 		// We recalculate when we actually attack.
-		const readonly_context::unit_stats_cache_t::key_type cache_key = std::make_pair(target, &up->type());
+		const readonly_context::unit_stats_cache_t::key_type cache_key = std::pair(target, &up->type());
 		const readonly_context::unit_stats_cache_t::iterator usc = ai_obj.unit_stats_cache().find(cache_key);
 		// Just check this attack is valid for this attacking unit (may be modified)
 		if (usc != ai_obj.unit_stats_cache().end() &&
@@ -147,7 +147,7 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 		old_bc.reset(nullptr);
 
 		if ( !from_cache ) {
-			ai_obj.unit_stats_cache().emplace(cache_key, std::make_pair(
+			ai_obj.unit_stats_cache().emplace(cache_key, std::pair(
 				bc->get_attacker_stats(),
 				bc->get_defender_stats()
 			));
@@ -156,7 +156,6 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 		// Note we didn't fight at all if defender is already dead.
 		double prob_fought = (1.0 - prob_dead_already);
 
-		/** @todo 1.9 add combatant.prob_killed */
 		double prob_killed = def.hp_dist[0] - prob_dead_already;
 		prob_dead_already = def.hp_dist[0];
 
@@ -166,7 +165,7 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 		double cost = up->cost();
 		const bool on_village = map.is_village(m->second);
 		// Up to double the value of a unit based on experience
-		cost += (double(up->experience()) / up->max_experience())*cost;
+		cost += (static_cast<double>(up->experience()) / up->max_experience())*cost;
 		resources_used += cost;
 		avg_losses += cost * prob_died;
 
@@ -182,7 +181,7 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 			avg_damage_taken -= game_config::poison_amount*2 * prob_survived;
 		}
 
-		terrain_quality += (double(bc->get_defender_stats().chance_to_hit)/100.0)*cost * (on_village ? 0.5 : 1.0);
+		terrain_quality += (static_cast<double>(bc->get_defender_stats().chance_to_hit)/100.0)*cost * (on_village ? 0.5 : 1.0);
 
 		double advance_prob = 0.0;
 		// The reward for advancing a unit is to get a 'negative' loss of that unit
@@ -194,7 +193,7 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 			if (xp_for_advance == 0)
 				xp_for_advance = 1;
 
-			int fight_xp = defend_it->level();
+			int fight_xp = game_config::combat_xp(defend_it->level());
 			int kill_xp = game_config::kill_xp(fight_xp);
 
 			if (fight_xp >= xp_for_advance) {
@@ -228,12 +227,7 @@ void attack_analysis::analyze(const gamemap& map, unit_map& units,
 		// If we didn't advance, we took this damage.
 		avg_damage_taken += (up->hitpoints() - att.average_hp()) * (1.0 - advance_prob);
 
-		/**
-		 * @todo 1.9: attack_prediction.cpp should understand advancement
-		 * directly.  For each level of attacker def gets 1 xp or
-		 * kill_experience.
-		 */
-		int fight_xp = up->level();
+		int fight_xp = game_config::combat_xp(up->level());
 		int kill_xp = game_config::kill_xp(fight_xp);
 		def_avg_experience += fight_xp * (1.0 - att.hp_dist[0]) + kill_xp * att.hp_dist[0];
 		if (m == movements.begin()) {
@@ -345,7 +339,7 @@ wfl::variant attack_analysis::get_value(const std::string& key) const
 		return variant(std::make_shared<location_callable>(target));
 	} else if(key == "movements") {
 		std::vector<variant> res;
-		for(size_t n = 0; n != movements.size(); ++n) {
+		for(std::size_t n = 0; n != movements.size(); ++n) {
 			auto item = std::make_shared<map_formula_callable>(nullptr);
 			item->add("src", variant(std::make_shared<location_callable>(movements[n].first)));
 			item->add("dst", variant(std::make_shared<location_callable>(movements[n].second)));
@@ -355,7 +349,7 @@ wfl::variant attack_analysis::get_value(const std::string& key) const
 		return variant(res);
 	} else if(key == "units") {
 		std::vector<variant> res;
-		for(size_t n = 0; n != movements.size(); ++n) {
+		for(std::size_t n = 0; n != movements.size(); ++n) {
 			res.emplace_back(std::make_shared<location_callable>(movements[n].first));
 		}
 
