@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -24,7 +25,6 @@
 #include "ai/composite/rca.hpp"
 #include "ai/composite/stage.hpp"
 #include "game_board.hpp"
-#include "game_classification.hpp"
 #include "game_data.hpp"
 #include "log.hpp"
 #include "map/map.hpp"
@@ -45,7 +45,6 @@ static lg::log_domain log_ai_testing_ai_default("ai/ca/testing_ai_default");
 #define WRN_AI_TESTING_AI_DEFAULT LOG_STREAM(warn, log_ai_testing_ai_default)
 #define ERR_AI_TESTING_AI_DEFAULT LOG_STREAM(err, log_ai_testing_ai_default)
 
-
 namespace ai {
 
 namespace ai_default_rca {
@@ -80,11 +79,15 @@ double goto_phase::evaluate()
 	for(std::vector<map_location>::const_iterator g = gotos.begin(); g != gotos.end(); ++g) {
 		unit_map::const_iterator ui = units_.find(*g);
 		// passive_leader: never moves or attacks
-		if(ui->can_recruit() && get_passive_leader() && !get_passive_leader_shares_keep()){
-			continue;//@todo: only bail out if goto is on keep
+		if(ui->can_recruit() && is_passive_leader(ui->id())){
+			continue;
 		}
 		// end of passive_leader
 
+		if(!is_allowed_unit(*ui)){
+			continue;
+		}
+
 		const pathfind::shortest_path_calculator calc(*ui, current_team(), resources::gameboard->teams(), resources::gameboard->map());
 
 		const pathfind::teleport_map allowed_teleports = pathfind::get_teleport_locations(*ui, current_team());
@@ -147,7 +150,6 @@ void goto_phase::execute()
 	}
 }
 
-
 //==============================================================
 
 combat_phase::combat_phase( rca_context &context, const config &cfg )
@@ -161,6 +163,7 @@ combat_phase::~combat_phase()
 
 double combat_phase::evaluate()
 {
+	const unit_map &units_ = resources::gameboard->units();
 	std::vector<std::string> options = get_recruitment_pattern();
 
 	choice_rating_ = -1000.0;
@@ -193,6 +196,22 @@ double combat_phase::evaluate()
 		if(skip_num > 0 && ((it - analysis.begin())%skip_num) && it->movements.size() > 1)
 			continue;
 
+		// This is somewhat inefficient. It would be faster to exclude these attacks
+		// in get_attacks() above, but the CA filter information is not available inside
+		// the attacks aspect code. Providing the filtering here is only done for consistency
+		// with other CAs though, the recommended method of filtering attacks is via
+		// 'filter_own' of the attacks aspect.
+		bool skip_attack = false;
+		for(std::size_t i = 0; i != it->movements.size(); ++i) {
+			const unit_map::const_iterator u = units_.find(it->movements[i].first);
+			if (!u.valid() || (!is_allowed_unit(*u))) {
+				skip_attack = true;
+				break;
+			}
+		}
+		if (skip_attack)
+			continue;
+
 		const double rating = it->rating(get_aggression(),*this);
 		LOG_AI_TESTING_AI_DEFAULT << "attack option rated at " << rating << " ("
 					  << (it->uses_leader ? get_leader_aggression() : get_aggression()) << ")\n";
@@ -206,7 +225,6 @@ double combat_phase::evaluate()
 	time_taken = SDL_GetTicks() - ticks;
 	LOG_AI_TESTING_AI_DEFAULT << "analysis took " << time_taken << " ticks\n";
 
-
 	// suokko tested the rating against current_team().caution()
 	// Bad mistake -- the AI became extremely reluctant to attack anything.
 	// Documenting this in case someone has this bright idea again...*don't*...
@@ -280,8 +298,21 @@ double move_leader_to_goals_phase::evaluate()
 		return BAD_SCORE;
 	}
 
-	const unit_map::iterator leader = resources::gameboard->units().find_leader(get_side());
-	if (!leader.valid() || leader->incapacitated()) {
+	const unit_map &units_ = resources::gameboard->units();
+	const std::vector<unit_map::const_iterator> leaders = units_.find_leaders(get_side());
+	if (leaders.empty()) {
+		return BAD_SCORE;
+	}
+
+	const unit* leader = nullptr;
+	for (const unit_map::const_iterator& l_itor : leaders) {
+		if (!l_itor->incapacitated() && l_itor->movement_left() > 0 && is_allowed_unit(*l_itor)) {
+			leader = &(*l_itor);
+			break;
+		}
+	}
+
+	if (leader == nullptr) {
 		WRN_AI_TESTING_AI_DEFAULT << "Leader not found" << std::endl;
 		return BAD_SCORE;
 	}
@@ -373,10 +404,7 @@ move_leader_to_keep_phase::~move_leader_to_keep_phase()
 
 double move_leader_to_keep_phase::evaluate()
 {
-	if (get_leader_ignores_keep()) {
-		return BAD_SCORE;
-	}
-	if (get_passive_leader() && !get_passive_leader_shares_keep()) {
+	if (is_keep_ignoring_leader("")) {
 		return BAD_SCORE;
 	}
 
@@ -400,7 +428,7 @@ double move_leader_to_keep_phase::evaluate()
 	int shortest_distance = 99999;
 
 	for (const unit_map::const_iterator& leader : leaders) {
-		if (leader->incapacitated() || leader->movement_left() == 0) {
+		if (leader->incapacitated() || leader->movement_left() == 0 || !is_allowed_unit(*leader) || is_keep_ignoring_leader(leader->id()) || (is_passive_leader(leader->id()) && !is_passive_keep_sharing_leader(leader->id()))) {
 			continue;
 		}
 
@@ -534,7 +562,6 @@ double get_villages_phase::evaluate()
 	return BAD_SCORE;
 }
 
-
 void get_villages_phase::execute()
 {
 	unit_map &units_ = resources::gameboard->units();
@@ -562,12 +589,6 @@ void get_villages_phase::execute()
 				    power_projection(i->first, get_enemy_dstsrc()) >= new_unit->hitpoints() / 4.0)
 				{
 					LOG_AI_TESTING_AI_DEFAULT << "found support target... " << new_unit->get_location() << '\n';
-					//FIXME: suokko tweaked the constant 1.0 to the formula:
-					//25.0* current_team().caution() * power_projection(loc,enemy_dstsrc) / new_unit->second.hitpoints()
-					//Is this an improvement?
-
-					///@todo 1.7 check if this an improvement
-					//add_target(target(new_unit->first,1.0,target::SUPPORT));
 				}
 			}
 		}
@@ -608,15 +629,14 @@ void get_villages_phase::get_villages(
 	treachmap reachmap;
 	for(unit_map::const_iterator u_itor = units_.begin();
 			u_itor != units_.end(); ++u_itor) {
-		if(u_itor->can_recruit() && get_passive_leader()){
+		if(u_itor->can_recruit() && is_passive_leader(u_itor->id())){
 			continue;
 		}
-		if(u_itor->side() == get_side() && u_itor->movement_left()) {
+		if(u_itor->side() == get_side() && u_itor->movement_left() && is_allowed_unit(*u_itor)) {
 			reachmap.emplace(u_itor->get_location(),	std::vector<map_location>());
 		}
 	}
 
-
 	DBG_AI_TESTING_AI_DEFAULT << reachmap.size() << " units found who can try to capture a village.\n";
 
 	find_villages(reachmap, moves_, dstsrc, enemy_dstsrc);
@@ -655,9 +675,7 @@ void get_villages_phase::find_villages(
 {
 	std::map<map_location, double> vulnerability;
 
-	const bool passive_leader = get_passive_leader();
-
-	size_t min_distance = 100000;
+	std::size_t min_distance = 100000;
 	const gamemap &map_ = resources::gameboard->map();
 	std::vector<team> &teams_ = resources::gameboard->teams();
 
@@ -671,11 +689,7 @@ void get_villages_phase::find_villages(
 		const map_location &current_loc = j->first;
 
 		if(j->second == leader_loc_) {
-			if(passive_leader) {
-				continue;
-			}
-
-			const size_t distance = distance_between(keep_loc_, current_loc);
+			const std::size_t distance = distance_between(keep_loc_, current_loc);
 			if(distance < min_distance) {
 				min_distance = distance;
 				best_leader_loc_ = current_loc;
@@ -692,7 +706,7 @@ void get_villages_phase::find_villages(
 		}
 
 		bool want_village = true, owned = false;
-		for(size_t n = 0; n != teams_.size(); ++n) {
+		for(std::size_t n = 0; n != teams_.size(); ++n) {
 			owned = teams_[n].owns_village(current_loc);
 			if(owned && !current_team().is_enemy(n+1)) {
 				want_village = false;
@@ -723,7 +737,7 @@ void get_villages_phase::find_villages(
 		}
 
 		const unit_map::const_iterator u = resources::gameboard->units().find(j->second);
-		if (u == resources::gameboard->units().end() || u->get_state("guardian")) {
+		if (u == resources::gameboard->units().end() || u->get_state("guardian") || !is_allowed_unit(*u) || (u->can_recruit() && is_passive_leader(u->id()))) {
 			continue;
 		}
 
@@ -776,7 +790,7 @@ void get_villages_phase::dispatch(treachmap& reachmap, tmoves& moves)
 	//    village only one can capture it, so use the first in the list.)
 	// 2. Villages which can only be reached by one unit get that unit dispatched
 	//    to them.
-	size_t village_count = 0;
+	std::size_t village_count = 0;
 	bool dispatched = true;
 	while(dispatched) {
 		dispatched = false;
@@ -870,7 +884,7 @@ bool get_villages_phase::dispatch_unit_simple(treachmap& reachmap, tmoves& moves
 }
 
 bool get_villages_phase::dispatch_village_simple(
-	treachmap& reachmap, tmoves& moves, size_t& village_count)
+	treachmap& reachmap, tmoves& moves, std::size_t& village_count)
 {
 
 	bool result = false;
@@ -955,13 +969,13 @@ get_villages_phase::treachmap::iterator get_villages_phase::remove_unit(
 }
 
 void get_villages_phase::dispatch_complex(
-	treachmap& reachmap, tmoves& moves, const size_t village_count)
+	treachmap& reachmap, tmoves& moves, const std::size_t village_count)
 {
 	// ***** ***** Init and dispatch if every unit can reach every village.
 
-	const size_t unit_count = reachmap.size();
+	const std::size_t unit_count = reachmap.size();
 	// The maximum number of villages we can capture with the available units.
-	const size_t max_result = unit_count < village_count ? unit_count : village_count;
+	const std::size_t max_result = unit_count < village_count ? unit_count : village_count;
 
 	assert(unit_count >= 2 && village_count >= 2);
 
@@ -973,28 +987,28 @@ void get_villages_phase::dispatch_complex(
 	}
 
 	std::vector<map_location> units(unit_count);
-	std::vector<size_t> villages_per_unit(unit_count);
+	std::vector<std::size_t> villages_per_unit(unit_count);
 	std::vector<map_location> villages;
-	std::vector<size_t> units_per_village(village_count);
+	std::vector<std::size_t> units_per_village(village_count);
 
 	// We want to test the units, the ones who can reach the least
 	// villages first so this is our lookup map.
-	std::multimap<size_t /* villages_per_unit value*/,
-		size_t /*villages_per_unit index*/> unit_lookup;
+	std::multimap<std::size_t /* villages_per_unit value*/,
+		std::size_t /*villages_per_unit index*/> unit_lookup;
 
 	std::vector</*unit*/boost::dynamic_bitset</*village*/>> matrix(reachmap.size(), boost::dynamic_bitset<>(village_count));
 
 	treachmap::const_iterator itor = reachmap.begin();
-	for(size_t u = 0; u < unit_count; ++u, ++itor) {
+	for(std::size_t u = 0; u < unit_count; ++u, ++itor) {
 		units[u] = itor->first;
 		villages_per_unit[u] = itor->second.size();
 		unit_lookup.emplace(villages_per_unit[u], u);
 
 		assert(itor->second.size() >= 2);
 
-		for(size_t v = 0; v < itor->second.size(); ++v) {
+		for(std::size_t v = 0; v < itor->second.size(); ++v) {
 
-			size_t v_index;
+			std::size_t v_index;
 			// find the index of the v in the villages
 			std::vector<map_location>::const_iterator v_itor =
 				std::find(villages.begin(), villages.end(), itor->second[v]);
@@ -1010,7 +1024,7 @@ void get_villages_phase::dispatch_complex(
 			matrix[u][v_index] = true;
 		}
 	}
-	for(std::vector<size_t>::const_iterator upv_it = units_per_village.begin();
+	for(std::vector<std::size_t>::const_iterator upv_it = units_per_village.begin();
 			upv_it != units_per_village.end(); ++upv_it) {
 
 		assert(*upv_it >=2);
@@ -1019,7 +1033,7 @@ void get_villages_phase::dispatch_complex(
 	if(debug_) {
 		// Print header
 		std::cerr << "Reach matrix:\n\nvillage";
-		size_t u, v;
+		std::size_t u, v;
 		for(v = 0; v < village_count; ++v) {
 			std::cerr << '\t' << villages[v];
 		}
@@ -1045,7 +1059,7 @@ void get_villages_phase::dispatch_complex(
 
 	// Test the special case, everybody can reach all villages
 	const bool reach_all = ((village_count == unit_count)
-		&& (std::accumulate(villages_per_unit.begin(), villages_per_unit.end(), size_t())
+		&& (std::accumulate(villages_per_unit.begin(), villages_per_unit.end(), std::size_t())
 		== (village_count * unit_count)));
 
 	if(reach_all) {
@@ -1056,12 +1070,12 @@ void get_villages_phase::dispatch_complex(
 	}
 
 	// ***** ***** Find a square
-	std::multimap<size_t /* villages_per_unit value*/, size_t /*villages_per_unit index*/>
+	std::multimap<std::size_t /* villages_per_unit value*/, std::size_t /*villages_per_unit index*/>
 		::const_iterator src_itor =  unit_lookup.begin();
 
 	while(src_itor != unit_lookup.end() && src_itor->first == 2) {
 
-		for(std::multimap<size_t, size_t>::const_iterator
+		for(std::multimap<std::size_t, std::size_t>::const_iterator
 				dst_itor = unit_lookup.begin();
 				dst_itor != unit_lookup.end(); ++ dst_itor) {
 
@@ -1071,13 +1085,13 @@ void get_villages_phase::dispatch_complex(
 			}
 
 			boost::dynamic_bitset<> result = matrix[src_itor->second] & matrix[dst_itor->second];
-			size_t matched = result.count();
+			std::size_t matched = result.count();
 
 			// we found a  solution, dispatch
 			if(matched == 2) {
 				// Collect data
-				size_t first = result.find_first();
-				size_t second = result.find_next(first);
+				std::size_t first = result.find_first();
+				std::size_t second = result.find_next(first);
 
 				const map_location village1 = villages[first];
 				const map_location village2 = villages[second];
@@ -1138,22 +1152,22 @@ void get_villages_phase::dispatch_complex(
 	// picked. In general we shouldn't reach this point too often if we do
 	// there are a lot of villages which are unclaimed and a lot of units
 	// to claim them.
-	const size_t max_options = 8;
+	const std::size_t max_options = 8;
 	if(unit_count >= max_options && village_count >= max_options) {
 
 		DBG_AI_TESTING_AI_DEFAULT << "Too many units " << unit_count << " and villages "
 			<< village_count<<" found, evaluate only the first "
 			<< max_options << " options;\n";
 
-		std::vector<size_t> perm (max_options, 0);
-		for(size_t i =0; i < max_options; ++i) {
+		std::vector<std::size_t> perm (max_options, 0);
+		for(std::size_t i =0; i < max_options; ++i) {
 			perm[i] = i;
 		}
 		while(std::next_permutation(perm.begin(), perm.end())) {
 
 			// Get result for current permutation.
 			std::vector<std::pair<map_location,map_location>> result;
-			for(size_t u = 0; u < max_options; ++u) {
+			for(std::size_t u = 0; u < max_options; ++u) {
 				if(matrix[u][perm[u]]) {
 					result.emplace_back(villages[perm[u]], units[u]);
 
@@ -1184,14 +1198,14 @@ void get_villages_phase::dispatch_complex(
 
 		DBG_AI_TESTING_AI_DEFAULT << "Unit major\n";
 
-		std::vector<size_t> perm (unit_count, 0);
-		for(size_t i =0; i < unit_count; ++i) {
+		std::vector<std::size_t> perm (unit_count, 0);
+		for(std::size_t i =0; i < unit_count; ++i) {
 			perm[i] = i;
 		}
 		while(std::next_permutation(perm.begin(), perm.end())) {
 			// Get result for current permutation.
 			std::vector<std::pair<map_location,map_location>> result;
-			for(size_t u = 0; u < unit_count; ++u) {
+			for(std::size_t u = 0; u < unit_count; ++u) {
 				if(matrix[u][perm[u]]) {
 					result.emplace_back(villages[perm[u]], units[u]);
 
@@ -1226,14 +1240,14 @@ void get_villages_phase::dispatch_complex(
 
 		DBG_AI_TESTING_AI_DEFAULT << "Village major\n";
 
-		std::vector<size_t> perm (village_count, 0);
-		for(size_t i =0; i < village_count; ++i) {
+		std::vector<std::size_t> perm (village_count, 0);
+		for(std::size_t i =0; i < village_count; ++i) {
 			perm[i] = i;
 		}
 		while(std::next_permutation(perm.begin(), perm.end())) {
 			// Get result for current permutation.
 			std::vector<std::pair<map_location,map_location>> result;
-			for(size_t v = 0; v < village_count; ++v) {
+			for(std::size_t v = 0; v < village_count; ++v) {
 				if(matrix[perm[v]][v]) {
 					result.emplace_back(villages[v], units[perm[v]]);
 
@@ -1269,7 +1283,7 @@ void get_villages_phase::dispatch_complex(
 void get_villages_phase::full_dispatch(treachmap& reachmap, tmoves& moves)
 {
 	treachmap::const_iterator itor = reachmap.begin();
-	for(size_t i = 0; i < reachmap.size(); ++i, ++itor) {
+	for(std::size_t i = 0; i < reachmap.size(); ++i, ++itor) {
 		DBG_AI_TESTING_AI_DEFAULT << "Dispatched unit at " << itor->first
 				<< " to village " << itor->second[i] << '\n';
 		moves.emplace_back(itor->second[i], itor->first);
@@ -1321,7 +1335,7 @@ double get_healing_phase::evaluate()
 	for(; u_it != units_.end(); ++u_it) {
 		unit &u = *u_it;
 
-		if(u.can_recruit() && get_passive_leader()){
+		if(u.can_recruit() && is_passive_leader(u.id())){
 			continue;
 		}
 
@@ -1331,7 +1345,7 @@ double get_healing_phase::evaluate()
 		if(u.side() == get_side() &&
 		   (u.max_hitpoints() - u.hitpoints() >= game_config::poison_amount/2
 		   || u.get_state(unit::STATE_POISONED)) &&
-		    !u.get_ability_bool("regenerate", *resources::gameboard))
+		    !u.get_ability_bool("regenerate") && is_allowed_unit(*u_it))
 		{
 			// Look for the village which is the least vulnerable to enemy attack.
 			typedef std::multimap<map_location,map_location>::const_iterator Itor;
@@ -1392,7 +1406,6 @@ retreat_phase::~retreat_phase()
 double retreat_phase::evaluate()
 {
 
-
 	// Get versions of the move map that assume that all units are at full movement
 	const unit_map& units_ = resources::gameboard->units();
 
@@ -1405,16 +1418,9 @@ double retreat_phase::evaluate()
 	calculate_possible_moves(dummy_possible_moves, fullmove_srcdst, fullmove_dstsrc,
 			false, true, &get_avoid());
 
-	/*adjacent_loc_array_t leader_adj;
-	if(leader != units_.end()) {
-		get_adjacent_tiles(leader->get_location(), leader_adj.data());
-	}*/
-	//int leader_adj_count = 0;
 	std::vector<map_location> leaders_adj_v;
 	for (unit_map::const_iterator leader : leaders) {
-		adjacent_loc_array_t tmp_leader_adj;
-		get_adjacent_tiles(leader->get_location(), tmp_leader_adj.data());
-		for (map_location &loc : tmp_leader_adj) {
+		for(const map_location& loc : get_adjacent_tiles(leader->get_location())) {
 			bool found = false;
 			for (map_location &new_loc : leaders_adj_v) {
 				if(new_loc == loc){
@@ -1429,13 +1435,12 @@ double retreat_phase::evaluate()
 	}
 	//leader_adj_count = leaders_adj_v.size();
 
-
 	for(unit_map::const_iterator i = units_.begin(); i != units_.end(); ++i) {
 		if (i->side() == get_side() &&
 		    i->movement_left() == i->total_movement() &&
 		    //leaders.find(*i) == leaders.end() && //unit_map::const_iterator(i) != leader &&
 		    std::find(leaders.begin(), leaders.end(), i) == leaders.end() &&
-		    !i->incapacitated())
+		    !i->incapacitated() && is_allowed_unit(*i))
 		{
 			// This unit still has movement left, and is a candidate to retreat.
 			// We see the amount of power of each side on the situation,
@@ -1520,8 +1525,6 @@ void retreat_phase::execute()
 	}
 }
 
-
-
 bool retreat_phase::should_retreat(const map_location& loc, const unit_map::const_iterator& un,  const move_map &srcdst, const move_map &dstsrc, double caution)
 {
 	const move_map &enemy_dstsrc = get_enemy_dstsrc();
@@ -1544,7 +1547,6 @@ bool retreat_phase::should_retreat(const map_location& loc, const unit_map::cons
 	return caution*their_power*(1.0+exposure) > our_power;
 }
 
-
 //==============================================================
 
 leader_control_phase::leader_control_phase( rca_context &context, const config &cfg )
@@ -1552,7 +1554,6 @@ leader_control_phase::leader_control_phase( rca_context &context, const config &
 {
 }
 
-
 leader_control_phase::~leader_control_phase()
 {
 }
@@ -1563,8 +1564,6 @@ double leader_control_phase::evaluate()
 	return BAD_SCORE;
 }
 
-
-
 void leader_control_phase::execute()
 {
 	ERR_AI_TESTING_AI_DEFAULT << get_name() << ": execute - not yet implemented" << std::endl;
@@ -1583,9 +1582,18 @@ leader_shares_keep_phase::~leader_shares_keep_phase()
 
 double leader_shares_keep_phase::evaluate()
 {
-	if(get_passive_leader() && !get_passive_leader_shares_keep()){
+	bool have_active_leader = false;
+	std::vector<unit_map::unit_iterator> ai_leaders = resources::gameboard->units().find_leaders(get_side());
+	for (unit_map::unit_iterator &ai_leader : ai_leaders) {
+	        if (!is_passive_leader(ai_leader->id()) || is_passive_keep_sharing_leader(ai_leader->id())) {
+			have_active_leader = true;
+			break;
+		}
+	}
+	if(!have_active_leader) {
 		return BAD_SCORE;
 	}
+
 	bool allied_leaders_available = false;
 	for(team &tmp_team : resources::gameboard->teams()) {
 		if(!current_team().is_enemy(tmp_team.side())){
@@ -1615,7 +1623,7 @@ void leader_shares_keep_phase::execute()
 
 	//check for each ai leader if he should move away from his keep
 	for (unit_map::unit_iterator &ai_leader : ai_leaders) {
-		if(!ai_leader.valid()) {
+		if(!ai_leader.valid() || !is_allowed_unit(*ai_leader) || (is_passive_leader(ai_leader->id()) && !is_passive_keep_sharing_leader(ai_leader->id()))) {
 			//This can happen if wml killed or moved a leader during a movement events of another leader
 			continue;
 		}
@@ -1684,18 +1692,11 @@ void leader_shares_keep_phase::execute()
 		}
 		ai_leader->remove_movement_ai();
 	}
-	// re-get the AI leaders, in case an event did something
-	ai_leaders = resources::gameboard->units().find_leaders(get_side());
-	for(unit_map::unit_iterator &leader : ai_leaders) {
-		leader->remove_movement_ai();
-	}
 	//ERR_AI_TESTING_AI_DEFAULT << get_name() << ": evaluate - not yet implemented" << std::endl;
 }
 
-
 //==============================================================
 
-
 } //end of namespace testing_ai_default
 
 } //end of namespace ai
