@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -42,8 +43,6 @@ static lg::log_domain log_ai_testing_ca_move_to_targets("ai/ca/move_to_targets")
 #define WRN_AI LOG_STREAM(warn, log_ai_testing_ca_move_to_targets)
 #define ERR_AI LOG_STREAM(err, log_ai_testing_ca_move_to_targets)
 
-
-
 struct move_cost_calculator : pathfind::cost_calculator
 {
 	move_cost_calculator(const unit& u, const gamemap& map,
@@ -85,7 +84,6 @@ private:
 	const bool avoid_enemies_;
 };
 
-
 class remove_wrong_targets {
 public:
 	remove_wrong_targets(const readonly_context &context)
@@ -122,24 +120,21 @@ move_to_targets_phase::move_to_targets_phase( rca_context &context, const config
 {
 }
 
-
 move_to_targets_phase::~move_to_targets_phase()
 {
 }
 
-
 double move_to_targets_phase::evaluate()
 {
 	return get_score();
 }
 
-
 void move_to_targets_phase::execute()
 {
 	unit_map::const_iterator leader = resources::gameboard->units().find_leader(get_side());
 	LOG_AI << "finding targets...\n";
 	std::vector<target> targets;
-	for(;;) {
+	while(true) {
 		if(targets.empty()) {
 			targets = find_targets(get_enemy_dstsrc());
 			targets.insert(targets.end(),additional_targets().begin(),
@@ -182,10 +177,6 @@ void move_to_targets_phase::execute()
 	}
 }
 
-
-
-
-
 // structure storing the maximal possible rating of a target
 struct rated_target{
 	rated_target(const std::vector<target>::iterator& t, double r) : tg(t), max_rating(r) {}
@@ -201,7 +192,6 @@ struct rated_target_comparer {
 	}
 };
 
-
 double move_to_targets_phase::rate_target(const target& tg, const unit_map::iterator& u,
 			const move_map& dstsrc, const move_map& enemy_dstsrc,
 			const pathfind::plain_route& rt)
@@ -266,8 +256,6 @@ double move_to_targets_phase::rate_target(const target& tg, const unit_map::iter
 	return rating;
 }
 
-
-
 std::pair<map_location,map_location> move_to_targets_phase::choose_move(std::vector<target>& targets)
 {
 	log_scope2(log_ai_testing_ca_move_to_targets, "choosing move");
@@ -280,17 +268,17 @@ std::pair<map_location,map_location> move_to_targets_phase::choose_move(std::vec
 
 	//take care of all the guardians first
 	for(u = units_.begin(); u != units_.end(); ++u) {
-		if (!(u->side() != get_side() || (u->can_recruit() && !get_leader_ignores_keep()) || u->movement_left() <= 0 || u->incapacitated())) {
+		if (!(u->side() != get_side() || (u->can_recruit() && !is_keep_ignoring_leader(u->id())) || u->movement_left() <= 0 || u->incapacitated())) {
 			if (u->get_state("guardian")) {
 				LOG_AI << u->type_id() << " is guardian, staying still\n";
-				return std::make_pair(u->get_location(), u->get_location());
+				return std::pair(u->get_location(), u->get_location());
 			}
 		}
 	}
 
 	//now find the first eligible remaining unit
 	for(u = units_.begin(); u != units_.end(); ++u) {
-		if (!(u->side() != get_side() || (u->can_recruit() && !get_leader_ignores_keep()) || u->movement_left() <= 0 || u->incapacitated())) {
+		if (!(u->side() != get_side() || (u->can_recruit() && !is_keep_ignoring_leader(u->id())) || u->movement_left() <= 0 || u->incapacitated() || !is_allowed_unit(*u))) {
 			break;
 		}
 	}
@@ -376,7 +364,7 @@ std::pair<map_location,map_location> move_to_targets_phase::choose_move(std::vec
 
 	if(best_rated_target == rated_targets.end()) {
 		LOG_AI << "no eligible targets found for unit at " << u->get_location() << std::endl;
-		return std::make_pair(u->get_location(), u->get_location());
+		return std::pair(u->get_location(), u->get_location());
 	}
 
 	assert(best_rating >= 0);
@@ -391,9 +379,9 @@ std::pair<map_location,map_location> move_to_targets_phase::choose_move(std::vec
 		LOG_AI << "complex targeting...\n";
 		//now see if any other unit can put a better bid forward
 		for(++u; u != units_.end(); ++u) {
-			if (u->side() != get_side() || (u->can_recruit() && !get_leader_ignores_keep()) ||
+			if (u->side() != get_side() || (u->can_recruit() && !is_keep_ignoring_leader(u->id())) ||
 			    u->movement_left() <= 0 || u->get_state("guardian") ||
-			    u->incapacitated())
+			    u->incapacitated() || !is_allowed_unit(*u))
 			{
 				continue;
 			}
@@ -402,12 +390,9 @@ std::pair<map_location,map_location> move_to_targets_phase::choose_move(std::vec
 
 			const move_cost_calculator calc(*u, map_, units_, enemy_dstsrc);
 
-			///@todo 1.9: lower this value for perf,
-			// but best_rating is too big for scout and support
-			// which give a too small locStopValue
-			// so keep costy A* for the moment.
-			//const double locStopValue = std::min(best_target->value / best_rating, (double) 100.0);
-
+			// locStopValue controls how quickly we give up on the A* search, due
+			// to it seeming futile. Be very cautious about changing this value,
+			// as it can cause the AI to give up on searches and just do nothing.
 			const double locStopValue = 500.0;
 			const pathfind::teleport_map allowed_teleports = pathfind::get_teleport_locations(*u, current_team());
 			pathfind::plain_route cur_route = pathfind::a_star_search(u->get_location(), best_target->loc, locStopValue, calc, map_.w(), map_.h(), &allowed_teleports);
@@ -462,7 +447,7 @@ std::pair<map_location,map_location> move_to_targets_phase::choose_move(std::vec
 			}
 
 			LOG_AI << "returning support...\n";
-			return std::make_pair(best->get_location(), best_loc);
+			return std::pair(best->get_location(), best_loc);
 		}
 	}
 
@@ -519,7 +504,7 @@ std::pair<map_location,map_location> move_to_targets_phase::choose_move(std::vec
 				LOG_AI << "group didn't move " << group.size() << "\n";
 
 				//the group didn't move, so end the first unit in the group's turn, to prevent an infinite loop
-				return std::make_pair(best->get_location(), best->get_location());
+				return std::pair(best->get_location(), best->get_location());
 
 			}
 		} else {
@@ -620,7 +605,7 @@ std::pair<map_location,map_location> move_to_targets_phase::choose_move(std::vec
 		//create this as a target, attempting to rally units around
 		targets.emplace_back(best->get_location(), best_target->value);
 		best_target = targets.end() - 1;
-		return std::make_pair(best->get_location(), best->get_location());
+		return std::pair(best->get_location(), best->get_location());
 	}
 
 	LOG_AI << "Could not find anywhere to move!\n";
@@ -652,7 +637,6 @@ void move_to_targets_phase::access_points(const move_map& srcdst, const map_loca
 	}
 }
 
-
 double move_to_targets_phase::compare_groups(const std::set<map_location>& our_group, const std::set<map_location>& their_group, const std::vector<map_location>& battlefield) const
 {
 	const double a = rate_group(our_group,battlefield);
@@ -660,14 +644,11 @@ double move_to_targets_phase::compare_groups(const std::set<map_location>& our_g
 	return a/b;
 }
 
-
 void move_to_targets_phase::enemies_along_path(const std::vector<map_location>& route, const move_map& dstsrc, std::set<map_location>& res)
 {
 	for(std::vector<map_location>::const_iterator i = route.begin(); i != route.end(); ++i) {
-		adjacent_loc_array_t adj;
-		get_adjacent_tiles(*i,adj.data());
-		for(size_t n = 0; n < adj.size(); ++n) {
-			const std::pair<move_map::const_iterator,move_map::const_iterator> itors = dstsrc.equal_range(adj[n]);
+		for(const map_location& adj : get_adjacent_tiles(*i)) {
+			const std::pair<move_map::const_iterator,move_map::const_iterator> itors = dstsrc.equal_range(adj);
 			for(move_map::const_iterator j = itors.first; j != itors.second; ++j) {
 				res.insert(j->second);
 			}
@@ -675,7 +656,6 @@ void move_to_targets_phase::enemies_along_path(const std::vector<map_location>&
 	}
 }
 
-
 map_location move_to_targets_phase::form_group(const std::vector<map_location>& route, const move_map& dstsrc, std::set<map_location>& res)
 {
 	unit_map &units_ = resources::gameboard->units();
@@ -689,7 +669,7 @@ map_location move_to_targets_phase::form_group(const std::vector<map_location>&
 			continue;
 		}
 
-		size_t n = 0, nunits = res.size();
+		std::size_t n = 0, nunits = res.size();
 
 		const std::pair<move_map::const_iterator,move_map::const_iterator> itors = dstsrc.equal_range(*i);
 		for(move_map::const_iterator j = itors.first; j != itors.second; ++j) {
@@ -697,7 +677,7 @@ map_location move_to_targets_phase::form_group(const std::vector<map_location>&
 				++n;
 			} else {
 				const unit_map::const_iterator un = units_.find(j->second);
-				if(un == units_.end() || (un->can_recruit() && !get_leader_ignores_keep()) || un->movement_left() < un->total_movement()) {
+				if(un == units_.end() || (un->can_recruit() && !is_keep_ignoring_leader(un->id())) || un->movement_left() < un->total_movement()) {
 					continue;
 				}
 
@@ -718,7 +698,6 @@ map_location move_to_targets_phase::form_group(const std::vector<map_location>&
 	return *i;
 }
 
-
 bool move_to_targets_phase::move_group(const map_location& dst, const std::vector<map_location>& route, const std::set<map_location>& units)
 {
 	unit_map &units_ = resources::gameboard->units();
@@ -733,7 +712,7 @@ bool move_to_targets_phase::move_group(const map_location& dst, const std::vecto
 
 	map_location next;
 
-	size_t direction = 0;
+	std::size_t direction = 0;
 
 	//find the direction the group is moving in
 	if(itor+1 != route.end()) {
@@ -743,9 +722,7 @@ bool move_to_targets_phase::move_group(const map_location& dst, const std::vecto
 	}
 
 	if(next.valid()) {
-		adjacent_loc_array_t adj;
-		get_adjacent_tiles(dst,adj.data());
-
+		const auto adj = get_adjacent_tiles(dst);
 		direction = std::distance(adj.begin(), std::find(adj.begin(), adj.end(), next));
 	}
 
@@ -794,7 +771,6 @@ bool move_to_targets_phase::move_group(const map_location& dst, const std::vecto
 			move_result_ptr move_res = execute_move_action(*i,best_loc);
 			gamestate_changed |= move_res->is_gamestate_changed();
 
-
 			//if we were ambushed or something went wrong,  abort the group's movement.
 			if (!move_res->is_ok()) {
 				return gamestate_changed;
@@ -803,9 +779,8 @@ bool move_to_targets_phase::move_group(const map_location& dst, const std::vecto
 			preferred_moves.erase(std::find(preferred_moves.begin(),preferred_moves.end(),best_loc));
 
 			//find locations that are 'perpendicular' to the direction of movement for further units to move to.
-			adjacent_loc_array_t adj;
-			get_adjacent_tiles(best_loc,adj.data());
-			for(size_t n = 0; n < adj.size(); ++n) {
+			const auto adj = get_adjacent_tiles(best_loc);
+			for(std::size_t n = 0; n < adj.size(); ++n) {
 				if(n != direction && ((n+3)%6) != direction && map_.on_board(adj[n]) &&
 				   units_.count(adj[n]) == 0 && std::count(preferred_moves.begin(),preferred_moves.end(),adj[n]) == 0) {
 					preferred_moves.push_front(adj[n]);
@@ -820,7 +795,6 @@ bool move_to_targets_phase::move_group(const map_location& dst, const std::vecto
 	return gamestate_changed;
 }
 
-
 double move_to_targets_phase::rate_group(const std::set<map_location>& group, const std::vector<map_location>& battlefield) const
 {
 	unit_map &units_ = resources::gameboard->units();
@@ -855,8 +829,6 @@ double move_to_targets_phase::rate_group(const std::set<map_location>& group, co
 	return strength;
 }
 
-
-
 bool move_to_targets_phase::should_retreat(const map_location& loc, const unit_map::const_iterator& un,
 		const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_dstsrc,
 		double caution)
