@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2013 - 2018 by Felix Bauer
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2013 - 2021
+	by Felix Bauer
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
- */
+	See the COPYING file for more details.
+*/
 
 /**
  * @file
@@ -49,6 +50,9 @@ static lg::log_domain log_ai_recruitment("ai/recruitment");
 #define LOG_AI_RECRUITMENT LOG_STREAM(info, log_ai_recruitment)
 #define ERR_AI_RECRUITMENT LOG_STREAM(err, log_ai_recruitment)
 
+static lg::log_domain log_wml("wml");
+#define ERR_WML LOG_STREAM(err, log_wml)
+
 namespace ai {
 
 namespace default_recruitment {
@@ -112,20 +116,20 @@ std::string data::to_string() const {
 }
 
 recruitment::recruitment(rca_context& context, const config& cfg)
-		: candidate_action(context, cfg),
-		  important_hexes_(),
-		  important_terrain_(),
-		  own_units_in_combat_counter_(0),
-		  average_local_cost_(),
-		  cheapest_unit_costs_(),
-		  combat_cache_(),
-		  recruit_situation_change_observer_(),
-		  average_lawful_bonus_(0.0),
-		  recruitment_instructions_(),
-		  recruitment_instructions_turn_(-1),
-		  own_units_count_(),
-		  total_own_units_(0),
-		  scouts_wanted_(0)
+	: candidate_action(context, cfg),
+	important_hexes_(),
+	important_terrain_(),
+	own_units_in_combat_counter_(0),
+	average_local_cost_(),
+	cheapest_unit_costs_(),
+	combat_cache_(),
+	recruit_situation_change_observer_(),
+	average_lawful_bonus_(0.0),
+	recruitment_instructions_(),
+	recruitment_instructions_turn_(-1),
+	own_units_count_(),
+	total_own_units_(0),
+	scouts_wanted_(0)
 {
 	if (cfg["state"] == "save_gold") {
 		state_ = SAVE_GOLD;
@@ -176,6 +180,12 @@ double recruitment::evaluate() {
 	const std::vector<unit_map::const_iterator> leaders = units.find_leaders(get_side());
 
 	for (const unit_map::const_iterator& leader : leaders) {
+		// Need to check this here, otherwise recruiting might be blacklisted
+		// if no allowed leader is on a keep yet
+		if (!is_allowed_unit(*leader)) {
+			continue;
+		}
+
 		if (leader == resources::gameboard->units().end()) {
 			return BAD_SCORE;
 		}
@@ -215,6 +225,10 @@ void recruitment::execute() {
 
 	for (const unit_map::const_iterator& leader : leaders) {
 		const map_location& keep = leader->get_location();
+		if (!is_allowed_unit(*leader)) {
+			LOG_AI_RECRUITMENT << "Leader " << leader->name() << " is not allowed recruiter. \n";
+			continue;
+		}
 		if (!resources::gameboard->map().is_keep(keep)) {
 			LOG_AI_RECRUITMENT << "Leader " << leader->name() << " is not on keep. \n";
 			continue;
@@ -236,7 +250,8 @@ void recruitment::execute() {
 		// Add team recruits.
 		for (const std::string& recruit : current_team().recruits()) {
 			if (!unit_types.find(recruit)) {
-				lg::wml_error() << "Unit-type \"" << recruit << "\" doesn't exist.\n";
+				lg::log_to_chat() << "Unit-type \"" << recruit << "\" doesn't exist.\n";
+				ERR_WML << "Unit-type \"" << recruit << "\" doesn't exist.";
 			}
 			data.recruits.insert(recruit);
 			data.scores[recruit] = 0.0;
@@ -246,7 +261,8 @@ void recruitment::execute() {
 		// Add extra recruits.
 		for (const std::string& recruit : leader->recruits()) {
 			if (!unit_types.find(recruit)) {
-				lg::wml_error() << "Unit-type \"" << recruit << "\" doesn't exist.\n";
+				lg::log_to_chat() << "Unit-type \"" << recruit << "\" doesn't exist.\n";
+				ERR_WML << "Unit-type \"" << recruit << "\" doesn't exist.";
 			}
 			data.recruits.insert(recruit);
 			data.scores[recruit] = 0.0;
@@ -256,7 +272,7 @@ void recruitment::execute() {
 		// Add recalls.
 		// Recalls are treated as recruits. While recruiting
 		// we'll check if we can do a recall instead of a recruitment.
-		for (const unit_const_ptr & recall : current_team().recall_list()) {
+		for (const unit_const_ptr recall : current_team().recall_list()) {
 			// Check if this leader is allowed to recall this unit.
 			const unit_filter ufilt( vconfig(leader->recall_filter()));
 			if (!ufilt(*recall, map_location::null_location())) {
@@ -513,7 +529,7 @@ const std::string* recruitment::get_appropriate_recall(const std::string& type,
 		const data& leader_data) const {
 	const std::string* best_recall_id = nullptr;
 	double best_recall_value = -1;
-	for (const unit_const_ptr & recall_unit : current_team().recall_list()) {
+	for (const unit_const_ptr recall_unit : current_team().recall_list()) {
 		if (type != recall_unit->type_id()) {
 			continue;
 		}
@@ -632,30 +648,30 @@ void recruitment::compare_cost_maps_and_update_important_hexes(
 	border_cost_map important_hexes_candidates;
 	double smallest_border_movecost = 999999;
 	double biggest_border_movecost = 0;
-	for(int x = 0; x < map.w(); ++x) {
-		for (int y = 0; y < map.h(); ++y) {
-			double my_cost_average = my_cost_map.get_average_cost_at(x, y);
-			double enemy_cost_average = enemy_cost_map.get_average_cost_at(x, y);
-			if (my_cost_average == -1 || enemy_cost_average == -1) {
-				continue;
-			}
-			// We multiply the threshold MAP_BORDER_THICKNESS by the average_local_cost
-			// to favor high cost hexes (a bit).
-			if (std::abs(my_cost_average - MAP_OFFENSIVE_SHIFT - enemy_cost_average) <
-					MAP_BORDER_THICKNESS * average_local_cost_[map_location(x, y)]) {
-				double border_movecost = (my_cost_average + enemy_cost_average) / 2;
 
-				important_hexes_candidates[map_location(x, y)] = border_movecost;
+	map.for_each_walkable_loc([&](map_location loc) {
+		double my_cost_average = my_cost_map.get_average_cost_at(loc);
+		double enemy_cost_average = enemy_cost_map.get_average_cost_at(loc);
+		if (my_cost_average == -1 || enemy_cost_average == -1) {
+			return;
+		}
+		// We multiply the threshold MAP_BORDER_THICKNESS by the average_local_cost
+		// to favor high cost hexes (a bit).
+		if (std::abs(my_cost_average - MAP_OFFENSIVE_SHIFT - enemy_cost_average) <
+				MAP_BORDER_THICKNESS * average_local_cost_[loc]) {
+			double border_movecost = (my_cost_average + enemy_cost_average) / 2;
+			important_hexes_candidates[loc] = border_movecost;
 
-				if (border_movecost < smallest_border_movecost) {
-					smallest_border_movecost = border_movecost;
-				}
-				if (border_movecost > biggest_border_movecost) {
-					biggest_border_movecost = border_movecost;
-				}
+			if (border_movecost < smallest_border_movecost) {
+				smallest_border_movecost = border_movecost;
 			}
-		}  // for
-	}  // for
+			if (border_movecost > biggest_border_movecost) {
+				biggest_border_movecost = border_movecost;
+			}
+		}
+
+	});
+
 	double threshold = (biggest_border_movecost - smallest_border_movecost) *
 			MAP_BORDER_WIDTH + smallest_border_movecost;
 	for (const border_cost_map::value_type& candidate : important_hexes_candidates) {
@@ -758,7 +774,7 @@ void recruitment::update_average_lawful_bonus() {
 		++counter;
 	}
 	if (counter > 0) {
-		average_lawful_bonus_ = round_double(static_cast<double>(sum) / counter);
+		average_lawful_bonus_ = std::round(static_cast<double>(sum) / counter);
 	}
 }
 
@@ -860,7 +876,7 @@ void recruitment::update_important_hexes() {
 		get_tiles_in_radius(village, MAP_VILLAGE_SURROUNDING, surrounding);
 		for (const map_location& hex : surrounding) {
 			// only add hex if one of our units can reach the hex
-			if (map.on_board(hex) && my_cost_map.get_cost_at(hex.x, hex.y) != -1) {
+			if (map.on_board(hex) && my_cost_map.get_cost_at(hex) != -1) {
 				important_hexes_.insert(hex);
 			}
 		}
@@ -989,7 +1005,7 @@ void recruitment::do_combat_analysis(std::vector<data>* leader_data) {
 			for (const std::string& recruit : leader.recruits) {
 				double score = compare_unit_types(recruit, enemy_unit);
 				score *= enemy_unit_hp;
-				score = pow(score, COMBAT_SCORE_POWER);
+				score = std::pow(score, COMBAT_SCORE_POWER);
 				temp_scores[recruit] += score;
 			}
 		}
@@ -1083,9 +1099,9 @@ struct attack_simulation {
 			attacker_type(attacker),
 			defender_type(defender),
 			attacker_stats(attacker, att_weapon, true, defender, def_weapon,
-					round_double(defender_defense), average_lawful_bonus),
+					std::round(defender_defense), average_lawful_bonus),
 			defender_stats(defender, def_weapon, false, attacker, att_weapon,
-					round_double(attacker_defense), average_lawful_bonus),
+					std::round(attacker_defense), average_lawful_bonus),
 			attacker_combatant(attacker_stats),
 			defender_combatant(defender_stats)
 	{
@@ -1433,7 +1449,7 @@ bool recruitment::remove_job_if_no_blocker(config* job) {
  */
 double recruitment::get_estimated_income(int turns) const {
 	const team& team = resources::gameboard->get_team(get_side());
-	const size_t own_villages = team.villages().size();
+	const std::size_t own_villages = team.villages().size();
 	const double village_gain = get_estimated_village_gain();
 	const double unit_gain = get_estimated_unit_gain();
 
@@ -1464,7 +1480,7 @@ double recruitment::get_estimated_village_gain() const {
 	const gamemap& map = resources::gameboard->map();
 	int neutral_villages = 0;
 	for (const map_location& village : map.villages()) {
-		if (resources::gameboard->village_owner(village) == -1) {
+		if (resources::gameboard->village_owner(village) == 0) {
 			++neutral_villages;
 		}
 	}
@@ -1617,7 +1633,7 @@ void recruitment::do_similarity_penalty(std::vector<data>* leader_data) const {
  * Called at the beginning and whenever the recruitment list changes.
  */
 int recruitment::get_cheapest_unit_cost_for_leader(const unit_map::const_iterator& leader) {
-	std::map<size_t, int>::const_iterator it = cheapest_unit_costs_.find(leader->underlying_id());
+	std::map<std::size_t, int>::const_iterator it = cheapest_unit_costs_.find(leader->underlying_id());
 	if (it != cheapest_unit_costs_.end()) {
 		return it->second;
 	}
@@ -1730,7 +1746,7 @@ void recruitment::update_scouts_wanted() {
 	// We recruit the initial allocation of scouts
 	// based on how many neutral villages there are.
 	for (const map_location& village : resources::gameboard->map().villages()) {
-		if (resources::gameboard->village_owner(village) == -1) {
+		if (resources::gameboard->village_owner(village) == 0) {
 			++neutral_villages;
 		}
 	}
@@ -1742,7 +1758,7 @@ void recruitment::update_scouts_wanted() {
 	// making us get twice as many scouts.
 	double villages_per_scout = (VILLAGE_PER_SCOUT_MULTIPLICATOR * get_villages_per_scout()) / 2;
 
-	scouts_wanted_ = (villages_per_scout > 0) ? round_double(our_share / villages_per_scout) : 0;
+	scouts_wanted_ = (villages_per_scout > 0) ? std::round(our_share / villages_per_scout) : 0;
 
 	if (scouts_wanted_ == 0) {
 		return;
@@ -1824,9 +1840,9 @@ recruitment_aspect::recruitment_aspect(readonly_context &context, const config &
 		create_limit(limits_, lim);
 	}
 	std::function<void(std::vector<std::shared_ptr<recruit_job>>&, const config&)> factory_jobs =
-		std::bind(&recruitment_aspect::create_job,*this,_1,_2);
+		std::bind(&recruitment_aspect::create_job, *this, std::placeholders::_1, std::placeholders::_2);
 	std::function<void(std::vector<std::shared_ptr<recruit_limit>>&, const config&)> factory_limits =
-		std::bind(&recruitment_aspect::create_limit,*this,_1,_2);
+		std::bind(&recruitment_aspect::create_limit, *this, std::placeholders::_1, std::placeholders::_2);
 	register_vector_property(property_handlers(), "recruit", jobs_, factory_jobs);
 	register_vector_property(property_handlers(), "limit", limits_, factory_limits);
 }
