@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -51,6 +52,7 @@
 #include "ai/formula/function_table.hpp"           // for ai_function_symbol_table
 #include "ai/game_info.hpp"  // for move_result_ptr, move_map, etc
 #include "ai/formula/candidates.hpp"               // for base_candidate_action, etc
+#include "utils/variant.hpp"
 
 #include <cassert>                     // for assert
 #include <ctime>                       // for time
@@ -65,7 +67,6 @@ static lg::log_domain log_formula_ai("ai/engine/fai");
 #define WRN_AI LOG_STREAM(warn, log_formula_ai)
 #define ERR_AI LOG_STREAM(err, log_formula_ai)
 
-
 using namespace wfl;
 
 namespace ai {
@@ -96,7 +97,6 @@ int formula_ai::get_recursion_count() const{
 	return recursion_counter_.get_count();
 }
 
-
 formula_ai::formula_ai(readonly_context &context, const config &cfg)
 	:
 	readonly_context_proxy(),
@@ -135,7 +135,7 @@ void formula_ai::handle_exception(const formula_error& e, const std::string& fai
 
 void formula_ai::display_message(const std::string& msg) const
 {
-	game_display::get_singleton()->get_chat_manager().add_chat_message(time(nullptr), "wfl", get_side(), msg,
+	game_display::get_singleton()->get_chat_manager().add_chat_message(std::time(nullptr), "wfl", get_side(), msg,
 				events::chat_handler::MESSAGE_PUBLIC, false);
 
 }
@@ -150,13 +150,11 @@ formula_ptr formula_ai::create_optional_formula(const std::string& formula_strin
 	}
 }
 
-
 void formula_ai::set_ai_context(ai_context *context)
 {
 	ai_ptr_ = context;
 }
 
-
 std::string formula_ai::evaluate(const std::string& formula_str)
 {
 	try{
@@ -191,7 +189,7 @@ wfl::variant formula_ai::make_action(wfl::const_formula_ptr formula_, const wfl:
 		throw formula_error("null formula passed to make_action","","formula",0);
 	}
 	LOG_AI << "do move...\n";
-	const variant var = formula_->evaluate(variables);///@todo 1.9 add formula_debugger
+	const variant var = formula_->evaluate(variables);
 	variant res;
 
 	if (ai_ptr_) {
@@ -221,10 +219,9 @@ pathfind::plain_route formula_ai::shortest_path_calculator(const map_location &s
         const map_location::DIRECTION preferred = destination.get_relative_dir(src);
 
         int best_rating = 100;//smaller is better
-        adjacent_loc_array_t adj;
-        get_adjacent_tiles(destination,adj.data());
+		const auto adj = get_adjacent_tiles(destination);
 
-        for(size_t n = 0; n < adj.size(); ++n) {
+        for(std::size_t n = 0; n < adj.size(); ++n) {
                 if(resources::gameboard->map().on_board(adj[n]) == false) {
                         continue;
                 }
@@ -233,8 +230,8 @@ pathfind::plain_route formula_ai::shortest_path_calculator(const map_location &s
                         continue;
                 }
 
-                static const size_t NDIRECTIONS = map_location::NDIRECTIONS;
-                unsigned int difference = std::abs(int(preferred - n));
+                static const std::size_t NDIRECTIONS = map_location::NDIRECTIONS;
+                unsigned int difference = std::abs(static_cast<int>(preferred - n));
                 if(difference > NDIRECTIONS/2) {
                         difference = NDIRECTIONS - difference;
                 }
@@ -264,14 +261,14 @@ pathfind::teleport_map formula_ai::get_allowed_teleports(unit_map::iterator& uni
 
 void formula_ai::add_formula_function(const std::string& name, const_formula_ptr formula, const_formula_ptr precondition, const std::vector<std::string>& args)
 {
-	formula_function_ptr fcn(new user_formula_function(name,formula,precondition,args));
-	function_table_.add_function(name, std::move(fcn));
+	function_table_.add_function(name, std::make_shared<user_formula_function>(name, formula, precondition, args));
 }
 
-namespace {
+namespace
+{
 template<typename Container>
-variant villages_from_set(const Container& villages,
-				          const std::set<map_location>* exclude=nullptr) {
+variant villages_from_set(const Container& villages, const std::set<map_location>* exclude = nullptr)
+{
 	std::vector<variant> vars;
 	for(const map_location& loc : villages) {
 		if(exclude && exclude->count(loc)) {
@@ -282,7 +279,22 @@ variant villages_from_set(const Container& villages,
 
 	return variant(vars);
 }
+
+// TODO: I have no damn idea what to name this function
+variant visit_helper(const utils::variant<bool, std::vector<std::string>>& input)
+{
+	return utils::visit(
+		[](const auto& v) {
+			if constexpr(utils::decayed_is_same<bool, decltype(v)>) {
+				return variant(v);
+			} else {
+				const std::vector<variant> vars(v.begin(), v.end());
+				return variant(vars);
+			}
+		},
+		input);
 }
+} // namespace
 
 variant formula_ai::get_value(const std::string& key) const
 {
@@ -292,10 +304,6 @@ variant formula_ai::get_value(const std::string& key) const
 	{
 		return variant(get_aggression()*1000,variant::DECIMAL_VARIANT);
 
-	} else if(key == "attack_depth")
-	{
-		return variant(get_attack_depth());
-
 	} else if(key == "avoid")
 	{
 		std::set<map_location> av_locs;
@@ -316,7 +324,7 @@ variant formula_ai::get_value(const std::string& key) const
 
 	} else if(key == "leader_ignores_keep")
 	{
-		return variant(get_leader_ignores_keep());
+		return visit_helper(get_leader_ignores_keep());
 
 	} else if(key == "leader_value")
 	{
@@ -324,11 +332,11 @@ variant formula_ai::get_value(const std::string& key) const
 
 	} else if(key == "passive_leader")
 	{
-		return variant(get_passive_leader());
+		return visit_helper(get_passive_leader());
 
 	} else if(key == "passive_leader_shares_keep")
 	{
-		return variant(get_passive_leader_shares_keep());
+		return visit_helper(get_passive_leader_shares_keep());
 
 	} else if(key == "recruitment_pattern")
 	{
@@ -339,6 +347,14 @@ variant formula_ai::get_value(const std::string& key) const
 		}
 		return variant(vars);
 
+	} else if(key == "retreat_enemy_weight")
+	{
+		return variant(get_retreat_enemy_weight()*1000,variant::DECIMAL_VARIANT);
+
+	} else if(key == "retreat_factor")
+	{
+		return variant(get_retreat_factor()*1000,variant::DECIMAL_VARIANT);
+
 	} else if(key == "scout_village_targeting")
 	{
 		return variant(get_scout_village_targeting()*1000,variant::DECIMAL_VARIANT);
@@ -386,7 +402,7 @@ variant formula_ai::get_value(const std::string& key) const
 	} else if(key == "allies")
 	{
 		std::vector<variant> vars;
-		for( size_t i = 0; i < resources::gameboard->teams().size(); ++i) {
+		for( std::size_t i = 0; i < resources::gameboard->teams().size(); ++i) {
 			if ( !current_team().is_enemy( i+1 ) )
 				vars.emplace_back(i);
 		}
@@ -395,7 +411,7 @@ variant formula_ai::get_value(const std::string& key) const
 	} else if(key == "enemies")
 	{
 		std::vector<variant> vars;
-		for( size_t i = 0; i < resources::gameboard->teams().size(); ++i) {
+		for( std::size_t i = 0; i < resources::gameboard->teams().size(); ++i) {
 			if ( current_team().is_enemy( i+1 ) )
 				vars.emplace_back(i);
 		}
@@ -428,7 +444,7 @@ variant formula_ai::get_value(const std::string& key) const
 
 		unit_types.build_all(unit_type::FULL);
 
-		for( size_t i = 0; i<resources::gameboard->teams().size(); ++i)
+		for( std::size_t i = 0; i<resources::gameboard->teams().size(); ++i)
 		{
 			std::vector<variant> v;
 			tmp.push_back( v );
@@ -447,7 +463,7 @@ variant formula_ai::get_value(const std::string& key) const
 			}
 		}
 
-		for( size_t i = 0; i<tmp.size(); ++i)
+		for( std::size_t i = 0; i<tmp.size(); ++i)
 			vars.emplace_back(tmp[i]);
 		return variant(vars);
 
@@ -463,7 +479,7 @@ variant formula_ai::get_value(const std::string& key) const
 	{
 		std::vector<variant> vars;
 		std::vector< std::vector< variant>> tmp;
-		for( size_t i = 0; i<resources::gameboard->teams().size(); ++i)
+		for( std::size_t i = 0; i<resources::gameboard->teams().size(); ++i)
 		{
 			std::vector<variant> v;
 			tmp.push_back( v );
@@ -471,7 +487,7 @@ variant formula_ai::get_value(const std::string& key) const
 		for(const unit &u : units) {
 			tmp[u.side() - 1].emplace_back(std::make_shared<unit_callable>(u));
 		}
-		for( size_t i = 0; i<tmp.size(); ++i)
+		for( std::size_t i = 0; i<tmp.size(); ++i)
 			vars.emplace_back(tmp[i]);
 		return variant(vars);
 
@@ -541,11 +557,11 @@ variant formula_ai::get_value(const std::string& key) const
 	} else if(key == "villages_of_side")
 	{
 		std::vector<variant> vars;
-		for(size_t i = 0; i<resources::gameboard->teams().size(); ++i)
+		for(std::size_t i = 0; i<resources::gameboard->teams().size(); ++i)
 		{
 			vars.emplace_back();
 		}
-		for(size_t i = 0; i<vars.size(); ++i)
+		for(std::size_t i = 0; i<vars.size(); ++i)
 		{
 			vars[i] = villages_from_set(resources::gameboard->teams()[i].villages());
 		}
@@ -602,14 +618,12 @@ variant formula_ai::get_keeps() const
 {
 	if(keeps_cache_.is_null()) {
 		std::vector<variant> vars;
-		for(size_t x = 0; x != size_t(resources::gameboard->map().w()); ++x) {
-			for(size_t y = 0; y != size_t(resources::gameboard->map().h()); ++y) {
+		for(std::size_t x = 0; x != std::size_t(resources::gameboard->map().w()); ++x) {
+			for(std::size_t y = 0; y != std::size_t(resources::gameboard->map().h()); ++y) {
 				const map_location loc(x,y);
 				if(resources::gameboard->map().is_keep(loc)) {
-					adjacent_loc_array_t adj;
-					get_adjacent_tiles(loc,adj.data());
-					for(size_t n = 0; n < adj.size(); ++n) {
-						if(resources::gameboard->map().is_castle(adj[n])) {
+					for(const map_location& adj : get_adjacent_tiles(loc)) {
+						if(resources::gameboard->map().is_castle(adj)) {
 							vars.emplace_back(std::make_shared<location_callable>(loc));
 							break;
 						}
@@ -649,7 +663,7 @@ void formula_ai::on_create(){
 		const t_string &inputs = func["inputs"];
 		const t_string &formula_str = func["formula"];
 
-		std::vector<std::string> args = utils::split(inputs);
+		std::vector<std::string> args = utils::split(inputs.str());
 		try {
 			add_formula_function(name,
 					     create_optional_formula(formula_str),
@@ -661,7 +675,6 @@ void formula_ai::on_create(){
 		}
 	}
 
-
 	vars_ = map_formula_callable();
 	if (const config &ai_vars = cfg_.child("vars"))
 	{
@@ -672,10 +685,8 @@ void formula_ai::on_create(){
 		}
 	}
 
-
 }
 
-
 void formula_ai::evaluate_candidate_action(ca_ptr fai_ca)
 {
 	fai_ca->evaluate(this,resources::gameboard->units());
