@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -48,11 +49,11 @@ namespace wfl {
 typedef std::pair< unit_map::unit_iterator, int> unit_formula_pair;
 
 struct unit_formula_compare {
-        bool operator() (const unit_formula_pair& left,
-                        const unit_formula_pair& right) const
-        {
-                return left.second > right.second;
-        }
+	bool operator() (const unit_formula_pair& left,
+					const unit_formula_pair& right) const
+	{
+			return left.second > right.second;
+	}
 };
 
 typedef std::multiset< unit_formula_pair, unit_formula_compare > unit_formula_set;
