@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Bartosz Waresiak <dragonking@o2.pl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Bartosz Waresiak <dragonking@o2.pl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -21,13 +22,11 @@
 #include "ai/formula/candidates.hpp"
 #include "game_board.hpp"
 #include "log.hpp"
-#include "resources.hpp"
 #include "units/unit.hpp"
 
 static lg::log_domain log_formula_ai("ai/engine/fai");
 #define ERR_AI LOG_STREAM(err, log_formula_ai)
 
-
 namespace wfl {
 
 base_candidate_action::base_candidate_action(const std::string& name,
@@ -66,7 +65,7 @@ candidate_action_with_filters::candidate_action_with_filters(
 	const config & filter_params = cfg.child("filter");
 
 	if( filter_params ) {
-		for(const config::attribute filter_param : filter_params.attribute_range())
+		for(const config::attribute& filter_param : filter_params.attribute_range())
 		{
 			const_formula_ptr filter_formula(
 					new formula(filter_param.second, function_table));
@@ -166,7 +165,7 @@ void attack_candidate_action::evaluate(ai::formula_ai* ai, unit_map& units)
 			}
 		} else
 		{
-			if (ai->current_team().is_enemy(i->side()) && !i->incapacitated() && !i->invisible(i->get_location(), *resources::gameboard)) {
+			if (ai->current_team().is_enemy(i->side()) && !i->incapacitated() && !i->invisible(i->get_location())) {
 				enemy_res.emplace_back(std::make_shared<unit_callable>(*i));
 			}
 		}
@@ -221,9 +220,9 @@ void attack_candidate_action::evaluate(ai::formula_ai* ai, unit_map& units)
 		enemy_units_flt.emplace_back(u_callable);
 	}
 
-	for( size_t my_unit = 0 ; my_unit < my_units_flt.size() ; ++my_unit){
+	for( std::size_t my_unit = 0 ; my_unit < my_units_flt.size() ; ++my_unit){
 		auto my_unit_callable = my_units_flt[my_unit].convert_to<unit_callable>();
-		for( size_t enemy_unit = 0 ; enemy_unit < enemy_units_flt.size() ; ++enemy_unit){
+		for( std::size_t enemy_unit = 0 ; enemy_unit < enemy_units_flt.size() ; ++enemy_unit){
 			auto enemy_unit_callable = enemy_units_flt[enemy_unit].convert_to<unit_callable>();
 			if(ai->can_reach_unit(my_unit_callable->get_location(), enemy_unit_callable->get_location())) {
 
