@@ -1,18 +1,18 @@
 /*
-   Copyright (C) 2009 - 2018 by Bartosz Waresiak <dragonking@o2.pl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Bartosz Waresiak <dragonking@o2.pl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-
 #include <queue>
 #include <set>
 #include <utility>
@@ -28,6 +28,7 @@
 #include "attack_prediction.hpp"
 #include "filesystem.hpp"
 #include "game_board.hpp"
+#include "global.hpp"
 #include "display.hpp"
 #include "log.hpp"
 #include "map/label.hpp"
@@ -38,6 +39,7 @@
 #include "color.hpp"
 #include "terrain/filter.hpp"
 #include "units/unit.hpp"
+#include "units/types.hpp"
 #include "pathfind/pathfind.hpp"
 
 static lg::log_domain log_formula_ai("ai/engine/fai");
@@ -89,7 +91,6 @@ class unit_adapter {
 			}
 		}
 
-
 	private:
 		const unit_type *unit_type_;
 		const unit* unit_;
@@ -111,7 +112,6 @@ class unit_adapter {
 																													   \
 	variant name##_function::execute(const formula_callable& variables, formula_debugger* fdb) const
 
-
 DEFINE_FAI_FUNCTION(distance_to_nearest_unowned_village, 1, 1)
 {
 	const map_location loc = args()[0]->evaluate(variables, add_debug_info(fdb, 0, "distance_to_nearest_unowned_village:location")).convert_to<location_callable>()->loc();
@@ -273,11 +273,11 @@ DEFINE_FAI_FUNCTION(calculate_map_ownership, 2, 5)
 	if( !units_input.is_list() )
 		return variant();
 
-	size_t number_of_teams = units_input.num_elements();
+	std::size_t number_of_teams = units_input.num_elements();
 
 	std::vector< std::vector<int>> scores( number_of_teams );
 
-	for( size_t i = 0; i< number_of_teams; ++i)
+	for( std::size_t i = 0; i< number_of_teams; ++i)
 		scores[i].resize(w*h);
 
 	/* // TODO: Do we need this?
@@ -288,20 +288,20 @@ DEFINE_FAI_FUNCTION(calculate_map_ownership, 2, 5)
 	}
 	*/
 
-	for(size_t side = 0 ; side < units_input.num_elements() ; ++side) {
+	for(std::size_t side = 0 ; side < units_input.num_elements() ; ++side) {
 		if( leaders_input[side].is_empty() )
 			continue;
 
 		const map_location loc = leaders_input[side][0].convert_to<location_callable>()->loc();
 		const variant units_of_side = units_input[side];
 
-		for(size_t unit_it = 0 ; unit_it < units_of_side.num_elements() ; ++unit_it) {
+		for(std::size_t unit_it = 0 ; unit_it < units_of_side.num_elements() ; ++unit_it) {
 			unit_adapter unit(units_of_side[unit_it]);
 			find_movemap( unit, loc, scores[side], true, ai_ );
 		}
 	}
 
-	size_t index = 0;
+	std::size_t index = 0;
 	for( std::vector< std::vector<int>>::iterator i = scores.begin() ; i != scores.end() ; ++i) {
 		for( std::vector<int>::iterator j = i->begin() ; j != i->end() ; ++j ) {
 			if(units_input[index].num_elements() != 0) {
@@ -316,12 +316,12 @@ DEFINE_FAI_FUNCTION(calculate_map_ownership, 2, 5)
 	//std::vector<variant> res;
 	std::map<variant, variant> res;
 
-	size_t current_side = ai_.get_side() - 1 ;
+	std::size_t current_side = ai_.get_side() - 1 ;
 
 	std::vector<int> enemies;
 	std::vector<int> allies;
 
-	for(size_t side = 0 ; side < units_input.num_elements() ; ++side) {
+	for(std::size_t side = 0 ; side < units_input.num_elements() ; ++side) {
 		if( side == current_side)
 			continue;
 
@@ -382,7 +382,7 @@ DEFINE_WFL_FUNCTION(nearest_loc, 2, 2)
 	int best = 1000000;
 	int best_i = -1;
 
-	for(size_t i = 0; i < items.num_elements(); ++i) {
+	for(std::size_t i = 0; i < items.num_elements(); ++i) {
 
 		const map_location move_loc = items[i].convert_to<location_callable>()->loc();
 		int distance = distance_between(loc, move_loc);
@@ -444,14 +444,11 @@ DEFINE_WFL_FUNCTION(castle_locs, 1, 1)
 
 				visited_locs.insert(loc);
 
-				adjacent_loc_array_t adj;
-				get_adjacent_tiles(loc, adj.data());
-
-				for(unsigned n = 0; n < adj.size(); ++n) {
-					if (resources::gameboard->map().on_board(adj[n]) && visited_locs.find( adj[n] ) == visited_locs.end() ) {
-						if (resources::gameboard->map().get_terrain_info(adj[n]).is_keep() ||
-								resources::gameboard->map().get_terrain_info(adj[n]).is_castle() ) {
-							queued_locs.push(adj[n]);
+				for(const map_location& adj : get_adjacent_tiles(loc)) {
+					if (resources::gameboard->map().on_board(adj) && visited_locs.find( adj ) == visited_locs.end() ) {
+						if (resources::gameboard->map().get_terrain_info(adj).is_keep() ||
+								resources::gameboard->map().get_terrain_info(adj).is_castle() ) {
+							queued_locs.push(adj);
 						}
 					}
 				}
@@ -577,7 +574,7 @@ DEFINE_FAI_FUNCTION(close_enemies, 2, 2)
 		WRN_AI << "close_enemies_function: range is negative (" << range_s << ")" << std::endl;
 		range_s = 0;
 	}
-	size_t range = static_cast<size_t>(range_s);
+	std::size_t range = static_cast<std::size_t>(range_s);
 	unit_map::const_iterator un = resources::gameboard->units().begin();
 	unit_map::const_iterator end = resources::gameboard->units().end();
 	while (un != end) {
@@ -616,7 +613,7 @@ DEFINE_WFL_FUNCTION(calculate_outcome, 3, 4)
 	}
 
 	battle_context bc(units, args()[1]->evaluate(variables, add_debug_info(fdb, 1, "calculate_outcome:attacker_attack_location")).convert_to<location_callable>()->loc(),
-		defender_location, weapon, -1, 1.0, nullptr, &*units.find(attacker_location));
+		defender_location, weapon, -1, 1.0, nullptr, units.find(attacker_location).get_shared_ptr());
 	std::vector<double> hp_dist = bc.get_attacker_combatant().hp_dist;
 	std::vector<double>::iterator it = hp_dist.begin();
 	int i = 0;
@@ -692,17 +689,6 @@ DEFINE_WFL_FUNCTION(outcomes, 1, 1)
 	return variant(vars);
 }
 
-/* // TODO: Not really sure what this was meant to do or if there's any reason left to keep it...
-DEFINE_FAI_FUNCTION(evaluate_for_position, 2, 2)
-{
-	variant position = args()[0]->evaluate(variables,add_debug_info(fdb,0,"evaluate_for_position:position"));
-	ai_.store_outcome_position(position);
-	position_callable* pos = position.convert_to<position_callable>();
-	position_callable::swapper swapper(ai_, *pos);
-	return args()[1]->evaluate(variables,add_debug_info(fdb,1,"evaluate_for_position:formula"));
-}
-*/
-
 DEFINE_FAI_FUNCTION(rate_action, 1, 1)
 {
 	variant act = args()[0]->evaluate(variables,add_debug_info(fdb,0,"rate_action:action"));
@@ -808,10 +794,11 @@ DEFINE_FAI_FUNCTION(simplest_path, 2, 3)
 	}
 
 	for (std::vector<map_location>::const_iterator loc_iter = route.steps.begin() + 1 ; loc_iter !=route.steps.end(); ++loc_iter) {
-		if (unit_it->movement_cost((resources::gameboard->map())[*loc_iter]) < movetype::UNREACHABLE )
+		if(unit_it->movement_cost(static_cast<const game_board*>(resources::gameboard)->map()[*loc_iter]) < movetype::UNREACHABLE) {
 			locations.emplace_back(std::make_shared<location_callable>(*loc_iter));
-		else
+		} else {
 			break;
+		}
 	}
 
 	return variant(locations);
@@ -941,7 +928,6 @@ DEFINE_FAI_FUNCTION(debug_label, 2, 2)
 	return variant(result);
 }
 
-
 DEFINE_WFL_FUNCTION(is_village, 2, 3)
 {
 	const gamemap& m = args()[0]->evaluate(variables, add_debug_info(fdb, 0, "is_village:map")).convert_to<gamemap_callable>()->get_gamemap();
@@ -1056,7 +1042,7 @@ namespace {
 			}
 		}
 
-		return std::make_pair(highest_melee_damage, highest_ranged_damage);
+		return std::pair(highest_melee_damage, highest_ranged_damage);
 	}
 }
 
