@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -17,7 +18,6 @@
  * Defines formula ai side formulas stage
  */
 
-
 #include "ai/formula/stage_side_formulas.hpp"
 #include "ai/formula/ai.hpp"
 
@@ -33,12 +33,11 @@ static lg::log_domain log_ai("ai/stage/side_formulas");
 namespace ai {
 
 stage_side_formulas::stage_side_formulas(ai_context &context, const config &cfg, formula_ai &fai)
-		: stage(context,cfg), cfg_(cfg), fai_(fai), move_formula_()
+	: stage(context,cfg), cfg_(cfg), fai_(fai), move_formula_()
 {
 
 }
 
-
 stage_side_formulas::~stage_side_formulas()
 {
 }
@@ -62,17 +61,14 @@ bool stage_side_formulas::do_play_stage()
 	return false;
 }
 
-
 void stage_side_formulas::on_create()
 {
 	move_formula_ = fai_.create_optional_formula(cfg_["move"]);
 }
 
-
 config stage_side_formulas::to_config() const
 {
 	config cfg = stage::to_config();
-	///@todo 1.7: serialize to config
 	cfg.append(cfg_);
 	return cfg;
 }
