@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -45,7 +46,7 @@ typedef std::multimap<map_location, map_location> move_map;
 typedef std::map<map_location, pathfind::paths> moves_map;
 
 class ai_composite;
-typedef std::shared_ptr<ai_composite> ai_ptr;
+typedef std::unique_ptr<ai_composite> composite_ai_ptr;
 
 class attack_analysis;
 typedef std::vector<attack_analysis> attacks_vector;
@@ -111,7 +112,6 @@ public:
 	{
 	}
 
-	/** hack. @todo 1.9 rework that via extended event system, or at least ensure it hurts no one */
 	std::set<map_location> recent_attacks;
 };
 
