@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -219,7 +220,7 @@ static int ai_attack(lua_State *L, bool exec)
 		}
 	}
 
-	//TODO: Right now, aggression is used by the attack execution functions to determine the weapon to be used.
+	// Note: Right now, aggression is used by the attack execution functions to determine the weapon to be used.
 	// If a decision is made to expand the function that determines the weapon, this block must be refactored
 	// to parse aggression if a single int is on the stack, or create a table of parameters, if a table is on the
 	// stack.
@@ -227,8 +228,7 @@ static int ai_attack(lua_State *L, bool exec)
 		aggression = lua_tonumber(L, 4);
 	}
 
-	unit_advancements_aspect advancements = context.get_advancements();
-	ai::attack_result_ptr attack_result = ai::actions::execute_attack_action(side,exec,attacker,defender,attacker_weapon,aggression,advancements);
+	ai::attack_result_ptr attack_result = ai::actions::execute_attack_action(side,exec,attacker,defender,attacker_weapon,aggression);
 	return transform_ai_action(L,attack_result);
 }
 
@@ -271,25 +271,6 @@ static int cfun_ai_check_stopunit(lua_State *L)
 	return ai_stopunit_select(L, false, true, true);
 }
 
-static int cfun_ai_execute_synced_command(lua_State *L)
-{
-	luaW_getglobal(L, "wesnoth", "deprecated_message");
-	lua_pushstring(L, "ai.synced_command");
-	lua_pushinteger(L, 4);
-	lua_pushstring(L, "1.14");
-	lua_pushstring(L, "Instead, you must register a custom synced command in wesnoth.custom_synced_commands, and invoke it with wesnoth.invoke_synced_command.");
-	luaW_pcall(L, 4, 0);
-	return 0; // Unreachable, as luaW_pcall throws an exception
-}
-
-static int cfun_ai_check_synced_command(lua_State *L)
-{
-	// This is basically the same as above but returns false instead of raising an error.
-	deprecated_message("ai.check_synced_command", DEP_LEVEL::REMOVED, "1.14", "Instead, you must register a custom synced command in wesnoth.custom_synced_commands, and invoke it with wesnoth.invoke_synced_command.");
-	lua_pushboolean(L, false);
-	return 1;
-}
-
 static int ai_recruit(lua_State *L, bool exec)
 {
 	const char *unit_name = luaL_checkstring(L, 1);
@@ -299,7 +280,6 @@ static int ai_recruit(lua_State *L, bool exec)
 		where.set_wml_x(lua_tonumber(L, 2));
 		where.set_wml_y(lua_tonumber(L, 3));
 	}
-	//TODO fendrin: talk to Crab about the from argument.
 	map_location from = map_location::null_location();
 	ai::recruit_result_ptr recruit_result = ai::actions::execute_recruit_action(side,exec,std::string(unit_name),where,from);
 	return transform_ai_action(L,recruit_result);
@@ -324,7 +304,6 @@ static int ai_recall(lua_State *L, bool exec)
 		where.set_wml_x(lua_tonumber(L, 2));
 		where.set_wml_y(lua_tonumber(L, 3));
 	}
-	//TODO fendrin: talk to Crab about the from argument.
 	map_location from = map_location::null_location();
 	ai::recall_result_ptr recall_result = ai::actions::execute_recall_action(side,exec,std::string(unit_id),where,from);
 	return transform_ai_action(L,recall_result);
@@ -347,7 +326,6 @@ static int cfun_ai_fallback_human(lua_State*)
 
 // Goals and targets
 
-
 static int cfun_ai_get_targets(lua_State *L)
 {
 	move_map enemy_dst_src = get_readonly_context(L).get_enemy_dstsrc();
@@ -362,7 +340,6 @@ static int cfun_ai_get_targets(lua_State *L)
 		//to factor out
 		lua_createtable(L, 3, 0);
 
-
 		lua_pushstring(L, "type");
 		lua_pushstring(L, it->type.to_string().c_str());
 		lua_rawset(L, -3);
@@ -396,14 +373,6 @@ static int cfun_ai_get_aggression(lua_State *L)
 	return 1;
 }
 
-static int cfun_ai_get_attack_depth(lua_State *L)
-{
-	DEPRECATED_ASPECT_MESSAGE("attack_depth");
-	int attack_depth = get_readonly_context(L).get_attack_depth();
-	lua_pushnumber(L, attack_depth);
-	return 1;
-}
-
 static int cfun_ai_get_attacks(lua_State *L)
 {
 	// Unlike the other aspect fetchers, this one is not deprecated!
@@ -465,11 +434,31 @@ static int cfun_ai_get_leader_goal(lua_State *L)
 	return 1;
 }
 
+namespace
+{
+// TODO: name this something better
+void visit_helper(lua_State* L, const utils::variant<bool, std::vector<std::string>>& input)
+{
+	utils::visit(
+		[L](const auto& v) {
+			if constexpr(utils::decayed_is_same<bool, decltype(v)>) {
+				lua_pushboolean(L, v);
+			} else {
+				lua_createtable(L, v.size(), 0);
+				for(const std::string& str : v) {
+					lua_pushlstring(L, str.c_str(), str.size());
+					lua_rawseti(L, -2, lua_rawlen(L, -2) + 1);
+				}
+			}
+		},
+		input);
+}
+} // namespace
+
 static int cfun_ai_get_leader_ignores_keep(lua_State *L)
 {
 	DEPRECATED_ASPECT_MESSAGE("leader_ignores_keep");
-	bool leader_ignores_keep = get_readonly_context(L).get_leader_ignores_keep();
-	lua_pushboolean(L, leader_ignores_keep);
+	visit_helper(L, get_readonly_context(L).get_leader_ignores_keep());
 	return 1;
 }
 
@@ -484,16 +473,14 @@ static int cfun_ai_get_leader_value(lua_State *L)
 static int cfun_ai_get_passive_leader(lua_State *L)
 {
 	DEPRECATED_ASPECT_MESSAGE("passive_leader");
-	bool passive_leader = get_readonly_context(L).get_passive_leader();
-	lua_pushboolean(L, passive_leader);
+	visit_helper(L, get_readonly_context(L).get_passive_leader());
 	return 1;
 }
 
 static int cfun_ai_get_passive_leader_shares_keep(lua_State *L)
 {
 	DEPRECATED_ASPECT_MESSAGE("passive_leader_shares_keep");
-	bool passive_leader_shares_keep = get_readonly_context(L).get_passive_leader_shares_keep();
-	lua_pushboolean(L, passive_leader_shares_keep);
+	visit_helper(L, get_readonly_context(L).get_passive_leader_shares_keep());
 	return 1;
 }
 
@@ -596,7 +583,6 @@ static void push_movements(lua_State *L, const std::vector< std::pair < map_loca
 		lua_rawseti(L, table_index, i); // setting  the pair as an element of the movements table
 	}
 
-
 }
 
 static void push_attack_analysis(lua_State *L, const attack_analysis& aa)
@@ -651,7 +637,7 @@ static void push_attack_analysis(lua_State *L, const attack_analysis& aa)
 	lua_rawset(L, -3);
 
 	lua_pushstring(L, "terrain_quality");
-	lua_pushnumber(L, aa.alternative_terrain_quality);
+	lua_pushnumber(L, aa.terrain_quality);
 	lua_rawset(L, -3);
 
 	lua_pushstring(L, "alternative_terrain_quality");
@@ -804,22 +790,8 @@ static int impl_ai_aspect_get(lua_State* L)
 		return 0;
 	}
 
-	typedef std::vector<std::string> string_list;
-	if(typesafe_aspect<bool>* aspect_as_bool = try_aspect_as<bool>(iter->second)) {
-		lua_pushboolean(L, aspect_as_bool->get());
-	} else if(typesafe_aspect<int>* aspect_as_int = try_aspect_as<int>(iter->second)) {
-		lua_pushinteger(L, aspect_as_int->get());
-	} else if(typesafe_aspect<double>* aspect_as_double = try_aspect_as<double>(iter->second)) {
-		lua_pushnumber(L, aspect_as_double->get());
-	} else if(typesafe_aspect<config>* aspect_as_config = try_aspect_as<config>(iter->second)) {
-		luaW_pushconfig(L, aspect_as_config->get());
-	} else if(typesafe_aspect<string_list>* aspect_as_string_list = try_aspect_as<string_list>(iter->second)) {
-		lua_push(L, aspect_as_string_list->get());
-	} else if(typesafe_aspect<terrain_filter>* aspect_as_terrain_filter = try_aspect_as<terrain_filter>(iter->second)) {
-		std::set<map_location> result;
-		aspect_as_terrain_filter->get().get_locations(result);
-		lua_push(L, result);
-	} else if(typesafe_aspect<attacks_vector>* aspect_as_attacks_vector = try_aspect_as<attacks_vector>(iter->second)) {
+	// A few aspects require special delicate handling...
+	if(typesafe_aspect<attacks_vector>* aspect_as_attacks_vector = try_aspect_as<attacks_vector>(iter->second)) {
 		using ai_default_rca::aspect_attacks_base;
 		aspect_attacks_base* real_aspect = dynamic_cast<aspect_attacks_base*>(aspect_as_attacks_vector);
 		while(real_aspect == nullptr) {
@@ -853,7 +825,6 @@ static int impl_ai_aspect_get(lua_State* L)
 			lua_rawseti(L, -2, i + 1);
 		}
 		lua_setfield(L, -2, "enemy");
-		return 1;
 	} else if(typesafe_aspect<unit_advancements_aspect>* aspect_as_unit_advancements_aspects = try_aspect_as<unit_advancements_aspect>(iter->second)) {
 		const unit_advancements_aspect& val = aspect_as_unit_advancements_aspects->get();
 		int my_side = get_engine(L).get_readonly_context().get_side();
@@ -868,7 +839,7 @@ static int impl_ai_aspect_get(lua_State* L)
 			lua_settable(L, -3);
 		}
 	} else {
-		lua_pushnil(L);
+		iter->second->get_lua(L);
 	}
 	return 1;
 }
@@ -916,7 +887,6 @@ static int impl_ai_get(lua_State* L)
 			// Aspects
 			{ "get_aggression", &cfun_ai_get_aggression },
 			{ "get_avoid", &cfun_ai_get_avoid },
-			{ "get_attack_depth", &cfun_ai_get_attack_depth },
 			{ "get_attacks", &cfun_ai_get_attacks },
 			{ "get_caution", &cfun_ai_get_caution },
 			{ "get_grouping", &cfun_ai_get_grouping },
@@ -943,7 +913,6 @@ static int impl_ai_get(lua_State* L)
 			{ "check_recall", &cfun_ai_check_recall },
 			{ "check_move", &cfun_ai_check_move },
 			{ "check_stopunit", &cfun_ai_check_stopunit },
-			{ "check_synced_command", &cfun_ai_check_synced_command },
 			{ "check_attack", &cfun_ai_check_attack },
 			{ "check_recruit", &cfun_ai_check_recruit },
 			//{ "",},
@@ -976,7 +945,6 @@ static int impl_ai_get(lua_State* L)
 			{ "stopunit_all", &cfun_ai_execute_stopunit_all },
 			{ "stopunit_attacks", &cfun_ai_execute_stopunit_attacks },
 			{ "stopunit_moves", &cfun_ai_execute_stopunit_moves },
-			{ "synced_command", &cfun_ai_execute_synced_command },
 			{ "fallback_human", &cfun_ai_fallback_human},
 			{ nullptr, nullptr } };
 	for (const luaL_Reg* p = mutating_callbacks; p->name; ++p) {
@@ -1078,12 +1046,15 @@ lua_ai_action_handler* lua_ai_action_handler::create(lua_State *L, char const *c
 	return new lua_ai_action_handler(L, context, length + 1);
 }
 
-
 int lua_ai_load::refcount = 0;
 
 lua_ai_load::lua_ai_load(lua_ai_context& ctx, bool read_only) : L(ctx.L), was_readonly(false)
 {
 	refcount++;
+	lua_getfield(L, LUA_REGISTRYINDEX, aisKey); // [-1: AI registry]
+	lua_rawgeti(L, -1, ctx.num_); // [-1: AI state  -2: AI registry]
+	lua_remove(L,-2); // [-1: AI state]
+
 	// Check if the AI table is already loaded. If so, we have less work to do.
 	lua_getglobal(L, "ai");
 	if(!lua_isnoneornil(L, -1)) {
@@ -1095,19 +1066,16 @@ lua_ai_load::lua_ai_load(lua_ai_context& ctx, bool read_only) : L(ctx.L), was_re
 		lua_pushstring(L, "read_only");
 		lua_pushboolean(L, read_only);
 		lua_rawset(L, -3);
-		return; // Leave the AI table on the stack, as requested
+		lua_pop(L, 1); // Pop the ai table off the stack
+	} else {
+		lua_pop(L, 1); // Pop the nil value off the stack
+		// Load the AI functions table into global scope
+		lua_getfield(L, -1, "ai"); // [-1: AI functions  -2: AI state]
+		lua_pushstring(L, "read_only"); // [-1: key  -2: AI functions  -3: AI state]
+		lua_pushboolean(L, read_only); // [-1: value  -2: key  -3: AI functions  -4: AI state]
+		lua_rawset(L, -3); // [-1: AI functions  -2: AI state]
+		lua_setglobal(L, "ai"); // [-1: AI state]
 	}
-	lua_pop(L, 1); // Pop the nil value off the stack
-	lua_getfield(L, LUA_REGISTRYINDEX, aisKey); // [-1: AI registry]
-	lua_rawgeti(L, -1, ctx.num_); // [-1: AI state  -2: AI registry]
-	lua_remove(L,-2); // [-1: AI state]
-
-	// Load the AI functions table into global scope
-	lua_getfield(L, -1, "ai"); // [-1: AI functions  -2: AI state]
-	lua_pushstring(L, "read_only"); // [-1: key  -2: AI functions  -3: AI state]
-	lua_pushboolean(L, read_only); // [-1: value  -2: key  -3: AI functions  -4: AI state]
-	lua_rawset(L, -3); // [-1: AI functions  -2: AI state]
-	lua_setglobal(L, "ai"); // [-1: AI state]
 }
 
 lua_ai_load::~lua_ai_load()
@@ -1136,7 +1104,7 @@ lua_ai_context::~lua_ai_context()
 	lua_pop(L, 1);
 }
 
-void lua_ai_action_handler::handle(const config &cfg, bool read_only, lua_object_ptr l_obj)
+void lua_ai_action_handler::handle(const config &cfg, const config &filter_own, bool read_only, lua_object_ptr l_obj)
 {
 	int initial_top = lua_gettop(L);//get the old stack size
 
@@ -1154,8 +1122,14 @@ void lua_ai_action_handler::handle(const config &cfg, bool read_only, lua_object
 	luaW_pushconfig(L, cfg);
 	lua_getfield(L, iState, "data");
 
+	int num = 3;
+	if (!filter_own.empty()) {
+		luaW_pushconfig(L, filter_own);
+		num=4;
+	}
+
 	// Call the function
-	luaW_pcall(L, 3, l_obj ? 1 : 0, true);
+	luaW_pcall(L, num, l_obj ? 1 : 0, true);
 	if (l_obj) {
 		l_obj->store(L, -1);
 	}
