@@ -1,22 +1,23 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
  * @file
  * Managing the AIs lifecycle - headers
- * @todo 1.9 Refactor history handling and internal commands.
- * @todo 1.9 AI Interface command to clear the history.
+ * TODO: Refactor history handling and internal commands.
+ * TODO: AI Interface command to clear the history.
  */
 
 #pragma once
@@ -31,6 +32,7 @@
 #include <string>                       // for string
 
 class game_launcher;
+namespace ai { class unit_advancements_aspect; }  // lines 45-45
 namespace ai { class ai_composite; }  // lines 45-45
 namespace ai { class ai_context; }  // lines 42-42
 namespace ai { class component; }  // lines 43-43
@@ -41,15 +43,12 @@ namespace ai { class side_context; }  // lines 38-38
 namespace events { class generic_event; }
 namespace events { class observer; }
 
-
-namespace ai {
-
-typedef std::shared_ptr<ai_composite> composite_ai_ptr;
+namespace ai
+{
 
 /**
  * Base class that holds the AI and current AI parameters.
  * It is an implementation detail.
- * @todo 1.9 move it out of public view
  */
 class holder{
 public:
@@ -65,16 +64,12 @@ public:
 
 	void modify_ai(const config& cfg);
 
-
 	void append_ai(const config& cfg);
 
-
 	const std::string get_ai_overview();
 
-
 	const std::string get_ai_structure();
 
-
 	const std::string get_ai_identifier() const;
 
 	component* get_component(component *root, const std::string &path); // Ai debug method
@@ -82,7 +77,6 @@ public:
 private:
 	void init( side_number side );
 
-
 	composite_ai_ptr ai_;
 	std::unique_ptr<side_context> side_context_;
 	std::unique_ptr<readonly_context> readonly_context_;
@@ -93,7 +87,7 @@ private:
 };
 
 /**
- * AI Command History Item. It is an implementation detail
+ * AI Command History Item. It is an implementation detail. See TODOs above.
  */
 class command_history_item{
 public:
@@ -109,7 +103,6 @@ public:
 private:
 	int number_;
 	std::string command_;
-
 };
 
 /**
@@ -124,7 +117,7 @@ public:
 	// CONSTANTS
 	// =======================================================================
 
-	static const size_t MAX_HISTORY_SIZE = 200;
+	static const std::size_t MAX_HISTORY_SIZE = 200;
 
 	static const std::string AI_TYPE_COMPOSITE_AI;
 	static const std::string AI_TYPE_SAMPLE_AI;
@@ -134,7 +127,6 @@ public:
 	static const std::string AI_TYPE_AI2;
 	static const std::string AI_TYPE_DEFAULT;
 
-
 	// =======================================================================
 	// CONSTRUCTORS AND DESTRUCTORS
 	// =======================================================================
@@ -164,33 +156,28 @@ public:
 	// LIFECYCLE
 	// =======================================================================
 
-
 	/**
 	 * Adds observer of game events.
 	 * Should be called in playsingle_controller 's constructor.
 	 */
 	void add_observer( events::observer* event_observer);
 
-
 	/**
 	 * Removes an observer of game events.
 	 * Should be called in playsingle_controller 's destructor.
 	 */
 	void remove_observer( events::observer* event_observer );
 
-
 	/**
 	 * Adds observer of game events except ai_user_interact event and ai_sync_network event
 	 */
 	void add_gamestate_observer( events::observer* event_observer);
 
-
 	/**
 	 * Removes an observer of game events except ai_user_interact event and ai_sync_network event
 	 */
 	void remove_gamestate_observer( events::observer* event_observer );
 
-
 	/**
 	 * Notifies all observers of 'ai_user_interact' event.
 	 * Function which should be called frequently to allow the user to interact
@@ -206,86 +193,72 @@ public:
 	 */
 	void raise_sync_network();
 
-
 	/**
 	 * Notifies all observers of 'ai_gamestate_changed' event.
 	 */
 	void raise_gamestate_changed();
 
-
 	/**
 	 * Notifies all observers of 'ai_tod_changed' event.
 	 */
 	void raise_tod_changed();
 
-
 	/**
 	 * Notifies all observers of 'ai_recruit_list_changed' event.
 	 */
 	void raise_recruit_list_changed();
 
-
 	/**
 	 * Notifies all observers of 'ai_turn_started' event.
 	 */
 	void raise_turn_started();
 
-
 	/**
 	 * Notifies all observers of 'ai_map_changed' event.
 	 */
 	void raise_map_changed();
 
-
 	/**
 	 * Adds an observer of 'ai_map_changed' event.
 	 */
 	void add_map_changed_observer( events::observer* event_observer );
 
-
 	/**
 	 * Adds an observer of 'ai_recruit_list_changed' event.
 	 */
 	void add_recruit_list_changed_observer( events::observer* event_observer );
 
-
 	/**
 	 * Adds an observer of 'ai_turn_started' event.
 	 */
 	void add_turn_started_observer( events::observer* event_observer );
 
-
 	/**
 	 * Adds an observer of 'ai_tod_changed' event.
 	 */
 	void add_tod_changed_observer( events::observer* event_observer );
 
-
 	/**
 	 * Deletes an observer of 'ai_map_changed' event.
 	 */
 	void remove_map_changed_observer( events::observer* event_observer );
 
 
-
 	/**
 	 * Deletes an observer of 'ai_recruit_list_changed' event.
 	 */
 	void remove_recruit_list_changed_observer( events::observer* event_observer );
 
-
 	/**
 	 * Deletes an observer of 'ai_turn_started' event.
 	 */
 	void remove_turn_started_observer( events::observer* event_observer );
 
-
 	/**
 	 * Deletes an observer of 'ai_tod_changed' event.
 	 */
 	void remove_tod_changed_observer( events::observer* event_observer );
 
-
 public:
 
 	// =======================================================================
@@ -303,7 +276,6 @@ public:
 	 */
 	const std::string evaluate_command( side_number side, const std::string& str );
 
-
 	// =======================================================================
 	// ADD, CREATE AIs, OR LIST AI TYPES
 	// =======================================================================
@@ -319,7 +291,6 @@ public:
 	 */
 	bool add_ai_for_side_from_file( side_number side, const std::string& file, bool replace = true );
 
-
 	/**
 	 * Adds active AI for specified @a side from @a cfg.
 	 * @note Running this command may invalidate references previously returned
@@ -331,19 +302,6 @@ public:
 	 */
 	bool add_ai_for_side_from_config(side_number side, const config &cfg, bool replace = true);
 
-
-	/**
-	 * Adds active AI for specified @a side from parameters.
-	 * @note Running this command may invalidate references previously returned
-	 *       by manager. AI is not initialized at this point.
-	 * @param side side number (1-based, as in game_info).
-	 * @param ai_algorithm_type type of AI algorithm to create.
-	 * @param replace should new ai replace the current ai or 'be placed on top of it'.
-	 * @return true if successful.
-	 */
-	bool add_ai_for_side( side_number side, const std::string& ai_algorithm_type, bool replace = true);
-
-
 	// =======================================================================
 	// REMOVE
 	// =======================================================================
@@ -356,7 +314,6 @@ public:
 	 */
 	void remove_ai_for_side( side_number side );
 
-
 	/**
 	 * Removes all AIs from @a side.
 	 * @note Running this command may invalidate references previously returned
@@ -365,7 +322,6 @@ public:
 	 */
 	void remove_all_ais_for_side( side_number side );
 
-
 	/**
 	 * Clears all the AIs.
 	 * @note Running this command may invalidate references previously returned
@@ -379,7 +335,6 @@ public:
 	// GET active AI parameters
 	// =======================================================================
 
-
 	/**
 	 * Gets AI info for active AI of the given @a side.
 	 * @param side side number (1-based).
@@ -387,7 +342,6 @@ public:
 	 */
 	game_info& get_active_ai_info_for_side( side_number side );
 
-
 	/**
 	 * Gets AI Overview for active AI of the given @a side
 	 * @param side side number (1-based)
@@ -395,7 +349,6 @@ public:
 	 */
 	std::string get_active_ai_overview_for_side( side_number side );
 
-
 	/**
 	 * Gets AI Structure for active AI of the given @a side
 	 * @param side side number (1-based)
@@ -425,13 +378,13 @@ public:
 	 */
 	config to_config( side_number side );
 
-
 	/**
 	 * Gets global AI-game info
 	 * @return a reference to the AI-game info.
 	 */
 	game_info& get_ai_info();
 
+	const ai::unit_advancements_aspect& get_advancement_aspect_for_side(side_number side);
 
 	// =======================================================================
 	// SET active AI parameters
@@ -465,7 +418,6 @@ public:
 	 */
 	void play_turn(side_number side);
 
-
 private:
 
 	typedef std::map< side_number, std::stack< holder >> AI_map_of_stacks;
@@ -488,7 +440,6 @@ private:
 
 	static manager* singleton_;
 
-
 	// =======================================================================
 	// EVALUATION
 	// =======================================================================
@@ -498,7 +449,7 @@ private:
 	 * @param side side number (1-based).
 	 * @param str string to evaluate.
 	 * @return string result of evaluation.
-	 * @todo 1.9 rewrite this function to use a fai or lua parser.
+	 * TODO: rewrite this function to use a fai or lua parser.
 	 */
 	const std::string internal_evaluate_command( side_number side, const std::string& str );
 
@@ -513,7 +464,6 @@ private:
 	// AI STACKS
 	// =======================================================================
 
-
 	/**
 	 * Gets the AI stack for the specified side, create it if it doesn't exist.
 	 */
@@ -523,7 +473,6 @@ private:
 	// AI HOLDERS
 	// =======================================================================
 
-
 	/**
 	 * Gets active holder for specified @a side.
 	 */
@@ -543,7 +492,6 @@ private:
 	 */
 	ai_composite& get_active_ai_for_side( side_number side );
 
-
 	friend class ::game_launcher;
 };
 
