@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -188,9 +189,6 @@ static register_aspect_factory< composite_aspect< unit_advancements_aspect >>
 static register_aspect_factory< composite_aspect<double>>
 	aggression__composite_aspect_factory("aggression*composite_aspect");
 
-static register_aspect_factory< composite_aspect<int>>
-	attack_depth__composite_aspect_factory("attack_depth*composite_aspect");
-
 static register_aspect_factory< composite_aspect< attacks_vector >>
 	attacks__composite_aspect_factory("attacks*composite_aspect");
 
@@ -209,16 +207,16 @@ static register_aspect_factory< composite_aspect<double>>
 static register_aspect_factory< composite_aspect<config>>
 	leader_goal__composite_aspect_factory("leader_goal*composite_aspect");
 
-static register_aspect_factory< composite_aspect<bool>>
+static register_aspect_factory< composite_aspect<utils::variant<bool, std::vector<std::string>>>>
 	leader_igores_keep__composite_aspect_factory("leader_ignores_keep*composite_aspect");
 
 static register_aspect_factory< composite_aspect<double>>
 	leader_value__composite_aspect_factory("leader_value*composite_aspect");
 
-static register_aspect_factory< composite_aspect<bool>>
+static register_aspect_factory< composite_aspect<utils::variant<bool, std::vector<std::string>>>>
 	passive_leader__composite_aspect_factory("passive_leader*composite_aspect");
 
-static register_aspect_factory< composite_aspect<bool>>
+static register_aspect_factory< composite_aspect<utils::variant<bool, std::vector<std::string>>>>
 	passive_leader_shares_keep__composite_aspect_factory("passive_leader_shares_keep*composite_aspect");
 
 static register_aspect_factory< composite_aspect<double>>
@@ -227,10 +225,10 @@ static register_aspect_factory< composite_aspect<double>>
 static register_aspect_factory< composite_aspect<config>>
 	recruitment_instructions__composite_aspect_factory("recruitment_instructions*composite_aspect");
 
-static register_aspect_factory< composite_aspect< std::vector<std::string>> >
+static register_aspect_factory< composite_aspect< std::vector<std::string>>>
 	recruitment_more__composite_aspect_factory("recruitment_more*composite_aspect");
 
-static register_aspect_factory< composite_aspect< std::vector<std::string>> >
+static register_aspect_factory< composite_aspect< std::vector<std::string>>>
 	recruitment_pattern__composite_aspect_factory("recruitment_pattern*composite_aspect");
 
 static register_aspect_factory< composite_aspect<int>>
@@ -239,6 +237,12 @@ static register_aspect_factory< composite_aspect<int>>
 static register_aspect_factory< composite_aspect<config>>
 	recruitment_save_gold__composite_aspect_factory("recruitment_save_gold*composite_aspect");
 
+static register_aspect_factory< composite_aspect<double>>
+	retreat_enemy_weight__composite_aspect_factory("retreat_enemy_weight*composite_aspect");
+
+static register_aspect_factory< composite_aspect<double>>
+	retreat_factor__composite_aspect_factory("retreat_factor*composite_aspect");
+
 static register_aspect_factory< composite_aspect<double>>
 	scout_village_targeting__composite_aspect_factory("scout_village_targeting*composite_aspect");
 
@@ -262,9 +266,6 @@ static register_aspect_factory< standard_aspect< unit_advancements_aspect >>
 static register_aspect_factory< standard_aspect<double>>
 	aggression__standard_aspect_factory("aggression*standard_aspect");
 
-static register_aspect_factory< standard_aspect<int>>
-	attack_depth__standard_aspect_factory("attack_depth*standard_aspect");
-
 static register_aspect_factory< ai_default_rca::aspect_attacks >
 	attacks__testing_ai_default_aspect_attacks_factory("attacks*ai_default_rca::aspect_attacks");
 
@@ -283,16 +284,16 @@ static register_aspect_factory< standard_aspect<double>>
 static register_aspect_factory< standard_aspect<config>>
 	leader_goal__standard_aspect_factory("leader_goal*standard_aspect");
 
-static register_aspect_factory< standard_aspect<bool>>
+static register_aspect_factory< standard_aspect<utils::variant<bool, std::vector<std::string>>>>
 	leader_ignores_keep__standard_aspect_factory("leader_ignores_keep*standard_aspect");
 
 static register_aspect_factory< standard_aspect<double>>
 	leader_value__standard_aspect_factory("leader_value*standard_aspect");
 
-static register_aspect_factory< standard_aspect<bool>>
+static register_aspect_factory< standard_aspect<utils::variant<bool, std::vector<std::string>>>>
 	passive_leader__standard_aspect_factory("passive_leader*standard_aspect");
 
-static register_aspect_factory< standard_aspect<bool>>
+static register_aspect_factory< standard_aspect<utils::variant<bool, std::vector<std::string>>>>
 	passive_leader_shares_keep__standard_aspect_factory("passive_leader_shares_keep*standard_aspect");
 
 static register_aspect_factory< standard_aspect<double>>
@@ -301,10 +302,10 @@ static register_aspect_factory< standard_aspect<double>>
 static register_aspect_factory< default_recruitment::recruitment_aspect >
 	recruitment_instructions__standard_aspect_factory("recruitment_instructions*standard_aspect");
 
-static register_aspect_factory< standard_aspect< std::vector<std::string>> >
+static register_aspect_factory< standard_aspect< std::vector<std::string>>>
 	recruitment_more__standard_aspect_factory("recruitment_more*standard_aspect");
 
-static register_aspect_factory< standard_aspect< std::vector<std::string>> >
+static register_aspect_factory< standard_aspect< std::vector<std::string>>>
 	recruitment_pattern__standard_aspect_factory("recruitment_pattern*standard_aspect");
 
 static register_aspect_factory< standard_aspect<int>>
@@ -313,6 +314,12 @@ static register_aspect_factory< standard_aspect<int>>
 static register_aspect_factory< standard_aspect<config>>
 	recruitment_save_gold__standard_aspect_factory("recruitment_save_gold*standard_aspect");
 
+static register_aspect_factory< standard_aspect<double>>
+	retreat_enemy_weight__standard_aspect_factory("retreat_enemy_weight*standard_aspect");
+
+static register_aspect_factory< standard_aspect<double>>
+	retreat_factor__standard_aspect_factory("retreat_factor*standard_aspect");
+
 static register_aspect_factory< standard_aspect<double>>
 	scout_village_targeting__standard_aspect_factory("scout_village_targeting*standard_aspect");
 
@@ -340,9 +347,6 @@ static register_aspect_factory< standard_aspect< unit_advancements_aspect >>
 static register_aspect_factory< standard_aspect<double>>
 	aggression__standard_aspect_factory2("aggression*");
 
-static register_aspect_factory< standard_aspect<int>>
-	attack_depth__standard_aspect_factory2("attack_depth*");
-
 static register_aspect_factory< ai_default_rca::aspect_attacks >
 	attacks__testing_ai_default_aspect_attacks_factory2("attacks*");
 
@@ -361,16 +365,16 @@ static register_aspect_factory< standard_aspect<double>>
 static register_aspect_factory< standard_aspect<config>>
 	leader_goal__standard_aspect_factory2("leader_goal*");
 
-static register_aspect_factory< standard_aspect<bool>>
+static register_aspect_factory< standard_aspect<utils::variant<bool, std::vector<std::string>>>>
 	leader_ignores_keep__standard_aspect_factory2("leader_ignores_keep*");
 
 static register_aspect_factory< standard_aspect<double>>
 	leader_value__standard_aspect_factory2("leader_value*");
 
-static register_aspect_factory< standard_aspect<bool>>
+static register_aspect_factory< standard_aspect<utils::variant<bool, std::vector<std::string>>>>
 	passive_leader__standard_aspect_factory2("passive_leader*");
 
-static register_aspect_factory< standard_aspect<bool>>
+static register_aspect_factory< standard_aspect<utils::variant<bool, std::vector<std::string>>>>
 	passive_leader_shares_keep__standard_aspect_factory2("passive_leader_shares_keep*");
 
 static register_aspect_factory< standard_aspect<double>>
@@ -379,10 +383,10 @@ static register_aspect_factory< standard_aspect<double>>
 static register_aspect_factory< default_recruitment::recruitment_aspect >
 	recruitment_instructions__standard_aspect_factory2("recruitment_instructions*");
 
-static register_aspect_factory< standard_aspect< std::vector<std::string>> >
+static register_aspect_factory< standard_aspect< std::vector<std::string>>>
 	recruitment_more__standard_aspect_factory2("recruitment_more*");
 
-static register_aspect_factory< standard_aspect< std::vector<std::string>> >
+static register_aspect_factory< standard_aspect< std::vector<std::string>>>
 	recruitment_pattern__standard_aspect_factory2("recruitment_pattern*");
 
 static register_aspect_factory< standard_aspect<int>>
@@ -391,6 +395,12 @@ static register_aspect_factory< standard_aspect<int>>
 static register_aspect_factory< standard_aspect<config>>
 	recruitment_save_gold__standard_aspect_factory2("recruitment_save_gold*");
 
+static register_aspect_factory< standard_aspect<double>>
+	retreat_enemy_weight__standard_aspect_factory2("retreat_enemy_weight*");
+
+static register_aspect_factory< standard_aspect<double>>
+	retreat_factor__standard_aspect_factory2("retreat_factor*");
+
 static register_aspect_factory< standard_aspect<double>>
 	scout_village_targeting__standard_aspect_factory2("scout_village_targeting*");
 
@@ -414,9 +424,6 @@ static register_lua_aspect_factory< lua_aspect< unit_advancements_aspect >>
 static register_lua_aspect_factory< lua_aspect<double>>
 	aggression__lua_aspect_factory("aggression*lua_aspect");
 
-static register_lua_aspect_factory< lua_aspect<int>>
-	attack_depth__lua_aspect_factory("attack_depth*lua_aspect");
-
 static register_lua_aspect_factory< aspect_attacks_lua >
 	attacks__lua_aspect_factory("attacks*lua_aspect");
 
@@ -435,18 +442,24 @@ static register_lua_aspect_factory< lua_aspect<double>>
 static register_lua_aspect_factory< lua_aspect<config>>
 	leader_goal__lua_aspect_factory("leader_goal*lua_aspect");
 
-static register_lua_aspect_factory< lua_aspect<bool>>
+static register_lua_aspect_factory< lua_aspect<utils::variant<bool, std::vector<std::string>>>>
 	leader_ignores_keep__lua_aspect_factory("leader_ignores_keep*lua_aspect");
 
 static register_lua_aspect_factory< lua_aspect<double>>
 	leader_value__lua_aspect_factory("leader_value*lua_aspect");
 
-static register_lua_aspect_factory< lua_aspect<bool>>
+static register_lua_aspect_factory< lua_aspect<utils::variant<bool, std::vector<std::string>>>>
 	passive_leader__lua_aspect_factory("passive_leader*lua_aspect");
 
-static register_lua_aspect_factory< lua_aspect<bool>>
+static register_lua_aspect_factory< lua_aspect<utils::variant<bool, std::vector<std::string>>>>
 	passive_leader_shares_keep__lua_aspect_factory("passive_leader_shares_keep*lua_aspect");
 
+static register_lua_aspect_factory< lua_aspect<double>>
+	retreat_enemy_weight__lua_aspect_factory("retreat_enemy_weight*lua_aspect");
+
+static register_lua_aspect_factory< lua_aspect<double>>
+	retreat_factor__lua_aspect_factory("retreat_factor*lua_aspect");
+
 static register_lua_aspect_factory< lua_aspect<double>>
 	scout_village_targeting__lua_aspect_factory("scout_village_targeting*lua_aspect");
 
@@ -462,7 +475,7 @@ static register_lua_aspect_factory< lua_aspect<double>>
 static register_lua_aspect_factory< lua_aspect<int>>
 	villages_per_scout__lua_aspect_factory("villages_per_scout*lua_aspect");
 
-static register_lua_aspect_factory< lua_aspect< std::vector<std::string>> >
+static register_lua_aspect_factory< lua_aspect< std::vector<std::string>>>
 	recruitment_pattern__lua_aspect_factory("recruitment_pattern*lua_aspect");
 
 
