@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2004 - 2018 by Philippe Plantier <ayin@anathas.org>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2004 - 2021
+	by Philippe Plantier <ayin@anathas.org>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -27,16 +28,6 @@ void new_animation_frame();
 int get_current_animation_tick();
 
 template<typename T>
-class void_value
-{
-public:
-	const T operator()() const
-	{
-		return T();
-	}
-};
-
-template<typename T, typename T_void_value = void_value<T>>
 class animated
 {
 public:
@@ -98,9 +89,9 @@ public:
 	int get_current_frame_duration() const;
 	int get_current_frame_time() const;
 	const T& get_first_frame() const;
-	const T& get_frame(size_t n) const;
+	const T& get_frame(std::size_t n) const;
 	const T& get_last_frame() const;
-	size_t get_frames_count() const;
+	std::size_t get_frames_count() const;
 
 	void force_change()
 	{
