@@ -1,20 +1,19 @@
 /*
-   Copyright (C) 2003 - 2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
-#include "global.hpp"
-
 #include <algorithm> // for max
 #include <cstdint>
 #include <ostream>
@@ -256,7 +255,7 @@ namespace std
 	template<>
 	struct hash<color_t>
 	{
-		size_t operator()(const color_t& c) const NOEXCEPT
+		std::size_t operator()(const color_t& c) const noexcept
 		{
 			return c.to_rgba_bytes();
 		}
