@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2018 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -24,26 +25,23 @@
 #include "lexical_cast.hpp"
 #include "log.hpp"
 #include "utils/const_clone.hpp"
-#include "utils/functional.hpp"
 #include "deprecation.hpp"
 #include "game_version.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <algorithm>
 #include <cstdlib>
 #include <cstring>
-#include <deque>
 #include <istream>
 #include <locale>
 
-#include <boost/variant/apply_visitor.hpp>
-#include <boost/variant/get.hpp>
-#include <boost/variant/static_visitor.hpp>
-#include <boost/variant/variant.hpp>
-
 static lg::log_domain log_config("config");
 #define ERR_CF LOG_STREAM(err, log_config)
 #define DBG_CF LOG_STREAM(debug, log_config)
 
+static lg::log_domain log_wml("wml");
+#define ERR_WML LOG_STREAM(err, log_wml)
+
 namespace
 {
 // std::map::operator[] does not support heterogeneous lookup so we need this to work around.
@@ -70,6 +68,7 @@ int map_erase_key(Map& map, Key&& key)
 	}
 	return 0;
 }
+
 }
 
 struct config_implementation
@@ -192,25 +191,21 @@ bool config::valid_tag(config_key_type name)
 	} else {
 		return std::all_of(name.begin(), name.end(), [](const char& c)
 		{
-			// Only alphanumeric ASCII characters and underscores are allowed
-			return std::isalnum(c, std::locale::classic()) || c == '_';
+			/* Only alphanumeric ASCII characters and underscores are allowed.
+
+			We're using a manual check mainly for performance. @gfgtdf measured
+			that a manual check can be up to 30 times faster than std::isalnum().
+
+			- Jyrki, 2019-01-19 */
+			return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') ||
+				(c >= '0' && c <= '9') || (c == '_');
 		});
 	}
 }
 
 bool config::valid_attribute(config_key_type name)
 {
-	if(name.empty()) {
-		return false;
-	}
-
-	for(char c : name) {
-		if(!std::isalnum(c, std::locale::classic()) && c != '_') {
-			return false;
-		}
-	}
-
-	return true;
+	return valid_tag(name);
 }
 
 bool config::has_attribute(config_key_type key) const
@@ -219,22 +214,6 @@ bool config::has_attribute(config_key_type key) const
 	return values_.find(key) != values_.end();
 }
 
-bool config::has_old_attribute(config_key_type key, const std::string& old_key, const std::string& msg) const
-{
-	check_valid();
-	if(values_.find(key) != values_.end()) {
-		return true;
-	} else if(values_.find(old_key) != values_.end()) {
-		if(!msg.empty()) {
-			lg::wml_error() << msg;
-		}
-
-		return true;
-	}
-
-	return false;
-}
-
 void config::remove_attribute(config_key_type key)
 {
 	check_valid();
@@ -416,7 +395,8 @@ bool config::has_child(config_key_type key) const
 {
 	check_valid();
 
-	return children_.find(key) != children_.end();
+	child_map::const_iterator i = children_.find(key);
+	return i != children_.end() && !i->second.empty();
 }
 
 config& config::child(config_key_type key, int n)
@@ -427,18 +407,28 @@ config& config::child(config_key_type key, int n)
 	if(i == children_.end()) {
 		DBG_CF << "The config object has no child named »" << key << "«.\n";
 
-		return invalid;
+		if(throw_when_child_not_found::do_throw()) {
+			throw error("Child not found");
+		} else {
+			return invalid;
+		}
 	}
 
-	if(n < 0)
+	if(n < 0) {
 		n = i->second.size() + n;
-	if(size_t(n) < i->second.size()) {
-		return *i->second[n];
-	} else {
+	}
+
+	try {
+		return *i->second.at(n);
+	} catch(const std::out_of_range&) {
 		DBG_CF << "The config object has only »" << i->second.size() << "« children named »" << key
 			   << "«; request for the index »" << n << "« cannot be honored.\n";
 
-		return invalid;
+		if(throw_when_child_not_found::do_throw()) {
+			throw error("Child at index not found");
+		} else {
+			return invalid;
+		}
 	}
 }
 
@@ -452,6 +442,26 @@ const config& config::child(config_key_type key, const std::string& parent) cons
 	return config_implementation::child(this, key, parent);
 }
 
+utils::optional_reference<config> config::optional_child(config_key_type key, int n)
+{
+	try {
+		throw_when_child_not_found raii_helper{};
+		return child(key, n);
+	} catch(const error&) {
+		return std::nullopt;
+	}
+}
+
+utils::optional_reference<const config> config::optional_child(config_key_type key, int n) const
+{
+	try {
+		throw_when_child_not_found raii_helper{};
+		return child(key, n);
+	} catch(const error&) {
+		return std::nullopt;
+	}
+}
+
 const config& config::child_or_empty(config_key_type key) const
 {
 	static const config empty_cfg;
@@ -475,6 +485,32 @@ config& config::child_or_add(config_key_type key)
 	return add_child(key);
 }
 
+utils::optional_reference<const config> config::get_deprecated_child(config_key_type old_key, const std::string& in_tag, DEP_LEVEL level, const std::string& message) const {
+	check_valid();
+
+	if(auto i = children_.find(old_key); i != children_.end() && !i->second.empty()) {
+		const std::string what = formatter() << "[" << in_tag << "][" << old_key << "]";
+		deprecated_message(what, level, "", message);
+		return *i->second.front();
+	}
+
+	return std::nullopt;
+}
+
+config::const_child_itors config::get_deprecated_child_range(config_key_type old_key, const std::string& in_tag, DEP_LEVEL level, const std::string& message) const {
+	check_valid();
+	static child_list dummy;
+	const child_list* p = &dummy;
+
+	if(auto i = children_.find(old_key); i != children_.end() && !i->second.empty()) {
+		const std::string what = formatter() << "[" << in_tag << "][" << old_key << "]";
+		deprecated_message(what, level, "", message);
+		p = &i->second;
+	}
+
+	return const_child_itors(const_child_iterator(p->begin()), const_child_iterator(p->end()));
+}
+
 config& config::add_child(config_key_type key)
 {
 	check_valid();
@@ -541,6 +577,56 @@ config& config::add_child_at(config_key_type key, const config& val, unsigned in
 	return *v[index];
 }
 
+size_t config::find_total_first_of(config_key_type key, size_t start)
+{
+	assert(start <= ordered_children.size());
+	const size_t npos = static_cast<size_t>(-1);
+
+	auto pos = std::find_if(ordered_begin() + start, ordered_end(), [&](const config::any_child& can){ return can.key == key; });
+
+	if(pos == ordered_end()) {
+		return npos;
+	}
+
+	return static_cast<size_t>(pos - ordered_begin());
+}
+
+config& config::add_child_at_total(config_key_type key, const config &val, size_t pos)
+{
+	assert(pos <= ordered_children.size());
+	if(pos == ordered_children.size()) {
+		//optimisation
+		return config::add_child(key, val);
+	}
+
+	auto end = ordered_children.end();
+	auto pos_it = ordered_children.begin() + pos;
+	auto next = std::find_if(pos_it, end,[&](const child_pos& p){ return p.pos->first == key; });
+
+	if(next == end) {
+		config& res = config::add_child(key, val);
+		//rotate the just inserted element to position pos.
+		std::rotate(ordered_children.begin() + pos, ordered_children.end() - 1, ordered_children.end());
+		return res;
+	}
+
+	auto pl = next->pos;
+	child_list& l = pl->second;
+	unsigned int index = next->index;
+	config& res = **(l.emplace(l.begin() + index, new config(val)));
+
+	for(auto ord = next; ord != end; ++ord) {
+		//this changes next->index and all later refernces to that tag.
+		if(ord->pos == pl) {
+			++ord->index;
+		}
+	}
+
+	//finally insert our new child in ordered_children.
+	ordered_children.insert(pos_it, { pl, index });
+	return res;
+}
+
 namespace
 {
 struct remove_ordered
@@ -711,11 +797,16 @@ config::attribute_value& config::operator[](config_key_type key)
 	return res->second;
 }
 
-const config::attribute_value& config::get_old_attribute(
-		config_key_type key, const std::string& old_key, const std::string& in_tag) const
+const config::attribute_value& config::get_old_attribute(config_key_type key, const std::string& old_key, const std::string& in_tag, const std::string& message) const
 {
 	check_valid();
 
+	if(has_attribute(old_key)) {
+		const std::string what = formatter() << "[" << in_tag << "]" << old_key << "=";
+		const std::string msg  = formatter() << "Use " << key << "= instead. " << message;
+		deprecated_message(what, DEP_LEVEL::INDEFINITE, "", msg);
+	}
+
 	attribute_map::const_iterator i = values_.find(key);
 	if(i != values_.end()) {
 		return i->second;
@@ -723,13 +814,19 @@ const config::attribute_value& config::get_old_attribute(
 
 	i = values_.find(old_key);
 	if(i != values_.end()) {
-		if(!in_tag.empty()) {
-			const std::string what = formatter() << "[" << in_tag << "]" << old_key << "=";
-			const std::string msg  = formatter() << "Use " << key << "= instead.";
-			deprecated_message(what, DEP_LEVEL::INDEFINITE, "", msg);
-			lg::wml_error() << msg;
-		}
+		return i->second;
+	}
 
+	static const attribute_value empty_attribute;
+	return empty_attribute;
+}
+
+const config::attribute_value& config::get_deprecated_attribute(config_key_type old_key, const std::string& in_tag, DEP_LEVEL level, const std::string& message) const {
+	check_valid();
+
+	if(auto i = values_.find(old_key); i != values_.end()) {
+		const std::string what = formatter() << "[" << in_tag << "]" << old_key << "=";
+		deprecated_message(what, level, "", message);
 		return i->second;
 	}
 
@@ -793,7 +890,11 @@ config& config::find_child(config_key_type key, const std::string& name, const s
 	if(i == children_.end()) {
 		DBG_CF << "Key »" << name << "« value »" << value << "« pair not found as child of key »" << key << "«.\n";
 
-		return invalid;
+		if(throw_when_child_not_found::do_throw()) {
+			throw error("Child not found");
+		} else {
+			return invalid;
+		}
 	}
 
 	const child_list::iterator j = std::find_if(i->second.begin(), i->second.end(),
@@ -809,7 +910,11 @@ config& config::find_child(config_key_type key, const std::string& name, const s
 
 	DBG_CF << "Key »" << name << "« value »" << value << "« pair not found as child of key »" << key << "«.\n";
 
-	return invalid;
+	if(throw_when_child_not_found::do_throw()) {
+		throw error("Child not found");
+	} else {
+		return invalid;
+	}
 }
 
 void config::clear()
@@ -966,8 +1071,8 @@ void config::get_diff(const config& c, config& res) const
 		const child_list& a = itor_a != children_.end() ? itor_a->second : dummy;
 		const child_list& b = itor_b != c.children_.end() ? itor_b->second : dummy;
 
-		size_t ndeletes = 0;
-		size_t ai = 0, bi = 0;
+		std::size_t ndeletes = 0;
+		std::size_t ai = 0, bi = 0;
 		while(ai != a.size() || bi != b.size()) {
 			// If the two elements are the same, nothing needs to be done.
 			if(ai < a.size() && bi < b.size() && *a[ai] == *b[bi]) {
@@ -1038,7 +1143,7 @@ void config::apply_diff(const config& diff, bool track /* = false */)
 	}
 
 	for(const config& i : diff.child_range("change_child")) {
-		const size_t index = lexical_cast<size_t>(i["index"].str());
+		const std::size_t index = lexical_cast<std::size_t>(i["index"].str());
 		for(const any_child& item : i.all_children_range()) {
 			if(item.key.empty()) {
 				continue;
@@ -1054,7 +1159,7 @@ void config::apply_diff(const config& diff, bool track /* = false */)
 	}
 
 	for(const config& i : diff.child_range("insert_child")) {
-		const size_t index = lexical_cast<size_t>(i["index"].str());
+		const std::size_t index = lexical_cast<std::size_t>(i["index"].str());
 		for(const any_child& item : i.all_children_range()) {
 			config& inserted = add_child_at(item.key, item.cfg, index);
 			if(track) {
@@ -1064,7 +1169,7 @@ void config::apply_diff(const config& diff, bool track /* = false */)
 	}
 
 	for(const config& i : diff.child_range("delete_child")) {
-		const size_t index = lexical_cast<size_t>(i["index"].str());
+		const std::size_t index = lexical_cast<std::size_t>(i["index"].str());
 		for(const any_child& item : i.all_children_range()) {
 			if(!track) {
 				remove_child(item.key, index);
@@ -1084,14 +1189,14 @@ void config::clear_diff_track(const config& diff)
 {
 	remove_attribute(diff_track_attribute);
 	for(const config& i : diff.child_range("delete_child")) {
-		const size_t index = lexical_cast<size_t>(i["index"].str());
+		const std::size_t index = lexical_cast<std::size_t>(i["index"].str());
 		for(const any_child& item : i.all_children_range()) {
 			remove_child(item.key, index);
 		}
 	}
 
 	for(const config& i : diff.child_range("change_child")) {
-		const size_t index = lexical_cast<size_t>(i["index"].str());
+		const std::size_t index = lexical_cast<std::size_t>(i["index"].str());
 		for(const any_child& item : i.all_children_range()) {
 			if(item.key.empty()) {
 				continue;
@@ -1178,23 +1283,50 @@ void config::inherit_from(const config& c)
 	swap(scratch);
 }
 
+/**
+ * Merge the attributes of config 'c' into this config, preserving this config's values.
+ */
+void config::inherit_attributes(const config& cfg)
+{
+	check_valid(cfg);
+	for(const attribute& v : cfg.values_) {
+		attribute_value& v2 = values_[v.first];
+		if(v2.blank()) {
+			v2 = v.second;
+		}
+	}
+}
 bool config::matches(const config& filter) const
 {
 	check_valid(filter);
 
+	bool result = true;
+
 	for(const attribute& i : filter.attribute_range()) {
-		const attribute_value* v = get(i.first);
-		if(!v || *v != i.second) {
-			return false;
+		if(i.first.compare(0, 8, "glob_on_") == 0) {
+			const attribute_value* v = get(i.first.substr(8));
+			if(!v || !utils::wildcard_string_match(v->str(), i.second.str())) {
+				result = false;
+				break;
+			}
+		} else {
+			const attribute_value* v = get(i.first);
+			if(!v || *v != i.second) {
+				result = false;
+				break;
+			}
 		}
 	}
 
 	for(const any_child& i : filter.all_children_range()) {
 		if(i.key == "not") {
-			if(matches(i.cfg)) {
-				return false;
-			}
-
+			result = result && !matches(i.cfg);
+			continue;
+		} else if(i.key == "and") {
+			result = result && matches(i.cfg);
+			continue;
+		} else if(i.key == "or") {
+			result = result || matches(i.cfg);
 			continue;
 		}
 
@@ -1206,12 +1338,10 @@ bool config::matches(const config& filter) const
 			}
 		}
 
-		if(!found) {
-			return false;
-		}
+		result = result && found;
 	}
 
-	return true;
+	return result;
 }
 
 std::string config::debug() const
@@ -1330,11 +1460,15 @@ void swap(config& lhs, config& rhs)
 
 bool config::validate_wml() const
 {
-	return std::all_of(children_.begin(), children_.end(), [](const child_map::value_type& pair)
+	return std::all_of(children_.begin(), children_.end(), [](const auto& pair)
 	{
 		return valid_tag(pair.first) &&
 			std::all_of(pair.second.begin(), pair.second.end(),
-			[](const std::unique_ptr<config>& c) { return c->validate_wml(); });
+			[](const auto& c) { return c->validate_wml(); });
+	}) &&
+		std::all_of(values_.begin(), values_.end(), [](const auto& pair)
+	{
+		return valid_attribute(pair.first);
 	});
 }
 
