@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -27,7 +28,9 @@
 
 #pragma once
 
-#include "global.hpp"
+#include "config_attribute_value.hpp"
+#include "exceptions.hpp"
+#include "utils/optional_reference.hpp"
 
 #include <climits>
 #include <ctime>
@@ -35,51 +38,17 @@
 #include <iosfwd>
 #include <iterator>
 #include <map>
+#include <memory>
 #include <string>
+#include <string_view>
+#include <type_traits>
 #include <utility>
 #include <vector>
-#include <type_traits>
-#include <memory>
 
-#include <boost/exception/exception.hpp>
-#include <boost/variant/apply_visitor.hpp>
-#include <boost/variant/variant.hpp>
 #include <boost/range/iterator_range.hpp>
 
-#include "config_attribute_value.hpp"
-#include "exceptions.hpp"
-
-#ifdef HAVE_CXX14
-#	ifdef __clang__ // Check this first, because clang also defines __GNUC__
-#		ifdef __apple_build_version__ // Apple clang
-#			if (__clang_major__ == 5 && __clang_minor__ >= 1) || __clang_major__ > 5 // Apple clang 5.1+
-#				define USE_HETEROGENOUS_LOOKUPS
-#			endif
-#		else // Non-Apple clang
-#			if (__clang_major__ == 3 && __clang_minor__ >= 4) || __clang_major__ > 3 // clang 3.4+
-#				define USE_HETEROGENOUS_LOOKUPS
-#			endif
-#		endif
-#	elif defined(__GNUC__) && __GNUC__ >= 5 // GCC 5.0+
-#		define USE_HETEROGENOUS_LOOKUPS
-#	endif
-#endif
-
-#if defined(_MSC_VER) && _MSC_VER >= 1900 // MSVC 2015
-#	define USE_HETEROGENOUS_LOOKUPS
-#endif
-
-#ifdef USE_HETEROGENOUS_LOOKUPS
-#if BOOST_VERSION > 106100
-#include <boost/utility/string_view.hpp>
-using config_key_type = boost::string_view;
-#else
-#include <boost/utility/string_ref.hpp>
-using config_key_type = boost::string_ref;
-#endif
-#else
-using config_key_type = const std::string &;
-#endif
+using config_key_type = std::string_view;
+enum class DEP_LEVEL : uint8_t;
 
 class config;
 class enum_tag;
@@ -124,7 +93,7 @@ public:
 	/**
 	 * Creates a config with several attributes and children.
 	 * Pass the keys/tags and values/children alternately.
-	 * @example config("key", 42, "value", config())
+	 * For example: config("key", 42, "value", config())
 	 */
 	template<typename... T>
 	explicit config(config_key_type first, T&&... args);
@@ -140,12 +109,11 @@ public:
 	explicit operator bool() const
 	{ return this != &invalid; }
 
+	static config& get_invalid()
+	{ return invalid; }
+
 	typedef std::vector<std::unique_ptr<config>> child_list;
-	typedef std::map<std::string, child_list
-#ifdef USE_HETEROGENOUS_LOOKUPS
-		, std::less<>
-#endif
-	> child_map;
+	typedef std::map<std::string, child_list, std::less<>> child_map;
 
 	struct const_child_iterator;
 
@@ -249,9 +217,7 @@ public:
 	typedef std::map<
 		std::string
 		, attribute_value
-#ifdef USE_HETEROGENOUS_LOOKUPS
 		, std::less<>
-#endif
 	> attribute_map;
 	typedef attribute_map::value_type attribute;
 	struct const_attribute_iterator;
@@ -337,13 +303,45 @@ public:
 	 */
 	const config & child_or_empty(config_key_type key) const;
 
+	/**
+	 * An object of this type will cause the following functions to throw a config::error instead
+	 * of returning a reference to the invalid config for the duration of its lfetime. If multiple
+	 * instances exist simultaneously, this behavior will persist until all objects are destroyed.
+	 *
+	 * - @c child
+	 * - @c find_child
+	 */
+	class throw_when_child_not_found
+	{
+	public:
+		friend class config;
+
+		throw_when_child_not_found()
+		{
+			instances++;
+		}
+
+		~throw_when_child_not_found()
+		{
+			instances--;
+		}
+
+		static bool do_throw()
+		{
+			return instances > 0;
+		}
+
+	private:
+		static inline unsigned instances = 0;
+	};
+
 	/**
 	 * Returns the nth child with the given @a key, or
 	 * a reference to an invalid config if there is none.
 	 * @note A negative @a n accesses from the end of the object.
 	 *       For instance, -1 is the index of the last child.
 	 */
-	config &child(config_key_type key, int n = 0);
+	config& child(config_key_type key, int n = 0);
 
 	/**
 	 * Returns the nth child with the given @a key, or
@@ -352,7 +350,14 @@ public:
 	 *       For instance, -1 is the index of the last child.
 	 */
 	const config& child(config_key_type key, int n = 0) const
-	{ return const_cast<config *>(this)->child(key, n); }
+	{ return const_cast<config*>(this)->child(key, n); }
+
+	/** Euivalent to @ref child, but returns an empty optional if the nth child was not found. */
+	utils::optional_reference<config> optional_child(config_key_type key, int n = 0);
+
+	/** Euivalent to @ref child, but returns an empty optional if the nth child was not found. */
+	utils::optional_reference<const config> optional_child(config_key_type key, int n = 0) const;
+
 	/**
 	 * Returns a mandatory child node.
 	 *
@@ -383,12 +388,35 @@ public:
 	 *
 	 * @returns                   The wanted child node.
 	 */
-	const config& child(
-		config_key_type key
-			, const std::string& parent) const;
+	const config& child(config_key_type key, const std::string& parent) const;
+
+	/**
+	 * Get a deprecated child and log a deprecation message
+	 * @param old_key The deprecated child to return if present
+	 * @param in_tag The name of the tag this child appears in
+	 * @param level The deprecation level
+	 * @param message An explanation of the deprecation, possibly mentioning an alternative
+	 * @note The deprecation message will be a level 3 deprecation.
+	 */
+	utils::optional_reference<const config> get_deprecated_child(config_key_type old_key, const std::string& in_tag, DEP_LEVEL level, const std::string& message) const;
+
+	/**
+	 * Get a deprecated child rangw and log a deprecation message
+	 * @param old_key The deprecated child to return if present
+	 * @param in_tag The name of the tag this child appears in
+	 * @param level The deprecation level
+	 * @param message An explanation of the deprecation, possibly mentioning an alternative
+	 * @note The deprecation message will be a level 3 deprecation.
+	 */
+	const_child_itors get_deprecated_child_range(config_key_type old_key, const std::string& in_tag, DEP_LEVEL level, const std::string& message) const;
 
 	config& add_child(config_key_type key);
 	config& add_child(config_key_type key, const config& val);
+	/**
+	 * @param key the tag name
+	 * @param val the contents of the tag
+	 * @param index is the index of the new child within all children of type key.
+	 */
 	config& add_child_at(config_key_type key, const config &val, unsigned index);
 
 	config &add_child(config_key_type key, config &&val);
@@ -405,7 +433,6 @@ public:
 	 */
 	const attribute_value& operator[](config_key_type key) const;
 
-#ifdef USE_HETEROGENOUS_LOOKUPS
 	/**
 	* Returns a reference to the attribute with the given @a key.
 	* Creates it if it does not exist.
@@ -423,7 +450,6 @@ public:
 	{
 		return operator[](config_key_type(key));
 	}
-#endif
 
 	/**
 	* Returns a reference to the attribute with the given @a key.
@@ -452,13 +478,24 @@ public:
 	/**
 	 * Function to handle backward compatibility
 	 * Get the value of key and if missing try old_key
-	 * and log msg as a WML error (if not empty)
+	 * and log a deprecation message
+	 * @param key The non-deprecated attribute to return
+	 * @param old_key The deprecated attribute to return if @a key is missing
+	 * @param in_tag The name of the tag these attributes appear in
+	 * @param message An explanation of the deprecation, to be output if @a old_key is present.
+	 * @note The deprecation message will be a level 1 deprecation.
 	*/
-	const attribute_value &get_old_attribute(config_key_type key, const std::string &old_key, const std::string& in_tag = "") const;
+	const attribute_value &get_old_attribute(config_key_type key, const std::string &old_key, const std::string& in_tag, const std::string& message = "") const;
+
 	/**
-	 * Returns a reference to the first child with the given @a key.
-	 * Creates the child if it does not yet exist.
+	 * Get a deprecated attribute without a direct substitute,
+	 * and log a deprecation message
+	 * @param old_key The deprecated attribute to return if present
+	 * @param in_tag The name of the tag this attribute appears in
+	 * @param level The deprecation level
+	 * @param message An explanation of the deprecation, possibly mentioning an alternative
 	 */
+	const attribute_value& get_deprecated_attribute(config_key_type old_key, const std::string& in_tag, DEP_LEVEL level, const std::string& message) const;
 
 	/**
 	 * Inserts an attribute into the config
@@ -471,25 +508,34 @@ public:
 		operator[](key) = std::forward<T>(value);
 	}
 
+	/**
+	 * Returns a reference to the first child with the given @a key.
+	 * Creates the child if it does not yet exist.
+	 */
 	config &child_or_add(config_key_type key);
 
 	bool has_attribute(config_key_type key) const;
-	/**
-	 * Function to handle backward compatibility
-	 * Check if has key or old_key
-	 * and log msg as a WML error (if not empty)
-	*/
-	bool has_old_attribute(config_key_type key, const std::string &old_key, const std::string& msg = "") const;
 
 	void remove_attribute(config_key_type key);
 	void merge_attributes(const config &);
 	template<typename... T>
 	void remove_attributes(T... keys) {
-		for(const std::string& key : {keys...}) {
+		for(const auto& key : {keys...}) {
 			remove_attribute(key);
 		}
 	}
 
+	template<typename... T>
+	void copy_attributes(const config& from, T... keys)
+	{
+		for(const auto& key : {keys...}) {
+			auto* attr = from.get(key);
+			if(attr) {
+				(*this)[key] = *attr;
+			}
+		}
+	}
+
 	const_attr_itors attribute_range() const;
 	attr_itors attribute_range();
 
@@ -504,7 +550,9 @@ public:
 		const std::string &value) const
 	{ return const_cast<config *>(this)->find_child(key, name, value); }
 
+private:
 	void clear_children_impl(config_key_type key);
+public:
 	template<typename... T>
 	void clear_children(T... keys) {
 		for(auto key : {keys...}) {
@@ -532,7 +580,7 @@ public:
 	std::string debug() const;
 	std::string hash() const;
 
-	struct error : public game::error, public boost::exception {
+	struct error : public game::error {
 		error(const std::string& message) : game::error(message) {}
 	};
 
@@ -624,7 +672,7 @@ public:
 		typedef std::vector<child_pos>::const_iterator Itor;
 		typedef const_all_children_iterator this_type;
 		explicit const_all_children_iterator(const Itor &i): i_(i) {}
-		const_all_children_iterator(all_children_iterator& i): i_(i.i_) {}
+		const_all_children_iterator(const all_children_iterator& i): i_(i.i_) {}
 
 		const_all_children_iterator &operator++() { ++i_; return *this; }
 		const_all_children_iterator operator++(int) { return const_all_children_iterator(i_++); }
@@ -659,6 +707,14 @@ public:
 		friend class config;
 	};
 
+	/**
+	 * @param key the tag name
+	 * @param val the contents of the tag
+	 * @param pos is the index of the new child in _all_ children.
+	 */
+	config& add_child_at_total(config_key_type key, const config &val, size_t pos);
+	size_t find_total_first_of(config_key_type key, size_t start = 0);
+
 	typedef boost::iterator_range<all_children_iterator> all_children_itors;
 	typedef boost::iterator_range<const_all_children_iterator> const_all_children_itors;
 
@@ -718,6 +774,10 @@ public:
 	 * Merge config 'c' into this config, preserving this config's values.
 	 */
 	void inherit_from(const config& c);
+	/**
+	 * Merge the attributes of config 'c' into this config, preserving this config's values.
+	 */
+	void inherit_attributes(const config& c);
 
 	bool matches(const config &filter) const;
 
@@ -840,10 +900,3 @@ inline config::config(config_key_type first, T&&... args)
 	detail::config_construct_unpacker<config_key_type, T...> unpack;
 	unpack.visit(*this, first, std::forward<T>(args)...);
 }
-
-class variable_set
-{
-public:
-	virtual ~variable_set() {}
-	virtual config::attribute_value get_variable_const(const std::string &id) const = 0;
-};
