@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2018 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -22,19 +23,14 @@
 
 #include "lexical_cast.hpp"
 #include "log.hpp"
-#include "utils/const_clone.hpp"
-#include "utils/functional.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <cstdlib>
 #include <cstring>
 #include <deque>
+#include <functional>
 #include <istream>
 
-#include <boost/variant/apply_visitor.hpp>
-#include <boost/variant/get.hpp>
-#include <boost/variant/static_visitor.hpp>
-#include <boost/variant/variant.hpp>
-
 static lg::log_domain log_config("config");
 #define ERR_CF LOG_STREAM(err, log_config)
 #define DBG_CF LOG_STREAM(debug, log_config)
@@ -244,9 +240,9 @@ void config_attribute_value::write_if_not_empty(const std::string& v)
 
 bool config_attribute_value::to_bool(bool def) const
 {
-	if(const yes_no* p = boost::get<const yes_no>(&value_))
+	if(const yes_no* p = utils::get_if<yes_no>(&value_))
 		return *p;
-	if(const true_false* p = boost::get<const true_false>(&value_))
+	if(const true_false* p = utils::get_if<true_false>(&value_))
 		return *p;
 
 	// No other types are ever recognized as boolean.
@@ -255,15 +251,18 @@ bool config_attribute_value::to_bool(bool def) const
 
 namespace
 {
-/// Visitor for converting a variant to a numeric type (T).
+/** Visitor for converting a variant to a numeric type (T). */
 template<typename T>
-class attribute_numeric_visitor : public boost::static_visitor<T>
+class attribute_numeric_visitor
+#ifdef USING_BOOST_VARIANT
+	: public boost::static_visitor<T>
+#endif
 {
 public:
 	// Constructor stores the default value.
 	attribute_numeric_visitor(T def) : def_(def) {}
 
-	T operator()(const boost::blank&) const { return def_; }
+	T operator()(const utils::monostate&) const { return def_; }
 	T operator()(bool)                 const { return def_; }
 	T operator()(int i)                const { return static_cast<T>(i); }
 	T operator()(unsigned long long u) const { return static_cast<T>(u); }
@@ -291,14 +290,14 @@ unsigned config_attribute_value::to_unsigned(unsigned def) const
 	return apply_visitor(attribute_numeric_visitor<unsigned>(def));
 }
 
-size_t config_attribute_value::to_size_t(size_t def) const
+std::size_t config_attribute_value::to_size_t(std::size_t def) const
 {
-	return apply_visitor(attribute_numeric_visitor<size_t>(def));
+	return apply_visitor(attribute_numeric_visitor<std::size_t>(def));
 }
 
-time_t config_attribute_value::to_time_t(time_t def) const
+std::time_t config_attribute_value::to_time_t(std::time_t def) const
 {
-	return apply_visitor(attribute_numeric_visitor<time_t>(def));
+	return apply_visitor(attribute_numeric_visitor<std::time_t>(def));
 }
 
 double config_attribute_value::to_double(double def) const
@@ -306,19 +305,22 @@ double config_attribute_value::to_double(double def) const
 	return apply_visitor(attribute_numeric_visitor<double>(def));
 }
 
-/// Visitor for converting a variant to a string.
-class config_attribute_value::string_visitor : public boost::static_visitor<std::string>
+/** Visitor for converting a variant to a string. */
+class config_attribute_value::string_visitor
+#ifdef USING_BOOST_VARIANT
+	: public boost::static_visitor<std::string>
+#endif
 {
 	const std::string default_;
 
 public:
 	string_visitor(const std::string& fallback) : default_(fallback) {}
 
-	std::string operator()(const boost::blank &) const { return default_; }
+	std::string operator()(const utils::monostate &) const { return default_; }
 	std::string operator()(const yes_no & b)     const { return b.str(); }
 	std::string operator()(const true_false & b) const { return b.str(); }
-	std::string operator()(int i)                const { return lexical_cast<std::string>(i); }
-	std::string operator()(unsigned long long u) const { return lexical_cast<std::string>(u); }
+	std::string operator()(int i)                const { return std::to_string(i); }
+	std::string operator()(unsigned long long u) const { return std::to_string(u); }
 	std::string operator()(double d)             const { return lexical_cast<std::string>(d); }
 	std::string operator()(const std::string& s) const { return s; }
 	std::string operator()(const t_string& s)    const { return s.str(); }
@@ -331,7 +333,7 @@ std::string config_attribute_value::str(const std::string& fallback) const
 
 t_string config_attribute_value::t_str() const
 {
-	if(const t_string* p = boost::get<const t_string>(&value_)) {
+	if(const t_string* p = utils::get_if<t_string>(&value_)) {
 		return *p;
 	}
 
@@ -343,7 +345,7 @@ t_string config_attribute_value::t_str() const
  */
 bool config_attribute_value::blank() const
 {
-	return boost::get<const boost::blank>(&value_) != nullptr;
+	return utils::holds_alternative<utils::monostate>(value_);
 }
 
 /**
@@ -351,19 +353,22 @@ bool config_attribute_value::blank() const
  */
 bool config_attribute_value::empty() const
 {
-	if(boost::get<const boost::blank>(&value_)) {
+	if(blank()) {
 		return true;
 	}
 
-	if(const std::string* p = boost::get<const std::string>(&value_)) {
+	if(const std::string* p = utils::get_if<std::string>(&value_)) {
 		return p->empty();
 	}
 
 	return false;
 }
 
-/// Visitor handling equality checks.
-class config_attribute_value::equality_visitor : public boost::static_visitor<bool>
+/** Visitor handling equality checks. */
+class config_attribute_value::equality_visitor
+#ifdef USING_BOOST_VARIANT
+	: public boost::static_visitor<bool>
+#endif
 {
 public:
 	// Most generic: not equal.
@@ -399,7 +404,7 @@ public:
  */
 bool config_attribute_value::operator==(const config_attribute_value& other) const
 {
-	return boost::apply_visitor(equality_visitor(), value_, other.value_);
+	return utils::visit(equality_visitor(), value_, other.value_);
 }
 
 /**
@@ -415,7 +420,7 @@ bool config_attribute_value::equals(const std::string& str) const
 	return *this == v;
 	// if c["a"] = "1" then this solution would have resulted in c["a"] == "1" being false
 	// because a["a"] is '1' and not '"1"'.
-	// return boost::apply_visitor(std::bind( equality_visitor(), _1, std::cref(str) ), value_);
+	// return boost::apply_visitor(std::bind( equality_visitor(), std::placeholders::_1, std::cref(str) ), value_);
 	// that's why we don't use it.
 }
 
@@ -423,5 +428,13 @@ std::ostream& operator<<(std::ostream& os, const config_attribute_value& v)
 {
 	// Simple implementation, but defined out-of-line because of the templating
 	// involved.
-	return os << v.value_;
+	v.apply_visitor([&os](const auto& val) { os << val; });
+	return os;
+}
+
+namespace utils
+{
+	std::vector<std::string> split(const config_attribute_value& val) {
+		return utils::split(val.str());
+	}
 }
