@@ -1,23 +1,23 @@
 /*
-   Copyright (C) 2006 - 2018 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "controller_base.hpp"
 
 #include "display.hpp"
 #include "events.hpp"
-#include "preferences/game.hpp"
 #include "game_config_manager.hpp"
 #include "hotkey/command_executor.hpp"
 #include "log.hpp"
@@ -26,6 +26,7 @@
 #include "preferences/game.hpp"
 #include "scripting/plugins/context.hpp"
 #include "show_dialog.hpp" //gui::in_dialog
+#include "gui/core/event/handler.hpp" // gui2::is_in_dialog
 #include "soundsource.hpp"
 #include "gui/core/timer.hpp"
 
@@ -42,7 +43,6 @@ controller_base::controller_base()
 	, scroll_down_(false)
 	, scroll_left_(false)
 	, scroll_right_(false)
-	, joystick_manager_()
 	, key_release_listener_(*this)
 	, last_mouse_is_touch_(false)
 	, long_touch_timer_(0)
@@ -247,7 +247,7 @@ void controller_base::handle_event(const SDL_Event& event)
 
 void controller_base::process(events::pump_info&)
 {
-	if(gui::in_dialog()) {
+	if(gui2::is_in_dialog()) {
 		return;
 	}
 
@@ -266,7 +266,7 @@ bool controller_base::have_keyboard_focus()
 	return true;
 }
 
-bool controller_base::handle_scroll(int mousex, int mousey, int mouse_flags, double x_axis, double y_axis)
+bool controller_base::handle_scroll(int mousex, int mousey, int mouse_flags)
 {
 	const bool mouse_in_window =
 		CVideo::get_singleton().window_has_flags(SDL_WINDOW_MOUSE_FOCUS)
@@ -322,7 +322,7 @@ bool controller_base::handle_scroll(int mousex, int mousey, int mouse_flags, dou
 			if(sdl::point_in_rect(mousex, mousey, rect) && mh_base.scroll_started()) {
 				// Scroll speed is proportional from the distance from the first
 				// middle click and scrolling speed preference.
-				const double speed = 0.04 * sqrt(static_cast<double>(scroll_speed));
+				const double speed = 0.04 * std::sqrt(static_cast<double>(scroll_speed));
 				const double snap_dist = 16; // Snap to horizontal/vertical scrolling
 				const double x_diff = (mousex - original_loc.x);
 				const double y_diff = (mousey - original_loc.y);
@@ -340,10 +340,6 @@ bool controller_base::handle_scroll(int mousex, int mousey, int mouse_flags, dou
 		}
 	}
 
-	// scroll with joystick
-	dx += round_double(x_axis * scroll_speed);
-	dy += round_double(y_axis * scroll_speed);
-
 	return get_display().scroll(dx, dy);
 }
 
@@ -388,44 +384,13 @@ void controller_base::play_slice(bool is_delay_enabled)
 
 	bool was_scrolling = scrolling_;
 
-	std::pair<double, double> values = joystick_manager_.get_scroll_axis_pair();
-	const double joystickx = values.first;
-	const double joysticky = values.second;
-
 	int mousex, mousey;
 	uint8_t mouse_flags = SDL_GetMouseState(&mousex, &mousey);
 
-	// TODO enable after an axis choosing mechanism is implemented
-#if 0
-	std::pair<double, double> values = joystick_manager_.get_mouse_axis_pair();
-	mousex += values.first * 10;
-	mousey += values.second * 10;
-	SDL_WarpMouse(mousex, mousey);
-#endif
-
-	scrolling_ = handle_scroll(mousex, mousey, mouse_flags, joystickx, joysticky);
+	scrolling_ = handle_scroll(mousex, mousey, mouse_flags);
 
 	map_location highlighted_hex = get_display().mouseover_hex();
 
-	// TODO: enable when the relative cursor movement is implemented well enough
-#if 0
-	const map_location& selected_hex = get_display().selected_hex();
-
-	if (selected_hex != map_location::null_location()) {
-		if (joystick_manager_.next_highlighted_hex(highlighted_hex, selected_hex)) {
-			get_mouse_handler_base().mouse_motion(0,0, true, true, highlighted_hex);
-			get_display().scroll_to_tile(highlighted_hex, display::ONSCREEN_WARP, false, true);
-			scrolling_ = true;
-		}
-	} else
-#endif
-
-	if(joystick_manager_.update_highlighted_hex(highlighted_hex) && get_display().get_map().on_board(highlighted_hex)) {
-		get_mouse_handler_base().mouse_motion(0, 0, true, true, highlighted_hex);
-		get_display().scroll_to_tile(highlighted_hex, display::ONSCREEN_WARP, false, true);
-		scrolling_ = true;
-	}
-
 	// be nice when window is not visible	// NOTE should be handled by display instead, to only disable drawing
 	if(is_delay_enabled && !CVideo::get_singleton().window_has_flags(SDL_WINDOW_SHOWN)) {
 		CVideo::delay(200);
@@ -489,7 +454,7 @@ bool controller_base::in_context_menu(hotkey::HOTKEY_COMMAND /*command*/) const
 	return true;
 }
 
-const config& controller_base::get_theme(const config& game_config, std::string theme_name)
+const config& controller_base::get_theme(const game_config_view& game_config, std::string theme_name)
 {
 	if(theme_name.empty()) {
 		theme_name = preferences::theme();
