@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -95,9 +96,9 @@ SDL_Cursor* create_cursor(surface surf)
 	// The width must be a multiple of 8 (SDL requirement)
 
 #ifdef __APPLE__
-	size_t cursor_width = 16;
+	std::size_t cursor_width = 16;
 #else
-	size_t cursor_width = surf->w;
+	std::size_t cursor_width = surf->w;
 	if((cursor_width % 8) != 0) {
 		cursor_width += 8 - (cursor_width % 8);
 	}
@@ -113,12 +114,12 @@ SDL_Cursor* create_cursor(surface surf)
 
 	for(int y = 0; y != surf->h; ++y) {
 		for(int x = 0; x != surf->w; ++x) {
-			if(static_cast<size_t>(x) < cursor_width) {
+			if(static_cast<std::size_t>(x) < cursor_width) {
 				uint8_t r, g, b, a;
 				SDL_GetRGBA(pixels[y * surf->w + x], surf->format, &r, &g, &b, &a);
 
-				const size_t index = y * cursor_width + x;
-				const size_t shift = 7 - (index % 8);
+				const std::size_t index = y * cursor_width + x;
+				const std::size_t shift = 7 - (index % 8);
 
 				const uint8_t trans = (a < 128 ? 0 : 1) << shift;
 				const uint8_t black = (trans == 0 || (r + g + b) / 3 > 128 ? 0 : 1) << shift;
