@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2015 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2015 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -24,14 +25,25 @@
 
 #include <cstring>
 
+#if defined(__APPLE__) || defined(_X11)
+#include <sys/utsname.h>
+#endif
+
 #if defined(__APPLE__)
 
 #include "apple_version.hpp"
+#include "serialization/string_utils.hpp"
+
+#include <map>
+#include <boost/algorithm/string/trim.hpp>
 
 #elif defined(_X11)
 
+#include "serialization/string_utils.hpp"
+
 #include <cerrno>
-#include <sys/utsname.h>
+#include <map>
+#include <boost/algorithm/string/trim.hpp>
 
 #endif
 
@@ -72,6 +84,49 @@ bool on_wine()
 
 	return GetProcAddress(ntdll, "wine_get_version") != nullptr;
 }
+
+/**
+ * Tries to find out the Windows 10 release number.
+ *
+ * This depends on the registry having special information in it. This may or
+ * may not break in the future or be faked by the application compatibility
+ * layer. Take with a grain of salt.
+ */
+std::string windows_release_id()
+{
+	char buf[256]{""};
+	DWORD size = sizeof(buf);
+
+	auto res = RegGetValueA(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", "DisplayVersion", RRF_RT_REG_SZ, nullptr, buf, &size);
+	if(res != ERROR_SUCCESS) {
+		res = RegGetValueA(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", "ReleaseId", RRF_RT_REG_SZ, nullptr, buf, &size);
+	}
+
+	return std::string{res == ERROR_SUCCESS ? buf : ""};
+}
+
+std::string windows_runtime_arch()
+{
+	SYSTEM_INFO si;
+	SecureZeroMemory(&si, sizeof(SYSTEM_INFO));
+	GetNativeSystemInfo(&si);
+
+	switch(si.wProcessorArchitecture) {
+		case PROCESSOR_ARCHITECTURE_INTEL:
+			return "x86";
+		case PROCESSOR_ARCHITECTURE_AMD64:
+			return "x86_64";
+		case PROCESSOR_ARCHITECTURE_ARM:
+			return "arm";
+		case PROCESSOR_ARCHITECTURE_ARM64:
+			return "arm64";
+		case PROCESSOR_ARCHITECTURE_IA64:
+			return "ia64";
+		default:
+			return _("cpu_architecture^<unknown>");
+	}
+}
+
 #endif
 
 #if defined(_X11)
@@ -114,24 +169,98 @@ std::string read_pipe_line(scoped_posix_pipe& p)
 
 	return ver;
 }
+
+std::map<std::string, std::string> parse_fdo_osrelease(const std::string& path)
+{
+	auto in = filesystem::istream_file(path);
+	if(!in->good()) {
+		return {};
+	}
+
+	std::map<std::string, std::string> res;
+
+	// NOTE:  intentionally basic "parsing" here. We are not supposed to see
+	//        more complex shell syntax anyway.
+	//        <https://www.freedesktop.org/software/systemd/man/os-release.html>
+	for(std::string s; std::getline(*in, s);) {
+		if(s.empty() || s.front() == '#') {
+			continue;
+		}
+
+		auto eqsign_pos = s.find('=');
+		if(!eqsign_pos || eqsign_pos == std::string::npos) {
+			continue;
+		}
+
+		auto lhs = s.substr(0, eqsign_pos),
+			 rhs = eqsign_pos + 1 < s.length() ? utils::unescape(s.substr(eqsign_pos + 1)) : "";
+
+		boost::algorithm::trim(lhs);
+		boost::algorithm::trim(rhs);
+
+		// Unquote if the quotes match on both sides
+		if(rhs.length() >= 2 && rhs.front() == '"' && rhs.back() == '"') {
+			rhs.pop_back();
+			rhs.erase(0, 1);
+		}
+
+		res.emplace(std::move(lhs), std::move(rhs));
+	}
+
+	return res;
+}
+
 #endif
 
 } // end anonymous namespace
 
 std::string os_version()
 {
+#if defined(__APPLE__) || defined(_X11)
+	utsname u;
+
+	if(uname(&u) != 0) {
+		ERR_DU << "os_version: uname error (" << strerror(errno) << ")\n";
+	}
+#endif
+
 #if defined(__APPLE__)
 
 	//
 	// Standard Mac OS X version
 	//
 
-	return desktop::apple::os_version();
+	return desktop::apple::os_version() + " " + u.machine;
 
 #elif defined(_X11)
 
 	//
-	// Linux Standard Base version.
+	// systemd/freedesktop.org method.
+	//
+
+	std::map<std::string, std::string> osrel;
+
+	static const std::string fdo_osrel_etc = "/etc/os-release";
+	static const std::string fdo_osrel_usr = "/usr/lib/os-release";
+
+	if(filesystem::file_exists(fdo_osrel_etc)) {
+		osrel = parse_fdo_osrelease(fdo_osrel_etc);
+	} else if(filesystem::file_exists(fdo_osrel_usr)) {
+		osrel = parse_fdo_osrelease(fdo_osrel_usr);
+	}
+
+	// Check both existence and emptiness in case some vendor sets PRETTY_NAME=""
+	auto osrel_distname = osrel["PRETTY_NAME"];
+	if(osrel_distname.empty()) {
+		osrel_distname = osrel["NAME"];
+	}
+
+	if(!osrel_distname.empty()) {
+		return osrel_distname + " " + u.machine;
+	}
+
+	//
+	// Linux Standard Base fallback.
 	//
 
 	static const std::string lsb_release_bin = "/usr/bin/lsb_release";
@@ -149,7 +278,7 @@ std::string os_version()
 
 		// Check this again in case we got "" above for some weird reason.
 		if(!ver.empty()) {
-			return ver;
+			return ver + " " + u.machine;
 		}
 	}
 
@@ -157,12 +286,6 @@ std::string os_version()
 	// POSIX uname version fallback.
 	//
 
-	utsname u;
-
-	if(uname(&u) != 0) {
-		ERR_DU << "os_version: uname error (" << strerror(errno) << ")\n";
-	}
-
 	return formatter() << u.sysname << ' '
 						<< u.release << ' '
 						<< u.version << ' '
@@ -177,7 +300,10 @@ std::string os_version()
 	static const std::string base
 			= !on_wine() ? "Microsoft Windows" : "Wine/Microsoft Windows";
 
-	OSVERSIONINFOEX v { sizeof(OSVERSIONINFOEX) };
+	OSVERSIONINFOEX v;
+
+	SecureZeroMemory(&v, sizeof(OSVERSIONINFOEX));
+	v.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
 
 #ifdef _MSC_VER
 // GetVersionEx is rather problematic, but it works for our usecase.
@@ -240,9 +366,15 @@ std::string os_version()
 			break;
 		case 1000:
 			if(v.wProductType == VER_NT_WORKSTATION) {
-				version = "10";
+				version = v.dwBuildNumber < 22000 ? "10" : "11";
+				const auto& release_id = windows_release_id();
+				if(!release_id.empty()) {
+					version += ' ';
+					version += release_id;
+				}
 				break;
 			} // else fallback to default
+			[[fallthrough]];
 		default:
 			if(v.wProductType != VER_NT_WORKSTATION) {
 				version = "Server";
@@ -262,7 +394,7 @@ std::string os_version()
 			<< v.dwBuildNumber;
 	version += ")";
 
-	return base + " " + version;
+	return base + " " + version + " " + windows_runtime_arch();
 
 #else
 
