@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2013 - 2018 by Maxim Biro <nurupo.contributions@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2013 - 2021
+	by Maxim Biro <nurupo.contributions@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "desktop/windows_tray_notification.hpp"
@@ -135,9 +136,7 @@ bool windows_tray_notification::create_tray_icon()
 	nid->uCallbackMessage = WM_TRAYNOTIFY;
 	nid->uID = ICON_ID;
 	nid->hIcon = icon;
-#if _WIN32_WINNT >= 0x600
 	nid->hBalloonIcon = icon;
-#endif
 	lstrcpyW(nid->szTip, wtip.c_str());
 
 	// creating icon notification
@@ -196,9 +195,9 @@ void windows_tray_notification::switch_to_wesnoth_window()
 	SetForegroundWindow(window);
 }
 
-std::wstring windows_tray_notification::string_to_wstring(const std::string& string, size_t maxlength)
+std::wstring windows_tray_notification::string_to_wstring(const std::string& string, std::size_t maxlength)
 {
-	utf16::string u16_string = unicode_cast<utf16::string>(string);
+	std::u16string u16_string = unicode_cast<std::u16string>(string);
 	if(u16_string.size() > maxlength) {
 		if((u16_string[maxlength-1] & 0xDC00) == 0xD800)
 			u16_string.resize(maxlength - 1);
