@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "display_chat_manager.hpp"
@@ -20,11 +21,12 @@
 #include "game_board.hpp" // <-- only needed for is_observer()
 #include "preferences/game.hpp"
 #include "log.hpp"
-#include "font/marked-up_text.hpp"
+#include "font/sdl_ttf_compat.hpp"
 #include "mp_ui_alerts.hpp"
 #include "serialization/string_utils.hpp"
 #include "color.hpp"
 #include "preferences/credentials.hpp"
+#include "serialization/utf8_exception.hpp"
 
 #include <SDL2/SDL_timer.h>
 
@@ -43,7 +45,7 @@ display_chat_manager::chat_message::chat_message(int speaker, int h)
 {}
 
 
-void display_chat_manager::add_chat_message(const time_t& time, const std::string& speaker,
+void display_chat_manager::add_chat_message(const std::time_t& time, const std::string& speaker,
 		int side, const std::string& message, events::chat_handler::MESSAGE_TYPE type,
 		bool bell)
 {
@@ -64,7 +66,7 @@ void display_chat_manager::add_chat_message(const time_t& time, const std::strin
 	if (!preferences::parse_should_show_lobby_join(sender, message)) return;
 	if (preferences::is_ignored(sender)) return;
 
-	preferences::parse_admin_authentication(sender, message);
+	//preferences::parse_admin_authentication(sender, message); TODO: replace
 
 	bool is_observer = false;
 	{ //TODO: Clean this block up somehow
@@ -103,7 +105,7 @@ void display_chat_manager::add_chat_message(const time_t& time, const std::strin
 	try {
 		// We've had a joker who send an invalid utf-8 message to crash clients
 		// so now catch the exception and ignore the message.
-		msg = my_disp_.video().faked() ? "" : font::word_wrap_text(msg,font::SIZE_15,my_disp_.map_outside_area().w*3/4);
+		msg = my_disp_.video().faked() ? "" : font::pango_word_wrap(msg,font::SIZE_15,my_disp_.map_outside_area().w*3/4);
 	} catch (utf8::invalid_utf8_exception&) {
 		ERR_NG << "Invalid utf-8 found, chat message is ignored." << std::endl;
 		return;
@@ -180,6 +182,7 @@ void display_chat_manager::add_chat_message(const time_t& time, const std::strin
 
 	prune_chat_messages();
 }
+
 static unsigned int safe_subtract(unsigned int a, unsigned int b)
 {
 	return (a > b) ? a - b : 0;
