@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -21,6 +22,8 @@
 
 #pragma once
 
+#include "units/orb_status.hpp"
+#include "units/ptr.hpp"
 #include <string>
 #include <vector>
 
@@ -60,16 +63,46 @@ public:
 	// Needed for reports
 
 	const unit * get_visible_unit(const map_location &loc, const team &current_team, bool see_all = false) const;
+	unit_const_ptr get_visible_unit_shared_ptr(const map_location &loc, const team &current_team, bool see_all = false) const;
 
-	// From actions:: namespace
+	struct can_move_result
+	{
+		/**
+		 * The unit can move to another hex, taking account of enemies' locations, ZoC and
+		 * terrain costs vs current movement points.
+		 */
+		bool move = false;
 
-	bool unit_can_move(const unit & u) const;
+		/**
+		 * The unit can make an attack from the hex that it's currently on, this
+		 * requires attack points and a non-petrified enemy in an adjacent hex.
+		 */
+		bool attack_here = false;
 
-	// From class team
+		operator bool() const
+		{
+			return move || attack_here;
+		}
+	};
 
 	/**
-	 * Given the location of a village, will return the 0-based index
-	 * of the team that currently owns it, and -1 if it is unowned.
+	 * Work out what @a u can do - this does not check which player's turn is currently active, the
+	 * result is calculated assuming that the unit's owner is currently active.
+	 */
+	can_move_result unit_can_move(const unit& u) const;
+
+	/**
+	 * Returns an enumurated summary of whether this unit can move and/or attack.
+	 *
+	 * This does not check which player's turn is currently active, the result is calculated
+	 * assuming that the unit's owner is currently active. For this reason this never returns
+	 * orb_status::enemy nor orb_status::allied.
+	 */
+	orb_status unit_orb_status(const unit& u) const;
+
+	/**
+	 * Given the location of a village, will return the 1-based number
+	 * of the team that currently owns it, and 0 if it is unowned.
 	 */
 	int village_owner(const map_location & loc) const;
 
@@ -85,11 +118,10 @@ public:
 
 	// Accessor from team.cpp
 
-	/// Check if we are an observer in this game
+	/** Check if we are an observer in this game */
 	bool is_observer() const;
 
 	// Dtor
-
 	virtual ~display_context() {}
 };
 
