@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Tomasz Sniatowski <kailoran@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Tomasz Sniatowski <kailoran@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -60,11 +61,11 @@ std::string editor_action::get_description() const
 	return "Unknown action";
 }
 
-editor_action* editor_action::perform(map_context& mc) const
+std::unique_ptr<editor_action> editor_action::perform(map_context& mc) const
 {
-	editor_action_ptr undo(new editor_action_whole_map(mc.map()));
+	auto undo = std::make_unique<editor_action_whole_map>(mc.map());
 	perform_without_undo(mc);
-	return undo.release();
+	return undo;
 }
 
 IMPLEMENT_ACTION(whole_map)
@@ -78,7 +79,7 @@ editor_action_chain::editor_action_chain(const editor::editor_action_chain& othe
 	: editor_action()
 	, actions_()
 {
-	for(editor_action* a : other.actions_) {
+	for(const auto& a : other.actions_) {
 		actions_.push_back(a->clone());
 	}
 }
@@ -89,31 +90,21 @@ editor_action_chain& editor_action_chain::operator=(const editor_action_chain& o
 		return *this;
 	}
 
-	for(editor_action* a : actions_) {
-		delete a;
-	}
-
 	actions_.clear();
 
-	for(editor_action* a : other.actions_) {
+	for(const auto& a : other.actions_) {
 		actions_.push_back(a->clone());
 	}
 
 	return *this;
 }
-editor_action_chain::~editor_action_chain()
-{
-	for(editor_action* a : actions_) {
-		delete a;
-	}
-}
 
 IMPLEMENT_ACTION(chain)
 
 int editor_action_chain::action_count() const
 {
 	int count = 0;
-	for(const editor_action* a : actions_) {
+	for(const auto& a : actions_) {
 		if(a) {
 			count += a->action_count();
 		}
@@ -122,14 +113,14 @@ int editor_action_chain::action_count() const
 	return count;
 }
 
-void editor_action_chain::append_action(editor_action* a)
+void editor_action_chain::append_action(std::unique_ptr<editor_action> a)
 {
-	actions_.push_back(a);
+	actions_.push_back(std::move(a));
 }
 
-void editor_action_chain::prepend_action(editor_action* a)
+void editor_action_chain::prepend_action(std::unique_ptr<editor_action> a)
 {
-	actions_.push_front(a);
+	actions_.push_front(std::move(a));
 }
 
 bool editor_action_chain::empty() const
@@ -137,43 +128,43 @@ bool editor_action_chain::empty() const
 	return actions_.empty();
 }
 
-editor_action* editor_action_chain::pop_last_action()
+std::unique_ptr<editor_action> editor_action_chain::pop_last_action()
 {
 	if(empty()) {
 		throw editor_action_exception("pop_last_action requested on an empty action_chain");
 	}
 
-	editor_action* last = actions_.back();
+	auto last = std::move(actions_.back());
 	actions_.pop_back();
 	return last;
 }
 
-editor_action* editor_action_chain::pop_first_action()
+std::unique_ptr<editor_action> editor_action_chain::pop_first_action()
 {
 	if(empty()) {
 		throw editor_action_exception("pop_first_action requested on an empty action_chain");
 	}
 
-	editor_action* last = actions_.front();
+	auto first = std::move(actions_.front());
 	actions_.pop_front();
-	return last;
+	return first;
 }
 
-editor_action_chain* editor_action_chain::perform(map_context& mc) const
+std::unique_ptr<editor_action> editor_action_chain::perform(map_context& mc) const
 {
-	std::unique_ptr<editor_action_chain> undo(new editor_action_chain());
-	for(editor_action* a : actions_) {
+	auto undo = std::make_unique<editor_action_chain>();
+	for(auto& a : actions_) {
 		if(a != nullptr) {
 			undo->append_action(a->perform(mc));
 		}
 	}
 
 	std::reverse(undo->actions_.begin(), undo->actions_.end());
-	return undo.release();
+	return undo;
 }
 void editor_action_chain::perform_without_undo(map_context& mc) const
 {
-	for(editor_action* a : actions_) {
+	for(const auto& a : actions_) {
 		if(a != nullptr) {
 			a->perform_without_undo(mc);
 		}
@@ -192,13 +183,13 @@ void editor_action_paste::extend(const editor_map& map, const std::set<map_locat
 	paste_.add_tiles(map, locs);
 }
 
-editor_action_paste* editor_action_paste::perform(map_context& mc) const
+std::unique_ptr<editor_action> editor_action_paste::perform(map_context& mc) const
 {
 	map_fragment mf(mc.map(), paste_.get_offset_area(offset_));
-	std::unique_ptr<editor_action_paste> undo(new editor_action_paste(mf));
+	auto undo = std::make_unique<editor_action_paste>(mf);
 
 	perform_without_undo(mc);
-	return undo.release();
+	return undo;
 }
 
 void editor_action_paste::perform_without_undo(map_context& mc) const
@@ -210,13 +201,13 @@ void editor_action_paste::perform_without_undo(map_context& mc) const
 
 IMPLEMENT_ACTION(paint_area)
 
-editor_action_paste* editor_action_paint_area::perform(map_context& mc) const
+std::unique_ptr<editor_action> editor_action_paint_area::perform(map_context& mc) const
 {
 	map_fragment mf(mc.map(), area_);
-	std::unique_ptr<editor_action_paste> undo(new editor_action_paste(mf));
+	auto undo = std::make_unique<editor_action_paste>(mf);
 
 	perform_without_undo(mc);
-	return undo.release();
+	return undo;
 }
 
 void editor_action_paint_area::perform_without_undo(map_context& mc) const
@@ -227,16 +218,15 @@ void editor_action_paint_area::perform_without_undo(map_context& mc) const
 
 IMPLEMENT_ACTION(fill)
 
-editor_action_paint_area* editor_action_fill::perform(map_context& mc) const
+std::unique_ptr<editor_action> editor_action_fill::perform(map_context& mc) const
 {
 	std::set<map_location> to_fill = mc.map().get_contiguous_terrain_tiles(loc_);
-	std::unique_ptr<editor_action_paint_area> undo(
-			new editor_action_paint_area(to_fill, mc.map().get_terrain(loc_)));
+	auto undo = std::make_unique<editor_action_paint_area>(to_fill, mc.map().get_terrain(loc_));
 
 	mc.draw_terrain(t_, to_fill, one_layer_);
 	mc.set_needs_terrain_rebuild();
 
-	return undo.release();
+	return undo;
 }
 
 void editor_action_fill::perform_without_undo(map_context& mc) const
@@ -248,28 +238,28 @@ void editor_action_fill::perform_without_undo(map_context& mc) const
 
 IMPLEMENT_ACTION(starting_position)
 
-editor_action* editor_action_starting_position::perform(map_context& mc) const
+std::unique_ptr<editor_action> editor_action_starting_position::perform(map_context& mc) const
 {
-	editor_action_ptr undo;
+	std::unique_ptr<editor_action> undo;
 
-	const std::string* old_loc_id = mc.map().is_starting_position(loc_);
+	const std::string* old_loc_id = mc.map().is_special_location(loc_);
 	map_location old_loc = mc.map().special_location(loc_id_);
 
 	if(old_loc_id != nullptr) {
 		// If another player was starting at the location, we actually perform two actions, so the undo is an
 		// action_chain.
-		editor_action_chain* undo_chain = new editor_action_chain();
+		auto undo_chain = std::make_unique<editor_action_chain>();
 
-		undo_chain->append_action(new editor_action_starting_position(loc_, *old_loc_id));
-		undo_chain->append_action(new editor_action_starting_position(old_loc, loc_id_));
+		undo_chain->append_action(std::make_unique<editor_action_starting_position>(loc_, *old_loc_id));
+		undo_chain->append_action(std::make_unique<editor_action_starting_position>(old_loc, loc_id_));
 
-		undo.reset(undo_chain);
+		undo = std::move(undo_chain);
 
 		LOG_ED << "ssp actual: " << *old_loc_id << " to " << map_location() << "\n";
 
 		mc.map().set_special_location(*old_loc_id, map_location());
 	} else {
-		undo.reset(new editor_action_starting_position(old_loc, loc_id_));
+		undo = std::make_unique<editor_action_starting_position>(old_loc, loc_id_);
 	}
 
 	LOG_ED << "ssp actual: " << loc_id_ << " to " << loc_ << "\n";
@@ -277,12 +267,12 @@ editor_action* editor_action_starting_position::perform(map_context& mc) const
 	mc.map().set_special_location(loc_id_, loc_);
 	mc.set_needs_labels_reset();
 
-	return undo.release();
+	return undo;
 }
 
 void editor_action_starting_position::perform_without_undo(map_context& mc) const
 {
-	const std::string* old_id = mc.map().is_starting_position(loc_);
+	const std::string* old_id = mc.map().is_special_location(loc_);
 	if(old_id != nullptr) {
 		mc.map().set_special_location(*old_id, map_location());
 	}
@@ -303,7 +293,7 @@ IMPLEMENT_ACTION(apply_mask)
 
 void editor_action_apply_mask::perform_without_undo(map_context& mc) const
 {
-	mc.map().overlay(mask_, config(), {0, 0});
+	mc.map().overlay(mask_, {0, 0, wml_loc()});
 	mc.set_needs_terrain_rebuild();
 }
 
@@ -317,13 +307,12 @@ void editor_action_create_mask::perform_without_undo(map_context& mc) const
 
 IMPLEMENT_ACTION(shuffle_area)
 
-editor_action_paste* editor_action_shuffle_area::perform(map_context& mc) const
+std::unique_ptr<editor_action> editor_action_shuffle_area::perform(map_context& mc) const
 {
 	map_fragment mf(mc.map(), area_);
-	std::unique_ptr<editor_action_paste> undo(new editor_action_paste(mf));
-
+	auto undo = std::make_unique<editor_action_paste>(mf);
 	perform_without_undo(mc);
-	return undo.release();
+	return undo;
 }
 
 void editor_action_shuffle_area::perform_without_undo(map_context& mc) const
