@@ -1,16 +1,18 @@
 /*
-   Copyright (C) 2008 - 2018 by Fabian Mueller <fabianmueller5@gmx.de>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Fabian Mueller <fabianmueller5@gmx.de>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
+
 #define GETTEXT_DOMAIN "wesnoth-editor"
 
 #include "editor/action/mouse/mouse_action_map_label.hpp"
@@ -27,41 +29,23 @@
 
 namespace editor {
 
-editor_action* mouse_action_map_label::click_left(editor_display& disp, int x, int y)
+std::unique_ptr<editor_action> mouse_action_map_label::click_left(editor_display& disp, int x, int y)
 {
 	click_ = true;
 	map_location hex = disp.hex_clicked_on(x, y);
 	clicked_on_ = hex;
-	last_draged_ = hex;
 	return nullptr;
 }
 
-editor_action* mouse_action_map_label::drag_left(editor_display& disp, int x, int y
-		, bool& partial, editor_action* /*last_undo*/)
+std::unique_ptr<editor_action> mouse_action_map_label::drag_left(editor_display& disp, int x, int y
+		, bool& /*partial*/, editor_action* /*last_undo*/)
 {
 	map_location hex = disp.hex_clicked_on(x, y);
-
-	/* Cursor is still on old hex field */
-	if (hex == last_draged_)
-		return nullptr;
-	click_ = false;
-
-	editor_action_chain* chain = nullptr;
-	const terrain_label* label = disp.get_controller().get_current_map_context().get_labels().get_label(last_draged_);
-
-
-	if (label) {
-		partial = true;
-		chain = new editor_action_chain(new editor_action_label_delete(last_draged_));
-		chain->append_action(new editor_action_label(hex, label->text(), label->team_name(), label->color(),
-			label->visible_in_fog(), label->visible_in_shroud(), label->immutable(), label->category()));
-	}
-
-	last_draged_ = hex;
-	return chain;
+	click_ = (hex == clicked_on_);
+	return nullptr;
 }
 
-editor_action* mouse_action_map_label::up_left(editor_display& disp, int x, int y)
+std::unique_ptr<editor_action> mouse_action_map_label::up_left(editor_display& disp, int x, int y)
 {
 	if (!click_) return nullptr;
 	click_ = false;
@@ -82,21 +66,21 @@ editor_action* mouse_action_map_label::up_left(editor_display& disp, int x, int
 
 	gui2::dialogs::editor_edit_label d(label, immutable, visible_fog, visible_shroud, color, category);
 
-	editor_action* a = nullptr;
+	std::unique_ptr<editor_action> a;
 	if(d.show()) {
-		a = new editor_action_label(hex, label, team_name, color
+		a = std::make_unique<editor_action_label>(hex, label, team_name, color
 				, visible_fog, visible_shroud, immutable, category);
 		update_brush_highlights(disp, hex);
 	}
 	return a;
 }
 
-editor_action* mouse_action_map_label::click_right(editor_display& /*disp*/, int /*x*/, int /*y*/)
+std::unique_ptr<editor_action> mouse_action_map_label::click_right(editor_display& /*disp*/, int /*x*/, int /*y*/)
 {
 	return nullptr;
 }
 
-editor_action* mouse_action_map_label::up_right(editor_display& disp, int x, int y)
+std::unique_ptr<editor_action> mouse_action_map_label::up_right(editor_display& disp, int x, int y)
 {
 	map_location hex = disp.hex_clicked_on(x, y);
 
@@ -105,7 +89,49 @@ editor_action* mouse_action_map_label::up_right(editor_display& disp, int x, int
 	//if (!clicked_label)
 	//	return nullptr;
 
-	return new editor_action_label_delete(hex);
+	return std::make_unique<editor_action_label_delete>(hex);
+}
+
+std::unique_ptr<editor_action> mouse_action_map_label::drag_end_left(editor_display& disp, int x, int y)
+{
+	if (click_) return nullptr;
+
+	map_location hex = disp.hex_clicked_on(x, y);
+	if (!disp.get_map().on_board(hex))
+		return nullptr;
+
+	const terrain_label* dragged_label = disp
+		.get_controller()
+		.get_current_map_context()
+		.get_labels()
+		.get_label(clicked_on_);
+
+	// Return nullptr if the dragged label doesn't exist
+	if (!dragged_label) {
+		return nullptr;
+	}
+
+	// Create action chain to:
+	// 1. Delete label in initial hex
+	// 2. Delete label in target hex if it exists, otherwise will no-op
+	// 3. Create label in target hex
+	auto chain = std::make_unique<editor_action_chain>();
+	chain->append_action(std::make_unique<editor_action_label_delete>(clicked_on_));
+	chain->append_action(std::make_unique<editor_action_label_delete>(hex));
+	chain->append_action(
+		std::make_unique<editor_action_label>(
+			hex,
+			dragged_label->text(),
+			dragged_label->team_name(),
+			dragged_label->color(),
+			dragged_label->visible_in_fog(),
+			dragged_label->visible_in_shroud(),
+			dragged_label->immutable(),
+			dragged_label->category()
+		)
+	);
+
+	return chain;
 }
 
 void mouse_action_map_label::set_mouse_overlay(editor_display& disp)
