@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Tomasz Sniatowski <kailoran@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Tomasz Sniatowski <kailoran@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -188,13 +189,13 @@ class editor_controller : public controller_base,
 		 * Perform an action, then delete the action object.
 		 * The pointer can be nullptr, in which case nothing will happen.
 		 */
-		void perform_delete(editor_action* action);
+		void perform_delete(std::unique_ptr<editor_action> action);
 
 		/**
 		 * Peform an action on the current map_context, then refresh the display
 		 * and delete the pointer. The pointer can be nullptr, in which case nothing will happen.
 		 */
-		void perform_refresh_delete(editor_action* action, bool drag_part = false);
+		void perform_refresh_delete(std::unique_ptr<editor_action> action, bool drag_part = false);
 
 
 		virtual std::vector<std::string> additional_actions_pressed() override;
@@ -205,10 +206,10 @@ class editor_controller : public controller_base,
 		void init_gui();
 
 		/** init the available time-of-day settings */
-		void init_tods(const config& game_config);
+		void init_tods(const game_config_view& game_config);
 
 		/** init background music for the editor */
-		void init_music(const config& game_config);
+		void init_music(const game_config_view& game_config);
 
 		/** Load editor-specific tooltips */
 		void load_tooltips();
@@ -241,7 +242,7 @@ class editor_controller : public controller_base,
 		const std::unique_ptr<editor_display> gui_;
 
 		/** Pre-defined time of day lighting settings for the settings dialog */
-		typedef std::map<std::string, std::pair<std::string ,std::vector<time_of_day>> > tods_map;
+		typedef std::map<std::string, std::pair<std::string ,std::vector<time_of_day>>> tods_map;
 		tods_map tods_;
 
 		/* managers */
