@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -20,6 +21,7 @@
 #include "preferences/editor.hpp"
 
 class map_generator;
+class game_config_view;
 
 namespace editor
 {
@@ -29,12 +31,12 @@ class context_manager : public filter_context
 public:
 	using context_ptr = std::unique_ptr<map_context>;
 
-	context_manager(editor_display& gui, const config& game_config);
+	context_manager(editor_display& gui, const game_config_view& game_config);
 	~context_manager();
 
 	bool is_active_transitions_hotkey(const std::string& item);
 
-	size_t modified_maps(std::string& modified);
+	std::size_t modified_maps(std::string& modified);
 
 	void set_update_transitions_mode(int mode)
 	{
@@ -63,7 +65,7 @@ public:
 		return current_context_index_;
 	}
 
-	size_t open_maps(void)
+	std::size_t open_maps(void)
 	{
 		return map_contexts_.size();
 	}
@@ -150,7 +152,7 @@ public:
 	/** Display a load map dialog and process user input. */
 	void resize_map_dialog();
 
-	size_t size()
+	std::size_t size()
 	{
 		return map_contexts_.size();
 	}
@@ -195,7 +197,7 @@ public:
 	class location_palette* locs_;
 private:
 	/** init available random map generators */
-	void init_map_generators(const config& game_config);
+	void init_map_generators(const game_config_view& game_config);
 
 	/**
 	 * Shows an are-you-sure dialog if the map was modified.
@@ -279,7 +281,7 @@ private:
 	 * @return index of the map context containing the given filename,
 	 *         or map_contexts_.size() if not found.
 	 */
-	size_t check_open_map(const std::string& fn) const;
+	std::size_t check_open_map(const std::string& fn) const;
 
 	/**
 	 * Check if a map is already open. If yes, switch to it
@@ -312,7 +314,7 @@ public:
 private:
 	editor_display& gui_;
 
-	const config& game_config_;
+	const game_config_view& game_config_;
 
 	/** Default directory for map load/save as dialogs */
 	std::string default_dir_;
