@@ -1,16 +1,18 @@
 /*
-   Copyright (C) 2008 - 2018 by Tomasz Sniatowski <kailoran@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Tomasz Sniatowski <kailoran@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
+
 #define GETTEXT_DOMAIN "wesnoth-editor"
 
 #include "editor/map/map_context.hpp"
@@ -27,8 +29,8 @@
 #include "serialization/binary_or_text.hpp"
 #include "serialization/parser.hpp"
 #include "team.hpp"
-#include "terrain/type_data.hpp"
 #include "units/unit.hpp"
+#include "game_config_view.hpp"
 
 #include <boost/regex.hpp>
 
@@ -51,7 +53,7 @@ editor_team_info::editor_team_info(const team& t)
 {
 }
 
-const size_t map_context::max_action_stack_size_ = 100;
+const std::size_t map_context::max_action_stack_size_ = 100;
 
 map_context::map_context(const editor_map& map, bool pure_map, const config& schedule)
 	: filename_()
@@ -71,7 +73,7 @@ map_context::map_context(const editor_map& map, bool pure_map, const config& sch
 	, scenario_id_()
 	, scenario_name_()
 	, scenario_description_()
-	, xp_mod_(70)
+	, xp_mod_()
 	, victory_defeated_(true)
 	, random_time_(false)
 	, active_area_(-1)
@@ -85,12 +87,12 @@ map_context::map_context(const editor_map& map, bool pure_map, const config& sch
 {
 }
 
-map_context::map_context(const config& game_config, const std::string& filename)
+map_context::map_context(const game_config_view& game_config, const std::string& filename)
 	: filename_(filename)
 	, map_data_key_()
 	, embedded_(false)
 	, pure_map_(false)
-	, map_(game_config)
+	, map_()
 	, undo_stack_()
 	, redo_stack_()
 	, actions_since_save_(0)
@@ -103,14 +105,14 @@ map_context::map_context(const config& game_config, const std::string& filename)
 	, scenario_id_()
 	, scenario_name_()
 	, scenario_description_()
-	, xp_mod_(70)
+	, xp_mod_()
 	, victory_defeated_(true)
 	, random_time_(false)
 	, active_area_(-1)
 	, labels_(nullptr)
 	, units_()
 	, teams_()
-	, tod_manager_(new tod_manager(game_config.find_child("editor_times", "id", "default")))
+	, tod_manager_(new tod_manager(game_config.find_child("editor_times", "id", "empty")))
 	, mp_settings_()
 	, game_classification_()
 	, music_tracks_()
@@ -160,7 +162,7 @@ map_context::map_context(const config& game_config, const std::string& filename)
 	if(!boost::regex_search(
 		file_string, matched_map_data, rexpression_map_data, boost::regex_constants::match_not_dot_null)
 	) {
-		map_ = editor_map::from_string(game_config, file_string); // throws on error
+		map_ = editor_map::from_string(file_string); // throws on error
 		pure_map_ = true;
 
 		add_to_recent_files();
@@ -181,7 +183,7 @@ map_context::map_context(const config& game_config, const std::string& filename)
 			LOG_ED << "Loading generated scenario file" << std::endl;
 			// 4.0 editor generated scenario
 			try {
-				load_scenario(game_config);
+				load_scenario();
 			} catch(const config::error& e) {
 				// We already caught and rethrew this exception in load_scenario
 				throw editor_map_load_exception("load_scenario", e.message);
@@ -190,7 +192,7 @@ map_context::map_context(const config& game_config, const std::string& filename)
 			LOG_ED << "Loading embedded map file" << std::endl;
 			embedded_ = true;
 			pure_map_ = true;
-			map_ = editor_map::from_string(game_config, map_data);
+			map_ = editor_map::from_string(map_data);
 		}
 
 		add_to_recent_files();
@@ -215,7 +217,7 @@ map_context::map_context(const config& game_config, const std::string& filename)
 
 	filename_ = new_filename;
 	file_string = filesystem::read_file(filename_);
-	map_ = editor_map::from_string(game_config, file_string);
+	map_ = editor_map::from_string(file_string);
 	pure_map_ = true;
 
 	add_to_recent_files();
@@ -312,7 +314,7 @@ void map_context::replace_local_schedule(const std::vector<time_of_day>& schedul
 	}
 }
 
-void map_context::load_scenario(const config& game_config)
+void map_context::load_scenario()
 {
 	config scenario;
 
@@ -327,12 +329,13 @@ void map_context::load_scenario(const config& game_config)
 	scenario_name_ = scenario["name"].str();
 	scenario_description_ = scenario["description"].str();
 
-	xp_mod_ = scenario["experience_modifier"].to_int();
-
+	if(const config::attribute_value* experience_modifier = scenario.get("experience_modifier")) {
+		xp_mod_ = experience_modifier->to_int();
+	}
 	victory_defeated_ = scenario["victory_when_enemies_defeated"].to_bool(true);
 	random_time_ = scenario["random_start_time"].to_bool(false);
 
-	map_ = editor_map::from_string(game_config, scenario["map_data"]); // throws on error
+	map_ = editor_map::from_string(scenario["map_data"]); // throws on error
 
 	labels_.read(scenario);
 
@@ -343,7 +346,7 @@ void map_context::load_scenario(const config& game_config)
 
 	for(const config& item : scenario.child_range("item")) {
 		const map_location loc(item);
-		overlays_.emplace(loc, overlay(item));
+		overlays_[loc].push_back(overlay(item));
 	}
 
 	for(const config& music : scenario.child_range("music")) {
@@ -475,8 +478,12 @@ config map_context::to_config()
 	scenario["name"] = t_string(scenario_name_);
 	scenario["description"] = scenario_description_;
 
-	scenario["experience_modifier"] = xp_mod_;
-	scenario["victory_when_enemies_defeated"] = victory_defeated_;
+	if(xp_mod_) {
+		scenario["experience_modifier"] = *xp_mod_;
+	}
+	if(victory_defeated_) {
+		scenario["victory_when_enemies_defeated"] = *victory_defeated_;
+	}
 	scenario["random_start_time"] = random_time_;
 
 	scenario.append(tod_manager_->to_config());
@@ -491,22 +498,22 @@ config map_context::to_config()
 	labels_.write(scenario);
 
 	for(const auto& overlay_pair : overlays_) {
-		config& item = scenario.add_child("item");
+		for(const overlay& o : overlay_pair.second) {
+			config& item = scenario.add_child("item");
 
-		// Write x,y location
-		overlay_pair.first.write(item);
+			// Write x,y location
+			overlay_pair.first.write(item);
 
-		const overlay& o = overlay_pair.second;
+			// These should always have a value
+			item["image"] = o.image;
+			item["visible_in_fog"] = o.visible_in_fog;
 
-		// These should always have a value
-		item["image"] = o.image;
-		item["visible_in_fog"] = o.visible_in_fog;
-
-		// Optional keys
-		item["id"].write_if_not_empty(o.id);
-		item["name"].write_if_not_empty(o.name);
-		item["team_name"].write_if_not_empty(o.team_name);
-		item["halo"].write_if_not_empty(o.halo);
+			// Optional keys
+			item["id"].write_if_not_empty(o.id);
+			item["name"].write_if_not_empty(o.name);
+			item["team_name"].write_if_not_empty(o.team_name);
+			item["halo"].write_if_not_empty(o.halo);
+		}
 	}
 
 	for(const music_map::value_type& track : music_tracks_) {
@@ -679,7 +686,7 @@ void map_context::perform_action(const editor_action& action)
 {
 	LOG_ED << "Performing action " << action.get_id() << ": " << action.get_name() << ", actions count is "
 		   << action.get_instance_count() << std::endl;
-	editor_action* undo = action.perform(*this);
+	auto undo = action.perform(*this);
 	if(actions_since_save_ < 0) {
 		// set to a value that will make it impossible to get to zero, as at this point
 		// it is no longer possible to get back the original map state using undo/redo
@@ -688,7 +695,7 @@ void map_context::perform_action(const editor_action& action)
 
 	++actions_since_save_;
 
-	undo_stack_.emplace_back(undo);
+	undo_stack_.emplace_back(std::move(undo));
 
 	trim_stack(undo_stack_);
 
@@ -708,10 +715,10 @@ void map_context::perform_partial_action(const editor_action& action)
 		throw editor_logic_exception("Last undo action not a chain in perform_partial_action()");
 	}
 
-	editor_action* undo = action.perform(*this);
+	auto undo = action.perform(*this);
 
 	// actions_since_save_ += action.action_count();
-	undo_chain->prepend_action(undo);
+	undo_chain->prepend_action(std::move(undo));
 
 	redo_stack_.clear();
 }
@@ -803,7 +810,7 @@ void map_context::partial_undo()
 
 	// a partial undo performs the first action form the current action's action_chain that would be normally performed
 	// i.e. the *first* one.
-	const editor_action_ptr first_action_in_chain(undo_chain->pop_first_action());
+	const auto first_action_in_chain = undo_chain->pop_first_action();
 	if(undo_chain->empty()) {
 		actions_since_save_--;
 		undo_stack_.pop_back();
@@ -830,13 +837,13 @@ void map_context::perform_action_between_stacks(action_stack& from, action_stack
 {
 	assert(!from.empty());
 
-	editor_action_ptr action(nullptr);
+	std::unique_ptr<editor_action> action;
 	action.swap(from.back());
 
 	from.pop_back();
 
-	editor_action* reverse_action = action->perform(*this);
-	to.emplace_back(reverse_action);
+	auto reverse_action = action->perform(*this);
+	to.emplace_back(std::move(reverse_action));
 
 	trim_stack(to);
 }
