@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2012 - 2018 by Fabian Mueller <fabianmueller5@gmx.de>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2012 - 2021
+	by Fabian Mueller <fabianmueller5@gmx.de>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -57,7 +58,7 @@ public:
 	std::vector<gui::widget>* get_widgets() { return nullptr; }
 
 	//group
-	virtual void set_group(size_t /*index*/) override {}
+	virtual void set_group(std::size_t /*index*/) override {}
 	virtual void next_group() override {}
 	virtual void prev_group() override {}
 	virtual const std::vector<item_group>& get_groups() const override { static const std::vector<item_group> empty; return empty; }
@@ -70,8 +71,8 @@ public:
 
     //item
 	virtual int num_items() override {return 0;}
-	virtual size_t start_num() override {return 0;}
-	virtual void set_start_item(size_t /*index*/) override {}
+	virtual std::size_t start_num() override {return 0;}
+	virtual void set_start_item(std::size_t /*index*/) override {}
 	virtual bool supports_swap() override { return false; }
 	virtual void swap() override {}
 
