@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2012 - 2018 by Fabian Mueller <fabianmueller5@gmx.de>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2012 - 2021
+	by Fabian Mueller <fabianmueller5@gmx.de>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -20,6 +21,7 @@
 
 #include "editor/palette/item_palette.hpp"
 #include "gettext.hpp"
+#include "game_config_view.hpp"
 
 #include <string>
 
@@ -30,7 +32,7 @@ std::string item_palette::get_help_string()
 	return selected_fg_item().name;
 }
 
-void item_palette::setup(const config& cfg)
+void item_palette::setup(const game_config_view& cfg)
 {
 	for(const config& group : cfg.child_range("item_group")) {
 		groups_.emplace_back(group);
@@ -81,7 +83,7 @@ void item_palette::draw_item(const overlay& item, surface& image, std::stringstr
 	tooltip_text << item.name;
 }
 
-item_palette::item_palette(editor_display &gui, const config& cfg,
+item_palette::item_palette(editor_display &gui, const game_config_view& cfg,
                            editor_toolkit &toolkit)
 //TODO avoid magic numbers
 	:	editor_palette<overlay>(gui, cfg, 36, 4, toolkit)
