@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2012 - 2018 by Fabian Mueller <fabianmueller5@gmx.de>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2012 - 2021
+	by Fabian Mueller <fabianmueller5@gmx.de>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -20,6 +21,7 @@
 
 #include "editor/palette/editor_palettes.hpp"
 #include "overlay.hpp"
+class game_config_view;
 
 namespace editor {
 
@@ -32,10 +34,10 @@ class item_palette : public editor_palette<overlay> {
 public:
 
 	item_palette(editor_display &gui,
-	             const config& cfg,
+	             const game_config_view& cfg,
 	             editor_toolkit &toolkit);
 
-	virtual void setup(const config& cfg);
+	virtual void setup(const game_config_view& cfg);
 
 	virtual std::string get_help_string();
 
