@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-editor"
@@ -17,7 +18,7 @@
 #include "editor/palette/location_palette.hpp"
 
 #include "gettext.hpp"
-#include "font/marked-up_text.hpp"
+#include "font/sdl_ttf_compat.hpp"
 #include "font/standard_colors.hpp"
 #include "tooltips.hpp"
 
@@ -50,7 +51,7 @@ public:
 		}
 
 	};
-	location_palette_item(CVideo& video, editor::location_palette& parent)
+	location_palette_item(CVideo& video, editor::location_palette* parent)
 		: gui::widget(video, true)
 		, parent_(parent)
 	{
@@ -64,7 +65,7 @@ public:
 		if (state_.selected) {
 			sdl::draw_rectangle(location(), {255, 255, 255, 255});
 		}
-		font::draw_text(&video(), location(), 16, font::NORMAL_COLOR, desc_.empty() ? id_ : desc_, location().x + 2, location().y, 0);
+		font::pango_draw_text(&video(), location(), 16, font::NORMAL_COLOR, desc_.empty() ? id_ : desc_, location().x + 2, location().y, 0);
 	}
 
 	//TODO move to widget
@@ -78,7 +79,7 @@ public:
 		if (!(hit(e.x, e.y)))
 			return;
 		if (e.button == SDL_BUTTON_LEFT) {
-			parent_.select_item(id_);
+			parent_->select_item(id_);
 		}
 		if (e.button == SDL_BUTTON_RIGHT) {
 			//TODO: add a context menu with the following options:
@@ -131,7 +132,7 @@ private:
 	std::string id_;
 	std::string desc_;
 	state_t state_;
-	editor::location_palette& parent_;
+	editor::location_palette* parent_;
 };
 
 class location_palette_button : public gui::button
@@ -158,7 +159,7 @@ protected:
 
 };
 namespace editor {
-location_palette::location_palette(editor_display &gui, const config& /*cfg*/,
+location_palette::location_palette(editor_display &gui, const game_config_view& /*cfg*/,
                                    editor_toolkit &toolkit)
 		: common_palette(gui.video())
 		, item_size_(20)
@@ -291,10 +292,8 @@ void location_palette::adjust_size(const SDL_Rect& target)
 	// If that happens, no items will fit and we'll have a negative number here.
 	// Just skip it in that case.
 	if(items_fitting > 0 && num_visible_items() != items_fitting) {
-		location_palette_item lpi(disp_.video(), *this);
-		//Why does this need a pointer to a non-const as second paraeter?
-		//TODO: we should write our own ptr_vector class, boost::ptr_vector has a lot of flaws.
-		buttons_.resize(items_fitting, &lpi);
+		location_palette_item lpi(disp_.video(), this);
+		buttons_.resize(items_fitting, lpi);
 	}
 
 	set_location(target);
@@ -419,8 +418,7 @@ void location_palette::add_item(const std::string& id)
 	const auto itor = std::upper_bound(items_.begin(), items_.end(), id, loc_id_comp);
 	if(itor == items_.begin() || *(itor - 1) != id) {
 		pos = std::distance(items_.begin(), items_.insert(itor, id));
-	}
-	else {
+	} else {
 		pos = std::distance(items_.begin(), itor);
 	}
 	selected_item_ = id;
