@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -18,11 +19,9 @@
 #include "editor/palette/common_palette.hpp"
 #include "editor/palette/tristate_button.hpp"
 
-#include <boost/ptr_container/ptr_vector.hpp>
-
 class location_palette_item;
 class location_palette_button;
-
+class game_config_view;
 namespace editor {
 
 class editor_toolkit;
@@ -31,15 +30,15 @@ class location_palette : public common_palette {
 
 public:
 
-	location_palette(editor_display &gui, const config& /*cfg*/,
+	location_palette(editor_display &gui, const game_config_view& /*cfg*/,
 	                 editor_toolkit &toolkit);
 
 
 	virtual sdl_handler_vector handler_members() override;
 
-	void set_start_item(size_t index) override { items_start_ = index; }
+	void set_start_item(std::size_t index) override { items_start_ = index; }
 
-	size_t start_num(void) override { return items_start_; }
+	std::size_t start_num(void) override { return items_start_; }
 
 	/** Menu expanding for palette group list */
 	void expand_palette_groups_menu(std::vector<config>& items, int i) override
@@ -47,7 +46,7 @@ public:
 		items.erase(items.begin() + i);
 	}
 
-	virtual void set_group(size_t /*index*/) override {}
+	virtual void set_group(std::size_t /*index*/) override {}
 	virtual void next_group() override {}
 	virtual void prev_group() override {}
 	virtual const std::vector<item_group>& get_groups() const override { static const std::vector<item_group> empty; return empty; }
@@ -115,7 +114,7 @@ private:
 	std::string selected_item_;
 	std::vector<std::string> items_;
 	editor_toolkit& toolkit_;
-	boost::ptr_vector<location_palette_item> buttons_;
+	std::vector<location_palette_item> buttons_;
 	std::unique_ptr<location_palette_button> button_add_;
 	std::unique_ptr<location_palette_button> button_delete_;
 	std::unique_ptr<location_palette_button> button_goto_;
