@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-editor"
@@ -22,7 +23,7 @@
 
 namespace editor {
 
-palette_manager::palette_manager(editor_display& gui, const config& cfg
+palette_manager::palette_manager(editor_display& gui, const game_config_view& cfg
                                , editor_toolkit& toolkit)
 		: gui::widget(gui.video()),
 		  gui_(gui),
@@ -39,7 +40,7 @@ palette_manager::palette_manager(editor_display& gui, const config& cfg
 	item_palette_->setup(cfg);
 }
 
-void palette_manager::set_group(size_t index)
+void palette_manager::set_group(std::size_t index)
 {
 	active_palette().set_group(index);
 	scroll_top();
