@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -22,6 +23,7 @@
 
 #include "gettext.hpp"
 #include "serialization/string_utils.hpp"
+#include "game_config_view.hpp"
 
 namespace {
 	static t_translation::terrain_code fg_terrain;
@@ -67,7 +69,7 @@ void terrain_palette::select_fg_item(const t_translation::terrain_code& terrain)
 }
 
 
-void terrain_palette::setup(const config& cfg)
+void terrain_palette::setup(const game_config_view& cfg)
 {
 	// Get the available terrains temporary in items
 	t_translation::ter_list items = map().get_terrain_list();
@@ -212,7 +214,7 @@ void terrain_palette::draw_item(const t_translation::terrain_code& terrain,
 	}
 }
 
-terrain_palette::terrain_palette(editor_display &gui, const config& cfg, editor_toolkit &toolkit)
+terrain_palette::terrain_palette(editor_display &gui, const game_config_view& cfg, editor_toolkit &toolkit)
 //TODO avoid magic numbers
 	:	editor_palette<t_translation::terrain_code>(gui, cfg, 36, 4, toolkit)
 {
@@ -227,8 +229,21 @@ const std::string& terrain_palette::get_id(const t_translation::terrain_code& te
 std::string terrain_palette::get_help_string()
 {
 	std::ostringstream msg;
-	msg << _("FG: ") << map().get_terrain_editor_string(selected_fg_item())	<< " | "
-		<< _("BG: ") << map().get_terrain_editor_string(selected_bg_item());
+	msg << _("Left-click: ") << map().get_terrain_editor_string(selected_fg_item())	<< " | "
+		<< _("Right-click: ") << map().get_terrain_editor_string(selected_bg_item()) << "\n";
+	if(selected_fg_item().base == t_translation::NO_LAYER) {
+		// TRANSLATORS: Similar behavior applies to shift + right-click. This message specifies left-click
+		// because the logic of whether to show the "overlay only" or "base only" version depends on the
+		// terrain currently selected for the left button.
+		msg << _("Shift + left-click: paint overlay layer only") << " | ";
+	} else {
+		msg << _("Shift + left-click: paint base layer only") << " | ";
+	}
+#ifdef __APPLE__
+	msg << _("Cmd + click: copy terrain") << std::endl;
+#else
+	msg << _("Ctrl + click: copy terrain") << std::endl;
+#endif
 	return msg.str();
 }
 
