@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2012 - 2018 by Fabian Mueller <fabianmueller5@gmx.de>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2012 - 2021
+	by Fabian Mueller <fabianmueller5@gmx.de>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -31,7 +32,7 @@ std::string unit_palette::get_help_string() {
 	return selected_fg_item().type_name();
 }
 
-void unit_palette::setup(const config& /*cfg*/)
+void unit_palette::setup(const game_config_view& /*cfg*/)
 {
 	for(const unit_type_data::unit_type_map::value_type &i : unit_types.types()) {
 		if(i.second.do_not_list())
@@ -94,11 +95,11 @@ void unit_palette::draw_item(const unit_type& u, surface& image, std::stringstre
 	tooltip_text << u.type_name();
 }
 
-unit_palette::unit_palette(editor_display &gui, const config& cfg,
+unit_palette::unit_palette(editor_display &gui, const game_config_view& cfg,
                            editor_toolkit &toolkit)
 //TODO avoid magic numbers
-	:	editor_palette<unit_type>(gui, cfg, 36, 4, toolkit),
-		selected_bg_items_()
+	: editor_palette<const unit_type&>(gui, cfg, 36, 4, toolkit)
+	, selected_bg_items_()
 {
 }
 
