@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2012 - 2018 by Fabian Mueller <fabianmueller5@gmx.de>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2012 - 2021
+	by Fabian Mueller <fabianmueller5@gmx.de>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -20,7 +21,7 @@
 #include "hotkey/hotkey_command.hpp"
 
 class config;
-
+class game_config_view;
 namespace editor {
 
 
@@ -28,7 +29,7 @@ class editor_toolkit {
 
 public:
 	editor_toolkit(editor_display& gui, const CKey& key,
-			const config& game_config, context_manager& c_manager);
+			const game_config_view& game_config, context_manager& c_manager);
 
 	~editor_toolkit();
 
@@ -38,10 +39,10 @@ public:
 
 private:
 	/** init the sidebar objects */
-	void init_sidebar(const config& game_config);
+	void init_sidebar(const game_config_view& game_config);
 
 	/** init the brushes */
-	void init_brushes(const config& game_config);
+	void init_brushes(const game_config_view& game_config);
 
 	/** init the mouse actions (tools) */
 	void init_mouse_actions(context_manager& c_manager);
