@@ -1,42 +1,43 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "events.hpp"
+
 #include "cursor.hpp"
 #include "desktop/clipboard.hpp"
 #include "log.hpp"
 #include "quit_confirmation.hpp"
-#include "video.hpp"
 #include "sdl/userevent.hpp"
+#include "utils/ranges.hpp"
+#include "video.hpp"
 
 #if defined _WIN32
 #include "desktop/windows_tray_notification.hpp"
 #endif
 
 #include <algorithm>
-#include <atomic>
 #include <cassert>
 #include <deque>
+#include <future>
 #include <iterator>
+#include <thread>
 #include <utility>
 #include <vector>
 
 #include <SDL2/SDL.h>
 
-#include <boost/range/adaptor/reversed.hpp>
-#include <boost/thread.hpp>
-
 #define ERR_GEN LOG_STREAM(err, lg::general)
 
 namespace
@@ -46,19 +47,14 @@ struct invoked_function_data
 	explicit invoked_function_data(const std::function<void(void)>& func)
 		: f(func)
 		, finished()
-		, thrown_exception()
 	{
-		finished = false;
 	}
 
 	/** The actual function to call. */
 	const std::function<void(void)>& f;
 
 	/** Whether execution in the main thread is complete. */
-	std::atomic_bool finished;
-
-	/** Stores any exception thrown during the execution of @ref f. */
-	std::exception_ptr thrown_exception;
+	std::promise<void> finished;
 
 	void call()
 	{
@@ -68,10 +64,11 @@ struct invoked_function_data
 			// Handle this exception in the main thread.
 			throw;
 		} catch(...) {
-			thrown_exception = std::current_exception();
+			finished.set_exception(std::current_exception());
+			return;
 		}
 
-		finished = true;
+		finished.set_value();
 	}
 };
 }
@@ -252,7 +249,7 @@ sdl_handler::sdl_handler(const sdl_handler &that)
 		event_contexts.front().add_handler(this);
 	} else if(has_joined_) {
 		bool found_context = false;
-		for(auto &context : boost::adaptors::reverse(event_contexts)) {
+		for(auto &context : utils::reversed_view(event_contexts)) {
 			if(context.has_handler(&that)) {
 				found_context = true;
 				context.add_handler(this);
@@ -271,7 +268,7 @@ sdl_handler &sdl_handler::operator=(const sdl_handler &that)
 	if(that.has_joined_global_) {
 		join_global();
 	} else if(that.has_joined_) {
-		for(auto &context : boost::adaptors::reverse(event_contexts)) {
+		for(auto &context : utils::reversed_view(event_contexts)) {
 			if(context.has_handler(&that)) {
 				join(context);
 				break;
@@ -332,7 +329,7 @@ void sdl_handler::join_same(sdl_handler* parent)
 		leave(); // should not be in multiple event contexts
 	}
 
-	for(auto& context : boost::adaptors::reverse(event_contexts)) {
+	for(auto& context : utils::reversed_view(event_contexts)) {
 		if(context.has_handler(parent)) {
 			join(context);
 			return;
@@ -354,7 +351,7 @@ void sdl_handler::leave()
 		member->leave();
 	}
 
-	for(auto& context : boost::adaptors::reverse(event_contexts)) {
+	for(auto& context : utils::reversed_view(event_contexts)) {
 		if(context.remove_handler(this)) {
 			break;
 		}
@@ -471,11 +468,11 @@ static bool remove_on_resize(const SDL_Event& a)
 }
 
 // TODO: I'm uncertain if this is always safe to call at static init; maybe set in main() instead?
-static const boost::thread::id main_thread = boost::this_thread::get_id();
+static const std::thread::id main_thread = std::this_thread::get_id();
 
 void pump()
 {
-	if(boost::this_thread::get_id() != main_thread) {
+	if(std::this_thread::get_id() != main_thread) {
 		// Can only call this on the main thread!
 		return;
 	}
@@ -514,7 +511,7 @@ void pump()
 		events.push_back(temp_event);
 	}
 
-	std::vector<SDL_Event>::iterator ev_it = events.begin();
+	auto ev_it = events.begin();
 	for(int i = 1; i < begin_ignoring; ++i) {
 		if(is_input(*ev_it)) {
 			// ignore user input events that occurred before the window was activated
@@ -524,14 +521,15 @@ void pump()
 		}
 	}
 
-	std::vector<SDL_Event>::iterator ev_end = events.end();
 	bool resize_found = false;
-	for(ev_it = events.begin(); ev_it != ev_end; ++ev_it) {
-		SDL_Event& event = *ev_it;
+	for(const SDL_Event& event : utils::reversed_view(events)) {
 		if(event.type == SDL_WINDOWEVENT && event.window.event == SDL_WINDOWEVENT_RESIZED) {
 			resize_found = true;
 			last_resize_event = event;
 			last_resize_event_used = false;
+
+			// Since we're working backwards, the first resize event found is the last in the list.
+			break;
 		}
 	}
 
@@ -554,15 +552,11 @@ void pump()
 		events.erase(first_draw_event + 1, events.end());
 	}
 
-	ev_end = events.end();
-
-	for(ev_it = events.begin(); ev_it != ev_end; ++ev_it) {
+	for(SDL_Event& event : events) {
 		for(context& c : event_contexts) {
 			c.add_staging_handlers();
 		}
 
-		SDL_Event& event = *ev_it;
-
 #ifdef MOUSE_TOUCH_EMULATION
 		switch (event.type) {
 			// TODO: Implement SDL_MULTIGESTURE. Some day.
@@ -872,7 +866,7 @@ void peek_for_resize()
 
 void call_in_main_thread(const std::function<void(void)>& f)
 {
-	if(boost::this_thread::get_id() == main_thread) {
+	if(std::this_thread::get_id() == main_thread) {
 		// nothing special to do if called from the main thread.
 		f();
 		return;
@@ -888,13 +882,8 @@ void call_in_main_thread(const std::function<void(void)>& f)
 
 	SDL_PushEvent(&sdl_event);
 
-	while(!fdata.finished) {
-		SDL_Delay(10);
-	}
-
-	if(fdata.thrown_exception) {
-		std::rethrow_exception(fdata.thrown_exception);
-	}
+	// Block until execution is complete in the main thread. Rethrows any exceptions.
+	fdata.finished.get_future().wait();
 }
 
 } // end events namespace
