@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "fake_unit_ptr.hpp"
@@ -19,8 +20,6 @@
 #include "units/unit.hpp"
 #include "units/ptr.hpp"
 
-#include <boost/swap.hpp>
-
 fake_unit_ptr::fake_unit_ptr() : unit_(), my_manager_(nullptr) {}
 fake_unit_ptr::fake_unit_ptr(const internal_ptr & u) : unit_(u), my_manager_(nullptr) {}
 fake_unit_ptr::fake_unit_ptr(const internal_ptr & u, fake_unit_manager * mgr) : unit_(u), my_manager_(nullptr)
@@ -40,7 +39,7 @@ fake_unit_ptr::fake_unit_ptr(fake_unit_ptr && ptr)
 }
 
 void fake_unit_ptr::swap (fake_unit_ptr & o) {
-	boost::swap(unit_, o.unit_);
+	std::swap(unit_, o.unit_);
 	std::swap(my_manager_, o.my_manager_);
 }
 
