@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -19,19 +20,18 @@
 
 #pragma once
 
-#include <algorithm>
 #include <ctime>
-#include <functional>
+#include <fstream>
 #include <iosfwd>
+#include <memory>
 #include <string>
 #include <vector>
-#include <memory>
 
 #include "exceptions.hpp"
-#include "serialization/string_utils.hpp"
+#include "game_version.hpp"
 
 class config;
-
+class game_config_view;
 struct SDL_RWops;
 
 namespace filesystem {
@@ -52,9 +52,9 @@ struct io_exception : public game::error {
 
 struct file_tree_checksum;
 
-enum file_name_option { ENTIRE_FILE_PATH, FILE_NAME_ONLY };
-enum file_filter_option { NO_FILTER, SKIP_MEDIA_DIR, SKIP_PBL_FILES };
-enum file_reorder_option { DONT_REORDER, DO_REORDER };
+enum class name_mode { ENTIRE_FILE_PATH, FILE_NAME_ONLY };
+enum class filter_mode { NO_FILTER, SKIP_MEDIA_DIR, SKIP_PBL_FILES };
+enum class reorder_mode { DONT_REORDER, DO_REORDER };
 
 // A list of file and directory blacklist patterns
 class blacklist_pattern_list
@@ -67,17 +67,9 @@ public:
 		: file_patterns_(file_patterns), directory_patterns_(directory_patterns)
 	{}
 
-	bool match_file(const std::string& name) const
-	{
-		return std::any_of(file_patterns_.begin(), file_patterns_.end(),
-			std::bind(&utils::wildcard_string_match, std::ref(name), std::placeholders::_1));
-	}
+	bool match_file(const std::string& name) const;
 
-	bool match_dir(const std::string& name) const
-	{
-		return std::any_of(directory_patterns_.begin(), directory_patterns_.end(),
-			std::bind(&utils::wildcard_string_match, std::ref(name), std::placeholders::_1));
-	}
+	bool match_dir(const std::string& name) const;
 
 	void add_file_pattern(const std::string& pattern)
 	{
@@ -130,25 +122,23 @@ static const blacklist_pattern_list default_blacklist{
 	}
 };
 
-/** Some tasks to run on startup. */
-void init();
-
 /**
- * Populates 'files' with all the files and
- * 'dirs' with all the directories in dir.
- * If files or dirs are nullptr they will not be used.
+ * Get a list of all files and/or directories in a given directory.
  *
- * mode: determines whether the entire path or just the filename is retrieved.
- * filter: determines if we skip images and sounds directories
- * reorder: triggers the special handling of _main.cfg and _final.cfg
- * checksum: can be used to store checksum info
+ * @param dir The directory to examine.
+ * @param[out] files The files in @a dir. Won't be used if nullptr.
+ * @param[out] dirs The directories in @a dir. Won't be used if nullptr.
+ * @param mode Determines whether the entire path or just the filename is retrieved.
+ * @param filter Determines if we skip images and sounds directories.
+ * @param reorder Triggers the special handling of _main.cfg and _final.cfg.
+ * @param[out] checksum Can be used to store checksum info.
  */
 void get_files_in_dir(const std::string &dir,
                       std::vector<std::string>* files,
                       std::vector<std::string>* dirs=nullptr,
-                      file_name_option mode = FILE_NAME_ONLY,
-                      file_filter_option filter = NO_FILTER,
-                      file_reorder_option reorder = DONT_REORDER,
+                      name_mode mode = name_mode::FILE_NAME_ONLY,
+                      filter_mode filter = filter_mode::NO_FILTER,
+                      reorder_mode reorder = reorder_mode::DONT_REORDER,
                       file_tree_checksum* checksum = nullptr);
 
 std::string get_dir(const std::string &dir);
@@ -162,6 +152,8 @@ std::string get_saves_dir();
 std::string get_intl_dir();
 std::string get_screenshot_dir();
 std::string get_addons_dir();
+const std::string get_version_path_suffix(const version_info& version);
+const std::string& get_version_path_suffix();
 
 /**
  * Get the next free filename using "name + number (3 digits) + extension"
@@ -175,23 +167,59 @@ std::string get_user_config_dir();
 std::string get_user_data_dir();
 std::string get_cache_dir();
 
+struct other_version_dir
+{
+	/**
+	 * Here the version is given as a string instead of a version_info, because the
+	 * logic of how many components are significant ("1.16" rather than
+	 * "1.16.0") is encapsulated in find_other_version_saves_dirs().
+	 */
+	std::string version;
+	std::string path;
+
+	// constructor because emplace_back() doesn't use aggregate initialization
+	other_version_dir(const std::string& v, const std::string& p)
+		: version(v)
+		, path(p)
+	{
+	}
+};
+
+/**
+ * Searches for directories containing saves created by other versions of Wesnoth.
+ *
+ * The directories returned will exist, but might not contain any saves. Changes to
+ * the filesystem (by running other versions or by deleting old directories) may
+ * change the results returned by the function.
+ */
+std::vector<other_version_dir> find_other_version_saves_dirs();
+
 std::string get_cwd();
+bool set_cwd(const std::string& dir);
+
 std::string get_exe_dir();
 
 bool make_directory(const std::string& dirname);
 bool delete_directory(const std::string& dirname, const bool keep_pbl = false);
-bool delete_file(const std::string &filename);
+bool delete_file(const std::string& filename);
 
 bool looks_like_pbl(const std::string& file);
 
 // Basic disk I/O:
 
 /** Basic disk I/O - read file. */
-std::string read_file(const std::string &fname);
+std::string read_file(const std::string& fname);
 filesystem::scoped_istream istream_file(const std::string& fname, bool treat_failure_as_error = true);
-filesystem::scoped_ostream ostream_file(const std::string& fname, bool create_directory = true);
+filesystem::scoped_ostream ostream_file(const std::string& fname, std::ios_base::openmode mode = std::ios_base::binary, bool create_directory = true);
 /** Throws io_exception if an error occurs. */
 void write_file(const std::string& fname, const std::string& data);
+/**
+ * Read a file and then writes it back out.
+ *
+ * @param src The source file.
+ * @param dest The destination of the copied file.
+ */
+void copy_file(const std::string& src, const std::string& dest);
 
 std::string read_map(const std::string& name);
 
@@ -219,7 +247,7 @@ bool is_directory(const std::string& fname);
 bool file_exists(const std::string& name);
 
 /** Get the modification time of a file. */
-time_t file_modified_time(const std::string& fname);
+std::time_t file_modified_time(const std::string& fname);
 
 /** Returns true if the file ends with '.gz'. */
 bool is_gzip_file(const std::string& filename);
@@ -231,6 +259,18 @@ inline bool is_compressed_file(const std::string& filename) {
 	return is_gzip_file(filename) || is_bzip2_file(filename);
 }
 
+/**
+ * Returns whether the given filename is a legal name for a user-created file.
+ *
+ * This is meant to be used for any files created by Wesnoth where user input
+ * is required, including save files and add-on files for uploading to the
+ * add-ons server.
+ *
+ * @param name                 File name to verify.
+ * @param allow_whitespace     Whether whitespace should be allowed.
+ */
+bool is_legal_user_file_name(const std::string& name, bool allow_whitespace = true);
+
 struct file_tree_checksum
 {
 	file_tree_checksum();
@@ -238,8 +278,8 @@ struct file_tree_checksum
 	void write(config& cfg) const;
 	void reset() {nfiles = 0;modified = 0;sum_size=0;}
 	// @todo make variables private!
-	size_t nfiles, sum_size;
-	time_t modified;
+	std::size_t nfiles, sum_size;
+	std::time_t modified;
 	bool operator==(const file_tree_checksum &rhs) const;
 	bool operator!=(const file_tree_checksum &rhs) const
 	{ return !operator==(rhs); }
@@ -360,10 +400,10 @@ char path_separator();
 struct binary_paths_manager
 {
 	binary_paths_manager();
-	binary_paths_manager(const config& cfg);
+	binary_paths_manager(const game_config_view& cfg);
 	~binary_paths_manager();
 
-	void set_paths(const config& cfg);
+	void set_paths(const game_config_view& cfg);
 
 private:
 	binary_paths_manager(const binary_paths_manager& o);
@@ -407,13 +447,13 @@ std::string get_wml_location(const std::string &filename,
 std::string get_short_wml_path(const std::string &filename);
 
 /**
- * Returns an image path to @a filename for binary path-independent use in saved games.
+ * Returns an asset path to @a filename for binary path-independent use in saved games.
  *
  * Example:
- *   units/konrad-fighter.png ->
+ *   images, units/konrad-fighter.png ->
  *   data/campaigns/Heir_To_The_Throne/images/units/konrad-fighter.png
  */
-std::string get_independent_image_path(const std::string &filename);
+std::string get_independent_binary_file_path(const std::string& type, const std::string &filename);
 
 /**
  * Returns the appropriate invocation for a Wesnoth-related binary, assuming
@@ -424,5 +464,9 @@ std::string get_independent_image_path(const std::string &filename);
  */
 std::string get_program_invocation(const std::string &program_name);
 
+/**
+ * Returns the localized version of the given filename, if it exists.
+ */
+std::string get_localized_path(const std::string& file, const std::string& suff = "");
 
 }
