@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2017-2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2017 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include <SDL2/SDL.h>
@@ -18,6 +19,7 @@
 #include "log.hpp"
 
 #include <algorithm>
+#include <cassert>
 
 static lg::log_domain log_filesystem("filesystem");
 #define ERR_FS LOG_STREAM(err, log_filesystem)
@@ -32,10 +34,10 @@ static int64_t ifs_size (struct SDL_RWops * context);
 static int64_t ofs_size (struct SDL_RWops * context);
 static int64_t SDLCALL ifs_seek(struct SDL_RWops *context, int64_t offset, int whence);
 static int64_t SDLCALL ofs_seek(struct SDL_RWops *context, int64_t offset, int whence);
-static size_t SDLCALL ifs_read(struct SDL_RWops *context, void *ptr, size_t size, size_t maxnum);
-static size_t SDLCALL ofs_read(struct SDL_RWops *context, void *ptr, size_t size, size_t maxnum);
-static size_t SDLCALL ifs_write(struct SDL_RWops *context, const void *ptr, size_t size, size_t num);
-static size_t SDLCALL ofs_write(struct SDL_RWops *context, const void *ptr, size_t size, size_t num);
+static std::size_t SDLCALL ifs_read(struct SDL_RWops *context, void *ptr, std::size_t size, std::size_t maxnum);
+static std::size_t SDLCALL ofs_read(struct SDL_RWops *context, void *ptr, std::size_t size, std::size_t maxnum);
+static std::size_t SDLCALL ifs_write(struct SDL_RWops *context, const void *ptr, std::size_t size, std::size_t num);
+static std::size_t SDLCALL ofs_write(struct SDL_RWops *context, const void *ptr, std::size_t size, std::size_t num);
 static int SDLCALL ifs_close(struct SDL_RWops *context);
 static int SDLCALL ofs_close(struct SDL_RWops *context);
 
@@ -115,11 +117,11 @@ typedef std::pair<int64_t, std::ios_base::seekdir> offset_dir;
 static offset_dir translate_seekdir(int64_t offset, int whence) {
 	switch(whence){
 	case RW_SEEK_SET:
-		return std::make_pair(std::max<int64_t>(0, offset), std::ios_base::beg);
+		return std::pair(std::max<int64_t>(0, offset), std::ios_base::beg);
 	case RW_SEEK_CUR:
-		return std::make_pair(offset, std::ios_base::cur);
+		return std::pair(offset, std::ios_base::cur);
 	case RW_SEEK_END:
-		return std::make_pair(std::min<int64_t>(0, offset), std::ios_base::end);
+		return std::pair(std::min<int64_t>(0, offset), std::ios_base::end);
 	default:
 		assert(false);
 		throw "assertion ignored";
@@ -160,7 +162,7 @@ static int64_t SDLCALL ofs_seek(struct SDL_RWops *context, int64_t offset, int w
 	return static_cast<int>(pos);
 }
 
-static size_t SDLCALL ifs_read(struct SDL_RWops *context, void *ptr, size_t size, size_t maxnum) {
+static std::size_t SDLCALL ifs_read(struct SDL_RWops *context, void *ptr, std::size_t size, std::size_t maxnum) {
 	std::istream *ifs = static_cast<std::istream*>(context->hidden.unknown.data1);
 
 	// This seems overly simplistic, but it's the same as mem_read's implementation
@@ -173,16 +175,16 @@ static size_t SDLCALL ifs_read(struct SDL_RWops *context, void *ptr, size_t size
 
 	return static_cast<int>(num);
 }
-static size_t SDLCALL ofs_read(struct SDL_RWops * /*context*/, void * /*ptr*/, size_t /*size*/, size_t /*maxnum*/) {
+static std::size_t SDLCALL ofs_read(struct SDL_RWops * /*context*/, void * /*ptr*/, std::size_t /*size*/, std::size_t /*maxnum*/) {
 	SDL_SetError("Reading not implemented");
 	return 0;
 }
 
-static size_t SDLCALL ifs_write(struct SDL_RWops * /*context*/, const void * /*ptr*/, size_t /*size*/, size_t /*num*/) {
+static std::size_t SDLCALL ifs_write(struct SDL_RWops * /*context*/, const void * /*ptr*/, std::size_t /*size*/, std::size_t /*num*/) {
 	SDL_SetError("Writing not implemented");
 	return 0;
 }
-static size_t SDLCALL ofs_write(struct SDL_RWops *context, const void *ptr, size_t size, size_t num) {
+static std::size_t SDLCALL ofs_write(struct SDL_RWops *context, const void *ptr, std::size_t size, std::size_t num) {
 	std::ostream *ofs = static_cast<std::ostream*>(context->hidden.unknown.data1);
 
 	const std::streampos before = ofs->tellp();
