@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "floating_label.hpp"
@@ -46,9 +47,12 @@ floating_label::floating_label(const std::string& text, const surface& surf)
 #else
 	: surf_(surf)
 	, buf_(nullptr)
+	, buf_pos_()
 #endif
+	, fadeout_(0)
+	, time_start_(0)
 	, text_(text)
-	, font_size_(SIZE_NORMAL)
+	, font_size_(SIZE_SMALL)
 	, color_(NORMAL_COLOR)
 	, bgcolor_()
 	, bgalpha_(0)
@@ -60,7 +64,6 @@ floating_label::floating_label(const std::string& text, const surface& surf)
 	, width_(-1)
 	, height_(-1)
 	, clip_rect_(CVideo::get_singleton().screen_area())
-	, alpha_change_(0)
 	, visible_(true)
 	, align_(CENTER_ALIGN)
 	, border_(0)
@@ -75,7 +78,7 @@ void floating_label::move(double xmove, double ymove)
 	ypos_ += ymove;
 }
 
-int floating_label::xpos(size_t width) const
+int floating_label::xpos(std::size_t width) const
 {
 	int xpos = int(xpos_);
 	if(align_ == font::CENTER_ALIGN) {
@@ -90,11 +93,18 @@ int floating_label::xpos(size_t width) const
 surface floating_label::create_surface()
 {
 	if(!surf_) {
-		font::pango_text text;
-		text.set_foreground_color(color_);
-		text.set_font_size(font_size_);
-		text.set_maximum_width(width_ < 0 ? clip_rect_.w : width_);
-		text.set_maximum_height(height_ < 0 ? clip_rect_.h : height_, true);
+		font::pango_text& text = font::get_text_renderer();
+
+		text.set_link_aware(false)
+			.set_family_class(font::FONT_SANS_SERIF)
+			.set_font_size(font_size_)
+			.set_font_style(font::pango_text::STYLE_NORMAL)
+			.set_alignment(PANGO_ALIGN_LEFT)
+			.set_foreground_color(color_)
+			.set_maximum_width(width_ < 0 ? clip_rect_.w : width_)
+			.set_maximum_height(height_ < 0 ? clip_rect_.h : height_, true)
+			.set_ellipse_mode(PANGO_ELLIPSIZE_END)
+			.set_characters_per_line(0);
 
 		// ignore last '\n'
 		if(!text_.empty() && *(text_.rbegin()) == '\n') {
@@ -154,7 +164,7 @@ surface floating_label::create_surface()
 	return surf_;
 }
 
-void floating_label::draw(surface screen)
+void floating_label::draw(int time, surface screen)
 {
 	if(!visible_) {
 		buf_ = nullptr;
@@ -177,10 +187,43 @@ void floating_label::draw(surface screen)
 		}
 	}
 
-	SDL_Rect rect = sdl::create_rect(xpos(surf_->w), ypos_, surf_->w, surf_->h);
+	SDL_Point pos = get_loc(time);
+	buf_pos_ = sdl::create_rect(pos.x, pos.y, surf_->w, surf_->h);
 	const clip_rect_setter clip_setter(screen, &clip_rect_);
-	sdl_copy_portion(screen,&rect,buf_,nullptr);
-	sdl_blit(surf_,nullptr,screen,&rect);
+	//important: make a copy of buf_pos_ because sdl_blit modifies dst_rect.
+	SDL_Rect rect = buf_pos_;
+	sdl_copy_portion(screen, &rect, buf_, nullptr);
+	sdl_blit(get_surface(time), nullptr, screen, &rect);
+}
+
+void floating_label::set_lifetime(int lifetime, int fadeout)
+{
+	lifetime_ = lifetime;
+	fadeout_ = fadeout;
+	time_start_	= SDL_GetTicks();
+}
+
+
+SDL_Point floating_label::get_loc(int time)
+{
+	int time_alive = get_time_alive(time);
+	return {
+		static_cast<int>(time_alive * xmove_ + xpos(surf_->w)),
+		static_cast<int>(time_alive * ymove_ + ypos_)
+ 	};
+}
+
+surface floating_label::get_surface(int time)
+{
+	if(lifetime_ >= 0 && fadeout_ > 0) {
+		int time_alive = get_time_alive(time);
+		if(time_alive >= lifetime_ && surf_ != nullptr) {
+			// fade out moving floating labels
+			int alpha_add = -255 * (time_alive - lifetime_) / fadeout_;
+			return adjust_surface_alpha_add(surf_, alpha_add);
+		}
+	}
+	return surf_;
 }
 
 void floating_label::undraw(surface screen)
@@ -188,18 +231,10 @@ void floating_label::undraw(surface screen)
 	if(screen == nullptr || buf_ == nullptr) {
 		return;
 	}
-	SDL_Rect rect = sdl::create_rect(xpos(surf_->w), ypos_, surf_->w, surf_->h);
-	const clip_rect_setter clip_setter(screen, &clip_rect_);
-	sdl_blit(buf_,nullptr,screen,&rect);
 
-	move(xmove_,ymove_);
-	if(lifetime_ > 0) {
-		--lifetime_;
-		if(alpha_change_ != 0 && (xmove_ != 0.0 || ymove_ != 0.0) && surf_ != nullptr) {
-			// fade out moving floating labels
-			surf_ = adjust_surface_alpha_add(surf_,alpha_change_);
-		}
-	}
+	const clip_rect_setter clip_setter(screen, &clip_rect_);
+	SDL_Rect rect = buf_pos_;
+	sdl_blit(buf_, nullptr, screen, &rect);
 }
 
 int add_floating_label(const floating_label& flabel)
@@ -231,10 +266,14 @@ void scroll_floating_labels(double xmove, double ymove)
 	}
 }
 
-void remove_floating_label(int handle)
+void remove_floating_label(int handle, int fadeout)
 {
 	const label_map::iterator i = labels.find(handle);
 	if(i != labels.end()) {
+		if(fadeout > 0) {
+			i->second.set_lifetime(0, fadeout);
+			return;
+		}
 		labels.erase(i);
 	}
 
@@ -265,11 +304,13 @@ SDL_Rect get_floating_label_rect(int handle)
 
 floating_label_context::floating_label_context()
 {
+	//TODO: 'pause' floating labels in other contexrs
 	label_contexts.emplace();
 }
 
 floating_label_context::~floating_label_context()
 {
+	//TODO: 'pause' floating labels in other contexrs
 	const std::set<int>& context = label_contexts.top();
 
 	while(!context.empty()) {
@@ -286,6 +327,7 @@ void draw_floating_labels(surface screen)
 	if(label_contexts.empty()) {
 		return;
 	}
+	int time = SDL_GetTicks();
 
 	const std::set<int>& context = label_contexts.top();
 
@@ -293,7 +335,7 @@ void draw_floating_labels(surface screen)
 	// are displayed over earlier added labels.
 	for(label_map::iterator i = labels.begin(); i != labels.end(); ++i) {
 		if(context.count(i->first) > 0) {
-			i->second.draw(screen);
+			i->second.draw(time, screen);
 		}
 	}
 }
@@ -303,6 +345,7 @@ void undraw_floating_labels(surface screen)
 	if(label_contexts.empty()) {
 		return;
 	}
+	int time = SDL_GetTicks();
 
 	std::set<int>& context = label_contexts.top();
 
@@ -316,7 +359,7 @@ void undraw_floating_labels(surface screen)
 
 	//remove expired labels
 	for(label_map::iterator j = labels.begin(); j != labels.end(); ) {
-		if(context.count(j->first) > 0 && j->second.expired()) {
+		if(context.count(j->first) > 0 && j->second.expired(time)) {
 			context.erase(j->first);
 			labels.erase(j++);
 		} else {
