@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2006 - 2018 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "floating_textbox.hpp"
@@ -33,7 +34,8 @@ namespace gui{
 		check_(nullptr),
 		mode_(TEXTBOX_NONE),
 		label_string_(),
-		label_(0)
+		label_(0),
+		command_history_()
 	{}
 
 	void floating_textbox::close(game_display& gui)
@@ -118,7 +120,7 @@ namespace gui{
 		}
 
 
-		box_.reset(new gui::textbox(gui.video(),100,"",true,256,font::SIZE_PLUS,0.8,0.6));
+		box_.reset(new gui::textbox(gui.video(),100,"",true,256,font::SIZE_NORMAL,0.8,0.6));
 
 		update_location(gui);
 	}
@@ -143,4 +145,18 @@ namespace gui{
 		}
 		box_->set_text(text);
 	}
+
+	void floating_textbox::memorize_command(const std::string& command)
+	{
+		if(command.empty()) {
+			return;
+		}
+
+		auto prev = std::find(command_history_.begin(), command_history_.end(), command);
+
+		if(prev != command_history_.end()) {
+			command_history_.erase(prev);
+		}
+		command_history_.emplace_back(command);
+	}
 }
