@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -31,7 +32,6 @@
 #include "serialization/string_utils.hpp"
 #include "serialization/unicode.hpp"
 #include "preferences/general.hpp"
-#include "utils/general.hpp"
 
 #include <boost/algorithm/string/replace.hpp>
 
@@ -45,7 +45,6 @@ pango_text::pango_text()
 	: context_(pango_font_map_create_context(pango_cairo_font_map_get_default()), g_object_unref)
 	, layout_(pango_layout_new(context_.get()), g_object_unref)
 	, rect_()
-	, sublayouts_()
 	, surface_()
 	, text_()
 	, markedup_text_(false)
@@ -65,6 +64,7 @@ pango_text::pango_text()
 	, calculation_dirty_(true)
 	, length_(0)
 	, surface_dirty_(true)
+	, rendered_viewport_()
 	, surface_buffer_()
 {
 	// With 72 dpi the sizes are the same as with SDL_TTF so hardcoded.
@@ -91,12 +91,19 @@ pango_text::pango_text()
 	cairo_font_options_destroy(fo);
 }
 
-surface& pango_text::render()
+surface& pango_text::render(const SDL_Rect& viewport)
 {
-	this->rerender();
+	rerender(viewport);
 	return surface_;
 }
 
+surface& pango_text::render()
+{
+	recalculate();
+	auto viewport = SDL_Rect{0, 0, rect_.x + rect_.width, rect_.y + rect_.height};
+	rerender(viewport);
+	return surface_;
+}
 
 int pango_text::get_width() const
 {
@@ -135,24 +142,13 @@ unsigned pango_text::insert_text(const unsigned offset, const std::string& text)
 	if (length_ + len > maximum_length_) {
 		len = maximum_length_ - length_;
 	}
-	const utf8::string insert = text.substr(0, utf8::index(text, len));
-	utf8::string tmp = text_;
+	const std::string insert = text.substr(0, utf8::index(text, len));
+	std::string tmp = text_;
 	this->set_text(utf8::insert(tmp, offset, insert), false);
 	// report back how many characters were actually inserted (e.g. to move the cursor selection)
 	return len;
 }
 
-bool pango_text::insert_unicode(const unsigned offset, ucs4::char_t unicode)
-{
-	return this->insert_unicode(offset, ucs4::string(1, unicode)) == 1;
-}
-
-unsigned pango_text::insert_unicode(const unsigned offset, const ucs4::string& unicode)
-{
-	const utf8::string insert = unicode_cast<utf8::string>(unicode);
-	return this->insert_text(offset, insert);
-}
-
 point pango_text::get_cursor_position(
 		const unsigned column, const unsigned line) const
 {
@@ -169,13 +165,13 @@ point pango_text::get_cursor_position(
 			return point(0, 0);
 		}
 
-		for(size_t i = 0; i < line; ++i) {
+		for(std::size_t i = 0; i < line; ++i) {
 			pango_layout_iter_next_line(itor.get());
 		}
 	}
 
 	// Go the wanted column.
-	for(size_t i = 0; i < column; ++i) {
+	for(std::size_t i = 0; i < column; ++i) {
 		if(!pango_layout_iter_next_char(itor.get())) {
 			// It seems that the documentation is wrong and causes and off by
 			// one error... the result should be false if already at the end of
@@ -198,7 +194,7 @@ point pango_text::get_cursor_position(
 	return point(PANGO_PIXELS(rect.x), PANGO_PIXELS(rect.y));
 }
 
-size_t pango_text::get_maximum_length() const
+std::size_t pango_text::get_maximum_length() const
 {
 	return maximum_length_;
 }
@@ -218,16 +214,16 @@ std::string pango_text::get_token(const point & position, const char * delim) co
 
 	std::string d(delim);
 
-	if (index < 0 || (static_cast<size_t>(index) >= txt.size()) || d.find(txt.at(index)) != std::string::npos) {
+	if (index < 0 || (static_cast<std::size_t>(index) >= txt.size()) || d.find(txt.at(index)) != std::string::npos) {
 		return ""; // if the index is out of bounds, or the index character is a delimiter, return nothing
 	}
 
-	size_t l = index;
+	std::size_t l = index;
 	while (l > 0 && (d.find(txt.at(l-1)) == std::string::npos)) {
 		--l;
 	}
 
-	size_t r = index + 1;
+	std::size_t r = index + 1;
 	while (r < txt.size() && (d.find(txt.at(r)) == std::string::npos)) {
 		++r;
 	}
@@ -275,7 +271,7 @@ point pango_text::get_column_line(const point& position) const
 	 * text is available. Haven't found what the best thing to do would be.
 	 * Until that time leave it as is.
 	 */
-	for(size_t i = 0; ; ++i) {
+	for(std::size_t i = 0; ; ++i) {
 		const int pos = this->get_cursor_position(i, line).x;
 
 		if(pos == offset) {
@@ -287,13 +283,12 @@ point pango_text::get_column_line(const point& position) const
 bool pango_text::set_text(const std::string& text, const bool markedup)
 {
 	if(markedup != markedup_text_ || text != text_) {
-		sublayouts_.clear();
 		if(layout_ == nullptr) {
 			layout_.reset(pango_layout_new(context_.get()));
 		}
 
-		const ucs4::string wide = unicode_cast<ucs4::string>(text);
-		const std::string narrow = unicode_cast<utf8::string>(wide);
+		const std::u32string wide = unicode_cast<std::u32string>(text);
+		const std::string narrow = unicode_cast<std::string>(wide);
 		if(text != narrow) {
 			ERR_GUI_L << "pango_text::" << __func__
 					<< " text '" << text
@@ -403,6 +398,11 @@ pango_text& pango_text::set_maximum_height(int height, bool multiline)
 	if(height != maximum_height_) {
 		// assert(context_);
 
+		// The maximum height is handled in this class' calculate_size() method.
+		//
+		// Although we also pass it to PangoLayout if multiline is true, the documentation of pango_layout_set_height
+		// makes me wonder whether we should avoid that function completely. For example, "at least one line is included
+		// in each paragraph regardless" and "may be changed in future, file a bug if you rely on the current behavior".
 		pango_layout_set_height(layout_.get(), !multiline ? -1 : height * PANGO_SCALE);
 		maximum_height_ = height;
 		calculation_dirty_ = true;
@@ -423,6 +423,13 @@ pango_text& pango_text::set_ellipse_mode(const PangoEllipsizeMode ellipse_mode)
 		surface_dirty_ = true;
 	}
 
+	// According to the docs of pango_layout_set_height, the behavior is undefined if a height other than -1 is combined
+	// with PANGO_ELLIPSIZE_NONE. Wesnoth's code currently always calls set_ellipse_mode after set_maximum_height, so do
+	// the cleanup here. The code in calculate_size() will still apply the maximum height after Pango's calculations.
+	if(ellipse_mode_ == PANGO_ELLIPSIZE_NONE) {
+		pango_layout_set_height(layout_.get(), -1);
+	}
+
 	return *this;
 }
 
@@ -437,12 +444,12 @@ pango_text &pango_text::set_alignment(const PangoAlignment alignment)
 	return *this;
 }
 
-pango_text& pango_text::set_maximum_length(const size_t maximum_length)
+pango_text& pango_text::set_maximum_length(const std::size_t maximum_length)
 {
 	if(maximum_length != maximum_length_) {
 		maximum_length_ = maximum_length;
 		if(length_ > maximum_length_) {
-			utf8::string tmp = text_;
+			std::string tmp = text_;
 			this->set_text(utf8::truncate(tmp, maximum_length_), false);
 		}
 	}
@@ -482,9 +489,29 @@ pango_text& pango_text::set_add_outline(bool do_add)
 	return *this;
 }
 
-void pango_text::recalculate(const bool force) const
+int pango_text::get_max_glyph_height() const
 {
-	if(calculation_dirty_ || force) {
+	p_font font{ get_font_families(font_class_), font_size_, font_style_ };
+
+	PangoFont* f = pango_font_map_load_font(
+		pango_cairo_font_map_get_default(),
+		context_.get(),
+		font.get());
+
+	PangoFontMetrics* m = pango_font_get_metrics(f, nullptr);
+
+	auto ascent = pango_font_metrics_get_ascent(m);
+	auto descent = pango_font_metrics_get_descent(m);
+
+	pango_font_metrics_unref(m);
+	g_object_unref(f);
+
+	return ceil(pango_units_to_double(ascent + descent));
+}
+
+void pango_text::recalculate() const
+{
+	if(calculation_dirty_) {
 		assert(layout_ != nullptr);
 
 		calculation_dirty_ = false;
@@ -523,6 +550,9 @@ PangoRectangle pango_text::calculate_size(PangoLayout& layout) const
 		w *= characters_per_line_;
 
 		maximum_width = ceil(pango_units_to_double(w));
+
+		pango_font_metrics_unref(m);
+		g_object_unref(f);
 	} else {
 		maximum_width = maximum_width_;
 	}
@@ -551,6 +581,7 @@ PangoRectangle pango_text::calculate_size(PangoLayout& layout) const
 		<< " maximum_height " << maximum_height_
 		<< " result " << size
 		<< ".\n";
+
 	if(maximum_width != -1 && size.x + size.width > maximum_width) {
 		DBG_GUI_L << "pango_text::" << __func__
 			<< " text '" << gui2::debug_truncate(text_)
@@ -559,6 +590,16 @@ PangoRectangle pango_text::calculate_size(PangoLayout& layout) const
 			<< ".\n";
 	}
 
+	// The maximum height is handled here instead of using the library - see the comments in set_maximum_height()
+	if(maximum_height_ != -1 && size.y + size.height > maximum_height_) {
+		DBG_GUI_L << "pango_text::" << __func__
+			<< " text '" << gui2::debug_truncate(text_)
+			<< " ' height " << size.y + size.height
+			<< " greater as the wanted maximum of " << maximum_height_
+			<< ".\n";
+		size.height = maximum_height_ - std::max(0, size.y);
+	}
+
 	return size;
 }
 
@@ -623,45 +664,30 @@ static void from_cairo_format(uint32_t & c)
 	// that, we work around that by increasing alpha at the expense of AA accuracy (which is
 	// not particularly noticeable if you don't know what you're looking for anyway).
 	if(a < 255) {
-		a = utils::clamp<unsigned>(unsigned(a) * 1.75, 0, 255);
+		a = std::clamp<unsigned>(unsigned(a) * 1.75, 0, 255);
 	}
 #endif
 
 	c = (static_cast<uint32_t>(a) << 24) | (static_cast<uint32_t>(r) << 16) | (static_cast<uint32_t>(g) << 8) | static_cast<uint32_t>(b);
 }
 
-void pango_text::render(PangoLayout& layout, const PangoRectangle& rect, const size_t surface_buffer_offset, const unsigned stride)
+void pango_text::render(PangoLayout& layout, const SDL_Rect& viewport, const unsigned stride)
 {
-	int width = rect.x + rect.width;
-	int height = rect.y + rect.height;
-	if(maximum_width_  > 0) { width = std::min(width, maximum_width_); }
-	if(maximum_height_ > 0) { height = std::min(height, maximum_height_); }
-
 	cairo_format_t format = CAIRO_FORMAT_ARGB32;
 
-	uint8_t* buffer = &surface_buffer_[surface_buffer_offset];
+	uint8_t* buffer = &surface_buffer_[0];
 
 	std::unique_ptr<cairo_surface_t, std::function<void(cairo_surface_t*)>> cairo_surface(
-		cairo_image_surface_create_for_data(buffer, format, width, height, stride), cairo_surface_destroy);
+		cairo_image_surface_create_for_data(buffer, format, viewport.w, viewport.h, stride), cairo_surface_destroy);
 	std::unique_ptr<cairo_t, std::function<void(cairo_t*)>> cr(cairo_create(cairo_surface.get()), cairo_destroy);
 
 	if(cairo_status(cr.get()) == CAIRO_STATUS_INVALID_SIZE) {
-		if(!is_surface_split()) {
-			split_surface();
-
-			PangoRectangle upper_rect = calculate_size(*sublayouts_[0]);
-			PangoRectangle lower_rect = calculate_size(*sublayouts_[1]);
-
-			render(*sublayouts_[0], upper_rect, 0u, stride);
-			render(*sublayouts_[1], lower_rect, upper_rect.height * stride, stride);
-
-			return;
-		} else {
-			// If this occurs in practice, it can be fixed by implementing recursive splitting.
-			throw std::length_error("Text is too long to render");
-		}
+		throw std::length_error("Text is too long to render");
 	}
 
+	// The top-left of the text, which can be outside the area to be rendered
+	cairo_move_to(cr.get(), -viewport.x, -viewport.y);
+
 	//
 	// TODO: the outline may be slightly cut off around certain text if it renders too
 	// close to the surface's edge. That causes the outline to extend just slightly
@@ -696,66 +722,58 @@ void pango_text::render(PangoLayout& layout, const PangoRectangle& rect, const s
 	pango_cairo_show_layout(cr.get(), &layout);
 }
 
-void pango_text::rerender(const bool force)
+void pango_text::rerender(const SDL_Rect& viewport)
 {
-	if(surface_dirty_ || force) {
+	if(surface_dirty_ || !SDL_RectEquals(&rendered_viewport_, &viewport)) {
 		assert(layout_.get());
 
-		this->recalculate(force);
+		this->recalculate();
 		surface_dirty_ = false;
-
-		int width  = rect_.x + rect_.width;
-		int height = rect_.y + rect_.height;
-		if(maximum_width_  > 0) { width  = std::min(width, maximum_width_); }
-		if(maximum_height_ > 0) { height = std::min(height, maximum_height_); }
+		rendered_viewport_ = viewport;
 
 		cairo_format_t format = CAIRO_FORMAT_ARGB32;
-		const unsigned stride = cairo_format_stride_for_width(format, width);
+		const int stride = cairo_format_stride_for_width(format, viewport.w);
 
-		this->create_surface_buffer(stride * height);
-
-		if (surface_buffer_.empty()) {
+		// The width and stride can be zero if the text is empty or the stride can be negative to indicate an error from
+		// Cairo. Width isn't tested here because it's implied by stride.
+		if(stride <= 0 || viewport.h <= 0) {
 			surface_ = surface(0, 0);
+			surface_buffer_.clear();
 			return;
 		}
 
-		render(*layout_, rect_, 0u, stride);
+		// Check to prevent arithmetic overflow when calculating (stride * height).
+		// The size of the viewport should already provide a far lower limit on the
+		// maximum size, but this is left in as a sanity check.
+		if(viewport.h > std::numeric_limits<int>::max() / stride) {
+			throw std::length_error("Text is too long to render");
+		}
+
+		// Resize buffer appropriately and set all pixel values to 0.
+		surface_ = nullptr; // Don't leave a dangling pointer to the old buffer
+		surface_buffer_.assign(viewport.h * stride, 0);
+
+		// Try rendering the whole text in one go. If this throws a length_error
+		// then leave it to the caller to handle; one reason it may throw is that
+		// cairo surfaces are limited to approximately 2**15 pixels in height.
+		render(*layout_, viewport, stride);
 
 		// The cairo surface is in CAIRO_FORMAT_ARGB32 which uses
 		// pre-multiplied alpha. SDL doesn't use that so the pixels need to be
 		// decoded again.
-		uint32_t * pixels = reinterpret_cast<uint32_t *>(&surface_buffer_[0]);
-
-		for(int y = 0; y < height; ++y) {
-			for(int x = 0; x < width; ++x) {
-				from_cairo_format(pixels[y * width + x]);
+		for(int y = 0; y < viewport.h; ++y) {
+			uint32_t* pixels = reinterpret_cast<uint32_t*>(&surface_buffer_[y * stride]);
+			for(int x = 0; x < viewport.w; ++x) {
+				from_cairo_format(pixels[x]);
 			}
 		}
 
-#if SDL_VERSION_ATLEAST(2, 0, 6)
 		surface_ = SDL_CreateRGBSurfaceWithFormatFrom(
-			&surface_buffer_[0], width, height, 32, stride, SDL_PIXELFORMAT_ARGB8888);
-#else
-		surface_ = SDL_CreateRGBSurfaceFrom(
-			&surface_buffer_[0], width, height, 32, stride, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
-#endif
-		if(!surface_) {
-			ERR_GUI_L << "pango_text: SDL_CreateRGBSurfaceWithFormatFrom Failed, w="
-			          << width << ", h=" << height << ", reason: " << SDL_GetError() << "\n";
-		}
+			&surface_buffer_[0], viewport.w, viewport.h, 32, stride, SDL_PIXELFORMAT_ARGB8888);
 	}
 }
 
-void pango_text::create_surface_buffer(const size_t size) const
-{
-	// Clear surface.
-	surface_ = nullptr;
-
-	// Resize buffer appropriately and clear all existing data (essentially sets all pixel values to 0).
-	surface_buffer_.assign(size, 0);
-}
-
-bool pango_text::set_markup(utils::string_view text, PangoLayout& layout) {
+bool pango_text::set_markup(std::string_view text, PangoLayout& layout) {
 	char* raw_text;
 	std::string semi_escaped;
 	bool valid = validate_markup(text, &raw_text, semi_escaped);
@@ -765,7 +783,7 @@ bool pango_text::set_markup(utils::string_view text, PangoLayout& layout) {
 
 	if(valid) {
 		if(link_aware_) {
-			std::string formatted_text = format_links(text.to_string());
+			std::string formatted_text = format_links(text);
 			pango_layout_set_markup(&layout, formatted_text.c_str(), formatted_text.size());
 		} else {
 			pango_layout_set_markup(&layout, text.data(), text.size());
@@ -774,7 +792,7 @@ bool pango_text::set_markup(utils::string_view text, PangoLayout& layout) {
 		ERR_GUI_L << "pango_text::" << __func__
 			<< " text '" << text
 			<< "' has broken markup, set to normal text.\n";
-		set_text(_("The text contains invalid Pango markup: ") + text.to_string(), false);
+		set_text(_("The text contains invalid Pango markup: ") + std::string(text), false);
 	}
 
 	return valid;
@@ -784,41 +802,45 @@ bool pango_text::set_markup(utils::string_view text, PangoLayout& layout) {
  * Replaces all instances of URLs in a given string with formatted links
  * and returns the result.
  */
-std::string pango_text::format_links(const std::string& text) const
+std::string pango_text::format_links(std::string_view text) const
 {
-	const std::string delim = " \n\r\t";
-	std::string result = "";
+	static const std::string delim = " \n\r\t";
+	std::ostringstream result;
 
-	int last_delim = -1;
-	for (size_t index = 0; index < text.size(); ++index) {
-		if (delim.find(text[index]) != std::string::npos) {
+	std::size_t tok_start = 0;
+	for(std::size_t pos = 0; pos < text.length(); ++pos) {
+		if(delim.find(text[pos]) == std::string::npos) {
+			continue;
+		}
+
+		if(const auto tok_length = pos - tok_start) {
 			// Token starts from after the last delimiter up to (but not including) this delimiter
-			std::string token = text.substr(last_delim + 1, index - last_delim - 1);
-
-			if (looks_like_url(token)) {
-				result += format_as_link(token, link_color_) + text[index];
+			auto token = text.substr(tok_start, tok_length);
+			if(looks_like_url(token)) {
+				result << format_as_link(std::string{token}, link_color_);
 			} else {
-				result += token + text[index];
+				result << token;
 			}
-
-			last_delim = index;
 		}
+
+		result << text[pos];
+		tok_start = pos + 1;
 	}
 
-	if (last_delim < static_cast<int>(text.size()) - 1) {
-		std::string token = text.substr(last_delim + 1, text.size() - last_delim - 1);
-
+	// Deal with the remainder token
+	if(tok_start < text.length()) {
+		auto token = text.substr(tok_start);
 		if(looks_like_url(token)) {
-			result += format_as_link(token, link_color_);
+			result << format_as_link(std::string{token}, link_color_);
 		} else {
-			result += token;
+			result << token;
 		}
 	}
 
-	return result;
+	return result.str();
 }
 
-bool pango_text::validate_markup(utils::string_view text, char** raw_text, std::string& semi_escaped) const
+bool pango_text::validate_markup(std::string_view text, char** raw_text, std::string& semi_escaped) const
 {
 	if(pango_parse_markup(text.data(), text.size(),
 		0, nullptr, raw_text, nullptr, nullptr)) {
@@ -833,7 +855,7 @@ bool pango_text::validate_markup(utils::string_view text, char** raw_text, std::
 	 * So only try to recover from broken ampersands, by simply replacing them
 	 * with the escaped version.
 	 */
-	semi_escaped = semi_escape_text(text.to_string());
+	semi_escaped = semi_escape_text(std::string(text));
 
 	/*
 	 * If at least one ampersand is replaced the semi-escaped string
@@ -856,27 +878,6 @@ bool pango_text::validate_markup(utils::string_view text, char** raw_text, std::
 	return true;
 }
 
-void pango_text::split_surface()
-{
-	auto text_parts = utils::vertical_split(text_);
-
-	PangoLayout* upper_layout = pango_layout_new(context_.get());
-	PangoLayout* lower_layout = pango_layout_new(context_.get());
-
-	set_markup(text_parts.first, *upper_layout);
-	set_markup(text_parts.second, *lower_layout);
-
-	copy_layout_properties(*layout_, *upper_layout);
-	copy_layout_properties(*layout_, *lower_layout);
-
-	sublayouts_.emplace_back(upper_layout, g_object_unref);
-	sublayouts_.emplace_back(lower_layout, g_object_unref);
-
-	// Freeing the old layout causes all text to use
-	// default line spacing in the future.
-	// layout_.reset(nullptr);
-}
-
 void pango_text::copy_layout_properties(PangoLayout& src, PangoLayout& dst)
 {
 	pango_layout_set_alignment(&dst, pango_layout_get_alignment(&src));
@@ -884,4 +885,46 @@ void pango_text::copy_layout_properties(PangoLayout& src, PangoLayout& dst)
 	pango_layout_set_ellipsize(&dst, pango_layout_get_ellipsize(&src));
 }
 
+std::vector<std::string> pango_text::get_lines() const
+{
+	this->recalculate();
+
+	PangoLayout* const layout = layout_.get();
+	std::vector<std::string> res;
+	int count = pango_layout_get_line_count(layout);
+
+	if(count < 1) {
+		return res;
+	}
+
+	using layout_iterator = std::unique_ptr<PangoLayoutIter, std::function<void(PangoLayoutIter*)>>;
+	layout_iterator i{pango_layout_get_iter(layout), pango_layout_iter_free};
+
+	res.reserve(count);
+
+	do {
+		PangoLayoutLine* ll = pango_layout_iter_get_line_readonly(i.get());
+		const char* begin = &pango_layout_get_text(layout)[ll->start_index];
+		res.emplace_back(begin, ll->length);
+	} while(pango_layout_iter_next_line(i.get()));
+
+	return res;
+}
+
+pango_text& get_text_renderer()
+{
+	static pango_text text_renderer;
+	return text_renderer;
+}
+
+int get_max_height(unsigned size, font::family_class fclass, pango_text::FONT_STYLE style)
+{
+	// Reset metrics to defaults
+	return get_text_renderer()
+		.set_family_class(fclass)
+		.set_font_style(style)
+		.set_font_size(size)
+		.get_max_glyph_height();
+}
+
 } // namespace font
