@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -18,7 +19,6 @@
 #include "color.hpp"
 #include "sdl/surface.hpp"
 #include "serialization/string_utils.hpp"
-#include "serialization/unicode_types.hpp"
 
 #include <pango/pango.h>
 #include <pango/pangocairo.h>
@@ -84,8 +84,20 @@ public:
 	/**
 	 * Returns the rendered text.
 	 *
-	 * Before rendering it tests whether a redraw is needed and if so it first
-	 * redraws the surface before returning it.
+	 * @param viewport Only this area needs to be drawn - the returned
+	 * surface's origin will correspond to viewport.x and viewport.y, the
+	 * width and height will be at least viewport.w and viewport.h (although
+	 * they may be larger).
+	 */
+	surface& render(const SDL_Rect& viewport);
+
+	/**
+	 * Equivalent to render(viewport), where the viewport's top-left is at
+	 * (0,0) and the area is large enough to contain the full text.
+	 *
+	 * The top-left of the viewport will be at (0,0), regardless of the values
+	 * of x and y.  If the x or y co-ordinates are non-zero, then x columns and
+	 * y rows of blank space are included in the amount of memory allocated.
 	 */
 	surface& render();
 
@@ -111,27 +123,6 @@ public:
 	 */
 	unsigned insert_text(const unsigned offset, const std::string& text);
 
-	/**
-	 * Inserts a unicode char.
-	 *
-	 * @param offset              The position to insert the char.
-	 * @param unicode             The character to insert.
-	 *
-	 * @returns                   True upon success, false otherwise.
-	 */
-	bool insert_unicode(const unsigned offset, ucs4::char_t unicode);
-
-	/**
-	 * Inserts unicode text.
-	 *
-	 * @param offset              The position to insert the text.
-	 * @param unicode             Vector with characters to insert.
-	 *
-	 * @returns                   The number of characters inserted.
-	 */
-	unsigned insert_unicode(
-		const unsigned offset, const ucs4::string& unicode);
-
 	/***** ***** ***** ***** Font flags ***** ***** ***** *****/
 
 	// NOTE: these values must be powers of 2 in order to be bit-unique
@@ -144,6 +135,15 @@ public:
 
 	/***** ***** ***** ***** Query details ***** ***** ***** *****/
 
+	/**
+	 * Returns the maximum glyph height of a font, in pixels.
+	 *
+	 * @returns                       The height of the tallest possible glyph for the selected
+	 *                                font. More specifically, the result is the sum of the maximum
+	 *                                ascent and descent lengths.
+	 */
+	int get_max_glyph_height() const;
+
 	/**
 	 * Gets the location for the cursor.
 	 *
@@ -163,13 +163,14 @@ public:
 	 * @returns                   The maximum length of the text. The length of text
 	 *                            should not exceed this value.
 	 */
-	size_t get_maximum_length() const;
+	std::size_t get_maximum_length() const;
 
 	/**
 	 * Gets the largest collection of characters, including the token at position,
 	 * and not including any characters from the delimiters set.
 	 *
 	 * @param position            The pixel position in the text area.
+	 * @param delimiters
 	 *
 	 * @returns                   The token containing position, and none of the
 	 * 			      delimiter characters. If position is out of bounds,
@@ -197,13 +198,24 @@ public:
 	 */
 	point get_column_line(const point& position) const;
 
+	/**
+	 * Retrieves a list of strings with contents for each rendered line.
+	 *
+	 * This method is not const because it requires rendering the text.
+	 *
+	 * @note This is only intended for renderer implementation details. This
+	 *       is a rather expensive function because it copies everything at
+	 *       least once.
+	 */
+	std::vector<std::string> get_lines() const;
+
 	/**
 	 * Gets the length of the text in bytes.
 	 *
 	 * The text set is UTF-8 so the length of the string might not be the length
 	 * of the text.
 	 */
-	size_t get_length() const { return length_; }
+	std::size_t get_length() const { return length_; }
 
 	/**
 	 * Sets the text to render.
@@ -241,7 +253,7 @@ public:
 
 	pango_text& set_alignment(const PangoAlignment alignment);
 
-	pango_text& set_maximum_length(const size_t maximum_length);
+	pango_text& set_maximum_length(const std::size_t maximum_length);
 
 	bool link_aware() const { return link_aware_; }
 
@@ -258,9 +270,6 @@ private:
 	std::unique_ptr<PangoLayout, std::function<void(void*)>> layout_;
 	mutable PangoRectangle rect_;
 
-	// Used if the text is too long to fit into a single Cairo surface.
-	std::vector<std::unique_ptr<PangoLayout, std::function<void(void*)>>> sublayouts_;
-
 	/** The SDL surface to render upon used as a cache. */
 	mutable surface surface_;
 
@@ -341,7 +350,7 @@ private:
 	PangoAlignment alignment_;
 
 	/** The maximum length of the text. */
-	size_t maximum_length_;
+	std::size_t maximum_length_;
 
 	/**
 	 * The text has two dirty states:
@@ -353,16 +362,12 @@ private:
 	mutable bool calculation_dirty_;
 
 	/** Length of the text. */
-	mutable size_t length_;
+	mutable std::size_t length_;
 
 	/**
 	 * Recalculates the text layout.
-	 *
-	 * When the text is recalculated the surface is dirtied.
-	 *
-	 * @param force               Recalculate even if not dirty?
 	 */
-	void recalculate(const bool force = false) const;
+	void recalculate() const;
 
 	/** Calculates surface size. */
 	PangoRectangle calculate_size(PangoLayout& layout) const;
@@ -370,18 +375,17 @@ private:
 	/** The dirty state of the surface. */
 	mutable bool surface_dirty_;
 
+	/** The area that's cached in surface_, which is the area that was rendered when surface_dirty_ was last set to false. */
+	SDL_Rect rendered_viewport_;
+
 	/**
 	 * Renders the text.
 	 *
 	 * It will do a recalculation first so no need to call both.
-	 *
-	 * @param force               Render even if not dirty? This parameter is
-	 *                            also send to recalculate().
 	 */
-	void rerender(const bool force = false);
+	void rerender(const SDL_Rect& viewport);
 
-	void render(PangoLayout& layout, const PangoRectangle& rect,
-		const size_t surface_buffer_offset, const unsigned stride);
+	void render(PangoLayout& layout, const SDL_Rect& viewport, const unsigned stride);
 
 	/**
 	 * Buffer to store the image on.
@@ -392,20 +396,6 @@ private:
 	 */
 	mutable std::vector<uint8_t> surface_buffer_;
 
-	/**
-	 * Creates a new buffer.
-	 *
-	 * If needed frees the other surface and then creates a new buffer and
-	 * initializes the entire buffer with values 0.
-	 *
-	 * NOTE even though we're clearly modifying function we don't change the
-	 * state of the object. The const is needed so other functions can also be
-	 * marked const (those also don't change the state of the object.
-	 *
-	 * @param size                The required size of the buffer.
-	 */
-	void create_surface_buffer(const size_t size) const;
-
 	/**
 	 * Sets the markup'ed text.
 	 *
@@ -413,37 +403,41 @@ private:
 	 * a bit harder to recover from the errors and still set the markup.
 	 *
 	 * @param text                The text to set as markup.
+	 * @param layout
 	 *
 	 * @returns                   Whether the markup was set or an
 	 *                            unrecoverable error occurred and the text is
 	 *                            set as plain text with an error message.
 	 */
-	bool set_markup(utils::string_view text, PangoLayout& layout);
+	bool set_markup(std::string_view text, PangoLayout& layout);
 
-	bool validate_markup(utils::string_view text, char** raw_text, std::string& semi_escaped) const;
-
-	/** Splits the text to two Cairo surfaces.
-	 *
-	 * The implementation isn't recursive: the function only splits the text once.
-	 * As a result, it only doubles the maximum surface height to 64,000 pixels
-	 * or so.
-	 * The reason for this is that a recursive implementation would be more complex
-	 * and it's unnecessary for now, as the longest surface in the game
-	 * (end credits) is only about 40,000 pixels high with the default_large widget
-	 * definition.
-	 * If we need even larger surfaces in the future, the implementation can be made
-	 * recursive.
-	 */
-	void split_surface();
-
-	bool is_surface_split() const
-	{
-		return sublayouts_.size() > 0;
-	}
+	bool validate_markup(std::string_view text, char** raw_text, std::string& semi_escaped) const;
 
 	static void copy_layout_properties(PangoLayout& src, PangoLayout& dst);
 
-	std::string format_links(const std::string& text) const;
+	std::string format_links(std::string_view text) const;
 };
 
+/**
+ * Returns a reference to a static pango_text object.
+ *
+ * Since the class is essentially a render pipeline, there's no need for individual
+ * areas of the game to own their own renderers. Not to mention it isn't a trivial
+ * class; constructing one is likely to be expensive.
+ */
+pango_text& get_text_renderer();
+
+/**
+ * Returns the maximum glyph height of a font, in pixels.
+ *
+ * @param size                    Desired font size in pixels.
+ * @param fclass                  Font family to use for measurement.
+ * @param style                   Font style to select the correct variant for measurement.
+ *
+ * @returns                       The height of the tallest possible glyph for the selected
+ *                                font. More specifically, the result is the sum of the maximum
+ *                                ascent and descent lengths.
+ */
+int get_max_height(unsigned size, font::family_class fclass = font::FONT_SANS_SERIF, pango_text::FONT_STYLE style = pango_text::STYLE_NORMAL);
+
 } // namespace font
