@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "formula/callable_objects.hpp"
@@ -22,7 +23,13 @@
 #include "units/attack_type.hpp"
 #include "units/formula_manager.hpp"
 #include "units/unit.hpp"
+#include "units/types.hpp"
 #include "log.hpp"
+#include "recall_list_manager.hpp"
+#include "deprecation.hpp"
+#include "game_board.hpp"
+#include "game_version.hpp"
+#include "resources.hpp"
 
 static lg::log_domain log_scripting_formula("scripting/formula");
 #define LOG_SF LOG_STREAM(info, log_scripting_formula)
@@ -183,6 +190,11 @@ variant unit_callable::get_value(const std::string& key) const
 		}
 
 		return variant(std::make_shared<location_callable>(loc_));
+	} else if(key == "terrain") {
+		if(loc_ == map_location::null_location()) {
+			return variant();
+		}
+		return variant(std::make_shared<terrain_callable>(*resources::gameboard, loc_));
 	} else if(key == "id") {
 		return variant(u_.id());
 	} else if(key == "type") {
@@ -232,7 +244,10 @@ variant unit_callable::get_value(const std::string& key) const
 	} else if(key == "states" || key == "status") {
 		return formula_callable::convert_set(u_.get_states());
 	} else if(key == "side") {
+		deprecated_message("unit.side", DEP_LEVEL::FOR_REMOVAL, version_info("1.17"), "This returns 0 for side 1 etc and should not be used. Use side_number instead.");
 		return variant(u_.side()-1);
+	} else if(key == "side_number") {
+		return variant(u_.side());
 	} else if(key == "cost") {
 		return variant(u_.cost());
 	} else if(key == "upkeep") {
@@ -260,6 +275,35 @@ variant unit_callable::get_value(const std::string& key) const
 		return variant(u_.alignment().to_string());
 	} else if(key == "facing") {
 		return variant(map_location::write_direction(u_.facing()));
+	} else if(key == "resistance" || key == "movement_cost" || key == "vision_cost" || key == "jamming_cost" || key == "defense") {
+		const auto& mt = u_.movement_type();
+		config cfg;
+		bool needs_flip = false;
+		if(key == "resistance") {
+			mt.get_resistances().write(cfg);
+			needs_flip = true;
+		} else if(key == "movement_cost") {
+			mt.get_movement().write(cfg);
+		} else if(key == "vision_cost") {
+			mt.get_vision().write(cfg);
+		} else if(key == "jamming_cost") {
+			mt.get_vision().write(cfg);
+		} else if(key == "defense") {
+			mt.get_defense().write(cfg);
+			needs_flip = true;
+		}
+		std::map<variant, variant> res;
+		for(const auto& p : cfg.attribute_range()) {
+			int val = p.second;
+			if(needs_flip) {
+				val = 100 - val;
+			}
+			res.emplace(variant(p.first), variant(val));
+		}
+
+		return variant(res);
+	} else if(key == "flying") {
+		return variant(u_.is_flying());
 	} else if(key == "vars") {
 		if(u_.formula_manager().formula_vars()) {
 			return variant(u_.formula_manager().formula_vars());
@@ -283,6 +327,7 @@ void unit_callable::get_inputs(formula_input_vector& inputs) const
 	add_input(inputs, "x");
 	add_input(inputs, "y");
 	add_input(inputs, "loc");
+	add_input(inputs, "terrain");
 	add_input(inputs, "id");
 	add_input(inputs, "type");
 	add_input(inputs, "name");
@@ -300,7 +345,7 @@ void unit_callable::get_inputs(formula_input_vector& inputs) const
 	add_input(inputs, "max_moves");
 	add_input(inputs, "attacks_left");
 	add_input(inputs, "max_attacks");
-	add_input(inputs, "side");
+	add_input(inputs, "side_number");
 	add_input(inputs, "extra_recruit");
 	add_input(inputs, "advances_to");
 	add_input(inputs, "status");
@@ -317,6 +362,12 @@ void unit_callable::get_inputs(formula_input_vector& inputs) const
 	add_input(inputs, "zoc");
 	add_input(inputs, "alignment");
 	add_input(inputs, "facing");
+	add_input(inputs, "resistance");
+	add_input(inputs, "movement_cost");
+	add_input(inputs, "vision_cost");
+	add_input(inputs, "jamming_cost");
+	add_input(inputs, "defense");
+	add_input(inputs, "flying");
 	add_input(inputs, "vars");
 	add_input(inputs, "wml_vars");
 }
@@ -367,6 +418,10 @@ variant unit_type_callable::get_value(const std::string& key) const
 		return variant(u_.movement());
 	} else if(key == "unpoisonable") {
 		return variant(u_.musthave_status("unpoisonable"));
+	} else if(key == "unslowable") {
+		return variant(u_.musthave_status("unslowable"));
+	} else if(key == "unpetrifiable") {
+		return variant(u_.musthave_status("unpetrifiable"));
 	} else if(key == "undrainable") {
 		return variant(u_.musthave_status("undrainable"));
 	} else if(key == "unplagueable") {
@@ -411,7 +466,10 @@ int unit_type_callable::do_compare(const formula_callable* callable) const
 	return u_.id().compare(u_callable->u_.id());
 }
 
-struct fai_variant_visitor : public boost::static_visitor<variant>
+struct fai_variant_visitor
+#ifdef USING_BOOST_VARIANT
+	: public boost::static_visitor<variant>
+#endif
 {
 	variant operator()(bool b) const               { return variant(b ? 1 : 0); }
 	variant operator()(int i) const                { return variant(i); }
@@ -422,7 +480,7 @@ struct fai_variant_visitor : public boost::static_visitor<variant>
 	// (or should we assume that such strings will be translatable?).
 	variant operator()(const std::string& s) const { return variant(s); }
 	variant operator()(const t_string& s) const    { return variant(s.str()); }
-	variant operator()(boost::blank) const         { return variant(); }
+	variant operator()(utils::monostate) const         { return variant(); }
 };
 
 variant config_callable::get_value(const std::string& key) const
@@ -531,6 +589,9 @@ variant terrain_callable::get_value(const std::string& key) const
 	} else if(key == "healing") {
 		return variant(t_.gives_healing());
 	} else if(key == "owner") {
+		deprecated_message("terrain.owner", DEP_LEVEL::FOR_REMOVAL, version_info("1.17"), "This returns 0 for side 1 etc and should not be used. Use owner_side instead.");
+		return variant(owner_ - 1);
+	} else if(key == "owner_side") {
 		return variant(owner_);
 	}
 
@@ -552,7 +613,7 @@ void terrain_callable::get_inputs(formula_input_vector& inputs) const
 	add_input(inputs, "castle");
 	add_input(inputs, "keep");
 	add_input(inputs, "healing");
-	add_input(inputs, "owner");
+	add_input(inputs, "owner_side");
 }
 
 int terrain_callable::do_compare(const formula_callable* callable) const
@@ -572,8 +633,6 @@ const gamemap& gamemap_callable::get_gamemap() const {
 
 void gamemap_callable::get_inputs(formula_input_vector& inputs) const
 {
-	add_input(inputs, "gamemap");
-	add_input(inputs, "terrain");
 	add_input(inputs, "w");
 	add_input(inputs, "h");
 }
@@ -592,6 +651,19 @@ variant gamemap_callable::get_value(const std::string& key) const
 			}
 		}
 
+		return variant(vars);
+	} else if(key == "gamemap") {
+		int w = get_gamemap().w();
+		int h = get_gamemap().h();
+
+		std::map<variant, variant> vars;
+		for(int i = 0; i < w; i++) {
+			for(int j = 0; j < h; j++) {
+				const map_location loc(i, j);
+				vars.emplace(std::make_shared<location_callable>(loc), std::make_shared<terrain_callable>(board_, loc));
+			}
+		}
+
 		return variant(vars);
 	} else if(key == "w") {
 		return variant(get_gamemap().w());
@@ -604,7 +676,7 @@ variant gamemap_callable::get_value(const std::string& key) const
 
 void team_callable::get_inputs(formula_input_vector& inputs) const
 {
-	add_input(inputs, "side");
+	add_input(inputs, "side_number");
 	add_input(inputs, "id");
 	add_input(inputs, "gold");
 	add_input(inputs, "start_gold");
@@ -637,6 +709,9 @@ void team_callable::get_inputs(formula_input_vector& inputs) const
 variant team_callable::get_value(const std::string& key) const
 {
 	if(key == "side") {
+		deprecated_message("team.side", DEP_LEVEL::INDEFINITE, version_info("1.17"), "Use side_number instead.");
+		return variant(team_.side());
+	} else if(key == "side_number") {
 		return variant(team_.side());
 	} else if(key == "id") {
 		return variant(team_.save_id());
@@ -679,7 +754,7 @@ variant team_callable::get_value(const std::string& key) const
 	} else if(key == "share_vision") {
 		return variant(team_.share_vision().to_string());
 	} else if(key == "carryover_bonus") {
-		return variant(team_.carryover_bonus());
+		return variant(team_.carryover_bonus(), variant::DECIMAL_VARIANT);
 	} else if(key == "carryover_percentage") {
 		return variant(team_.carryover_percentage());
 	} else if(key == "carryover_add") {
@@ -689,7 +764,12 @@ variant team_callable::get_value(const std::string& key) const
 		for(const auto& recruit : team_.recruits()) {
 			result.emplace_back(recruit);
 		}
-
+		return variant(result);
+	} else if(key == "recall") {
+		std::vector<variant> result;
+		for(const auto& u : team_.recall_list()) {
+			result.push_back(std::make_shared<unit_callable>(*u));
+		}
 		return variant(result);
 	} else if(key == "wml_vars") {
 		return variant(std::make_shared<config_callable>(team_.variables()));
