@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "formula/function.hpp"
@@ -19,6 +20,7 @@
 #include "formula/debugger.hpp"
 #include "game_config.hpp"
 #include "game_display.hpp"
+#include "global.hpp"
 #include "log.hpp"
 
 #include <boost/math/constants/constants.hpp>
@@ -116,7 +118,7 @@ DEFINE_WFL_FUNCTION(dir, 1, 1)
 	formula_input_vector inputs = callable->inputs();
 
 	std::vector<variant> res;
-	for(size_t i = 0; i < inputs.size(); ++i) {
+	for(std::size_t i = 0; i < inputs.size(); ++i) {
 		const formula_input& input = inputs[i];
 		res.emplace_back(input.name);
 	}
@@ -126,7 +128,7 @@ DEFINE_WFL_FUNCTION(dir, 1, 1)
 
 DEFINE_WFL_FUNCTION(if, 2, -1)
 {
-	for(size_t n = 0; n < args().size() - 1; n += 2) {
+	for(std::size_t n = 0; n < args().size() - 1; n += 2) {
 		if(args()[n]->evaluate(variables, fdb).as_bool()) {
 			return args()[n + 1]->evaluate(variables, fdb);
 		}
@@ -143,7 +145,7 @@ DEFINE_WFL_FUNCTION(switch, 3, -1)
 {
 	variant var = args()[0]->evaluate(variables, fdb);
 
-	for(size_t n = 1; n < args().size() - 1; n += 2) {
+	for(std::size_t n = 1; n < args().size() - 1; n += 2) {
 		variant val = args()[n]->evaluate(variables, fdb);
 
 		if(val == var) {
@@ -181,7 +183,7 @@ DEFINE_WFL_FUNCTION(min, 1, -1)
 		res = *std::min_element(res.begin(), res.end());
 	}
 
-	for(size_t n = 1; n < args().size(); ++n) {
+	for(std::size_t n = 1; n < args().size(); ++n) {
 		variant v = args()[n]->evaluate(variables, fdb);
 
 		if(v.is_list()) {
@@ -211,7 +213,7 @@ DEFINE_WFL_FUNCTION(max, 1, -1)
 		res = *std::max_element(res.begin(), res.end());
 	}
 
-	for(size_t n = 1; n < args().size(); ++n) {
+	for(std::size_t n = 1; n < args().size(); ++n) {
 		variant v = args()[n]->evaluate(variables, fdb);
 
 		if(v.is_list()) {
@@ -270,9 +272,9 @@ DEFINE_WFL_FUNCTION(debug_print, 1, 2)
 
 		LOG_SF << str1 << std::endl;
 
-		if(game_config::debug) {
+		if(game_config::debug && game_display::get_singleton()) {
 			game_display::get_singleton()->get_chat_manager().add_chat_message(
-				time(nullptr), "WFL", 0, str1, events::chat_handler::MESSAGE_PUBLIC, false);
+				std::time(nullptr), "WFL", 0, str1, events::chat_handler::MESSAGE_PUBLIC, false);
 		}
 
 		return var1;
@@ -286,7 +288,7 @@ DEFINE_WFL_FUNCTION(debug_print, 1, 2)
 
 		if(game_config::debug && game_display::get_singleton()) {
 			game_display::get_singleton()->get_chat_manager().add_chat_message(
-				time(nullptr), str1, 0, str2, events::chat_handler::MESSAGE_PUBLIC, false);
+				std::time(nullptr), str1, 0, str2, events::chat_handler::MESSAGE_PUBLIC, false);
 		}
 
 		return var2;
@@ -319,7 +321,7 @@ DEFINE_WFL_FUNCTION(debug_profile, 1, 2)
 
 	if(game_config::debug && game_display::get_singleton()) {
 		game_display::get_singleton()->get_chat_manager().add_chat_message(
-			time(nullptr), speaker, 0, str.str(), events::chat_handler::MESSAGE_PUBLIC, false);
+			std::time(nullptr), speaker, 0, str.str(), events::chat_handler::MESSAGE_PUBLIC, false);
 	}
 
 	return value;
@@ -361,7 +363,7 @@ DEFINE_WFL_FUNCTION(tomap, 1, 2)
 			return variant();
 		}
 
-		for(size_t i = 0; i < var_1.num_elements(); ++i) {
+		for(std::size_t i = 0; i < var_1.num_elements(); ++i) {
 			tmp[var_1[i]] = var_2[i];
 		}
 	} else {
@@ -395,7 +397,7 @@ DEFINE_WFL_FUNCTION(substring, 2, 3)
 			offset = 0;
 		}
 	} else {
-		if(static_cast<size_t>(offset) >= result.size()) {
+		if(static_cast<std::size_t>(offset) >= result.size()) {
 			return variant(std::string());
 		}
 	}
@@ -427,7 +429,7 @@ DEFINE_WFL_FUNCTION(replace, 3, 4)
 			offset = 0;
 		}
 	} else {
-		if(static_cast<size_t>(offset) >= result.size()) {
+		if(static_cast<std::size_t>(offset) >= result.size()) {
 			return variant(result);
 		}
 	}
@@ -458,7 +460,7 @@ DEFINE_WFL_FUNCTION(insert, 3, 3)
 		if(offset < 0) {
 			offset = 0;
 		}
-	} else if(static_cast<size_t>(offset) >= result.size()) {
+	} else if(static_cast<std::size_t>(offset) >= result.size()) {
 		return variant(result + insert);
 	}
 
@@ -497,21 +499,21 @@ DEFINE_WFL_FUNCTION(str_lower, 1, 1)
 DEFINE_WFL_FUNCTION(sin, 1, 1)
 {
 	const double angle = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = sin(angle * pi<double>() / 180.0);
+	const double result = std::sin(angle * pi<double>() / 180.0);
 	return variant(result, variant::DECIMAL_VARIANT);
 }
 
 DEFINE_WFL_FUNCTION(cos, 1, 1)
 {
 	const double angle = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = cos(angle * pi<double>() / 180.0);
+	const double result = std::cos(angle * pi<double>() / 180.0);
 	return variant(result, variant::DECIMAL_VARIANT);
 }
 
 DEFINE_WFL_FUNCTION(tan, 1, 1)
 {
 	const double angle = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = tan(angle * pi<double>() / 180.0);
+	const double result = std::tan(angle * pi<double>() / 180.0);
 	if(std::isnan(result) || result <= INT_MIN || result >= INT_MAX) {
 		return variant();
 	}
@@ -522,7 +524,7 @@ DEFINE_WFL_FUNCTION(tan, 1, 1)
 DEFINE_WFL_FUNCTION(asin, 1, 1)
 {
 	const double num = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = asin(num) * 180.0 / pi<double>();
+	const double result = std::asin(num) * 180.0 / pi<double>();
 	if(std::isnan(result)) {
 		return variant();
 	}
@@ -533,7 +535,7 @@ DEFINE_WFL_FUNCTION(asin, 1, 1)
 DEFINE_WFL_FUNCTION(acos, 1, 1)
 {
 	const double num = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = acos(num) * 180.0 / pi<double>();
+	const double result = std::acos(num) * 180.0 / pi<double>();
 	if(std::isnan(result)) {
 		return variant();
 	}
@@ -544,14 +546,14 @@ DEFINE_WFL_FUNCTION(acos, 1, 1)
 DEFINE_WFL_FUNCTION(atan, 1, 1)
 {
 	const double num = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = atan(num) * 180.0 / pi<double>();
+	const double result = std::atan(num) * 180.0 / pi<double>();
 	return variant(result, variant::DECIMAL_VARIANT);
 }
 
 DEFINE_WFL_FUNCTION(sqrt, 1, 1)
 {
 	const double num = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = sqrt(num);
+	const double result = std::sqrt(num);
 	if(std::isnan(result)) {
 		return variant();
 	}
@@ -562,7 +564,7 @@ DEFINE_WFL_FUNCTION(sqrt, 1, 1)
 DEFINE_WFL_FUNCTION(cbrt, 1, 1)
 {
 	const double num = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = num < 0 ? -pow(-num, 1.0 / 3.0) : pow(num, 1.0 / 3.0);
+	const double result = num < 0 ? -std::pow(-num, 1.0 / 3.0) : std::pow(num, 1.0 / 3.0);
 	return variant(result, variant::DECIMAL_VARIANT);
 }
 
@@ -570,7 +572,7 @@ DEFINE_WFL_FUNCTION(root, 2, 2)
 {
 	const double base = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
 	const double root = args()[1]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = base < 0 && fmod(root, 2) == 1 ? -pow(-base, 1.0 / root) : pow(base, 1.0 / root);
+	const double result = base < 0 && std::fmod(root, 2) == 1 ? -std::pow(-base, 1.0 / root) : std::pow(base, 1.0 / root);
 	if(std::isnan(result)) {
 		return variant();
 	}
@@ -582,7 +584,7 @@ DEFINE_WFL_FUNCTION(log, 1, 2)
 {
 	const double num = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
 	if(args().size() == 1) {
-		const double result = log(num);
+		const double result = std::log(num);
 		if(std::isnan(result)) {
 			return variant();
 		}
@@ -591,7 +593,7 @@ DEFINE_WFL_FUNCTION(log, 1, 2)
 	}
 
 	const double base = args()[1]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = log(num) / log(base);
+	const double result = std::log(num) / std::log(base);
 	if(std::isnan(result)) {
 		return variant();
 	}
@@ -602,7 +604,7 @@ DEFINE_WFL_FUNCTION(log, 1, 2)
 DEFINE_WFL_FUNCTION(exp, 1, 1)
 {
 	const double num = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	const double result = exp(num);
+	const double result = std::exp(num);
 	if(result == 0 || result >= INT_MAX) {
 		// These are range errors rather than NaNs,
 		// but I figure it's better than returning INT_MIN.
@@ -623,7 +625,7 @@ DEFINE_WFL_FUNCTION(hypot, 2, 2)
 {
 	const double x = args()[0]->evaluate(variables, fdb).as_decimal() / 1000.0;
 	const double y = args()[1]->evaluate(variables, fdb).as_decimal() / 1000.0;
-	return variant(hypot(x, y), variant::DECIMAL_VARIANT);
+	return variant(std::hypot(x, y), variant::DECIMAL_VARIANT);
 }
 
 DEFINE_WFL_FUNCTION(index_of, 2, 2)
@@ -631,7 +633,7 @@ DEFINE_WFL_FUNCTION(index_of, 2, 2)
 	const variant value = args()[0]->evaluate(variables, fdb);
 	const variant list = args()[1]->evaluate(variables, fdb);
 
-	for(size_t i = 0; i < list.num_elements(); ++i) {
+	for(std::size_t i = 0; i < list.num_elements(); ++i) {
 		if(list[i] == value) {
 			return variant(i);
 		}
@@ -683,7 +685,7 @@ DEFINE_WFL_FUNCTION(wave, 1, 1)
 {
 	const int value = args()[0]->evaluate(variables, fdb).as_int() % 1000;
 	const double angle = 2.0 * pi<double>() * (static_cast<double>(value) / 1000.0);
-	return variant(static_cast<int>(sin(angle) * 1000.0));
+	return variant(static_cast<int>(std::sin(angle) * 1000.0));
 }
 
 namespace
@@ -736,7 +738,7 @@ DEFINE_WFL_FUNCTION(sort, 1, 2)
 	std::vector<variant> vars;
 	vars.reserve(list.num_elements());
 
-	for(size_t n = 0; n != list.num_elements(); ++n) {
+	for(std::size_t n = 0; n != list.num_elements(); ++n) {
 		vars.push_back(list[n]);
 	}
 
@@ -781,7 +783,7 @@ DEFINE_WFL_FUNCTION(find_string, 2, 2)
 	const std::string str = args()[0]->evaluate(variables, fdb).as_string();
 	const std::string key = args()[1]->evaluate(variables, fdb).as_string();
 
-	size_t pos = str.find(key);
+	std::size_t pos = str.find(key);
 	return variant(static_cast<int>(pos));
 }
 
@@ -922,7 +924,7 @@ namespace
 {
 struct indexer
 {
-	explicit indexer(size_t i)
+	explicit indexer(std::size_t i)
 		: i(i)
 	{
 	}
@@ -936,7 +938,7 @@ struct indexer
 		}
 	}
 
-	size_t i;
+	std::size_t i;
 };
 
 /** @todo: replace with lambda? */
@@ -976,10 +978,10 @@ DEFINE_WFL_FUNCTION(zip, 1, -1)
 
 	// So basically this does [[a,b,c],[d,e,f],[x,y,z]] -> [[a,d,x],[b,e,y],[c,f,z]]
 	// Or [[a,b,c,d],[x,y,z]] -> [[a,x],[b,y],[c,z],[d,null()]]
-	size_t max_i = std::max_element(input.begin(), input.end(), comparator())->num_elements();
+	std::size_t max_i = std::max_element(input.begin(), input.end(), comparator())->num_elements();
 	output.reserve(max_i);
 
-	for(size_t i = 0; i < max_i; i++) {
+	for(std::size_t i = 0; i < max_i; i++) {
 		std::vector<variant> elem(input.size());
 		std::transform(input.begin(), input.end(), elem.begin(), indexer(i));
 		output.emplace_back(elem);
@@ -1042,7 +1044,7 @@ DEFINE_WFL_FUNCTION(sum, 1, 2)
 		}
 	}
 
-	for(size_t n = 0; n != items.num_elements(); ++n) {
+	for(std::size_t n = 0; n != items.num_elements(); ++n) {
 		res = res + items[n];
 	}
 
@@ -1103,7 +1105,7 @@ DEFINE_WFL_FUNCTION(size, 1, 1)
 DEFINE_WFL_FUNCTION(null, 0, -1)
 {
 	if(!args().empty()) {
-		for(size_t i = 0; i < args().size(); ++i) {
+		for(std::size_t i = 0; i < args().size(); ++i) {
 			args()[i]->evaluate(variables, fdb);
 		}
 	}
@@ -1232,12 +1234,9 @@ DEFINE_WFL_FUNCTION(adjacent_locs, 1, 1)
 		.convert_to<location_callable>()
 		->loc();
 
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(loc, adj.data());
-
 	std::vector<variant> v;
-	for(unsigned n = 0; n < adj.size(); ++n) {
-		v.emplace_back(std::make_shared<location_callable>(adj[n]));
+	for(const map_location& adj : get_adjacent_tiles(loc)) {
+		v.emplace_back(std::make_shared<location_callable>(adj));
 	}
 
 	return variant(v);
@@ -1392,7 +1391,7 @@ variant formula_function_expression::execute(const formula_callable& variables,
 	const int begin_time = SDL_GetTicks();
 	map_formula_callable callable;
 
-	for(size_t n = 0; n != arg_names_.size(); ++n) {
+	for(std::size_t n = 0; n != arg_names_.size(); ++n) {
 		variant var = args()[n]->evaluate(variables, fdb);
 		callable.add(arg_names_[n], var);
 
@@ -1405,7 +1404,7 @@ variant formula_function_expression::execute(const formula_callable& variables,
 		if(!precondition_->evaluate(callable, fdb).as_bool()) {
 			DBG_NG << "FAILED function precondition for function '" << formula_->str() << "' with arguments: ";
 
-			for(size_t n = 0; n != arg_names_.size(); ++n) {
+			for(std::size_t n = 0; n != arg_names_.size(); ++n) {
 				DBG_NG << "  arg " << (n + 1) << ": " << args()[n]->evaluate(variables, fdb).to_debug_string() << "\n";
 			}
 		}
