@@ -1,16 +1,16 @@
-
 /*
-   Copyright (C) 2008 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -108,11 +108,11 @@ public:
 		: formula_expression(name)
 		, args_(args)
 	{
-		if(min_args >= 0 && args_.size() < static_cast<size_t>(min_args)) {
+		if(min_args >= 0 && args_.size() < static_cast<std::size_t>(min_args)) {
 			throw formula_error("Too few arguments", "", "", 0);
 		}
 
-		if(max_args >= 0 && args_.size() > static_cast<size_t>(max_args)) {
+		if(max_args >= 0 && args_.size() > static_cast<std::size_t>(max_args)) {
 			throw formula_error("Too many arguments", "", "", 0);
 		}
 	}
