@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "formula/tokenizer.hpp"
@@ -24,7 +25,7 @@ namespace tokenizer
 
 namespace {
 
-void raise_exception(iterator& i1, iterator i2, std::string str) {
+[[noreturn]] void raise_exception(iterator& i1, iterator i2, std::string str) {
 	std::ostringstream expr;
 	while( (i1 != i2) && (*i1 != '\n') ) {
 		if( (*i1 != '\t') )
@@ -283,8 +284,7 @@ token get_token(iterator& i1, const iterator i2) {
 				raise_exception(it, i2, std::string() );
 		}
 	}
-	raise_exception(it, i2, std::string() );
-	return token();
+	raise_exception(it, i2, std::string());
 }
 
 }
