@@ -1,26 +1,28 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
 #include "display_context.hpp"
 #include "team.hpp"
+#include "terrain/translation.hpp"
 #include "terrain/type_data.hpp"
 #include "units/map.hpp"
 #include "units/id.hpp"
+#include <optional>
 
-#include <boost/optional.hpp>
 #include <set>
 #include <vector>
 
@@ -46,10 +48,8 @@ namespace events {
  * code at all points in the engine which modify the relevant data.
  *
  **/
-
 class game_board : public display_context
 {
-
 	std::vector<team> teams_;
 	std::vector<std::string> labels_;
 
@@ -57,13 +57,6 @@ class game_board : public display_context
 	n_unit::id_manager unit_id_manager_;
 	unit_map units_;
 
-	//TODO: Remove these when we have refactored enough to make it possible.
-	friend class play_controller;
-	friend class events::mouse_handler;
-	friend class events::menu_handler;
-	friend class game_state;
-	friend class game_lua_kernel;
-
 	/**
 	 * Temporary unit move structs:
 	 *
@@ -86,7 +79,7 @@ public:
 	n_unit::id_manager& unit_id_manager() { return unit_id_manager_; }
 	// Constructors, trivial dtor, and const accessors
 
-	game_board(const ter_data_cache & tdata, const config & level);
+	game_board(const config& level);
 	virtual ~game_board();
 
 	virtual const std::vector<team>& teams() const override
@@ -111,6 +104,11 @@ public:
 		return *map_;
 	}
 
+	gamemap& map()
+	{
+		return *map_;
+	}
+
 	virtual const unit_map& units() const override
 	{
 		return units_;
@@ -155,30 +153,29 @@ public:
 	// Manipulator from playturn
 
 	void side_drop_to (int side_num, team::CONTROLLER ctrl, team::PROXY_CONTROLLER proxy = team::PROXY_CONTROLLER::PROXY_HUMAN);
-	void side_change_controller (int side_num, bool is_local, const std::string& pname = "");
+	void side_change_controller (int side_num, bool is_local, const std::string& pname, const std::string& controller_type);
 
 	// Manipulator from actionwml
 
 	bool try_add_unit_to_recall_list(const map_location& loc, const unit_ptr u);
-	boost::optional<std::string> replace_map (const gamemap & r);
-	void overlay_map (const gamemap & o, const config & cfg, map_location loc);
+	std::optional<std::string> replace_map(const gamemap & r);
 
-	bool change_terrain(const map_location &loc, const std::string &t,
-	                    const std::string & mode, bool replace_if_failed); //used only by lua
+	bool change_terrain(const map_location &loc, const std::string &t, const std::string & mode, bool replace_if_failed); //used only by lua and debug commands
+	bool change_terrain(const map_location &loc, const t_translation::terrain_code &t, terrain_type_data::merge_mode& mode, bool replace_if_failed); //used only by lua and debug commands
 
 	// Global accessor from unit.hpp
 
 	unit_map::iterator find_visible_unit(const map_location &loc, const team& current_team, bool see_all = false);
-	unit_map::iterator find_visible_unit(const map_location & loc, size_t team, bool see_all = false) { return find_visible_unit(loc, teams_[team], see_all); }
+	unit_map::iterator find_visible_unit(const map_location & loc, std::size_t team, bool see_all = false) { return find_visible_unit(loc, teams_[team], see_all); }
 	bool has_visible_unit (const map_location & loc, const team & team, bool see_all = false) const;
-	bool has_visible_unit (const map_location & loc, size_t team, bool see_all = false) const { return has_visible_unit(loc, teams_[team], see_all); }
+	bool has_visible_unit (const map_location & loc, std::size_t team, bool see_all = false) const { return has_visible_unit(loc, teams_[team], see_all); }
 
 	unit* get_visible_unit(const map_location &loc, const team &current_team, bool see_all = false); //TODO: can this not return a pointer?
 
 	// Wrapped functions from unit_map. These should ultimately provide notification to observers, pathfinding.
 
 	unit_map::iterator find_unit(const map_location & loc) { return units_.find(loc); }
-	/// Calculates whether a team is defeated
+	/** Calculates whether a team is defeated */
 	bool team_is_defeated(const team& t) const;
 };
 
