@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -17,10 +18,11 @@
 #include "utils/make_enum.hpp"
 
 #include <vector>
+#include <set>
 
 class config;
 
-/// The default difficulty setting for campaigns.
+/** The default difficulty setting for campaigns. */
 extern const std::string DEFAULT_DIFFICULTY;
 
 //meta information of the game
@@ -30,10 +32,6 @@ public:
 	game_classification() = default;
 	explicit game_classification(const config& cfg);
 
-	config to_config() const;
-	std::string get_tagname() const;
-	bool is_normal_mp_game() const;
-
 	std::string label;                               /**< Name of the game (e.g. name of save file). */
 	std::string version;                             /**< Version game was created with. */
 	MAKE_ENUM (CAMPAIGN_TYPE,			 /**< Type of the game - campaign, multiplayer etc. */
@@ -49,6 +47,9 @@ public:
 	std::string era_define;                          /**< If there is a define the era uses to customize data */
 	std::vector<std::string> mod_defines;            /**< If there are defines the modifications use to customize data */
 
+	std::vector<std::string> active_mods;
+	std::string era_id;
+
 	std::string campaign;                            /**< The id of the campaign being played */
 	std::string campaign_name;                       /**< The name of the campaign being played. */
 
@@ -59,4 +60,33 @@ public:
 	std::string difficulty = DEFAULT_DIFFICULTY; /**< The difficulty level the game is being played on. */
 	std::string random_mode = "";
 	bool oos_debug = false;
+
+	config to_config() const;
+	std::string get_tagname() const;
+	std::set<std::string> active_addons(const std::string& scenario_id) const;
+
+	bool is_normal_mp_game() const
+	{
+		return is_multiplayer() && campaign.empty();
+	}
+
+	bool is_scenario() const
+	{
+		return campaign_type == CAMPAIGN_TYPE::SCENARIO;
+	}
+
+	bool is_multiplayer() const
+	{
+		return campaign_type == CAMPAIGN_TYPE::MULTIPLAYER;
+	}
+
+	bool is_test() const
+	{
+		return campaign_type == CAMPAIGN_TYPE::TEST;
+	}
+
+	bool is_tutorial() const
+	{
+		return campaign_type == CAMPAIGN_TYPE::TUTORIAL;
+	}
 };
