@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "game_config.hpp"
@@ -18,14 +19,10 @@
 #include "config.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
-#include "utils/general.hpp"
 #include "utils/math.hpp"
 #include "game_version.hpp"
 #include "wesconfig.h"
 #include "serialization/string_utils.hpp"
-#ifdef LOAD_REVISION
-#include "revision.h"
-#endif
 
 static lg::log_domain log_engine("engine");
 #define LOG_NG LOG_STREAM(info, log_engine)
@@ -33,24 +30,9 @@ static lg::log_domain log_engine("engine");
 
 namespace game_config
 {
-
 //
-// Path and revision info
+// Path info
 //
-const std::string version = VERSION;
-
-const version_info wesnoth_version(VERSION);
-const version_info min_savegame_version(MIN_SAVEGAME_VERSION);
-const version_info test_version("test");
-
-#ifdef REVISION
-const std::string revision = VERSION " (" REVISION ")";
-#elif defined(VCS_SHORT_HASH) && defined(VCS_WC_MODIFIED)
-const std::string revision = std::string(VERSION) + " (" + VCS_SHORT_HASH + (VCS_WC_MODIFIED ? "-Modified" : "-Clean") + ")";
-#else
-const std::string revision = VERSION;
-#endif
-
 #ifdef WESNOTH_PATH
 std::string path = WESNOTH_PATH;
 #else
@@ -62,6 +44,7 @@ std::string default_preferences_path = DEFAULT_PREFS_PATH;
 #else
 std::string default_preferences_path = "";
 #endif
+bool check_migration = false;
 
 std::string wesnoth_program_dir;
 
@@ -73,6 +56,7 @@ int village_income   = 1;
 int village_support  = 1;
 int recall_cost      = 20;
 int kill_experience  = 8;
+int combat_experience  = 1;
 
 int poison_amount    = 8;
 int rest_heal_amount = 2;
@@ -106,16 +90,19 @@ unsigned lobby_refresh        = 4000;
 
 const std::string observer_team_name = "observer";
 
-const size_t max_loop = 65536;
+const std::size_t max_loop = 65536;
 
 std::vector<server_info> server_list;
 
+bool allow_insecure = false;
+
 //
 // Gamestate flags
 //
 bool
 	debug_impl           = false,
 	debug_lua            = false,
+	strict_lua           = false,
 	editor               = false,
 	ignore_replay_errors = false,
 	mp_debug             = false,
@@ -144,14 +131,14 @@ void set_debug(bool new_debug) {
 }
 
 //
-// Orb display flahs
+// Orb display flags
 //
-bool
-	show_ally_orb,
-	show_enemy_orb,
-	show_moved_orb,
-	show_partial_orb,
-	show_unmoved_orb;
+bool show_ally_orb;
+bool show_disengaged_orb;
+bool show_enemy_orb;
+bool show_moved_orb;
+bool show_partial_orb;
+bool show_unmoved_orb;
 
 //
 // Music constants
@@ -173,23 +160,23 @@ static std::vector<color_t> blue_white_scale;
 static std::vector<color_t> blue_white_scale_text;
 
 std::map<std::string, color_range> team_rgb_range;
+// Map [color_range]id to [color_range]name, or "" if no name
 std::map<std::string, t_string> team_rgb_name;
 
 std::map<std::string, std::vector<color_t>> team_rgb_colors;
 
 std::vector<std::string> default_colors;
 
-namespace colors {
-
-std::string
-	moved_orb_color,
-	unmoved_orb_color,
-	partial_orb_color,
-	enemy_orb_color,
-	ally_orb_color,
-	default_color_list;
-
-} // colors
+namespace colors
+{
+std::string ally_orb_color;
+std::string disengaged_orb_color;
+std::string enemy_orb_color;
+std::string moved_orb_color;
+std::string partial_orb_color;
+std::string unmoved_orb_color;
+std::string default_color_list;
+} // namespace colors
 
 //
 // Image constants
@@ -211,6 +198,7 @@ std::string
 	victory_laurel_easy,
 	// orbs and hp/xp bar
 	orb,
+	orb_two_color,
 	energy,
 	// top bar icons
 	battery_icon,
@@ -231,7 +219,7 @@ std::string
 	observer,
 	tod_bright,
 	tod_dark,
-	///@todo de-hardcode this
+	// TODO: de-hardcode this
 	selected_menu   = "buttons/radiobox-pressed.png",
 	deselected_menu = "buttons/radiobox.png",
 	checked_menu    = "buttons/checkbox-pressed.png",
@@ -284,8 +272,8 @@ std::string
 
 } // sounds
 
-static void add_color_info(const config& v, bool build_defaults);
-void add_color_info(const config& v)
+static void add_color_info(const game_config_view& v, bool build_defaults);
+void add_color_info(const game_config_view& v)
 {
 	add_color_info(v, false);
 }
@@ -299,6 +287,7 @@ void load_config(const config &v)
 	rest_heal_amount = v["rest_heal_amount"].to_int(2);
 	recall_cost      = v["recall_cost"].to_int(20);
 	kill_experience  = v["kill_experience"].to_int(8);
+	combat_experience= v["combat_experience"].to_int(1);
 	lobby_refresh    = v["lobby_refresh"].to_int(2000);
 	default_terrain  = v["default_terrain"].str();
 	tile_size        = v["tile_size"].to_int(72);
@@ -325,6 +314,7 @@ void load_config(const config &v)
 		partial_orb_color  = i["partial_orb_color"].str();
 		enemy_orb_color    = i["enemy_orb_color"].str();
 		ally_orb_color     = i["ally_orb_color"].str();
+		disengaged_orb_color = i["disengaged_orb_color"].str();
 	} // colors
 
 	show_ally_orb     = v["show_ally_orb"].to_bool(true);
@@ -332,6 +322,7 @@ void load_config(const config &v)
 	show_moved_orb    = v["show_moved_orb"].to_bool(true);
 	show_partial_orb  = v["show_partly_orb"].to_bool(true);
 	show_unmoved_orb  = v["show_unmoved_orb"].to_bool(true);
+	show_disengaged_orb = v["show_disengaged_orb"].to_bool(true);
 
 	if(const config& i = v.child("images")){
 		using namespace game_config::images;
@@ -346,6 +337,7 @@ void load_config(const config &v)
 		victory_laurel_easy = i["victory_laurel_easy"].str();
 
 		orb    = i["orb"].str();
+		orb_two_color = i["orb_two_color"].str();
 		energy = i["energy"].str();
 
 		battery_icon = i["battery_icon"].str();
@@ -382,7 +374,7 @@ void load_config(const config &v)
 	shroud_prefix = v["shroud_prefix"].str();
 	fog_prefix    = v["fog_prefix"].str();
 
-	add_color_info(v, true);
+	add_color_info(game_config_view::wrap(v), true);
 
 	if(const config::attribute_value* a = v.get("flag_rgb")) {
 		flag_rgb = a->str();
@@ -457,7 +449,7 @@ void load_config(const config &v)
 	}
 }
 
-void add_color_info(const config& v, bool build_defaults)
+void add_color_info(const game_config_view& v, bool build_defaults)
 {
 	if(build_defaults) {
 		default_colors.clear();
@@ -569,7 +561,7 @@ color_t red_to_green(int val, bool for_text)
 {
 	const std::vector<color_t>& color_scale = for_text ? red_green_scale_text : red_green_scale;
 
-	val = utils::clamp(val, 0, 100);
+	val = std::clamp(val, 0, 100);
 	const int lvl = (color_scale.size() - 1) * val / 100;
 
 	return color_scale[lvl];
@@ -579,7 +571,7 @@ color_t blue_to_white(int val, bool for_text)
 {
 	const std::vector<color_t>& color_scale = for_text ? blue_white_scale_text : blue_white_scale;
 
-	val = utils::clamp(val, 0, 100);
+	val = std::clamp(val, 0, 100);
 	const int lvl = (color_scale.size() - 1) * val / 100;
 
 	return color_scale[lvl];
