@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -39,10 +40,10 @@
 #include "tod_manager.hpp"
 #include "color.hpp"
 #include "synced_context.hpp"
-#include "terrain/type_data.hpp"
 #include "units/unit.hpp"
 #include "units/drawer.hpp"
 #include "whiteboard/manager.hpp"
+#include "overlay.hpp"
 
 static lg::log_domain log_display("display");
 #define ERR_DP LOG_STREAM(err, log_display)
@@ -61,41 +62,40 @@ std::map<map_location,fixed_t> game_display::debugHighlights_;
  */
 std::vector<surface> footsteps_images(const map_location& loc, const pathfind::marked_route & route_, const display_context * dc_);
 
-game_display::game_display(game_board& board, std::weak_ptr<wb::manager> wb,
-		reports & reports_object,
+game_display::game_display(game_board& board,
+		std::weak_ptr<wb::manager> wb,
+		reports& reports_object,
 		const config& theme_cfg,
-		const config& level,
-		bool) :
-		display(&board, wb, reports_object, theme_cfg, level, false),
-		overlay_map_(),
-		attack_indicator_src_(),
-		attack_indicator_dst_(),
-		route_(),
-		displayedUnitHex_(),
-		sidebarScaling_(1.0),
-		first_turn_(true),
-		in_game_(false),
-		chat_man_(new display_chat_manager(*this)),
-		mode_(RUNNING),
-		needs_rebuild_(false)
+		const config& level)
+	: display(&board, wb, reports_object, theme_cfg, level, false)
+	, overlay_map_()
+	, attack_indicator_src_()
+	, attack_indicator_dst_()
+	, route_()
+	, displayedUnitHex_()
+	, in_game_(false)
+	, chat_man_(new display_chat_manager(*this))
+	, mode_(RUNNING)
+	, needs_rebuild_(false)
 {
-	replace_overlay_map(&overlay_map_);
 	video().clear_screen();
 }
 
 game_display::~game_display()
 {
 	try {
-	// SDL_FreeSurface(minimap_);
-	chat_man_->prune_chat_messages(true);
-	} catch (...) {}
+		chat_man_->prune_chat_messages(true);
+	} catch(...) {
+	}
 }
 
 void game_display::new_turn()
 {
+	static bool first_turn = true;
 	const time_of_day& tod = resources::tod_manager->get_time_of_day();
 
-	if( !first_turn_) {
+	// We want to skip this on the first run of this function
+	if(!first_turn) {
 		const time_of_day& old_tod = resources::tod_manager->get_previous_time_of_day();
 
 		if(old_tod.image_mask != tod.image_mask) {
@@ -133,7 +133,7 @@ void game_display::new_turn()
 		tod_hex_mask2 = nullptr;
 	}
 
-	first_turn_ = false;
+	first_turn = false;
 
 	display::update_tod();
 
@@ -200,7 +200,7 @@ void game_display::scroll_to_leader(int side, SCROLL_TYPE scroll_type,bool force
 {
 	unit_map::const_iterator leader = dc_->units().find_leader(side);
 
-	if(leader.valid() && leader->is_visible_to_team(dc_->get_team(viewing_side()), *dc_, false)) {
+	if(leader.valid() && leader->is_visible_to_team(dc_->get_team(viewing_side()), false)) {
 		scroll_to_tile(leader->get_location(), scroll_type, true, force);
 	}
 }
@@ -613,11 +613,11 @@ void game_display::float_label(const map_location& loc, const std::string& text,
 	}
 
 	font::floating_label flabel(text);
-	flabel.set_font_size(font::SIZE_XLARGE);
+	flabel.set_font_size(font::SIZE_FLOAT_LABEL);
 	flabel.set_color(color);
 	flabel.set_position(get_location_x(loc)+zoom_/2, get_location_y(loc));
-	flabel.set_move(0, -2 * turbo_speed());
-	flabel.set_lifetime(60/turbo_speed());
+	flabel.set_move(0, -0.1 * turbo_speed());
+	flabel.set_lifetime(1000/turbo_speed());
 	flabel.set_scroll_mode(font::ANCHOR_LABEL_MAP);
 
 	font::add_floating_label(flabel);
@@ -680,3 +680,8 @@ bool game_display::maybe_rebuild() {
 	}
 	return false;
 }
+
+display::overlay_map& game_display::get_overlays()
+{
+	return overlay_map_;
+}
