@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #pragma once
 
 class config;
@@ -38,8 +37,7 @@ public:
 			std::weak_ptr<wb::manager> wb,
 			reports & reports_object,
 			const config& theme_cfg,
-			const config& level,
-			bool dummy=false);
+			const config& level);
 
 	~game_display();
 	static game_display* get_singleton()
@@ -114,6 +112,12 @@ public:
 	 * valid after being set.
 	 */
 	void set_route(const pathfind::marked_route *route);
+	/**
+	 * Gets the route along which footsteps are drawn to show movement of a
+	 * unit. If no route is currently being shown, the array get_route().steps
+	 * will be empty.
+	 */
+	const pathfind::marked_route& get_route() { return route_; }
 
 	/** Function to float a label above a tile */
 	void float_label(const map_location& loc, const std::string& text, const color_t& color);
@@ -148,6 +152,9 @@ protected:
 
 	virtual void draw_hex(const map_location& loc) override;
 
+	/** Inherited from display. */
+	virtual overlay_map& get_overlays() override;
+
 	std::set<map_location> units_that_can_reach_goal_;
 
 public:
@@ -176,11 +183,9 @@ public:
 	static int& debug_highlight(const map_location& loc);
 	static void clear_debug_highlights() { debugHighlights_.clear(); }
 
-
 	/** The playing team is the team whose turn it is. */
 	virtual int playing_side() const override { return activeTeam_ + 1; }
 
-
 	std::string current_team_name() const;
 
 	display_chat_manager & get_chat_manager() { return *chat_man_; }
@@ -204,10 +209,10 @@ public:
 
 	void set_game_mode(const game_mode mode);
 
-	/// Sets whether the screen (map visuals) needs to be rebuilt. This is typically after the map has been changed by wml.
+	/** Sets whether the screen (map visuals) needs to be rebuilt. This is typically after the map has been changed by wml. */
 	void needs_rebuild(bool b);
 
-	/// Rebuilds the screen if needs_rebuild(true) was previously called, and resets the flag.
+	/** Rebuilds the screen if needs_rebuild(true) was previously called, and resets the flag. */
 	bool maybe_rebuild();
 
 private:
@@ -228,9 +233,7 @@ private:
 
 	map_location displayedUnitHex_;
 
-	double sidebarScaling_;
-
-	bool first_turn_, in_game_;
+	bool in_game_;
 
 	const std::unique_ptr<display_chat_manager> chat_man_;
 
