@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2006 - 2018 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -35,7 +36,11 @@ MAKE_ENUM(LEVEL_RESULT,
 	(DEFEAT,       "defeat")
 	(QUIT,         "quit")
 	(OBSERVER_END, "observer_end")
-)
+	(TEST_NOT_SET, "result_not_set")
+	(TEST_PASS,    "pass")
+	(TEST_FAIL,    "fail")
+	(TEST_INVALID, "test_result_value_invalid")
+);
 
 /**
  * Exception used to escape form the ai or ui code to playsingle_controller::play_side.
@@ -50,7 +55,7 @@ public:
 		, std::exception()
 	{
 	}
-	const char * what() const NOEXCEPT { return "return_to_play_side_exception"; }
+	const char * what() const noexcept { return "return_to_play_side_exception"; }
 private:
 
 	IMPLEMENT_LUA_JAILBREAK_EXCEPTION(return_to_play_side_exception)
@@ -67,7 +72,7 @@ public:
 		, std::exception()
 	{
 	}
-	const char * what() const NOEXCEPT { return "quit_game_exception"; }
+	const char * what() const noexcept { return "quit_game_exception"; }
 private:
 	IMPLEMENT_LUA_JAILBREAK_EXCEPTION(quit_game_exception)
 };
@@ -98,13 +103,15 @@ struct end_level_data
 	bool replay_save;                  /**< Should a replay save be made? */
 	bool proceed_to_next_level;        /**< whether to proceed to the next scenario, equals is_victory in sp. We need to save this in saves during linger mode. > */
 	bool is_victory;
+	std::string test_result;           /**< result to use if this is a unit test */
 	transient_end_level transient;
 	void write(config& cfg) const;
 
 	void read(const config& cfg);
 
 	config to_config() const;
-	config to_config_full() const; ///< Includes the transient data
+	/** Includes the transient data */
+	config to_config_full() const;
 };
 inline void throw_quit_game_exception()
 {
