@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -29,7 +30,6 @@
 #include "ai/manager.hpp"
 #include "fake_unit_ptr.hpp"
 #include "filesystem.hpp"
-#include "game_classification.hpp"
 #include "game_display.hpp"
 #include "preferences/game.hpp"
 #include "gettext.hpp"
@@ -59,10 +59,10 @@
 #include "units/animation_component.hpp"
 #include "units/udisplay.hpp"
 #include "units/filter.hpp"
+#include "units/types.hpp"
 #include "wml_exception.hpp"
 #include "whiteboard/manager.hpp"
-
-#include <boost/regex.hpp>
+#include "deprecation.hpp"
 
 static lg::log_domain log_engine("engine");
 #define DBG_NG LOG_STREAM(debug, log_engine)
@@ -109,7 +109,7 @@ namespace { // Support functions
 		std::string variation = cfg["variation"];
 		std::string img_mods = cfg["image_mods"];
 
-		size_t side_num = cfg["side"].to_int(1);
+		std::size_t side_num = cfg["side"].to_int(1);
 		if (!resources::gameboard->has_team(side_num)) {
 			side_num = 1;
 		}
@@ -144,7 +144,7 @@ namespace { // Support functions
 		std::vector<map_location> path;
 		map_location src;
 		map_location dst;
-		for(size_t i = 0; i != std::min(xvals.size(),yvals.size()); ++i) {
+		for(std::size_t i = 0; i != std::min(xvals.size(),yvals.size()); ++i) {
 			if(i==0){
 				try {
 					src.set_wml_x(std::stoi(xvals[i]));
@@ -253,15 +253,17 @@ wml_action::wml_action(const std::string & tag, handler function)
 	static void wml_func_##pname(const queued_event& pei, const vconfig& pcfg)
 
 
-/// Experimental data persistence
-/// @todo Finish experimenting.
+/**
+ * Experimental data persistence
+ * @todo Finish experimenting.
+ */
 WML_HANDLER_FUNCTION(clear_global_variable,,pcfg)
 {
 	if (!resources::controller->is_replay())
 		verify_and_clear_global_variable(pcfg);
 }
 
-static void on_replay_error(const std::string& message, bool /*b*/)
+static void on_replay_error(const std::string& message)
 {
 	ERR_NG << "Error via [do_command]:" << std::endl;
 	ERR_NG << message << std::endl;
@@ -316,10 +318,10 @@ WML_HANDLER_FUNCTION(do_command,, cfg)
 		ERR_NG << "[do_command] can only be used from clients that control the currently playing side" << std::endl;
 		return;
 	}
-	for(vconfig::all_children_iterator i = cfg.ordered_begin(); i != cfg.ordered_end(); ++i)
+	for(const auto& [key, child] : cfg.all_ordered())
 	{
-		if(allowed_tags.find( i.get_key()) == allowed_tags.end()) {
-			ERR_NG << "unsupported tag [" << i.get_key() << "] in [do_command]" << std::endl;
+		if(allowed_tags.find(key) == allowed_tags.end()) {
+			ERR_NG << "unsupported tag [" << key << "] in [do_command]" << std::endl;
 			std::stringstream o;
 			std::copy(allowed_tags.begin(), allowed_tags.end(), std::ostream_iterator<std::string>(o, " "));
 			ERR_NG << "allowed tags: " << o.str() << std::endl;
@@ -331,17 +333,20 @@ WML_HANDLER_FUNCTION(do_command,, cfg)
 		//Note that this fires related events and everything else that also happens normally.
 		//have to watch out with the undo stack, therefore forbid [auto_shroud] and [update_shroud] here...
 		synced_context::run_in_synced_context_if_not_already(
-			/*commandname*/ i.get_key(),
-			/*data*/ i.get_child().get_parsed_config(),
+			/*commandname*/ key,
+			/*data*/ child.get_parsed_config(),
 			/*use_undo*/ true,
 			/*show*/ true,
 			/*error_handler*/ &on_replay_error
 		);
+		ai::manager::get_singleton().raise_gamestate_changed();
 	}
 }
 
-/// Experimental data persistence
-/// @todo Finish experimenting.
+/**
+ * Experimental data persistence
+ * @todo Finish experimenting.
+ */
 WML_HANDLER_FUNCTION(get_global_variable,,pcfg)
 {
 	verify_and_get_global_variable(pcfg);
@@ -372,10 +377,13 @@ WML_HANDLER_FUNCTION(modify_turns,, cfg)
 	}
 }
 
-/// Moving a 'unit' - i.e. a dummy unit
-/// that is just moving for the visual effect
+/**
+ * Moving a 'unit' - i.e. a dummy unit
+ * that is just moving for the visual effect
+ */
 WML_HANDLER_FUNCTION(move_unit_fake,, cfg)
 {
+	set_scontext_unsynced leave_synced_context;
 	events::command_disabler command_disabler;
 	fake_unit_ptr dummy_unit(create_fake_unit(cfg));
 	if(!dummy_unit.get())
@@ -398,11 +406,13 @@ WML_HANDLER_FUNCTION(move_unit_fake,, cfg)
 
 WML_HANDLER_FUNCTION(move_units_fake,, cfg)
 {
+	set_scontext_unsynced leave_synced_context;
 	events::command_disabler command_disabler;
 	LOG_NG << "Processing [move_units_fake]\n";
 
+	const bool force_scroll = cfg["force_scroll"].to_bool();
 	const vconfig::child_list unit_cfgs = cfg.get_children("fake_unit");
-	size_t num_units = unit_cfgs.size();
+	std::size_t num_units = unit_cfgs.size();
 	std::vector<fake_unit_ptr > units;
 	units.reserve(num_units);
 	std::vector<std::vector<map_location>> paths;
@@ -410,7 +420,7 @@ WML_HANDLER_FUNCTION(move_units_fake,, cfg)
 
 	LOG_NG << "Moving " << num_units << " units\n";
 
-	size_t longest_path = 0;
+	std::size_t longest_path = 0;
 
 	for (const vconfig& config : unit_cfgs) {
 		const std::vector<std::string> xvals = utils::split(config["x"]);
@@ -432,15 +442,15 @@ WML_HANDLER_FUNCTION(move_units_fake,, cfg)
 
 	std::vector<map_location> path_step(2);
 	path_step.resize(2);
-	for(size_t step = 1; step < longest_path; ++step) {
+	for(std::size_t step = 1; step < longest_path; ++step) {
 		DBG_NG << "Doing step " << step << "...\n";
-		for(size_t un = 0; un < num_units; ++un) {
+		for(std::size_t un = 0; un < num_units; ++un) {
 			if(step >= paths[un].size() || paths[un][step - 1] == paths[un][step])
 				continue;
 			DBG_NG << "Moving unit " << un << ", doing step " << step << '\n';
 			path_step[0] = paths[un][step - 1];
 			path_step[1] = paths[un][step];
-			unit_display::move_unit(path_step, units[un].get_unit_ptr());
+			unit_display::move_unit(path_step, units[un].get_unit_ptr(), true, map_location::NDIRECTIONS, force_scroll);
 			units[un]->set_location(path_step[1]);
 			units[un]->anim_comp().set_standing(false);
 		}
@@ -449,9 +459,11 @@ WML_HANDLER_FUNCTION(move_units_fake,, cfg)
 	LOG_NG << "Units moved\n";
 }
 
-/// If we should recall units that match a certain description.
-// If you change attributes specific to [recall] (that is, not a Standard Unit Filter)
-// be sure to update data/lua/wml_tag, auto_recall feature for [role] to reflect your changes.
+/**
+ * If we should recall units that match a certain description.
+ * If you change attributes specific to [recall] (that is, not a Standard Unit Filter)
+ * be sure to update data/lua/wml_tag, auto_recall feature for [role] to reflect your changes.
+ */
 WML_HANDLER_FUNCTION(recall,, cfg)
 {
 	events::command_disabler command_disabler;
@@ -460,16 +472,17 @@ WML_HANDLER_FUNCTION(recall,, cfg)
 	// Prevent the recall unit filter from using the location as a criterion
 
 	/**
-	 * @todo FIXME: we should design the WML to avoid these types of
+	 * FIXME: we should design the WML to avoid these types of
 	 * collisions; filters should be named consistently and always have a
 	 * distinct scope.
 	 */
 	temp_config["x"] = "recall";
 	temp_config["y"] = "recall";
+	temp_config.remove_attribute("location_id");
 	vconfig unit_filter_cfg(temp_config);
 	const vconfig & leader_filter = cfg.child("secondary_unit");
 
-	for(int index = 0; index < int(resources::gameboard->teams().size()); ++index) {
+	for(int index = 0; index < static_cast<int>(resources::gameboard->teams().size()); ++index) {
 		LOG_NG << "for side " << index + 1 << "...\n";
 		const std::string player_id = resources::gameboard->teams()[index].save_id_or_number();
 
@@ -492,9 +505,15 @@ WML_HANDLER_FUNCTION(recall,, cfg)
 				const unit_ptr to_recruit = *u;
 				const unit* pass_check = to_recruit.get();
 				if(!cfg["check_passability"].to_bool(true)) pass_check = nullptr;
-				const map_location cfg_loc = cfg_to_loc(cfg);
+				map_location cfg_loc = cfg_to_loc(cfg);
+				if(cfg.has_attribute("location_id")) {
+					const auto& special_locs = resources::gameboard->map().special_locations().left;
+					const auto& iter = special_locs.find(cfg["location_id"]);
+					if(iter != special_locs.end()) {
+						cfg_loc = iter->second;
+					}
+				}
 
-				/// @todo fendrin: comment this monster
 				for (unit_map::const_unit_iterator leader : leaders) {
 					DBG_NG << "...considering " + leader->id() + " as the recalling leader...\n";
 					map_location loc = cfg_loc;
@@ -543,43 +562,31 @@ namespace {
 	{
 		map_choice(const std::string& filename) : filename_(filename) {}
 		std::string filename_;
-		virtual config query_user(int /*side*/) const
+		virtual config query_user(int /*side*/) const override
 		{
-			//Do a regex check for the file format to prevent sending arbitrary files to other clients.
-			//Note: this allows only the new format.
-			static const std::string s_simple_terrain = R"""([A-Za-z\\|/]{1,4})""";
-			static const std::string s_terrain = s_simple_terrain + R"""((\^)""" + s_simple_terrain + ")?";
-			static const std::string s_sep = "(, |\\n)";
-			static const std::string s_prefix = R"""((\d+ )?)""";
-			static const std::string s_all = "(" + s_prefix + s_terrain + s_sep + ")+";
-			static const boost::regex r_all(s_all);
-
-			const std::string& mapfile = filesystem::get_wml_location(filename_);
-			std::string res = "";
-			if(filesystem::file_exists(mapfile)) {
-				res = filesystem::read_file(mapfile);
-			}
-			config retv;
-			if(boost::regex_match(res, r_all))
-			{
-				retv["map_data"] = res;
-			}
-			return retv;
+			std::string res = filesystem::read_map(filename_);
+			return config {"map_data", res};
 		}
-		virtual config random_choice(int /*side*/) const
+		virtual config random_choice(int /*side*/) const override
 		{
 			return config();
 		}
-		virtual std::string description() const
+		virtual std::string description() const override
 		{
 			return "Map Data";
 		}
-
+		virtual bool is_visible() const override
+		{
+			// Allow query_user() to be called during prestart events, as it doesn't show any UI.
+			return false;
+		}
 	};
 }
 
-/// Experimental map replace
-/// @todo Finish experimenting.
+/**
+ * Experimental map replace
+ * @todo Finish experimenting.
+ */
 WML_HANDLER_FUNCTION(replace_map,, cfg)
 {
 	/*
@@ -599,12 +606,16 @@ WML_HANDLER_FUNCTION(replace_map,, cfg)
 		if(!cfg["map_file"].empty()) {
 			config file_cfg = mp_sync::get_user_choice("map_data", map_choice(cfg["map_file"].str()));
 			map.read(file_cfg["map_data"].str(), false);
+		} else if(!cfg["map_data"].empty()) {
+			map.read(cfg["map_data"], false);
 		} else {
+			deprecated_message("[replace_map]map=", DEP_LEVEL::INDEFINITE, "1.16", "Use map_data= instead.");
 			map.read(cfg["map"], false);
 		}
 	} catch(const incorrect_map_format_error&) {
-		const std::string log_map_name = cfg["map"].empty() ? cfg["file"] : std::string("from inline data");
-		lg::wml_error() << "replace_map: Unable to load map " << log_map_name << std::endl;
+		const std::string log_map_name = cfg["map"].empty() ? cfg["map_file"] : std::string("from inline data");
+		lg::log_to_chat() << "replace_map: Unable to load map " << log_map_name << '\n';
+		ERR_WML << "replace_map: Unable to load map " << log_map_name;
 		return;
 	} catch(const wml_exception& e) {
 		e.show();
@@ -614,7 +625,8 @@ WML_HANDLER_FUNCTION(replace_map,, cfg)
 	if (map.total_width() > game_map->total_width()
 	|| map.total_height() > game_map->total_height()) {
 		if (!cfg["expand"].to_bool()) {
-			lg::wml_error() << "replace_map: Map dimension(s) increase but expand is not set" << std::endl;
+			lg::log_to_chat() << "replace_map: Map dimension(s) increase but expand is not set\n";
+			ERR_WML << "replace_map: Map dimension(s) increase but expand is not set";
 			return;
 		}
 	}
@@ -622,15 +634,17 @@ WML_HANDLER_FUNCTION(replace_map,, cfg)
 	if (map.total_width() < game_map->total_width()
 	|| map.total_height() < game_map->total_height()) {
 		if (!cfg["shrink"].to_bool()) {
-			lg::wml_error() << "replace_map: Map dimension(s) decrease but shrink is not set" << std::endl;
+			lg::log_to_chat() << "replace_map: Map dimension(s) decrease but shrink is not set\n";
+			ERR_WML << "replace_map: Map dimension(s) decrease but shrink is not set";
 			return;
 		}
 	}
 
-	boost::optional<std::string> errmsg = resources::gameboard->replace_map(map);
+	std::optional<std::string> errmsg = resources::gameboard->replace_map(map);
 
 	if (errmsg) {
-		lg::wml_error() << *errmsg << std::endl;
+		lg::log_to_chat() << *errmsg << '\n';
+		ERR_WML << *errmsg;
 	}
 
 	display::get_singleton()->reload_map();
@@ -638,8 +652,10 @@ WML_HANDLER_FUNCTION(replace_map,, cfg)
 	ai::manager::get_singleton().raise_map_changed();
 }
 
-/// Experimental data persistence
-/// @todo Finish experimenting.
+/**
+ * Experimental data persistence
+ * @todo Finish experimenting.
+ */
 WML_HANDLER_FUNCTION(set_global_variable,,pcfg)
 {
 	if (!resources::controller->is_replay())
@@ -711,7 +727,7 @@ WML_HANDLER_FUNCTION(set_variables,, cfg)
 
 				for(std::vector<std::string>::iterator i=split_vector.begin(); i!=split_vector.end(); ++i)
 				{
-					data.emplace_back(config {key_name, *i});
+					data.emplace_back(key_name, *i);
 				}
 			}
 		}
@@ -751,9 +767,11 @@ WML_HANDLER_FUNCTION(set_variables,, cfg)
 	}
 }
 
-/// Store the relative direction from one hex to another in a WML variable.
-/// This is mainly useful as a diagnostic tool, but could be useful
-/// for some kind of scenario.
+/**
+ * Store the relative direction from one hex to another in a WML variable.
+ * This is mainly useful as a diagnostic tool, but could be useful
+ * for some kind of scenario.
+ */
 WML_HANDLER_FUNCTION(store_relative_direction,, cfg)
 {
 	if (!cfg.child("source")) {
@@ -786,10 +804,12 @@ WML_HANDLER_FUNCTION(store_relative_direction,, cfg)
 	}
 }
 
-/// Store the rotation of one hex around another in a WML variable.
-/// In increments of 60 degrees, clockwise.
-/// This is mainly useful as a diagnostic tool, but could be useful
-/// for some kind of scenario.
+/**
+ * Store the rotation of one hex around another in a WML variable.
+ * In increments of 60 degrees, clockwise.
+ * This is mainly useful as a diagnostic tool, but could be useful
+ * for some kind of scenario.
+ */
 WML_HANDLER_FUNCTION(store_rotate_map_location,, cfg)
 {
 	if (!cfg.child("source")) {
@@ -823,67 +843,6 @@ WML_HANDLER_FUNCTION(store_rotate_map_location,, cfg)
 	}
 }
 
-
-/// Store time of day config in a WML variable. This is useful for those who
-/// are too lazy to calculate the corresponding time of day for a given turn,
-/// or if the turn / time-of-day sequence mutates in a scenario.
-WML_HANDLER_FUNCTION(store_time_of_day,, cfg)
-{
-	const map_location loc = cfg_to_loc(cfg);
-	int turn = cfg["turn"];
-	// using 0 will use the current turn
-	const time_of_day& tod = resources::tod_manager->get_time_of_day(loc,turn);
-
-	std::string variable = cfg["variable"];
-	if(variable.empty()) {
-		variable = "time_of_day";
-	}
-	try
-	{
-		variable_access_create store = resources::gamedata->get_variable_access_write(variable);
-		tod.write(store.as_container());
-	}
-	catch(const invalid_variablename_exception&)
-	{
-		ERR_NG << "Found invalid variablename " << variable << " in [store_time_of_day] with " << cfg.get_config().debug() << "\n";
-	}
-}
-
-/// Creating a mask of the terrain
-WML_HANDLER_FUNCTION(terrain_mask,, cfg)
-{
-	map_location loc = cfg_to_loc(cfg, 1, 1);
-
-	gamemap mask_map(resources::gameboard->map().tdata(), "");
-
-	try {
-		if(!cfg["mask_file"].empty()) {
-			const std::string& maskfile = filesystem::get_wml_location(cfg["mask_file"].str());
-
-			if(filesystem::file_exists(maskfile)) {
-				mask_map.read(filesystem::read_file(maskfile), false);
-			} else {
-				throw incorrect_map_format_error("Invalid file path");
-			}
-		} else {
-			mask_map.read(cfg["mask"], false);
-		}
-	} catch(const incorrect_map_format_error&) {
-		ERR_NG << "terrain mask is in the incorrect format, and couldn't be applied" << std::endl;
-		return;
-	} catch(const wml_exception& e) {
-		e.show();
-		return;
-	}
-
-	if (!cfg["border"].to_bool(true)) {
-		mask_map.add_fog_border();
-	}
-
-	resources::gameboard->overlay_map(mask_map, cfg.get_parsed_config(), loc);
-	game_display::get_singleton()->needs_rebuild(true);
-}
-
 WML_HANDLER_FUNCTION(tunnel,, cfg)
 {
 	const bool remove = cfg["remove"].to_bool(false);
@@ -909,7 +868,7 @@ WML_HANDLER_FUNCTION(tunnel,, cfg)
 	}
 }
 
-/// If we should spawn a new unit on the map somewhere
+/** If we should spawn a new unit on the map somewhere */
 WML_HANDLER_FUNCTION(unit,, cfg)
 {
 	events::command_disabler command_disabler;
