@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -37,12 +38,15 @@
 static lg::log_domain log_engine("engine");
 #define WRN_NG LOG_STREAM(warn, log_engine)
 
+static lg::log_domain log_wml("wml");
+#define ERR_WML LOG_STREAM(err, log_wml)
+
 
 // This file is in the game_events namespace.
 namespace game_events {
 
 namespace builtin_conditions {
-	std::vector<std::pair<int,int>> default_counts = utils::parse_ranges("1-99999");
+	std::vector<std::pair<int,int>> default_counts = utils::parse_ranges("1-infinity");
 
 	bool have_unit(const vconfig& cfg)
 	{
@@ -67,7 +71,7 @@ namespace builtin_conditions {
 				if(counts == default_counts && match_count) {
 					break;
 				}
-				for(size_t t = 0; t < team.recall_list().size(); ++t) {
+				for(std::size_t t = 0; t < team.recall_list().size(); ++t) {
 					if(counts == default_counts && match_count) {
 						break;
 					}
@@ -84,7 +88,7 @@ namespace builtin_conditions {
 	bool have_location(const vconfig& cfg)
 	{
 		std::set<map_location> res;
-		terrain_filter(cfg, resources::filter_con).get_locations(res);
+		terrain_filter(cfg, resources::filter_con, false).get_locations(res);
 
 		std::vector<std::pair<int,int>> counts = cfg.has_attribute("count")
 		? utils::parse_ranges(cfg["count"]) : default_counts;
@@ -93,15 +97,28 @@ namespace builtin_conditions {
 
 	bool variable_matches(const vconfig& values)
 	{
+		if(values["name"].blank()) {
+			lg::log_to_chat() << "[variable] with missing name=\n";
+			ERR_WML << "[variable] with missing name=";
+			return true;
+		}
 		const std::string name = values["name"];
 		config::attribute_value value = resources::gamedata->get_variable_const(name);
 
+		if(auto n = values.get_config().attribute_count(); n > 2) {
+			lg::log_to_chat() << "[variable] name='" << name << "' found with multiple comparison attributes\n";
+			ERR_WML << "[variable] name='" << name << "' found with multiple comparison attributes";
+		} else if(n < 2) {
+			lg::log_to_chat() << "[variable] name='" << name << "' found with no comparison attribute\n";
+			ERR_WML << "[variable] name='" << name << "' found with no comparison attribute";
+		}
+
 #define TEST_STR_ATTR(name, test) \
 		do { \
 			if (values.has_attribute(name)) { \
 				std::string attr_str = values[name].str(); \
 				std::string str_value = value.str(); \
-				if (!(test)) return false; \
+				return (test); \
 			} \
 		} while (0)
 
@@ -110,7 +127,7 @@ namespace builtin_conditions {
 			if (values.has_attribute(name)) { \
 				double attr_num = values[name].to_double(); \
 				double num_value = value.to_double(); \
-				if (!(test)) return false; \
+				return (test); \
 			} \
 		} while (0)
 
@@ -119,7 +136,7 @@ namespace builtin_conditions {
 			if (values.has_attribute(name)) { \
 				bool attr_bool = values[name].to_bool(); \
 				bool bool_value = value.to_bool(); \
-				if (!(test)) return false; \
+				return (test); \
 			} \
 		} while (0)
 
@@ -138,6 +155,7 @@ namespace builtin_conditions {
 #undef TEST_STR_ATTR
 #undef TEST_NUM_ATTR
 #undef TEST_BOL_ATTR
+
 		return true;
 	}
 }
@@ -152,19 +170,15 @@ namespace { // Support functions
 			return false;
 		}
 
-		vconfig::all_children_iterator cond_end = cond.ordered_end();
-		static const std::set<std::string> skip =
+		static const std::set<std::string> skip
 			{"then", "else", "elseif", "not", "and", "or", "do"};
 
-		for(vconfig::all_children_iterator it = cond.ordered_begin(); it != cond_end; ++it) {
-			std::string key = it.get_key();
-			bool result = true;
+		for(const auto& [key, filter] : cond.all_ordered()) {
 			if(std::find(skip.begin(), skip.end(), key) == skip.end()) {
 				assert(resources::lua_kernel);
-				result = resources::lua_kernel->run_wml_conditional(key, it.get_child());
-			}
-			if (!result) {
-				return false;
+				if(!resources::lua_kernel->run_wml_conditional(key, filter)) {
+					return false;
+				}
 			}
 		}
 
@@ -179,30 +193,21 @@ bool conditional_passed(const vconfig& cond)
 	bool matches = internal_conditional_passed(cond);
 
 	// Handle [and], [or], and [not] with in-order precedence
-	vconfig::all_children_iterator cond_i = cond.ordered_begin();
-	vconfig::all_children_iterator cond_end = cond.ordered_end();
-	while(cond_i != cond_end)
-	{
-		const std::string& cond_name = cond_i.get_key();
-		const vconfig& cond_filter = cond_i.get_child();
-
+	for(const auto& [key, filter] : cond.all_ordered()) {
 		// Handle [and]
-		if(cond_name == "and")
-		{
-			matches = matches && conditional_passed(cond_filter);
+		if(key == "and") {
+			matches = matches && conditional_passed(filter);
 		}
 		// Handle [or]
-		else if(cond_name == "or")
-		{
-			matches = matches || conditional_passed(cond_filter);
+		else if(key == "or") {
+			matches = matches || conditional_passed(filter);
 		}
 		// Handle [not]
-		else if(cond_name == "not")
-		{
-			matches = matches && !conditional_passed(cond_filter);
+		else if(key == "not") {
+			matches = matches && !conditional_passed(filter);
 		}
-		++cond_i;
 	}
+
 	return matches;
 }
 
