@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -36,7 +37,7 @@ const entity_location entity_location::null_entity(map_location::null_location()
  * Can also be used if the event has a unit and the caller already has the
  * unit's location and underlying ID.
  */
-entity_location::entity_location(const map_location &loc, size_t id)
+entity_location::entity_location(const map_location &loc, std::size_t id)
 	: map_location(loc), id_(id), filter_loc_(loc)
 {}
 
@@ -44,7 +45,7 @@ entity_location::entity_location(const map_location &loc, size_t id)
  * Constructor for when an event has a unit that needs to be filtered as if
  * it was in a different location.
  */
-entity_location::entity_location(const map_location &loc, size_t id,
+entity_location::entity_location(const map_location &loc, std::size_t id,
                                  const map_location & filter_loc)
 	: map_location(loc), id_(id), filter_loc_(filter_loc)
 {}
