@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -31,7 +32,6 @@
 #include "synced_context.hpp"
 #include "terrain/filter.hpp"
 #include "deprecation.hpp"
-#include "gui/auxiliary/old_markup.hpp"
 
 static lg::log_domain log_engine("engine");
 #define ERR_NG LOG_STREAM(err, log_engine)
@@ -80,17 +80,11 @@ wml_menu_item::wml_menu_item(const std::string& id, const config& cfg)
 	, use_hotkey_(cfg["use_hotkey"].to_bool(true))
 	, use_wml_menu_(cfg["use_hotkey"].str() != "only")
 	, is_synced_(cfg["synced"].to_bool(true))
+	, persistent_(cfg["persistent"].to_bool(true))
 {
 	if(cfg.has_attribute("needs_select")) {
 		deprecated_message("needs_select", DEP_LEVEL::INDEFINITE, {1, 15, 0});
 	}
-	gui2::legacy_menu_item parsed(cfg["description"].str(), "Multiple columns in [set_menu_item] are no longer supported; the image is specified by image=.");
-	if(parsed.contained_markup()) {
-		description_ = parsed.label();
-		if(!parsed.description().empty()) {
-			description_ += " " + parsed.description();
-		}
-	}
 }
 
 // Constructor for items defined in an event.
@@ -108,6 +102,7 @@ wml_menu_item::wml_menu_item(const std::string& id, const vconfig& definition)
 	, use_hotkey_(true)
 	, use_wml_menu_(true)
 	, is_synced_(true)
+	, persistent_(true)
 {
 	// On the off-chance that update() doesn't do it, add the hotkey here.
 	// (Update can always modify it.)
@@ -132,6 +127,7 @@ wml_menu_item::wml_menu_item(const std::string& id, const vconfig& definition, c
 	, use_hotkey_(original.use_hotkey_)
 	, use_wml_menu_(original.use_wml_menu_)
 	, is_synced_(original.is_synced_)
+	, persistent_(original.persistent_)
 {
 	// Apply WML.
 	update(definition);
@@ -152,7 +148,7 @@ bool wml_menu_item::can_show(const map_location& hex, const game_data& data, fil
 	}
 
 	// Failing the [fiter_location] tag means no show.
-	if(!filter_location_.empty() && !terrain_filter(filter_location_, &filter_con)(hex)) {
+	if(!filter_location_.empty() && !terrain_filter(filter_location_, &filter_con, false)(hex)) {
 		return false;
 	}
 
@@ -277,15 +273,7 @@ void wml_menu_item::update(const vconfig& vcfg)
 	}
 
 	if(vcfg.has_attribute("description")) {
-		gui2::legacy_menu_item parsed(vcfg["description"].str(), "Multiple columns in [set_menu_item] are no longer supported; the image is specified by image=.");
-		if(parsed.contained_markup()) {
-			description_ = parsed.label();
-			if(!parsed.description().empty()) {
-				description_ += " " + parsed.description();
-			}
-		} else {
-			description_ = vcfg["description"].t_str();
-		}
+		description_ = vcfg["description"].t_str();
 		hotkey_updated = true;
 	}
 
@@ -298,6 +286,10 @@ void wml_menu_item::update(const vconfig& vcfg)
 		is_synced_ = vcfg["synced"].to_bool(true);
 	}
 
+	if(vcfg.has_attribute("persistent")) {
+		persistent_ = vcfg["persistent"].to_bool(true);
+	}
+
 	if(const vconfig& child = vcfg.child("show_if")) {
 		show_if_ = child;
 		show_if_.make_safe();
