@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -84,7 +85,9 @@ public:
 	/**
 	 * Returns whether or not *this is applicable given the context.
 	 * Assumes game variables x1, y1, and unit have been set.
-	 * @param[in]  hex  The hex where the menu will appear.
+	 * @param[in] hex The hex where the menu will appear.
+	 * @param[in] data Used to check whether to show if selecting is required.
+	 * @param[in] context Used to check whether the menu's filter matches.
 	 */
 	bool can_show(const map_location& hex, const game_data& data, filter_context& context) const;
 
@@ -189,6 +192,9 @@ private:
 	 * The command shall not change the gamestate if !is_synced_
 	 */
 	bool is_synced_;
+
+	/** If true, keep this menu item in later scenarios. */
+	bool persistent_;
 };
 
 } // end namespace game_events
