@@ -1,20 +1,20 @@
 /*
-   Copyright (C) 2013 - 2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2013 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "game_initialization/configure_engine.hpp"
 
 #include "formula/string_utils.hpp"
-#include "game_config_manager.hpp"
 #include "mp_game_settings.hpp"
 #include "preferences/credentials.hpp"
 #include "preferences/game.hpp"
@@ -145,11 +145,6 @@ bool configure_engine::allow_observers_default() const
 	return preferences::allow_observers();
 }
 
-bool configure_engine::registered_users_only_default() const
-{
-	return preferences::registered_users_only();
-}
-
 bool configure_engine::shuffle_sides_default() const
 {
 	return preferences::shuffle_sides();
