@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by Nathan Walker <nathan.b.walker@vanderbilt.edu>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Nathan Walker <nathan.b.walker@vanderbilt.edu>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -116,11 +117,6 @@ public:
 		return parameters_.allow_observers;
 	}
 
-	bool registered_users_only() const
-	{
-		return parameters_.registered_users_only;
-	}
-
 	bool shuffle_sides() const
 	{
 		return parameters_.shuffle_sides;
@@ -140,12 +136,12 @@ public:
 	// Setter methods
 	//
 
-	void set_game_name(std::string name)
+	void set_game_name(const std::string& name)
 	{
 		parameters_.name = name;
 	}
 
-	void set_game_password(std::string name)
+	void set_game_password(const std::string& name)
 	{
 		parameters_.password = name;
 	}
@@ -220,9 +216,9 @@ public:
 		parameters_.allow_observers = val;
 	}
 
-	void set_registered_users_only(bool val)
+	void set_private_replay(bool val)
 	{
-		parameters_.registered_users_only = val;
+		parameters_.private_replay = val;
 	}
 
 	void set_oos_debug(bool val)
@@ -261,7 +257,6 @@ public:
 	bool fog_game_default() const;
 	bool shroud_game_default() const;
 	bool allow_observers_default() const;
-	bool registered_users_only_default() const;
 	bool shuffle_sides_default() const;
 	mp_game_settings::RANDOM_FACTION_MODE random_faction_mode_default() const;
 	const config& options_default() const;
@@ -283,7 +278,7 @@ private:
 
 	mp_game_settings& parameters_;
 
-	/// Never nullptr.
+	/** Never nullptr. */
 	const config* initial_;
 
 	/**
