@@ -1,16 +1,18 @@
 /*
-   Copyright (C) 2013 - 2018 by Andrius Silinskas <silinskas.andrius@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2013 - 2021
+	by Andrius Silinskas <silinskas.andrius@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
+
 #include "game_initialization/connect_engine.hpp"
 
 #include "ai/configuration.hpp"
@@ -23,10 +25,11 @@
 #include "log.hpp"
 #include "map/map.hpp"
 #include "mt_rng.hpp"
-#include "team.hpp"
 #include "tod_manager.hpp"
+#include "team.hpp"
 #include "wesnothd_connection.hpp"
 
+#include <array>
 #include <cstdlib>
 #include <ctime>
 
@@ -43,22 +46,31 @@ static lg::log_domain log_mp_connect_engine("mp/connect/engine");
 static lg::log_domain log_network("network");
 #define LOG_NW LOG_STREAM(info, log_network)
 
-static const std::string controller_names[] {
+namespace
+{
+const std::array<std::string, 5> controller_names {{
 	"human",
 	"human",
 	"ai",
 	"null",
 	"reserved"
+}};
+
+const std::set<std::string> children_to_swap {
+	"village",
+	"unit",
+	"ai"
 };
+} // end anon namespace
 
 namespace ng {
 
-connect_engine::connect_engine(saved_game& state, const bool first_scenario, mp_campaign_info* campaign_info)
+connect_engine::connect_engine(saved_game& state, const bool first_scenario, mp_game_metadata* metadata)
 	: level_()
 	, state_(state)
 	, params_(state.mp_settings())
-	, default_controller_(campaign_info ? CNTR_NETWORK : CNTR_LOCAL)
-	, campaign_info_(campaign_info)
+	, default_controller_(metadata ? CNTR_NETWORK : CNTR_LOCAL)
+	, mp_metadata_(metadata)
 	, first_scenario_(first_scenario)
 	, force_lock_settings_()
 	, side_engines_()
@@ -72,13 +84,15 @@ connect_engine::connect_engine(saved_game& state, const bool first_scenario, mp_
 	}
 
 	const bool is_mp = state_.classification().is_normal_mp_game();
-	force_lock_settings_ = (!state.mp_settings().saved_game) && scenario()["force_lock_settings"].to_bool(!is_mp);
+	force_lock_settings_ = (state.mp_settings().saved_game != mp_game_settings::SAVED_GAME_MODE::MIDGAME) && scenario()["force_lock_settings"].to_bool(!is_mp);
 
 	// Original level sides.
 	config::child_itors sides = current_config()->child_range("side");
 
 	// AI algorithms.
-	ai::configuration::add_era_ai_from_config(level_.child("era"));
+	if(const config& era = level_.child("era")) {
+		ai::configuration::add_era_ai_from_config(era);
+	}
 	ai::configuration::add_mod_ai_from_config(level_.child_range("modification"));
 
 	// Set the team name lists and modify the original level sides if necessary.
@@ -181,14 +195,12 @@ connect_engine::connect_engine(saved_game& state, const bool first_scenario, mp_
 		return translation::compare(lhs["name"].str(), rhs["name"].str()) < 0;
 	});
 
-	game_config::add_color_info(scenario());
+	game_config::add_color_info(game_config_view::wrap(scenario()));
 
 	// Create side engines.
 	int index = 0;
 	for(const config& s : sides) {
-		side_engines_.emplace_back(new side_engine(s, *this, index));
-
-		index++;
+		side_engines_.emplace_back(new side_engine(s, *this, index++));
 	}
 
 	if(first_scenario_) {
@@ -229,7 +241,7 @@ void connect_engine::import_user(const config& data, const bool observer, int si
 {
 	const std::string& username = data["name"];
 	assert(!username.empty());
-	if(campaign_info_) {
+	if(mp_metadata_) {
 		connected_users_rw().insert(username);
 	}
 
@@ -352,39 +364,18 @@ bool connect_engine::can_start_game() const
 
 void connect_engine::send_to_server(const config& cfg) const
 {
-	if(campaign_info_) {
-		campaign_info_->connection.send_data(cfg);
+	if(mp_metadata_) {
+		mp_metadata_->connection.send_data(cfg);
 	}
 }
 
-bool connect_engine::receive_from_server(config& dst) const
-{
-	if(campaign_info_) {
-		return campaign_info_->connection.receive_data(dst);
-	}
-	else {
-		return false;
-	}
-}
-
-std::vector<std::string> side_engine::get_children_to_swap()
-{
-	std::vector<std::string> children;
-
-	children.push_back("village");
-	children.push_back("unit");
-	children.push_back("ai");
-
-	return children;
-}
-
 std::multimap<std::string, config> side_engine::get_side_children()
 {
 	std::multimap<std::string, config> children;
 
-	for(const std::string& children_to_swap : get_children_to_swap()) {
-		for(const config& child : cfg_.child_range(children_to_swap)) {
-			children.emplace(children_to_swap, child);
+	for(const std::string& to_swap : children_to_swap) {
+		for(const config& child : cfg_.child_range(to_swap)) {
+			children.emplace(to_swap, child);
 		}
 	}
 
@@ -393,7 +384,7 @@ std::multimap<std::string, config> side_engine::get_side_children()
 
 void side_engine::set_side_children(std::multimap<std::string, config> children)
 {
-	for(const std::string& children_to_remove : get_children_to_swap()) {
+	for(const std::string& children_to_remove : children_to_swap) {
 		cfg_.clear_children(children_to_remove);
 	}
 
@@ -448,31 +439,22 @@ void connect_engine::start_game()
 
 		// Fisher-Yates shuffle.
 		for(int i = playable_sides.size(); i > 1; i--) {
-			int j_side = playable_sides[rng.get_next_random() % i];
-			int i_side = playable_sides[i - 1];
+			const int j_side = playable_sides[rng.get_next_random() % i];
+			const int i_side = playable_sides[i - 1];
 
 			if(i_side == j_side) continue; //nothing to swap
 
 			// First we swap everything about a side with another
-			side_engine_ptr tmp_side = side_engines_[j_side];
-			side_engines_[j_side] = side_engines_[i_side];
-			side_engines_[i_side] = tmp_side;
+			std::swap(side_engines_[j_side], side_engines_[i_side]);
 
-			// Some 'child' variables such as village ownership and
-			// initial side units need to be swapped over as well
+			// Some 'child' variables such as village ownership and initial side units need to be swapped over as well
 			std::multimap<std::string, config> tmp_side_children = side_engines_[j_side]->get_side_children();
 			side_engines_[j_side]->set_side_children(side_engines_[i_side]->get_side_children());
 			side_engines_[i_side]->set_side_children(tmp_side_children);
 
-			// Then we revert the swap for fields that are unique to
-			// player control and the team they selected
-			int tmp_index = side_engines_[j_side]->index();
-			side_engines_[j_side]->set_index(side_engines_[i_side]->index());
-			side_engines_[i_side]->set_index(tmp_index);
-
-			int tmp_team = side_engines_[j_side]->team();
-			side_engines_[j_side]->set_team(side_engines_[i_side]->team());
-			side_engines_[i_side]->set_team(tmp_team);
+			// Then we revert the swap for fields that are unique to player control and the team they selected
+			std::swap(side_engines_[j_side]->index_, side_engines_[i_side]->index_);
+			std::swap(side_engines_[j_side]->team_,  side_engines_[i_side]->team_);
 		}
 	}
 
@@ -490,7 +472,7 @@ void connect_engine::start_game()
 	send_to_server(config("start_game"));
 }
 
-void connect_engine::start_game_commandline(const commandline_options& cmdline_opts, const config& game_config)
+void connect_engine::start_game_commandline(const commandline_options& cmdline_opts, const game_config_view& game_config)
 {
 	DBG_MP << "starting a new game in commandline mode" << std::endl;
 
@@ -504,7 +486,7 @@ void connect_engine::start_game_commandline(const commandline_options& cmdline_o
 
 		// Set the faction, if commandline option is given.
 		if(cmdline_opts.multiplayer_side) {
-			for(const mp_option& option : *cmdline_opts.multiplayer_side) {
+			for(const mp_option option : *cmdline_opts.multiplayer_side) {
 
 				if(std::get<0>(option) == num) {
 					if(std::find_if(era_factions_.begin(), era_factions_.end(), [&option](const config* faction) { return (*faction)["id"] == std::get<1>(option); }) != era_factions_.end()) {
@@ -521,7 +503,7 @@ void connect_engine::start_game_commandline(const commandline_options& cmdline_o
 
 		// Set the controller, if commandline option is given.
 		if(cmdline_opts.multiplayer_controller) {
-			for(const mp_option& option : *cmdline_opts.multiplayer_controller) {
+			for(const mp_option option : *cmdline_opts.multiplayer_controller) {
 
 				if(std::get<0>(option) == num) {
 					DBG_MP << "\tsetting side " << std::get<0>(option) <<
@@ -537,7 +519,7 @@ void connect_engine::start_game_commandline(const commandline_options& cmdline_o
 		std::string ai_algorithm = game_config.child("ais")["default_ai_algorithm"].str();
 		side->set_ai_algorithm(ai_algorithm);
 		if(cmdline_opts.multiplayer_algorithm) {
-			for(const mp_option& option : *cmdline_opts.multiplayer_algorithm) {
+			for(const mp_option option : *cmdline_opts.multiplayer_algorithm) {
 
 				if(std::get<0>(option) == num) {
 					DBG_MP << "\tsetting side " << std::get<0>(option) <<
@@ -564,7 +546,7 @@ void connect_engine::start_game_commandline(const commandline_options& cmdline_o
 
 	for(config &side : scenario().child_range("side")) {
 		if(cmdline_opts.multiplayer_ai_config) {
-			for(const mp_option& option : *cmdline_opts.multiplayer_ai_config) {
+			for(const mp_option option : *cmdline_opts.multiplayer_ai_config) {
 
 				if(std::get<0>(option) == side["side"].to_unsigned()) {
 					DBG_MP << "\tsetting side " << side["side"] <<
@@ -615,7 +597,7 @@ void connect_engine::leave_game()
 
 std::pair<bool, bool> connect_engine::process_network_data(const config& data)
 {
-	std::pair<bool, bool> result(std::make_pair(false, true));
+	std::pair<bool, bool> result(false, true);
 
 	if(data.child("leave_game")) {
 		result.first = true;
@@ -763,7 +745,7 @@ std::pair<bool, bool> connect_engine::process_network_data(const config& data)
 
 int connect_engine::find_user_side_index_by_id(const std::string& id) const
 {
-	size_t i = 0;
+	std::size_t i = 0;
 	for(side_engine_ptr side : side_engines_) {
 		if(side->player_id() == id) {
 			break;
@@ -787,11 +769,11 @@ void connect_engine::send_level_data() const
 			"create_game", config {
 				"name", params_.name,
 				"password", params_.password,
+				"ignored", preferences::get_ignored_delim(),
 			},
 		});
 		send_to_server(level_);
 	} else {
-		send_to_server(config {"update_game", config()});
 		config next_level;
 		next_level.add_child("store_next_scenario", level_);
 		send_to_server(next_level);
@@ -835,7 +817,7 @@ void connect_engine::load_previous_sides_users()
 	//Do this in an extra loop to make sure we import each user only once.
 	for(const std::string& name : names)
 	{
-		if(connected_users().find(name) != connected_users().end() || !campaign_info_) {
+		if(connected_users().find(name) != connected_users().end() || !mp_metadata_) {
 			import_user(name, false);
 		}
 	}
@@ -850,8 +832,8 @@ void connect_engine::update_side_controller_options()
 
 const std::set<std::string>& connect_engine::connected_users() const
 {
-	if(campaign_info_) {
-		return campaign_info_->connected_players;
+	if(mp_metadata_) {
+		return mp_metadata_->connected_players;
 	}
 
 	static std::set<std::string> empty;
@@ -860,8 +842,8 @@ const std::set<std::string>& connect_engine::connected_users() const
 
 std::set<std::string>& connect_engine::connected_users_rw()
 {
-	assert(campaign_info_);
-	return campaign_info_->connected_players;
+	assert(mp_metadata_);
+	return mp_metadata_->connected_players;
 }
 
 side_engine::side_engine(const config& cfg, connect_engine& parent_engine, const int index)
@@ -882,13 +864,15 @@ side_engine::side_engine(const config& cfg, connect_engine& parent_engine, const
 	, ai_algorithm_()
 	, chose_random_(cfg["chose_random"].to_bool(false))
 	, disallow_shuffle_(cfg["disallow_shuffle"].to_bool(false))
-	, flg_(parent_.era_factions_, cfg_, parent_.force_lock_settings_, parent_.params_.use_map_settings, parent_.params_.saved_game)
-	, allow_changes_(!parent_.params_.saved_game && !(flg_.choosable_factions().size() == 1 && flg_.choosable_leaders().size() == 1 && flg_.choosable_genders().size() == 1))
+	, flg_(parent_.era_factions_, cfg_, parent_.force_lock_settings_, parent_.params_.use_map_settings, parent_.params_.saved_game == mp_game_settings::SAVED_GAME_MODE::MIDGAME)
+	, allow_changes_(parent_.params_.saved_game != mp_game_settings::SAVED_GAME_MODE::MIDGAME && !(flg_.choosable_factions().size() == 1 && flg_.choosable_leaders().size() == 1 && flg_.choosable_genders().size() == 1))
 	, waiting_to_choose_faction_(allow_changes_)
 	, color_options_(game_config::default_colors)
-	, color_id_(color_options_[color_])
+	//TODO: what should we do if color_ is out of range?
+	, color_id_(color_options_.at(color_))
 {
-	// Save default attributes that could be overwirtten by the faction, so that correct faction lists would be
+
+	// Save default attributes that could be overwritten by the faction, so that correct faction lists would be
 	// initialized by flg_manager when the new side config is sent over network.
 	cfg_.add_child("default_faction", config {
 		"type",    cfg_["type"],
@@ -904,7 +888,7 @@ side_engine::side_engine(const config& cfg, connect_engine& parent_engine, const
 	cfg_["side"] = index_ + 1;
 
 	// Check if this side should give its control to some other side.
-	const size_t side_cntr_index = cfg_["controller"].to_int(-1) - 1;
+	const std::size_t side_cntr_index = cfg_["controller"].to_int(-1) - 1;
 	if(side_cntr_index < parent_.side_engines().size()) {
 		// Remove this attribute to avoid locking side
 		// to non-existing controller type.
@@ -922,7 +906,7 @@ side_engine::side_engine(const config& cfg, connect_engine& parent_engine, const
 	update_controller_options();
 
 	// Tweak the controllers.
-	if(parent_.state_.classification().campaign_type == game_classification::CAMPAIGN_TYPE::SCENARIO && cfg_["controller"].blank()) {
+	if(parent_.state_.classification().is_scenario() && cfg_["controller"].blank()) {
 		cfg_["controller"] = "ai";
 	}
 
@@ -1007,7 +991,7 @@ config side_engine::new_config() const
 	res["side"] = index_ + 1;
 
 	// If the user is allowed to change type, faction, leader etc,  then import their new values in the config.
-	if(!parent_.params_.saved_game) {
+	if(parent_.params_.saved_game != mp_game_settings::SAVED_GAME_MODE::MIDGAME) {
 		// Merge the faction data to res.
 		config faction = flg_.current_faction();
 		LOG_MP << "side_engine::new_config: side=" << index_ + 1 << " faction=" << faction["id"] << " recruit=" << faction["recruit"] << "\n";
@@ -1026,7 +1010,7 @@ config side_engine::new_config() const
 
 	// This function (new_config) is only meant to be called by the host's machine, which is why this check
 	// works. It essentially certifies that whatever side has the player_id that matches the host's login
-	// will be flagged. The reason we cannot check mp_campaign_info::is_host is because that flag is *always*
+	// will be flagged. The reason we cannot check mp_game_metadata::is_host is because that flag is *always*
 	// true on the host's machine, meaning this flag would be set to true for every side.
 	res["is_host"] = player_id_ == preferences::login();
 
@@ -1052,7 +1036,7 @@ config side_engine::new_config() const
 		// AI algorithm, we do nothing. Otherwise we add the chosen AI and if this
 		// is a saved game, we also remove the old stages from the AI config.
 		if(ai_algorithm_ != "use_saved") {
-			if(parent_.params_.saved_game) {
+			if(parent_.params_.saved_game == mp_game_settings::SAVED_GAME_MODE::MIDGAME) {
 				for (config &ai_config : res.child_range("ai")) {
 					ai_config.clear_children("stage");
 				}
@@ -1084,7 +1068,7 @@ config side_engine::new_config() const
 	res["allow_changes"] = allow_changes_;
 	res["chose_random"] = chose_random_;
 
-	if(!parent_.params_.saved_game) {
+	if(parent_.params_.saved_game != mp_game_settings::SAVED_GAME_MODE::MIDGAME) {
 		// Find a config where a default leader is and set a new type and gender values for it.
 		config* leader = &res;
 
@@ -1167,7 +1151,8 @@ config side_engine::new_config() const
 		res["income"] = income_;
 	}
 
-	if(parent_.params_.use_map_settings && !parent_.params_.saved_game) {
+
+	if(parent_.params_.use_map_settings && parent_.params_.saved_game != mp_game_settings::SAVED_GAME_MODE::MIDGAME) {
 		if(cfg_.has_attribute("name")){
 			res["name"] = cfg_["name"];
 		}
@@ -1231,7 +1216,7 @@ bool side_engine::available_for_user(const std::string& name) const
 
 void side_engine::resolve_random(randomness::mt_rng & rng, const std::vector<std::string> & avoid_faction_ids)
 {
-	if(parent_.params_.saved_game) {
+	if(parent_.params_.saved_game == mp_game_settings::SAVED_GAME_MODE::MIDGAME) {
 		return;
 	}
 
@@ -1250,7 +1235,7 @@ void side_engine::reset()
 	set_waiting_to_choose_status(false);
 	set_controller(parent_.default_controller_);
 
-	if(!parent_.params_.saved_game) {
+	if(parent_.params_.saved_game != mp_game_settings::SAVED_GAME_MODE::MIDGAME) {
 		flg_.set_current_faction(0);
 	}
 }
@@ -1284,7 +1269,7 @@ void side_engine::update_controller_options()
 	controller_options_.clear();
 
 	// Default options.
-	if(parent_.campaign_info_) {
+	if(parent_.mp_metadata_) {
 		add_controller_option(CNTR_NETWORK, _("Network Player"), "human");
 	}
 
@@ -1372,9 +1357,7 @@ void side_engine::set_controller_commandline(const std::string& controller_name)
 void side_engine::add_controller_option(ng::controller controller,
 		const std::string& name, const std::string& controller_value)
 {
-	if(controller_lock_ && !cfg_["controller"].empty() &&
-		cfg_["controller"] != controller_value) {
-
+	if(controller_lock_ && !cfg_["controller"].empty() && cfg_["controller"] != controller_value) {
 		return;
 	}
 
