@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2013 - 2018 by Andrius Silinskas <silinskas.andrius@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2013 - 2021
+	by Andrius Silinskas <silinskas.andrius@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "game_initialization/create_engine.hpp"
@@ -67,8 +68,7 @@ void scenario::set_metadata()
 	const std::string& map_data = data_["map_data"];
 
 	try {
-		map_.reset(new gamemap(game_config_manager::get()->terrain_types(),
-			map_data));
+		map_.reset(new gamemap(map_data));
 	} catch(const incorrect_map_format_error& e) {
 		data_["description"] = _("Map could not be loaded: ") + e.message;
 
@@ -182,8 +182,8 @@ campaign::campaign(const config& data)
 	, id_(data["id"])
 	, allow_era_choice_(level::allow_era_choice())
 	, image_label_()
-	, min_players_(2)
-	, max_players_(2)
+	, min_players_(1)
+	, max_players_(1)
 {
 	if(data.has_attribute("start_year")) {
 		dates_.first = irdya_date::read_date(data["start_year"]);
@@ -207,8 +207,8 @@ void campaign::set_metadata()
 {
 	image_label_ = data_["image"].str();
 
-	int min = data_["min_players"].to_int(2);
-	int max = data_["max_players"].to_int(2);
+	int min = data_["min_players"].to_int(1);
+	int max = data_["max_players"].to_int(1);
 
 	min_players_ = max_players_ =  min;
 
@@ -259,18 +259,17 @@ create_engine::create_engine(saved_game& state)
 	state_.clear();
 	state_.classification().campaign_type = type;
 
-	game_config_manager::get()->load_game_config_for_create(type == game_classification::CAMPAIGN_TYPE::MULTIPLAYER);
+	game_config_manager::get()->load_game_config_for_create(state_.classification().is_multiplayer());
 
 	// Initialize dependency_manager_ after refreshing game config.
-	dependency_manager_.reset(new depcheck::manager(
-		game_config_, type == game_classification::CAMPAIGN_TYPE::MULTIPLAYER));
+	dependency_manager_.reset(new depcheck::manager(game_config_, state_.classification().is_multiplayer()));
 
 	// TODO: the editor dir is already configurable, is the preferences value
 	filesystem::get_files_in_dir(filesystem::get_user_data_dir() + "/editor/maps", &user_map_names_,
-		nullptr, filesystem::FILE_NAME_ONLY);
+		nullptr, filesystem::name_mode::FILE_NAME_ONLY);
 
 	filesystem::get_files_in_dir(filesystem::get_user_data_dir() + "/editor/scenarios", &user_scenario_names_,
-		nullptr, filesystem::FILE_NAME_ONLY);
+		nullptr, filesystem::name_mode::FILE_NAME_ONLY);
 
 	DBG_MP << "initializing all levels, eras and mods\n";
 
@@ -278,15 +277,15 @@ create_engine::create_engine(saved_game& state)
 	init_extras(ERA);
 	init_extras(MOD);
 
-	state_.mp_settings().saved_game = false;
+	state_.mp_settings().saved_game = mp_game_settings::SAVED_GAME_MODE::NONE;
 
-	for(const std::string& str : preferences::modifications(state_.classification().campaign_type == game_classification::CAMPAIGN_TYPE::MULTIPLAYER)) {
+	for(const std::string& str : preferences::modifications(state_.classification().is_multiplayer())) {
 		if(game_config_.find_child("modification", "id", str)) {
-			state_.mp_settings().active_mods.push_back(str);
+			state_.classification().active_mods.push_back(str);
 		}
 	}
 
-	dependency_manager_->try_modifications(state_.mp_settings().active_mods, true);
+	dependency_manager_->try_modifications(state_.classification().active_mods, true);
 
 	reset_level_filters();
 }
@@ -371,8 +370,9 @@ void create_engine::prepare_for_new_level()
 
 void create_engine::prepare_for_era_and_mods()
 {
-	state_.classification().era_define = game_config_.find_child("era", "id", get_parameters().mp_era)["define"].str();
-	for(const std::string& mod_id : get_parameters().active_mods) {
+	get_parameters();
+	state_.classification().era_define = game_config_.find_child("era", "id", state_.classification().era_id)["define"].str();
+	for(const std::string& mod_id : state_.classification().active_mods) {
 		state_.classification().mod_defines.push_back(game_config_.find_child("modification", "id", mod_id)["define"].str());
 	}
 }
@@ -406,6 +406,7 @@ void create_engine::prepare_for_campaign(const std::string& difficulty)
 
 	state_.classification().end_text = current_level_data["end_text"].str();
 	state_.classification().end_text_duration = current_level_data["end_text_duration"];
+	state_.classification().end_credits = current_level_data["end_credits"].to_bool(true);
 
 	state_.classification().campaign_define = current_level_data["define"].str();
 	state_.classification().campaign_xtra_defines =
@@ -425,10 +426,6 @@ std::string create_engine::select_campaign_difficulty(int set_value)
 		difficulties.push_back(d["define"]);
 	}
 
-	if(difficulties.empty()) {
-		difficulties = utils::split(current_level().data()["difficulties"]);
-	}
-
 	// No difficulties found. Exit
 	if(difficulties.empty()) {
 		return "";
@@ -470,11 +467,11 @@ void create_engine::prepare_for_saved_game()
 {
 	DBG_MP << "preparing mp_game_settings for saved game\n";
 
-	game_config_manager::get()->load_game_config_for_game(state_.classification());
+	game_config_manager::get()->load_game_config_for_game(state_.classification(), state_.get_scenario_id());
 
 	// The save might be a start-of-scenario save so make sure we have the scenario data loaded.
 	state_.expand_scenario();
-	state_.mp_settings().saved_game = true;
+	state_.mp_settings().saved_game = state_.is_mid_game_save() ? mp_game_settings::SAVED_GAME_MODE::MIDGAME : mp_game_settings::SAVED_GAME_MODE::SCENARIO_START;
 }
 
 void create_engine::prepare_for_other()
@@ -516,7 +513,7 @@ const create_engine::extras_metadata& create_engine::current_era() const
 	return *get_const_extras_by_type(ERA)[current_era_index_];
 }
 
-void create_engine::set_current_level(const size_t index)
+void create_engine::set_current_level(const std::size_t index)
 {
 	try {
 		current_level_index_ = type_map_.at(current_level_type_.v).games_filtered.at(index);
@@ -535,12 +532,12 @@ void create_engine::set_current_level(const size_t index)
 		generator_.reset(nullptr);
 	}
 
-	if(state_.classification().campaign_type == game_classification::CAMPAIGN_TYPE::MULTIPLAYER) {
+	if(state_.classification().is_multiplayer()) {
 		dependency_manager_->try_scenario(current_level().id());
 	}
 }
 
-void create_engine::set_current_era_index(const size_t index, bool force)
+void create_engine::set_current_era_index(const std::size_t index, bool force)
 {
 	current_era_index_ = index;
 
@@ -554,7 +551,7 @@ bool create_engine::toggle_mod(int index, bool force)
 	bool is_active = dependency_manager_->is_modification_active(index);
 	dependency_manager_->try_modification_by_index(index, !is_active, force);
 
-	state_.mp_settings().active_mods = dependency_manager_->get_modifications();
+	state_.classification().active_mods = dependency_manager_->get_modifications();
 
 	return !is_active;
 }
@@ -579,7 +576,7 @@ std::pair<level::TYPE, int> create_engine::find_level_by_id(const std::string& i
 	for(const auto& type : type_map_) {
 		int i = 0;
 
-		for(const auto game : type.second.games) {
+		for(const auto& game : type.second.games) {
 			if(game->id() == id) {
 				return {type.first, i};
 			}
@@ -606,12 +603,12 @@ int create_engine::find_extra_by_id(const MP_EXTRA extra_type, const std::string
 
 void create_engine::init_active_mods()
 {
-	state_.mp_settings().active_mods = dependency_manager_->get_modifications();
+	state_.classification().active_mods = dependency_manager_->get_modifications();
 }
 
 std::vector<std::string>& create_engine::active_mods()
 {
-	return state_.mp_settings().active_mods;
+	return state_.classification().active_mods;
 }
 
 std::vector<create_engine::extras_metadata_ptr> create_engine::active_mods_data()
@@ -637,7 +634,7 @@ const mp_game_settings& create_engine::get_parameters()
 	DBG_MP << "getting parameter values" << std::endl;
 
 	int era_index = current_level().allow_era_choice() ? current_era_index_ : 0;
-	state_.mp_settings().mp_era = eras_[era_index]->id;
+	state_.classification().era_id = eras_[era_index]->id;
 	state_.mp_settings().mp_era_name = eras_[era_index]->name;
 
 	return state_.mp_settings();
@@ -650,7 +647,7 @@ void create_engine::init_all_levels()
 
 		// User maps.
 		int dep_index_offset = 0;
-		for(size_t i = 0; i < user_map_names_.size(); i++)
+		for(std::size_t i = 0; i < user_map_names_.size(); i++)
 		{
 			config user_map_data = gen_mp_data;
 			user_map_data["map_data"] = filesystem::read_map(user_map_names_[i]);
@@ -661,7 +658,7 @@ void create_engine::init_all_levels()
 			bool add_map = true;
 			std::unique_ptr<gamemap> map;
 			try {
-				map.reset(new gamemap(game_config_manager::get()->terrain_types(), user_map_data["map_data"]));
+				map.reset(new gamemap(user_map_data["map_data"]));
 			} catch (const incorrect_map_format_error& e) {
 				user_map_data["description"] = _("Map could not be loaded: ") + e.message;
 
@@ -684,7 +681,7 @@ void create_engine::init_all_levels()
 
 		// User made scenarios.
 		dep_index_offset = 0;
-		for(size_t i = 0; i < user_scenario_names_.size(); i++)
+		for(std::size_t i = 0; i < user_scenario_names_.size(); i++)
 		{
 			config data;
 			try {
@@ -737,7 +734,7 @@ void create_engine::init_all_levels()
 		}
 
 		const std::string& type = data["type"];
-		bool mp = state_.classification().campaign_type == game_classification::CAMPAIGN_TYPE::MULTIPLAYER;
+		const bool mp = state_.classification().is_multiplayer();
 
 		if(type == "mp" || (type == "hybrid" && mp)) {
 			type_map_[level::TYPE::CAMPAIGN].games.emplace_back(new campaign(data));
@@ -774,7 +771,7 @@ void create_engine::init_extras(const MP_EXTRA extra_type)
 	for(const config& extra : game_config_.child_range(extra_name))
 	{
 		ng::depcheck::component_availability type = extra["type"].to_enum(default_availabilty);
-		bool mp = state_.classification().campaign_type == game_classification::CAMPAIGN_TYPE::MULTIPLAYER;
+		const bool mp = state_.classification().is_multiplayer();
 
 		if((type != ng::depcheck::component_availability::MP || mp) && (type != ng::depcheck::component_availability::SP || !mp) )
 		{
@@ -805,7 +802,7 @@ void create_engine::apply_level_filters()
 std::vector<create_engine::level_ptr> create_engine::get_levels_by_type_unfiltered(level::TYPE type) const
 {
 	std::vector<level_ptr> levels;
-	for(const level_ptr lvl : type_map_.at(type.v).games) {
+	for(const level_ptr& lvl : type_map_.at(type.v).games) {
 		levels.push_back(lvl);
 	}
 
@@ -817,14 +814,14 @@ std::vector<create_engine::level_ptr> create_engine::get_levels_by_type(level::T
     auto& g_list = type_map_.at(type.v);
 
 	std::vector<level_ptr> levels;
-	for(size_t level : g_list.games_filtered) {
+	for(std::size_t level : g_list.games_filtered) {
 		levels.push_back(g_list.games[level]);
 	}
 
 	return levels;
 }
 
-std::vector<size_t> create_engine::get_filtered_level_indices(level::TYPE type) const
+std::vector<std::size_t> create_engine::get_filtered_level_indices(level::TYPE type) const
 {
 	return type_map_.at(type.v).games_filtered;
 }
