@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2012 - 2018 by Boldizsár Lipka <lipkab@zoho.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2012 - 2021
+	by Boldizsár Lipka <lipkab@zoho.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "game_initialization/depcheck.hpp"
@@ -24,6 +25,7 @@
 #include "gui/dialogs/depcheck_confirm_change.hpp"
 #include "gui/dialogs/depcheck_select_new.hpp"
 #include "gui/dialogs/message.hpp"
+#include "game_config_view.hpp"
 
 static lg::log_domain log_mp_create_depcheck("mp/create/depcheck");
 #define DBG_MP LOG_STREAM(debug, log_mp_create_depcheck)
@@ -55,7 +57,7 @@ namespace ng
 {
 namespace depcheck
 {
-manager::manager(const config& gamecfg, bool mp)
+manager::manager(const game_config_view& gamecfg, bool mp)
 	: depinfo_()
 	, era_()
 	, scenario_()
@@ -182,7 +184,7 @@ std::vector<std::string> manager::get_required(const elem& e) const
 	config data = depinfo_.find_child(e.type, "id", e.id);
 
 	if(data.has_attribute("force_modification")) {
-		result = utils::split(data["force_modification"], ',');
+		result = utils::split(data["force_modification"].str(), ',');
 	}
 
 	return result;
@@ -207,7 +209,7 @@ std::vector<std::string> manager::get_conflicting_enabled(const elem& e) const
 	std::vector<std::string> result;
 
 	for(const std::string& mod : mods_) {
-		if(conflicts(elem(mod, "modification"), e)) {
+		if(does_conflict(elem(mod, "modification"), e)) {
 			result.push_back(mod);
 		}
 	}
@@ -215,7 +217,7 @@ std::vector<std::string> manager::get_conflicting_enabled(const elem& e) const
 	return result;
 }
 
-bool manager::conflicts(const elem& elem1, const elem& elem2, bool directonly) const
+bool manager::does_conflict(const elem& elem1, const elem& elem2, bool directonly) const
 {
 	if(elem1 == elem2) {
 		return false;
@@ -257,7 +259,7 @@ bool manager::conflicts(const elem& elem1, const elem& elem2, bool directonly) c
 	if(data1.has_attribute("allow_" + elem2.type)) {
 		std::vector<std::string> allowed = utils::split(data1["allow_" + elem2.type]);
 
-		result = !utils::contains(allowed, elem2.id) && !requires(elem1, elem2);
+		result = !utils::contains(allowed, elem2.id) && !does_require(elem1, elem2);
 	} else if(data1.has_attribute("disallow_" + elem2.type)) {
 		std::vector<std::string> disallowed = utils::split(data1["disallow_" + elem2.type]);
 
@@ -267,7 +269,7 @@ bool manager::conflicts(const elem& elem1, const elem& elem2, bool directonly) c
 	if(data2.has_attribute("allow_" + elem1.type)) {
 		std::vector<std::string> allowed = utils::split(data2["allow_" + elem1.type]);
 
-		result = result || (!utils::contains(allowed, elem1.id) && !requires(elem2, elem1));
+		result = result || (!utils::contains(allowed, elem1.id) && !does_require(elem2, elem1));
 	} else if(data2.has_attribute("disallow_" + elem1.type)) {
 		std::vector<std::string> disallowed = utils::split(data2["disallow_" + elem1.type]);
 
@@ -285,7 +287,7 @@ bool manager::conflicts(const elem& elem1, const elem& elem2, bool directonly) c
 		for(const std::string& s : req1) {
 			elem m(s, "modification");
 
-			if(conflicts(elem2, m, true)) {
+			if(does_conflict(elem2, m, true)) {
 				return true;
 			}
 		}
@@ -293,7 +295,7 @@ bool manager::conflicts(const elem& elem1, const elem& elem2, bool directonly) c
 		for(const std::string& s : req2) {
 			elem m(s, "modification");
 
-			if(conflicts(elem1, m, true)) {
+			if(does_conflict(elem1, m, true)) {
 				return true;
 			}
 		}
@@ -304,7 +306,7 @@ bool manager::conflicts(const elem& elem1, const elem& elem2, bool directonly) c
 			for(const std::string& id2 : req2) {
 				elem m2(id2, "modification");
 
-				if(conflicts(m1, m2)) {
+				if(does_conflict(m1, m2)) {
 					return true;
 				}
 			}
@@ -314,7 +316,7 @@ bool manager::conflicts(const elem& elem1, const elem& elem2, bool directonly) c
 	return false;
 }
 
-bool manager::requires(const elem& elem1, const elem& elem2) const
+bool manager::does_require(const elem& elem1, const elem& elem2) const
 {
 	if(elem2.type != "modification") {
 		return false;
@@ -440,8 +442,7 @@ bool manager::enable_mods_dialog(const std::vector<std::string>& mods, const std
 		items.push_back(depinfo_.find_child("modification", "id", mod)["name"]);
 	}
 
-	gui2::dialogs::depcheck_confirm_change dialog(true, items, requester);
-	return dialog.show();
+	return gui2::dialogs::depcheck_confirm_change::execute(true, items, requester);
 }
 
 bool manager::disable_mods_dialog(const std::vector<std::string>& mods, const std::string& requester)
@@ -451,8 +452,7 @@ bool manager::disable_mods_dialog(const std::vector<std::string>& mods, const st
 		items.push_back(depinfo_.find_child("modification", "id", mod)["name"]);
 	}
 
-	gui2::dialogs::depcheck_confirm_change dialog(false, items, requester);
-	return dialog.show();
+	return gui2::dialogs::depcheck_confirm_change::execute(false, items, requester);
 }
 
 std::string manager::change_era_dialog(const std::vector<std::string>& eras)
@@ -553,13 +553,13 @@ bool manager::change_scenario(const std::string& id)
 
 	// Now checking if the currently selected era conflicts the scenario
 	// and changing era if necessary
-	if(!conflicts(scen, elem(era_, "era"))) {
+	if(!does_conflict(scen, elem(era_, "era"))) {
 		return true;
 	}
 
 	std::vector<std::string> compatible;
 	for(const config& i : depinfo_.child_range("era")) {
-		if(!conflicts(scen, elem(i["id"], "era"))) {
+		if(!does_conflict(scen, elem(i["id"], "era"))) {
 			compatible.push_back(i["id"]);
 		}
 	}
@@ -621,13 +621,13 @@ bool manager::change_era(const std::string& id)
 
 	// Now checking if the currently selected scenario conflicts the era
 	// and changing scenario if necessary
-	if(!conflicts(era, elem(scenario_, "scenario"))) {
+	if(!does_conflict(era, elem(scenario_, "scenario"))) {
 		return true;
 	}
 
 	std::vector<std::string> compatible;
 	for(const config& i : depinfo_.child_range("scenario")) {
-		if(!conflicts(era, elem(i["id"], "scenario"))) {
+		if(!does_conflict(era, elem(i["id"], "scenario"))) {
 			compatible.push_back(i["id"]);
 		}
 	}
@@ -655,7 +655,7 @@ bool manager::change_modifications(const std::vector<std::string>& modifications
 		elem ei(i, "modification");
 
 		for(const std::string& j : filtered) {
-			ok = ok && !conflicts(ei, elem(j, "modification"));
+			ok = ok && !does_conflict(ei, elem(j, "modification"));
 		}
 
 		if(ok) {
@@ -678,7 +678,7 @@ bool manager::change_modifications(const std::vector<std::string>& modifications
 		bool ok = true;
 
 		for(const std::string& s : mods_) {
-			ok = ok && !conflicts(era, elem(s, "modification"));
+			ok = ok && !does_conflict(era, elem(s, "modification"));
 		}
 
 		if(ok) {
@@ -715,7 +715,7 @@ bool manager::change_modifications(const std::vector<std::string>& modifications
 		elem scen(c["id"], "scenario");
 		bool ok = true;
 		for(const std::string& s : mods_) {
-			ok = ok && !conflicts(scen, elem(s, "modification"));
+			ok = ok && !does_conflict(scen, elem(s, "modification"));
 		}
 
 		if(ok) {
