@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2013 - 2018 by Andrius Silinskas <silinskas.andrius@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2013 - 2021
+	by Andrius Silinskas <silinskas.andrius@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -24,7 +25,7 @@ namespace ng {
 
 const std::string random_enemy_picture("units/random-dice.png");
 
-/// FLG stands for faction, leader and gender.
+/** FLG stands for faction, leader and gender. */
 class flg_manager
 {
 public:
@@ -82,7 +83,7 @@ public:
 
 private:
 	flg_manager(const flg_manager&) = delete;
-	void operator=(const flg_manager&) = delete;
+	flg_manager& operator=(const flg_manager&) = delete;
 
 	void update_available_factions();
 	void update_available_leaders();
@@ -97,9 +98,9 @@ private:
 	void select_default_faction();
 
 	int faction_index(const config& faction) const;
-	/// returns -1 if no leader with that name was found
+	/** returns -1 if no leader with that name was found */
 	int leader_index(const std::string& leader) const;
-	/// returns -1 if no gender with that name was found
+	/** returns -1 if no gender with that name was found */
 	int gender_index(const std::string& gender) const;
 
 	const std::vector<const config*>& era_factions_;
