@@ -1,36 +1,39 @@
 /*
-   Copyright (C) 2009 - 2018 by Tomasz Sniatowski <kailoran@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Tomasz Sniatowski <kailoran@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "game_initialization/lobby_data.hpp"
 
 #include "config.hpp"
-#include "preferences/credentials.hpp"
-#include "preferences/game.hpp"
-#include "gui/dialogs/campaign_difficulty.hpp"
 #include "filesystem.hpp"
 #include "font/pango/escape.hpp"
 #include "formatter.hpp"
 #include "formula/string_utils.hpp"
 #include "game_config_manager.hpp"
+#include "game_config_view.hpp"
+#include "game_initialization/multiplayer.hpp"
+#include "game_version.hpp"
 #include "gettext.hpp"
+#include "gui/dialogs/campaign_difficulty.hpp"
 #include "lexical_cast.hpp"
 #include "log.hpp"
-#include "map/map.hpp"
 #include "map/exception.hpp"
-#include "terrain/type_data.hpp"
+#include "map/map.hpp"
+#include "mp_game_settings.hpp"
+#include "preferences/credentials.hpp"
+#include "preferences/game.hpp"
 #include "wml_exception.hpp"
-#include "game_version.hpp"
 
 #include <iterator>
 
@@ -48,7 +51,7 @@ static lg::log_domain log_lobby("lobby");
 
 namespace mp {
 
-chat_message::chat_message(const time_t& timestamp,
+chat_message::chat_message(const std::time_t& timestamp,
 						   const std::string& user,
 						   const std::string& message)
 	: timestamp(timestamp), user(user), message(message)
@@ -59,7 +62,7 @@ chat_session::chat_session() : history_()
 {
 }
 
-void chat_session::add_message(const time_t& timestamp,
+void chat_session::add_message(const std::time_t& timestamp,
 						   const std::string& user,
 						   const std::string& message)
 {
@@ -69,7 +72,7 @@ void chat_session::add_message(const time_t& timestamp,
 
 void chat_session::add_message(const std::string& user, const std::string& message)
 {
-	add_message(time(nullptr), user, message);
+	add_message(std::time(nullptr), user, message);
 }
 
 void chat_session::clear()
@@ -107,30 +110,27 @@ void room_info::process_room_members(const config& data)
 
 user_info::user_info(const config& c)
 	: name(c["name"])
+	, forum_id(c["forum_id"].to_int())
 	, game_id(c["game_id"])
-	, relation(ME)
-	, state(game_id == 0 ? LOBBY : GAME)
+	, relation(user_relation::ME)
+	, state(game_id == 0 ? user_state::LOBBY : user_state::GAME)
 	, registered(c["registered"].to_bool())
 	, observing(c["status"] == "observing")
+	, moderator(c["moderator"].to_bool(false))
 {
 	update_relation();
 }
 
-void user_info::update_state(int selected_game_id,
-							 const room_info* current_room /*= nullptr*/)
+void user_info::update_state(int selected_game_id)
 {
 	if(game_id != 0) {
 		if(game_id == selected_game_id) {
-			state = SEL_GAME;
+			state = user_state::SEL_GAME;
 		} else {
-			state = GAME;
+			state = user_state::GAME;
 		}
 	} else {
-		if(current_room != nullptr && current_room->is_member(name)) {
-			state = SEL_ROOM;
-		} else {
-			state = LOBBY;
-		}
+		state = user_state::LOBBY;
 	}
 	update_relation();
 }
@@ -138,13 +138,13 @@ void user_info::update_state(int selected_game_id,
 void user_info::update_relation()
 {
 	if(name == preferences::login()) {
-		relation = ME;
+		relation = user_relation::ME;
 	} else if(preferences::is_ignored(name)) {
-		relation = IGNORED;
+		relation = user_relation::IGNORED;
 	} else if(preferences::is_friend(name)) {
-		relation = FRIEND;
+		relation = user_relation::FRIEND;
 	} else {
-		relation = NEUTRAL;
+		relation = user_relation::NEUTRAL;
 	}
 }
 
@@ -161,30 +161,7 @@ const std::string& spaced_em_dash()
 	return res;
 }
 
-// Returns an abbreviated form of the provided string - ie, 'Ageless Era' should become 'AE'
-std::string make_short_name(const std::string& long_name)
-{
-	if(long_name.empty()) {
-		return "";
-	}
-
-	size_t pos = 0;
-
-	std::stringstream ss;
-	ss << long_name[pos];
-
-	while(pos < long_name.size()) {
-		pos = long_name.find(' ', pos + 1);
-
-		if(pos <= long_name.size() - 2) {
-			ss << long_name[pos + 1];
-		}
-	}
-
-	return ss.str();
-}
-
-std::string make_game_type_marker(std::string text, bool color_for_missing)
+std::string make_game_type_marker(const std::string& text, bool color_for_missing)
 {
 	if(color_for_missing) {
 		return formatter() << "<b><span color='#f00'>[" << text << "]</span></b> ";
@@ -205,7 +182,6 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 	, map_info()
 	, map_size_info()
 	, era()
-	, era_short()
 	, gold(game["mp_village_gold"])
 	, support(game["mp_village_support"])
 	, xp(game["experience_modifier"].str() + "%")
@@ -214,14 +190,13 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 	, time_limit()
 	, vacant_slots()
 	, current_turn(0)
-	, reloaded(game["savegame"].to_bool())
+	, reloaded(game["savegame"].to_enum<mp_game_settings::SAVED_GAME_MODE>(mp_game_settings::SAVED_GAME_MODE::NONE) != mp_game_settings::SAVED_GAME_MODE::NONE)
 	, started(false)
 	, fog(game["mp_fog"].to_bool())
 	, shroud(game["mp_shroud"].to_bool())
 	, observers(game["observer"].to_bool(true))
 	, shuffle_sides(game["shuffle_sides"].to_bool(true))
 	, use_map_settings(game["mp_use_map_settings"].to_bool())
-	, registered_users_only(game["registered_users_only"].to_bool())
 	, private_replay(game["private_replay"].to_bool())
 	, verified(true)
 	, password_required(game["password"].to_bool())
@@ -229,19 +204,19 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 	, have_all_mods(true)
 	, has_friends(false)
 	, has_ignored(false)
-	, display_status(NEW)
+	, display_status(disp_status::NEW)
 	, required_addons()
-	, addons_outcome(SATISFIED)
+	, addons_outcome(addon_req::SATISFIED)
 {
-	const config& game_config = game_config_manager::get()->game_config();
+	const game_config_view& game_config = game_config_manager::get()->game_config();
 
 	// Parse the list of addons required to join this game.
 	for(const config& addon : game.child_range("addon")) {
-		if(addon.has_attribute("id")) {
+		if(addon.has_attribute("id") && addon["require"].to_bool(false)) {
 			if(std::find(installed_addons.begin(), installed_addons.end(), addon["id"].str()) == installed_addons.end()) {
 				required_addon r;
 				r.addon_id = addon["id"].str();
-				r.outcome = NEED_DOWNLOAD;
+				r.outcome = addon_req::NEED_DOWNLOAD;
 
 				// Use addon name if provided, else fall back on the addon id.
 				if(addon.has_attribute("name")) {
@@ -252,8 +227,8 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 
 				required_addons.push_back(std::move(r));
 
-				if(addons_outcome == SATISFIED) {
-					addons_outcome = NEED_DOWNLOAD;
+				if(addons_outcome == addon_req::SATISFIED) {
+					addons_outcome = addon_req::NEED_DOWNLOAD;
 				}
 			}
 		}
@@ -264,28 +239,22 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 		const bool require = game["require_era"].to_bool(true);
 		if(era_cfg) {
 			era = era_cfg["name"].str();
-			era_short = era_cfg["short_name"].str();
-			if(era_short.empty()) {
-				era_short = make_short_name(era);
-			}
 
 			if(require) {
-				ADDON_REQ result = check_addon_version_compatibility(era_cfg, game);
+				addon_req result = check_addon_version_compatibility(era_cfg, game);
 				addons_outcome = std::max(addons_outcome, result); // Elevate to most severe error level encountered so far
 			}
 		} else {
 			have_era = !require;
 			era = game["mp_era_name"].str();
-			era_short = make_short_name(era);
 			verified = false;
 
 			if(!have_era) {
-				addons_outcome = NEED_DOWNLOAD;
+				addons_outcome = addon_req::NEED_DOWNLOAD;
 			}
 		}
 	} else {
 		era = _("Unknown era");
-		era_short = "??";
 		verified = false;
 	}
 
@@ -298,18 +267,18 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 
 		if(cfg["require_modification"].to_bool(false)) {
 			if(const config& mod = game_config.find_child("modification", "id", cfg["id"])) {
-				ADDON_REQ result = check_addon_version_compatibility(mod, game);
+				addon_req result = check_addon_version_compatibility(mod, game);
 				addons_outcome = std::max(addons_outcome, result); // Elevate to most severe error level encountered so far
 			} else {
 				have_all_mods = false;
 				mod_info.back().second = false;
 
-				addons_outcome = NEED_DOWNLOAD;
+				addons_outcome = addon_req::NEED_DOWNLOAD;
 			}
 		}
 	}
 
-	std::sort(mod_info.begin(), mod_info.end(), [](const std::pair<std::string, bool>& lhs, const std::pair<std::string, bool>& rhs) {
+	std::sort(mod_info.begin(), mod_info.end(), [](const auto& lhs, const auto& rhs) {
 		return translation::icompare(lhs.first, rhs.first) < 0;
 	});
 
@@ -323,7 +292,7 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 		info_stream << " — ??×??";
 	} else {
 		try {
-			gamemap map(std::make_shared<terrain_type_data>(game_config), map_data);
+			gamemap map(map_data);
 			std::ostringstream msi;
 			msi << map.w() << font::unicode_multiplication_sign << map.h();
 			map_size_info = msi.str();
@@ -379,12 +348,12 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 			}
 
 			if(require) {
-				ADDON_REQ result = check_addon_version_compatibility((*level_cfg), game);
+				addon_req result = check_addon_version_compatibility((*level_cfg), game);
 				addons_outcome = std::max(addons_outcome, result); // Elevate to most severe error level encountered so far
 			}
 		} else {
 			if(require) {
-				addons_outcome = std::max(addons_outcome, NEED_DOWNLOAD); // Elevate to most severe error level encountered so far
+				addons_outcome = std::max(addons_outcome, addon_req::NEED_DOWNLOAD); // Elevate to most severe error level encountered so far
 			}
 			type_marker = make_game_type_marker(_("scenario_abbreviation^S"), true);
 			scenario = game["mp_scenario_name"].str();
@@ -415,7 +384,7 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 
 			// TODO: should we have this?
 			//if(game["require_scenario"].to_bool(false)) {
-				ADDON_REQ result = check_addon_version_compatibility(campaign_cfg, game);
+				addon_req result = check_addon_version_compatibility(campaign_cfg, game);
 				addons_outcome = std::max(addons_outcome, result); // Elevate to most severe error level encountered so far
 			//}
 		} else {
@@ -430,7 +399,11 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 		verified = false;
 	}
 
-	// Remove any newlines that might have been in game titles
+	// Remove any newlines that might have been in game names (the player-set ones)
+	// No idea how this could happen, but I've seen it (vultraz, 2020-10-26)
+	boost::erase_all(name, "\n");
+
+	// Remove any newlines that might have been in game titles (scenario/campaign name, etc.)
 	boost::replace_all(scenario, "\n", " " + font::unicode_em_dash + " ");
 
 	if(reloaded) {
@@ -493,14 +466,18 @@ game_info::game_info(const config& game, const std::vector<std::string>& install
 	map_info = info_stream.str();
 }
 
-game_info::ADDON_REQ game_info::check_addon_version_compatibility(const config& local_item, const config& game)
+game_info::addon_req game_info::check_addon_version_compatibility(const config& local_item, const config& game)
 {
 	if(!local_item.has_attribute("addon_id") || !local_item.has_attribute("addon_version")) {
-		return SATISFIED;
+		return addon_req::SATISFIED;
 	}
 
 	if(const config& game_req = game.find_child("addon", "id", local_item["addon_id"])) {
-		required_addon r {local_item["addon_id"].str(), SATISFIED, ""};
+		if(!game_req["require"].to_bool(false)) {
+			return addon_req::SATISFIED;
+		}
+
+		required_addon r{local_item["addon_id"].str(), addon_req::SATISFIED, ""};
 
 		// Local version
 		const version_info local_ver(local_item["addon_version"].str());
@@ -525,7 +502,7 @@ game_info::ADDON_REQ game_info::check_addon_version_compatibility(const config&
 				<< "' addon_version='" << local_item["addon_version"]
 				<< "' remote_ver='" << remote_ver.str()
 				<< "'\n";
-			r.outcome = CANNOT_SATISFY;
+			r.outcome = addon_req::CANNOT_SATISFY;
 
 			r.message = VGETTEXT("The host's version of <i>$addon</i> is incompatible. They have version <b>$host_ver</b> while you have version <b>$local_ver</b>.", {
 				{"addon",     local_item["addon_title"].str()},
@@ -539,7 +516,7 @@ game_info::ADDON_REQ game_info::check_addon_version_compatibility(const config&
 
 		// Check if our version is too out of date to play.
 		if(remote_min_ver > local_ver) {
-			r.outcome = NEED_DOWNLOAD;
+			r.outcome = addon_req::NEED_DOWNLOAD;
 
 			r.message = VGETTEXT("Your version of <i>$addon</i> is incompatible. You have version <b>$local_ver</b> while the host has version <b>$host_ver</b>.", {
 				{"addon",     local_item["addon_title"].str()},
@@ -552,7 +529,7 @@ game_info::ADDON_REQ game_info::check_addon_version_compatibility(const config&
 		}
 	}
 
-	return SATISFIED;
+	return addon_req::SATISFIED;
 }
 
 bool game_info::can_join() const
@@ -562,22 +539,22 @@ bool game_info::can_join() const
 
 bool game_info::can_observe() const
 {
-	return observers || preferences::is_authenticated();
+	return observers || mp::logged_in_as_moderator();
 }
 
 const char* game_info::display_status_string() const
 {
 	switch(display_status) {
-		case game_info::CLEAN:
+		case game_info::disp_status::CLEAN:
 			return "clean";
-		case game_info::NEW:
+		case game_info::disp_status::NEW:
 			return "new";
-		case game_info::DELETED:
+		case game_info::disp_status::DELETED:
 			return "deleted";
-		case game_info::UPDATED:
+		case game_info::disp_status::UPDATED:
 			return "updated";
 		default:
-			ERR_CF << "BAD display_status " << display_status << " in game " << id << "\n";
+			ERR_CF << "BAD display_status " << static_cast<int>(display_status) << " in game " << id << "\n";
 			return "?";
 	}
 }
@@ -587,9 +564,9 @@ bool game_info::match_string_filter(const std::string& filter) const
 	const std::string& s1 = name;
 	const std::string& s2 = map_info;
 	return std::search(s1.begin(), s1.end(), filter.begin(), filter.end(),
-			chars_equal_insensitive) != s1.end()
+			utils::chars_equal_insensitive) != s1.end()
 	    || std::search(s2.begin(), s2.end(), filter.begin(), filter.end(),
-			chars_equal_insensitive) != s2.end();
+			utils::chars_equal_insensitive) != s2.end();
 }
 
 }
