@@ -1,19 +1,21 @@
 /*
-   Copyright (C) 2009 - 2018 by Tomasz Sniatowski <kailoran@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Tomasz Sniatowski <kailoran@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
+#include <ctime>
 #include <set>
 #include <deque>
 #include <functional>
@@ -28,11 +30,11 @@ namespace mp {
 struct chat_message
 {
 	/** Create a chat message */
-	chat_message(const time_t& timestamp,
+	chat_message(const std::time_t& timestamp,
 				 const std::string& user,
 				 const std::string& message);
 
-	time_t timestamp;
+	std::time_t timestamp;
 	std::string user;
 	std::string message;
 };
@@ -43,7 +45,7 @@ class chat_session
 public:
 	chat_session();
 
-	void add_message(const time_t& timestamp,
+	void add_message(const std::time_t& timestamp,
 					 const std::string& user,
 					 const std::string& message);
 
@@ -104,19 +106,19 @@ struct user_info
 {
 	explicit user_info(const config& c);
 
-	void update_state(int selected_game_id,
-					  const room_info* current_room = nullptr);
+	void update_state(int selected_game_id);
+
 	void update_relation();
 
-	enum user_relation {
+	enum class user_relation {
 		ME,
 		FRIEND,
 		NEUTRAL,
 		IGNORED
 	};
-	enum user_state {
+
+	enum class user_state {
 		LOBBY,
-		SEL_ROOM,
 		GAME,
 		SEL_GAME
 	};
@@ -124,11 +126,13 @@ struct user_info
 	bool operator<(const user_info& b) const;
 
 	std::string name;
+	int forum_id;
 	int game_id;
 	user_relation relation;
 	user_state state;
 	bool registered;
 	bool observing;
+	bool moderator;
 };
 
 /**
@@ -150,7 +154,6 @@ struct game_info
 	std::string map_info;
 	std::string map_size_info;
 	std::string era;
-	std::string era_short;
 
 	/** List of modification names and whether they're installed or not. */
 	std::vector<std::pair<std::string, bool>> mod_info;
@@ -161,7 +164,7 @@ struct game_info
 	std::string vision;
 	std::string status; // vacant slots or turn info
 	std::string time_limit;
-	size_t vacant_slots;
+	std::size_t vacant_slots;
 
 	unsigned int current_turn;
 	bool reloaded;
@@ -171,7 +174,6 @@ struct game_info
 	bool observers;
 	bool shuffle_sides;
 	bool use_map_settings;
-	bool registered_users_only;
 	bool private_replay;
 	bool verified;
 	bool password_required;
@@ -181,26 +183,27 @@ struct game_info
 	bool has_friends;
 	bool has_ignored;
 
-	enum game_display_status {
+	enum class disp_status {
 		CLEAN,
 		NEW,
 		UPDATED,
 		DELETED
 	};
-	game_display_status display_status;
 
-	enum ADDON_REQ { SATISFIED, NEED_DOWNLOAD, CANNOT_SATISFY };
+	disp_status display_status;
+
+	enum class addon_req { SATISFIED, NEED_DOWNLOAD, CANNOT_SATISFY };
 
 	struct required_addon {
 		std::string addon_id;
-		ADDON_REQ outcome;
+		addon_req outcome;
 		std::string message;
 	};
 
 	std::vector<required_addon> required_addons;
-	ADDON_REQ addons_outcome;
+	addon_req addons_outcome;
 
-	ADDON_REQ check_addon_version_compatibility(const config& local_item, const config& game);
+	addon_req check_addon_version_compatibility(const config& local_item, const config& game);
 
 	const char* display_status_string() const;
 
