@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Tomasz Sniatowski <kailoran@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Tomasz Sniatowski <kailoran@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "game_initialization/lobby_info.hpp"
@@ -39,11 +40,9 @@ lobby_info::lobby_info(const std::vector<std::string>& installed_addons)
 	: installed_addons_(installed_addons)
 	, gamelist_()
 	, gamelist_initialized_(false)
-	, rooms_()
 	, games_by_id_()
 	, games_()
 	, users_()
-	, whispers_()
 	, game_filters_()
 	, game_filter_invert_(false)
 	, games_visibility_()
@@ -134,11 +133,11 @@ bool lobby_info::process_gamelist_diff(const config& data)
 		// the gamelist is now corrupted, stop further processing and wait for a fresh list.
 		gamelist_initialized_ = false;
 		return false;
-	}
-	else {
+	} else {
 		return true;
 	}
 }
+
 bool lobby_info::process_gamelist_diff_impl(const config& data)
 {
 	SCOPE_LB;
@@ -184,19 +183,19 @@ bool lobby_info::process_gamelist_diff_impl(const config& data)
 
 			// Had a game with that id, so update it and mark it as such
 			current_i->second = game_info(c, installed_addons_);
-			current_i->second.display_status = game_info::UPDATED;
+			current_i->second.display_status = game_info::disp_status::UPDATED;
 		} else if(diff_result == "deleted") {
 			if(current_i == games_by_id_.end()) {
 				WRN_LB << "Would have to delete a game that I don't have: " << game_id << std::endl;
 				continue;
 			}
 
-			if(current_i->second.display_status == game_info::NEW) {
+			if(current_i->second.display_status == game_info::disp_status::NEW) {
 				// This means the game never made it through to the user interface,
 				// so just deleting it is fine.
 				games_by_id_.erase(current_i);
 			} else {
-				current_i->second.display_status = game_info::DELETED;
+				current_i->second.display_status = game_info::disp_status::DELETED;
 			}
 		}
 	}
@@ -239,10 +238,10 @@ void lobby_info::process_userlist()
 		}
 
 		switch(ui.relation) {
-		case user_info::FRIEND:
+		case user_info::user_relation::FRIEND:
 			g->has_friends = true;
 			break;
-		case user_info::IGNORED:
+		case user_info::user_relation::IGNORED:
 			g->has_ignored = true;
 			break;
 		default:
@@ -259,10 +258,10 @@ void lobby_info::sync_games_display_status()
 	auto i = games_by_id_.begin();
 
 	while(i != games_by_id_.end()) {
-		if(i->second.display_status == game_info::DELETED) {
+		if(i->second.display_status == game_info::disp_status::DELETED) {
 			i = games_by_id_.erase(i);
 		} else {
-			i->second.display_status = game_info::CLEAN;
+			i->second.display_status = game_info::disp_status::CLEAN;
 			++i;
 		}
 	}
@@ -284,29 +283,25 @@ const game_info* lobby_info::get_game_by_id(int id) const
 	return i == games_by_id_.end() ? nullptr : &i->second;
 }
 
-room_info* lobby_info::get_room(const std::string& name)
+room_info* chat_info::get_room(const std::string& name)
 {
-	for(auto& r : rooms_) {
-		if(r.name() == name) {
-			return &r;
-		}
+	try {
+		return &rooms_.at(name);
+	} catch(const std::out_of_range&) {
+		return nullptr;
 	}
-
-	return nullptr;
 }
 
-const room_info* lobby_info::get_room(const std::string& name) const
+const room_info* chat_info::get_room(const std::string& name) const
 {
-	for(const auto& r : rooms_) {
-		if(r.name() == name) {
-			return &r;
-		}
+	try {
+		return &rooms_.at(name);
+	} catch(const std::out_of_range&) {
+		return nullptr;
 	}
-
-	return nullptr;
 }
 
-bool lobby_info::has_room(const std::string& name) const
+bool chat_info::has_room(const std::string& name) const
 {
 	return get_room(name) != nullptr;
 }
@@ -322,20 +317,16 @@ user_info* lobby_info::get_user(const std::string& name)
 	return nullptr;
 }
 
-void lobby_info::open_room(const std::string& name)
+void chat_info::open_room(const std::string& name)
 {
-	if(!has_room(name)) {
-		rooms_.emplace_back(name);
-	}
+	// TODO: use try_emplace with C++20
+	rooms_.emplace(name, room_info(name));
 }
 
-void lobby_info::close_room(const std::string& name)
+void chat_info::close_room(const std::string& name)
 {
 	DBG_LB << "lobby info: closing room " << name << std::endl;
-
-	if(room_info* r = get_room(name)) {
-		rooms_.erase(rooms_.begin() + (r - &rooms_[0]));
-	}
+	rooms_.erase(name);
 }
 
 void lobby_info::make_games_vector()
@@ -353,6 +344,24 @@ void lobby_info::make_games_vector()
 	games_visibility_.flip();
 }
 
+bool lobby_info::is_game_visible(const game_info& game) {
+	bool show = true;
+
+	for(const auto& filter_func : game_filters_) {
+		show = filter_func(game);
+
+		if(!show) {
+			break;
+		}
+	}
+
+	if(game_filter_invert_) {
+		show = !show;
+	}
+
+	return show;
+}
+
 void lobby_info::apply_game_filter()
 {
 	// Since games_visibility_ is a visibility mask over games_,
@@ -360,28 +369,14 @@ void lobby_info::apply_game_filter()
 	assert(games_visibility_.size() == games_.size());
 
 	for(unsigned i = 0; i < games_.size(); ++i) {
-		bool show = true;
-
-		for(const auto& filter_func : game_filters_) {
-			show = filter_func(*games_[i]);
-
-			if(!show) {
-				break;
-			}
-		}
-
-		if(game_filter_invert_) {
-			show = !show;
-		}
-
-		games_visibility_[i] = show;
+		games_visibility_[i] = is_game_visible(*games_[i]);
 	}
 }
 
-void lobby_info::update_user_statuses(int game_id, const room_info* room)
+void lobby_info::update_user_statuses(int game_id)
 {
 	for(auto& user : users_) {
-		user.update_state(game_id, room);
+		user.update_state(game_id);
 	}
 }
 
