@@ -1,26 +1,26 @@
 /*
-   Copyright (C) 2013 - 2018 by Andrius Silinskas <silinskas.andrius@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2013 - 2021
+	by Andrius Silinskas <silinskas.andrius@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
+
 #include "game_initialization/mp_game_utils.hpp"
 
 #include "config.hpp"
 #include "formula/string_utils.hpp"
-#include "game_config.hpp"
 #include "game_config_manager.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
 #include "saved_game.hpp"
-#include "wesnothd_connection_error.hpp"
 
 static lg::log_domain log_engine("engine");
 #define LOG_NG LOG_STREAM(info, log_engine)
@@ -45,6 +45,8 @@ static void add_multiplayer_classification(config& multiplayer, saved_game& stat
 	multiplayer["difficulty_define"] = state.classification().difficulty;
 	multiplayer["mp_campaign"] = state.classification().campaign;
 	multiplayer["mp_campaign_name"] = state.classification().campaign_name;
+	multiplayer["mp_era"] = state.classification().era_id;
+	multiplayer["active_mods"] = utils::join(state.classification().active_mods, ",");
 }
 
 config initial_level_config(saved_game& state)
@@ -63,7 +65,7 @@ config initial_level_config(saved_game& state)
 	}
 
 	config& scenario = state.get_starting_point();
-	if(!state.mp_settings().saved_game) {
+	if(state.mp_settings().saved_game == mp_game_settings::SAVED_GAME_MODE::NONE) {
 		state.set_random_seed();
 	}
 
@@ -82,7 +84,7 @@ config initial_level_config(saved_game& state)
 	add_multiplayer_classification(level.child_or_add("multiplayer"), state);
 
 	// [multiplayer] mp_era= should be persistent over saves.
-	std::string era = params.mp_era;
+	std::string era = state.classification().era_id;
 
 	/**
 	 * [era] and [modification]s are toplevel tags here.
@@ -93,36 +95,20 @@ config initial_level_config(saved_game& state)
 	 * -- vultraz, 2017-11-24
 	 */
 
-	const config& game_config = game_config_manager::get()->game_config();
+	const game_config_view& game_config = game_config_manager::get()->game_config();
 	const config& era_cfg = game_config.find_child("era", "id", era);
 
-	if(const config& temp_cfg = game_config.find_child("scenario", "id", level.child("multiplayer")["mp_scenario"])) {
-		level.child("multiplayer")["mp_scenario_addon_id"] = temp_cfg["addon_id"].str();
-		level.child("multiplayer")["mp_scenario_addon_version"] = temp_cfg["addon_version"].str();
-	} else if(const config& temp_cfg = game_config.find_child("multiplayer", "id", level.child("multiplayer")["mp_scenario"])) {
-		level.child("multiplayer")["mp_scenario_addon_id"] = temp_cfg["addon_id"].str();
-		level.child("multiplayer")["mp_scenario_addon_version"] = temp_cfg["addon_version"].str();
-	} else {
-		level.child("multiplayer")["mp_scenario_addon_id"] = "";
-		level.child("multiplayer")["mp_scenario_addon_version"] = "";
-	}
-
 	if(!era_cfg) {
-		if(!params.saved_game) {
-			throw config::error(VGETTEXT("Cannot find era $era", {{"era", era}}));
+		if(params.saved_game == mp_game_settings::SAVED_GAME_MODE::NONE) {
+			throw config::error(VGETTEXT("Cannot find era '$era'", {{"era", era}}));
 		}
 
 		// FIXME: @todo We should tell user about missing era but still load game...
-		WRN_CF << "Missing era in MP load game " << era << std::endl;
+		WRN_CF << "Missing era in MP load game '" << era << "'" << std::endl;
 
-		// Otherwise we get an error when when we try to add ai algorithms in mp_staging.
-		level.add_child("era");
 	} else {
 		level.add_child("era", era_cfg);
 
-		level.child("multiplayer")["mp_era_addon_id"] = era_cfg["addon_id"].str();
-		level.child("multiplayer")["mp_era_addon_version"] = era_cfg["addon_version"].str();
-
 		// Initialize the list of sides available for the current era.
 		// We also need this so not to get a segfault in mp_staging for ai configuration.
 		const config& custom_side = game_config.find_child("multiplayer_side", "id", "Custom");
@@ -130,23 +116,16 @@ config initial_level_config(saved_game& state)
 	}
 
 	// Add modifications, needed for ai algorithms which are applied in mp_staging.
-	const std::vector<std::string>& mods = params.active_mods;
-	std::vector<std::string> mod_versions;
-	std::vector<std::string> mod_addon_ids;
+	const std::vector<std::string>& mods = state.classification().active_mods;
 
 	for(unsigned i = 0; i < mods.size(); ++i) {
 		if(const config& mod_cfg = game_config.find_child("modification", "id", mods[i])) {
-			mod_versions.push_back(mod_cfg["addon_version"].str());
-			mod_addon_ids.push_back(mod_cfg["addon_id"].str());
 			level.add_child("modification", mod_cfg);
 		}
 	}
 
-	level.child("multiplayer")["active_mod_versions"] = utils::join(mod_versions);
-	level.child("multiplayer")["active_mod_addon_ids"] = utils::join(mod_addon_ids);
-
 	// This will force connecting clients to be using the same version number as us.
-	level["version"] = game_config::version;
+	level["version"] = game_config::wesnoth_version.str();
 	return level;
 }
 
@@ -157,15 +136,4 @@ void level_to_gamestate(const config& level, saved_game& state)
 	state.classification().campaign_type = type;
 }
 
-void check_response(bool res, const config& data)
-{
-	if(!res) {
-		throw wesnothd_error(_("Connection timed out"));
-	}
-
-	if(const config& err = data.child("error")) {
-		throw wesnothd_error(err["message"]);
-	}
-}
-
 } // end namespace mp
