@@ -1,20 +1,28 @@
 /*
-   Copyright (C) 2008 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "game_version.hpp"
+
+#include "utils/math.hpp"
 #include "lexical_cast.hpp"
 #include "serialization/string_utils.hpp"
+#include "wesconfig.h"
+
+#ifdef LOAD_REVISION
+#include "revision.h"
+#endif
 
 #include <cassert>
 #include <functional>
@@ -22,6 +30,22 @@
 
 #include <boost/algorithm/string.hpp>
 
+namespace game_config
+{
+const version_info wesnoth_version(VERSION);
+const version_info min_savegame_version(MIN_SAVEGAME_VERSION);
+const version_info test_version("test");
+
+#ifdef REVISION
+const std::string revision = VERSION " (" REVISION ")";
+#elif defined(VCS_SHORT_HASH) && defined(VCS_WC_MODIFIED)
+const std::string revision = std::string(VERSION) + " (" + VCS_SHORT_HASH + (VCS_WC_MODIFIED ? "-Modified" : "-Clean") + ")";
+#else
+const std::string revision = VERSION;
+#endif
+
+} // namespace game_config
+
 version_info::version_info()
 	: nums_(3,0), special_(""), special_separator_('\0')
 {
@@ -82,7 +106,7 @@ version_info::version_info(const std::string& str)
 	}
 
 	const std::vector<std::string> components = utils::split(left_side, '.');
-	const size_t s = components.size();
+	const std::size_t s = components.size();
 	if(s == 0) {
 		return;
 	}
@@ -90,17 +114,17 @@ version_info::version_info(const std::string& str)
 		nums_.resize(s, 0);
 	}
 
-	for(size_t i = 0; (i < s); ++i) {
+	for(std::size_t i = 0; (i < s); ++i) {
 		nums_[i] = lexical_cast_default<unsigned int>(components[i]);
 	}
 }
 
 std::string version_info::str() const
 {
-	const size_t s = nums_.size();
+	const std::size_t s = nums_.size();
 
 	std::ostringstream o;
-	for(size_t k = 0; k < s; ++k) {
+	for(std::size_t k = 0; k < s; ++k) {
 		o << nums_[k];
 
 		if(s != 1+k) {
@@ -147,6 +171,10 @@ bool version_info::is_canonical() const {
 	return nums_.size() <= 3;
 }
 
+bool version_info::is_dev_version() const {
+	return is_canonical() && is_odd(minor_version());
+}
+
 namespace {
 	template<template<typename> class Fcn>
 	bool version_comparison_internal(const version_info& l, const version_info& r)
@@ -154,9 +182,9 @@ namespace {
 		std::vector<unsigned int> lc = l.components();
 		std::vector<unsigned int> rc = r.components();
 
-		const size_t lsize = lc.size();
-		const size_t rsize = rc.size();
-		const size_t csize = std::max(lsize, rsize);
+		const std::size_t lsize = lc.size();
+		const std::size_t rsize = rc.size();
+		const std::size_t csize = std::max(lsize, rsize);
 
 		// make compatible, missing items default to zero
 		if(lsize < csize) lc.resize(csize, 0);
