@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -46,6 +47,7 @@ public:
 	version_info();                    /**< Default constructor. */
 	version_info(const std::string&);  /**< String constructor. */
 	version_info(const char* str);
+	version_info(std::nullptr_t) = delete;
 
 	/** Simple list constructor. */
 	version_info(unsigned int major, unsigned int minor, unsigned int revision_level,
@@ -59,6 +61,11 @@ public:
 	 */
 	bool is_canonical() const;
 
+	/**
+	 * Whether this version represents a development version of Wesnoth aka whether the minor version odd.
+	 */
+	bool is_dev_version() const;
+
 	/**
 	 * Serializes the version number into string form.
 	 *
@@ -146,7 +153,7 @@ public:
 	 * @throw std::out_of_range If the number of components is less than
 	 *                          <tt>index - 1</tt>.
 	 */
-	unsigned int get_component(size_t index) const
+	unsigned int get_component(std::size_t index) const
 	{
 		return nums_.at(index);
 	}
@@ -160,7 +167,7 @@ public:
 	 * @throw std::out_of_range If the number of components is less than
 	 *                          <tt>index - 1</tt>.
 	 */
-	void set_component(size_t index, unsigned int value)
+	void set_component(std::size_t index, unsigned int value)
 	{
 		nums_.at(index) = value;
 	}
@@ -204,3 +211,13 @@ enum VERSION_COMP_OP {
 
 VERSION_COMP_OP parse_version_op(const std::string& op_str);
 bool do_version_check(const version_info& a, VERSION_COMP_OP op, const version_info& b);
+
+namespace game_config
+{
+extern const version_info wesnoth_version;
+extern const version_info min_savegame_version;
+extern const version_info test_version;
+
+extern const std::string revision;
+
+} // namespace game_config
