@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -27,6 +28,9 @@
 static lg::log_domain log_engine("engine");
 #define LOG_NG LOG_STREAM(info, log_engine)
 
+static lg::log_domain log_wml("wml");
+#define ERR_WML LOG_STREAM(err, log_wml)
+
 cave_map_generator::cave_map_generator(const config &cfg) :
 	wall_(t_translation::CAVE_WALL),
 	clear_(t_translation::CAVE),
@@ -51,7 +55,7 @@ std::string cave_map_generator::config_name() const
 	return "";
 }
 
-size_t cave_map_generator::cave_map_generator_job::translate_x(size_t x) const
+std::size_t cave_map_generator::cave_map_generator_job::translate_x(std::size_t x) const
 {
 	if(flipx_) {
 		x = params.width_ - x - 1;
@@ -60,7 +64,7 @@ size_t cave_map_generator::cave_map_generator_job::translate_x(size_t x) const
 	return x;
 }
 
-size_t cave_map_generator::cave_map_generator_job::translate_y(size_t y) const
+std::size_t cave_map_generator::cave_map_generator_job::translate_y(std::size_t y) const
 {
 	if(flipy_) {
 		y = params.height_ - y - 1;
@@ -69,19 +73,19 @@ size_t cave_map_generator::cave_map_generator_job::translate_y(size_t y) const
 	return y;
 }
 
-std::string cave_map_generator::create_map(boost::optional<uint32_t> randomseed)
+std::string cave_map_generator::create_map(std::optional<uint32_t> randomseed)
 {
 	const config res = create_scenario(randomseed);
 	return res["map_data"];
 }
 
-config cave_map_generator::create_scenario(boost::optional<uint32_t> randomseed)
+config cave_map_generator::create_scenario(std::optional<uint32_t> randomseed)
 {
 	cave_map_generator_job job(*this, randomseed);
 	return job.res_;
 }
 
-cave_map_generator::cave_map_generator_job::cave_map_generator_job(const cave_map_generator& pparams, boost::optional<uint32_t> randomseed)
+cave_map_generator::cave_map_generator_job::cave_map_generator_job(const cave_map_generator& pparams, std::optional<uint32_t> randomseed)
 	: params(pparams)
 	, flipx_(false)
 	, flipy_(false)
@@ -101,7 +105,7 @@ cave_map_generator::cave_map_generator_job::cave_map_generator_job(const cave_ma
 			"message", "Use the Lua cave generator instead, with scenario_generation=lua and create_scenario= (see wiki for details).",
 		},
 	});
-	uint32_t seed = randomseed.get_ptr() ? *randomseed.get_ptr() : seed_rng::next_seed();
+	uint32_t seed = randomseed ? *randomseed : seed_rng::next_seed();
 	rng_.seed(seed);
 	LOG_NG << "creating random cave with seed: " << seed << '\n';
 	flipx_ = static_cast<int>(rng_() % 100) < params.flipx_chance_;
@@ -125,18 +129,16 @@ cave_map_generator::cave_map_generator_job::cave_map_generator_job(const cave_ma
 	res_["map_data"] = t_translation::write_game_map(map_, starting_positions_);
 }
 
-void cave_map_generator::cave_map_generator_job::build_chamber(map_location loc, std::set<map_location>& locs, size_t size, size_t jagged)
+void cave_map_generator::cave_map_generator_job::build_chamber(map_location loc, std::set<map_location>& locs, std::size_t size, std::size_t jagged)
 {
 	if(size == 0 || locs.count(loc) != 0 || !params.on_board(loc))
 		return;
 
 	locs.insert(loc);
 
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(loc,adj.data());
-	for(size_t n = 0; n < adj.size(); ++n) {
+	for(const map_location& adj : get_adjacent_tiles(loc)) {
 		if(static_cast<int>(rng_() % 100) < (100l - static_cast<long>(jagged))) {
-			build_chamber(adj[n],locs,size-1,jagged);
+			build_chamber(adj,locs,size-1,jagged);
 		}
 	}
 }
@@ -153,7 +155,7 @@ void cave_map_generator::cave_map_generator_job::generate_chambers()
 		const std::string &xpos = ch["x"];
 		const std::string &ypos = ch["y"];
 
-		size_t min_xpos = 0, min_ypos = 0, max_xpos = params.width_, max_ypos = params.height_;
+		std::size_t min_xpos = 0, min_ypos = 0, max_xpos = params.width_, max_ypos = params.height_;
 
 		if (!xpos.empty()) {
 			const std::vector<std::string>& items = utils::split(xpos, '-');
@@ -162,7 +164,8 @@ void cave_map_generator::cave_map_generator_job::generate_chambers()
 					min_xpos = std::stoi(items.front()) - 1;
 					max_xpos = std::stoi(items.back());
 				} catch(const std::invalid_argument&) {
-					lg::wml_error() << "Invalid min/max coordinates in cave_map_generator: " << items.front() << ", " << items.back() << "\n";
+					lg::log_to_chat() << "Invalid min/max coordinates in cave_map_generator: " << items.front() << ", " << items.back() << "\n";
+					ERR_WML << "Invalid min/max coordinates in cave_map_generator: " << items.front() << ", " << items.back();
 					continue;
 				}
 			}
@@ -175,12 +178,13 @@ void cave_map_generator::cave_map_generator_job::generate_chambers()
 					min_ypos = std::stoi(items.front()) - 1;
 					max_ypos = std::stoi(items.back());
 				} catch(const std::invalid_argument&) {
-					lg::wml_error() << "Invalid min/max coordinates in cave_map_generator: " << items.front() << ", " << items.back() << "\n";
+					lg::log_to_chat() << "Invalid min/max coordinates in cave_map_generator: " << items.front() << ", " << items.back() << "\n";
+					ERR_WML << "Invalid min/max coordinates in cave_map_generator: " << items.front() << ", " << items.back();
 				}
 			}
 		}
-		const size_t x = translate_x(min_xpos + (rng_()%(max_xpos-min_xpos)));
-		const size_t y = translate_y(min_ypos + (rng_()%(max_ypos-min_ypos)));
+		const std::size_t x = translate_x(min_xpos + (rng_()%(max_xpos-min_xpos)));
+		const std::size_t y = translate_y(min_ypos + (rng_()%(max_ypos-min_ypos)));
 
 		int chamber_size = ch["size"].to_int(3);
 		int jagged_edges = ch["jagged"];
@@ -204,7 +208,7 @@ void cave_map_generator::cave_map_generator_job::generate_chambers()
 			const std::string &dst = p["destination"];
 
 			// Find the destination of this passage
-			const std::map<std::string,size_t>::const_iterator itor = chamber_ids_.find(dst);
+			const std::map<std::string,std::size_t>::const_iterator itor = chamber_ids_.find(dst);
 			if(itor == chamber_ids_.end())
 				continue;
 
@@ -223,7 +227,7 @@ void cave_map_generator::cave_map_generator_job::place_chamber(const chamber& c)
 
 	if (c.items == nullptr || c.locs.empty()) return;
 
-	size_t index = 0;
+	std::size_t index = 0;
 	for (const config::any_child &it : c.items->all_children_range())
 	{
 		config cfg = it.cfg;
@@ -279,7 +283,7 @@ struct passage_path_calculator : pathfind::cost_calculator
 {
 	passage_path_calculator(const t_translation::ter_map& mapdata,
 	                        const t_translation::terrain_code & wall,
-	                        double laziness, size_t windiness,
+	                        double laziness, std::size_t windiness,
 							std::mt19937& rng) :
 		map_(mapdata), wall_(wall), laziness_(laziness), windiness_(windiness), rng_(rng)
 	{}
@@ -289,7 +293,7 @@ private:
 	const t_translation::ter_map& map_;
 	t_translation::terrain_code wall_;
 	double laziness_;
-	size_t windiness_;
+	std::size_t windiness_;
 	std::mt19937& rng_;
 };
 
@@ -361,9 +365,7 @@ void cave_map_generator::cave_map_generator_job::place_castle(int starting_posit
 		starting_positions_.insert(t_translation::starting_positions::value_type(std::to_string(starting_position), coord));
 	}
 
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(loc,adj.data());
-	for(size_t n = 0; n < adj.size(); ++n) {
-		set_terrain(adj[n], params.castle_);
+	for(const map_location& adj : get_adjacent_tiles(loc)) {
+		set_terrain(adj, params.castle_);
 	}
 }
