@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -254,8 +255,6 @@ default_map_generator_job::default_map_generator_job(uint32_t seed)
  */
 height_map default_map_generator_job::generate_height_map(size_t width, size_t height, size_t iterations, size_t hill_size, size_t island_size, size_t island_off_center)
 {
-	height_map res(width, std::vector<int>(height,0));
-
 	size_t center_x = width/2;
 	size_t center_y = height/2;
 
@@ -285,9 +284,15 @@ height_map default_map_generator_job::generate_height_map(size_t width, size_t h
 			break;
 		}
 	}
+	return generate_height_map(width, height, iterations, hill_size, island_size, center_x, center_y);
+}
+
+height_map default_map_generator_job::generate_height_map(size_t width, size_t height, size_t iterations, size_t hill_size, size_t island_size, size_t center_x, size_t center_y)
+{
+	height_map res(width, std::vector<int>(height,0));
 
 	DBG_NG << iterations << " iterations\n";
-	for(size_t i = 0; i != iterations; ++i) {
+	for(std::size_t i = 0; i != iterations; ++i) {
 
 		// (x1,y1) is the location of the hill,
 		// and 'radius' is the radius of the hill.
@@ -307,9 +312,9 @@ height_map default_map_generator_job::generate_height_map(size_t width, size_t h
 
 		// We have to check whether this is actually a valley
 		if(island_size != 0) {
-			const size_t diffx = std::abs(x1 - int(center_x));
-			const size_t diffy = std::abs(y1 - int(center_y));
-			const size_t dist = size_t(std::sqrt(double(diffx*diffx + diffy*diffy)));
+			const std::size_t diffx = std::abs(x1 - static_cast<int>(center_x));
+			const std::size_t diffy = std::abs(y1 - static_cast<int>(center_y));
+			const std::size_t dist = std::size_t(std::sqrt(static_cast<double>(diffx*diffx + diffy*diffy)));
 			is_valley = dist > island_size;
 		}
 
@@ -345,8 +350,8 @@ height_map default_map_generator_job::generate_height_map(size_t width, size_t h
 
 	// Find the highest and lowest points on the map for normalization:
 	int highest = 0, lowest = 100000, x;
-	for(x = 0; size_t(x) != res.size(); ++x) {
-		for(int y = 0; size_t(y) != res[x].size(); ++y) {
+	for(x = 0; std::size_t(x) != res.size(); ++x) {
+		for(int y = 0; std::size_t(y) != res[x].size(); ++y) {
 			if(res[x][y] > highest) {
 				highest = res[x][y];
 			}
@@ -362,8 +367,8 @@ height_map default_map_generator_job::generate_height_map(size_t width, size_t h
 		<< " highest =" << highest << " \n";
 	// Normalize the heights to the range 0-1000:
 	highest -= lowest;
-	for(x = 0; size_t(x) != res.size(); ++x) {
-		for(int y = 0; size_t(y) != res[x].size(); ++y) {
+	for(x = 0; std::size_t(x) != res.size(); ++x) {
+		for(int y = 0; std::size_t(y) != res[x].size(); ++y) {
 			res[x][y] -= lowest;
 			res[x][y] *= 1000;
 			if(highest != 0) {
@@ -462,8 +467,7 @@ bool default_map_generator_job::generate_river_internal(const height_map& height
 	}
 
 	map_location current_loc(x,y);
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(current_loc,adj.data());
+	auto adj = get_adjacent_tiles(current_loc);
 	std::shuffle(std::begin(adj), std::end(adj), rng_);
 
 	// Mark that we have attempted from this map_location
@@ -500,7 +504,7 @@ std::vector<map_location> default_map_generator_job::generate_river(const height
  * Returns a random tile at one of the borders of a map that is of the given
  * dimensions.
  */
-map_location default_map_generator_job::random_point_at_side(size_t width, size_t height)
+map_location default_map_generator_job::random_point_at_side(std::size_t width, std::size_t height)
 {
 	const int side = rng_()%4;
 	if(side < 2) {
@@ -522,14 +526,14 @@ static std::string output_map(const terrain_map& terrain,
 	// All other segments of the map are there only to give
 	// the important middle part some context.
 	// We also have a border so also adjust for that.
-	const size_t begin_x = terrain.w / 3 - gamemap::default_border ;
-	const size_t end_x = terrain.w * 2 / 3 + gamemap::default_border;
-	const size_t begin_y = terrain.h / 3 - gamemap::default_border;
-	const size_t end_y = terrain.h * 2 / 3 + gamemap::default_border;
+	const std::size_t begin_x = terrain.w / 3 - gamemap::default_border ;
+	const std::size_t end_x = terrain.w * 2 / 3 + gamemap::default_border;
+	const std::size_t begin_y = terrain.h / 3 - gamemap::default_border;
+	const std::size_t end_y = terrain.h * 2 / 3 + gamemap::default_border;
 
 	terrain_map map(end_x - begin_x, end_y - begin_y);
-	for(size_t y = begin_y; y != end_y; ++y) {
-		for(size_t x = begin_x; x != end_x; ++x) {
+	for(std::size_t y = begin_y; y != end_y; ++y) {
+		for(std::size_t x = begin_x; x != end_x; ++x) {
 			map[x - begin_x][y - begin_y] = terrain[x][y];
 		}
 	}
@@ -542,14 +546,14 @@ static std::string output_map(const terrain_map& terrain,
 	return t_translation::write_game_map(map, starting_positions);
 }
 
-static int rank_castle_location(int x, int y, const is_valid_terrain& valid_terrain, int min_x, int max_x, int min_y, int max_y, size_t min_distance, const std::vector<map_location>& other_castles, int highest_ranking)
+static int rank_castle_location(int x, int y, const is_valid_terrain& valid_terrain, int min_x, int max_x, int min_y, int max_y, std::size_t min_distance, const std::vector<map_location>& other_castles, int highest_ranking)
 {
 	const map_location loc(x,y);
 
-	size_t avg_distance = 0, lowest_distance = 1000;
+	std::size_t avg_distance = 0, lowest_distance = 1000;
 
 	for(std::vector<map_location>::const_iterator c = other_castles.begin(); c != other_castles.end(); ++c) {
-		const size_t distance = distance_between(loc,*c);
+		const std::size_t distance = distance_between(loc,*c);
 		if(distance < 6) {
 			return 0;
 		}
@@ -607,7 +611,7 @@ static int rank_castle_location(int x, int y, const is_valid_terrain& valid_terr
 typedef std::map<t_translation::terrain_code, t_translation::ter_list> tcode_list_cache;
 
 static map_location place_village(const t_translation::ter_map& map,
-	const size_t x, const size_t y, const size_t radius, const config& cfg,
+	const std::size_t x, const std::size_t y, const std::size_t radius, const config& cfg,
 	tcode_list_cache &adj_liked_cache)
 {
 	const map_location loc(x,y);
@@ -635,17 +639,12 @@ static map_location place_village(const t_translation::ter_map& map,
 			}
 
 			int rating = child["rating"];
-			adjacent_loc_array_t adj;
-			get_adjacent_tiles(map_location(i.x,i.y),adj.data());
-			for(size_t n = 0; n < adj.size(); ++n) {
-				if(adj[n].x < 0 || adj[n].y < 0 ||
-						adj[n].x >= map.w ||
-						adj[n].y >= map.h) {
-
+			for(const map_location& adj : get_adjacent_tiles({i.x, i.y})) {
+				if(adj.x < 0 || adj.y < 0 || adj.x >= map.w || adj.y >= map.h) {
 					continue;
 				}
 
-				const t_translation::terrain_code t2 = map[adj[n].x][adj[n].y];
+				const t_translation::terrain_code t2 = map[adj.x][adj.y];
 				rating += std::count(adjacent_liked->begin(),adjacent_liked->end(),t2);
 			}
 
@@ -663,28 +662,25 @@ static map_location place_village(const t_translation::ter_map& map,
 static void flood_name(const map_location& start, const std::string& name, std::map<map_location,std::string>& tile_names,
 	const t_translation::ter_match& tile_types, const terrain_map& terrain,
 	unsigned width, unsigned height,
-	size_t label_count, std::map<map_location,std::string>* labels, const std::string& full_name) {
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(start,adj.data());
-	size_t n;
+	std::size_t label_count, std::map<map_location,std::string>* labels, const std::string& full_name) {
+
 	//if adjacent tiles are tiles and unnamed, name them
-	for(n = 0; n < 6; n++) {
+	for(const map_location& adj : get_adjacent_tiles(start)) {
 		//we do not care for tiles outside the middle part
 		//cast to unsigned to skip x < 0 || y < 0 as well.
-		if(static_cast<unsigned>(adj[n].x) >= width / 3 || static_cast<unsigned>(adj[n].y) >= height / 3) {
+		if(static_cast<unsigned>(adj.x) >= width / 3 || static_cast<unsigned>(adj.y) >= height / 3) {
 			continue;
 		}
 
-		const t_translation::terrain_code terr = terrain[adj[n].x + (width / 3)][adj[n].y + (height / 3)];
-		const map_location loc(adj[n].x, adj[n].y);
-		if((t_translation::terrain_matches(terr, tile_types)) && (tile_names.find(loc) == tile_names.end())) {
-			tile_names.emplace(loc, name);
+		const t_translation::terrain_code terr = terrain[adj.x + (width / 3)][adj.y + (height / 3)];
+		if((t_translation::terrain_matches(terr, tile_types)) && (tile_names.find(adj) == tile_names.end())) {
+			tile_names.emplace(adj, name);
 			//labeling decision: this is result of trial and error on what looks best in game
 			if(label_count % 6 == 0) { //ensure that labels do not occur more often than every 6 recursions
-				labels->emplace(loc, full_name);
+				labels->emplace(adj, full_name);
 				label_count++; //ensure that no adjacent tiles get labeled
 			}
-			flood_name(adj[n], name, tile_names, tile_types, terrain, width, height, label_count++, labels, full_name);
+			flood_name(adj, name, tile_names, tile_types, terrain, width, height, label_count++, labels, full_name);
 		}
 	}
 }
@@ -773,8 +769,8 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 	}
 
 	terrain_map terrain(data.width, data.height, grassland);
-	for(size_t x = 0; x != heights.size(); ++x) {
-		for(size_t y = 0; y != heights[x].size(); ++y) {
+	for(std::size_t x = 0; x != heights.size(); ++x) {
+		for(std::size_t y = 0; y != heights[x].size(); ++y) {
 			for(auto i : height_conversion) {
 				if(i.convert_terrain(heights[x][y])) {
 					terrain[x][y] = i.convert_to();
@@ -805,8 +801,8 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 
 	std::map<map_location, std::string> river_names, lake_names, road_names, bridge_names, mountain_names, forest_names, swamp_names;
 
-	const size_t nlakes = data.max_lakes > 0 ? (rng_()%data.max_lakes) : 0;
-	for(size_t lake = 0; lake != nlakes; ++lake) {
+	const std::size_t nlakes = data.max_lakes > 0 ? (rng_()%data.max_lakes) : 0;
+	for(std::size_t lake = 0; lake != nlakes; ++lake) {
 		for(int tries = 0; tries != 100; ++tries) {
 			const int x = rng_()%data.width;
 			const int y = rng_()%data.height;
@@ -822,7 +818,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 				const std::string& name = river_name_generator->generate({{"base",  base_name}});
 				LOG_NG << "Named river '" << name << "'\n";
 
-				size_t name_frequency = 20;
+				std::size_t name_frequency = 20;
 				for(std::vector<map_location>::const_iterator r = river.begin(); r != river.end(); ++r) {
 					const map_location loc(r->x-data.width/3,r->y-data.height/3);
 
@@ -879,7 +875,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 	LOG_NG << "Generated rivers. " << (SDL_GetTicks() - ticks) << " ticks elapsed" << "\n";
 	ticks = SDL_GetTicks();
 
-	const size_t default_dimensions = 40*40*9;
+	const std::size_t default_dimensions = 40*40*9;
 
 	/*
 	 * Convert grassland terrain to other types of flat terrain.
@@ -970,7 +966,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 			}
 
 			if(best_ranking == 0) {
-				ERR_NG << "No castle location found, aborting." << std::endl;
+				ERR_NG << "No castle location found, for " << data.nplayers << " players aborting. " << std::endl;
 				const std::string error = _("No valid castle location found. Too many or too few mountain hexes? (please check the 'max hill size' parameter)");
 				throw mapgen_exception(error);
 			}
@@ -1015,8 +1011,8 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 		dst.y += data.height/3 - 1;
 
 		if(data.link_castles && road < static_cast<int>(castles.size() * castles.size())) {
-			const size_t src_castle = road/castles.size();
-			const size_t dst_castle = road%castles.size();
+			const std::size_t src_castle = road/castles.size();
+			const std::size_t dst_castle = road%castles.size();
 			if(src_castle >= dst_castle) {
 				continue;
 			}
@@ -1080,9 +1076,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 				const map_location& last = *(step-1);
 				const map_location& next = *(step+1);
 
-				adjacent_loc_array_t adj;
-				get_adjacent_tiles(*step,adj.data());
-
+				const auto adj = get_adjacent_tiles(*step);
 				int direction = -1;
 
 				// If we are going north-south
@@ -1112,7 +1106,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 
 				if(direction != -1) {
 					const std::vector<std::string> items = utils::split(convert_to_bridge);
-					if(size_t(direction) < items.size() && !items[direction].empty()) {
+					if(std::size_t(direction) < items.size() && !items[direction].empty()) {
 						terrain[x][y] = t_translation::read_terrain_code(items[direction]);
 					}
 
@@ -1191,7 +1185,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 				if(t_translation::terrain_matches(terr, t_translation::ALL_MOUNTAINS)) {
 					//name every 15th mountain
 					if((rng_() % 15) == 0) {
-						for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+						for(std::size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
 							base_name = base_name_generator->generate();
 							name = mountain_name_generator->generate({{"base",  base_name}});
 						}
@@ -1202,7 +1196,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 					// If the forest tile is not named yet, name it
 					const std::map<map_location, std::string>::const_iterator forest_name = forest_names.find(loc);
 					if(forest_name == forest_names.end()) {
-						for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+						for(std::size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
 							base_name = base_name_generator->generate();
 							name = forest_name_generator->generate({{"base",  base_name}});
 						}
@@ -1214,7 +1208,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 					// If the swamp tile is not named yet, name it
 					const std::map<map_location, std::string>::const_iterator swamp_name = swamp_names.find(loc);
 					if(swamp_name == swamp_names.end()) {
-						for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+						for(std::size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
 							base_name = base_name_generator->generate();
 							name = swamp_name_generator->generate({{"base",  base_name}});
 						}
@@ -1244,8 +1238,8 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 	if(data.nvillages > 0) {
 
 		// First we work out the size of the x and y distance between villages
-		const size_t tiles_per_village = ((data.width*data.height)/9)/data.nvillages;
-		size_t village_x = 1, village_y = 1;
+		const std::size_t tiles_per_village = ((data.width*data.height)/9)/data.nvillages;
+		std::size_t village_x = 1, village_y = 1;
 
 		// Alternate between incrementing the x and y value.
 		// When they are high enough to equal or exceed the tiles_per_village,
@@ -1274,9 +1268,9 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 			LOG_NG << "village at " << vx << "\n";
 
 			for(int vy = rng_()%village_y; vy < data.height; vy += village_y) {
-				const size_t add = rng_()%3;
-				const size_t x = (vx + add) - 1;
-				const size_t y = (vy + add) - 1;
+				const std::size_t add = rng_()%3;
+				const std::size_t x = (vx + add) - 1;
+				const std::size_t y = (vy + add) - 1;
 
 				const map_location res = place_village(terrain, x, y, 2, cfg, adj_liked_cache);
 
@@ -1312,9 +1306,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 					(village_naming.has_attribute("base_names") || village_naming.has_attribute("base_name_generator")) ? "base" : "male" );
 
 				const map_location loc(res.x-data.width/3,res.y-data.height/3);
-
-				adjacent_loc_array_t adj;
-				get_adjacent_tiles(loc,adj.data());
+				const auto adj = get_adjacent_tiles(loc);
 
 				std::string name_type = "village";
 				const t_translation::ter_list
@@ -1323,11 +1315,11 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 					mountain = t_translation::ter_list(1, t_translation::MOUNTAIN),
 					hill	 = t_translation::ter_list(1, t_translation::HILL);
 
-				size_t field_count = 0, forest_count = 0, mountain_count = 0, hill_count = 0;
+				std::size_t field_count = 0, forest_count = 0, mountain_count = 0, hill_count = 0;
 
 				std::map<std::string,std::string> symbols;
 
-				size_t n;
+				std::size_t n;
 				for(n = 0; n != 6; ++n) {
 					const std::map<map_location,std::string>::const_iterator road_name = road_names.find(adj[n]);
 					if(road_name != road_names.end()) {
@@ -1409,7 +1401,7 @@ std::string default_map_generator_job::default_generate_map(generator_data data,
 				symbols["base"] = base_name_generator->generate();
 				std::shared_ptr<name_generator> village_name_generator = village_name_generator_factory.get_name_generator(name_type);
 
-				for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+				for(std::size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
 					name = village_name_generator->generate(symbols);
 				}
 
