@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -284,7 +285,7 @@ public:
 		, callback_load_value_(callback_load_value)
 		, callback_save_value_(callback_save_value)
 	{
-		static_assert(!std::is_same<styled_widget, W>::value, "Second template argument cannot be styled_widget");
+		static_assert(!std::is_same_v<styled_widget, W>, "Second template argument cannot be styled_widget");
 	}
 
 	/**
@@ -308,7 +309,7 @@ public:
 		, callback_load_value_(nullptr)
 		, callback_save_value_(nullptr)
 	{
-		static_assert(!std::is_same<styled_widget, W>::value, "Second template argument cannot be styled_widget");
+		static_assert(!std::is_same_v<styled_widget, W>, "Second template argument cannot be styled_widget");
 	}
 
 	/**
@@ -335,7 +336,7 @@ public:
 		, callback_load_value_(nullptr)
 		, callback_save_value_(nullptr)
 	{
-		static_assert(std::is_same<styled_widget, W>::value, "Second template argument must be styled_widget");
+		static_assert(std::is_same_v<styled_widget, W>, "Second template argument must be styled_widget");
 	}
 
 	/** Inherited from field_base. */
@@ -578,7 +579,7 @@ private:
 					callback_change_(*widget);
 				}
 
-				connect_signal_notify_modified(*widget, std::bind(callback_change_, _1));
+				connect_signal_notify_modified(*widget, std::bind(callback_change_, std::placeholders::_1));
 			}
 		}
 	}
