@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2007 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2007 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -63,11 +64,15 @@ public:
 		 *
 		 * @param canvas          The resulting image will be blitted upon this
 		 *                        canvas.
+		 * @param renderer        The SDL_Renderer to use.
+		 * @param view_bounds     Part of the shape to render - this is the location of @a canvas
+		 *                        within the coordinates of the shape.
 		 * @param variables       The canvas can have formulas in it's
 		 *                        definition, this parameter contains the values
 		 *                        for these formulas.
 		 */
 		virtual void draw(surface& canvas, SDL_Renderer* renderer,
+		                  const SDL_Rect& view_bounds,
 		                  wfl::map_formula_callable& variables) = 0;
 
 		bool immutable() const
@@ -83,25 +88,25 @@ public:
 		bool immutable_;
 	};
 
-	typedef std::shared_ptr<shape> shape_ptr;
-	typedef std::shared_ptr<const shape> const_shape_ptr;
-
 	canvas();
 	canvas(const canvas&) = delete;
-	canvas(canvas&& c) NOEXCEPT;
-
-	~canvas();
+	canvas& operator=(const canvas&) = delete;
+	canvas(canvas&& c) noexcept;
 
+	private:
 	/**
-	 * Draws the canvas.
+	 * Internal part of the blit() function - prepares the contents of the internal viewport_
+	 * surface, reallocating that surface if necessary.
 	 *
+	 * @param area_to_draw        Currently-visible part of the widget, any area outside here won't be blitted to the parent.
 	 * @param force               If the canvas isn't dirty it isn't redrawn
 	 *                            unless force is set to true.
 	 */
-	void draw(const bool force = false);
+	void draw(const SDL_Rect& area_to_draw, const bool force = false);
 
+	public:
 	/**
-	 * Blits the canvas unto another surface.
+	 * Draw the canvas' shapes onto another surface.
 	 *
 	 * It makes sure the image on the canvas is up to date. Also executes the
 	 * pre-blitting functions.
@@ -114,9 +119,8 @@ public:
 	/**
 	 * Sets the config.
 	 *
-	 * @param cfg                 The config object with the data to draw, see
-	 *                            https://www.wesnoth.org/wiki/GUICanvasWML for
-	 *                            more information.
+	 * @param cfg                 The config object with the data to draw.
+	 * @param force               Whether to clear all shapes or not.
 	 */
 	void set_cfg(const config& cfg, const bool force = false)
 	{
@@ -128,9 +132,7 @@ public:
 	/**
 	 * Appends data to the config.
 	 *
-	 * @param cfg                 The config object with the data to draw, see
-	 *                            https://www.wesnoth.org/wiki/GUICanvasWML for
-	 *                            more information.
+	 * @param cfg                 The config object with the data to draw.
 	 */
 	void append_cfg(const config& cfg)
 	{
@@ -163,14 +165,9 @@ public:
 		return h_;
 	}
 
-	surface& surf()
+	void set_variable(const std::string& key, wfl::variant&& value)
 	{
-		return canvas_;
-	}
-
-	void set_variable(const std::string& key, const wfl::variant& value)
-	{
-		variables_.add(key, value);
+		variables_.add(key, std::move(value));
 		set_is_dirty(true);
 		invalidate_cache();
 	}
@@ -182,11 +179,7 @@ public:
 
 private:
 	/** Vector with the shapes to draw. */
-	std::vector<shape_ptr> shapes_;
-
-	/** All shapes which have been already drawn. Kept around in case
-	 * the cache needs to be invalidated. */
-	std::vector<shape_ptr> drawn_shapes_;
+	std::vector<std::unique_ptr<shape>> shapes_;
 
 	/**
 	 * The depth of the blur to use in the pre committing.
@@ -198,16 +191,21 @@ private:
 	 */
 	unsigned blur_depth_;
 
-	/** Width of the canvas. */
+	/** Width of the canvas (the full size, not limited to the view_bounds_). */
 	unsigned w_;
 
-	/** Height of the canvas. */
+	/** Height of the canvas (the full size, not limited to the view_bounds_). */
 	unsigned h_;
 
 	/** The surface we draw all items on. */
-	surface canvas_;
+	surface viewport_;
 
-	SDL_Renderer* renderer_;
+	/**
+	 * The placement and size of viewport_ in the coordinates of this widget; value is not useful when bool(viewport_) is false.
+	 *
+	 * For large widgets, a small viewport_ may be used that contains only the currently-visible part of the widget.
+	 */
+	SDL_Rect view_bounds_;
 
 	/** The variables of the canvas. */
 	wfl::map_formula_callable variables_;
@@ -225,13 +223,16 @@ private:
 	 * the config object is no longer required and thus not stored in the
 	 * object.
 	 *
-	 * @param cfg                 The config object with the data to draw, see
-	 *                            https://www.wesnoth.org/wiki/GUICanvasWML
+	 * @param cfg                 The config object with the data to draw, see @ref GUICanvasWML
 	 */
 	void parse_cfg(const config& cfg);
 
 	void clear_shapes(const bool force);
-	void invalidate_cache();
+
+	void invalidate_cache()
+	{
+		viewport_ = nullptr;
+	}
 };
 
 } // namespace gui2
