@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -57,79 +58,17 @@ void dispatcher::connect()
 bool dispatcher::has_event(const ui_event event, const event_queue_type event_type)
 {
 #if 0
-	// Debug code to test whether the event is in the right queue.
-	std::cerr << "Event '" << event
-			<< "' event "
-			<< find<set_event>(event, dispatcher_implementation
-				::has_handler(event_type, *this))
-			<< " mouse "
-			<< find<set_event_mouse>(event, dispatcher_implementation
-				::has_handler(event_type, *this))
-			<< " keyboard "
-			<< find<set_event_touch_motion>(event, dispatcher_implementation
-				::has_handler(event_type, *this))
-			<< " touch_motion "
-			<< find<set_event_touch_gesture>(event, dispatcher_implementation
-				::has_handler(event_type, *this))
-			<< " touch_gesture "
-			<< find<set_event_keyboard>(event, dispatcher_implementation
-				::has_handler(event_type, *this))
-			<< " notification "
-			<< find<set_event_notification>(event, dispatcher_implementation
-				::has_handler(event_type, *this))
-			<< " message "
-			<< find<set_event_message>(event, dispatcher_implementation
-				::has_handler(event_type, *this))
-			<< ".\n";
+	const bool res = dispatcher_implementation::has_handler(*this, event_type, event);
+	std::cerr << "Event '" << event << " '" << (res ? "found" : "not found") << "in queue\n";
+	return res;
+#else
+	return dispatcher_implementation::has_handler(*this, event_type, event);
 #endif
-
-	return find<set_event>(
-			event, dispatcher_implementation::has_handler(event_type, *this))
-	    || find<set_event_mouse>(
-			event, dispatcher_implementation::has_handler(event_type, *this))
-	    || find<set_event_keyboard>(
-			event, dispatcher_implementation::has_handler(event_type, *this))
-	    || find<set_event_text_input>(
-			event, dispatcher_implementation::has_handler(event_type, *this))
-		   || find<set_event_touch_motion>(
-			event, dispatcher_implementation::has_handler(event_type, *this))
-		   || find<set_event_touch_gesture>(
-			event, dispatcher_implementation::has_handler(event_type, *this))
-	    || find<set_event_notification>(
-			event, dispatcher_implementation::has_handler(event_type, *this))
-	    || find<set_event_message>(
-			event, dispatcher_implementation::has_handler(event_type, *this))
-	    || find<set_event_raw_event>(
-			event, dispatcher_implementation::has_handler(event_type, *this));
 }
 
-/**
- * Helper class to do a runtime test whether an event is in a set.
- *
- * The class is supposed to be used in combination with find function. This
- * function is used in the fire functions to make sure an event is send to the
- * proper handler. If not there will be a run-time assertion failure. This
- * makes developing and testing the code easier, a wrong handler terminates
- * Wesnoth instead of silently not working.
- */
-class event_in_set
-{
-public:
-	/**
-	 * If found we get executed to set the result.
-	 *
-	 * Since we need to return true if found we always return true.
-	 */
-	template <class T>
-	bool oper(ui_event)
-	{
-		return true;
-	}
-};
-
 bool dispatcher::fire(const ui_event event, widget& target)
 {
-	assert(find<set_event>(event, event_in_set()));
+	assert(is_general_event(event));
 	switch(event) {
 		case LEFT_BUTTON_DOUBLE_CLICK:
 			return fire_event_double_click<LEFT_BUTTON_CLICK, LEFT_BUTTON_DOUBLE_CLICK,
@@ -150,7 +89,7 @@ bool dispatcher::fire(const ui_event event, widget& target)
 
 bool dispatcher::fire(const ui_event event, widget& target, const point& coordinate)
 {
-	assert(find<set_event_mouse>(event, event_in_set()));
+	assert(is_mouse_event(event));
 	return fire_event<signal_mouse_function>(event, this, &target, coordinate);
 }
 
@@ -158,46 +97,45 @@ bool dispatcher::fire(const ui_event event,
 		widget& target,
 		const SDL_Keycode key,
 		const SDL_Keymod modifier,
-		const utf8::string& unicode)
+		const std::string& unicode)
 {
-	assert(find<set_event_keyboard>(event, event_in_set()));
+	assert(is_keyboard_event(event));
 	return fire_event<signal_keyboard_function>(event, this, &target, key, modifier, unicode);
 }
 
 bool dispatcher::fire(const ui_event event, widget& target, const point& pos, const point& distance)
 {
-	assert(find<set_event_touch_motion>(event, event_in_set()));
+	assert(is_touch_motion_event(event));
 	return fire_event<signal_touch_motion_function>(event, this, &target, pos, distance);
 }
 
-bool dispatcher::fire(const ui_event event, widget& target, const point& center, float dTheta, float dDist, Uint8 numFingers)
+bool dispatcher::fire(const ui_event event, widget& target, const point& center, float dTheta, float dDist, uint8_t numFingers)
 {
-	assert(find<set_event_touch_gesture>(event, event_in_set()));
+	assert(is_touch_gesture_event(event));
 	return fire_event<signal_touch_gesture_function>(event, this, &target, center, dTheta, dDist, numFingers);
 }
 
 bool dispatcher::fire(const ui_event event, widget& target, const SDL_Event& sdlevent)
 {
-	assert(find<set_event_raw_event>(event, event_in_set()));
+	assert(is_raw_event_event(event));
 	return fire_event<signal_raw_event_function>(event, this, &target, sdlevent);
 }
 
 bool dispatcher::fire(const ui_event event, widget& target, const std::string& text, int32_t start, int32_t len)
 {
-	assert(find<set_event_text_input>(event, event_in_set()));
+	assert(is_text_input_event(event));
 	return fire_event<signal_text_input_function>(event, this, &target, text, start, len);
 }
 
 bool dispatcher::fire(const ui_event event, widget& target, void*)
 {
-	assert(find<set_event_notification>(event, event_in_set()));
+	assert(is_notification_event(event));
 	return fire_event<signal_notification_function>(event, this, &target, nullptr);
 }
 
-// TODO: is there any reason msg isn't a const reference?
-bool dispatcher::fire(const ui_event event, widget& target, message& msg)
+bool dispatcher::fire(const ui_event event, widget& target, const message& msg)
 {
-	assert(find<set_event_message>(event, event_in_set()));
+	assert(is_message_event(event));
 	return fire_event<signal_message_function>(event, this, &target, msg);
 }
 
@@ -251,6 +189,12 @@ void connect_signal_notify_modified(dispatcher& dispatcher, const signal_notific
 	dispatcher.connect_signal<NOTIFY_MODIFIED>(signal);
 }
 
+void connect_signal_on_draw(dispatcher& dispatcher, const signal_function& signal)
+{
+	// TODO: evaluate whether draw events need go in this queue position.
+	dispatcher.connect_signal<DRAW>(signal, dispatcher::front_child);
+}
+
 } // namespace event
 
 } // namespace gui2
@@ -258,7 +202,7 @@ void connect_signal_notify_modified(dispatcher& dispatcher, const signal_notific
 /**
  * @page event_dispatching Event dispatching.
  *
- * @section introduction Introduction
+ * @section introduction-event_dispatching Introduction
  *
  * This page describes how the new event handling system works, since the
  * system is still work in progress it might be out of date with the actual
@@ -318,7 +262,7 @@ void connect_signal_notify_modified(dispatcher& dispatcher, const signal_notific
  * tries sees where the event sticks. This following sections describe how the
  * events are tried for this usage scenario.
  *
- * @subsubsection unhandled Unhandled event
+ * @subsubsection unhandled-all_queues Unhandled event
  *
  * - W pre child
  * - C pre child
@@ -330,13 +274,13 @@ void connect_signal_notify_modified(dispatcher& dispatcher, const signal_notific
  * - C post child
  * - B post child
  *
- * @subsubsection mouse_down Mouse down
+ * @subsubsection mouse_down-all_queues Mouse down
  *
  * - W pre child
  * - C pre child -> set focus -> !handled
  * - B pre child -> set pressed state -> handled
  *
- * @subsubsection mouse_wheel Mouse wheel
+ * @subsubsection mouse_wheel-all_queues Mouse wheel
  *
  * - W pre child
  * - C pre child
@@ -353,7 +297,7 @@ void connect_signal_notify_modified(dispatcher& dispatcher, const signal_notific
  * the last possible widget and to the child of the last widget. The pre queue
  * will be send from top to bottom, the post queue from bottom to top.
  *
- * @subsubsection unhandled Unhandled event
+ * @subsubsection unhandled-chain Unhandled event
  *
  * - W pre child
  * - C pre child
@@ -361,13 +305,13 @@ void connect_signal_notify_modified(dispatcher& dispatcher, const signal_notific
  * - C post child
  * - W post child
  *
- * @subsubsection mouse_down Mouse down
+ * @subsubsection mouse_down-chain Mouse down
  *
  * - W pre child
  * - C pre child -> set focus -> !handled
  * - B child -> set pressed state -> handled
  *
- * @subsubsection mouse_wheel Mouse wheel
+ * @subsubsection mouse_wheel-chain Mouse wheel
  *
  * - W pre child
  * - C pre child
