@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -25,7 +26,7 @@
 #include "gui/widgets/text_box_base.hpp"
 #include "sdl/userevent.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 namespace gui2
 {
@@ -33,44 +34,6 @@ namespace gui2
 namespace event
 {
 
-/**
- * SDL_AddTimer() callback for the hover event.
- *
- * When this callback is called it pushes a new hover event in the event queue.
- *
- * @param interval                The time parameter of SDL_AddTimer.
- * @param param                   Pointer to a widget that's able to show the
- *                                tooltip (will be used as a dispatcher).
- *
- * @returns                       The new timer interval, 0 to stop.
- */
-
-#if 0
-/**
- * SDL_AddTimer() callback for the popup event.
- *
- * This event makes sure the popup is removed again.
- *
- * @param interval                The time parameter of SDL_AddTimer.
- * @param param                   Pointer to parameter structure.
- *
- * @returns                       The new timer interval, 0 to stop.
- */
-static uint32_t popup_callback(uint32_t /*interval*/, void* /*param*/)
-{
-	DBG_GUI_E << "Pushing popup removal event in queue.\n";
-
-	SDL_Event event;
-	sdl::UserEvent data(HOVER_REMOVE_POPUP_EVENT);
-
-	event.type = HOVER_REMOVE_POPUP_EVENT;
-	event.user = data;
-
-	SDL_PushEvent(&event);
-	return 0;
-}
-#endif
-
 /**
  * Small helper to keep a resource (boolean) locked.
  *
@@ -120,35 +83,35 @@ mouse_motion::mouse_motion(widget& owner,
 	owner.connect_signal<event::SDL_MOUSE_MOTION>(
 			std::bind(&mouse_motion::signal_handler_sdl_mouse_motion,
 						this,
-						_2,
-						_3,
-						_5),
+						std::placeholders::_2,
+						std::placeholders::_3,
+						std::placeholders::_5),
 			queue_position);
 
 	owner.connect_signal<event::SDL_TOUCH_MOTION>(
 			std::bind(&mouse_motion::signal_handler_sdl_touch_motion,
 						this,
-						_2,
-						_3,
-						_5,
-						_6),
+						std::placeholders::_2,
+						std::placeholders::_3,
+						std::placeholders::_5,
+						std::placeholders::_6),
 			queue_position);
 
 	owner_.connect_signal<event::SDL_WHEEL_UP>(std::bind(
-			&mouse_motion::signal_handler_sdl_wheel, this, _2, _3, _5));
+			&mouse_motion::signal_handler_sdl_wheel, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5));
 	owner_.connect_signal<event::SDL_WHEEL_DOWN>(std::bind(
-			&mouse_motion::signal_handler_sdl_wheel, this, _2, _3, _5));
+			&mouse_motion::signal_handler_sdl_wheel, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5));
 	owner_.connect_signal<event::SDL_WHEEL_LEFT>(std::bind(
-			&mouse_motion::signal_handler_sdl_wheel, this, _2, _3, _5));
+			&mouse_motion::signal_handler_sdl_wheel, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5));
 	owner_.connect_signal<event::SDL_WHEEL_RIGHT>(std::bind(
-			&mouse_motion::signal_handler_sdl_wheel, this, _2, _3, _5));
+			&mouse_motion::signal_handler_sdl_wheel, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5));
 
 	owner.connect_signal<event::SHOW_HELPTIP>(
 			std::bind(&mouse_motion::signal_handler_show_helptip,
 						this,
-						_2,
-						_3,
-						_5),
+						std::placeholders::_2,
+						std::placeholders::_3,
+						std::placeholders::_5),
 			queue_position);
 }
 
@@ -395,47 +358,92 @@ void mouse_motion::stop_hover_timer()
 
 #undef LOG_HEADER
 #define LOG_HEADER                                                             \
-	"distributor mouse button " << name_ << " [" << owner_.id() << "]: "
+	"distributor mouse button " << events_.name << " [" << owner_.id() << "]: "
 
-template<typename T>
-mouse_button<T>::mouse_button(const std::string& name_, widget& owner,
+mouse_button::mouse_button(const mouse_button_event_types& events, widget& owner,
 		const dispatcher::queue_position queue_position)
 	: mouse_motion(owner, queue_position)
 	, last_click_stamp_(0)
 	, last_clicked_widget_(nullptr)
 	, focus_(nullptr)
-	, name_(name_)
+	, events_(events)
 	, is_down_(false)
 	, signal_handler_sdl_button_down_entered_(false)
 	, signal_handler_sdl_button_up_entered_(false)
 {
-	owner_.connect_signal<T::sdl_button_down_event>(
-			std::bind(&mouse_button<T>::signal_handler_sdl_button_down,
-						this,
-						_2,
-						_3,
-						_5),
-			queue_position);
-	owner_.connect_signal<T::sdl_button_up_event>(
-			std::bind(&mouse_button<T>::signal_handler_sdl_button_up,
-						this,
-						_2,
-						_3,
-						_5),
-			queue_position);
+	// The connect_signal framework is currently using SFINAE checking to ensure that we only
+	// register mouse button signal handlers for mouse buttons. That causes us to need this
+	// hardcoded (either directly or by making mouse_button a templated class), the manual handling
+	// of the three cases here is the current progress on refactoring.
+	switch(events_.sdl_button_down_event) {
+	case event::SDL_LEFT_BUTTON_DOWN: {
+		owner_.connect_signal<event::SDL_LEFT_BUTTON_DOWN>(
+				std::bind(&mouse_button::signal_handler_sdl_button_down,
+							this,
+							std::placeholders::_2,
+							std::placeholders::_3,
+							std::placeholders::_5),
+				queue_position);
+		owner_.connect_signal<event::SDL_LEFT_BUTTON_UP>(
+				std::bind(&mouse_button::signal_handler_sdl_button_up,
+							this,
+							std::placeholders::_2,
+							std::placeholders::_3,
+							std::placeholders::_5),
+				queue_position);
+		break;
+	}
+	case event::SDL_MIDDLE_BUTTON_DOWN: {
+		owner_.connect_signal<event::SDL_MIDDLE_BUTTON_DOWN>(
+				std::bind(&mouse_button::signal_handler_sdl_button_down,
+							this,
+							std::placeholders::_2,
+							std::placeholders::_3,
+							std::placeholders::_5),
+				queue_position);
+		owner_.connect_signal<event::SDL_MIDDLE_BUTTON_UP>(
+				std::bind(&mouse_button::signal_handler_sdl_button_up,
+							this,
+							std::placeholders::_2,
+							std::placeholders::_3,
+							std::placeholders::_5),
+				queue_position);
+		break;
+	}
+	case event::SDL_RIGHT_BUTTON_DOWN: {
+		owner_.connect_signal<event::SDL_RIGHT_BUTTON_DOWN>(
+				std::bind(&mouse_button::signal_handler_sdl_button_down,
+							this,
+							std::placeholders::_2,
+							std::placeholders::_3,
+							std::placeholders::_5),
+				queue_position);
+		owner_.connect_signal<event::SDL_RIGHT_BUTTON_UP>(
+				std::bind(&mouse_button::signal_handler_sdl_button_up,
+							this,
+							std::placeholders::_2,
+							std::placeholders::_3,
+							std::placeholders::_5),
+				queue_position);
+		break;
+	}
+	default: {
+		// There's exactly three instances of this class per instance of distributor, so this assert
+		// will be caught during the build-time-tests.
+		assert(!"Hardcoded assumption about button being LEFT / MIDDLE / RIGHT failed");
+	}
+	}
 }
 
-template<typename T>
-void mouse_button<T>::initialize_state(const bool is_down)
+void mouse_button::initialize_state(int32_t button_state)
 {
 	last_click_stamp_ = 0;
 	last_clicked_widget_ = nullptr;
 	focus_ = 0;
-	is_down_ = is_down;
+	is_down_ = button_state & events_.mask;
 }
 
-template<typename T>
-void mouse_button<T>::signal_handler_sdl_button_down(const event::ui_event event, bool& handled,
+void mouse_button::signal_handler_sdl_button_down(const event::ui_event event, bool& handled,
 		const point& coordinate)
 {
 	if(signal_handler_sdl_button_down_entered_) {
@@ -458,10 +466,10 @@ void mouse_button<T>::signal_handler_sdl_button_down(const event::ui_event event
 	if(mouse_captured_) {
 		assert(mouse_focus_);
 		focus_ = mouse_focus_;
-		DBG_GUI_E << LOG_HEADER << "Firing: " << T::sdl_button_down_event << ".\n";
-		if(!owner_.fire(T::sdl_button_down_event, *focus_, coordinate)) {
-			DBG_GUI_E << LOG_HEADER << "Firing: " << T::button_down_event << ".\n";
-			owner_.fire(T::button_down_event, *mouse_focus_);
+		DBG_GUI_E << LOG_HEADER << "Firing: " << events_.sdl_button_down_event << ".\n";
+		if(!owner_.fire(events_.sdl_button_down_event, *focus_, coordinate)) {
+			DBG_GUI_E << LOG_HEADER << "Firing: " << events_.button_down_event << ".\n";
+			owner_.fire(events_.button_down_event, *mouse_focus_);
 		}
 	} else {
 		widget* mouse_over = owner_.find_at(coordinate, true);
@@ -478,17 +486,16 @@ void mouse_button<T>::signal_handler_sdl_button_down(const event::ui_event event
 		}
 
 		focus_ = mouse_over;
-		DBG_GUI_E << LOG_HEADER << "Firing: " << T::sdl_button_down_event << ".\n";
-		if(!owner_.fire(T::sdl_button_down_event, *focus_, coordinate)) {
-			DBG_GUI_E << LOG_HEADER << "Firing: " << T::button_down_event << ".\n";
-			owner_.fire(T::button_down_event, *focus_);
+		DBG_GUI_E << LOG_HEADER << "Firing: " << events_.sdl_button_down_event << ".\n";
+		if(!owner_.fire(events_.sdl_button_down_event, *focus_, coordinate)) {
+			DBG_GUI_E << LOG_HEADER << "Firing: " << events_.button_down_event << ".\n";
+			owner_.fire(events_.button_down_event, *focus_);
 		}
 	}
 	handled = true;
 }
 
-template<typename T>
-void mouse_button<T>::signal_handler_sdl_button_up(const event::ui_event event, bool& handled,
+void mouse_button::signal_handler_sdl_button_up(const event::ui_event event, bool& handled,
 		const point& coordinate)
 {
 	if(signal_handler_sdl_button_up_entered_) {
@@ -499,20 +506,26 @@ void mouse_button<T>::signal_handler_sdl_button_up(const event::ui_event event,
 	DBG_GUI_E << LOG_HEADER << event << ".\n";
 
 	if(!is_down_) {
+#ifdef GUI2_SHOW_UNHANDLED_EVENT_WARNINGS
 		WRN_GUI_E << LOG_HEADER << event
 				  << ". The mouse button is already up, we missed an event.\n";
+#endif
 		return;
 	}
 	is_down_ = false;
 
 	if(focus_) {
-		DBG_GUI_E << LOG_HEADER << "Firing: " << T::sdl_button_up_event << ".\n";
-		if(!owner_.fire(T::sdl_button_up_event, *focus_, coordinate)) {
-			DBG_GUI_E << LOG_HEADER << "Firing: " << T::button_up_event << ".\n";
-			owner_.fire(T::button_up_event, *focus_);
+		DBG_GUI_E << LOG_HEADER << "Firing: " << events_.sdl_button_up_event << ".\n";
+		if(!owner_.fire(events_.sdl_button_up_event, *focus_, coordinate)) {
+			DBG_GUI_E << LOG_HEADER << "Firing: " << events_.button_up_event << ".\n";
+			owner_.fire(events_.button_up_event, *focus_);
 		}
 	}
 
+	// FIXME: The block below is strange diamond inheritance - it's code that could be in
+	// mouse_motion which applies to all three buttons, but it will be run in one of the
+	// three mouse_button<T> subclasses, and then the other two mouse_button<T> subclasses
+	// will reach here with mouse_captured_ == false.
 	widget* mouse_over = owner_.find_at(coordinate, true);
 	if(mouse_captured_) {
 		const unsigned mask = SDL_BUTTON_LMASK | SDL_BUTTON_MMASK
@@ -539,23 +552,22 @@ void mouse_button<T>::signal_handler_sdl_button_up(const event::ui_event event,
 	handled = true;
 }
 
-template<typename T>
-void mouse_button<T>::mouse_button_click(widget* widget)
+void mouse_button::mouse_button_click(widget* widget)
 {
 	uint32_t stamp = SDL_GetTicks();
 	if(last_click_stamp_ + settings::double_click_time >= stamp
 	   && last_clicked_widget_ == widget) {
 
-		DBG_GUI_E << LOG_HEADER << "Firing: " << T::button_double_click_event << ".\n";
+		DBG_GUI_E << LOG_HEADER << "Firing: " << events_.button_double_click_event << ".\n";
 
-		owner_.fire(T::button_double_click_event, *widget);
+		owner_.fire(events_.button_double_click_event, *widget);
 		last_click_stamp_ = 0;
 		last_clicked_widget_ = nullptr;
 
 	} else {
 
-		DBG_GUI_E << LOG_HEADER << "Firing: " << T::button_click_event << ".\n";
-		owner_.fire(T::button_click_event, *widget);
+		DBG_GUI_E << LOG_HEADER << "Firing: " << events_.button_click_event << ".\n";
+		owner_.fire(events_.button_click_event, *widget);
 		last_click_stamp_ = stamp;
 		last_clicked_widget_ = widget;
 	}
@@ -566,6 +578,40 @@ void mouse_button<T>::mouse_button_click(widget* widget)
 #undef LOG_HEADER
 #define LOG_HEADER "distributor mouse motion [" << owner_.id() << "]: "
 
+namespace
+{
+
+const auto mouse_button_left_events = mouse_button_event_types {
+		SDL_LEFT_BUTTON_DOWN,
+		SDL_LEFT_BUTTON_UP,
+		LEFT_BUTTON_DOWN,
+		LEFT_BUTTON_UP,
+		LEFT_BUTTON_CLICK,
+		LEFT_BUTTON_DOUBLE_CLICK,
+		SDL_BUTTON_LMASK,
+		"left"};
+
+const auto mouse_button_middle_events = mouse_button_event_types {
+		SDL_MIDDLE_BUTTON_DOWN,
+		SDL_MIDDLE_BUTTON_UP,
+		MIDDLE_BUTTON_DOWN,
+		MIDDLE_BUTTON_UP,
+		MIDDLE_BUTTON_CLICK,
+		MIDDLE_BUTTON_DOUBLE_CLICK,
+		SDL_BUTTON_MMASK,
+		"middle"};
+
+const auto mouse_button_right_events = mouse_button_event_types {
+		SDL_RIGHT_BUTTON_DOWN,
+		SDL_RIGHT_BUTTON_UP,
+		RIGHT_BUTTON_DOWN,
+		RIGHT_BUTTON_UP,
+		RIGHT_BUTTON_CLICK,
+		RIGHT_BUTTON_DOUBLE_CLICK,
+		SDL_BUTTON_RMASK,
+		"right"};
+} // anonymous namespace
+
 /**
  * @todo Test whether the state is properly tracked when an input blocker is
  * used.
@@ -573,17 +619,9 @@ void mouse_button<T>::mouse_button_click(widget* widget)
 distributor::distributor(widget& owner,
 						   const dispatcher::queue_position queue_position)
 	: mouse_motion(owner, queue_position)
-	, mouse_button_left("left", owner, queue_position)
-	, mouse_button_middle("middle", owner, queue_position)
-	, mouse_button_right("right", owner, queue_position)
-#if 0
-	, hover_pending_(false)
-	, hover_id_(0)
-	, hover_box_()
-	, had_hover_(false)
-	, tooltip_(nullptr)
-	, help_popup_(nullptr)
-#endif
+	, mouse_button_left(mouse_button_left_events, owner, queue_position)
+	, mouse_button_middle(mouse_button_middle_events, owner, queue_position)
+	, mouse_button_right(mouse_button_right_events, owner, queue_position)
 	, keyboard_focus_(nullptr)
 	, keyboard_focus_chain_()
 {
@@ -594,16 +632,16 @@ distributor::distributor(widget& owner,
 	}
 
 	owner_.connect_signal<event::SDL_KEY_DOWN>(std::bind(
-			&distributor::signal_handler_sdl_key_down, this, _5, _6, _7));
+			&distributor::signal_handler_sdl_key_down, this, std::placeholders::_5, std::placeholders::_6, std::placeholders::_7));
 
 	owner_.connect_signal<event::SDL_TEXT_INPUT>(std::bind(
-			&distributor::signal_handler_sdl_text_input, this, _5, _6, _7));
+			&distributor::signal_handler_sdl_text_input, this, std::placeholders::_5, std::placeholders::_6, std::placeholders::_7));
 
 	owner_.connect_signal<event::SDL_TEXT_EDITING>(std::bind(
-			&distributor::signal_handler_sdl_text_editing, this, _5, _6, _7));
+			&distributor::signal_handler_sdl_text_editing, this, std::placeholders::_5, std::placeholders::_6, std::placeholders::_7));
 
 	owner_.connect_signal<event::NOTIFY_REMOVAL>(std::bind(
-			&distributor::signal_handler_notify_removal, this, _1, _2));
+			&distributor::signal_handler_notify_removal, this, std::placeholders::_1, std::placeholders::_2));
 
 	initialize_state();
 }
@@ -611,25 +649,25 @@ distributor::distributor(widget& owner,
 distributor::~distributor()
 {
 	owner_.disconnect_signal<event::SDL_KEY_DOWN>(std::bind(
-			&distributor::signal_handler_sdl_key_down, this, _5, _6, _7));
+			&distributor::signal_handler_sdl_key_down, this, std::placeholders::_5, std::placeholders::_6, std::placeholders::_7));
 
 	owner_.disconnect_signal<event::SDL_TEXT_INPUT>(std::bind(
-			&distributor::signal_handler_sdl_text_input, this, _5, _6, _7));
+			&distributor::signal_handler_sdl_text_input, this, std::placeholders::_5, std::placeholders::_6, std::placeholders::_7));
 
 	owner_.disconnect_signal<event::SDL_TEXT_EDITING>(std::bind(
-			&distributor::signal_handler_sdl_text_editing, this, _5, _6, _7));
+			&distributor::signal_handler_sdl_text_editing, this, std::placeholders::_5, std::placeholders::_6, std::placeholders::_7));
 
 	owner_.disconnect_signal<event::NOTIFY_REMOVAL>(std::bind(
-			&distributor::signal_handler_notify_removal, this, _1, _2));
+			&distributor::signal_handler_notify_removal, this, std::placeholders::_1, std::placeholders::_2));
 }
 
 void distributor::initialize_state()
 {
-	const uint8_t button_state = SDL_GetMouseState(nullptr, nullptr);
+	const uint32_t button_state = SDL_GetMouseState(nullptr, nullptr);
 
-	mouse_button_left::initialize_state((button_state & SDL_BUTTON(1)) != 0);
-	mouse_button_middle::initialize_state((button_state & SDL_BUTTON(2)) != 0);
-	mouse_button_right::initialize_state((button_state & SDL_BUTTON(3)) != 0);
+	mouse_button_left::initialize_state(button_state);
+	mouse_button_middle::initialize_state(button_state);
+	mouse_button_right::initialize_state(button_state);
 
 	init_mouse_location();
 }
@@ -671,8 +709,7 @@ void distributor::keyboard_add_to_chain(widget* widget)
 void distributor::keyboard_remove_from_chain(widget* w)
 {
 	assert(w);
-	std::vector<widget*>::iterator itor = std::find(
-			keyboard_focus_chain_.begin(), keyboard_focus_chain_.end(), w);
+	auto itor = std::find(keyboard_focus_chain_.begin(), keyboard_focus_chain_.end(), w);
 
 	if(itor != keyboard_focus_chain_.end()) {
 		keyboard_focus_chain_.erase(itor);
@@ -744,17 +781,17 @@ void distributor::signal_handler_keyboard_internal(event::ui_event evt, P1&& p1,
 	}
 }
 
-void distributor::signal_handler_sdl_key_down(const SDL_Keycode key, const SDL_Keymod modifier, const utf8::string& unicode)
+void distributor::signal_handler_sdl_key_down(const SDL_Keycode key, const SDL_Keymod modifier, const std::string& unicode)
 {
 	signal_handler_keyboard_internal<signal_keyboard_function>(event::SDL_KEY_DOWN, key, modifier, unicode);
 }
 
-void distributor::signal_handler_sdl_text_input(const utf8::string& unicode, int32_t start, int32_t end)
+void distributor::signal_handler_sdl_text_input(const std::string& unicode, int32_t start, int32_t end)
 {
 	signal_handler_keyboard_internal<signal_text_input_function>(event::SDL_TEXT_INPUT, unicode, start, end);
 }
 
-void distributor::signal_handler_sdl_text_editing(const utf8::string& unicode, int32_t start, int32_t end)
+void distributor::signal_handler_sdl_text_editing(const std::string& unicode, int32_t start, int32_t end)
 {
 	signal_handler_keyboard_internal<signal_text_input_function>(event::SDL_TEXT_EDITING, unicode, start, end);
 }
@@ -803,10 +840,7 @@ void distributor::signal_handler_notify_removal(dispatcher& w,
 	if(keyboard_focus_ == &w) {
 		keyboard_focus_ = nullptr;
 	}
-	const std::vector<widget*>::iterator itor
-			= std::find(keyboard_focus_chain_.begin(),
-						keyboard_focus_chain_.end(),
-						&w);
+	const auto itor = std::find(keyboard_focus_chain_.begin(), keyboard_focus_chain_.end(), &w);
 	if(itor != keyboard_focus_chain_.end()) {
 		keyboard_focus_chain_.erase(itor);
 	}
