@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -26,11 +27,10 @@
 #include "video.hpp"
 #include "serialization/unicode_cast.hpp"
 #include "sdl/userevent.hpp"
+#include "utils/ranges.hpp"
 
 #include <cassert>
 
-#include <boost/range/adaptor/reversed.hpp>
-
 /**
  * @todo The items below are not implemented yet.
  *
@@ -127,10 +127,10 @@ public:
 	~sdl_event_handler();
 
 	/** Inherited from events::sdl_handler. */
-	void handle_event(const SDL_Event& event);
+	void handle_event(const SDL_Event& event) override;
 
 	/** Inherited from events::sdl_handler. */
-	void handle_window_event(const SDL_Event& event);
+	void handle_window_event(const SDL_Event& event) override;
 
 	/**
 	 * Connects a dispatcher.
@@ -170,7 +170,7 @@ private:
 
 	/** Fires a draw event. */
 	using events::sdl_handler::draw;
-	void draw();
+	void draw() override;
 	void draw_everything();
 
 	/**
@@ -252,7 +252,7 @@ private:
 	 * @param dDist					the amount that the fingers pinched during this motion
 	 * @param numFingers			the number of fingers used in the gesture
 	 */
-	void touch_multi_gesture(const point& center, float dTheta, float dDist, Uint8 numFingers);
+	void touch_multi_gesture(const point& center, float dTheta, float dDist, uint8_t numFingers);
 
 	/**
 	 * Handles a hat motion event.
@@ -295,7 +295,7 @@ private:
 	 */
 	void key_down(const SDL_Keycode key,
 				  const SDL_Keymod modifier,
-				  const utf8::string& unicode);
+				  const std::string& unicode);
 
 	/**
 	 * Fires a text input event.
@@ -344,7 +344,7 @@ private:
 	 * NOTE the keyboard events aren't really wired in yet so doesn't do much.
 	 */
 	dispatcher* keyboard_focus_;
-	friend void capture_keyboard(dispatcher*);
+	friend void capture_keyboard(dispatcher* dispatcher);
 };
 
 sdl_event_handler::sdl_event_handler()
@@ -379,8 +379,8 @@ void sdl_event_handler::handle_event(const SDL_Event& event)
 		return;
 	}
 
-	Uint8 button = event.button.button;
-	CVideo& video = dynamic_cast<window&>(*dispatchers_.back()).video();
+	uint8_t button = event.button.button;
+	CVideo& video = CVideo::get_singleton();
 
 	switch(event.type) {
 		case SDL_MOUSEMOTION:
@@ -426,7 +426,7 @@ void sdl_event_handler::handle_event(const SDL_Event& event)
 			break;
 
 		case TIMER_EVENT:
-			execute_timer(reinterpret_cast<size_t>(event.user.data1));
+			execute_timer(reinterpret_cast<std::size_t>(event.user.data1));
 			break;
 
 		case CLOSE_WINDOW_EVENT:
@@ -598,9 +598,7 @@ void sdl_event_handler::draw()
 	}
 
 	if(!dispatchers_.empty()) {
-		CVideo& video = dynamic_cast<window&>(*dispatchers_.back()).video();
-
-		video.flip();
+		CVideo::get_singleton().flip();
 	}
 }
 
@@ -641,7 +639,7 @@ void sdl_event_handler::mouse(const ui_event event, const point& position)
 		return;
 	}
 
-	for(auto& dispatcher : boost::adaptors::reverse(dispatchers_)) {
+	for(auto& dispatcher : utils::reversed_view(dispatchers_)) {
 		if(dispatcher->get_mouse_behavior() == dispatcher::all) {
 			dispatcher->fire(event, dynamic_cast<widget&>(*dispatcher), position);
 			break;
@@ -721,7 +719,7 @@ dispatcher* sdl_event_handler::keyboard_dispatcher()
 		return keyboard_focus_;
 	}
 
-	for(auto& dispatcher : boost::adaptors::reverse(dispatchers_)) {
+	for(auto& dispatcher : utils::reversed_view(dispatchers_)) {
 		if(dispatcher->get_want_keyboard_input()) {
 			return dispatcher;
 		}
@@ -732,28 +730,28 @@ dispatcher* sdl_event_handler::keyboard_dispatcher()
 
 void sdl_event_handler::touch_motion(const point& position, const point& distance)
 {
-	for(auto& dispatcher : boost::adaptors::reverse(dispatchers_)) {
+	for(auto& dispatcher : utils::reversed_view(dispatchers_)) {
 		dispatcher->fire(SDL_TOUCH_MOTION , dynamic_cast<widget&>(*dispatcher), position, distance);
 	}
 }
 
 void sdl_event_handler::touch_up(const point& position)
 {
-	for(auto& dispatcher : boost::adaptors::reverse(dispatchers_)) {
+	for(auto& dispatcher : utils::reversed_view(dispatchers_)) {
 		dispatcher->fire(SDL_TOUCH_UP, dynamic_cast<widget&>(*dispatcher), position);
 	}
 }
 
 void sdl_event_handler::touch_down(const point& position)
 {
-	for(auto& dispatcher : boost::adaptors::reverse(dispatchers_)) {
+	for(auto& dispatcher : utils::reversed_view(dispatchers_)) {
 		dispatcher->fire(SDL_TOUCH_DOWN, dynamic_cast<widget&>(*dispatcher), position);
 	}
 }
 
-void sdl_event_handler::touch_multi_gesture(const point& center, float dTheta, float dDist, Uint8 numFingers)
+void sdl_event_handler::touch_multi_gesture(const point& center, float dTheta, float dDist, uint8_t numFingers)
 {
-	for(auto& dispatcher : boost::adaptors::reverse(dispatchers_)) {
+	for(auto& dispatcher : utils::reversed_view(dispatchers_)) {
 		dispatcher->fire(SDL_TOUCH_MULTI_GESTURE, dynamic_cast<widget&>(*dispatcher), center, dTheta, dDist, numFingers);
 	}
 }
@@ -831,7 +829,7 @@ bool sdl_event_handler::hotkey_pressed(const hotkey::hotkey_ptr key)
 
 void sdl_event_handler::key_down(const SDL_Keycode key,
 						const SDL_Keymod modifier,
-						const utf8::string& unicode)
+						const std::string& unicode)
 {
 	DBG_GUI_E << "Firing: " << SDL_KEY_DOWN << ".\n";
 
