@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -29,7 +30,7 @@
 
 #pragma once
 
-#include "utils/functional.hpp"
+#include <functional>
 
 #include <SDL2/SDL_types.h>
 
@@ -49,8 +50,8 @@ namespace gui2
  * @returns                       The id of the timer.
  * @retval [0]                    Failed to create a timer.
  */
-size_t add_timer(const uint32_t interval,
-				 const std::function<void(size_t id)>& callback,
+std::size_t add_timer(const uint32_t interval,
+				 const std::function<void(std::size_t id)>& callback,
 				 const bool repeat = false);
 
 /**
@@ -66,7 +67,7 @@ size_t add_timer(const uint32_t interval,
  * @returns                       Status, false if the timer couldn't be
  *                                removed.
  */
-bool remove_timer(const size_t id);
+bool remove_timer(const std::size_t id);
 
 /**
  * Executes a timer.
@@ -80,6 +81,6 @@ bool remove_timer(const size_t id);
  * @returns                       Status, false if the timer couldn't be
  *                                executed.
  */
-bool execute_timer(const size_t id);
+bool execute_timer(const std::size_t id);
 
 } // namespace gui2
