@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2011 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2011 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "gui/dialogs/addon/uninstall_list.hpp"
@@ -23,32 +24,8 @@
 
 #include <algorithm>
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_addon_uninstall_list
- *
- * == Add-on uninstall list ==
- *
- * Dialog with a checkbox list for choosing installed add-ons to remove.
- *
- * @begin{table}{dialog_widgets}
- *
- * addons_list & & listbox & m &
- *     A listbox containing add-on selection entries. $
- *
- * -checkbox & & toggle_button & m &
- *     A toggle button allowing the user to mark/unmark the add-on. $
- *
- * -name & & styled_widget & o &
- *     The name of the add-on. $
- *
- * @end{table}
- */
 
 REGISTER_DIALOG(addon_uninstall_list)
 
@@ -113,4 +90,3 @@ std::vector<std::string> addon_uninstall_list::selected_addons() const
 }
 
 } // namespace dialogs
-} // namespace gui2
