@@ -1,25 +1,29 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "gui/dialogs/campaign_selection.hpp"
 
+#include "font/text_formatting.hpp"
+#include "gui/dialogs/campaign_difficulty.hpp"
 #include "gui/auxiliary/find_widget.hpp"
 #include "gui/widgets/button.hpp"
 #include "gui/widgets/image.hpp"
 #include "gui/widgets/listbox.hpp"
+#include "gui/widgets/menu_button.hpp"
 #include "gui/widgets/multi_page.hpp"
 #include "gui/widgets/multimenu_button.hpp"
 #include "gui/widgets/scroll_label.hpp"
@@ -32,56 +36,17 @@
 #include "lexical_cast.hpp"
 #include "preferences/game.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 #include "utils/irdya_datetime.hpp"
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_campaign_selection
- *
- * == Campaign selection ==
- *
- * This shows the dialog which allows the user to choose which campaign to
- * play.
- *
- * @begin{table}{dialog_widgets}
- *
- * campaign_tree & & tree_view & m &
- *         A tree_view that contains all available campaigns. $
- *
- * -icon & & image & o &
- *         The icon for the campaign. $
- *
- * -name & & styled_widget & o &
- *         The name of the campaign. $
- *
- * -victory & & image & o &
- *         The icon to show when the user finished the campaign. The engine
- *         determines whether or not the user has finished the campaign and
- *         sets the visible flag for the widget accordingly. $
- *
- * campaign_details & & multi_page & m &
- *         A multi page widget that shows more details for the selected
- *         campaign. $
- *
- * -image & & image & o &
- *         The image for the campaign. $
- *
- * -description & & styled_widget & o &
- *         The description of the campaign. $
- *
- * @end{table}
- */
 
 REGISTER_DIALOG(campaign_selection)
 
-void campaign_selection::campaign_selected(window& window)
+void campaign_selection::campaign_selected()
 {
-	tree_view& tree = find_widget<tree_view>(&window, "campaign_tree", false);
+	tree_view& tree = find_widget<tree_view>(get_window(), "campaign_tree", false);
 	if(tree.empty()) {
 		return;
 	}
@@ -96,14 +61,91 @@ void campaign_selection::campaign_selected(window& window)
 			return;
 		}
 
-		multi_page& pages = find_widget<multi_page>(&window, "campaign_details", false);
+		multi_page& pages = find_widget<multi_page>(get_window(), "campaign_details", false);
 		pages.select_page(choice);
 
 		engine_.set_current_level(choice);
+
+		styled_widget& background = find_widget<styled_widget>(get_window(), "campaign_background", false);
+		background.set_label(engine_.current_level().data()["background"].str());
+
+		// Rebuild difficulty menu
+		difficulties_.clear();
+
+		auto& diff_menu = find_widget<menu_button>(get_window(), "difficulty_menu", false);
+
+		const auto& diff_config = generate_difficulty_config(engine_.current_level().data());
+		diff_menu.set_active(diff_config.child_count("difficulty") > 1);
+
+		if(!diff_config.empty()) {
+			std::vector<config> entry_list;
+			unsigned n = 0, selection = 0, max_n = diff_config.child_count("difficulty");
+
+			for(const auto& cfg : diff_config.child_range("difficulty")) {
+				config entry;
+
+				// FIXME: description may have markup that will display weird on the menu_button proper
+				entry["label"] = cfg["label"].str() + " (" + cfg["description"].str() + ")";
+				entry["image"] = cfg["image"].str("misc/blank-hex.png");
+
+				if(preferences::is_campaign_completed(tree.selected_item()->id(), cfg["define"])) {
+					std::string laurel;
+
+					if(n + 1 >= max_n) {
+						laurel = game_config::images::victory_laurel_hardest;
+					} else if(n == 0) {
+						laurel = game_config::images::victory_laurel_easy;
+					} else {
+						laurel = game_config::images::victory_laurel;
+					}
+
+					entry["image"] = laurel + "~BLIT(" + entry["image"] + ")";
+				}
+
+				if(!cfg["description"].empty()) {
+					std::string desc;
+					if(cfg["auto_markup"].to_bool(true) == false) {
+						desc = cfg["description"].str();
+					} else {
+						//desc = "<small>";
+						if(!cfg["old_markup"].to_bool()) {
+							desc += font::span_color(font::GRAY_COLOR) + "(" + cfg["description"].str() + ")</span>";
+						} else {
+							desc += font::span_color(font::GRAY_COLOR) + cfg["description"].str() + "</span>";
+						}
+						//desc += "</small>";
+					}
+
+					// Icons get displayed instead of the labels on the dropdown menu itself,
+					// so we want to prepend each label to its description here
+					desc = cfg["label"].str() + "\n" + desc;
+
+					entry["details"] = std::move(desc);
+				}
+
+				entry_list.emplace_back(std::move(entry));
+				difficulties_.emplace_back(cfg["define"].str());
+
+				if(cfg["default"].to_bool(false)) {
+					selection = n;
+				}
+
+				++n;
+			}
+
+			diff_menu.set_values(entry_list);
+			diff_menu.set_selected(selection);
+		}
 	}
 }
 
-void campaign_selection::sort_campaigns(window& window, campaign_selection::CAMPAIGN_ORDER order, bool ascending)
+void campaign_selection::difficulty_selected()
+{
+	const std::size_t selection = find_widget<menu_button>(get_window(), "difficulty_menu", false).get_value();
+	current_difficulty_ = difficulties_.at(std::min(difficulties_.size() - 1, selection));
+}
+
+void campaign_selection::sort_campaigns(campaign_selection::CAMPAIGN_ORDER order, bool ascending)
 {
 	using level_ptr = ng::create_engine::level_ptr;
 
@@ -147,7 +189,7 @@ void campaign_selection::sort_campaigns(window& window, campaign_selection::CAMP
 		break;
 	}
 
-	tree_view& tree = find_widget<tree_view>(&window, "campaign_tree", false);
+	tree_view& tree = find_widget<tree_view>(get_window(), "campaign_tree", false);
 
 	// Remember which campaign was selected...
 	std::string was_selected;
@@ -182,7 +224,7 @@ void campaign_selection::sort_campaigns(window& window, campaign_selection::CAMP
 	bool exists_in_filtered_result = false;
 	for(unsigned i = 0; i < levels.size(); ++i) {
 		if(show_items[i]) {
-			add_campaign_to_tree(window, levels[i]->data());
+			add_campaign_to_tree(levels[i]->data());
 
 			if (!exists_in_filtered_result) {
 				exists_in_filtered_result = levels[i]->id() == was_selected;
@@ -191,13 +233,13 @@ void campaign_selection::sort_campaigns(window& window, campaign_selection::CAMP
 	}
 
 	if(!was_selected.empty() && exists_in_filtered_result) {
-		find_widget<tree_view_node>(&window, was_selected, false).select_node();
+		find_widget<tree_view_node>(get_window(), was_selected, false).select_node();
 	} else {
-		campaign_selected(window);
+		campaign_selected();
 	}
 }
 
-void campaign_selection::toggle_sorting_selection(window& window, CAMPAIGN_ORDER order)
+void campaign_selection::toggle_sorting_selection(CAMPAIGN_ORDER order)
 {
 	static bool force = false;
 	if(force) {
@@ -221,18 +263,18 @@ void campaign_selection::toggle_sorting_selection(window& window, CAMPAIGN_ORDER
 		force = true;
 
 		if(order == NAME) {
-			find_widget<toggle_button>(&window, "sort_time", false).set_value(0);
+			find_widget<toggle_button>(get_window(), "sort_time", false).set_value(0);
 		} else if(order == DATE) {
-			find_widget<toggle_button>(&window, "sort_name", false).set_value(0);
+			find_widget<toggle_button>(get_window(), "sort_name", false).set_value(0);
 		}
 
 		force = false;
 	}
 
-	sort_campaigns(window, current_sorting_, currently_sorted_asc_);
+	sort_campaigns(current_sorting_, currently_sorted_asc_);
 }
 
-void campaign_selection::filter_text_changed(text_box_base* textbox, const std::string& text)
+void campaign_selection::filter_text_changed(const std::string& text)
 {
 	const std::vector<std::string> words = utils::split(text, ' ');
 
@@ -241,29 +283,29 @@ void campaign_selection::filter_text_changed(text_box_base* textbox, const std::
 	}
 
 	last_search_words_ = words;
-	window& window = *textbox->get_window();
-	sort_campaigns(window, current_sorting_, currently_sorted_asc_);
+	sort_campaigns(current_sorting_, currently_sorted_asc_);
 }
 
 void campaign_selection::pre_show(window& window)
 {
 	text_box* filter = find_widget<text_box>(&window, "filter_box", false, true);
 	filter->set_text_changed_callback(
-			std::bind(&campaign_selection::filter_text_changed, this, _1, _2));
+			std::bind(&campaign_selection::filter_text_changed, this, std::placeholders::_2));
 
 	/***** Setup campaign tree. *****/
 	tree_view& tree = find_widget<tree_view>(&window, "campaign_tree", false);
 
-	tree.set_selection_change_callback(std::bind(&campaign_selection::campaign_selected, this, std::ref(window)));
+	connect_signal_notify_modified(tree,
+		std::bind(&campaign_selection::campaign_selected, this));
 
 	toggle_button& sort_name = find_widget<toggle_button>(&window, "sort_name", false);
 	toggle_button& sort_time = find_widget<toggle_button>(&window, "sort_time", false);
 
 	connect_signal_notify_modified(sort_name,
-		std::bind(&campaign_selection::toggle_sorting_selection, this, std::ref(window), NAME));
+		std::bind(&campaign_selection::toggle_sorting_selection, this, NAME));
 
 	connect_signal_notify_modified(sort_time,
-		std::bind(&campaign_selection::toggle_sorting_selection, this, std::ref(window), DATE));
+		std::bind(&campaign_selection::toggle_sorting_selection, this, DATE));
 
 	window.keyboard_capture(filter);
 	window.add_to_keyboard_chain(&tree);
@@ -275,7 +317,7 @@ void campaign_selection::pre_show(window& window)
 		const config& campaign = level->data();
 
 		/*** Add tree item ***/
-		add_campaign_to_tree(window, campaign);
+		add_campaign_to_tree(campaign);
 
 		/*** Add detail item ***/
 		std::map<std::string, string_map> data;
@@ -317,18 +359,26 @@ void campaign_selection::pre_show(window& window)
 		mods_menu.set_values(mod_menu_values);
 		mods_menu.select_options(mod_states_);
 
-		connect_signal_notify_modified(mods_menu, std::bind(&campaign_selection::mod_toggled, this, std::ref(window)));
+		connect_signal_notify_modified(mods_menu, std::bind(&campaign_selection::mod_toggled, this));
 	} else {
 		mods_menu.set_active(false);
 		mods_menu.set_label(_("active_modifications^None"));
 	}
 
-	campaign_selected(window);
+	//
+	// Set up Difficulty dropdown
+	//
+	menu_button& diff_menu = find_widget<menu_button>(get_window(), "difficulty_menu", false);
+
+	diff_menu.set_use_markup(true);
+	connect_signal_notify_modified(diff_menu, std::bind(&campaign_selection::difficulty_selected, this));
+
+	campaign_selected();
 }
 
-void campaign_selection::add_campaign_to_tree(window& window, const config& campaign)
+void campaign_selection::add_campaign_to_tree(const config& campaign) const
 {
-	tree_view& tree = find_widget<tree_view>(&window, "campaign_tree", false);
+	tree_view& tree = find_widget<tree_view>(get_window(), "campaign_tree", false);
 	std::map<std::string, string_map> data;
 	string_map item;
 
@@ -389,15 +439,16 @@ void campaign_selection::post_show(window& window)
 		}
 	}
 
-	deterministic_ = find_widget<toggle_button>(&window, "checkbox_deterministic", false).get_value_bool();
+
+	rng_mode_ = RNG_MODE(std::clamp<unsigned>(find_widget<menu_button>(&window, "rng_menu", false).get_value(), RNG_DEFAULT, RNG_BIASED));
 
 	preferences::set_modifications(engine_.active_mods(), false);
 }
 
-void campaign_selection::mod_toggled(window& window)
+void campaign_selection::mod_toggled()
 {
 	boost::dynamic_bitset<> new_mod_states =
-		find_widget<multimenu_button>(&window, "mods_menu", false).get_toggle_states();
+		find_widget<multimenu_button>(get_window(), "mods_menu", false).get_toggle_states();
 
 	// Get a mask of any mods that were toggled, regardless of new state
 	mod_states_ = mod_states_ ^ new_mod_states;
@@ -413,4 +464,3 @@ void campaign_selection::mod_toggled(window& window)
 }
 
 } // namespace dialogs
-} // namespace gui2
