@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -24,56 +25,21 @@
 #include "gui/widgets/settings.hpp"
 #include "gui/widgets/window.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_core_selection
- *
- * == Core selection ==
- *
- * This shows the dialog which allows the user to choose which core to
- * play.
- *
- * @begin{table}{dialog_widgets}
- *
- * core_list & & listbox & m &
- *         A listbox that contains all available cores. $
- *
- * -icon & & image & o &
- *         The icon for the core. $
- *
- * -name & & styled_widget & o &
- *         The name of the core. $
- *
- * core_details & & multi_page & m &
- *         A multi page widget that shows more details for the selected
- *         core. $
- *
- * -image & & image & o &
- *         The image for the core. $
- *
- * -description & & styled_widget & o &
- *         The description of the core. $
- *
- * @end{table}
- */
 
 REGISTER_DIALOG(core_selection)
 
-void core_selection::core_selected(window& window)
+void core_selection::core_selected() const
 {
 	const int selected_row
-			= find_widget<listbox>(&window, "core_list", false)
+			= find_widget<listbox>(get_window(), "core_list", false)
 					  .get_selected_row();
 
 	multi_page& pages
-			= find_widget<multi_page>(&window, "core_details", false);
+			= find_widget<multi_page>(get_window(), "core_details", false);
 
 	pages.select_page(selected_row);
 }
@@ -83,7 +49,7 @@ void core_selection::pre_show(window& window)
 	/***** Setup core list. *****/
 	listbox& list = find_widget<listbox>(&window, "core_list", false);
 
-	connect_signal_notify_modified(list, std::bind(&core_selection::core_selected, this, std::ref(window)));
+	connect_signal_notify_modified(list, std::bind(&core_selection::core_selected, this));
 
 	window.keyboard_capture(&list);
 
@@ -118,7 +84,7 @@ void core_selection::pre_show(window& window)
 	}
 	list.select_row(choice_, true);
 
-	core_selected(window);
+	core_selected();
 }
 
 void core_selection::post_show(window& window)
@@ -128,4 +94,3 @@ void core_selection::post_show(window& window)
 }
 
 } // namespace dialogs
-} // namespace gui2
