@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -24,56 +25,12 @@
 #include "gui/widgets/pane.hpp"
 #include "gui/widgets/progress_bar.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 #include <ctime>
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_clock
- *
- * == Clock ==
- *
- * This shows the dialog for keeping track of the drawing events related to the
- * current time. (This window is used for debug purposes only.)
- *
- * @begin{table}{dialog_widgets}
- * hour_percentage   & & progress_bar     & o &
- *         This shows the hours as a percentage, where 24 hours is 100%. $
- * minute_percentage & & progress_bar     & o &
- *         This shows the minutes as a percentage, where 60 minutes is 100%. $
- * second_percentage & & progress_bar     & o &
- *         This shows the seconds as a percentage, where 60 seconds is 100%. $
- *
- * hour              & & integer_selector & o &
- *         This shows the seconds since the beginning of the day. The styled_widget
- *         should have a ''minimum_value'' of 0 and a ''maximum_value'' of 86399
- *         (24 * 60 * 60 - 1). $
- *
- * minute            & & integer_selector & o &
- *         This shows the seconds since the beginning of the current hour. The
- *         styled_widget should have a ''minimum_value'' of 0 and a ''maximum_value''
- *         of 3599 (60 * 60 - 1). $
- *
- * minute            & & integer_selector & o &
- *         This shows the seconds since the beginning of the current minute. The
- *         styled_widget should have a ''minimum_value'' of 0 and a ''maximum_value''
- *         of 59. $
- *
- * clock             & & styled_widget          & o &
- *         A styled_widget which will have set three variables in its canvas:
- *         @* hour, the same value as the hour integer_selector.
- *         @* minute, the same value as the minute integer_selector.
- *         @* second, the same value as the second integer_selector.
- *         @- the styled_widget can then should the time in its own preferred
- *         format(s). $
- * @end{table}
- */
 
 REGISTER_DIALOG(debug_clock)
 
@@ -104,8 +61,7 @@ void debug_clock::pre_show(window& window)
 	clock_ = find_widget<styled_widget>(&window, "clock", false, false);
 
 	signal_ = std::bind(&debug_clock::update_time, this, false);
-	window.connect_signal<event::DRAW>(signal_,
-									   event::dispatcher::front_child);
+	connect_signal_on_draw(window, signal_);
 
 	time_.set_current_time();
 	update_time(true);
@@ -174,8 +130,8 @@ debug_clock::time::time() : hour(0), minute(0), second(0), millisecond(0)
 
 void debug_clock::time::set_current_time()
 {
-	time_t now = ::time(nullptr);
-	tm* stamp = localtime(&now);
+	std::time_t now = ::std::time(nullptr);
+	std::tm* stamp = std::localtime(&now);
 
 	hour = stamp->tm_hour;
 	minute = stamp->tm_min;
@@ -214,4 +170,3 @@ bool debug_clock::time::step(const unsigned milliseconds)
 }
 
 } // namespace dialogs
-} // namespace gui2
