@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2012 - 2018 by Boldizsár Lipka <lipkab@zoho.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2012 - 2021
+	by Boldizsár Lipka <lipkab@zoho.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -22,37 +23,8 @@
 #include "gui/widgets/listbox.hpp"
 #include "gettext.hpp"
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_depcheck_select_new
- *
- * == SP/MP Dependency Check: Select New ==
- *
- * Offers a list of compatible items if a currently selected one is
- * incompatible. Currently used for switching era or map.
- *
- * @begin{table}{dialog_widgets}
- *
- * message & & label & m &
- * 		displays the details of the required changes $
- *
- * itemlist & & listbox & m &
- * 		displays the available items to choose from $
- *
- * cancel & & button & m &
- * 		refuse to apply any changes $
- *
- * ok & & button & m &
- * 		select the chosen item $
- *
- * @end{table}
- *
- */
 
 REGISTER_DIALOG(depcheck_select_new)
 
@@ -108,4 +80,3 @@ void depcheck_select_new::post_show(window& window)
 	}
 }
 } // namespace dialogs
-} // namespace gui2
