@@ -1,41 +1,65 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "gui/dialogs/drop_down_menu.hpp"
-#include "gui/widgets/listbox.hpp"
+
 #include "gui/auxiliary/find_widget.hpp"
+#include "gui/core/event/dispatcher.hpp"
 #include "gui/dialogs/modal_dialog.hpp"
 #include "gui/widgets/image.hpp"
 #include "gui/widgets/integer_selector.hpp"
+#include "gui/widgets/listbox.hpp"
 #include "gui/widgets/scrollbar.hpp"
 #include "gui/widgets/settings.hpp"
+#include "gui/widgets/styled_widget.hpp"
 #include "gui/widgets/toggle_button.hpp"
 #include "gui/widgets/toggle_panel.hpp"
 #include "gui/widgets/window.hpp"
 
 #include "sdl/rect.hpp"
-#include "utils/functional.hpp"
+#include <functional>
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
 REGISTER_DIALOG(drop_down_menu)
 
+drop_down_menu::entry_data::entry_data(const config& cfg)
+	: checkbox()
+	, icon(cfg["icon"].str())
+	, image()
+	, label(cfg["label"].t_str())
+	, details()
+	, tooltip(cfg["tooltip"].t_str())
+{
+	// Checkboxes take precedence in column 1
+	if(cfg.has_attribute("checkbox")) {
+		checkbox = cfg["checkbox"].to_bool(false);
+	}
+
+	// Images take precedence in column 2
+	if(cfg.has_attribute("image")) {
+		image = cfg["image"].str();
+	}
+
+	if(cfg.has_attribute("details")) {
+		details = cfg["details"].t_str();
+	}
+}
+
 namespace
 {
 	void callback_flip_embedded_toggle(window& window)
@@ -55,17 +79,40 @@ namespace
 	void resize_callback(window& window)
 	{
 		window.set_retval(retval::CANCEL);
-		window.close();
 	}
 }
 
-void drop_down_menu::mouse_up_callback(window& window, bool&, bool&, const point& coordinate)
+drop_down_menu::drop_down_menu(styled_widget* parent, const std::vector<config>& items, int selected_item, bool keep_open)
+	: parent_(parent)
+	, items_(items.begin(), items.end())
+	, button_pos_(parent->get_rectangle())
+	, selected_item_(selected_item)
+	, use_markup_(parent->get_use_markup())
+	, keep_open_(keep_open)
+	, mouse_down_happened_(false)
+{
+	set_restore(true);
+}
+
+drop_down_menu::drop_down_menu(SDL_Rect button_pos, const std::vector<config>& items, int selected_item, bool use_markup, bool keep_open)
+	: parent_(nullptr)
+	, items_(items.begin(), items.end())
+	, button_pos_(button_pos)
+	, selected_item_(selected_item)
+	, use_markup_(use_markup)
+	, keep_open_(keep_open)
+	, mouse_down_happened_(false)
+{
+	set_restore(true);
+}
+
+void drop_down_menu::mouse_up_callback(bool&, bool&, const point& coordinate)
 {
 	if(!mouse_down_happened_) {
 		return;
 	}
 
-	listbox& list = find_widget<listbox>(&window, "list", true);
+	listbox& list = find_widget<listbox>(get_window(), "list", true);
 
 	/* Disregard clicks on scrollbars and toggle buttons so the dropdown menu can be scrolled or have an embedded
 	 * toggle button selected without the menu closing.
@@ -73,7 +120,7 @@ void drop_down_menu::mouse_up_callback(window& window, bool&, bool&, const point
 	 * This works since this mouse_up_callback function is called before widgets' left-button-up handlers.
 	 *
 	 * Additionally, this is done before row deselection so selecting/deselecting a toggle button doesn't also leave
-	 * the list with no row visually selected. Oddly, the visial deselection doesn't seem to cause any crashes, and
+	 * the list with no row visually selected. Oddly, the visual deselection doesn't seem to cause any crashes, and
 	 * the previously selected row is reselected when the menu is opened again. Still, it's odd to see your selection
 	 * vanish.
 	 */
@@ -81,7 +128,7 @@ void drop_down_menu::mouse_up_callback(window& window, bool&, bool&, const point
 		return;
 	}
 
-	if(dynamic_cast<toggle_button*>(window.find_at(coordinate, true)) != nullptr) {
+	if(dynamic_cast<toggle_button*>(get_window()->find_at(coordinate, true)) != nullptr) {
 		return;
 	}
 
@@ -97,11 +144,11 @@ void drop_down_menu::mouse_up_callback(window& window, bool&, bool&, const point
 		list.select_row(sel, false);
 	}
 
-	SDL_Rect rect = window.get_rectangle();
+	SDL_Rect rect = get_window()->get_rectangle();
 	if(!sdl::point_in_rect(coordinate, rect)) {
-		window.set_retval(retval::CANCEL);
+		set_retval(retval::CANCEL);
 	} else if(!keep_open_) {
-		window.set_retval(retval::OK);
+		set_retval(retval::OK);
 	}
 }
 
@@ -119,55 +166,55 @@ void drop_down_menu::pre_show(window& window)
 
 	listbox& list = find_widget<listbox>(&window, "list", true);
 
-	std::map<std::string, string_map> data;
-	string_map item;
-
 	for(const auto& entry : items_) {
-		data.clear();
+		std::map<std::string, string_map> data;
+		string_map item;
 
-		item["label"] = entry["icon"];
-		data.emplace("icon", item);
+		//
+		// These widgets can be initialized here since they don't require widget type swapping.
+		//
+		item["use_markup"] = utils::bool_string(use_markup_);
+		if(!entry.checkbox) {
+			item["label"] = entry.icon;
+			data.emplace("icon", item);
+		}
 
-		if(!entry.has_attribute("image")) {
-			item["label"] = entry["label"];
-			item["use_markup"] = utils::bool_string(use_markup_);
+		if(!entry.image) {
+			item["label"] = entry.label;
 			data.emplace("label", item);
 		}
 
-		if(entry.has_attribute("details")) {
-			item["label"] = entry["details"];
-			item["use_markup"] = utils::bool_string(use_markup_);
+		if(entry.details) {
+			item["label"] = *entry.details;
 			data.emplace("details", item);
 		}
 
 		grid& new_row = list.add_row(data);
+		grid& mi_grid = find_widget<grid>(&new_row, "menu_item", false);
 
 		// Set the tooltip on the whole panel
-		find_widget<toggle_panel>(&new_row, "panel", false).set_tooltip(entry["tooltip"]);
+		find_widget<toggle_panel>(&new_row, "panel", false).set_tooltip(entry.tooltip);
 
-		if(entry.has_attribute("image")) {
-			image* img = build_single_widget_instance<image>("image");
-			img->set_label(entry["image"]);
+		if(entry.checkbox) {
+			toggle_button* checkbox = build_single_widget_instance<toggle_button>();
+			checkbox->set_id("checkbox");
+			checkbox->set_value_bool(*entry.checkbox);
 
-			grid* mi_grid = dynamic_cast<grid*>(new_row.find("menu_item", false));
-			if(mi_grid) {
-				mi_grid->swap_child("label", img, false);
+			// Fire a NOTIFIED_MODIFIED event in the parent widget when the toggle state changes
+			if(parent_) {
+				connect_signal_notify_modified(*checkbox, std::bind([this]() {
+					parent_->fire(event::NOTIFY_MODIFIED, *parent_, nullptr);
+				}));
 			}
+
+			mi_grid.swap_child("icon", checkbox, false);
 		}
 
-		if(entry.has_attribute("checkbox")) {
-			toggle_button* checkbox = build_single_widget_instance<toggle_button>("toggle_button");
-			checkbox->set_id("checkbox");
-			checkbox->set_value_bool(entry["checkbox"].to_bool(false));
+		if(entry.image) {
+			image* img = build_single_widget_instance<image>();
+			img->set_label(*entry.image);
 
-			if(callback_toggle_state_change_ != nullptr) {
-				connect_signal_notify_modified(*checkbox, std::bind(callback_toggle_state_change_));
-			}
-
-			grid* mi_grid = dynamic_cast<grid*>(new_row.find("menu_item", false));
-			if(mi_grid) {
-				mi_grid->swap_child("icon", checkbox, false);
-			}
+			mi_grid.swap_child("label", img, false);
 		}
 	}
 
@@ -177,30 +224,28 @@ void drop_down_menu::pre_show(window& window)
 
 	window.keyboard_capture(&list);
 
-	// Dismiss on click outside the window
+	// Dismiss on clicking outside the window.
 	window.connect_signal<event::SDL_LEFT_BUTTON_UP>(
-		std::bind(&drop_down_menu::mouse_up_callback, this, std::ref(window), _3, _4, _5), event::dispatcher::front_child);
+		std::bind(&drop_down_menu::mouse_up_callback, this, std::placeholders::_3, std::placeholders::_4, std::placeholders::_5), event::dispatcher::front_child);
 
 	window.connect_signal<event::SDL_RIGHT_BUTTON_UP>(
-		std::bind(&drop_down_menu::mouse_up_callback, this, std::ref(window), _3, _4, _5), event::dispatcher::front_child);
+		std::bind(&drop_down_menu::mouse_up_callback, this, std::placeholders::_3, std::placeholders::_4, std::placeholders::_5), event::dispatcher::front_child);
 
 	window.connect_signal<event::SDL_LEFT_BUTTON_DOWN>(
 		std::bind(&drop_down_menu::mouse_down_callback, this), event::dispatcher::front_child);
 
-	// Handle embedded button toggling.
-	// For some reason this works as a listbox value callback but don't ask me why.
-	// - vultraz, 2017-02-17
-	connect_signal_notify_modified(list, std::bind(&callback_flip_embedded_toggle, std::ref(window)));
+	// Dismiss on resize.
+	window.connect_signal<event::SDL_VIDEO_RESIZE>(
+		std::bind(&resize_callback, std::ref(window)), event::dispatcher::front_child);
 
-	// Dismiss on resize
-	window.connect_signal<event::SDL_VIDEO_RESIZE>(std::bind(&resize_callback, std::ref(window)), event::dispatcher::front_child);
+	// Handle embedded button toggling.
+	connect_signal_notify_modified(list,
+		std::bind(&callback_flip_embedded_toggle, std::ref(window)));
 }
 
 void drop_down_menu::post_show(window& window)
 {
-	listbox& list = find_widget<listbox>(&window, "list", true);
-
-	selected_item_ = list.get_selected_row();
+	selected_item_ = find_widget<listbox>(&window, "list", true).get_selected_row();
 }
 
 boost::dynamic_bitset<> drop_down_menu::get_toggle_states() const
@@ -223,4 +268,3 @@ boost::dynamic_bitset<> drop_down_menu::get_toggle_states() const
 }
 
 } // namespace dialogs
-} // namespace gui2
