@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Jörg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Jörg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -18,32 +19,34 @@
 
 #include "desktop/open.hpp"
 #include "filesystem.hpp"
+#include "font/pango/escape.hpp"
 #include "formatter.hpp"
 #include "formula/string_utils.hpp"
-#include "gettext.hpp"
-#include "game_config.hpp"
-#include "preferences/game.hpp"
 #include "game_classification.hpp"
+#include "game_config.hpp"
+#include "gettext.hpp"
 #include "gui/auxiliary/field.hpp"
 #include "gui/core/log.hpp"
 #include "gui/dialogs/game_delete.hpp"
 #include "gui/widgets/button.hpp"
 #include "gui/widgets/image.hpp"
 #include "gui/widgets/label.hpp"
-#include "gui/widgets/scroll_label.hpp"
 #include "gui/widgets/listbox.hpp"
 #include "gui/widgets/minimap.hpp"
+#include "gui/widgets/scroll_label.hpp"
 #include "gui/widgets/settings.hpp"
 #include "gui/widgets/text_box.hpp"
 #include "gui/widgets/toggle_button.hpp"
 #include "gui/widgets/window.hpp"
-#include "picture.hpp"
 #include "language.hpp"
+#include "picture.hpp"
+#include "preferences/game.hpp"
 #include "serialization/string_utils.hpp"
 #include "utils/general.hpp"
+#include <functional>
+#include "game_config_view.hpp"
 
 #include <cctype>
-#include "utils/functional.hpp"
 
 static lg::log_domain log_gameloaddlg{"gui/dialogs/game_load_dialog"};
 #define ERR_GAMELOADDLG   LOG_STREAM(err,   log_gameloaddlg)
@@ -51,57 +54,40 @@ static lg::log_domain log_gameloaddlg{"gui/dialogs/game_load_dialog"};
 #define LOG_GAMELOADDLG   LOG_STREAM(info,  log_gameloaddlg)
 #define DBG_GAMELOADDLG   LOG_STREAM(debug, log_gameloaddlg)
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_game_load
- *
- * == Load a game ==
- *
- * This shows the dialog to select and load a savegame file.
- *
- * @begin{table}{dialog_widgets}
- *
- * txtFilter & & text & m &
- *         The filter for the listbox items. $
- *
- * savegame_list & & listbox & m &
- *         List of savegames. $
- *
- * -filename & & styled_widget & m &
- *         Name of the savegame. $
- *
- * -date & & styled_widget & o &
- *         Date the savegame was created. $
- *
- * -minimap & & minimap & m &
- *         Minimap of the selected savegame. $
- *
- * -imgLeader & & image & m &
- *         The image of the leader in the selected savegame. $
- *
- * -lblScenario & & label & m &
- *         The name of the scenario of the selected savegame. $
- *
- * -lblSummary & & label & m &
- *         Summary of the selected savegame. $
- *
- * @end{table}
- */
 
 REGISTER_DIALOG(game_load)
 
-game_load::game_load(const config& cache_config, savegame::load_game_metadata& data)
+bool game_load::execute(const game_config_view& cache_config, savegame::load_game_metadata& data)
+{
+	if(savegame::save_index_class::default_saves_dir()->get_saves_list().empty()) {
+		bool found_files = false;
+		for(const auto& dir : filesystem::find_other_version_saves_dirs()) {
+			if(!found_files) {
+				// this needs to be a shared_ptr because get_saves_list() uses shared_from_this
+				auto index = std::make_shared<savegame::save_index_class>(dir.path);
+				found_files = !index->get_saves_list().empty();
+			}
+		}
+
+		if(!found_files) {
+			gui2::show_transient_message(_("No Saved Games"), _("There are no saved games to load."));
+			return false;
+		}
+	}
+
+	return game_load(cache_config, data).show();
+}
+
+game_load::game_load(const game_config_view& cache_config, savegame::load_game_metadata& data)
 	: filename_(data.filename)
+	, save_index_manager_(data.manager)
 	, change_difficulty_(register_bool("change_difficulty", true, data.select_difficulty))
 	, show_replay_(register_bool("show_replay", true, data.show_replay))
 	, cancel_orders_(register_bool("cancel_orders", true, data.cancel_orders))
 	, summary_(data.summary)
-	, games_({savegame::get_saves_list()})
+	, games_()
 	, cache_config_(cache_config)
 	, last_words_()
 {
@@ -110,25 +96,71 @@ game_load::game_load(const config& cache_config, savegame::load_game_metadata& d
 void game_load::pre_show(window& window)
 {
 	// Allow deleting saves with the Delete key.
-	connect_signal_pre_key_press(window, std::bind(&game_load::key_press_callback, this, std::ref(window), _5));
-
-	find_widget<minimap>(&window, "minimap", false).set_config(&cache_config_);
+	connect_signal_pre_key_press(window, std::bind(&game_load::key_press_callback, this, std::placeholders::_5));
 
 	text_box* filter = find_widget<text_box>(&window, "txtFilter", false, true);
 
-	filter->set_text_changed_callback(
-			std::bind(&game_load::filter_text_changed, this, _1, _2));
+	filter->set_text_changed_callback(std::bind(&game_load::filter_text_changed, this, std::placeholders::_2));
 
 	listbox& list = find_widget<listbox>(&window, "savegame_list", false);
 
-	connect_signal_notify_modified(list,
-			std::bind(&game_load::display_savegame, this, std::ref(window)));
+	connect_signal_notify_modified(list, std::bind(&game_load::display_savegame, this));
 
 	window.keyboard_capture(filter);
 	window.add_to_keyboard_chain(&list);
 
+	list.register_sorting_option(0, [this](const int i) { return games_[i].name(); });
+	list.register_sorting_option(1, [this](const int i) { return games_[i].modified(); });
+
+	populate_game_list();
+
+	connect_signal_mouse_left_click(find_widget<button>(&window, "delete", false),
+			std::bind(&game_load::delete_button_callback, this));
+
+	connect_signal_mouse_left_click(find_widget<button>(&window, "browse_saves_folder", false),
+			std::bind(&game_load::browse_button_callback, this));
+
+	menu_button& dir_list = find_widget<menu_button>(&window, "dirList", false);
+
+	dir_list.set_use_markup(true);
+	set_save_dir_list(dir_list);
+
+	connect_signal_notify_modified(dir_list, std::bind(&game_load::handle_dir_select, this));
+
+	display_savegame();
+}
+
+void game_load::set_save_dir_list(menu_button& dir_list)
+{
+	const auto other_dirs = filesystem::find_other_version_saves_dirs();
+	if(other_dirs.empty()) {
+		dir_list.set_visible(widget::visibility::invisible);
+		return;
+	}
+
+	std::vector<config> options;
+
+	// The first option in the list is the current version's save dir
+	options.emplace_back("label",  _("game_version^Current Version"), "path", "");
+
+	for(const auto& known_dir : other_dirs) {
+		options.emplace_back(
+			"label", VGETTEXT("game_version^Wesnoth $version", utils::string_map{{"version", known_dir.version}}),
+			"path", known_dir.path
+		);
+	}
+
+	dir_list.set_values(options);
+}
+
+void game_load::populate_game_list()
+{
+	listbox& list = find_widget<listbox>(get_window(), "savegame_list", false);
+
 	list.clear();
 
+	games_ = save_index_manager_->get_saves_list();
+
 	for(const auto& game : games_) {
 		std::map<std::string, string_map> data;
 		string_map item;
@@ -144,41 +176,21 @@ void game_load::pre_show(window& window)
 		list.add_row(data);
 	}
 
-	list.register_sorting_option(0, [this](const int i) { return games_[i].name(); });
-	list.register_sorting_option(1, [this](const int i) { return games_[i].modified(); });
-
-	connect_signal_mouse_left_click(
-			find_widget<button>(&window, "delete", false),
-			std::bind(&game_load::delete_button_callback,
-					this, std::ref(window)));
-
-	connect_signal_mouse_left_click(
-		find_widget<button>(&window, "browse_saves_folder", false),
-		bind_void(&desktop::open_object, filesystem::get_saves_dir()));
-
-	display_savegame(window);
+	find_widget<button>(get_window(), "delete", false).set_active(!save_index_manager_->read_only());
 }
 
-void game_load::display_savegame_internal(window& window)
+void game_load::display_savegame_internal(const savegame::save_info& game)
 {
-	const int selected_row =
-		find_widget<listbox>(&window, "savegame_list", false).get_selected_row();
-
-	if(selected_row == -1) {
-		return;
-	}
-
-	savegame::save_info& game = games_[selected_row];
 	filename_ = game.name();
 	summary_  = game.summary();
 
-	find_widget<minimap>(&window, "minimap", false)
+	find_widget<minimap>(get_window(), "minimap", false)
 			.set_map_data(summary_["map_data"]);
 
-	find_widget<label>(&window, "lblScenario", false)
+	find_widget<label>(get_window(), "lblScenario", false)
 			.set_label(summary_["label"]);
 
-	listbox& leader_list = find_widget<listbox>(&window, "leader_list", false);
+	listbox& leader_list = find_widget<listbox>(get_window(), "leader_list", false);
 
 	leader_list.clear();
 
@@ -193,7 +205,7 @@ void game_load::display_savegame_internal(window& window)
 		// work, we fallback on unknown-unit.png.
 		std::string leader_image = leader["leader_image"].str();
 		if(!::image::exists(leader_image)) {
-			leader_image = filesystem::get_independent_image_path(leader_image);
+			leader_image = filesystem::get_independent_binary_file_path("images", leader_image);
 
 			// The leader TC modifier isn't appending if the independent image path can't
 			// be resolved during save_index entry creation, so we need to add it here.
@@ -232,8 +244,8 @@ void game_load::display_savegame_internal(window& window)
 	evaluate_summary_string(str, summary_);
 
 	// The new label value may have more or less lines than the previous value, so invalidate the layout.
-	find_widget<scroll_label>(&window, "slblSummary", false).set_label(str.str());
-	window.invalidate_layout();
+	find_widget<scroll_label>(get_window(), "slblSummary", false).set_label(str.str());
+	get_window()->invalidate_layout();
 
 	toggle_button& replay_toggle            = dynamic_cast<toggle_button&>(*show_replay_->get_widget());
 	toggle_button& cancel_orders_toggle     = dynamic_cast<toggle_button&>(*cancel_orders_->get_widget());
@@ -255,22 +267,34 @@ void game_load::display_savegame_internal(window& window)
 
 // This is a wrapper that prevents a corrupted save file (if it happens to be
 // the first in the list) from making the dialog fail to open.
-void game_load::display_savegame(window& window)
+void game_load::display_savegame()
 {
+	bool successfully_displayed_a_game = false;
+
 	try {
-		game_load::display_savegame_internal(window);
+		const int selected_row = find_widget<listbox>(get_window(), "savegame_list", false).get_selected_row();
+		if(selected_row < 0) {
+			find_widget<button>(get_window(), "delete", false).set_active(false);
+		} else {
+			find_widget<button>(get_window(), "delete", false).set_active(!save_index_manager_->read_only());
+			game_load::display_savegame_internal(games_[selected_row]);
+			successfully_displayed_a_game = true;
+		}
 	} catch(const config::error& e) {
 		// Clear the UI widgets, show an error message.
 		const std::string preamble = _("The selected file is corrupt: ");
 		const std::string message = e.message.empty() ? "(no details)" : e.message;
 		ERR_GAMELOADDLG << preamble << message << "\n";
-		find_widget<minimap>(&window, "minimap", false).set_map_data("");
-		find_widget<label>(&window, "lblScenario", false)
-			.set_label(preamble);
-		find_widget<scroll_label>(&window, "slblSummary", false)
-			.set_label(message);
+	}
 
-		listbox& leader_list = find_widget<listbox>(&window, "leader_list", false);
+	if(!successfully_displayed_a_game) {
+		find_widget<minimap>(get_window(), "minimap", false).set_map_data("");
+		find_widget<label>(get_window(), "lblScenario", false)
+			.set_label("");
+		find_widget<scroll_label>(get_window(), "slblSummary", false)
+			.set_label("");
+
+		listbox& leader_list = find_widget<listbox>(get_window(), "leader_list", false);
 		leader_list.clear();
 
 		toggle_button& replay_toggle            = dynamic_cast<toggle_button&>(*show_replay_->get_widget());
@@ -281,13 +305,17 @@ void game_load::display_savegame(window& window)
 		cancel_orders_toggle.set_active(false);
 		change_difficulty_toggle.set_active(false);
 	}
+
+	// Disable Load button if nothing is selected or if the currently selected file can't be loaded
+	find_widget<button>(get_window(), "ok", false).set_active(successfully_displayed_a_game);
+
+	// Disable 'Enter' loading in the same circumstance
+	get_window()->set_enter_disabled(!successfully_displayed_a_game);
 }
 
-void game_load::filter_text_changed(text_box_base* textbox, const std::string& text)
+void game_load::filter_text_changed(const std::string& text)
 {
-	window& window = *textbox->get_window();
-
-	listbox& list = find_widget<listbox>(&window, "savegame_list", false);
+	listbox& list = find_widget<listbox>(get_window(), "savegame_list", false);
 
 	const std::vector<std::string> words = utils::split(text, ' ');
 
@@ -299,21 +327,16 @@ void game_load::filter_text_changed(text_box_base* textbox, const std::string& t
 	show_items.resize(list.get_item_count(), true);
 
 	if(!text.empty()) {
-		for(unsigned int i = 0; i < list.get_item_count(); i++) {
-			grid* row = list.get_row_grid(i);
-
-			grid::iterator it = row->begin();
-			label& filename_label = find_widget<label>(*it, "filename", false);
-
+		for(unsigned int i = 0; i < list.get_item_count() && i < games_.size(); i++) {
 			bool found = false;
 			for(const auto & word : words)
 			{
-				found = std::search(filename_label.get_label().str().begin(),
-									filename_label.get_label().str().end(),
+				found = std::search(games_[i].name().begin(),
+									games_[i].name().end(),
 									word.begin(),
 									word.end(),
-									chars_equal_insensitive)
-						!= filename_label.get_label().str().end();
+									utils::chars_equal_insensitive)
+						!= games_[i].name().end();
 
 				if(!found) {
 					// one word doesn't match, we don't reach words.end()
@@ -326,32 +349,25 @@ void game_load::filter_text_changed(text_box_base* textbox, const std::string& t
 	}
 
 	list.set_row_shown(show_items);
-
-	const bool any_shown = list.any_rows_shown();
-
-	// Disable Load button if no games are available
-	find_widget<button>(&window, "ok", false).set_active(any_shown);
-
-	// Disable 'Enter' loading if no games are available
-	window.set_enter_disabled(!any_shown);
 }
 
 void game_load::evaluate_summary_string(std::stringstream& str, const config& cfg_summary)
 {
-	std::string difficulty_human_str = string_table[cfg_summary["difficulty"]];
 	if(cfg_summary["corrupt"].to_bool()) {
 		str << "\n<span color='#f00'>" << _("(Invalid)") << "</span>";
-
+		// \todo: this skips the catch() statement in display_savegame. Low priority, as the
+		// dialog's state is reasonable; the "load" button is inactive, the "delete" button is
+		// active, and (cosmetic bug) it leaves the "change difficulty" toggle active. Can be
+		// triggered by creating an empty file in the save directory.
 		return;
 	}
 
 	const std::string& campaign_type = cfg_summary["campaign_type"];
+	const std::string campaign_id = cfg_summary["campaign"];
 
 	try {
 		switch(game_classification::CAMPAIGN_TYPE::string_to_enum(campaign_type).v) {
 			case game_classification::CAMPAIGN_TYPE::SCENARIO: {
-				const std::string campaign_id = cfg_summary["campaign"];
-
 				const config* campaign = nullptr;
 				if(!campaign_id.empty()) {
 					if(const config& c = cache_config_.find_child("campaign", "id", campaign_id)) {
@@ -359,16 +375,6 @@ void game_load::evaluate_summary_string(std::stringstream& str, const config& cf
 					}
 				}
 
-				if (campaign != nullptr) {
-					try {
-						const config &difficulty = campaign->find_child("difficulty", "define", cfg_summary["difficulty"]);
-						std::ostringstream ss;
-						ss << difficulty["label"] << " (" << difficulty["description"] << ")";
-						difficulty_human_str = ss.str();
-					} catch(const config::error&) {
-					}
-				}
-
 				utils::string_map symbols;
 				if(campaign != nullptr) {
 					symbols["campaign_name"] = (*campaign)["name"];
@@ -409,8 +415,49 @@ void game_load::evaluate_summary_string(std::stringstream& str, const config& cf
 		str << _("Scenario start");
 	}
 
-	str << "\n" << _("Difficulty: ")
-		<< difficulty_human_str;
+	str << "\n" << _("Difficulty: ");
+	try {
+		switch (game_classification::CAMPAIGN_TYPE::string_to_enum(campaign_type).v) {
+		case game_classification::CAMPAIGN_TYPE::SCENARIO:
+		case game_classification::CAMPAIGN_TYPE::MULTIPLAYER: {
+			const config* campaign = nullptr;
+			if (!campaign_id.empty()) {
+				if (const config& c = cache_config_.find_child("campaign", "id", campaign_id)) {
+					campaign = &c;
+				}
+			}
+
+			// 'SCENARIO' or SP should only ever be campaigns
+			// 'MULTIPLAYER' may be a campaign with difficulty or single scenario without difficulty
+			// For the latter do not show the difficulty - even though it will be listed as
+			// NORMAL -> Medium in the save file it should not be considered valid (GitHub Issue #5321)
+			if (campaign != nullptr) {
+				try {
+					const config& difficulty = campaign->find_child("difficulty", "define", cfg_summary["difficulty"]);
+					std::ostringstream ss;
+					ss << difficulty["label"] << " (" << difficulty["description"] << ")";
+					str << ss.str();
+				}
+				catch (const config::error&) {
+					// fall back to standard difficulty string in case of exception
+					str << string_table[cfg_summary["difficulty"]];
+				}
+			}
+			else {
+				str << "—";
+			}
+
+			break;
+		}
+		case game_classification::CAMPAIGN_TYPE::TUTORIAL:
+		case game_classification::CAMPAIGN_TYPE::TEST:
+			str << "—";
+			break;
+		}
+	}
+	catch (const bad_enum_cast&) {
+		str << "—";
+	}
 
 	if(!cfg_summary["version"].empty()) {
 		str << "\n" << _("Version: ") << cfg_summary["version"];
@@ -432,12 +479,16 @@ void game_load::evaluate_summary_string(std::stringstream& str, const config& cf
 		}
 	}
 }
-
-void game_load::delete_button_callback(window& window)
+void game_load::browse_button_callback()
 {
-	listbox& list = find_widget<listbox>(&window, "savegame_list", false);
+	desktop::open_object(save_index_manager_->dir());
+}
 
-	const size_t index = size_t(list.get_selected_row());
+void game_load::delete_button_callback()
+{
+	listbox& list = find_widget<listbox>(get_window(), "savegame_list", false);
+
+	const std::size_t index = std::size_t(list.get_selected_row());
 	if(index < games_.size()) {
 
 		// See if we should ask the user for deletion confirmation
@@ -448,23 +499,18 @@ void game_load::delete_button_callback(window& window)
 		}
 
 		// Delete the file
-		savegame::delete_game(games_[index].name());
+		save_index_manager_->delete_game(games_[index].name());
 
 		// Remove it from the list of saves
 		games_.erase(games_.begin() + index);
 
 		list.remove_row(index);
 
-		// Close the dialog if there are no more saves
-		if(list.get_item_count() == 0) {
-			window.set_retval(retval::CANCEL);
-		}
-
-		display_savegame(window);
+		display_savegame();
 	}
 }
 
-void game_load::key_press_callback(window& window, const SDL_Keycode key)
+void game_load::key_press_callback(const SDL_Keycode key)
 {
 	//
 	// Don't delete games when we're typing in the textbox!
@@ -475,14 +521,28 @@ void game_load::key_press_callback(window& window, const SDL_Keycode key)
 	//
 	// - vultraz, 2017-08-28
 	//
-	if(find_widget<text_box>(&window, "txtFilter", false).get_state() == text_box_base::FOCUSED) {
+	if(find_widget<text_box>(get_window(), "txtFilter", false).get_state() == text_box_base::FOCUSED) {
 		return;
 	}
 
 	if(key == SDLK_DELETE) {
-		delete_button_callback(window);
+		delete_button_callback();
 	}
 }
 
+void game_load::handle_dir_select()
+{
+	menu_button& dir_list = find_widget<menu_button>(get_window(), "dirList", false);
+
+	const auto& path = dir_list.get_value_config()["path"].str();
+	if(path.empty()) {
+		save_index_manager_ = savegame::save_index_class::default_saves_dir();
+	} else {
+		save_index_manager_ = std::make_shared<savegame::save_index_class>(path);
+	}
+
+	populate_game_list();
+	display_savegame();
+}
+
 } // namespace dialogs
-} // namespace gui2
