@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Jörg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Jörg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -22,29 +23,8 @@
 #include "gui/widgets/label.hpp"
 #include "gui/widgets/settings.hpp"
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_game_save
- *
- * == Save a game ==
- *
- * This shows the dialog to create a savegame file.
- *
- * @begin{table}{dialog_widgets}
- *
- * lblTitle & & label & m &
- *         The title of the window. $
- *
- * txtFilename & & text_box & m &
- *         The name of the savefile. $
- *
- * @end{table}
- */
 
 REGISTER_DIALOG(game_save)
 
@@ -86,4 +66,3 @@ game_save_oos::game_save_oos(bool& ignore_all,
 }
 
 } // namespace dialogs
-} // namespace gui2
