@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -43,7 +44,7 @@
 #include "filter_context.hpp"
 
 #include <vector>
-#include "utils/functional.hpp"
+#include <functional>
 
 namespace
 {
@@ -66,35 +67,8 @@ inline std::string config_to_string(const config& cfg, std::string only_children
 
 }
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_gamestate_inspector
- *
- * == Gamestate inspector ==
- *
- * This shows the gamestate inspector
- *
- * @begin{table}{dialog_widgets}
- *
- * inspector_name & & styled_widget & m &
- *         Name of the inspector. $
- *
- * stuff_list & & styled_widget & m &
- *         List of various stuff that can be viewed. $
- *
- * inspect & & styled_widget & m &
- *         The state of the variable or event. $
- *
- * copy & & button & m &
- *         A button to copy the state to clipboard. $
- *
- * @end{table}
- */
 
 class gamestate_inspector::model
 {
@@ -108,7 +82,7 @@ public:
 
 	std::string get_data_paged(int which_page)
 	{
-		if(size_t(which_page) >= pages.size()) {
+		if(std::size_t(which_page) >= pages.size()) {
 			return "";
 		}
 		return data.substr(pages[which_page].first, pages[which_page].second);
@@ -134,9 +108,9 @@ private:
 	void repaginate()
 	{
 		pages.clear();
-		size_t start = 0;
+		std::size_t start = 0;
 		while(start < data.size()) {
-			size_t end = data.find_last_of('\n', start + max_inspect_win_len);
+			std::size_t end = data.find_last_of('\n', start + max_inspect_win_len);
 			if(end == std::string::npos) {
 				end = data.size() - 1;
 			}
@@ -147,7 +121,7 @@ private:
 	}
 	static const unsigned int max_inspect_win_len = 20000;
 	std::string data;
-	std::vector<std::pair<size_t,int>> pages;
+	std::vector<std::pair<std::size_t,int>> pages;
 };
 
 class stuff_list_adder
@@ -380,14 +354,14 @@ public:
 	void set_node_callback(const std::vector<int>& node_path, void (C::* fcn)(tree_view_node&))
 	{
 		C& sub_controller = *get_controller<C>();
-		callbacks.emplace(node_path, std::bind(fcn, sub_controller, _1));
+		callbacks.emplace(node_path, std::bind(fcn, sub_controller, std::placeholders::_1));
 	}
 
 	template<typename C, typename T>
 	void set_node_callback(const std::vector<int>& node_path, void (C::* fcn)(tree_view_node&, T), T param)
 	{
 		C& sub_controller = *get_controller<C>();
-		callbacks.emplace(node_path, std::bind(fcn, sub_controller, _1, param));
+		callbacks.emplace(node_path, std::bind(fcn, sub_controller, std::placeholders::_1, param));
 	}
 
 	void bind(window& window)
@@ -398,7 +372,8 @@ public:
 		auto left_button = find_widget<button>(&window, "page_left", false, true);
 		auto right_button = find_widget<button>(&window, "page_right", false, true);
 
-		stuff_list->set_selection_change_callback(std::bind(&gamestate_inspector::controller::handle_stuff_list_item_clicked, this, _1));
+		connect_signal_notify_modified(*stuff_list,
+			std::bind(&gamestate_inspector::controller::handle_stuff_list_item_clicked, this, std::placeholders::_1));
 
 		connect_signal_mouse_left_click(
 				*copy_button,
@@ -529,7 +504,7 @@ void variable_mode_controller::show_list(tree_view_node& node)
 			&variable_mode_controller::show_var);
 	}
 
-	std::map<std::string, size_t> wml_array_sizes;
+	std::map<std::string, std::size_t> wml_array_sizes;
 
 	for(const auto& ch : vars().all_children_range())
 	{
@@ -559,8 +534,8 @@ void variable_mode_controller::show_array(tree_view_node& node)
 	widget* w = node.find("name", false);
 	if(label* lbl = dynamic_cast<label*>(w)) {
 		const std::string& var = lbl->get_label();
-		size_t n_start = var.find_last_of('[') + 1;
-		size_t n_len = var.size() - n_start - 1;
+		std::size_t n_start = var.find_last_of('[') + 1;
+		std::size_t n_len = var.size() - n_start - 1;
 		int n = std::stoi(var.substr(n_start, n_len));
 		model().set_data(config_to_string(vars().child(var.substr(1, n_start - 3), n)));
 	}
@@ -754,7 +729,7 @@ void team_mode_controller::show_recall(tree_view_node& node, int side)
 		return;
 	}
 
-	for(const unit_ptr u : dc().get_team(side).recall_list()) {
+	for(const unit_ptr& u : dc().get_team(side).recall_list()) {
 		auto progress = view().stuff_list_entry(&node, "unit");
 		add_unit_entry(progress, *u, dc());
 		c.set_node_callback(progress.add(), &team_mode_controller::show_recall_unit, side);
@@ -826,4 +801,3 @@ void gamestate_inspector::pre_show(window& window)
 }
 
 } // namespace dialogs
-} // namespace gui2
