@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2017-2018 by Charles Dang <exodia339@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2017 - 2021
+	by Charles Dang <exodia339@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -37,9 +38,7 @@
 
 #include "help/help.hpp"
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 REGISTER_DIALOG(help_browser)
@@ -55,7 +54,8 @@ void help_browser::pre_show(window& window)
 	tree_view& topic_tree = find_widget<tree_view>(&window, "topic_tree", false);
 	multi_page& topic_pages = find_widget<multi_page>(&window, "topic_text_pages", false);
 
-	topic_tree.set_selection_change_callback(std::bind(&help_browser::on_topic_select, this, std::ref(window)));
+	connect_signal_notify_modified(topic_tree,
+		std::bind(&help_browser::on_topic_select, this));
 
 	window.keyboard_capture(&topic_tree);
 
@@ -82,12 +82,12 @@ void help_browser::pre_show(window& window)
 		++id;
 	}
 
-	on_topic_select(window);
+	on_topic_select();
 }
 
-void help_browser::on_topic_select(window& window)
+void help_browser::on_topic_select()
 {
-	tree_view& tree = find_widget<tree_view>(&window, "topic_tree", false);
+	tree_view& tree = find_widget<tree_view>(get_window(), "topic_tree", false);
 
 	if(tree.empty()) {
 		return;
@@ -100,9 +100,8 @@ void help_browser::on_topic_select(window& window)
 	}
 
 	const unsigned topic_i = lexical_cast<unsigned>(tree.selected_item()->id());
-	find_widget<multi_page>(&window, "topic_text_pages", false).select_page(topic_i);
+	find_widget<multi_page>(get_window(), "topic_text_pages", false).select_page(topic_i);
 
 }
 
 } // namespace dialogs
-} // namespace gui2
