@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2017-2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2017 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -32,9 +33,7 @@
 
 #include <vector>
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 REGISTER_DIALOG(label_settings)
 
@@ -113,7 +112,7 @@ void label_settings::pre_show(window& window)
 		toggle_button& status = find_widget<toggle_button>(grid, "cat_status", false);
 		status.set_value(visible);
 
-		connect_signal_notify_modified(status, std::bind(&label_settings::toggle_category, this, _1, category));
+		connect_signal_notify_modified(status, std::bind(&label_settings::toggle_category, this, std::placeholders::_1, category));
 
 		if(category.substr(0, 5) == "side:") {
 			label& cat_name = find_widget<label>(grid, "cat_name", false);
@@ -143,4 +142,3 @@ void label_settings::toggle_category(widget& box, const std::string& category)
 }
 
 } // namespace dialogs
-} // namespace gui2
