@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -23,9 +24,7 @@
 #include "scripting/plugins/manager.hpp"
 #include "video.hpp"
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 modal_dialog::modal_dialog()
 	: window_(nullptr)
@@ -74,7 +73,7 @@ bool modal_dialog::show(const unsigned auto_close_time)
 		return false;
 	}
 
-	window_.reset(build_window());
+	window_ = build_window();
 	assert(window_.get());
 
 	post_build(*window_);
@@ -118,6 +117,13 @@ bool modal_dialog::show(const unsigned auto_close_time)
 	return retval_ == retval::OK;
 }
 
+void modal_dialog::set_retval(int retval)
+{
+	if(window_) {
+		window_->set_retval(retval);
+	}
+}
+
 field_bool* modal_dialog::register_bool(
 		const std::string& id,
 		const bool mandatory,
@@ -218,7 +224,7 @@ field_label* modal_dialog::register_label(const std::string& id,
 	return field;
 }
 
-window* modal_dialog::build_window() const
+std::unique_ptr<window> modal_dialog::build_window() const
 {
 	return build(window_id());
 }
@@ -265,24 +271,3 @@ void modal_dialog::finalize_fields(window& window, const bool save_fields)
 }
 
 } // namespace dialogs
-} // namespace gui2
-
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 1
- *
- * {{Autogenerated}}
- *
- * = Window definition =
- *
- * The window definition define how the windows shown in the dialog look.
- */
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = ZZZZZZ_footer
- *
- * [[Category: WML Reference]]
- * [[Category: GUI WML Reference]]
- */
