@@ -1,29 +1,28 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
 #include "gui/auxiliary/field-fwd.hpp"
 #include "gui/core/static_registry.hpp"
-#include "utils/functional.hpp"
+#include <functional>
 
 #include <string>
 #include <vector>
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 /**
  * Registers a window.
@@ -61,7 +60,6 @@ namespace dialogs
  * Call this function to register a window. In the header of the class it adds
  * the following code:
  *@code
- *  // Inherited from modal_dialog, implemented by REGISTER_DIALOG.
  *	virtual const std::string& id() const;
  *@endcode
  * Then use this macro in the implementation, inside the gui2 namespace.
@@ -90,10 +88,10 @@ namespace dialogs
 
 /**
  * Adds a bare-bones static `display` function to a dialog class that immediately
- * invokes the dialogs's @ref modal_dialog::show function. If more complex behavior
+ * invokes the dialogs's modal_dialog::show function. If more complex behavior
  * is desired, the function should be defined manually.
  *
- * See the @ref modal_dialog documentation (below) for more info.
+ * See the modal_dialog documentation (below) for more info.
  */
 #define DEFINE_SIMPLE_DISPLAY_WRAPPER(dialog)                                                                          \
 	template<typename... T>                                                                                            \
@@ -104,10 +102,10 @@ namespace dialogs
 
 /**
  * Adds a bare-bonesstatic `execute` function to a dialog class that immediately
- * invokes and return the result of the dialogs's @ref modal_dialog::show function.
+ * invokes and return the result of the dialogs's modal_dialog::show function.
  * If more complex behavior is desired, the function should be defined manually.
  *
- * See the @ref modal_dialog documentation (below) for more info.
+ * See the modal_dialog documentation (below) for more info.
  */
 #define DEFINE_SIMPLE_EXECUTE_WRAPPER(dialog)                                                                          \
 	template<typename... T>                                                                                            \
@@ -139,7 +137,7 @@ namespace dialogs
  * When a function only has 'in parameters' it should return a void value and
  * the function should be called @p display, if it has 'in + out parameters' it
  * must return a bool value. This value indicates whether or not the OK button
- * was pressed to close the dialog. See @ref editor_new_map::execute for an
+ * was pressed to close the dialog. See editor_new_map::execute for an
  * example.
  */
 class modal_dialog
@@ -170,7 +168,6 @@ public:
 	 */
 	bool show(const unsigned auto_close_time = 0);
 
-
 	/***** ***** ***** setters / getters for members ***** ****** *****/
 
 	/** Returns a pointer to the dialog's window. Will be null if it hasn't been built yet. */
@@ -179,11 +176,15 @@ public:
 		return window_.get();
 	}
 
+	/** Returns the cached window exit code. */
 	int get_retval() const
 	{
 		return retval_;
 	}
 
+	/** Convenience wrapper to set the window's exit code. */
+	void set_retval(int retval);
+
 	void set_always_save_fields(const bool always_save_fields)
 	{
 		always_save_fields_ = always_save_fields;
@@ -221,6 +222,7 @@ protected:
 	 *                            with ok.
 	 * @param callback_change     When the value of the widget changes this
 	 *                            callback is called.
+	 * @param initial_fire
 	 *
 	 * @returns                   Pointer to the created widget.
 	 */
@@ -244,6 +246,7 @@ protected:
 	 *                            @ref field::field for more information.
 	 * @param callback_change     When the value of the widget changes this
 	 *                            callback is called.
+	 * @param initial_fire
 	 *
 	 * @returns                   Pointer to the created widget.
 	 */
@@ -330,7 +333,16 @@ protected:
 	std::unique_ptr<window> window_;
 
 private:
-	/** Returns the window exit status, 0 means not shown. */
+	/**
+	 * The window's exit code (return value).
+	 *
+	 * We keep a copy here so it may be accessed even after the dialog is closed and
+	 * the window object is destroyed.
+	 *
+	 * This value is initially set to 0 (retval::NONE) meaning the dialog was not
+	 * shown. After @ref show returns, it will hold the most recent retval of the
+	 * window object, including any modifications made in @ref post_show.
+	 */
 	int retval_;
 
 	/**
@@ -392,7 +404,7 @@ private:
 	 *
 	 * @returns                   The window to show.
 	 */
-	window* build_window() const;
+	std::unique_ptr<window> build_window() const;
 
 	/**
 	 * Actions to be taken directly after the window is build.
@@ -442,4 +454,3 @@ private:
 };
 
 } // namespace dialogs
-} // namespace gui2
