@@ -1,20 +1,23 @@
 /*
-   Copyright (C) 2009 - 2018 by Tomasz Sniatowski <kailoran@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Tomasz Sniatowski <kailoran@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
+
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "gui/dialogs/multiplayer/lobby.hpp"
 
+#include "gui/auxiliary/field.hpp"
 #include "gui/auxiliary/find_widget.hpp"
 #include "gui/dialogs/message.hpp"
 #include "gui/dialogs/multiplayer/mp_join_game_password_prompt.hpp"
@@ -35,20 +38,24 @@
 #include "gui/widgets/toggle_button.hpp"
 #include "gui/widgets/toggle_panel.hpp"
 #include "gui/widgets/tree_view_node.hpp"
+#include "gui/dialogs/server_info_dialog.hpp"
 
 #include "addon/client.hpp"
 #include "addon/manager_ui.hpp"
 #include "chat_log.hpp"
+#include "desktop/open.hpp"
 #include "font/text_formatting.hpp"
 #include "formatter.hpp"
 #include "formula/string_utils.hpp"
+#include "game_config_manager.hpp"
 #include "preferences/game.hpp"
 #include "gettext.hpp"
+#include "help/help.hpp"
 #include "preferences/lobby.hpp"
 #include "playmp_controller.hpp"
 #include "wesnothd_connection.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 static lg::log_domain log_lobby("lobby");
 #define DBG_LB LOG_STREAM(debug, log_lobby)
@@ -56,9 +63,7 @@ static lg::log_domain log_lobby("lobby");
 #define ERR_LB LOG_STREAM(err, log_lobby)
 #define SCOPE_LB log_scope2(log_lobby, __func__)
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 REGISTER_DIALOG(mp_lobby)
@@ -101,10 +106,7 @@ void sub_player_list::update_player_count_label()
 void player_list::init(window& w)
 {
 	active_game.init(w, _("Selected Game"), true);
-	other_rooms.init(w, _("Lobby"), true);
-#ifdef ENABLE_ROOM_MEMBER_TREE
-	active_room.init(w, _("Current Room"));
-#endif
+	lobby_players.init(w, _("Lobby"), true);
 	other_games.init(w, _("Other Games"));
 
 	tree = find_widget<tree_view>(&w, "player_tree", false, true);
@@ -115,16 +117,35 @@ bool mp_lobby::logout_prompt()
 	return show_prompt(_("Do you really want to log out?"));
 }
 
-mp_lobby::mp_lobby(const config& game_config, mp::lobby_info& info, wesnothd_connection &connection)
+std::string mp_lobby::announcements_ = "";
+std::string mp_lobby::server_information_ = "";
+
+mp_lobby::mp_lobby(mp::lobby_info& info, wesnothd_connection& connection, int& joined_game)
 	: quit_confirmation(&mp_lobby::logout_prompt)
-	, game_config_(game_config)
+	, game_config_(game_config_manager::get()->game_config())
 	, gamelistbox_(nullptr)
 	, lobby_info_(info)
 	, chatbox_(nullptr)
-	, filter_friends_(nullptr)
-	, filter_ignored_(nullptr)
-	, filter_slots_(nullptr)
-	, filter_invert_(nullptr)
+	, filter_friends_(register_bool("filter_with_friends",
+		  true,
+		  preferences::fi_friends_in_game,
+		  preferences::set_fi_friends_in_game,
+		  std::bind(&mp_lobby::game_filter_change_callback, this)))
+	, filter_ignored_(register_bool("filter_with_ignored",
+		  true,
+		  preferences::fi_blocked_in_game,
+		  preferences::set_fi_blocked_in_game,
+		  std::bind(&mp_lobby::game_filter_change_callback, this)))
+	, filter_slots_(register_bool("filter_vacant_slots",
+		  true,
+		  preferences::fi_vacant_slots,
+		  preferences::set_fi_vacant_slots,
+		  std::bind(&mp_lobby::game_filter_change_callback, this)))
+	, filter_invert_(register_bool("filter_invert",
+		  true,
+		  preferences::fi_invert,
+		  preferences::set_fi_invert,
+		  std::bind(&mp_lobby::game_filter_change_callback, this)))
 	, filter_text_(nullptr)
 	, selected_game_id_()
 	, player_list_()
@@ -137,11 +158,12 @@ mp_lobby::mp_lobby(const config& game_config, mp::lobby_info& info, wesnothd_con
 	, gamelist_id_at_row_()
 	, delay_playerlist_update_(false)
 	, delay_gamelist_update_(false)
-	, joined_game_id_(0)
+	, joined_game_id_(joined_game)
 {
 	// Need to set this in the constructor, pre_show() is too late
 	set_show_even_without_video(true);
 	set_allow_plugin_skip(false);
+	set_always_save_fields(true);
 }
 
 struct lobby_delay_gamelist_update_guard
@@ -167,8 +189,11 @@ mp_lobby::~mp_lobby()
 void mp_lobby::post_build(window& win)
 {
 	/*** Local hotkeys. ***/
+	win.register_hotkey(hotkey::HOTKEY_HELP,
+		std::bind(&mp_lobby::show_help_callback, this));
+
 	win.register_hotkey(hotkey::HOTKEY_PREFERENCES,
-		std::bind(&mp_lobby::show_preferences_button_callback, this, std::ref(win)));
+		std::bind(&mp_lobby::show_preferences_button_callback, this));
 }
 
 namespace
@@ -204,10 +229,10 @@ std::string colorize(const std::string& str, const color_t& color)
 	return (formatter() << font::span_color(color) << str << "</span>").str();
 }
 
-bool handle_addon_requirements_gui(const std::vector<mp::game_info::required_addon>& reqs, mp::game_info::ADDON_REQ addon_outcome)
+bool handle_addon_requirements_gui(const std::vector<mp::game_info::required_addon>& reqs, mp::game_info::addon_req addon_outcome)
 {
-	if(addon_outcome == mp::game_info::CANNOT_SATISFY) {
-		std::string e_title = _("Incompatible User-made Content.");
+	if(addon_outcome == mp::game_info::addon_req::CANNOT_SATISFY) {
+		std::string e_title = _("Incompatible User-made Content");
 		std::string err_msg = _("This game cannot be joined because the host has out-of-date add-ons that are incompatible with your version. You might wish to suggest that the host's add-ons be updated.");
 
 		err_msg +="\n\n";
@@ -215,15 +240,15 @@ bool handle_addon_requirements_gui(const std::vector<mp::game_info::required_add
 		err_msg += "\n";
 
 		for(const mp::game_info::required_addon & a : reqs) {
-			if (a.outcome == mp::game_info::CANNOT_SATISFY) {
+			if (a.outcome == mp::game_info::addon_req::CANNOT_SATISFY) {
 				err_msg += font::unicode_bullet + " " + a.message + "\n";
 			}
 		}
 		gui2::show_message(e_title, err_msg, message::auto_close, true);
 
 		return false;
-	} else if(addon_outcome == mp::game_info::NEED_DOWNLOAD) {
-		std::string e_title = _("Missing User-made Content.");
+	} else if(addon_outcome == mp::game_info::addon_req::NEED_DOWNLOAD) {
+		std::string e_title = _("Missing User-made Content");
 		std::string err_msg = _("This game requires one or more user-made addons to be installed or updated in order to join.\nDo you want to try to install them?");
 
 		err_msg +="\n\n";
@@ -232,7 +257,7 @@ bool handle_addon_requirements_gui(const std::vector<mp::game_info::required_add
 
 		std::vector<std::string> needs_download;
 		for(const mp::game_info::required_addon & a : reqs) {
-			if(a.outcome == mp::game_info::NEED_DOWNLOAD) {
+			if(a.outcome == mp::game_info::addon_req::NEED_DOWNLOAD) {
 				err_msg += font::unicode_bullet + " " + a.message + "\n";
 
 				needs_download.push_back(a.addon_id);
@@ -307,10 +332,12 @@ void mp_lobby::update_gamelist_diff()
 	for(unsigned i = 0; i < lobby_info_.games().size(); ++i) {
 		const mp::game_info& game = *lobby_info_.games()[i];
 
-		if(game.display_status == mp::game_info::NEW) {
+		if(game.display_status == mp::game_info::disp_status::NEW) {
 			// call void do_notify(notify_mode mode, const std::string& sender, const std::string& message)
 			// sender will be the game_info.scenario (std::string) and message will be game_info.name (std::string)
-			do_notify(mp::NOTIFY_GAME_CREATED, game.scenario, game.name);
+			if (lobby_info_.is_game_visible(game)) {
+				do_notify(mp::NOTIFY_GAME_CREATED, game.scenario, game.name);
+			}
 
 			LOG_LB << "Adding game to listbox " << game.id << "\n";
 
@@ -352,14 +379,14 @@ void mp_lobby::update_gamelist_diff()
 				return;
 			}
 
-			if(game.display_status == mp::game_info::UPDATED) {
+			if(game.display_status == mp::game_info::disp_status::UPDATED) {
 				LOG_LB << "Modifying game in listbox " << game.id << " (row " << list_i << ")\n";
 				grid* grid = gamelistbox_->get_row_grid(list_i);
 				modify_grid_with_data(grid, make_game_row_data(game));
 				adjust_game_row_contents(game, grid, false);
 				++list_i;
 				next_gamelist_id_at_row.push_back(game.id);
-			} else if(game.display_status == mp::game_info::DELETED) {
+			} else if(game.display_status == mp::game_info::disp_status::DELETED) {
 				LOG_LB << "Deleting game from listbox " << game.id << " (row "
 					   << list_i << ")\n";
 				gamelistbox_->remove_row(list_i);
@@ -429,7 +456,7 @@ std::map<std::string, string_map> mp_lobby::make_game_row_data(const mp::game_in
 	item["label"] = game.vacant_slots > 0 ? colorize(game.name, color_string) : game.name;
 	data.emplace("name", item);
 
-	item["label"] = colorize("<i>" + game.type_marker + scenario_text + "</i>", font::GRAY_COLOR);
+	item["label"] = colorize(game.type_marker + "<i>" + scenario_text + "</i>", font::GRAY_COLOR);
 	data.emplace("scenario", item);
 
 	item["label"] = colorize(game.status, color_string);
@@ -534,7 +561,6 @@ void mp_lobby::adjust_game_row_contents(const mp::game_info& game, grid* grid, b
 	//
 	minimap& map = find_widget<minimap>(grid, "minimap", false);
 
-	map.set_config(&game_config_);
 	map.set_map_data(game.map_data);
 
 	if(!add_callbacks) {
@@ -563,106 +589,63 @@ void mp_lobby::update_playerlist()
 
 	SCOPE_LB;
 	DBG_LB << "Playerlist update: " << lobby_info_.users().size() << "\n";
-	lobby_info_.update_user_statuses(selected_game_id_, chatbox_->active_window_room());
-
-#ifdef ENABLE_ROOM_MEMBER_TREE
-	bool lobby = false;
-	if(mp::room_info* ri = chatbox_->active_window_room()) {
-		if(ri->name() == "lobby") {
-			lobby = true;
-		}
-	}
-#endif
+	lobby_info_.update_user_statuses(selected_game_id_);
 
 	assert(player_list_.active_game.tree);
-#ifdef ENABLE_ROOM_MEMBER_TREE
-	assert(player_list_.active_room.tree);
-#endif
 	assert(player_list_.other_games.tree);
-	assert(player_list_.other_rooms.tree);
+	assert(player_list_.lobby_players.tree);
 
 	unsigned scrollbar_position = player_list_.tree->get_vertical_scrollbar_item_position();
 
 	player_list_.active_game.tree->clear();
-#ifdef ENABLE_ROOM_MEMBER_TREE
-	player_list_.active_room.tree->clear();
-#endif
 	player_list_.other_games.tree->clear();
-	player_list_.other_rooms.tree->clear();
-
-	for(auto& user : lobby_info_.users()) {
-		sub_player_list* target_list(nullptr);
+	player_list_.lobby_players.tree->clear();
 
+	std::map<std::string, std::map<std::string, string_map>> lobby_player_items;
+	std::map<std::string, std::map<std::string, string_map>> active_game_items;
+	std::map<std::string, std::map<std::string, string_map>> other_game_items;
+	for(const auto& user : lobby_info_.users()) {
 		std::string name = user.name;
 
 		std::stringstream icon_ss;
+
 		icon_ss << "lobby/status";
 		switch(user.state) {
-#ifdef ENABLE_ROOM_MEMBER_TREE
-			case mp::user_info::SEL_ROOM:
+			case mp::user_info::user_state::LOBBY:
 				icon_ss << "-lobby";
-				target_list = &player_list_.active_room;
-				if(lobby) {
-					target_list = &player_list_.other_rooms;
-				}
 				break;
-#endif
-			case mp::user_info::LOBBY:
-				icon_ss << "-lobby";
-				target_list = &player_list_.other_rooms;
-				break;
-			case mp::user_info::SEL_GAME:
+			case mp::user_info::user_state::SEL_GAME:
 				name = colorize(name, {0, 255, 255});
 				icon_ss << (user.observing ? "-obs" : "-playing");
-				target_list = &player_list_.active_game;
 				break;
-			case mp::user_info::GAME:
-				name = colorize(name, font::BAD_COLOR);
+			case mp::user_info::user_state::GAME:
+				name = colorize(name, font::GRAY_COLOR);
 				icon_ss << (user.observing ? "-obs" : "-playing");
-				target_list = &player_list_.other_games;
 				break;
 			default:
-				ERR_LB << "Bad user state in lobby: " << user.name << ": "
-					   << user.state << "\n";
+				ERR_LB << "Bad user state in lobby: " << user.name << ": " << static_cast<int>(user.state) << "\n";
 				continue;
 		}
 
 		switch(user.relation) {
-			case mp::user_info::ME:
+			case mp::user_info::user_relation::ME:
 				icon_ss << "-s";
 				break;
-			case mp::user_info::NEUTRAL:
+			case mp::user_info::user_relation::NEUTRAL:
 				icon_ss << "-n";
 				break;
-			case mp::user_info::FRIEND:
+			case mp::user_info::user_relation::FRIEND:
 				icon_ss << "-f";
 				break;
-			case mp::user_info::IGNORED:
+			case mp::user_info::user_relation::IGNORED:
 				icon_ss << "-i";
 				break;
 			default:
-				ERR_LB << "Bad user relation in lobby: " << user.relation
-					   << "\n";
+				ERR_LB << "Bad user relation in lobby: " << static_cast<int>(user.relation) << "\n";
 		}
 
-		// TODO: on the official server this results in every name being bold since we
-		// require a registered account. Leaving this commented out in case we ever
-		// walk that back and want to have such an indication again (it's useless for
-		// custom servers since registered logins aren't supported there).
-#if 0
-		if(user.registered) {
-			name = "<b>" + name + "</b>";
-		}
-#endif
-
 		icon_ss << ".png";
 
-		if(!preferences::playerlist_group_players()) {
-			target_list = &player_list_.other_rooms;
-		}
-
-		assert(target_list->tree);
-
 		string_map tree_group_field;
 		std::map<std::string, string_map> tree_group_item;
 
@@ -674,17 +657,37 @@ void mp_lobby::update_playerlist()
 		tree_group_field["use_markup"] = "true";
 		tree_group_item["name"] = tree_group_field;
 
-		tree_view_node& player = target_list->tree->add_child("player", tree_group_item);
+		switch(user.state) {
+			case mp::user_info::user_state::LOBBY:
+				lobby_player_items[user.name] = tree_group_item;
+				break;
+			case mp::user_info::user_state::SEL_GAME:
+				active_game_items[user.name] = tree_group_item;
+				break;
+			case mp::user_info::user_state::GAME:
+				other_game_items[user.name] = tree_group_item;
+				break;
+			default:
+				ERR_LB << "Bad user state in lobby: " << user.name << ": " << static_cast<int>(user.state) << "\n";
+				continue;
+		}
+	}
 
-		connect_signal_mouse_left_double_click(find_widget<toggle_panel>(&player, "tree_view_node_label", false),
-			std::bind(&mp_lobby::user_dialog_callback, this, &user));
+	for(const auto& player : player_list_.active_game.tree->replace_children("player", active_game_items)) {
+		 connect_signal_mouse_left_double_click(find_widget<toggle_panel>(player.second.get(), "tree_view_node_label", false),
+		 	std::bind(&mp_lobby::user_dialog_callback, this, lobby_info_.get_user(player.first)));
+	}
+	for(const auto& player : player_list_.lobby_players.tree->replace_children("player", lobby_player_items)) {
+		 connect_signal_mouse_left_double_click(find_widget<toggle_panel>(player.second.get(), "tree_view_node_label", false),
+		 	std::bind(&mp_lobby::user_dialog_callback, this, lobby_info_.get_user(player.first)));
+	}
+	for(const auto& player : player_list_.other_games.tree->replace_children("player", other_game_items)) {
+		 connect_signal_mouse_left_double_click(find_widget<toggle_panel>(player.second.get(), "tree_view_node_label", false),
+		 	std::bind(&mp_lobby::user_dialog_callback, this, lobby_info_.get_user(player.first)));
 	}
 
 	player_list_.active_game.update_player_count_label();
-#ifdef ENABLE_ROOM_MEMBER_TREE
-	player_list_.active_room.update_player_count_label();
-#endif
-	player_list_.other_rooms.update_player_count_label();
+	player_list_.lobby_players.update_player_count_label();
 	player_list_.other_games.update_player_count_label();
 
 	// Don't attempt to restore the scroll position if the window hasn't been laid out yet
@@ -739,13 +742,12 @@ void mp_lobby::pre_show(window& window)
 	window.set_enter_disabled(true);
 
 	// Exit hook to add a confirmation when quitting the Lobby.
-	window.set_exit_hook(std::bind(&mp_lobby::exit_hook, this, std::ref(window)));
+	window.set_exit_hook(std::bind(&mp_lobby::exit_hook, this, std::placeholders::_1));
 
 	chatbox_ = find_widget<chatbox>(&window, "chat", false, true);
 
 	window.keyboard_capture(chatbox_);
 
-	chatbox_->set_lobby_info(lobby_info_);
 	chatbox_->set_wesnothd_connection(network_connection_);
 	chatbox_->set_active_window_changed_callback([this]() { player_list_dirty_ = true; });
 	chatbox_->load_log(default_chat_log, true);
@@ -754,7 +756,7 @@ void mp_lobby::pre_show(window& window)
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&window, "show_preferences", false),
-		std::bind(&mp_lobby::show_preferences_button_callback, this, std::ref(window)));
+		std::bind(&mp_lobby::show_preferences_button_callback, this));
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&window, "join_global", false),
@@ -766,6 +768,10 @@ void mp_lobby::pre_show(window& window)
 		find_widget<button>(&window, "observe_global", false),
 		std::bind(&mp_lobby::enter_selected_game, this, DO_OBSERVE));
 
+	connect_signal_mouse_left_click(
+		find_widget<button>(&window, "server_info", false),
+		std::bind(&mp_lobby::show_server_info, this));
+
 	find_widget<button>(&window, "observe_global", false).set_active(false);
 
 	menu_button& replay_options = find_widget<menu_button>(&window, "replay_options", false);
@@ -779,29 +785,13 @@ void mp_lobby::pre_show(window& window)
 	}
 
 	connect_signal_notify_modified(replay_options,
-		std::bind(&mp_lobby::skip_replay_changed_callback, this, std::ref(window)));
+		std::bind(&mp_lobby::skip_replay_changed_callback, this));
 
-	filter_friends_ = find_widget<toggle_button>(&window, "filter_with_friends", false, true);
-	filter_ignored_ = find_widget<toggle_button>(&window, "filter_without_ignored", false, true);
-	filter_slots_   = find_widget<toggle_button>(&window, "filter_vacant_slots", false, true);
-	filter_invert_  = find_widget<toggle_button>(&window, "filter_invert", false, true);
 	filter_text_    = find_widget<text_box>(&window, "filter_text", false, true);
 
-	connect_signal_notify_modified(*filter_friends_,
-		std::bind(&mp_lobby::game_filter_change_callback, this));
-
-	connect_signal_notify_modified(*filter_ignored_,
-		std::bind(&mp_lobby::game_filter_change_callback, this));
-
-	connect_signal_notify_modified(*filter_slots_,
-		std::bind(&mp_lobby::game_filter_change_callback, this));
-
-	connect_signal_notify_modified(*filter_invert_,
-		std::bind(&mp_lobby::game_filter_change_callback, this));
-
 	connect_signal_pre_key_press(
 			*filter_text_,
-			std::bind(&mp_lobby::game_filter_keypress_callback, this, _5));
+			std::bind(&mp_lobby::game_filter_keypress_callback, this, std::placeholders::_5));
 
 	chatbox_->room_window_open(N_("lobby"), true, false);
 	chatbox_->active_window_changed();
@@ -818,6 +808,29 @@ void mp_lobby::pre_show(window& window)
 	lobby_update_timer_ = add_timer(
 		game_config::lobby_network_timer, std::bind(&mp_lobby::network_handler, this), true);
 
+	//
+	// Profile box
+	//
+	if(auto* profile_panel = find_widget<panel>(&window, "profile", false, false)) {
+		auto your_info = std::find_if(lobby_info_.users().begin(), lobby_info_.users().end(),
+			[](const auto& u) { return u.relation == mp::user_info::user_relation::ME; });
+
+		if(your_info != lobby_info_.users().end()) {
+			find_widget<label>(profile_panel, "username", false).set_label(your_info->name);
+
+			auto& profile_button = find_widget<button>(profile_panel, "view_profile", false);
+			if(your_info->forum_id != 0) {
+				connect_signal_mouse_left_click(profile_button,
+					std::bind(&desktop::open_object, mp::get_profile_link(your_info->forum_id)));
+			} else {
+				profile_button.set_active(false);
+			}
+
+			// TODO: implement
+			find_widget<button>(profile_panel, "view_match_history", false).set_active(false);
+		}
+	}
+
 	// Set up Lua plugin context
 	plugins_context_.reset(new plugins_context("Multiplayer Lobby"));
 
@@ -838,7 +851,7 @@ void mp_lobby::pre_show(window& window)
 	}, true);
 
 	plugins_context_->set_accessor("game_list",   [this](const config&) { return lobby_info_.gamelist(); });
-	plugins_context_->set_accessor("game_config", [this](const config&) { return game_config_; });
+	//plugins_context_->set_accessor("game_config", [this](const config&) { return game_config_; });
 }
 
 void mp_lobby::post_show(window& /*window*/)
@@ -892,6 +905,14 @@ void mp_lobby::process_network_data(const config& data)
 		process_gamelist(data);
 	} else if(const config& gamelist_diff = data.child("gamelist_diff")) {
 		process_gamelist_diff(gamelist_diff);
+	} else if(const config& info = data.child("message")) {
+		if(info["type"] == "server_info") {
+			server_information_ = info["message"].str();
+			return;
+		} else if(info["type"] == "announcements") {
+			announcements_ = info["message"].str();
+			return;
+		}
 	}
 
 	chatbox_->process_network_data(data);
@@ -959,13 +980,20 @@ void mp_lobby::enter_game(const mp::game_info& game, JOIN_MODE mode)
 		break;
 	}
 
+	// prompt moderators for whether they want to join a game with observers disabled
+	if(!game.observers && mp::logged_in_as_moderator()) {
+		if(gui2::show_message(_("Observe"), _("This game doesn't allow observers. Observe using moderator rights anyway?"), gui2::dialogs::message::yes_no_buttons) != gui2::retval::OK) {
+			return;
+		}
+	}
+
 	const bool try_join = mode == DO_JOIN;
 	const bool try_obsv = mode == DO_OBSERVE;
 
 	window& window = *get_window();
 
 	// Prompt user to download this game's required addons if its requirements have not been met
-	if(game.addons_outcome != mp::game_info::SATISFIED) {
+	if(game.addons_outcome != mp::game_info::addon_req::SATISFIED) {
 		if(game.required_addons.empty()) {
 			gui2::show_error_message(_("Something is wrong with the addon version check database supporting the multiplayer lobby. Please report this at https://bugs.wesnoth.org."));
 			return;
@@ -987,7 +1015,11 @@ void mp_lobby::enter_game(const mp::game_info& game, JOIN_MODE mode)
 	join_data["id"] = std::to_string(game.id);
 	join_data["observe"] = try_obsv;
 
-	if(!join_data.empty() && try_join && game.password_required) {
+	if(mp::logged_in_as_moderator() && game.password_required) {
+		if(gui2::show_message(_("Join"), _("This game is password protected. Join using moderator rights anyway?"), gui2::dialogs::message::yes_no_buttons) != gui2::retval::OK) {
+			return;
+		}
+	} else if(!join_data.empty() && game.password_required) {
 		std::string password;
 
 		if(!gui2::dialogs::mp_join_game_password_prompt::execute(password)) {
@@ -1037,16 +1069,21 @@ void mp_lobby::refresh_lobby()
 	network_connection_.send_data(config("refresh_lobby"));
 }
 
-void mp_lobby::show_preferences_button_callback(window& window)
+void mp_lobby::show_help_callback()
 {
-	gui2::dialogs::preferences_dialog::display(game_config_);
+	help::show_help();
+}
+
+void mp_lobby::show_preferences_button_callback()
+{
+	gui2::dialogs::preferences_dialog::display();
 
 	/**
 	 * The screen size might have changed force an update of the size.
 	 *
 	 * @todo This might no longer be needed when gui2 is done.
 	 */
-	const SDL_Rect rect = window.video().screen_area();
+	const SDL_Rect rect = CVideo::get_singleton().screen_area();
 
 	gui2::settings::gamemap_width  += rect.w - gui2::settings::screen_width;
 	gui2::settings::gamemap_height += rect.h - gui2::settings::screen_height;
@@ -1058,11 +1095,16 @@ void mp_lobby::show_preferences_button_callback(window& window)
 	 *
 	 * @todo This might no longer be needed when gui2 is done.
 	 */
-	window.invalidate_layout();
+	get_window()->invalidate_layout();
 
 	refresh_lobby();
 }
 
+void mp_lobby::show_server_info()
+{
+	server_info::display(server_information_, announcements_);
+}
+
 void mp_lobby::game_filter_reload()
 {
 	lobby_info_.clear_game_filter();
@@ -1073,26 +1115,31 @@ void mp_lobby::game_filter_reload()
 		});
 	}
 
+	window& window = *get_window();
+
 	// TODO: make changing friend/ignore lists trigger a refresh
-	if(filter_friends_->get_value()) {
+	if(filter_friends_->get_widget_value(window)) {
 		lobby_info_.add_game_filter([](const mp::game_info& info)->bool {
 			return info.has_friends == true;
 		});
 	}
 
-	if(filter_ignored_->get_value()) {
+	// Unlike the friends filter, this is an inclusion filter (do we want to also show
+	// games with blocked players) rather than an exclusion filter (do we want to show
+	// only games with friends).
+	if(filter_ignored_->get_widget_value(window) == false) {
 		lobby_info_.add_game_filter([](const mp::game_info& info)->bool {
 			return info.has_ignored == false;
 		});
 	}
 
-	if(filter_slots_->get_value()) {
+	if(filter_slots_->get_widget_value(window)) {
 		lobby_info_.add_game_filter([](const mp::game_info& info)->bool {
 			return info.vacant_slots > 0;
 		});
 	}
 
-	lobby_info_.set_game_filter_invert(filter_invert_->get_value_bool());
+	lobby_info_.set_game_filter_invert(filter_invert_->get_widget_value(window));
 }
 
 void mp_lobby::game_filter_keypress_callback(const SDL_Keycode key)
@@ -1143,13 +1190,12 @@ void mp_lobby::user_dialog_callback(mp::user_info* info)
 	refresh_lobby();
 }
 
-void mp_lobby::skip_replay_changed_callback(window& window)
+void mp_lobby::skip_replay_changed_callback()
 {
 	// TODO: this prefence should probably be controlled with an enum
-	const int value = find_widget<menu_button>(&window, "replay_options", false).get_value();
+	const int value = find_widget<menu_button>(get_window(), "replay_options", false).get_value();
 	preferences::set_skip_mp_replay(value == 1);
 	preferences::set_blindfold_replay(value == 2);
 }
 
 } // namespace dialogs
-} // namespace gui2
