@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Tomasz Sniatowski <kailoran@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Tomasz Sniatowski <kailoran@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -23,6 +24,8 @@
 #include "quit_confirmation.hpp"
 
 class wesnothd_connection;
+class game_config_view;
+
 namespace gui2
 {
 
@@ -52,8 +55,7 @@ struct player_list
 	void init(window& w);
 
 	sub_player_list active_game;
-	sub_player_list active_room;
-	sub_player_list other_rooms;
+	sub_player_list lobby_players;
 	sub_player_list other_games;
 
 	tree_view* tree;
@@ -62,12 +64,10 @@ struct player_list
 class mp_lobby : public modal_dialog, public quit_confirmation, private plugin_executor
 {
 public:
-	mp_lobby(const config& game_config, mp::lobby_info& info, wesnothd_connection &connection);
+	mp_lobby(mp::lobby_info& info, wesnothd_connection& connection, int& joined_game);
 
 	~mp_lobby();
 
-	int get_joined_game_id() const { return joined_game_id_; }
-
 	void update_gamelist();
 
 protected:
@@ -134,7 +134,11 @@ private:
 	/** Enter game by index, where index is the selected game listbox row. */
 	void enter_selected_game(JOIN_MODE mode);
 
-	void show_preferences_button_callback(window& window);
+	void show_help_callback();
+
+	void show_preferences_button_callback();
+
+	void show_server_info();
 
 	void refresh_lobby();
 
@@ -150,25 +154,21 @@ private:
 
 	void user_dialog_callback(mp::user_info* info);
 
-	void skip_replay_changed_callback(window& window);
+	void skip_replay_changed_callback();
 
 	bool exit_hook(window& window);
 
 	static bool logout_prompt();
 
-	/** Inherited from modal_dialog, implemented by REGISTER_DIALOG. */
 	virtual const std::string& window_id() const override;
 
-	/** Inherited from modal_dialog. */
 	virtual void post_build(window& window) override;
 
-	/** Inherited from modal_dialog. */
 	virtual void pre_show(window& window) override;
 
-	/** Inherited from modal_dialog. */
 	virtual void post_show(window& window) override;
 
-	const config& game_config_;
+	const game_config_view& game_config_;
 
 	listbox* gamelistbox_;
 
@@ -176,13 +176,10 @@ private:
 
 	chatbox* chatbox_;
 
-	toggle_button* filter_friends_;
-
-	toggle_button* filter_ignored_;
-
-	toggle_button* filter_slots_;
-
-	toggle_button* filter_invert_;
+	field_bool* filter_friends_;
+	field_bool* filter_ignored_;
+	field_bool* filter_slots_;
+	field_bool* filter_invert_;
 
 	text_box* filter_text_;
 
@@ -201,7 +198,7 @@ private:
 	wesnothd_connection &network_connection_;
 
 	/** Timer for updating the lobby. */
-	size_t lobby_update_timer_;
+	std::size_t lobby_update_timer_;
 
 	std::vector<int> gamelist_id_at_row_;
 
@@ -209,9 +206,12 @@ private:
 
 	bool delay_gamelist_update_;
 
-	int joined_game_id_;
+	int& joined_game_id_;
 
 	friend struct lobby_delay_gamelist_update_guard;
+
+	static std::string server_information_;
+	static std::string announcements_;
 };
 
 } // namespace dialogs
