@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -17,6 +18,9 @@
 #include "gui/dialogs/multiplayer/mp_create_game.hpp"
 
 #include "filesystem.hpp"
+#include "formatter.hpp"
+#include "formula/string_utils.hpp"
+#include "game_config.hpp"
 #include "game_config_manager.hpp"
 #include "game_initialization/lobby_data.hpp"
 #include "gettext.hpp"
@@ -27,14 +31,9 @@
 #include "gui/widgets/button.hpp"
 #include "gui/widgets/image.hpp"
 #include "gui/widgets/integer_selector.hpp"
-#include "gui/widgets/menu_button.hpp"
-#include "preferences/game.hpp"
 #include "gui/widgets/listbox.hpp"
-#include "formatter.hpp"
-#include "formula/string_utils.hpp"
-#include "game_config.hpp"
+#include "gui/widgets/menu_button.hpp"
 #include "gui/widgets/minimap.hpp"
-#include "gui/widgets/settings.hpp"
 #include "gui/widgets/slider.hpp"
 #include "gui/widgets/stacked_widget.hpp"
 #include "gui/widgets/status_label_helper.hpp"
@@ -42,8 +41,10 @@
 #include "gui/widgets/toggle_button.hpp"
 #include "gui/widgets/toggle_panel.hpp"
 #include "log.hpp"
-#include "savegame.hpp"
 #include "map_settings.hpp"
+#include "preferences/game.hpp"
+#include "save_index.hpp"
+#include "savegame.hpp"
 
 #include <boost/algorithm/string.hpp>
 
@@ -53,9 +54,7 @@ static lg::log_domain log_mp_create("mp/create");
 #define WRN_MP LOG_STREAM(warn, log_mp_create)
 #define ERR_MP LOG_STREAM(err, log_mp_create)
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 // Special retval value for loading a game
@@ -66,9 +65,8 @@ namespace prefs = preferences;
 
 REGISTER_DIALOG(mp_create_game)
 
-mp_create_game::mp_create_game(const config& cfg, saved_game& state, bool local_mode, mp::user_info* host_info)
-	: cfg_(cfg)
-	, create_engine_(state)
+mp_create_game::mp_create_game(saved_game& state, bool local_mode)
+	: create_engine_(state)
 	, config_engine_()
 	, options_manager_()
 	, selected_game_index_(-1)
@@ -82,7 +80,6 @@ mp_create_game::mp_create_game(const config& cfg, saved_game& state, bool local_
 		std::bind(&mp_create_game::update_map_settings, this)))
 	, shuffle_sides_(register_bool("shuffle_sides", true, prefs::shuffle_sides, prefs::set_shuffle_sides))
 	, observers_(register_bool("observers", true, prefs::allow_observers, prefs::set_allow_observers))
-	, registered_users_(register_bool("registered_users", true, prefs::registered_users_only, prefs::set_registered_users_only))
 	, strict_sync_(register_bool("strict_sync", true))
 	, private_replay_(register_bool("private_replay", true))
 	, turns_(register_integer("turn_count", true, prefs::turns, prefs::set_turns))
@@ -96,7 +93,6 @@ mp_create_game::mp_create_game(const config& cfg, saved_game& state, bool local_
 	, mod_list_()
 	, eras_menu_button_()
 	, local_mode_(local_mode)
-	, host_info_(host_info)
 {
 	level_types_ = {
 		{ng::level::TYPE::SCENARIO, _("Scenarios")},
@@ -131,21 +127,19 @@ mp_create_game::mp_create_game(const config& cfg, saved_game& state, bool local_
 
 void mp_create_game::pre_show(window& win)
 {
-	find_widget<minimap>(&win, "minimap", false).set_config(&cfg_);
-
 	find_widget<text_box>(&win, "game_name", false).set_value(local_mode_ ? "" : ng::configure_engine::game_name_default());
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&win, "random_map_regenerate", false),
-		std::bind(&mp_create_game::regenerate_random_map, this, std::ref(win)));
+		std::bind(&mp_create_game::regenerate_random_map, this));
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&win, "random_map_settings", false),
-		std::bind(&mp_create_game::show_generator_settings, this, std::ref(win)));
+		std::bind(&mp_create_game::show_generator_settings, this));
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&win, "load_game", false),
-		std::bind(&mp_create_game::load_game_callback, this, std::ref(win)));
+		std::bind(&mp_create_game::load_game_callback, this));
 
 	// Custom dialog close hook
 	win.set_exit_hook_ok_only([this](window& w)->bool { return dialog_exit_hook(w); });
@@ -159,12 +153,12 @@ void mp_create_game::pre_show(window& win)
 	// Set up filtering
 	//
 	connect_signal_notify_modified(find_widget<slider>(&win, "num_players", false),
-		std::bind(&mp_create_game::on_filter_change<slider>, this, std::ref(win), "num_players", true));
+		std::bind(&mp_create_game::on_filter_change<slider>, this, "num_players", true));
 
 	text_box& filter = find_widget<text_box>(&win, "game_filter", false);
 
 	filter.set_text_changed_callback(
-		std::bind(&mp_create_game::on_filter_change<text_box>, this, std::ref(win), "game_filter", true));
+		std::bind(&mp_create_game::on_filter_change<text_box>, this, "game_filter", true));
 
 	// Note this cannot be in the keyboard chain or it will capture focus from other text boxes
 	win.keyboard_capture(&filter);
@@ -200,7 +194,7 @@ void mp_create_game::pre_show(window& win)
 	game_menu_button.set_values(game_types, get_initial_type_index());
 
 	connect_signal_notify_modified(game_menu_button,
-		std::bind(&mp_create_game::update_games_list, this, std::ref(win)));
+		std::bind(&mp_create_game::update_games_list, this));
 
 	//
 	// Set up mods list
@@ -227,7 +221,7 @@ void mp_create_game::pre_show(window& win)
 			mog_toggle.set_value_bool(true);
 		}
 
-		connect_signal_notify_modified(mog_toggle, std::bind(&mp_create_game::on_mod_toggle, this, std::ref(win), i, &mog_toggle));
+		connect_signal_notify_modified(mog_toggle, std::bind(&mp_create_game::on_mod_toggle, this, i, &mog_toggle));
 	}
 
 	// No mods, hide the header
@@ -253,14 +247,14 @@ void mp_create_game::pre_show(window& win)
 	eras_menu_button_->set_values(era_names);
 
 	connect_signal_notify_modified(*eras_menu_button_,
-		std::bind(&mp_create_game::on_era_select, this, std::ref(win)));
+		std::bind(&mp_create_game::on_era_select, this));
 
 	const int era_selection = create_engine_.find_extra_by_id(ng::create_engine::ERA, preferences::era());
 	if(era_selection >= 0) {
 		eras_menu_button_->set_selected(era_selection);
 	}
 
-	on_era_select(win);
+	on_era_select();
 
 	//
 	// Set up random faction mode menu_button
@@ -288,9 +282,9 @@ void mp_create_game::pre_show(window& win)
 	rfm_menu_button.set_values(rfm_options, initial_index);
 
 	connect_signal_notify_modified(rfm_menu_button,
-		std::bind(&mp_create_game::on_random_faction_mode_select, this, std::ref(win)));
+		std::bind(&mp_create_game::on_random_faction_mode_select, this));
 
-	on_random_faction_mode_select(win);
+	on_random_faction_mode_select();
 
 	//
 	// Set up the setting status labels
@@ -309,11 +303,6 @@ void mp_create_game::pre_show(window& win)
 	// Disable certain settings if we're playing a local game.
 	//
 
-	// Don't allow a 'registered users only' game if the host themselves isn't registered.
-	if(local_mode_ || (host_info_ && !host_info_->registered)) {
-		registered_users_->widget_set_enabled(win, false, false);
-	}
-
 	if(local_mode_) {
 		find_widget<text_box>(&win, "game_name", false).set_active(false);
 		find_widget<text_box>(&win, "game_password", false).set_active(false);
@@ -329,7 +318,7 @@ void mp_create_game::pre_show(window& win)
 	listbox& tab_bar = find_widget<listbox>(&win, "tab_bar", false);
 
 	connect_signal_notify_modified(tab_bar,
-		std::bind(&mp_create_game::on_tab_select, this, std::ref(win)));
+		std::bind(&mp_create_game::on_tab_select, this));
 
 	// Allow the settings stack to find widgets in all pages, regardless of which is selected.
 	// This ensures settings (especially game settings) widgets are appropriately updated when
@@ -344,15 +333,15 @@ void mp_create_game::pre_show(window& win)
 	listbox& list = find_widget<listbox>(&win, "games_list", false);
 
 	connect_signal_notify_modified(list,
-		std::bind(&mp_create_game::on_game_select, this, std::ref(win)));
+		std::bind(&mp_create_game::on_game_select, this));
 
 	win.add_to_keyboard_chain(&list);
 
 	// This handles the initial game selection as well
-	display_games_of_type(win, level_types_[get_initial_type_index()].first, preferences::level());
+	display_games_of_type(level_types_[get_initial_type_index()].first, preferences::level());
 
 	// Initial tab selection must be done after game selection so the field widgets are set to their correct active state.
-	on_tab_select(win);
+	on_tab_select();
 
 	//
 	// Set up the Lua plugin context
@@ -361,7 +350,7 @@ void mp_create_game::pre_show(window& win)
 
 	plugins_context_->set_callback("create", [&win](const config&) { win.set_retval(retval::OK); }, false);
 	plugins_context_->set_callback("quit",   [&win](const config&) { win.set_retval(retval::CANCEL); }, false);
-	plugins_context_->set_callback("load",   [this, &win](const config&) { load_game_callback(win); }, false);
+	plugins_context_->set_callback("load",   [this](const config&) { load_game_callback(); }, false);
 
 #define UPDATE_ATTRIBUTE(field, convert) \
 	do { if(cfg.has_attribute(#field)) { field##_->set_widget_value(win, cfg[#field].convert()); } } while(false) \
@@ -380,7 +369,6 @@ void mp_create_game::pre_show(window& win)
 		UPDATE_ATTRIBUTE(reservoir, to_int);
 		UPDATE_ATTRIBUTE(action_bonus, to_int);
 		UPDATE_ATTRIBUTE(observers, to_bool);
-		UPDATE_ATTRIBUTE(registered_users, to_bool);
 		UPDATE_ATTRIBUTE(strict_sync, to_bool);
 		UPDATE_ATTRIBUTE(private_replay, to_bool);
 		UPDATE_ATTRIBUTE(shuffle_sides, to_bool);
@@ -405,11 +393,10 @@ void mp_create_game::pre_show(window& win)
 	plugins_context_->set_callback("select_era",   [this](const config& cfg) {
 		create_engine_.set_current_era_index(cfg["index"].to_int()); }, true);
 
-	plugins_context_->set_callback("select_mod",   [this, &win](const config& cfg) {
-		on_mod_toggle(win, cfg["index"].to_int(), nullptr);
+	plugins_context_->set_callback("select_mod",   [this](const config& cfg) {
+		on_mod_toggle(cfg["index"].to_int(), nullptr);
 	}, true);
 
-	plugins_context_->set_accessor("game_config",  [this](const config&) {return cfg_; });
 	plugins_context_->set_accessor("get_selected", [this](const config&) {
 		const ng::level& current_level = create_engine_.current_level();
 		return config {
@@ -439,7 +426,7 @@ void mp_create_game::pre_show(window& win)
 	});
 }
 
-void mp_create_game::sync_with_depcheck(window& window)
+void mp_create_game::sync_with_depcheck()
 {
 	DBG_MP << "sync_with_depcheck: start\n";
 
@@ -463,22 +450,22 @@ void mp_create_game::sync_with_depcheck(window& window)
 			create_engine_.set_current_level(new_level_index.second);
 			selected_game_index_ = new_level_index.second;
 
-			auto& game_types_list = find_widget<menu_button>(&window, "game_types", false);
+			auto& game_types_list = find_widget<menu_button>(get_window(), "game_types", false);
 			game_types_list.set_value(std::distance(level_types_.begin(), std::find_if(level_types_.begin(), level_types_.begin(), [&](const level_type_info& info){ return info.first == new_level_index.first; })));
 
 			if(different_type) {
-				display_games_of_type(window, new_level_index.first, create_engine_.current_level().id());
+				display_games_of_type(new_level_index.first, create_engine_.current_level().id());
 			} else {
 				// This function (or rather on_game_select) might be triggered by a listbox callback, in
 				// which case we cannot use display_games_of_type since it destroys the list (and its
 				// elements) which might result in segfaults. Instead, we assume that a listbox-triggered
 				// sync_with_depcheck call never changes the game type and goes to this branch instead.
-				find_widget<listbox>(&window, "games_list", false).select_row(new_level_index.second);
+				find_widget<listbox>(get_window(), "games_list", false).select_row(new_level_index.second);
 
 				// Override the last selection so on_game_select selects the new level
 				selected_game_index_ = -1;
 
-				on_game_select(window);
+				on_game_select();
 			}
 		}
 	}
@@ -493,27 +480,27 @@ void mp_create_game::sync_with_depcheck(window& window)
 }
 
 template<typename widget>
-void mp_create_game::on_filter_change(window& window, const std::string& id, bool do_select)
+void mp_create_game::on_filter_change(const std::string& id, bool do_select)
 {
-	create_engine_.apply_level_filter(find_widget<widget>(&window, id, false).get_value());
+	create_engine_.apply_level_filter(find_widget<widget>(get_window(), id, false).get_value());
 
-	listbox& game_list = find_widget<listbox>(&window, "games_list", false);
+	listbox& game_list = find_widget<listbox>(get_window(), "games_list", false);
 
 	boost::dynamic_bitset<> filtered(game_list.get_item_count());
-	for(const size_t i : create_engine_.get_filtered_level_indices(create_engine_.current_level_type())) {
+	for(const std::size_t i : create_engine_.get_filtered_level_indices(create_engine_.current_level_type())) {
 		filtered[i] = true;
 	}
 
 	game_list.set_row_shown(filtered);
 
 	if(do_select) {
-		on_game_select(window);
+		on_game_select();
 	}
 }
 
-void mp_create_game::on_game_select(window& window)
+void mp_create_game::on_game_select()
 {
-	const int selected_game = find_widget<listbox>(&window, "games_list", false).get_selected_row();
+	const int selected_game = find_widget<listbox>(get_window(), "games_list", false).get_selected_row();
 
 	if(selected_game == selected_game_index_) {
 		return;
@@ -524,9 +511,9 @@ void mp_create_game::on_game_select(window& window)
 
 	create_engine_.set_current_level(selected_game_index_);
 
-	sync_with_depcheck(window);
+	sync_with_depcheck();
 
-	update_details(window);
+	update_details();
 
 	// General settings
 	const bool can_select_era = create_engine_.current_level().allow_era_choice();
@@ -546,13 +533,13 @@ void mp_create_game::on_game_select(window& window)
 	update_map_settings();
 }
 
-void mp_create_game::on_tab_select(window& window)
+void mp_create_game::on_tab_select()
 {
-	const int i = find_widget<listbox>(&window, "tab_bar", false).get_selected_row();
-	find_widget<stacked_widget>(&window, "pager", false).select_layer(i);
+	const int i = find_widget<listbox>(get_window(), "tab_bar", false).get_selected_row();
+	find_widget<stacked_widget>(get_window(), "pager", false).select_layer(i);
 }
 
-void mp_create_game::on_mod_toggle(window& window, const int index, toggle_button* sender)
+void mp_create_game::on_mod_toggle(const int index, toggle_button* sender)
 {
 	if(sender && (sender->get_value_bool() == create_engine_.dependency_manager().is_modification_active(index))) {
 		ERR_MP << "ignoring on_mod_toggle that is already set\n";
@@ -561,47 +548,47 @@ void mp_create_game::on_mod_toggle(window& window, const int index, toggle_butto
 
 	create_engine_.toggle_mod(index);
 
-	sync_with_depcheck(window);
+	sync_with_depcheck();
 
 	options_manager_->update_mod_options();
 }
 
-void mp_create_game::on_era_select(window& window)
+void mp_create_game::on_era_select()
 {
 	create_engine_.set_current_era_index(eras_menu_button_->get_value());
 
 	eras_menu_button_->set_tooltip(create_engine_.current_era().description);
 
-	sync_with_depcheck(window);
+	sync_with_depcheck();
 
 	options_manager_->update_era_options();
 }
 
-void mp_create_game::on_random_faction_mode_select(window& window)
+void mp_create_game::on_random_faction_mode_select()
 {
-	selected_rfm_index_ = find_widget<menu_button>(&window, "random_faction_mode", false).get_value();
+	selected_rfm_index_ = find_widget<menu_button>(get_window(), "random_faction_mode", false).get_value();
 }
 
-void mp_create_game::show_description(window& window, const std::string& new_description)
+void mp_create_game::show_description(const std::string& new_description)
 {
-	styled_widget& description = find_widget<styled_widget>(&window, "description", false);
+	styled_widget& description = find_widget<styled_widget>(get_window(), "description", false);
 
 	description.set_label(!new_description.empty() ? new_description : _("No description available."));
 	description.set_use_markup(true);
 }
 
-void mp_create_game::update_games_list(window& window)
+void mp_create_game::update_games_list()
 {
-	const int index = find_widget<menu_button>(&window, "game_types", false).get_value();
+	const int index = find_widget<menu_button>(get_window(), "game_types", false).get_value();
 
-	display_games_of_type(window, level_types_[index].first, create_engine_.current_level().id());
+	display_games_of_type(level_types_[index].first, create_engine_.current_level().id());
 }
 
-void mp_create_game::display_games_of_type(window& window, ng::level::TYPE type, const std::string& level)
+void mp_create_game::display_games_of_type(ng::level::TYPE type, const std::string& level)
 {
 	create_engine_.set_current_level_type(type);
 
-	listbox& list = find_widget<listbox>(&window, "games_list", false);
+	listbox& list = find_widget<listbox>(get_window(), "games_list", false);
 
 	list.clear();
 
@@ -622,50 +609,50 @@ void mp_create_game::display_games_of_type(window& window, ng::level::TYPE type,
 
 	if(!level.empty() && !list.get_rows_shown().empty()) {
 		// Recalculate which rows should be visible
-		on_filter_change<slider>(window, "num_players", false);
-		on_filter_change<text_box>(window, "game_filter", false);
+		on_filter_change<slider>("num_players", false);
+		on_filter_change<text_box>("game_filter", false);
 
 		int level_index = create_engine_.find_level_by_id(level).second;
-		if(level_index >= 0 && size_t(level_index) < list.get_item_count()) {
+		if(level_index >= 0 && std::size_t(level_index) < list.get_item_count()) {
 			list.select_row(level_index);
 		}
 	}
 
 	const bool is_random_map = type == ng::level::TYPE::RANDOM_MAP;
 
-	find_widget<button>(&window, "random_map_regenerate", false).set_active(is_random_map);
-	find_widget<button>(&window, "random_map_settings", false).set_active(is_random_map);
+	find_widget<button>(get_window(), "random_map_regenerate", false).set_active(is_random_map);
+	find_widget<button>(get_window(), "random_map_settings", false).set_active(is_random_map);
 
 	// Override the last selection so on_game_select selects the new level
 	selected_game_index_ = -1;
 
-	on_game_select(window);
+	on_game_select();
 }
 
-void mp_create_game::show_generator_settings(window& window)
+void mp_create_game::show_generator_settings()
 {
 	create_engine_.generator_user_config();
 
-	regenerate_random_map(window);
+	regenerate_random_map();
 }
 
-void mp_create_game::regenerate_random_map(window& window)
+void mp_create_game::regenerate_random_map()
 {
 	create_engine_.init_generated_level_data();
 
-	update_details(window);
+	update_details();
 }
 
 int mp_create_game::convert_to_game_filtered_index(const unsigned int initial_index)
 {
-	const std::vector<size_t>& filtered_indices = create_engine_.get_filtered_level_indices(create_engine_.current_level_type());
+	const std::vector<std::size_t>& filtered_indices = create_engine_.get_filtered_level_indices(create_engine_.current_level_type());
 	return std::distance(filtered_indices.begin(), std::find(filtered_indices.begin(), filtered_indices.end(), initial_index));
 }
 
-void mp_create_game::update_details(window& win)
+void mp_create_game::update_details()
 {
-	styled_widget& players = find_widget<styled_widget>(&win, "map_num_players", false);
-	styled_widget& map_size = find_widget<styled_widget>(&win, "map_size", false);
+	styled_widget& players = find_widget<styled_widget>(get_window(), "map_num_players", false);
+	styled_widget& map_size = find_widget<styled_widget>(get_window(), "map_size", false);
 
 	if(create_engine_.current_level_type() == ng::level::TYPE::RANDOM_MAP) {
 		// If the current random map doesn't have data, generate it
@@ -675,7 +662,7 @@ void mp_create_game::update_details(window& win)
 			create_engine_.init_generated_level_data();
 		}
 
-		find_widget<button>(&win, "random_map_settings", false).set_active(create_engine_.generator_has_settings());
+		find_widget<button>(get_window(), "random_map_settings", false).set_active(create_engine_.generator_has_settings());
 	}
 
 	create_engine_.current_level().set_metadata();
@@ -688,9 +675,9 @@ void mp_create_game::update_details(window& win)
 	// Set the title, with newlines replaced. Newlines are sometimes found in SP Campaign names
 	std::string title = create_engine_.current_level().name();
 	boost::replace_all(title, "\n", " " + font::unicode_em_dash + " ");
-	find_widget<styled_widget>(&win, "game_title", false).set_label(title);
+	find_widget<styled_widget>(get_window(), "game_title", false).set_label(title);
 
-	show_description(win, create_engine_.current_level().description());
+	show_description(create_engine_.current_level().description());
 
 	switch(create_engine_.current_level_type().v) {
 		case ng::level::TYPE::SCENARIO:
@@ -703,15 +690,14 @@ void mp_create_game::update_details(window& win)
 
 			create_engine_.get_state().classification().campaign = "";
 
-			find_widget<stacked_widget>(&win, "minimap_stack", false).select_layer(0);
-			const std::string map_data = !current_scenario->data()["map_data"].empty()
-				? current_scenario->data()["map_data"]
-				: filesystem::read_map(current_scenario->data()["map_file"]);
-			if (current_scenario->data()["map_data"].empty()) {
-				current_scenario->data()["map_data"] = map_data;
+			find_widget<stacked_widget>(get_window(), "minimap_stack", false).select_layer(0);
+
+			if(current_scenario->data()["map_data"].empty()) {
+				saved_game::expand_map_file(current_scenario->data());
 				current_scenario->set_metadata();
 			}
-			find_widget<minimap>(&win, "minimap", false).set_map_data(map_data);
+
+			find_widget<minimap>(get_window(), "minimap", false).set_map_data(current_scenario->data()["map_data"]);
 
 			players.set_label(std::to_string(current_scenario->num_players()));
 			map_size.set_label(current_scenario->map_size());
@@ -728,8 +714,8 @@ void mp_create_game::update_details(window& win)
 
 			const std::string img = formatter() << current_campaign->data()["image"] << "~SCALE_INTO(265,265)";
 
-			find_widget<stacked_widget>(&win, "minimap_stack", false).select_layer(1);
-			find_widget<image>(&win, "campaign_image", false).set_image(img);
+			find_widget<stacked_widget>(get_window(), "minimap_stack", false).select_layer(1);
+			find_widget<image>(get_window(), "campaign_image", false).set_image(img);
 
 			const int p_min = current_campaign->min_players();
 			const int p_max = current_campaign->max_players();
@@ -790,9 +776,9 @@ void mp_create_game::update_map_settings()
 	}
 }
 
-void mp_create_game::load_game_callback(window& window)
+void mp_create_game::load_game_callback()
 {
-	savegame::loadgame load(cfg_, create_engine_.get_state());
+	savegame::loadgame load(savegame::save_index_class::default_saves_dir(), create_engine_.get_state());
 
 	if(!load.load_multiplayer_game()) {
 		return;
@@ -802,7 +788,7 @@ void mp_create_game::load_game_callback(window& window)
 		create_engine_.get_state().cancel_orders();
 	}
 
-	window.set_retval(LOAD_GAME);
+	set_retval(LOAD_GAME);
 }
 
 std::vector<std::string> mp_create_game::get_active_mods()
@@ -839,7 +825,7 @@ bool mp_create_game::dialog_exit_hook(window& /*window*/)
 	if(!create_engine_.current_level().can_launch_game()) {
 		std::stringstream msg;
 		// TRANSLATORS: This sentence will be followed by some details of the error, most likely the "Map could not be loaded" message from create_engine.cpp
-		msg << _("The selected game can not be created.");
+		msg << _("The selected game cannot be created.");
 		msg << "\n\n";
 		msg << create_engine_.current_level().description();
 		gui2::show_transient_error_message(msg.str());
@@ -862,6 +848,9 @@ void mp_create_game::post_show(window& window)
 
 	if(get_retval() == LOAD_GAME) {
 		create_engine_.prepare_for_saved_game();
+
+		// We don't need the LOAD_GAME retval past this point. For convenience, reset it to OK so we can use the execute wrapper, then exit.
+		set_retval(retval::OK);
 		return;
 	}
 
@@ -942,7 +931,6 @@ void mp_create_game::post_show(window& window)
 		config_engine_->set_mp_countdown_action_bonus(action_bonus_->get_widget_value(window));
 
 		config_engine_->set_allow_observers(observers_->get_widget_value(window));
-		config_engine_->set_registered_users_only(registered_users_->get_widget_value(window));
 		config_engine_->set_private_replay(private_replay_->get_widget_value(window));
 		config_engine_->set_oos_debug(strict_sync_->get_widget_value(window));
 		config_engine_->set_shuffle_sides(shuffle_sides_->get_widget_value(window));
@@ -970,4 +958,3 @@ void mp_create_game::post_show(window& window)
 }
 
 } // namespace dialogs
-} // namespace gui2
