@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2008 - 2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -22,24 +23,25 @@
 #include "game_config.hpp"
 #include "game_config_manager.hpp"
 #include "game_initialization/mp_game_utils.hpp"
-#include "preferences/credentials.hpp"
 #include "gettext.hpp"
 #include "gui/auxiliary/find_widget.hpp"
 #include "gui/core/timer.hpp"
 #include "gui/dialogs/loading_screen.hpp"
 #include "gui/dialogs/multiplayer/faction_select.hpp"
+#include "gui/dialogs/multiplayer/player_list_helper.hpp"
 #include "gui/dialogs/transient_message.hpp"
 #include "gui/widgets/button.hpp"
 #include "gui/widgets/chatbox.hpp"
-#include "gui/widgets/menu_button.hpp"
 #include "gui/widgets/image.hpp"
 #include "gui/widgets/label.hpp"
 #include "gui/widgets/listbox.hpp"
-#include "gui/widgets/settings.hpp"
+#include "gui/widgets/menu_button.hpp"
 #include "gui/widgets/tree_view.hpp"
 #include "gui/widgets/tree_view_node.hpp"
 #include "log.hpp"
 #include "mp_ui_alerts.hpp"
+#include "preferences/credentials.hpp"
+#include "saved_game.hpp"
 #include "statistics.hpp"
 #include "units/types.hpp"
 #include "utils/scope_exit.hpp"
@@ -51,17 +53,14 @@ static lg::log_domain log_mp_connect_engine("mp/connect/engine");
 #define WRN_MP LOG_STREAM(warn, log_mp_connect_engine)
 #define ERR_MP LOG_STREAM(err, log_mp_connect_engine)
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 REGISTER_DIALOG(mp_join_game)
 
-mp_join_game::mp_join_game(saved_game& state, mp::lobby_info& lobby_info, wesnothd_connection& connection, const bool first_scenario, const bool observe_game)
+mp_join_game::mp_join_game(saved_game& state, wesnothd_connection& connection, const bool first_scenario, const bool observe_game)
 	: level_()
 	, state_(state)
-	, lobby_info_(lobby_info)
 	, network_connection_(connection)
 	, update_timer_(0)
 	, first_scenario_(first_scenario)
@@ -94,16 +93,16 @@ bool mp_join_game::fetch_game_config()
 	bool has_scenario_and_controllers = false;
 	while(!has_scenario_and_controllers) {
 		config revc;
-		const bool data_res =
-			network_connection_.fetch_data_with_loading_screen(revc, loading_stage::download_level_data);
 
-		if(!data_res) {
-			return false;
-		}
+		gui2::dialogs::loading_screen::display([&]() {
+			gui2::dialogs::loading_screen::progress(loading_stage::download_level_data);
 
-		mp::check_response(data_res, revc);
+			network_connection_.wait_and_receive_data(revc);
+		});
 
-		if(revc.child("leave_game")) {
+		if(const config& err = revc.child("error")) {
+			throw wesnothd_error(err["message"]);
+		} else if(revc.child("leave_game")) {
 			return false;
 		} else if(config& next_scenario = revc.child("next_scenario")) {
 			level_.swap(next_scenario);
@@ -134,10 +133,22 @@ bool mp_join_game::fetch_game_config()
 		state_.classification() = game_classification(level_);
 
 		// Make sure that we have the same config as host, if possible.
-		game_config_manager::get()->load_game_config_for_game(state_.classification());
+		std::string scenario_id = state_.get_scenario_id();
+		// since add-ons are now only enabled when used, the scenario ID may still not be known
+		// so check in the MP info sent from the server for the scenario ID if that's the case
+		if(scenario_id == "") {
+			for(const auto& addon : level_.child("multiplayer").child_range("addon")) {
+				for(const auto& content : addon.child_range("content")) {
+					if(content["type"] == "scenario") {
+						scenario_id = content["id"].str();
+					}
+				}
+			}
+		}
+		game_config_manager::get()->load_game_config_for_game(state_.classification(), scenario_id);
 	}
 
-	game_config::add_color_info(get_scenario());
+	game_config::add_color_info(game_config_view::wrap(get_scenario()));
 
 	// If we're just an observer, we don't need to find an appropriate side and set faction selection
 	if(observe_game_) {
@@ -230,19 +241,19 @@ void mp_join_game::pre_show(window& window)
 	// Set title
 	//
 	label& title = find_widget<label>(&window, "title", false);
-	title.set_label((formatter() << title.get_label() << " " << font::unicode_em_dash << " " << get_scenario()["name"].t_str()).str());
+	// FIXME: very hacky way to get the game name...
+	title.set_label((formatter() << level_.child("multiplayer")["scenario"] << " " << font::unicode_em_dash << " " << get_scenario()["name"].t_str()).str());
 
 	//
 	// Set up sides list
 	//
-	generate_side_list(window);
+	generate_side_list();
 
 	//
 	// Initialize chatbox and game rooms
 	//
 	chatbox& chat = find_widget<chatbox>(&window, "chat", false);
 
-	chat.set_lobby_info(lobby_info_);
 	chat.set_wesnothd_connection(network_connection_);
 
 	chat.room_window_open(N_("this game"), true, false);
@@ -257,7 +268,7 @@ void mp_join_game::pre_show(window& window)
 	//
 	// Set up the network handling
 	//
-	update_timer_ = add_timer(game_config::lobby_network_timer, std::bind(&mp_join_game::network_handler, this, std::ref(window)), true);
+	update_timer_ = add_timer(game_config::lobby_network_timer, std::bind(&mp_join_game::network_handler, this), true);
 
 	//
 	// Set up the Lua plugin context
@@ -301,14 +312,16 @@ bool mp_join_game::show_flg_select(int side_num, bool first_time)
 		const bool is_mp = state_.classification().is_normal_mp_game();
 		const bool lock_settings = get_scenario()["force_lock_settings"].to_bool(!is_mp);
 		const bool use_map_settings = level_.child("multiplayer")["mp_use_map_settings"].to_bool();
-		const bool saved_game = level_.child("multiplayer")["savegame"].to_bool();
+		const mp_game_settings::SAVED_GAME_MODE saved_game = level_.child("multiplayer")["savegame"].to_enum<mp_game_settings::SAVED_GAME_MODE>(mp_game_settings::SAVED_GAME_MODE::NONE);
 
-		ng::flg_manager flg(era_factions, side_choice, lock_settings, use_map_settings, saved_game);
+		ng::flg_manager flg(era_factions, side_choice, lock_settings, use_map_settings, saved_game == mp_game_settings::SAVED_GAME_MODE::MIDGAME);
 
 		{
 			gui2::dialogs::faction_select flg_dialog(flg, color, side_num);
 			flg_dialog_ = &flg_dialog;
-			utils::scope_exit se([this]() { flg_dialog_ = nullptr; });
+			ON_SCOPE_EXIT(this) {
+				flg_dialog_ = nullptr;
+			};
 
 			if(!flg_dialog.show() && !first_time) {
 				return true;
@@ -331,15 +344,13 @@ bool mp_join_game::show_flg_select(int side_num, bool first_time)
 	return true;
 }
 
-void mp_join_game::generate_side_list(window& window)
+void mp_join_game::generate_side_list()
 {
 	if(stop_updates_) {
 		return;
 	}
 
-	tree_view& tree = find_widget<tree_view>(&window, "side_list", false);
-
-	window.keyboard_capture(&tree);
+	tree_view& tree = find_widget<tree_view>(get_window(), "side_list", false);
 
 	tree.clear();
 	team_tree_map_.clear();
@@ -357,7 +368,7 @@ void mp_join_game::generate_side_list(window& window)
 			std::map<std::string, string_map> data;
 			string_map item;
 
-			item["label"] = (formatter() << _("Team:") << " " << t_string::from_serialized(side["user_team_name"])).str();
+			item["label"] = t_string::from_serialized(side["user_team_name"]);
 			data.emplace("tree_view_node_label", item);
 
 			tree_view_node& team_node = tree.add_node("team_header", data);
@@ -460,7 +471,7 @@ void mp_join_game::generate_side_list(window& window)
 					handled = halt = true;
 				};
 
-				connect_signal_mouse_left_click(*select_leader_button, std::bind(handler, _3, _4));
+				connect_signal_mouse_left_click(*select_leader_button, std::bind(handler, std::placeholders::_3, std::placeholders::_4));
 			} else {
 				select_leader_button->set_visible(widget::visibility::hidden);
 			}
@@ -482,11 +493,11 @@ void mp_join_game::close_faction_select_dialog_if_open()
 	}
 }
 
-void mp_join_game::network_handler(window& window)
+void mp_join_game::network_handler()
 {
 	// If the game has already started, close the dialog immediately.
 	if(level_["started"].to_bool()) {
-		window.set_retval(retval::OK);
+		set_retval(retval::OK);
 		return;
 	}
 
@@ -496,7 +507,7 @@ void mp_join_game::network_handler(window& window)
 	}
 
 	// Update chat
-	find_widget<chatbox>(&window, "chat", false).process_network_data(data);
+	find_widget<chatbox>(get_window(), "chat", false).process_network_data(data);
 
 	if(!data["message"].empty()) {
 		gui2::show_transient_message(_("Response") , data["message"]);
@@ -505,16 +516,16 @@ void mp_join_game::network_handler(window& window)
 	if(data["failed"].to_bool()) {
 		close_faction_select_dialog_if_open();
 
-		window.set_retval(retval::CANCEL);
+		set_retval(retval::CANCEL);
 	} else if(data.child("start_game")) {
 		close_faction_select_dialog_if_open();
 
 		level_["started"] = true;
-		window.set_retval(retval::OK);
+		set_retval(retval::OK);
 	} else if(data.child("leave_game")) {
 		close_faction_select_dialog_if_open();
 
-		window.set_retval(retval::CANCEL);
+		set_retval(retval::CANCEL);
 	}
 
 	if(data.child("stop_updates")) {
@@ -523,7 +534,7 @@ void mp_join_game::network_handler(window& window)
 		// TODO: We should catch config::error and then leave the game.
 		level_.apply_diff(c);
 
-		generate_side_list(window);
+		generate_side_list();
 	} else if(const config& change = data.child("change_controller")) {
 		if(config& side_to_change = get_scenario().find_child("side", "side", change["side"])) {
 			side_to_change.merge_with(change);
@@ -535,7 +546,7 @@ void mp_join_game::network_handler(window& window)
 	} else if(data.has_child("scenario") || data.has_child("snapshot") || data.child("next_scenario")) {
 		level_ = first_scenario_ ? data : data.child("next_scenario");
 
-		generate_side_list(window);
+		generate_side_list();
 	}
 
 	if(data.has_child("turn")) {
@@ -583,4 +594,3 @@ void mp_join_game::post_show(window& window)
 }
 
 } // namespace dialogs
-} // namespace gui2
