@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2008 - 2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -27,9 +28,7 @@
 #include "gui/widgets/window.hpp"
 #include "deprecation.hpp"
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 mp_options_helper::mp_options_helper(window& window, ng::create_engine& create_engine)
@@ -252,11 +251,7 @@ void mp_options_helper::display_custom_options(const std::string& type, int node
 			} else if(opt.key == "spacer") {
 				option_node.add_child("options_spacer_node", empty_map);
 
-			} else if(opt.key == "choice" || opt.key == "combo") {
-				if(opt.key == "combo") {
-					deprecated_message("combo", DEP_LEVEL::FOR_REMOVAL, {1, 15, 0}, "Use [choice] instead.");
-				}
-
+			} else if(opt.key == "choice") {
 				if(!option_cfg.has_child("item")) {
 					continue;
 				}
@@ -343,4 +338,3 @@ config mp_options_helper::get_options_config()
 }
 
 } // namespace dialogs
-} // namespace gui2 // namespace gui2
