@@ -1,20 +1,22 @@
 /*
-   Copyright (C) 2008 - 2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "gui/dialogs/multiplayer/mp_staging.hpp"
 
+#include "ai/configuration.hpp"
 #include "chat_log.hpp"
 #include "font/text_formatting.hpp"
 #include "formatter.hpp"
@@ -22,13 +24,14 @@
 #include "gettext.hpp"
 #include "gui/auxiliary/find_widget.hpp"
 #include "gui/dialogs/multiplayer/faction_select.hpp"
+#include "gui/dialogs/multiplayer/player_list_helper.hpp"
 #include "gui/widgets/button.hpp"
 #include "gui/widgets/chatbox.hpp"
 #include "gui/widgets/drawing.hpp"
-#include "gui/widgets/menu_button.hpp"
 #include "gui/widgets/image.hpp"
-#include "gui/widgets/listbox.hpp"
 #include "gui/widgets/label.hpp"
+#include "gui/widgets/listbox.hpp"
+#include "gui/widgets/menu_button.hpp"
 #include "gui/widgets/settings.hpp"
 #include "gui/widgets/slider.hpp"
 #include "gui/widgets/status_label_helper.hpp"
@@ -39,17 +42,14 @@
 #include "units/types.hpp"
 #include "wesnothd_connection.hpp"
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 REGISTER_DIALOG(mp_staging)
 
-mp_staging::mp_staging(ng::connect_engine& connect_engine, mp::lobby_info& lobby_info, wesnothd_connection* connection)
+mp_staging::mp_staging(ng::connect_engine& connect_engine, wesnothd_connection* connection)
 	: connect_engine_(connect_engine)
 	, ai_algorithms_(ai::configuration::get_available_ais())
-	, lobby_info_(lobby_info)
 	, network_connection_(connection)
 	, update_timer_(0)
 	, state_changed_(false)
@@ -79,36 +79,16 @@ void mp_staging::pre_show(window& window)
 	// Set title and status widget states
 	//
 	label& title = find_widget<label>(&window, "title", false);
-	title.set_label((formatter() << title.get_label() << " " << font::unicode_em_dash << " " << connect_engine_.scenario()["name"].t_str()).str());
+	title.set_label((formatter() << connect_engine_.params().name << " " << font::unicode_em_dash << " " << connect_engine_.scenario()["name"].t_str()).str());
 
-	update_status_label_and_buttons(window);
-
-	//
-	// Set up teams
-	//
-	for(const ng::connect_engine::team_data_pod& team : connect_engine_.team_data()) {
-		tree_view& tree = find_widget<tree_view>(&window, "side_list", false);
-		static const std::map<std::string, string_map> empty_map;
-
-		std::map<std::string, string_map> tree_data;
-		string_map tree_item;
-
-		tree_item["label"] = (formatter() << _("Team:") << " " << t_string::from_serialized(team.user_team_name)).str();
-		tree_data.emplace("tree_view_node_label", tree_item);
-
-		tree_view_node& team_node = tree.add_node("team_header", tree_data);
-		team_node.add_sibling("side_spacer", empty_map);
-
-		team_tree_map_[team.team_name] = &team_node;
-	}
+	update_status_label_and_buttons();
 
 	//
 	// Set up sides list
-	// This must be after setting up teams because add_side_node() uses team_tree_map_
 	//
 	for(const auto& side : connect_engine_.side_engines()) {
 		if(side->allow_player() || game_config::debug) {
-			add_side_node(window, side);;
+			add_side_node(side);;
 		}
 	}
 
@@ -117,8 +97,6 @@ void mp_staging::pre_show(window& window)
 	//
 	chatbox& chat = find_widget<chatbox>(&window, "chat", false);
 
-	chat.set_lobby_info(lobby_info_);
-
 	if(network_connection_) {
 		chat.set_wesnothd_connection(*network_connection_);
 	}
@@ -135,7 +113,7 @@ void mp_staging::pre_show(window& window)
 	//
 	// Set up the network handling
 	//
-	update_timer_ = add_timer(game_config::lobby_network_timer, std::bind(&mp_staging::network_handler, this, std::ref(window)), true);
+	update_timer_ = add_timer(game_config::lobby_network_timer, std::bind(&mp_staging::network_handler, this), true);
 
 	//
 	// Set up the Lua plugin context
@@ -159,7 +137,35 @@ int mp_staging::get_side_node_position(ng::side_engine_ptr side) const
 	return position;
 }
 
-void mp_staging::add_side_node(window& window, ng::side_engine_ptr side)
+template<typename... T>
+tree_view_node& mp_staging::add_side_to_team_node(ng::side_engine_ptr side, T&&... params)
+{
+	static const std::map<std::string, string_map> empty_map;
+
+	// If there is no team node in the map, this will return nullptr
+	tree_view_node* team_node = team_tree_map_[side->team_name()];
+
+	// Add a team node if none exists
+	if(team_node == nullptr) {
+		tree_view& tree = find_widget<tree_view>(get_window(), "side_list", false);
+
+		std::map<std::string, string_map> tree_data;
+		string_map tree_item;
+
+		tree_item["label"] = side->user_team_name();
+		tree_data.emplace("tree_view_node_label", tree_item);
+
+		team_node = &tree.add_node("team_header", tree_data);
+		team_node->add_sibling("side_spacer", empty_map);
+
+		team_tree_map_[side->team_name()] = team_node;
+	}
+
+	assert(team_node && "No team node found!");
+	return team_node->add_child(std::forward<T>(params)...);
+}
+
+void mp_staging::add_side_node(ng::side_engine_ptr side)
 {
 	std::map<std::string, string_map> data;
 	string_map item;
@@ -174,7 +180,7 @@ void mp_staging::add_side_node(window& window, ng::side_engine_ptr side)
 	item["label"] = "icons/icon-random.png";
 	data.emplace("leader_gender", item);
 
-	tree_view_node& node = team_tree_map_[side->team_name()]->add_child("side_panel", data, get_side_node_position(side));
+	tree_view_node& node = add_side_to_team_node(side, "side_panel", data, get_side_node_position(side));
 
 	side_tree_map_[side] = &node;
 
@@ -191,7 +197,7 @@ void mp_staging::add_side_node(window& window, ng::side_engine_ptr side)
 	const bool lock_team   = side->cfg()["team_lock"].to_bool(fls);
 	const bool lock_color  = side->cfg()["color_lock"].to_bool(fls);
 
-	const bool saved_game = connect_engine_.params().saved_game;
+	const bool saved_game = connect_engine_.params().saved_game == mp_game_settings::SAVED_GAME_MODE::MIDGAME;
 
 	//
 	// AI Algorithm
@@ -245,6 +251,7 @@ void mp_staging::add_side_node(window& window, ng::side_engine_ptr side)
 	//
 	button& leader_select = find_widget<button>(&row_grid, "select_leader", false);
 
+	//todo: shouldn't this also be disabled when the flg settings are locked.
 	leader_select.set_active(!saved_game);
 
 	connect_signal_mouse_left_click(leader_select,
@@ -284,10 +291,11 @@ void mp_staging::add_side_node(window& window, ng::side_engine_ptr side)
 	menu_button& team_selection = find_widget<menu_button>(&row_grid, "side_team", false);
 
 	team_selection.set_values(team_names, initial_team_selection);
+	//todo: shouldn't this also be disabled when team settings are locked.
 	team_selection.set_active(!saved_game);
 
 	connect_signal_notify_modified(team_selection,
-		std::bind(&mp_staging::on_team_select, this, std::ref(window), side, std::ref(team_selection)));
+		std::bind(&mp_staging::on_team_select, this, side, std::ref(team_selection)));
 
 	//
 	// Colors
@@ -393,7 +401,7 @@ void mp_staging::on_color_select(ng::side_engine_ptr side, grid& row_grid)
 	set_state_changed();
 }
 
-void mp_staging::on_team_select(window& window, ng::side_engine_ptr side, menu_button& team_menu)
+void mp_staging::on_team_select(ng::side_engine_ptr side, menu_button& team_menu)
 {
 	// Since we're not necessarily displaying every every team in the menu, we can't just
 	// use the selected index to set a side's team. Instead, we grab the index we stored
@@ -405,13 +413,29 @@ void mp_staging::on_team_select(window& window, ng::side_engine_ptr side, menu_b
 		return;
 	}
 
+	// Note the old team so we can remove the node if empty after the side move.
+	// Do this *before* setting the new team!
+	const std::string old_team = side->team_name();
 	side->set_team(team_index);
 
+	auto& tree = find_widget<tree_view>(get_window(), "side_list", false);
+
 	// First, remove the node from the tree
-	auto node = find_widget<tree_view>(&window, "side_list", false).remove_node(side_tree_map_[side]);
+	auto node = tree.remove_node(side_tree_map_[side]);
 
 	// Then add a new node as a child to the appropriate team's node
-	team_tree_map_[side->team_name()]->add_child(std::move(node.first), get_side_node_position(side));
+	add_side_to_team_node(side, std::move(node.first), get_side_node_position(side));
+
+	tree_view_node* old_team_node = team_tree_map_[old_team];
+
+	// Last, remove the old team node if it's now empty
+	if(old_team_node->empty()) {
+		// Decor node will be immediately after team node. Remove this first!
+		tree.remove_node(old_team_node->get_node_below());
+		tree.remove_node(old_team_node);
+
+		team_tree_map_[old_team] = nullptr;
+	}
 
 	set_state_changed();
 }
@@ -476,18 +500,18 @@ void mp_staging::update_leader_display(ng::side_engine_ptr side, grid& row_grid)
 	}
 }
 
-void mp_staging::update_status_label_and_buttons(window& window)
+void mp_staging::update_status_label_and_buttons()
 {
-	find_widget<label>(&window, "status_label", false).set_label(
+	find_widget<label>(get_window(), "status_label", false).set_label(
 		connect_engine_.can_start_game() ? "" : connect_engine_.sides_available()
 			? _("Waiting for players to join...")
 			: _("Waiting for players to choose factions...")
 	);
 
-	find_widget<button>(&window, "ok", false).set_active(connect_engine_.can_start_game());
+	find_widget<button>(get_window(), "ok", false).set_active(connect_engine_.can_start_game());
 }
 
-void mp_staging::network_handler(window& window)
+void mp_staging::network_handler()
 {
 	// First, send off any changes if they've been accumulated
 	if(state_changed_) {
@@ -501,7 +525,7 @@ void mp_staging::network_handler(window& window)
 	}
 
 	// Update chat
-	find_widget<chatbox>(&window, "chat", false).process_network_data(data);
+	find_widget<chatbox>(get_window(), "chat", false).process_network_data(data);
 
 	// TODO: why is this needed...
 	const bool was_able_to_start = connect_engine_.can_start_game();
@@ -510,7 +534,7 @@ void mp_staging::network_handler(window& window)
 	std::tie(quit_signal_received, std::ignore) = connect_engine_.process_network_data(data);
 
 	if(quit_signal_received) {
-		window.set_retval(retval::CANCEL);
+		set_retval(retval::CANCEL);
 	}
 
 	// Update side leader displays
@@ -539,7 +563,7 @@ void mp_staging::network_handler(window& window)
 	}
 
 	// Update status label and buttons
-	update_status_label_and_buttons(window);
+	update_status_label_and_buttons();
 
 	if(!was_able_to_start && connect_engine_.can_start_game()) {
 		mp_ui_alerts::ready_for_start();
@@ -563,4 +587,3 @@ void mp_staging::post_show(window& window)
 }
 
 } // namespace dialogs
-} // namespace gui2
