@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2017-2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2017 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -20,14 +21,12 @@
 #include "game_config.hpp"
 #include <memory>
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 class plugin_executor
 {
-	size_t timer_id;
+	std::size_t timer_id;
 
 	void play_slice() {
 		if(plugins_context_) {
@@ -55,4 +54,3 @@ protected:
 };
 
 } // namespace dialogs
-} // namespace gui2
