@@ -1,38 +1,37 @@
 /*
-   Copyright (C) 2011, 2015 by Iris Morelle <shadowm2006@gmail.com>
-   Copyright (C) 2016 - 2018 by Charles Dang <exodia339gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2016 - 2021
+	by Charles Dang <exodia339gmail.com>
+	Copyright (C) 2011, 2015 by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "gui/dialogs/preferences_dialog.hpp"
 
-#include "gettext.hpp"
 #include "filesystem.hpp"
 #include "formatter.hpp"
 #include "formula/string_utils.hpp"
-#include "preferences/game.hpp"
+#include "gettext.hpp"
 #include "hotkey/hotkey_item.hpp"
+#include "lexical_cast.hpp"
 #include "preferences/credentials.hpp"
-#include "preferences/lobby.hpp"
-#include "preferences/general.hpp"
 #include "preferences/display.hpp"
-#include "utils/functional.hpp"
-#include "utils/general.hpp"
+#include "preferences/game.hpp"
+#include "preferences/general.hpp"
+#include "preferences/lobby.hpp"
 #include "video.hpp"
 
 // Sub-dialog includes
-#include "gui/dialogs/advanced_graphics_options.hpp"
 #include "gui/dialogs/game_cache_options.hpp"
 #include "gui/dialogs/hotkey_bind.hpp"
 #include "gui/dialogs/log_settings.hpp"
@@ -58,17 +57,11 @@
 #include "gui/widgets/text_box.hpp"
 #include "gui/widgets/toggle_button.hpp"
 #include "gui/widgets/window.hpp"
-#include "lexical_cast.hpp"
 
-#if BOOST_VERSION >= 106700
-#include <boost/integer/common_factor_rt.hpp>
-#else
-#include <boost/math/common_factor_rt.hpp>
-#endif
+#include <functional>
+#include <numeric>
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 namespace
 {
@@ -89,7 +82,7 @@ void disable_widget_on_toggle(window& window, widget& w, const std::string& id)
 // number of pager layers (since get_layer_count returns one-past-end).
 int index_in_pager_range(const int first, const stacked_widget& pager)
 {
-	return utils::clamp<int>(first, 0, pager.get_layer_count() - 1);
+	return std::clamp<int>(first, 0, pager.get_layer_count() - 1);
 }
 
 // Helper to make it easier to immediately apply sound toggles immediately.
@@ -112,27 +105,19 @@ void volume_setter_on_change(widget& w)
 } // end anon namespace
 
 using namespace preferences;
+using avp = preferences::advanced_manager::option;
 
 REGISTER_DIALOG(preferences_dialog)
 
-preferences_dialog::preferences_dialog(const config& game_cfg, const PREFERENCE_VIEW& initial_view)
-	: resolutions_() // should be populated by set_resolution_list before use
-	, adv_preferences_cfg_()
+preferences_dialog::preferences_dialog(const PREFERENCE_VIEW initial_view)
+	: adv_preferences_(preferences::get_advanced_preferences())
+	, resolutions_() // should be populated by set_resolution_list before use
 	, last_selected_item_(0)
 	, accl_speeds_({0.25, 0.5, 0.75, 1, 1.25, 1.5, 1.75, 2, 3, 4, 8, 16})
 	, visible_hotkeys_()
 	, cat_names_()
 	, initial_index_(pef_view_map[initial_view])
 {
-	for(const config& adv : game_cfg.child_range("advanced_preference")) {
-		adv_preferences_cfg_.push_back(adv);
-	}
-
-	std::sort(adv_preferences_cfg_.begin(), adv_preferences_cfg_.end(),
-		[](const config& lhs, const config& rhs) {
-			return lhs["name"].t_str().str() < rhs["name"].t_str().str();
-		});
-
 	for(const auto& name : hotkey::get_category_names()) {
 		// Don't include categories with no hotkeys
 		if(!hotkey::get_hotkeys_by_category(name.first).empty()) {
@@ -141,27 +126,17 @@ preferences_dialog::preferences_dialog(const config& game_cfg, const PREFERENCE_
 	}
 }
 
-void preferences_dialog::on_filtertext_changed(text_box_base* textbox)
-{
-	hotkey_filter_callback(*textbox->get_window());
-}
-
 // Helper function to refresh resolution list
-void preferences_dialog::set_resolution_list(menu_button& res_list, CVideo& video)
+void preferences_dialog::set_resolution_list(menu_button& res_list)
 {
-	resolutions_ = video.get_available_resolutions(true);
+	resolutions_ = CVideo::get_singleton().get_available_resolutions(true);
 
 	std::vector<config> options;
 	for(const point& res : resolutions_) {
 		config option;
 		option["label"] = formatter() << res.x << font::unicode_multiplication_sign << res.y;
 
-#if BOOST_VERSION >= 106700
-		const int div = boost::integer::gcd(res.x, res.y);
-#else
-		const int div = boost::math::gcd(res.x, res.y);
-#endif
-
+		const int div = std::gcd(res.x, res.y);
 		const int x_ratio = res.x / div;
 		const int y_ratio = res.y / div;
 
@@ -173,7 +148,7 @@ void preferences_dialog::set_resolution_list(menu_button& res_list, CVideo& vide
 	}
 
 	const unsigned current_res = std::distance(resolutions_.begin(), std::find(resolutions_.begin(), resolutions_.end(),
-		video.current_resolution()));
+		CVideo::get_singleton().current_resolution()));
 
 	res_list.set_values(options, current_res);
 }
@@ -225,7 +200,7 @@ void preferences_dialog::on_friends_list_select(listbox& list, text_box& textbox
 	textbox.set_value(who->second.get_nick() + " " + who->second.get_notes());
 }
 
-void preferences_dialog::update_friends_list_controls(window& window, listbox& list)
+void preferences_dialog::update_friends_list_controls(listbox& list)
 {
 	const bool list_empty = list.get_item_count() == 0;
 
@@ -233,13 +208,13 @@ void preferences_dialog::update_friends_list_controls(window& window, listbox& l
 		list.select_row(std::min(static_cast<int>(list.get_item_count()) - 1, list.get_selected_row()));
 	}
 
-	find_widget<button>(&window, "remove", false).set_active(!list_empty);
+	find_widget<button>(get_window(), "remove", false).set_active(!list_empty);
 
-	find_widget<label>(&window, "no_friends_notice", false).set_visible(
+	find_widget<label>(get_window(), "no_friends_notice", false).set_visible(
 		list_empty ? widget::visibility::visible : widget::visibility::invisible);
 }
 
-void preferences_dialog::add_friend_list_entry(const bool is_friend, text_box& textbox, window& window)
+void preferences_dialog::add_friend_list_entry(const bool is_friend, text_box& textbox)
 {
 	std::string username = textbox.text();
 	if(username.empty()) {
@@ -249,16 +224,13 @@ void preferences_dialog::add_friend_list_entry(const bool is_friend, text_box& t
 
 	std::string reason;
 
-	size_t pos = username.find_first_of(' ');
+	std::size_t pos = username.find_first_of(' ');
 	if(pos != std::string::npos) {
 		reason = username.substr(pos + 1);
 		username = username.substr(0, pos);
 	}
 
-	acquaintance* entry = nullptr;
-	bool added_new;
-
-	std::tie(entry, added_new) = add_acquaintance(username, (is_friend ? "friend": "ignore"), reason);
+	auto [entry, added_new] = add_acquaintance(username, (is_friend ? "friend": "ignore"), reason);
 
 	if(!entry) {
 		gui2::show_transient_message(_("Error"), _("Invalid username"), "", false, false, true);
@@ -267,7 +239,7 @@ void preferences_dialog::add_friend_list_entry(const bool is_friend, text_box& t
 
 	textbox.clear();
 
-	listbox& list = find_widget<listbox>(&window, "friends_list", false);
+	listbox& list = find_widget<listbox>(get_window(), "friends_list", false);
 
 	//
 	// If this is a new entry, just add a new row. If it's not, we find the relevant
@@ -290,10 +262,10 @@ void preferences_dialog::add_friend_list_entry(const bool is_friend, text_box& t
 		}
 	}
 
-	update_friends_list_controls(window, list);
+	update_friends_list_controls(list);
 }
 
-void preferences_dialog::remove_friend_list_entry(listbox& friends_list, text_box& textbox, window& window)
+void preferences_dialog::remove_friend_list_entry(listbox& friends_list, text_box& textbox)
 {
 	const int selected_row = std::max(0, friends_list.get_selected_row());
 
@@ -314,10 +286,10 @@ void preferences_dialog::remove_friend_list_entry(listbox& friends_list, text_bo
 
 	textbox.clear();
 
-	listbox& list = find_widget<listbox>(&window, "friends_list", false);
+	listbox& list = find_widget<listbox>(get_window(), "friends_list", false);
 	list.remove_row(selected_row);
 
-	update_friends_list_controls(window, list);
+	update_friends_list_controls(list);
 }
 
 template<bool(*toggle_getter)(), bool(*toggle_setter)(bool), int(*vol_getter)(), void(*vol_setter)(int)>
@@ -333,8 +305,8 @@ void preferences_dialog::initialize_sound_option_group(const std::string& id_suf
 	// the callback the setter callback is duplicated in the on-change callback. The field
 	// class could possibly use some reworking to make this less redundant, but for now it
 	// works well enough.
-	register_bool(toggle_widget_id, true, toggle_getter, bind_void(toggle_setter, _1),
-		std::bind(sound_toggle_on_change<toggle_setter>, std::ref(window), volume_widget_id, _1), true);
+	register_bool(toggle_widget_id, true, toggle_getter, std::bind(toggle_setter, std::placeholders::_1),
+		std::bind(sound_toggle_on_change<toggle_setter>, std::ref(window), volume_widget_id, std::placeholders::_1), true);
 
 	// Set up the volume slider. integer_field doesn't have a callback-on-changed mechanism.
 	// To add one would either mean adding it to the base field class or make it a proper
@@ -343,7 +315,7 @@ void preferences_dialog::initialize_sound_option_group(const std::string& id_suf
 
 	// Callback to actually immediately apply the volume effect.
 	connect_signal_notify_modified(find_widget<slider>(&window, volume_widget_id, false),
-		std::bind(volume_setter_on_change<vol_setter>, _1));
+		std::bind(volume_setter_on_change<vol_setter>, std::placeholders::_1));
 }
 
 /**
@@ -430,8 +402,7 @@ void preferences_dialog::post_build(window& window)
 
 	// We bind a special callback function, so setup_single_toggle() is not used
 	connect_signal_mouse_left_click(toggle_fullscreen, std::bind(
-			&preferences_dialog::fullscreen_toggle_callback,
-			this, std::ref(window)));
+			&preferences_dialog::fullscreen_toggle_callback, this));
 
 	/* SET RESOLUTION */
 	menu_button& res_list = find_widget<menu_button>(&window, "resolution_set", false);
@@ -439,10 +410,10 @@ void preferences_dialog::post_build(window& window)
 	res_list.set_use_markup(true);
 	res_list.set_active(!fullscreen());
 
-	set_resolution_list(res_list, window.video());
+	set_resolution_list(res_list);
 
 	connect_signal_notify_modified(res_list,
-		std::bind(&preferences_dialog::handle_res_select, this, std::ref(window)));
+		std::bind(&preferences_dialog::handle_res_select, this));
 
 	/* SHOW FLOATING LABELS */
 	register_bool("show_floating_labels", true,
@@ -474,11 +445,11 @@ void preferences_dialog::post_build(window& window)
 
 	register_integer("idle_anim_frequency", true,
 		idle_anim_rate, set_idle_anim_rate);
-#if 0
+
 	/* FONT SCALING */
-	register_integer("scaling_slider", true,
-		font_scaling, set_font_scaling);
-#endif
+	//register_integer("scaling_slider", true,
+	//	font_scaling, set_font_scaling);
+
 	/* FPS LIMITER */
 	register_bool("fps_limiter", true,
 		[]() { return draw_delay() != 0; }, [](bool v) { set_draw_delay(v ? -1 : 0); });
@@ -486,7 +457,7 @@ void preferences_dialog::post_build(window& window)
 	/* SELECT THEME */
 	connect_signal_mouse_left_click(
 			find_widget<button>(&window, "choose_theme", false),
-			bind_void(&show_theme_dialog));
+			std::bind(&show_theme_dialog));
 
 	//
 	// SOUND PANEL
@@ -547,7 +518,7 @@ void preferences_dialog::post_build(window& window)
 		friends_list.add_row(get_friends_list_row_data(entry.second));
 	}
 
-	update_friends_list_controls(window, friends_list);
+	update_friends_list_controls(friends_list);
 
 	text_box& textbox = find_widget<text_box>(&window, "friend_name_box", false);
 
@@ -555,23 +526,20 @@ void preferences_dialog::post_build(window& window)
 		find_widget<button>(&window, "add_friend", false), std::bind(
 			&preferences_dialog::add_friend_list_entry,
 			this, true,
-			std::ref(textbox),
-			std::ref(window)));
+			std::ref(textbox)));
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&window, "add_ignored", false), std::bind(
 			&preferences_dialog::add_friend_list_entry,
 			this, false,
-			std::ref(textbox),
-			std::ref(window)));
+			std::ref(textbox)));
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&window, "remove", false), std::bind(
 			&preferences_dialog::remove_friend_list_entry,
 			this,
 			std::ref(friends_list),
-			std::ref(textbox),
-			std::ref(window)));
+			std::ref(textbox)));
 
 	connect_signal_notify_modified(friends_list, std::bind(
 			&preferences_dialog::on_friends_list_select,
@@ -598,18 +566,10 @@ void preferences_dialog::post_build(window& window)
 
 	std::map<std::string, string_map> row_data;
 
-	for(const config& option : adv_preferences_cfg_) {
-		// Details about the current option
-		ADVANCED_PREF_TYPE pref_type;
-		try {
-			pref_type = ADVANCED_PREF_TYPE::string_to_enum(option["type"].str());
-		} catch(const bad_enum_cast&) {
-			continue;
-		}
+	for(const auto& option : adv_preferences_) {
+		const std::string& pref_name = option.field;
 
-		const std::string& pref_name = option["field"].str();
-
-		row_data["pref_name"]["label"] = option["name"];
+		row_data["pref_name"]["label"] = option.name;
 		advanced.add_row(row_data);
 
 		const int this_row = advanced.get_item_count() - 1;
@@ -625,16 +585,16 @@ void preferences_dialog::post_build(window& window)
 		toggle_button& toggle_box = find_widget<toggle_button>(main_grid, "value_toggle", false);
 		toggle_box.set_visible(widget::visibility::hidden);
 
-		if(!option["description"].empty()) {
-			find_widget<styled_widget>(main_grid, "description", false).set_label(option["description"]);
+		if(!option.description.empty()) {
+			find_widget<styled_widget>(main_grid, "description", false).set_label(option.description);
 		}
 
-		switch(pref_type.v) {
-			case ADVANCED_PREF_TYPE::TOGGLE: {
+		switch(option.type) {
+			case avp::avd_type::TOGGLE: {
 				//main_grid->remove_child("setter");
 
 				toggle_box.set_visible(widget::visibility::visible);
-				toggle_box.set_value(get(pref_name, option["default"].to_bool()));
+				toggle_box.set_value(get(pref_name, option.cfg["default"].to_bool()));
 
 				// We need to bind a lambda here since preferences::set is overloaded.
 				// A lambda alone would be more verbose because it'd need to specify all the parameters.
@@ -648,18 +608,18 @@ void preferences_dialog::post_build(window& window)
 				break;
 			}
 
-			case ADVANCED_PREF_TYPE::SLIDER: {
-				slider* setter_widget = build_single_widget_instance<slider>("slider", config {"definition", "minimal"});
+			case avp::avd_type::SLIDER: {
+				slider* setter_widget = build_single_widget_instance<slider>(config {"definition", "minimal"});
 				setter_widget->set_id("setter");
 				// Maximum must be set first or this will assert
-				setter_widget->set_value_range(option["min"].to_int(), option["max"].to_int());
-				setter_widget->set_step_size(option["step"].to_int(1));
+				setter_widget->set_value_range(option.cfg["min"].to_int(), option.cfg["max"].to_int());
+				setter_widget->set_step_size(option.cfg["step"].to_int(1));
 
 				details_grid.swap_child("setter", setter_widget, true);
 
 				slider& slide = find_widget<slider>(&details_grid, "setter", false);
 
-				slide.set_value(lexical_cast_default<int>(get(pref_name), option["default"].to_int()));
+				slide.set_value(lexical_cast_default<int>(get(pref_name), option.cfg["default"].to_int()));
 
 				// We need to bind a lambda here since preferences::set is overloaded.
 				// A lambda alone would be more verbose because it'd need to specify all the parameters.
@@ -672,11 +632,11 @@ void preferences_dialog::post_build(window& window)
 				break;
 			}
 
-			case ADVANCED_PREF_TYPE::COMBO: {
+			case avp::avd_type::COMBO: {
 				std::vector<config> menu_data;
 				std::vector<std::string> option_ids;
 
-				for(const config& choice : option.child_range("option")) {
+				for(const config& choice : option.cfg.child_range("option")) {
 					config menu_item;
 					menu_item["label"] = choice["name"];
 					if(choice.has_attribute("description")) {
@@ -688,7 +648,7 @@ void preferences_dialog::post_build(window& window)
 
 				// Attempt to find an initial selection
 				int selected = std::distance(option_ids.begin(), std::find(option_ids.begin(), option_ids.end(),
-					get(pref_name, option["default"].str())
+					get(pref_name, option.cfg["default"].str())
 				));
 
 				// If the saved option value was invalid, reset selection to 0.
@@ -696,7 +656,7 @@ void preferences_dialog::post_build(window& window)
 					selected = 0;
 				}
 
-				menu_button* setter_widget = build_single_widget_instance<menu_button>("menu_button");
+				menu_button* setter_widget = build_single_widget_instance<menu_button>();
 				setter_widget->set_id("setter");
 
 				details_grid.swap_child("setter", setter_widget, true);
@@ -709,7 +669,7 @@ void preferences_dialog::post_build(window& window)
 				// We need to bind a lambda here since preferences::set is overloaded.
 				// A lambda alone would be more verbose because it'd need to specify all the parameters.
 				connect_signal_notify_modified(menu,
-					std::bind([=](widget& w) { set(pref_name, option_ids[dynamic_cast<menu_button&>(w).get_value()]); }, _1));
+					std::bind([=](widget& w) { set(pref_name, option_ids[dynamic_cast<menu_button&>(w).get_value()]); }, std::placeholders::_1));
 
 				gui2::bind_status_label<menu_button>(main_grid, "setter", [](menu_button& m)->std::string {
 					return m.get_value_string();
@@ -718,10 +678,10 @@ void preferences_dialog::post_build(window& window)
 				break;
 			}
 
-			case ADVANCED_PREF_TYPE::SPECIAL: {
+			case avp::avd_type::SPECIAL: {
 				//main_grid->remove_child("setter");
 
-				image* value_widget = build_single_widget_instance<image>("image");
+				image* value_widget = build_single_widget_instance<image>();
 				value_widget->set_label("icons/arrows/arrows_blank_right_25.png~CROP(3,3,18,18)");
 
 				main_grid->swap_child("value", value_widget, true);
@@ -752,12 +712,12 @@ void preferences_dialog::post_build(window& window)
 	hotkey_menu.set_values(hotkey_category_entries);
 
 	connect_signal_notify_modified(hotkey_menu,
-		std::bind(&preferences_dialog::hotkey_filter_callback, this, std::ref(window)));
+		std::bind(&preferences_dialog::hotkey_filter_callback, this));
 
-	listbox& hotkey_list = setup_hotkey_list(window);
+	listbox& hotkey_list = setup_hotkey_list();
 
 	text_box& filter = find_widget<text_box>(&window, "filter", false);
-	filter.set_text_changed_callback(std::bind(&preferences_dialog::on_filtertext_changed, this, _1));
+	filter.set_text_changed_callback(std::bind(&preferences_dialog::hotkey_filter_callback, this));
 
 	// Action column
 	hotkey_list.register_translatable_sorting_option(0, [this](const int i) { return visible_hotkeys_[i]->description.str(); });
@@ -770,7 +730,7 @@ void preferences_dialog::post_build(window& window)
 	hotkey_list.register_sorting_option(3, [this](const int i) { return !visible_hotkeys_[i]->scope[hotkey::SCOPE_EDITOR]; });
 	hotkey_list.register_sorting_option(4, [this](const int i) { return !visible_hotkeys_[i]->scope[hotkey::SCOPE_MAIN_MENU]; });
 
-	hotkey_list.set_active_sorting_option({0, listbox::SORT_ASCENDING}, true);
+	hotkey_list.set_active_sorting_option({0, preferences::SORT_ORDER::ASCENDING}, true);
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&window, "btn_add_hotkey", false), std::bind(
@@ -787,11 +747,10 @@ void preferences_dialog::post_build(window& window)
 	connect_signal_mouse_left_click(
 		find_widget<button>(&window, "btn_reset_hotkeys", false), std::bind(
 			&preferences_dialog::default_hotkey_callback,
-			this,
-			std::ref(window)));
+			this));
 }
 
-listbox& preferences_dialog::setup_hotkey_list(window& window)
+listbox& preferences_dialog::setup_hotkey_list()
 {
 	const std::string& default_icon = "misc/empty.png~CROP(0,0,15,15)";
 
@@ -805,10 +764,10 @@ listbox& preferences_dialog::setup_hotkey_list(window& window)
 	t_string& row_is_g_markup = row_data["lbl_is_game"]["use_markup"];
 	t_string& row_is_e        = row_data["lbl_is_editor"]["label"];
 	t_string& row_is_e_markup = row_data["lbl_is_editor"]["use_markup"];
-	t_string& row_is_t        = row_data["lbl_is_titlescreen"]["label"];
-	t_string& row_is_t_markup = row_data["lbl_is_titlescreen"]["use_markup"];
+	t_string& row_is_m        = row_data["lbl_is_mainmenu"]["label"];
+	t_string& row_is_m_markup = row_data["lbl_is_mainmenu"]["use_markup"];
 
-	listbox& hotkey_list = find_widget<listbox>(&window, "list_hotkeys", false);
+	listbox& hotkey_list = find_widget<listbox>(get_window(), "list_hotkeys", false);
 
 	hotkey_list.clear();
 	visible_hotkeys_.clear();
@@ -816,7 +775,7 @@ listbox& preferences_dialog::setup_hotkey_list(window& window)
 	// These translated initials should match those used in data/gui/window/preferences/02_hotkeys.cfg
 	std::string text_game_feature_on = "<span color='#0f0'>" + _("game_hotkeys^G") + "</span>";
 	std::string text_editor_feature_on = "<span color='#0f0'>" + _("editor_hotkeys^E") + "</span>";
-	std::string text_title_feature_on = "<span color='#0f0'>" + _("titlescreen_hotkeys^T") + "</span>";
+	std::string text_mainmenu_feature_on = "<span color='#0f0'>" + _("mainmenu_hotkeys^M") + "</span>";
 
 	for(const auto& hotkey_item : hotkey::get_hotkey_commands()) {
 		if(hotkey_item.hidden) {
@@ -837,8 +796,8 @@ listbox& preferences_dialog::setup_hotkey_list(window& window)
 		row_is_g_markup = "true";
 		row_is_e = hotkey_item.scope[hotkey::SCOPE_EDITOR]    ? text_editor_feature_on : "";
 		row_is_e_markup = "true";
-		row_is_t = hotkey_item.scope[hotkey::SCOPE_MAIN_MENU] ? text_title_feature_on : "";
-		row_is_t_markup = "true";
+		row_is_m = hotkey_item.scope[hotkey::SCOPE_MAIN_MENU] ? text_mainmenu_feature_on : "";
+		row_is_m_markup = "true";
 
 		hotkey_list.add_row(row_data);
 	}
@@ -896,13 +855,13 @@ void preferences_dialog::add_hotkey_callback(listbox& hotkeys)
 	hotkey::add_hotkey(newhk);
 
 	// We need to recalculate all hotkey names in because we might have removed a hotkey from another command.
-	for(size_t i = 0; i < hotkeys.get_item_count(); ++i) {
+	for(std::size_t i = 0; i < hotkeys.get_item_count(); ++i) {
 		const hotkey::hotkey_command& hotkey_item_row = *visible_hotkeys_[i];
 		find_widget<label>(hotkeys.get_row_grid(i), "lbl_hotkey", false).set_label(hotkey::get_names(hotkey_item_row.command));
 	}
 }
 
-void preferences_dialog::default_hotkey_callback(window& window)
+void preferences_dialog::default_hotkey_callback()
 {
 	gui2::show_transient_message(_("Hotkeys Reset"), _("All hotkeys have been reset to their default values."),
 			std::string(), false, false, true);
@@ -910,10 +869,10 @@ void preferences_dialog::default_hotkey_callback(window& window)
 	clear_hotkeys();
 
 	// Set up the list again and reselect the default sorting option.
-	listbox& hotkey_list = setup_hotkey_list(window);
-	hotkey_list.set_active_sorting_option({0, listbox::SORT_ASCENDING}, true);
+	listbox& hotkey_list = setup_hotkey_list();
+	hotkey_list.set_active_sorting_option({0, preferences::SORT_ORDER::ASCENDING}, true);
 
-	find_widget<multimenu_button>(&window, "hotkey_category_menu", false).reset_toggle_states();
+	find_widget<multimenu_button>(get_window(), "hotkey_category_menu", false).reset_toggle_states();
 }
 
 void preferences_dialog::remove_hotkey_callback(listbox& hotkeys)
@@ -929,10 +888,10 @@ void preferences_dialog::remove_hotkey_callback(listbox& hotkeys)
 	find_widget<label>(hotkeys.get_row_grid(row_number), "lbl_hotkey", false).set_label(hotkey::get_names(hotkey_item.command));
 }
 
-void preferences_dialog::hotkey_filter_callback(window& window) const
+void preferences_dialog::hotkey_filter_callback() const
 {
-	const multimenu_button& hotkey_menu = find_widget<const multimenu_button>(&window, "hotkey_category_menu", false);
-	const text_box& name_filter = find_widget<const text_box>(&window, "filter", false);
+	const multimenu_button& hotkey_menu = find_widget<const multimenu_button>(get_window(), "hotkey_category_menu", false);
+	const text_box& name_filter = find_widget<const text_box>(get_window(), "filter", false);
 
 	boost::dynamic_bitset<> toggle_states = hotkey_menu.get_toggle_states();
 	boost::dynamic_bitset<> res(visible_hotkeys_.size());
@@ -977,35 +936,29 @@ void preferences_dialog::hotkey_filter_callback(window& window) const
 		}
 	}
 
-	find_widget<listbox>(&window, "list_hotkeys", false).set_row_shown(res);
+	find_widget<listbox>(get_window(), "list_hotkeys", false).set_row_shown(res);
 }
 
 void preferences_dialog::on_advanced_prefs_list_select(listbox& list)
 {
 	const int selected_row = list.get_selected_row();
+	const auto& pref = adv_preferences_[selected_row];
 
-	const ADVANCED_PREF_TYPE& selected_type = ADVANCED_PREF_TYPE::string_to_enum(
-		adv_preferences_cfg_[selected_row]["type"].str());
-
-	const std::string& selected_field = adv_preferences_cfg_[selected_row]["field"].str();
-
-	if(selected_type == ADVANCED_PREF_TYPE::SPECIAL) {
-		if(selected_field == "advanced_graphic_options") {
-			gui2::dialogs::advanced_graphics_options::display();
-		} else if(selected_field == "logging") {
+	if(pref.type == avp::avd_type::SPECIAL) {
+		if(pref.field == "logging") {
 			gui2::dialogs::log_settings::display();
-		} else if(selected_field == "orb_color") {
+		} else if(pref.field == "orb_color") {
 			gui2::dialogs::select_orb_colors::display();
 		} else {
-			WRN_GUI_L << "Invalid or unimplemented custom advanced prefs option: " << selected_field << "\n";
+			WRN_GUI_L << "Invalid or unimplemented custom advanced prefs option: " << pref.field << "\n";
 		}
 
 		// Add more options here as needed
 	}
 
-	const bool has_description = !adv_preferences_cfg_[selected_row]["description"].empty();
+	const bool has_description = !pref.description.empty();
 
-	if(has_description || (selected_type != ADVANCED_PREF_TYPE::SPECIAL && selected_type != ADVANCED_PREF_TYPE::TOGGLE)) {
+	if(has_description || (pref.type != avp::avd_type::SPECIAL && pref.type != avp::avd_type::TOGGLE)) {
 		find_widget<widget>(get_advanced_row_grid(list, selected_row), "prefs_setter_grid", false)
 			.set_visible(widget::visibility::visible);
 	}
@@ -1018,13 +971,13 @@ void preferences_dialog::on_advanced_prefs_list_select(listbox& list)
 	}
 }
 
-void preferences_dialog::initialize_tabs(window& window, listbox& selector)
+void preferences_dialog::initialize_tabs(listbox& selector)
 {
 	//
 	// MULTIPLAYER TABS
 	//
 	connect_signal_notify_modified(selector,
-		std::bind(&preferences_dialog::on_tab_select, this, std::ref(window)));
+		std::bind(&preferences_dialog::on_tab_select, this));
 }
 
 void preferences_dialog::pre_show(window& window)
@@ -1039,23 +992,20 @@ void preferences_dialog::pre_show(window& window)
 	// is not the case for those in Advanced
 	//
 
-	gui2::bind_status_label<slider>(&window, "max_saves_slider", [](slider& s)->std::string {
-		return s.get_value() == INFINITE_AUTO_SAVES ? _("∞") : s.get_value_label().str();
-	});
-
+	gui2::bind_status_label<slider>(&window, "max_saves_slider");
 	gui2::bind_status_label<slider>(&window, "turbo_slider");
-#if 0
-	gui2::bind_status_label<slider>(&window, "scaling_slider",   [](slider& s)->std::string {
-		return s.get_value_label() + "%";
-	});
-#endif
+
+	//gui2::bind_status_label<slider>(&window, "scaling_slider",   [](slider& s)->std::string {
+	//	return s.get_value_label() + "%";
+	//});
+
 	listbox& selector = find_widget<listbox>(&window, "selector", false);
 	stacked_widget& pager = find_widget<stacked_widget>(&window, "pager", false);
 
 	pager.set_find_in_all_layers(true);
 
 	connect_signal_notify_modified(selector,
-		std::bind(&preferences_dialog::on_page_select, this, std::ref(window)));
+		std::bind(&preferences_dialog::on_page_select, this));
 
 	window.keyboard_capture(&selector);
 
@@ -1080,7 +1030,7 @@ void preferences_dialog::pre_show(window& window)
 			const int to_select = (ii == main_index ? tab_index : 0);
 
 			// Initialize tabs for this page
-			initialize_tabs(window, *tab_selector);
+			initialize_tabs(*tab_selector);
 
 			tab_selector->select_row(to_select);
 			tab_pager->select_layer(to_select);
@@ -1092,44 +1042,44 @@ void preferences_dialog::pre_show(window& window)
 	pager.select_layer(main_index);
 }
 
-void preferences_dialog::set_visible_page(window& window, unsigned int page, const std::string& pager_id)
+void preferences_dialog::set_visible_page(unsigned int page, const std::string& pager_id)
 {
-	find_widget<stacked_widget>(&window, pager_id, false).select_layer(page);
+	find_widget<stacked_widget>(get_window(), pager_id, false).select_layer(page);
 }
 
 // Special fullsceen callback
-void preferences_dialog::fullscreen_toggle_callback(window& window)
+void preferences_dialog::fullscreen_toggle_callback()
 {
-	const bool ison = find_widget<toggle_button>(&window, "fullscreen", false).get_value_bool();
-	window.video().set_fullscreen(ison);
+	const bool ison = find_widget<toggle_button>(get_window(), "fullscreen", false).get_value_bool();
+	CVideo::get_singleton().set_fullscreen(ison);
 
-	menu_button& res_list = find_widget<menu_button>(&window, "resolution_set", false);
+	menu_button& res_list = find_widget<menu_button>(get_window(), "resolution_set", false);
 
-	set_resolution_list(res_list, window.video());
+	set_resolution_list(res_list);
 	res_list.set_active(!ison);
 }
 
-void preferences_dialog::handle_res_select(window& window)
+void preferences_dialog::handle_res_select()
 {
-	menu_button& res_list = find_widget<menu_button>(&window, "resolution_set", false);
+	menu_button& res_list = find_widget<menu_button>(get_window(), "resolution_set", false);
 
-	if(window.video().set_resolution(resolutions_[res_list.get_value()])) {
-		set_resolution_list(res_list, window.video());
+	if(CVideo::get_singleton().set_resolution(resolutions_[res_list.get_value()])) {
+		set_resolution_list(res_list);
 	}
 }
 
-void preferences_dialog::on_page_select(window& window)
+void preferences_dialog::on_page_select()
 {
 	const int selected_row =
-		std::max(0, find_widget<listbox>(&window, "selector", false).get_selected_row());
-	set_visible_page(window, static_cast<unsigned int>(selected_row), "pager");
+		std::max(0, find_widget<listbox>(get_window(), "selector", false).get_selected_row());
+	set_visible_page(static_cast<unsigned int>(selected_row), "pager");
 }
 
-void preferences_dialog::on_tab_select(window& window)
+void preferences_dialog::on_tab_select()
 {
 	const int selected_row =
-		std::max(0, find_widget<listbox>(&window, "tab_selector", false).get_selected_row());
-	set_visible_page(window, static_cast<unsigned int>(selected_row), "tab_pager");
+		std::max(0, find_widget<listbox>(get_window(), "tab_selector", false).get_selected_row());
+	set_visible_page(static_cast<unsigned int>(selected_row), "tab_pager");
 }
 
 void preferences_dialog::post_show(window& /*window*/)
@@ -1142,4 +1092,3 @@ void preferences_dialog::post_show(window& /*window*/)
 }
 
 } // namespace dialogs
-} // namespace gui2
