@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -23,41 +24,8 @@
 #include "gui/widgets/settings.hpp"
 #include "gui/widgets/window.hpp"
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_simple_item_selector
- *
- * == Simple item selector ==
- *
- * A simple one-column listbox with OK and Cancel buttons.
- *
- * @begin{table}{dialog_widgets}
- *
- * title & & label & m &
- *         Dialog title label. $
- *
- * message & & styled_widget & m &
- *         Text label displaying a description or instructions. $
- *
- * listbox & & listbox & m &
- *         Listbox displaying user choices. $
- *
- * -item & & styled_widget & m &
- *         Widget which shows a listbox item label. $
- *
- * ok & & button & m &
- *         OK button. $
- *
- * cancel & & button & m &
- *         Cancel button. $
- *
- * @end{table}
- */
 
 REGISTER_DIALOG(simple_item_selector)
 
@@ -122,4 +90,3 @@ void simple_item_selector::post_show(window& window)
 }
 
 } // namespace dialogs
-} // namespace gui2
