@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -18,11 +19,22 @@
 
 #include <vector>
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
+/**
+ * @ingroup GUIWindowDefinitionWML
+ *
+ * A simple one-column listbox with OK and Cancel buttons.
+ * Key               |Type          |Mandatory|Description
+ * ------------------|--------------|---------|-----------
+ * title             | @ref label   |yes      |Dialog title label.
+ * message           | control      |yes      |Text label displaying a description or instructions.
+ * listbox           | @ref listbox |yes      |Listbox displaying user choices.
+ * item              | control      |yes      |Widget which shows a listbox item label.
+ * ok                | @ref button  |yes      |OK button.
+ * cancel            | @ref button  |yes      |Cancel button.
+ */
 class simple_item_selector : public modal_dialog
 {
 public:
@@ -87,14 +99,10 @@ private:
 
 	std::string ok_label_, cancel_label_;
 
-	/** Inherited from modal_dialog, implemented by REGISTER_DIALOG. */
 	virtual const std::string& window_id() const override;
 
-	/** Inherited from modal_dialog. */
 	virtual void pre_show(window& window) override;
 
-	/** Inherited from modal_dialog. */
 	virtual void post_show(window& window) override;
 };
 } // namespace dialogs
-} // namespace gui2
