@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2017-2018 by Charles Dang <exodia339@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2017 - 2021
+	by Charles Dang <exodia339@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -20,11 +21,7 @@
 #include "storyscreen/controller.hpp"
 #include "storyscreen/part.hpp"
 
-class CVideo;
-
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 /** Dialog to view the storyscreen. */
@@ -46,37 +43,35 @@ public:
 	}
 
 private:
-	/** Inherited from modal_dialog, implemented by REGISTER_DIALOG. */
 	virtual const std::string& window_id() const override;
 
-	/** Inherited from modal_dialog. */
 	virtual void pre_show(window& window) override;
 
 	void clear_image_timer();
 
 	void update_current_part_ptr();
 
-	void display_part(window& window);
+	void display_part();
 
 	using floating_image_list = std::vector<storyscreen::floating_image>;
-	void draw_floating_image(window& window, floating_image_list::const_iterator image_iter, int this_part_index);
+	void draw_floating_image(floating_image_list::const_iterator image_iter, int this_part_index);
 
 	enum NAV_DIRECTION {
 		DIR_FORWARD,
 		DIR_BACKWARDS
 	};
 
-	void nav_button_callback(window& window, NAV_DIRECTION direction);
+	void nav_button_callback( NAV_DIRECTION direction);
 
-	void key_press_callback(window& window, const SDL_Keycode key);
+	void key_press_callback(const SDL_Keycode key);
 
 	void set_next_draw();
 	void begin_fade_draw(bool fade_in);
 	void halt_fade_draw();
 
-	void draw_callback(window& window);
+	void draw_callback();
 
-	void flag_stack_as_dirty(window& window);
+	void flag_stack_as_dirty();
 
 	storyscreen::controller controller_;
 
@@ -84,8 +79,8 @@ private:
 
 	storyscreen::controller::part_pointer_type current_part_;
 
-	size_t timer_id_;
-	size_t next_draw_;
+	std::size_t timer_id_;
+	std::size_t next_draw_;
 
 	int fade_step_;
 
@@ -99,4 +94,3 @@ private:
 };
 
 } // namespace dialogs
-} // namespace gui2
