@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2016 - 2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2016 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -24,9 +25,7 @@
 #include "gui/widgets/window.hpp"
 #include "picture.hpp"
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 REGISTER_DIALOG(terrain_layers)
@@ -63,7 +62,7 @@ void terrain_layers::pre_show(window& window)
 	listbox& list = find_widget<listbox>(&window, "layer_list", false);
 
 	int order = 1;
-	for(const terrain_builder::tile::log_details det : tile_logs_) {
+	for(const terrain_builder::tile::log_details& det : tile_logs_) {
 		const terrain_builder::tile::rule_image_rand& ri   = *det.first;
 		const terrain_builder::rule_image_variant& variant = *det.second;
 
@@ -151,4 +150,3 @@ void terrain_layers::pre_show(window& window)
 }
 
 } // namespace dialogs
-} // namespace gui2
