@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "gui/dialogs/theme_list.hpp"
@@ -20,32 +21,8 @@
 #include "gui/widgets/window.hpp"
 #include "theme.hpp"
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_theme_list
- *
- * == Theme list ==
- *
- * Dialog for selecting a GUI theme.
- *
- * @begin{table}{dialog_widgets}
- *
- * themes & & listbox & m &
- *         Listbox displaying user choices. $
- *
- * -name & & styled_widget & m &
- *         Widget which shows a theme item name. $
- *
- * -description & & styled_widget & m &
- *         Widget which shows a theme item description. $
- *
- * @end{table}
- */
 
 REGISTER_DIALOG(theme_list)
 
@@ -94,4 +71,3 @@ void theme_list::post_show(window& window)
 	index_ = list.get_selected_row();
 }
 } // namespace dialogs
-} // namespace gui2
