@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -34,8 +35,10 @@
 #include "gui/dialogs/message.hpp"
 #include "gui/dialogs/multiplayer/mp_host_game_prompt.hpp"
 #include "gui/dialogs/multiplayer/mp_method_selection.hpp"
+#include "gui/dialogs/preferences_dialog.hpp"
 #include "gui/dialogs/screenshot_notification.hpp"
 #include "gui/dialogs/simple_item_selector.hpp"
+#include "language.hpp"
 #include "log.hpp"
 #include "preferences/game.hpp"
 //#define DEBUG_TOOLTIP
@@ -52,86 +55,19 @@
 #include "hotkey/hotkey_command.hpp"
 #include "sdl/surface.hpp"
 #include "sdl/utils.hpp"
-#include "utils/functional.hpp"
 #include "video.hpp"
 
 #include <algorithm>
+#include <functional>
+
+#include <boost/algorithm/string/erase.hpp>
 
 static lg::log_domain log_config("config");
 #define ERR_CF LOG_STREAM(err, log_config)
 #define WRN_CF LOG_STREAM(warn, log_config)
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_title_screen
- *
- * == Title screen ==
- *
- * This shows the title screen.
- *
- * @begin{table}{dialog_widgets}
- * tutorial & & button & m &
- *         The button to start the tutorial. $
- *
- * campaign & & button & m &
- *         The button to start a campaign. $
- *
- * multiplayer & & button & m &
- *         The button to start multiplayer mode. $
- *
- * load & & button & m &
- *         The button to load a saved game. $
- *
- * editor & & button & m &
- *         The button to start the editor. $
- *
- * addons & & button & m &
- *         The button to start managing the addons. $
- *
- * cores & & button & m &
- *         The button to start managing the cores. $
- *
- * language & & button & m &
- *         The button to select the game language. $
- *
- * credits & & button & m &
- *         The button to show Wesnoth's contributors. $
- *
- * quit & & button & m &
- *         The button to quit Wesnoth. $
- *
- * tips & & multi_page & m &
- *         A multi_page to hold all tips, when this widget is used the area of
- *         the tips doesn't need to be resized when the next or previous button
- *         is pressed. $
- *
- * -tip & & label & o &
- *         Shows the text of the current tip. $
- *
- * -source & & label & o &
- *         The source (the one who's quoted or the book referenced) of the
- *         current tip. $
- *
- * next_tip & & button & m &
- *         The button show the next tip of the day. $
- *
- * previous_tip & & button & m &
- *         The button show the previous tip of the day. $
- *
- * logo & & image & o &
- *         The Wesnoth logo. $
- *
- * revision_number & & styled_widget & o &
- *         A widget to show the version number when the version number is
- *         known. $
- *
- * @end{table}
- */
 
 REGISTER_DIALOG(title_screen)
 
@@ -159,7 +95,11 @@ static void register_button(window& win, const std::string& id, hotkey::HOTKEY_C
 		win.register_hotkey(hk, std::bind(callback));
 	}
 
-	connect_signal_mouse_left_click(find_widget<button>(&win, id, false), std::bind(callback));
+	auto b = find_widget<button>(&win, id, false, false);
+	if(b != nullptr)
+	{
+		connect_signal_mouse_left_click(*b, std::bind(callback));
+	}
 }
 
 static void launch_lua_console()
@@ -167,12 +107,12 @@ static void launch_lua_console()
 	gui2::dialogs::lua_interpreter::display(gui2::dialogs::lua_interpreter::APP);
 }
 
-static void make_screenshot(window& win)
+static void make_screenshot()
 {
-	surface screenshot = win.video().getSurface().clone();
+	surface screenshot = CVideo::get_singleton().getSurface().clone();
 	if(screenshot) {
 		std::string filename = filesystem::get_screenshot_dir() + "/" + _("Screenshot") + "_";
-		filename = filesystem::get_next_filename(filename, ".png");
+		filename = filesystem::get_next_filename(filename, ".jpg");
 		gui2::dialogs::screenshot_notification::display(filename, screenshot);
 	}
 }
@@ -213,11 +153,11 @@ void title_screen::pre_show(window& win)
 
 #ifdef DEBUG_TOOLTIP
 	win.connect_signal<event::SDL_MOUSE_MOTION>(
-			std::bind(debug_tooltip, std::ref(win), _3, _5),
+			std::bind(debug_tooltip, std::ref(win), std::placeholders::_3, std::placeholders::_5),
 			event::dispatcher::front_child);
 #endif
 
-	win.connect_signal<event::SDL_VIDEO_RESIZE>(std::bind(&title_screen::on_resize, this, std::ref(win)));
+	win.connect_signal<event::SDL_VIDEO_RESIZE>(std::bind(&title_screen::on_resize, this));
 
 	//
 	// General hotkeys
@@ -226,13 +166,13 @@ void title_screen::pre_show(window& win)
 		std::bind(&gui2::window::set_retval, std::ref(win), RELOAD_GAME_DATA, true));
 
 	win.register_hotkey(hotkey::TITLE_SCREEN__TEST,
-		std::bind(&title_screen::hotkey_callback_select_tests, this, std::ref(win)));
+		std::bind(&title_screen::hotkey_callback_select_tests, this));
 
 	// A wrapper is needed here since the relevant display function is overloaded, and
 	// since the wrapper's signature doesn't exactly match what register_hotkey expects.
 	win.register_hotkey(hotkey::LUA_CONSOLE, std::bind(&launch_lua_console));
 
-	win.register_hotkey(hotkey::HOTKEY_SCREENSHOT, std::bind(&make_screenshot, std::ref(win)));
+	win.register_hotkey(hotkey::HOTKEY_SCREENSHOT, std::bind(&make_screenshot));
 
 	//
 	// Background and logo images
@@ -266,35 +206,36 @@ void title_screen::pre_show(window& win)
 	//
 	// Tip-of-the-day browser
 	//
-	multi_page& tip_pages = find_widget<multi_page>(&win, "tips", false);
+	multi_page* tip_pages = find_widget<multi_page>(&win, "tips", false, false);
 
-	std::vector<game_tip> tips = tip_of_the_day::shuffle(settings::tips);
-	if(tips.empty()) {
-		WRN_CF << "There are no tips of day available." << std::endl;
+	if(tip_pages != nullptr) {
+		std::vector<game_tip> tips = tip_of_the_day::shuffle(settings::tips);
+		if(tips.empty()) {
+			WRN_CF << "There are no tips of day available." << std::endl;
+		}
+		for(const auto& tip : tips)	{
+			string_map widget;
+			std::map<std::string, string_map> page;
+
+			widget["use_markup"] = "true";
+
+			widget["label"] = tip.text();
+			page.emplace("tip", widget);
+
+			widget["label"] = tip.source();
+			page.emplace("source", widget);
+
+			tip_pages->add_page(page);
+		}
+
+		update_tip(true);
 	}
 
-	for(const auto& tip : tips)	{
-		string_map widget;
-		std::map<std::string, string_map> page;
-
-		widget["use_markup"] = "true";
-
-		widget["label"] = tip.text();
-		page.emplace("tip", widget);
-
-		widget["label"] = tip.source();
-		page.emplace("source", widget);
-
-		tip_pages.add_page(page);
-	}
-
-	update_tip(win, true);
-
 	register_button(win, "next_tip", hotkey::TITLE_SCREEN__NEXT_TIP,
-		std::bind(&title_screen::update_tip, this, std::ref(win), true));
+		std::bind(&title_screen::update_tip, this, true));
 
 	register_button(win, "previous_tip", hotkey::TITLE_SCREEN__PREVIOUS_TIP,
-		std::bind(&title_screen::update_tip, this, std::ref(win), false));
+		std::bind(&title_screen::update_tip, this, false));
 
 	//
 	// Help
@@ -304,7 +245,6 @@ void title_screen::pre_show(window& win)
 			gui2::dialogs::help_browser::display();
 		}
 
-		help::help_manager help_manager(&game_config_manager::get()->game_config());
 		help::show_help();
 	});
 
@@ -313,14 +253,6 @@ void title_screen::pre_show(window& win)
 	//
 	register_button(win, "about", hotkey::HOTKEY_NULL, std::bind(&game_version::display<>));
 
-	//
-	// Tutorial
-	//
-	register_button(win, "tutorial", hotkey::TITLE_SCREEN__TUTORIAL, [this, &win]() {
-		game_.set_tutorial();
-		win.set_retval(LAUNCH_GAME);
-	});
-
 	//
 	// Campaign
 	//
@@ -338,7 +270,7 @@ void title_screen::pre_show(window& win)
 	// Multiplayer
 	//
 	register_button(win, "multiplayer", hotkey::TITLE_SCREEN__MULTIPLAYER,
-		std::bind(&title_screen::button_callback_multiplayer, this, std::ref(win)));
+		std::bind(&title_screen::button_callback_multiplayer, this));
 
 	//
 	// Load game
@@ -346,20 +278,15 @@ void title_screen::pre_show(window& win)
 	register_button(win, "load", hotkey::HOTKEY_LOAD_GAME, [this, &win]() {
 		if(game_.load_game()) {
 			win.set_retval(LAUNCH_GAME);
-		} else {
-			game_.clear_loaded_game();
 		}
 	});
 
 	//
 	// Addons
 	//
-	register_button(win, "addons", hotkey::TITLE_SCREEN__ADDONS, []() {
-		// NOTE: we need the help_manager to get access to the Add-ons section in the game help!
-		help::help_manager help_manager(&game_config_manager::get()->game_config());
-
+	register_button(win, "addons", hotkey::TITLE_SCREEN__ADDONS, [&win]() {
 		if(manage_addons()) {
-			game_config_manager::get()->reload_changed_game_config();
+			win.set_retval(RELOAD_GAME_DATA);
 		}
 	});
 
@@ -371,32 +298,48 @@ void title_screen::pre_show(window& win)
 	//
 	// Cores
 	//
-	register_button(win, "cores", hotkey::TITLE_SCREEN__CORES,
+	win.register_hotkey(hotkey::TITLE_SCREEN__CORES,
 		std::bind(&title_screen::button_callback_cores, this));
 
-	if(game_config_manager::get()->game_config().child_range("core").size() <= 1) {
-		find_widget<button>(&win, "cores", false).set_visible(widget::visibility::invisible);
-	}
-
 	//
 	// Language
 	//
 	register_button(win, "language", hotkey::HOTKEY_LANGUAGE, [this, &win]() {
 		try {
 			if(game_.change_language()) {
-				t_string::reset_translations();
-				::image::flush_cache();
-				on_resize(win);
+				on_resize();
 			}
 		} catch(const std::runtime_error& e) {
 			gui2::show_error_message(e.what());
 		}
 	});
 
+	if(auto* lang_button = find_widget<button>(&win, "language", false, false); lang_button) {
+		const auto& locale = translation::get_effective_locale_info();
+		const auto& langs = get_languages(true);
+
+		auto lang_def = std::find_if(langs.begin(), langs.end(), [&](language_def const& lang) {
+			// Just assume everything is UTF-8 (it should be as long as we're called Wesnoth)
+			// and strip the charset from the Boost locale identifier.
+			const auto& boost_name = boost::algorithm::erase_first_copy(locale.name(), ".UTF-8");
+			// std::cerr << lang.localename << '/' << boost_name << '\n';
+			return lang.localename == boost_name;
+		});
+
+		// If the locale doesn't match a known translation, we might be using the System Default
+		// Language option (issue #6205 <https://github.com/wesnoth/wesnoth/issues/6205>) or
+		// something weirder is up. Use the WML-defined default label instead in that case.
+		if(lang_def != langs.end()) {
+			lang_button->set_label(lang_def->language.str());
+		}
+	}
+
 	//
 	// Preferences
 	//
-	register_button(win, "preferences", hotkey::HOTKEY_PREFERENCES, [this]() { game_.show_preferences(); });
+	register_button(win, "preferences", hotkey::HOTKEY_PREFERENCES, []() {
+		gui2::dialogs::preferences_dialog::display();
+	});
 
 	//
 	// Credits
@@ -407,6 +350,8 @@ void title_screen::pre_show(window& win)
 	// Quit
 	//
 	register_button(win, "quit", hotkey::HOTKEY_QUIT_TO_DESKTOP, [&win]() { win.set_retval(QUIT_GAME); });
+	// A sanity check, exit immediately if the .cfg file didn't have a "quit" button.
+	find_widget<button>(&win, "quit", false, true);
 
 	//
 	// Debug clock
@@ -414,37 +359,41 @@ void title_screen::pre_show(window& win)
 	register_button(win, "clock", hotkey::HOTKEY_NULL,
 		std::bind(&title_screen::show_debug_clock_window, this));
 
-	find_widget<button>(&win, "clock", false).set_visible(show_debug_clock_button
-		? widget::visibility::visible
-		: widget::visibility::invisible);
+	auto clock = find_widget<button>(&win, "clock", false, false);
+	if(clock) {
+		clock->set_visible(show_debug_clock_button ? widget::visibility::visible : widget::visibility::invisible);
+	}
 }
 
-void title_screen::on_resize(window& win)
+void title_screen::on_resize()
 {
-	win.set_retval(REDRAW_BACKGROUND);
+	set_retval(REDRAW_BACKGROUND);
 }
 
-void title_screen::update_tip(window& win, const bool previous)
+void title_screen::update_tip(const bool previous)
 {
-	multi_page& tips = find_widget<multi_page>(&win, "tips", false);
-	if(tips.get_page_count() == 0) {
+	multi_page* tip_pages = find_widget<multi_page>(get_window(), "tips", false, false);
+	if(tip_pages == nullptr) {
+		return;
+	}
+	if(tip_pages->get_page_count() == 0) {
 		return;
 	}
 
-	int page = tips.get_selected_page();
+	int page = tip_pages->get_selected_page();
 	if(previous) {
 		if(page <= 0) {
-			page = tips.get_page_count();
+			page = tip_pages->get_page_count();
 		}
 		--page;
 	} else {
 		++page;
-		if(static_cast<unsigned>(page) >= tips.get_page_count()) {
+		if(static_cast<unsigned>(page) >= tip_pages->get_page_count()) {
 			page = 0;
 		}
 	}
 
-	tips.select_page(page);
+	tip_pages->select_page(page);
 
 	/**
 	 * @todo Look for a proper fix.
@@ -453,7 +402,7 @@ void title_screen::update_tip(window& win, const bool previous)
 	 * Not entirely sure why, but since we plan to move to SDL2 that change
 	 * will probably fix this issue automatically.
 	 */
-	win.set_is_dirty(true);
+	get_window()->set_is_dirty(true);
 }
 
 void title_screen::show_debug_clock_window()
@@ -468,7 +417,7 @@ void title_screen::show_debug_clock_window()
 	}
 }
 
-void title_screen::hotkey_callback_select_tests(window& window)
+void title_screen::hotkey_callback_select_tests()
 {
 	game_config_manager::get()->load_game_config_for_create(false, true);
 
@@ -487,11 +436,11 @@ void title_screen::hotkey_callback_select_tests(window& window)
 	int choice = dlg.selected_index();
 	if(choice >= 0) {
 		game_.set_test(options[choice]);
-		window.set_retval(LAUNCH_GAME);
+		set_retval(LAUNCH_GAME);
 	}
 }
 
-void title_screen::button_callback_multiplayer(window& window)
+void title_screen::button_callback_multiplayer()
 {
 	while(true) {
 		gui2::dialogs::mp_method_selection dlg;
@@ -501,29 +450,29 @@ void title_screen::button_callback_multiplayer(window& window)
 			return;
 		}
 
-		const int res = dlg.get_choice();
+		const auto res = dlg.get_choice();
 
-		if(res == 2 && preferences::mp_server_warning_disabled() < 2) {
+		if(res == decltype(dlg)::choice::HOST && preferences::mp_server_warning_disabled() < 2) {
 			if(!gui2::dialogs::mp_host_game_prompt::execute()) {
 				continue;
 			}
 		}
 
 		switch(res) {
-		case 0:
-			game_.select_mp_server(preferences::server_list().front().address);
-			window.set_retval(MP_CONNECT);
+		case decltype(dlg)::choice::JOIN:
+			game_.select_mp_server(preferences::builtin_servers_list().front().address);
+			get_window()->set_retval(MP_CONNECT);
 			break;
-		case 1:
+		case decltype(dlg)::choice::CONNECT:
 			game_.select_mp_server("");
-			window.set_retval(MP_CONNECT);
+			get_window()->set_retval(MP_CONNECT);
 			break;
-		case 2:
+		case decltype(dlg)::choice::HOST:
 			game_.select_mp_server("localhost");
-			window.set_retval(MP_HOST);
+			get_window()->set_retval(MP_HOST);
 			break;
-		case 3:
-			window.set_retval(MP_LOCAL);
+		case decltype(dlg)::choice::LOCAL:
+			get_window()->set_retval(MP_LOCAL);
 			break;
 		}
 
@@ -549,9 +498,8 @@ void title_screen::button_callback_cores()
 		const std::string& core_id = cores[core_dlg.get_choice()]["id"];
 
 		preferences::set_core_id(core_id);
-		game_config_manager::get()->reload_changed_game_config();
+		get_window()->set_retval(RELOAD_GAME_DATA);
 	}
 }
 
 } // namespace dialogs
-} // namespace gui2
