@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -22,11 +23,7 @@
 #include "gui/widgets/button.hpp"
 #include "gui/widgets/label.hpp"
 #include "gui/widgets/image.hpp"
-#ifdef GUI2_EXPERIMENTAL_LISTBOX
-#include "gui/widgets/list.hpp"
-#else
 #include "gui/widgets/listbox.hpp"
-#endif
 #include "gui/widgets/settings.hpp"
 #include "gui/widgets/unit_preview_pane.hpp"
 #include "gui/widgets/window.hpp"
@@ -38,64 +35,36 @@
 #include "color.hpp"
 #include "units/unit.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
-namespace gui2
+namespace gui2::dialogs
 {
-namespace dialogs
-{
-
-/*WIKI
- * @page = GUIWindowDefinitionWML
- * @order = 2_unit_attack
- *
- * == Unit attack ==
- *
- * This shows the dialog for attacking units.
- *
- * @begin{table}{dialog_widgets}
- *                                     $
- * attacker_icon     & & image   & o & Shows the icon of the attacking unit. $
- * attacker_name     & & styled_widget & o & Shows the name of the attacking unit. $
- *
- * defender_portrait & & image   & o & Shows the portrait of the defending unit.
- *                                     $
- * defender_icon     & & image   & o & Shows the icon of the defending unit. $
- * defender_name     & & styled_widget & o & Shows the name of the defending unit. $
- *
- *
- * weapon_list       & & listbox & m & The list with weapons to choose from. $
- * -attacker_weapon  & & styled_widget & o & The weapon for the attacker to use. $
- * -defender_weapon  & & styled_widget & o & The weapon for the defender to use. $
- *
- * @end{table}
- */
 
 REGISTER_DIALOG(unit_attack)
 
 unit_attack::unit_attack(const unit_map::iterator& attacker_itor,
 						   const unit_map::iterator& defender_itor,
-						   const std::vector<battle_context>& weapons,
+						   std::vector<battle_context>&& weapons,
 						   const int best_weapon)
 	: selected_weapon_(-1)
 	, attacker_itor_(attacker_itor)
 	, defender_itor_(defender_itor)
-	, weapons_(weapons)
+	, weapons_(std::move(weapons))
 	, best_weapon_(best_weapon)
 {
 }
 
-void unit_attack::damage_calc_callback(window& window)
+void unit_attack::damage_calc_callback()
 {
-	const size_t index = find_widget<listbox>(&window, "weapon_list", false).get_selected_row();
-	attack_predictions::display(weapons_[index], *attacker_itor_, *defender_itor_);
+	const std::size_t index = find_widget<listbox>(get_window(), "weapon_list", false).get_selected_row();
+	attack_predictions::display(weapons_[index], attacker_itor_.get_shared_ptr(), defender_itor_.get_shared_ptr());
 }
 
 void unit_attack::pre_show(window& window)
 {
 	connect_signal_mouse_left_click(
 			find_widget<button>(&window, "damage_calculation", false),
-			std::bind(&unit_attack::damage_calc_callback, this, std::ref(window)));
+			std::bind(&unit_attack::damage_calc_callback, this));
 
 	find_widget<unit_preview_pane>(&window, "attacker_pane", false)
 		.set_displayed_unit(*attacker_itor_);
@@ -147,7 +116,8 @@ void unit_attack::pre_show(window& window)
 		);
 
 		std::string attw_specials = attacker_weapon.weapon_specials(true, attacker.backstab_pos);
-		std::string defw_specials = defender_weapon.weapon_specials(true);
+		bool defender_attack = !(defender_weapon.name().empty() && defender_weapon.damage() == 0 && defender_weapon.num_attacks() == 0 && defender.chance_to_hit == 0);
+		std::string defw_specials = defender_attack ? defender_weapon.weapon_specials(true) : "";
 
 		if(!attw_specials.empty()) {
 			attw_specials = " " + attw_specials;
@@ -208,4 +178,3 @@ void unit_attack::post_show(window& window)
 }
 
 } // namespace dialogs
-} // namespace gui2
