@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2016 - 2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2016 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -33,31 +34,31 @@
 #include "game_board.hpp"
 #include "gettext.hpp"
 #include "replay_helper.hpp"
+#include "play_controller.hpp"
 #include "resources.hpp"
 #include "synced_context.hpp"
 #include "team.hpp"
 #include "units/types.hpp"
 #include "units/unit.hpp"
 #include "units/ptr.hpp"
-#include "utils/functional.hpp"
+#include <functional>
+#include "whiteboard/manager.hpp"
 
 #include <boost/dynamic_bitset.hpp>
 
 static lg::log_domain log_display("display");
 #define LOG_DP LOG_STREAM(info, log_display)
 
-namespace gui2
-{
-namespace dialogs
+namespace gui2::dialogs
 {
 
 // Index 2 is by-level
-static listbox::order_pair sort_last    {-1, listbox::SORT_NONE};
-static listbox::order_pair sort_default { 2, listbox::SORT_DESCENDING};
+static listbox::order_pair sort_last    {-1, preferences::SORT_ORDER::NONE};
+static listbox::order_pair sort_default { 2, preferences::SORT_ORDER::DESCENDING};
 
 REGISTER_DIALOG(unit_recall)
 
-unit_recall::unit_recall(recalls_ptr_vector& recall_list, team& team)
+unit_recall::unit_recall(std::vector<unit_const_ptr>& recall_list, team& team)
 	: recall_list_(recall_list)
 	, team_(team)
 	, selected_index_()
@@ -73,18 +74,35 @@ static void dump_recall_list_to_console(const T& units)
 
 	LOG_DP << "size: " << units.size() << "\n";
 
-	size_t idx = 0;
-	for(const unit_const_ptr& u_ptr : units) {
+	std::size_t idx = 0;
+	for(const auto& u_ptr : units) {
 		LOG_DP << "\tunit[" << (idx++) << "]: " << u_ptr->id() << " name = '" << u_ptr->name() << "'\n";
 	}
 }
 
-static std::string format_level_string(const int level)
+static const color_t inactive_row_color(0x96, 0x96, 0x96);
+
+static const inline std::string maybe_inactive(const std::string& str, bool active)
+{
+	if(active)
+		return str;
+	else
+		return font::span_color(inactive_row_color, str);
+}
+
+static std::string format_level_string(const int level, bool recallable)
 {
 	std::string lvl = std::to_string(level);
 
-	if(level < 1) {
-		return "<span color='#969696'>" + lvl + "</span>";
+	if(!recallable) {
+		// Same logic as when recallable, but always in inactive_row_color.
+		if(level < 2) {
+			return font::span_color(inactive_row_color, lvl);
+		} else {
+			return font::span_color(inactive_row_color, "<b>" + lvl + "</b>");
+		}
+	} else if(level < 1) {
+		return font::span_color(inactive_row_color, lvl);
 	} else if(level == 1) {
 		return lvl;
 	} else if(level == 2) {
@@ -148,11 +166,11 @@ void unit_recall::pre_show(window& window)
 			= find_widget<text_box>(&window, "filter_box", false, true);
 
 	filter->set_text_changed_callback(
-			std::bind(&unit_recall::filter_text_changed, this, _1, _2));
+			std::bind(&unit_recall::filter_text_changed, this, std::placeholders::_2));
 
 	listbox& list = find_widget<listbox>(&window, "recall_list", false);
 
-	connect_signal_notify_modified(list, std::bind(&unit_recall::list_item_clicked, this, std::ref(window)));
+	connect_signal_notify_modified(list, std::bind(&unit_recall::list_item_clicked, this));
 
 	list.clear();
 
@@ -161,11 +179,11 @@ void unit_recall::pre_show(window& window)
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&window, "rename", false),
-		std::bind(&unit_recall::rename_unit, this, std::ref(window)));
+		std::bind(&unit_recall::rename_unit, this));
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&window, "dismiss", false),
-		std::bind(&unit_recall::dismiss_unit, this, std::ref(window)));
+		std::bind(&unit_recall::dismiss_unit, this));
 
 	connect_signal_mouse_left_click(
 		find_widget<button>(&window, "show_help", false),
@@ -177,6 +195,19 @@ void unit_recall::pre_show(window& window)
 
 		std::string mods = unit->image_mods();
 
+		int wb_gold = 0;
+		if(resources::controller) {
+			if(const std::shared_ptr<wb::manager>& whiteb = resources::controller->get_whiteboard()) {
+				wb::future_map future; // So gold takes into account planned spending
+				wb_gold = whiteb->get_spent_gold_for(team_.side());
+			}
+		}
+
+		// Note: Our callers apply [filter_recall], but leave it to us
+		// to apply cost-based filtering.
+		const int recall_cost = (unit->recall_cost() > -1 ? unit->recall_cost() : team_.recall_cost());
+		const bool recallable = (recall_cost <= team_.gold() - wb_gold);
+
 		if(unit->can_recruit()) {
 			mods += "~BLIT(" + unit::leader_crown() + ")";
 		}
@@ -185,58 +216,86 @@ void unit_recall::pre_show(window& window)
 			mods += "~BLIT(" + overlay + ")";
 		}
 
+		if(!recallable) {
+			mods += "~GS()";
+
+			// Just set the tooltip on every single element in this row.
+			if(wb_gold > 0)
+				column["tooltip"] = _("This unit cannot be recalled because you will not have enough gold at this point in your plan.");
+			else
+				column["tooltip"] = _("This unit cannot be recalled because you do not have enough gold.");
+		}
+
 		column["use_markup"] = "true";
 
 		column["label"] = unit->absolute_image() + mods;
 		row_data.emplace("unit_image", column);
 
-		column["label"] = unit->type_name();
+		column["label"] = maybe_inactive(unit->type_name(), recallable);
 		row_data.emplace("unit_type", column);
 
-		column["label"] = format_cost_string(unit->recall_cost(), team_.recall_cost());
+		// gold_icon is handled below
+
+		column["label"] =
+			recallable
+			? format_cost_string(unit->recall_cost(), team_.recall_cost())
+			: maybe_inactive(std::to_string(recall_cost), recallable);
 		row_data.emplace("unit_recall_cost", column);
 
 		const std::string& name = !unit->name().empty() ? unit->name().str() : font::unicode_en_dash;
-		column["label"] = name;
+		column["label"] = maybe_inactive(name, recallable);
 		row_data.emplace("unit_name", column);
 
-		column["label"] = format_level_string(unit->level());
+		column["label"] = format_level_string(unit->level(), recallable);
 		row_data.emplace("unit_level", column);
 
 		std::stringstream exp_str;
-		exp_str << font::span_color(unit->xp_color());
 		if(unit->can_advance()) {
 			exp_str << unit->experience() << "/" << unit->max_experience();
 		} else {
 			exp_str << font::unicode_en_dash;
 		}
-		exp_str << "</span>";
 
-		column["label"] = exp_str.str();
+		column["label"] = font::span_color(recallable ? unit->xp_color() : inactive_row_color, exp_str.str());
 		row_data.emplace("unit_experience", column);
 
 		// Since the table widgets use heavy formatting, we save a bare copy
 		// of certain options to filter on.
 		std::string filter_text = unit->type_name() + " " + name + " " + std::to_string(unit->level());
 
+		if(recallable) {
+			// This is to allow filtering for recallable units by typing "vvv" in the search box.
+			// That's intended to be easy to type and unlikely to match unit or type names.
+			//
+			// TODO: document this. (Also, implement a "Hide non-recallable units" checkbox.)
+			filter_text += " " + std::string("vvv");
+		}
+
 		std::string traits;
 		for(const std::string& trait : unit->trait_names()) {
 			traits += (traits.empty() ? "" : "\n") + trait;
 			filter_text += " " + trait;
 		}
 
-		column["label"] = !traits.empty() ? traits : font::unicode_en_dash;
+		column["label"] = maybe_inactive(
+					!traits.empty() ? traits : font::unicode_en_dash,
+					recallable);
 		row_data.emplace("unit_traits", column);
 
-		list.add_row(row_data);
 		filter_options_.push_back(filter_text);
+		grid& grid = list.add_row(row_data);
+		if(!recallable) {
+			image *gold_icon = dynamic_cast<image*>(grid.find("gold_icon", false));
+			assert(gold_icon);
+			gold_icon->set_image(gold_icon->get_image() + "~GS()");
+		}
 	}
 
 	list.register_translatable_sorting_option(0, [this](const int i) { return recall_list_[i]->type_name().str(); });
 	list.register_translatable_sorting_option(1, [this](const int i) { return recall_list_[i]->name().str(); });
 	list.register_sorting_option(2, [this](const int i) {
 		const unit& u = *recall_list_[i];
-		return std::make_tuple(u.level(), -static_cast<int>(u.experience_to_advance()));
+		return std::tuple(u.level(), -static_cast<int>(u.experience_to_advance()));
 	});
 	list.register_sorting_option(3, [this](const int i) { return recall_list_[i]->experience(); });
 	list.register_translatable_sorting_option(4, [this](const int i) {
@@ -245,14 +304,18 @@ void unit_recall::pre_show(window& window)
 
 	list.set_active_sorting_option(sort_last.first >= 0 ? sort_last	: sort_default, true);
 
-	list_item_clicked(window);
+	list_item_clicked();
 }
 
-void unit_recall::rename_unit(window& window)
+void unit_recall::rename_unit()
 {
-	listbox& list = find_widget<listbox>(&window, "recall_list", false);
+	listbox& list = find_widget<listbox>(get_window(), "recall_list", false);
 
 	const int index = list.get_selected_row();
+	if (index == -1) {
+		return;
+	}
+
 	unit& selected_unit = const_cast<unit&>(*recall_list_[index].get());
 
 	std::string name = selected_unit.name();
@@ -272,17 +335,20 @@ void unit_recall::rename_unit(window& window)
 		}
 		filter_options_.insert(filter_options_.begin() + index, filter_text.str());
 
-		list_item_clicked(window);
-		window.invalidate_layout();
+		list_item_clicked();
+		get_window()->invalidate_layout();
 	}
 }
 
-void unit_recall::dismiss_unit(window& window)
+void unit_recall::dismiss_unit()
 {
 	LOG_DP << "Recall list units:\n"; dump_recall_list_to_console(recall_list_);
 
-	listbox& list = find_widget<listbox>(&window, "recall_list", false);
+	listbox& list = find_widget<listbox>(get_window(), "recall_list", false);
 	const int index = list.get_selected_row();
+	if (index == -1) {
+		return;
+	}
 
 	const unit& u = *recall_list_[index].get();
 
@@ -316,7 +382,7 @@ void unit_recall::dismiss_unit(window& window)
 
 	// Remove the entry from the dialog list
 	list.remove_row(index);
-	list_item_clicked(window);
+	list_item_clicked();
 
 	// Remove the entry from the filter list
 	filter_options_.erase(filter_options_.begin() + index);
@@ -335,7 +401,7 @@ void unit_recall::dismiss_unit(window& window)
 
 	// Close the dialog if all units are dismissed
 	if(list.get_item_count() == 0) {
-		window.set_retval(retval::CANCEL);
+		set_retval(retval::CANCEL);
 	}
 }
 
@@ -344,10 +410,10 @@ void unit_recall::show_help()
 	help::show_help("recruit_and_recall");
 }
 
-void unit_recall::list_item_clicked(window& window)
+void unit_recall::list_item_clicked()
 {
 	const int selected_row
-		= find_widget<listbox>(&window, "recall_list", false).get_selected_row();
+		= find_widget<listbox>(get_window(), "recall_list", false).get_selected_row();
 
 	if(selected_row == -1) {
 		return;
@@ -355,10 +421,10 @@ void unit_recall::list_item_clicked(window& window)
 
 	const unit& selected_unit = *recall_list_[selected_row].get();
 
-	find_widget<unit_preview_pane>(&window, "unit_details", false)
+	find_widget<unit_preview_pane>(get_window(), "unit_details", false)
 		.set_displayed_unit(selected_unit);
 
-	find_widget<button>(&window, "rename", false).set_active(!selected_unit.unrenamable());
+	find_widget<button>(get_window(), "rename", false).set_active(!selected_unit.unrenamable());
 }
 
 void unit_recall::post_show(window& window)
@@ -371,11 +437,9 @@ void unit_recall::post_show(window& window)
 	}
 }
 
-void unit_recall::filter_text_changed(text_box_base* textbox, const std::string& text)
+void unit_recall::filter_text_changed(const std::string& text)
 {
-	window& window = *textbox->get_window();
-
-	listbox& list = find_widget<listbox>(&window, "recall_list", false);
+	listbox& list = find_widget<listbox>(get_window(), "recall_list", false);
 
 	const std::vector<std::string> words = utils::split(text, ' ');
 
@@ -404,7 +468,11 @@ void unit_recall::filter_text_changed(text_box_base* textbox, const std::string&
 	}
 
 	list.set_row_shown(show_items);
+
+	// Disable rename and dismiss buttons if no units are shown
+	const bool any_shown = list.any_rows_shown();
+	find_widget<button>(get_window(), "rename", false).set_active(any_shown);
+	find_widget<button>(get_window(), "dismiss", false).set_active(any_shown);
 }
 
 } // namespace dialogs
-} // namespace gui2
