@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2016 - 2018 The Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2016 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -31,6 +32,7 @@
 #include "font/pango/escape.hpp"
 #include "formatter.hpp"
 #include "formula/string_utils.hpp"
+#include "game_initialization/multiplayer.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
 #include "preferences/credentials.hpp"
@@ -61,7 +63,7 @@ chatbox::chatbox(const implementation::builder_chatbox& builder)
 	, chat_input_(nullptr)
 	, active_window_(0)
 	, active_window_changed_callback_()
-	, lobby_info_(nullptr)
+	, chat_info_()
 	, wesnothd_connection_(nullptr)
 	, log_(nullptr)
 {
@@ -72,7 +74,7 @@ chatbox::chatbox(const implementation::builder_chatbox& builder)
 	// loss itself when applicable. Nothing else happens in the interim while
 	// keyboard_focus_ equals `this` to warrent cleanup.
 	connect_signal<event::RECEIVE_KEYBOARD_FOCUS>(
-		std::bind(&chatbox::signal_handler_receive_keyboard_focus, this, _2));
+		std::bind(&chatbox::signal_handler_receive_keyboard_focus, this, std::placeholders::_2));
 }
 
 void chatbox::finalize_setup()
@@ -89,7 +91,7 @@ void chatbox::finalize_setup()
 	chat_input_ = find_widget<text_box>(this, "chat_input", false, true);
 
 	connect_signal_pre_key_press(*chat_input_,
-		std::bind(&chatbox::chat_input_keypress_callback, this, _5));
+		std::bind(&chatbox::chat_input_keypress_callback, this, std::placeholders::_5));
 }
 
 void chatbox::load_log(std::map<std::string, chatroom_log>& log, bool show_lobby)
@@ -127,7 +129,7 @@ void chatbox::switch_to_window(lobby_chat_window* t)
 	switch_to_window(t - &open_windows_[0]);
 }
 
-void chatbox::switch_to_window(size_t id)
+void chatbox::switch_to_window(std::size_t id)
 {
 	active_window_ = id;
 	assert(active_window_ < open_windows_.size());
@@ -148,6 +150,8 @@ void chatbox::chat_input_keypress_callback(const SDL_Keycode key)
 		return;
 	}
 
+	lobby_chat_window& t = open_windows_[active_window_];
+
 	switch(key) {
 	case SDLK_RETURN:
 	case SDLK_KP_ENTER: {
@@ -157,8 +161,6 @@ void chatbox::chat_input_keypress_callback(const SDL_Keycode key)
 			//       the other party without having to specify it's nick.
 			chat_handler::do_speak(input);
 		} else {
-			lobby_chat_window& t = open_windows_[active_window_];
-
 			if(t.whisper) {
 				send_whisper(t.name, input);
 				add_whisper_sent(t.name, input);
@@ -175,9 +177,14 @@ void chatbox::chat_input_keypress_callback(const SDL_Keycode key)
 	}
 
 	case SDLK_TAB: {
+		auto* li = mp::get_lobby_info();
+		if(!li) {
+			break;
+		}
+
 		// TODO: very inefficient! Very! D:
 		std::vector<std::string> matches;
-		for(const auto& ui : lobby_info_->users()) {
+		for(const auto& ui : li->users()) {
 			if(ui.name != preferences::login()) {
 				matches.push_back(ui.name);
 			}
@@ -211,7 +218,7 @@ void chatbox::append_to_chatbox(const std::string& text, const bool force_scroll
 	append_to_chatbox(text, active_window_, force_scroll);
 }
 
-void chatbox::append_to_chatbox(const std::string& text, size_t id, const bool force_scroll)
+void chatbox::append_to_chatbox(const std::string& text, std::size_t id, const bool force_scroll)
 {
 	grid& grid = chat_log_container_->page_grid(id);
 
@@ -220,7 +227,7 @@ void chatbox::append_to_chatbox(const std::string& text, size_t id, const bool f
 	const unsigned chatbox_position = log.get_vertical_scrollbar_item_position();
 
 	const std::string new_text = formatter()
-		<< log.get_label() << "\n" << "<span color='#bcb088'>" << preferences::get_chat_timestamp(time(0)) << text << "</span>";
+		<< log.get_label() << "\n" << "<span color='#bcb088'>" << preferences::get_chat_timestamp(std::time(0)) << text << "</span>";
 
 	log.set_use_markup(true);
 	log.set_label(new_text);
@@ -242,7 +249,7 @@ void chatbox::append_to_chatbox(const std::string& text, size_t id, const bool f
 
 void chatbox::send_chat_message(const std::string& message, bool /*allies_only*/)
 {
-	add_chat_message(time(nullptr), preferences::login(), 0, message);
+	add_chat_message(std::time(nullptr), preferences::login(), 0, message);
 
 	::config c {"message", ::config {"message", message, "sender", preferences::login()}};
 	send_to_server(c);
@@ -255,7 +262,7 @@ void chatbox::user_relation_changed(const std::string& /*name*/)
 	}
 }
 
-void chatbox::add_chat_message(const time_t& /*time*/,
+void chatbox::add_chat_message(const std::time_t& /*time*/,
 	const std::string& speaker,
 	int /*side*/,
 	const std::string& message,
@@ -286,7 +293,7 @@ void chatbox::add_whisper_sent(const std::string& receiver, const std::string& m
 		add_active_window_whisper(VGETTEXT("whisper to $receiver", {{"receiver", receiver}}), message, true);
 	}
 
-	lobby_info_->get_whisper_log(receiver).add_message(preferences::login(), message);
+	chat_info_.get_whisper_log(receiver).add_message(preferences::login(), message);
 }
 
 void chatbox::add_whisper_received(const std::string& sender, const std::string& message)
@@ -294,7 +301,7 @@ void chatbox::add_whisper_received(const std::string& sender, const std::string&
 	bool can_go_to_active = !preferences::whisper_friends_only() || preferences::is_friend(sender);
 	bool can_open_new = preferences::auto_open_whisper_windows() && can_go_to_active;
 
-	lobby_info_->get_whisper_log(sender).add_message(sender, message);
+	chat_info_.get_whisper_log(sender).add_message(sender, message);
 
 	if(whisper_window_open(sender, can_open_new)) {
 		if(whisper_window_active(sender)) {
@@ -324,7 +331,7 @@ void chatbox::add_chat_room_message_sent(const std::string& room, const std::str
 	}
 
 	// Do not open room window here. The player should be in the room before sending messages
-	mp::room_info* ri = lobby_info_->get_room(room);
+	mp::room_info* ri = chat_info_.get_room(room);
 	assert(ri);
 
 	if(!room_window_active(room)) {
@@ -339,7 +346,7 @@ void chatbox::add_chat_room_message_received(const std::string& room,
 	const std::string& speaker,
 	const std::string& message)
 {
-	mp::room_info* ri = lobby_info_->get_room(room);
+	mp::room_info* ri = chat_info_.get_room(room);
 	if(!ri) {
 		LOG_LB << "Discarding message to room " << room << " from " << speaker << " (room not open)\n";
 		return;
@@ -383,14 +390,14 @@ bool chatbox::room_window_active(const std::string& room)
 lobby_chat_window* chatbox::room_window_open(const std::string& room, const bool open_new, const bool allow_close)
 {
 	return find_or_create_window(room, false, open_new, allow_close,
-		VGETTEXT("Room <i>“$name”</i> joined", { { "name", translation::dsgettext("wesnoth-lib", room.c_str()) } }));
+		VGETTEXT("Joined <i>$name</i>", { { "name", translation::dsgettext("wesnoth-lib", room.c_str()) } }));
 }
 
 lobby_chat_window* chatbox::whisper_window_open(const std::string& name, bool open_new)
 {
 	return find_or_create_window(name, true, open_new, true,
-		VGETTEXT("Whisper session with <i>“$name”</i> started. "
-		"If you do not want to receive messages from this user, type <i>/ignore $name</i>", { { "name", name } }));
+		VGETTEXT("Started private message with <i>$name</i>. "
+		"If you do not want to receive messages from this player, type <i>/ignore $name</i>", { { "name", name } }));
 }
 
 lobby_chat_window* chatbox::find_or_create_window(const std::string& name,
@@ -420,7 +427,7 @@ lobby_chat_window* chatbox::find_or_create_window(const std::string& name,
 	std::map<std::string, string_map> data{{"log_text", item}};
 
 	if(!whisper) {
-		lobby_info_->open_room(name);
+		chat_info_.open_room(name);
 	}
 
 	if(log_ != nullptr) {
@@ -454,7 +461,7 @@ lobby_chat_window* chatbox::find_or_create_window(const std::string& name,
 		close_button.set_visible(widget::visibility::hidden);
 	} else {
 		connect_signal_mouse_left_click(close_button,
-			std::bind(&chatbox::close_window_button_callback, this, open_windows_.back().name, _3, _4));
+			std::bind(&chatbox::close_window_button_callback, this, open_windows_.back().name, std::placeholders::_3, std::placeholders::_4));
 	}
 
 	return &open_windows_.back();
@@ -528,7 +535,7 @@ void chatbox::add_active_window_whisper(const std::string& sender,
 	append_to_chatbox(text, force_scroll);
 }
 
-void chatbox::close_window(size_t idx)
+void chatbox::close_window(std::size_t idx)
 {
 	const lobby_chat_window& t = open_windows_[idx];
 
@@ -539,16 +546,6 @@ void chatbox::close_window(size_t idx)
 		return;
 	}
 
-	if(t.whisper == false) {
-		// closing a room window -- send a part to the server
-		::config data, msg;
-		msg["room"] = t.name;
-		msg["player"] = preferences::login();
-		data.add_child("room_part", std::move(msg));
-
-		send_to_server(data);
-	}
-
 	// Check if we're closing the currently-active window.
 	const bool active_changed = idx == active_window_;
 
@@ -557,9 +554,9 @@ void chatbox::close_window(size_t idx)
 	}
 
 	if(t.whisper) {
-		lobby_info_->get_whisper_log(t.name).clear();
+		chat_info_.get_whisper_log(t.name).clear();
 	} else {
-		lobby_info_->close_room(t.name);
+		chat_info_.close_room(t.name);
 	}
 
 	if(log_ != nullptr) {
@@ -608,120 +605,7 @@ mp::room_info* chatbox::active_window_room()
 		return nullptr;
 	}
 
-	return lobby_info_->get_room(t.name);
-}
-
-void chatbox::process_room_join(const ::config& data)
-{
-	const std::string& room = data["room"];
-	const std::string& player = data["player"];
-
-	DBG_LB << "room join: " << room << " " << player << "\n";
-
-	mp::room_info* r = lobby_info_->get_room(room);
-	if(r) {
-		if(player == preferences::login()) {
-			if(const auto& members = data.child("members")) {
-				r->process_room_members(members);
-			}
-		} else {
-			r->add_member(player);
-
-			/* TODO: add/use preference */
-			add_room_window_message(room, "server", VGETTEXT("$player has entered the room", {{"player", player}}));
-		}
-
-		if(r == active_window_room()) {
-			active_window_changed_callback_();
-		}
-	} else {
-		if(player == preferences::login()) {
-			lobby_chat_window* t = room_window_open(room, true);
-
-			lobby_info_->open_room(room);
-			r = lobby_info_->get_room(room);
-			assert(r);
-
-			if(const auto& members = data.child("members")) {
-				r->process_room_members(members);
-			}
-
-			switch_to_window(t);
-
-			const std::string& topic = data["topic"];
-			if(!topic.empty()) {
-				add_chat_room_message_received("room", "server", room + ": " + topic);
-			}
-		} else {
-			LOG_LB << "Discarding join info for a room the player is not in\n";
-		}
-	}
-}
-
-void chatbox::process_room_part(const ::config& data)
-{
-	// TODO: close room window when the part message is sent
-	const std::string& room = data["room"];
-	const std::string& player = data["player"];
-
-	DBG_LB << "Room part: " << room << " " << player << "\n";
-
-	if(mp::room_info* r = lobby_info_->get_room(room)) {
-		r->remove_member(player);
-
-		/* TODO: add/use preference */
-		add_room_window_message(room, "server", VGETTEXT("$player has left the room", {{"player", player}}));
-		if(active_window_room() == r) {
-			active_window_changed_callback_();
-		}
-	} else {
-		LOG_LB << "Discarding part info for a room the player is not in\n";
-	}
-}
-
-void chatbox::process_room_query_response(const ::config& data)
-{
-	const std::string& room = data["room"];
-	const std::string& message = data["message"];
-
-	DBG_LB << "room query response: " << room << " " << message << "\n";
-
-	if(room.empty()) {
-		if(!message.empty()) {
-			add_active_window_message("server", message);
-		}
-
-		if(const ::config& rooms = data.child("rooms")) {
-			// TODO: this should really open a nice join room dialog instead
-			std::stringstream ss;
-			ss << "Rooms:";
-
-			for(const auto & r : rooms.child_range("room")) {
-				ss << " " << r["name"];
-			}
-
-			add_active_window_message("server", ss.str());
-		}
-	} else {
-		if(room_window_open(room, false)) {
-			if(!message.empty()) {
-				add_chat_room_message_received(room, "server", message);
-			}
-
-			if(const ::config& members = data.child("members")) {
-				mp::room_info* r = lobby_info_->get_room(room);
-				assert(r);
-				r->process_room_members(members);
-				if(r == active_window_room()) {
-					active_window_changed_callback_();
-				}
-			}
-		} else {
-			if(!message.empty()) {
-				add_active_window_message("server", room + ": " + message);
-			}
-		}
-	}
+	return chat_info_.get_room(t.name);
 }
 
 void chatbox::process_message(const ::config& data, bool whisper /*= false*/)
@@ -735,11 +619,13 @@ void chatbox::process_message(const ::config& data, bool whisper /*= false*/)
 	}
 
 	const std::string& message = data["message"];
-	preferences::parse_admin_authentication(sender, message);
+	//preferences::parse_admin_authentication(sender, message); TODO: replace
 
 	if(whisper) {
 		add_whisper_received(sender, message);
 	} else {
+		if (!preferences::parse_should_show_lobby_join(sender, message)) return;
+
 		std::string room = data["room"];
 
 		// Attempt to send to the currently active room first.
@@ -754,6 +640,15 @@ void chatbox::process_message(const ::config& data, bool whisper /*= false*/)
 			room = "lobby";
 		}
 
+		if(log_ != nullptr && data["type"].str() == "motd") {
+			if(log_->at("lobby").received_motd == message) {
+				LOG_LB << "Ignoring repeated motd\n";
+				return;
+			} else {
+				log_->at("lobby").received_motd = message;
+			}
+		}
+
 		add_chat_room_message_received(room, sender, message);
 	}
 
@@ -769,12 +664,6 @@ void chatbox::process_network_data(const ::config& data)
 		process_message(message);
 	} else if(const ::config& whisper = data.child("whisper")) {
 		process_message(whisper, true);
-	} else if(const ::config& room_join = data.child("room_join")) {
-		process_room_join(room_join);
-	} else if(const ::config& room_part = data.child("room_part")) {
-		process_room_part(room_part);
-	} else if(const ::config& room_query_response = data.child("room_query_response")) {
-		process_room_query_response(room_query_response);
 	}
 }
 
@@ -825,7 +714,7 @@ widget* builder_chatbox::build() const
 	const auto conf = widget->cast_config_to<chatbox_definition>();
 	assert(conf);
 
-	widget->init_grid(conf->grid);
+	widget->init_grid(*conf->grid);
 	widget->finalize_setup();
 
 	return widget;
