@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2016 - 2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2016 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -84,11 +85,6 @@ public:
 		active_window_changed_callback_ = f;
 	}
 
-	void set_lobby_info(mp::lobby_info& i)
-	{
-		lobby_info_ = &i;
-	}
-
 	void set_wesnothd_connection(wesnothd_connection& c)
 	{
 		wesnothd_connection_ = &c;
@@ -107,7 +103,7 @@ protected:
 	virtual void user_relation_changed(const std::string& name) override;
 
 	/** Inherited form @ref chat_handler */
-	virtual void add_chat_message(const time_t& time,
+	virtual void add_chat_message(const std::time_t& time,
 		const std::string& speaker,
 		int side,
 		const std::string& message,
@@ -140,11 +136,11 @@ private:
 
 	std::vector<lobby_chat_window> open_windows_;
 
-	size_t active_window_;
+	std::size_t active_window_;
 
 	std::function<void(void)> active_window_changed_callback_;
 
-	mp::lobby_info* lobby_info_;
+	mp::chat_info chat_info_;
 
 	wesnothd_connection* wesnothd_connection_;
 
@@ -168,7 +164,7 @@ private:
 
 	void append_to_chatbox(const std::string& text, const bool force_scroll = false);
 
-	void append_to_chatbox(const std::string& text, size_t id, const bool force_scroll = false);
+	void append_to_chatbox(const std::string& text, std::size_t id, const bool force_scroll = false);
 
 	/** @returns true if the whisper window for "name" is the active window. */
 	bool whisper_window_active(const std::string& name);
@@ -203,7 +199,7 @@ private:
 		const std::string& message,
 		const bool force_scroll = false);
 
-	void close_window(size_t idx);
+	void close_window(std::size_t idx);
 
 public:
 	/** Inherited form @ref chat_handler */
@@ -215,7 +211,7 @@ public:
 	 */
 	void switch_to_window(lobby_chat_window* t);
 
-	void switch_to_window(size_t id);
+	void switch_to_window(std::size_t id);
 
 	void active_window_changed();
 
@@ -248,12 +244,6 @@ public:
 
 	void close_window_button_callback(std::string room_name, bool& handled, bool& halt);
 
-	void process_room_join(const ::config& data);
-
-	void process_room_part(const ::config& data);
-
-	void process_room_query_response(const ::config& data);
-
 	void process_message(const ::config& data, bool whisper = false);
 
 	void process_network_data(const ::config& data);
@@ -288,7 +278,7 @@ public:
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 private:
 };
