@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -25,9 +26,9 @@
 #include "gui/widgets/scrollbar_container.hpp"
 #include "gui/widgets/window.hpp"
 #include "serialization/string_utils.hpp"
-#include "utils/io.hpp"
 
 #include <fstream>
+#include <iomanip>
 #include <iostream>
 
 namespace gui2
@@ -81,8 +82,8 @@ std::string get_base_filename()
 {
 	std::ostringstream ss;
 
-	time_t t = time(nullptr);
-	ss << utils::put_time(std::localtime(&t), "%Y%m%d_%H%M%S");
+	std::time_t t = std::time(nullptr);
+	ss << std::put_time(std::localtime(&t), "%Y%m%d_%H%M%S");
 
 	static unsigned counter = 0;
 	++counter;
@@ -246,7 +247,7 @@ void debug_layout_graph::widget_generate_info(std::ostream& out,
 		const generator_base* generator = dynamic_cast<const generator_base*>(widget);
 
 		if(generator) {
-			for(size_t i = 0; i < generator->get_item_count(); ++i) {
+			for(std::size_t i = 0; i < generator->get_item_count(); ++i) {
 
 				const std::string child_id = id + "_I_"
 											 + std::to_string(i);
