@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -30,10 +31,23 @@ namespace implementation
 // ------------ WIDGET -----------{
 
 /**
- * A widget to draw upon.
+ * @ingroup GUIWidgetWML
  *
- * This widget has a fixed size like the spacer, but allows the user to
- * manual draw items. The widget is display only.
+ * A drawing is widget with a fixed size and gives access to the canvas of the widget in the window instance.
+ * It has a fixed size like the spacer, but allows the user to manually draw items.
+ * This widget is display only.
+ *
+ * The widget normally has no event interaction so only one state is defined:
+ * * state_enabled - the drawing is enabled. The state is a dummy since the things really drawn are placed in the window instance.
+ *
+ * If either the width or the height is not zero the drawing functions as a fixed size drawing.
+ * Key          |Type                                    |Default  |Description
+ * -------------|----------------------------------------|---------|-----------
+ * width        | @ref guivartype_f_unsigned "f_unsigned"|0        |The width of the drawing.
+ * height       | @ref guivartype_f_unsigned "f_unsigned"|0        |The height of the drawing.
+ * draw         | @ref guivartype_config "config"        |mandatory|The config containing the drawing.
+ *
+ * The variables available are the same as for the window resolution see @ref builder_window::window_resolution for the list of items.
  */
 class drawing : public styled_widget
 {
@@ -135,7 +149,7 @@ struct builder_drawing : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 	/** The width of the widget. */
 	typed_formula<unsigned> width;
