@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -20,6 +21,7 @@
 #include "wml_exception.hpp"
 
 #include <numeric>
+#include <cmath>
 
 namespace gui2
 {
@@ -119,7 +121,7 @@ point horizontal_list::calculate_best_size() const
 	// The best size is the sum of the widths and the greatest height.
 	point result(0, 0);
 
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(i)) {
 			continue;
 		}
@@ -148,7 +150,7 @@ void horizontal_list::place(const point& origin, const point& size)
 
 	point current_origin = origin;
 
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -175,7 +177,7 @@ void horizontal_list::place(const point& origin, const point& size)
 void horizontal_list::set_origin(const point& origin)
 {
 	point current_origin = origin;
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -195,7 +197,7 @@ void horizontal_list::set_visible_rectangle(const SDL_Rect& rectangle)
 	 * function in the generator template class and call it from the wanted
 	 * placement functions.
 	 */
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		grid& grid = item_ordered(i);
 		grid.set_visible_rectangle(rectangle);
 	}
@@ -205,7 +207,7 @@ widget* horizontal_list::find_at(const point& coordinate, const bool must_be_act
 {
 	assert(get_window());
 
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(i)) {
 			continue;
 		}
@@ -224,7 +226,7 @@ const widget* horizontal_list::find_at(const point& coordinate, const bool must_
 {
 	assert(get_window());
 
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(i)) {
 			continue;
 		}
@@ -283,7 +285,7 @@ void horizontal_list::handle_key_right_arrow(SDL_Keymod /*modifier*/, bool& hand
 	}
 
 	if(get_selected_item_count() == 0) {
-		for(size_t i = get_ordered_index(0); i < get_item_count(); i++) {
+		for(std::size_t i = get_ordered_index(0); i < get_item_count(); i++) {
 			if(get_item_shown(get_item_at_ordered(i))) {
 				// TODO: Check if active?
 				handled = true;
@@ -297,7 +299,7 @@ void horizontal_list::handle_key_right_arrow(SDL_Keymod /*modifier*/, bool& hand
 	// NOTE maybe this should only work if we can select only one item...
 	handled = true;
 
-	for(size_t i = get_ordered_index(get_selected_item()) + 1; i < get_item_count(); ++i) {
+	for(std::size_t i = get_ordered_index(get_selected_item()) + 1; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -331,7 +333,7 @@ point vertical_list::calculate_best_size() const
 {
 	// The best size is the sum of the heights and the greatest width.
 	point result(0, 0);
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(i)) {
 			continue;
 		}
@@ -359,7 +361,7 @@ void vertical_list::place(const point& origin, const point& size)
 	 */
 
 	point current_origin = origin;
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -386,7 +388,7 @@ void vertical_list::place(const point& origin, const point& size)
 void vertical_list::set_origin(const point& origin)
 {
 	point current_origin = origin;
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -406,7 +408,7 @@ void vertical_list::set_visible_rectangle(const SDL_Rect& rectangle)
 	 * function in the generator template class and call it from the wanted
 	 * placement functions.
 	 */
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		grid& grid = item(i);
 		grid.set_visible_rectangle(rectangle);
 	}
@@ -416,7 +418,7 @@ widget* vertical_list::find_at(const point& coordinate, const bool must_be_activ
 {
 	assert(get_window());
 
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(i)) {
 			continue;
 		}
@@ -434,7 +436,7 @@ const widget* vertical_list::find_at(const point& coordinate, const bool must_be
 {
 	assert(get_window());
 
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(i)) {
 			continue;
 		}
@@ -491,7 +493,7 @@ void vertical_list::handle_key_down_arrow(SDL_Keymod /*modifier*/, bool& handled
 	}
 
 	if(get_selected_item_count() == 0) {
-		for(size_t i = get_ordered_index(0); i < get_item_count(); i++) {
+		for(std::size_t i = get_ordered_index(0); i < get_item_count(); i++) {
 			if(get_item_shown(get_item_at_ordered(i))) {
 				// TODO: Check if active?
 				handled = true;
@@ -505,7 +507,7 @@ void vertical_list::handle_key_down_arrow(SDL_Keymod /*modifier*/, bool& handled
 	// NOTE maybe this should only work if we can select only one item...
 	handled = true;
 
-	for(size_t i = get_ordered_index(get_selected_item()) + 1; i < get_item_count(); ++i) {
+	for(std::size_t i = get_ordered_index(get_selected_item()) + 1; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -548,11 +550,11 @@ point table::calculate_best_size() const
 	 * - vultraz, 2017-08-25
 	 */
 
-	size_t n_items = get_item_count();
-	size_t max_cols = sqrt(n_items) + 2;
+	std::size_t n_items = get_item_count();
+	std::size_t max_cols = std::sqrt(n_items) + 2;
 
 	std::vector<point> item_sizes;
-	for(size_t i = 0; i < n_items; i++) {
+	for(std::size_t i = 0; i < n_items; i++) {
 		if(get_item_shown(i)) {
 			item_sizes.push_back(item(i).get_best_size());
 		}
@@ -572,7 +574,7 @@ point table::calculate_best_size() const
 		[](point a, point b) { return a.x < b.x; }
 	)->x / 2;
 
-	for(size_t cells_in_1st_row = 2; cells_in_1st_row <= max_cols; cells_in_1st_row++) {
+	for(std::size_t cells_in_1st_row = 2; cells_in_1st_row <= max_cols; cells_in_1st_row++) {
 		int row_min_width = std::accumulate(item_sizes.begin(), item_sizes.begin() + cells_in_1st_row, 0,
 			[](int a, point b) { return a + b.x; }
 		);
@@ -582,7 +584,7 @@ point table::calculate_best_size() const
 
 		point row_size, total_size;
 
-		for(size_t n = 0; n < item_sizes.size(); n++) {
+		for(std::size_t n = 0; n < item_sizes.size(); n++) {
 			if(row_size.x + item_sizes[n].x > row_max_width) {
 				// Start new row
 				row++;
@@ -632,7 +634,7 @@ void table::place(const point& origin, const point& size)
 	// TODO: Make sure all cells in a row are the same height
 	point current_origin = origin;
 	int row_height = 0;
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -668,8 +670,8 @@ void table::place(const point& origin, const point& size)
 void table::set_origin(const point& origin)
 {
 	point current_origin = origin;
-	size_t row_height = 0;
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	std::size_t row_height = 0;
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -698,7 +700,7 @@ void table::set_visible_rectangle(const SDL_Rect& rectangle)
 	 * function in the generator template class and call it from the wanted
 	 * placement functions.
 	 */
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		grid& grid = item(i);
 		grid.set_visible_rectangle(rectangle);
 	}
@@ -708,7 +710,7 @@ widget* table::find_at(const point& coordinate, const bool must_be_active)
 {
 	assert(get_window());
 
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(i)) {
 			continue;
 		}
@@ -726,7 +728,7 @@ const widget* table::find_at(const point& coordinate, const bool must_be_active)
 {
 	assert(get_window());
 
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(!get_item_shown(i)) {
 			continue;
 		}
@@ -785,7 +787,7 @@ void table::handle_key_down_arrow(SDL_Keymod /*modifier*/, bool& handled)
 	}
 
 	if(get_selected_item_count() == 0) {
-		for(size_t i = get_ordered_index(0); i < get_item_count(); i++) {
+		for(std::size_t i = get_ordered_index(0); i < get_item_count(); i++) {
 			if(get_item_shown(get_item_at_ordered(i))) {
 				// TODO: Check if active?
 				handled = true;
@@ -800,7 +802,7 @@ void table::handle_key_down_arrow(SDL_Keymod /*modifier*/, bool& handled)
 	// NOTE maybe this should only work if we can select only one item...
 	handled = true;
 
-	for(size_t i = get_ordered_index(get_selected_item()) + 1; i < get_item_count(); ++i) {
+	for(std::size_t i = get_ordered_index(get_selected_item()) + 1; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -859,7 +861,7 @@ void table::handle_key_right_arrow(SDL_Keymod /*modifier*/, bool& handled)
 	}
 
 	if(get_selected_item_count() == 0) {
-		for(size_t i = get_ordered_index(0); i < get_item_count(); i++) {
+		for(std::size_t i = get_ordered_index(0); i < get_item_count(); i++) {
 			if(get_item_shown(get_item_at_ordered(i))) {
 				// TODO: Check if active?
 				handled = true;
@@ -874,7 +876,7 @@ void table::handle_key_right_arrow(SDL_Keymod /*modifier*/, bool& handled)
 	// NOTE maybe this should only work if we can select only one item...
 	handled = true;
 
-	for(size_t i = get_ordered_index(get_selected_item()) + 1; i < get_item_count(); ++i) {
+	for(std::size_t i = get_ordered_index(get_selected_item()) + 1; i < get_item_count(); ++i) {
 		if(!get_item_shown(get_item_at_ordered(i))) {
 			continue;
 		}
@@ -891,7 +893,7 @@ void table::handle_key_right_arrow(SDL_Keymod /*modifier*/, bool& handled)
 
 void independent::request_reduce_width(const unsigned maximum_width)
 {
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		grid& grid = item(i);
 		grid.request_reduce_width(maximum_width);
 	}
@@ -899,7 +901,7 @@ void independent::request_reduce_width(const unsigned maximum_width)
 
 void independent::request_reduce_height(const unsigned maximum_height)
 {
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		grid& grid = item(i);
 		grid.request_reduce_height(maximum_height);
 	}
@@ -913,7 +915,7 @@ point independent::calculate_best_size() const
 	 */
 	point result(0, 0);
 
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		const grid& grid = item(i);
 
 		const point best_size = grid.get_best_size();
@@ -932,7 +934,7 @@ point independent::calculate_best_size() const
 
 void independent::place(const point& origin, const point& size)
 {
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		grid& grid = item(i);
 		grid.place(origin, size);
 	}
@@ -946,7 +948,7 @@ void independent::set_origin(const point& origin)
 	 * @todo evaluate whether setting it only for the visible item is better
 	 * and what the consequences are.
 	 */
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		grid& grid = item(i);
 		grid.set_origin(origin);
 	}
@@ -980,7 +982,7 @@ const widget* independent::find_at(const point& coordinate, const bool must_be_a
 
 widget* independent::find(const std::string& id, const bool must_be_active)
 {
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(is_selected(i)) {
 			if(widget* widget = item(i).find(id, must_be_active)) {
 				return widget;
@@ -993,7 +995,7 @@ widget* independent::find(const std::string& id, const bool must_be_active)
 
 const widget* independent::find(const std::string& id, const bool must_be_active) const
 {
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		if(is_selected(i)) {
 			if(const widget* widget = item(i).find(id, must_be_active)) {
 				return widget;
@@ -1012,7 +1014,7 @@ void independent::set_visible_rectangle(const SDL_Rect& rectangle)
 	 * @todo evaluate whether setting it only for the visible item is better
 	 * and what the consequences are.
 	 */
-	for(size_t i = 0; i < get_item_count(); ++i) {
+	for(std::size_t i = 0; i < get_item_count(); ++i) {
 		grid& grid = item(i);
 		grid.set_visible_rectangle(rectangle);
 	}
@@ -1027,7 +1029,8 @@ namespace select_action
 void selection::select(grid& grid, const bool select)
 {
 	selectable_item* selectable = dynamic_cast<selectable_item*>(grid.get_widget(0, 0));
-	assert(selectable);
+	//the check in selection::init is not strict enouth to guaranetee this.
+	VALIDATE(selectable, "Only toggle buttons and panels are allowed as the cells of a list definition.");
 
 	selectable->set_value(select);
 }
@@ -1046,7 +1049,7 @@ void selection::init(grid* g,
 			toggle_panel* panel = dynamic_cast<toggle_panel*>(widget);
 
 			if(btn) {
-				connect_signal_notify_modified(*btn, std::bind(callback, _1));
+				connect_signal_notify_modified(*btn, std::bind(callback, std::placeholders::_1));
 
 				std::map<std::string, string_map>::const_iterator itor = data.find(btn->id());
 
@@ -1057,7 +1060,7 @@ void selection::init(grid* g,
 					btn->set_members(itor->second);
 				}
 			} else if(panel) {
-				connect_signal_notify_modified(*panel, std::bind(callback, _1));
+				connect_signal_notify_modified(*panel, std::bind(callback, std::placeholders::_1));
 
 				panel->set_child_members(data);
 			} else if(child_grid) {
