@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -22,9 +23,7 @@
 
 namespace gui2
 {
-
 struct builder_grid;
-typedef std::shared_ptr<const builder_grid> builder_grid_const_ptr;
 
 class grid;
 
@@ -176,7 +175,7 @@ public:
 	 * @returns                   A reference to the newly created grid.
 	 */
 	virtual grid& create_item(const int index,
-							   builder_grid_const_ptr list_builder,
+							   const builder_grid& list_builder,
 							   const string_map& item_data,
 							   const std::function<void(widget&)>& callback)
 			= 0;
@@ -200,7 +199,7 @@ public:
 	 */
 	virtual grid&
 	create_item(const int index,
-				builder_grid_const_ptr list_builder,
+				const builder_grid& list_builder,
 				const std::map<std::string /* widget id */, string_map>& data,
 				const std::function<void(widget&)>& callback) = 0;
 
@@ -220,7 +219,7 @@ public:
 	 *                            in the grid is (de)selected.
 	 */
 	virtual void create_items(const int index,
-							  builder_grid_const_ptr list_builder,
+							  const builder_grid& list_builder,
 							  const std::vector<string_map>& data,
 							  const std::function<void(widget&)>& callback)
 			= 0;
@@ -242,7 +241,7 @@ public:
 	 */
 	virtual void create_items(
 			const int index,
-			builder_grid_const_ptr list_builder,
+			const builder_grid& list_builder,
 			const std::vector<std::map<std::string /*widget id*/, string_map>>&
 					data,
 			const std::function<void(widget&)>& callback) = 0;
