@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -139,6 +140,7 @@ struct one_item : public virtual generator_base
 	 * @pre                       @p select == @c true
 	 *
 	 * @param index               The item to select.
+	 * @param select              Should always be true.
 	 */
 	void select_item(const unsigned index, const bool select) override
 	{
@@ -671,7 +673,7 @@ public:
 		{
 			return last_selected_item_;
 		} else {
-			for(size_t i = 0; i < items_.size(); ++i) {
+			for(std::size_t i = 0; i < items_.size(); ++i) {
 				if((*items_[i]).selected) {
 					return i;
 				}
@@ -714,7 +716,7 @@ public:
 
 	/** Inherited from generator_base. */
 	grid& create_item(const int index,
-			builder_grid_const_ptr list_builder,
+			const builder_grid& list_builder,
 			const string_map& item_data,
 			const std::function<void(widget&)>& callback) override
 	{
@@ -726,15 +728,14 @@ public:
 
 	/** Inherited from generator_base. */
 	grid& create_item(const int index,
-			builder_grid_const_ptr list_builder,
+			const builder_grid& list_builder,
 			const std::map<std::string /* widget id */, string_map>& item_data,
 			const std::function<void(widget&)>& callback) override
 	{
-		assert(list_builder);
 		assert(index == -1 || static_cast<unsigned>(index) <= items_.size());
 
 		child* item = new child;
-		list_builder->build(&item->child_grid);
+		list_builder.build(&item->child_grid);
 
 		init(&item->child_grid, item_data, callback);
 
@@ -757,7 +758,7 @@ public:
 
 	/** Inherited from generator_base. */
 	virtual void create_items(const int index,
-			builder_grid_const_ptr list_builder,
+			const builder_grid& list_builder,
 			const std::vector<std::map<std::string /*widget id*/, string_map>>& data,
 			const std::function<void(widget&)>& callback) override
 	{
@@ -766,7 +767,7 @@ public:
 
 	/** Inherited from generator_base. */
 	virtual void create_items(const int index,
-			builder_grid_const_ptr list_builder,
+			const builder_grid& list_builder,
 			const std::vector<string_map>& data,
 			const std::function<void(widget&)>& callback) override
 	{
@@ -958,7 +959,7 @@ private:
 		 */
 		bool shown;
 
-		size_t ordered_index;
+		std::size_t ordered_index;
 	};
 
 	/** The number of selected items. */
@@ -972,7 +973,7 @@ private:
 	child_list items_;
 
 	/** the elements of order_ are indexes to items_ */
-	mutable std::vector<size_t> order_;
+	mutable std::vector<std::size_t> order_;
 	/** whether need to recalculate order_dirty_ */
 	mutable bool order_dirty_;
 
@@ -997,7 +998,7 @@ private:
 		{
 		}
 
-		bool operator()(size_t a, size_t b)
+		bool operator()(std::size_t a, std::size_t b)
 		{
 			return order_func_(a, b);
 		}
@@ -1023,7 +1024,7 @@ private:
 			if(order_.size() != items_.size()) {
 				order_.resize(items_.size());
 
-				for(size_t i = 0; i < items_.size(); ++i) {
+				for(std::size_t i = 0; i < items_.size(); ++i) {
 					order_[i] = i;
 				}
 			}
@@ -1032,7 +1033,7 @@ private:
 				std::stable_sort(order_.begin(), order_.end(), calculate_order_helper(order_func_, items_));
 			}
 
-			for(size_t i = 0; i < order_.size(); ++i) {
+			for(std::size_t i = 0; i < order_.size(); ++i) {
 				items_[order_[i]]->ordered_index = i;
 			}
 
@@ -1072,7 +1073,7 @@ private:
 	 */
 	template<class T>
 	void impl_create_items(const int index,
-			builder_grid_const_ptr list_builder,
+			const builder_grid& list_builder,
 			const std::vector<T>& data,
 			const std::function<void(widget&)>& callback)
 	{
