@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -238,7 +239,7 @@ void grid::request_reduce_width(const unsigned maximum_width)
 
 	const unsigned too_wide = size.x - maximum_width;
 	unsigned reduced = 0;
-	for(size_t col = 0; col < cols_; ++col) {
+	for(std::size_t col = 0; col < cols_; ++col) {
 		if(too_wide - reduced >= col_width_[col]) {
 			DBG_GUI_L << LOG_HEADER << " column " << col
 					  << " is too small to be reduced.\n";
@@ -316,7 +317,7 @@ void grid::request_reduce_height(const unsigned maximum_height)
 
 	const unsigned too_high = size.y - maximum_height;
 	unsigned reduced = 0;
-	for(size_t row = 0; row < rows_; ++row) {
+	for(std::size_t row = 0; row < rows_; ++row) {
 		unsigned wanted_height = row_height_[row] - (too_high - reduced);
 		/**
 		 * @todo Improve this code.
@@ -1032,13 +1033,13 @@ unsigned grid_implementation::row_request_reduce_height(
 	// The minimum height required.
 	unsigned required_height = 0;
 
-	for(size_t x = 0; x < grid.cols_; ++x) {
+	for(std::size_t x = 0; x < grid.cols_; ++x) {
 		grid::child& cell = grid.get_child(row, x);
 		cell_request_reduce_height(cell, maximum_height);
 
 		const point size(cell.get_best_size());
 
-		if(required_height == 0 || static_cast<size_t>(size.y)
+		if(required_height == 0 || static_cast<std::size_t>(size.y)
 								   > required_height) {
 
 			required_height = size.y;
@@ -1057,13 +1058,13 @@ unsigned grid_implementation::column_request_reduce_width(
 	// The minimum width required.
 	unsigned required_width = 0;
 
-	for(size_t y = 0; y < grid.rows_; ++y) {
+	for(std::size_t y = 0; y < grid.rows_; ++y) {
 		grid::child& cell = grid.get_child(y, column);
 		cell_request_reduce_width(cell, maximum_width);
 
 		const point size(cell.get_best_size());
 
-		if(required_width == 0 || static_cast<size_t>(size.x)
+		if(required_width == 0 || static_cast<std::size_t>(size.x)
 								  > required_width) {
 
 			required_width = size.x;
@@ -1115,210 +1116,3 @@ void set_single_child(grid& grid, widget* widget)
 }
 
 } // namespace gui2
-
-
-/*WIKI
- * @page = GUILayout
- *
- * {{Autogenerated}}
- *
- * = Abstract =
- *
- * In the widget library the placement and sizes of elements is determined by
- * a grid. Therefore most widgets have no fixed size.
- *
- *
- * = Theory =
- *
- * We have two examples for the addon dialog, the first example the lower
- * buttons are in one grid, that means if the remove button gets wider
- * (due to translations) the connect button (4.1 - 2.2) will be aligned
- * to the left of the remove button. In the second example the connect
- * button will be partial underneath the remove button.
- *
- * A grid exists of x rows and y columns for all rows the number of columns
- * needs to be the same, there is no column (nor row) span. If spanning is
- * required place a nested grid to do so. In the examples every row has 1 column
- * but rows 3, 4 (and in the second 5) have a nested grid to add more elements
- * per row.
- *
- * In the grid every cell needs to have a widget, if no widget is wanted place
- * the special widget ''spacer''. This is a non-visible item which normally
- * shouldn't have a size. It is possible to give a spacer a size as well but
- * that is discussed elsewhere.
- *
- * Every row and column has a ''grow_factor'', since all columns in a grid are
- * aligned only the columns in the first row need to define their grow factor.
- * The grow factor is used to determine with the extra size available in a
- * dialog. The algorithm determines the extra size work like this:
- *
- * * determine the extra size
- * * determine the sum of the grow factors
- * * if this sum is 0 set the grow factor for every item to 1 and sum to sum of items.
- * * divide the extra size with the sum of grow factors
- * * for every item multiply the grow factor with the division value
- *
- * eg
- *  extra size 100
- *  grow factors 1, 1, 2, 1
- *  sum 5
- *  division 100 / 5 = 20
- *  extra sizes 20, 20, 40, 20
- *
- * Since we force the factors to 1 if all zero it's not possible to have non
- * growing cells. This can be solved by adding an extra cell with a spacer and a
- * grow factor of 1. This is used for the buttons in the examples.
- *
- * Every cell has a ''border_size'' and ''border'' the ''border_size'' is the
- * number of pixels in the cell which aren't available for the widget. This is
- * used to make sure the items in different cells aren't put side to side. With
- * ''border'' it can be determined which sides get the border. So a border is
- * either 0 or ''border_size''.
- *
- * If the widget doesn't grow when there's more space available the alignment
- * determines where in the cell the widget is placed.
- *
- * == Examples ==
- *
- *  |---------------------------------------|
- *  | 1.1                                   |
- *  |---------------------------------------|
- *  | 2.1                                   |
- *  |---------------------------------------|
- *  | |-----------------------------------| |
- *  | | 3.1 - 1.1          | 3.1 - 1.2    | |
- *  | |-----------------------------------| |
- *  |---------------------------------------|
- *  | |-----------------------------------| |
- *  | | 4.1 - 1.1 | 4.1 - 1.2 | 4.1 - 1.3 | |
- *  | |-----------------------------------| |
- *  | | 4.1 - 2.1 | 4.1 - 2.2 | 4.1 - 2.3 | |
- *  | |-----------------------------------| |
- *  |---------------------------------------|
- *
- *
- *  1.1       label : title
- *  2.1       label : description
- *  3.1 - 1.1 label : server
- *  3.1 - 1.2 text box : server to connect to
- *  4.1 - 1.1 spacer
- *  4.1 - 1.2 spacer
- *  4.1 - 1.3 button : remove addon
- *  4.1 - 2.1 spacer
- *  4.1 - 2.2 button : connect
- *  4.1 - 2.3 button : cancel
- *
- *
- *  |---------------------------------------|
- *  | 1.1                                   |
- *  |---------------------------------------|
- *  | 2.1                                   |
- *  |---------------------------------------|
- *  | |-----------------------------------| |
- *  | | 3.1 - 1.1          | 3.1 - 1.2    | |
- *  | |-----------------------------------| |
- *  |---------------------------------------|
- *  | |-----------------------------------| |
- *  | | 4.1 - 1.1         | 4.1 - 1.2     | |
- *  | |-----------------------------------| |
- *  |---------------------------------------|
- *  | |-----------------------------------| |
- *  | | 5.1 - 1.1 | 5.1 - 1.2 | 5.1 - 2.3 | |
- *  | |-----------------------------------| |
- *  |---------------------------------------|
- *
- *
- *  1.1       label : title
- *  2.1       label : description
- *  3.1 - 1.1 label : server
- *  3.1 - 1.2 text box : server to connect to
- *  4.1 - 1.1 spacer
- *  4.1 - 1.2 button : remove addon
- *  5.1 - 1.1 spacer
- *  5.1 - 1.2 button : connect
- *  5.1 - 1.3 button : cancel
- *
- * = Praxis =
- *
- * This is the code needed to create the skeleton for the structure the extra
- * flags are omitted.
- *
- *  	[grid]
- *  		[row]
- *  			[column]
- *  				[label]
- *  					# 1.1
- *  				[/label]
- *  			[/column]
- *  		[/row]
- *  		[row]
- *  			[column]
- *  				[label]
- *  					# 2.1
- *  				[/label]
- *  			[/column]
- *  		[/row]
- *  		[row]
- *  			[column]
- *  				[grid]
- *  					[row]
- *  						[column]
- *  							[label]
- *  								# 3.1 - 1.1
- *  							[/label]
- *  						[/column]
- *  						[column]
- *  							[text_box]
- *  								# 3.1 - 1.2
- *  							[/text_box]
- *  						[/column]
- *  					[/row]
- *  				[/grid]
- *  			[/column]
- *  		[/row]
- *  		[row]
- *  			[column]
- *  				[grid]
- *  					[row]
- *  						[column]
- *  							[spacer]
- *  								# 4.1 - 1.1
- *  							[/spacer]
- *  						[/column]
- *  						[column]
- *  							[spacer]
- *  								# 4.1 - 1.2
- *  							[/spacer]
- *  						[/column]
- *  						[column]
- *  							[button]
- *  								# 4.1 - 1.3
- *  							[/button]
- *  						[/column]
- *  					[/row]
- *  					[row]
- *  						[column]
- *  							[spacer]
- *  								# 4.1 - 2.1
- *  							[/spacer]
- *  						[/column]
- *  						[column]
- *  							[button]
- *  								# 4.1 - 2.2
- *  							[/button]
- *  						[/column]
- *  						[column]
- *  							[button]
- *  								# 4.1 - 2.3
- *  							[/button]
- *  						[/column]
- *  					[/row]
- *  				[/grid]
- *  			[/column]
- *  		[/row]
- *  	[/grid]
- *
- *
- * [[Category: WML Reference]]
- * [[Category: GUI WML Reference]]
- */
