@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -343,16 +344,7 @@ private:
 		 * of copied when the grid's child vector is resized. std::vector will
 		 * utilize a move constructor as long as a non-throwing one is provided.
 		 */
-#if defined(_MSC_VER) && _MSC_VER <= 1800 // MSVC 2013
-		child(child&& c) NOEXCEPT
-			: flags_(c.flags_)
-			, border_size_(c.border_size_)
-			, widget_(std::move(c.widget_))
-		{
-		}
-#else
-		child(child&&) NOEXCEPT = default;
-#endif
+		child(child&&) noexcept = default;
 
 		/** Returns the best size for the cell. */
 		point get_best_size() const;
@@ -418,10 +410,7 @@ private:
 		 */
 		std::unique_ptr<widget> free_widget()
 		{
-			std::unique_ptr<widget> temp(nullptr);
-			widget_.swap(temp);
-
-			return temp;
+			return std::exchange(widget_, nullptr);
 		}
 
 	private:
