@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -100,6 +101,7 @@ void get_screen_size_variables(wfl::map_formula_callable& variable)
 {
 	variable.add("screen_width", wfl::variant(settings::screen_width));
 	variable.add("screen_height", wfl::variant(settings::screen_height));
+	variable.add("screen_pitch_microns", wfl::variant(settings::screen_pitch_microns));
 	variable.add("gamemap_width", wfl::variant(settings::gamemap_width));
 	variable.add("gamemap_height", wfl::variant(settings::gamemap_height));
 	variable.add("gamemap_x_offset", wfl::variant(settings::gamemap_x_offset));
