@@ -1,17 +1,17 @@
 /*
-   Copyright (C) 2012 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2012 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
-
 #pragma once
 
 #include "gui/widgets/styled_widget.hpp"
@@ -95,7 +95,15 @@ public:
 
 typedef control_NEW<state_default> tbase;
 
-/** The matrix class. */
+/**
+ * @ingroup GUIWidgetWML
+ *
+ * List with the matrix specific variables:
+ * Key                      |Type                                            |Default     |Description
+ * -------------------------|------------------------------------------------|------------|-----------
+ * vertical_scrollbar_mode  | @ref guivartype_scrollbar_mode "scrollbar_mode"|initial_auto|Determines whether or not to show the scrollbar.
+ * horizontal_scrollbar_mode| @ref guivartype_scrollbar_mode "scrollbar_mode"|initial_auto|Determines whether or not to show the scrollbar.
+ */
 class matrix : public tbase
 {
 	friend class debug_layout_graph;
@@ -157,7 +165,7 @@ public:
 	 *
 	 * @param compare_functor     The functor to use to sort the items.
 	 */
-	void sort(const pane::tcompare_functor& compare_functor)
+	void sort(const pane::compare_functor_t& compare_functor)
 	{
 		/********************** OUTLINE *******************/
 		pane_->sort(compare_functor);
@@ -172,7 +180,7 @@ public:
 	 * @param filter_functor      The functor to determine whether an item
 	 *                            should be shown or hidden.
 	 */
-	void filter(const pane::tfilter_functor& filter_functor)
+	void filter(const pane::filter_functor_t& filter_functor)
 	{
 		/********************** OUTLINE *******************/
 		pane_->filter(filter_functor);
@@ -259,7 +267,7 @@ struct builder_matrix : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 	scrollbar_container::scrollbar_mode vertical_scrollbar_mode;
 	scrollbar_container::scrollbar_mode horizontal_scrollbar_mode;
