@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -20,7 +21,6 @@
 #include "gui/core/window_builder.hpp"
 
 class config;
-
 namespace gui2
 {
 namespace implementation
@@ -31,10 +31,14 @@ struct builder_minimap;
 // ------------ WIDGET -----------{
 
 /**
- * The basic minimap class.
+ * @ingroup GUIWidgetWML
  *
- * This minimap can only show a minimap, but it can't be interacted with. For
- * that the tminimap_interactive class will be created.
+ * The basic minimap class.
+ * This minimap can only show a minimap, but it can't be interacted with.
+ *
+ * The following states exist:
+ * * state_enabled - the minimap is enabled.
+ * A minimap has no extra fields.
  */
 class minimap : public styled_widget
 {
@@ -75,22 +79,10 @@ public:
 		return map_data_;
 	}
 
-	void set_config(const ::config* terrain)
-	{
-		terrain_ = terrain;
-	}
-
 private:
 	/** The map data to be used to generate the map. */
 	std::string map_data_;
 
-	/**
-	 * The config object with the terrain data.
-	 *
-	 * This config must be set before the object can be drawn.
-	 */
-	const ::config* terrain_;
-
 	/**
 	 * Gets the image for the minimap.
 	 *
@@ -138,7 +130,7 @@ struct builder_minimap : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 };
 
 } // namespace implementation
