@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -31,7 +32,20 @@ struct builder_multi_page;
 
 class generator_base;
 
-/** The multi page class. */
+/**
+ * @ingroup GUIWidgetWML
+ *
+ * A multi page is a control that contains several 'pages' of which only one is visible.
+ * The pages can contain the same widgets containing the same 'kind' of data or look completely different.
+ * Key          |Type                        |Default  |Description
+ * -------------|----------------------------|---------|-----------
+ * grid         | @ref guivartype_grid "grid"|mandatory|Defines the grid with the widgets to place on the page.
+ * A multipage has no states. List with the multi page specific variables:
+ * Key            |Type                              |Default  |Description
+ * ---------------|----------------------------------|---------|-----------
+ * page_definition| @ref guivartype_section "section"|mandatory|This defines how a multi page item looks. It must contain the grid definition for at least one page.
+ * page_data      | @ref guivartype_section "section"|[]       |A grid alike section which stores the initial data for the multi page. Every row must have the same number of columns as the 'page_definition'.
+ */
 class multi_page : public container_base
 {
 	friend struct implementation::builder_multi_page;
@@ -53,6 +67,7 @@ public:
 	 * @returns                   The grid of the newly added page.
 	 */
 	grid& add_page(const string_map& item);
+
 	/**
 	 * Adds single page to the grid.
 	 *
@@ -88,6 +103,7 @@ public:
 	 * @returns                   The grid of the newly added page.
 	 */
 	grid& add_page(const std::map<std::string /* widget id */, string_map>& data);
+
 	/**
 	 * Adds single page to the grid.
 	 *
@@ -103,7 +119,7 @@ public:
 	 *                            members. Having both empty and non-empty
 	 *                            id's gives undefined behavior.
 	 *
-	 * @param type                the id of the [page_definition] that shoduol be used
+	 * @param type                the id of the [page_definition] that should be used
 	 *
 	 * @param insert_pos          the position where th new page is inserted, usually
 	 *                            -1 for 'at end'
@@ -129,7 +145,7 @@ public:
 	unsigned get_page_count() const;
 
 	/**
-	 * Selectes a page.
+	 * Selects a page.
 	 *
 	 * @param page                The page to select.
 	 * @param select              Select or deselect the page.
@@ -172,6 +188,7 @@ public:
 	/** See @ref styled_widget::get_state. */
 	virtual unsigned get_state() const override;
 
+private:
 	/***** ***** ***** setters / getters for members ***** ****** *****/
 
 	void set_page_builders(const std::map<std::string, builder_grid_const_ptr>& page_builders)
@@ -180,7 +197,6 @@ public:
 		page_builders_ = page_builders;
 	}
 
-private:
 	/**
 	 * Finishes the building initialization of the widget.
 	 *
@@ -242,7 +258,7 @@ struct builder_multi_page : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 	std::map<std::string, builder_grid_const_ptr> builders;
 
