@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2012 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2012 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -26,7 +27,7 @@
 #include "lexical_cast.hpp"
 #include "sdl/rect.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 #define LOG_SCOPE_HEADER "pane [" + id() + "] " + __func__
 #define LOG_HEADER LOG_SCOPE_HEADER + ':'
@@ -104,19 +105,6 @@ struct pane_implementation
 	}
 };
 
-pane::pane(const builder_grid_ptr item_builder)
-	: widget()
-	, items_()
-	, item_builder_(item_builder)
-	, item_id_generator_(0)
-	, placer_(placer_base::build(placer_base::tgrow_direction::vertical, 1))
-{
-	connect_signal<event::REQUEST_PLACEMENT>(
-			std::bind(
-					&pane::signal_handler_request_placement, this, _1, _2, _3),
-			event::dispatcher::back_pre_child);
-}
-
 pane::pane(const implementation::builder_pane& builder)
 	: widget(builder)
 	, items_()
@@ -126,7 +114,7 @@ pane::pane(const implementation::builder_pane& builder)
 {
 	connect_signal<event::REQUEST_PLACEMENT>(
 			std::bind(
-					&pane::signal_handler_request_placement, this, _1, _2, _3),
+					&pane::signal_handler_request_placement, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3),
 			event::dispatcher::back_pre_child);
 }
 
@@ -208,14 +196,14 @@ void pane::child_populate_dirty_list(window& caller,
 	}
 }
 
-void pane::sort(const tcompare_functor& compare_functor)
+void pane::sort(const compare_functor_t& compare_functor)
 {
 	items_.sort(compare_functor);
 
 	set_origin_children();
 }
 
-void pane::filter(const tfilter_functor& filter_functor)
+void pane::filter(const filter_functor_t& filter_functor)
 {
 	for(auto & item : items_)
 	{
@@ -382,47 +370,13 @@ void pane::signal_handler_request_placement(dispatcher& dispatcher,
 
 // }---------- BUILDER -----------{
 
-/*WIKI_MACRO
- * @begin{macro}{pane_description}
- *
- *        A pane is a container where new members can be added and removed
- *        during run-time.
- * @end{macro}
- */
-
-/*WIKI
- * @page = GUIWidgetInstanceWML
- * @order = 2_viewport
- * @begin{parent}{name="gui/window/resolution/grid/row/column/"}
- * @begin{tag}{name="pane"}{min=0}{max=-1}{super="generic/widget_instance"}
- * == Label ==
- *
- * @macro = viewport_description
- *
- * List with the label specific variables:
- * @begin{table}{config}
- *     grow_direction & grow_direction & &
- *                                The direction in which new items grow. $
- *     parallel_items & unsigned & &
- *                                The number of items that are growing in
- *                                parallel. $
- *     item_definition & section & &
- *                                The definition of a new item. $
- * @end{table}
- *
- * @begin{tag}{name="item_definition"}{min=1}{max=1}{super="gui/window/resolution/grid"}
- * @end{tag}{name="item_definition"}
- * @end{tag}{name="pane"}
- * @end{parent}{name="gui/window/resolution/grid/row/column/"}
- */
-
 namespace implementation
 {
 
 builder_pane::builder_pane(const config& cfg)
 	: builder_widget(cfg)
 	, grow_direction(
-			  lexical_cast<placer_base::tgrow_direction>(cfg["grow_direction"]))
+			  lexical_cast<placer_base::grow_direction>(cfg["grow_direction"]))
 	, parallel_items(cfg["parallel_items"])
 	, item_definition(new builder_grid(cfg.child("item_definition", "[pane]")))
 {
