@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -29,10 +30,30 @@ struct builder_panel;
 // ------------ WIDGET -----------{
 
 /**
- * Visible container to hold multiple widgets.
+ * @ingroup GUIWidgetWML
  *
- * This widget can draw items beyond the widgets it holds and in front of them.
- * A panel is always active so these functions return dummy values.
+ * A panel is a visible container to hold multiple widgets.
+ * The difference between a grid and a panel is that it's possible to define how a panel looks.
+ * A grid in an invisible container to just hold the items.
+ * A panel is always enabled and can't be disabled.
+ * Instead it uses the states as layers to draw on and can draw items beyond the widgets it holds and in front of them.
+ * A panel is always active so some functions return dummy values.
+ *
+ * The widget instance has the following:
+ * Key          |Type                        |Default  |Description
+ * -------------|----------------------------|---------|-----------
+ * grid         | @ref guivartype_grid "grid"|mandatory|Defines the grid with the widgets to place on the panel.
+ *
+ * The resolution for a panel also contains the following keys:
+ * Key          |Type                                |Default|Description
+ * -------------|------------------------------------|-------|-------------
+ * top_border   | @ref guivartype_unsigned "unsigned"|0      |The size which isn't used for the client area.
+ * bottom_border| @ref guivartype_unsigned "unsigned"|0      |The size which isn't used for the client area.
+ * left_border  | @ref guivartype_unsigned "unsigned"|0      |The size which isn't used for the client area.
+ * right_border | @ref guivartype_unsigned "unsigned"|0      |The size which isn't used for the client area.
+ * The following layers exist:
+ * * background - the background of the panel.
+ * * foreground - the foreground of the panel.
  */
 class panel : public container_base
 {
@@ -106,7 +127,7 @@ struct builder_panel : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 	builder_grid_ptr grid;
 };
