@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -29,6 +30,18 @@ struct builder_repeating_button;
 
 // ------------ WIDGET -----------{
 
+/**
+ * @ingroup GUIWidgetWML
+ *
+ * A repeating_button is a control that can be pushed down and repeat a certain action.
+ * Once the button is down every x milliseconds it is down a new down event is triggered.
+ *
+ * The following states exist:
+ * * state_enabled - the repeating_button is enabled.
+ * * state_disabled - the repeating_button is disabled.
+ * * state_pressed - the left mouse repeating_button is down.
+ * * state_focussed - the mouse is over the repeating_button.
+ */
 class repeating_button : public styled_widget, public clickable_item
 {
 public:
@@ -100,7 +113,7 @@ private:
 	state_t state_;
 
 	/** The timer for the repeating events. */
-	size_t repeat_timer_;
+	std::size_t repeat_timer_;
 
 public:
 	/** Static type getter that does not rely on the widget being constructed. */
@@ -147,7 +160,7 @@ public:
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 };
 
 } // namespace implementation
