@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -26,7 +27,7 @@
 #include "sdl/rect.hpp"
 
 #include <algorithm>
-#include "utils/functional.hpp"
+#include <functional>
 
 #define LOG_SCOPE_HEADER get_control_type() + " [" + id() + "] " + __func__
 #define LOG_HEADER LOG_SCOPE_HEADER + ':'
@@ -81,31 +82,31 @@ scrollbar_container::scrollbar_container(
 	, content_visible_area_()
 {
 	connect_signal<event::SDL_KEY_DOWN>(
-		std::bind(&scrollbar_container::signal_handler_sdl_key_down, this, _2, _3, _5, _6));
+		std::bind(&scrollbar_container::signal_handler_sdl_key_down, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5, std::placeholders::_6));
 
 	connect_signal<event::SDL_WHEEL_UP>(
-		std::bind(&scrollbar_container::signal_handler_sdl_wheel_up, this, _2, _3),
+		std::bind(&scrollbar_container::signal_handler_sdl_wheel_up, this, std::placeholders::_2, std::placeholders::_3),
 		event::dispatcher::back_post_child);
 
 	connect_signal<event::SDL_WHEEL_DOWN>(
-		std::bind(&scrollbar_container::signal_handler_sdl_wheel_down, this, _2, _3),
+		std::bind(&scrollbar_container::signal_handler_sdl_wheel_down, this, std::placeholders::_2, std::placeholders::_3),
 		event::dispatcher::back_post_child);
 
 	connect_signal<event::SDL_WHEEL_LEFT>(
-		std::bind(&scrollbar_container::signal_handler_sdl_wheel_left, this, _2, _3),
+		std::bind(&scrollbar_container::signal_handler_sdl_wheel_left, this, std::placeholders::_2, std::placeholders::_3),
 		event::dispatcher::back_post_child);
 
 	connect_signal<event::SDL_WHEEL_RIGHT>(
-		std::bind(&scrollbar_container::signal_handler_sdl_wheel_right, this, _2, _3),
+		std::bind(&scrollbar_container::signal_handler_sdl_wheel_right, this, std::placeholders::_2, std::placeholders::_3),
 		event::dispatcher::back_post_child);
 
 	connect_signal<event::SDL_TOUCH_MOTION>(
 			std::bind(&scrollbar_container::signal_handler_sdl_touch_motion,
 						this,
-						_2,
-						_3,
-						_5,
-						_6),
+						std::placeholders::_2,
+						std::placeholders::_3,
+						std::placeholders::_5,
+						std::placeholders::_6),
 			event::dispatcher::back_post_child);
 }
 
@@ -768,7 +769,7 @@ void scrollbar_container::finalize_setup()
 	}
 
 	/***** Setup the content *****/
-	content_ = build_single_widget_instance<spacer>("spacer");
+	content_ = build_single_widget_instance<spacer>();
 
 	// TODO: possibly move this unique_ptr casting functionality to a helper function.
 	content_grid_.reset(dynamic_cast<grid*>(get_grid().swap_child("_content_grid", content_, true).release()));
