@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -23,7 +24,7 @@
 #include "gettext.hpp"
 #include "wml_exception.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 namespace gui2
 {
@@ -62,31 +63,6 @@ scrollbar_panel_definition::scrollbar_panel_definition(const config& cfg)
 	load_resolutions<resolution>(cfg);
 }
 
-/*WIKI
- * @page = GUIWidgetDefinitionWML
- * @order = 1_scrollbar_panel
- *
- * == Scrollbar panel ==
- *
- * @begin{parent}{name="gui/"}
- * @begin{tag}{name="scrollbar_panel_definition"}{min=0}{max=-1}{super="generic/widget_definition"}
- * @begin{tag}{name="resolution"}{min=0}{max=-1}{super="gui/window_definition/resolution"}
- * The definition of a panel with scrollbars. A panel is a container holding
- * other elements in its grid. A panel is always enabled and can't be
- * disabled. Instead it uses the states as layers to draw on.
- *
- * @begin{table}{config}
- *     grid & grid & &                    A grid containing the widgets for main
- *                                     widget. $
- * @end{table}
- * The following layers exist:
- * * background, the background of the panel.
- * * foreground, the foreground of the panel.
- *
- * @end{tag}{name="resolution"}
- * @end{tag}{name="scrollbar_panel_definition"}
- * @end{parent}{name="gui/"}
- */
 scrollbar_panel_definition::resolution::resolution(const config& cfg)
 	: resolution_definition(cfg), grid()
 {
@@ -102,35 +78,6 @@ scrollbar_panel_definition::resolution::resolution(const config& cfg)
 
 // }---------- BUILDER -----------{
 
-/*WIKI
- * @page = GUIWidgetInstanceWML
- * @order = 2_scrollbar_panel
- *
- * == Scrollbar panel ==
- * @begin{parent}{name="gui/window/resolution/grid/row/column/"}
- * @begin{tag}{name="scrollbar_panel"}{min="0"}{max="-1"}{super="generic/widget_instance"}
- * Instance of a scrollbar_panel.
- *
- * List with the scrollbar_panel specific variables:
- * @begin{table}{config}
- *     vertical_scrollbar_mode & scrollbar_mode & initial_auto &
- *                                     Determines whether or not to show the
- *                                     scrollbar. $
- *     horizontal_scrollbar_mode & scrollbar_mode & initial_auto &
- *                                     Determines whether or not to show the
- *                                     scrollbar. $
- *
- *     definition & section & &        This defines how a scrollbar_panel item
- *                                     looks. It must contain the grid
- *                                     definition for 1 row of the list. $
- *
- * @end{table}
- * @begin{tag}{name="definition"}{min=0}{max=1}{super="gui/window/resolution/grid"}
- * @end{tag}{name="definition"}
- * @end{tag}{name="scrollbar_panel"}
- * @end{parent}{name="gui/window/resolution/grid/row/column/"}
- */
-
 namespace implementation
 {
 
@@ -162,7 +109,7 @@ widget* builder_scrollbar_panel::build() const
 	const auto conf = panel->cast_config_to<scrollbar_panel_definition>();
 	assert(conf);
 
-	panel->init_grid(conf->grid);
+	panel->init_grid(*conf->grid);
 	panel->finalize_setup();
 
 	/*** Fill the content grid. ***/
