@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -30,10 +31,28 @@ struct builder_scrollbar_panel;
 }
 
 /**
+ * @ingroup GUIWidgetWML
+ *
  * Visible container to hold multiple widgets.
  *
- * This widget can draw items beyond the widgets it holds and in front of
- * them. A panel is always active so these functions return dummy values.
+ * This widget can draw items beyond the widgets it holds and in front of them.
+ * A panel is always active so these functions return dummy values.
+ *
+ * A panel is a container holding other elements in its grid.
+ * It uses the states as layers to draw on.
+ *
+ * Key          |Type                        |Default  |Description
+ * -------------|----------------------------|---------|-----------
+ * grid         | @ref guivartype_grid "grid"|mandatory|A grid containing the widgets for main widget.
+ * The following layers exist:
+ * * background - the background of the panel.
+ * * foreground - the foreground of the panel.
+ * List with the scrollbar_panel specific variables:
+ * Key                      |Type                                            |Default     |Description
+ * -------------------------|------------------------------------------------|------------|-----------
+ * vertical_scrollbar_mode  | @ref guivartype_scrollbar_mode "scrollbar_mode"|initial_auto|Determines whether or not to show the scrollbar.
+ * horizontal_scrollbar_mode| @ref guivartype_scrollbar_mode "scrollbar_mode"|initial_auto|Determines whether or not to show the scrollbar.
+ * definition               | @ref guivartype_section "section"              |mandatory   |This defines how a scrollbar_panel item looks. It must contain the grid definition for 1 row of the list.
  */
 class scrollbar_panel : public scrollbar_container
 {
@@ -88,7 +107,7 @@ struct builder_scrollbar_panel : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 	scrollbar_container::scrollbar_mode vertical_scrollbar_mode;
 	scrollbar_container::scrollbar_mode horizontal_scrollbar_mode;
