@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2007 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2007 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -38,11 +39,12 @@ extern bool new_widgets;
 namespace settings
 {
 /**
- * The screen resolution should be available for all widgets since
- * their drawing method will depend on it.
+ * The screen resolution and pixel pitch should be available for all widgets since
+ * their drawing method might depend on it.
  */
 extern unsigned screen_width;
 extern unsigned screen_height;
+extern unsigned screen_pitch_microns; /* Pixel pitch in microns */
 
 /**
  * The offset between the left edge of the screen and the gamemap.
