@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -25,17 +26,11 @@
 #include "gui/widgets/window.hpp"
 #include "sdl/rect.hpp"
 #include "sound.hpp"
-#include "utils/general.hpp"
 #include "gettext.hpp"
 #include "wml_exception.hpp"
 
-#include "utils/functional.hpp"
-
-#if BOOST_VERSION >= 106700
-#include <boost/integer/common_factor_rt.hpp>
-#else
-#include <boost/math/common_factor_rt.hpp>
-#endif
+#include <functional>
+#include <numeric>
 
 #define LOG_SCOPE_HEADER get_control_type() + " [" + id() + "] " + __func__
 #define LOG_HEADER LOG_SCOPE_HEADER + ':'
@@ -56,12 +51,12 @@ slider::slider(const implementation::builder_slider& builder)
 	, value_label_generator_()
 	, current_item_mouse_position_(0, 0)
 {
-	connect_signal<event::SDL_KEY_DOWN>(std::bind(&slider::signal_handler_sdl_key_down, this, _2, _3, _5));
+	connect_signal<event::SDL_KEY_DOWN>(std::bind(&slider::signal_handler_sdl_key_down, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5));
 
 	// connect_signal<event::LEFT_BUTTON_DOWN>(
-	//		std::bind(&slider::signal_handler_left_button_down, this, _2, _3));
+	//		std::bind(&slider::signal_handler_left_button_down, this, std::placeholders::_2, std::placeholders::_3));
 
-	connect_signal<event::LEFT_BUTTON_UP>(std::bind(&slider::signal_handler_left_button_up, this, _2, _3));
+	connect_signal<event::LEFT_BUTTON_UP>(std::bind(&slider::signal_handler_left_button_up, this, std::placeholders::_2, std::placeholders::_3));
 }
 
 point slider::calculate_best_size() const
@@ -85,7 +80,7 @@ point slider::calculate_best_size() const
 
 void slider::set_value(int value)
 {
-	value = utils::clamp(value, minimum_value_, get_maximum_value());
+	value = std::clamp(value, minimum_value_, get_maximum_value());
 	int old_value = get_value();
 
 	if(value == old_value) {
@@ -158,8 +153,8 @@ bool slider::on_positioner(const point& coordinate) const
 
 int slider::on_bar(const point& coordinate) const
 {
-	const unsigned x = static_cast<size_t>(coordinate.x);
-	const unsigned y = static_cast<size_t>(coordinate.y);
+	const unsigned x = static_cast<std::size_t>(coordinate.x);
+	const unsigned y = static_cast<std::size_t>(coordinate.y);
 
 	// Not on the widget, leave.
 	if(x > get_width() || y > get_height()) {
@@ -247,7 +242,7 @@ static t_string default_value_label_generator(const std::vector<t_string>& value
 void slider::set_value_labels(const std::vector<t_string>& value_labels)
 {
 	// Don't use std::ref because we want to store value_labels in the closure.
-	set_value_labels(std::bind(&default_value_label_generator, value_labels, _1, _2));
+	set_value_labels(std::bind(&default_value_label_generator, value_labels, std::placeholders::_1, std::placeholders::_2));
 }
 
 
@@ -266,12 +261,7 @@ void slider::set_value_range(int min_value, int max_value)
 	int diff = max_value - min_value;
 	int old_value = get_value();
 
-#if BOOST_VERSION >= 106700
-	step_size_ = boost::integer::gcd(diff, step_size_);
-#else
-	step_size_ = boost::math::gcd(diff, step_size_);
-#endif
-
+	step_size_ = std::gcd(diff, step_size_);
 	minimum_value_ = min_value;
 
 	slider_set_item_last(diff / step_size_);
@@ -290,11 +280,8 @@ void slider::set_step_size(int step_size)
 	const int range_diff = get_item_count() - 1;
 	const int old_value = get_value();
 
-#if BOOST_VERSION >= 106700
-	step_size_ = boost::integer::gcd(range_diff, step_size);
-#else
-	step_size_ = boost::math::gcd(range_diff, step_size);
-#endif
+	step_size_ = std::gcd(range_diff, step_size);
+
 	slider_set_item_last(range_diff / step_size_);
 	set_value(old_value);
 
@@ -312,53 +299,6 @@ slider_definition::slider_definition(const config& cfg)
 	load_resolutions<resolution>(cfg);
 }
 
-/*WIKI
- * @page = GUIWidgetDefinitionWML
- * @order = 1_slider
- *
- * == Slider ==
- *
- * @macro = slider_description
- *
- * @begin{parent}{name="gui/"}
- * @begin{tag}{name="slider_definition"}{min=0}{max=-1}{super="generic/widget_definition"}
- * @begin{tag}{name="resolution"}{min=0}{max=-1}{super="generic/widget_definition/resolution"}
- * @begin{table}{config}
- *     minimum_positioner_length & unsigned & &
- *                                     The minimum size the positioner is
- *                                     allowed to be. The engine needs to know
- *                                     this in order to calculate the best size
- *                                     for the positioner. $
- *     maximum_positioner_length & unsigned & 0 &
- *                                     The maximum size the positioner is
- *                                     allowed to be. If minimum and maximum are
- *                                     the same value the positioner is fixed
- *                                     size. If the maximum is 0 (and the
- *                                     minimum not) there's no maximum. $
- *     left_offset & unsigned & 0 &    The number of pixels at the left side
- *                                     which can't be used by the positioner. $
- *     right_offset & unsigned & 0 &   The number of pixels at the right side
- *                                     which can't be used by the positioner. $
- * @end{table}
- *
- * The following states exist:
- * * state_enabled, the slider is enabled.
- * * state_disabled, the slider is disabled.
- * * state_pressed, the left mouse button is down on the positioner of the
- *   slider.
- * * state_focused, the mouse is over the positioner of the slider.
- * @begin{tag}{name="state_enabled"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_enabled"}
- * @begin{tag}{name="state_disabled"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_disabled"}
- * @begin{tag}{name="state_pressed"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_pressed"}
- * @begin{tag}{name="state_focused"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_focused"}
- * @end{tag}{name="resolution"}
- * @end{tag}{name="slider_definition"}
- * @end{parent}{name="gui/"}
- */
 slider_definition::resolution::resolution(const config& cfg)
 	: resolution_definition(cfg)
 	, positioner_length(cfg["minimum_positioner_length"])
@@ -376,57 +316,6 @@ slider_definition::resolution::resolution(const config& cfg)
 
 // }---------- BUILDER -----------{
 
-/*WIKI_MACRO
- * @begin{macro}{slider_description}
- * A slider is a styled_widget that can select a value by moving a grip on a groove.
- * @end{macro}
- */
-
-/*WIKI
- * @page = GUIWidgetInstanceWML
- * @order = 3_slider
- * @begin{parent}{name="gui/window/resolution/grid/row/column/"}
- * @begin{tag}{name="slider"}{min="0"}{max="-1"}{super="generic/widget_instance"}
- * == Slider ==
- *
- * @macro = slider_description
- *
- * @begin{table}{config}
- *     best_slider_length & unsigned & 0 &
- *                                    The best length for the sliding part. $
- *     minimum_value & int & 0 &        The minimum value the slider can have. $
- *     maximum_value & int & 0 &        The maximum value the slider can have. $
- *
- *     step_size & unsigned & 0 &       The number of items the slider's value
- *                                    increases with one step. $
- *     value & int & 0 &                The value of the slider. $
- *
- *     minimum_value_label & t_string & "" &
- *                                    If the minimum value is chosen there
- *                                    might be the need for a special value
- *                                    (eg off). When this key has a value
- *                                    that value will be shown if the minimum
- *                                    is selected. $
- *     maximum_value_label & t_string & "" &
- *                                    If the maximum value is chosen there
- *                                    might be the need for a special value
- *                                    (eg unlimited)). When this key has a
- *                                    value that value will be shown if the
- *                                    maximum is selected. $
- *     value_labels & [] &              It might be the labels need to be shown
- *                                    are not a linear number sequence eg
- *                                    (0.5, 1, 2, 4) in that case for all
- *                                    items this section can be filled with
- *                                    the values, which should be the same
- *                                    number of items as the items in the
- *                                    slider. NOTE if this option is used,
- *                                    'minimum_value_label' and
- *                                    'maximum_value_label' are ignored. $
- * @end{table}
- * @end{tag}{name="slider"}
- * @end{parent}{name="gui/window/resolution/grid/row/column/"}
- */
-
 namespace implementation
 {
 builder_slider::builder_slider(const config& cfg)
@@ -462,7 +351,7 @@ widget* builder_slider::build() const
 	widget->finalize_setup();
 
 	if(!value_labels_.empty()) {
-		VALIDATE(value_labels_.size() == static_cast<size_t>(widget->get_item_count()),
+		VALIDATE(value_labels_.size() == static_cast<std::size_t>(widget->get_item_count()),
 				 _("The number of value_labels and values don't match."));
 
 		widget->set_value_labels(value_labels_);
