@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -29,7 +30,32 @@ struct builder_slider;
 
 // ------------ WIDGET -----------{
 
-/** A slider. */
+/**
+ * @ingroup GUIWidgetWML
+ *
+ * A slider is a control that can select a value by moving a grip on a groove.
+ * Key                      |Type                                |Default  |Description
+ * -------------------------|------------------------------------|---------|-----------
+ * minimum_positioner_length| @ref guivartype_unsigned "unsigned"|mandatory|The minimum size the positioner is allowed to be. The engine needs to know this in order to calculate the best size for the positioner.
+ * maximum_positioner_length| @ref guivartype_unsigned "unsigned"|0        |The maximum size the positioner is allowed to be. If minimum and maximum are the same value the positioner is fixed size. If the maximum is 0 (and the minimum not) there's no maximum.
+ * left_offset              | @ref guivartype_unsigned "unsigned"|0        |The number of pixels at the left side which can't be used by the positioner.
+ * right_offset             | @ref guivartype_unsigned "unsigned"|0        |The number of pixels at the right side which can't be used by the positioner.
+ * Variables:
+ * Key                      |Type                                |Default  |Description
+ * -------------------------|------------------------------------|---------|-----------
+ * best_slider_length       | @ref guivartype_unsigned "unsigned"|0        |The best length for the sliding part.
+ * minimum_value            | @ref guivartype_int "int"          |0        |The minimum value the slider can have.
+ * maximum_value            | @ref guivartype_int "int"          |0        |The maximum value the slider can have.
+ * step_size                | @ref guivartype_unsigned "unsigned"|0        |The number of items the slider's value increases with one step.
+ * value                    | @ref guivartype_int "int"          |0        |The value of the slider.
+ * minimum_value_label      | @ref guivartype_t_string "t_string"|""       |If the minimum value is chosen there might be the need for a special value (eg off). When this key has a value that value will be shown if the minimum is selected.
+ * maximum_value_label      | @ref guivartype_t_string "t_string"|""       |If the maximum value is chosen there might be the need for a special value (eg unlimited)). When this key has a value that value will be shown if the maximum is selected.
+ * The following states exist:
+ * * state_enabled - the slider is enabled.
+ * * state_disabled - the slider is disabled.
+ * * state_pressed - the left mouse button is down on the positioner of the slider.
+ * * state_focussed - the mouse is over the positioner of the slider.
+ */
 class slider : public slider_base, public integer_selector
 {
 	friend struct implementation::builder_slider;
@@ -196,6 +222,11 @@ private:
 
 	// void update_current_item_mouse_position();
 
+public:
+	/** Static type getter that does not rely on the widget being constructed. */
+	static const std::string& type();
+
+private:
 	/** Inherited from styled_widget, implemented by REGISTER_WIDGET. */
 	virtual const std::string& get_control_type() const override;
 
@@ -244,7 +275,7 @@ struct builder_slider : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 private:
 	unsigned best_slider_length_;
