@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -19,7 +20,7 @@
 #include "gui/core/register_widget.hpp"
 #include "gui/widgets/settings.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 namespace gui2
 {
@@ -115,20 +116,6 @@ spacer_definition::spacer_definition(const config& cfg)
 	load_resolutions<resolution>(cfg);
 }
 
-/*WIKI
- * @page = GUIWidgetDefinitionWML
- * @order = 1_spacer
- *
- * == Spacer ==
- *
- * @macro = spacer_description
- *
- * A spacer has no states so nothing to load.
- * @begin{parent}{name="gui/"}
- * @begin{tag}{name="spacer_definition"}{min=0}{max=-1}{super="generic/widget_definition"}
- * @end{tag}{name="spacer_definition"}
- * @end{parent}{name="gui/"}
- */
 spacer_definition::resolution::resolution(const config& cfg)
 	: resolution_definition(cfg)
 {
@@ -136,39 +123,6 @@ spacer_definition::resolution::resolution(const config& cfg)
 
 // }---------- BUILDER -----------{
 
-/*WIKI_MACRO
- * @begin{macro}{spacer_description}
- *
- *        A spacer is a dummy item to either fill in a widget since no empty
- *        items are allowed or to reserve a fixed space.
- * @end{macro}
- */
-
-
-/*WIKI
- * @page = GUIWidgetInstanceWML
- * @order = 2_spacer
- * @begin{parent}{name="gui/window/resolution/grid/row/column/"}
- * @begin{tag}{name="spacer"}{min=0}{max=-1}{super="generic/widget_instance"}
- * == Spacer ==
- *
- * @macro = spacer_description
- *
- * If either the width or the height is non-zero the spacer functions as a
- * fixed size spacer.
- *
- * @begin{table}{config}
- *     width & f_unsigned & 0 &          The width of the spacer. $
- *     height & f_unsigned & 0 &         The height of the spacer. $
- * @end{table}
- *
- * The variable available are the same as for the window resolution see
- * https://www.wesnoth.org/wiki/GUIToolkitWML#Resolution_2 for the list of
- * items.
- * @end{tag}{name="spacer"}
- * @end{parent}{name="gui/window/resolution/grid/row/column/"}
- */
-
 namespace implementation
 {
 
