@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -30,13 +31,20 @@ struct builder_spacer;
 // ------------ WIDGET -----------{
 
 /**
- * An empty widget.
+ * @ingroup GUIWidgetWML
  *
- * Since every grid cell needs a widget this is a blank widget. This widget can
- * also be used to 'force' sizes.
+ * An empty widget to either fill in a widget since no empty items are allowed or to reserve a fixed space.
  *
- * Since we're a kind of dummy class we're always active, our drawing does
- * nothing.
+ * Since every grid cell needs a widget this is a blank widget.
+ * This widget can also be used to 'force' sizes - if either the width or the height is non-zero the spacer functions as a fixed size spacer.
+ *
+ * Since we're a kind of dummy class we're always active, our drawing does nothing.
+ *
+ * Key          |Type                                    |Default  |Description
+ * -------------|----------------------------------------|---------|-----------
+ * width        | @ref guivartype_f_unsigned "f_unsigned"|0        |The width of the spacer.
+ * height       | @ref guivartype_f_unsigned "f_unsigned"|0        |The height of the spacer.
+ * The variable available are the same as for the window resolution see @ref builder_window::window_resolution for the list of items.
  */
 class spacer : public styled_widget
 {
@@ -115,7 +123,7 @@ struct builder_spacer : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 private:
 	// We store these as strings since they could contain formulas.
