@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -33,6 +34,15 @@ struct builder_stacked_widget;
 
 class generator_base;
 
+/**
+ * @ingroup GUIWidgetWML
+ *
+ * A stacked widget holds several widgets on top of each other.
+ * This can be used for various effects; add an optional overlay to an image, stack it with a spacer to force a minimum size of a widget.
+ * The latter is handy to avoid making a separate definition for a single instance with a fixed size.
+ *
+ * A stacked widget has no states.
+ */
 class stacked_widget : public container_base
 {
 	friend struct stacked_widget_implementation;
@@ -120,10 +130,10 @@ private:
 	/**
 	 * Finishes the building initialization of the widget.
 	 *
-	 * @param widget_builder      The builder to build the contents of the
+	 * @param widget_builders     The builder to build the contents of the
 	 *                            widget.
 	 */
-	void finalize(std::vector<builder_grid_const_ptr> widget_builder);
+	void finalize(const std::vector<builder_grid>& widget_builders);
 
 	/**
 	 * Contains a pointer to the generator.
@@ -205,10 +215,10 @@ struct builder_stacked_widget : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 	/** The builders for all layers of the stack .*/
-	std::vector<builder_grid_const_ptr> stack;
+	std::vector<builder_grid> stack;
 };
 
 } // namespace implementation
