@@ -1,29 +1,29 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "gui/widgets/text_box.hpp"
 
-#include "font/sdl_ttf.hpp"
 #include "gui/core/log.hpp"
 #include "gui/core/register_widget.hpp"
 #include "gui/widgets/settings.hpp"
 #include "gui/widgets/window.hpp"
 #include "preferences/game.hpp"
 #include "serialization/unicode.hpp"
-#include "utils/functional.hpp"
+#include <functional>
 
 #define LOG_SCOPE_HEADER get_control_type() + " [" + id() + "] " + __func__
 #define LOG_HEADER LOG_SCOPE_HEADER + ':'
@@ -107,18 +107,18 @@ text_box::text_box(const implementation::builder_styled_widget& builder)
 	set_wants_mouse_left_double_click();
 
 	connect_signal<event::MOUSE_MOTION>(std::bind(
-			&text_box::signal_handler_mouse_motion, this, _2, _3, _5));
+			&text_box::signal_handler_mouse_motion, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5));
 	connect_signal<event::LEFT_BUTTON_DOWN>(std::bind(
-			&text_box::signal_handler_left_button_down, this, _2, _3));
+			&text_box::signal_handler_left_button_down, this, std::placeholders::_2, std::placeholders::_3));
 	connect_signal<event::LEFT_BUTTON_UP>(std::bind(
-			&text_box::signal_handler_left_button_up, this, _2, _3));
+			&text_box::signal_handler_left_button_up, this, std::placeholders::_2, std::placeholders::_3));
 	connect_signal<event::LEFT_BUTTON_DOUBLE_CLICK>(std::bind(
-			&text_box::signal_handler_left_button_double_click, this, _2, _3));
+			&text_box::signal_handler_left_button_double_click, this, std::placeholders::_2, std::placeholders::_3));
 
 	const auto conf = cast_config_to<text_box_definition>();
 	assert(conf);
 
-	set_font_size(conf->text_font_size);
+	set_font_size(get_text_font_size());
 	set_font_style(conf->text_font_style);
 
 	update_offsets();
@@ -242,7 +242,7 @@ void text_box::delete_selection()
 		start -= len;
 	}
 
-	utf8::string tmp = get_value();
+	std::string tmp = get_value();
 	set_value(utf8::erase(tmp, start, len));
 	set_cursor(start, false);
 }
@@ -278,8 +278,7 @@ void text_box::update_offsets()
 	const auto conf = cast_config_to<text_box_definition>();
 	assert(conf);
 
-	// FIXME: This should use pango-cairo code path instead of sdl_ttf code path
-	text_height_ = font::get_max_height(conf->text_font_size);
+	text_height_ = font::get_max_height(get_text_font_size());
 
 	wfl::map_formula_callable variables;
 	variables.add("height", wfl::variant(get_height()));
@@ -403,44 +402,6 @@ text_box_definition::text_box_definition(const config& cfg)
 	load_resolutions<resolution>(cfg);
 }
 
-/*WIKI
- * @page = GUIWidgetDefinitionWML
- * @order = 1_text_box
- *
- * == Text box ==
- *
- * The definition of a text box.
- *
- * @begin{parent}{name="gui/"}
- * @begin{tag}{name="ext_box_definition"}{min=0}{max=-1}{super="generic/widget_definition"}
- * The resolution for a text box also contains the following keys:
- * @begin{tag}{name="resolution"}{min=0}{max=-1}{super=generic/widget_definition/resolution}
- * @begin{table}{config}
- *     text_x_offset & f_unsigned & "" & The x offset of the text in the text
- *                                     box. This is needed for the code to
- *                                     determine where in the text the mouse
- *                                     clicks, so it can set the cursor
- *                                     properly. $
- *     text_y_offset & f_unsigned & "" & The y offset of the text in the text
- *                                     box. $
- * @end{table}
- *
- * The following states exist:
- * * state_enabled, the text box is enabled.
- * * state_disabled, the text box is disabled.
- * * state_focused, the text box has the focus of the keyboard.
- * @begin{tag}{name="state_enabled"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_enabled"}
- * @begin{tag}{name="state_disabled"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_disabled"}
- * @begin{tag}{name="state_focused"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_focused"}
- * @begin{tag}{name="state_hovered"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_hovered"}
- * @end{tag}{name="resolution"}
- * @end{tag}{name="ext_box_definition"}
- * @end{parent}{name="gui/"}
- */
 text_box_definition::resolution::resolution(const config& cfg)
 	: resolution_definition(cfg)
 	, text_x_offset(cfg["text_x_offset"])
@@ -455,28 +416,6 @@ text_box_definition::resolution::resolution(const config& cfg)
 
 // }---------- BUILDER -----------{
 
-/*WIKI
- * @page = GUIWidgetInstanceWML
- * @order = 2_text_box
- *
- * == Text box ==
- * @begin{parent}{name="gui/window/resolution/grid/row/column/"}
- * @begin{tag}{name="text_box"}{min="0"}{max="-1"}{super="generic/widget_instance"}
- * @begin{table}{config}
- *     label & t_string & "" &          The initial text of the text box. $
- *     history & string & "" &         The name of the history for the text
- *                                     box.
- *                                     A history saves the data entered in a
- *                                     text box between the games. With the up
- *                                     and down arrow it can be accessed. To
- *                                     create a new history item just add a
- *                                     new unique name for this field and the
- *                                     engine will handle the rest. $
- * @end{table}
- * @end{tag}{name="text_box"}
- * @end{parent}{name="gui/window/resolution/grid/row/column/"}
- */
-
 namespace implementation
 {
 
