@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -18,11 +19,12 @@
 
 #include "cursor.hpp"
 #include "desktop/clipboard.hpp"
+#include "gui/core/gui_definition.hpp"
 #include "gui/core/log.hpp"
 #include "gui/core/timer.hpp"
 #include "serialization/unicode.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 #include <limits>
 
@@ -45,27 +47,30 @@ text_box_base::text_box_base(const implementation::builder_styled_widget& builde
 	, cursor_blink_rate_ms_(750)
 	, text_changed_callback_()
 {
+	auto cfg = get_control(control_type, builder.definition);
+	text_.set_family_class(cfg->text_font_family);
+
 #ifdef __unix__
 	// pastes on UNIX systems.
 	connect_signal<event::MIDDLE_BUTTON_CLICK>(std::bind(
-			&text_box_base::signal_handler_middle_button_click, this, _2, _3));
+			&text_box_base::signal_handler_middle_button_click, this, std::placeholders::_2, std::placeholders::_3));
 
 #endif
 
 	connect_signal<event::SDL_KEY_DOWN>(std::bind(
-			&text_box_base::signal_handler_sdl_key_down, this, _2, _3, _5, _6));
-	connect_signal<event::SDL_TEXT_INPUT>(std::bind(&text_box_base::handle_commit, this, _3, _5));
-	connect_signal<event::SDL_TEXT_EDITING>(std::bind(&text_box_base::handle_editing, this, _3, _5, _6, _7));
+			&text_box_base::signal_handler_sdl_key_down, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5, std::placeholders::_6));
+	connect_signal<event::SDL_TEXT_INPUT>(std::bind(&text_box_base::handle_commit, this, std::placeholders::_3, std::placeholders::_5));
+	connect_signal<event::SDL_TEXT_EDITING>(std::bind(&text_box_base::handle_editing, this, std::placeholders::_3, std::placeholders::_5, std::placeholders::_6, std::placeholders::_7));
 
 	connect_signal<event::RECEIVE_KEYBOARD_FOCUS>(std::bind(
-			&text_box_base::signal_handler_receive_keyboard_focus, this, _2));
+			&text_box_base::signal_handler_receive_keyboard_focus, this, std::placeholders::_2));
 	connect_signal<event::LOSE_KEYBOARD_FOCUS>(
-			std::bind(&text_box_base::signal_handler_lose_keyboard_focus, this, _2));
+			std::bind(&text_box_base::signal_handler_lose_keyboard_focus, this, std::placeholders::_2));
 
 	connect_signal<event::MOUSE_ENTER>(
-			std::bind(&text_box_base::signal_handler_mouse_enter, this, _2, _3));
+			std::bind(&text_box_base::signal_handler_mouse_enter, this, std::placeholders::_2, std::placeholders::_3));
 	connect_signal<event::MOUSE_LEAVE>(
-			std::bind(&text_box_base::signal_handler_mouse_leave, this, _2, _3));
+			std::bind(&text_box_base::signal_handler_mouse_leave, this, std::placeholders::_2, std::placeholders::_3));
 
 	toggle_cursor_timer(true);
 }
@@ -93,7 +98,7 @@ unsigned text_box_base::get_state() const
 	return state_;
 }
 
-void text_box_base::set_maximum_length(const size_t maximum_length)
+void text_box_base::set_maximum_length(const std::size_t maximum_length)
 {
 	if(maximum_length == 0) {
 		return;
@@ -128,7 +133,7 @@ void text_box_base::set_value(const std::string& text)
 	}
 }
 
-void text_box_base::set_cursor(const size_t offset, const bool select)
+void text_box_base::set_cursor(const std::size_t offset, const bool select)
 {
 	reset_cursor_state();
 
@@ -157,7 +162,7 @@ void text_box_base::set_cursor(const size_t offset, const bool select)
 	}
 }
 
-void text_box_base::insert_char(const utf8::string& unicode)
+void text_box_base::insert_char(const std::string& unicode)
 {
 	delete_selection();
 
@@ -200,7 +205,7 @@ void text_box_base::copy_selection(const bool mouse)
 	}
 
 	unsigned end, start = selection_start_;
-	const utf8::string txt = text_.text();
+	const std::string txt = text_.text();
 
 	if(selection_length_ > 0) {
 		end = utf8::index(txt, start + selection_length_);
@@ -229,7 +234,7 @@ void text_box_base::paste_selection(const bool mouse)
 	fire(event::NOTIFY_MODIFIED, *this, nullptr);
 }
 
-void text_box_base::set_selection_start(const size_t selection_start)
+void text_box_base::set_selection_start(const std::size_t selection_start)
 {
 	if(selection_start != selection_start_) {
 		selection_start_ = selection_start;
@@ -245,9 +250,9 @@ void text_box_base::set_selection_length(const int selection_length)
 	}
 }
 
-void text_box_base::set_selection(size_t start, int length)
+void text_box_base::set_selection(std::size_t start, int length)
 {
-	const size_t text_size = text_.get_length();
+	const std::size_t text_size = text_.get_length();
 
 	if(start >= text_size) {
 		start = text_size;
@@ -260,8 +265,8 @@ void text_box_base::set_selection(size_t start, int length)
 
 	// The text pos/size type differs in both signedness and size with the
 	// selection length. Such is life.
-	const int sel_start = std::min<size_t>(start, std::numeric_limits<int>::max());
-	const int sel_max_length = std::min<size_t>(text_size - start, std::numeric_limits<int>::max());
+	const int sel_start = std::min<std::size_t>(start, std::numeric_limits<int>::max());
+	const int sel_max_length = std::min<std::size_t>(text_size - start, std::numeric_limits<int>::max());
 
 	const bool backwards = length < 0;
 
@@ -311,7 +316,7 @@ void text_box_base::cursor_timer_callback()
 			cursor_alpha_ = 255;
 			return;
 		default:
-			if(!open_window_stack.empty() && get_window() != open_window_stack.back()) {
+			if(get_window() != open_window_stack.back()) {
 				cursor_alpha_ = 0;
 			} else {
 				cursor_alpha_ = (~cursor_alpha_) & 0xFF;
@@ -363,7 +368,7 @@ void text_box_base::handle_key_right_arrow(SDL_Keymod modifier, bool& handled)
 	DBG_GUI_E << LOG_SCOPE_HEADER << '\n';
 
 	handled = true;
-	const size_t offset = selection_start_ + 1 + selection_length_;
+	const std::size_t offset = selection_start_ + 1 + selection_length_;
 	if(offset <= text_.get_length()) {
 		set_cursor(offset, (modifier & KMOD_SHIFT) != 0);
 	}
@@ -429,7 +434,7 @@ void text_box_base::handle_key_delete(SDL_Keymod /*modifier*/, bool& handled)
 	fire(event::NOTIFY_MODIFIED, *this, nullptr);
 }
 
-void text_box_base::handle_commit(bool& handled, const utf8::string& unicode)
+void text_box_base::handle_commit(bool& handled, const std::string& unicode)
 {
 	DBG_GUI_E << LOG_SCOPE_HEADER << '\n';
 
@@ -451,11 +456,11 @@ void text_box_base::handle_commit(bool& handled, const utf8::string& unicode)
 /**
  * SDL_TEXTEDITING handler. See example at https://wiki.libsdl.org/Tutorials/TextInput
  */
-void text_box_base::handle_editing(bool& handled, const utf8::string& unicode, int32_t start, int32_t len)
+void text_box_base::handle_editing(bool& handled, const std::string& unicode, int32_t start, int32_t len)
 {
 	if(unicode.size() > 1 || unicode[0] != 0) {
 		handled = true;
-		size_t new_len = utf8::size(unicode);
+		std::size_t new_len = utf8::size(unicode);
 		if(!is_composing()) {
 			ime_composing_ = true;
 			delete_selection();
