@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2008 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2008 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -25,7 +26,7 @@
 #include "sound.hpp"
 #include "wml_exception.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 #define LOG_SCOPE_HEADER get_control_type() + " [" + id() + "] " + __func__
 #define LOG_HEADER LOG_SCOPE_HEADER + ':'
@@ -38,7 +39,7 @@ namespace gui2
 REGISTER_WIDGET(toggle_panel)
 
 toggle_panel::toggle_panel(const implementation::builder_toggle_panel& builder)
-	: panel(builder, get_control_type())
+	: panel(builder, type())
 	, state_(ENABLED)
 	, state_num_(0)
 	, retval_(retval::NONE)
@@ -46,34 +47,34 @@ toggle_panel::toggle_panel(const implementation::builder_toggle_panel& builder)
 	set_wants_mouse_left_double_click();
 
 	connect_signal<event::MOUSE_ENTER>(std::bind(
-			&toggle_panel::signal_handler_mouse_enter, this, _2, _3));
+			&toggle_panel::signal_handler_mouse_enter, this, std::placeholders::_2, std::placeholders::_3));
 	connect_signal<event::MOUSE_LEAVE>(std::bind(
-			&toggle_panel::signal_handler_mouse_leave, this, _2, _3));
+			&toggle_panel::signal_handler_mouse_leave, this, std::placeholders::_2, std::placeholders::_3));
 #if 0
 	connect_signal<event::LEFT_BUTTON_CLICK>(
 			std::bind(&toggle_panel::signal_handler_pre_left_button_click,
 						this,
-						_2),
+						std::placeholders::_2),
 			event::dispatcher::back_pre_child);
 #endif
 	connect_signal<event::LEFT_BUTTON_CLICK>(std::bind(
-			&toggle_panel::signal_handler_left_button_click, this, _2, _3));
+			&toggle_panel::signal_handler_left_button_click, this, std::placeholders::_2, std::placeholders::_3));
 	connect_signal<event::LEFT_BUTTON_CLICK>(
 			std::bind(&toggle_panel::signal_handler_left_button_click,
 						this,
-						_2,
-						_3),
+						std::placeholders::_2,
+						std::placeholders::_3),
 			event::dispatcher::back_post_child);
 	connect_signal<event::LEFT_BUTTON_DOUBLE_CLICK>(
 			std::bind(&toggle_panel::signal_handler_left_button_double_click,
 						this,
-						_2,
-						_3));
+						std::placeholders::_2,
+						std::placeholders::_3));
 	connect_signal<event::LEFT_BUTTON_DOUBLE_CLICK>(
 			std::bind(&toggle_panel::signal_handler_left_button_double_click,
 						this,
-						_2,
-						_3),
+						std::placeholders::_2,
+						std::placeholders::_3),
 			event::dispatcher::back_post_child);
 }
 
@@ -293,55 +294,6 @@ toggle_panel_definition::toggle_panel_definition(const config& cfg)
 	load_resolutions<resolution>(cfg);
 }
 
-/*WIKI
- * @page = GUIWidgetDefinitionWML
- * @order = 1_toggle_panel
- *
- * == Toggle panel ==
- *
- * @begin{parent}{name="gui/"}
- * @begin{tag}{name="oggle_panel_definition"}{min=0}{max=-1}{super="generic/widget_definition"}
- * The definition of a toggle panel. A toggle panel is like a toggle button, but
- * instead of being a button it's a panel. This means it can hold multiple child
- * items.
- *
- * @begin{tag}{name="resolution"}{min=0}{max=-1}{super=generic/widget_definition/resolution}
- * The resolution for a toggle panel also contains the following keys:
- * @begin{table}{config}
- *     top_border & unsigned & 0 &     The size which isn't used for the client
- *                                   area. $
- *     bottom_border & unsigned & 0 &  The size which isn't used for the client
- *                                   area. $
- *     left_border & unsigned & 0 &    The size which isn't used for the client
- *                                   area. $
- *     right_border & unsigned & 0 &   The size which isn't used for the client
- *                                   area. $
- * @end{table}
- *
- * The following states exist:
- * * state_enabled, the panel is enabled and not selected.
- * * state_disabled, the panel is disabled and not selected.
- * * state_focused, the mouse is over the panel and not selected.
- *
- * * state_enabled_selected, the panel is enabled and selected.
- * * state_disabled_selected, the panel is disabled and selected.
- * * state_focused_selected, the mouse is over the panel and selected.
- * @begin{tag}{name="state_enabled"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_enabled"}
- * @begin{tag}{name="state_disabled"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_disabled"}
- * @begin{tag}{name="state_focused"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_focused"}
- * @begin{tag}{name="state_enabled_selected"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_enabled_selected"}
- * @begin{tag}{name="state_disabled_selected"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_disabled_selected"}
- * @begin{tag}{name="state_focused_selected"}{min=0}{max=1}{super="generic/state"}
- * @end{tag}{name="state_focused_selected"}
- * @end{tag}{name="resolution"}
- * @end{tag}{name="oggle_panel_definition"}
- * @end{parent}{name="gui/"}
- */
 toggle_panel_definition::resolution::resolution(const config& cfg)
 	: resolution_definition(cfg)
 	, top_border(cfg["top_border"])
@@ -360,34 +312,6 @@ toggle_panel_definition::resolution::resolution(const config& cfg)
 
 // }---------- BUILDER -----------{
 
-/*WIKI
- * @page = GUIWidgetInstanceWML
- * @order = 2_toggle_panel
- * @begin{parent}{name="gui/window/resolution/grid/row/column/"}
- * @begin{tag}{name="toggle_panel"}{min=0}{max=-1}{super="generic/widget_instance"}
- * == Toggle panel ==
- *
- * A toggle panel is an item which can hold other items. The difference between
- * a grid and a panel is that it's possible to define how a panel looks. A grid
- * in an invisible container to just hold the items. The toggle panel is a
- * combination of the panel and a toggle button, it allows a toggle button with
- * its own grid.
- *
- * @begin{table}{config}
- *     grid & grid & &                 Defines the grid with the widgets to
- *                                     place on the panel. $
- *     return_value_id & string & "" & The return value id, see
- *                                     [[GUIToolkitWML#Button]] for more
- *                                     information. $
- *     return_value & int & 0 &        The return value, see
- *                                     [[GUIToolkitWML#Button]] for more
- *                                     information. $
- * @end{table}
- * @allow{link}{name="gui/window/resolution/grid"}
- * @end{tag}{name="toggle_panel"}
- * @end{parent}{name="gui/window/resolution/grid/row/column/"}
- */
-
 namespace implementation
 {
 
@@ -413,7 +337,7 @@ widget* builder_toggle_panel::build() const
 	DBG_GUI_G << "Window builder: placed toggle panel '" << id
 			  << "' with definition '" << definition << "'.\n";
 
-	widget->init_grid(grid);
+	widget->init_grid(*grid);
 	return widget;
 }
 
