@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -36,6 +37,26 @@ namespace implementation {
 
 class tree_view_node;
 
+/**
+ * @ingroup GUIWidgetWML
+ *
+ * A tree view is a control that holds several items of the same or different types.
+ * The items shown are called tree view nodes and when a node has children, these can be shown or hidden.
+ * Nodes that contain children need to provide a clickable button in order to fold or unfold the children.
+ *
+ * The following states exist:
+ * * state_enabled - the listbox is enabled.
+ * * state_disabled - the listbox is disabled.
+ * List with the tree view specific variables:
+ * Key                      |Type                                            |Default     |Description
+ * -------------------------|------------------------------------------------|------------|-------------
+ * vertical_scrollbar_mode  | @ref guivartype_scrollbar_mode "scrollbar_mode"|initial_auto|Determines whether or not to show the scrollbar.
+ * horizontal_scrollbar_mode| @ref guivartype_scrollbar_mode "scrollbar_mode"|initial_auto|Determines whether or not to show the scrollbar.
+ * indention_step_size      | @ref guivartype_unsigned "unsigned"            |0           |The number of pixels every level of nodes is indented from the previous level.
+ * node                     | @ref guivartype_unsigned "unsigned"            |mandatory   |The tree view can contain multiple node sections. This part needs more documentation.
+ * id                       | @ref guivartype_unsigned "unsigned"            |""          |.
+ * return_value_id          | @ref guivartype_unsigned "unsigned"            |""          |.
+ */
 class tree_view : public scrollbar_container
 {
 	friend struct implementation::builder_tree_view;
@@ -68,7 +89,7 @@ public:
 	 * @returns         A pair consisting of a smart pointer managing the removed
 	 *                  node, and its position before removal.
 	 */
-	std::pair<tree_view_node::ptr_t, int> remove_node(tree_view_node* node);
+	std::pair<std::shared_ptr<tree_view_node>, int> remove_node(tree_view_node* node);
 
 	void clear();
 
@@ -102,11 +123,6 @@ public:
 		return selected_item_;
 	}
 
-	void set_selection_change_callback(std::function<void(widget&)> callback)
-	{
-		selection_change_callback_ = callback;
-	}
-
 	const std::vector<node_definition>& get_node_definitions() const
 	{
 		return node_definitions_;
@@ -143,8 +159,6 @@ private:
 
 	tree_view_node* selected_item_;
 
-	std::function<void(widget&)> selection_change_callback_;
-
 	/**
 	 * Resizes the content.
 	 *
@@ -159,10 +173,12 @@ private:
 	 *                            * negative values reduce height.
 	 *                            * zero leave height as is.
 	 *                            * positive values increase height.
+	 * @param width_modification_pos
+	 * @param height_modification_pos
 	 */
 	void resize_content(const int width_modification,
 						const int height_modification,
-						const int width__modification_pos = -1,
+						const int width_modification_pos = -1,
 						const int height_modification_pos = -1);
 
 	/** Layouts the children if needed. */
@@ -216,7 +232,7 @@ struct builder_tree_view : public builder_styled_widget
 
 	using builder_styled_widget::build;
 
-	widget* build() const;
+	virtual widget* build() const override;
 
 	scrollbar_container::scrollbar_mode vertical_scrollbar_mode;
 	scrollbar_container::scrollbar_mode horizontal_scrollbar_mode;
