@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -24,17 +25,14 @@
 #include "gui/widgets/toggle_panel.hpp"
 #include "gui/widgets/tree_view.hpp"
 #include "sdl/rect.hpp"
-
-#include "utils/functional.hpp"
+#include <functional>
 
 #define LOG_SCOPE_HEADER get_control_type() + " [" + get_tree_view().id() + "] " + __func__
 #define LOG_HEADER LOG_SCOPE_HEADER + ':'
 
 namespace gui2
 {
-
-tree_view_node::tree_view_node(
-		const std::string& id,
+tree_view_node::tree_view_node(const std::string& id,
 		tree_view_node* parent_node,
 		tree_view& parent_tree_view,
 		const std::map<std::string /* widget id */, string_map>& data)
@@ -77,11 +75,12 @@ tree_view_node::tree_view_node(
 		if(toggle_) {
 			toggle_widget->set_visible(widget::visibility::hidden);
 
-			toggle_widget->connect_signal<event::LEFT_BUTTON_CLICK>(std::bind(
-					&tree_view_node::signal_handler_left_button_click, this, _2));
-			toggle_widget->connect_signal<event::LEFT_BUTTON_CLICK>(std::bind(
-					&tree_view_node::signal_handler_left_button_click, this, _2),
-					event::dispatcher::back_post_child);
+			toggle_widget->connect_signal<event::LEFT_BUTTON_CLICK>(
+				std::bind(&tree_view_node::signal_handler_left_button_click, this, std::placeholders::_2));
+
+			toggle_widget->connect_signal<event::LEFT_BUTTON_CLICK>(
+				std::bind(&tree_view_node::signal_handler_left_button_click, this, std::placeholders::_2),
+				event::dispatcher::back_post_child);
 
 			if(unfolded_) {
 				toggle_->set_value(1);
@@ -92,12 +91,13 @@ tree_view_node::tree_view_node(
 		label_ = dynamic_cast<selectable_item*>(label_widget);
 
 		if(label_) {
-			label_widget->connect_signal<event::LEFT_BUTTON_CLICK>(std::bind(
-					&tree_view_node::signal_handler_label_left_button_click, this, _2, _3, _4),
-					event::dispatcher::front_child);
-			label_widget->connect_signal<event::LEFT_BUTTON_CLICK>(std::bind(
-					&tree_view_node::signal_handler_label_left_button_click, this, _2, _3, _4),
-					event::dispatcher::front_pre_child);
+			label_widget->connect_signal<event::LEFT_BUTTON_CLICK>(
+				std::bind(&tree_view_node::signal_handler_label_left_button_click, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_4),
+				event::dispatcher::front_child);
+
+			label_widget->connect_signal<event::LEFT_BUTTON_CLICK>(
+				std::bind(&tree_view_node::signal_handler_label_left_button_click, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_4),
+				event::dispatcher::front_pre_child);
 
 			if(!get_tree_view().selected_item_) {
 				get_tree_view().selected_item_ = this;
@@ -127,11 +127,11 @@ void tree_view_node::clear_before_destruct()
 	}
 }
 
-tree_view_node& tree_view_node::add_child_impl(ptr_t&& new_node, const int index)
+tree_view_node& tree_view_node::add_child_impl(std::shared_ptr<tree_view_node>&& new_node, const int index)
 {
 	auto itor = children_.end();
 
-	if(static_cast<size_t>(index) < children_.size()) {
+	if(static_cast<std::size_t>(index) < children_.size()) {
 		itor = children_.begin() + index;
 	}
 
@@ -187,6 +187,82 @@ tree_view_node& tree_view_node::add_child_impl(ptr_t&& new_node, const int index
 	return node;
 }
 
+std::map<std::string, std::shared_ptr<gui2::tree_view_node>> tree_view_node::replace_children(const std::string& id, const std::map<std::string, std::map<std::string /* widget id */, string_map>>& data)
+{
+	std::map<std::string, std::shared_ptr<gui2::tree_view_node>> nodes;
+	clear();
+
+	if(data.size() == 0)
+	{
+		return nodes;
+	}
+
+	int width_modification = 0;
+
+	for(const auto& d : data)
+	{
+		std::shared_ptr<gui2::tree_view_node> new_node = std::make_shared<tree_view_node>(id, this, get_tree_view(), d.second);
+		std::shared_ptr<gui2::tree_view_node> node = *children_.insert(children_.end(), std::move(new_node));
+
+		// NOTE: we currently don't support moving nodes between different trees, so this
+		// just ensures that wasn't tried. Remove this if we implement support for that.
+		assert(node->tree_view_ == tree_view_);
+
+		// Safety check. Might only fail if someone accidentally removed the parent_node_ setter in add_child().
+		assert(node->parent_node_ == this);
+
+		nodes[d.first] = node;
+
+		if(is_folded()) {
+			continue;
+		}
+
+		assert(get_tree_view().content_grid());
+		const point current_size = get_tree_view().content_grid()->get_size();
+
+		// Calculate width modification.
+		// This increases tree width if the width of the new node is greater than the current width.
+		int best_size = node->get_best_size().x;
+		best_size += get_indentation_level() * get_tree_view().indentation_step_size_;
+
+		int new_width = best_size > current_size.x
+			? best_size - current_size.x
+			: 0;
+
+		if(new_width > width_modification)
+		{
+			width_modification = new_width;
+		}
+	}
+
+	if(is_folded()) {
+		return nodes;
+	}
+
+	// Calculate height modification.
+	// For this, we only increase height if the best size of the tree (that is, the size with the new node)
+	// is larger than its current size. This prevents the scrollbar being reserved even when there's obviously
+	// enough visual space.
+
+	// Throw away cached best size to force a recalculation.
+	get_tree_view().layout_initialize(false);
+
+	const point current_size = get_tree_view().content_grid()->get_size();
+	const point tree_best_size = get_tree_view().get_best_size();
+
+	const int height_modification = tree_best_size.y > current_size.y && get_tree_view().layout_size() == point()
+		? tree_best_size.y - current_size.y
+		: 0;
+
+	assert(height_modification >= 0);
+
+	// Request new size.
+	auto& last_node = children_.at(children_.size()-1);
+	get_tree_view().resize_content(width_modification, height_modification, -1, last_node->calculate_ypos());
+
+	return nodes;
+}
+
 unsigned tree_view_node::get_indentation_level() const
 {
 	unsigned level = 0;
@@ -281,8 +357,7 @@ void tree_view_node::clear()
 	int height_reduction = 0;
 
 	if(!is_folded()) {
-		for(const auto & node : children_)
-		{
+		for(const auto& node : children_) {
 			height_reduction += node->get_current_size().y;
 		}
 	}
@@ -293,31 +368,29 @@ void tree_view_node::clear()
 		return;
 	}
 
-	get_tree_view().resize_content(0, -height_reduction,  -1, calculate_ypos());
+	get_tree_view().resize_content(0, -height_reduction, -1, calculate_ypos());
 }
 
 struct tree_view_node_implementation
 {
 private:
-	template <class W, class It>
-	static W* find_at_aux(It begin,
-						  It end,
-						  const point& coordinate,
-						  const bool must_be_active)
+	template<class W, class It>
+	static W* find_at_aux(It begin, It end, const point& coordinate, const bool must_be_active)
 	{
 		for(It it = begin; it != end; ++it) {
 			if(W* widget = (*it)->find_at(coordinate, must_be_active)) {
 				return widget;
 			}
 		}
+
 		return nullptr;
 	}
 
 public:
-	template <class W>
+	template<class W>
 	static W* find_at(utils::const_clone_ref<tree_view_node, W> tree_view_node,
-					  const point& coordinate,
-					  const bool must_be_active)
+			const point& coordinate,
+			const bool must_be_active)
 	{
 		if(W* widget = tree_view_node.grid_.find_at(coordinate, must_be_active)) {
 			return widget;
@@ -327,10 +400,8 @@ public:
 			return nullptr;
 		}
 
-		return find_at_aux<W>(tree_view_node.children_.begin(),
-							  tree_view_node.children_.end(),
-							  coordinate,
-							  must_be_active);
+		return find_at_aux<W>(
+			tree_view_node.children_.begin(), tree_view_node.children_.end(), coordinate, must_be_active);
 	}
 };
 
@@ -444,6 +515,7 @@ point tree_view_node::get_folded_size() const
 	if(get_indentation_level() > 1) {
 		size.x += (get_indentation_level() - 1) * get_tree_view().indentation_step_size_;
 	}
+
 	return size;
 }
 
@@ -468,8 +540,7 @@ point tree_view_node::get_unfolded_size() const
 	return size;
 }
 
-point tree_view_node::calculate_best_size(const int indentation_level,
-											const unsigned indentation_step_size) const
+point tree_view_node::calculate_best_size(const int indentation_level, const unsigned indentation_step_size) const
 {
 	log_scope2(log_gui_layout, LOG_SCOPE_HEADER);
 
@@ -515,9 +586,7 @@ void tree_view_node::place(const point& origin, const point& size)
 	get_tree_view().layout_children(true);
 }
 
-unsigned tree_view_node::place(const unsigned indentation_step_size,
-								point origin,
-								unsigned width)
+unsigned tree_view_node::place(const unsigned indentation_step_size, point origin, unsigned width)
 {
 	log_scope2(log_gui_layout, LOG_SCOPE_HEADER);
 	DBG_GUI_L << LOG_HEADER << " origin " << origin << ".\n";
@@ -542,7 +611,7 @@ unsigned tree_view_node::place(const unsigned indentation_step_size,
 	}
 
 	DBG_GUI_L << LOG_HEADER << " set children.\n";
-	for(auto & node : children_) {
+	for(auto& node : children_) {
 		origin.y += node->place(indentation_step_size, origin, width);
 	}
 
@@ -564,7 +633,7 @@ void tree_view_node::set_visible_rectangle(const SDL_Rect& rectangle)
 		return;
 	}
 
-	for(auto & node : children_) {
+	for(auto& node : children_) {
 		node->set_visible_rectangle(rectangle);
 	}
 }
@@ -579,7 +648,7 @@ void tree_view_node::impl_draw_children(surface& frame_buffer,
 		return;
 	}
 
-	for(auto & node : children_) {
+	for(auto& node : children_) {
 		node->impl_draw_children(frame_buffer, x_offset, y_offset);
 	}
 }
@@ -602,11 +671,10 @@ void tree_view_node::signal_handler_left_button_click(const event::ui_event even
 	unfolded_ = unfolded_new;
 	is_folded() ? fold_internal() : unfold_internal();
 
-	fire(event::NOTIFY_MODIFIED, *this, nullptr);
+	get_tree_view().fire(event::NOTIFY_MODIFIED, get_tree_view(), nullptr);
 }
 
-void tree_view_node::signal_handler_label_left_button_click(
-		const event::ui_event event, bool& handled, bool& halt)
+void tree_view_node::signal_handler_label_left_button_click(const event::ui_event event, bool& handled, bool& halt)
 {
 	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
 
@@ -627,14 +695,10 @@ void tree_view_node::signal_handler_label_left_button_click(
 
 	get_tree_view().selected_item_ = this;
 
-	if(get_tree_view().selection_change_callback_) {
-		get_tree_view().selection_change_callback_(get_tree_view());
-	}
+	get_tree_view().fire(event::NOTIFY_MODIFIED, get_tree_view(), nullptr);
 }
 
-void tree_view_node::init_grid(
-		grid* g,
-		const std::map<std::string /* widget id */, string_map>& data)
+void tree_view_node::init_grid(grid* g, const std::map<std::string /* widget id */, string_map>& data)
 {
 	assert(g);
 
@@ -675,7 +739,7 @@ const std::string& tree_view_node::get_control_type() const
 
 tree_view_node& tree_view_node::get_child_at(int index)
 {
-	assert(static_cast<size_t>(index) < children_.size());
+	assert(static_cast<std::size_t>(index) < children_.size());
 	return *children_[index];
 }
 
@@ -686,7 +750,7 @@ std::vector<int> tree_view_node::describe_path()
 	}
 
 	std::vector<int> res = parent_node_->describe_path();
-	for(size_t i = 0; i < parent_node_->count_children(); ++i) {
+	for(std::size_t i = 0; i < parent_node_->count_children(); ++i) {
 		if(parent_node_->children_[i].get() == this) {
 			res.push_back(i);
 			return res;
@@ -694,7 +758,7 @@ std::vector<int> tree_view_node::describe_path()
 	}
 
 	assert(!"tree_view_node was not found in parent nodes children");
-	throw "assertion ignored"; //To silence 'no return value in this codepath' warning.
+	throw "assertion ignored"; // To silence 'no return value in this codepath' warning.
 }
 
 int tree_view_node::calculate_ypos()
@@ -730,7 +794,7 @@ tree_view_node* tree_view_node::get_node_above()
 	assert(!is_root_node());
 
 	tree_view_node* cur = nullptr;
-	for(size_t i = 0; i < parent_node_->count_children(); ++i) {
+	for(std::size_t i = 0; i < parent_node_->count_children(); ++i) {
 		if(parent_node_->children_[i].get() == this) {
 			if(i == 0) {
 				return parent_node_->is_root_node() ? nullptr : parent_node_;
@@ -745,8 +809,11 @@ tree_view_node* tree_view_node::get_node_above()
 		cur = &cur->get_child_at(cur->count_children() - 1);
 	}
 
-	if (!cur)
-		throw std::domain_error("tree_view_node::get_node_above(): Cannot determine which node is this line, or which node is the line above this one, if any.");
+	if(!cur) {
+		throw std::domain_error(
+			"tree_view_node::get_node_above(): Cannot determine which node is this line, or which "
+			"node is the line above this one, if any.");
+	}
 
 	return cur;
 }
@@ -762,13 +829,14 @@ tree_view_node* tree_view_node::get_node_below()
 	while(cur->parent_node_ != nullptr) {
 		tree_view_node& parent = *cur->parent_node_;
 
-		for(size_t i = 0; i < parent.count_children(); ++i) {
+		for(std::size_t i = 0; i < parent.count_children(); ++i) {
 			if(parent.children_[i].get() == cur) {
 				if(i < parent.count_children() - 1) {
 					return parent.children_[i + 1].get();
 				} else {
 					cur = &parent;
 				}
+
 				break;
 			}
 		}
@@ -783,6 +851,7 @@ tree_view_node* tree_view_node::get_selectable_node_above()
 	do {
 		above = above->get_node_above();
 	} while(above != nullptr && above->label_ == nullptr);
+
 	return above;
 }
 
@@ -792,6 +861,7 @@ tree_view_node* tree_view_node::get_selectable_node_below()
 	do {
 		below = below->get_node_below();
 	} while(below != nullptr && below->label_ == nullptr);
+
 	return below;
 }
 
@@ -811,11 +881,11 @@ void tree_view_node::select_node(bool expand_parents)
 	if(get_tree_view().selected_item_ && get_tree_view().selected_item_->label_) {
 		get_tree_view().selected_item_->label_->set_value(false);
 	}
+
 	get_tree_view().selected_item_ = this;
 
-	if(get_tree_view().selection_change_callback_) {
-		get_tree_view().selection_change_callback_(get_tree_view());
-	}
+	get_tree_view().fire(event::NOTIFY_MODIFIED, get_tree_view(), nullptr);
+
 	label_->set_value_bool(true);
 }
 
@@ -826,7 +896,7 @@ void tree_view_node::layout_initialize(const bool full_initialization)
 	grid_.layout_initialize(full_initialization);
 
 	// Clear child caches.
-	for(auto & child : children_) {
+	for(auto& child : children_) {
 		child->layout_initialize(full_initialization);
 	}
 }
