@@ -1,21 +1,23 @@
 /*
-   Copyright (C) 2010 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
 #include "gui/widgets/widget.hpp"
 #include "gui/widgets/grid.hpp"
+
 #include <memory>
 
 namespace gui2
@@ -34,8 +36,7 @@ class tree_view_node : public widget
 	friend class tree_view;
 
 public:
-	using ptr_t = std::unique_ptr<tree_view_node>;
-	using node_children_vector = std::vector<ptr_t>;
+	using node_children_vector = std::vector<std::shared_ptr<tree_view_node>>;
 
 	bool operator==(const tree_view_node& node)
 	{
@@ -69,10 +70,22 @@ public:
 			const std::map<std::string /* widget id */, string_map>& data,
 			const int index = -1)
 	{
-		ptr_t new_node(new tree_view_node(id, this, get_tree_view(), data));
-		return add_child_impl(std::move(new_node), index);
+		return add_child_impl(std::make_shared<tree_view_node>(id, this, get_tree_view(), data), index);
 	}
 
+	/**
+	 * Replaces all children of this tree with new children.
+	 * The motivation here is to provide a way to add multiple children without calculating the trees size for each child added.
+	 * This is a waste of time since the results of that resizing will be immediately thrown out except for the final child added.
+	 *
+	 * @param id The id of the node definition to use for the new nodes.
+	 * @param data data.first is a unique identifying value to be associated to the respective tree_view_node that's returned.
+	 * 			   data.second is the data to provide to the tree_node_view's constructor.
+	 * @return return_value.first is data.first
+	 * 		   return_value.second is the tree_view_node created from data.second
+	 */
+	std::map<std::string, std::shared_ptr<gui2::tree_view_node>> replace_children(const std::string& id, const std::map<std::string, std::map<std::string /* widget id */, string_map>>& data);
+
 	/**
 	 * Adds a previously-constructed node as a child of this node at the given position.
 	 *
@@ -80,7 +93,7 @@ public:
 	 * @param index               The item before which to add the new item,
 	 *                            0 == begin, -1 == end.
 	 */
-	tree_view_node& add_child(ptr_t new_node, const int index = -1)
+	tree_view_node& add_child(std::shared_ptr<tree_view_node> new_node, const int index = -1)
 	{
 		new_node->parent_node_ = this;
 		return add_child_impl(std::move(new_node), index);
@@ -109,7 +122,7 @@ public:
 
 private:
 	/** Implementation detail for @ref add_child. */
-	tree_view_node& add_child_impl(ptr_t&& new_node, const int index);
+	tree_view_node& add_child_impl(std::shared_ptr<tree_view_node>&& new_node, const int index);
 
 public:
 	/**
@@ -119,8 +132,7 @@ public:
 	 * node. This node has no parent node and some other special features so
 	 * several code paths need to check whether they are the parent node.
 	 *
-	 * This also returns true for a detecthed node returned with @ref tree_view::
-	 * remove_node.
+	 * This also returns true for a detecthed node returned with @ref tree_view::remove_node.
 	 */
 	bool is_root_node() const
 	{
@@ -170,6 +182,12 @@ public:
 		return children_;
 	}
 
+	node_children_vector& siblings()
+	{
+		assert(!is_root_node());
+		return parent_node().children();
+	}
+
 	/** See @ref widget::find_at. */
 	virtual widget* find_at(const point& coordinate,
 							 const bool must_be_active) override;
@@ -188,7 +206,7 @@ public:
 	/**
 	 * The number of children in this widget.
 	 */
-	size_t count_children() const
+	std::size_t count_children() const
 	{
 		return children_.size();
 	}
