@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2016 - 2018 The Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2016 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -38,12 +39,13 @@
 #include "resources.hpp"
 #include "team.hpp"
 #include "terrain/movement.hpp"
+#include "terrain/type_data.hpp"
 #include "units/attack_type.hpp"
 #include "units/types.hpp"
 #include "units/helper.hpp"
 #include "units/unit.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 namespace gui2
 {
@@ -122,10 +124,10 @@ static inline std::string get_hp_tooltip(const utils::string_map& res, const std
 		const int res_def = 100 - get(resist.first, false);
 
 		if(res_att == res_def) {
-			line << "<span color='" << unit_helper::resistance_color(res_def) << "'>" << utils::signed_percent(res_def) << "</span>";
+			line << "<span color='" << unit_helper::resistance_color(res_def) << "'>\t" << utils::signed_percent(res_def) << "</span>";
 		} else {
-			line << "<span color='" << unit_helper::resistance_color(res_att) << "'>" << utils::signed_percent(res_att) << "</span>" << "/"
-			     << "<span color='" << unit_helper::resistance_color(res_def) << "'>" << utils::signed_percent(res_def) << "</span>";
+			line << "<span color='" << unit_helper::resistance_color(res_att) << "'>\t" << utils::signed_percent(res_att) << "</span>" << "/"
+			     << "<span color='" << unit_helper::resistance_color(res_def) << "'>"   << utils::signed_percent(res_def) << "</span>";
 			att_def_diff = true;
 		}
 
@@ -150,7 +152,7 @@ static inline std::string get_mp_tooltip(int total_movement, std::function<int (
 	std::ostringstream tooltip;
 	tooltip << "<big>" << _("Movement Costs:") << "</big>";
 
-	ter_data_cache tdata = help::load_terrain_types_data();
+	std::shared_ptr<terrain_type_data> tdata = help::load_terrain_types_data();
 
 	if(!tdata) {
 		return "";
@@ -162,13 +164,11 @@ static inline std::string get_mp_tooltip(int total_movement, std::function<int (
 		}
 
 		const terrain_type& info = tdata->get_terrain_info(terrain);
-		if(info.union_type().size() == 1 && info.union_type()[0] == info.number() && info.is_nonnull()) {
+		if(info.is_indivisible() && info.is_nonnull()) {
 			terrain_moves.emplace(info.name(), get(terrain));
 		}
 	}
 
-
-
 	for(const terrain_movement& tm: terrain_moves)
 	{
 		tooltip << '\n' << font::unicode_bullet << " " << tm.name << ": ";
@@ -224,10 +224,14 @@ void unit_preview_pane::print_attack_details(T attacks, tree_view_node& parent_n
 		const bool range_png_exists = ::image::locator(range_png).file_exists();
 		const bool type_png_exists = ::image::locator(type_png).file_exists();
 
+		const t_string& range = string_table["range_" + a.range()];
+		const t_string& type = string_table["type_" + a.type()];
+
 		const std::string label = (formatter()
 			 << font::span_color(font::unit_type_color)
 			 << a.damage() << font::weapon_numbers_sep << a.num_attacks()
 			 << " " << a.name() << "</span>").str();
+
 		auto& subsection = header_node.add_child(
 			"item_image",
 			{
@@ -237,11 +241,18 @@ void unit_preview_pane::print_attack_details(T attacks, tree_view_node& parent_n
 			}
 		);
 
+		find_widget<styled_widget>(&subsection, "image_range", true).set_tooltip(range);
+		find_widget<styled_widget>(&subsection, "image_type", true).set_tooltip(type);
+
 		if(!range_png_exists || !type_png_exists) {
 			add_name_tree_node(
 				subsection,
 				"item",
-				(formatter() << font::span_color(font::weapon_details_color) << string_table["range_" + a.range()] << font::weapon_details_sep << string_table["type_" + a.type()] << "</span>").str()
+				(formatter()
+				 << font::span_color(font::weapon_details_color)
+				 << range << font::weapon_details_sep
+				 << type << "</span>"
+				 ).str()
 			);
 		}
 
@@ -270,7 +281,7 @@ void unit_preview_pane::set_displayed_type(const unit_type& type)
 				 + ")";
 		}
 
-		mods += "~SCALE_INTO_SHARP(144,144)" + image_mods_;
+		mods += "~XBRZ(2)~SCALE_INTO_SHARP(144,144)" + image_mods_;
 
 		icon_type_->set_label((type.icon().empty() ? type.image() : type.icon()) + mods);
 	}
@@ -284,6 +295,7 @@ void unit_preview_pane::set_displayed_type(const unit_type& type)
 		std::string l_str = VGETTEXT("Lvl $lvl", {{"lvl", std::to_string(type.level())}});
 
 		label_level_->set_label("<b>" + l_str + "</b>");
+		label_level_->set_tooltip(unit_helper::unit_level_tooltip(type));
 		label_level_->set_use_markup(true);
 	}
 
@@ -331,12 +343,12 @@ void unit_preview_pane::set_displayed_type(const unit_type& type)
 		tree_details_->clear();
 		tree_details_->add_node("hp_xp_mp", {
 			{ "hp",{
-				{ "label", (formatter() << "<small>" << "<span color='#21e100'>" << "<b>" << _("HP: ") << "</b>" << type.hitpoints() << "</span>" << " | </small>").str() },
+				{ "label", (formatter() << "<small>" << font::span_color(unit::hp_color_max()) << "<b>" << _("HP: ") << "</b>" << type.hitpoints() << "</span>" << " | </small>").str() },
 				{ "use_markup", "true" },
 				{ "tooltip", get_hp_tooltip(type.movement_type().get_resistances().damage_table(), [&type](const std::string& dt, bool is_attacker) { return type.resistance_against(dt, is_attacker); }) }
 			} },
 			{ "xp",{
-				{ "label", (formatter() << "<small>" << "<span color='#00a0e1'>" << "<b>" << _("XP: ") << "</b>" << type.experience_needed() << "</span>" << " | </small>").str() },
+				{ "label", (formatter() << "<small>" << font::span_color(unit::xp_color(100, type.can_advance(), true)) << "<b>" << _("XP: ") << "</b>" << type.experience_needed() << "</span>" << " | </small>").str() },
 				{ "use_markup", "true" },
 				{ "tooltip", (formatter() << _("Experience Modifier: ") << unit_experience_accelerator::get_acceleration() << '%').str() }
 			} },
@@ -414,7 +426,7 @@ void unit_preview_pane::set_displayed_unit(const unit& u)
 			mods += "~BLIT(" + overlay + ")";
 		}
 
-		mods += "~SCALE_INTO_SHARP(144,144)" + image_mods_;
+		mods += "~XBRZ(2)~SCALE_INTO_SHARP(144,144)" + image_mods_;
 
 		icon_type_->set_label(u.absolute_image() + mods);
 	}
@@ -435,6 +447,7 @@ void unit_preview_pane::set_displayed_unit(const unit& u)
 		std::string l_str = VGETTEXT("Lvl $lvl", {{"lvl", std::to_string(u.level())}});
 
 		label_level_->set_label("<b>" + l_str + "</b>");
+		label_level_->set_tooltip(unit_helper::unit_level_tooltip(u));
 		label_level_->set_use_markup(true);
 	}
 
@@ -514,7 +527,7 @@ void unit_preview_pane::set_displayed_unit(const unit& u)
 			);
 
 			assert(u.trait_names().size() == u.trait_descriptions().size());
-			for (size_t i = 0; i < u.trait_names().size(); ++i) {
+			for (std::size_t i = 0; i < u.trait_names().size(); ++i) {
 				add_name_tree_node(
 					header_node,
 					"item",
@@ -619,7 +632,7 @@ widget* builder_unit_preview_pane::build() const
 	const auto conf = widget->cast_config_to<unit_preview_pane_definition>();
 	assert(conf);
 
-	widget->init_grid(conf->grid);
+	widget->init_grid(*conf->grid);
 	widget->finalize_setup();
 	widget->set_image_mods(image_mods_);
 
