@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2007 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2007 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -28,7 +29,8 @@ namespace gui2
 /***** ***** ***** Constructor and destructor. ***** ***** *****/
 
 widget::widget()
-	: id_("")
+	: enable_lua_ptr<widget>(this)
+	, id_("")
 	, parent_(nullptr)
 	, x_(-1)
 	, y_(-1)
@@ -50,7 +52,8 @@ widget::widget()
 }
 
 widget::widget(const builder_widget& builder)
-	: id_(builder.id)
+	: enable_lua_ptr<widget>(this)
+	, id_(builder.id)
 	, parent_(nullptr)
 	, x_(-1)
 	, y_(-1)
