@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2007 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2007 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -17,6 +18,8 @@
 #include "gui/core/event/dispatcher.hpp"
 #include "sdl/point.hpp"
 #include "gui/widgets/event_executor.hpp"
+#include "scripting/lua_ptr.hpp"
+
 #include "color.hpp"
 
 #include <string>
@@ -44,7 +47,7 @@ class walker_base;
  * info needed for a real widget and some pure abstract functions which need to
  * be implemented by classes deriving from this class.
  */
-class widget : public event_executor, public event::dispatcher
+class widget : public event_executor, public event::dispatcher, public enable_lua_ptr<widget>
 {
 	friend class debug_layout_graph;
 	friend class window; // needed for modifying the layout_size.
@@ -104,7 +107,7 @@ public:
 		/**
 		 * The widget is fully visible.
 		 *
-		 * The widget should be drawn if @ref dirty_ is @c true. The entire
+		 * The widget should be drawn if dirty_ is @c true. The entire
 		 * widget's rectangle should be redrawn.
 		 */
 		full,
@@ -112,7 +115,7 @@ public:
 		/**
 		 * The widget is partly visible.
 		 *
-		 * The should be drawn if @ref dirty_ is @c true. The rectangle to
+		 * The should be drawn if dirty_ is @c true. The rectangle to
 		 * redraw in determined by @ref clipping_rectangle_
 		 */
 		partly,
@@ -120,7 +123,7 @@ public:
 		/**
 		 * The widget is not visible.
 		 *
-		 * The widget should not be drawn if @ref dirty_ is @c true.
+		 * The widget should not be drawn if dirty_ is @c true.
 		 */
 		none
 	};
@@ -236,7 +239,7 @@ public:
 	 *
 	 * @param full_initialization For widgets with scrollbars it hides them
 	 *                            unless the mode is
-	 *                            @ref scrollbar_mode::ALWAYS_VISIBLE. For
+	 *                            scrollbar_mode::ALWAYS_VISIBLE. For
 	 *                            other widgets this flag is a @em NOP.
 	 */
 	virtual void layout_initialize(const bool full_initialization);
@@ -302,7 +305,6 @@ public:
 	 * size as determined during the layout phase.
 	 *
 	 * @returns                      The best size for the widget.
-	 * @retval 0,0                   The best size is 0,0.
 	 */
 	point get_best_size() const;
 
@@ -315,7 +317,6 @@ private:
 	 * their children since it is meant to update itself.
 	 *
 	 * @returns                      The best size for the widget.
-	 * @retval 0,0                   The best size is 0,0.
 	 */
 	virtual point calculate_best_size() const = 0;
 
