@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2007 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2007 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -61,7 +62,7 @@
 #include "wml_exception.hpp"
 #include "sdl/userevent.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 #include <algorithm>
 #include <iterator>
@@ -98,7 +99,7 @@ public:
 
 	using builder_styled_widget::build;
 
-	widget* build() const
+	virtual widget* build() const override
 	{
 		return nullptr;
 	}
@@ -271,12 +272,11 @@ window* manager::get_window(const unsigned id)
 
 } // namespace
 
-window::window(const builder_window::window_resolution* definition)
-	: panel(implementation::builder_window(::config {"definition", definition->definition}), type())
-	, cursor::setter(cursor::NORMAL)
+window::window(const builder_window::window_resolution& definition)
+	: panel(implementation::builder_window(::config {"definition", definition.definition}), type())
 	, video_(CVideo::get_singleton())
-	, status_(NEW)
-	, show_mode_(none)
+	, status_(status::NEW)
+	, show_mode_(show_mode::none)
 	, retval_(retval::NONE)
 	, owner_(nullptr)
 	, need_layout_(true)
@@ -286,19 +286,19 @@ window::window(const builder_window::window_resolution* definition)
 	, restore_(true)
 	, is_toplevel_(!is_in_dialog())
 	, restorer_()
-	, automatic_placement_(definition->automatic_placement)
-	, horizontal_placement_(definition->horizontal_placement)
-	, vertical_placement_(definition->vertical_placement)
-	, maximum_width_(definition->maximum_width)
-	, maximum_height_(definition->maximum_height)
-	, x_(definition->x)
-	, y_(definition->y)
-	, w_(definition->width)
-	, h_(definition->height)
-	, reevaluate_best_size_(definition->reevaluate_best_size)
-	, functions_(definition->functions)
-	, tooltip_(definition->tooltip)
-	, helptip_(definition->helptip)
+	, automatic_placement_(definition.automatic_placement)
+	, horizontal_placement_(definition.horizontal_placement)
+	, vertical_placement_(definition.vertical_placement)
+	, maximum_width_(definition.maximum_width)
+	, maximum_height_(definition.maximum_height)
+	, x_(definition.x)
+	, y_(definition.y)
+	, w_(definition.width)
+	, h_(definition.height)
+	, reevaluate_best_size_(definition.reevaluate_best_size)
+	, functions_(definition.functions)
+	, tooltip_(definition.tooltip)
+	, helptip_(definition.helptip)
 	, click_dismiss_(false)
 	, enter_disabled_(false)
 	, escape_disabled_(false)
@@ -322,7 +322,7 @@ window::window(const builder_window::window_resolution* definition)
 	}
 
 	connect_signal<event::SDL_VIDEO_RESIZE>(std::bind(
-			&window::signal_handler_sdl_video_resize, this, _2, _3, _5));
+			&window::signal_handler_sdl_video_resize, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5));
 
 	connect_signal<event::SDL_ACTIVATE>(std::bind(
 			&event::distributor::initialize_state, event_distributor_.get()));
@@ -330,54 +330,54 @@ window::window(const builder_window::window_resolution* definition)
 	connect_signal<event::SDL_LEFT_BUTTON_UP>(
 			std::bind(&window::signal_handler_click_dismiss,
 						this,
-						_2,
-						_3,
-						_4,
+						std::placeholders::_2,
+						std::placeholders::_3,
+						std::placeholders::_4,
 						SDL_BUTTON_LMASK),
 			event::dispatcher::front_child);
 	connect_signal<event::SDL_MIDDLE_BUTTON_UP>(
 			std::bind(&window::signal_handler_click_dismiss,
 						this,
-						_2,
-						_3,
-						_4,
+						std::placeholders::_2,
+						std::placeholders::_3,
+						std::placeholders::_4,
 						SDL_BUTTON_MMASK),
 			event::dispatcher::front_child);
 	connect_signal<event::SDL_RIGHT_BUTTON_UP>(
 			std::bind(&window::signal_handler_click_dismiss,
 						this,
-						_2,
-						_3,
-						_4,
+						std::placeholders::_2,
+						std::placeholders::_3,
+						std::placeholders::_4,
 						SDL_BUTTON_RMASK),
 			event::dispatcher::front_child);
 
 	connect_signal<event::SDL_KEY_DOWN>(
 			std::bind(
-					&window::signal_handler_sdl_key_down, this, _2, _3, _5, _6, true),
+					&window::signal_handler_sdl_key_down, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5, std::placeholders::_6, true),
 			event::dispatcher::back_post_child);
 	connect_signal<event::SDL_KEY_DOWN>(std::bind(
-			&window::signal_handler_sdl_key_down, this, _2, _3, _5, _6, false));
+			&window::signal_handler_sdl_key_down, this, std::placeholders::_2, std::placeholders::_3, std::placeholders::_5, std::placeholders::_6, false));
 
 	connect_signal<event::MESSAGE_SHOW_TOOLTIP>(
 			std::bind(&window::signal_handler_message_show_tooltip,
 						this,
-						_2,
-						_3,
-						_5),
+						std::placeholders::_2,
+						std::placeholders::_3,
+						std::placeholders::_5),
 			event::dispatcher::back_pre_child);
 
 	connect_signal<event::MESSAGE_SHOW_HELPTIP>(
 			std::bind(&window::signal_handler_message_show_helptip,
 						this,
-						_2,
-						_3,
-						_5),
+						std::placeholders::_2,
+						std::placeholders::_3,
+						std::placeholders::_5),
 			event::dispatcher::back_pre_child);
 
 	connect_signal<event::REQUEST_PLACEMENT>(
 			std::bind(
-					&window::signal_handler_request_placement, this, _2, _3),
+					&window::signal_handler_request_placement, this, std::placeholders::_2, std::placeholders::_3),
 			event::dispatcher::back_pre_child);
 
 	connect_signal<event::CLOSE_WINDOW>(std::bind(&window::signal_handler_close_window, this));
@@ -411,7 +411,7 @@ window::~window()
 	 * Another issue is that on smallgui and an MP game the tooltip not
 	 * unrendered properly can capture the mouse and make playing impossible.
 	 */
-	if(show_mode_ == modal) {
+	if(show_mode_ == show_mode::modal) {
 		dialogs::tip::remove();
 	}
 
@@ -429,15 +429,6 @@ window* window::window_instance(const unsigned handle)
 	return manager::instance().get_window(handle);
 }
 
-/*WIKI
- * @page = GUIToolkitWML
- * @order = 3_widget_window_2
- *
- * List if the id's that have generate a return value:
- * * ok confirms the dialog.
- * * cancel cancels the dialog.
- *
- */
 retval window::get_retval_by_id(const std::string& id)
 {
 	// Note it might change to a map later depending on the number
@@ -457,12 +448,12 @@ void window::show_tooltip(/*const unsigned auto_close_timeout*/)
 
 	generate_dot_file("show", SHOW);
 
-	assert(status_ == NEW);
+	assert(status_ == status::NEW);
 
 	set_mouse_behavior(event::dispatcher::none);
 	set_want_keyboard_input(false);
 
-	show_mode_ = tooltip;
+	show_mode_ = show_mode::tooltip;
 
 	/*
 	 * Before show has been called, some functions might have done some testing
@@ -479,11 +470,11 @@ void window::show_non_modal(/*const unsigned auto_close_timeout*/)
 
 	generate_dot_file("show", SHOW);
 
-	assert(status_ == NEW);
+	assert(status_ == status::NEW);
 
 	set_mouse_behavior(event::dispatcher::hit);
 
-	show_mode_ = modeless;
+	show_mode_ = show_mode::modeless;
 
 	/*
 	 * Before show has been called, some functions might have done some testing
@@ -506,14 +497,14 @@ int window::show(const bool restore, const unsigned auto_close_timeout)
 	 */
 	dialogs::tip::remove();
 
-	show_mode_ = modal;
+	show_mode_ = show_mode::modal;
 	restore_ = restore;
 
 	log_scope2(log_gui_draw, LOG_SCOPE_HEADER);
 
 	generate_dot_file("show", SHOW);
 
-	assert(status_ == NEW);
+	assert(status_ == status::NEW);
 
 	/*
 	 * Before show has been called, some functions might have done some testing
@@ -542,7 +533,7 @@ int window::show(const bool restore, const unsigned auto_close_timeout)
 	{
 		// Start our loop drawing will happen here as well.
 		bool mouse_button_state_initialized = false;
-		for(status_ = SHOWING; status_ != CLOSED;) {
+		for(status_ = status::SHOWING; status_ != status::CLOSED;) {
 			push_draw_event();
 
 			// process installed callback if valid, to allow e.g. network
@@ -564,8 +555,8 @@ int window::show(const bool restore, const unsigned auto_close_timeout)
 				mouse_button_state_initialized = true;
 			}
 
-			if(status_ == REQUEST_CLOSE) {
-				status_ = exit_hook_(*this) ? CLOSED : SHOWING;
+			if(status_ == status::REQUEST_CLOSE) {
+				status_ = exit_hook_(*this) ? status::CLOSED : status::SHOWING;
 			}
 
 			// Add a delay so we don't keep spinning if there's no event.
@@ -684,7 +675,7 @@ void window::draw()
 		assert(!item.empty());
 
 		const SDL_Rect dirty_rect
-				= new_widgets ? video().screen_area()
+				= new_widgets ? video_.screen_area()
 							  : item.back()->get_dirty_rectangle();
 
 // For testing we disable the clipping rect and force the entire screen to
@@ -902,19 +893,15 @@ void window::layout()
 
 	get_screen_size_variables(variables_);
 
-	int maximum_width = 0;
-	int maximum_height = 0;
+	unsigned int maximum_width = maximum_width_(variables_, &functions_);
+	unsigned int maximum_height = maximum_height_(variables_, &functions_);
 
 	if(automatic_placement_) {
-		if(maximum_width_ > 0) {
-			maximum_width = std::min(maximum_width_, settings::screen_width);
-		} else {
+		if(maximum_width == 0 || maximum_width > settings::screen_width) {
 			maximum_width = settings::screen_width;
 		}
 
-		if(maximum_height_ > 0) {
-			maximum_height = std::min(maximum_height_, settings::screen_height);
-		} else {
+		if(maximum_height == 0 || maximum_height > settings::screen_height) {
 			maximum_height = settings::screen_height;
 		}
 	} else {
@@ -1006,7 +993,12 @@ void window::layout()
 
 	/***** Get the best location for the window *****/
 	size = get_best_size();
-	assert(size.x <= maximum_width && size.y <= maximum_height);
+	/* Although 0-size windows might not seem valid/useful, there are
+	   a handful of windows that request 0 size just to get a position
+	   chosen via the code below, so at least for now allow them:
+	*/
+	assert(size.x >= 0 && static_cast<unsigned>(size.x) <= maximum_width
+	       && size.y >= 0 && static_cast<unsigned>(size.y) <= maximum_height);
 
 	point origin(0, 0);
 
@@ -1186,9 +1178,9 @@ void window::redraw_windows_on_top() const
 	}
 }
 
-void window::finalize(const std::shared_ptr<builder_grid>& content_grid)
+void window::finalize(const builder_grid& content_grid)
 {
-	window_swap_grid(nullptr, &get_grid(), content_grid->build(), "_window_content_grid");
+	window_swap_grid(nullptr, &get_grid(), content_grid.build(), "_window_content_grid");
 }
 
 #ifdef DEBUG_WINDOW_LAYOUT_GRAPHS
@@ -1406,12 +1398,12 @@ void window::signal_handler_sdl_key_down(const event::ui_event event,
 
 void window::signal_handler_message_show_tooltip(const event::ui_event event,
 												  bool& handled,
-												  event::message& message)
+												  const event::message& message)
 {
 	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
 
-	event::message_show_tooltip& request
-			= dynamic_cast<event::message_show_tooltip&>(message);
+	const event::message_show_tooltip& request
+			= dynamic_cast<const event::message_show_tooltip&>(message);
 
 	dialogs::tip::show(tooltip_.id, request.message, request.location, request.source_rect);
 
@@ -1420,12 +1412,12 @@ void window::signal_handler_message_show_tooltip(const event::ui_event event,
 
 void window::signal_handler_message_show_helptip(const event::ui_event event,
 												  bool& handled,
-												  event::message& message)
+												  const event::message& message)
 {
 	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
 
-	event::message_show_helptip& request
-			= dynamic_cast<event::message_show_helptip&>(message);
+	const event::message_show_helptip& request
+			= dynamic_cast<const event::message_show_helptip&>(message);
 
 	dialogs::tip::show(helptip_.id, request.message, request.location, request.source_rect);
 
@@ -1449,25 +1441,6 @@ void window::signal_handler_close_window()
 
 // }---------- DEFINITION ---------{
 
-/*WIKI
- * @page = GUIWidgetDefinitionWML
- * @order = 1_window
- *
- * == Window ==
- *
- * The definition of a window. A window is a kind of panel see the panel for
- * which fields exist
- *
- * @begin{parent}{name="gui/"}
- * @begin{tag}{name="window_definition"}{min=0}{max=-1}{super="generic/widget_definition"}
- * @begin{tag}{name="resolution"}{min=0}{max=-1}{super="gui/panel_definition/resolution"}
- * @allow{link}{name="gui/window/resolution/grid"}
- * @allow{link}{name="gui/panel_definition/resolution/background"}
- * @allow{link}{name="gui/panel_definition/resolution/foreground"}
- * @end{tag}{name="resolution"}
- * @end{tag}{name="window_definition"}
- * @end{parent}{name="gui/"}
- */
 window_definition::window_definition(const config& cfg)
 	: styled_widget_definition(cfg)
 {
@@ -1496,7 +1469,7 @@ window_definition::resolution::resolution(const config& cfg)
 /**
  * @page layout_algorithm Layout algorithm
  *
- * @section introduction Introduction
+ * @section introduction-layout_algorithm Introduction
  *
  * This page describes how the layout engine for the dialogs works. First
  * a global overview of some terms used in this document.
@@ -1534,8 +1507,8 @@ window_definition::resolution::resolution(const config& cfg)
  *
  * - All visible grid cells; A grid cell is visible when the widget inside
  *   of it doesn't have the state visibility::invisible. Widgets which have the
- *   state @ref visibility::hidden are sized properly since when they become
- *   @ref visibility::visible the layout shouldn't be invalidated. A grid cell
+ *   state visibility::hidden are sized properly since when they become
+ *   visibility::visible the layout shouldn't be invalidated. A grid cell
  *   that's invisible has size 0,0.
  *
  * - All resizable grid cells; A grid cell is resizable under the following
