@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2007 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2007 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -56,19 +57,21 @@ class distributor;
 } // namespace event
 
 /**
- * base class of top level items, the only item
- * which needs to store the final canvases to draw on
+ * @ingroup GUIWidgetWML
+ *
+ * base class of top level items, the only item which needs to store the final canvases to draw on.
+ * A window is a kind of panel see the panel for which fields exist.
  */
 class window : public panel
 {
 	friend class debug_layout_graph;
-	friend window* build(const builder_window::window_resolution*);
+	friend std::unique_ptr<window> build(const builder_window::window_resolution&);
 	friend struct window_implementation;
 	friend class invalidate_layout_blocker;
 	friend class pane;
 
 public:
-	explicit window(const builder_window::window_resolution* definition);
+	explicit window(const builder_window::window_resolution& definition);
 
 	~window();
 
@@ -163,14 +166,11 @@ public:
 	}
 
 	/** The status of the window. */
-	enum status {
-		NEW,		   /**< The window is new and not yet shown. */
-		SHOWING,	   /**< The window is being shown. */
-		REQUEST_CLOSE, /**< The window has been requested to be
-						*   closed but still needs to evaluate the
-						*   request.
-						*/
-		CLOSED /**< The window has been closed. */
+	enum class status {
+		NEW,           /**< The window is new and not yet shown. */
+		SHOWING,       /**< The window is being shown. */
+		REQUEST_CLOSE, /**< The window has been requested to be closed but still needs to evaluate the request. */
+		CLOSED         /**< The window has been closed. */
 	};
 
 	/**
@@ -181,7 +181,7 @@ public:
 	 */
 	void close()
 	{
-		status_ = REQUEST_CLOSE;
+		status_ = status::REQUEST_CLOSE;
 	}
 
 	/**
@@ -349,11 +349,6 @@ public:
 
 	/***** ***** ***** setters / getters for members ***** ****** *****/
 
-	CVideo& video()
-	{
-		return video_;
-	}
-
 	/**
 	 * Sets there return value of the window.
 	 *
@@ -430,7 +425,7 @@ public:
 		callback_next_draw_ = func;
 	}
 
-	enum show_mode {
+	enum class show_mode {
 		none,
 		modal,
 		modeless,
@@ -503,10 +498,10 @@ private:
 	const unsigned vertical_placement_;
 
 	/** The maximum width if automatic_placement_ is true. */
-	unsigned maximum_width_;
+	typed_formula<unsigned> maximum_width_;
 
 	/** The maximum height if automatic_placement_ is true. */
-	unsigned maximum_height_;
+	typed_formula<unsigned> maximum_height_;
 
 	/** The formula to calculate the x value of the dialog. */
 	typed_formula<unsigned> x_;
@@ -665,7 +660,7 @@ private:
 	 *
 	 * @param content_grid        The new contents for the content grid.
 	 */
-	void finalize(const std::shared_ptr<builder_grid>& content_grid);
+	void finalize(const builder_grid& content_grid);
 
 #ifdef DEBUG_WINDOW_LAYOUT_GRAPHS
 	debug_layout_graph* debug_layout_;
@@ -755,11 +750,11 @@ private:
 
 	void signal_handler_message_show_tooltip(const event::ui_event event,
 											 bool& handled,
-											 event::message& message);
+											 const event::message& message);
 
 	void signal_handler_message_show_helptip(const event::ui_event event,
 											 bool& handled,
-											 event::message& message);
+											 const event::message& message);
 
 	void signal_handler_request_placement(const event::ui_event event,
 										  bool& handled);
