@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -354,7 +355,7 @@ void halo_impl::remove(int handle)
 
 void halo_impl::unrender(std::set<map_location> invalidated_locations)
 {
-	if(preferences::show_haloes() == false || haloes.empty()) {
+	if(haloes.empty()) {
 		return;
 	}
 	//assert(invalidated_haloes.empty());
@@ -429,7 +430,7 @@ void halo_impl::unrender(std::set<map_location> invalidated_locations)
 
 void halo_impl::render()
 {
-	if(preferences::show_haloes() == false || haloes.empty() ||
+	if(haloes.empty() ||
 			invalidated_haloes.empty()) {
 		return;
 	}
@@ -508,12 +509,8 @@ halo_record::~halo_record()
 
 	std::shared_ptr<halo_impl> man = my_manager_.lock();
 
-	if (man) {
-		try {
-			man->remove(id_);
-		} catch (std::exception & e) {
-			std::cerr << "Caught an exception in halo::halo_record destructor: \n" << e.what() << std::endl;
-		} catch (...) {}
+	if(man) {
+		man->remove(id_);
 	}
 }
 
