@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -25,6 +26,7 @@
 #include "events.hpp"                   // for raise_draw_event, pump, etc
 #include "font/constants.hpp"           // for relative_size
 #include "preferences/game.hpp"
+#include "game_config_manager.hpp"
 #include "gettext.hpp"                  // for _
 #include "gui/dialogs/transient_message.hpp"
 #include "help/help_browser.hpp"             // for help_browser
@@ -50,23 +52,41 @@ static lg::log_domain log_display("display");
 #define WRN_DP LOG_STREAM(warn, log_display)
 
 static lg::log_domain log_help("help");
-#define WRN_HP LOG_STREAM(warn, log_help)
-#define DBG_HP LOG_STREAM(debug, log_help)
+#define ERR_HELP LOG_STREAM(err, log_help)
 
 namespace help {
+/**
+ * Open a help dialog using a specified toplevel.
+ *
+ * This would allow for complete customization of the contents, although not in a
+ * very easy way. It's used as the internal implementation of the other help*
+ * functions.
+ *
+ *@pre The help_manager must already exist; this is different to the functions
+ * declared in help.hpp, which is why this one's declaration is in the .cpp
+ * file. Because this takes a section as an argument, it wouldn't make sense
+ * for it to call ensure_cache_lifecycle() internally - if the help_manager
+ * doesn't already exist, that would likely destroy the referenced object at
+ * the point that this function exited.
+ */
+void show_with_toplevel(const section &toplevel, const std::string& show_topic="", int xloc=-1, int yloc=-1);
+
 
 void show_unit_description(const unit &u)
 {
+	auto cache_lifecycle = ensure_cache_lifecycle();
 	help::show_unit_description(u.type());
 }
 
 void show_terrain_description(const terrain_type &t)
 {
-	help::show_terrain_help(t.id(), t.hide_in_editor() || t.is_combined());
+	auto cache_lifecycle = ensure_cache_lifecycle();
+	help::show_terrain_help(t.id(), t.hide_help());
 }
 
 void show_unit_description(const unit_type &t)
 {
+	auto cache_lifecycle = ensure_cache_lifecycle();
 	std::string var_id = t.get_cfg()["variation_id"].str();
 	if (var_id.empty())
 		var_id = t.get_cfg()["variation_name"].str();
@@ -88,18 +108,26 @@ void show_unit_description(const unit_type &t)
 		help::show_unit_help(t.id(), t.show_variations_in_help(), hide_help);
 }
 
-extern config dummy_cfg;
-
-help_manager::help_manager(const config *cfg) //, gamemap *_map)
+help_manager::help_manager(const game_config_view *cfg)
 {
-	game_cfg = cfg == nullptr ? &dummy_cfg : cfg;
-//	map = _map;
+	assert(!game_cfg);
+	assert(cfg);
+	// This is a global rawpointer in the help:: namespace.
+	game_cfg = cfg;
+}
+
+std::unique_ptr<help_manager> ensure_cache_lifecycle()
+{
+	// The internals of help_manager are that this global raw pointer is
+	// non-null if and only if an instance of help_manager already exists.
+	if(game_cfg)
+		return nullptr;
+	return std::make_unique<help_manager>(&game_config_manager::get()->game_config());
 }
 
 help_manager::~help_manager()
 {
 	game_cfg = nullptr;
-//	map = nullptr;
 	default_toplevel.clear();
 	hidden_sections.clear();
     // These last numbers must be reset so that the content is regenerated.
@@ -115,7 +143,8 @@ help_manager::~help_manager()
  */
 void show_help(const std::string& show_topic, int xloc, int yloc)
 {
-	show_help(default_toplevel, show_topic, xloc, yloc);
+	auto cache_lifecycle = ensure_cache_lifecycle();
+	show_with_toplevel(default_toplevel, show_topic, xloc, yloc);
 }
 
 /**
@@ -125,7 +154,8 @@ void show_help(const std::string& show_topic, int xloc, int yloc)
  */
 void show_unit_help(const std::string& show_topic, bool has_variations, bool hidden, int xloc, int yloc)
 {
-	show_help(default_toplevel,
+	auto cache_lifecycle = ensure_cache_lifecycle();
+	show_with_toplevel(default_toplevel,
 			  hidden_symbol(hidden) + (has_variations ? ".." : "") + unit_prefix + show_topic, xloc, yloc);
 }
 
@@ -136,7 +166,8 @@ void show_unit_help(const std::string& show_topic, bool has_variations, bool hid
  */
 void show_terrain_help(const std::string& show_topic, bool hidden, int xloc, int yloc)
 {
-	show_help(default_toplevel, hidden_symbol(hidden) + terrain_prefix + show_topic, xloc, yloc);
+	auto cache_lifecycle = ensure_cache_lifecycle();
+	show_with_toplevel(default_toplevel, hidden_symbol(hidden) + terrain_prefix + show_topic, xloc, yloc);
 }
 
 /**
@@ -144,7 +175,8 @@ void show_terrain_help(const std::string& show_topic, bool hidden, int xloc, int
  */
 void show_variation_help(const std::string& unit, const std::string &variation, bool hidden, int xloc, int yloc)
 {
-	show_help(default_toplevel, hidden_symbol(hidden) + variation_prefix + unit + "_" + variation, xloc, yloc);
+	auto cache_lifecycle = ensure_cache_lifecycle();
+	show_with_toplevel(default_toplevel, hidden_symbol(hidden) + variation_prefix + unit + "_" + variation, xloc, yloc);
 }
 
 /**
@@ -153,7 +185,7 @@ void show_variation_help(const std::string& unit, const std::string &variation,
  * This allows for complete customization of the contents, although not in a
  * very easy way.
  */
-void show_help(const section &toplevel_sec,
+void show_with_toplevel(const section &toplevel_sec,
 			   const std::string& show_topic,
 			   int xloc, int yloc)
 {
@@ -181,7 +213,7 @@ void show_help(const section &toplevel_sec,
 	gui::button close_button_(video, _("Close"));
 	buttons_ptr.push_back(&close_button_);
 
-	gui::dialog_frame f(video, _("The Battle for Wesnoth Help"), gui::dialog_frame::default_style,
+	gui::dialog_frame f(video, _("Help"), gui::dialog_frame::default_style,
 					 true, &buttons_ptr);
 	f.layout(xloc, yloc, width, height);
 	f.draw();
@@ -234,11 +266,12 @@ void show_help(const section &toplevel_sec,
 			CVideo::delay(10);
 		}
 	}
-	catch (parse_error& /*e*/) {
-		// Disabled due to issue #2587
+	catch (const parse_error& e) {
+		ERR_HELP << _("Parse error when parsing help text:") << " " << e.message << std::endl;
 #if 0
+		// Displaying in the UI is disabled due to issue #2587
 		std::stringstream msg;
-		msg << _("Parse error when parsing help text: ") << "'" << e.message << "'";
+		msg << _("Parse error when parsing help text:") << " '" << e.message << "'";
 		gui2::show_transient_message("", msg.str());
 #endif
 	}
