@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -26,7 +27,7 @@ struct SDL_Rect;
 
 namespace help {
 
-/// A help browser widget.
+/** A help browser widget. */
 class help_browser : public gui::widget
 {
 public:
@@ -34,8 +35,10 @@ public:
 
 	void adjust_layout();
 
-	/// Display the topic with the specified identifier. Open the menu
-	/// on the right location and display the topic in the text area.
+	/**
+	 * Display the topic with the specified identifier. Open the menu
+	 * on the right location and display the topic in the text area.
+	 */
 	void show_topic(const std::string &topic_id);
 
 protected:
@@ -44,14 +47,18 @@ protected:
 	virtual void handle_event(const SDL_Event &event);
 
 private:
-	/// Update the current cursor, set it to the reference cursor if
-	/// mousex, mousey is over a cross-reference, otherwise, set it to
-	/// the normal cursor.
+	/**
+	 * Update the current cursor, set it to the reference cursor if
+	 * mousex, mousey is over a cross-reference, otherwise, set it to
+	 * the normal cursor.
+	 */
 	void update_cursor();
 	void show_topic(const topic &t, bool save_in_history=true);
-	/// Move in the topic history. Pop an element from from and insert
-	/// it in to. Pop at the fronts if the maximum number of elements is
-	/// exceeded.
+	/**
+	 * Move in the topic history. Pop an element from from and insert
+	 * it in to. Pop at the fronts if the maximum number of elements is
+	 * exceeded.
+	 */
 	void move_in_history(std::deque<const topic *> &from, std::deque<const topic *> &to);
 	help_menu menu_;
 	help_text_area text_area_;
