@@ -1,40 +1,43 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "help/help_impl.hpp"
 
 #include "about.hpp"                    // for get_text
+#include "actions/attack.hpp"           // for time_of_day bonus
 #include "display.hpp"                  // for display
 #include "display_context.hpp"          // for display_context
+#include "formula/string_utils.hpp"     // for VGETTEXT
 #include "game_config.hpp"              // for debug, menu_contract, etc
 #include "game_config_manager.hpp"      // for game_config_manager
 #include "preferences/game.hpp"         // for encountered_terrains, etc
 #include "gettext.hpp"                  // for _, gettext, N_
 #include "help/help_topic_generators.hpp"
 #include "hotkey/hotkey_command.hpp"    // for is_scope_active, etc
-#include "picture.hpp"            // for get_image, locator
+#include "picture.hpp"                    // for get_image, locator
 #include "log.hpp"                      // for LOG_STREAM, logger, etc
 #include "utils/make_enum.hpp"          // for operator<<
 #include "map/map.hpp"                  // for gamemap
-#include "font/marked-up_text.hpp"      // for is_cjk_char, word_wrap_text
 #include "font/standard_colors.hpp"     // for NORMAL_COLOR
+#include "font/sdl_ttf_compat.hpp"
 #include "units/race.hpp"               // for unit_race, etc
 #include "resources.hpp"                // for tod_manager, config_manager
 #include "sdl/surface.hpp"                // for surface
 #include "serialization/string_utils.hpp"  // for split, quoted_split, etc
 #include "serialization/unicode_cast.hpp"  // for unicode_cast
-#include "serialization/unicode_types.hpp"  // for char_t, etc
+#include "serialization/utf8_exception.hpp"  // for char_t, etc
 #include "terrain/terrain.hpp"          // for terrain_type
 #include "terrain/translation.hpp"      // for operator==, ter_list, etc
 #include "terrain/type_data.hpp"        // for terrain_type_data, etc
@@ -51,7 +54,6 @@
 #include <iterator>                     // for back_insert_iterator, etc
 #include <map>                          // for map, etc
 #include <set>
-#include <SDL.h>
 
 static lg::log_domain log_display("display");
 #define WRN_DP LOG_STREAM(warn, log_display)
@@ -62,7 +64,7 @@ static lg::log_domain log_help("help");
 
 namespace help {
 
-const config *game_cfg = nullptr;
+const game_config_view *game_cfg = nullptr;
 // The default toplevel.
 help::section default_toplevel;
 // All sections and topics not referenced from the default toplevel.
@@ -72,11 +74,10 @@ int last_num_encountered_units = -1;
 int last_num_encountered_terrains = -1;
 boost::tribool last_debug_state = boost::indeterminate;
 
-config dummy_cfg;
 std::vector<std::string> empty_string_vector;
 const int max_section_level = 15;
 const int title_size = font::SIZE_LARGE;
-const int title2_size = font::SIZE_15;
+const int title2_size = font::SIZE_PLUS;
 const int box_width = 2;
 const int normal_font_size = font::SIZE_NORMAL;
 const unsigned max_history = 100;
@@ -94,6 +95,61 @@ const std::string era_prefix = "era_";
 const std::string variation_prefix = "variation_";
 const std::string ability_prefix = "ability_";
 
+static bool is_cjk_char(const char32_t ch)
+{
+	/**
+	 * You can check these range at http://unicode.org/charts/
+	 * see the "East Asian Scripts" part.
+	 * Notice that not all characters in that part is still in use today, so don't list them all here.
+	 * Below are characters that I guess may be used in wesnoth translations.
+	 */
+
+	//FIXME add range from Japanese-specific and Korean-specific section if you know the characters are used today.
+
+	if (ch < 0x2e80) return false; // shortcut for common non-CJK
+
+	return
+		//Han Ideographs: all except Supplement
+		(ch >= 0x4e00 && ch < 0x9fcf) ||
+		(ch >= 0x3400 && ch < 0x4dbf) ||
+		(ch >= 0x20000 && ch < 0x2a6df) ||
+		(ch >= 0xf900 && ch < 0xfaff) ||
+		(ch >= 0x3190 && ch < 0x319f) ||
+
+		//Radicals: all except Ideographic Description
+		(ch >= 0x2e80 && ch < 0x2eff) ||
+		(ch >= 0x2f00 && ch < 0x2fdf) ||
+		(ch >= 0x31c0 && ch < 0x31ef) ||
+
+		//Chinese-specific: Bopomofo and Bopomofo Extended
+		(ch >= 0x3104 && ch < 0x312e) ||
+		(ch >= 0x31a0 && ch < 0x31bb) ||
+
+		//Yi-specific: Yi Radicals, Yi Syllables
+		(ch >= 0xa490 && ch < 0xa4c7) ||
+		(ch >= 0xa000 && ch < 0xa48d) ||
+
+		//Japanese-specific: Hiragana, Katakana, Kana Supplement
+		(ch >= 0x3040 && ch <= 0x309f) ||
+		(ch >= 0x30a0 && ch <= 0x30ff) ||
+		(ch >= 0x1b000 && ch <= 0x1b001) ||
+
+		//Ainu-specific: Katakana Phonetic Extensions
+		(ch >= 0x31f0 && ch <= 0x31ff) ||
+
+		//Korean-specific: Hangul Syllables, Hangul Jamo, Hangul Jamo Extended-A, Hangul Jamo Extended-B
+		(ch >= 0xac00 && ch < 0xd7af) ||
+		(ch >= 0x1100 && ch <= 0x11ff) ||
+		(ch >= 0xa960 && ch <= 0xa97c) ||
+		(ch >= 0xd7b0 && ch <= 0xd7fb) ||
+
+		//CJK Symbols and Punctuation
+		(ch >= 0x3000 && ch < 0x303f) ||
+
+		//Halfwidth and Fullwidth Forms
+		(ch >= 0xff00 && ch < 0xffef);
+}
+
 bool section_is_referenced(const std::string &section_id, const config &cfg)
 {
 	if (const config &toplevel = cfg.child("toplevel"))
@@ -151,7 +207,6 @@ void parse_config_internal(const config *help_cfg, const config *section_cfg,
 	}
 	else if (section_cfg != nullptr) {
 		const std::vector<std::string> sections = utils::quoted_split((*section_cfg)["sections"]);
-		sec.level = level;
 		std::string id = level == 0 ? "toplevel" : (*section_cfg)["id"].str();
 		if (level != 0) {
 			if (!is_valid_id(id)) {
@@ -181,9 +236,8 @@ void parse_config_internal(const config *help_cfg, const config *section_cfg,
 		}
 
 		generate_sections(help_cfg, (*section_cfg)["sections_generator"], sec, level);
-		//TODO: harmonize topics/sections sorting
 		if ((*section_cfg)["sort_sections"] == "yes") {
-			std::sort(sec.sections.begin(),sec.sections.end(), section_less());
+			sec.sections.sort(section_less());
 		}
 
 		bool sort_topics = false;
@@ -293,7 +347,7 @@ void generate_sections(const config *help_cfg, const std::string &generator, sec
 	if (generator == "races") {
 		generate_races_sections(help_cfg, sec, level);
 	} else if (generator == "terrains") {
-		generate_terrain_sections(help_cfg, sec, level);
+		generate_terrain_sections(sec, level);
 	} else if (generator == "eras") {
 		DBG_HP << "Generating eras...\n";
 		generate_era_sections(help_cfg, sec, level);
@@ -325,22 +379,8 @@ std::string generate_topic_text(const std::string &generator, const config *help
 	return empty_string;
 }
 
-topic_text::~topic_text()
+topic_text& topic_text::operator=(std::shared_ptr<topic_generator> g)
 {
-	if (generator_ && --generator_->count == 0)
-		delete generator_;
-}
-
-topic_text::topic_text(const topic_text& t): parsed_text_(t.parsed_text_), generator_(t.generator_)
-{
-	if (generator_)
-		++generator_->count;
-}
-
-topic_text &topic_text::operator=(topic_generator *g)
-{
-	if (generator_ && --generator_->count == 0)
-		delete generator_;
 	generator_ = g;
 	return *this;
 }
@@ -349,59 +389,61 @@ const std::vector<std::string>& topic_text::parsed_text() const
 {
 	if (generator_) {
 		parsed_text_ = parse_text((*generator_)());
-		if (--generator_->count == 0)
-			delete generator_;
-		generator_ = nullptr;
+		// This caches the result, so doesn't need the generator any more
+		generator_.reset();
 	}
 	return parsed_text_;
 }
 
+std::string time_of_day_bonus_colored(const int time_of_day_bonus)
+{
+	return std::string("<format>color='") + (time_of_day_bonus > 0 ? "green" : (time_of_day_bonus < 0 ? "red" : "white")) + "' text='" + std::to_string(time_of_day_bonus) + "'</format>";
+}
+
 std::vector<topic> generate_time_of_day_topics(const bool /*sort_generated*/)
 {
 	std::vector<topic> topics;
 	std::stringstream toplevel;
 
-	// HACK: Wesnoth 1.14 only, to avoid breaking a string freeze in the middle
-	//       of the RC phase. The string already exists in the wesnoth-help
-	//       (WML) textdomain and it's used for the help sections tree on the
-	//       left. It was supposed to be used here as well but someone forgot
-	//       to mark the string translatable. Future versions will just use the
-	//       regular textdomain for this file.
-	const std::string& tod_schedule_heading =
-			translation::dsgettext("wesnoth-help", "Time of Day Schedule");
-
 	if (! resources::tod_manager) {
 		toplevel << _("Only available during a scenario.");
-		topics.emplace_back(tod_schedule_heading, "..schedule", toplevel.str());
+		topics.emplace_back(_("Time of Day Schedule"), "..schedule", toplevel.str());
 		return topics;
 	}
 
-	// HACK: Wesnoth 1.14 only, to avoid breaking a string freeze in the middle
-	//       of the RC phase. The string already exists in the wesnoth-lib
-	//       textdomain and it's used for a similar purpose in the Editor UI.
-	//       Future versions will just use the regular textdomain for this file.
-	const std::string& lawful_bonus_label =
-			translation::dsgettext("wesnoth-lib", "Lawful Bonus:");
-
 	const std::vector<time_of_day>& times = resources::tod_manager->times();
 	for (const time_of_day& time : times)
 	{
 		const std::string id = "time_of_day_" + time.id;
 		const std::string image = "<img>src='" + time.image + "'</img>";
+		const std::string image_lawful = "<img>src='icons/alignments/alignment_lawful_30.png'</img>";
+		const std::string image_neutral = "<img>src='icons/alignments/alignment_neutral_30.png'</img>";
+		const std::string image_chaotic = "<img>src='icons/alignments/alignment_chaotic_30.png'</img>";
+		const std::string image_liminal = "<img>src='icons/alignments/alignment_liminal_30.png'</img>";
 		std::stringstream text;
 
-		toplevel << make_link(time.name.str(), id) << jump_to(160) <<
-				image << jump(30) << time.lawful_bonus << '\n';
+		const int lawful_bonus = generic_combat_modifier(time.lawful_bonus, unit_type::ALIGNMENT::LAWFUL, false, resources::tod_manager->get_max_liminal_bonus());
+		const int neutral_bonus = generic_combat_modifier(time.lawful_bonus, unit_type::ALIGNMENT::NEUTRAL, false, resources::tod_manager->get_max_liminal_bonus());
+		const int chaotic_bonus = generic_combat_modifier(time.lawful_bonus, unit_type::ALIGNMENT::CHAOTIC, false, resources::tod_manager->get_max_liminal_bonus());
+		const int liminal_bonus = generic_combat_modifier(time.lawful_bonus, unit_type::ALIGNMENT::LIMINAL, false, resources::tod_manager->get_max_liminal_bonus());
 
-		text << image << '\n' <<
-				time.description.str() << '\n' <<
-				lawful_bonus_label << ' ' << time.lawful_bonus << '\n' <<
-				'\n' << make_link(_("Schedule"), "..schedule");
+		toplevel << make_link(time.name.str(), id) << jump_to(160) << image << jump(30) <<
+			image_lawful << time_of_day_bonus_colored(lawful_bonus) << jump_to(390) <<
+			image_neutral << time_of_day_bonus_colored(neutral_bonus) << jump_to(450) <<
+			image_chaotic << time_of_day_bonus_colored(chaotic_bonus) << jump_to(520) <<
+			image_liminal << time_of_day_bonus_colored(liminal_bonus) << '\n';
+
+		text << image << '\n' << time.description.str() << '\n' <<
+			image_lawful << _("Lawful Bonus:") << ' ' << time_of_day_bonus_colored(lawful_bonus) << '\n' <<
+			image_neutral << _("Neutral Bonus:") << ' ' << time_of_day_bonus_colored(neutral_bonus) << '\n' <<
+			image_chaotic << _("Chaotic Bonus:") << ' ' << time_of_day_bonus_colored(chaotic_bonus) << '\n' <<
+			image_liminal << _("Liminal Bonus:") << ' ' << time_of_day_bonus_colored(liminal_bonus) << '\n' <<
+			'\n' << make_link(_("Schedule"), "..schedule");
 
 		topics.emplace_back(time.name.str(), id, text.str());
 	}
 
-	topics.emplace_back(tod_schedule_heading, "..schedule", toplevel.str());
+	topics.emplace_back(_("Time of Day Schedule"), "..schedule", toplevel.str());
 	return topics;
 }
 
@@ -423,7 +465,7 @@ std::vector<topic> generate_weapon_special_topics(const bool sort_generated)
 		for (const attack_type& atk : type.attacks()) {
 
 			std::vector<std::pair<t_string, t_string>> specials = atk.special_tooltips();
-			for ( size_t i = 0; i != specials.size(); ++i )
+			for ( std::size_t i = 0; i != specials.size(); ++i )
 			{
 				special_description.emplace(specials[i].first, specials[i].second);
 
@@ -632,7 +674,6 @@ std::vector<topic> generate_faction_topics(const config & era, const bool sort_g
 				if (const unit_race *r = unit_types.find_race(type.race_id())) {
 					races.insert(make_link(r->plural_name(), std::string("..") + race_prefix + r->id()));
 				}
-				DBG_HP << type.alignment() << " -> " << type.alignment_description(type.alignment(), type.genders().front()) << "\n";
 				alignments.insert(make_link(type.alignment_description(type.alignment(), type.genders().front()), "time_of_day"));
 			}
 		}
@@ -694,7 +735,8 @@ std::vector<topic> generate_trait_topics(const bool sort_generated)
 	for (const unit_type_data::unit_type_map::value_type &i : unit_types.types())
 	{
 		const unit_type &type = i.second;
-		if (description_type(type) == FULL_DESCRIPTION) {
+		const auto desc_type = description_type(type);
+		if (desc_type == FULL_DESCRIPTION || desc_type == HIDDEN_BUT_SHOW_MACROS) {
 			if (config::const_child_itors traits = type.possible_traits()) {
 				for (const config & trait : traits) {
 					const std::string trait_id = trait["id"];
@@ -846,9 +888,9 @@ void generate_era_sections(const config* help_cfg, section & sec, int level)
 	}
 }
 
-void generate_terrain_sections(const config* /*help_cfg*/, section& sec, int /*level*/)
+void generate_terrain_sections(section& sec, int /*level*/)
 {
-	ter_data_cache tdata = load_terrain_types_data();
+	std::shared_ptr<terrain_type_data> tdata = load_terrain_types_data();
 
 	if (!tdata) {
 		WRN_HP << "When building terrain help sections, couldn't acquire terrain types data, aborting.\n";
@@ -863,7 +905,7 @@ void generate_terrain_sections(const config* /*help_cfg*/, section& sec, int /*l
 
 		const terrain_type& info = tdata->get_terrain_info(t);
 
-		bool hidden = info.is_combined() || info.hide_help();
+		bool hidden = info.hide_help();
 
 		if (preferences::encountered_terrains().find(t)
 				== preferences::encountered_terrains().end() && !info.is_overlay())
@@ -872,7 +914,7 @@ void generate_terrain_sections(const config* /*help_cfg*/, section& sec, int /*l
 		topic terrain_topic;
 		terrain_topic.title = info.editor_name();
 		terrain_topic.id    = hidden_symbol(hidden) + terrain_prefix + info.id();
-		terrain_topic.text  = new terrain_topic_generator(info);
+		terrain_topic.text  = std::make_shared<terrain_topic_generator>(info);
 
 		t_translation::ter_list base_terrains = tdata->underlying_union_terrain(t);
 		for (const t_translation::terrain_code& base : base_terrains) {
@@ -893,12 +935,12 @@ void generate_terrain_sections(const config* /*help_cfg*/, section& sec, int /*l
 		}
 	}
 
-	for (std::map<std::string, section>::const_iterator it = base_map.begin(); it != base_map.end(); ++it) {
-		sec.add_section(it->second);
+	for (const auto& base : base_map) {
+		sec.add_section(base.second);
 	}
 }
 
-void generate_unit_sections(const config* /*help_cfg*/, section& sec, int level, const bool /*sort_generated*/, const std::string& race)
+void generate_unit_sections(const config* /*help_cfg*/, section& sec, int /*level*/, const bool /*sort_generated*/, const std::string& race)
 {
 	for (const unit_type_data::unit_type_map::value_type &i : unit_types.types()) {
 		const unit_type &type = i.second;
@@ -917,7 +959,7 @@ void generate_unit_sections(const config* /*help_cfg*/, section& sec, int level,
 			const std::string var_ref = hidden_symbol(var_type.hide_help()) + variation_prefix + var_type.id() + "_" + variation_id;
 
 			topic var_topic(topic_name, var_ref, "");
-			var_topic.text = new unit_topic_generator(var_type, variation_id);
+			var_topic.text = std::make_shared<unit_topic_generator>(var_type, variation_id);
 			base_unit.topics.push_back(var_topic);
 		}
 
@@ -926,7 +968,6 @@ void generate_unit_sections(const config* /*help_cfg*/, section& sec, int level,
 
 		base_unit.id = ref_id;
 		base_unit.title = type_name;
-		base_unit.level = level +1;
 
 		sec.add_section(base_unit);
 	}
@@ -950,11 +991,12 @@ std::vector<topic> generate_unit_topics(const bool sort_generated, const std::st
 		if (desc_type != FULL_DESCRIPTION)
 			continue;
 
-		const std::string type_name = type.type_name();
+		const std::string debug_suffix = (game_config::debug ? " (" + type.id() + ")" : "");
+		const std::string type_name = type.type_name() + (type.id() == type.type_name().str() ? "" : debug_suffix);
 		const std::string real_prefix = type.show_variations_in_help() ? ".." : "";
 		const std::string ref_id = hidden_symbol(type.hide_help()) + real_prefix + unit_prefix +  type.id();
 		topic unit_topic(type_name, ref_id, "");
-		unit_topic.text = new unit_topic_generator(type);
+		unit_topic.text = std::make_shared<unit_topic_generator>(type);
 		topics.push_back(unit_topic);
 
 		if (!type.hide_help()) {
@@ -971,9 +1013,11 @@ std::vector<topic> generate_unit_topics(const bool sort_generated, const std::st
 	std::string race_id = "..race_"+race;
 	std::string race_name;
 	std::string race_description;
+	std::string race_help_taxonomy;
 	if (const unit_race *r = unit_types.find_race(race)) {
 		race_name = r->plural_name();
 		race_description = r->description();
+		race_help_taxonomy = r->help_taxonomy();
 		// if (description.empty()) description =  _("No description Available");
 		for (const config &additional_topic : r->additional_topics())
 		  {
@@ -990,6 +1034,17 @@ std::vector<topic> generate_unit_topics(const bool sort_generated, const std::st
 		// description =  _("Here put the description of the Miscellaneous race");
 	}
 
+	// Find any other races whose [race]help_taxonomy points to the current race
+	std::map<std::string, t_string> subgroups;
+	for (const auto &r : unit_types.races()) {
+		if (r.second.help_taxonomy() == race) {
+			if (!r.second.plural_name().empty())
+				subgroups[r.first] = r.second.plural_name();
+			else
+				subgroups[r.first] = r.first;
+		}
+	}
+
 	std::stringstream text;
 
 	if (!race_description.empty()) {
@@ -1005,9 +1060,40 @@ std::vector<topic> generate_unit_topics(const bool sort_generated, const std::st
 		text << "\n\n";
 	}
 
-	text << _("<header>text='Units of this race'</header>") << "\n";
-	for (std::set<std::string, string_less>::iterator u = race_units.begin(); u != race_units.end(); ++u) {
-		text << font::unicode_bullet << " " << (*u) << "\n";
+	if (!race_help_taxonomy.empty()) {
+		utils::string_map symbols;
+		symbols["topic_id"] = "..race_"+race_help_taxonomy;
+		if (const unit_race *r = unit_types.find_race(race_help_taxonomy)) {
+			symbols["help_taxonomy"] = r->plural_name();
+		} else {
+			// Fall back to using showing the race id for the race that we couldn't find.
+			// Not great, but probably useful if UMC has a broken link.
+			symbols["help_taxonomy"] = race_help_taxonomy;
+		}
+		// TRANSLATORS: this is expected to say "[Dunefolk are] a group of units, all of whom are Humans",
+		// or "[Quenoth Elves are] a group of units, all of whom are Elves".
+		text << VGETTEXT("This is a group of units, all of whom are <ref>dst='$topic_id' text='$help_taxonomy'</ref>", symbols) << "\n\n";
+	}
+
+	if (!subgroups.empty()) {
+		if (!race_help_taxonomy.empty()) {
+			text << _("<header>text='Subgroups of units within this group'</header>") << "\n";
+		} else {
+			text << _("<header>text='Groups of units within this race'</header>") << "\n";
+		}
+		for (const auto &sg : subgroups) {
+			text << font::unicode_bullet << " " << make_link(sg.second, "..race_" + sg.first) << "\n";
+		}
+		text << "\n";
+	}
+
+	if (!race_help_taxonomy.empty()) {
+		text << _("<header>text='Units of this group'</header>") << "\n";
+	} else {
+		text << _("<header>text='Units of this race'</header>") << "\n";
+	}
+	for (const auto &u : race_units) {
+		text << font::unicode_bullet << " " << u << "\n";
 	}
 
 	topics.emplace_back(race_name, race_id, text.str());
@@ -1029,6 +1115,12 @@ UNIT_DESCRIPTION_TYPE description_type(const unit_type &type)
 	if (encountered_units.find(type.id()) != encountered_units.end()) {
 		return FULL_DESCRIPTION;
 	}
+
+	// See the docs of HIDDEN_BUT_SHOW_MACROS
+	if (type.id() == "Fog Clearer") {
+		return HIDDEN_BUT_SHOW_MACROS;
+	}
+
 	return NO_DESCRIPTION;
 }
 
@@ -1074,18 +1166,16 @@ std::string generate_contents_links(const section &sec, const std::vector<topic>
 {
 		std::stringstream res;
 
-		section_list::const_iterator s;
-		for (s = sec.sections.begin(); s != sec.sections.end(); ++s) {
-			if (is_visible_id((*s)->id)) {
-				std::string link = make_link((*s)->title, ".."+(*s)->id);
+		for (auto &s : sec.sections) {
+			if (is_visible_id(s.id)) {
+				std::string link = make_link(s.title, ".."+s.id);
 				res << font::unicode_bullet << " " << link << "\n";
 			}
 		}
 
-		std::vector<topic>::const_iterator t;
-		for (t = topics.begin(); t != topics.end(); ++t) {
-			if (is_visible_id(t->id)) {
-				std::string link = make_link(t->title, t->id);
+		for (auto &t : topics) {
+			if (is_visible_id(t.id)) {
+				std::string link = make_link(t.title, t.id);
 				res << font::unicode_bullet << " " << link << "\n";
 			}
 		}
@@ -1102,34 +1192,6 @@ bool topic::operator<(const topic &t) const
 	return id < t.id;
 }
 
-section::~section()
-{
-	std::for_each(sections.begin(), sections.end(), delete_section());
-}
-
-section::section(const section &sec) :
-	title(sec.title),
-	id(sec.id),
-	topics(sec.topics),
-	sections(),
-	level(sec.level)
-{
-	std::transform(sec.sections.begin(), sec.sections.end(),
-				   std::back_inserter(sections), create_section());
-}
-
-section& section::operator=(const section &sec)
-{
-	title = sec.title;
-	id = sec.id;
-	level = sec.level;
-	topics.insert(topics.end(), sec.topics.begin(), sec.topics.end());
-	std::transform(sec.sections.begin(), sec.sections.end(),
-				   std::back_inserter(sections), create_section());
-	return *this;
-}
-
-
 bool section::operator==(const section &sec) const
 {
 	return sec.id == id;
@@ -1142,18 +1204,15 @@ bool section::operator<(const section &sec) const
 
 void section::add_section(const section &s)
 {
-	sections.push_back(new section(s));
+	sections.emplace_back(s);
 }
 
 void section::clear()
 {
 	topics.clear();
-	std::for_each(sections.begin(), sections.end(), delete_section());
 	sections.clear();
 }
 
-
-
 const topic *find_topic(const section &sec, const std::string &id)
 {
 	topic_list::const_iterator tit =
@@ -1161,9 +1220,8 @@ const topic *find_topic(const section &sec, const std::string &id)
 	if (tit != sec.topics.end()) {
 		return &(*tit);
 	}
-	section_list::const_iterator sit;
-	for (sit = sec.sections.begin(); sit != sec.sections.end(); ++sit) {
-		const topic *t = find_topic(*(*sit), id);
+	for (const auto &s : sec.sections) {
+		const auto t = find_topic(s, id);
 		if (t != nullptr) {
 			return t;
 		}
@@ -1173,13 +1231,13 @@ const topic *find_topic(const section &sec, const std::string &id)
 
 const section *find_section(const section &sec, const std::string &id)
 {
-	section_list::const_iterator sit =
+	const auto &sit =
 		std::find_if(sec.sections.begin(), sec.sections.end(), has_id(id));
 	if (sit != sec.sections.end()) {
-		return *sit;
+		return &*sit;
 	}
-	for (sit = sec.sections.begin(); sit != sec.sections.end(); ++sit) {
-		const section *s = find_section(*(*sit), id);
+	for (const auto &subsection : sec.sections) {
+		const auto s = find_section(subsection, id);
 		if (s != nullptr) {
 			return s;
 		}
@@ -1187,13 +1245,18 @@ const section *find_section(const section &sec, const std::string &id)
 	return nullptr;
 }
 
+section *find_section(section &sec, const std::string &id)
+{
+	return const_cast<section *>(find_section(const_cast<const section &>(sec), id));
+}
+
 std::vector<std::string> parse_text(const std::string &text)
 {
 	std::vector<std::string> res;
 	bool last_char_escape = false;
 	const char escape_char = '\\';
 	std::stringstream ss;
-	size_t pos;
+	std::size_t pos;
 	enum { ELEMENT_NAME, OTHER } state = OTHER;
 	for (pos = 0; pos < text.size(); ++pos) {
 		const char c = text[pos];
@@ -1228,7 +1291,7 @@ std::vector<std::string> parse_text(const std::string &text)
 					ss.str("");
 					s << "</" << element_name << ">";
 					const std::string end_element_name = s.str();
-					size_t end_pos = text.find(end_element_name, pos);
+					std::size_t end_pos = text.find(end_element_name, pos);
 					if (end_pos == std::string::npos) {
 						std::stringstream msg;
 						msg << "Unterminated element: " << element_name;
@@ -1271,7 +1334,7 @@ std::string convert_to_wml(const std::string &element_name, const std::string &c
 	// No checks are made for the equal sign or something like that.
 	// Attributes are just separated by spaces or newlines.
 	// Attributes that contain spaces must be in single quotes.
-	for (size_t pos = 0; pos < contents.size(); ++pos) {
+	for (std::size_t pos = 0; pos < contents.size(); ++pos) {
 		const char c = contents[pos];
 		if (c == escape_char && !last_char_escape) {
 			last_char_escape = true;
@@ -1330,7 +1393,7 @@ color_t string_to_color(const std::string &cmp_str)
 	}
 	// a #rrggbb color in pango format.
 	if (*cmp_str.c_str() == '#' && cmp_str.size() == 7) {
-		return color_t::from_argb_bytes(strtoul(cmp_str.c_str() + 1, nullptr, 16));
+		return color_t::from_hex_string(cmp_str.substr(1));
 	}
 	return font::NORMAL_COLOR;
 }
@@ -1340,11 +1403,11 @@ std::vector<std::string> split_in_width(const std::string &s, const int font_siz
 {
 	std::vector<std::string> res;
 	try {
-	const std::string& first_line = font::word_wrap_text(s, font_size, width, -1, 1, true);
-	res.push_back(first_line);
-	if(s.size() > first_line.size()) {
-		res.push_back(s.substr(first_line.size()));
-	}
+		const std::string& first_line = font::pango_word_wrap(s, font_size, width, -1, 1, true);
+		res.push_back(first_line);
+		if(s.size() > first_line.size()) {
+			res.push_back(s.substr(first_line.size()));
+		}
 	}
 	catch (utf8::invalid_utf8_exception&)
 	{
@@ -1364,11 +1427,11 @@ std::string remove_first_space(const std::string& text)
 
 std::string get_first_word(const std::string &s)
 {
-	size_t first_word_start = s.find_first_not_of(' ');
+	std::size_t first_word_start = s.find_first_not_of(' ');
 	if (first_word_start == std::string::npos) {
 		return s;
 	}
-	size_t first_word_end = s.find_first_of(" \n", first_word_start);
+	std::size_t first_word_end = s.find_first_of(" \n", first_word_start);
 	if( first_word_end == first_word_start ) {
 		// This word is '\n'.
 		first_word_end = first_word_start+1;
@@ -1381,9 +1444,9 @@ std::string get_first_word(const std::string &s)
 	if (ch == utf8::iterator::end(re))
 		return re;
 
-	ucs4::char_t firstchar = *ch;
-	if (font::is_cjk_char(firstchar)) {
-		re = unicode_cast<utf8::string>(firstchar);
+	char32_t firstchar = *ch;
+	if (is_cjk_char(firstchar)) {
+		re = unicode_cast<std::string>(firstchar);
 	}
 	return re;
 }
@@ -1393,10 +1456,7 @@ void generate_contents()
 	default_toplevel.clear();
 	hidden_sections.clear();
 	if (game_cfg != nullptr) {
-		const config *help_config = &game_cfg->child("help");
-		if (!*help_config) {
-			help_config = &dummy_cfg;
-		}
+		const config *help_config = &game_cfg->child_or_empty("help");
 		try {
 			default_toplevel = parse_config(help_config);
 			// Create a config object that contains everything that is
@@ -1460,9 +1520,11 @@ bool is_visible_id(const std::string &id) {
 	return (id.empty() || id[0] != '.');
 }
 
-/// Return true if the id is valid for user defined topics and
-/// sections. Some IDs are special, such as toplevel and may not be
-/// be defined in the config.
+/**
+ * Return true if the id is valid for user defined topics and
+ * sections. Some IDs are special, such as toplevel and may not be
+ * be defined in the config.
+ */
 bool is_valid_id(const std::string &id) {
 	if (id == "toplevel") {
 		return false;
@@ -1494,9 +1556,18 @@ unsigned image_width(const std::string &filename)
 	return 0;
 }
 
-void push_tab_pair(std::vector<std::pair<std::string, unsigned int>> &v, const std::string &s)
+void push_tab_pair(std::vector<help::item> &v, const std::string &s, const std::optional<std::string> &image, unsigned padding)
 {
-	v.emplace_back(s, font::line_width(s, normal_font_size));
+	help::item item(s, font::pango_line_width(s, normal_font_size));
+	if (image) {
+		// If the image doesn't exist, don't add padding.
+		auto width = image_width(*image);
+		padding = (width ? padding : 0);
+
+		item.first = "<img>src='" + *image + "'</img>" + (padding ? jump(padding) : "") + s;
+		item.second += width + padding;
+	}
+	v.emplace_back(item);
 }
 
 std::string generate_table(const table_spec &tab, const unsigned int spacing)
@@ -1541,20 +1612,21 @@ std::string generate_table(const table_spec &tab, const unsigned int spacing)
 	return ss.str();
 }
 
-/// Prepend all chars with meaning inside attributes with a backslash.
+/** Prepend all chars with meaning inside attributes with a backslash. */
 std::string escape(const std::string &s)
 {
 	return utils::escape(s, "'\\");
 }
 
-/// Load the appropriate terrain types data to use
-ter_data_cache load_terrain_types_data() {
+/** Load the appropriate terrain types data to use */
+std::shared_ptr<terrain_type_data> load_terrain_types_data()
+{
 	if (display::get_singleton()) {
 		return display::get_singleton()->get_disp_context().map().tdata();
 	} else if (game_config_manager::get()){
 		return game_config_manager::get()->terrain_types();
 	} else {
-		return ter_data_cache();
+		return {};
 	}
 }
 
