@@ -1,22 +1,25 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "help/help_text_area.hpp"
 
 #include "config.hpp"                   // for config, etc
 #include "game_config.hpp"              // for debug
+#include "font/sdl_ttf_compat.hpp"
 #include "help/help_impl.hpp"           // for parse_error, box_width, etc
+#include "lexical_cast.hpp"
 #include "picture.hpp"                    // for get_image
 #include "log.hpp"                      // for LOG_STREAM, log_domain, etc
 #include "preferences/general.hpp"              // for font_scaled
@@ -31,6 +34,11 @@
 static lg::log_domain log_display("display");
 #define WRN_DP LOG_STREAM(warn, log_display)
 
+static lg::log_domain log_help("help");
+#define ERR_HP LOG_STREAM(err, log_help)
+#define WRN_HP LOG_STREAM(warn, log_help)
+#define DBG_HP LOG_STREAM(debug, log_help)
+
 namespace help {
 
 help_text_area::help_text_area(CVideo &video, const section &toplevel) :
@@ -41,7 +49,7 @@ help_text_area::help_text_area(CVideo &video, const section &toplevel) :
 	shown_topic_(nullptr),
 	title_spacing_(16),
 	curr_loc_(0, 0),
-	min_row_height_(font::get_max_height(normal_font_size)),
+	min_row_height_(4 + font::get_max_height(normal_font_size)),
 	curr_row_height_(min_row_height_),
 	contents_height_(0)
 {
@@ -60,6 +68,7 @@ void help_text_area::show_topic(const topic &t)
 	shown_topic_ = &t;
 	set_items();
 	set_dirty(true);
+	DBG_HP << "Showing topic: " << t.id << ": " << t.title << std::endl;
 }
 
 
@@ -103,9 +112,8 @@ void help_text_area::set_items()
 	curr_row_height_ = min_row_height_;
 	// Add the title item.
 	const std::string show_title =
-		font::make_text_ellipsis(shown_topic_->title, title_size, inner_location().w);
-	surface surf(font::get_rendered_text(show_title, title_size,
-					     font::NORMAL_COLOR, TTF_STYLE_BOLD));
+		font::pango_line_ellipsize(shown_topic_->title, title_size, inner_location().w);
+	surface surf = font::pango_render_text(show_title, title_size, font::NORMAL_COLOR, font::pango_text::STYLE_BOLD);
 	if (surf != nullptr) {
 		add_item(item(surf, 0, 0, show_title));
 		curr_loc_.second = title_spacing_;
@@ -301,7 +309,7 @@ void help_text_area::add_text_item(const std::string& text, const std::string& r
 	if (text.empty())
 		return;
 	const int remaining_width = get_remaining_width();
-	size_t first_word_start = text.find_first_not_of(" ");
+	std::size_t first_word_start = text.find_first_not_of(" ");
 	if (first_word_start == std::string::npos) {
 		first_word_start = 0;
 	}
@@ -313,11 +321,11 @@ void help_text_area::add_text_item(const std::string& text, const std::string& r
 		return;
 	}
 	const std::string first_word = get_first_word(text);
-	int state = 0;
-	state |= bold ? TTF_STYLE_BOLD : 0;
-	state |= italic ? TTF_STYLE_ITALIC : 0;
+	int state = font::pango_text::STYLE_NORMAL;
+	state |= bold ? font::pango_text::STYLE_BOLD : 0;
+	state |= italic ? font::pango_text::STYLE_ITALIC : 0;
 	if (curr_loc_.first != get_min_x(curr_loc_.second, curr_row_height_)
-		&& remaining_width < font::line_width(first_word, scaled_font_size, state)) {
+		&& remaining_width < font::pango_line_width(first_word, scaled_font_size, font::pango_text::FONT_STYLE(state))) {
 		// The first word does not fit, and we are not at the start of
 		// the line. Move down.
 		down_one_line();
@@ -343,7 +351,7 @@ void help_text_area::add_text_item(const std::string& text, const std::string& r
 			down_one_line();
 		}
 		else {
-			surface surf(font::get_rendered_text(first_part, scaled_font_size, color, state));
+			surface surf = font::pango_render_text(first_part, scaled_font_size, color, font::pango_text::FONT_STYLE(state));
 			if (surf)
 				add_item(item(surf, curr_loc_.first, curr_loc_.second, first_part, ref_dst));
 		}
@@ -353,16 +361,16 @@ void help_text_area::add_text_item(const std::string& text, const std::string& r
 
 			const std::string first_word_before = get_first_word(s);
 			const std::string first_word_after = get_first_word(remove_first_space(s));
-			if (get_remaining_width() >= font::line_width(first_word_after, scaled_font_size, state)
+			if (get_remaining_width() >= font::pango_line_width(first_word_after, scaled_font_size, font::pango_text::FONT_STYLE(state))
 				&& get_remaining_width()
-				< font::line_width(first_word_before, scaled_font_size, state)) {
+				< font::pango_line_width(first_word_before, scaled_font_size, font::pango_text::FONT_STYLE(state))) {
 				// If the removal of the space made this word fit, we
 				// must move down a line, otherwise it will be drawn
 				// without a space at the end of the line.
 				s = remove_first_space(s);
 				down_one_line();
 			}
-			else if (!(font::line_width(first_word_before, scaled_font_size, state)
+			else if (!(font::pango_line_width(first_word_before, scaled_font_size, font::pango_text::FONT_STYLE(state))
 					   < get_remaining_width())) {
 				s = remove_first_space(s);
 			}
