@@ -1,20 +1,23 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
+#define GETTEXT_DOMAIN "wesnoth-help"
+
 #include "help/help_topic_generators.hpp"
 
-#include "font/sdl_ttf.hpp"             // for line_width
+#include "font/sdl_ttf_compat.hpp"
 #include "game_config.hpp"              // for debug, menu_contract, etc
 #include "preferences/game.hpp"         // for encountered_terrains, etc
 #include "gettext.hpp"                  // for _, gettext, N_
@@ -28,13 +31,12 @@
 #include "tstring.hpp"                  // for t_string, operator<<
 #include "units/helper.hpp"             // for resistance_color
 #include "units/types.hpp"              // for unit_type, unit_type_data, etc
+#include <optional>
 #include "video.hpp"                    // fore current_resolution
 
-#include <boost/optional.hpp>  // for optional
 #include <iostream>                     // for operator<<, basic_ostream, etc
 #include <map>                          // for map, etc
 #include <set>
-#include <SDL2/SDL.h>
 
 static lg::log_domain log_help("help");
 #define WRN_HP LOG_STREAM(warn, log_help)
@@ -67,13 +69,13 @@ static std::string best_str(bool best) {
 
 typedef t_translation::ter_list::const_iterator ter_iter;
 // Gets an english description of a terrain ter_list alias behavior: "Best of cave, hills", "Worst of Swamp, Forest" etc.
-static std::string print_behavior_description(ter_iter start, ter_iter end, const ter_data_cache & tdata, bool first_level = true, bool begin_best = true)
+static std::string print_behavior_description(ter_iter start, ter_iter end, const std::shared_ptr<terrain_type_data> & tdata, bool first_level = true, bool begin_best = true)
 {
 
 	if (start == end) return "";
 	if (*start == t_translation::MINUS || *start == t_translation::PLUS) return print_behavior_description(start+1, end, tdata, first_level, *start == t_translation::PLUS); //absorb any leading mode changes by calling again, with a new default value begin_best.
 
-	boost::optional<ter_iter> last_change_pos;
+	std::optional<ter_iter> last_change_pos;
 
 	bool best = begin_best;
 	for (ter_iter i = start; i != end; ++i) {
@@ -100,7 +102,7 @@ static std::string print_behavior_description(ter_iter start, ter_iter end, cons
 		if (!first_level) ss << "( ";
 		ss << names.at(0);
 
-		for (size_t i = 1; i < names.size(); i++) {
+		for (std::size_t i = 1; i < names.size(); i++) {
 			ss << ", " << names.at(i);
 		}
 
@@ -144,24 +146,21 @@ std::string terrain_topic_generator::operator()() const {
 	else
 		ss << "\n";
 
-	ter_data_cache tdata = load_terrain_types_data();
+	std::shared_ptr<terrain_type_data> tdata = load_terrain_types_data();
 
 	if (!tdata) {
 		WRN_HP << "When building terrain help topics, we couldn't acquire any terrain types data\n";
 		return ss.str();
 	}
 
-	if (!(type_.union_type().size() == 1 && type_.union_type()[0] == type_.number() && type_.is_nonnull())) {
-
-		const t_translation::ter_list& underlying_mvt_terrains = tdata->underlying_mvt_terrain(type_.number());
-
+	if (!type_.is_indivisible()) {
 		ss << "\n" << _("Base Terrain: ");
 
 		bool first = true;
-		for (const t_translation::terrain_code& underlying_terrain : underlying_mvt_terrains) {
-			const terrain_type& mvt_base = tdata->get_terrain_info(underlying_terrain);
+		for (const auto& underlying_terrain : type_.union_type()) {
+			const terrain_type& base = tdata->get_terrain_info(underlying_terrain);
 
-			if (mvt_base.editor_name().empty()) continue;
+			if (base.editor_name().empty()) continue;
 
 			if (!first) {
 				ss << ", ";
@@ -169,15 +168,16 @@ std::string terrain_topic_generator::operator()() const {
 				first = false;
 			}
 
-			ss << make_link(mvt_base.editor_name(), ".." + terrain_prefix + mvt_base.id());
+			ss << make_link(base.editor_name(), ".." + terrain_prefix + base.id());
 		}
 
 		ss << "\n";
 
+		const t_translation::ter_list& underlying_mvt_terrains = type_.mvt_type();
 		ss << "\n" << _("Movement properties: ");
 		ss << print_behavior_description(underlying_mvt_terrains.begin(), underlying_mvt_terrains.end(), tdata) << "\n";
 
-		const t_translation::ter_list& underlying_def_terrains = tdata->underlying_def_terrain(type_.number());
+		const t_translation::ter_list& underlying_def_terrains = type_.def_type();
 		ss << "\n" << _("Defense properties: ");
 		ss << print_behavior_description(underlying_def_terrains.begin(), underlying_def_terrains.end(), tdata) << "\n";
 	}
@@ -231,13 +231,13 @@ std::string terrain_topic_generator::operator()() const {
 
 
 //Typedef to help with formatting list of traits
-//Maps localized trait name to trait help topic ID
+// Maps localized trait name to trait help topic ID
 typedef std::pair<std::string, std::string> trait_data;
 
 //Helper function for printing a list of trait data
 static void print_trait_list(std::stringstream & ss, const std::vector<trait_data> & l)
 {
-	size_t i = 0;
+	std::size_t i = 0;
 	ss << make_link(l[i].first, l[i].second);
 
 	// This doesn't skip traits with empty names
@@ -410,7 +410,7 @@ std::string unit_topic_generator::operator()() const {
 		std::vector<trait_data> random_traits;
 		int must_have_nameless_traits = 0;
 
-		for (const config & trait : traits) {
+		for(const config& trait : traits) {
 			const std::string& male_name = trait["male_name"].str();
 			const std::string& female_name = trait["female_name"].str();
 			std::string trait_name;
@@ -444,8 +444,8 @@ std::string unit_topic_generator::operator()() const {
 				std::stringstream random_count;
 				random_count << " (" << (type_.num_traits() - must_have_traits.size() - must_have_nameless_traits) << ") : ";
 
-				int second_line_whitespace = font::line_width(traits_label+must_have_count.str(), normal_font_size)
-					- font::line_width(random_count.str(), normal_font_size);
+				int second_line_whitespace = font::pango_line_width(traits_label+must_have_count.str(), normal_font_size)
+					- font::pango_line_width(random_count.str(), normal_font_size);
 				// This ensures that the second line is justified so that the ':' characters are aligned.
 
 				ss << must_have_count.str();
@@ -552,7 +552,7 @@ std::string unit_topic_generator::operator()() const {
 		<< _("Alignment:") << font::nbsp
 		<< make_link(type_.alignment_description(type_.alignment(), type_.genders().front()), "time_of_day")
 		<< jump(30);
-	if (type_.can_advance()) {
+	if (type_.can_advance() || type_.modification_advancements()) {
 		// TRANSLATORS: This string is used in the help page of a single unit.  It uses
 		// non-breaking spaces to prevent unpleasant line breaks (issue #3256).  In the
 		// translation use non-breaking spaces as appropriate for the target language.
@@ -560,7 +560,13 @@ std::string unit_topic_generator::operator()() const {
 	}
 
 	// Print the detailed description about the unit.
-		ss << "\n\n" << detailed_description;
+	ss << "\n\n" << detailed_description;
+	if(const auto notes = type_.special_notes(); !notes.empty()) {
+		ss << "\n\n" << _("Special Notes:") << '\n';
+		for(const auto& note : notes) {
+			ss << "• " << note << '\n';
+		}
+	}
 
 	// Print the different attacks a unit has, if it has any.
 	if (!type_.attacks().empty()) {
@@ -573,9 +579,9 @@ std::string unit_topic_generator::operator()() const {
 		// Dummy element, icons are below.
 		first_row.push_back(item("", 0));
 		push_header(first_row, _("unit help^Name"));
-		push_header(first_row, _("Type"));
 		push_header(first_row, _("Strikes"));
 		push_header(first_row, _("Range"));
+		push_header(first_row, _("Type"));
 		push_header(first_row, _("Special"));
 		table.push_back(first_row);
 		// Print information about every attack.
@@ -584,28 +590,44 @@ std::string unit_topic_generator::operator()() const {
 			std::string lang_type = string_table["type_" + attack.type()];
 			std::vector<item> row;
 			std::stringstream attack_ss;
+
+			// Attack icon
 			attack_ss << "<img>src='" << attack.icon() << "'</img>";
 			row.emplace_back(attack_ss.str(),image_width(attack.icon()));
-			push_tab_pair(row, lang_weapon);
-			push_tab_pair(row, lang_type);
 			attack_ss.str(clear_stringstream);
+
+			// Attack name
+			push_tab_pair(row, lang_weapon);
+
+			// damage x strikes
 			attack_ss << attack.damage() << font::weapon_numbers_sep << attack.num_attacks()
 				<< " " << attack.accuracy_parry_description();
 			push_tab_pair(row, attack_ss.str());
 			attack_ss.str(clear_stringstream);
+
+			// Padding for range and damage type icons
+			const auto padding = 5; // TODO amount of padding?
+
+			// Range, with icon
+			const std::string range_icon = "icons/profiles/" + attack.range() + "_attack.png";
 			if (attack.min_range() > 1 || attack.max_range() > 1) {
 				attack_ss << attack.min_range() << "-" << attack.max_range() << ' ';
 			}
 			attack_ss << string_table["range_" + attack.range()];
-			push_tab_pair(row, attack_ss.str());
+			push_tab_pair(row, attack_ss.str(), range_icon, padding);
 			attack_ss.str(clear_stringstream);
+
+			// Damage type, with icon
+			const std::string type_icon = "icons/profiles/" + attack.type() + ".png";
+			push_tab_pair(row, lang_type, type_icon, padding);
+
 			// Show this attack's special, if it has any. Cross
 			// reference it to the section describing the special.
 			std::vector<std::pair<t_string, t_string>> specials = attack.special_tooltips();
 			if (!specials.empty()) {
 				std::string lang_special = "";
-				const size_t specials_size = specials.size();
-				for (size_t i = 0; i != specials_size; ++i) {
+				const std::size_t specials_size = specials.size();
+				for (std::size_t i = 0; i != specials_size; ++i) {
 					const std::string ref_id = std::string("weaponspecial_")
 						+ specials[i].first.base_str();
 					lang_special = (specials[i].first);
@@ -614,7 +636,7 @@ std::string unit_topic_generator::operator()() const {
 						attack_ss << ", "; //comma placed before next special
 					}
 				}
-				row.emplace_back(attack_ss.str(), font::line_width(lang_special, normal_font_size));
+				row.emplace_back(attack_ss.str(), font::pango_line_width(lang_special, normal_font_size));
 			}
 			table.push_back(row);
 		}
@@ -653,7 +675,7 @@ std::string unit_topic_generator::operator()() const {
 			resistance -= std::stoi(dam_it.second);
 		} catch(std::invalid_argument&) {}
 		std::string resist = std::to_string(resistance) + '%';
-		const size_t pos = resist.find('-');
+		const std::size_t pos = resist.find('-');
 		if (pos != std::string::npos) {
 			resist.replace(pos, 1, font::unicode_minus);
 		}
@@ -665,12 +687,12 @@ std::string unit_topic_generator::operator()() const {
 		const std::string markup = str.str();
 		str.str(clear_stringstream);
 		str << resist;
-		row.emplace_back(markup, font::line_width(str.str(), normal_font_size));
+		row.emplace_back(markup, font::pango_line_width(str.str(), normal_font_size));
 		resistance_table.push_back(row);
 	}
 	ss << generate_table(resistance_table);
 
-	if (ter_data_cache tdata = load_terrain_types_data()) {
+	if (std::shared_ptr<terrain_type_data> tdata = load_terrain_types_data()) {
 		// Print the terrain modifier table of the unit.
 		ss << "\n\n<header>text='" << escape(_("Terrain Modifiers"))
 			<< "'</header>\n\n";
@@ -709,7 +731,7 @@ std::string unit_topic_generator::operator()() const {
 				continue;
 			}
 
-			if (info.union_type().size() == 1 && info.union_type()[0] == info.number() && info.is_nonnull()) {
+			if (info.is_indivisible() && info.is_nonnull()) {
 				terrain_movement_info movement_info =
 				{
 					info.name(),
@@ -737,7 +759,7 @@ std::string unit_topic_generator::operator()() const {
 
 			row.emplace_back("<img>src='" + final_image + "'</img> " +
 					make_link(m.name, "..terrain_" + m.id),
-				font::line_width(m.name, normal_font_size) + (high_res ? 32 : 16) );
+				font::pango_line_width(m.name, normal_font_size) + (high_res ? 32 : 16) );
 
 			//defense  -  range: +10 % .. +70 %
 			// passing false to select the more saturated red-to-green scale
@@ -748,7 +770,7 @@ std::string unit_topic_generator::operator()() const {
 			std::string markup = str.str();
 			str.str(clear_stringstream);
 			str << m.defense << "%";
-			row.emplace_back(markup, font::line_width(str.str(), normal_font_size));
+			row.emplace_back(markup, font::pango_line_width(str.str(), normal_font_size));
 
 			//movement  -  range: 1 .. 5, movetype::UNREACHABLE=impassable
 			str.str(clear_stringstream);
@@ -772,7 +794,7 @@ std::string unit_topic_generator::operator()() const {
 			markup = str.str();
 			str.str(clear_stringstream);
 			str << m.movement_cost;
-			row.emplace_back(markup, font::line_width(str.str(), normal_font_size));
+			row.emplace_back(markup, font::pango_line_width(str.str(), normal_font_size));
 
 			//defense cap
 			if (has_terrain_defense_caps) {
@@ -782,6 +804,7 @@ std::string unit_topic_generator::operator()() const {
 				} else {
 					str << "<format>color=white text='" << font::unicode_figure_dash << "'</format>";
 				}
+
 				markup = str.str();
 				str.str(clear_stringstream);
 				if (m.defense_cap) {
@@ -789,7 +812,7 @@ std::string unit_topic_generator::operator()() const {
 				} else {
 					str << font::unicode_figure_dash;
 				}
-				row.emplace_back(markup, font::line_width(str.str(), normal_font_size));
+				row.emplace_back(markup, font::pango_line_width(str.str(), normal_font_size));
 			}
 
 			//vision
@@ -817,7 +840,7 @@ std::string unit_topic_generator::operator()() const {
 				markup = str.str();
 				str.str(clear_stringstream);
 				str << m.vision_cost;
-				row.emplace_back(markup, font::line_width(str.str(), normal_font_size));
+				row.emplace_back(markup, font::pango_line_width(str.str(), normal_font_size));
 			}
 
 			//jamming
@@ -856,10 +879,8 @@ std::string unit_topic_generator::operator()() const {
 	return ss.str();
 }
 
-
-
 void unit_topic_generator::push_header(std::vector< item > &row,  const std::string& name) const {
-	row.emplace_back(bold(name), font::line_width(name, normal_font_size, TTF_STYLE_BOLD));
+	row.emplace_back(bold(name), font::pango_line_width(name, normal_font_size, font::pango_text::STYLE_BOLD));
 }
 
 } // end namespace help
