@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -41,8 +42,7 @@ class unit_topic_generator: public topic_generator
 {
 	const unit_type& type_;
 	const std::string variation_;
-	typedef std::pair< std::string, unsigned > item;
-	void push_header(std::vector< item > &row,  const std::string& name) const;
+	void push_header(std::vector< help::item > &row,  const std::string& name) const;
 public:
 	unit_topic_generator(const unit_type &t, std::string variation="") : type_(t), variation_(variation) {}
 	virtual std::string operator()() const;
