@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "hotkey/command_executor.hpp"
@@ -33,9 +34,9 @@
 #include "../resources.hpp"
 #include "../playmp_controller.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
-#include <SDL_image.h>
+#include <SDL2/SDL_image.h>
 
 #include <cassert>
 #include <ios>
@@ -55,9 +56,9 @@ namespace {
 void make_screenshot(const std::string& name, bool map_screenshot)
 {
 	surface screenshot = display::get_singleton()->screenshot(map_screenshot);
-	if(!screenshot.null()) {
+	if(screenshot) {
 		std::string filename = filesystem::get_screenshot_dir() + "/" + name + "_";
-		filename = filesystem::get_next_filename(filename, ".png");
+		filename = filesystem::get_next_filename(filename, ".jpg");
 		gui2::dialogs::screenshot_notification::display(filename, screenshot);
 	}
 }
@@ -247,6 +248,10 @@ bool command_executor::do_execute_command(const hotkey_command&  cmd, int /*inde
 		case HOTKEY_HELP:
 			show_help();
 			break;
+		case HOTKEY_HELP_ABOUT_SAVELOAD:
+			// although HOTKEY_HELP uses a virtual call to allow context-specific help, this one is already a specific topic
+			help::show_help("saveload");
+			break;
 		case HOTKEY_CHAT_LOG:
 			show_chat_log();
 			break;
@@ -407,12 +412,11 @@ void command_executor::show_menu(const std::vector<config>& items_arg, int xloc,
 	{
 		SDL_Rect pos {xloc, yloc, 1, 1};
 		gui2::dialogs::drop_down_menu mmenu(pos, items, -1, true, false); // TODO: last value should be variable
-		mmenu.show();
-		if(mmenu.get_retval() == gui2::retval::OK) {
+		if(mmenu.show()) {
 			res = mmenu.selected_item();
 		}
 	} // This will kill the dialog.
-	if (res < 0 || size_t(res) >= items.size()) return;
+	if (res < 0 || std::size_t(res) >= items.size()) return;
 
 	const theme::menu* submenu = gui.get_theme().get_menu_item(items[res]["id"]);
 	if (submenu) {
@@ -448,7 +452,7 @@ std::string command_executor::get_menu_image(display& disp, const std::string& c
 
 	// TODO: Find a way to do away with the fugly special markup
 	if(command[0] == '&') {
-		size_t n = command.find_first_of('=');
+		std::size_t n = command.find_first_of('=');
 		if(n != std::string::npos)
 			return command.substr(1, n - 1);
 	}
@@ -489,7 +493,7 @@ std::string command_executor::get_menu_image(display& disp, const std::string& c
 
 void command_executor::get_menu_images(display& disp, std::vector<config>& items)
 {
-	for(size_t i = 0; i < items.size(); ++i) {
+	for(std::size_t i = 0; i < items.size(); ++i) {
 		config& item = items[i];
 
 		const std::string& item_id = item["id"];
@@ -578,6 +582,7 @@ static void event_queue(const SDL_Event& event, command_executor* executor)
 {
 	if (!executor) return;
 	executor->queue_command(event);
+	executor->set_button_state();
 }
 
 void command_executor::queue_command(const SDL_Event& event, int index)
@@ -663,39 +668,6 @@ void command_executor::execute_command_wrap(const command_executor::queued_comma
 	}
 }
 
-// Removes duplicate commands caused by both SDL_KEYDOWN and SDL_TEXTINPUT triggering hotkeys.
-// See https://github.com/wesnoth/wesnoth/issues/1736
-std::vector<command_executor::queued_command> command_executor::filter_command_queue()
-{
-	std::vector<queued_command> filtered_commands;
-
-	/** A command plus "key released" flag. Otherwise, we will filter out key releases that are preceded by a keypress. */
-	using command_with_keyrelease = std::pair<const hotkey_command*, bool>;
-	std::set<command_with_keyrelease> seen_commands;
-
-	for(const queued_command& cmd : command_queue_) {
-		command_with_keyrelease command_key(cmd.command, cmd.release);
-		if(seen_commands.find(command_key) == seen_commands.end()) {
-			seen_commands.insert(command_key);
-			filtered_commands.push_back(cmd);
-		}
-	}
-
-	command_queue_.clear();
-
-	return filtered_commands;
-}
-
-bool command_executor::run_queued_commands()
-{
-	std::vector<queued_command> commands = filter_command_queue();
-	for(const queued_command& cmd : commands) {
-		execute_command_wrap(cmd);
-	}
-
-	return !commands.empty();
-}
-
 void command_executor_default::set_button_state()
 {
 	display& disp = get_display();
@@ -758,6 +730,39 @@ void command_executor_default::set_button_state()
 	}
 }
 
+// Removes duplicate commands caused by both SDL_KEYDOWN and SDL_TEXTINPUT triggering hotkeys.
+// See https://github.com/wesnoth/wesnoth/issues/1736
+std::vector<command_executor::queued_command> command_executor::filter_command_queue()
+{
+	std::vector<queued_command> filtered_commands;
+
+	/** A command plus "key released" flag. Otherwise, we will filter out key releases that are preceded by a keypress. */
+	using command_with_keyrelease = std::pair<const hotkey_command*, bool>;
+	std::set<command_with_keyrelease> seen_commands;
+
+	for(const queued_command& cmd : command_queue_) {
+		command_with_keyrelease command_key(cmd.command, cmd.release);
+		if(seen_commands.find(command_key) == seen_commands.end()) {
+			seen_commands.insert(command_key);
+			filtered_commands.push_back(cmd);
+		}
+	}
+
+	command_queue_.clear();
+
+	return filtered_commands;
+}
+
+bool command_executor::run_queued_commands()
+{
+	std::vector<queued_command> commands = filter_command_queue();
+	for(const queued_command& cmd : commands) {
+		execute_command_wrap(cmd);
+	}
+
+	return !commands.empty();
+}
+
 void command_executor_default::recalculate_minimap()
 {
 	get_display().recalculate_minimap();
@@ -795,7 +800,7 @@ void command_executor_default::zoom_out()
 void command_executor_default::zoom_default()
 {
 	if(!get_display().view_locked()) {
-		get_display().set_default_zoom();
+		get_display().toggle_default_zoom();
 	}
 }
 
