@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -54,6 +55,7 @@ enum HOTKEY_COMMAND {
 	HOTKEY_LABEL_TEAM_TERRAIN, HOTKEY_LABEL_TERRAIN, HOTKEY_CLEAR_LABELS,HOTKEY_SHOW_ENEMY_MOVES, HOTKEY_BEST_ENEMY_MOVES,
 	HOTKEY_DELAY_SHROUD, HOTKEY_UPDATE_SHROUD, HOTKEY_CONTINUE_MOVE,
 	HOTKEY_SEARCH, HOTKEY_SPEAK_ALLY, HOTKEY_SPEAK_ALL, HOTKEY_HELP,
+	HOTKEY_HELP_ABOUT_SAVELOAD,
 	HOTKEY_CHAT_LOG, HOTKEY_LANGUAGE, HOTKEY_ANIMATE_MAP,
 
 	// Replay
@@ -95,7 +97,6 @@ enum HOTKEY_COMMAND {
 	TITLE_SCREEN__RELOAD_WML,
 	TITLE_SCREEN__NEXT_TIP,
 	TITLE_SCREEN__PREVIOUS_TIP,
-	TITLE_SCREEN__TUTORIAL,
 	TITLE_SCREEN__CAMPAIGN,
 	TITLE_SCREEN__MULTIPLAYER,
 	TITLE_SCREEN__ADDONS,
@@ -125,7 +126,6 @@ enum HOTKEY_COMMAND {
 	// Unit
 	HOTKEY_EDITOR_UNIT_TOGGLE_CANRECRUIT, HOTKEY_EDITOR_UNIT_TOGGLE_RENAMEABLE,
 	HOTKEY_EDITOR_UNIT_CHANGE_ID, HOTKEY_EDITOR_UNIT_TOGGLE_LOYAL,
-	HOTKEY_EDITOR_UNIT_RECRUIT_ASSIGN,
 
 	// Brushes
 	HOTKEY_EDITOR_BRUSH_NEXT, HOTKEY_EDITOR_BRUSH_DEFAULT,
@@ -135,7 +135,7 @@ enum HOTKEY_COMMAND {
 	HOTKEY_EDITOR_TOOL_NEXT,
 	HOTKEY_EDITOR_TOOL_PAINT, HOTKEY_EDITOR_TOOL_FILL,
 	HOTKEY_EDITOR_TOOL_SELECT, HOTKEY_EDITOR_TOOL_STARTING_POSITION, HOTKEY_EDITOR_TOOL_LABEL,
-	HOTKEY_EDITOR_TOOL_UNIT, HOTKEY_EDITOR_TOOL_VILLAGE, HOTKEY_EDITOR_TOOL_ITEM, HOTKEY_EDITOR_TOOL_SOUNDSOURCE,
+	HOTKEY_EDITOR_TOOL_UNIT, HOTKEY_EDITOR_TOOL_VILLAGE, HOTKEY_EDITOR_TOOL_ITEM,
 
 	// Select
 	HOTKEY_EDITOR_SELECT_ALL, HOTKEY_EDITOR_SELECT_INVERSE,	HOTKEY_EDITOR_SELECT_NONE,
@@ -148,7 +148,7 @@ enum HOTKEY_COMMAND {
 	HOTKEY_EDITOR_SELECTION_ROTATE, HOTKEY_EDITOR_SELECTION_FLIP,
 	HOTKEY_EDITOR_SELECTION_FILL,
 	HOTKEY_EDITOR_SELECTION_EXPORT,
-	HOTKEY_EDITOR_SELECTION_GENERATE, HOTKEY_EDITOR_SELECTION_RANDOMIZE,
+	HOTKEY_EDITOR_SELECTION_RANDOMIZE,
 
 	// Map
 	HOTKEY_EDITOR_MAP_NEW, HOTKEY_EDITOR_MAP_LOAD, HOTKEY_EDITOR_MAP_SAVE,
@@ -156,7 +156,7 @@ enum HOTKEY_COMMAND {
 	HOTKEY_EDITOR_MAP_REVERT, HOTKEY_EDITOR_MAP_INFO,
 	HOTKEY_EDITOR_MAP_CLOSE,
 	HOTKEY_EDITOR_MAP_SWITCH,
-	HOTKEY_EDITOR_MAP_RESIZE, HOTKEY_EDITOR_MAP_ROTATE,
+	HOTKEY_EDITOR_MAP_RESIZE,
 	HOTKEY_EDITOR_MAP_GENERATE, HOTKEY_EDITOR_MAP_APPLY_MASK,
 	HOTKEY_EDITOR_MAP_CREATE_MASK_TO,
 
@@ -223,16 +223,18 @@ std::list<HOTKEY_COMMAND> get_hotkeys_by_category(HOTKEY_CATEGORY category);
 
 typedef std::bitset<SCOPE_COUNT> hk_scopes;
 
-/// Do not use this outside hotkeys.cpp.
-/// hotkey_command uses t_string which might cause bugs when used at program startup,
-/// so use this for the master hotkey list (and only there).
+/**
+ * Do not use this outside hotkeys.cpp.
+ * hotkey_command uses t_string which might cause bugs when used at program startup,
+ * so use this for the master hotkey list (and only there).
+ */
 struct hotkey_command_temp
 {
 	HOTKEY_COMMAND id;
 
 	std::string command;
 
-	/// description, tooltip are untranslated
+	/** description, tooltip are untranslated */
 	std::string description;
 
 	bool hidden;
@@ -243,8 +245,10 @@ struct hotkey_command_temp
 	std::string tooltip;
 };
 
-/// Stores all information related to functions that can be bound to hotkeys.
-/// this is currently a semi struct: it haves a constructor, but only const-public members.
+/**
+ * Stores all information related to functions that can be bound to hotkeys.
+ * this is currently a semi struct: it haves a constructor, but only const-public members.
+ */
 struct hotkey_command
 {
 	hotkey_command() = delete;
@@ -257,40 +261,46 @@ struct hotkey_command
 	hotkey_command(const hotkey_command&) = default;
 	hotkey_command& operator=(const hotkey_command&) = default;
 
-	/// the names are strange: the "hotkey::HOTKEY_COMMAND" is named id, and the string to identify the object is called "command"
-	/// there is some inconstancy with that names in this file.
-	/// This binds the command to a function. Does not need to be unique.
+	/**
+	 * the names are strange: the "hotkey::HOTKEY_COMMAND" is named id, and the string to identify the object is called "command"
+	 * there is some inconstancy with that names in this file.
+	 * This binds the command to a function. Does not need to be unique.
+	 */
 	HOTKEY_COMMAND id;
 
-	/// The command is unique.
+	/** The command is unique. */
 	std::string command;
 
 	// since the wml_menu hotkey_command s can have different textdomains we need t_string now.
 	t_string description;
 
-	/// If hidden then don't show the command in the hotkey preferences.
+	/** If hidden then don't show the command in the hotkey preferences. */
 	bool hidden;
 
-	/// Toggle hotkeys have some restrictions on what can be bound to them.
-	/// They require a binding that has two states, "pressed" and "released".
+	/**
+	 * Toggle hotkeys have some restrictions on what can be bound to them.
+	 * They require a binding that has two states, "pressed" and "released"
+	 */
 	bool toggle;
 
-	/// The visibility scope of the command.
+	/** The visibility scope of the command. */
 	hk_scopes scope;
 
-	/// The category of the command.
+	/** The category of the command. */
 	HOTKEY_CATEGORY category;
 
 	t_string tooltip;
 
-	/// checks weather this is the null hotkey_command
+	/** checks weather this is the null hotkey_command */
 	bool null() const;
 
-	/// returns the command that is treated as null
+	/** returns the command that is treated as null */
 	static const hotkey_command& null_command();
 
-	/// the execute_command argument was changed from HOTKEY_COMMAND to hotkey_command,
-	/// to be able to call it with HOTKEY_COMMAND, this function was created
+	/**
+	 * the execute_command argument was changed from HOTKEY_COMMAND to hotkey_command,
+	 * to be able to call it with HOTKEY_COMMAND, this function was created
+	 */
 	static const hotkey_command& get_command_by_command(HOTKEY_COMMAND command);
 };
 
@@ -302,14 +312,16 @@ private:
 	hk_scopes prev_scope_active_;
 };
 
-/// returns a container that contains all currently active hotkey_commands.
-/// everything that wants a hotkey, must be in this container.
+/**
+ * returns a container that contains all currently active hotkey_commands.
+ * everything that wants a hotkey, must be in this container
+ */
 const std::vector<hotkey_command>& get_hotkey_commands();
 
-/// returns the hotkey_command with the given name
+/** returns the hotkey_command with the given name */
 const hotkey_command& get_hotkey_command(const std::string& command);
 
-/// returns the hotkey_command that is treated as null.
+/** returns the hotkey_command that is treated as null. */
 const hotkey_command& get_hotkey_null();
 
 void deactivate_all_scopes();
@@ -318,16 +330,17 @@ void set_active_scopes(hk_scopes s);
 bool is_scope_active(scope s);
 bool is_scope_active(hk_scopes s);
 
-///
 bool has_hotkey_command(const std::string& id);
 
-/// adds a new wml hotkey to the list, but only if there is no hotkey with that id yet on the list.
-/// the object that is created here will be deleted in "delete_all_wml_hotkeys()"
+/**
+ * adds a new wml hotkey to the list, but only if there is no hotkey with that id yet on the list.
+ * the object that is created here will be deleted in "delete_all_wml_hotkeys()"
+ */
 void add_wml_hotkey(const std::string& id, const t_string& description, const config& default_hotkey);
 
-/// deletes all wml hotkeys, should be called after a game has ended
+/** deletes all wml hotkeys, should be called after a game has ended */
 void delete_all_wml_hotkeys();
-///removes a wml hotkey with the given id, returns true if the deletion was successful
+/** removes a wml hotkey with the given id, returns true if the deletion was successful */
 bool remove_wml_hotkey(const std::string& id);
 
 const std::string& get_description(const std::string& command);
@@ -337,6 +350,6 @@ void init_hotkey_commands();
 
 void clear_hotkey_commands();
 
-/// returns get_hotkey_command(command).id
+/** returns get_hotkey_command(command).id */
 HOTKEY_COMMAND get_id(const std::string& command);
 }
