@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "hotkey/hotkey_handler.hpp"
@@ -64,7 +65,7 @@ const game_state & play_controller::hotkey_handler::gamestate() const {
 bool play_controller::hotkey_handler::browse() const { return play_controller_.is_browsing(); }
 bool play_controller::hotkey_handler::linger() const { return play_controller_.is_lingering(); }
 
-const team & play_controller::hotkey_handler::viewing_team() const { return play_controller_.get_teams_const()[gui()->viewing_team()]; }
+const team & play_controller::hotkey_handler::viewing_team() const { return play_controller_.get_teams()[gui()->viewing_team()]; }
 bool play_controller::hotkey_handler::viewing_team_is_playing() const { return gui()->viewing_team() == gui()->playing_team(); }
 
 void play_controller::hotkey_handler::objectives(){
@@ -162,6 +163,10 @@ void play_controller::hotkey_handler::cycle_back_units(){
 	mouse_handler_.cycle_back_units(browse());
 }
 
+void play_controller::hotkey_handler::speak(){
+	menu_handler_.speak();
+}
+
 void play_controller::hotkey_handler::show_chat_log(){
 	menu_handler_.show_chat_log();
 }
@@ -210,15 +215,18 @@ void play_controller::hotkey_handler::toggle_accelerated_speed()
 {
 	preferences::set_turbo(!preferences::turbo());
 
+	display::announce_options ao;
+	ao.discard_previous = true;
+
 	if (preferences::turbo())
 	{
 		utils::string_map symbols;
 		symbols["hk"] = hotkey::get_names(hotkey::hotkey_command::get_command_by_command(hotkey::HOTKEY_ACCELERATED).command);
-		gui()->announce(_("Accelerated speed enabled!") + "\n" + VGETTEXT("(press $hk to disable)", symbols), font::NORMAL_COLOR);
+		gui()->announce(_("Accelerated speed enabled!") + "\n" + VGETTEXT("(press $hk to disable)", symbols), font::NORMAL_COLOR, ao);
 	}
 	else
 	{
-		gui()->announce(_("Accelerated speed disabled!"), font::NORMAL_COLOR);
+		gui()->announce(_("Accelerated speed disabled!"), font::NORMAL_COLOR, ao);
 	}
 }
 
@@ -262,7 +270,8 @@ bool play_controller::hotkey_handler::do_execute_command(const hotkey::hotkey_co
 		std::string name = cmd.command.substr(prefixlen);
 		const map_location& hex = mouse_handler_.get_last_hex();
 
-		return gamestate().get_wml_menu_items().fire_item(name, hex, gamestate().gamedata_, gamestate(), gamestate().board_.units_, !press);
+		return gamestate().get_wml_menu_items().fire_item(
+			name, hex, gamestate().gamedata_, gamestate(), play_controller_.get_units(), !press);
 	}
 	return command_executor::do_execute_command(cmd, index, press, release);
 }
@@ -304,6 +313,7 @@ bool play_controller::hotkey_handler::can_execute_command(const hotkey::hotkey_c
 	case hotkey::HOTKEY_QUIT_TO_DESKTOP:
 	case hotkey::HOTKEY_SEARCH:
 	case hotkey::HOTKEY_HELP:
+	case hotkey::HOTKEY_HELP_ABOUT_SAVELOAD:
 	case hotkey::HOTKEY_USER_CMD:
 	case hotkey::HOTKEY_CUSTOM_CMD:
 	case hotkey::HOTKEY_AI_FORMULA:
@@ -329,8 +339,8 @@ bool play_controller::hotkey_handler::can_execute_command(const hotkey::hotkey_c
 		return true;
 
 	case hotkey::HOTKEY_SURRENDER: {
-		size_t humans_notme_cnt = 0;
-		for(const auto& t : play_controller_.get_teams_const()) {
+		std::size_t humans_notme_cnt = 0;
+		for(const auto& t : play_controller_.get_teams()) {
 			if(t.is_network_human()) {
 				++humans_notme_cnt;
 			}
@@ -349,6 +359,9 @@ bool play_controller::hotkey_handler::can_execute_command(const hotkey::hotkey_c
 	case hotkey::HOTKEY_LOAD_GAME:
 		return !play_controller_.is_networked_mp(); // Can only load games if not in a network game
 
+	case hotkey::HOTKEY_SPEAK:
+	case hotkey::HOTKEY_SPEAK_ALLY:
+	case hotkey::HOTKEY_SPEAK_ALL:
 	case hotkey::HOTKEY_CHAT_LOG:
 		return true;
 
@@ -368,7 +381,7 @@ bool play_controller::hotkey_handler::can_execute_command(const hotkey::hotkey_c
 			menu_handler_.current_unit().valid() &&
 			!(menu_handler_.current_unit()->unrenamable()) &&
 			menu_handler_.current_unit()->side() == gui()->viewing_side() &&
-			play_controller_.get_teams_const()[menu_handler_.current_unit()->side() - 1].is_local_human();
+			play_controller_.get_teams()[menu_handler_.current_unit()->side() - 1].is_local_human();
 
 	default:
 		return false;
@@ -410,13 +423,13 @@ void play_controller::hotkey_handler::expand_autosaves(std::vector<config>& item
 
 		if(savegame::save_game_exists(name, comp_format)) {
 			newsaves.emplace_back(name + compression::format_extension(comp_format));
-			newitems.emplace_back(config {"label", _("Back to Turn ") + std::to_string(turn)});
+			newitems.emplace_back("label", _("Back to Turn ") + std::to_string(turn));
 		}
 	}
 
 	if(savegame::save_game_exists(start_name, comp_format)) {
 		newsaves.emplace_back(start_name + compression::format_extension(comp_format));
-		newitems.emplace_back(config {"label", _("Back to Start")});
+		newitems.emplace_back("label", _("Back to Start"));
 	}
 
 	// Make sure list doesn't get too long: keep top two, midpoint and bottom.
@@ -436,7 +449,7 @@ void play_controller::hotkey_handler::expand_wml_commands(std::vector<config>& i
 	std::vector<config> newitems;
 
 	gamestate().get_wml_menu_items().get_items(mouse_handler_.get_last_hex(), wml_commands_, newitems,
-		gamestate(), gamestate().gamedata_, gamestate().board_.units_);
+		gamestate(), gamestate().gamedata_, play_controller_.get_units());
 
 	// Replace this placeholder entry with available menu items.
 	items.insert(pos, newitems.begin(), newitems.end());
@@ -455,7 +468,7 @@ void play_controller::hotkey_handler::show_menu(const std::vector<config>& items
 		const hotkey::hotkey_command& command = hotkey::get_hotkey_command(id);
 
 		if(id == "wml" || (can_execute_command(command) && (!context_menu || in_context_menu(command.id)))) {
-			items.emplace_back(config {"id", id});
+			items.emplace_back("id", id);
 		}
 	}
 
@@ -492,8 +505,8 @@ bool play_controller::hotkey_handler::in_context_menu(hotkey::HOTKEY_COMMAND com
 
 		// A quick check to save us having to create the future map and
 		// possibly loop through all units.
-		if ( !play_controller_.get_map_const().is_keep(last_hex)  &&
-		     !play_controller_.get_map_const().is_castle(last_hex) )
+		if ( !play_controller_.get_map().is_keep(last_hex)  &&
+		     !play_controller_.get_map().is_castle(last_hex) )
 			return false;
 
 		wb::future_map future; /* lasts until method returns. */
@@ -541,5 +554,5 @@ hotkey::ACTION_STATE play_controller::hotkey_handler::get_action_state(hotkey::H
 
 void play_controller::hotkey_handler::load_autosave(const std::string& filename)
 {
-	throw savegame::load_game_exception(filename);
+	throw savegame::load_game_exception({savegame::save_index_class::default_saves_dir(), filename});
 }
