@@ -1,19 +1,21 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "hotkey/hotkey_handler_sp.hpp"
 
+#include "filesystem.hpp" // for get_saves_dir()
 #include "font/standard_colors.hpp"
 #include "formula/string_utils.hpp"
 #include "hotkey/hotkey_command.hpp"
@@ -241,7 +243,7 @@ bool playsingle_controller::hotkey_handler::can_execute_command(const hotkey::ho
 		case hotkey::HOTKEY_CREATE_UNIT:
 		case hotkey::HOTKEY_CHANGE_SIDE:
 		case hotkey::HOTKEY_KILL_UNIT:
-			return !events::commands_disabled && game_config::debug && play_controller_.get_map_const().on_board(mouse_handler_.get_last_hex()) && play_controller_.current_team().is_local();
+			return !events::commands_disabled && game_config::debug && play_controller_.get_map().on_board(mouse_handler_.get_last_hex()) && play_controller_.current_team().is_local();
 
 		case hotkey::HOTKEY_CLEAR_LABELS:
 			res = !is_observer();
@@ -249,7 +251,7 @@ bool playsingle_controller::hotkey_handler::can_execute_command(const hotkey::ho
 		case hotkey::HOTKEY_LABEL_TEAM_TERRAIN:
 		case hotkey::HOTKEY_LABEL_TERRAIN: {
 			const terrain_label *label = gui()->labels().get_label(mouse_handler_.get_last_hex());
-			res = !events::commands_disabled && play_controller_.get_map_const().on_board(mouse_handler_.get_last_hex())
+			res = !events::commands_disabled && play_controller_.get_map().on_board(mouse_handler_.get_last_hex())
 				&& !gui()->shrouded(mouse_handler_.get_last_hex())
 				&& !is_observer()
 				&& (!label || !label->immutable());
@@ -262,7 +264,7 @@ bool playsingle_controller::hotkey_handler::can_execute_command(const hotkey::ho
 			if( (menu_handler_.current_unit().valid())
 				&& (menu_handler_.current_unit()->move_interrupted()))
 				return true;
-			const unit_map::const_iterator i = play_controller_.get_units_const().find(mouse_handler_.get_selected_hex());
+			const unit_map::const_iterator i = play_controller_.get_units().find(mouse_handler_.get_selected_hex());
 			if (!i.valid()) return false;
 			return i->move_interrupted();
 		}
@@ -279,6 +281,7 @@ bool playsingle_controller::hotkey_handler::can_execute_command(const hotkey::ho
 		case hotkey::HOTKEY_WB_SUPPOSE_DEAD:
 		{
 			//@todo re-enable this once we figure out a decent UI for suppose_dead
+			//@todo when re-enabling this, change 'true' to 'false' in master_hotkey_list for this hotkey
 			return false;
 		}
 
@@ -295,7 +298,8 @@ bool playsingle_controller::hotkey_handler::can_execute_command(const hotkey::ho
 			return playsingle_controller_.get_replay_controller() && playsingle_controller_.get_replay_controller()->can_execute_command(cmd, index);
 		case hotkey::HOTKEY_REPLAY_EXIT:
 			return playsingle_controller_.is_replay() && (!playsingle_controller_.is_networked_mp() || resources::recorder->at_end());
-		default: return play_controller::hotkey_handler::can_execute_command(cmd, index);
+		default:
+			return play_controller::hotkey_handler::can_execute_command(cmd, index);
 	}
 	return res;
 }
@@ -306,7 +310,7 @@ void playsingle_controller::hotkey_handler::load_autosave(const std::string& fil
 	{
 		config savegame;
 		std::string error_log;
-		savegame::read_save_file(filename, savegame, &error_log);
+		savegame::read_save_file(filesystem::get_saves_dir(), filename, savegame, &error_log);
 
 		if(!error_log.empty() || savegame.child_or_empty("snapshot")["replay_pos"].to_int(-1) < 0 ) {
 			gui2::show_error_message(_("The file you have tried to load is corrupt: '") + error_log);
