@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -21,6 +22,7 @@
 #include <string>
 #include <boost/algorithm/string.hpp>
 
+class game_config_view;
 class config;
 namespace hotkey {
 
@@ -251,7 +253,7 @@ public:
 		keycode_ = keycode;
 	}
 
-	void set_text(std::string text)
+	void set_text(const std::string& text)
 	{
 		text_ = text;
 		boost::algorithm::to_lower(text_);
@@ -341,15 +343,6 @@ protected:
 	virtual bool bindings_equal_helper (hotkey_ptr other) const;
 };
 
-/**
- * @todo not implemented
- */
-class hotkey_joystick: public hotkey_base
-{
-protected:
-	int button_;
-};
-
 /**
  * Create and instantiate a hotkey from a config element.
  * @param cfg The config element to read for data.
@@ -398,7 +391,7 @@ const hotkey_ptr get_hotkey(const SDL_Event &event);
  * @param set_as_default Indicates whether the config struct should be treated as the
  * default game settings.
  */
-void load_hotkeys(const config& cfg, bool set_as_default = false);
+void load_hotkeys(const game_config_view& cfg, bool set_as_default = false);
 
 /**
  * Reset all hotkeys to the defaults.
