@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "image_modifications.hpp"
@@ -21,6 +22,7 @@
 #include "lexical_cast.hpp"
 #include "log.hpp"
 #include "serialization/string_utils.hpp"
+#include "team.hpp"
 
 #include "formula/formula.hpp"
 #include "formula/callable.hpp"
@@ -56,9 +58,9 @@ void modification_queue::pop()
 }
 
 /** Returns the number of elements in the queue. */
-size_t modification_queue::size() const
+std::size_t modification_queue::size() const
 {
-	size_t count = 0;
+	std::size_t count = 0;
 	for(const map_type::value_type& pair : priorities_) {
 		count += pair.second.size();
 	}
@@ -75,7 +77,7 @@ modification * modification_queue::top() const
 
 namespace {
 
-/// A function used to parse modification arguments
+/** A function used to parse modification arguments */
 using mod_parser = std::function<modification*(const std::string&)>;
 
 /** A map of all registered mod parsers
@@ -232,6 +234,8 @@ public:
 	{
 		add_input(inputs, "x");
 		add_input(inputs, "y");
+		add_input(inputs, "u");
+		add_input(inputs, "v");
 		add_input(inputs, "red");
 		add_input(inputs, "green");
 		add_input(inputs, "blue");
@@ -259,6 +263,10 @@ public:
 			return variant(w);
 		} else if(key == "height") {
 			return variant(h);
+		} else if(key == "u") {
+			return variant(p.x / static_cast<float>(w));
+		} else if(key == "v") {
+			return variant(p.y / static_cast<float>(h));
 		}
 
 		return variant();
@@ -631,23 +639,10 @@ REGISTER_MOD_PARSER(TC, args)
 		return nullptr;
 	}
 
-	int side_n = lexical_cast_default<int>(params[0], -1);
-	std::string team_color;
+	const int side_n = lexical_cast_default<int>(params[0], -1);
 	if(side_n < 1) {
-		ERR_DP << "invalid team (" << side_n
-		       << ") passed to the ~TC() function\n";
+		ERR_DP << "Invalid side (" << side_n << ") passed to the ~TC() function\n";
 		return nullptr;
-	} else if(side_n <= static_cast<int>(image::get_team_colors().size())) {
-		team_color = image::get_team_colors()[side_n - 1];
-	} else {
-		// This side is not initialized; use default "n"
-		try {
-			team_color = std::to_string(side_n);
-		} catch(const bad_lexical_cast&) {
-			ERR_DP << "bad things happen" << std::endl;
-
-			return nullptr;
-		}
 	}
 
 	//
@@ -663,7 +658,7 @@ REGISTER_MOD_PARSER(TC, args)
 
 	color_range_map rc_map;
 	try {
-		const color_range& new_color = game_config::color_info(team_color);
+		const color_range& new_color = team::get_side_color_range(side_n);
 		const std::vector<color_t>& old_color = game_config::tc_info(params[1]);
 
 		rc_map = recolor_range(new_color,old_color);
@@ -726,7 +721,7 @@ REGISTER_MOD_PARSER(PAL, args)
 		const std::vector<color_t>& old_palette = game_config::tc_info(remap_params[0]);
 		const std::vector<color_t>& new_palette =game_config::tc_info(remap_params[1]);
 
-		for(size_t i = 0; i < old_palette.size() && i < new_palette.size(); ++i) {
+		for(std::size_t i = 0; i < old_palette.size() && i < new_palette.size(); ++i) {
 			rc_map[old_palette[i]] = new_palette[i];
 		}
 
@@ -756,7 +751,7 @@ REGISTER_MOD_PARSER(FL, args)
 REGISTER_MOD_PARSER(ROTATE, args)
 {
 	const std::vector<std::string>& slice_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const size_t s = slice_params.size();
+	const std::size_t s = slice_params.size();
 
 	switch(s) {
 		case 0:
@@ -913,7 +908,7 @@ REGISTER_MOD_PARSER(CHAN, args)
 REGISTER_MOD_PARSER(CS, args)
 {
 	std::vector<std::string> const factors = utils::split(args, ',');
-	const size_t s = factors.size();
+	const std::size_t s = factors.size();
 
 	if(s == 0) {
 		ERR_DP << "no arguments passed to the ~CS() function" << std::endl;
@@ -968,7 +963,7 @@ REGISTER_MOD_PARSER(BLEND, args)
 REGISTER_MOD_PARSER(CROP, args)
 {
 	const std::vector<std::string>& slice_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const size_t s = slice_params.size();
+	const std::size_t s = slice_params.size();
 
 	if(s == 0 || (s == 1 && slice_params[0].empty())) {
 		ERR_DP << "no arguments passed to the ~CROP() function" << std::endl;
@@ -1004,7 +999,7 @@ static bool check_image(const image::locator& img, std::stringstream & message)
 REGISTER_MOD_PARSER(BLIT, args)
 {
 	std::vector<std::string> param = utils::parenthetical_split(args, ',');
-	const size_t s = param.size();
+	const std::size_t s = param.size();
 
 	if(s == 0 || (s == 1 && param[0].empty())){
 		ERR_DP << "no arguments passed to the ~BLIT() function" << std::endl;
@@ -1037,7 +1032,7 @@ REGISTER_MOD_PARSER(BLIT, args)
 REGISTER_MOD_PARSER(MASK, args)
 {
 	std::vector<std::string> param = utils::parenthetical_split(args, ',');
-	const size_t s = param.size();
+	const std::size_t s = param.size();
 
 	if(s == 0 || (s == 1 && param[0].empty())){
 		ERR_DP << "no arguments passed to the ~MASK() function" << std::endl;
@@ -1083,7 +1078,7 @@ REGISTER_MOD_PARSER(L, args)
 REGISTER_MOD_PARSER(SCALE, args)
 {
 	const std::vector<std::string>& scale_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const size_t s = scale_params.size();
+	const std::size_t s = scale_params.size();
 
 	if(s == 0 || (s == 1 && scale_params[0].empty())) {
 		ERR_DP << "no arguments passed to the ~SCALE() function" << std::endl;
@@ -1104,7 +1099,7 @@ REGISTER_MOD_PARSER(SCALE, args)
 REGISTER_MOD_PARSER(SCALE_SHARP, args)
 {
 	const std::vector<std::string>& scale_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const size_t s = scale_params.size();
+	const std::size_t s = scale_params.size();
 
 	if(s == 0 || (s == 1 && scale_params[0].empty())) {
 		ERR_DP << "no arguments passed to the ~SCALE_SHARP() function" << std::endl;
@@ -1125,7 +1120,7 @@ REGISTER_MOD_PARSER(SCALE_SHARP, args)
 REGISTER_MOD_PARSER(SCALE_INTO, args)
 {
 	const std::vector<std::string>& scale_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const size_t s = scale_params.size();
+	const std::size_t s = scale_params.size();
 
 	if(s == 0 || (s == 1 && scale_params[0].empty())) {
 		ERR_DP << "no arguments passed to the ~SCALE_INTO() function" << std::endl;
@@ -1146,7 +1141,7 @@ REGISTER_MOD_PARSER(SCALE_INTO, args)
 REGISTER_MOD_PARSER(SCALE_INTO_SHARP, args)
 {
 	const std::vector<std::string>& scale_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const size_t s = scale_params.size();
+	const std::size_t s = scale_params.size();
 
 	if(s == 0 || (s == 1 && scale_params[0].empty())) {
 		ERR_DP << "no arguments passed to the ~SCALE_INTO_SHARP() function" << std::endl;
