@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "filesystem.hpp"
@@ -19,6 +20,7 @@
 #include "preferences/general.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
+#include "game_config_manager.hpp"
 
 #include <stdexcept>
 #include <clocale>
@@ -89,6 +91,16 @@ const t_string& symbol_table::operator[](const char* key) const
 	return (*this)[std::string(key)];
 }
 
+utils::string_map::const_iterator symbol_table::find(const std::string& key) const
+{
+	return strings_.find(key);
+}
+
+utils::string_map::const_iterator symbol_table::end() const
+{
+	return strings_.end();
+}
+
 bool load_language_list()
 {
 	config cfg;
@@ -112,13 +124,13 @@ bool load_language_list()
 	return true;
 }
 
-language_list get_languages()
+language_list get_languages(bool all)
 {
 	// We sort every time, the local might have changed which can modify the
 	// sort order.
 	std::sort(known_languages.begin(), known_languages.end());
 
-	if(min_translation_percent == 0) {
+	if(all || min_translation_percent == 0) {
 		return known_languages;
 	}
 
@@ -133,6 +145,67 @@ void set_min_translation_percent(int percent) {
 	min_translation_percent = percent;
 }
 
+#ifdef _WIN32
+// Simplified translation table from unix locale symbols to win32 locale strings
+static const std::map<std::string, std::string> win32_locales_map = {
+	{ "af", "Afrikaans" },
+	{ "ang", "C" },
+	{ "ar", "Arabic" },
+	{ "bg", "Bulgarian" },
+	{ "ca", "Catalan" },
+	{ "cs", "Czech" },
+	{ "da", "Danish" },
+	{ "de", "German" },
+	{ "el", "Greek" },
+	{ "en", "English" },
+	{ "eo", "C" },
+	{ "es", "Spanish" },
+	{ "et", "Estonian" },
+	{ "eu", "Basque" },
+	{ "fi", "Finnish" },
+	{ "fr", "French" },
+	{ "fur", "C" },
+	{ "ga", "Irish_Ireland" }, // Yes, "Irish" alone does not work
+	{ "gl", "Galician" },
+	{ "he", "Hebrew" },
+	{ "hr", "Croatian" },
+	{ "hu", "Hungarian" },
+	{ "id", "Indonesian" },
+	{ "is", "Icelandic" },
+	{ "it", "Italian" },
+	{ "ja", "Japanese" },
+	{ "ko", "Korean" },
+	{ "la", "C" },
+	{ "lt", "Lithuanian" },
+	{ "lv", "Latvian" },
+	{ "mk", "Macedonian" },
+	{ "mr", "C" },
+	{ "nb", "Norwegian" },
+	{ "nl", "Dutch" },
+	{ "pl", "Polish" },
+	{ "pt", "Portuguese" },
+	{ "racv", "C" },
+	{ "ro", "Romanian" },
+	{ "ru", "Russian" },
+	{ "sk", "Slovak" },
+	{ "sl", "Slovenian" },
+	{ "sr", "Serbian" },
+	{ "sv", "Swedish" },
+	{ "tl", "Filipino" },
+	{ "tr", "Turkish" },
+	{ "uk", "Ukrainian" },
+	{ "vi", "Vietnamese" },
+	{ "zh", "Chinese" },
+};
+
+static const std::string& posix_locale_to_win32(const std::string& posix)
+{
+	auto it = win32_locales_map.find(posix);
+	return it != win32_locales_map.end() ? it->second : posix;
+}
+
+#endif
+
 static void wesnoth_setlocale(int category, const std::string& slocale,
 	std::vector<std::string> const *alternates)
 {
@@ -155,15 +228,8 @@ static void wesnoth_setlocale(int category, const std::string& slocale,
 #endif
 
 #ifdef _WIN32
-	std::string win_locale(locale, 0, 2);
-	#include "language_win32.ii"
-	//if(category == LC_MESSAGES) {
-	//	SetEnvironmentVariableA("LANG", win_locale.c_str());
-	//	std::string env = "LANGUAGE=" + locale;
-	//	_putenv(env.c_str());
-	//	return;
-	//}
-	locale = win_locale;
+	std::string lang_code{locale, 0, locale.find_first_of("_@.")};
+	locale = posix_locale_to_win32(lang_code);
 #endif
 
 	char *res = nullptr;
@@ -281,9 +347,7 @@ const language_def& get_locale()
 #if 0
 	const char* const locale = getenv("LANG");
 	#ifdef _WIN32
-	    std::string win_locale = locale
-		#include "language_win32.ii"
-		return win_locale;
+	    return posix_locale_to_win32(locale);
 	#endif
 	if(locale != nullptr && strlen(locale) >= 2) {
 		//we can't pass pointers into the string to the std::string
@@ -300,7 +364,7 @@ const language_def& get_locale()
 	return known_languages[0];
 }
 
-void init_textdomains(const config& cfg)
+void init_textdomains(const game_config_view& cfg)
 {
 	for (const config &t : cfg.child_range("textdomain"))
 	{
@@ -323,7 +387,7 @@ void init_textdomains(const config& cfg)
 	}
 }
 
-bool init_strings(const config& cfg)
+bool init_strings(const game_config_view& cfg)
 {
 	languages_.clear();
 	for (const config &l : cfg.child_range("language")) {
