@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -45,9 +46,7 @@
 #define __LONG_LONG_SUPPORTED
 #endif
 
-#include "global.hpp"
-
-#include "utils/type_trait_aliases.hpp"
+#include <optional>
 
 #include <cstdlib>
 #include <limits>
@@ -55,9 +54,6 @@
 #include <sstream>
 #include <type_traits>
 
-#include <boost/mpl/set.hpp>
-#include <boost/optional.hpp>
-
 #define DEBUG_THROW(id)
 #endif
 
@@ -93,7 +89,7 @@ namespace implementation {
 template<typename To, typename From>
 inline To lexical_cast(From value)
 {
-	return implementation::lexical_caster<To, From>().operator()(value, boost::none);
+	return implementation::lexical_caster<To, From>().operator()(value, std::nullopt);
 }
 
 /**
@@ -116,7 +112,7 @@ inline To lexical_cast_default(From value, To fallback = To())
 /** Thrown when a lexical_cast fails. */
 struct bad_lexical_cast : std::exception
 {
-	const char* what() const NOEXCEPT
+	const char* what() const noexcept
 	{
 		return "bad_lexical_cast";
 	}
@@ -143,7 +139,7 @@ template<
 >
 struct lexical_caster
 {
-	To operator()(From value, boost::optional<To> fallback) const
+	To operator()(From value, std::optional<To> fallback) const
 	{
 		DEBUG_THROW("generic");
 
@@ -151,7 +147,7 @@ struct lexical_caster
 		std::stringstream sstr;
 
 		if(!(sstr << value && sstr >> result)) {
-			if(fallback) { return fallback.get(); }
+			if(fallback) { return *fallback; }
 
 			throw bad_lexical_cast();
 		} else {
@@ -171,10 +167,10 @@ struct lexical_caster<
 	  std::string
 	, From
 	, void
-	, utils::enable_if_t<std::is_integral<utils::remove_pointer_t<From>>::value>
+	, std::enable_if_t<std::is_integral_v<std::remove_pointer_t<From>>>
 >
 {
-	std::string operator()(From value, boost::optional<std::string>) const
+	std::string operator()(From value, std::optional<std::string>) const
 	{
 		DEBUG_THROW("specialized - To std::string - From integral (pointer)");
 
@@ -196,15 +192,15 @@ struct lexical_caster<
 	  long long
 	, From
 	, void
-	, utils::enable_if_t<boost::mpl::has_key<boost::mpl::set<char*, const char*> , From>::value>
+	, std::enable_if_t<std::is_same_v<From, const char*> || std::is_same_v<From, char*>>
 	>
 {
-	long long operator()(From value, boost::optional<long long> fallback) const
+	long long operator()(From value, std::optional<long long> fallback) const
 	{
 		DEBUG_THROW("specialized - To long long - From (const) char*");
 
 		if(fallback) {
-			return lexical_cast_default<long long>(std::string(value), fallback.get());
+			return lexical_cast_default<long long>(std::string(value), *fallback);
 		} else {
 			return lexical_cast<long long>(std::string(value));
 		}
@@ -224,7 +220,7 @@ struct lexical_caster<
 	, std::string
 	>
 {
-	long long operator()(const std::string& value, boost::optional<long long> fallback) const
+	long long operator()(const std::string& value, std::optional<long long> fallback) const
 	{
 		DEBUG_THROW("specialized - To long long - From std::string");
 
@@ -235,7 +231,7 @@ struct lexical_caster<
 		}
 
 		if(fallback) {
-			return fallback.get();
+			return *fallback;
 		} else {
 			throw bad_lexical_cast();
 		}
@@ -251,16 +247,16 @@ template <class To, class From>
 struct lexical_caster<
 	  To
 	, From
-	, utils::enable_if_t<std::is_integral<To>::value && std::is_signed<To>::value && !std::is_same<To, long long>::value>
-	, utils::enable_if_t<boost::mpl::has_key<boost::mpl::set<char*, const char*> , From>::value>
+	, std::enable_if_t<std::is_integral_v<To> && std::is_signed_v<To> && !std::is_same_v<To, long long>>
+	, std::enable_if_t<std::is_same_v<From, const char*> || std::is_same_v<From, char*>>
 	>
 {
-	To operator()(From value, boost::optional<To> fallback) const
+	To operator()(From value, std::optional<To> fallback) const
 	{
 		DEBUG_THROW("specialized - To signed - From (const) char*");
 
 		if(fallback) {
-			return lexical_cast_default<To>(std::string(value), fallback.get());
+			return lexical_cast_default<To>(std::string(value), *fallback);
 		} else {
 			return lexical_cast<To>(std::string(value));
 		}
@@ -276,10 +272,10 @@ template <class To>
 struct lexical_caster<
 	  To
 	, std::string
-	, utils::enable_if_t<std::is_integral<To>::value && std::is_signed<To>::value && !std::is_same<To, long long>::value>
+	, std::enable_if_t<std::is_integral_v<To> && std::is_signed_v<To> && !std::is_same_v<To, long long>>
 	>
 {
-	To operator()(const std::string& value, boost::optional<To> fallback) const
+	To operator()(const std::string& value, std::optional<To> fallback) const
 	{
 		DEBUG_THROW("specialized - To signed - From std::string");
 
@@ -293,7 +289,7 @@ struct lexical_caster<
 		}
 
 		if(fallback) {
-			return fallback.get();
+			return *fallback;
 		} else {
 			throw bad_lexical_cast();
 		}
@@ -309,16 +305,16 @@ template <class To, class From>
 struct lexical_caster<
 	  To
 	, From
-	, utils::enable_if_t<std::is_floating_point<To>::value>
-	, utils::enable_if_t<boost::mpl::has_key<boost::mpl::set<char*, const char*> , From>::value>
+	, std::enable_if_t<std::is_floating_point_v<To>>
+	, std::enable_if_t<std::is_same_v<From, const char*> || std::is_same_v<From, char*>>
 	>
 {
-	To operator()(From value, boost::optional<To> fallback) const
+	To operator()(From value, std::optional<To> fallback) const
 	{
 		DEBUG_THROW("specialized - To floating point - From (const) char*");
 
 		if(fallback) {
-			return lexical_cast_default<To>(std::string(value), fallback.get());
+			return lexical_cast_default<To>(std::string(value), *fallback);
 		} else {
 			return lexical_cast<To>(std::string(value));
 		}
@@ -334,17 +330,17 @@ template <class To>
 struct lexical_caster<
 	  To
 	, std::string
-	, utils::enable_if_t<std::is_floating_point<To>::value>
+	, std::enable_if_t<std::is_floating_point_v<To>>
 	>
 {
-	To operator()(const std::string& value, boost::optional<To> fallback) const
+	To operator()(const std::string& value, std::optional<To> fallback) const
 	{
 		DEBUG_THROW("specialized - To floating point - From std::string");
 
 		// Explicitly reject hexadecimal values. Unit tests of the config class require that.
 		if(value.find_first_of("Xx") != std::string::npos) {
 			if(fallback) {
-				return fallback.get();
+				return *fallback;
 			} else {
 				throw bad_lexical_cast();
 			}
@@ -360,7 +356,7 @@ struct lexical_caster<
 		}
 
 		if(fallback) {
-			return fallback.get();
+			return *fallback;
 		} else {
 			throw bad_lexical_cast();
 		}
@@ -379,16 +375,16 @@ struct lexical_caster<
 	  unsigned long long
 	, From
 	, void
-	, utils::enable_if_t<boost::mpl::has_key<boost::mpl::set<char*, const char*> , From>::value>
+	, std::enable_if_t<std::is_same_v<From, const char*> || std::is_same_v<From, char*>>
 	>
 {
-	unsigned long long operator()(From value, boost::optional<unsigned long long> fallback) const
+	unsigned long long operator()(From value, std::optional<unsigned long long> fallback) const
 	{
 		DEBUG_THROW(
 				"specialized - To unsigned long long - From (const) char*");
 
 		if(fallback) {
-			return lexical_cast_default<unsigned long long>(std::string(value), fallback.get());
+			return lexical_cast_default<unsigned long long>(std::string(value), *fallback);
 		} else {
 			return lexical_cast<unsigned long long>(std::string(value));
 		}
@@ -408,7 +404,7 @@ struct lexical_caster<
 	, std::string
 	>
 {
-	unsigned long long operator()(const std::string& value, boost::optional<unsigned long long> fallback) const
+	unsigned long long operator()(const std::string& value, std::optional<unsigned long long> fallback) const
 	{
 		DEBUG_THROW("specialized - To unsigned long long - From std::string");
 
@@ -419,7 +415,7 @@ struct lexical_caster<
 		}
 
 		if(fallback) {
-			return fallback.get();
+			return *fallback;
 		} else {
 			throw bad_lexical_cast();
 		}
@@ -435,16 +431,16 @@ template <class To, class From>
 struct lexical_caster<
 	  To
 	, From
-	, utils::enable_if_t<std::is_unsigned<To>::value && !std::is_same<To, unsigned long long>::value>
-	, utils::enable_if_t<boost::mpl::has_key<boost::mpl::set<char*, const char*> , From>::value>
+	, std::enable_if_t<std::is_unsigned_v<To> && !std::is_same_v<To, unsigned long long>>
+	, std::enable_if_t<std::is_same_v<From, const char*> || std::is_same_v<From, char*>>
 	>
 {
-	To operator()(From value, boost::optional<To> fallback) const
+	To operator()(From value, std::optional<To> fallback) const
 	{
 		DEBUG_THROW("specialized - To unsigned - From (const) char*");
 
 		if(fallback) {
-			return lexical_cast_default<To>(std::string(value), fallback.get());
+			return lexical_cast_default<To>(std::string(value), *fallback);
 		} else {
 			return lexical_cast<To>(std::string(value));
 		}
@@ -460,10 +456,10 @@ template <class To>
 struct lexical_caster<
 	  To
 	, std::string
-	, utils::enable_if_t<std::is_unsigned<To>::value>
+	, std::enable_if_t<std::is_unsigned_v<To>>
 	>
 {
-	To operator()(const std::string& value, boost::optional<To> fallback) const
+	To operator()(const std::string& value, std::optional<To> fallback) const
 	{
 		DEBUG_THROW("specialized - To unsigned - From std::string");
 
@@ -478,13 +474,52 @@ struct lexical_caster<
 		}
 
 		if(fallback) {
-			return fallback.get();
+			return *fallback;
 		} else {
 			throw bad_lexical_cast();
 		}
 	}
 };
 
+/**
+ * Specialized conversion class.
+ *
+ * Specialized for returning a bool from a std::string.
+ * @note is specialized to silence C4804 from MSVC.
+ */
+template <>
+struct lexical_caster<bool, std::string>
+{
+	bool operator()(const std::string& value, std::optional<bool>) const
+	{
+		DEBUG_THROW("specialized - To bool - From std::string");
+
+		return value == "1";
+	}
+};
+
+/**
+ * Specialized conversion class.
+ *
+ * Specialized for returning a bool from a (const) char*.
+ * @note is specialized to silence C4804 from MSVC.
+ */
+template <class From>
+struct lexical_caster<
+	  bool
+	, From
+	, void
+	, std::enable_if_t<std::is_same_v<From, const char*> || std::is_same_v<From, char*>>
+	>
+{
+	bool operator()(From value, std::optional<bool>) const
+	{
+		DEBUG_THROW("specialized - To bool - From (const) char*");
+
+		return lexical_cast<bool>(std::string(value));
+	}
+};
+
 } // namespace implementation
 
 #endif
