@@ -1,9 +1,10 @@
 /*
-   By Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2015 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   The contents of this file are placed in the public domain.
- */
+	The contents of this file are placed in the public domain.
+*/
 
 #pragma once
 
@@ -25,7 +26,7 @@ public:
 	{
 	}
 
-	virtual ~libc_error() NOEXCEPT
+	virtual ~libc_error() noexcept
 	{
 	}
 
@@ -42,7 +43,7 @@ public:
 	}
 
 	/** Returns an explanatory string describing the exception. */
-	const char* what() const NOEXCEPT
+	const char* what() const noexcept
 	{
 		return msg_.c_str();
 	}
