@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-                 2004 - 2015 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2004 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -21,14 +22,11 @@
 
 #include "log.hpp"
 
-#include <boost/date_time.hpp>
-
 #include <map>
 #include <sstream>
 #include <ctime>
-
-#include "global.hpp"
-#include "utils/io.hpp"
+#include <mutex>
+#include <iomanip>
 
 namespace {
 
@@ -45,8 +43,8 @@ static std::ostream null_ostream(new null_streambuf);
 static int indent = 0;
 static bool timestamp = true;
 static bool precise_timestamp = false;
+static std::mutex log_mutex;
 
-static boost::posix_time::time_facet facet("%Y%m%d %H:%M:%S%F ");
 static std::ostream *output_stream = nullptr;
 
 static std::ostream& output()
@@ -173,18 +171,18 @@ bool broke_strict() {
 	return strict_threw_;
 }
 
-std::string get_timestamp(const time_t& t, const std::string& format) {
+std::string get_timestamp(const std::time_t& t, const std::string& format) {
 	std::ostringstream ss;
 
-	ss << utils::put_time(std::localtime(&t), format.c_str());
+	ss << std::put_time(std::localtime(&t), format.c_str());
 
 	return ss.str();
 }
-std::string get_timespan(const time_t& t) {
+std::string get_timespan(const std::time_t& t) {
 	std::ostringstream sout;
 	// There doesn't seem to be any library function for this
-	const time_t minutes = t / 60;
-	const time_t days = minutes / 60 / 24;
+	const std::time_t minutes = t / 60;
+	const std::time_t days = minutes / 60 / 24;
 	if(t <= 0) {
 		sout << "expired";
 	} else if(minutes == 0) {
@@ -197,18 +195,19 @@ std::string get_timespan(const time_t& t) {
 	return sout.str();
 }
 
-static void print_precise_timestamp(std::ostream & out) NOEXCEPT
+static void print_precise_timestamp(std::ostream& out) noexcept
 {
 	try {
-		facet.put(
-			std::ostreambuf_iterator<char>(out),
-			out,
-			' ',
-			boost::posix_time::microsec_clock::local_time());
+		int64_t micros = std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::high_resolution_clock::now().time_since_epoch()).count();
+		std::time_t seconds = micros/1'000'000;
+		int fractional = micros-(seconds*1'000'000);
+		char c = out.fill('0');
+		out << std::put_time(std::localtime(&seconds), "%Y%m%d %H:%M:%S") << "." << std::setw(6) << fractional << ' ';
+		out.fill(c);
 	} catch(...) {}
 }
 
-std::ostream &logger::operator()(const log_domain& domain, bool show_names, bool do_indent) const
+log_in_progress logger::operator()(const log_domain& domain, bool show_names, bool do_indent) const
 {
 	if (severity_ > domain.domain_->second) {
 		return null_ostream;
@@ -219,59 +218,78 @@ std::ostream &logger::operator()(const log_domain& domain, bool show_names, bool
 			std::cerr << ss.str() << std::endl;
 			strict_threw_ = true;
 		}
-		std::ostream& stream = output();
+		log_in_progress stream = output();
 		if(do_indent) {
-			for(int i = 0; i != indent; ++i)
-				stream << "  ";
-			}
+			stream.set_indent(indent);
+		}
 		if (timestamp) {
-			if(precise_timestamp) {
-				print_precise_timestamp(stream);
-			} else {
-				stream << get_timestamp(time(nullptr));
-			}
+			stream.enable_timestamp();
 		}
 		if (show_names) {
-			stream << name_ << ' ' << domain.domain_->first << ": ";
+			stream.set_prefix(formatter() << name_ << ' ' << domain.domain_->first << ": ");
 		}
 		return stream;
 	}
 }
 
-void scope_logger::do_log_entry(const log_domain& domain, const std::string& str) NOEXCEPT
+log_in_progress::log_in_progress(std::ostream& stream)
+	: stream_(stream)
+{}
+
+void log_in_progress::operator|(formatter&& message)
+{
+	std::lock_guard lock(log_mutex);
+	for(int i = 0; i < indent; ++i)
+		stream_ << "  ";
+	if(timestamp_) {
+		if(precise_timestamp) {
+			print_precise_timestamp(stream_);
+		} else {
+			stream_ << get_timestamp(std::time(nullptr));
+		}
+	}
+	stream_ << prefix_ << message.str();
+}
+
+void log_in_progress::set_indent(int level) {
+	indent_ = level;
+}
+
+void log_in_progress::enable_timestamp() {
+	timestamp_ = true;
+}
+
+void log_in_progress::set_prefix(const std::string& prefix) {
+	prefix_ = prefix;
+}
+
+void scope_logger::do_log_entry(const std::string& str) noexcept
 {
-	output_ = &debug()(domain, false, true);
 	str_ = str;
 	try {
-		ticks_ = boost::posix_time::microsec_clock::local_time();
+		ticks_ = std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::high_resolution_clock::now().time_since_epoch()).count();
 	} catch(...) {}
-	(*output_) << "{ BEGIN: " << str_ << "\n";
+	debug()(domain_, false, true) | formatter() << "{ BEGIN: " << str_ << "\n";
 	++indent;
 }
 
-void scope_logger::do_log_exit() NOEXCEPT
+void scope_logger::do_log_exit() noexcept
 {
 	long ticks = 0;
 	try {
-		ticks = (boost::posix_time::microsec_clock::local_time() - ticks_).total_milliseconds();
+		ticks = std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::high_resolution_clock::now().time_since_epoch()).count() - ticks_;
 	} catch(...) {}
 	--indent;
-	do_indent();
-	if (timestamp) (*output_) << get_timestamp(time(nullptr));
-	(*output_) << "} END: " << str_ << " (took " << ticks << "ms)\n";
+	auto output = debug()(domain_, false, true);
+	output.set_indent(indent);
+	if(timestamp) output.enable_timestamp();
+	output | formatter() << "} END: " << str_ << " (took " << ticks << "ms)\n";
 }
 
-void scope_logger::do_indent() const
-{
-	for(int i = 0; i != indent; ++i)
-		(*output_) << "  ";
-}
-
-std::stringstream& wml_error()
+std::stringstream& log_to_chat()
 {
 	static std::stringstream lg;
 	return lg;
 }
 
 } // end namespace lg
-
