@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 // For some reason, it became necessary to include this before the header
@@ -28,8 +29,6 @@
 
 #include <boost/algorithm/string/predicate.hpp>
 
-#include "utils/io.hpp"
-
 #ifndef UNICODE
 #define UNICODE
 #endif
@@ -44,6 +43,16 @@ static lg::log_domain log_setup("logsetup");
 #define LOG_LS LOG_STREAM(info,  log_setup)
 #define DBG_LS LOG_STREAM(debug, log_setup)
 
+namespace filesystem
+{
+
+std::string get_logs_dir()
+{
+	return filesystem::get_user_data_dir() + "/logs";
+}
+
+}
+
 namespace lg
 {
 
@@ -86,7 +95,7 @@ void rotate_logs(const std::string& log_dir)
 
 	std::sort(files.begin(), files.end());
 
-	for(size_t j = 0; j < files.size() - max_logs; ++j) {
+	for(std::size_t j = 0; j < files.size() - max_logs; ++j) {
 		const std::string path = log_dir + '/' + files[j];
 		LOG_LS << "rotate_logs(): delete " << path << '\n';
 		if(!filesystem::delete_file(path)) {
@@ -111,8 +120,8 @@ std::string unique_log_filename()
 
 	o << log_file_prefix;
 
-	const time_t cur = time(nullptr);
-	o << utils::put_time(std::localtime(&cur), "%Y%m%d-%H%M%S-");
+	const std::time_t cur = std::time(nullptr);
+	o << std::put_time(std::localtime(&cur), "%Y%m%d-%H%M%S-");
 
 	o << GetCurrentProcessId() << log_file_suffix;
 
@@ -151,7 +160,7 @@ void log_init_panic(const std::string& msg)
 
 	// It may seem excessive to quit over something like this, but it's a good
 	// indicator of possible configuration issues with the user data dir that
-	// may cause much weirder symptoms later (see http://r.wesnoth.org/t42970
+	// may cause much weirder symptoms later (see https://r.wesnoth.org/t42970
 	// for an example).
 	exit(1);
 }
@@ -424,8 +433,12 @@ void log_file_manager::enable_native_console_output()
 		LOG_LS << "Allocated own console.\n";
 		created_wincon_ = true;
 	} else {
-		ERR_LS << "Console attachment or allocation failed!\n";
-		return;
+		// Wine as of version 4.21 just goes ERROR_ACCESS_DENIED when trying
+		// to allocate a console for a GUI subsystem application. We can ignore
+		// this since the user purportedly knows what they're doing and if they
+		// get radio silence from Wesnoth and no log files they'll realize that
+		// something went wrong.
+		WRN_LS << "Cannot attach or allocate a console, continuing anyway (is this Wine?)\n";
 	}
 
 	DBG_LS << "stderr to console\n";
@@ -463,12 +476,19 @@ std::string log_file_path()
 	return "";
 }
 
-void early_log_file_setup()
+static bool disable_redirect;
+
+void early_log_file_setup(bool disable)
 {
 	if(lfm) {
 		return;
 	}
 
+	if(disable) {
+		disable_redirect = true;
+		return;
+	}
+
 	lfm.reset(new log_file_manager());
 }
 
@@ -489,8 +509,9 @@ bool using_own_console()
 
 void finish_log_file_setup()
 {
+	if(disable_redirect) return;
 	// Make sure the LFM is actually set up just in case.
-	early_log_file_setup();
+	early_log_file_setup(false);
 
 	if(lfm->console_enabled()) {
 		// Nothing to do if running in console mode.
@@ -504,7 +525,7 @@ void finish_log_file_setup()
 		return;
 	}
 
-	const std::string log_dir = filesystem::get_user_data_dir() + "/logs";
+	const std::string log_dir = filesystem::get_logs_dir();
 	if(!filesystem::file_exists(log_dir) && !filesystem::make_directory(log_dir)) {
 		log_init_panic(std::string("Could not create logs directory at ") +
 					   log_dir + ".");
