@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2011 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2011 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "lua_jailbreak_exception.hpp"
@@ -18,7 +19,7 @@
 
 lua_jailbreak_exception *lua_jailbreak_exception::jailbreak_exception = nullptr;
 
-void lua_jailbreak_exception::store() const NOEXCEPT
+void lua_jailbreak_exception::store() const noexcept
 {
 	/*
 	 * It should not be  possible to call this function with an exception still
@@ -54,7 +55,7 @@ void lua_jailbreak_exception::rethrow()
 	assert(false);
 }
 
-void lua_jailbreak_exception::clear() NOEXCEPT
+void lua_jailbreak_exception::clear() noexcept
 {
 	delete jailbreak_exception;
 	jailbreak_exception = nullptr;
