@@ -1,16 +1,17 @@
 /*
-Copyright (C) 2006 - 2018 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
-Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -89,8 +90,8 @@ private:
 	void advance_to_arg(unsigned n) const
 	{
 		while (n < args.size() && !args_end) {
-			size_t first_space = str_.find_first_of(' ', args.back());
-			size_t next_arg_begin = str_.find_first_not_of(' ', first_space);
+			std::size_t first_space = str_.find_first_of(' ', args.back());
+			std::size_t next_arg_begin = str_.find_first_not_of(' ', first_space);
 			if (next_arg_begin != std::string::npos) {
 				args.push_back(next_arg_begin);
 			}
@@ -100,7 +101,7 @@ private:
 		}
 	}
 	std::string str_;
-	mutable std::vector<size_t> args;
+	mutable std::vector<std::size_t> args;
 	mutable bool args_end;
 };
 
@@ -127,9 +128,9 @@ public:
 		command_handler handler;
 		std::string help; //long help text
 		std::string usage; //only args info
-		std::string flags;
-		explicit command(command_handler h, const std::string help = "",
-			const std::string& usage = "", const std::string flags = "")
+		std::string flags; //< see implementation of get_command_flags_description() for meaning of flags
+		explicit command(command_handler h, const std::string& help = "",
+			const std::string& usage = "", const std::string& flags = "")
 			: handler(h), help(help), usage(usage), flags(flags)
 		{
 		}
@@ -354,7 +355,7 @@ protected:
 		help_on_unknown_ = value;
 	}
 	//this is display-only
-	static void set_cmd_prefix(std::string value)
+	static void set_cmd_prefix(const std::string& value)
 	{
 		cmd_prefix_ = value;
 	}
