@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2006 - 2018 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "mouse_events.hpp"
@@ -21,7 +22,6 @@
 #include "config.hpp"                        // for config
 #include "cursor.hpp"                        // for set, CURSOR_TYPE::NORMAL, etc
 #include "game_board.hpp"                    // for game_board, etc
-#include "game_config.hpp"                   // for red_to_green
 #include "game_events/pump.hpp"              // for fire
 #include "gettext.hpp"                       // for _
 #include "gui/dialogs/transient_message.hpp" // for show_transient_message
@@ -54,6 +54,9 @@ static lg::log_domain log_engine("engine");
 #define ERR_NG LOG_STREAM(err, log_engine)
 #define LOG_NG LOG_STREAM(info, log_engine)
 
+static lg::log_domain log_wml("wml");
+#define ERR_WML LOG_STREAM(err, log_wml)
+
 namespace events
 {
 mouse_handler::mouse_handler(game_display* gui, play_controller& pc)
@@ -155,6 +158,9 @@ void mouse_handler::touch_motion(int x, int y, const bool browse, bool update, m
 
 	// now copy-pasting mouse_handler::mouse_motion()
 
+	// Note for anyone reconciling this code with the version in mouse_handler::mouse_motion:
+	// commit 27a40a82aeea removed the game_board& board from mouse_motion, but didn't update
+	// the corresponding code here in touch_motion.
 	game_board & board = pc_.gamestate().board_;
 
 	if(new_hex == map_location::null_location())
@@ -162,7 +168,7 @@ void mouse_handler::touch_motion(int x, int y, const bool browse, bool update, m
 
 	if(new_hex != last_hex_) {
 		update = true;
-		if( pc_.get_map_const().on_board(last_hex_) ) {
+		if( pc_.get_map().on_board(last_hex_) ) {
 			// we store the previous hexes used to propose attack direction
 			previous_hex_ = last_hex_;
 			// the hex of the selected unit is also "free"
@@ -198,7 +204,7 @@ void mouse_handler::touch_motion(int x, int y, const bool browse, bool update, m
 
 	// reset current_route_ and current_paths if not valid anymore
 	// we do it before cursor selection, because it uses current_paths_
-	if( !pc_.get_map_const().on_board(new_hex) ) {
+	if( !pc_.get_map().on_board(new_hex) ) {
 		current_route_.steps.clear();
 		gui().set_route(nullptr);
 		pc_.get_whiteboard()->erase_temp_move();
@@ -332,14 +338,21 @@ void mouse_handler::touch_motion(int x, int y, const bool browse, bool update, m
 		else
 			un.reset();
 	} //end planned unit map scope
+}
 
+void mouse_handler::show_reach_for_unit(const unit_ptr& un)
+{
 	if( (!selected_hex_.valid()) && un && current_paths_.destinations.empty() &&
 		 !gui().fogged(un->get_location()))
 	{
-		if (un->side() == side_num_) {
-			//unit is on our team, show path if the unit has one
+		// If the unit has a path set and is either ours or allied then show the path.
+		//
+		// Exception: allied AI sides' moves are still hidden, on the assumption that
+		// campaign authors won't want to leak goto_x,goto_y tricks to the player.
+		if(!viewing_team().is_enemy(un->side()) && !pc_.get_teams()[un->side() - 1].is_ai()) {
+			//unit is on our team or an allied team, show path if the unit has one
 			const map_location go_to = un->get_goto();
-			if(board.map().on_board(go_to)) {
+			if(pc_.get_map().on_board(go_to)) {
 				pathfind::marked_route route;
 				{ // start planned unit map scope
 					wb::future_map_if_active raii;
@@ -366,7 +379,6 @@ void mouse_handler::touch_motion(int x, int y, const bool browse, bool update, m
 		unselected_paths_ = true;
 		gui().highlight_reach(current_paths_);
 	}
-
 }
 
 void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update, map_location new_hex)
@@ -382,8 +394,6 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update, m
 		return;
 	}
 
-	game_board& board = pc_.gamestate().board_;
-
 	if(new_hex == map_location::null_location()) {
 		new_hex = gui().hex_clicked_on(x, y);
 	}
@@ -395,7 +405,7 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update, m
 
 		update = true;
 
-		if(pc_.get_map_const().on_board(last_hex_)) {
+		if(pc_.get_map().on_board(last_hex_)) {
 			// we store the previous hexes used to propose attack direction
 			previous_hex_ = last_hex_;
 
@@ -440,7 +450,7 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update, m
 
 	// reset current_route_ and current_paths if not valid anymore
 	// we do it before cursor selection, because it uses current_paths_
-	if(!pc_.get_map_const().on_board(new_hex)) {
+	if(!pc_.get_map().on_board(new_hex)) {
 		current_route_.steps.clear();
 		gui().set_route(nullptr);
 		pc_.get_whiteboard()->erase_temp_move();
@@ -527,8 +537,7 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update, m
 			current_route_.steps.clear();
 			gui().set_route(nullptr);
 			pc_.get_whiteboard()->erase_temp_move();
-		} else if(!current_paths_.destinations.empty() && board.map().on_board(selected_hex_)
-				  && board.map().on_board(new_hex)) {
+		} else if(!current_paths_.destinations.empty() && pc_.get_map().on_board(selected_hex_) && pc_.get_map().on_board(new_hex)) {
 			if(selected_unit && !selected_unit->incapacitated()) {
 				// Show the route from selected unit to mouseover hex
 				current_route_ = get_route(&*selected_unit, dest, viewing_team());
@@ -541,7 +550,7 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update, m
 			}
 		}
 
-		if(board.map().on_board(selected_hex_) && !selected_unit && mouseover_unit.valid() && mouseover_unit) {
+		if(pc_.get_map().on_board(selected_hex_) && !selected_unit && mouseover_unit.valid() && mouseover_unit) {
 			// Show the route from selected hex to mouseover unit
 			current_route_ = get_route(&*mouseover_unit, selected_hex_, viewing_team());
 
@@ -563,43 +572,14 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update, m
 		}
 	} /*end planned unit map scope*/
 
-	if(!selected_hex_.valid() && un && current_paths_.destinations.empty() && !gui().fogged(un->get_location())) {
-		/*
-		 * Only process unit if toggler not preventing normal unit
-		 * processing. This can happen e.g. if, after activating 'show
-		 * [best possible] enemy movements' through the UI menu, the
-		 * mouse cursor lands on a hex with unit in it.
-		 */
-		if(!preventing_units_highlight_) {
-			if(un->side() == side_num_) {
-				// unit is on our team, show path if the unit has one
-				const map_location go_to = un->get_goto();
-				if(board.map().on_board(go_to)) {
-					pathfind::marked_route route;
-					{ // start planned unit map scope
-						wb::future_map_if_active raii;
-						route = get_route(un.get(), go_to, current_team());
-					} // end planned unit map scope
-					gui().set_route(&route);
-				}
-				over_route_ = true;
-
-				wb::future_map_if_active raii;
-				current_paths_ = pathfind::paths(*un, false, true, viewing_team(), path_turns_);
-			} else {
-				// unit under cursor is not on our team
-				// Note: planned unit map must be activated after this is done,
-				// since the future state includes changes to units' movement.
-				unit_movement_resetter move_reset(*un);
-
-				wb::future_map_if_active raii;
-				current_paths_ = pathfind::paths(*un, false, true, viewing_team(), path_turns_);
-			}
-
-			unselected_paths_ = true;
-			gui().highlight_reach(current_paths_);
-
-		}
+	/*
+	 * Only highlight unit's reach if toggler not preventing normal unit
+	 * processing. This can happen e.g. if, after activating 'show
+	 * [best possible] enemy movements' through the UI menu, the
+	 * mouse cursor lands on a hex with unit in it.
+	 */
+	if(!preventing_units_highlight_) {
+		show_reach_for_unit(un);
 	}
 
 	if(!un && preventing_units_highlight_) {
@@ -625,7 +605,7 @@ unit_map::iterator mouse_handler::find_unit(const map_location& hex)
 		return it;
 	}
 
-	return pc_.gamestate().board_.units_.end();
+	return pc_.get_units().end();
 }
 
 unit_map::const_iterator mouse_handler::find_unit(const map_location& hex) const
@@ -732,11 +712,10 @@ map_location mouse_handler::current_unit_attacks_from(const map_location& loc) c
 	int best_rating = 100; // smaller is better
 
 	map_location res;
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(loc, adj.data());
+	const auto adj = get_adjacent_tiles(loc);
 
-	for(size_t n = 0; n < adj.size(); ++n) {
-		if(pc_.gamestate().board_.map().on_board(adj[n]) == false) {
+	for(std::size_t n = 0; n < adj.size(); ++n) {
+		if(pc_.get_map().on_board(adj[n]) == false) {
 			continue;
 		}
 
@@ -745,7 +724,7 @@ map_location mouse_handler::current_unit_attacks_from(const map_location& loc) c
 		}
 
 		if(current_paths_.destinations.contains(adj[n])) {
-			static const size_t NDIRECTIONS = map_location::NDIRECTIONS;
+			static const std::size_t NDIRECTIONS = map_location::NDIRECTIONS;
 
 			unsigned int difference = std::abs(static_cast<int>(preferred - n));
 			if(difference > NDIRECTIONS / 2) {
@@ -785,11 +764,6 @@ pathfind::marked_route mouse_handler::get_route(const unit* un, map_location go_
 	return mark_route(route);
 }
 
-void mouse_handler::mouse_press(const SDL_MouseButtonEvent& event, const bool browse)
-{
-	mouse_handler_base::mouse_press(event, browse);
-}
-
 bool mouse_handler::right_click_show_menu(int x, int y, const bool /*browse*/)
 {
 	if(selected_hex_.valid() || unselected_reach_) {
@@ -802,7 +776,7 @@ bool mouse_handler::right_click_show_menu(int x, int y, const bool /*browse*/)
 
 void mouse_handler::select_or_action(bool browse)
 {
-	if(!pc_.get_map_const().on_board(last_hex_)) {
+	if(!pc_.get_map().on_board(last_hex_)) {
 		tooltips::click(drag_from_x_, drag_from_y_);
 		return;
 	}
@@ -839,12 +813,12 @@ void mouse_handler::move_action(bool browse)
 	// TODO
 	//	// Clicks on border hexes mean to deselect.
 	//	// (Check this before doing processing that might not be needed.)
-	//	if ( !pc_.get_map_const().on_board(hex) ) {
+	//	if ( !pc_.get_map().on_board(hex) ) {
 	//		deselect_hex();
 	//		return false;
 	//	}
 
-	unit_map::iterator u;
+	unit* u = nullptr;
 	const unit* clicked_u = nullptr;
 
 	map_location src;
@@ -853,7 +827,7 @@ void mouse_handler::move_action(bool browse)
 
 	{ // start planned unit map scope
 		wb::future_map_if_active planned_unit_map;
-		u = find_unit(selected_hex_);
+		u = find_unit_nonowning(selected_hex_);
 
 		// if the unit is selected and then itself clicked on,
 		// any goto command is canceled
@@ -875,7 +849,7 @@ void mouse_handler::move_action(bool browse)
 			return;
 		}
 
-		if(((u.valid() && u->side() == side_num_) || pc_.get_whiteboard()->is_active()) && clicked_u != nullptr) {
+		if(((u != nullptr && u->side() == side_num_) || pc_.get_whiteboard()->is_active()) && clicked_u != nullptr) {
 			if(attack_from == selected_hex_) { // no move needed
 				int choice = -1;
 				{
@@ -911,8 +885,7 @@ void mouse_handler::move_action(bool browse)
 
 					// block where we temporary move the unit
 					{
-						temporary_unit_mover temp_mover(
-								pc_.gamestate().board_.units_, src, attack_from, itor->move_left);
+						temporary_unit_mover temp_mover(pc_.get_units(), src, attack_from, itor->move_left);
 						choice = show_attack_dialog(attack_from, clicked_u->get_location());
 					}
 
@@ -955,7 +928,7 @@ void mouse_handler::move_action(bool browse)
 			(!browse || pc_.get_whiteboard()->is_active())
 			&& selected_hex_.valid()
 			&& selected_hex_ != hex
-			&& u.valid()
+			&& u != nullptr
 			&& (u->side() == side_num_ || pc_.get_whiteboard()->is_active())
 			&& !clicked_u
 			&& !current_route_.steps.empty()
@@ -1000,7 +973,7 @@ void mouse_handler::move_action(bool browse)
 		} else {
 			// Don't move if the unit already has actions
 			// from the whiteboard.
-			if(pc_.get_whiteboard()->unit_has_actions(u ? &*u : clicked_u)) {
+			if(pc_.get_whiteboard()->unit_has_actions(u ? u : clicked_u)) {
 				return;
 			}
 
@@ -1089,9 +1062,9 @@ void mouse_handler::select_hex(const map_location& hex, const bool browse, const
 		pathfind::paths clicked_location;
 		clicked_location.destinations.insert(hex);
 
-		for(unit_map::iterator u = pc_.gamestate().board_.units_.begin(); u != pc_.gamestate().board_.units_.end();
+		for(unit_map::iterator u = pc_.get_units().begin(); u != pc_.get_units().end();
 				++u) {
-			bool invisible = u->invisible(u->get_location(), gui_->get_disp_context());
+			bool invisible = u->invisible(u->get_location());
 
 			if(!gui_->fogged(u->get_location()) && !u->incapacitated() && !invisible) {
 				const pathfind::paths& path =
@@ -1106,7 +1079,7 @@ void mouse_handler::select_hex(const map_location& hex, const bool browse, const
 
 		gui_->highlight_another_reach(reaching_unit_locations);
 	} else {
-		if(!pc_.gamestate().board_.units_.find(last_hex_)) {
+		if(!pc_.get_units().find(last_hex_)) {
 			unselected_reach_ = gui_->unhighlight_reach();
 		}
 
@@ -1144,7 +1117,7 @@ bool mouse_handler::move_unit_along_current_route()
 
 	bool interrupted = false;
 	if(steps.size() > 1) {
-		size_t num_moves = move_unit_along_route(steps, interrupted);
+		std::size_t num_moves = move_unit_along_route(steps, interrupted);
 
 		interrupted = interrupted || num_moves + 1 < steps.size();
 		next_unit_ = steps[num_moves];
@@ -1175,7 +1148,7 @@ bool mouse_handler::move_unit_along_current_route()
  *          steps is not empty (the return value is guaranteed to be less
  *          than steps.size() ).
  */
-size_t mouse_handler::move_unit_along_route(const std::vector<map_location>& steps, bool& interrupted)
+std::size_t mouse_handler::move_unit_along_route(const std::vector<map_location>& steps, bool& interrupted)
 {
 	if(steps.empty()) {
 		interrupted = false;
@@ -1187,8 +1160,8 @@ size_t mouse_handler::move_unit_along_route(const std::vector<map_location>& ste
 
 	// If this is a leader on a keep, ask permission to the whiteboard to move it
 	// since otherwise it may cause planned recruits to be erased.
-	if(pc_.get_map_const().is_keep(steps.front())) {
-		unit_map::const_iterator const u = pc_.gamestate().board_.units().find(steps.front());
+	if(pc_.get_map().is_keep(steps.front())) {
+		unit_map::const_iterator const u = pc_.get_units().find(steps.front());
 
 		if(u && u->can_recruit() && u->side() == gui().viewing_side()
 				&& !pc_.get_whiteboard()->allow_leader_to_move(*u)) {
@@ -1199,7 +1172,7 @@ size_t mouse_handler::move_unit_along_route(const std::vector<map_location>& ste
 	}
 
 	LOG_NG << "move unit along route  from " << steps.front() << " to " << steps.back() << "\n";
-	size_t moves = actions::move_unit_and_record(steps, &pc_.get_undo_stack(), false, true, &interrupted);
+	std::size_t moves = actions::move_unit_and_record(steps, &pc_.get_undo_stack(), false, true, &interrupted);
 
 	cursor::set(cursor::NORMAL);
 	gui().invalidate_game_status();
@@ -1253,7 +1226,7 @@ int mouse_handler::fill_weapon_choices(
 	for(unsigned int i = 0; i < attacker->attacks().size(); i++) {
 		// skip weapons with attack_weight=0
 		if(attacker->attacks()[i].attack_weight() > 0) {
-			battle_context bc(pc_.gamestate().board_.units_, attacker->get_location(), defender->get_location(), i);
+			battle_context bc(pc_.get_units(), attacker->get_location(), defender->get_location(), i);
 
 			// Don't include if the attacker's weapon has at least one active "disable" special.
 			if(bc.get_attacker_stats().disable) {
@@ -1276,8 +1249,8 @@ int mouse_handler::show_attack_dialog(const map_location& attacker_loc, const ma
 {
 	game_board& board = pc_.gamestate().board_;
 
-	unit_map::iterator attacker = board.units_.find(attacker_loc);
-	unit_map::iterator defender = board.units_.find(defender_loc);
+	unit_map::iterator attacker = board.units().find(attacker_loc);
+	unit_map::iterator defender = board.units().find(defender_loc);
 
 	if(!attacker || !defender) {
 		ERR_NG << "One fighter is missing, can't attack";
@@ -1307,7 +1280,8 @@ void mouse_handler::attack_enemy(const map_location& attacker_loc, const map_loc
 	try {
 		attack_enemy_(attacker_loc, defender_loc, choice);
 	} catch(const std::bad_alloc&) {
-		lg::wml_error() << "Memory exhausted a unit has either a lot hitpoints or a negative amount.\n";
+		lg::log_to_chat() << "Memory exhausted a unit has either a lot hitpoints or a negative amount.\n";
+		ERR_WML << "Memory exhausted a unit has either a lot hitpoints or a negative amount.";
 	}
 }
 
@@ -1340,7 +1314,7 @@ void mouse_handler::attack_enemy_(const map_location& att_loc, const map_locatio
 		defender = &*defender_it;
 	}
 
-	if(size_t(choice) >= bc_vector.size()) {
+	if(std::size_t(choice) >= bc_vector.size()) {
 		return;
 	}
 
@@ -1362,9 +1336,9 @@ void mouse_handler::attack_enemy_(const map_location& att_loc, const map_locatio
 	gui().unhighlight_reach();
 
 	current_team().set_action_bonus_count(1 + current_team().action_bonus_count());
-	///@todo change ToD to be location specific for the defender
+	// TODO: change ToD to be location specific for the defender
 
-	const tod_manager& tod_man = pc_.get_tod_manager_const();
+	const tod_manager& tod_man = pc_.get_tod_manager();
 
 	synced_context::run_and_throw("attack",
 		replay_helper::get_attack(
@@ -1386,12 +1360,9 @@ std::set<map_location> mouse_handler::get_adj_enemies(const map_location& loc, i
 {
 	std::set<map_location> res;
 
-	const team& uteam = pc_.gamestate().board_.teams_[side - 1];
+	const team& uteam = pc_.get_teams()[side - 1];
 
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(loc, adj.data());
-
-	for(const map_location& aloc : adj) {
+	for(const map_location& aloc : get_adjacent_tiles(loc)) {
 		unit_map::const_iterator i = find_unit(aloc);
 
 		if(i && uteam.is_enemy(i->side())) {
@@ -1417,21 +1388,18 @@ void mouse_handler::show_attack_options(const unit_map::const_iterator& u)
 
 	// Get the teams involved.
 	const team& cur_team = current_team();
-	const team& u_team = pc_.gamestate().board_.teams_[u->side() - 1];
+	const team& u_team = pc_.get_teams()[u->side() - 1];
 
 	// Check each adjacent hex.
-	adjacent_loc_array_t adj;
-	get_adjacent_tiles(u->get_location(), adj.data());
-
-	for(const map_location& loc : adj) {
+	for(const map_location& loc : get_adjacent_tiles(u->get_location())) {
 		// No attack option shown if no visible unit present.
 		// (Visible to current team, not necessarily the unit's team.)
-		if(!pc_.get_map_const().on_board(loc)) {
+		if(!pc_.get_map().on_board(loc)) {
 			continue;
 		}
 
-		unit_map::const_iterator i = pc_.gamestate().board_.units().find(loc);
-		if(!i || !i->is_visible_to_team(cur_team, gui_->get_disp_context(), false)) {
+		unit_map::const_iterator i = pc_.get_units().find(loc);
+		if(!i || !i->is_visible_to_team(cur_team, false)) {
 			continue;
 		}
 
@@ -1456,7 +1424,7 @@ bool mouse_handler::unit_in_cycle(unit_map::const_iterator it)
 		return false;
 	}
 
-	if(current_team().is_enemy(int(gui().viewing_team() + 1)) && it->invisible(it->get_location(), gui().get_disp_context())) {
+	if(current_team().is_enemy(static_cast<int>(gui().viewing_team() + 1)) && it->invisible(it->get_location())) {
 		return false;
 	}
 
@@ -1469,29 +1437,29 @@ bool mouse_handler::unit_in_cycle(unit_map::const_iterator it)
 
 void mouse_handler::cycle_units(const bool browse, const bool reverse)
 {
-	game_board& board = pc_.gamestate().board_;
+	unit_map& units = pc_.get_units();
 
-	if(board.units().begin() == board.units().end()) {
+	if(units.begin() == units.end()) {
 		return;
 	}
 
 	unit_map::const_iterator it = find_unit(next_unit_);
 	if(!it) {
-		it = board.units().begin();
+		it = units.begin();
 	}
 
 	const unit_map::const_iterator itx = it;
 
 	do {
 		if(reverse) {
-			if(it == board.units().begin()) {
-				it = board.units().end();
+			if(it == units.begin()) {
+				it = units.end();
 			}
 
 			--it;
 		} else {
-			if(it == board.units().end()) {
-				it = board.units().begin();
+			if(it == units.end()) {
+				it = units.begin();
 			} else {
 				++it;
 			}
@@ -1520,17 +1488,17 @@ void mouse_handler::set_current_paths(const pathfind::paths& new_paths)
 
 team& mouse_handler::viewing_team()
 {
-	return pc_.gamestate().board_.teams_[gui().viewing_team()];
+	return pc_.get_teams()[gui().viewing_team()];
 }
 
 const team& mouse_handler::viewing_team() const
 {
-	return pc_.gamestate().board_.teams()[gui().viewing_team()];
+	return pc_.get_teams()[gui().viewing_team()];
 }
 
 team& mouse_handler::current_team()
 {
-	return pc_.gamestate().board_.teams_[side_num_ - 1];
+	return pc_.get_teams()[side_num_ - 1];
 }
 
 mouse_handler* mouse_handler::singleton_ = nullptr;
