@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -24,7 +25,6 @@
 #include "log.hpp"
 #include "map/map.hpp"
 #include "terrain/translation.hpp"
-#include "terrain/type_data.hpp"
 #include "units/types.hpp" // for attack_type
 
 static lg::log_domain log_config("config");
@@ -37,13 +37,13 @@ static lg::log_domain log_config("config");
 
 namespace { // Some functions for use with parameters::eval.
 
-	/// Converts config defense values to a "max" value.
+	/** Converts config defense values to a "max" value. */
 	int config_to_max(int value)
 	{
 		return value < 0 ? -value : value;
 	}
 
-	/// Converts config defense values to a "min" value.
+	/** Converts config defense values to a "min" value. */
 	int config_to_min(int value)
 	{
 		return value < 0 ? -value : 0;
@@ -51,17 +51,23 @@ namespace { // Some functions for use with parameters::eval.
 }
 
 
-/// The parameters used when calculating a terrain-based value.
+/** The parameters used when calculating a terrain-based value. */
 struct movetype::terrain_info::parameters
 {
-	int min_value;     /// The smallest allowable value.
-	int max_value;     /// The largest allowable value.
-	int default_value; /// The default value (if no data is available).
+	/** The smallest allowable value. */
+	int min_value;
+	/** The largest allowable value. */
+	int max_value;
+	/** The default value (if no data is available). */
+	int default_value;
 
-	int (*eval)(int);  /// Converter for values taken from a config. May be nullptr.
+	/** Converter for values taken from a config. May be nullptr. */
+	int (*eval)(int);
 
-	bool use_move;     /// Whether to look at underlying movement or defense terrains.
-	bool high_is_good; /// Whether we are looking for highest or lowest (unless inverted by the underlying terrain).
+	/** Whether to look at underlying movement or defense terrains. */
+	bool use_move;
+	/** Whether we are looking for highest or lowest (unless inverted by the underlying terrain). */
+	bool high_is_good;
 
 	parameters(int min, int max, int (*eval_fun)(int)=nullptr, bool move=true, bool high=false) :
 		min_value(min), max_value(max), default_value(high ? min : max),
@@ -70,7 +76,7 @@ struct movetype::terrain_info::parameters
 };
 
 
-/// Limits for movement, vision and jamming
+/** Limits for movement, vision and jamming */
 const movetype::terrain_info::parameters
 	movetype::mvj_params_{1, movetype::UNREACHABLE};
 
@@ -86,13 +92,17 @@ const movetype::terrain_info::parameters
 class movetype::terrain_info::data
 {
 public:
-	/// Constructor.
-	/// @a params must be long-lived (typically a static variable).
+	/**
+	 * Constructor.
+	 * @a params must be long-lived (typically a static variable).
+	 */
 	explicit data(const parameters & params) :
 		cfg_(), cache_(), params_(params)
 	{}
-	/// Constructor.
-	/// @a params must be long-lived (typically a static variable).
+	/**
+	 * Constructor.
+	 * @a params must be long-lived (typically a static variable).
+	 */
 	data(const config & cfg, const parameters & params) :
 		cfg_(cfg), cache_(), params_(params)
 	{}
@@ -103,42 +113,42 @@ public:
 		cfg_(that.cfg_), cache_(), params_(that.params_)
 	{}
 
-	/// Clears the cached data (presumably our fallback has changed).
+	/** Clears the cached data (presumably our fallback has changed). */
 	void clear_cache() const;
-	/// Tests if merging @a new_values would result in changes.
+	/** Tests if merging @a new_values would result in changes. */
 	bool config_has_changes(const config & new_values, bool overwrite) const;
-	/// Tests for no data in this object.
+	/** Tests for no data in this object. */
 	bool empty() const { return cfg_.empty(); }
-	/// Merges the given config over the existing costs.
+	/** Merges the given config over the existing costs. */
 	void merge(const config & new_values, bool overwrite);
-	/// Read-only access to our parameters.
+	/** Read-only access to our parameters. */
 	const parameters & params() const { return params_; }
-	/// Returns the value associated with the given terrain.
+	/** Returns the value associated with the given terrain. */
 	int value(const t_translation::terrain_code & terrain,
 	          const terrain_info * fallback) const
 	{ return value(terrain, fallback, 0); }
-	/// If there is data, writes it to the config.
+	/** If there is data, writes it to the config. */
 	void write(config & out_cfg, const std::string & child_name) const;
-	/// If there is (merged) data, writes it to the config.
+	/** If there is (merged) data, writes it to the config. */
 	void write(config & out_cfg, const std::string & child_name,
 	           const terrain_info * fallback) const;
 
 private:
-	/// Calculates the value associated with the given terrain.
+	/** Calculates the value associated with the given terrain. */
 	int calc_value(const t_translation::terrain_code & terrain,
 	               const terrain_info * fallback, unsigned recurse_count) const;
-	/// Returns the value associated with the given terrain (possibly cached).
+	/** Returns the value associated with the given terrain (possibly cached). */
 	int value(const t_translation::terrain_code & terrain,
 	          const terrain_info * fallback, unsigned recurse_count) const;
 
 private:
 	typedef std::map<t_translation::terrain_code, int> cache_t;
 
-	/// Config describing the terrain values.
+	/** Config describing the terrain values. */
 	config cfg_;
-	/// Cache of values based on the config.
+	/** Cache of values based on the config. */
 	mutable cache_t cache_;
-	/// Various parameters used when calculating values.
+	/** Various parameters used when calculating values. */
 	const parameters & params_;
 };
 
@@ -277,7 +287,7 @@ int movetype::terrain_info::data::calc_value(
 		return params_.default_value;
 	}
 
-	ter_data_cache tdata;
+	std::shared_ptr<terrain_type_data> tdata;
 	if (game_config_manager::get()){
 		tdata = game_config_manager::get()->terrain_types(); //This permits to get terrain info in unit help pages from the help in title screen, even if there is no residual gamemap object
 	}
@@ -287,10 +297,8 @@ int movetype::terrain_info::data::calc_value(
 	const t_translation::ter_list & underlying = params_.use_move ?
 			tdata->underlying_mvt_terrain(terrain) :
 			tdata->underlying_def_terrain(terrain);
-	assert(!underlying.empty());
 
-
-	if ( underlying.size() == 1  &&  underlying.front() == terrain )
+	if (terrain_type::is_indivisible(terrain, underlying))
 	{
 		// This is not an alias; get the value directly.
 		int result = params_.default_value;
@@ -421,10 +429,7 @@ movetype::terrain_info::terrain_info(const config & cfg, const parameters & para
  */
 std::unique_ptr<movetype::terrain_costs> movetype::read_terrain_costs(const config & cfg)
 {
-	// todoc++14: use std::make_unique
-	std::unique_ptr<terrain_costs> ret;
-	ret.reset(new terrain_info(cfg, mvj_params_, nullptr));
-	return ret;
+	return std::make_unique<terrain_info> (cfg, movetype::mvj_params_, nullptr);
 }
 
 /**
@@ -514,6 +519,7 @@ void swap(movetype & a, movetype & b)
 	swap(a.defense_, b.defense_);
 	std::swap(a.resist_, b.resist_);
 	std::swap(a.flying_, b.flying_);
+	std::swap(a.special_notes_, b.special_notes_);
 }
 
 movetype & movetype::operator=(const movetype & that)
@@ -604,11 +610,10 @@ void movetype::terrain_info::write(config & cfg, const std::string & child_name,
  */
 std::unique_ptr<movetype::terrain_costs> movetype::terrain_info::make_standalone() const
 {
-	// todoc++14: use std::make_unique
 	std::unique_ptr<terrain_costs> t;
 	if(!fallback_) {
 		// Call the copy constructor, which will make_data_shareable().
-		t.reset(new terrain_info(*this, nullptr));
+		t = std::make_unique<terrain_info>(*this, nullptr);
 	}
 	else if(get_data().empty()) {
 		// Pure fallback.
@@ -618,7 +623,7 @@ std::unique_ptr<movetype::terrain_costs> movetype::terrain_info::make_standalone
 		// Need to merge data.
 		config merged;
 		write(merged, "", true);
-		t.reset(new terrain_info(merged, get_data().params(), nullptr));
+		t = std::make_unique<terrain_info>(merged, get_data().params(), nullptr);
 	}
 	return t;
 }
@@ -705,8 +710,10 @@ movetype::terrain_defense & movetype::terrain_defense::operator=(terrain_defense
 	max_.swap_data(that.max_);
 	return *this;
 }
-/// Merges the given config over the existing costs.
-/// (Not overwriting implies adding.)
+/**
+ * Merges the given config over the existing costs.
+ * (Not overwriting implies adding.)
+ */
 void movetype::terrain_defense::merge(const config & new_data, bool overwrite)
 {
 	min_.merge(new_data, overwrite, {});
@@ -796,7 +803,8 @@ movetype::movetype() :
 	jamming_(mvj_params_, &vision_),
 	defense_(),
 	resist_(),
-	flying_(false)
+	flying_(false),
+	special_notes_()
 {
 }
 
@@ -811,8 +819,13 @@ movetype::movetype(const config & cfg) :
 	defense_(cfg.child_or_empty("defense")),
 	resist_(cfg.child_or_empty("resistance")),
 	flying_(cfg["flies"].to_bool(false))
-	// \todo standardize on "flying" instead of "flies"
 {
+	// 1.15 will support both "flying" and "flies", with "flies" being deprecated
+	flying_ = cfg["flying"].to_bool(flying_);
+
+	for(const config& sn : cfg.child_range("special_note")) {
+		special_notes_.push_back(sn["note"]);
+	}
 }
 
 
@@ -825,7 +838,8 @@ movetype::movetype(const movetype & that) :
 	jamming_(that.jamming_, &vision_),
 	defense_(that.defense_),
 	resist_(that.resist_),
-	flying_(that.flying_)
+	flying_(that.flying_),
+	special_notes_(that.special_notes_)
 {
 }
 
@@ -838,7 +852,8 @@ movetype::movetype(movetype && that) :
 	jamming_(std::move(that.jamming_), &vision_),
 	defense_(std::move(that.defense_)),
 	resist_(std::move(that.resist_)),
-	flying_(std::move(that.flying_))
+	flying_(std::move(that.flying_)),
+	special_notes_(std::move(that.special_notes_))
 {
 }
 
@@ -868,6 +883,7 @@ void movetype::merge(const config & new_cfg, bool overwrite)
 	// "flies" is used when WML defines a movetype.
 	// "flying" is used when WML defines a unit.
 	// It's easier to support both than to track which case we are in.
+	// Note: in 1.15 "flies" is deprecated, with "flying" preferred in movetype too.
 	flying_ = new_cfg["flies"].to_bool(flying_);
 	flying_ = new_cfg["flying"].to_bool(flying_);
 }
@@ -902,9 +918,6 @@ const std::set<std::string> movetype::effects {"movement_costs",
 
 /**
  * Writes the movement type data to the provided config.
- *
- * Note that this writes the movement part of SingleUnitWML, not the WML for a
- * [movetype] tag, because it uses the key "flying" instead of "flies".
  */
 void movetype::write(config & cfg) const
 {
@@ -916,4 +929,8 @@ void movetype::write(config & cfg) const
 
 	if ( flying_ )
 		cfg["flying"] = true;
+
+	for(const auto& note : special_notes_) {
+		cfg.add_child("special_note", config{"note", note});
+	}
 }
