@@ -1,27 +1,26 @@
 /*
-   Copyright (C) 2003 - 2018 by Jörg Hinrichs
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #pragma once
 
 #include "config.hpp"
+#include "game_version.hpp"
 #include "gettext.hpp"
 #include "utils/make_enum.hpp"
-#include "game_version.hpp"
-
-#include <boost/optional.hpp>
+#include <optional>
 
 struct mp_game_settings
 {
@@ -35,12 +34,10 @@ struct mp_game_settings
 	std::string name;
 	std::string password;
 	std::string hash;
-	std::string mp_era;
 	std::string mp_era_name;
 	std::string mp_scenario;
 	std::string mp_scenario_name;
 	std::string mp_campaign;
-	std::vector<std::string> active_mods;
 	std::map<std::string, std::string> side_users;
 
 	int num_turns;
@@ -57,11 +54,16 @@ struct mp_game_settings
 	bool fog_game;
 	bool shroud_game;
 	bool allow_observers;
-	bool registered_users_only;
 	bool private_replay;
 	bool shuffle_sides;
 
-	bool saved_game;
+	MAKE_ENUM(SAVED_GAME_MODE,
+		(NONE, "no")
+		(MIDGAME, "midgame")
+		(SCENARIO_START, "scenaro_start")
+	)
+
+	SAVED_GAME_MODE saved_game;
 
 	// HACK: The Create Game dialog has special knowledge of these strings
 	//       and the fact that they're placed in the default (wesnoth)
@@ -77,18 +79,27 @@ struct mp_game_settings
 
 	config options;
 
+	struct addon_content
+	{
+		std::string id;
+		std::string name;
+		std::string type;
+	};
+
 	struct addon_version_info
 	{
-		boost::optional<version_info> version;
-		boost::optional<version_info> min_version;
-
+		std::optional<version_info> version;
+		std::optional<version_info> min_version;
 		std::string name;
+		bool required;
+		std::vector<addon_content> content;
 
 		explicit addon_version_info(const config &);
 		void write(config &) const;
 	};
 
-	std::map<std::string, addon_version_info> addons; // the key is the addon_id
+	/** the key is the addon_id */
+	std::map<std::string, addon_version_info> addons;
 
 	/**
 	 * Takes a config with addon metadata (id, name, version, min_version) and adds
