@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #pragma once
 
 #include "map/location.hpp"
@@ -49,7 +48,7 @@ void get_tiles_in_radius(const map_location& center, const int radius,
  * of @a center (including @a center itself). @a result must be a std::set
  * of locations.
  */
-void get_tiles_radius(const map_location& center, size_t radius,
+void get_tiles_radius(const map_location& center, std::size_t radius,
                       std::set<map_location>& result);
 
 /**
@@ -58,7 +57,7 @@ void get_tiles_radius(const map_location& center, size_t radius,
  * @a result must be a std::set of locations.
  */
 void get_tiles_radius(const gamemap& map, const std::vector<map_location>& locs,
-                      size_t radius, std::set<map_location>& result,
+                      std::size_t radius, std::set<map_location>& result,
                       bool with_border=false);
 
 /**
@@ -68,5 +67,5 @@ void get_tiles_radius(const gamemap& map, const std::vector<map_location>& locs,
  * @a result must be a std::set of locations.
  */
 void get_tiles_radius(const gamemap& map, const std::vector<map_location>& locs,
-                      size_t radius, std::set<map_location>& result,
+                      std::size_t radius, std::set<map_location>& result,
                       bool with_border, const xy_pred &pred);
