@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Jody Northup
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Jody Northup
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "filesystem.hpp"
@@ -79,7 +80,7 @@ bool persist_file_context::clear_var(const std::string &global, bool immediate)
 				{
 					const std::string::const_iterator index_end = std::find(global.begin(),global.end(),']');
 					const std::string index_str(index_start+1,index_end);
-					size_t index = static_cast<size_t>(lexical_cast_default<int>(index_str));
+					std::size_t index = static_cast<std::size_t>(lexical_cast_default<int>(index_str));
 					cfg.remove_child(global,index);
 					if (immediate) bactive.remove_child(global,index);
 				} else {
@@ -163,9 +164,9 @@ config persist_file_context::get_var(const std::string &global) const
 	const config *active = get_node(cfg_, namespace_);
 	if (active && (active->has_child("variables"))) {
 		const config &cfg = active->child("variables");
-		size_t arrsize = cfg.child_count(global);
+		std::size_t arrsize = cfg.child_count(global);
 		if (arrsize > 0) {
-			for (size_t i = 0; i < arrsize; i++)
+			for (std::size_t i = 0; i < arrsize; i++)
 				ret.add_child(global,cfg.child(global,i));
 		} else {
 			ret = pack_scalar(global,cfg[global]);
