@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Jody Northup
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Jody Northup
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "game_data.hpp"
@@ -71,12 +72,12 @@ static void get_global_variable(persist_context &ctx, const vconfig &pcfg)
 	try
 	{
 		if (cfg) {
-			size_t arrsize = cfg.child_count(global);
+			std::size_t arrsize = cfg.child_count(global);
 			if (arrsize == 0) {
 				resources::gamedata->set_variable(local,cfg[global]);
 			} else {
 				resources::gamedata->clear_variable(local);
-				for (size_t i = 0; i < arrsize; i++)
+				for (std::size_t i = 0; i < arrsize; i++)
 					resources::gamedata->add_variable_cfg(local,cfg.child(global,i));
 			}
 		} else {
@@ -104,7 +105,7 @@ static void set_global_variable(persist_context &ctx, const vconfig &pcfg)
 		std::string local = pcfg["from_local"];
 		config val;
 		const config &vars = resources::gamedata->get_variables();
-		size_t arraylen = vars.child_count(local);
+		std::size_t arraylen = vars.child_count(local);
 		if (arraylen == 0) {
 			try
 			{
@@ -115,7 +116,7 @@ static void set_global_variable(persist_context &ctx, const vconfig &pcfg)
 				val.clear();
 			}
 		} else {
-			for (size_t i = 0; i < arraylen; i++)
+			for (std::size_t i = 0; i < arraylen; i++)
 				val.add_child(global,vars.child(local,i));
 		}
 		ctx.set_var(global, val, pcfg["immediate"].to_bool());
@@ -204,7 +205,7 @@ void verify_and_clear_global_variable(const vconfig &pcfg)
 {
 	bool valid = true;
 	if (!pcfg.has_attribute("global")) {
-		ERR_PERSIST << "[clear_global_variable] missing required attribute \"from_global\"";
+		ERR_PERSIST << "[clear_global_variable] missing required attribute \"global\"";
 		valid = false;
 	}
 	if (!pcfg.has_attribute("namespace")) {
