@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -17,26 +18,22 @@
  * Routines for images: load, scale, re-color, etc.
  */
 
-#define GETTEXT_DOMAIN "wesnoth-lib"
-
 #include "picture.hpp"
 
 #include "config.hpp"
 #include "display.hpp"
 #include "filesystem.hpp"
 #include "game_config.hpp"
-#include "gettext.hpp"
 #include "image_modifications.hpp"
 #include "log.hpp"
 #include "preferences/general.hpp"
 #include "serialization/base64.hpp"
 #include "serialization/string_utils.hpp"
 #include "sdl/rect.hpp"
-#include "utils/general.hpp"
 
 #include <SDL2/SDL_image.h>
 
-#include "utils/functional.hpp"
+#include <functional>
 
 #include <boost/algorithm/string.hpp>
 #include <boost/functional/hash_fwd.hpp>
@@ -76,9 +73,9 @@ namespace std
 template<>
 struct hash<image::locator::value>
 {
-	size_t operator()(const image::locator::value& val) const
+	std::size_t operator()(const image::locator::value& val) const
 	{
-		size_t hash = std::hash<unsigned>{}(val.type_);
+		std::size_t hash = std::hash<unsigned>{}(val.type_);
 
 		if(val.type_ == image::locator::FILE || val.type_ == image::locator::SUB_FILE) {
 			boost::hash_combine(hash, val.filename_);
@@ -183,36 +180,27 @@ std::map<surface, surface> reversed_images_;
 
 int red_adjust = 0, green_adjust = 0, blue_adjust = 0;
 
-/** List of colors used by the TC image modification */
-std::vector<std::string> team_colors;
-
 unsigned int zoom = tile_size;
 unsigned int cached_zoom = 0;
 
-/** Algorithm choices */
-// typedef std::function<surface(const surface &, int, int)> scaling_function;
-typedef surface (*scaling_function)(const surface&, int, int);
-scaling_function scale_to_zoom_func;
-scaling_function scale_to_hex_func;
-
 const std::string data_uri_prefix = "data:";
 struct parsed_data_URI{
-	explicit parsed_data_URI(utils::string_view data_URI);
-	utils::string_view scheme;
-	utils::string_view mime;
-	utils::string_view base64;
-	utils::string_view data;
+	explicit parsed_data_URI(std::string_view data_URI);
+	std::string_view scheme;
+	std::string_view mime;
+	std::string_view base64;
+	std::string_view data;
 	bool good;
 };
-parsed_data_URI::parsed_data_URI(utils::string_view data_URI)
+parsed_data_URI::parsed_data_URI(std::string_view data_URI)
 {
-	const size_t colon = data_URI.find(':');
-	const utils::string_view after_scheme = data_URI.substr(colon + 1);
+	const std::size_t colon = data_URI.find(':');
+	const std::string_view after_scheme = data_URI.substr(colon + 1);
 
-	const size_t comma = after_scheme.find(',');
-	const utils::string_view type_info = after_scheme.substr(0, comma);
+	const std::size_t comma = after_scheme.find(',');
+	const std::string_view type_info = after_scheme.substr(0, comma);
 
-	const size_t semicolon = type_info.find(';');
+	const std::size_t semicolon = type_info.find(';');
 
 	scheme = data_URI.substr(0, colon);
 	base64 = type_info.substr(semicolon + 1);
@@ -280,15 +268,15 @@ void locator::parse_arguments()
 		parsed_data_URI parsed{fn};
 
 		if(!parsed.good) {
-			utils::string_view view{ fn };
-			utils::string_view stripped = view.substr(0, view.find(","));
+			std::string_view view{ fn };
+			std::string_view stripped = view.substr(0, view.find(","));
 			ERR_DP << "Invalid data URI: " << stripped << std::endl;
 		}
 
 		val_.is_data_uri_ = true;
 	}
 
-	size_t markup_field = fn.find('~');
+	std::size_t markup_field = fn.find('~');
 
 	if(markup_field != std::string::npos) {
 		val_.type_ = SUB_FILE;
@@ -452,94 +440,6 @@ bool locator::value::operator<(const value& a) const
 	return false;
 }
 
-// Check if localized file is up-to-date according to l10n track index.
-// Make sure only that the image is not explicitly recorded as fuzzy,
-// in order to be able to use non-tracked images (e.g. from UMC).
-static std::set<std::string> fuzzy_localized_files;
-static bool localized_file_uptodate(const std::string& loc_file)
-{
-	if(fuzzy_localized_files.empty()) {
-		// First call, parse track index to collect fuzzy files by path.
-		std::string fsep = "\xC2\xA6"; // UTF-8 for "broken bar"
-		// Issue #4716 is that passing an empty string as the first argument of get_binary_file_location
-		// causes that function to find the file in both "wesnoth_dir//l10n-track" and "wesnoth_dir/l10n-track",
-		// triggering the warning about conflicting files with the same name.
-		std::string trackpath = filesystem::get_binary_file_location("workaround_for_issue_4716", "l10n-track");
-
-		// l10n-track file not present. Assume image is up-to-date.
-		if(trackpath.empty()) {
-			return true;
-		}
-
-		std::string contents = filesystem::read_file(trackpath);
-
-		for(const std::string& line : utils::split(contents, '\n')) {
-			size_t p1 = line.find(fsep);
-			if(p1 == std::string::npos) {
-				continue;
-			}
-
-			std::string state = line.substr(0, p1);
-			boost::trim(state);
-			if(state == "fuzzy") {
-				size_t p2 = line.find(fsep, p1 + fsep.length());
-				if(p2 == std::string::npos) {
-					continue;
-				}
-
-				std::string relpath = line.substr(p1 + fsep.length(), p2 - p1 - fsep.length());
-				fuzzy_localized_files.insert(game_config::path + '/' + relpath);
-			}
-		}
-
-		fuzzy_localized_files.insert(""); // make sure not empty any more
-	}
-
-	return fuzzy_localized_files.count(loc_file) == 0;
-}
-
-// Return path to localized counterpart of the given file, if any, or empty string.
-// Localized counterpart may also be requested to have a suffix to base name.
-static std::string get_localized_path(const std::string& file, const std::string& suff = "")
-{
-	std::string dir = filesystem::directory_name(file);
-	std::string base = filesystem::base_name(file);
-
-	const size_t pos_ext = base.rfind(".");
-
-	std::string loc_base;
-	if(pos_ext != std::string::npos) {
-		loc_base = base.substr(0, pos_ext) + suff + base.substr(pos_ext);
-	} else {
-		loc_base = base + suff;
-	}
-
-	// TRANSLATORS: This is the language code which will be used
-	// to store and fetch localized non-textual resources, such as images,
-	// when they exist. Normally it is just the code of the PO file itself,
-	// e.g. "de" of de.po for German. But it can also be a comma-separated
-	// list of language codes by priority, when the localized resource
-	// found for first of those languages will be used. This is useful when
-	// two languages share sufficient commonality, that they can use each
-	// other's resources rather than duplicating them. For example,
-	// Swedish (sv) and Danish (da) are such, so Swedish translator could
-	// translate this message as "sv,da", while Danish as "da,sv".
-	std::vector<std::string> langs = utils::split(_("language code for localized resources^en_US"));
-
-	// In case even the original image is split into base and overlay,
-	// add en_US with lowest priority, since the message above will
-	// not have it when translated.
-	langs.push_back("en_US");
-	for(const std::string& lang : langs) {
-		std::string loc_file = dir + "/" + "l10n" + "/" + lang + "/" + loc_base;
-		if(filesystem::file_exists(loc_file) && localized_file_uptodate(loc_file)) {
-			return loc_file;
-		}
-	}
-
-	return "";
-}
-
 // Load overlay image and compose it with the original surface.
 static void add_localized_overlay(const std::string& ovr_file, surface& orig_surf)
 {
@@ -563,7 +463,7 @@ static surface load_image_file(const image::locator& loc)
 	{
 		if(!location.empty()) {
 			// Check if there is a localized image.
-			const std::string loc_location = get_localized_path(location);
+			const std::string loc_location = filesystem::get_localized_path(location);
 			if(!loc_location.empty()) {
 				location = loc_location;
 			}
@@ -573,7 +473,7 @@ static surface load_image_file(const image::locator& loc)
 
 			// If there was no standalone localized image, check if there is an overlay.
 			if(res && loc_location.empty()) {
-				const std::string ovr_location = get_localized_path(location, "--overlay");
+				const std::string ovr_location = filesystem::get_localized_path(location, "--overlay");
 				if(!ovr_location.empty()) {
 					add_localized_overlay(ovr_location, res);
 				}
@@ -605,10 +505,17 @@ static surface load_image_sub_file(const image::locator& loc)
 		try {
 			surf = (*mod)(surf);
 		} catch(const image::modification::imod_exception& e) {
+			std::ostringstream ss;
+			ss << "\n";
+
+			for(const std::string& mod_name : utils::parenthetical_split(loc.get_modifications(), '~')) {
+				ss << "\t" << mod_name << "\n";
+			}
+
 			ERR_CFG << "Failed to apply a modification to an image:\n"
-					<< "Image: " << loc.get_filename() << ".\n"
-					<< "Modifications: " << loc.get_modifications() << ".\n"
-					<< "Error: " << e.message;
+					<< "Image: " << loc.get_filename() << "\n"
+					<< "Modifications: " << ss.str() << "\n"
+					<< "Error: " << e.message << "\n";
 		}
 
 		// NOTE: do this *after* applying the mod or you'll get crashes!
@@ -654,8 +561,8 @@ static surface load_image_data_uri(const image::locator& loc)
 	parsed_data_URI parsed{loc.get_filename()};
 
 	if(!parsed.good) {
-		utils::string_view fn = loc.get_filename();
-		utils::string_view stripped = fn.substr(0, fn.find(","));
+		std::string_view fn = loc.get_filename();
+		std::string_view stripped = fn.substr(0, fn.find(","));
 		ERR_DP << "Invalid data URI: " << stripped << std::endl;
 	} else if(parsed.mime.substr(0, 5) != "image") {
 		ERR_DP << "Data URI not of image MIME type: " << parsed.mime << std::endl;
@@ -724,7 +631,7 @@ static surface apply_light(surface surf, const light_string& ls)
 		};
 
 		// decompose into atomic lightmap operations (4 chars)
-		for(size_t c = 0; c + 3 < ls.size(); c += 4) {
+		for(std::size_t c = 0; c + 3 < ls.size(); c += 4) {
 			light_string sls = ls.substr(c, 4);
 
 			// get the corresponding image and apply the lightmap operation to it
@@ -795,20 +702,6 @@ void set_color_adjustment(int r, int g, int b)
 	}
 }
 
-void set_team_colors(const std::vector<std::string>* colors)
-{
-	if(colors == nullptr) {
-		team_colors.clear();
-	} else {
-		team_colors = *colors;
-	}
-}
-
-const std::vector<std::string>& get_team_colors()
-{
-	return team_colors;
-}
-
 void set_zoom(unsigned int amount)
 {
 	if(amount != zoom) {
@@ -829,42 +722,6 @@ void set_zoom(unsigned int amount)
 	}
 }
 
-// F should be a scaling algorithm without "integral" zoom limitations
-template<scaling_function F>
-static surface scale_xbrz_helper(const surface& res, int w, int h)
-{
-	int best_integer_zoom = std::min(w / res->w, h / res->h);
-	int legal_zoom = utils::clamp(best_integer_zoom, 1, 5);
-	return F(scale_surface_xbrz(res, legal_zoom), w, h);
-}
-
-using SCALING_ALGORITHM = preferences::SCALING_ALGORITHM;
-
-static scaling_function select_algorithm(SCALING_ALGORITHM algo)
-{
-	switch(algo.v) {
-	case SCALING_ALGORITHM::LINEAR: {
-		scaling_function result = &scale_surface;
-		return result;
-	}
-	case SCALING_ALGORITHM::NEAREST_NEIGHBOR: {
-		scaling_function result = &scale_surface_nn;
-		return result;
-	}
-	case SCALING_ALGORITHM::XBRZ_LIN: {
-		scaling_function result = &scale_xbrz_helper<scale_surface>;
-		return result;
-	}
-	case SCALING_ALGORITHM::XBRZ_NN: {
-		scaling_function result = &scale_xbrz_helper<scale_surface_nn>;
-		return result;
-	}
-	default:
-		assert(false && "I don't know how to implement this scaling algorithm");
-		throw 42;
-	}
-}
-
 static surface get_hexed(const locator& i_locator)
 {
 	surface image(get_image(i_locator, UNSCALED));
@@ -881,7 +738,7 @@ static surface get_scaled_to_hex(const locator& i_locator)
 	// return scale_surface(img, zoom, zoom);
 
 	if(img) {
-		return scale_to_hex_func(img, zoom, zoom);
+		return scale_surface_nn(img, zoom, zoom);
 	}
 
 	return surface(nullptr);
@@ -902,7 +759,7 @@ static surface get_scaled_to_zoom(const locator& i_locator)
 	surface res(get_image(i_locator, UNSCALED));
 	// For some reason haloes seems to have invalid images, protect against crashing
 	if(res) {
-		return scale_to_zoom_func(res, ((res->w * zoom) / tile_size), ((res->h * zoom) / tile_size));
+		return scale_surface_nn(res, ((res->w * zoom) / tile_size), ((res->h * zoom) / tile_size));
 	}
 
 	return surface(nullptr);
@@ -914,7 +771,7 @@ static surface get_brightened(const locator& i_locator)
 	return brighten_image(image, ftofxp(game_config::hex_brightening));
 }
 
-/// translate type to a simpler one when possible
+/** translate type to a simpler one when possible */
 static TYPE simplify_type(const image::locator& i_locator, TYPE type)
 {
 	switch(type) {
@@ -992,12 +849,10 @@ surface get_image(const image::locator& i_locator, TYPE type)
 	}
 
 	// return the image if already cached
-	bool tmp;
-	tmp = i_locator.in_cache(*imap);
+	bool tmp = i_locator.in_cache(*imap);
 
 	if(tmp) {
-		surface result;
-		result = i_locator.locate_in_cache(*imap);
+		surface result = i_locator.locate_in_cache(*imap);
 		return result;
 	}
 
@@ -1071,7 +926,7 @@ surface get_lighted_image(const image::locator& i_locator, const light_string& l
 	case SCALED_TO_HEX:
 		// we light before scaling to reuse the unscaled cache
 		res = get_lighted_image(i_locator, ls, HEXED);
-		res = scale_surface(res, zoom, zoom);
+		res = scale_surface_nn(res, zoom, zoom);
 		break;
 	default:
 		break;
@@ -1163,10 +1018,7 @@ bool exists(const image::locator& i_locator)
 
 	// The insertion will fail if there is already an element in the cache
 	// and this will point to the existing element.
-	auto iter = image_existence_map.begin();
-	bool success;
-
-	std::tie(iter, success) = image_existence_map.emplace(i_locator.get_filename(), false);
+	auto [iter, success] = image_existence_map.emplace(i_locator.get_filename(), false);
 
 	bool& cache = iter->second;
 	if(success) {
@@ -1195,8 +1047,8 @@ static void precache_file_existence_internal(const std::string& dir, const std::
 
 	std::vector<std::string> files_found;
 	std::vector<std::string> dirs_found;
-	filesystem::get_files_in_dir(checked_dir, &files_found, &dirs_found, filesystem::FILE_NAME_ONLY,
-			filesystem::NO_FILTER, filesystem::DONT_REORDER);
+	filesystem::get_files_in_dir(checked_dir, &files_found, &dirs_found, filesystem::name_mode::FILE_NAME_ONLY,
+			filesystem::filter_mode::NO_FILTER, filesystem::reorder_mode::DONT_REORDER);
 
 	for(const auto& f : files_found) {
 		image_existence_map[subdir + f] = true;
@@ -1237,16 +1089,12 @@ save_result save_image(const surface& surf, const std::string& filename)
 		return save_result::no_image;
 	}
 
-#ifdef SDL_IMAGE_VERSION_ATLEAST
-#if SDL_IMAGE_VERSION_ATLEAST(2, 0, 2)
 	if(filesystem::ends_with(filename, ".jpeg") || filesystem::ends_with(filename, ".jpg") || filesystem::ends_with(filename, ".jpe")) {
 		LOG_DP << "Writing a JPG image to " << filename << std::endl;
 
 		const int err = IMG_SaveJPG_RW(surf, filesystem::make_write_RWops(filename).release(), true, 75); // SDL takes ownership of the RWops
 		return err == 0 ? save_result::success : save_result::save_failed;
 	}
-#endif
-#endif
 
 	if(filesystem::ends_with(filename, ".png")) {
 		LOG_DP << "Writing a PNG image to " << filename << std::endl;
@@ -1264,25 +1112,4 @@ save_result save_image(const surface& surf, const std::string& filename)
 	return save_result::unsupported_format;
 }
 
-bool update_from_preferences()
-{
-	SCALING_ALGORITHM algo = preferences::default_scaling_algorithm;
-	try {
-		algo = SCALING_ALGORITHM::string_to_enum(preferences::get("scale_hex"));
-	} catch(const bad_enum_cast&) {
-	}
-
-	scale_to_hex_func = select_algorithm(algo);
-
-	algo = preferences::default_scaling_algorithm;
-	try {
-		algo = SCALING_ALGORITHM::string_to_enum(preferences::get("scale_zoom"));
-	} catch(const bad_enum_cast&) {
-	}
-
-	scale_to_zoom_func = select_algorithm(algo);
-
-	return true;
-}
-
 } // end namespace image
