@@ -1,31 +1,31 @@
 /*
-   Copyright (C) 2006 - 2018 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
 #include "controller_base.hpp"
-#include "floating_label.hpp"
 #include "game_end_exceptions.hpp"
 #include "help/help.hpp"
 #include "hotkey/command_executor.hpp"
 #include "menu_events.hpp"
 #include "mouse_events.hpp"
 #include "persist_manager.hpp"
-#include "terrain/type_data.hpp"
 #include "tod_manager.hpp"
 #include "game_state.hpp"
+#include <optional>
 
 #include <set>
 
@@ -44,6 +44,10 @@ namespace actions {
 	class undo_list;
 }
 
+namespace font {
+	struct floating_label_context;
+}
+
 namespace game_events {
 	class manager;
 	class wml_event_pump;
@@ -76,8 +80,7 @@ class game_state;
 class play_controller : public controller_base, public events::observer, public quit_confirmation
 {
 public:
-	play_controller(const config& level, saved_game& state_of_game,
-		const ter_data_cache& tdata, bool skip_replay);
+	play_controller(const config& level, saved_game& state_of_game, bool skip_replay);
 	virtual ~play_controller();
 
 	//event handler, overridden from observer
@@ -125,41 +128,78 @@ public:
 	 */
 	virtual void process_oos(const std::string& msg) const;
 
-	void set_end_level_data(const end_level_data& data) {
+	void set_end_level_data(const end_level_data& data)
+	{
 		gamestate().end_level_data_ = data;
 	}
-	void reset_end_level_data() {
-		gamestate().end_level_data_ = boost::none;
-	}
-	bool is_regular_game_end() const {
-		return gamestate().end_level_data_.get_ptr() != nullptr;
-	}
-	const end_level_data& get_end_level_data_const() const {
-		return *gamestate().end_level_data_;
-	}
-	const std::vector<team>& get_teams_const() const {
-		return gamestate().board_.teams_;
+
+	void reset_end_level_data()
+	{
+		gamestate().end_level_data_.reset();
 	}
 
-	const unit_map& get_units_const() const {
+	bool is_regular_game_end() const
+	{
+		return gamestate().end_level_data_.has_value();
+	}
+
+	const end_level_data& get_end_level_data() const
+	{
+		return *gamestate().end_level_data_;
+	}
+
+	std::vector<team>& get_teams()
+	{
+		return gamestate().board_.teams();
+	}
+
+	const std::vector<team>& get_teams() const
+	{
+		return gamestate().board_.teams();
+	}
+
+	const unit_map& get_units() const
+	{
 		return gamestate().board_.units();
 	}
 
-	const gamemap& get_map_const() const{
+	unit_map& get_units()
+	{
+		return gamestate().board_.units();
+	}
+
+	const gamemap& get_map() const
+	{
 		return gamestate().board_.map();
 	}
-	const tod_manager& get_tod_manager_const() const{
-			return gamestate().tod_manager_;
-		}
 
-	bool is_observer() const {
+	const tod_manager& get_tod_manager() const
+	{
+		return gamestate().tod_manager_;
+	}
+
+	bool is_observer() const
+	{
 		return gamestate().board_.is_observer();
 	}
 
-	game_state& gamestate() {
+	bool do_healing() const
+	{
+		return gamestate().do_healing_;
+	}
+
+	void set_do_healing(bool do_healing)
+	{
+		gamestate().do_healing_ = do_healing;
+	}
+
+	game_state& gamestate()
+	{
 		return *gamestate_;
 	}
-	const game_state& gamestate() const {
+
+	const game_state& gamestate() const
+	{
 		return *gamestate_;
 	}
 
@@ -170,7 +210,7 @@ public:
 	 */
 	void check_victory();
 
-	size_t turn() const {return gamestate().tod_manager_.turn();}
+	std::size_t turn() const {return gamestate().tod_manager_.turn();}
 
 	/**
 	 * Returns the number of the side whose turn it is.
@@ -230,6 +270,11 @@ public:
 		return level_["disallow_recall"].to_bool();
 	}
 
+	std::string get_loaded_resources() const
+	{
+		return level_["loaded_resources"].str();
+	}
+
 	std::string theme() const
 	{
 		return gamestate_->get_game_data()->get_theme();
@@ -268,16 +313,19 @@ public:
 	virtual bool is_networked_mp() const { return false; }
 	virtual void send_to_wesnothd(const config&, const std::string& = "unknown") const { }
 	virtual bool receive_from_wesnothd(config&) const { return false; }
-	/// Reevaluate [show_if] conditions and build a new objectives string.
-	void refresh_objectives();
+	/** Reevaluate [show_if] conditions and build a new objectives string. */
+	void refresh_objectives() const;
 	void show_objectives() const;
+
 	struct scoped_savegame_snapshot
 	{
 		scoped_savegame_snapshot(const play_controller& controller);
 		~scoped_savegame_snapshot();
 		const play_controller& controller_;
 	};
+
 	saved_game& get_saved_game() { return saved_game_; }
+
 protected:
 	friend struct scoped_savegame_snapshot;
 	void play_slice_catch();
@@ -287,7 +335,7 @@ protected:
 	void process_keyup_event(const SDL_Event& event) override;
 
 	void init_managers();
-	///preload events cannot be synced
+	/** preload events cannot be synced */
 	void fire_preload();
 	void fire_prestart();
 	void fire_start();
@@ -298,11 +346,12 @@ protected:
 	bool enemies_visible() const;
 
 	void enter_textbox();
+	void textbox_move_vertically(bool up);
 	void tab();
 
 
 	bool is_team_visible(int team_num, bool observer) const;
-	/// returns 0 if no such team was found.
+	/** returns 0 if no such team was found. */
 	int find_last_visible_team() const;
 
 private:
@@ -310,7 +359,6 @@ private:
 
 protected:
 	//gamestate
-	const ter_data_cache& tdata_;
 	std::unique_ptr<game_state> gamestate_;
 	config level_;
 	saved_game& saved_game_;
@@ -325,7 +373,7 @@ protected:
 	std::unique_ptr<plugins_context> plugins_context_;
 
 	//more managers
-	font::floating_label_context labels_manager_;
+	std::unique_ptr<font::floating_label_context> labels_manager_;
 	help::help_manager help_manager_;
 	events::mouse_handler mouse_handler_;
 	events::menu_handler menu_handler_;
@@ -359,6 +407,11 @@ private:
 
 	void init(const config& level);
 
+	/**
+	 * This shows a warning dialog if either [scenario]next_scenario or any [endlevel]next_scenario would lead to an "Unknown Scenario" dialog.
+	 */
+	void check_next_scenario_is_known();
+
 	bool victory_when_enemies_defeated_;
 	bool remove_from_carryover_on_defeat_;
 	std::vector<std::string> victory_music_;
