@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2006 - 2018 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -19,12 +20,11 @@
 #include "syncmp_handler.hpp"
 
 class turn_info;
-struct mp_campaign_info;
+struct mp_game_metadata;
 class playmp_controller : public playsingle_controller, public syncmp_handler
 {
 public:
-	playmp_controller(const config& level, saved_game& state_of_game,
-		const ter_data_cache & tdata, mp_campaign_info* mp_info);
+	playmp_controller(const config& level, saved_game& state_of_game, mp_game_metadata* mp_info);
 	virtual ~playmp_controller();
 
 	void maybe_linger() override;
@@ -70,5 +70,5 @@ private:
 	void set_end_scenario_button();
 	void reset_end_scenario_button();
 	void process_network_data(bool chat_only = false);
-	mp_campaign_info* mp_info_;
+	mp_game_metadata* mp_info_;
 };
