@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2006 - 2018 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -37,8 +38,7 @@ struct reset_gamestate_exception : public std::exception
 class playsingle_controller : public play_controller
 {
 public:
-	playsingle_controller(const config& level, saved_game& state_of_game,
-		const ter_data_cache & tdata, bool skip_replay);
+	playsingle_controller(const config& level, saved_game& state_of_game, bool skip_replay);
 
 	LEVEL_RESULT play_scenario(const config& level);
 	void play_scenario_init();
@@ -84,11 +84,11 @@ protected:
 	turn_info turn_data_;
 	enum END_TURN_STATE
 	{
-		/// The turn was not ended yet
+		/** The turn was not ended yet */
 		END_TURN_NONE,
-		/// And endturn was required eigher by the player, by the ai or by [end_turn]
+		/** And endturn was required eigher by the player, by the ai or by [end_turn] */
 		END_TURN_REQUIRED,
-		/// An [end_turn] was added to the replay.
+		/** An [end_turn] was added to the replay. */
 		END_TURN_SYNCED,
 	};
 	END_TURN_STATE end_turn_;
