@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "playturn.hpp"
@@ -79,7 +80,7 @@ turn_info::PROCESS_DATA_RESULT turn_info::sync_network()
 
 void turn_info::send_data()
 {
-	const bool send_everything = synced_context::is_unsynced() ? !resources::undo_stack->can_undo() : synced_context::is_simultaneously();
+	const bool send_everything = synced_context::is_unsynced() ? !resources::undo_stack->can_undo() : synced_context::is_simultaneous();
 	if ( !send_everything ) {
 		replay_sender_.sync_non_undoable();
 	} else {
@@ -137,13 +138,13 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 
 	if (const config &message = cfg.child("message"))
 	{
-		game_display::get_singleton()->get_chat_manager().add_chat_message(time(nullptr), message["sender"], message["side"],
+		game_display::get_singleton()->get_chat_manager().add_chat_message(std::time(nullptr), message["sender"], message["side"],
 				message["message"], events::chat_handler::MESSAGE_PUBLIC,
 				preferences::message_bell());
 	}
 	else if (const config &whisper = cfg.child("whisper") /*&& is_observer()*/)
 	{
-		game_display::get_singleton()->get_chat_manager().add_chat_message(time(nullptr), "whisper: " + whisper["sender"].str(), 0,
+		game_display::get_singleton()->get_chat_manager().add_chat_message(std::time(nullptr), "whisper: " + whisper["sender"].str(), 0,
 				whisper["message"], events::chat_handler::MESSAGE_PRIVATE,
 				preferences::message_bell());
 	}
@@ -157,7 +158,7 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 	}
 	else if (cfg.child("leave_game")) {
 		const bool has_reason = cfg.child("leave_game").has_attribute("reason");
-		throw ingame_wesnothd_error(has_reason ? cfg.child("leave_game")["reason"].str() : "");
+		throw leavegame_wesnothd_error(has_reason ? cfg.child("leave_game")["reason"].str() : "");
 	}
 	else if (const config &turn = cfg.child("turn"))
 	{
@@ -178,7 +179,8 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 		const int side = change["side"].to_int();
 		const bool is_local = change["is_local"].to_bool();
 		const std::string player = change["player"];
-		const size_t index = side - 1;
+		const std::string controller_type = change["controller"];
+		const std::size_t index = side - 1;
 		if(index >= resources::gameboard->teams().size()) {
 			ERR_NW << "Bad [change_controller] signal from server, side out of bounds: " << change.debug() << std::endl;
 			return PROCESS_CONTINUE;
@@ -187,7 +189,7 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 		const team & tm = resources::gameboard->teams().at(index);
 		const bool was_local = tm.is_local();
 
-		resources::gameboard->side_change_controller(side, is_local, player);
+		resources::gameboard->side_change_controller(side, is_local, player, controller_type);
 
 		if (!was_local && tm.is_local()) {
 			resources::controller->on_not_observer();
@@ -221,7 +223,7 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 	else if (const config &side_drop_c = cfg.child("side_drop"))
 	{
 		const int  side_drop = side_drop_c["side_num"].to_int(0);
-		size_t index = side_drop -1;
+		std::size_t index = side_drop -1;
 
 		bool restart = side_drop == game_display::get_singleton()->playing_side();
 
