@@ -1,14 +1,15 @@
 /*
-Copyright (C) 2017-2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2017 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "credentials.hpp"
@@ -36,6 +37,7 @@ See the COPYING file for more details.
 #endif
 
 static lg::log_domain log_config("config");
+#define DBG_CFG LOG_STREAM(debug , log_config)
 #define ERR_CFG LOG_STREAM(err , log_config)
 
 class secure_buffer : public std::vector<unsigned char>
@@ -60,7 +62,7 @@ struct login_info
 	login_info(const std::string& username, const std::string& server)
 		: username(username), server(server), key()
 	{}
-	size_t size() const
+	std::size_t size() const
 	{
 		return 3 + username.size() + server.size() + key.size();
 	}
@@ -86,7 +88,7 @@ static std::string get_system_username()
 	if(GetUserNameW(buffer, &size)) {
 		//size includes a terminating null character.
 		assert(size > 0);
-		res = unicode_cast<utf8::string>(boost::iterator_range<wchar_t*>(buffer, buffer + size - 1));
+		res = unicode_cast<std::string>(boost::iterator_range<wchar_t*>(buffer, buffer + size - 1));
 	}
 #else
 	if(char* const login = getenv("USER")) {
@@ -153,6 +155,7 @@ namespace preferences
 
 	std::string password(const std::string& server, const std::string& login)
 	{
+		DBG_CFG << "Retrieving password for server: '" << server << "', login: '" << login << "'\n";
 		auto login_clean = login;
 		boost::trim(login_clean);
 
@@ -176,6 +179,7 @@ namespace preferences
 
 	void set_password(const std::string& server, const std::string& login, const std::string& key)
 	{
+		DBG_CFG << "Setting password for server: '" << server << "', login: '" << login << "'\n";
 		auto login_clean = login;
 		boost::trim(login_clean);
 
@@ -214,8 +218,8 @@ namespace preferences
 			return;
 		}
 		for(const std::string& elem : utils::split(std::string(data.begin(), data.end()), CREDENTIAL_SEPARATOR, utils::REMOVE_EMPTY)) {
-			size_t at = elem.find_last_of('@');
-			size_t eq = elem.find_first_of('=', at + 1);
+			std::size_t at = elem.find_last_of('@');
+			std::size_t eq = elem.find_first_of('=', at + 1);
 			if(at != std::string::npos && eq != std::string::npos) {
 				secure_buffer key(elem.begin() + eq + 1, elem.end());
 				credentials.emplace_back(elem.substr(0, at), elem.substr(at + 1, eq - at - 1), unescape(key));
@@ -229,21 +233,15 @@ namespace preferences
 			filesystem::delete_file(filesystem::get_credentials_file());
 			return;
 		}
-		secure_buffer credentials_data(1, CREDENTIAL_SEPARATOR);
-		size_t offset = 1;
+		secure_buffer credentials_data;
 		for(const auto& cred : credentials) {
-			credentials_data.resize(credentials_data.size() + cred.size(), CREDENTIAL_SEPARATOR);
-			std::copy(cred.username.begin(), cred.username.end(), credentials_data.begin() + offset);
-			offset += cred.username.size();
-			credentials_data[offset++] = '@';
-			std::copy(cred.server.begin(), cred.server.end(), credentials_data.begin() + offset);
-			offset += cred.server.size();
-			credentials_data[offset++] = '=';
+			credentials_data.push_back(CREDENTIAL_SEPARATOR);
+			credentials_data.insert(credentials_data.end(), cred.username.begin(), cred.username.end());
+			credentials_data.push_back('@');
+			credentials_data.insert(credentials_data.end(), cred.server.begin(), cred.server.end());
+			credentials_data.push_back('=');
 			secure_buffer key_escaped = escape(cred.key);
-			// Escaping may increase the length, so resize again if so
-			credentials_data.resize(credentials_data.size() + key_escaped.size() - cred.key.size());
-			std::copy(key_escaped.begin(), key_escaped.end(), credentials_data.begin() + offset);
-			offset += key_escaped.size() + 1;
+			credentials_data.insert(credentials_data.end(), key_escaped.begin(), key_escaped.end());
 		}
 		try {
 			filesystem::scoped_ostream credentials_file = filesystem::ostream_file(filesystem::get_credentials_file());
@@ -259,7 +257,7 @@ namespace preferences
 secure_buffer build_key(const std::string& server, const std::string& login)
 {
 	std::string sysname = get_system_username();
-	secure_buffer result(std::max<size_t>(server.size() + login.size() + sysname.size(), 32));
+	secure_buffer result(std::max<std::size_t>(server.size() + login.size() + sysname.size(), 32));
 	unsigned char i = 0;
 	std::generate(result.begin(), result.end(), [&i]() {return 'x' ^ i++;});
 	std::copy(login.begin(), login.end(), result.begin());
@@ -274,14 +272,14 @@ static secure_buffer rc4_crypt(const secure_buffer& text, const secure_buffer& k
 #ifndef __APPLE__
 	RC4_KEY cipher_key;
 	RC4_set_key(&cipher_key, key.size(), key.data());
-	const size_t block_size = key.size();
-	const size_t blocks = text.size() / block_size;
-	const size_t extra = text.size() % block_size;
-	for(size_t i = 0; i < blocks * block_size; i += block_size) {
+	const std::size_t block_size = key.size();
+	const std::size_t blocks = text.size() / block_size;
+	const std::size_t extra = text.size() % block_size;
+	for(std::size_t i = 0; i < blocks * block_size; i += block_size) {
 		RC4(&cipher_key, block_size, text.data() + i, result.data() + i);
 	}
 	if(extra) {
-		size_t i = blocks * block_size;
+		std::size_t i = blocks * block_size;
 		RC4(&cipher_key, extra, text.data() + i, result.data() + i);
 	}
 #else
