@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -31,8 +32,8 @@
 #include "sdl/point.hpp"
 #include "serialization/parser.hpp"
 #include "sound.hpp"
-#include "utils/general.hpp"
 #include "video.hpp" // non_interactive()
+#include "game_config_view.hpp"
 
 #include <sys/stat.h> // for setting the permissions of the preferences file
 #ifndef _WIN32
@@ -71,8 +72,6 @@ const int def_window_height = 720;
 const int min_font_scaling  = 80;
 const int max_font_scaling  = 150;
 
-const SCALING_ALGORITHM default_scaling_algorithm = SCALING_ALGORITHM::XBRZ_NN;
-
 class prefs_event_handler : public events::sdl_handler {
 public:
 	virtual void handle_event(const SDL_Event &) {}
@@ -86,25 +85,7 @@ base_manager::base_manager()
 {
 	event_handler_.join_global();
 
-	try{
-#ifdef DEFAULT_PREFS_PATH
-		filesystem::scoped_istream stream = filesystem::istream_file(filesystem::get_default_prefs_file(),false);
-		read(prefs, *stream);
-
-		config user_prefs;
-		stream = filesystem::istream_file(filesystem::get_prefs_file());
-		read(user_prefs, *stream);
-
-		prefs.merge_with(user_prefs);
-#else
-		filesystem::scoped_istream stream = filesystem::istream_file(filesystem::get_prefs_file(),false);
-		read(prefs, *stream);
-#endif
-	} catch(const config::error& e) {
-		ERR_CFG << "Error loading preference, message: "
-				<< e.what()
-				<< std::endl;
-	}
+	preferences::load_base_prefs();
 	preferences::load_credentials();
 }
 
@@ -252,6 +233,29 @@ config* get_prefs(){
 	return pointer;
 }
 
+void load_base_prefs() {
+	try{
+#ifdef DEFAULT_PREFS_PATH
+		filesystem::scoped_istream stream = filesystem::istream_file(filesystem::get_default_prefs_file(),false);
+		read(prefs, *stream);
+
+		config user_prefs;
+		stream = filesystem::istream_file(filesystem::get_prefs_file());
+		read(user_prefs, *stream);
+
+		prefs.merge_with(user_prefs);
+#else
+		prefs.clear();
+		filesystem::scoped_istream stream = filesystem::istream_file(filesystem::get_prefs_file(),false);
+		read(prefs, *stream);
+#endif
+	} catch(const config::error& e) {
+		ERR_CFG << "Error loading preference, message: "
+				<< e.what()
+				<< std::endl;
+	}
+}
+
 
 bool show_allied_orb() {
 	return get("show_ally_orb", game_config::show_ally_orb);
@@ -288,6 +292,12 @@ void set_show_partial_orb(bool show_orb) {
 	prefs["show_partial_orb"] = show_orb;
 }
 
+bool show_disengaged_orb() {
+	return get("show_disengaged_orb", game_config::show_disengaged_orb);
+}
+void set_show_disengaged_orb(bool show_orb) {
+	prefs["show_disengaged_orb"] = show_orb;
+}
 
 static std::string fix_orb_color_name(const std::string& color) {
 	if (color.substr(0,4) == "orb_") {
@@ -360,6 +370,16 @@ void set_partial_color(const std::string& color_id) {
 	prefs["partial_orb_color"] = color_id;
 }
 
+std::string disengaged_color() {
+	std::string disengaged_color = get("disengaged_orb_color");
+	if (disengaged_color.empty())
+		return game_config::colors::disengaged_orb_color;
+	return fix_orb_color_name(disengaged_color);
+}
+void set_disengaged_color(const std::string& color_id) {
+	prefs["disengaged_orb_color"] = color_id;
+}
+
 bool scroll_to_action()
 {
 	return get("scroll_to_action", true);
@@ -444,7 +464,7 @@ int font_scaling()
 
 void set_font_scaling(int scale)
 {
-	prefs["font_scale"] = utils::clamp(scale, min_font_scaling, max_font_scaling);
+	prefs["font_scale"] = std::clamp(scale, min_font_scaling, max_font_scaling);
 }
 
 int font_scaled(int size)
@@ -512,20 +532,20 @@ void _set_grid(bool ison)
 	preferences::set("grid", ison);
 }
 
-size_t sound_buffer_size()
+std::size_t sound_buffer_size()
 {
 	// Sounds don't sound good on Windows unless the buffer size is 4k,
 	// but this seems to cause crashes on other systems...
 	#ifdef _WIN32
-		const size_t buf_size = 4096;
+		const std::size_t buf_size = 4096;
 	#else
-		const size_t buf_size = 1024;
+		const std::size_t buf_size = 1024;
 	#endif
 
 	return prefs["sound_buffer_size"].to_int(buf_size);
 }
 
-void save_sound_buffer_size(const size_t size)
+void save_sound_buffer_size(const std::size_t size)
 {
 	#ifdef _WIN32
 		const char* buf_size = "4096";
@@ -602,6 +622,16 @@ void set_UI_volume(int vol)
 	sound::set_UI_volume(UI_volume());
 }
 
+unsigned int tile_size()
+{
+	return prefs["tile_size"].to_unsigned();
+}
+
+void set_tile_size(const unsigned int size)
+{
+	prefs["tile_size"] = size;
+}
+
 bool turn_bell()
 {
 	return get("turn_bell", true);
@@ -718,147 +748,9 @@ namespace {
 	double scroll = 0.2;
 }
 
-bool joystick_support_enabled()
-{
-	return get("joystick_support_enabled", false);
-}
-
-int joystick_mouse_deadzone()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_scroll_deadzone"), 1500), 0, 16000);
-	return value;
-}
-
-int joystick_num_mouse_xaxis()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_num_scroll_xaxis"), 0), -1, 3);
-	return value;
-}
-
-int joystick_mouse_xaxis_num()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_scroll_xaxis_num"), 0), 0, 7);
-	return value;
-}
-
-int joystick_num_mouse_yaxis()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_num_scroll_yaxis"), 0), -1, 3);
-	return value;
-}
-
-int joystick_mouse_yaxis_num()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_scroll_yaxis_num"), 1), 0, 7);
-	return value;
-}
-
-int joystick_scroll_deadzone()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_scroll_deadzone"), 1500), 0, 16000);
-	return value;
-}
-
-int joystick_cursor_deadzone()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_cursor_deadzone"), 1500), 0, 16000);
-	return value;
-}
-
-int joystick_thrusta_deadzone()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_thrusta_deadzone"), 1500), 0, 16000);
-	return value;
-}
-
-int joystick_thrustb_deadzone()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_thrustb_deadzone"), 1500), 0, 16000);
-	return value;
-}
-
-int joystick_cursor_threshold()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_cursor_threshold"), 10000), 0, 16000);
-	return value;
-}
-
-int joystick_num_scroll_xaxis()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_num_scroll_xaxis"), 0), -1, 3);
-	return value;
-}
-
-int joystick_scroll_xaxis_num()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_scroll_xaxis_num"), 0), 0, 7);
-	return value;
-}
-
-int joystick_num_scroll_yaxis()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_num_scroll_yaxis"), 0), -1, 3);
-	return value;
-}
-
-int joystick_scroll_yaxis_num()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_scroll_yaxis_num"), 1), 0, 7);
-	return value;
-}
-
-int joystick_num_cursor_xaxis()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_num_cursor_xaxis"), 0), -1, 3);
-	return value;
-}
-
-int joystick_cursor_xaxis_num()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_cursor_xaxis_num"), 3), 0, 7);
-	return value;
-}
-
-int joystick_num_cursor_yaxis()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_num_cursor_yaxis"), 0), -1, 3);
-	return value;
-}
-
-int joystick_cursor_yaxis_num()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_cursor_yaxis_num"), 4), 0, 7);
-	return value;
-}
-
-int joystick_num_thrusta_axis()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_num_thrusta_axis"), 0), -1, 3);
-	return value;
-}
-
-int joystick_thrusta_axis_num()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_thrusta_axis_num"), 2), 0, 7);
-	return value;
-}
-
-int joystick_num_thrustb_axis()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_num_thrustb_axis"), 0), -1, 3);
-	return value;
-}
-
-int joystick_thrustb_axis_num()
-{
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("joystick_thrustb_axis_num"), 2), 0, 7);
-	return value;
-}
-
-
 int scroll_speed()
 {
-	const int value = utils::clamp<int>(lexical_cast_default<int>(get("scroll"), 50), 1, 100);
+	const int value = std::clamp<int>(lexical_cast_default<int>(get("scroll"), 50), 1, 100);
 	scroll = value/100.0;
 
 	return value;
@@ -992,7 +884,7 @@ void _set_color_cursors(bool value)
 
 void load_hotkeys()
 {
-	hotkey::load_hotkeys(prefs, false);
+	hotkey::load_hotkeys(game_config_view::wrap(prefs), false);
 }
 
 void save_hotkeys()
@@ -1054,16 +946,6 @@ void set_disable_auto_moves(bool value)
 	preferences::set("disable_auto_moves", value);
 }
 
-bool disable_loadingscreen_animation()
-{
-	return get("disable_loadingscreen_animation", false);
-}
-
-void set_disable_loadingscreen_animation(bool value)
-{
-	set("disable_loadingscreen_animation", value);
-}
-
 bool damage_prediction_allow_monte_carlo_simulation()
 {
 	return get("damage_prediction_allow_monte_carlo_simulation", true);
@@ -1074,4 +956,25 @@ void set_damage_prediction_allow_monte_carlo_simulation(bool value)
 	set("damage_prediction_allow_monte_carlo_simulation", value);
 }
 
+std::string addon_manager_saved_order_name()
+{
+	return get("addon_manager_saved_order_name");
+}
+
+void set_addon_manager_saved_order_name(const std::string& value)
+{
+	set("addon_manager_saved_order_name", value);
+}
+
+SORT_ORDER addon_manager_saved_order_direction()
+{
+	return SORT_ORDER::string_to_enum(get("addon_manager_saved_order_direction"), SORT_ORDER::NONE);
+}
+
+void set_addon_manager_saved_order_direction(SORT_ORDER value)
+{
+	set("addon_manager_saved_order_direction", SORT_ORDER::enum_to_string(value));
+}
+
+
 } // end namespace preferences
