@@ -1,15 +1,15 @@
 /*
-   Copyright (C) 2015 - 2018 by the Battle for Wesnoth Project
-   <https://www.wesnoth.org/>
+	Copyright (C) 2015 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -20,7 +20,7 @@ class CVideo;
 #include <vector>
 #include <string>
 
-#include "utils/functional.hpp"
+#include <functional>
 
 /**
  * Implements a quit confirmation dialog.
@@ -53,8 +53,9 @@ private:
 	// noncopyable
 	quit_confirmation(const quit_confirmation&) = delete;
 	const quit_confirmation& operator=(const quit_confirmation&) = delete;
-	static std::vector<quit_confirmation*> blockers_;
-	static bool open_;
+
+	static inline std::vector<quit_confirmation*> blockers_ {};
+	static inline bool open_ = false;
 
 	std::function<bool()> prompt_;
 };
