@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -39,6 +40,7 @@
 #include "whiteboard/manager.hpp"
 #include "replay_recorder_base.hpp"
 
+#include <array>
 #include <set>
 #include <map>
 
@@ -61,7 +63,7 @@ static void verify(const unit_map& units, const config& cfg) {
 	std::stringstream errbuf;
 	LOG_REPLAY << "verifying unit structure...\n";
 
-	const size_t nunits = cfg["num_units"].to_size_t();
+	const std::size_t nunits = cfg["num_units"].to_size_t();
 	if(nunits != units.size()) {
 		errbuf << "SYNC VERIFICATION FAILED: number of units from data source differ: "
 			   << nunits << " according to data source. " << units.size() << " locally\n";
@@ -104,12 +106,12 @@ static void verify(const unit_map& units, const config& cfg) {
 		u->write(u_cfg);
 
 		bool is_ok = true;
-		static const std::string fields[] {"type","hitpoints","experience","side",""};
-		for(const std::string* str = fields; str->empty() == false; ++str) {
-			if (u_cfg[*str] != un[*str]) {
-				errbuf << "ERROR IN FIELD '" << *str << "' for unit at "
-					   << loc << " data source: '" << un[*str]
-					   << "' local: '" << u_cfg[*str] << "'\n";
+		static const std::array<std::string, 4> fields {{"type","hitpoints","experience","side"}};
+		for(const std::string& field : fields) {
+			if (u_cfg[field] != un[field]) {
+				errbuf << "ERROR IN FIELD '" << field << "' for unit at "
+					   << loc << " data source: '" << un[field]
+					   << "' local: '" << u_cfg[field] << "'\n";
 				is_ok = false;
 			}
 		}
@@ -124,9 +126,9 @@ static void verify(const unit_map& units, const config& cfg) {
 	LOG_REPLAY << "verification passed\n";
 }
 
-static time_t get_time(const config &speak)
+static std::time_t get_time(const config &speak)
 {
-	time_t time;
+	std::time_t time;
 	if (!speak["time"].empty())
 	{
 		std::stringstream ss(speak["time"].str());
@@ -135,7 +137,7 @@ static time_t get_time(const config &speak)
 	else
 	{
 		//fallback in case sender uses wesnoth that doesn't send timestamps
-		time = ::time(nullptr);
+		time = std::time(nullptr);
 	}
 	return time;
 }
@@ -156,7 +158,7 @@ chat_msg::chat_msg(const config &cfg)
 	if (side==0) {
 		color_ = "white";//observers
 	} else {
-		color_ = team::get_side_highlight_pango(side-1);
+		color_ = team::get_side_highlight_pango(side);
 	}
 	time_ = get_time(cfg);
 	/*
@@ -447,8 +449,10 @@ config& replay::get_last_real_command()
 	assert(false && "replay::get_last_real_command called with no existent command.");
 	throw "replay::get_last_real_command called with no existent command.";
 }
-/// fixes a rename command when undoing a earlier command.
-/// @return: true if the command should be removed.
+/**
+ * fixes a rename command when undoing a earlier command.
+ * @return: true if the command should be removed.
+ */
 static bool fix_rename_command(const config& c, config& async_child)
 {
 	if (const config &child = c.child("move"))
@@ -669,7 +673,7 @@ bool replay::add_start_if_not_there_yet()
 	}
 }
 
-static void show_oos_error_error_function(const std::string& message, bool /*heavy*/)
+static void show_oos_error_error_function(const std::string& message)
 {
 	replay::process_error(message);
 }
@@ -822,7 +826,7 @@ REPLAY_RETURN do_replay_handle(bool one_move)
 		{
 			int val = countdown_update["value"];
 			int tval = countdown_update["team"];
-			if (tval <= 0  || tval > int(resources::gameboard->teams().size())) {
+			if (tval <= 0  || tval > static_cast<int>(resources::gameboard->teams().size())) {
 				std::stringstream errbuf;
 				errbuf << "Illegal countdown update \n"
 					<< "Received update for :" << tval << " Current user :"
