@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -21,6 +22,7 @@
 
 #include "map/location.hpp"
 
+#include <ctime>
 #include <deque>
 #include <iterator>
 #include <map>
@@ -35,14 +37,14 @@ public:
 	const std::string &text() const { return text_; }
 	const std::string &nick() const { return nick_; }
 	const std::string &color() const { return color_; }
-	const time_t &time() const { return time_; }
+	const std::time_t &time() const { return time_; }
 	chat_msg(const config &cfg);
 	virtual ~chat_msg();
 private:
 	std::string color_;
 	std::string nick_;
 	std::string text_;
-	time_t time_;
+	std::time_t time_;
 };
 
 class replay
@@ -63,10 +65,12 @@ public:
 	*/
 	config& get_last_real_command();
 	/**
-		adds a user_input to the replay
-		@param from_side the side that had to make the decision, -1 for 'server'
+	 * adds a user_input to the replay
+	 * @param name The tag name of the config to add
+	 * @param input the contents of the config to add
+	 * @param from_side the side that had to make the decision, -1 for 'server'
 	*/
-	void user_input(const std::string &, const config &, int from_side);
+	void user_input(const std::string &name, const config &input, int from_side);
 	void add_label(const terrain_label*);
 	void clear_labels(const std::string&, bool);
 	void add_rename(const std::string& name, const map_location& loc);
