@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "actions/attack.hpp"
@@ -18,18 +19,23 @@
 #include "font/pango/escape.hpp"
 #include "font/text_formatting.hpp"
 #include "formatter.hpp"
+#include "formula/string_utils.hpp"
 #include "preferences/game.hpp"
 #include "gettext.hpp"
 #include "language.hpp"
 #include "map/map.hpp"
 #include "mouse_events.hpp"
+#include "pathfind/pathfind.hpp"
+#include "picture.hpp"
 #include "reports.hpp"
+#include "resources.hpp"
 #include "color.hpp"
 #include "team.hpp"
 #include "terrain/movement.hpp"
 #include "tod_manager.hpp"
 #include "units/unit.hpp"
 #include "units/helper.hpp"
+#include "units/types.hpp"
 #include "whiteboard/manager.hpp"
 
 #include <cassert>
@@ -39,8 +45,6 @@
 #include <boost/format.hpp>
 #include <boost/lexical_cast.hpp>
 
-#include "utils/io.hpp"
-
 static void add_text(config &report, const std::string &text,
 	const std::string &tooltip, const std::string &help = "")
 {
@@ -138,6 +142,13 @@ static const unit *get_selected_unit(reports::context & rc)
 		rc.screen().show_everything());
 }
 
+static unit_const_ptr get_selected_unit_ptr(reports::context & rc)
+{
+	return rc.dc().get_visible_unit_shared_ptr(rc.screen().selected_hex(),
+		rc.teams()[rc.screen().viewing_team()],
+		rc.screen().show_everything());
+}
+
 static config gray_inactive(reports::context & rc, const std::string &str, const std::string& tooltip = "")
 {
 	if ( rc.screen().viewing_side() == rc.screen().playing_side() )
@@ -182,6 +193,12 @@ static config unit_type(const unit* u)
 	str << u->type_name();
 	tooltip << _("Type: ") << "<b>" << u->type_name() << "</b>\n"
 		<< u->unit_description();
+	if(const auto& notes = u->unit_special_notes(); !notes.empty()) {
+		tooltip << "\n\n" << _("Special Notes:") << '\n';
+		for(const auto& note : notes) {
+			tooltip << font::unicode_bullet << " " << note << '\n';
+		}
+	}
 	return text_report(str.str(), tooltip.str(), has_variations_prefix + "unit_" + u->type_id());
 }
 REPORT_GENERATOR(unit_type, rc)
@@ -214,6 +231,16 @@ REPORT_GENERATOR(selected_unit_race, rc)
 	return unit_race(u);
 }
 
+static std::string side_tooltip(const team& team)
+{
+	if(team.side_name().empty())
+		return "";
+
+	return VGETTEXT("Side: <b>$side_name</b> ($color_name)",
+			{{"side_name", team.side_name()},
+			 {"color_name", team::get_side_color_name_for_UI(team.side()) }});
+}
+
 static config unit_side(reports::context & rc, const unit* u)
 {
 	if (!u) return config();
@@ -230,12 +257,9 @@ static config unit_side(reports::context & rc, const unit* u)
 	std::stringstream text;
 	text << " " << u->side();
 
-	std::ostringstream tooltip;
-	if (!u_team.side_name().empty()) {
-		tooltip << _("Side:") << " <b>" <<  u_team.side_name() << "</b>";
-	}
-	add_image(report, flag_icon + mods, tooltip.str(), "");
-	add_text(report, text.str(), tooltip.str(), "");
+	const std::string& tooltip = side_tooltip(u_team);
+	add_image(report, flag_icon + mods, tooltip, "");
+	add_text(report, text.str(), tooltip, "");
 	return report;
 }
 REPORT_GENERATOR(unit_side, rc)
@@ -252,29 +276,7 @@ REPORT_GENERATOR(selected_unit_side, rc)
 static config unit_level(const unit* u)
 {
 	if (!u) return config();
-	std::ostringstream str, tooltip;
-	str << u->level();
-	tooltip << _("Level: ") << "<b>" << u->level() << "</b>\n";
-	const std::vector<std::string> &adv_to_types = u->advances_to_translated();
-	const std::vector<config> &adv_to_mods = u->get_modification_advances();
-	const bool has_advancements = !adv_to_types.empty() || !adv_to_mods.empty();
-	if(has_advancements) {
-		tooltip << _("Advancements:") << "\n<b>\t";
-		if(!adv_to_types.empty())
-			tooltip << utils::join(adv_to_types, "\n\t");
-		if(!adv_to_mods.empty()) {
-			if(!adv_to_types.empty())
-				tooltip << "\n\t";
-			std::vector<std::string> descriptions;
-			for(const config& adv : adv_to_mods)
-				descriptions.push_back(adv["description"].str());
-			tooltip << utils::join(descriptions, "\n\t");
-		}
-		tooltip << "</b>";
-	} else {
-		tooltip << _("No advancement");
-	}
-	return text_report(str.str(), tooltip.str());
+	return text_report(std::to_string(u->level()), unit_helper::unit_level_tooltip(*u));
 }
 REPORT_GENERATOR(unit_level, rc)
 {
@@ -334,7 +336,7 @@ static config unit_status(reports::context & rc, const unit* u)
 	if (!u) return config();
 	config res;
 	map_location displayed_unit_hex = rc.screen().displayed_unit_hex();
-	if (rc.map().on_board(displayed_unit_hex) && u->invisible(displayed_unit_hex, rc.dc())) {
+	if (rc.map().on_board(displayed_unit_hex) && u->invisible(displayed_unit_hex)) {
 		add_status(res, "misc/invisible.png", N_("invisible: "),
 			N_("This unit is invisible. It cannot be seen or attacked by enemy units."));
 	}
@@ -407,32 +409,35 @@ static config unit_abilities(const unit* u, const map_location& loc)
 	config res;
 
 	boost::dynamic_bitset<> active;
-	const std::vector<std::tuple<std::string, t_string,t_string,t_string>> &abilities = u->ability_tooltips(active, loc);
+	const auto abilities = u->ability_tooltips(active, loc);
+
 	const std::size_t abilities_size = abilities.size();
-	for ( std::size_t i = 0; i != abilities_size; ++i )
-	{
+	for(std::size_t i = 0; i != abilities_size; ++i) {
 		// Aliases for readability:
-		const std::string& id        = std::get<0>(abilities[i]);
-		const std::string& base_name = std::get<1>(abilities[i]).base_str();
-		const t_string& display_name = std::get<2>(abilities[i]);
-		const t_string& description  = std::get<3>(abilities[i]);
+		const auto& [id, base_name, display_name, description] = abilities[i];
 
 		std::ostringstream str, tooltip;
 
-		if ( active[i] )
+		if(active[i]) {
 			str << display_name;
-		else
+		} else {
 			str << span_color(font::inactive_ability_color) << display_name << naps;
-		if ( i + 1 != abilities_size )
+		}
+
+		if(i + 1 != abilities_size) {
 			str << ", ";
+		}
 
 		tooltip << _("Ability: ") << "<b>" << display_name << "</b>";
-		if ( !active[i] )
+		if(!active[i]) {
 			tooltip << "<i>" << _(" (inactive)") << "</i>";
+		}
+
 		tooltip << '\n' << description;
 
-		add_text(res, str.str(), tooltip.str(), "ability_" + id + base_name);
+		add_text(res, str.str(), tooltip.str(), "ability_" + id + base_name.base_str());
 	}
+
 	return res;
 }
 REPORT_GENERATOR(unit_abilities, rc)
@@ -539,8 +544,7 @@ static config unit_advancement_options(const unit* u)
 {
 	if (!u) return config();
 	config res;
-	typedef std::pair<std::string, std::string> pair_string;
-	for (const pair_string &ps : u->advancement_icons()) {
+	for (const auto& ps : u->advancement_icons()) {
 		add_image(res, ps.first, ps.second);
 	}
 	return res;
@@ -595,19 +599,31 @@ static config unit_defense(reports::context & rc, const unit* u, const map_locat
 	}
 
 	tooltip << "<b>" << _("Defense: ") << span_color(color)  << def << '%' << naps << "</b>";
-	return text_report(str.str(), tooltip.str());
+	const std::string has_variations_prefix = (u->type().show_variations_in_help() ? ".." : "");
+	return text_report(str.str(), tooltip.str(), has_variations_prefix + "unit_" + u->type_id());
 }
 REPORT_GENERATOR(unit_defense,rc)
 {
 	const unit *u = get_visible_unit(rc);
+	const team &viewing_team = rc.teams()[rc.screen().viewing_team()];
+	const map_location& mouseover_hex = rc.screen().mouseover_hex();
 	const map_location& displayed_unit_hex = rc.screen().displayed_unit_hex();
-	return unit_defense(rc, u, displayed_unit_hex);
+	const map_location& hex = (mouseover_hex.valid() && !viewing_team.shrouded(mouseover_hex)) ? mouseover_hex : displayed_unit_hex;
+	return unit_defense(rc, u, hex);
 }
 REPORT_GENERATOR(selected_unit_defense, rc)
 {
 	const unit *u = get_selected_unit(rc);
-	const map_location& selected_hex = rc.screen().selected_hex();
-	return unit_defense(rc, u, selected_hex);
+	const map_location& attack_indicator_src = game_display::get_singleton()->get_attack_indicator_src();
+	if(attack_indicator_src.valid())
+		return unit_defense(rc, u, attack_indicator_src);
+
+	const map_location& mouseover_hex = rc.screen().mouseover_hex();
+	const unit *visible_unit = get_visible_unit(rc);
+	if(visible_unit && u && visible_unit->id() != u->id() && mouseover_hex.valid())
+		return unit_defense(rc, u, mouseover_hex);
+	else
+		return unit_defense(rc, u, u->get_location());
 }
 
 static config unit_vision(const unit* u)
@@ -638,7 +654,7 @@ REPORT_GENERATOR(selected_unit_vision, rc)
 	return unit_vision(u);
 }
 
-static config unit_moves(reports::context & rc, const unit* u)
+static config unit_moves(reports::context & rc, const unit* u, bool is_visible_unit)
 {
 	if (!u) return config();
 	std::ostringstream str, tooltip;
@@ -689,18 +705,39 @@ static config unit_moves(reports::context & rc, const unit* u)
 
 	int grey = 128 + static_cast<int>((255 - 128) * movement_frac);
 	color_t c = color_t(grey, grey, grey);
-	str << span_color(c) << u->movement_left() << '/' << u->total_movement() << naps;
+	int numerator = u->movement_left();
+	if(is_visible_unit) {
+		const pathfind::marked_route& route = game_display::get_singleton()->get_route();
+		if(route.steps.size() > 0) {
+			numerator -= route.move_cost;
+			if(numerator < 0) {
+				// Multi-turn move
+				// TODO: what to show in this case?
+				numerator = 0;
+			}
+
+			// If the route captures a village, assume that uses up all remaining MP.
+			const auto& end = route.marks.find(route.steps.back());
+			if(end != route.marks.end() && end->second.capture) {
+				numerator = 0;
+			}
+		} else {
+			// TODO: if the mouseover hex is unreachable (for example, a deep water hex
+			// and the current unit is a land unit), what to show?
+		}
+	}
+	str << span_color(c) << numerator << '/' << u->total_movement() << naps;
 	return text_report(str.str(), tooltip.str());
 }
 REPORT_GENERATOR(unit_moves, rc)
 {
 	const unit *u = get_visible_unit(rc);
-	return unit_moves(rc, u);
+	return unit_moves(rc, u, true);
 }
 REPORT_GENERATOR(selected_unit_moves, rc)
 {
 	const unit *u = get_selected_unit(rc);
-	return unit_moves(rc, u);
+	return unit_moves(rc, u, false);
 }
 
 static inline const color_t attack_info_percent_color(int resistance)
@@ -711,19 +748,25 @@ static inline const color_t attack_info_percent_color(int resistance)
 	return font::YELLOW_COLOR;
 }
 
-static int attack_info(reports::context & rc, const attack_type &at, config &res, const unit &u, const map_location &hex)
+static int attack_info(reports::context & rc, const attack_type &at, config &res, const unit &u, const map_location &hex, const unit* sec_u = nullptr, const_attack_ptr sec_u_weapon = nullptr)
 {
 	std::ostringstream str, tooltip;
 	int damage = 0;
 
+	struct string_with_tooltip {
+		std::string str;
+		std::string tooltip;
+	};
+
 	{
-		auto ctx = at.specials_context(unit_const_ptr(&u), hex, u.side() == rc.screen().playing_side());
+		auto ctx = at.specials_context(u.shared_from_this(), hex, u.side() == rc.screen().playing_side());
 		int base_damage = at.damage();
 		int specials_damage = at.modified_damage(false);
 		int damage_multiplier = 100;
+		const_attack_ptr weapon  = at.shared_from_this();
 		int tod_bonus = combat_modifier(get_visible_time_of_day_at(rc, hex), u.alignment(), u.is_fearless());
 		damage_multiplier += tod_bonus;
-		int leader_bonus = under_leadership(u, hex);
+		int leader_bonus = under_leadership(u, hex, weapon);
 		if (leader_bonus != 0)
 			damage_multiplier += leader_bonus;
 
@@ -804,14 +847,22 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 			tooltip << '\t' << _("With specials: ") << num_attacks << '\n';
 		}
 
-		add_text(res, flush(str), flush(tooltip));
+		const string_with_tooltip damage_and_num_attacks {flush(str), flush(tooltip)};
 
 		std::string range = string_table["range_" + at.range()];
 		std::string lang_type = string_table["type_" + at.type()];
 
-		str << span_color(font::weapon_details_color) << "  " << "  "
-			<< range << font::weapon_details_sep
-			<< lang_type << "</span>\n";
+		// SCALE_INTO_SHARP() is needed in case the 72x72 images/misc/missing-image.png is substituted.
+		const std::string range_png = std::string("icons/profiles/") + at.range() + "_attack.png~SCALE_INTO_SHARP(16,16)";
+		const std::string type_png = std::string("icons/profiles/") + at.type() + ".png~SCALE_INTO_SHARP(16,16)";
+		const bool range_png_exists = image::locator(range_png).file_exists();
+		const bool type_png_exists = image::locator(type_png).file_exists();
+
+		if(!range_png_exists || !type_png_exists) {
+			str << span_color(font::weapon_details_color) << "  " << "  "
+				<< range << font::weapon_details_sep
+				<< lang_type << "</span>\n";
+		}
 
 		tooltip << _("Weapon range: ") << "<b>" << range << "</b>\n"
 			<< _("Damage type: ")  << "<b>" << lang_type << "</b>\n"
@@ -830,8 +881,8 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 			if (!unit_team.is_enemy(enemy.side()))
 				continue;
 			const map_location &loc = enemy.get_location();
-			if (viewing_team.fogged(loc) ||
-				(viewing_team.is_enemy(enemy.side()) && enemy.invisible(loc, rc.dc())))
+			const bool see_all = game_config::debug || rc.screen().show_everything();
+			if (!enemy.is_visible_to_team(viewing_team, see_all))
 				continue;
 			bool new_type = seen_types.insert(enemy.type_id()).second;
 			if (new_type) {
@@ -840,8 +891,7 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 			}
 		}
 
-		typedef std::pair<int, std::set<std::string>> resist_units;
-		for (const resist_units &resist : resistances) {
+		for (const auto& resist : resistances) {
 			int damage_with_resistance = round_damage(specials_damage, damage_multiplier * resist.first, damage_divisor);
 			tooltip << "<b>" << damage_with_resistance << "</b>  "
 				<< "<span color='" << attack_info_percent_color(resist.first-100).to_hex_string() << "'>"
@@ -850,7 +900,23 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 				<< " :  \t" // spaces to align the tab to a multiple of 8
 				<< utils::join(resist.second, " " + font::unicode_bullet + " ") << '\n';
 		}
-		add_text(res, flush(str), flush(tooltip));
+		const string_with_tooltip damage_versus {flush(str), flush(tooltip)};
+
+#if 0
+		// We wanted to use the attack icon here, but couldn't find a good layout.
+		// The default images are 60x60 and have a 2-pixel transparent border. Trim it.
+		// The first SCALE() accounts for theoretically possible add-ons attack images larger than 60x60.
+		const std::string attack_icon = at.icon() + "~SCALE_INTO_SHARP(60,60)~CROP(2,2,56,56)~SCALE_INTO_SHARP(32,32)";
+		add_image(res, attack_icon, at.name());
+		add_text(res, " ", "");
+#endif
+
+		// The icons are 16x16. We add 5px padding for alignment reasons (placement of the icon in relation to ascender and descender letters).
+		const std::string spacer = "misc/blank.png~CROP(0, 0, 16, 21)"; // 21 == 16+5
+		if (range_png_exists) add_image(res, spacer + "~BLIT(" + range_png + ",0,5)", damage_versus.tooltip);
+		if (type_png_exists) add_image(res, spacer + "~BLIT(" + type_png + ",0,5)", damage_versus.tooltip);
+		add_text(res, damage_and_num_attacks.str, damage_and_num_attacks.tooltip);
+		add_text(res, damage_versus.str, damage_versus.tooltip); // This string is usually empty
 
 		const std::string &accuracy_parry = at.accuracy_parry_description();
 		if (!accuracy_parry.empty())
@@ -872,11 +938,15 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 	}
 
 	{
-		auto ctx = at.specials_context_for_listing(u.side() == rc.screen().playing_side());
+		//If we have a second unit, do the 2-unit specials_context
+		bool attacking = (u.side() == rc.screen().playing_side());
+		auto ctx = (sec_u == nullptr) ? at.specials_context_for_listing(attacking) :
+						at.specials_context(u.shared_from_this(), sec_u->shared_from_this(), hex, sec_u->get_location(), attacking, sec_u_weapon);
+
 		boost::dynamic_bitset<> active;
 		const std::vector<std::pair<t_string, t_string>> &specials = at.special_tooltips(&active);
-		const size_t specials_size = specials.size();
-		for ( size_t i = 0; i != specials_size; ++i )
+		const std::size_t specials_size = specials.size();
+		for ( std::size_t i = 0; i != specials_size; ++i )
 		{
 			// Aliases for readability:
 			const t_string &name = specials[i].first;
@@ -893,6 +963,15 @@ static int attack_info(reports::context & rc, const attack_type &at, config &res
 
 			add_text(res, flush(str), flush(tooltip), help_page);
 		}
+		if(!specials.empty()) {
+			// Add some padding so the end of the specials list
+			// isn't too close vertically to the attack icons of
+			// the next attack. Also for symmetry with the padding
+			// above the list of specials (below the attack icon line).
+			const std::string spacer = "misc/blank.png~CROP(0, 0, 1, 5)";
+			add_image(res, spacer, "");
+			add_text(res, "\n", "");
+		}
 	}
 	return damage;
 }
@@ -917,11 +996,12 @@ static std::string format_hp(unsigned hp)
 	return res.str();
 }
 
-static config unit_weapons(reports::context & rc, const unit *attacker, const map_location &attacker_pos, const unit *defender, bool show_attacker)
+static config unit_weapons(reports::context & rc, unit_const_ptr attacker, const map_location &attacker_pos, const unit *defender, bool show_attacker)
 {
 	if (!attacker || !defender) return config();
 
-	const unit* u = show_attacker ? attacker : defender;
+	const unit* u = show_attacker ? attacker.get() : defender;
+	const unit* sec_u = !show_attacker ? attacker.get() : defender;
 	const map_location unit_loc = show_attacker ? attacker_pos : defender->get_location();
 
 	std::ostringstream str, tooltip;
@@ -931,8 +1011,7 @@ static config unit_weapons(reports::context & rc, const unit *attacker, const ma
 	for (unsigned int i = 0; i < attacker->attacks().size(); i++) {
 		// skip weapons with attack_weight=0
 		if (attacker->attacks()[i].attack_weight() > 0) {
-			battle_context weapon(rc.units(), attacker_pos, defender->get_location(), i, -1, 0.0, nullptr, attacker);
-			weapons.emplace_back(std::move(weapon));
+			weapons.emplace_back(rc.units(), attacker_pos, defender->get_location(), i, -1, 0.0, nullptr, attacker);
 		}
 	}
 
@@ -945,6 +1024,8 @@ static config unit_weapons(reports::context & rc, const unit *attacker, const ma
 
 		const battle_context_unit_stats& context_unit_stats =
 				show_attacker ? weapon.get_attacker_stats() : weapon.get_defender_stats();
+		const battle_context_unit_stats& other_context_unit_stats =
+				!show_attacker ? weapon.get_attacker_stats() : weapon.get_defender_stats();
 
 		int total_damage = 0;
 		int base_damage = 0;
@@ -954,7 +1035,7 @@ static config unit_weapons(reports::context & rc, const unit *attacker, const ma
 
 		color_t dmg_color = font::weapon_color;
 		if (context_unit_stats.weapon) {
-			base_damage = attack_info(rc, *context_unit_stats.weapon, res, *u, unit_loc);
+			base_damage = attack_info(rc, *context_unit_stats.weapon, res, *u, unit_loc, sec_u, other_context_unit_stats.weapon);
 			total_damage = context_unit_stats.damage;
 			num_blows = context_unit_stats.num_blows;
 			chance_to_hit = context_unit_stats.chance_to_hit;
@@ -1048,6 +1129,16 @@ static config unit_weapons(reports::context & rc, const unit *u, const map_locat
 		add_text(res,  span_color(font::weapon_details_color)
 				+ attack_headline + "</span>" + '\n', "");
 
+		const auto left = u->attacks_left(false), max = u->max_attacks();
+		if(max != 1) {
+			// TRANSLATORS: This string is shown in the sidebar beneath the word "Attacks" when a unit can attack multiple times per turn
+			const std::string line = VGETTEXT("Remaining: $left/$max",
+							{{"left", std::to_string(left)},
+							 {"max",  std::to_string(max)}});
+			add_text(res, "  " + span_color(font::weapon_details_color, line) + "\n",
+				_("This unit can attack multiple times per turn."));
+		}
+
 		for (const attack_type &at : u->attacks())
 		{
 			attack_info(rc, at, res, *u, hex);
@@ -1067,11 +1158,11 @@ REPORT_GENERATOR(unit_weapons, rc)
 }
 REPORT_GENERATOR(highlighted_unit_weapons, rc)
 {
-	const unit *u = get_selected_unit(rc);
+	unit_const_ptr u = get_selected_unit_ptr(rc);
 	const unit *sec_u = get_visible_unit(rc);
 
-	if (!u) return config();
-	if (!sec_u || u == sec_u) return unit_weapons(rc, sec_u, rc.screen().mouseover_hex());
+	if (!u) return report_unit_weapons(rc);
+	if (!sec_u || u.get() == sec_u) return unit_weapons(rc, sec_u, rc.screen().mouseover_hex());
 
 	map_location highlighted_hex = rc.screen().displayed_unit_hex();
 	map_location attack_loc;
@@ -1081,15 +1172,16 @@ REPORT_GENERATOR(highlighted_unit_weapons, rc)
 	if (!attack_loc.valid())
 		return unit_weapons(rc, sec_u, rc.screen().mouseover_hex());
 
+	//TODO: shouldn't this pass sec_u as secodn parameter ?
 	return unit_weapons(rc, u, attack_loc, sec_u, false);
 }
 REPORT_GENERATOR(selected_unit_weapons, rc)
 {
-	const unit *u = get_selected_unit(rc);
+	unit_const_ptr u = get_selected_unit_ptr(rc);
 	const unit *sec_u = get_visible_unit(rc);
 
 	if (!u) return config();
-	if (!sec_u || u == sec_u) return unit_weapons(rc, u, u->get_location());
+	if (!sec_u || u.get() == sec_u) return unit_weapons(rc, u.get(), u->get_location());
 
 	map_location highlighted_hex = rc.screen().displayed_unit_hex();
 	map_location attack_loc;
@@ -1097,7 +1189,7 @@ REPORT_GENERATOR(selected_unit_weapons, rc)
 		attack_loc = rc.mhb()->current_unit_attacks_from(highlighted_hex);
 
 	if (!attack_loc.valid())
-		return unit_weapons(rc, u, u->get_location());
+		return unit_weapons(rc, u.get(), u->get_location());
 
 	return unit_weapons(rc, u, attack_loc, sec_u, true);
 }
@@ -1128,17 +1220,12 @@ REPORT_GENERATOR(unit_profile, rc)
 	return image_report(u->small_profile());
 }
 
-REPORT_GENERATOR(tod_stats, rc)
+static config tod_stats_at(reports::context & rc, const map_location& hex)
 {
 	std::ostringstream tooltip;
 	std::ostringstream text;
 
-	const map_location& selected_hex = rc.screen().selected_hex();
-	const map_location& mouseover_hex = rc.screen().mouseover_hex();
-
-	const map_location& hex = mouseover_hex.valid() ? mouseover_hex : selected_hex;
-
-	const map_location& tod_schedule_hex = display::get_singleton()->shrouded(hex) ? map_location::null_location() : hex;
+	const map_location& tod_schedule_hex = (hex.valid() && !display::get_singleton()->shrouded(hex)) ? hex : map_location::null_location();
 	const std::vector<time_of_day>& schedule = rc.tod().times(tod_schedule_hex);
 
 	tooltip << _("Time of day schedule:") << " \n";
@@ -1156,6 +1243,21 @@ REPORT_GENERATOR(tod_stats, rc)
 
 	return text_report(text.str(), tooltip.str(), "..schedule");
 }
+REPORT_GENERATOR(tod_stats, rc)
+{
+	map_location mouseover_hex = rc.screen().mouseover_hex();
+	if (mouseover_hex.valid()) return tod_stats_at(rc, mouseover_hex);
+	return tod_stats_at(rc, rc.screen().selected_hex());
+}
+REPORT_GENERATOR(selected_tod_stats, rc)
+{
+	const unit *u = get_selected_unit(rc);
+	if(!u) return tod_stats_at(rc, map_location::null_location());
+	const map_location& attack_indicator_src = game_display::get_singleton()->get_attack_indicator_src();
+	const map_location& hex =
+		attack_indicator_src.valid() ? attack_indicator_src : u->get_location();
+	return tod_stats_at(rc, hex);
+}
 
 static config time_of_day_at(reports::context & rc, const map_location& mouseover_hex)
 {
@@ -1163,16 +1265,17 @@ static config time_of_day_at(reports::context & rc, const map_location& mouseove
 	time_of_day tod = get_visible_time_of_day_at(rc, mouseover_hex);
 
 	int b = tod.lawful_bonus;
-
+	int l = generic_combat_modifier(b, unit_type::ALIGNMENT::LIMINAL, false, rc.tod().get_max_liminal_bonus());
 	std::string  lawful_color("white");
 	std::string chaotic_color("white");
 	std::string liminal_color("white");
 
-	// Use same red/green colouring scheme as time_of_day_bonus_colored() in help/help_impl.cpp for consistency
 	if (b != 0) {
-		lawful_color  = (b > 0) ? "green" : "red";
-		chaotic_color = (b < 0) ? "green" : "red";
-		liminal_color = "red";
+		lawful_color  = (b > 0) ? "#0f0" : "#f00";
+		chaotic_color = (b < 0) ? "#0f0" : "#f00";
+	}
+	if (l != 0) {
+		liminal_color = (l > 0) ? "#0f0" : "#f00";
 	}
 	tooltip << _("Time of day:") << " <b>" << tod.name << "</b>\n"
 		<< _("Lawful units: ") << "<span foreground=\"" << lawful_color  << "\">"
@@ -1181,7 +1284,7 @@ static config time_of_day_at(reports::context & rc, const map_location& mouseove
 		<< _("Chaotic units: ") << "<span foreground=\"" << chaotic_color << "\">"
 		<< utils::signed_percent(-b) << "</span>\n"
 		<< _("Liminal units: ") << "<span foreground=\"" << liminal_color << "\">"
-		<< utils::signed_percent(-(std::abs(b))) << "</span>\n";
+		<< utils::signed_percent(l) << "</span>\n";
 
 	std::string tod_image = tod.image;
 	if(tod.bonus_modified > 0) {
@@ -1198,6 +1301,15 @@ REPORT_GENERATOR(time_of_day, rc)
 	if (mouseover_hex.valid()) return time_of_day_at(rc, mouseover_hex);
 	return time_of_day_at(rc, rc.screen().selected_hex());
 }
+REPORT_GENERATOR(selected_time_of_day, rc)
+{
+	const unit *u = get_selected_unit(rc);
+	if(!u) return time_of_day_at(rc, map_location::null_location());
+	const map_location& attack_indicator_src = game_display::get_singleton()->get_attack_indicator_src();
+	const map_location& hex =
+		attack_indicator_src.valid() ? attack_indicator_src : u->get_location();
+	return time_of_day_at(rc, hex);
+}
 
 static config unit_box_at(reports::context & rc, const map_location& mouseover_hex)
 {
@@ -1206,6 +1318,7 @@ static config unit_box_at(reports::context & rc, const map_location& mouseover_h
 	time_of_day local_tod = get_visible_time_of_day_at(rc, mouseover_hex);
 
 	int bonus = local_tod.lawful_bonus;
+	int l = generic_combat_modifier(bonus, unit_type::ALIGNMENT::LIMINAL, false, rc.tod().get_max_liminal_bonus());
 
 	std::string  lawful_color("white");
 	std::string chaotic_color("white");
@@ -1214,7 +1327,9 @@ static config unit_box_at(reports::context & rc, const map_location& mouseover_h
 	if (bonus != 0) {
 		lawful_color  = (bonus > 0) ? "green" : "red";
 		chaotic_color = (bonus < 0) ? "green" : "red";
-		liminal_color = "red";
+	}
+	if (l != 0) {
+		liminal_color = (l > 0) ? "green" : "red";
 	}
 	tooltip << local_tod.name << '\n'
 		<< _("Lawful units: ") << "<span foreground=\"" << lawful_color  << "\">"
@@ -1223,7 +1338,7 @@ static config unit_box_at(reports::context & rc, const map_location& mouseover_h
 		<< _("Chaotic units: ") << "<span foreground=\"" << chaotic_color << "\">"
 		<< utils::signed_percent(-bonus) << "</span>\n"
 		<< _("Liminal units: ") << "<span foreground=\"" << liminal_color << "\">"
-		<< utils::signed_percent(-(std::abs(bonus))) << "</span>\n";
+		<< utils::signed_percent(l) << "</span>\n";
 
 	std::string local_tod_image  = "themes/classic/" + local_tod.image;
 	std::string global_tod_image = "themes/classic/" + global_tod.image;
@@ -1453,7 +1568,7 @@ REPORT_GENERATOR(terrain, rc)
 	std::ostringstream str;
 	if (map.is_village(mouseover_hex))
 	{
-		int owner = rc.dc().village_owner(mouseover_hex) + 1;
+		int owner = rc.dc().village_owner(mouseover_hex);
 		if (owner == 0 || viewing_team.fogged(mouseover_hex)) {
 			str << map.get_terrain_info(terrain).income_description();
 		} else if (owner == viewing_side) {
@@ -1537,7 +1652,7 @@ REPORT_GENERATOR(side_playing, rc)
 	std::string mods = "~RC(" + old_rgb + ">" + new_rgb + ")";
 	if (flag_icon.empty())
 		flag_icon = game_config::images::flag_icon;
-	return image_report(flag_icon + mods, active_team.side_name());
+	return image_report(flag_icon + mods, side_tooltip(active_team));
 }
 
 REPORT_GENERATOR(observers, rc)
@@ -1587,8 +1702,8 @@ REPORT_GENERATOR(report_clock, /*rc*/)
 		? "%I:%M %p"
 		: "%H:%M";
 
-	time_t t = std::time(nullptr);
-	ss << utils::put_time(std::localtime(&t), format);
+	std::time_t t = std::time(nullptr);
+	ss << std::put_time(std::localtime(&t), format);
 	add_text(report, ss.str(), _("Clock"));
 
 	return report;
