@@ -1,16 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by Jörg Hinrichs, refactored from various
-   places formerly created by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by Jörg Hinrichs, David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "save_index.hpp"
@@ -43,11 +43,11 @@ void extract_summary_from_config(config&, config&);
 
 void save_index_class::rebuild(const std::string& name)
 {
-	time_t modified = filesystem::file_modified_time(filesystem::get_saves_dir() + "/" + name);
+	std::time_t modified = filesystem::file_modified_time(dir_ + "/" + name);
 	rebuild(name, modified);
 }
 
-void save_index_class::rebuild(const std::string& name, const time_t& modified)
+void save_index_class::rebuild(const std::string& name, const std::time_t& modified)
 {
 	log_scope("load_summary_from_file");
 
@@ -56,7 +56,7 @@ void save_index_class::rebuild(const std::string& name, const time_t& modified)
 	try {
 		config full;
 		std::string dummy;
-		read_save_file(name, full, &dummy);
+		read_save_file(dir_, name, full, &dummy);
 
 		extract_summary_from_config(full, summary);
 	} catch(const game::load_game_failed&) {
@@ -70,11 +70,11 @@ void save_index_class::rebuild(const std::string& name, const time_t& modified)
 void save_index_class::remove(const std::string& name)
 {
 	config& root = data();
-	root.remove_attribute(name);
+	root.remove_children("save", [&name](const config& d) { return name == d["save"]; });
 	write_save_index();
 }
 
-void save_index_class::set_modified(const std::string& name, const time_t& modified)
+void save_index_class::set_modified(const std::string& name, const std::time_t& modified)
 {
 	modified_[name] = modified;
 }
@@ -82,20 +82,51 @@ void save_index_class::set_modified(const std::string& name, const time_t& modif
 config& save_index_class::get(const std::string& name)
 {
 	config& result = data(name);
-	time_t m = modified_[name];
+	std::time_t m = modified_[name];
 
 	config::attribute_value& mod_time = result["mod_time"];
-	if(mod_time.empty() || static_cast<time_t>(mod_time.to_int()) != m) {
+	if(mod_time.empty() || mod_time.to_time_t() != m) {
 		rebuild(name, m);
 	}
 
 	return result;
 }
 
+const std::string& save_index_class::dir() const
+{
+	return dir_;
+}
+
+void save_index_class::clean_up_index()
+{
+	config &root = data();
+
+	std::vector<std::string> filenames;
+	filesystem::get_files_in_dir(dir(), &filenames);
+
+	if(root.all_children_count() > filenames.size()) {
+		root.remove_children("save", [&filenames](const config& d)
+			{
+				return std::find(filenames.begin(), filenames.end(), d["save"]) == filenames.end();
+			}
+		);
+	}
+}
+
 void save_index_class::write_save_index()
 {
 	log_scope("write_save_index()");
 
+	if(read_only_) {
+		LOG_SAVE << "no-op: read_only instance";
+		return;
+	}
+
+	if(clean_up_index_) {
+		clean_up_index();
+		clean_up_index_ = false;
+	}
+
 	try {
 		filesystem::scoped_ostream stream = filesystem::ostream_file(filesystem::get_save_index_file());
 
@@ -110,13 +141,22 @@ void save_index_class::write_save_index()
 	}
 }
 
-save_index_class::save_index_class()
+save_index_class::save_index_class(const std::string& dir)
 	: loaded_(false)
 	, data_()
 	, modified_()
+	, dir_(dir)
+	, read_only_(true)
+	, clean_up_index_(true)
 {
 }
 
+save_index_class::save_index_class(create_for_default_saves_dir)
+	: save_index_class(filesystem::get_saves_dir())
+{
+	read_only_ = false;
+}
+
 config& save_index_class::data(const std::string& name)
 {
 	config& cfg = data();
@@ -167,8 +207,13 @@ void save_index_class::fix_leader_image_path(config& data)
 	}
 }
 
-save_index_class save_index_manager;
+std::shared_ptr<save_index_class> save_index_class::default_saves_dir()
+{
+	static auto instance = std::make_shared<save_index_class>(create_for_default_saves_dir::yes);
+	return instance;
+}
 
+/** Filter file names based on input string. */
 class filename_filter
 {
 public:
@@ -186,14 +231,30 @@ private:
 	std::string filter_;
 };
 
-/** Get a list of available saves. */
-std::vector<save_info> get_saves_list(const std::string* dir, const std::string* filter)
+/** Ignore certain files - in particular, the auto-generated Steam cloud record. */
+class filename_ignore
 {
-	create_save_info creator(dir);
+public:
+	bool operator()(const std::string& filename) const
+	{
+		return std::find(files_to_ignore_.begin(), files_to_ignore_.end(), filename) != files_to_ignore_.end();
+	}
+
+private:
+	/* Steam documentation indicates games can ignore its auto-generated 'steam_autocloud.vdf'.
+	   Reference: https://partner.steamgames.com/doc/features/cloud (under Steam Auto-Cloud section as of September 2021) */
+	const std::vector<std::string> files_to_ignore_ = std::vector {std::string {"steam_autocloud.vdf"}};
+};
+
+/** Get a list of available saves. */
+std::vector<save_info> save_index_class::get_saves_list(const std::string* filter)
+{
+	create_save_info creator(shared_from_this());
 
 	std::vector<std::string> filenames;
-	filesystem::get_files_in_dir(creator.dir, &filenames);
+	filesystem::get_files_in_dir(dir(), &filenames);
 
+	filenames.erase(std::remove_if(filenames.begin(), filenames.end(), filename_ignore()), filenames.end());
 	if(filter) {
 		filenames.erase(
 			std::remove_if(filenames.begin(), filenames.end(), filename_filter(*filter)), filenames.end());
@@ -208,15 +269,17 @@ std::vector<save_info> get_saves_list(const std::string* dir, const std::string*
 
 const config& save_info::summary() const
 {
-	return save_index_manager.get(name());
+	return save_index_->get(name());
 }
 
 std::string save_info::format_time_local() const
 {
-	if(tm* tm_l = localtime(&modified())) {
+	if(std::tm* tm_l = std::localtime(&modified())) {
 		const std::string format = preferences::use_twelve_hour_clock_format()
-			? _("%a %b %d %I:%M %p %Y")
-			: _("%a %b %d %H:%M %Y");
+			// TRANSLATORS: Day of week + month + day of month + year + 12-hour time, eg 'Tue Nov 02 2021, 1:59 PM'. Format for your locale.
+			? _("%a %b %d %Y, %I:%M %p")
+			// TRANSLATORS: Day of week + month + day of month + year + 24-hour time, eg 'Tue Nov 02 2021, 13:59'. Format for your locale.
+			: _("%a %b %d %Y, %H:%M");
 
 		return translation::strftime(format, tm_l);
 	}
@@ -227,36 +290,40 @@ std::string save_info::format_time_local() const
 
 std::string save_info::format_time_summary() const
 {
-	time_t t = modified();
+	std::time_t t = modified();
 	return utils::format_time_summary(t);
 }
 
 bool save_info_less_time::operator()(const save_info& a, const save_info& b) const
 {
+	// This translatable string must be same one as in replay_savegame::create_initial_filename.
+	// TODO: we really shouldn't be relying on translatable strings like this, especially since
+	// old savefiles may have been created in a different language than the current UI language
+	const std::string replay_str = " " + _("replay");
 	if(a.modified() > b.modified()) {
 		return true;
 	} else if(a.modified() < b.modified()) {
 		return false;
-	} else if(a.name().find(_(" replay")) == std::string::npos && b.name().find(_(" replay")) != std::string::npos) {
+	} else if(a.name().find(replay_str) == std::string::npos && b.name().find(replay_str) != std::string::npos) {
 		// Special funky case; for files created in the same second,
 		// a replay file sorts less than a non-replay file.  Prevents
 		// a timing-dependent bug where it may look like, at the end
 		// of a scenario, the replay and the autosave for the next
 		// scenario are displayed in the wrong order.
 		return true;
-	} else if(a.name().find(_(" replay")) != std::string::npos && b.name().find(_(" replay")) == std::string::npos) {
+	} else if(a.name().find(replay_str) != std::string::npos && b.name().find(replay_str) == std::string::npos) {
 		return false;
 	} else {
 		return a.name() > b.name();
 	}
 }
 
-static filesystem::scoped_istream find_save_file(
+static filesystem::scoped_istream find_save_file(const std::string& dir,
 		const std::string& name, const std::vector<std::string>& suffixes)
 {
 	for(const std::string& suf : suffixes) {
 		filesystem::scoped_istream file_stream =
-			filesystem::istream_file(filesystem::get_saves_dir() + "/" + name + suf);
+			filesystem::istream_file(dir + "/" + name + suf);
 
 		if(!file_stream->fail()) {
 			return file_stream;
@@ -267,10 +334,10 @@ static filesystem::scoped_istream find_save_file(
 	throw game::load_game_failed();
 }
 
-void read_save_file(const std::string& name, config& cfg, std::string* error_log)
+void read_save_file(const std::string& dir, const std::string& name, config& cfg, std::string* error_log)
 {
 	static const std::vector<std::string> suffixes{"", ".gz", ".bz2"};
-	filesystem::scoped_istream file_stream = find_save_file(name, suffixes);
+	filesystem::scoped_istream file_stream = find_save_file(dir, name, suffixes);
 
 	cfg.clear();
 	try {
@@ -308,8 +375,14 @@ void read_save_file(const std::string& name, config& cfg, std::string* error_log
 	}
 }
 
-void remove_old_auto_saves(const int autosavemax, const int infinite_auto_saves)
+void save_index_class::delete_old_auto_saves(const int autosavemax, const int infinite_auto_saves)
 {
+	log_scope("delete_old_auto_saves()");
+	if(read_only_) {
+		LOG_SAVE << "no-op: read_only instance";
+		return;
+	}
+
 	const std::string auto_save = _("Auto-Save");
 
 	int countdown = autosavemax;
@@ -317,7 +390,7 @@ void remove_old_auto_saves(const int autosavemax, const int infinite_auto_saves)
 		return;
 	}
 
-	std::vector<save_info> games = get_saves_list(nullptr, &auto_save);
+	std::vector<save_info> games = get_saves_list(&auto_save);
 	for(std::vector<save_info>::iterator i = games.begin(); i != games.end(); ++i) {
 		if(countdown-- <= 0) {
 			LOG_SAVE << "Deleting savegame '" << i->name() << "'\n";
@@ -326,23 +399,28 @@ void remove_old_auto_saves(const int autosavemax, const int infinite_auto_saves)
 	}
 }
 
-void delete_game(const std::string& name)
+void save_index_class::delete_game(const std::string& name)
 {
-	filesystem::delete_file(filesystem::get_saves_dir() + "/" + name);
+	if(read_only_) {
+		log_scope("delete_game()");
+		LOG_SAVE << "no-op: read_only instance";
+		return;
+	}
 
-	save_index_manager.remove(name);
+	filesystem::delete_file(dir() + "/" + name);
+	remove(name);
 }
 
-create_save_info::create_save_info(const std::string* d)
-	: dir(d ? *d : filesystem::get_saves_dir())
+create_save_info::create_save_info(const std::shared_ptr<save_index_class>& manager)
+	: manager_(manager)
 {
 }
 
 save_info create_save_info::operator()(const std::string& filename) const
 {
-	time_t modified = filesystem::file_modified_time(dir + "/" + filename);
-	save_index_manager.set_modified(filename, modified);
-	return save_info(filename, modified);
+	std::time_t modified = filesystem::file_modified_time(manager_->dir() + "/" + filename);
+	manager_->set_modified(filename, modified);
+	return save_info(filename, manager_, modified);
 }
 
 void extract_summary_from_config(config& cfg_save, config& cfg_summary)
@@ -434,11 +512,11 @@ void extract_summary_from_config(config& cfg_save, config& cfg_summary)
 
 			// We need a binary path-independent path to the leader image here so it can be displayed
 			// for campaign-specific units even when the campaign isn't loaded yet.
-			std::string leader_image_path = filesystem::get_independent_image_path(leader_image);
+			std::string leader_image_path = filesystem::get_independent_binary_file_path("images", leader_image);
 
 			// If the image path was found, we append the leader TC modifier. If it's not (such as in
 			// the case where the binary path hasn't been loaded yet, perhaps due to save_index being
-			// deleted), the unaltered image path is used and will be parsed by get_independent_image_path
+			// deleted), the unaltered image path is used and will be parsed by get_independent_binary_file_path
 			// at runtime.
 			if(!leader_image_path.empty()) {
 				leader_image_path += leader_image_tc_modifier;
