@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2003 - 2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -123,7 +124,7 @@ saved_game::saved_game()
 	, replay_start_()
 	, classification_()
 	, mp_settings_()
-	, starting_point_type_(STARTING_POINT_NONE)
+	, starting_point_type_(starting_point::NONE)
 	, starting_point_()
 	, replay_data_()
 	, skip_story_(false)
@@ -136,11 +137,10 @@ saved_game::saved_game(config cfg)
 	, replay_start_()
 	, classification_(cfg)
 	, mp_settings_()
-	, starting_point_type_(STARTING_POINT_NONE)
+	, starting_point_type_(starting_point::NONE)
 	, starting_point_()
 	, replay_data_()
 	, skip_story_(false)
-
 {
 	set_data(cfg);
 }
@@ -181,7 +181,7 @@ void saved_game::write_config(config_writer& out) const
 	write_general_info(out);
 	write_starting_point(out);
 
-	if(!this->replay_start_.empty()) {
+	if(!replay_start_.empty()) {
 		out.write_child("replay_start", replay_start_);
 	}
 
@@ -194,9 +194,9 @@ void saved_game::write_config(config_writer& out) const
 
 void saved_game::write_starting_point(config_writer& out) const
 {
-	if(starting_point_type_ == STARTING_POINT_SNAPSHOT) {
+	if(starting_point_type_ == starting_point::SNAPSHOT) {
 		out.write_child("snapshot", starting_point_);
-	} else if(starting_point_type_ == STARTING_POINT_SCENARIO) {
+	} else if(starting_point_type_ == starting_point::SCENARIO) {
 		out.write_child("scenario", starting_point_);
 	}
 }
@@ -215,14 +215,20 @@ void saved_game::write_general_info(config_writer& out) const
 
 void saved_game::set_defaults()
 {
-	const bool is_loaded_game = this->starting_point_type_ != STARTING_POINT_SCENARIO;
+	const bool is_loaded_game = starting_point_type_ != starting_point::SCENARIO;
 	const bool is_multiplayer_tag = classification().get_tagname() == "multiplayer";
+	const game_config_view& game_config = game_config_manager::get()->game_config();
 
 	static const std::vector<std::string> team_defaults {
 		"carryover_percentage",
 		"carryover_add",
 	};
 
+	if(const config& campaign = game_config.find_child("campaign", "id", classification_.campaign)) {
+		bool require_campaign = campaign["require_campaign"].to_bool(true);
+		starting_point_["require_scenario"] = require_campaign;
+	}
+
 	for(config& side : starting_point_.child_range("side")) {
 		// Set save_id default value directly after loading to its default to prevent different default behaviour in
 		// mp_connect code and sp code.
@@ -261,16 +267,16 @@ void saved_game::set_defaults()
 
 void saved_game::expand_scenario()
 {
-	if(this->starting_point_type_ == STARTING_POINT_NONE && !has_carryover_expanded_) {
-		game_config_manager::get()->load_game_config_for_game(this->classification());
+	if(starting_point_type_ == starting_point::NONE && !has_carryover_expanded_) {
+		game_config_manager::get()->load_game_config_for_game(classification(), carryover_["next_scenario"]);
 
-		const config& game_config = game_config_manager::get()->game_config();
+		const game_config_view& game_config = game_config_manager::get()->game_config();
 		const config& scenario =
 			game_config.find_child(classification().get_tagname(), "id", carryover_["next_scenario"]);
 
 		if(scenario) {
-			this->starting_point_type_ = STARTING_POINT_SCENARIO;
-			this->starting_point_ = scenario;
+			starting_point_type_ = starting_point::SCENARIO;
+			starting_point_ = scenario;
 
 			// A hash has to be generated using an unmodified scenario data.
 			mp_settings_.hash = scenario.hash();
@@ -281,69 +287,76 @@ void saved_game::expand_scenario()
 			set_defaults();
 		} else {
 			ERR_NG << "Couldn't find [" << classification().get_tagname() << "] with id=" << carryover_["next_scenario"] << std::endl;
-			this->starting_point_type_ = STARTING_POINT_INVALID;
-			this->starting_point_.clear();
+			starting_point_type_ = starting_point::INVALID;
+			starting_point_.clear();
 		}
 	}
 }
 
 void saved_game::check_require_scenario()
 {
-	if(!starting_point_["require_scenario"].to_bool(false)) {
-		return;
-	}
-	if(starting_point_["addon_id"].empty()) {
-		//ERR_NG << "cannot handle require_scenario=yes because we don't know from which addon that scenario came from\n";
-		return;
-	}
+	const std::string version_default = starting_point_["addon_id"].empty() ? game_config::wesnoth_version.str() : "";
+	config scenario;
+	scenario["id"] = starting_point_["addon_id"].str("mainline");
+	scenario["name"] = starting_point_["addon_title"].str("mainline");
+	scenario["version"] = starting_point_["addon_version"].str(version_default);
+	scenario["min_version"] = starting_point_["addon_min_version"];
+	scenario["required"] = starting_point_["require_scenario"].to_bool(false);
+	config& content = scenario.add_child("content");
+	content["id"] = starting_point_["id"];
+	content["name"] = starting_point_["name"];
+	content["type"] = "scenario";
 
-	config required_scenario;
-
-	required_scenario["id"] = starting_point_["addon_id"];
-	required_scenario["name"] = starting_point_["addon_title"];
-	required_scenario["version"] = starting_point_["addon_version"];
-	required_scenario["min_version"] = starting_point_["addon_min_version"];
-
-	mp_settings_.update_addon_requirements(required_scenario);
+	mp_settings_.update_addon_requirements(scenario);
 }
 
-void saved_game::load_mod(const std::string& type, const std::string& id)
+// "non scenario" at the time of writing this meaning any era, campaign, mods, or resources (see expand_mp_events() below).
+void saved_game::load_non_scenario(const std::string& type, const std::string& id, size_t pos)
 {
 	if(const config& cfg = game_config_manager::get()->game_config().find_child(type, "id", id)) {
 		// Note the addon_id if this mod is required to play the game in mp.
 		std::string require_attr = "require_" + type;
 
 		// By default, eras have "require_era = true", and mods have "require_modification = false".
-		bool require_default = (type == "era");
+		// anything with no addon_id is from mainline, and therefore isn't required in the sense that all players already have it
+		const bool require_default = cfg["addon_id"].empty() ? false : type == "era";
+		const std::string version_default = cfg["addon_id"].empty() ? game_config::wesnoth_version.str() : "";
+		config non_scenario;
+		// if there's no addon_id, then this isn't an add-on
+		non_scenario["id"] = cfg["addon_id"].str("mainline");
+		non_scenario["name"] = cfg["addon_title"].str("mainline");
+		non_scenario["version"] = cfg["addon_version"].str(version_default);
+		non_scenario["min_version"] = cfg["addon_min_version"];
+		non_scenario["required"] = cfg[require_attr].to_bool(require_default);
+		config& content = non_scenario.add_child("content");
+		content["id"] = id;
+		content["name"] = cfg["addon_title"].str(cfg["name"].str(""));
+		content["type"] = type;
 
-		if(!cfg["addon_id"].empty() && cfg[require_attr].to_bool(require_default)) {
-			config required_mod;
-
-			required_mod["id"] = cfg["addon_id"];
-			required_mod["name"] = cfg["addon_title"];
-			required_mod["version"] = cfg["addon_version"];
-			required_mod["min_version"] = cfg["addon_min_version"];
-
-			mp_settings_.update_addon_requirements(required_mod);
-		}
+		mp_settings_.update_addon_requirements(non_scenario);
 
 		// Copy events
 		for(const config& modevent : cfg.child_range("event")) {
 			if(modevent["enable_if"].empty()
 				|| variable_to_bool(carryover_.child_or_empty("variables"), modevent["enable_if"])
 			) {
-				this->starting_point_.add_child("event", modevent);
+				starting_point_.add_child_at_total("event", modevent, pos++);
 			}
 		}
 
 		// Copy lua
 		for(const config& modlua : cfg.child_range("lua")) {
-			this->starting_point_.add_child("lua", modlua);
+			starting_point_.add_child_at_total("lua", modlua, pos++);
+		}
+
+		// Copy modify_unit_type
+		for(const config& modlua : cfg.child_range("modify_unit_type")) {
+			starting_point_.add_child_at_total("modify_unit_type", modlua, pos++);
 		}
 
 		// Copy load_resource
 		for(const config& load_resource : cfg.child_range("load_resource")) {
-			this->starting_point_.add_child("load_resource", load_resource);
+			starting_point_.add_child_at_total("load_resource", load_resource, pos++);
 		}
 	} else {
 		// TODO: A user message instead?
@@ -359,63 +372,60 @@ void saved_game::expand_mp_events()
 {
 	expand_scenario();
 
-	if(this->starting_point_type_ == STARTING_POINT_SCENARIO && !this->starting_point_["has_mod_events"].to_bool(false)) {
+	if(starting_point_type_ == starting_point::SCENARIO && !starting_point_["has_mod_events"].to_bool(false)) {
 		std::vector<modevents_entry> mods;
 		std::set<std::string> loaded_resources;
 
-		std::transform(mp_settings_.active_mods.begin(), mp_settings_.active_mods.end(), std::back_inserter(mods),
+		std::transform(classification_.active_mods.begin(), classification_.active_mods.end(), std::back_inserter(mods),
 			[](const std::string& id) { return modevents_entry("modification", id); }
 		);
 
 		// We don't want the error message below if there is no era (= if this is a sp game).
-		if(!mp_settings_.mp_era .empty()) {
-			mods.emplace_back("era", mp_settings_.mp_era);
+		if(!classification_.era_id.empty()) {
+			mods.emplace_back("era", classification_.era_id);
 		}
 
 		if(!classification_.campaign.empty()) {
 			mods.emplace_back("campaign", classification_.campaign);
 		}
 
-		// In the first iteration mod contains no [resource]s in all other iterations, mods contains only [resource]s.
-		do {
-			for(modevents_entry& mod : mods) {
-				load_mod(mod.type, mod.id);
+		for(modevents_entry& mod : mods) {
+			load_non_scenario(mod.type, mod.id, starting_point_.all_children_count());
+		}
+		mods.clear();
+
+		while(starting_point_.has_child("load_resource")) {
+			assert(starting_point_.child_count("load_resource") > 0);
+			std::string id = starting_point_.child("load_resource")["id"];
+			size_t pos = starting_point_.find_total_first_of("load_resource");
+			starting_point_.remove_child("load_resource", 0);
+			if(loaded_resources.find(id) == loaded_resources.end()) {
+				loaded_resources.insert(id);
+				load_non_scenario("resource", id, pos);
 			}
-
-			mods.clear();
-
-			for(const config& cfg : starting_point_.child_range("load_resource")) {
-				if(loaded_resources.find(cfg["id"].str()) == loaded_resources.end()) {
-					mods.emplace_back("resource", cfg["id"].str());
-
-					loaded_resources.insert(cfg["id"].str());
-				}
-			}
-
-			starting_point_.clear_children("load_resource");
-		} while(!mods.empty());
-
-		this->starting_point_["has_mod_events"] = true;
+		}
+		starting_point_["has_mod_events"] = true;
+		starting_point_["loaded_resources"] = utils::join(loaded_resources);
 	}
 }
 
 void saved_game::expand_mp_options()
 {
-	if(starting_point_type_ == STARTING_POINT_SCENARIO && !has_carryover_expanded_) {
+	if(starting_point_type_ == starting_point::SCENARIO && !has_carryover_expanded_) {
 		std::vector<modevents_entry> mods;
 
-		std::transform(mp_settings_.active_mods.begin(), mp_settings_.active_mods.end(), std::back_inserter(mods),
+		std::transform(classification_.active_mods.begin(), classification_.active_mods.end(), std::back_inserter(mods),
 			[](const std::string& id) { return modevents_entry("modification", id); }
 		);
 
-		mods.emplace_back("era", mp_settings_.mp_era);
+		mods.emplace_back("era", classification_.era_id);
 		mods.emplace_back("multiplayer", get_scenario_id());
 		mods.emplace_back("campaign", classification().campaign);
 
 		config& variables = carryover_.child_or_add("variables");
 
 		for(modevents_entry& mod : mods) {
-			if(const config& cfg = this->mp_settings().options.find_child(mod.type, "id", mod.id)) {
+			if(const config& cfg = mp_settings().options.find_child(mod.type, "id", mod.id)) {
 				for(const config& option : cfg.child_range("option")) {
 					try {
 						variable_access_create(option["id"], variables).as_scalar() = option["value"];
@@ -430,31 +440,61 @@ void saved_game::expand_mp_options()
 	}
 }
 
+static void inherit_scenario(config& scenario, config& map_scenario)
+{
+	config sides;
+	sides.splice_children(map_scenario, "side");
+	scenario.append_children(map_scenario);
+	scenario.inherit_attributes(map_scenario);
+	for(config& side_from : sides.child_range("side")) {
+		config& side_to = scenario.find_child("side", "side", side_from["side"]);
+		if(side_to) {
+			side_to.inherit_attributes(side_from);
+			side_to.append_children(side_from);
+		} else {
+			scenario.add_child("side", side_from);
+		}
+	}
+}
+
+void saved_game::expand_map_file(config& scenario)
+{
+	if(scenario["map_data"].empty() && !scenario["map_file"].empty()) {
+		std::string map_data = filesystem::read_map(scenario["map_file"]);
+		if(map_data.find("map_data") != std::string::npos) {
+			// we have a scenario, gnerated by the editor
+			config map_data_cfg;
+			read(map_data_cfg, map_data);
+			inherit_scenario(scenario, map_data_cfg);
+		} else {
+			// we have an plain map_data file
+			scenario["map_data"] = map_data;
+		}
+	}
+}
+
 void saved_game::expand_random_scenario()
 {
 	expand_scenario();
 
-	if(this->starting_point_type_ == STARTING_POINT_SCENARIO) {
+	if(starting_point_type_ == starting_point::SCENARIO) {
 		// If the entire scenario should be randomly generated
 		if(!starting_point_["scenario_generation"].empty()) {
 			LOG_NG << "randomly generating scenario...\n";
 			const cursor::setter cursor_setter(cursor::WAIT);
 
 			config scenario_new =
-				random_generate_scenario(starting_point_["scenario_generation"], starting_point_.child("generator"));
+				random_generate_scenario(starting_point_["scenario_generation"], starting_point_.child("generator"), &carryover_.child_or_empty("variables"));
 
 			post_scenario_generation(starting_point_, scenario_new);
-			starting_point_ = scenario_new;
+			starting_point_ = std::move(scenario_new);
 
 			update_label();
 			set_defaults();
 		}
 
 		// If no map_data is provided, try to load the specified file directly
-		if(starting_point_["map_data"].empty() && !starting_point_["map_file"].empty()) {
-			starting_point_["map_data"] = filesystem::read_map(starting_point_["map_file"]);
-		}
-
+		expand_map_file(starting_point_);
 		// If the map should be randomly generated
 		// We don’t want that we accidentally to this twice so we check for starting_point_["map_data"].empty()
 		if(starting_point_["map_data"].empty() && !starting_point_["map_generation"].empty()) {
@@ -462,7 +502,7 @@ void saved_game::expand_random_scenario()
 			const cursor::setter cursor_setter(cursor::WAIT);
 
 			starting_point_["map_data"] =
-				random_generate_map(starting_point_["map_generation"], starting_point_.child("generator"));
+				random_generate_map(starting_point_["map_generation"], starting_point_.child("generator"), &carryover_.child_or_empty("variables"));
 		}
 	}
 }
@@ -497,7 +537,7 @@ void saved_game::post_scenario_generation(const config& old_scenario, config& ge
 void saved_game::expand_carryover()
 {
 	expand_scenario();
-	if(this->starting_point_type_ == STARTING_POINT_SCENARIO && !has_carryover_expanded_) {
+	if(starting_point_type_ == starting_point::SCENARIO && !has_carryover_expanded_) {
 		carryover_info sides(carryover_);
 
 		sides.transfer_to(get_starting_point());
@@ -512,21 +552,21 @@ void saved_game::expand_carryover()
 
 bool saved_game::valid() const
 {
-	return this->starting_point_type_ != STARTING_POINT_INVALID;
+	return starting_point_type_ != starting_point::INVALID;
 }
 
 config& saved_game::set_snapshot(config snapshot)
 {
-	this->starting_point_type_ = STARTING_POINT_SNAPSHOT;
-	this->starting_point_.swap(snapshot);
+	starting_point_type_ = starting_point::SNAPSHOT;
+	starting_point_.swap(snapshot);
 
-	return this->starting_point_;
+	return starting_point_;
 }
 
 void saved_game::set_scenario(config scenario)
 {
-	this->starting_point_type_ = STARTING_POINT_SCENARIO;
-	this->starting_point_.swap(scenario);
+	starting_point_type_ = starting_point::SCENARIO;
+	starting_point_.swap(scenario);
 
 	has_carryover_expanded_ = false;
 
@@ -535,8 +575,8 @@ void saved_game::set_scenario(config scenario)
 
 void saved_game::remove_snapshot()
 {
-	this->starting_point_type_ = STARTING_POINT_NONE;
-	this->starting_point_.clear();
+	starting_point_type_ = starting_point::NONE;
+	starting_point_.clear();
 }
 
 config& saved_game::get_starting_point()
@@ -556,16 +596,16 @@ const config& saved_game::get_replay_starting_point()
 		expand_carryover();
 	}
 
-	if(starting_point_type_ == STARTING_POINT_SCENARIO) {
+	if(starting_point_type_ == starting_point::SCENARIO) {
 		return starting_point_;
 	}
 
-	return this->replay_start_.child("some_non_existet_invalid");
+	return replay_start_.child("some_non_existet_invalid");
 }
 
 void saved_game::convert_to_start_save()
 {
-	assert(starting_point_type_ == STARTING_POINT_SNAPSHOT);
+	assert(starting_point_type_ == starting_point::SNAPSHOT);
 
 	carryover_info sides(starting_point_, true);
 
@@ -587,15 +627,15 @@ config saved_game::to_config() const
 	// TODO: remove this code duplication with write_... functions.
 	config r = classification_.to_config();
 
-	if(!this->replay_start_.empty()) {
+	if(!replay_start_.empty()) {
 		r.add_child("replay_start", replay_start_);
 	}
 
 	replay_data_.write(r.add_child("replay"));
 
-	if(starting_point_type_ == STARTING_POINT_SNAPSHOT) {
+	if(starting_point_type_ == starting_point::SNAPSHOT) {
 		r.add_child("snapshot", starting_point_);
-	} else if(starting_point_type_ == STARTING_POINT_SCENARIO) {
+	} else if(starting_point_type_ == starting_point::SCENARIO) {
 		r.add_child("scenario", starting_point_);
 	}
 
@@ -605,14 +645,16 @@ config saved_game::to_config() const
 	return r;
 }
 
-std::string saved_game::get_scenario_id()
+std::string saved_game::get_scenario_id() const
 {
 	std::string scenario_id;
 
-	if(this->starting_point_type_ == STARTING_POINT_SNAPSHOT || this->starting_point_type_ == STARTING_POINT_SCENARIO) {
+	if(starting_point_type_ == starting_point::SNAPSHOT || starting_point_type_ == starting_point::SCENARIO) {
 		scenario_id = starting_point_["id"].str();
 	} else if(!has_carryover_expanded_) {
 		scenario_id = carryover_["next_scenario"].str();
+	} else if(!replay_start_.empty()) {
+		scenario_id = replay_start_["id"].str();
 	} else {
 		assert(!"cannot figure out scenario_id");
 		throw "assertion ignored";
@@ -642,7 +684,7 @@ void saved_game::update_label()
 
 void saved_game::cancel_orders()
 {
-	for(config& side : this->starting_point_.child_range("side")) {
+	for(config& side : starting_point_.child_range("side")) {
 		// for humans "goto_x/y" is used for multi-turn-moves
 		// for the ai "goto_x/y" is a way for wml to order the ai to move a unit to a certain place.
 		// we want to cancel human order but not to break wml.
@@ -659,7 +701,7 @@ void saved_game::cancel_orders()
 
 void saved_game::unify_controllers()
 {
-	for(config& side : this->starting_point_.child_range("side")) {
+	for(config& side : starting_point_.child_range("side")) {
 		side.remove_attribute("is_local");
 		//TODO: the old code below is probably not needed anymore
 		if(side["controller"] == "network") {
@@ -674,7 +716,7 @@ void saved_game::unify_controllers()
 
 saved_game& saved_game::operator=(saved_game&& other)
 {
-	this->swap(other);
+	swap(other);
 	return *this;
 }
 
@@ -724,14 +766,14 @@ void saved_game::set_data(config& cfg)
 	replay_data_.set_to_end();
 
 	if(config& snapshot = cfg.child("snapshot")) {
-		this->starting_point_type_ = STARTING_POINT_SNAPSHOT;
-		this->starting_point_.swap(snapshot);
+		starting_point_type_ = starting_point::SNAPSHOT;
+		starting_point_.swap(snapshot);
 	} else if(config& scenario = cfg.child("scenario")) {
-		this->starting_point_type_ = STARTING_POINT_SCENARIO;
-		this->starting_point_.swap(scenario);
+		starting_point_type_ = starting_point::SCENARIO;
+		starting_point_.swap(scenario);
 	} else {
-		this->starting_point_type_ = STARTING_POINT_NONE;
-		this->starting_point_.clear();
+		starting_point_type_ = starting_point::NONE;
+		starting_point_.clear();
 	}
 
 	LOG_NG << "scenario: '" << carryover_["next_scenario"].str() << "'\n";
@@ -741,8 +783,8 @@ void saved_game::set_data(config& cfg)
 		statistics::read_stats(stats);
 	}
 
-	classification_ = game_classification(cfg);
-	mp_settings_ = mp_game_settings(cfg.child_or_empty("multiplayer"));
+	classification_ = game_classification{ cfg };
+	mp_settings_ = { cfg.child_or_empty("multiplayer") };
 
 	cfg.clear();
 }
@@ -750,13 +792,13 @@ void saved_game::set_data(config& cfg)
 void saved_game::clear()
 {
 	carryover_.clear();
-	classification_ = game_classification();
+	classification_ = {};
 	has_carryover_expanded_ = false;
-	mp_settings_ = mp_game_settings();
-	replay_data_.swap(replay_recorder_base());
+	mp_settings_ = {};
+	replay_data_.swap({});
 	replay_start_.clear();
 	starting_point_.clear();
-	starting_point_type_ = STARTING_POINT_NONE;
+	starting_point_type_ = starting_point::NONE;
 }
 
 void swap(saved_game& lhs, saved_game& rhs)
