@@ -1,51 +1,54 @@
 /*
-   Copyright (C) 2003 - 2018 by Jörg Hinrichs, refactored from various
-   places formerly created by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by Jörg Hinrichs, David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include <boost/iostreams/filter/gzip.hpp>
 
 #include "savegame.hpp"
 
-#include "save_index.hpp"
 #include "carryover.hpp"
 #include "cursor.hpp"
 #include "format_time_summary.hpp"
 #include "formatter.hpp"
 #include "formula/string_utils.hpp"
+#include "game_config_manager.hpp"
+#include "game_config_view.hpp"
 #include "game_end_exceptions.hpp"
 #include "game_errors.hpp"
-#include "preferences/game.hpp"
+#include "game_version.hpp"
 #include "gettext.hpp"
+#include "gui/dialogs/campaign_difficulty.hpp"
 #include "gui/dialogs/game_load.hpp"
 #include "gui/dialogs/game_save.hpp"
 #include "gui/dialogs/message.hpp"
-#include "gui/dialogs/campaign_difficulty.hpp"
 #include "gui/dialogs/transient_message.hpp"
-#include "gui/widgets/settings.hpp"
 #include "gui/widgets/retval.hpp"
+#include "gui/widgets/settings.hpp"
 #include "log.hpp"
 #include "persist_manager.hpp"
+#include "preferences/game.hpp"
 #include "resources.hpp"
 #include "save_index.hpp"
 #include "saved_game.hpp"
 #include "serialization/binary_or_text.hpp"
 #include "serialization/parser.hpp"
+#include "serialization/utf8_exception.hpp"
 #include "statistics.hpp"
-#include "game_version.hpp"
 #include "video.hpp"
 
 #include <algorithm>
+#include <iomanip>
 
 static lg::log_domain log_engine("engine");
 #define LOG_SAVE LOG_STREAM(info, log_engine)
@@ -54,13 +57,13 @@ static lg::log_domain log_engine("engine");
 static lg::log_domain log_enginerefac("enginerefac");
 #define LOG_RG LOG_STREAM(info, log_enginerefac)
 
-
-namespace savegame {
-
+namespace savegame
+{
 bool save_game_exists(std::string name, compression::format compressed)
 {
 	name += compression::format_extension(compressed);
-	return filesystem::file_exists(filesystem::get_saves_dir() + "/" + name);
+	auto manager = save_index_class::default_saves_dir();
+	return filesystem::file_exists(manager->dir() + "/" + name);
 }
 
 void clean_saves(const std::string& label)
@@ -68,19 +71,21 @@ void clean_saves(const std::string& label)
 	const std::string prefix = label + "-" + _("Auto-Save");
 	LOG_SAVE << "Cleaning saves with prefix '" << prefix << "'\n";
 
-	for(const auto& save : get_saves_list()) {
+	auto manager = save_index_class::default_saves_dir();
+	for(const auto& save : manager->get_saves_list()) {
 		if(save.name().compare(0, prefix.length(), prefix) == 0) {
 			LOG_SAVE << "Deleting savegame '" << save.name() << "'\n";
-			delete_game(save.name());
+			manager->delete_game(save.name());
 		}
 	}
 }
 
-loadgame::loadgame(const config& game_config, saved_game& gamestate)
-	: game_config_(game_config)
+loadgame::loadgame(const std::shared_ptr<save_index_class>& index, saved_game& gamestate)
+	: game_config_(game_config_manager::get()->game_config())
 	, gamestate_(gamestate)
-	, load_data_()
-{}
+	, load_data_{index}
+{
+}
 
 bool loadgame::show_difficulty_dialog()
 {
@@ -90,14 +95,12 @@ bool loadgame::show_difficulty_dialog()
 
 	std::string campaign_id = load_data_.summary["campaign"];
 
-	for(const config &campaign : game_config_.child_range("campaign"))
-	{
+	for(const config& campaign : game_config_.child_range("campaign")) {
 		if(campaign["id"] != campaign_id) {
 			continue;
 		}
 
 		gui2::dialogs::campaign_difficulty difficulty_dlg(campaign);
-		difficulty_dlg.show();
 
 		// Return if canceled, since otherwise load_data_.difficulty will be set to 'CANCEL'
 		if(!difficulty_dlg.show()) {
@@ -105,6 +108,7 @@ bool loadgame::show_difficulty_dialog()
 		}
 
 		load_data_.difficulty = difficulty_dlg.selected_difficulty();
+		load_data_.select_difficulty = false;
 
 		// Exit loop
 		break;
@@ -135,19 +139,23 @@ bool loadgame::load_game_ingame()
 		}
 	}
 
+	if(!load_data_.manager) {
+		ERR_SAVE << "Null pointer in save index" << std::endl;
+		return false;
+	}
+
 	load_data_.show_replay |= is_replay_save(load_data_.summary);
 
 	// Confirm the integrity of the file before throwing the exception.
 	// Use the summary in the save_index for this.
-	const config & summary = save_index_manager.get(load_data_.filename);
+	const config& summary = load_data_.manager->get(load_data_.filename);
 
-	if (summary["corrupt"].to_bool(false)) {
-		gui2::show_error_message(
-				_("The file you have tried to load is corrupt: '"));
+	if(summary["corrupt"].to_bool(false)) {
+		gui2::show_error_message(_("The file you have tried to load is corrupt: '"));
 		return false;
 	}
 
-	if (!loadgame::check_version_compatibility(summary["version"].str())) {
+	if(!loadgame::check_version_compatibility(summary["version"].str())) {
 		return false;
 	}
 
@@ -158,10 +166,11 @@ bool loadgame::load_game()
 {
 	bool skip_version_check = true;
 
-	if(load_data_.filename.empty()){
+	if(load_data_.filename.empty()) {
 		if(!gui2::dialogs::game_load::execute(game_config_, load_data_)) {
 			return false;
 		}
+
 		skip_version_check = false;
 		load_data_.show_replay |= is_replay_save(load_data_.summary);
 	}
@@ -176,34 +185,37 @@ bool loadgame::load_game()
 		}
 	}
 
+	if(!load_data_.manager) {
+		ERR_SAVE << "Null pointer in save index" << std::endl;
+		return false;
+	}
+
 	std::string error_log;
-	read_save_file(load_data_.filename, load_data_.load_config, &error_log);
+	read_save_file(load_data_.manager->dir(), load_data_.filename, load_data_.load_config, &error_log);
 
 	convert_old_saves(load_data_.load_config);
 
-	for (config& side : load_data_.load_config.child_range("side")) {
+	for(config& side : load_data_.load_config.child_range("side")) {
 		side.remove_attribute("is_local");
 	}
 
 	if(!error_log.empty()) {
-        try {
-		    gui2::show_error_message(
-				    _("Warning: The file you have tried to load is corrupt. Loading anyway.\n") +
-				    error_log);
-        } catch (const utf8::invalid_utf8_exception&) {
-		    gui2::show_error_message(
-				    _("Warning: The file you have tried to load is corrupt. Loading anyway.\n") +
-                    std::string("(UTF-8 ERROR)"));
-        }
+		try {
+			gui2::show_error_message(
+				_("Warning: The file you have tried to load is corrupt. Loading anyway.\n") + error_log);
+		} catch(const utf8::invalid_utf8_exception&) {
+			gui2::show_error_message(_("Warning: The file you have tried to load is corrupt. Loading anyway.\n")
+				+ std::string("(UTF-8 ERROR)"));
+		}
 	}
 
-	if (!load_data_.difficulty.empty()){
+	if(!load_data_.difficulty.empty()) {
 		load_data_.load_config["difficulty"] = load_data_.difficulty;
 	}
 	// read classification to for loading the game_config config object.
 	gamestate_.classification() = game_classification(load_data_.load_config);
 
-	if (skip_version_check) {
+	if(skip_version_check) {
 		return true;
 	}
 
@@ -215,31 +227,28 @@ bool loadgame::check_version_compatibility()
 	return loadgame::check_version_compatibility(gamestate_.classification().version);
 }
 
-bool loadgame::check_version_compatibility(const version_info & save_version)
+bool loadgame::check_version_compatibility(const version_info& save_version)
 {
-	if (save_version == game_config::version) {
+	if(save_version == game_config::wesnoth_version) {
 		return true;
 	}
 
-	const version_info &wesnoth_version = game_config::wesnoth_version;
+	const version_info& wesnoth_version = game_config::wesnoth_version;
 
 	// Even minor version numbers indicate stable releases which are
 	// compatible with each other.
-	if (wesnoth_version.minor_version() % 2 == 0 &&
-	    wesnoth_version.major_version() == save_version.major_version() &&
-	    wesnoth_version.minor_version() == save_version.minor_version())
-	{
+	if(wesnoth_version.minor_version() % 2 == 0 && wesnoth_version.major_version() == save_version.major_version()
+		&& wesnoth_version.minor_version() == save_version.minor_version()) {
 		return true;
 	}
 
 	// Do not load if too old. If either the savegame or the current
 	// game has the version 'test', load. This 'test' version is never
 	// supposed to occur, except when Soliton is testing MP servers.
-	if (save_version < game_config::min_savegame_version &&
-	    save_version != game_config::test_version &&
-	    wesnoth_version != game_config::test_version)
-	{
-		const std::string message = _("This save is from an old, unsupported version ($version_number|) and cannot be loaded.");
+	if(save_version < game_config::min_savegame_version && save_version != game_config::test_version
+		&& wesnoth_version != game_config::test_version) {
+		const std::string message
+			= _("This save is from an old, unsupported version ($version_number|) and cannot be loaded.");
 		utils::string_map symbols;
 		symbols["version_number"] = save_version.str();
 		gui2::show_error_message(utils::interpolate_variables_into_string(message, &symbols));
@@ -247,11 +256,23 @@ bool loadgame::check_version_compatibility(const version_info & save_version)
 	}
 
 	if(preferences::confirm_load_save_from_different_version()) {
-		const std::string message = _("This save is from a different version of the game ($version_number|). Do you wish to try to load it?");
+		const std::string message
+			= _("This save is from a different version of the game ($version_number|), and might not work with this "
+				"version.\n"
+				"\n"
+				"<b>Warning:</b> saves in the middle of campaigns are especially likely to fail, and you should either "
+				"use the old version or restart the campaign. Even when a saved game seems to load successfully, "
+				"subtler aspects like gameplay balance and story progression could be impacted. The difficulty, the "
+				"challenge, the <i>fun</i> may be missing.\n"
+				"\n"
+				"For example, the campaign may have been rebalanced with fewer enemies in the early scenarios, but "
+				"expecting your recall list to have correspondingly less experience in the late scenarios.\n"
+				"\n"
+				"Do you wish to continue?");
 		utils::string_map symbols;
 		symbols["version_number"] = save_version.str();
 		const int res = gui2::show_message(_("Load Game"), utils::interpolate_variables_into_string(message, &symbols),
-			gui2::dialogs::message::yes_no_buttons);
+			gui2::dialogs::message::yes_no_buttons, true);
 		return res == gui2::retval::OK;
 	}
 
@@ -269,12 +290,16 @@ bool loadgame::load_multiplayer_game()
 		return false;
 	}
 
-
 	load_data_.show_replay |= is_replay_save(load_data_.summary);
 	if(load_data_.filename.empty()) {
 		return false;
 	}
 
+	if(!load_data_.manager) {
+		ERR_SAVE << "Null pointer in save index" << std::endl;
+		return false;
+	}
+
 	// read_save_file needs to be called before we can verify the classification so the data has
 	// been populated. Since we do that, we report any errors in that process first.
 	std::string error_log;
@@ -282,14 +307,12 @@ bool loadgame::load_multiplayer_game()
 		cursor::setter cur(cursor::WAIT);
 		log_scope("load_game");
 
-		read_save_file(load_data_.filename, load_data_.load_config, &error_log);
+		read_save_file(load_data_.manager->dir(), load_data_.filename, load_data_.load_config, &error_log);
 		copy_era(load_data_.load_config);
 	}
 
 	if(!error_log.empty()) {
-		gui2::show_error_message(
-				_("The file you have tried to load is corrupt: '") +
-				error_log);
+		gui2::show_error_message(_("The file you have tried to load is corrupt: '") + error_log);
 		return false;
 	}
 
@@ -310,16 +333,22 @@ bool loadgame::load_multiplayer_game()
 	return check_version_compatibility();
 }
 
-void loadgame::copy_era(config &cfg)
+void loadgame::copy_era(config& cfg)
 {
-	const config &replay_start = cfg.child("replay_start");
-	if (!replay_start) return;
+	const config& replay_start = cfg.child("replay_start");
+	if(!replay_start) {
+		return;
+	}
 
-	const config &era = replay_start.child("era");
-	if (!era) return;
+	const config& era = replay_start.child("era");
+	if(!era) {
+		return;
+	}
 
-	config &snapshot = cfg.child("snapshot");
-	if (!snapshot) return;
+	config& snapshot = cfg.child("snapshot");
+	if(!snapshot) {
+		return;
+	}
 
 	snapshot.add_child("era", era);
 }
@@ -327,21 +356,24 @@ void loadgame::copy_era(config &cfg)
 savegame::savegame(saved_game& gamestate, const compression::format compress_saves, const std::string& title)
 	: filename_()
 	, title_(title)
+	, save_index_manager_(save_index_class::default_saves_dir())
 	, gamestate_(gamestate)
 	, error_message_(_("The game could not be saved: "))
 	, show_confirmation_(false)
 	, compress_saves_(compress_saves)
-{}
+{
+}
 
 bool savegame::save_game_automatic(bool ask_for_overwrite, const std::string& filename)
 {
-	if (filename.empty())
+	if(filename.empty()) {
 		filename_ = create_filename();
-	else
+	} else {
 		filename_ = filename;
+	}
 
-	if (ask_for_overwrite){
-		if (!check_overwrite()) {
+	if(ask_for_overwrite) {
+		if(!check_overwrite()) {
 			return save_game_interactive("", savegame::OK_CANCEL);
 		}
 	}
@@ -356,11 +388,11 @@ bool savegame::save_game_interactive(const std::string& message, DIALOG_TYPE dia
 
 	const int res = show_save_dialog(message, dialog_type);
 
-	if (res == 2) {
-		throw_quit_game_exception(); //Quit game
+	if(res == 2) {
+		throw_quit_game_exception(); // Quit game
 	}
 
-	if (res == gui2::retval::OK && check_overwrite()) {
+	if(res == gui2::retval::OK && check_overwrite()) {
 		return save_game();
 	}
 
@@ -371,18 +403,17 @@ int savegame::show_save_dialog(const std::string& message, DIALOG_TYPE dialog_ty
 {
 	int res = 0;
 
-	if (dialog_type == OK_CANCEL){
+	if(dialog_type == OK_CANCEL) {
 		gui2::dialogs::game_save dlg(filename_, title_);
 		dlg.show();
 		res = dlg.get_retval();
-	}
-	else if (dialog_type == YES_NO){
+	} else if(dialog_type == YES_NO) {
 		gui2::dialogs::game_save_message dlg(filename_, title_, message);
 		dlg.show();
 		res = dlg.get_retval();
 	}
 
-	if (!check_filename(filename_)) {
+	if(!check_filename(filename_)) {
 		res = gui2::retval::CANCEL;
 	}
 
@@ -399,14 +430,17 @@ bool savegame::check_overwrite()
 	message << _("Save already exists. Do you want to overwrite it?") << "\n" << _("Name: ") << filename_;
 	const int res = gui2::show_message(_("Overwrite?"), message.str(), gui2::dialogs::message::yes_no_buttons);
 	return res == gui2::retval::OK;
-
 }
 
 bool savegame::check_filename(const std::string& filename)
 {
-	if (filesystem::is_compressed_file(filename)) {
-		gui2::show_error_message(_("Save names should not end on '.gz' or '.bz2'. "
-			"Please remove the extension."));
+	if(filesystem::is_compressed_file(filename)) {
+		gui2::show_error_message(_("Save names should not end on '.gz' or '.bz2'. Please remove the extension."));
+		return false;
+	} else if(!filesystem::is_legal_user_file_name(filename)) {
+		// This message is not all-inclusive. This is on purpose. Few people
+		// need to know about DOS device names or the 255 character limit.
+		gui2::show_error_message(_("Save names may not end with a dot, or contain two dots or any of the following characters:\n    \" * / : < > ? \\ | ~"));
 		return false;
 	}
 
@@ -424,20 +458,20 @@ void savegame::before_save()
 
 bool savegame::save_game(const std::string& filename)
 {
-
 	try {
 		uint32_t start, end;
 		start = SDL_GetTicks();
 
-		if (filename_.empty())
+		if(filename_.empty()) {
 			filename_ = filename;
+		}
 
 		before_save();
 
 		write_game_to_disk(filename_);
-		if (resources::persist != nullptr) {
+		if(resources::persist != nullptr) {
 			resources::persist->end_transaction();
-			resources::persist ->start_transaction();
+			resources::persist->start_transaction();
 		}
 
 		// Create an entry in the save_index. Doing this here ensures all leader image paths
@@ -447,21 +481,22 @@ bool savegame::save_game(const std::string& filename)
 		// a player saves a game and exits the game or reloads the cache, the leader image will
 		// only be available within that specific binary context (when playing another game from
 		// the came campaign, for example).
-		save_index_manager.rebuild(filename_);
+		save_index_manager_->rebuild(filename_);
 
 		end = SDL_GetTicks();
 		LOG_SAVE << "Milliseconds to save " << filename_ << ": " << end - start << std::endl;
 
-		if (show_confirmation_)
+		if(show_confirmation_) {
 			gui2::show_transient_message(_("Saved"), _("The game has been saved."));
+		}
+
 		return true;
 	} catch(const game::save_game_failed& e) {
 		ERR_SAVE << error_message_ << e.message << std::endl;
-
 		gui2::show_error_message(error_message_ + e.message);
-		//do not bother retrying, since the user can just try to save the game again
-		//maybe show a yes-no dialog for "disable autosaves now"?
 
+		// do not bother retrying, since the user can just try to save the game again
+		// maybe show a yes-no dialog for "disable autosaves now"?
 		return false;
 	};
 }
@@ -479,19 +514,20 @@ void savegame::write_game_to_disk(const std::string& filename)
 		write_game(out);
 		finish_save_game(out);
 	}
+
 	filesystem::scoped_ostream os(open_save_game(filename_));
 	(*os) << ss.str();
 
-	if (!os->good()) {
+	if(!os->good()) {
 		throw game::save_game_failed(_("Could not write to file"));
 	}
 }
 
-void savegame::write_game(config_writer &out)
+void savegame::write_game(config_writer& out)
 {
 	log_scope("write_game");
 
-	out.write_key_val("version", game_config::version);
+	out.write_key_val("version", game_config::wesnoth_version.str());
 
 	gamestate_.write_general_info(out);
 	out.open_child("statistics");
@@ -499,29 +535,28 @@ void savegame::write_game(config_writer &out)
 	out.close_child("statistics");
 }
 
-void savegame::finish_save_game(const config_writer &out)
+void savegame::finish_save_game(const config_writer& out)
 {
 	try {
 		if(!out.good()) {
 			throw game::save_game_failed(_("Could not write to file"));
 		}
-		save_index_manager.remove(gamestate_.classification().label);
 	} catch(const filesystem::io_exception& e) {
 		throw game::save_game_failed(e.what());
 	}
 }
 
 // Throws game::save_game_failed
-filesystem::scoped_ostream savegame::open_save_game(const std::string &label)
+filesystem::scoped_ostream savegame::open_save_game(const std::string& label)
 {
 	try {
-		return filesystem::ostream_file(filesystem::get_saves_dir() + "/" + label);
+		return filesystem::ostream_file(save_index_manager_->dir() + "/" + label);
 	} catch(const filesystem::io_exception& e) {
 		throw game::save_game_failed(e.what());
 	}
 }
 
-scenariostart_savegame::scenariostart_savegame(saved_game &gamestate, const compression::format compress_saves)
+scenariostart_savegame::scenariostart_savegame(saved_game& gamestate, const compression::format compress_saves)
 	: savegame(gamestate, compress_saves)
 {
 	filename_ = create_filename();
@@ -532,21 +567,29 @@ std::string scenariostart_savegame::create_initial_filename(unsigned int) const
 	return gamestate().classification().label;
 }
 
-void scenariostart_savegame::write_game(config_writer &out){
+void scenariostart_savegame::write_game(config_writer& out)
+{
 	savegame::write_game(out);
 	gamestate().write_carryover(out);
 }
 
-replay_savegame::replay_savegame(saved_game &gamestate, const compression::format compress_saves)
+replay_savegame::replay_savegame(saved_game& gamestate, const compression::format compress_saves)
 	: savegame(gamestate, compress_saves, _("Save Replay"))
-{}
+{
+}
 
 std::string replay_savegame::create_initial_filename(unsigned int) const
 {
-	return formatter() << gamestate().classification().label << " " << _("replay");
+	time_t t = std::time(nullptr);
+	tm tm = *std::localtime(&t);
+	auto time = std::put_time(&tm, "%Y%m%d-%H%M%S");
+
+	// TRANSLATORS: This string is used as part of a filename, as in, "HttT-The Elves Besieged replay.gz"
+	return formatter() << gamestate().classification().label << " " << _("replay") << " " << time;
 }
 
-void replay_savegame::write_game(config_writer &out) {
+void replay_savegame::write_game(config_writer& out)
+{
 	savegame::write_game(out);
 
 	gamestate().write_carryover(out);
@@ -555,10 +598,9 @@ void replay_savegame::write_game(config_writer &out) {
 	out.open_child("replay");
 	gamestate().get_replay().write(out);
 	out.close_child("replay");
-
 }
 
-autosave_savegame::autosave_savegame(saved_game &gamestate, const compression::format compress_saves)
+autosave_savegame::autosave_savegame(saved_game& gamestate, const compression::format compress_saves)
 	: ingame_savegame(gamestate, compress_saves)
 {
 	set_error_message(_("Could not auto save the game. Please save the game manually."));
@@ -571,16 +613,18 @@ void autosave_savegame::autosave(const bool disable_autosave, const int autosave
 
 	save_game_automatic();
 
-	remove_old_auto_saves(autosave_max, infinite_autosaves);
+	auto manager = save_index_class::default_saves_dir();
+	manager->delete_old_auto_saves(autosave_max, infinite_autosaves);
 }
 
 std::string autosave_savegame::create_initial_filename(unsigned int turn_number) const
 {
 	std::string filename;
-	if(gamestate().classification().label.empty())
+	if(gamestate().classification().label.empty()) {
 		filename = _("Auto-Save");
-	else
+	} else {
 		filename = gamestate().classification().label + "-" + _("Auto-Save") + std::to_string(turn_number);
+	}
 
 	return filename;
 }
@@ -588,37 +632,38 @@ std::string autosave_savegame::create_initial_filename(unsigned int turn_number)
 oos_savegame::oos_savegame(saved_game& gamestate, bool& ignore)
 	: ingame_savegame(gamestate, preferences::save_compression_format())
 	, ignore_(ignore)
-{}
+{
+}
 
 int oos_savegame::show_save_dialog(const std::string& message, DIALOG_TYPE /*dialog_type*/)
 {
 	int res = 0;
 
-	if (!ignore_){
+	if(!ignore_) {
 		gui2::dialogs::game_save_oos dlg(ignore_, filename_, title(), message);
 		dlg.show();
 		res = dlg.get_retval();
 	}
 
-	if (!check_filename(filename_)) {
+	if(!check_filename(filename_)) {
 		res = gui2::retval::CANCEL;
 	}
 
 	return res;
 }
 
-ingame_savegame::ingame_savegame(saved_game &gamestate, const compression::format compress_saves)
+ingame_savegame::ingame_savegame(saved_game& gamestate, const compression::format compress_saves)
 	: savegame(gamestate, compress_saves, _("Save Game"))
 {
 }
 
 std::string ingame_savegame::create_initial_filename(unsigned int turn_number) const
 {
-	return formatter() << gamestate().classification().label
-		<< " " << _("Turn") << " " << turn_number;
+	return formatter() << gamestate().classification().label << " " << _("Turn") << " " << turn_number;
 }
 
-void ingame_savegame::write_game(config_writer &out) {
+void ingame_savegame::write_game(config_writer& out)
+{
 	log_scope("write_game");
 
 	if(!gamestate().get_starting_point().validate_wml()) {
@@ -628,17 +673,17 @@ void ingame_savegame::write_game(config_writer &out) {
 	savegame::write_game(out);
 
 	gamestate().write_carryover(out);
-	out.write_child("snapshot",gamestate().get_starting_point());
+	out.write_child("snapshot", gamestate().get_starting_point());
 	out.write_child("replay_start", gamestate().replay_start());
 	out.open_child("replay");
 	gamestate().get_replay().write(out);
 	out.close_child("replay");
 }
 
-//changes done during 1.11.0-dev
+// changes done during 1.11.0-dev
 static void convert_old_saves_1_11_0(config& cfg)
 {
-	if(!cfg.has_child("snapshot")){
+	if(!cfg.has_child("snapshot")) {
 		return;
 	}
 
@@ -646,61 +691,63 @@ static void convert_old_saves_1_11_0(config& cfg)
 	const config& replay_start = cfg.child("replay_start");
 	const config& replay = cfg.child("replay");
 
-	if(!cfg.has_child("carryover_sides") && !cfg.has_child("carryover_sides_start")){
+	if(!cfg.has_child("carryover_sides") && !cfg.has_child("carryover_sides_start")) {
 		config carryover;
-		//copy rng and menu items from toplevel to new carryover_sides
+		// copy rng and menu items from toplevel to new carryover_sides
 		carryover["random_seed"] = cfg["random_seed"];
 		carryover["random_calls"] = cfg["random_calls"];
+
 		for(const config& menu_item : cfg.child_range("menu_item")) {
 			carryover.add_child("menu_item", menu_item);
 		}
+
 		carryover["difficulty"] = cfg["difficulty"];
 		carryover["random_mode"] = cfg["random_mode"];
-		//the scenario to be played is always stored as next_scenario in carryover_sides_start
+		// the scenario to be played is always stored as next_scenario in carryover_sides_start
 		carryover["next_scenario"] = cfg["scenario"];
 
 		config carryover_start = carryover;
 
-		//copy sides from either snapshot or replay_start to new carryover_sides
-		if(!snapshot.empty()){
+		// copy sides from either snapshot or replay_start to new carryover_sides
+		if(!snapshot.empty()) {
 			for(const config& side : snapshot.child_range("side")) {
 				carryover.add_child("side", side);
 			}
-			//for compatibility with old savegames that use player instead of side
+			// for compatibility with old savegames that use player instead of side
 			for(const config& side : snapshot.child_range("player")) {
 				carryover.add_child("side", side);
 			}
-			//save the sides from replay_start in carryover_sides_start
+			// save the sides from replay_start in carryover_sides_start
 			for(const config& side : replay_start.child_range("side")) {
 				carryover_start.add_child("side", side);
 			}
-			//for compatibility with old savegames that use player instead of side
+			// for compatibility with old savegames that use player instead of side
 			for(const config& side : replay_start.child_range("player")) {
 				carryover_start.add_child("side", side);
 			}
-		} else if (!replay_start.empty()){
+		} else if(!replay_start.empty()) {
 			for(const config& side : replay_start.child_range("side")) {
 				carryover.add_child("side", side);
 				carryover_start.add_child("side", side);
 			}
-			//for compatibility with old savegames that use player instead of side
+			// for compatibility with old savegames that use player instead of side
 			for(const config& side : replay_start.child_range("player")) {
 				carryover.add_child("side", side);
 				carryover_start.add_child("side", side);
 			}
 		}
 
-		//get variables according to old hierarchy and copy them to new carryover_sides
-		if(!snapshot.empty()){
-			if(const config& variables_from_snapshot = snapshot.child("variables")){
+		// get variables according to old hierarchy and copy them to new carryover_sides
+		if(!snapshot.empty()) {
+			if(const config& variables_from_snapshot = snapshot.child("variables")) {
 				carryover.add_child("variables", variables_from_snapshot);
 				carryover_start.add_child("variables", replay_start.child_or_empty("variables"));
-			} else if (const config& variables_from_cfg = cfg.child("variables")){
+			} else if(const config& variables_from_cfg = cfg.child("variables")) {
 				carryover.add_child("variables", variables_from_cfg);
 				carryover_start.add_child("variables", variables_from_cfg);
 			}
-		} else if (!replay_start.empty()){
-			if(const config& variables = replay_start.child("variables")){
+		} else if(!replay_start.empty()) {
+			if(const config& variables = replay_start.child("variables")) {
 				carryover.add_child("variables", variables);
 				carryover_start.add_child("variables", variables);
 			}
@@ -713,70 +760,69 @@ static void convert_old_saves_1_11_0(config& cfg)
 		cfg.add_child("carryover_sides_start", carryover_start);
 	}
 
-	//if replay and snapshot are empty we've got a start of scenario save and don't want replay_start either
-	if(replay.empty() && snapshot.empty()){
-		LOG_RG<<"removing replay_start \n";
+	// if replay and snapshot are empty we've got a start of scenario save and don't want replay_start either
+	if(replay.empty() && snapshot.empty()) {
+		LOG_RG << "removing replay_start \n";
 		cfg.clear_children("replay_start");
 	}
 
-	//remove empty replay or snapshot so type of save can be detected more easily
-	if(replay.empty()){
-		LOG_RG<<"removing replay \n";
+	// remove empty replay or snapshot so type of save can be detected more easily
+	if(replay.empty()) {
+		LOG_RG << "removing replay \n";
 		cfg.clear_children("replay");
 	}
 
-	if(snapshot.empty()){
-		LOG_RG<<"removing snapshot \n";
+	if(snapshot.empty()) {
+		LOG_RG << "removing snapshot \n";
 		cfg.clear_children("snapshot");
 	}
 }
-//changes done during 1.13.0-dev
+// changes done during 1.13.0-dev
 static void convert_old_saves_1_13_0(config& cfg)
 {
-	if(config& carryover_sides_start = cfg.child("carryover_sides_start"))
-	{
-		if(!carryover_sides_start.has_attribute("next_underlying_unit_id"))
-		{
+	if(config& carryover_sides_start = cfg.child("carryover_sides_start")) {
+		if(!carryover_sides_start.has_attribute("next_underlying_unit_id")) {
 			carryover_sides_start["next_underlying_unit_id"] = cfg["next_underlying_unit_id"];
 		}
 	}
-	if(cfg.child_or_empty("snapshot").empty())
-	{
+
+	if(cfg.child_or_empty("snapshot").empty()) {
 		cfg.clear_children("snapshot");
 	}
-	if(cfg.child_or_empty("replay_start").empty())
-	{
+
+	if(cfg.child_or_empty("replay_start").empty()) {
 		cfg.clear_children("replay_start");
 	}
-	if(config& snapshot = cfg.child("snapshot"))
-	{
-		//make [end_level] -> [end_level_data] since its alo called [end_level_data] in the carryover.
-		if(config& end_level = cfg.child("end_level") )
-		{
+
+	if(config& snapshot = cfg.child("snapshot")) {
+		// make [end_level] -> [end_level_data] since its alo called [end_level_data] in the carryover.
+		if(config& end_level = cfg.child("end_level")) {
 			snapshot.add_child("end_level_data", end_level);
 			snapshot.clear_children("end_level");
 		}
-		//if we have a snapshot then we already applied carryover so there is no reason to keep this data.
-		if(cfg.has_child("carryover_sides_start"))
-		{
+		// if we have a snapshot then we already applied carryover so there is no reason to keep this data.
+		if(cfg.has_child("carryover_sides_start")) {
 			cfg.clear_children("carryover_sides_start");
 		}
 	}
-	if(!cfg.has_child("snapshot") && !cfg.has_child("replay_start"))
-	{
+
+	if(!cfg.has_child("snapshot") && !cfg.has_child("replay_start")) {
 		cfg.clear_children("carryover_sides");
 	}
-	//This code is needed because for example otherwise it won't find the (empty) era
+
+	// This code is needed because for example otherwise it won't find the (empty) era
 	if(!cfg.has_child("multiplayer")) {
-		cfg.add_child("multiplayer", config {
-			"mp_era", "era_blank",
-			"mp_use_map_settings", true,
-		});
+		cfg.add_child("multiplayer",
+			config{
+				"mp_era",
+				"era_blank",
+				"mp_use_map_settings",
+				true,
+			});
 	}
 }
 
-
-//changes done during 1.13.0+dev
+// changes done during 1.13.0+dev
 static void convert_old_saves_1_13_1(config& cfg)
 {
 	if(config& multiplayer = cfg.child("multiplayer")) {
@@ -784,17 +830,13 @@ static void convert_old_saves_1_13_1(config& cfg)
 			multiplayer["mp_era"] = "era_default";
 		}
 	}
-	//This currently only fixes start-of-scenario saves.
-	if(config& carryover_sides_start = cfg.child("carryover_sides_start"))
-	{
-		for(config& side : carryover_sides_start.child_range("side"))
-		{
-			for(config& unit : side.child_range("unit"))
-			{
-				if(config& modifications = unit.child("modifications"))
-				{
-					for(config& advancement : modifications.child_range("advance"))
-					{
+
+	// This currently only fixes start-of-scenario saves.
+	if(config& carryover_sides_start = cfg.child("carryover_sides_start")) {
+		for(config& side : carryover_sides_start.child_range("side")) {
+			for(config& unit : side.child_range("unit")) {
+				if(config& modifications = unit.child("modifications")) {
+					for(config& advancement : modifications.child_range("advance")) {
 						modifications.add_child("advancement", advancement);
 					}
 					modifications.clear_children("advance");
@@ -802,36 +844,54 @@ static void convert_old_saves_1_13_1(config& cfg)
 			}
 		}
 	}
+
 	for(config& snapshot : cfg.child_range("snapshot")) {
-		if (snapshot.has_attribute("used_items")) {
+		if(snapshot.has_attribute("used_items")) {
 			config used_items;
 			for(const std::string& item : utils::split(snapshot["used_items"])) {
 				used_items[item] = true;
 			}
+
 			snapshot.remove_attribute("used_items");
 			snapshot.add_child("used_items", used_items);
 		}
 	}
 }
 
+// changes done during 1.15.3+dev
+static void convert_old_saves_1_15_3(config& cfg)
+{
+	if(cfg["era_id"].empty()) {
+		cfg["era_id"] = cfg.child_or_empty("multiplayer")["mp_era"];
+	}
+
+	if(cfg["active_mods"].empty()) {
+		cfg["active_mods"] = cfg.child_or_empty("multiplayer")["active_mods"];
+	}
+}
+
 void convert_old_saves(config& cfg)
 {
 	version_info loaded_version(cfg["version"]);
-	if(loaded_version < version_info("1.12.0"))
-	{
+	if(loaded_version < version_info("1.12.0")) {
 		convert_old_saves_1_11_0(cfg);
 	}
+
 	// '<= version_info("1.13.0")' doesn't work
-	//because version_info cannot handle 1.13.0-dev versions correctly.
-	if(loaded_version < version_info("1.13.1"))
-	{
+	// because version_info cannot handle 1.13.0-dev versions correctly.
+	if(loaded_version < version_info("1.13.1")) {
 		convert_old_saves_1_13_0(cfg);
 	}
-	if(loaded_version <= version_info("1.13.1"))
-	{
+
+	if(loaded_version <= version_info("1.13.1")) {
 		convert_old_saves_1_13_1(cfg);
 	}
-	LOG_RG<<"cfg after conversion "<<cfg<<"\n";
+
+	if(loaded_version < version_info("1.15.4")) {
+		convert_old_saves_1_15_3(cfg);
+	}
+
+	LOG_RG << "cfg after conversion " << cfg << "\n";
 }
 
-}
+} // namespace savegame
