@@ -1,16 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by Jörg Hinrichs, refactored from various
-   places formerly created by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by Jörg Hinrichs, David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -18,53 +18,62 @@
 #include "config.hpp"
 #include "filesystem.hpp"
 #include "lua_jailbreak_exception.hpp"
+#include "save_index.hpp"
 #include "saved_game.hpp"
 #include "serialization/compression.hpp"
 
 #include <exception>
 
 class config_writer;
+class game_config_view;
 class version_info;
 
-namespace savegame {
+namespace savegame
+{
 /** converts saves from older versions of wesnoth*/
 void convert_old_saves(config& cfg);
-/** Returns true if there is already a savegame with that name. */
+
+/**
+ * Returns true if there is already a savegame with this name, looking only in the default save
+ * directory. Only expected to be used to check whether a subsequent save would overwrite an
+ * existing file, therefore only expected to be used for the default save dir.
+ */
 bool save_game_exists(std::string name, compression::format compressed);
 
-/** Delete all autosaves of a certain scenario. */
+/**
+ * Delete all autosaves of a certain scenario from the default save directory.
+ *
+ * This is only expected to be called when the player starts the next scenario (or finishes the
+ * campaign, in the case of the last scenario), so it's expected to correspond to the next scenario
+ * being written to the default save directory.
+ */
 void clean_saves(const std::string& label);
 
-struct load_game_metadata {
-	/** Name of the savefile to be loaded. */
-	std::string filename;
+struct load_game_metadata
+{
+	/** There may be different instances of the index for different directories */
+	std::shared_ptr<save_index_class> manager {};
+
+	/** Name of the savefile to be loaded (not including the directory). */
+	std::string filename = "";
 
 	/** The difficulty the save is meant to be loaded with. */
-	std::string difficulty;
+	std::string difficulty = "";
 
 	/** State of the "show_replay" checkbox in the load-game dialog. */
-	bool show_replay;
+	bool show_replay = false;
 
 	/** State of the "cancel_orders" checkbox in the load-game dialog. */
-	bool cancel_orders;
+	bool cancel_orders = false;
 
 	/** State of the "change_difficulty" checkbox in the load-game dialog. */
-	bool select_difficulty;
+	bool select_difficulty = false;
 
 	/** Summary config of the save selected in the load game dialog. */
-	config summary;
+	config summary {};
 
 	/** Config information of the savefile to be loaded. */
-	config load_config;
-
-	explicit load_game_metadata(const std::string& fname = "", const std::string& hard = "",
-			bool replay = false, bool stop = false, bool change = false,
-			const config& summary = config(), const config& info = config())
-		: filename(fname), difficulty(hard)
-		, show_replay(replay), cancel_orders(stop), select_difficulty(change)
-		, summary(summary), load_config(info)
-	{
-	}
+	config load_config {};
 };
 
 /**
@@ -75,13 +84,7 @@ class load_game_exception
 	: public lua_jailbreak_exception, public std::exception
 {
 public:
-	load_game_exception(const std::string& fname)
-		: lua_jailbreak_exception()
-		, data_(fname)
-	{
-	}
-
-	load_game_exception(load_game_metadata&& data)
+	explicit load_game_exception(load_game_metadata&& data)
 		: lua_jailbreak_exception()
 		, data_(data)
 	{
@@ -96,7 +99,7 @@ private:
 class loadgame
 {
 public:
-	loadgame(const config& game_config, saved_game& gamestate);
+	loadgame(const std::shared_ptr<save_index_class>& index, saved_game& gamestate);
 	virtual ~loadgame() {}
 
 	/* In any of the following three function, a bool value of false indicates
@@ -133,7 +136,7 @@ private:
 	/** Copy era information into the snapshot. */
 	void copy_era(config& cfg);
 
-	const config& game_config_;
+	const game_config_view& game_config_;
 
 	saved_game& gamestate_; /** Primary output information. */
 
@@ -143,6 +146,9 @@ private:
  * The base class for all savegame stuff.
  * This should not be used directly, as it does not directly produce usable saves.
  * Instead, use one of the derived classes.
+ *
+ * Saves are only created in filesystem::get_saves_dir() - files can be loaded
+ * from elsewhere, but writes are only to that directory.
  */
 class savegame
 {
@@ -207,6 +213,13 @@ protected:
 	/** Title of the savegame dialog */
 	std::string title_;
 
+	/** Will (at the time of writing) be save_index_class::default_saves_dir().
+		There may be different instances the index for different directories, however
+		writing is only expected to happen in the default save directory.
+
+		Making this a class member anyway, while I'm refactoring. */
+	std::shared_ptr<save_index_class> save_index_manager_;
+
 private:
 	/** Subclass-specific part of filename building. */
 	virtual std::string create_initial_filename(unsigned int turn_number) const = 0;
