@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -22,6 +23,7 @@
 #include <string>                       // for string
 
 class config;
+class game_config_view;
 class unit;
 class vconfig;
 namespace ai { class engine_lua; }
@@ -61,7 +63,7 @@ class game_lua_kernel : public lua_kernel_base
 
 	const game_events::queued_event & get_event_info();
 
-	static void extract_preload_scripts(const config& game_config);
+	static void extract_preload_scripts(const game_config_view& game_config);
 	static std::vector<config> preload_scripts;
 	static config preload_config;
 
@@ -73,6 +75,7 @@ class game_lua_kernel : public lua_kernel_base
 	int intf_cancel_action(lua_State *);
 	int intf_add_time_area(lua_State *);
 	int intf_remove_time_area(lua_State *);
+	int intf_get_time_area(lua_State *);
 	int intf_animate_unit(lua_State *);
 	int intf_gamestate_inspector(lua_State *);
 	int impl_run_animation(lua_State *);
@@ -91,30 +94,31 @@ class game_lua_kernel : public lua_kernel_base
 	int intf_unit_ability(lua_State *L);
 	int intf_view_locked(lua_State *L);
 	int intf_lock_view(lua_State *L);
-	int intf_get_terrain(lua_State *L);
-	int intf_set_terrain(lua_State *L);
-	int intf_get_terrain_info(lua_State *L);
+	int impl_get_terrain_info(lua_State *L);
+	template<bool consider_illuminates>
 	int intf_get_time_of_day(lua_State *L);
+	int impl_schedule_get(lua_State *L);
+	int impl_schedule_len(lua_State *L);
+	void luaW_push_schedule(lua_State* L, int area_index);
 	int intf_get_village_owner(lua_State *L);
 	int intf_set_village_owner(lua_State *L);
 	int intf_get_map_size(lua_State *L);
 	int intf_get_mouseover_tile(lua_State *L);
 	int intf_get_selected_tile(lua_State *L);
-	int intf_get_starting_location(lua_State* L);
 	int impl_game_config_get(lua_State *L) override;
 	int impl_game_config_set(lua_State *L) override;
+	int impl_scenario_get(lua_State *L);
+	int impl_scenario_set(lua_State *L);
 	int impl_current_get(lua_State *L);
 	int intf_clear_messages(lua_State*);
-	int intf_end_level(lua_State*);
 	int impl_end_level_data_set(lua_State*);
-	int intf_get_end_level_data(lua_State*);
 	int intf_end_turn(lua_State*);
+	int intf_find_cost_map(lua_State *L);
 	int intf_find_path(lua_State *L);
 	int intf_find_reach(lua_State *L);
-	int intf_find_cost_map(lua_State *L);
+	int intf_find_vision_range(lua_State *L);
 	int intf_heal_unit(lua_State *L);
 	int intf_message(lua_State *L);
-	int intf_open_help(lua_State *L);
 	int intf_play_sound(lua_State *L);
 	int intf_print(lua_State *L);
 	void put_unit_helper(const map_location& loc);
@@ -127,12 +131,12 @@ class game_lua_kernel : public lua_kernel_base
 	int intf_set_end_campaign_credits(lua_State *L);
 	int intf_set_end_campaign_text(lua_State *L);
 	int intf_clear_menu_item(lua_State *L);
+	int intf_create_side(lua_State *L);
 	int intf_set_menu_item(lua_State *L);
-	int intf_set_next_scenario(lua_State *L);
-	int intf_shroud_op(lua_State *L, bool place_shroud);
+	int intf_toggle_shroud(lua_State *L, bool place_shroud);
+	int intf_override_shroud(lua_State *L);
 	int intf_simulate_combat(lua_State *L);
 	int intf_scroll_to_tile(lua_State *L);
-	int intf_select_hex(lua_State *L);
 	int intf_select_unit(lua_State *L);
 	int intf_deselect_hex(lua_State *L);
 	int intf_is_skipping_messages(lua_State *L);
@@ -144,16 +148,18 @@ class game_lua_kernel : public lua_kernel_base
 	int intf_set_side_id(lua_State *L);
 	int intf_modify_ai_wml(lua_State *L);
 	int intf_get_sides(lua_State* L);
+	int intf_get_side(lua_State* L);
 	int intf_add_tile_overlay(lua_State *L);
 	int intf_remove_tile_overlay(lua_State *L);
 	int intf_add_event(lua_State *L);
 	int intf_remove_event(lua_State *L);
 	int intf_color_adjust(lua_State *L);
 	int intf_delay(lua_State *L);
-	int intf_label(lua_State *L);
+	int intf_label(lua_State *L, bool add);
+	int intf_get_label(lua_State* L);
 	int intf_redraw(lua_State *L);
 	int intf_replace_schedule(lua_State *l);
-	int intf_set_time_of_day(lua_State *L);
+	int impl_schedule_set(lua_State *L);
 	int intf_scroll(lua_State *L);
 	int intf_get_all_vars(lua_State *L);
 	int impl_theme_item(lua_State *L, std::string name);
@@ -164,9 +170,6 @@ class game_lua_kernel : public lua_kernel_base
 	int intf_fire_event(lua_State *L, const bool by_id);
 	int intf_fire_wml_menu_item(lua_State *L);
 	int intf_teleport(lua_State *L);
-	int intf_remove_sound_source(lua_State *L);
-	int intf_add_sound_source(lua_State *L);
-	int intf_get_sound_source(lua_State *L);
 	int intf_log(lua_State *L);
 	int intf_toggle_fog(lua_State *L, const bool clear);
 	int intf_get_fog_or_shroud(lua_State *L, bool fog);
@@ -182,12 +185,13 @@ public:
 	game_board & board();
 	std::vector<team> & teams();
 	const gamemap & map() const;
+	game_display * get_display() const { return game_display_; }
 	/**
 		A value != 0 means that the shouldn't remove any units from the map, usually because
 		we are currently operating on a unit& and removing it might cause memory corruptions
 		note that we don't check for the dtor of lua owned units because we assume that
 		we operate on such a unit that the lua function that invoked the operation on that unit
-		(like wesnoth.add_modification, wesnoth.match_unit ..) have a local copy of that
+		(like wesnoth.units.add_modification, wesnoth.units.matches ..) have a local copy of that
 		lua_unit* userdata in its stack that prevents it from being collected.
 	*/
 	int map_locked_;
@@ -210,6 +214,7 @@ public:
 		const game_events::queued_event&);
 	bool run_filter(char const *name, const unit& u);
 	bool run_filter(char const *name, const map_location& l);
+	bool run_filter(char const *name, const team& t);
 	bool run_filter(char const *name, int nArgs);
 	bool run_wml_conditional(const std::string&, const vconfig&);
 
