@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -24,6 +25,7 @@
 #include "scripting/lua_common.hpp"
 
 #include "config.hpp"
+#include "scripting/push_check.hpp"
 #include "scripting/lua_unit.hpp"
 #include "tstring.hpp"                  // for t_string
 #include "variable.hpp" // for vconfig
@@ -52,6 +54,9 @@ static lg::log_domain log_scripting_lua("scripting/lua");
 #define WRN_LUA LOG_STREAM(warn, log_scripting_lua)
 #define ERR_LUA LOG_STREAM(err, log_scripting_lua)
 
+static lg::log_domain log_wml("wml");
+#define ERR_WML LOG_STREAM(err, log_wml)
+
 namespace lua_common {
 
 /**
@@ -75,6 +80,15 @@ static int impl_gettext(lua_State *L)
 	return 1;
 }
 
+static int impl_gettext_tostr(lua_State* L)
+{
+	char* d = static_cast<char*>(lua_touserdata(L, 1));
+	using namespace std::literals;
+	std::string str = "textdomain: "s + d;
+	lua_push(L, str);
+	return 1;
+}
+
 /**
  * Creates an interface for gettext
  * - Arg 1: string containing the domain.
@@ -82,10 +96,10 @@ static int impl_gettext(lua_State *L)
  */
 int intf_textdomain(lua_State *L)
 {
-	size_t l;
+	std::size_t l;
 	char const *m = luaL_checklstring(L, 1, &l);
 
-	void *p = lua_newuserdata(L, l + 1);
+	void *p = lua_newuserdatauv(L, l + 1, 0);
 	memcpy(p, m, l + 1);
 
 	luaL_setmetatable(L, gettextKey);
@@ -301,8 +315,20 @@ static int impl_vconfig_pairs_collect(lua_State *L)
 {
 	typedef config::const_attr_itors const_attr_itors;
 	void* p = lua_touserdata(L, 1);
+
+	// Triggers a false positive of C4189 with Visual Studio. Suppress.
+#if defined(_MSC_VER)
+#pragma warning(push)
+#pragma warning(disable: 4189)
+#endif
+
 	const_attr_itors* cai = static_cast<const_attr_itors*>(p);
 	cai->~const_attr_itors();
+
+#if defined(_MSC_VER)
+#pragma warning(pop)
+#endif
+
 	return 0;
 }
 
@@ -391,6 +417,7 @@ std::string register_gettext_metatable(lua_State *L)
 
 	static luaL_Reg const callbacks[] {
 		{ "__call", 	    &impl_gettext},
+		{ "__tostring",     &impl_gettext_tostr},
 		{ nullptr, nullptr }
 	};
 	luaL_setfuncs(L, callbacks, 0);
@@ -420,6 +447,13 @@ std::string register_tstring_metatable(lua_State *L)
 	};
 	luaL_setfuncs(L, callbacks, 0);
 
+	lua_createtable(L, 0, 1);
+	luaW_getglobal(L, "string", "format");
+	lua_setfield(L, -2, "format");
+	luaW_getglobal(L, "stringx", "vformat");
+	lua_setfield(L, -2, "vformat");
+	lua_setfield(L, -2, "__index");
+
 	lua_pushstring(L, "translatable string");
 	lua_setfield(L, -2, "__metatable");
 
@@ -466,12 +500,12 @@ std::string register_vconfig_metatable(lua_State *L)
 
 } // end namespace lua_common
 
-void* operator new(size_t sz, lua_State *L)
+void* operator new(std::size_t sz, lua_State *L, int nuv)
 {
-	return lua_newuserdata(L, sz);
+	return lua_newuserdatauv(L, sz, nuv);
 }
 
-void operator delete(void*, lua_State *L)
+void operator delete(void*, lua_State *L, int)
 {
 	// Not sure if this is needed since it's a no-op
 	// It's only called if a constructor throws while using the above operator new
@@ -508,12 +542,15 @@ void luaW_pushtstring(lua_State *L, const t_string& v)
 
 
 namespace {
-	struct luaW_pushscalar_visitor : boost::static_visitor<>
+	struct luaW_pushscalar_visitor
+#ifdef USING_BOOST_VARIANT
+		: boost::static_visitor<>
+#endif
 	{
 		lua_State *L;
 		luaW_pushscalar_visitor(lua_State *l): L(l) {}
 
-		void operator()(const boost::blank&) const
+		void operator()(const utils::monostate&) const
 		{ lua_pushnil(L); }
 		void operator()(bool b) const
 		{ lua_pushboolean(L, b); }
@@ -697,6 +734,39 @@ map_location luaW_checklocation(lua_State *L, int index)
 	return result;
 }
 
+int luaW_push_locationset(lua_State* L, const std::set<map_location>& locs)
+{
+	lua_createtable(L, locs.size(), 0);
+	int i = 1;
+	for(const map_location& loc : locs) {
+		lua_createtable(L, 2, 0);
+		lua_pushinteger(L, loc.wml_x());
+		lua_rawseti(L, -2, 1);
+		lua_pushinteger(L, loc.wml_y());
+		lua_rawseti(L, -2, 2);
+		lua_rawseti(L, -2, i);
+		++i;
+	}
+	return 1;
+}
+
+std::set<map_location> luaW_check_locationset(lua_State* L, int idx)
+{
+	std::set<map_location> locs;
+	if(!lua_istable(L, idx)) {
+		luaW_type_error(L, idx, "array of locations");
+	}
+	lua_len(L, idx);
+	int len = luaL_checkinteger(L, -1);
+	for(int i = 1; i <= len; i++) {
+		lua_geti(L, idx, i);
+		locs.insert(luaW_checklocation(L, -1));
+		lua_pop(L, 1);
+	}
+	return locs;
+
+}
+
 void luaW_pushconfig(lua_State *L, const config& cfg)
 {
 	lua_newtable(L);
@@ -745,7 +815,7 @@ bool luaW_toconfig(lua_State *L, int index, config &cfg)
 		if (!lua_istable(L, -1)) return_misformed();
 		lua_rawgeti(L, -1, 1);
 		char const *m = lua_tostring(L, -1);
-		if (!m || !config::valid_tag(m) || m[0] == '_') return_misformed();
+		if (!m || !config::valid_tag(m)) return_misformed();
 		lua_rawgeti(L, -2, 2);
 		if (!luaW_toconfig(L, -1, cfg.add_child(m)))
 			return_misformed();
@@ -758,7 +828,9 @@ bool luaW_toconfig(lua_State *L, int index, config &cfg)
 		int indextype = lua_type(L, -2);
 		if (indextype == LUA_TNUMBER) continue;
 		if (indextype != LUA_TSTRING) return_misformed();
-		config::attribute_value &v = cfg[lua_tostring(L, -2)];
+		const char* m = lua_tostring(L, -2);
+		if(!m || !config::valid_attribute(m)) return_misformed();
+		config::attribute_value &v = cfg[m];
 		if (lua_istable(L, -1)) {
 			int subindex = lua_absindex(L, -1);
 			std::ostringstream str;
@@ -916,7 +988,7 @@ bool luaW_checkvariable(lua_State *L, variable_access_create& v, int n)
 				if (luaW_toconfig(L, n, cfg)) {
 					return true;
 				}
-				FALLTHROUGH;
+				[[fallthrough]];
 			}
 		default:
 		default_explicit:
@@ -931,10 +1003,42 @@ bool luaW_checkvariable(lua_State *L, variable_access_create& v, int n)
 	}
 }
 
+bool luaW_tableget(lua_State *L, int index, const char* key)
+{
+	index = lua_absindex(L, index);
+	lua_pushstring(L, key);
+	lua_gettable(L, index);
+	if(lua_isnoneornil(L, -1)) {
+		lua_pop(L, 1);
+		return false;
+	}
+	return true;
+}
+
+std::string_view luaW_tostring(lua_State *L, int index)
+{
+	size_t len = 0;
+	const char* str = lua_tolstring(L, index, &len);
+	if(!str) {
+		throw luaL_error (L, "not a string");
+	}
+	return std::string_view(str, len);
+}
+
+std::string_view luaW_tostring_or_default(lua_State *L, int index, std::string_view def)
+{
+	size_t len = 0;
+	const char* str = lua_tolstring(L, index, &len);
+	if(!str) {
+		return def;
+	}
+	return std::string_view(str, len);
+}
+
 void chat_message(const std::string& caption, const std::string& msg)
 {
 	if (!game_display::get_singleton()) return;
-	game_display::get_singleton()->get_chat_manager().add_chat_message(time(nullptr), caption, 0, msg,
+	game_display::get_singleton()->get_chat_manager().add_chat_message(std::time(nullptr), caption, 0, msg,
 														   events::chat_handler::MESSAGE_PUBLIC, false);
 }
 
@@ -982,7 +1086,8 @@ bool luaW_pcall(lua_State *L, int nArgs, int nRets, bool allow_wml_error)
 			if (allow_wml_error && strncmp(m, "~wml:", 5) == 0) {
 				m += 5;
 				char const *e = strstr(m, "stack traceback");
-				lg::wml_error() << std::string(m, e ? e - m : strlen(m));
+				lg::log_to_chat() << std::string(m, e ? e - m : strlen(m)) << '\n';
+				ERR_WML << std::string(m, e ? e - m : strlen(m));
 			} else if (allow_wml_error && strncmp(m, "~lua:", 5) == 0) {
 				m += 5;
 				char const *e = nullptr, *em = m;
