@@ -1,20 +1,19 @@
 /*
-   Copyright (C) 2017-2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2017 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "scripting/lua_formula_bridge.hpp"
 
-#include "boost/variant/static_visitor.hpp"
-
 #include "game_board.hpp"
 #include "scripting/game_lua_kernel.hpp"
 #include "scripting/lua_unit.hpp"
@@ -44,11 +43,11 @@ public:
 	variant get_value(const std::string& key) const {
 		if(key == "__list") {
 			std::vector<variant> values;
-			size_t n = lua_rawlen(mState, table_i);
+			std::size_t n = lua_rawlen(mState, table_i);
 			if(n == 0) {
 				return variant();
 			}
-			for(size_t i = 1; i <= n; i++) {
+			for(std::size_t i = 1; i <= n; i++) {
 				lua_pushinteger(mState, i);
 				lua_gettable(mState, table_i);
 				values.push_back(luaW_tofaivariant(mState, -1));
