@@ -1,52 +1,53 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "scripting/lua_kernel_base.hpp"
 
 #include "game_config.hpp"
 #include "game_errors.hpp"
-#include "gui/core/gui_definition.hpp"
+#include "gui/core/gui_definition.hpp" // for remove_single_widget_definition
 #include "log.hpp"
 #include "lua_jailbreak_exception.hpp"  // for lua_jailbreak_exception
-#include "random.hpp"
 #include "seed_rng.hpp"
 #include "deprecation.hpp"
+#include "language.hpp"                 // for get_language
 
 #ifdef DEBUG_LUA
 #include "scripting/debug_lua.hpp"
 #endif
 
+#include "scripting/lua_color.hpp"
 #include "scripting/lua_common.hpp"
 #include "scripting/lua_cpp_function.hpp"
 #include "scripting/lua_fileops.hpp"
 #include "scripting/lua_formula_bridge.hpp"
 #include "scripting/lua_gui2.hpp"
+#include "scripting/lua_wml.hpp"
+#include "scripting/lua_stringx.hpp"
 #include "scripting/lua_map_location_ops.hpp"
+#include "scripting/lua_mathx.hpp"
 #include "scripting/lua_rng.hpp"
+#include "scripting/lua_widget.hpp"
 #include "scripting/push_check.hpp"
 
 #include "game_version.hpp"                  // for do_version_check, etc
-#include "picture.hpp"
 
-#include "formula/string_utils.hpp"
-#include "serialization/string_utils.hpp"
-#include "serialization/parser.hpp"
-#include "utils/functional.hpp"
+#include <functional>
 #include "utils/name_generator.hpp"
 #include "utils/markov_generator.hpp"
 #include "utils/context_free_grammar_generator.hpp"
-#include "variable.hpp" // for config_variable_set
 
 #include <cstring>
 #include <exception>
@@ -68,27 +69,132 @@ static lg::log_domain log_user("scripting/lua/user");
 
 // Registry key for metatable
 static const char * Gen = "name generator";
+static const char * Version = "version";
+// Registry key for lua interpreter environment
+static const char * Interp = "lua interpreter";
 
 // Callback implementations
 
 /**
- * Compares 2 version strings - which is newer.
- * - Args 1,3: version strings
- * - Arg 2: comparison operator (string)
+ * Compares two versions.
+ * - Args 1,2: version strings
  * - Ret 1: comparison result
  */
-static int intf_compare_versions(lua_State* L)
+template<VERSION_COMP_OP vop>
+static int impl_version_compare(lua_State* L)
 {
-	char const *v1 = luaL_checkstring(L, 1);
-
-	const VERSION_COMP_OP vop = parse_version_op(luaL_checkstring(L, 2));
-	if(vop == OP_INVALID) return luaL_argerror(L, 2, "unknown version comparison operator - allowed are ==, !=, <, <=, > and >=");
-
-	char const *v2 = luaL_checkstring(L, 3);
-
-	const bool result = do_version_check(version_info(v1), vop, version_info(v2));
+	version_info& v1 = *static_cast<version_info*>(luaL_checkudata(L, 1, Version));
+	version_info& v2 = *static_cast<version_info*>(luaL_checkudata(L, 2, Version));
+	const bool result = do_version_check(v1, vop, v2);
 	lua_pushboolean(L, result);
+	return 1;
+}
 
+/**
+ * Decomposes a version into its component parts
+ */
+static int impl_version_get(lua_State* L)
+{
+	version_info& vers = *static_cast<version_info*>(luaL_checkudata(L, 1, Version));
+	if(lua_isinteger(L, 2)) {
+		int n = lua_tointeger(L, 2) - 1;
+		auto& components = vers.components();
+		if(n >= 0 && size_t(n) < components.size()) {
+			lua_pushinteger(L, vers.components()[n]);
+		} else {
+			lua_pushnil(L);
+		}
+		return 1;
+	}
+	char const *m = luaL_checkstring(L, 2);
+	return_int_attrib("major", vers.major_version());
+	return_int_attrib("minor", vers.minor_version());
+	return_int_attrib("revision", vers.revision_level());
+	return_bool_attrib("is_canonical", vers.is_canonical());
+	return_string_attrib("special", vers.special_version());
+	if(char sep = vers.special_version_separator()) {
+		return_string_attrib("sep", std::string(1, sep));
+	} else if(strcmp(m, "sep") == 0) {
+		lua_pushnil(L);
+		return 1;
+	}
+	return 0;
+}
+
+/**
+ * Destroy a version
+ */
+static int impl_version_finalize(lua_State* L)
+{
+	version_info* vers = static_cast<version_info*>(luaL_checkudata(L, 1, Version));
+	vers->~version_info();
+	return 0;
+}
+
+/**
+ * Convert a version to string form
+ */
+static int impl_version_tostring(lua_State* L)
+{
+	version_info& vers = *static_cast<version_info*>(luaL_checkudata(L, 1, Version));
+	lua_push(L, vers.str());
+	return 1;
+}
+
+/**
+ * Builds a version from its component parts, or parses it from a string
+ */
+static int intf_make_version(lua_State* L)
+{
+	// If passed a version, just return it unchanged
+	if(luaL_testudata(L, 1, Version)) {
+		lua_settop(L, 1);
+		return 1;
+	}
+	// If it's a string, parse it; otherwise build from components
+	// The components method only supports canonical versions
+	if(lua_type(L, 1) == LUA_TSTRING) {
+		new(L) version_info(lua_check<std::string>(L, 1));
+	} else {
+		int major = luaL_checkinteger(L, 1), minor = luaL_optinteger(L, 2, 0), rev = luaL_optinteger(L, 3, 0);
+		std::string sep, special;
+		if(lua_type(L, -1) == LUA_TSTRING) {
+			special = lua_tostring(L, -1);
+			if(!special.empty() && std::isalpha(special[0])) {
+				sep.push_back('+');
+			} else {
+				sep.push_back(special[0]);
+				special = special.substr(1);
+			}
+		} else {
+			sep.push_back(0);
+		}
+		new(L) version_info(major, minor, rev, sep[0], special);
+	}
+	if(luaL_newmetatable(L, Version)) {
+		static const luaL_Reg metafuncs[] {
+			{ "__index", &impl_version_get },
+			{ "__tostring", &impl_version_tostring },
+			{ "__lt", &impl_version_compare<VERSION_COMP_OP::OP_LESS> },
+			{ "__le", &impl_version_compare<VERSION_COMP_OP::OP_LESS_OR_EQUAL> },
+			{ "__eq", &impl_version_compare<VERSION_COMP_OP::OP_EQUAL> },
+			{ "__gc", &impl_version_finalize },
+			{ nullptr, nullptr }
+		};
+		luaL_setfuncs(L, metafuncs, 0);
+		luaW_table_set<std::string>(L, -1, "__metatable", Version);
+	}
+	lua_setmetatable(L, -2);
+	return 1;
+}
+
+/**
+ * Returns the current Wesnoth version
+ */
+static int intf_current_version(lua_State* L) {
+	lua_settop(L, 0);
+	lua_push(L, game_config::wesnoth_version.str());
+	intf_make_version(L);
 	return 1;
 }
 
@@ -99,10 +205,10 @@ static int intf_compare_versions(lua_State* L)
 int lua_kernel_base::intf_print(lua_State* L)
 {
 	DBG_LUA << "intf_print called:\n";
-	size_t nargs = lua_gettop(L);
+	std::size_t nargs = lua_gettop(L);
 
 	lua_getglobal(L, "tostring");
-	for (size_t i = 1; i <= nargs; ++i) {
+	for (std::size_t i = 1; i <= nargs; ++i) {
 		lua_pushvalue(L, -1); // function to call: "tostring"
 		lua_pushvalue(L, i); // value to pass through tostring() before printing
 		lua_call(L, 1, 1);
@@ -126,6 +232,28 @@ int lua_kernel_base::intf_print(lua_State* L)
 	return 0;
 }
 
+static void impl_warn(void* p, const char* msg, int tocont) {
+	static const char*const prefix = "Warning:\n  ";
+	static std::ostringstream warning(prefix);
+	warning.seekp(0, std::ios::end);
+	warning << msg << ' ';
+	if(!tocont) {
+		auto L = reinterpret_cast<lua_State*>(p);
+		luaW_getglobal(L, "debug", "traceback");
+		lua_push(L, warning.str());
+		lua_pushinteger(L, 2);
+		lua_call(L, 2, 1);
+		auto& lk = lua_kernel_base::get_lua_kernel<lua_kernel_base>(L);
+		lk.add_log_to_console(luaL_checkstring(L, -1));
+		warning.str(prefix);
+	}
+}
+
+void lua_kernel_base::add_log_to_console(const std::string& msg) {
+	cmd_log_ << msg << "\n";
+	DBG_LUA << "'" << msg << "'\n";
+}
+
 /**
  * Replacement load function. Mostly the same as regular load, but disallows loading binary chunks
  * due to CVE-2018-1999023.
@@ -164,24 +292,6 @@ static int intf_load(lua_State* L)
 	return 1;
 }
 
-// Same for loadstring.
-static int intf_loadstring(lua_State* L)
-{
-	std::string string = luaL_checkstring(L, 1);
-	const char* name = luaL_optstring(L, 2, string.c_str());
-
-	deprecated_message("loadstring()", DEP_LEVEL::FOR_REMOVAL, {1, 15, 0}, "Use load() instead.");
-
-	int result = luaL_loadbufferx(L, string.data(), string.length(), name, "t");
-	if(result != LUA_OK) {
-		lua_pushnil(L);
-		lua_insert(L, -2);
-		return 2;
-	}
-
-	return 1;
-}
-
 // The show lua console callback is similarly a method of lua kernel
 int lua_kernel_base::intf_show_lua_console(lua_State *L)
 {
@@ -219,7 +329,7 @@ static int intf_name_generator(lua_State *L)
 			if(lua_istable(L, 2)) {
 				input = lua_check<std::vector<std::string>>(L, 2);
 			} else {
-				input = utils::parenthetical_split(luaL_checkstring(L, 2), ',');
+				input = utils::parenthetical_split(luaW_checktstring(L, 2), ',');
 			}
 			int chain_sz = luaL_optinteger(L, 3, 2);
 			int max_len = luaL_optinteger(L, 4, 12);
@@ -235,11 +345,16 @@ static int intf_name_generator(lua_State *L)
 						return lua_error(L);
 					}
 					if(lua_isstring(L, -1)) {
-						data[lua_tostring(L,-2)] = utils::split(lua_tostring(L,-1),'|');
+						auto& productions = data[lua_tostring(L,-2)] = utils::split(luaW_checktstring(L,-1).str(), '|');
+						if(productions.size() > 1) {
+							deprecated_message("wesnoth.name_generator('cfg', {nonterminal = 'a|b'})", DEP_LEVEL::INDEFINITE, "1.17", "Non-terminals should now be assigned an array of productions instead of a single string containing productions separated by | - but a single string is fine if it's only one production");
+						}
 					} else if(lua_istable(L, -1)) {
-						data[lua_tostring(L,-2)] = lua_check<std::vector<std::string>>(L, -1);
+						const auto& split = lua_check<std::vector<t_string>>(L, -1);
+						auto& productions = data[lua_tostring(L,-2)];
+						std::transform(split.begin(), split.end(), std::back_inserter(productions), std::mem_fn(&t_string::str));
 					} else {
-						lua_pushstring(L, "CFG generator: invalid noterminal value (must be a string or list of strings)");
+						lua_pushstring(L, "CFG generator: invalid nonterminal value (must be a string or list of strings)");
 						return lua_error(L);
 					}
 				}
@@ -247,7 +362,7 @@ static int intf_name_generator(lua_State *L)
 					gen = new(L) context_free_grammar_generator(data);
 				}
 			} else {
-				gen = new(L) context_free_grammar_generator(luaL_checkstring(L, 2));
+				gen = new(L) context_free_grammar_generator(luaW_checktstring(L, 2));
 			}
 			if(gen) {
 				assert(static_cast<void*>(gen) == dynamic_cast<context_free_grammar_generator*>(gen));
@@ -269,44 +384,6 @@ static int intf_name_generator(lua_State *L)
 	return 1;
 }
 
-/**
-* Returns a random numer, same interface as math.random.
-*/
-static int intf_random(lua_State *L)
-{
-	if (lua_isnoneornil(L, 1)) {
-		double r = static_cast<double>(randomness::generator->next_random());
-		double r_max = static_cast<double>(std::numeric_limits<uint32_t>::max());
-		lua_push(L, r / (r_max + 1));
-		return 1;
-	}
-	else {
-		int32_t min;
-		int32_t max;
-		if (lua_isnumber(L, 2)) {
-			min = lua_check<int32_t>(L, 1);
-			max = lua_check<int32_t>(L, 2);
-		}
-		else {
-			min = 1;
-			max = lua_check<int32_t>(L, 1);
-		}
-		if (min > max) {
-			return luaL_argerror(L, 1, "min > max");
-		}
-		lua_push(L, randomness::generator->get_random_int(min, max));
-		return 1;
-	}
-}
-
-static int intf_wml_matches_filter(lua_State* L)
-{
-	config cfg = luaW_checkconfig(L, 1);
-	config filter = luaW_checkconfig(L, 2);
-	lua_pushboolean(L, cfg.matches(filter));
-	return 1;
-}
-
 /**
 * Logs a message
 * Arg 1: (optional) Logger
@@ -344,74 +421,29 @@ static int intf_deprecated_message(lua_State* L) {
 	const DEP_LEVEL level = DEP_LEVEL(luaL_checkinteger(L, 2));
 	const std::string ver_str = lua_isnoneornil(L, 3) ? "" : luaL_checkstring(L, 3);
 	const std::string detail = luaW_checktstring(L, 4);
-	const version_info ver = ver_str.empty() ? game_config::version : ver_str;
+	const version_info ver = ver_str.empty() ? game_config::wesnoth_version.str() : ver_str;
 	const std::string msg = deprecated_message(elem, level, ver, detail);
 	if(level < DEP_LEVEL::INDEFINITE || level >= DEP_LEVEL::REMOVED) {
 		// Invalid deprecation level or level 4 deprecation should raise an interpreter error
 		lua_push(L, msg);
 		return lua_error(L);
 	}
+	lua_warning(L, msg.c_str(), false);
 	return 0;
 }
 
-/**
-* Gets the dimension of an image.
-* - Arg 1: string.
-* - Ret 1: width.
-* - Ret 2: height.
-*/
-static int intf_get_image_size(lua_State *L) {
-	char const *m = luaL_checkstring(L, 1);
-	image::locator img(m);
-	if(!img.file_exists()) return 0;
-	surface s = get_image(img);
-	lua_pushinteger(L, s->w);
-	lua_pushinteger(L, s->h);
-	return 2;
-}
-
 /**
 * Returns the time stamp, exactly as [set_variable] time=stamp does.
 * - Ret 1: integer
 */
-static int intf_get_time_stamp(lua_State *L) {
+static int intf_ms_since_init(lua_State *L) {
 	lua_pushinteger(L, SDL_GetTicks());
 	return 1;
 }
 
-static int intf_format(lua_State* L)
+static int intf_get_language(lua_State* L)
 {
-	config cfg = luaW_checkconfig(L, 2);
-	config_variable_set variables(cfg);
-	if(lua_isstring(L, 1)) {
-		std::string str = lua_tostring(L, 1);
-		lua_push(L, utils::interpolate_variables_into_string(str, variables));
-		return 1;
-	}
-	t_string str = luaW_checktstring(L, 1);
-	lua_push(L, utils::interpolate_variables_into_tstring(str, variables));
-	return 1;
-}
-
-template<bool conjunct>
-static int intf_format_list(lua_State* L)
-{
-	const t_string empty = luaW_checktstring(L, 1);
-	auto values = lua_check<std::vector<t_string>>(L, 2);
-	lua_push(L, (conjunct ? utils::format_conjunct_list : utils::format_disjunct_list)(empty, values));
-	return 1;
-}
-
-/**
-* Dumps a wml table or userdata wml object into a pretty string.
-* - Arg 1: wml table or vconfig userdata
-* - Ret 1: string
-*/
-static int intf_wml_tostring(lua_State* L) {
-	const config& arg = luaW_checkconfig(L, 1);
-	std::ostringstream stream;
-	write(stream, arg);
-	lua_pushstring(L, stream.str().c_str());
+	lua_push(L, get_language().localename);
 	return 1;
 }
 
@@ -435,6 +467,12 @@ lua_kernel_base::lua_kernel_base()
 
 	cmd_log_ << "Initializing " << my_name() << "...\n";
 
+	// Define the CPP_function metatable ( so we can override print to point to a C++ member function, add certain functions for this kernel, etc. )
+	// Do it first of all in case C++ functions are ever used in the core Wesnoth libs loaded in the next step
+	cmd_log_ << "Adding boost function proxy...\n";
+
+	lua_cpp::register_metatable(L);
+
 	// Open safe libraries.
 	// Debug and OS are not, but most of their functions will be disabled below.
 	cmd_log_ << "Adding standard libs...\n";
@@ -448,11 +486,17 @@ lua_kernel_base::lua_kernel_base()
 		{ "debug",  luaopen_debug  },
 		{ "os",     luaopen_os     },
 		{ "utf8",	luaopen_utf8   }, // added in Lua 5.3
+		// Wesnoth libraries
+		{ "stringx",lua_stringx::luaW_open },
+		{ "mathx",  lua_mathx::luaW_open },
+		{ "wml",    lua_wml::luaW_open },
+		{ "gui",    lua_gui2::luaW_open },
+		{ "filesystem", lua_fileops::luaW_open },
 		{ nullptr, nullptr }
 	};
 	for (luaL_Reg const *lib = safe_libs; lib->func; ++lib)
 	{
-		luaL_requiref(L, lib->name, lib->func, 1);
+		luaL_requiref(L, lib->name, lib->func, strlen(lib->name));
 		lua_pop(L, 1);  /* remove lib */
 	}
 
@@ -469,18 +513,6 @@ lua_kernel_base::lua_kernel_base()
 	}
 	lua_pop(L, 1);
 
-	// Disable functions from debug which we don't want.
-	lua_getglobal(L, "debug");
-	lua_pushnil(L);
-	while(lua_next(L, -2) != 0) {
-		lua_pop(L, 1);
-		char const* function = lua_tostring(L, -1);
-		if(strcmp(function, "traceback") == 0 || strcmp(function, "getinfo") == 0) continue;	//traceback is needed for our error handler
-		lua_pushnil(L);										//getinfo is needed for ilua strict mode
-		lua_setfield(L, -3, function);
-	}
-	lua_pop(L, 1);
-
 	// Delete dofile and loadfile.
 	lua_pushnil(L);
 	lua_setglobal(L, "dofile");
@@ -491,63 +523,26 @@ lua_kernel_base::lua_kernel_base()
 	cmd_log_ << "Adding error handler...\n";
 	push_error_handler(L);
 
-	// Create the gettext metatable.
-	cmd_log_ << lua_common::register_gettext_metatable(L);
-
-	// Create the tstring metatable.
-	cmd_log_ << lua_common::register_tstring_metatable(L);
-
 
 	lua_settop(L, 0);
 
-	// Define the CPP_function metatable ( so we can override print to point to a C++ member function, add "show_dialog" for this kernel, etc. )
-	cmd_log_ << "Adding boost function proxy...\n";
-
-	lua_cpp::register_metatable(L);
-
 	// Add some callback from the wesnoth lib
 	cmd_log_ << "Registering basic wesnoth API...\n";
 
 	static luaL_Reg const callbacks[] {
-		{ "compare_versions",         &intf_compare_versions         		},
-		{ "debug",                    &intf_wml_tostring                           },
 		{ "deprecated_message",       &intf_deprecated_message              },
-		{ "have_file",                &lua_fileops::intf_have_file          },
-		{ "read_file",                &lua_fileops::intf_read_file          },
 		{ "textdomain",               &lua_common::intf_textdomain   		},
-		{ "tovconfig",                &lua_common::intf_tovconfig		},
-		{ "get_dialog_value",         &lua_gui2::intf_get_dialog_value		},
-		{ "set_dialog_active",        &lua_gui2::intf_set_dialog_active		},
-		{ "set_dialog_visible",       &lua_gui2::intf_set_dialog_visible    },
-		{ "add_dialog_tree_node",     &lua_gui2::intf_add_dialog_tree_node	},
-		{ "add_widget_definition",    &lua_gui2::intf_add_widget_definition },
-		{ "set_dialog_callback",      &lua_gui2::intf_set_dialog_callback	},
-		{ "set_dialog_canvas",        &lua_gui2::intf_set_dialog_canvas		},
-		{ "set_dialog_focus",         &lua_gui2::intf_set_dialog_focus      },
-		{ "set_dialog_markup",        &lua_gui2::intf_set_dialog_markup		},
-		{ "set_dialog_value",         &lua_gui2::intf_set_dialog_value		},
-		{ "remove_dialog_item",       &lua_gui2::intf_remove_dialog_item    },
 		{ "dofile",                   &dispatch<&lua_kernel_base::intf_dofile>           },
 		{ "require",                  &dispatch<&lua_kernel_base::intf_require>          },
 		{ "kernel_type",              &dispatch<&lua_kernel_base::intf_kernel_type>          },
-		{ "show_dialog",              &lua_gui2::show_dialog   },
-		{ "show_menu",                &lua_gui2::show_menu  },
-		{ "show_message_dialog",      &lua_gui2::show_message_dialog },
-		{ "show_popup_dialog",        &lua_gui2::show_popup_dialog   },
-		{ "show_story",               &lua_gui2::show_story          },
-		{ "show_message_box",         &lua_gui2::show_message_box    },
-		{ "show_lua_console",	      &dispatch<&lua_kernel_base::intf_show_lua_console> },
 		{ "compile_formula",          &lua_formula_bridge::intf_compile_formula},
 		{ "eval_formula",             &lua_formula_bridge::intf_eval_formula},
 		{ "name_generator",           &intf_name_generator           },
-		{ "random",                   &intf_random                   },
-		{ "wml_matches_filter",       &intf_wml_matches_filter       },
 		{ "log",                      &intf_log                      },
-		{ "get_image_size",           &intf_get_image_size           },
-		{ "get_time_stamp",           &intf_get_time_stamp           },
-		{ "format",                   &intf_format                   },
-		{ "format_conjunct_list",     &intf_format_list<true>        },
-		{ "format_disjunct_list",     &intf_format_list<false>       },
+		{ "ms_since_init",            &intf_ms_since_init           },
+		{ "get_language",             &intf_get_language             },
+		{ "version",                  &intf_make_version       },
+		{ "current_version",          &intf_current_version    },
 		{ nullptr, nullptr }
 	};
 
@@ -559,6 +554,13 @@ lua_kernel_base::lua_kernel_base()
 	//lua_cpp::set_functions(L, cpp_callbacks, 0);
 	lua_setglobal(L, "wesnoth");
 
+	// Create the gettext metatable.
+	cmd_log_ << lua_common::register_gettext_metatable(L);
+	// Create the tstring metatable.
+	cmd_log_ << lua_common::register_tstring_metatable(L);
+
+	lua_widget::register_metatable(L);
+
 	// Override the print function
 	cmd_log_ << "Redirecting print function...\n";
 
@@ -566,12 +568,13 @@ lua_kernel_base::lua_kernel_base()
 	lua_setglobal(L, "std_print"); //storing original impl as 'std_print'
 	lua_settop(L, 0); //clear stack, just to be sure
 
+	lua_setwarnf(L, &::impl_warn, L);
 	lua_pushcfunction(L, &dispatch<&lua_kernel_base::intf_print>);
 	lua_setglobal(L, "print");
 
 	lua_pushcfunction(L, intf_load);
 	lua_setglobal(L, "load");
-	lua_pushcfunction(L, intf_loadstring);
+	lua_pushnil(L);
 	lua_setglobal(L, "loadstring");
 
 	cmd_log_ << "Initializing package repository...\n";
@@ -596,8 +599,9 @@ lua_kernel_base::lua_kernel_base()
 		{ "vector_diff",			&lua_map_location::intf_vector_diff			},
 		{ "vector_negation",		&lua_map_location::intf_vector_negation			},
 		{ "rotate_right_around_center",	&lua_map_location::intf_rotate_right_around_center	},
-		{ "tiles_adjacent",		&lua_map_location::intf_tiles_adjacent			},
-		{ "get_adjacent_tiles",		&lua_map_location::intf_get_adjacent_tiles		},
+		{ "are_hexes_adjacent",		&lua_map_location::intf_tiles_adjacent			},
+		{ "get_adjacent_hexes",		&lua_map_location::intf_get_adjacent_tiles		},
+		{ "get_hexes_in_radius",		&lua_map_location::intf_get_tiles_in_radius		},
 		{ "distance_between",		&lua_map_location::intf_distance_between		},
 		{ "get_in_basis_N_NE",		&lua_map_location::intf_get_in_basis_N_NE		},
 		{ "get_relative_dir",		&lua_map_location::intf_get_relative_dir		},
@@ -615,7 +619,7 @@ lua_kernel_base::lua_kernel_base()
 	cmd_log_ << "Adding game_config table...\n";
 
 	lua_getglobal(L, "wesnoth");
-	lua_newuserdata(L, 0);
+	lua_newuserdatauv(L, 0, 0);
 	lua_createtable(L, 0, 3);
 	lua_pushcfunction(L, &dispatch<&lua_kernel_base::impl_game_config_get>);
 	lua_setfield(L, -2, "__index");
@@ -643,6 +647,18 @@ lua_kernel_base::lua_kernel_base()
 	// Create formula bridge metatables
 	cmd_log_ << lua_formula_bridge::register_metatables(L);
 
+	cmd_log_ << lua_colors::register_metatables(L);
+
+	// Create the Lua interpreter table
+	cmd_log_ << "Sandboxing Lua interpreter...\nTo make variables visible outside the interpreter, assign to _G.variable.\n";
+	cmd_log_ << "The special variable _ holds the result of the last expression (if any).\n";
+	lua_newtable(L);
+	lua_createtable(L, 0, 1);
+	lua_getglobal(L, "_G");
+	lua_setfield(L, -2, "__index");
+	lua_setmetatable(L, -2);
+	lua_setfield(L, LUA_REGISTRYINDEX, Interp);
+
 	// Loading ilua:
 	cmd_log_ << "Loading ilua...\n";
 
@@ -653,7 +669,7 @@ lua_kernel_base::lua_kernel_base()
 		//run "ilua.set_strict()"
 		lua_pushstring(L, "set_strict");
 		lua_gettable(L, -2);
-		if (!this->protected_call(0,0, std::bind(&lua_kernel_base::log_error, this, _1, _2))) {
+		if (!this->protected_call(0,0, std::bind(&lua_kernel_base::log_error, this, std::placeholders::_1, std::placeholders::_2))) {
 			cmd_log_ << "Failed to activate strict mode.\n";
 		} else {
 			cmd_log_ << "Activated strict mode.\n";
@@ -664,6 +680,19 @@ lua_kernel_base::lua_kernel_base()
 		cmd_log_ << "Error: failed to load ilua.\n";
 	}
 	lua_settop(L, 0);
+
+	// Disable functions from debug which we don't want.
+	// We do this last because ilua needs to be able to use debug.getmetatable
+	lua_getglobal(L, "debug");
+	lua_pushnil(L);
+	while(lua_next(L, -2) != 0) {
+		lua_pop(L, 1);
+		char const* function = lua_tostring(L, -1);
+		if(strcmp(function, "traceback") == 0 || strcmp(function, "getinfo") == 0) continue;	//traceback is needed for our error handler
+		lua_pushnil(L);										//getinfo is needed for ilua strict mode
+		lua_setfield(L, -3, function);
+	}
+	lua_pop(L, 1);
 }
 
 lua_kernel_base::~lua_kernel_base()
@@ -686,14 +715,14 @@ void lua_kernel_base::throw_exception(char const * msg, char const * context)
 
 bool lua_kernel_base::protected_call(int nArgs, int nRets)
 {
-	error_handler eh = std::bind(&lua_kernel_base::log_error, this, _1, _2 );
+	error_handler eh = std::bind(&lua_kernel_base::log_error, this, std::placeholders::_1, std::placeholders::_2 );
 	return this->protected_call(nArgs, nRets, eh);
 }
 
-bool lua_kernel_base::load_string(char const * prog)
+bool lua_kernel_base::load_string(char const * prog, const std::string& name)
 {
-	error_handler eh = std::bind(&lua_kernel_base::log_error, this, _1, _2 );
-	return this->load_string(prog, eh);
+	error_handler eh = std::bind(&lua_kernel_base::log_error, this, std::placeholders::_1, std::placeholders::_2 );
+	return this->load_string(prog, name, eh);
 }
 
 bool lua_kernel_base::protected_call(int nArgs, int nRets, error_handler e_h)
@@ -715,8 +744,6 @@ bool lua_kernel_base::protected_call(lua_State * L, int nArgs, int nRets, error_
 			context += "Lua error in attached debugger: ";
 		} else if (errcode == LUA_ERRMEM) {
 			context += "Lua out of memory error: ";
-		} else if (errcode == LUA_ERRGCMM) {
-			context += "Lua error in garbage collection metamethod: ";
 		} else {
 			context += "unknown lua error: ";
 		}
@@ -736,11 +763,10 @@ bool lua_kernel_base::protected_call(lua_State * L, int nArgs, int nRets, error_
 	return true;
 }
 
-bool lua_kernel_base::load_string(char const * prog, error_handler e_h)
+bool lua_kernel_base::load_string(char const * prog, const std::string& name, error_handler e_h)
 {
 	// pass 't' to prevent loading bytecode which is unsafe and can be used to escape the sandbox.
-	// todo: maybe allow a 'name' parameter to give better error messages.
-	int errcode = luaL_loadbufferx(mState, prog, strlen(prog), /*name*/ prog, "t");
+	int errcode = luaL_loadbufferx(mState, prog, strlen(prog), name.empty() ? name.c_str() : prog, "t");
 	if (errcode != LUA_OK) {
 		char const * msg = lua_tostring(mState, -1);
 		std::string message = msg ? msg : "null string";
@@ -751,8 +777,6 @@ bool lua_kernel_base::load_string(char const * prog, error_handler e_h)
 			context += " a syntax error";
 		} else if(errcode == LUA_ERRMEM){
 			context += " a memory error";
-		} else if(errcode == LUA_ERRGCMM) {
-			context += " an error in garbage collection metamethod";
 		} else {
 			context += " an unknown error";
 		}
@@ -773,24 +797,29 @@ void lua_kernel_base::run_lua_tag(const config& cfg)
 		luaW_pushconfig(this->mState, args);
 		++nArgs;
 	}
-	this->run(cfg["code"].str().c_str(), nArgs);
+	this->run(cfg["code"].str().c_str(), cfg["name"].str(), nArgs);
 }
 // Call load_string and protected call. Make them throw exceptions.
 //
-void lua_kernel_base::throwing_run(const char * prog, int nArgs)
+void lua_kernel_base::throwing_run(const char * prog, const std::string& name, int nArgs, bool in_interpreter)
 {
 	cmd_log_ << "$ " << prog << "\n";
-	error_handler eh = std::bind(&lua_kernel_base::throw_exception, this, _1, _2 );
-	this->load_string(prog, eh);
+	error_handler eh = std::bind(&lua_kernel_base::throw_exception, this, std::placeholders::_1, std::placeholders::_2 );
+	this->load_string(prog, name, eh);
+	if(in_interpreter) {
+		lua_getfield(mState, LUA_REGISTRYINDEX, Interp);
+		if(lua_setupvalue(mState, -2, 1) == nullptr)
+			lua_pop(mState, 1);
+	}
 	lua_insert(mState, -nArgs - 1);
-	this->protected_call(nArgs, 0, eh);
+	this->protected_call(nArgs, in_interpreter ? LUA_MULTRET : 0, eh);
 }
 
 // Do a throwing run, but if we catch a lua_error, reformat it with signature for this function and log it.
-void lua_kernel_base::run(const char * prog, int nArgs)
+void lua_kernel_base::run(const char * prog, const std::string& name, int nArgs)
 {
 	try {
-		this->throwing_run(prog, nArgs);
+		this->throwing_run(prog, name, nArgs);
 	} catch (const game::lua_error & e) {
 		cmd_log_ << e.what() << "\n";
 		lua_kernel_base::log_error(e.what(), "In function lua_kernel::run()");
@@ -799,22 +828,48 @@ void lua_kernel_base::run(const char * prog, int nArgs)
 
 // Tests if a program resolves to an expression, and pretty prints it if it is, otherwise it runs it normally. Throws exceptions.
 void lua_kernel_base::interactive_run(char const * prog) {
-	std::string experiment = "ilua._pretty_print(";
+	std::string experiment = "return ";
 	experiment += prog;
-	experiment += ")";
+	int top = lua_gettop(mState);
 
-	error_handler eh = std::bind(&lua_kernel_base::throw_exception, this, _1, _2 );
+	error_handler eh = std::bind(&lua_kernel_base::throw_exception, this, std::placeholders::_1, std::placeholders::_2 );
+	luaW_getglobal(mState, "ilua", "_pretty_print");
 
 	try {
 		// Try to load the experiment without syntax errors
-		this->load_string(experiment.c_str(), eh);
+		this->load_string(experiment.c_str(), "interactive", eh);
+		lua_getfield(mState, LUA_REGISTRYINDEX, Interp);
+		if(lua_setupvalue(mState, -2, 1) == nullptr)
+			lua_pop(mState, 1);
 	} catch (const game::lua_error &) {
-		this->throwing_run(prog, 0);	// Since it failed, fall back to the usual throwing_run, on the original input.
+		this->throwing_run(prog, "interactive", 0, true);	// Since it failed, fall back to the usual throwing_run, on the original input.
+		if(lua_gettop(mState) == top + 1) {
+			// Didn't return anything
+			lua_settop(mState, top);
 		return;
+		} else goto PRINT;
 	}
 	// experiment succeeded, now run but log normally.
 	cmd_log_ << "$ " << prog << "\n";
-	this->protected_call(0, 0, eh);
+	this->protected_call(0, LUA_MULTRET, eh);
+PRINT:
+	int nRets = lua_gettop(mState) - top - 1;
+	{
+		// Assign first result to _
+		lua_getfield(mState, LUA_REGISTRYINDEX, Interp);
+		int env_idx = lua_gettop(mState);
+		lua_pushvalue(mState, top + 2);
+		lua_setfield(mState, -2, "_");
+		// Now duplicate EVERY result and pass it to table.pack, assigning to _all
+		luaW_getglobal(mState, "table", "pack");
+		for(int i = top + 2; i < env_idx; i++)
+			lua_pushvalue(mState, i);
+		this->protected_call(nRets, 1, eh);
+		lua_setfield(mState, -2, "_all");
+		lua_pop(mState, 1);
+	}
+	// stack is now ilua._pretty_print followed by any results of prog
+	this->protected_call(lua_gettop(mState) - top - 1, 0, eh);
 }
 /**
  * Loads and executes a Lua file.
@@ -869,7 +924,7 @@ int lua_kernel_base::intf_require(lua_State* L)
 	}
 	DBG_LUA << "require: loaded a file, now calling it\n";
 
-	if (!this->protected_call(L, 0, 1, std::bind(&lua_kernel_base::log_error, this, _1, _2))) {
+	if (!this->protected_call(L, 0, 1, std::bind(&lua_kernel_base::log_error, this, std::placeholders::_1, std::placeholders::_2))) {
 		// historically if wesnoth.require fails it just yields nil and some logging messages, not a lua error
 		return 0;
     }
@@ -899,9 +954,11 @@ int lua_kernel_base::impl_game_config_get(lua_State* L)
 	return_int_attrib("rest_heal_amount", game_config::rest_heal_amount);
 	return_int_attrib("recall_cost", game_config::recall_cost);
 	return_int_attrib("kill_experience", game_config::kill_experience);
-	return_string_attrib("version", game_config::version);
+	return_int_attrib("combat_experience", game_config::combat_experience);
+	return_string_attrib_deprecated("version", "wesnoth.game_config", INDEFINITE, "1.17", "Use wesnoth.current_version() instead", game_config::wesnoth_version.str());
 	return_bool_attrib("debug", game_config::debug);
 	return_bool_attrib("debug_lua", game_config::debug_lua);
+	return_bool_attrib("strict_lua", game_config::strict_lua);
 	return_bool_attrib("mp_debug", game_config::mp_debug);
 	return 0;
 }
@@ -931,7 +988,7 @@ void lua_kernel_base::load_core()
 	lua_settop(L, 0);
 	cmd_log_ << "Loading core...\n";
 	luaW_getglobal(L, "wesnoth", "require");
-	lua_pushstring(L, "lua/core.lua");
+	lua_pushstring(L, "lua/core");
 	if(!protected_call(1, 1)) {
 		cmd_log_ << "Error: Failed to load core.\n";
 	}
@@ -968,7 +1025,7 @@ std::vector<std::string> lua_kernel_base::get_attribute_names(const std::string
 {
 	std::vector<std::string> ret;
 	std::string base_path = input;
-	size_t last_dot = base_path.find_last_of('.');
+	std::size_t last_dot = base_path.find_last_of('.');
 	std::string partial_name = base_path.substr(last_dot + 1);
 	base_path.erase(last_dot);
 	std::string load = "return " + base_path;
