@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "scripting/lua_unit.hpp"
@@ -27,6 +28,8 @@
 #include "units/unit.hpp"
 #include "units/map.hpp"
 #include "units/animation_component.hpp"
+#include "game_version.hpp"
+#include "deprecation.hpp"
 
 #include "lua/lauxlib.h"
 #include "lua/lua.h"                    // for lua_State, lua_settop, etc
@@ -43,7 +46,7 @@ lua_unit::~lua_unit()
 {
 }
 
-unit* lua_unit::get()
+unit* lua_unit::get() const
 {
 	if (ptr) return ptr.get();
 	if (c_ptr) return c_ptr;
@@ -54,7 +57,7 @@ unit* lua_unit::get()
 	if (!ui.valid()) return nullptr;
 	return ui.get_shared_ptr().get(); //&*ui would not be legal, must get new shared_ptr by copy ctor because the unit_map itself is holding a boost shared pointer.
 }
-unit_ptr lua_unit::get_shared()
+unit_ptr lua_unit::get_shared() const
 {
 	if (ptr) return ptr;
 	if (side) {
@@ -72,10 +75,7 @@ unit_ptr lua_unit::get_shared()
 bool lua_unit::put_map(const map_location &loc)
 {
 	if (ptr) {
-		unit_map::unit_iterator unit_it;
-		bool success = false;
-
-		std::tie(unit_it, success) = resources::gameboard->units().replace(loc, ptr);
+		auto [unit_it, success] = resources::gameboard->units().replace(loc, ptr);
 
 		if(success) {
 			ptr.reset();
@@ -239,6 +239,32 @@ static int impl_unit_equality(lua_State* L)
 	return 1;
 }
 
+/**
+ * Turns a lua proxy unit to string. (__tostring metamethod)
+ */
+static int impl_unit_tostring(lua_State* L)
+{
+	const lua_unit* lu = luaW_tounit_ref(L, 1);
+	unit &u = *lu->get();
+	std::ostringstream str;
+
+	str << "unit: <";
+	if(!u.id().empty()) {
+		str << u.id() << " ";
+	} else {
+		str << u.type_id() << " ";
+	}
+	if(int side = lu->on_recall_list()) {
+		str << "at (side " << side << " recall list)";
+	} else {
+		str << "at (" << u.get_location() << ")";
+	}
+	str << '>';
+
+	lua_push(L, str.str());
+	return 1;
+}
+
 /**
  * Gets some data on a unit (__index metamethod).
  * - Arg 1: full userdata containing the unit id.
@@ -275,15 +301,20 @@ static int impl_unit_get(lua_State *L)
 	return_int_attrib("x", u.get_location().wml_x());
 	return_int_attrib("y", u.get_location().wml_y());
 	if(strcmp(m, "loc") == 0) {
-		lua_pushinteger(L, u.get_location().wml_x());
-		lua_pushinteger(L, u.get_location().wml_y());
-		return 2;
+		luaW_pushlocation(L, u.get_location());
+		return 1;
+	}
+	if(strcmp(m, "goto") == 0) {
+		luaW_pushlocation(L, u.get_goto());
+		return 1;
 	}
 	return_int_attrib("side", u.side());
 	return_string_attrib("id", u.id());
 	return_string_attrib("type", u.type_id());
 	return_string_attrib("image_mods", u.effect_image_mods());
 	return_string_attrib("usage", u.usage());
+	return_string_attrib("ellipse", u.image_ellipse());
+	return_string_attrib("halo", u.image_halo());
 	return_int_attrib("hitpoints", u.hitpoints());
 	return_int_attrib("max_hitpoints", u.max_hitpoints());
 	return_int_attrib("experience", u.experience());
@@ -294,7 +325,9 @@ static int impl_unit_get(lua_State *L)
 	return_int_attrib("max_attacks", u.max_attacks());
 	return_int_attrib("attacks_left", u.attacks_left());
 	return_tstring_attrib("name", u.name());
+	return_tstring_attrib("description", u.unit_description());
 	return_bool_attrib("canrecruit", u.can_recruit());
+	return_bool_attrib("renamable", !u.unrenamable());
 	return_int_attrib("level", u.level());
 	return_int_attrib("cost", u.cost());
 
@@ -310,10 +343,10 @@ static int impl_unit_get(lua_State *L)
 		unit::upkeep_t upkeep = u.upkeep_raw();
 
 		// Need to keep these separate in order to ensure an int value is always used if applicable.
-		if(int* v = boost::get<int>(&upkeep)) {
+		if(int* v = utils::get_if<int>(&upkeep)) {
 			lua_push(L, *v);
 		} else {
-			const std::string type = boost::apply_visitor(unit::upkeep_type_visitor(), upkeep);
+			const std::string type = utils::visit(unit::upkeep_type_visitor{}, upkeep);
 			lua_push(L, type);
 		}
 
@@ -353,21 +386,28 @@ static int impl_unit_get(lua_State *L)
 		push_unit_attacks_table(L, 1);
 		return 1;
 	}
+	if(strcmp(m, "petrified") == 0) {
+		deprecated_message("(unit).petrified", DEP_LEVEL::INDEFINITE, {1,17,0}, "use (unit).status.petrified instead");
+		lua_pushboolean(L, u.incapacitated());
+		return 1;
+	}
 	return_vector_string_attrib("animations", u.anim_comp().get_flags());
 	return_cfg_attrib("recall_filter", cfg = u.recall_filter());
 	return_bool_attrib("hidden", u.get_hidden());
-	return_bool_attrib("petrified", u.incapacitated());
 	return_bool_attrib("resting", u.resting());
 	return_string_attrib("role", u.get_role());
 	return_string_attrib("race", u.race()->id());
 	return_string_attrib("gender", gender_string(u.gender()));
 	return_string_attrib("variation", u.variation());
+	return_string_attrib("undead_variation", u.undead_variation());
 	return_bool_attrib("zoc", u.get_emit_zoc());
 	return_string_attrib("facing", map_location::write_direction(u.facing()));
-	return_string_attrib("portrait", u.big_profile() == u.absolute_image() ? u.absolute_image() + u.image_mods() : u.big_profile());
+	return_string_attrib("portrait", u.big_profile() == u.absolute_image()
+		? u.absolute_image() + u.image_mods() + "~XBRZ(2)"
+		: u.big_profile());
 	return_cfg_attrib("__cfg", u.write(cfg); u.get_location().write(cfg));
 
-	if(luaW_getmetafield(L, 1, m)) {
+	if(luaW_getglobal(L, "wesnoth", "units", m)) {
 		return 1;
 	}
 	return 0;
@@ -391,23 +431,35 @@ static int impl_unit_set(lua_State *L)
 	//modify_int_attrib_check_range("side", u.set_side(value), 1, static_cast<int>(teams().size())); TODO: Figure out if this is a good idea, to refer to teams() and make this depend on having a gamestate
 	modify_int_attrib("side", u.set_side(value));
 	modify_int_attrib("moves", u.set_movement(value));
+	modify_int_attrib("max_moves", u.set_total_movement(value));
+	modify_int_attrib("max_attacks", u.set_max_attacks(value));
 	modify_int_attrib("hitpoints", u.set_hitpoints(value));
+	modify_int_attrib("max_hitpoints", u.set_max_hitpoints(value));
 	modify_int_attrib("experience", u.set_experience(value));
+	modify_int_attrib("max_experience", u.set_max_experience(value));
 	modify_int_attrib("recall_cost", u.set_recall_cost(value));
 	modify_int_attrib("attacks_left", u.set_attacks(value));
 	modify_int_attrib("level", u.set_level(value));
 	modify_bool_attrib("resting", u.set_resting(value));
 	modify_tstring_attrib("name", u.set_name(value));
+	modify_tstring_attrib("description", u.set_unit_description(value));
+	modify_string_attrib("portrait", u.set_big_profile(value));
 	modify_string_attrib("role", u.set_role(value));
 	modify_string_attrib("facing", u.set_facing(map_location::parse_direction(value)));
+	modify_string_attrib("usage", u.set_usage(value));
+	modify_string_attrib("undead_variation", u.set_undead_variation(value));
+	modify_string_attrib("ellipse", u.set_image_ellipse(value));
+	modify_string_attrib("halo", u.set_image_halo(value));
 	modify_bool_attrib("hidden", u.set_hidden(value));
 	modify_bool_attrib("zoc", u.set_emit_zoc(value));
 	modify_bool_attrib("canrecruit", u.set_can_recruit(value));
+	modify_bool_attrib("renamable", u.set_unrenamable(!value));
+	modify_cfg_attrib("recall_filter", u.set_recall_filter(cfg));
 
 	modify_vector_string_attrib("extra_recruit", u.set_recruits(value));
 	modify_vector_string_attrib("advances_to", u.set_advances_to(value));
 	if(strcmp(m, "alignment") == 0) {
-		u.set_alignment(lua_check<unit_type::ALIGNMENT>(L, 3));
+		u.set_alignment(lua_check<UNIT_ALIGNMENT>(L, 3));
 		return 0;
 	}
 
@@ -418,7 +470,7 @@ static int impl_unit_set(lua_State *L)
 
 	if(strcmp(m, "upkeep") == 0) {
 		if(lua_isnumber(L, 3)) {
-			u.set_upkeep(luaL_checkinteger(L, 3));
+			u.set_upkeep(static_cast<int>(luaL_checkinteger(L, 3)));
 			return 0;
 		}
 		const char* v = luaL_checkstring(L, 3);
@@ -439,6 +491,11 @@ static int impl_unit_set(lua_State *L)
 		modify_int_attrib("x", loc.set_wml_x(value); u.set_location(loc));
 		modify_int_attrib("y", loc.set_wml_y(value); u.set_location(loc));
 		modify_string_attrib("id", u.set_id(value));
+		if(strcmp(m, "loc") == 0) {
+			luaW_tolocation(L, 3, loc);
+			u.set_location(loc);
+			return 0;
+		}
 	} else {
 		const bool is_key_x = strcmp(m, "x") == 0;
 		const bool is_key_y = strcmp(m, "y") == 0;
@@ -471,10 +528,7 @@ static int impl_unit_set(lua_State *L)
 					return luaL_argerror(L, 2, err_msg.c_str());
 				}
 
-				unit_map::iterator unit_iterator = gb->units().end();
-				bool success = false;
-
-				std::tie(unit_iterator, success) = gb->units().move(src, dst);
+				auto [unit_iterator, success] = gb->units().move(src, dst);
 
 				if(success) {
 					unit_iterator->anim_comp().set_standing();
@@ -485,6 +539,11 @@ static int impl_unit_set(lua_State *L)
 		}
 	}
 
+	if(strcmp(m, "goto") == 0) {
+		u.set_goto(luaW_checklocation(L, 3));
+		return 0;
+	}
+
 	std::string err_msg = "unknown modifiable property of unit: ";
 	err_msg += m;
 	return luaL_argerror(L, 2, err_msg.c_str());
@@ -572,10 +631,7 @@ static int impl_unit_variables_set(lua_State *L)
 		return luaL_argerror(L, 2, "unknown unit");
 	}
 	char const *m = luaL_checkstring(L, 2);
-	if(strcmp(m, "__cfg") == 0) {
-		u->variables() = luaW_checkconfig(L, 3);
-		return 0;
-	}
+	modify_cfg_attrib("__cfg", u->variables() = cfg);
 	config& vars = u->variables();
 	if(lua_isnoneornil(L, 3)) {
 		try {
@@ -602,47 +658,14 @@ namespace lua_units {
 		lua_setfield(L, -2, "__gc");
 		lua_pushcfunction(L, impl_unit_equality);
 		lua_setfield(L, -2, "__eq");
+		lua_pushcfunction(L, impl_unit_tostring);
+		lua_setfield(L, -2, "__tostring");
 		lua_pushcfunction(L, impl_unit_get);
 		lua_setfield(L, -2, "__index");
 		lua_pushcfunction(L, impl_unit_set);
 		lua_setfield(L, -2, "__newindex");
 		lua_pushstring(L, "unit");
 		lua_setfield(L, -2, "__metatable");
-		// Unit methods
-		luaW_getglobal(L, "wesnoth", "match_unit");
-		lua_setfield(L, -2, "matches");
-		luaW_getglobal(L, "wesnoth", "put_recall_unit");
-		lua_setfield(L, -2, "to_recall");
-		luaW_getglobal(L, "wesnoth", "put_unit");
-		lua_setfield(L, -2, "to_map");
-		luaW_getglobal(L, "wesnoth", "erase_unit");
-		lua_setfield(L, -2, "erase");
-		luaW_getglobal(L, "wesnoth", "copy_unit");
-		lua_setfield(L, -2, "clone");
-		luaW_getglobal(L, "wesnoth", "extract_unit");
-		lua_setfield(L, -2, "extract");
-		luaW_getglobal(L, "wesnoth", "advance_unit");
-		lua_setfield(L, -2, "advance");
-		luaW_getglobal(L, "wesnoth", "add_modification");
-		lua_setfield(L, -2, "add_modification");
-		luaW_getglobal(L, "wesnoth", "unit_resistance");
-		lua_setfield(L, -2, "resistance");
-		luaW_getglobal(L, "wesnoth", "remove_modifications");
-		lua_setfield(L, -2, "remove_modifications");
-		luaW_getglobal(L, "wesnoth", "unit_defense");
-		lua_setfield(L, -2, "defense");
-		luaW_getglobal(L, "wesnoth", "unit_movement_cost");
-		lua_setfield(L, -2, "movement");
-		luaW_getglobal(L, "wesnoth", "unit_vision_cost");
-		lua_setfield(L, -2, "vision");
-		luaW_getglobal(L, "wesnoth", "unit_jamming_cost");
-		lua_setfield(L, -2, "jamming");
-		luaW_getglobal(L, "wesnoth", "unit_ability");
-		lua_setfield(L, -2, "ability");
-		luaW_getglobal(L, "wesnoth", "transform_unit");
-		lua_setfield(L, -2, "transform");
-		luaW_getglobal(L, "wesnoth", "select_unit");
-		lua_setfield(L, -2, "select");
 
 		// Create the unit status metatable.
 		cmd_out << "Adding unit status metatable...\n";
