@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "scripting/lua_unit_attacks.hpp"
@@ -17,10 +18,11 @@
 #include "scripting/lua_common.hpp"
 #include "scripting/lua_unit.hpp"
 #include "scripting/lua_unit_type.hpp"
+#include "scripting/push_check.hpp"
 #include "units/unit.hpp"
+#include "units/types.hpp"
 #include "units/attack_type.hpp"
 #include "utils/const_clone.hpp"
-#include "utils/type_trait_aliases.hpp"
 
 #include "lua/lauxlib.h"
 #include "lua/lua.h"                    // for lua_State, lua_settop, etc
@@ -90,7 +92,7 @@ attack_type& luaW_checkweapon(lua_State* L, int idx)
 }
 
 template<typename T>
-using attack_ptr_in = std::shared_ptr<utils::const_clone_t<attack_type, utils::remove_pointer_t<T>>>;
+using attack_ptr_in = std::shared_ptr<utils::const_clone_t<attack_type, std::remove_pointer_t<T>>>;
 
 // Note that these two templates are designed on the assumption that T is either unit or unit_type
 template<typename T>
@@ -106,10 +108,10 @@ auto find_attack(T* u, const std::string& id) -> attack_ptr_in<T>
 }
 
 template<typename T>
-auto find_attack(T* u, size_t i) -> attack_ptr_in<T>
+auto find_attack(T* u, std::size_t i) -> attack_ptr_in<T>
 {
 	auto attacks = u->attacks();
-	if(i < static_cast<size_t>(attacks.size())) {
+	if(i < static_cast<std::size_t>(attacks.size())) {
 		auto iter = attacks.begin();
 		iter += i;
 		return *iter.base();
@@ -315,6 +317,18 @@ static int impl_unit_attack_equal(lua_State* L)
 	return 1;
 }
 
+/**
+ * Turns a lua proxy attack to string. (__tostring metamethod)
+ */
+static int impl_unit_attack_tostring(lua_State* L)
+{
+	const_attack_ptr atk = luaW_checkweapon_ref(L, 1).cattack;
+	std::ostringstream str;
+	str << "weapon: <" << atk->id() << '>';
+	lua_push(L, str.str());
+	return 1;
+}
+
 static int impl_unit_attack_match(lua_State* L)
 {
 	const_attack_ptr atk = luaW_toweapon(L, 1);
@@ -333,7 +347,7 @@ static int impl_unit_attack_collect(lua_State* L)
 	return 0;
 }
 
-static int intf_create_attack(lua_State* L)
+int intf_create_attack(lua_State* L)
 {
 	auto atk = std::make_shared<attack_type>(luaW_checkconfig(L, 1));
 	luaW_pushweapon(L, atk);
@@ -368,6 +382,8 @@ namespace lua_units {
 		lua_setfield(L, -2, "__newindex");
 		lua_pushcfunction(L, impl_unit_attack_equal);
 		lua_setfield(L, -2, "__eq");
+		lua_pushcfunction(L, impl_unit_attack_tostring);
+		lua_setfield(L, -2, "__tostring");
 		lua_pushcfunction(L, impl_unit_attack_collect);
 		lua_setfield(L, -2, "__gc");
 		lua_pushstring(L, uattackKey);
@@ -375,12 +391,6 @@ namespace lua_units {
 		lua_pushcfunction(L, impl_unit_attack_match);
 		lua_setfield(L, -2, "matches");
 
-		// Add create_attack
-		luaW_getglobal(L, "wesnoth");
-		lua_pushcfunction(L, intf_create_attack);
-		lua_setfield(L, -2, "create_weapon");
-		lua_pop(L, 1);
-
 		return cmd_out.str();
 	}
 }
