@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2014 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -19,7 +20,6 @@
  * Contains the SDL_Rect helper code.
  */
 
-#include "global.hpp"
 #include "utils.hpp"
 
 #include <SDL2/SDL_rect.h>
@@ -29,7 +29,7 @@ struct point;
 namespace sdl
 {
 
-CONSTEXPR const SDL_Rect empty_rect { 0, 0, 0, 0 };
+constexpr const SDL_Rect empty_rect { 0, 0, 0, 0 };
 
 /**
  * Creates an SDL_Rect with the given dimensions.
