@@ -1,23 +1,25 @@
 /*
-   Copyright (C) 2003 - 2018 the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
-#include "global.hpp"
 #include "utils/const_clone.hpp"
 
 #include <SDL2/SDL.h>
 
+#include <ostream>
+
 class CVideo;
 
 class surface
@@ -36,7 +38,7 @@ public:
 		add_surface_ref(surface_);
 	}
 
-	surface(surface&& s) NOEXCEPT : surface_(s.get())
+	surface(surface&& s) noexcept : surface_(s.get())
 	{
 		s.surface_ = nullptr;
 	}
@@ -52,7 +54,7 @@ public:
 		return *this;
 	}
 
-	surface& operator=(surface&& s) NOEXCEPT
+	surface& operator=(surface&& s) noexcept
 	{
 		free_surface();
 		surface_ = s.surface_;
@@ -60,9 +62,6 @@ public:
 		return *this;
 	}
 
-	// Intended to be used when SDL has already freed the surface
-	void clear_without_free() { surface_ = nullptr; }
-
 	/**
 	 * Check that the surface is neutral bpp 32.
 	 *
@@ -113,6 +112,8 @@ private:
 
 bool operator<(const surface& a, const surface& b);
 
+std::ostream& operator<<(std::ostream& stream, const surface& surf);
+
 struct surface_restorer
 {
 	surface_restorer();
