@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -119,7 +120,7 @@ surface stretch_surface_vertical(
 	return dst;
 }
 
-surface scale_surface_xbrz(const surface & surf, size_t z)
+surface scale_surface_xbrz(const surface & surf, std::size_t z)
 {
 	if(surf == nullptr)
 		return nullptr;
@@ -1667,10 +1668,10 @@ surface cut_surface(const surface &surf, const SDL_Rect& r)
 		return nullptr;
 	}
 
-	size_t sbpp = surf->format->BytesPerPixel;
-	size_t spitch = surf->pitch;
-	size_t rbpp = res->format->BytesPerPixel;
-	size_t rpitch = res->pitch;
+	std::size_t sbpp = surf->format->BytesPerPixel;
+	std::size_t spitch = surf->pitch;
+	std::size_t rbpp = res->format->BytesPerPixel;
+	std::size_t rpitch = res->pitch;
 
 	// compute the areas to copy
 	SDL_Rect src_rect = r;
@@ -1705,7 +1706,7 @@ surface cut_surface(const surface &surf, const SDL_Rect& r)
 	for(int y = 0; y < src_rect.h && (src_rect.y + y) < surf->h; ++y) {
 		const uint8_t* line_src  = src  + (src_rect.y + y) * spitch + src_rect.x * sbpp;
 		uint8_t* line_dest = dest + (dst_rect.y + y) * rpitch + dst_rect.x * rbpp;
-		size_t size = src_rect.w + src_rect.x <= surf->w ? src_rect.w : surf->w - src_rect.x;
+		std::size_t size = src_rect.w + src_rect.x <= surf->w ? src_rect.w : surf->w - src_rect.x;
 
 		assert(rpitch >= src_rect.w * rbpp);
 		memcpy(line_dest, line_src, size * rbpp);
@@ -1768,8 +1769,8 @@ surface rotate_any_surface(const surface& surf, float angle, int zoom, int offse
 		float max_x, max_y;
 		// convert angle to radiant (angle * 2 * PI) / 360
 		const float radians = angle * boost::math::constants::pi<float>() / 180;
-		cosine = cos(radians);
-		sine   = sin(radians);
+		cosine = std::cos(radians);
+		sine   = std::sin(radians);
 		// calculate the size of the dst image
 		src_w = surf->w * zoom;
 		src_h = surf->h * zoom;
@@ -1830,15 +1831,15 @@ void put_pixel(const surface& surf, surface_lock& surf_lock, int x, int y, uint3
 		*reinterpret_cast<uint16_t*>(dst) = pixel;
 		break;
 	case 3:
-		if (SDL_BYTEORDER == SDL_BIG_ENDIAN) {
-			dst[0] = (pixel >> 16) & 0xff;
-			dst[1] = (pixel >> 8) & 0xff;
-			dst[2] = pixel & 0xff;
-		} else {
-			dst[0] = pixel & 0xff;
-			dst[1] = (pixel >> 8) & 0xff;
-			dst[2] = (pixel >> 16) & 0xff;
-		}
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+		dst[0] = (pixel >> 16) & 0xff;
+		dst[1] = (pixel >> 8) & 0xff;
+		dst[2] = pixel & 0xff;
+#else
+		dst[0] = pixel & 0xff;
+		dst[1] = (pixel >> 8) & 0xff;
+		dst[2] = (pixel >> 16) & 0xff;
+#endif
 		break;
 	case 4:
 		*reinterpret_cast<uint32_t*>(dst) = pixel;
@@ -1859,11 +1860,11 @@ uint32_t get_pixel(const surface& surf, const const_surface_lock& surf_lock, int
 	case 2:
 		return *reinterpret_cast<const uint16_t*>(src);
 	case 3:
-		if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
-			return src[0] << 16 | src[1] << 8 | src[2];
-		else
-			return src[0] | src[1] << 8 | src[2] << 16;
-		break;
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+		return src[0] << 16 | src[1] << 8 | src[2];
+#else
+		return src[0] | src[1] << 8 | src[2] << 16;
+#endif
 	case 4:
 		return *reinterpret_cast<const uint32_t*>(src);
 	}
