@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #pragma once
 
 #include "color_range.hpp"
@@ -22,7 +21,7 @@
 #include "utils/math.hpp"
 #include "game_version.hpp"
 
-#include <SDL.h>
+#include <SDL2/SDL.h>
 
 #include <cstdlib>
 #include <map>
@@ -41,20 +40,6 @@ inline void sdl_copy_portion(const surface& screen, SDL_Rect* screen_rect, surfa
 	SDL_SetSurfaceBlendMode(screen, SDL_BLENDMODE_BLEND);
 }
 
-/**
- * Check that the surface is neutral bpp 32.
- *
- * The surface may have an empty alpha channel.
- *
- * @param surf                    The surface to test.
- *
- * @returns                       The status @c true if neutral, @c false if not.
- */
-bool is_neutral(const surface& surf);
-
-surface make_neutral_surface(const surface &surf);
-surface create_neutral_surface(int w, int h);
-
 /**
  * Stretches a surface in the horizontal direction.
  *
@@ -94,7 +79,7 @@ surface stretch_surface_vertical(
  *  @param z                 The scaling factor. Should be an integer 2-5 (1 is tolerated).
  *  @return		     The scaled surface
  */
-surface scale_surface_xbrz(const surface & surf, size_t z);
+surface scale_surface_xbrz(const surface & surf, std::size_t z);
 
 /** Scale a surface using the nearest neighbor algorithm (provided by xBRZ lib)
  *  @param surf		     The sources surface
@@ -300,7 +285,6 @@ surface rotate_90_surface(const surface &surf, bool clockwise);
 
 surface flip_surface(const surface &surf);
 surface flop_surface(const surface &surf);
-surface create_compatible_surface(const surface &surf, int width = -1, int height = -1);
 
 /**
  * Replacement for sdl_blit.
@@ -333,6 +317,7 @@ SDL_Rect get_non_transparent_portion(const surface &surf);
  * @param surf_lock      The locked surface to make sure the pointers are valid.
  * @param x              The position in the row of the pixel.
  * @param y              The row of the pixel.
+ * @param pixel          The pixel value.
  */
 void put_pixel(const surface& surf, surface_lock& surf_lock, int x, int y, uint32_t pixel);
 uint32_t get_pixel(const surface& surf, const const_surface_lock& surf_lock, int x, int y);
