@@ -1,20 +1,22 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "serialization/base64.hpp"
 
 #include <vector>
+#include <string>
 
 namespace {
 const std::string base64_itoa_map = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -48,7 +50,7 @@ char itoa(unsigned value, const std::string& map)
 	return map[value & 0x3f];
 }
 
-std::vector<uint8_t> generic_decode_be(utils::string_view in, const std::vector<int>& atoi_map)
+std::vector<uint8_t> generic_decode_be(std::string_view in, const std::vector<int>& atoi_map)
 {
 	const int last_char = in.find_last_not_of("=");
 	const int num_chars = last_char + 1;
@@ -82,7 +84,7 @@ std::vector<uint8_t> generic_decode_be(utils::string_view in, const std::vector<
 	return out;
 }
 
-std::vector<uint8_t> generic_decode_le(utils::string_view in, const std::vector<int>& atoi_map)
+std::vector<uint8_t> generic_decode_le(std::string_view in, const std::vector<int>& atoi_map)
 {
 	const int last_char = in.find_last_not_of("=");
 	const int length = last_char * 6 / 8;
@@ -211,7 +213,7 @@ std::string generic_encode_le(utils::byte_string_view in, const std::string& ito
 }
 
 namespace base64 {
-std::vector<uint8_t> decode(utils::string_view in)
+std::vector<uint8_t> decode(std::string_view in)
 {
 	return generic_decode_be(in, base64_atoi_map());
 }
@@ -221,7 +223,7 @@ std::string encode(utils::byte_string_view bytes)
 }
 }
 namespace crypt64{
-std::vector<uint8_t> decode(utils::string_view in)
+std::vector<uint8_t> decode(std::string_view in)
 {
 	return generic_decode_le(in, crypt64_atoi_map());
 }
@@ -231,7 +233,7 @@ std::string encode(utils::byte_string_view bytes)
 }
 int decode(char encoded_char)
 {
-	size_t pos = crypt64_itoa_map.find(encoded_char);
+	std::size_t pos = crypt64_itoa_map.find(encoded_char);
 	return pos == std::string::npos ? -1 : pos;
 }
 char encode(int value)
