@@ -1,17 +1,18 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 - 2018 by Philippe Plantier <ayin@anathas.org>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2005 - 2021
+	by Philippe Plantier <ayin@anathas.org>
+	Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -46,8 +47,6 @@
 #pragma warning(pop)
 #endif
 
-#include <boost/variant/static_visitor.hpp>
-
 #include <stack>
 
 static lg::log_domain log_config("config");
@@ -55,7 +54,7 @@ static lg::log_domain log_config("config");
 #define WRN_CF LOG_STREAM(warn, log_config)
 #define LOG_CF LOG_STREAM(info, log_config)
 
-static const size_t max_recursion_levels = 1000;
+static const std::size_t max_recursion_levels = 1000;
 
 namespace
 {
@@ -125,6 +124,10 @@ void parser::operator()()
 	cfg_.clear();
 	elements.emplace(&cfg_, "");
 
+	if(validator_) {
+		validator_->open_tag("", cfg_, tok_.get_start_line(), tok_.get_file());
+	}
+
 	do {
 		tok_.next_token();
 
@@ -163,6 +166,12 @@ void parser::operator()()
 	// The main element should be there. If it is not, this is a parser error.
 	assert(!elements.empty());
 
+	if(validator_) {
+		element& el = elements.top();
+		validator_->validate(*el.cfg, el.name, el.start_line, el.file);
+		validator_->close_tag();
+	}
+
 	if(elements.size() != 1) {
 		utils::string_map i18n_symbols;
 		i18n_symbols["tag"] = elements.top().name;
@@ -184,6 +193,7 @@ void parser::parse_element()
 
 	std::string elname;
 	config* current_element = nullptr;
+	config* parent = nullptr;
 
 	switch(tok_.current_token().type) {
 	case token::STRING: // [element]
@@ -194,11 +204,12 @@ void parser::parse_element()
 		}
 
 		// Add the element
-		current_element = &(elements.top().cfg->add_child(elname));
+		parent = elements.top().cfg;
+		current_element = &(parent->add_child(elname));
 		elements.emplace(current_element, elname, tok_.get_start_line(), tok_.get_file());
 
 		if(validator_) {
-			validator_->open_tag(elname, tok_.get_start_line(), tok_.get_file());
+			validator_->open_tag(elname, *parent, tok_.get_start_line(), tok_.get_file());
 		}
 
 		break;
@@ -215,17 +226,18 @@ void parser::parse_element()
 		}
 
 		// Find the last child of the current element whose name is element
-		if(config& c = elements.top().cfg->child(elname, -1)) {
+		parent = elements.top().cfg;
+		if(config& c = parent->child(elname, -1)) {
 			current_element = &c;
 
 			if(validator_) {
-				validator_->open_tag(elname, tok_.get_start_line(), tok_.get_file(), true);
+				validator_->open_tag(elname, *parent, tok_.get_start_line(), tok_.get_file(), true);
 			}
 		} else {
-			current_element = &elements.top().cfg->add_child(elname);
+			current_element = &parent->add_child(elname);
 
 			if(validator_) {
-				validator_->open_tag(elname, tok_.get_start_line(), tok_.get_file());
+				validator_->open_tag(elname, *parent, tok_.get_start_line(), tok_.get_file());
 			}
 		}
 
@@ -319,7 +331,7 @@ void parser::parse_variable()
 
 	bool ignore_next_newlines = false, previous_string = false;
 
-	while(1) {
+	while(true) {
 		tok_.next_token();
 		assert(curvar != variables.end());
 
@@ -333,7 +345,7 @@ void parser::parse_variable()
 				}
 
 				if(validator_) {
-					validator_->validate_key(cfg, *curvar, buffer.value(), tok_.get_start_line(), tok_.get_file());
+					validator_->validate_key(cfg, *curvar, cfg[*curvar], tok_.get_start_line(), tok_.get_file());
 				}
 
 				buffer = t_string_base();
@@ -378,7 +390,7 @@ void parser::parse_variable()
 				buffer += " ";
 			}
 
-			FALLTHROUGH;
+			[[fallthrough]];
 
 		default:
 			buffer += tok_.current_token().value;
@@ -397,7 +409,7 @@ void parser::parse_variable()
 				continue;
 			}
 
-			FALLTHROUGH;
+			[[fallthrough]];
 
 		case token::END:
 			goto finish;
@@ -416,7 +428,7 @@ finish:
 	}
 
 	if(validator_) {
-		validator_->validate_key(cfg, *curvar, buffer.value(), tok_.get_start_line(), tok_.get_file());
+		validator_->validate_key(cfg, *curvar, cfg[*curvar], tok_.get_start_line(), tok_.get_file());
 	}
 
 	while(++curvar != variables.end()) {
@@ -511,7 +523,10 @@ inline std::string escaped_string(const std::string& value)
 	return escaped_string(value.begin(), value.end());
 }
 
-class write_key_val_visitor : public boost::static_visitor<void>
+class write_key_val_visitor
+#ifdef USING_BOOST_VARIANT
+	: public boost::static_visitor<void>
+#endif
 {
 public:
 	write_key_val_visitor(std::ostream& out, unsigned level, std::string& textdomain, const std::string& key)
@@ -534,7 +549,7 @@ public:
 	// Specialized visitors for things that go in quotes:
 	//
 
-	void operator()(const boost::blank&) const
+	void operator()(const utils::monostate&) const
 	{
 		// Treat blank values as nonexistent which fits better than treating them as empty strings.
 	}
@@ -695,7 +710,7 @@ void write_close_child(std::ostream& out, const std::string& child, unsigned int
 	out << std::string(level, '\t') << "[/" << child << "]\n";
 }
 
-static void write_internal(const config& cfg, std::ostream& out, std::string& textdomain, size_t tab = 0)
+static void write_internal(const config& cfg, std::ostream& out, std::string& textdomain, std::size_t tab = 0)
 {
 	if(tab > max_recursion_levels) {
 		throw config::error("Too many recursion levels in config write");
@@ -722,7 +737,7 @@ static void write_internal(const config& cfg, std::ostream& out, std::string& te
 	}
 }
 
-static void write_internal(const configr_of& cfg, std::ostream& out, std::string& textdomain, size_t tab = 0)
+static void write_internal(const configr_of& cfg, std::ostream& out, std::string& textdomain, std::size_t tab = 0)
 {
 	if(tab > max_recursion_levels) {
 		throw config::error("Too many recursion levels in config write");
