@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2018 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2005 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -23,7 +24,6 @@
 #include "buffered_istream.hpp"
 #include "config.hpp"
 #include "filesystem.hpp"
-#include "game_config.hpp"
 #include "log.hpp"
 #include "serialization/binary_or_text.hpp"
 #include "serialization/parser.hpp"
@@ -41,8 +41,10 @@ static lg::log_domain log_preprocessor("preprocessor");
 #define LOG_PREPROC LOG_STREAM(info, log_preprocessor)
 #define DBG_PREPROC LOG_STREAM(debug, log_preprocessor)
 
-static std::string current_file_str = "CURRENT_FILE";
-static std::string current_dir_str = "CURRENT_DIRECTORY";
+static const std::string current_file_str = "CURRENT_FILE";
+static const std::string current_dir_str = "CURRENT_DIRECTORY";
+static const std::string left_curly_str = "LEFT_BRACE";
+static const std::string right_curly_str = "RIGHT_BRACE";
 
 // map associating each filename encountered to a number
 static std::map<std::string, int> file_number_map;
@@ -159,6 +161,17 @@ void preproc_define::write_argument(config_writer& writer, const std::string& ar
 	writer.close_child(key);
 }
 
+void preproc_define::write_argument(config_writer& writer, const std::string& arg, const std::string& default_value) const
+{
+	const std::string key = "argument";
+
+	writer.open_child(key);
+
+	writer.write_key_val("name", arg);
+	writer.write_key_val("default", default_value);
+	writer.close_child(key);
+}
+
 void preproc_define::write(config_writer& writer, const std::string& name) const
 {
 	const std::string key = "preproc_define";
@@ -170,16 +183,32 @@ void preproc_define::write(config_writer& writer, const std::string& name) const
 	writer.write_key_val("linenum", std::to_string(linenum));
 	writer.write_key_val("location", get_location(location));
 
+	if(is_deprecated()) {
+		writer.open_child("deprecated");
+		writer.write_key_val("level", int(*deprecation_level));
+		writer.write_key_val("version", deprecation_version.str());
+		writer.write_key_val("message", deprecation_message);
+		writer.close_child("deprecated");
+	}
+
 	for(const std::string& arg : arguments) {
 		write_argument(writer, arg);
 	}
 
+	for(const auto& [key, default_value] : optional_arguments) {
+		write_argument(writer, key, default_value);
+	}
+
 	writer.close_child(key);
 }
 
 void preproc_define::read_argument(const config& cfg)
 {
-	arguments.push_back(cfg["name"]);
+	if(cfg.has_attribute("default")) {
+		optional_arguments.emplace(cfg["name"], cfg["default"]);
+	} else {
+		arguments.push_back(cfg["name"]);
+	}
 }
 
 void preproc_define::read(const config& cfg)
@@ -189,6 +218,12 @@ void preproc_define::read(const config& cfg)
 	linenum = cfg["linenum"];
 	location = cfg["location"].str();
 
+	if(auto deprecated = cfg.optional_child("deprecated")) {
+		deprecation_level = DEP_LEVEL(deprecated.value()["level"].to_int());
+		deprecation_version = deprecated.value()["version"].str();
+		deprecation_message = deprecated.value()["message"].str();
+	}
+
 	for(const config& arg : cfg.child_range("argument")) {
 		read_argument(arg);
 	}
@@ -344,7 +379,7 @@ private:
 	/** Buffer read by the STL stream. */
 	std::string out_buffer_;
 
-	/** Buffer filled by the #current_ preprocessor. */
+	/** Buffer filled by the _current_ preprocessor. */
 	std::stringstream buffer_;
 
 	/** Input preprocessor queue. */
@@ -381,7 +416,7 @@ preprocessor::preprocessor(preprocessor_streambuf& t)
 
 /**
  * Called by an STL stream whenever it has reached the end of #out_buffer_.
- * Fills #buffer_ by calling the #current_ preprocessor, then copies its
+ * Fills #buffer_ by calling the _current_ preprocessor, then copies its
  * content into #out_buffer_.
  * @return the first character of #out_buffer_ if any, EOF otherwise.
  */
@@ -566,7 +601,7 @@ class preprocessor_file : public preprocessor
 {
 public:
 	/** Constructor. It relies on preprocessor_data so it's implemented after that class is declared. */
-	preprocessor_file(preprocessor_streambuf& t, const std::string& name, size_t symbol_index = -1);
+	preprocessor_file(preprocessor_streambuf& t, const std::string& name, std::size_t symbol_index = -1);
 
 	virtual void init() override;
 
@@ -739,7 +774,7 @@ bool operator!=(char lhs, preprocessor_data::token_desc::TOKEN_TYPE rhs)
 }
 
 /** preprocessor_file constructor. */
-preprocessor_file::preprocessor_file(preprocessor_streambuf& t, const std::string& name, size_t symbol_index)
+preprocessor_file::preprocessor_file(preprocessor_streambuf& t, const std::string& name, std::size_t symbol_index)
 	: preprocessor(t)
 	, files_()
 	, pos_()
@@ -749,13 +784,13 @@ preprocessor_file::preprocessor_file(preprocessor_streambuf& t, const std::strin
 {
 	if(is_directory_) {
 		filesystem::get_files_in_dir(name, &files_, nullptr,
-			filesystem::ENTIRE_FILE_PATH,
-			filesystem::SKIP_MEDIA_DIR,
-			filesystem::DO_REORDER
+			filesystem::name_mode::ENTIRE_FILE_PATH,
+			filesystem::filter_mode::SKIP_MEDIA_DIR,
+			filesystem::reorder_mode::DO_REORDER
 		);
 
 		for(const std::string& fname : files_) {
-			size_t cpos = fname.rfind(" ");
+			std::size_t cpos = fname.rfind(" ");
 
 			if(cpos != std::string::npos && cpos >= symbol_index) {
 				std::stringstream ss;
@@ -916,7 +951,7 @@ void preprocessor_data::pop_token()
 
 void preprocessor_data::skip_spaces()
 {
-	for(;;) {
+	while(true) {
 		int c = in_.peek();
 
 		if(in_.eof() || (c != ' ' && c != '\t')) {
@@ -929,7 +964,7 @@ void preprocessor_data::skip_spaces()
 
 void preprocessor_data::skip_eol()
 {
-	for(;;) {
+	while(true) {
 		int c = in_.get();
 
 		if(c == '\n') {
@@ -947,7 +982,7 @@ std::string preprocessor_data::read_word()
 {
 	std::string res;
 
-	for(;;) {
+	while(true) {
 		int c = in_.peek();
 
 		if(c == preprocessor_streambuf::traits_type::eof() || utils::portable_isspace(c)) {
@@ -964,7 +999,7 @@ std::string preprocessor_data::read_line()
 {
 	std::string res;
 
-	for(;;) {
+	while(true) {
 		int c = in_.get();
 
 		if(c == '\n') {
@@ -1097,7 +1132,7 @@ bool preprocessor_data::get_chunk()
 	if(c == OUTPUT_SEPARATOR) {
 		std::string buffer(1, c);
 
-		for(;;) {
+		while(true) {
 			char d = static_cast<char>(in_.get());
 
 			if(in_.eof() || d == '\n') {
@@ -1156,10 +1191,10 @@ bool preprocessor_data::get_chunk()
 			std::string symbol = items.front();
 			items.erase(items.begin());
 			int found_arg = 0, found_enddef = 0, found_deprecate = 0;
-			boost::optional<DEP_LEVEL> deprecation_level = boost::none;
+			std::optional<DEP_LEVEL> deprecation_level;
 			std::string buffer, deprecation_detail;
 			version_info deprecation_version = game_config::wesnoth_version;
-			for(;;) {
+			while(true) {
 				if(in_.eof())
 					break;
 				char d = static_cast<char>(in_.get());
@@ -1186,7 +1221,7 @@ bool preprocessor_data::get_chunk()
 							std::string argbuffer;
 
 							int found_endarg = 0;
-							for(;;) {
+							while(true) {
 								if(in_.eof()) {
 									break;
 								}
@@ -1285,33 +1320,24 @@ bool preprocessor_data::get_chunk()
 
 				LOG_PREPROC << "defining macro " << symbol << " (location " << get_location(parent_.location_) << ")\n";
 			}
-		} else if(command == "ifdef") {
+		} else if(command == "ifdef" || command == "ifndef") {
+			const bool negate = command[2] == 'n';
 			skip_spaces();
 			const std::string& symbol = read_word();
 			bool found = parent_.defines_->count(symbol) != 0;
 			DBG_PREPROC << "testing for macro " << symbol << ": " << (found ? "defined" : "not defined") << '\n';
-			conditional_skip(!found);
-		} else if(command == "ifndef") {
-			skip_spaces();
-			const std::string& symbol = read_word();
-			bool found = parent_.defines_->count(symbol) != 0;
-			DBG_PREPROC << "testing for macro " << symbol << ": " << (found ? "defined" : "not defined") << '\n';
-			conditional_skip(found);
-		} else if(command == "ifhave") {
+			conditional_skip(negate ? found : !found);
+		} else if(command == "ifhave" || command == "ifnhave") {
+			const bool negate = command[2] == 'n';
 			skip_spaces();
 			const std::string& symbol = read_word();
 			bool found = !filesystem::get_wml_location(symbol, directory_).empty();
 			DBG_PREPROC << "testing for file or directory " << symbol << ": " << (found ? "found" : "not found")
 						<< '\n';
-			conditional_skip(!found);
-		} else if(command == "ifnhave") {
-			skip_spaces();
-			const std::string& symbol = read_word();
-			bool found = !filesystem::get_wml_location(symbol, directory_).empty();
-			DBG_PREPROC << "testing for file or directory " << symbol << ": " << (found ? "found" : "not found")
-						<< '\n';
-			conditional_skip(found);
+			conditional_skip(negate ? found : !found);
 		} else if(command == "ifver" || command == "ifnver") {
+			const bool negate = command[2] == 'n';
+
 			skip_spaces();
 			const std::string& vsymstr = read_word();
 			skip_spaces();
@@ -1337,7 +1363,7 @@ bool preprocessor_data::get_chunk()
 				DBG_PREPROC << "testing version '" << version1.str() << "' against '" << version2.str() << "' ("
 							<< vopstr << "): " << (found ? "match" : "no match") << '\n';
 
-				conditional_skip(command == "ifver" ? !found : found);
+				conditional_skip(negate ? found : !found);
 			} else {
 				std::string err = "Undefined macro in #ifver/#ifnver first argument: '";
 				err += vsymstr;
@@ -1455,7 +1481,7 @@ bool preprocessor_data::get_chunk()
 			//	strings_.pop_back();
 			//}
 
-			if(strings_.size() <= static_cast<size_t>(token.stack_pos)) {
+			if(strings_.size() <= static_cast<std::size_t>(token.stack_pos)) {
 				parent_.error("No macro or file substitution target specified", linenum_);
 			}
 
@@ -1477,6 +1503,12 @@ bool preprocessor_data::get_chunk()
 			} else if(symbol == current_dir_str && strings_.size() - token.stack_pos == 1) {
 				pop_token();
 				put(filesystem::directory_name(parent_.get_current_file()));
+			} else if(symbol == left_curly_str && strings_.size() - token.stack_pos == 1) {
+				pop_token();
+				put("{");
+			} else if(symbol == right_curly_str && strings_.size() - token.stack_pos == 1) {
+				pop_token();
+				put("}");
 			} else if(local_defines_ && (arg = local_defines_->find(symbol)) != local_defines_->end()) {
 				if(strings_.size() - token.stack_pos != 1) {
 					std::ostringstream error;
@@ -1492,8 +1524,8 @@ bool preprocessor_data::get_chunk()
 				put(v.str());
 			} else if(parent_.depth() < 100 && (macro = parent_.defines_->find(symbol)) != parent_.defines_->end()) {
 				const preproc_define& val = macro->second;
-				size_t nb_arg = strings_.size() - token.stack_pos - 1;
-				size_t optional_arg_num = 0;
+				std::size_t nb_arg = strings_.size() - token.stack_pos - 1;
+				std::size_t optional_arg_num = 0;
 
 				std::unique_ptr<std::map<std::string, std::string>> defines{new std::map<std::string, std::string>};
 				const std::string& dir = filesystem::directory_name(val.location.substr(0, val.location.find(' ')));
@@ -1502,7 +1534,7 @@ bool preprocessor_data::get_chunk()
 					deprecated_message(symbol, *val.deprecation_level, val.deprecation_version, val.deprecation_message);
 				}
 
-				for(size_t i = 0; i < nb_arg; ++i) {
+				for(std::size_t i = 0; i < nb_arg; ++i) {
 					if(i < val.arguments.size()) {
 						// Normal mandatory arguments
 
@@ -1511,10 +1543,10 @@ bool preprocessor_data::get_chunk()
 						// These should be optional argument overrides
 
 						std::string str = strings_[token.stack_pos + i + 1];
-						size_t equals_pos = str.find_first_of("=");
+						std::size_t equals_pos = str.find_first_of("=");
 
 						if(equals_pos != std::string::npos) {
-							size_t argname_pos = str.substr(0, equals_pos).find_last_of(" \n") + 1;
+							std::size_t argname_pos = str.substr(0, equals_pos).find_last_of(" \n") + 1;
 
 							std::string argname = str.substr(argname_pos, equals_pos - argname_pos);
 
@@ -1567,8 +1599,9 @@ bool preprocessor_data::get_chunk()
 
 				if(nb_arg - optional_arg_num != val.arguments.size()) {
 					const std::vector<std::string>& locations = utils::quoted_split(val.location, ' ');
+					const std::string filename = locations.empty() ? "<command-line>" : get_filename(locations[0]);
 					std::ostringstream error;
-					error << "Preprocessor symbol '" << symbol << "' defined at " << get_filename(locations[0]) << ":"
+					error << "Preprocessor symbol '" << symbol << "' defined at " << filename << ":"
 						  << val.linenum << " expects " << val.arguments.size() << " arguments, but has "
 						  << nb_arg - optional_arg_num << " arguments";
 					parent_.error(error.str(), linenum_);
@@ -1723,8 +1756,8 @@ void preprocess_resource(const std::string& res_name,
 	if(filesystem::is_directory(res_name)) {
 		std::vector<std::string> dirs, files;
 
-		filesystem::get_files_in_dir(res_name, &files, &dirs, filesystem::ENTIRE_FILE_PATH, filesystem::SKIP_MEDIA_DIR,
-				filesystem::DO_REORDER);
+		filesystem::get_files_in_dir(res_name, &files, &dirs, filesystem::name_mode::ENTIRE_FILE_PATH, filesystem::filter_mode::SKIP_MEDIA_DIR,
+				filesystem::reorder_mode::DO_REORDER);
 
 		// Subdirectories
 		for(const std::string& dir : dirs) {
