@@ -1,17 +1,18 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 - 2018 by Philippe Plantier <ayin@anathas.org>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2005 - 2021
+	by Philippe Plantier <ayin@anathas.org>
+	Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -26,6 +27,7 @@
 #include "utils/general.hpp"
 #include <cassert>
 #include <array>
+#include <limits>
 #include <stdexcept>
 
 #include <boost/algorithm/string.hpp>
@@ -58,35 +60,41 @@ bool notspace(const char c)
 	return !portable_isspace(c);
 }
 
+void trim(std::string_view& s)
+{
+	s.remove_prefix(std::min(s.find_first_not_of(" \t\r\n"), s.size()));
+	if(s.empty()) {
+		return;
+	}
+	//find_last_not_of never returns npos because !s.empty()
+	size_t first_to_trim = s.find_last_not_of(" \t\r\n") + 1;
+	s = s.substr(0, first_to_trim);
+}
+
 /**
  * Splits a (comma-)separated string into a vector of pieces.
- * @param[in]  val    A (comma-)separated string.
- * @param[in]  c      The separator character (usually a comma).
+ * @param[in]  s      A (comma-)separated string.
+ * @param[in]  sep    The separator character (usually a comma).
  * @param[in]  flags  Flags controlling how the split is done.
  *                    This is a bit field with two settings (both on by default):
  *                    REMOVE_EMPTY causes empty pieces to be skipped/removed.
  *                    STRIP_SPACES causes the leading and trailing spaces of each piece to be ignored/stripped.
- *
- *                    Basic method taken from http://stackoverflow.com/a/236803
  */
-std::vector<std::string> split(const std::string& val, const char c, const int flags)
+std::vector<std::string> split(std::string_view s, const char sep, const int flags)
 {
 	std::vector<std::string> res;
+	split_foreach(s, sep, flags, [&](std::string_view item) {
+		res.emplace_back(item);
+	});
+	return res;
+}
 
-	std::stringstream ss;
-	ss.str(val);
-
-	std::string item;
-	while(std::getline(ss, item, c)) {
-		if(flags & STRIP_SPACES) {
-			boost::trim(item);
-		}
-
-		if(!(flags & REMOVE_EMPTY) || !item.empty()) {
-			res.push_back(std::move(item));
-		}
-	}
-
+std::set<std::string> split_set(std::string_view s, char sep, const int flags)
+{
+	std::set<std::string> res;
+	split_foreach(s, sep, flags, [&](std::string_view item) {
+		res.emplace(item);
+	});
 	return res;
 }
 
@@ -129,14 +137,14 @@ std::vector<std::string> square_parenthetical_split(const std::string& val,
 	while (true) {
 		if(i2 == val.end() || (!in_parenthesis && *i2 == separator)) {
 			//push back square contents
-			size_t size_square_exp = 0;
-			for (size_t i=0; i < square_left.size(); i++) {
+			std::size_t size_square_exp = 0;
+			for (std::size_t i=0; i < square_left.size(); i++) {
 				std::string tmp_val(square_left[i]+1,square_right[i]);
 				std::vector< std::string > tmp = split(tmp_val);
 				for(const std::string& piece : tmp) {
-					size_t found_tilde = piece.find_first_of('~');
+					std::size_t found_tilde = piece.find_first_of('~');
 					if (found_tilde == std::string::npos) {
-						size_t found_asterisk = piece.find_first_of('*');
+						std::size_t found_asterisk = piece.find_first_of('*');
 						if (found_asterisk == std::string::npos) {
 							std::string tmp2(piece);
 							boost::trim(tmp2);
@@ -155,7 +163,7 @@ std::vector<std::string> square_parenthetical_split(const std::string& val,
 						std::string s_begin = piece.substr(0,found_tilde);
 						boost::trim(s_begin);
 						int begin = std::stoi(s_begin);
-						size_t padding = 0, padding_end = 0;
+						std::size_t padding = 0, padding_end = 0;
 						while (padding<s_begin.size() && s_begin[padding]=='0') {
 							padding++;
 						}
@@ -175,7 +183,7 @@ std::vector<std::string> square_parenthetical_split(const std::string& val,
 						end+=increment; //include end in expansion
 						for (int k=begin; k!=end; k+=increment) {
 							std::string pb = std::to_string(k);
-							for (size_t p=pb.size(); p<=padding; p++)
+							for (std::size_t p=pb.size(); p<=padding; p++)
 								pb = std::string("0") + pb;
 							square_expansion.push_back(pb);
 						}
@@ -190,17 +198,17 @@ std::vector<std::string> square_parenthetical_split(const std::string& val,
 			}
 
 			//combine square contents and rest of string for comma zone block
-			size_t j = 0;
-			size_t j_max = 0;
+			std::size_t j = 0;
+			std::size_t j_max = 0;
 			if (!square_left.empty())
 				j_max = square_expansion.size() / square_left.size();
 			do {
 				j1 = i1;
 				std::string new_val;
-				for (size_t i=0; i < square_left.size(); i++) {
+				for (std::size_t i=0; i < square_left.size(); i++) {
 					std::string tmp_val(j1, square_left[i]);
 					new_val.append(tmp_val);
-					size_t k = j+i*j_max;
+					std::size_t k = j+i*j_max;
 					if (k < square_expansion.size())
 						new_val.append(square_expansion[k]);
 					j1 = square_right[i]+1;
@@ -236,7 +244,7 @@ std::vector<std::string> square_parenthetical_split(const std::string& val,
 			continue;
 		}
 		bool found=false;
-		for(size_t i=0; i < lp.size(); i++) {
+		for(std::size_t i=0; i < lp.size(); i++) {
 			if (*i2 == lp[i]){
 				if (*i2 == '[')
 					square_left.push_back(i2);
@@ -273,7 +281,7 @@ std::map<std::string, std::string> map_split(
 	std::map< std::string, std::string > res;
 
 	for( std::vector< std::string >::iterator i = v.begin(); i != v.end(); ++i) {
-		size_t pos = i->find_first_of(minor);
+		std::size_t pos = i->find_first_of(minor);
 		std::string key, value;
 
 		if(pos == std::string::npos) {
@@ -346,7 +354,7 @@ std::vector<std::string> parenthetical_split(const std::string& val,
 			continue;
 		}
 		bool found=false;
-		for(size_t i=0; i < lp.size(); i++){
+		for(std::size_t i=0; i < lp.size(); i++){
 			if (*i2 == lp[i]){
 				if (!separator && part.empty()){
 					std::string new_val(i1, i2);
@@ -382,35 +390,6 @@ std::vector<std::string> parenthetical_split(const std::string& val,
 	return res;
 }
 
-std::pair<string_view, string_view> vertical_split(const std::string& val)
-{
-	// Count the number of lines.
-	int num_lines = std::count(val.begin(), val.end(), '\n') + 1;
-
-	if(num_lines < 2) {
-		throw std::logic_error("utils::vertical_split: the string contains only one line");
-	}
-
-	// Split the string at the point where we have encountered
-	// (number of lines / 2 - 1) line separators.
-	int split_point = 0;
-	int num_found_line_separators = 0;
-	for(size_t i = 0; i < val.size(); ++i) {
-		if(val[i] == '\n') {
-			++num_found_line_separators;
-			if(num_found_line_separators >= num_lines / 2 - 1) {
-				split_point = i;
-				break;
-			}
-		}
-	}
-
-	assert(split_point != 0);
-
-	return { string_view(val.data(), split_point),
-		string_view(&val[split_point + 1], val.size() - (split_point + 1)) };
-}
-
 // Modify a number by string representing integer difference, or optionally %
 int apply_modifier( const int number, const std::string &amount, const int minimum ) {
 	// wassert( amount.empty() == false );
@@ -524,9 +503,30 @@ std::string half_signed_value(int val)
 
 static void si_string_impl_stream_write(std::stringstream &ss, double input) {
 	std::streamsize oldprec = ss.precision();
+#ifdef _MSC_VER
+	// For MSVC, default mode misbehaves, so we use fixed instead.
 	ss.precision(1);
 	ss << std::fixed
 	   << input;
+#else
+	// In default mode, precision sets the number of significant figures.
+
+	// 999.5 and above will render as 1000+, however, only numbers above 1000 will use 4 digits
+	// Rounding everything from 100 up (at which point we stop using decimals anyway) avoids this.
+	if (input >= 100) {
+		input = std::round(input);
+	}
+
+	// When in binary mode, numbers of up to 1023.9999 can be passed
+	// We should render those with 4 digits, instead of as 1e+3.
+	// Input should be an integer number now, but doubles can do strange things, so check the halfway point instead.
+	if (input >= 999.5) {
+		ss.precision(4);
+	} else {
+		ss.precision(3);
+	}
+	ss << input;
+#endif
 	ss.precision(oldprec);
 }
 
@@ -535,6 +535,10 @@ std::string si_string(double input, bool base2, const std::string& unit) {
 
 	typedef std::array<std::string, 9> strings9;
 
+	if(input < 0){
+		return font::unicode_minus + si_string(std::abs(input), base2, unit);
+	}
+
 	strings9 prefixes;
 	strings9::const_iterator prefix;
 	if (input == 0.0) {
@@ -602,8 +606,8 @@ static bool is_wildcard_char(char c) {
 }
 
 bool isvalid_username(const std::string& username) {
-	const size_t alnum = std::count_if(username.begin(), username.end(), isalnum);
-	const size_t valid_char =
+	const std::size_t alnum = std::count_if(username.begin(), username.end(), isalnum);
+	const std::size_t valid_char =
 			std::count_if(username.begin(), username.end(), is_username_char);
 	if ((alnum + valid_char != username.size())
 			|| valid_char == username.size() || username.empty() )
@@ -614,10 +618,10 @@ bool isvalid_username(const std::string& username) {
 }
 
 bool isvalid_wildcard(const std::string& username) {
-    const size_t alnum = std::count_if(username.begin(), username.end(), isalnum);
-	const size_t valid_char =
+    const std::size_t alnum = std::count_if(username.begin(), username.end(), isalnum);
+	const std::size_t valid_char =
 			std::count_if(username.begin(), username.end(), is_username_char);
-    const size_t wild_char =
+    const std::size_t wild_char =
             std::count_if(username.begin(), username.end(), is_wildcard_char);
 	if ((alnum + valid_char + wild_char != username.size())
 			|| valid_char == username.size() || username.empty() )
@@ -630,7 +634,7 @@ bool isvalid_wildcard(const std::string& username) {
 
 bool word_completion(std::string& text, std::vector<std::string>& wordlist) {
 	std::vector<std::string> matches;
-	const size_t last_space = text.rfind(" ");
+	const std::size_t last_space = text.rfind(" ");
 	// If last character is a space return.
 	if (last_space == text.size() -1) {
 		wordlist = matches;
@@ -653,7 +657,7 @@ bool word_completion(std::string& text, std::vector<std::string>& wordlist) {
 	{
 		if (word->size() < semiword.size()
 		|| !std::equal(semiword.begin(), semiword.end(), word->begin(),
-				chars_equal_insensitive))
+				utils::chars_equal_insensitive))
 		{
 			continue;
 		}
@@ -680,10 +684,10 @@ static bool is_word_boundary(char c) {
 }
 
 bool word_match(const std::string& message, const std::string& word) {
-	size_t first = message.find(word);
+	std::size_t first = message.find(word);
 	if (first == std::string::npos) return false;
 	if (first == 0 || is_word_boundary(message[first - 1])) {
-		size_t next = first + word.size();
+		std::size_t next = first + word.size();
 		if (next == message.size() || is_word_boundary(message[next])) {
 			return true;
 		}
@@ -732,7 +736,21 @@ bool wildcard_string_match(const std::string& str, const std::string& match) {
 	return matches;
 }
 
-std::string indent(const std::string& string, size_t indent_size)
+void to_sql_wildcards(std::string& str, bool underscores)
+{
+	std::replace(str.begin(), str.end(), '*', '%');
+	if(underscores)
+	{
+		std::size_t n = 0;
+		while((n = str.find("_", n)) != std::string::npos)
+		{
+			str.replace(n, 1, "\\_");
+			n += 2;
+		}
+	}
+}
+
+std::string indent(const std::string& string, std::size_t indent_size)
 {
 	if(indent_size == 0) {
 		return string;
@@ -747,7 +765,7 @@ std::string indent(const std::string& string, size_t indent_size)
 	const std::vector<std::string>& lines = split(string, '\x0A', 0);
 	std::string res;
 
-	for(size_t lno = 0; lno < lines.size();) {
+	for(std::size_t lno = 0; lno < lines.size();) {
 		const std::string& line = lines[lno];
 
 		// Lines containing only a carriage return count as empty.
@@ -811,30 +829,35 @@ std::pair<int, int> parse_range(const std::string& str)
 	const std::string b = dash != str.end() ? std::string(dash + 1, str.end()) : a;
 	std::pair<int,int> res {0,0};
 	try {
-		res = std::make_pair(std::stoi(a), std::stoi(b));
+		if (b == "infinity") {
+			res = std::pair(std::stoi(a), std::numeric_limits<int>::max());
+		} else {
+			res = std::pair(std::stoi(a), std::stoi(b));
+		}
+
 		if (res.second < res.first) {
 			res.second = res.first;
 		}
-	} catch(const std::invalid_argument&) {}
+	} catch(const std::invalid_argument&) {
+	    ERR_GENERAL << "Invalid range: "<< str << std::endl;
+	}
 
 	return res;
 }
 
 std::vector<std::pair<int, int>> parse_ranges(const std::string& str)
 {
-	std::vector< std::pair< int, int >> to_return;
-	std::vector<std::string> strs = utils::split(str);
-	std::vector<std::string>::const_iterator i, i_end=strs.end();
-	for(i = strs.begin(); i != i_end; ++i) {
-		to_return.push_back(parse_range(*i));
+	std::vector<std::pair<int, int>> to_return;
+	for(const std::string& r : utils::split(str)) {
+		to_return.push_back(parse_range(r));
 	}
+
 	return to_return;
 }
 
-
-void ellipsis_truncate(std::string& str, const size_t size)
+void ellipsis_truncate(std::string& str, const std::size_t size)
 {
-	const size_t prev_size = str.length();
+	const std::size_t prev_size = str.length();
 
 	utf8::truncate(str, size);
 
