@@ -1,22 +1,23 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2018 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2005 - 2021
+	by Philippe Plantier <ayin@anathas.org>
+	Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
 #include "font/constants.hpp"
-#include "serialization/string_view.hpp"
 
 #include <algorithm>
 #include <map>
@@ -24,6 +25,7 @@
 #include <set>
 #include <sstream>
 #include <string>
+#include <string_view>
 #include <utility>
 #include <vector>
 
@@ -42,8 +44,44 @@ enum {
 	STRIP_SPACES = 0x02  /** STRIP_SPACES: strips leading and trailing blank spaces. */
 };
 
+void trim(std::string_view& s);
+
+template<typename F>
+void split_foreach_impl(std::string_view s, char sep, const F& f)
+{
+	if(s.empty()) {
+		return;
+	}
+	while(true)
+	{
+		int partend = s.find(sep);
+		if(partend == int(std::string_view::npos)) {
+			break;
+		}
+		f(s.substr(0, partend));
+		s.remove_prefix(partend + 1);
+	}
+	f(s);
+}
+
+template<typename F>
+void split_foreach(std::string_view s, char sep, const int flags, const F& f)
+{
+	split_foreach_impl(s, sep, [&](std::string_view item) {
+		if(flags & STRIP_SPACES) {
+			trim(item);
+		}
+		if(!(flags & REMOVE_EMPTY) || !item.empty()) {
+			f(item);
+		}
+	});
+}
+
+
+
 /** Splits a (comma-)separated string into a vector of pieces. */
-std::vector<std::string> split(const std::string& val, const char c = ',', const int flags = REMOVE_EMPTY | STRIP_SPACES);
+std::vector<std::string> split(std::string_view val, const char c = ',', const int flags = REMOVE_EMPTY | STRIP_SPACES);
+std::set<std::string> split_set(std::string_view val, const char c = ',', const int flags = REMOVE_EMPTY | STRIP_SPACES);
 
 /**
  * This function is identical to split(), except it does not split when it otherwise would if the
@@ -122,18 +160,21 @@ std::vector< std::string > parenthetical_split(
  *
  * Examples:
  *
- * INPUT:   ("a[1-3](1,[5,6,7]),b[8,9]", ",")
+ * INPUT:   ("a[1~3](1,[5,6,7]),b[8,9]", ",")
  * RETURNS: {"a1(1,5)", "a2(1,6)", "a3(1,7)", "b8", "b9"}
  *
- * INPUT:   ("abc[07-10]")
+ * INPUT:   ("abc[07~10]")
  * RETURNS: {"abc07", "abc08", "abc09", "abc10"}
  *
- * INPUT:   ("a[1,2]b[3-4]:c[5,6]")
+ * INPUT:   ("a[1,2]b[3~4]:c[5,6]")
  * RETURNS: {"a1b3:c5", "a2b4:c6"}
  *
- * INPUT:   ("abc[3,1].png")
+ * INPUT:   ("abc[3~1].png")
  * RETURNS: {"abc3.png", "abc2.png", "abc1.png"}
  *
+ * INPUT:   ("abc[3,1].png")
+ * RETURNS: {"abc3.png", "abc1.png"}
+ *
  * INPUT:   ("abc[de,xyz]")
  * RETURNS: {"abcde", "abcxyz"}
  *
@@ -147,19 +188,6 @@ std::vector<std::string> square_parenthetical_split(
 	const std::string& right = ")]",
 	const int flags = REMOVE_EMPTY | STRIP_SPACES);
 
-/**
- * Splits a string into two parts as evenly as possible based on lines.
- * For example, if the string contains 3288 lines, then both parts will
- * be 1644 lines long.
- *
- * The line separator in between won't be in either of the parts the
- * function returns.
- *
- * Because this function is intended for extremely long strings
- * (kilobytes long), it returns string_views for performance.
- */
-std::pair<string_view, string_view> vertical_split(const std::string& val);
-
 /**
  * Generates a new string joining container items in a list.
  *
@@ -210,7 +238,7 @@ std::string join_map(
  * @param bullet The leading bullet string.
  */
 template<typename T>
-std::string bullet_list(const T& v, size_t indent = 4, const std::string& bullet = font::unicode_bullet)
+std::string bullet_list(const T& v, std::size_t indent = 4, const std::string& bullet = font::unicode_bullet)
 {
 	std::ostringstream str;
 
@@ -235,7 +263,7 @@ std::string bullet_list(const T& v, size_t indent = 4, const std::string& bullet
  * @param string      Text to indent.
  * @param indent_size Number of indentation units to use.
  */
-std::string indent(const std::string& string, size_t indent_size = 4);
+std::string indent(const std::string& string, std::size_t indent_size = 4);
 
 std::pair<int, int> parse_range(const std::string& str);
 
@@ -326,6 +354,14 @@ bool word_match(const std::string& message, const std::string& word);
  */
 bool wildcard_string_match(const std::string& str, const std::string& match);
 
+/**
+ * Converts '*' to '%' and optionally escapes '_'.
+ *
+ * @param str The original string.
+ * @param underscores Whether to escape underscore characters as well.
+ */
+void to_sql_wildcards(std::string& str, bool underscores = true);
+
 /**
  * Check if the username contains only valid characters.
  *
@@ -344,6 +380,6 @@ bool isvalid_wildcard(const std::string& login);
 /**
  * Truncates a string to a given utf-8 character count and then appends an ellipsis.
  */
-void ellipsis_truncate(std::string& str, const size_t size);
+void ellipsis_truncate(std::string& str, const std::size_t size);
 
 } // end namespace utils
