@@ -1,14 +1,16 @@
 /*
-   Copyright (C) 2017-2018 by the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2017 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -18,8 +20,6 @@
 #include <cstddef>   //ptrdiff_t
 #include <cassert>   //assert
 
-#include "unicode_types.hpp"
-
 namespace ucs4
 {
 	template<typename string_type, typename update_implementation>
@@ -27,15 +27,15 @@ namespace ucs4
 	{
 	public:
 		typedef std::input_iterator_tag iterator_category;
-		typedef ucs4::char_t value_type;
+		typedef char32_t value_type;
 		typedef ptrdiff_t difference_type;
-		typedef ucs4::char_t* pointer;
-		typedef ucs4::char_t& reference;
+		typedef char32_t* pointer;
+		typedef char32_t& reference;
 
 		iterator_base(const string_type& str)
 			: current_char(0)
 			, string_end(str.end())
-			, current_substr(std::make_pair(str.begin(), str.begin()))
+			, current_substr(str.begin(), str.begin())
 		{
 			update();
 		}
@@ -43,7 +43,7 @@ namespace ucs4
 		iterator_base(typename string_type::const_iterator const& begin, typename string_type::const_iterator const& end)
 			: current_char(0)
 			, string_end(end)
-			, current_substr(std::make_pair(begin, begin))
+			, current_substr(begin, begin)
 		{
 			update();
 		}
@@ -75,7 +75,7 @@ namespace ucs4
 			return *this;
 		}
 
-		ucs4::char_t operator*() const
+		char32_t operator*() const
 		{
 			return current_char;
 		}
@@ -100,7 +100,7 @@ namespace ucs4
 			current_char = update_implementation::read(current_substr.second, string_end);
 		}
 
-		ucs4::char_t current_char;
+		char32_t current_char;
 		typename string_type::const_iterator string_end;
 		std::pair<typename string_type::const_iterator, typename string_type::const_iterator> current_substr;
 	};
