@@ -1,17 +1,18 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 - 2018 by Philippe Plantier <ayin@anathas.org>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by Philippe Plantier <ayin@anathas.org>
+	Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -48,18 +49,18 @@ static int byte_size_from_utf8_first(const unsigned char ch)
 	return count;
 }
 
-utf8::string lowercase(const utf8::string& s)
+std::string lowercase(const std::string& s)
 {
 	if(!s.empty()) {
 		utf8::iterator itor(s);
-		utf8::string res;
+		std::string res;
 
 		for(;itor != utf8::iterator::end(s); ++itor) {
-			ucs4::char_t uchar = *itor;
+			char32_t uchar = *itor;
 			// If wchar_t is less than 32 bits wide, we cannot apply towlower() to all codepoints
-			if(uchar <= static_cast<ucs4::char_t>(std::numeric_limits<wchar_t>::max()))
+			if(uchar <= static_cast<char32_t>(std::numeric_limits<wchar_t>::max()))
 				uchar = towlower(static_cast<wchar_t>(uchar));
-			res += unicode_cast<utf8::string>(uchar);
+			res += unicode_cast<std::string>(uchar);
 		}
 
 		res.append(itor.substr().second, s.end());
@@ -68,7 +69,7 @@ utf8::string lowercase(const utf8::string& s)
 	return s;
 }
 
-size_t index(const utf8::string& str, const size_t index)
+std::size_t index(const std::string& str, const std::size_t index)
 {
 	// chr counts characters, i is the codepoint index
 	// remark: several functions rely on the fallback to str.length()
@@ -83,7 +84,7 @@ size_t index(const utf8::string& str, const size_t index)
 	return i;
 }
 
-size_t size(const utf8::string& str)
+std::size_t size(const std::string& str)
 {
 	unsigned int chr, i = 0, len = str.size();
 	try {
@@ -96,12 +97,12 @@ size_t size(const utf8::string& str)
 	return chr;
 }
 
-utf8::string& insert(utf8::string& str, const size_t pos, const utf8::string& insert)
+std::string& insert(std::string& str, const std::size_t pos, const std::string& insert)
 {
 	return str.insert(index(str, pos), insert);
 }
 
-utf8::string& erase(utf8::string& str, const size_t start, const size_t len)
+std::string& erase(std::string& str, const std::size_t start, const std::size_t len)
 {
 	if (start > size(str)) return str;
 	unsigned pos = index(str, start);
@@ -114,17 +115,17 @@ utf8::string& erase(utf8::string& str, const size_t start, const size_t len)
 	}
 }
 
-utf8::string& truncate(utf8::string& str, const size_t size)
+std::string& truncate(std::string& str, const std::size_t size)
 {
 	return erase(str, size);
 }
 
-void truncate_as_ucs4(utf8::string &str, const size_t size)
+void truncate_as_ucs4(std::string &str, const std::size_t size)
 {
-	ucs4::string u4_str = unicode_cast<ucs4::string>(str);
+	std::u32string u4_str = unicode_cast<std::u32string>(str);
 	if(u4_str.size() > size) {
 		u4_str.resize(size);
-		str = unicode_cast<utf8::string>(u4_str);
+		str = unicode_cast<std::string>(u4_str);
 	}
 }
 
