@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -29,7 +30,7 @@ namespace ucs4_convert_impl
 		oitor_t& out_;
 		iteratorwriter(oitor_t& out) : out_(out) {}
 
-		bool can_push(size_t /*count*/)
+		bool can_push(std::size_t /*count*/)
 		{
 			return true;
 		}
@@ -90,11 +91,11 @@ typename ucs4_convert_impl::enableif<TD, typename TS::value_type>::type unicode_
  * @return An instance of TD.
  */
 template<typename TD>
-TD unicode_cast(ucs4::char_t onechar)
+TD unicode_cast(char32_t onechar)
 {
 	using namespace ucs4_convert_impl;
 	typedef typename convert_impl<typename TD::value_type>::type impl_writer;
-	typedef convert_impl<ucs4::char_t>::type impl_reader;
+	typedef convert_impl<char32_t>::type impl_reader;
 	typedef typename std::back_insert_iterator<TD> output_itor;
 
 	TD res;
