@@ -1,16 +1,18 @@
 /*
-   Copyright (C) 2011 - 2018 by Sytyi Nick <nsytyi@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2011 - 2021
+	by Sytyi Nick <nsytyi@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
+
 /**
  * @file validator.hpp
  * This file contains information about validation abstract level interface.
@@ -23,7 +25,7 @@
 #include <string>
 
 class config;
-
+class config_attribute_value;
 extern bool strict_validation_enabled;
 
 /**
@@ -39,19 +41,22 @@ public:
 	 * Constructor of validator can throw validator::error
 	 * @throws abstract_validator::error
 	 */
-	abstract_validator(){}
+	abstract_validator(const std::string& name) : name_(name) {}
 
 	virtual ~abstract_validator(){}
 	/**
 	 * Is called when parser opens tag.
 	 * @param name        Name of tag
+	 * @param parent      The parent config
 	 * @param start_line  Line in file
 	 * @param file        Name of file
+	 * @param addition
 	 */
 	virtual void open_tag(const std::string & name,
+						  const config& parent,
 						  int start_line,
 						  const std::string &file,
-						  bool addittion = false) = 0;
+						  bool addition = false) = 0;
 	/**
 	 * As far as parser is built on stack, some realizations can store stack
 	 * too. So they need to know if tag was closed.
@@ -74,13 +79,14 @@ public:
 	 * What exactly is validated depends on validator realization
 	 * @param cfg         Config to be validated.
 	 * @param name        Name of tag
+	 * @param value       The key's value
 	 * @param start_line  Line in file
 	 * @param file        Name of file
 	 */
 
 	virtual void validate_key(const config & cfg,
 							  const std::string & name,
-							  const std::string & value,
+							  const config_attribute_value & value,
 							  int start_line,
 							  const std::string &file) = 0;
 	/**
@@ -91,4 +97,6 @@ public:
 	struct error : public game::error {
 		error(const std::string& message) : game::error(message) {}
 	};
+
+	const std::string name_;
 };
