@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -17,14 +18,13 @@
 #include "show_dialog.hpp"
 
 #include "floating_label.hpp"
-#include "font/sdl_ttf.hpp"
 #include "picture.hpp"
 #include "gettext.hpp"
 #include "gui/core/event/handler.hpp"
 #include "help/help.hpp"
 #include "hotkey/command_executor.hpp"
 #include "log.hpp"
-#include "font/marked-up_text.hpp"
+#include "font/sdl_ttf_compat.hpp"
 #include "font/standard_colors.hpp"
 #include "sdl/rect.hpp"
 
@@ -44,9 +44,6 @@ const int ButtonVPadding = 10;
 
 //note: style names are directly related to the panel image file names
 const dialog_frame::style dialog_frame::default_style("opaque", 0);
-const dialog_frame::style dialog_frame::message_style("translucent65", 3);
-const dialog_frame::style dialog_frame::preview_style("../dialogs/selection", 0);
-const dialog_frame::style dialog_frame::titlescreen_style("translucent54", 1);
 
 const int dialog_frame::title_border_w = 10;
 const int dialog_frame::title_border_h = 5;
@@ -192,7 +189,7 @@ dialog_frame::dimension_measurements dialog_frame::layout(int x, int y, int w, i
 		dim_.button_row.w += ButtonHPadding;
 	}
 
-	size_t buttons_width = dim_.button_row.w;
+	std::size_t buttons_width = dim_.button_row.w;
 
 	if(help_button_ != nullptr) {
 		buttons_width += help_button_->width() + ButtonHPadding*2;
@@ -325,8 +322,8 @@ void dialog_frame::draw_background()
 SDL_Rect dialog_frame::draw_title(CVideo* video)
 {
 	SDL_Rect rect = CVideo::get_singleton().screen_area();
-	return font::draw_text(video, rect, font::SIZE_TITLE, font::TITLE_COLOR,
-	                       title_, dim_.title.x, dim_.title.y, false, TTF_STYLE_NORMAL);
+	return font::pango_draw_text(video, rect, font::SIZE_TITLE, font::TITLE_COLOR,
+	                       title_, dim_.title.x, dim_.title.y, false, font::pango_text::STYLE_NORMAL);
 }
 
 void dialog_frame::draw()
