@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2010 - 2018 by Yurii Chernyi <terraninfo@terraninfo.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2010 - 2021
+	by Yurii Chernyi <terraninfo@terraninfo.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -23,6 +24,7 @@
 #include "variable.hpp"
 #include "team.hpp"
 #include "serialization/string_utils.hpp"
+#include "scripting/game_lua_kernel.hpp"
 #include "play_controller.hpp"
 #include "resources.hpp"
 #include "synced_context.hpp"
@@ -30,6 +32,7 @@
 #include "units/filter.hpp"
 #include "units/map.hpp"
 #include "variable.hpp"
+#include "filter_context.hpp"
 #include "formula/callable_objects.hpp"
 #include "formula/formula.hpp"
 #include "formula/function_gamestate.hpp"
@@ -37,6 +40,9 @@
 static lg::log_domain log_engine_sf("engine/side_filter");
 #define ERR_NG LOG_STREAM(err, log_engine_sf)
 
+static lg::log_domain log_wml("wml");
+#define ERR_WML LOG_STREAM(err, log_wml)
+
 side_filter::~side_filter() {}
 
 side_filter::side_filter(const vconfig& cfg, const filter_context * fc,  bool flat_tod)
@@ -138,7 +144,7 @@ bool side_filter::match_internal(const team &t) const
 			}
 		}
 		if(!found && ufilt_cfg["search_recall_list"].to_bool(false)) {
-			for(const unit_const_ptr & u : t.recall_list()) {
+			for(const unit_const_ptr u : t.recall_list()) {
 				scoped_recall_unit this_unit("this_unit", t.save_id_or_number(), t.recall_list().find_index(u->id()));
 				if(ufilter_->matches(*u)) {
 					found = true;
@@ -158,7 +164,7 @@ bool side_filter::match_internal(const team &t) const
 		const std::vector<int>& teams = enemy_filter_->get_teams();
 		if(teams.empty()) return false;
 		for(const int side : teams) {
-			if(!(fc_->get_disp_context().teams())[side - 1].is_enemy(t.side()))
+			if(!fc_->get_disp_context().get_team(side).is_enemy(t.side()))
 				return false;
 		}
 	}
@@ -170,7 +176,7 @@ bool side_filter::match_internal(const team &t) const
 		const std::vector<int>& teams = allied_filter_->get_teams();
 		if(teams.empty()) return false;
 		for(const int side : teams) {
-			if((fc_->get_disp_context().teams())[side - 1].is_enemy(t.side()))
+			if(fc_->get_disp_context().get_team(side).is_enemy(t.side()))
 				return false;
 		}
 	}
@@ -182,7 +188,7 @@ bool side_filter::match_internal(const team &t) const
 		const std::vector<int>& teams = has_enemy_filter_->get_teams();
 		bool found = false;
 		for(const int side : teams) {
-			if((fc_->get_disp_context().teams())[side - 1].is_enemy(t.side()))
+			if(fc_->get_disp_context().get_team(side).is_enemy(t.side()))
 			{
 				found = true;
 				break;
@@ -198,7 +204,7 @@ bool side_filter::match_internal(const team &t) const
 		const std::vector<int>& teams = has_ally_filter_->get_teams();
 		bool found = false;
 		for(const int side : teams) {
-			if(!(fc_->get_disp_context().teams())[side - 1].is_enemy(t.side()))
+			if(!fc_->get_disp_context().get_team(side).is_enemy(t.side()))
 			{
 				found = true;
 				break;
@@ -237,12 +243,23 @@ bool side_filter::match_internal(const team &t) const
 			}
 			return true;
 		} catch(const wfl::formula_error& e) {
-			lg::wml_error() << "Formula error in side filter: " << e.type << " at " << e.filename << ':' << e.line << ")\n";
+			lg::log_to_chat() << "Formula error in side filter: " << e.type << " at " << e.filename << ':' << e.line << ")\n";
+			ERR_WML << "Formula error in side filter: " << e.type << " at " << e.filename << ':' << e.line << ")";
 			// Formulae with syntax errors match nothing
 			return false;
 		}
 	}
 
+	if (cfg_.has_attribute("lua_function")) {
+		std::string lua_function = cfg_["lua_function"].str();
+		if (!lua_function.empty() && fc_->get_lua_kernel()) {
+			if (!fc_->get_lua_kernel()->run_filter(lua_function.c_str(), t)) {
+				return false;
+			}
+		}
+	}
+
+
 	return true;
 }
 
@@ -256,29 +273,21 @@ bool side_filter::match(const team& t) const
 {
 	bool matches = match_internal(t);
 
-	//handle [and], [or], and [not] with in-order precedence
-	vconfig::all_children_iterator cond = cfg_.ordered_begin();
-	vconfig::all_children_iterator cond_end = cfg_.ordered_end();
-	while (cond != cond_end) {
-		const std::string& cond_name = cond.get_key();
-		const vconfig& cond_cfg = cond.get_child();
-
-		//handle [and]
-		if(cond_name == "and")
-		{
-			matches = matches && side_filter(cond_cfg, fc_, flat_).match(t);
+	// Handle [and], [or], and [not] with in-order precedence
+	for(const auto& [key, filter] : cfg_.all_ordered()) {
+		// Handle [and]
+		if(key == "and") {
+			matches = matches && side_filter(filter, fc_, flat_).match(t);
 		}
-		//handle [or]
-		else if(cond_name == "or")
-		{
-			matches = matches || side_filter(cond_cfg, fc_, flat_).match(t);
+		// Handle [or]
+		else if(key == "or") {
+			matches = matches || side_filter(filter, fc_, flat_).match(t);
 		}
-		//handle [not]
-		else if(cond_name == "not")
-		{
-			matches = matches && !side_filter(cond_cfg, fc_, flat_).match(t);
+		// Handle [not]
+		else if(key == "not") {
+			matches = matches && !side_filter(filter, fc_, flat_).match(t);
 		}
-			++cond;
 	}
+
 	return matches;
 }
