@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "sound.hpp"
@@ -59,20 +60,20 @@ bool no_fading = false;
 bool unload_music = false;
 
 // number of allocated channels,
-const size_t n_of_channels = 32;
+const std::size_t n_of_channels = 32;
 
 // we need 2 channels, because we it for timer as well
-const size_t bell_channel = 0;
-const size_t timer_channel = 1;
+const std::size_t bell_channel = 0;
+const std::size_t timer_channel = 1;
 
 // number of channels reserved for sound sources
-const size_t source_channels = 8;
-const size_t source_channel_start = timer_channel + 1;
-const size_t source_channel_last = source_channel_start + source_channels - 1;
-const size_t UI_sound_channels = 2;
-const size_t UI_sound_channel_start = source_channel_last + 1;
-const size_t UI_sound_channel_last = UI_sound_channel_start + UI_sound_channels - 1;
-const size_t n_reserved_channels = UI_sound_channel_last + 1; // sources, bell, timer and UI
+const std::size_t source_channels = 8;
+const std::size_t source_channel_start = timer_channel + 1;
+const std::size_t source_channel_last = source_channel_start + source_channels - 1;
+const std::size_t UI_sound_channels = 2;
+const std::size_t UI_sound_channel_start = source_channel_last + 1;
+const std::size_t UI_sound_channel_last = UI_sound_channel_start + UI_sound_channels - 1;
+const std::size_t n_reserved_channels = UI_sound_channel_last + 1; // sources, bell, timer and UI
 
 // Max number of sound chunks that we want to cache
 // Keep this above number of available channels to avoid busy-looping
@@ -191,7 +192,13 @@ std::vector<std::shared_ptr<sound::music_track>>::const_iterator find_track(cons
 
 namespace sound
 {
-boost::optional<unsigned int> get_current_track_index()
+void flush_cache()
+{
+	sound_cache.clear();
+	music_cache.clear();
+}
+
+std::optional<unsigned int> get_current_track_index()
 {
 	if(current_track_index >= current_track_list.size()){
 		return {};
@@ -402,6 +409,37 @@ static void channel_finished_hook(int channel)
 	channel_ids[channel] = -1;
 }
 
+std::string current_driver()
+{
+	const char* const drvname = SDL_GetCurrentAudioDriver();
+	return drvname ? drvname : "<not initialized>";
+}
+
+std::vector<std::string> enumerate_drivers()
+{
+	std::vector<std::string> res;
+	int num_drivers = SDL_GetNumVideoDrivers();
+
+	for(int n = 0; n < num_drivers; ++n) {
+		const char* drvname = SDL_GetAudioDriver(n);
+		res.emplace_back(drvname ? drvname : "<invalid driver>");
+	}
+
+	return res;
+}
+
+driver_status driver_status::query()
+{
+	driver_status res{mix_ok, 0, 0, 0, 0};
+
+	if(mix_ok) {
+		Mix_QuerySpec(&res.frequency, &res.format, &res.channels);
+		res.chunk_size = preferences::sound_buffer_size();
+	}
+
+	return res;
+}
+
 bool init_sound()
 {
 	LOG_AUDIO << "Initializing audio...\n";
@@ -609,7 +647,8 @@ static void play_new_music()
 		return;
 	}
 
-	const std::string& filename = current_track->file_path();
+	const std::string localized = filesystem::get_localized_path(current_track->file_path());
+	const std::string& filename = localized.empty() ? current_track->file_path() : localized;
 
 	auto itor = music_cache.find(filename);
 	if(itor == music_cache.end()) {
@@ -701,12 +740,12 @@ void play_music_config(const config& music_node, bool allow_interrupt_current_tr
 	// in choose_track(), 2 tracks with the same name will always return the
 	// current track and track_ok() doesn't allow that.
 	if(iter == current_track_list.end()) {
-		if(i < 0 || static_cast<size_t>(i) >= current_track_list.size()) {
+		if(i < 0 || static_cast<std::size_t>(i) >= current_track_list.size()) {
 			current_track_list.emplace_back(new music_track(track));
 			iter = current_track_list.end() - 1;
 		} else {
 			iter = current_track_list.emplace(current_track_list.begin() + 1, new music_track(track));
-			if(current_track_index >= static_cast<size_t>(i)) {
+			if(current_track_index >= static_cast<std::size_t>(i)) {
 				current_track_index++;
 			}
 		}
@@ -857,7 +896,8 @@ static Mix_Chunk* load_chunk(const std::string& file, channel_group group)
 	sound_cache_iterator it;
 
 	sound_cache_chunk temp_chunk(file); // search the sound cache on this key
-	it_bgn = sound_cache.begin(), it_end = sound_cache.end();
+	it_bgn = sound_cache.begin();
+	it_end = sound_cache.end();
 	it = std::find(it_bgn, it_end, temp_chunk);
 
 	if(it != it_end) {
@@ -887,9 +927,10 @@ static Mix_Chunk* load_chunk(const std::string& file, channel_group group)
 
 		temp_chunk.group = group;
 		const std::string& filename = filesystem::get_binary_file_location("sounds", file);
+		const std::string localized = filesystem::get_localized_path(filename);
 
 		if(!filename.empty()) {
-			filesystem::rwops_ptr rwops = filesystem::make_read_RWops(filename);
+			filesystem::rwops_ptr rwops = filesystem::make_read_RWops(localized.empty() ? filename : localized);
 			temp_chunk.set_data(Mix_LoadWAV_RW(rwops.release(), true)); // SDL takes ownership of rwops
 		} else {
 			ERR_AUDIO << "Could not load sound file '" << file << "'." << std::endl;
