@@ -1,23 +1,24 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
 #include "events.hpp"
 #include "sound_music_track.hpp"
+#include <optional>
 
-#include <boost/optional.hpp>
 #include <string>
 
 class config;
@@ -33,6 +34,20 @@ enum channel_group {
 	SOUND_FX
 };
 
+std::string current_driver();
+std::vector<std::string> enumerate_drivers();
+
+struct driver_status
+{
+	bool initialized;
+	int frequency;
+	uint16_t format;
+	int channels;
+	int chunk_size;
+
+	static driver_status query();
+};
+
 bool init_sound();
 void close_sound();
 void reset_sound();
@@ -105,7 +120,7 @@ void set_sound_volume(int vol);
 void set_bell_volume(int vol);
 void set_UI_volume(int vol);
 
-boost::optional<unsigned int> get_current_track_index();
+std::optional<unsigned int> get_current_track_index();
 std::shared_ptr<sound::music_track> get_current_track();
 std::shared_ptr<sound::music_track> get_previous_music_track();
 void set_previous_track(std::shared_ptr<music_track>);
@@ -113,4 +128,6 @@ unsigned int get_num_tracks();
 void remove_track(unsigned int i);
 void play_track(unsigned int i);
 
+void flush_cache();
+
 }
