@@ -1,16 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Copyright (C) 2009 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>, Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "sound_music_track.hpp"
@@ -19,9 +19,7 @@
 #include "filesystem.hpp"
 #include "log.hpp"
 #include "serialization/string_utils.hpp"
-#if !defined(_WIN32) && !defined(__APPLE__)
 #include "vorbis/vorbisfile.h"
-#endif
 
 static lg::log_domain log_audio("audio");
 #define ERR_AUDIO LOG_STREAM(err, log_audio)
@@ -83,22 +81,10 @@ void music_track::resolve()
 		return;
 	}
 
-
-#if !defined(_WIN32) && !defined(__APPLE__)
 	if (title_.empty()) {
-		FILE* f;
-		f = fopen(file_path_.c_str(), "r");
-		if (f == nullptr) {
-			LOG_AUDIO << "Error opening file '" << file_path_
-					<< "' for track identification\n";
-			return;
-		}
-
 		OggVorbis_File vf;
-		if(ov_open(f, &vf, nullptr, 0) < 0) {
-			LOG_AUDIO << "track does not appear to be an Ogg file '"
-					<< id_ << "', cannot be identified\n";
-			ov_clear(&vf);
+		if(ov_fopen(file_path_.c_str(), &vf) < 0) {
+			LOG_AUDIO << "Error opening file '" << file_path_ << "' for track identification\n";
 			return;
 		}
 
@@ -121,7 +107,6 @@ void music_track::resolve()
 
 	ov_clear(&vf);
 	}
-#endif
 
 	LOG_AUDIO << "resolved music track '" << id_ << "' into '" << file_path_ << "'\n";
 }
