@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2006 - 2018 by Karol Nowak <grzywacz@sul.uni.lodz.pl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Karol Nowak <grzywacz@sul.uni.lodz.pl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "display.hpp"
@@ -52,7 +53,7 @@ void manager::add(const sourcespec &spec)
 	sources_[spec.id()].reset(new positional_source(spec));
 }
 
-config manager::get(const std::string &id)
+sourcespec manager::get(const std::string &id)
 {
 	config cfg;
 	positional_source_iterator it = sources_.find(id);
@@ -73,6 +74,11 @@ void manager::remove(const std::string &id)
 	}
 }
 
+bool manager::contains(const std::string& id)
+{
+	return sources_.find(id) != sources_.end();
+}
+
 void manager::update()
 {
 	unsigned int time = SDL_GetTicks();
@@ -215,6 +221,20 @@ void positional_source::write_config(config& cfg) const
 	write_locations(locations_, cfg);
 }
 
+void sourcespec::write(config& cfg) const
+{
+	cfg["id"] = id_;
+	cfg["sounds"] = files_;
+	cfg["delay"] = min_delay_;
+	cfg["chance"] = chance_;
+	cfg["check_fogged"] = check_fogged_;
+	cfg["check_shrouded"] = check_shrouded_;
+	cfg["loop"] = loops_;
+	cfg["full_range"] = range_;
+	cfg["fade_range"] = faderange_;
+	write_locations(locations_, cfg);
+}
+
 sourcespec::sourcespec(const config& cfg) :
 	id_(cfg["id"]),
 	files_(cfg["sounds"]),
