@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2006 - 2018 by Karol Nowak <grzywacz@sul.uni.lodz.pl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Karol Nowak <grzywacz@sul.uni.lodz.pl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -93,7 +94,8 @@ public:
 	// add or replace a soundsource
 	void add(const sourcespec &source);
 	void remove(const std::string &id);
-	config get(const std::string &id);
+	sourcespec get(const std::string &id);
+	bool contains(const std::string& id);
 	void update();
 
 	// checks which sound sources are visible
@@ -114,7 +116,7 @@ public:
 class sourcespec
 {
 	const std::string id_;
-	const std::string files_;
+	std::string files_;
 
 	int min_delay_;
 	int chance_;
@@ -172,6 +174,10 @@ public:
 		return locations_;
 	}
 
+	void set_locations(const std::vector<map_location>& locs) {
+		locations_ = locs;
+	}
+
 	int full_range() const { return range_; }
 
 	void set_full_range(int value) {
@@ -199,6 +205,10 @@ public:
 	const std::string& id() const { return id_; }
 
 	const std::string& files() const { return files_; }
+
+	void set_files(const std::string& f) {
+		files_ = f;
+	}
 };
 
 } // namespace soundsource
