@@ -1,16 +1,18 @@
 /*
-   Copyright (C) 2014 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
+
 #include "synced_commands.hpp"
 #include <cassert>
 
@@ -77,19 +79,19 @@ SYNCED_COMMAND_HANDLER_FUNCTION(recruit, child, use_undo, show, error_handler)
 		// This will be the case for AI recruits in replays saved
 		// before 1.11.2, so it is not more severe than a warning.
 		// EDIT: we broke compatibility with 1.11.2 anyway so we should give an error.
-		error_handler("Missing leader location for recruitment.\n", false);
+		error_handler("Missing leader location for recruitment.\n");
 	}
 	else if ( resources::gameboard->units().find(from) == resources::gameboard->units().end() ) {
 		// Sync problem?
 		std::stringstream errbuf;
 		errbuf << "Recruiting leader not found at " << from << ".\n";
-		error_handler(errbuf.str(), false);
+		error_handler(errbuf.str());
 	}
 
 	// Get the unit_type ID.
 	std::string type_id = child["type"];
 	if ( type_id.empty() ) {
-		error_handler("Recruitment is missing a unit type.", true);
+		error_handler("Recruitment is missing a unit type.");
 		return false;
 	}
 
@@ -97,7 +99,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(recruit, child, use_undo, show, error_handler)
 	if (!u_type) {
 		std::stringstream errbuf;
 		errbuf << "Recruiting illegal unit: '" << type_id << "'.\n";
-		error_handler(errbuf.str(), true);
+		error_handler(errbuf.str());
 		return false;
 	}
 
@@ -106,7 +108,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(recruit, child, use_undo, show, error_handler)
 	{
 		std::stringstream errbuf;
 		errbuf << "cannot recruit unit: " << res << "\n";
-		error_handler(errbuf.str(), true);
+		error_handler(errbuf.str());
 		return false;
 		//we are already oos because the unit wasn't created, no need to keep the bookkeeping right...
 	}
@@ -118,7 +120,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(recruit, child, use_undo, show, error_handler)
 		std::stringstream errbuf;
 		errbuf << "unit '" << type_id << "' is too expensive to recruit: "
 			<< u_type->cost() << "/" << beginning_gold << "\n";
-		error_handler(errbuf.str(), false);
+		error_handler(errbuf.str());
 	}
 
 	actions::recruit_unit(*u_type, current_team_num, loc, from, show, use_undo);
@@ -140,7 +142,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(recall, child, use_undo, show, error_handler)
 	map_location from(child.child_or_empty("from"), resources::gamedata);
 
 	if ( !actions::recall_unit(unit_id, current_team, loc, from, map_location::NDIRECTIONS, show, use_undo) ) {
-		error_handler("illegal recall: unit_id '" + unit_id + "' could not be found within the recall list.\n", true);
+		error_handler("illegal recall: unit_id '" + unit_id + "' could not be found within the recall list.\n");
 		//when recall_unit returned false nothing happened so we can safety return false;
 		return false;
 	}
@@ -154,12 +156,12 @@ SYNCED_COMMAND_HANDLER_FUNCTION(attack, child, /*use_undo*/, show, error_handler
 	//check_checksums(*cfg);
 
 	if (!destination) {
-		error_handler("no destination found in attack\n", true);
+		error_handler("no destination found in attack\n");
 		return false;
 	}
 
 	if (!source) {
-		error_handler("no source found in attack \n", true);
+		error_handler("no source found in attack \n");
 		return false;
 	}
 
@@ -183,7 +185,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(attack, child, /*use_undo*/, show, error_handler
 
 	unit_map::iterator u = resources::gameboard->units().find(src);
 	if (!u.valid()) {
-		error_handler("unfound location for source of attack\n", true);
+		error_handler("unfound location for source of attack\n");
 		return false;
 	}
 
@@ -195,7 +197,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(attack, child, /*use_undo*/, show, error_handler
 	}
 
 	if (static_cast<unsigned>(weapon_num) >= u->attacks().size()) {
-		error_handler("illegal weapon type in attack\n", true);
+		error_handler("illegal weapon type in attack\n");
 		return false;
 	}
 
@@ -204,7 +206,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(attack, child, /*use_undo*/, show, error_handler
 	if (!tgt.valid()) {
 		std::stringstream errbuf;
 		errbuf << "unfound defender for attack: " << src << " -> " << dst << '\n';
-		error_handler(errbuf.str(), true);
+		error_handler(errbuf.str());
 		return false;
 	}
 
@@ -217,7 +219,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(attack, child, /*use_undo*/, show, error_handler
 
 	if (def_weapon_num >= static_cast<int>(tgt->attacks().size())) {
 
-		error_handler("illegal defender weapon type in attack\n", true);
+		error_handler("illegal defender weapon type in attack\n");
 		return false;
 	}
 
@@ -235,7 +237,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(disband, child, /*use_undo*/, /*show*/, error_ha
 	team &current_team = resources::gameboard->get_team(current_team_num);
 
 	const std::string& unit_id = child["value"];
-	size_t old_size = current_team.recall_list().size();
+	std::size_t old_size = current_team.recall_list().size();
 
 	// Find the unit in the recall list.
 	unit_ptr dismissed_unit = current_team.recall_list().find_if_matches_id(unit_id);
@@ -246,7 +248,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(disband, child, /*use_undo*/, /*show*/, error_ha
 	current_team.recall_list().erase_if_matches_id(unit_id);
 
 	if (old_size == current_team.recall_list().size()) {
-		error_handler("illegal disband\n", true);
+		error_handler("illegal disband\n");
 		return false;
 	}
 	return true;
@@ -294,7 +296,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(move, child,  use_undo, show, error_handler)
 		std::stringstream errbuf;
 		errbuf << "unfound location for source of movement: "
 			<< src << " -> " << dst << '\n';
-		error_handler(errbuf.str(), true);
+		error_handler(errbuf.str());
 		return false;
 	}
 	bool skip_sighted = false;
@@ -377,7 +379,6 @@ SYNCED_COMMAND_HANDLER_FUNCTION(auto_shroud, child,  use_undo, /*show*/, /*error
  * Updates fog/shroud based on the undo stack, then updates stack as needed.
  * Call this when "updating shroud now".
  * This may fire events and change the game state.
- * @param[in]  is_replay  Set to true when this is called during a replay.
  *
  * This means it is a synced command like any other.
  */
@@ -387,7 +388,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(update_shroud, /*child*/,  use_undo, /*show*/, e
 	assert(use_undo);
 	team &current_team = resources::controller->current_team();
 	if(current_team.auto_shroud_updates()) {
-		error_handler("Team has DSU disabled but we found an explicit shroud update", false);
+		error_handler("Team has DSU disabled but we found an explicit shroud update");
 	}
 	resources::undo_stack->commit_vision();
 	resources::undo_stack->add_update_shroud();
@@ -416,7 +417,7 @@ namespace
 		} else {
 			message = VGETTEXT(text.c_str(), i18n_vars);
 		}
-		
+
 		if(show_long_message && !ignore) {
 			play_controller::scoped_savegame_snapshot snapshot(controller);
 			std::stringstream sbuilder;
@@ -442,6 +443,26 @@ namespace
 	}
 }
 
+SYNCED_COMMAND_HANDLER_FUNCTION(debug_terrain, child, use_undo, /*show*/, /*error_handler*/)
+{
+	if(use_undo) {
+		resources::undo_stack->clear();
+	}
+	debug_cmd_notification("terrain");
+
+	map_location loc(child);
+	const std::string& terrain_type = child["terrain_type"];
+	const std::string& mode_str = child["mode_str"];
+
+	bool result = resources::gameboard->change_terrain(loc, terrain_type, mode_str, false);
+	if(result) {
+		display::get_singleton()->invalidate(loc);
+		game_display::get_singleton()->needs_rebuild(result);
+		game_display::get_singleton()->maybe_rebuild();
+	}
+	return true;
+}
+
 SYNCED_COMMAND_HANDLER_FUNCTION(debug_unit, child,  use_undo, /*show*/, /*error_handler*/)
 {
 	if(use_undo) {
@@ -516,10 +537,11 @@ SYNCED_COMMAND_HANDLER_FUNCTION(debug_create_unit, child,  use_undo, /*show*/, e
 	debug_notification(N_("A unit was created using debug mode during $player’s turn"));
 	map_location loc(child);
 	resources::whiteboard->on_kill_unit();
+	const std::string& variation = child["variation"].str();
 	const unit_race::GENDER gender = string_gender(child["gender"], unit_race::NUM_GENDERS);
 	const unit_type *u_type = unit_types.find(child["type"]);
 	if (!u_type) {
-		error_handler("Invalid unit type", true);
+		error_handler("Invalid unit type");
 		return false;
 	}
 
@@ -527,7 +549,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(debug_create_unit, child,  use_undo, /*show*/, e
 			? resources::controller->current_side() : 1;
 
 	// Create the unit.
-	unit_ptr created = unit::create(*u_type, side_num, true, gender);
+	unit_ptr created = unit::create(*u_type, side_num, true, gender, variation);
 	created->new_turn();
 
 	unit_map::unit_iterator unit_it;
@@ -560,7 +582,7 @@ SYNCED_COMMAND_HANDLER_FUNCTION(debug_lua, child, use_undo, /*show*/, /*error_ha
 		resources::undo_stack->clear();
 	}
 	debug_cmd_notification("lua");
-	resources::lua_kernel->run(child["code"].str().c_str());
+	resources::lua_kernel->run(child["code"].str().c_str(), "debug command");
 	resources::controller->pump().flush_messages();
 
 	return true;
