@@ -1,14 +1,15 @@
 /*
-   Copyright (C) 2015 - 2018 by the Battle for Wesnoth Project
+	Copyright (C) 2015 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 
@@ -81,7 +82,7 @@ namespace
 			assert(label_id_ == -1);
 			SDL_Rect area = display::get_singleton()->map_outside_area();
 			font::floating_label flabel(message_);
-			flabel.set_font_size(font::SIZE_XLARGE);
+			flabel.set_font_size(font::SIZE_LARGE);
 			flabel.set_color(font::NORMAL_COLOR);
 			flabel.set_position(area.w/2, area.h/4);
 			flabel.set_lifetime(-1);
@@ -240,7 +241,7 @@ user_choice_manager::user_choice_manager(const std::string &name, const mp_sync:
 		assert(!t.is_empty());
 		if(side != current_side_)
 		{
-			synced_context::set_is_simultaneously();
+			synced_context::set_is_simultaneous();
 		}
 	}
 
@@ -330,7 +331,7 @@ void user_choice_manager::update_local_choice()
 		"waiting for $desc from side $sides",
 		"waiting for $desc from sides $sides",
 		sides_str.size(),
-		{std::make_pair("desc", uch_.description()), std::make_pair("sides", utils::format_conjunct_list("", sides_str))}
+		{std::pair("desc", uch_.description()), std::pair("sides", utils::format_conjunct_list("", sides_str))}
 	);
 	if(local_choice_prev != local_choice_) {
 		changed_event_.notify_observers();
@@ -348,7 +349,7 @@ void user_choice_manager::ask_local_choice()
 	config cfg = uch_.query_user(local_choice_);
 	if(res_.find(local_choice_) != res_.end()) {
 		// It might be possible that we this choice was already made by another client while we were in uch_.query_user
-		// because our side might be ressigned while we made our choice.
+		// because our side might be reassigned while we made our choice.
 		WRN_REPLAY << "Discarding a local choice because we found it already on the replay";
 		return;
 	}
@@ -357,8 +358,8 @@ void user_choice_manager::ask_local_choice()
 
 	//send data to others.
 	//but if there wasn't any data sent during this turn, we don't want to begin with that now.
-	//TODO: we should send user choices during nonundoable actions immideatley.
-	if(synced_context::is_simultaneously() || current_side_ != local_choice_)
+	//TODO: we should send user choices during nonundoable actions immediately.
+	if(synced_context::is_simultaneous() || current_side_ != local_choice_)
 	{
 		synced_context::send_user_choice();
 	}
@@ -400,8 +401,7 @@ static void wait_ingame(user_choice_manager& man)
 
 static void wait_prestart(user_choice_manager& man)
 {
-	gui2::dialogs::synched_choice_wait scw(man);
-	scw.show();
+	gui2::dialogs::synched_choice_wait::display(man);
 }
 
 std::map<int, config> user_choice_manager::get_user_choice_internal(const std::string &name, const mp_sync::user_choice &uch, const std::set<int>& sides)
