@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -21,6 +22,7 @@
 
 #include "ai/manager.hpp"
 #include "color.hpp"
+#include "formula/string_utils.hpp"     // for VGETTEXT
 #include "game_data.hpp"
 #include "game_events/pump.hpp"
 #include "lexical_cast.hpp"
@@ -34,8 +36,6 @@
 #include "units/types.hpp"
 #include "whiteboard/side_actions.hpp"
 
-#include <boost/dynamic_bitset.hpp>
-
 static lg::log_domain log_engine("engine");
 #define DBG_NG LOG_STREAM(debug, log_engine)
 #define LOG_NG LOG_STREAM(info, log_engine)
@@ -74,6 +74,7 @@ const std::set<std::string> team::attributes {
 	"lost",
 	"recall_cost",
 	"recruit",
+	"previous_recruits",
 	"save_id",
 	"scroll_to_leader",
 	"share_vision",
@@ -423,6 +424,18 @@ void team::write(config& cfg) const
 	cfg["action_bonus_count"] = action_bonus_count_;
 }
 
+void team::fix_villages(const gamemap &map)
+{
+	for (auto it = villages_.begin(); it != villages_.end(); ) {
+		if (map.is_village(*it)) {
+			++it;
+		}
+		else {
+			it = villages_.erase(it);
+		}
+	}
+}
+
 game_events::pump_result_t team::get_village(const map_location& loc, const int owner_side, game_data* gamedata)
 {
 	villages_.insert(loc);
@@ -492,7 +505,7 @@ int team::minimum_recruit_price() const
 	return info_.minimum_recruit_price;
 }
 
-void team::calculate_enemies(size_t index) const
+void team::calculate_enemies(std::size_t index) const
 {
 	if(!resources::gameboard || index >= resources::gameboard->teams().size()) {
 		return;
@@ -503,7 +516,7 @@ void team::calculate_enemies(size_t index) const
 	}
 }
 
-bool team::calculate_is_enemy(size_t index) const
+bool team::calculate_is_enemy(std::size_t index) const
 {
 	// We're not enemies of ourselves
 	if(&resources::gameboard->teams()[index] == this) {
@@ -544,13 +557,13 @@ public:
 	{
 	}
 
-	/// We are in a game with no mp server and need to do this choice locally
+	/** We are in a game with no mp server and need to do this choice locally */
 	virtual config local_choice() const
 	{
 		return config{"controller", new_controller_, "is_local", true};
 	}
 
-	/// tThe request which is sent to the mp server.
+	/** The request which is sent to the mp server. */
 	virtual config request() const
 	{
 		return config{
@@ -663,10 +676,10 @@ bool team::fogged(const map_location& loc) const
 	return fog_.shared_value(ally_fog(resources::gameboard->teams()), loc.wml_x(), loc.wml_y());
 }
 
-const std::vector<const team::shroud_map*>& team::ally_shroud(const std::vector<team>& teams) const
+const std::vector<const shroud_map*>& team::ally_shroud(const std::vector<team>& teams) const
 {
 	if(ally_shroud_.empty()) {
-		for(size_t i = 0; i < teams.size(); ++i) {
+		for(std::size_t i = 0; i < teams.size(); ++i) {
 			if(!is_enemy(i + 1) && (&(teams[i]) == this || teams[i].share_view() || teams[i].share_maps())) {
 				ally_shroud_.push_back(&(teams[i].shroud_));
 			}
@@ -676,10 +689,10 @@ const std::vector<const team::shroud_map*>& team::ally_shroud(const std::vector<
 	return ally_shroud_;
 }
 
-const std::vector<const team::shroud_map*>& team::ally_fog(const std::vector<team>& teams) const
+const std::vector<const shroud_map*>& team::ally_fog(const std::vector<team>& teams) const
 {
 	if(ally_fog_.empty()) {
-		for(size_t i = 0; i < teams.size(); ++i) {
+		for(std::size_t i = 0; i < teams.size(); ++i) {
 			if(!is_enemy(i + 1) && (&(teams[i]) == this || teams[i].share_view())) {
 				ally_fog_.push_back(&(teams[i].fog_));
 			}
@@ -689,7 +702,7 @@ const std::vector<const team::shroud_map*>& team::ally_fog(const std::vector<tea
 	return ally_fog_;
 }
 
-bool team::knows_about_team(size_t index) const
+bool team::knows_about_team(std::size_t index) const
 {
 	const team& t = resources::gameboard->teams()[index];
 
@@ -753,7 +766,20 @@ void validate_side(int side)
 	}
 }
 
-bool team::shroud_map::clear(int x, int y)
+int shroud_map::width() const
+{
+	return data_.size();
+}
+
+int shroud_map::height() const
+{
+	if(data_.size() == 0) return 0;
+	return std::max_element(data_.begin(), data_.end(), [](const auto& a, const auto& b) {
+		return a.size() < b.size();
+	})->size();
+}
+
+bool shroud_map::clear(int x, int y)
 {
 	if(enabled_ == false || x < 0 || y < 0) {
 		return false;
@@ -775,7 +801,7 @@ bool team::shroud_map::clear(int x, int y)
 	return false;
 }
 
-void team::shroud_map::place(int x, int y)
+void shroud_map::place(int x, int y)
 {
 	if(enabled_ == false || x < 0 || y < 0) {
 		return;
@@ -792,7 +818,7 @@ void team::shroud_map::place(int x, int y)
 	}
 }
 
-void team::shroud_map::reset()
+void shroud_map::reset()
 {
 	if(enabled_ == false) {
 		return;
@@ -803,7 +829,7 @@ void team::shroud_map::reset()
 	}
 }
 
-bool team::shroud_map::value(int x, int y) const
+bool shroud_map::value(int x, int y) const
 {
 	if(!enabled_) {
 		return false;
@@ -823,7 +849,7 @@ bool team::shroud_map::value(int x, int y) const
 	return !data_[x][y];
 }
 
-bool team::shroud_map::shared_value(const std::vector<const shroud_map*>& maps, int x, int y) const
+bool shroud_map::shared_value(const std::vector<const shroud_map*>& maps, int x, int y) const
 {
 	if(!enabled_) {
 		return false;
@@ -844,7 +870,7 @@ bool team::shroud_map::shared_value(const std::vector<const shroud_map*>& maps,
 	return true;
 }
 
-std::string team::shroud_map::write() const
+std::string shroud_map::write() const
 {
 	std::stringstream shroud_str;
 	for(const auto& sh : data_) {
@@ -860,7 +886,7 @@ std::string team::shroud_map::write() const
 	return shroud_str.str();
 }
 
-void team::shroud_map::read(const std::string& str)
+void shroud_map::read(const std::string& str)
 {
 	data_.clear();
 
@@ -879,10 +905,10 @@ void team::shroud_map::read(const std::string& str)
 	}
 }
 
-void team::shroud_map::merge(const std::string& str)
+void shroud_map::merge(const std::string& str)
 {
 	int x = 0, y = 0;
-	for(size_t i = 1; i < str.length(); ++i) {
+	for(std::size_t i = 1; i < str.length(); ++i) {
 		if(str[i] == '|') {
 			y = 0;
 			x++;
@@ -895,7 +921,7 @@ void team::shroud_map::merge(const std::string& str)
 	}
 }
 
-bool team::shroud_map::copy_from(const std::vector<const shroud_map*>& maps)
+bool shroud_map::copy_from(const std::vector<const shroud_map*>& maps)
 {
 	if(enabled_ == false) {
 		return false;
@@ -908,8 +934,8 @@ bool team::shroud_map::copy_from(const std::vector<const shroud_map*>& maps)
 		}
 
 		const std::vector<std::vector<bool>>& v = m->data_;
-		for(size_t x = 0; x != v.size(); ++x) {
-			for(size_t y = 0; y != v[x].size(); ++y) {
+		for(std::size_t x = 0; x != v.size(); ++x) {
+			for(std::size_t y = 0; y != v[x].size(); ++y) {
 				if(v[x][y]) {
 					cleared |= clear(x, y);
 				}
@@ -971,6 +997,18 @@ std::string team::get_side_color_id(unsigned side)
 	}
 }
 
+const t_string team::get_side_color_name_for_UI(unsigned side)
+{
+	const std::string& color_id = team::get_side_color_id(side);
+	const auto& rgb_name = game_config::team_rgb_name[color_id];
+	if(rgb_name.empty())
+		// TRANSLATORS: $color_id is the internal identifier of a side color, for example, 'lightred'.
+		// Translate the quotation marks only; leave "color_id" untranslated, as it's a variable name.
+		return VGETTEXT("“$color_id”", {{ "color_id", color_id }});
+	else
+		return rgb_name;
+}
+
 std::string team::get_side_color_id_from_config(const config& cfg)
 {
 	const config::attribute_value& c = cfg["color"];
