@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -37,6 +38,33 @@ namespace wb {
 	class side_actions;
 }
 
+class shroud_map {
+public:
+	shroud_map() : enabled_(false), data_() {}
+
+	void place(int x, int y);
+	bool clear(int x, int y);
+	void reset();
+
+	bool value(int x, int y) const;
+	bool shared_value(const std::vector<const shroud_map*>& maps, int x, int y) const;
+
+	bool copy_from(const std::vector<const shroud_map*>& maps);
+
+	std::string write() const;
+	void read(const std::string& shroud_data);
+	void merge(const std::string& shroud_data);
+
+	bool enabled() const { return enabled_; }
+	void set_enabled(bool enabled) { enabled_ = enabled; }
+
+	int width() const;
+	int height() const;
+private:
+	bool enabled_;
+	std::vector<std::vector<bool>> data_;
+};
+
 /**
  * This class stores all the data for a single 'side' (in game nomenclature).
  * E.g., there is only one leader unit per team.
@@ -71,30 +99,6 @@ public:
 	)
 
 private:
-	class shroud_map {
-	public:
-		shroud_map() : enabled_(false), data_() {}
-
-		void place(int x, int y);
-		bool clear(int x, int y);
-		void reset();
-
-		bool value(int x, int y) const;
-		bool shared_value(const std::vector<const shroud_map*>& maps, int x, int y) const;
-
-		bool copy_from(const std::vector<const shroud_map*>& maps);
-
-		std::string write() const;
-		void read(const std::string& shroud_data);
-		void merge(const std::string& shroud_data);
-
-		bool enabled() const { return enabled_; }
-		void set_enabled(bool enabled) { enabled_ = enabled; }
-	private:
-		bool enabled_;
-		std::vector<std::vector<bool>> data_;
-	};
-
 	struct team_info
 	{
 		team_info();
@@ -169,15 +173,24 @@ public:
 	team();
 	virtual ~team();
 
-	/// Stores the attributes recognized by [side]. These should be stripped
-	/// from a side's config before using it to create the side's leader.
+	/**
+	 * Stores the attributes recognized by [side]. These should be stripped
+	 * from a side's config before using it to create the side's leader.
+	 */
 	static const std::set<std::string> attributes;
 
 	void build(const config &cfg, const gamemap &map, int gold = default_team_gold_);
 
 	void write(config& cfg) const;
 
-	game_events::pump_result_t get_village(const map_location&, const int owner_side, game_data * fire_event); //!< Acquires a village from owner_side. Pointer fire_event should be the game_data for the game if it is desired to fire an event -- a "capture" event with owner_side variable scoped in will be fired. For no event, pass it nullptr. Default is the resources::gamedata pointer
+	void fix_villages(const gamemap &map);
+	/**
+	 * Acquires a village from owner_side.
+	 * Pointer fire_event should be the game_data for the game if it is desired to fire an event -- a "capture" event with owner_side variable scoped in will be fired.
+	 * For no event, pass it nullptr.
+	 * Default is the resources::gamedata pointer.
+	 */
+	game_events::pump_result_t get_village(const map_location&, const int owner_side, game_data * fire_event);
 	void lose_village(const map_location&);
 	void clear_villages() { villages_.clear(); }
 	const std::set<map_location>& villages() const { return villages_; }
@@ -240,7 +253,7 @@ public:
 	bool objectives_changed() const { return info_.objectives_changed; }
 
 	bool is_enemy(int n) const {
-		const size_t index = size_t(n-1);
+		const std::size_t index = std::size_t(n-1);
 		if(index >= enemies_.size()) {
 			calculate_enemies(index);
 		}
@@ -253,6 +266,7 @@ public:
 
 	CONTROLLER controller() const { return info_.controller; }
 	const std::string& color() const { return info_.color; }
+	/** @note Call display::reinit_flag_for_side() after calling this */
 	void set_color(const std::string& color) { info_.color = color; }
 	bool is_empty() const { return info_.controller == CONTROLLER::EMPTY; }
 
@@ -299,12 +313,13 @@ public:
 	const std::string& flag() const { return info_.flag; }
 	const std::string& flag_icon() const { return info_.flag_icon; }
 
+	/** @note Call display::reinit_flag_for_side() after calling this */
 	void set_flag(const std::string& flag) { info_.flag = flag; }
 	void set_flag_icon(const std::string& flag_icon) { info_.flag_icon = flag_icon; }
 
 	const std::string& side_name() const { return info_.side_name.empty() ? info_.current_player : info_.side_name.str(); }
 	t_string side_name_tstr() const { return info_.side_name.empty() ? t_string(info_.current_player) : info_.side_name; }
-	void set_side_name(const t_string& new_name) {info_.side_name = new_name;}
+	void set_side_name(const t_string& new_name) {info_.side_name = new_name == info_.current_player ? "" : new_name;}
 	const std::string& faction() const { return info_.faction; }
 	const t_string& faction_name() const { return info_.faction_name; }
 	//Returns true if the hex is shrouded/fogged for this side, or
@@ -322,14 +337,15 @@ public:
 	void refog() { fog_.reset(); }
 	void set_shroud(bool shroud) { shroud_.set_enabled(shroud); }
 	void set_fog(bool fog) { fog_.set_enabled(fog); }
+	std::string shroud_data() const { return shroud_.write(); }
 
 	/** Merge a WML shroud map with the shroud data of this player. */
 	void merge_shroud_map_data(const std::string& shroud_data) { shroud_.merge(shroud_data); }
 
-	bool knows_about_team(size_t index) const;
-	/// Records hexes that were cleared of fog via WML.
+	bool knows_about_team(std::size_t index) const;
+	/** Records hexes that were cleared of fog via WML. */
 	void add_fog_override(const std::set<map_location> &hexes) { fog_clearer_.insert(hexes.begin(), hexes.end()); }
-	/// Removes the record of hexes that were cleared of fog via WML.
+	/** Removes the record of hexes that were cleared of fog via WML. */
 	void remove_fog_override(const std::set<map_location> &hexes);
 
 	bool auto_shroud_updates() const { return auto_shroud_updates_; }
@@ -338,7 +354,7 @@ public:
 	bool no_leader() const { return info_.no_leader; }
 	DEFEAT_CONDITION defeat_condition() const { return info_.defeat_condition; }
 	void set_defeat_condition(DEFEAT_CONDITION value) { info_.defeat_condition = value; }
-	///sets the defeat condition if @param value is a valid defeat condition, otherwise nothing happes.
+	/** sets the defeat condition if @param value is a valid defeat condition, otherwise nothing happes. */
 	void set_defeat_condition_string(const std::string& value) { info_.defeat_condition.parse(value); }
 	void have_leader(bool value=true) { info_.no_leader = !value; }
 	bool hidden() const { return info_.hidden; }
@@ -369,6 +385,7 @@ public:
 	static color_t get_minimap_color(int side);
 
 	static std::string get_side_color_id(unsigned side);
+	static const t_string get_side_color_name_for_UI(unsigned side);
 	static std::string get_side_color_id_from_config(const config& cfg);
 	static std::string get_side_highlight_pango(int side);
 
@@ -417,7 +434,7 @@ private:
 	std::set<map_location> villages_;
 
 	shroud_map shroud_, fog_;
-	/// Stores hexes that have been cleared of fog via WML.
+	/** Stores hexes that have been cleared of fog via WML. */
 	std::set<map_location> fog_clearer_;
 
 	bool auto_shroud_updates_;
@@ -431,8 +448,8 @@ private:
 	std::string last_recruit_;
 
 private:
-	void calculate_enemies(size_t index) const;
-	bool calculate_is_enemy(size_t index) const;
+	void calculate_enemies(std::size_t index) const;
+	bool calculate_is_enemy(std::size_t index) const;
 	mutable boost::dynamic_bitset<> enemies_;
 
 	mutable std::vector<const shroud_map*> ally_shroud_, ally_fog_;
