@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2004 - 2018 by Philippe Plantier <ayin@anathas.org>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2004 - 2021
+	by Philippe Plantier <ayin@anathas.org>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -25,6 +26,7 @@
 #include "map/map.hpp"
 #include "preferences/game.hpp"
 #include "serialization/string_utils.hpp"
+#include "game_config_view.hpp"
 
 static lg::log_domain log_engine("engine");
 #define ERR_NG LOG_STREAM(err, log_engine)
@@ -85,7 +87,7 @@ static map_location legacy_difference(const map_location& me, const map_location
  */
 
 terrain_builder::building_ruleset terrain_builder::building_rules_;
-const config* terrain_builder::rules_cfg_ = nullptr;
+const game_config_view* terrain_builder::rules_cfg_ = nullptr;
 
 terrain_builder::rule_image::rule_image(int layer, int x, int y, bool global_image, int cx, int cy, bool is_water)
 	: layer(layer)
@@ -152,7 +154,7 @@ void terrain_builder::tile::rebuild_cache(const std::string& tod, logs* log)
 			const animated<image::locator>& anim = variant.images[rnd % variant.images.size()];
 
 			bool is_empty = true;
-			for(size_t i = 0; i < anim.get_frames_count(); ++i) {
+			for(std::size_t i = 0; i < anim.get_frames_count(); ++i) {
 				if(!image::is_empty_hex(anim.get_frame(i))) {
 					is_empty = false;
 					break;
@@ -287,7 +289,7 @@ void terrain_builder::flush_local_rules()
 	}
 }
 
-void terrain_builder::set_terrain_rules_cfg(const config& cfg)
+void terrain_builder::set_terrain_rules_cfg(const game_config_view& cfg)
 {
 	rules_cfg_ = &cfg;
 	// use the swap trick to clear the rules cache and get a fresh one.
@@ -452,7 +454,7 @@ bool terrain_builder::load_images(building_rule& rule)
 						const std::vector<std::string> items = utils::split(frame, ':');
 						const std::string& str = items.front();
 
-						const size_t tilde = str.find('~');
+						const std::size_t tilde = str.find('~');
 						bool has_tilde = tilde != std::string::npos;
 						const std::string filename = "terrain/" + (has_tilde ? str.substr(0, tilde) : str);
 
@@ -779,7 +781,7 @@ void terrain_builder::add_constraints(terrain_builder::constraint_set& constrain
 
 {
 	terrain_constraint& constraint = add_constraints(
-			constraints, loc, t_translation::ter_match(cfg["type"], t_translation::WILDCARD), global_images);
+			constraints, loc, t_translation::ter_match(cfg["type"].str(), t_translation::WILDCARD), global_images);
 
 	std::vector<std::string> item_string = utils::square_parenthetical_split(cfg["set_flag"], ',', "[", "]");
 	constraint.set_flag.insert(constraint.set_flag.end(), item_string.begin(), item_string.end());
@@ -860,7 +862,7 @@ void terrain_builder::add_rotated_rules(building_ruleset& rules, building_rule&
 	} else {
 		const std::vector<std::string>& rot = utils::split(rotations, ',');
 
-		for(size_t angle = 0; angle < rot.size(); ++angle) {
+		for(std::size_t angle = 0; angle < rot.size(); ++angle) {
 			/* Only 5% of the rules have valid images, so most of
 			   them will be discarded. If the ratio was higher,
 			   it would be more efficient to insert a copy of the
@@ -881,6 +883,11 @@ void terrain_builder::add_rotated_rules(building_ruleset& rules, building_rule&
 }
 
 void terrain_builder::parse_config(const config& cfg, bool local)
+{
+	return parse_config(game_config_view::wrap(cfg), local);
+}
+
+void terrain_builder::parse_config(const game_config_view& cfg, bool local)
 {
 	log_scope("terrain_builder::parse_config");
 	int n = 0;
@@ -1012,7 +1019,7 @@ void terrain_builder::add_off_map_rule(const std::string& image)
 	item["set_flag"] = "base";
 
 	// Parse the object
-	parse_global_config(cfg);
+	parse_global_config(game_config_view::wrap(cfg));
 }
 
 bool terrain_builder::rule_matches(const terrain_builder::building_rule& rule,
@@ -1155,7 +1162,7 @@ void terrain_builder::build_terrains()
 		// Find the constraint that contains the less terrain of all terrain rules.
 		// We will keep a track of the matching terrains of this constraint
 		// and later try to apply the rule only on them
-		size_t min_size = INT_MAX;
+		std::size_t min_size = INT_MAX;
 		t_translation::ter_list min_types = t_translation::ter_list(); // <-- This must be explicitly initialized, just
 																	   // as min_constraint is, at start of loop, or we
 																	   // get a null pointer dereference when we go
@@ -1165,13 +1172,13 @@ void terrain_builder::build_terrains()
 		for(const terrain_constraint& constraint : rule.constraints) {
 			const t_translation::ter_match& match = constraint.terrain_types_match;
 			t_translation::ter_list matching_types;
-			size_t constraint_size = 0;
+			std::size_t constraint_size = 0;
 
 			for(terrain_by_type_map::iterator type_it = terrain_by_type_.begin(); type_it != terrain_by_type_.end();
 					++type_it) {
 				const t_translation::terrain_code t = type_it->first;
 				if(terrain_matches(t, match)) {
-					const size_t match_size = type_it->second.size();
+					const std::size_t match_size = type_it->second.size();
 					constraint_size += match_size;
 					if(constraint_size >= min_size) {
 						break; // not a minimum, bail out
