@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2004 - 2018 by Philippe Plantier <ayin@anathas.org>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2004 - 2021
+	by Philippe Plantier <ayin@anathas.org>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -25,6 +26,8 @@
 #include "terrain/translation.hpp"
 
 class config;
+class game_config_view;
+
 class gamemap;
 namespace image
 {
@@ -81,7 +84,7 @@ public:
 	 *						off map image (see add_off_map_rule()).
 	 *						This image automatically gets the 'terrain/' prefix
 	 *						and '.png' suffix
-	 * @para draw_border    Whether the map border flag should be set to allow
+	 * @param draw_border    Whether the map border flag should be set to allow
 	 *                      its drawing.
 	 */
 	terrain_builder(const config& level, const gamemap* map, const std::string& offmap_image, bool draw_border);
@@ -92,7 +95,7 @@ public:
 	 * @param cfg			The main game configuration object, where the
 	 *						[terrain_graphics] rule reside.
 	 */
-	static void set_terrain_rules_cfg(const config& cfg);
+	static void set_terrain_rules_cfg(const game_config_view& cfg);
 
 	const gamemap& map() const
 	{
@@ -328,6 +331,7 @@ public:
 		 * to select the correct images.
 		 *
 		 * @param tod    The current time-of-day
+		 * @param log
 		 */
 		void rebuild_cache(const std::string& tod, logs* log = nullptr);
 
@@ -755,8 +759,9 @@ private:
 	 * @param local     Mark the rules as local only.
 	 */
 	void parse_config(const config& cfg, bool local = true);
+	void parse_config(const game_config_view& cfg, bool local = true);
 
-	void parse_global_config(const config& cfg)
+	void parse_global_config(const game_config_view& cfg)
 	{
 		parse_config(cfg, false);
 	}
@@ -858,5 +863,5 @@ private:
 	static building_ruleset building_rules_;
 
 	/** Config used to parse global terrain rules */
-	static const config* rules_cfg_;
+	static const game_config_view* rules_cfg_;
 };
