@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -27,6 +28,7 @@
 #include "tod_manager.hpp"
 #include "units/unit.hpp"
 #include "units/filter.hpp"
+#include "units/alignment.hpp"
 #include "variable.hpp"
 #include "formula/callable_objects.hpp"
 #include "formula/formula.hpp"
@@ -39,6 +41,9 @@ static lg::log_domain log_engine("engine");
 #define ERR_NG LOG_STREAM(err, log_engine)
 #define WRN_NG LOG_STREAM(warn, log_engine)
 
+static lg::log_domain log_wml("wml");
+#define ERR_WML LOG_STREAM(err, log_wml)
+
 terrain_filter::~terrain_filter()
 {
 }
@@ -89,14 +94,6 @@ terrain_filter::terrain_filter_cache::terrain_filter_cache() :
 	ufilter_()
 {}
 
-namespace {
-	struct cfg_isor {
-		bool operator() (std::pair<const std::string,const vconfig> val) const {
-			return val.first == "or";
-		}
-	};
-} //end anonymous namespace
-
 bool terrain_filter::match_internal(const map_location& loc, const unit* ref_unit, const bool ignore_xy) const
 {
 	if (!this->fc_->get_disp_context().map().on_board_with_border(loc)) {
@@ -115,9 +112,13 @@ bool terrain_filter::match_internal(const map_location& loc, const unit* ref_uni
 		fc_->get_tod_man().get_area_by_id(cfg_["area"]).count(loc) == 0)
 		return false;
 
+	if(cfg_.has_attribute("gives_income") &&
+		cfg_["gives_income"].to_bool() != fc_->get_disp_context().map().is_village(loc))
+		return false;
+
 	if(cfg_.has_attribute("terrain")) {
 		if(cache_.parsed_terrain == nullptr) {
-			cache_.parsed_terrain.reset(new t_translation::ter_match(cfg_["terrain"]));
+			cache_.parsed_terrain.reset(new t_translation::ter_match(std::string_view(cfg_["terrain"].str())));
 		}
 		if(!cache_.parsed_terrain->is_empty) {
 			const t_translation::terrain_code letter = fc_->get_disp_context().map().get_terrain_info(loc).number();
@@ -155,7 +156,14 @@ bool terrain_filter::match_internal(const map_location& loc, const unit* ref_uni
 			}
 		}
 		if (cfg_.has_attribute("location_id")) {
-			if (loc != fc_->get_disp_context().map().special_location(cfg_["location_id"])) {
+			std::set<map_location> matching_locs;
+			for(const auto& id : utils::split(cfg_["location_id"])) {
+				map_location test_loc = fc_->get_disp_context().map().special_location(id);
+				if(test_loc.valid()) {
+					matching_locs.insert(test_loc);
+				}
+			}
+			if (matching_locs.count(loc) == 0) {
 				return false;
 			}
 		}
@@ -199,8 +207,7 @@ bool terrain_filter::match_internal(const map_location& loc, const unit* ref_uni
 
 	//Allow filtering on adjacent locations
 	if(cfg_.has_child("filter_adjacent_location")) {
-		adjacent_loc_array_t adjacent;
-		get_adjacent_tiles(loc, adjacent.data());
+		const auto adjacent = get_adjacent_tiles(loc);
 		const vconfig::child_list& adj_cfgs = cfg_.get_children("filter_adjacent_location");
 		vconfig::child_list::const_iterator i, i_end, i_begin = adj_cfgs.begin();
 		for (i = i_begin, i_end = adj_cfgs.end(); i != i_end; ++i) {
@@ -210,7 +217,7 @@ bool terrain_filter::match_internal(const map_location& loc, const unit* ref_uni
 				? map_location::parse_directions((*i)["adjacent"]) : map_location::default_dirs();
 			std::vector<map_location::DIRECTION>::const_iterator j, j_end = dirs.end();
 			for (j = dirs.begin(); j != j_end; ++j) {
-				map_location &adj = adjacent[*j];
+				const map_location &adj = adjacent[*j];
 				if (fc_->get_disp_context().map().on_board(adj)) {
 					if(cache_.adjacent_matches == nullptr) {
 						while(index >= std::distance(cache_.adjacent_match_cache.begin(), cache_.adjacent_match_cache.end())) {
@@ -268,14 +275,15 @@ bool terrain_filter::match_internal(const map_location& loc, const unit* ref_uni
 		if(!tod_type.empty()) {
 			const std::vector<std::string>& vals = utils::split(tod_type);
 			if(tod.lawful_bonus<0) {
-				if(std::find(vals.begin(),vals.end(),unit_type::ALIGNMENT::enum_to_string(unit_type::ALIGNMENT::CHAOTIC)) == vals.end()) {
+				if(std::find(vals.begin(),vals.end(),UNIT_ALIGNMENT::enum_to_string(UNIT_ALIGNMENT::CHAOTIC)) == vals.end()) {
 					return false;
 				}
 			} else if(tod.lawful_bonus>0) {
-				if(std::find(vals.begin(),vals.end(),unit_type::ALIGNMENT::enum_to_string(unit_type::ALIGNMENT::LAWFUL)) == vals.end()) {
+				if(std::find(vals.begin(),vals.end(),UNIT_ALIGNMENT::enum_to_string(UNIT_ALIGNMENT::LAWFUL)) == vals.end()) {
 					return false;
 				}
-			} else if(std::find(vals.begin(),vals.end(),unit_type::ALIGNMENT::enum_to_string(unit_type::ALIGNMENT::NEUTRAL)) == vals.end()) {
+			} else if(std::find(vals.begin(),vals.end(),UNIT_ALIGNMENT::enum_to_string(UNIT_ALIGNMENT::NEUTRAL)) == vals.end() &&
+				std::find(vals.begin(),vals.end(),UNIT_ALIGNMENT::enum_to_string(UNIT_ALIGNMENT::LIMINAL)) == vals.end()) {
 				return false;
 			}
 		}
@@ -308,7 +316,7 @@ bool terrain_filter::match_internal(const map_location& loc, const unit* ref_uni
 		side_filter ssf(filter_owner, fc_);
 		const std::vector<int>& sides = ssf.get_teams();
 		bool found = false;
-		if(sides.empty() && fc_->get_disp_context().village_owner(loc) == -1)
+		if(sides.empty() && fc_->get_disp_context().village_owner(loc) == 0)
 			found = true;
 		for(const int side : sides) {
 			if(fc_->get_disp_context().get_team(side).owns_village(loc)) {
@@ -320,8 +328,8 @@ bool terrain_filter::match_internal(const map_location& loc, const unit* ref_uni
 			return false;
 	}
 	else if(!owner_side.empty()) {
-		const int side_index = owner_side.to_int(0) - 1;
-		if(fc_->get_disp_context().village_owner(loc) != side_index) {
+		const int side_num = owner_side.to_int(0);
+		if(fc_->get_disp_context().village_owner(loc) != side_num) {
 			return false;
 		}
 	}
@@ -342,7 +350,8 @@ bool terrain_filter::match_internal(const map_location& loc, const unit* ref_uni
 			}
 			return true;
 		} catch(const wfl::formula_error& e) {
-			lg::wml_error() << "Formula error in location filter: " << e.type << " at " << e.filename << ':' << e.line << ")\n";
+			lg::log_to_chat() << "Formula error in location filter: " << e.type << " at " << e.filename << ':' << e.line << ")\n";
+			ERR_WML << "Formula error in location filter: " << e.type << " at " << e.filename << ':' << e.line << ")";
 			// Formulae with syntax errors match nothing
 			return false;
 		}
@@ -379,7 +388,7 @@ bool terrain_filter::match_impl(const map_location& loc, const unit* ref_unit) c
 	}
 
 	//handle radius
-	size_t radius = cfg_["radius"].to_size_t(0);
+	std::size_t radius = cfg_["radius"].to_size_t(0);
 	if(radius > max_loop_) {
 		ERR_NG << "terrain_filter: radius greater than " << max_loop_
 		<< ", restricting\n";
@@ -398,36 +407,27 @@ bool terrain_filter::match_impl(const map_location& loc, const unit* ref_unit) c
 		get_tiles_radius(fc_->get_disp_context().map(), loc_vec, radius, hexes);
 	}
 
-	size_t loop_count = 0;
+	std::size_t loop_count = 0;
 	std::set<map_location>::const_iterator i;
 	for(i = hexes.begin(); i != hexes.end(); ++i) {
 		bool matches = match_internal(*i, ref_unit, false);
 
-		//handle [and], [or], and [not] with in-order precedence
-		vconfig::all_children_iterator cond = cfg_.ordered_begin();
-		vconfig::all_children_iterator cond_end = cfg_.ordered_end();
-		while(cond != cond_end)
-		{
-			const std::string& cond_name = cond.get_key();
-			const vconfig& cond_cfg = cond.get_child();
-
-			//handle [and]
-			if(cond_name == "and")
-			{
-				matches = matches && terrain_filter(cond_cfg, *this).match_impl(*i, ref_unit);
+		// Handle [and], [or], and [not] with in-order precedence
+		for(const auto& [key, filter] : cfg_.all_ordered()) {
+			// Handle [and]
+			if(key == "and") {
+				matches = matches && terrain_filter(filter, *this).match_impl(*i, ref_unit);
 			}
-			//handle [or]
-			else if(cond_name == "or")
-			{
-				matches = matches || terrain_filter(cond_cfg, *this).match_impl(*i, ref_unit);
+			// Handle [or]
+			else if(key == "or") {
+				matches = matches || terrain_filter(filter, *this).match_impl(*i, ref_unit);
 			}
-			//handle [not]
-			else if(cond_name == "not")
-			{
-				matches = matches && !terrain_filter(cond_cfg, *this).match_impl(*i, ref_unit);
+			// Handle [not]
+			else if(key == "not") {
+				matches = matches && !terrain_filter(filter, *this).match_impl(*i, ref_unit);
 			}
-			++cond;
 		}
+
 		if(matches) {
 			return true;
 		}
@@ -467,8 +467,14 @@ public:
 	static void filter_special_loc(T&& src, location_set& dest, const terrain_filter& filter, const F1& f1, const F2& f2)
 	{
 		if (filter.cfg_.has_attribute("location_id")) {
-			map_location loc2 = filter.fc_->get_disp_context().map().special_location(filter.cfg_["location_id"]);
-			filter_final(src, dest, filter, f1, f2, [loc2](const map_location& loc) { return loc == loc2; });
+			std::set<map_location> matching_locs;
+			for(const auto& id : utils::split(filter.cfg_["location_id"])) {
+				map_location test_loc = filter.fc_->get_disp_context().map().special_location(id);
+				if(test_loc.valid()) {
+					matching_locs.insert(test_loc);
+				}
+			}
+			filter_final(src, dest, filter, f1, f2, [matching_locs](const map_location& loc) { return matching_locs.count(loc) > 0; });
 		}
 		else {
 			filter_final(src, dest, filter, f1, f2, no_filter());
@@ -544,11 +550,17 @@ void terrain_filter::get_locs_impl(std::set<map_location>& locs, const unit* ref
 		terrain_filterimpl::filter_special_loc(area, match_set, *this, terrain_filterimpl::no_filter(), terrain_filterimpl::no_filter());
 	}
 	else if (cfg_.has_attribute("location_id")) {
-		map_location loc2 = fc_->get_disp_context().map().special_location(cfg_["location_id"]);
-		if (loc2.valid()) {
-			match_set.insert(loc2);
+		for(const auto& id : utils::split(cfg_["location_id"])) {
+			map_location test_loc = fc_->get_disp_context().map().special_location(id);
+			if(test_loc.valid()) {
+				match_set.insert(test_loc);
+			}
 		}
 	}
+	else if (cfg_["gives_income"].to_bool()) {
+		auto ar = fc_->get_disp_context().map().villages();
+		terrain_filterimpl::filter_xy(ar, match_set, *this, with_border);
+	}
 	else {
 		//consider all locations on the map
 		int bs = fc_->get_disp_context().map().border_size();
@@ -588,24 +600,19 @@ void terrain_filter::get_locs_impl(std::set<map_location>& locs, const unit* ref
 		}
 	}
 
-	//handle [and], [or], and [not] with in-order precedence
-	vconfig::all_children_iterator cond = cfg_.ordered_begin();
-	vconfig::all_children_iterator cond_end = cfg_.ordered_end();
-	int ors_left = std::count_if(cond, cond_end, cfg_isor());
-	while(cond != cond_end)
-	{
+	int ors_left = std::count_if(cfg_.ordered_begin(), cfg_.ordered_end(), [](const auto& val) { return val.first == "or"; });
+
+	// Handle [and], [or], and [not] with in-order precedence
+	for(const auto& [key, filter] : cfg_.all_ordered()) {
 		//if there are no locations or [or] conditions left, go ahead and return empty
 		if(match_set.empty() && ors_left <= 0) {
 			return;
 		}
 
-		const std::string& cond_name = cond.get_key();
-		const vconfig& cond_cfg = cond.get_child();
-
-		//handle [and]
-		if(cond_name == "and") {
+		// Handle [and]
+		if(key == "and") {
 			std::set<map_location> intersect_hexes;
-			terrain_filter(cond_cfg, *this).get_locations(intersect_hexes, with_border);
+			terrain_filter(filter, *this).get_locations(intersect_hexes, with_border);
 			std::set<map_location>::iterator intersect_itor = match_set.begin();
 			while(intersect_itor != match_set.end()) {
 				if(intersect_hexes.find(*intersect_itor) == intersect_hexes.end()) {
@@ -615,10 +622,10 @@ void terrain_filter::get_locs_impl(std::set<map_location>& locs, const unit* ref
 				}
 			}
 		}
-		//handle [or]
-		else if(cond_name == "or") {
+		// Handle [or]
+		else if(key == "or") {
 			std::set<map_location> union_hexes;
-			terrain_filter(cond_cfg, *this).get_locations(union_hexes, with_border);
+			terrain_filter(filter, *this).get_locations(union_hexes, with_border);
 			//match_set.insert(union_hexes.begin(), union_hexes.end()); //doesn't compile on MSVC
 			std::set<map_location>::iterator insert_itor = union_hexes.begin();
 			while(insert_itor != union_hexes.end()) {
@@ -626,23 +633,22 @@ void terrain_filter::get_locs_impl(std::set<map_location>& locs, const unit* ref
 			}
 			--ors_left;
 		}
-		//handle [not]
-		else if(cond_name == "not") {
+		// Handle [not]
+		else if(key == "not") {
 			std::set<map_location> removal_hexes;
-			terrain_filter(cond_cfg, *this).get_locations(removal_hexes, with_border);
+			terrain_filter(filter, *this).get_locations(removal_hexes, with_border);
 			std::set<map_location>::iterator erase_itor = removal_hexes.begin();
 			while(erase_itor != removal_hexes.end()) {
 				match_set.erase(*erase_itor++);
 			}
 		}
-		++cond;
 	}
 	if(match_set.empty()) {
 		return;
 	}
 
 	//handle radius
-	size_t radius = cfg_["radius"].to_size_t(0);
+	std::size_t radius = cfg_["radius"].to_size_t(0);
 	if(radius > max_loop_) {
 		ERR_NG << "terrain_filter: radius greater than " << max_loop_
 		<< ", restricting\n";
