@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2006 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #pragma once
 
 #include <vector>
@@ -26,7 +25,8 @@
 
 #include "exceptions.hpp"
 #include "map/location.hpp"
-#include "serialization/string_view.hpp"
+
+#include <string_view>
 
 namespace t_translation {
 
@@ -49,7 +49,7 @@ namespace t_translation {
 	 */
 	struct terrain_code {
 		terrain_code(const std::string& b, const std::string& o);
-		terrain_code(const std::string& b, ter_layer o = NO_LAYER);
+		explicit terrain_code(const std::string& b, ter_layer o = NO_LAYER);
 		terrain_code(ter_layer b, ter_layer o) : base(b), overlay(o) {}
 		terrain_code() : base(0), overlay(NO_LAYER) {}
 
@@ -80,18 +80,15 @@ namespace t_translation {
 
 		ter_map() = default;
 		ter_map(const ter_map&) = default;
-#if !defined(_MSC_VER) || _MSC_VER >= 1900
 		ter_map(ter_map&&) = default;
-#endif
+
 		ter_map(int w, int h, terrain_code fill = terrain_code()) : data(w * h, fill), w(w), h(h) {}
 
 		ter_map & operator= (const ter_map &) = default;
-#if !defined(_MSC_VER) || _MSC_VER >= 1900
 		ter_map & operator= (ter_map &&) = default;
-#endif
 
-		terrain_code& get(int x, int y) { size_t index = x * h + y; return data.at(index); }
-		const terrain_code& get(int x, int y) const { size_t index = x * h + y; return data.at(index); }
+		terrain_code& get(int x, int y) { std::size_t index = x * h + y; return data.at(index); }
+		const terrain_code& get(int x, int y) const { std::size_t index = x * h + y; return data.at(index); }
 
 		std::vector<terrain_code> data;
 		int w;
@@ -107,7 +104,7 @@ namespace t_translation {
 	 */
 	struct ter_match{
 		ter_match();
-		ter_match(const std::string& str, const ter_layer filler = NO_LAYER);
+		ter_match(std::string_view str, const ter_layer filler = NO_LAYER);
 		ter_match(const terrain_code& tcode);
 
 		ter_list terrain;
@@ -185,7 +182,7 @@ namespace t_translation {
 	 *
 	 * @return			A single terrain code
 	 */
-	terrain_code read_terrain_code(const std::string& str, const ter_layer filler = NO_LAYER);
+	terrain_code read_terrain_code(std::string_view str, const ter_layer filler = NO_LAYER);
 
 	/**
 	 * Writes a single terrain code to a string.
@@ -207,7 +204,7 @@ namespace t_translation {
 	 *
 	 * @returns		A vector which contains the terrain codes found in the string
 	 */
-	 ter_list read_list(utils::string_view str, const ter_layer filler = NO_LAYER);
+	 ter_list read_list(std::string_view str, const ter_layer filler = NO_LAYER);
 
 	/**
 	 * Writes a list of terrains to a string, only writes the new format.
@@ -248,17 +245,19 @@ namespace t_translation {
 	 *					the last is stored. The returned value is a map:
 	 *					* first		the starting locations
 	 *					* second	a coordinate structure where the location was found
+	 * @param border_offset
 	 *
 	 * @returns			A 2D vector with the terrains found the vector data is stored
 	 *					like result[x][y] where x the column number is and y the row number.
 	 */
-	ter_map read_game_map(const std::string& str, starting_positions& positions, coordinate border_offset = coordinate{ 0, 0 });
+	ter_map read_game_map(std::string_view str, starting_positions& positions, coordinate border_offset = coordinate{ 0, 0 });
 
 	/**
 	 * Write a gamemap in to a vector string.
 	 *
 	 * @param map				 A terrain vector, as returned from read_game_map
 	 * @param positions A starting positions map, as returned from read_game_map
+	 * @param border_offset
 	 *
 	 * @returns			A terrain string which can be read with read_game_map.
 	 *					For readability the map is padded to groups of 12 chars,
