@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 // In this domain since it compares a shared string from this domain.
@@ -20,8 +21,8 @@
 #include "config_cache.hpp"
 #include "filesystem.hpp"
 #include "formula/debugger.hpp"
-#include "game_classification.hpp"
 #include "game_config.hpp"
+#include "game_config_view.hpp"
 #include "game_display.hpp"
 #include "game_events/manager.hpp"
 #include "game_initialization/create_engine.hpp"
@@ -31,10 +32,11 @@
 #include "generators/map_create.hpp"
 #include "gettext.hpp"
 #include "gui/core/layout_exception.hpp"
+#include "gui/dialogs/addon/addon_auth.hpp"
 #include "gui/dialogs/addon/connect.hpp"
 #include "gui/dialogs/addon/install_dependencies.hpp"
+#include "gui/dialogs/addon/license_prompt.hpp"
 #include "gui/dialogs/addon/manager.hpp"
-#include "gui/dialogs/advanced_graphics_options.hpp"
 #include "gui/dialogs/attack_predictions.hpp"
 #include "gui/dialogs/campaign_difficulty.hpp"
 #include "gui/dialogs/campaign_selection.hpp"
@@ -53,7 +55,6 @@
 #include "gui/dialogs/editor/generator_settings.hpp"
 #include "gui/dialogs/editor/new_map.hpp"
 #include "gui/dialogs/editor/resize_map.hpp"
-#include "gui/dialogs/editor/set_starting_position.hpp"
 #include "gui/dialogs/end_credits.hpp"
 #include "gui/dialogs/file_dialog.hpp"
 #include "gui/dialogs/folder_create.hpp"
@@ -111,11 +112,12 @@
 #include "language.hpp"
 #include "map/map.hpp"
 #include "replay.hpp"
+#include "save_index.hpp"
 #include "saved_game.hpp"
 #include "terrain/type_data.hpp"
 #include "tests/utils/fake_display.hpp"
 //#include "scripting/lua_kernel_base.hpp"
-#include "utils/functional.hpp"
+#include <functional>
 #include "video.hpp"
 #include "wesnothd_connection.hpp"
 #include "wml_exception.hpp"
@@ -156,13 +158,6 @@ window* unit_test_window(const modeless_dialog& dialog)
 	return dialog.window_.get();
 }
 
-class mp_server_list;
-
-modal_dialog* unit_test_mp_server_list()
-{
-	return mp_connect::mp_server_list_for_unit_test();
-}
-
 } // namespace dialogs
 } // namespace gui2
 
@@ -170,6 +165,7 @@ namespace {
 
 	/** The main config, which contains the entire WML tree. */
 	config main_config;
+	game_config_view game_config_view_ = game_config_view::wrap(main_config);
 
 	/**
 	 * Helper class to generate a dialog.
@@ -329,7 +325,7 @@ void test()
 {
 	gui2::new_widgets = false;
 
-//	for(size_t i = 0; i < 2; ++i) {
+//	for(std::size_t i = 0; i < 2; ++i) {
 
 		test_resolutions<T>(get_gui_resolutions());
 
@@ -343,7 +339,7 @@ void test_popup()
 {
 	gui2::new_widgets = false;
 
-	for(size_t i = 0; i < 2; ++i) {
+	for(std::size_t i = 0; i < 2; ++i) {
 
 		test_popup_resolutions<T>(get_gui_resolutions());
 
@@ -355,7 +351,7 @@ void test_tip(const std::string& id)
 {
 	gui2::new_widgets = false;
 
-	for(size_t i = 0; i < 2; ++i) {
+	for(std::size_t i = 0; i < 2; ++i) {
 
 		test_tip_resolutions(get_gui_resolutions(), id);
 
@@ -411,7 +407,7 @@ BOOST_AUTO_TEST_CASE(test_gui2)
 	cache.add_define("MULTIPLAYER");
 	cache.get_config(game_config::path +"/data", main_config);
 
-	const filesystem::binary_paths_manager bin_paths_manager(main_config);
+	const filesystem::binary_paths_manager bin_paths_manager(game_config_view_);
 
 	load_language_list();
 	game_config::load_config(main_config.child("game_config"));
@@ -419,9 +415,10 @@ BOOST_AUTO_TEST_CASE(test_gui2)
 	/**** Run the tests. *****/
 
 	/* The modal_dialog classes. */
+	test<addon_auth>();
 	test<addon_connect>();
+	test<addon_license_prompt>();
 	//test<addon_manager>();
-	test<advanced_graphics_options>();
 	//test<attack_predictions>();
 	test<campaign_difficulty>();
 	test<campaign_selection>();
@@ -438,7 +435,6 @@ BOOST_AUTO_TEST_CASE(test_gui2)
 	test<editor_generate_map>();
 	test<editor_new_map>();
 	test<editor_resize_map>();
-	test<editor_set_starting_position>();
 	//test<end_credits>();
 	test<faction_select>();
 	test<file_dialog>();
@@ -472,7 +468,6 @@ BOOST_AUTO_TEST_CASE(test_gui2)
 	test<mp_join_game_password_prompt>();
 	test<mp_login>();
 	test<mp_method_selection>();
-	test<mp_server_list>();
 	//test<mp_staging>();
 	//test<outro>();
 	test<simple_item_selector>();
@@ -546,6 +541,7 @@ BOOST_AUTO_TEST_CASE(test_gui2)
 		"mp_change_control", // Basically useless without a game_board object, so disabling
 		"game_stats", // segfault with LTO
 		"gamestate_inspector", // segfault with LTO
+		"server_info",
 	};
 
 	std::vector<std::string> missing;
@@ -576,6 +572,16 @@ BOOST_AUTO_TEST_CASE(test_make_test_fake)
 
 namespace {
 
+template<>
+struct dialog_tester<addon_auth>
+{
+	config cfg;
+	addon_auth* create()
+	{
+		return new addon_auth(cfg);
+	}
+};
+
 template<>
 struct dialog_tester<addon_connect>
 {
@@ -586,6 +592,18 @@ struct dialog_tester<addon_connect>
 	}
 };
 
+template<>
+struct dialog_tester<addon_license_prompt>
+{
+	std::string license_terms = R"""(Lorem ipsum dolor sit amet, consectetur adipiscing elit. Duis ante nibh, dignissim ullamcorper tristique eget, condimentum sit amet enim. Aenean dictum pulvinar lacinia. Etiam eleifend, leo sed efficitur consectetur, augue nulla ornare lectus, vitae molestie lacus risus vitae libero. Quisque odio nunc, porttitor eget fermentum sit amet, faucibus eu risus. Praesent sit amet lacus tortor. Suspendisse volutpat quam vitae ipsum fermentum, in vulputate metus egestas. Nulla id consequat ex. Nulla ac dignissim nisl, nec euismod lectus. Duis vitae dolor ornare, convallis justo in, porta dui.
+
+Sed faucibus nibh sit amet ligula porta, non malesuada nibh tristique. Maecenas aliquam diam non eros convallis mattis. Proin rhoncus condimentum leo, sed condimentum magna. Phasellus cursus condimentum lacus, sed sodales lacus. Sed pharetra dictum metus, eget dictum nibh lobortis imperdiet. Nunc tempus sollicitudin bibendum. In porttitor interdum orci. Curabitur vitae nibh vestibulum, condimentum lectus quis, condimentum dui. In quis cursus nisl. Maecenas semper neque eu ipsum aliquam, id porta ligula lacinia. Integer sed blandit ex, eu accumsan magna.)""";
+	addon_license_prompt* create()
+	{
+		return new addon_license_prompt(license_terms);
+	}
+};
+
 template<>
 struct dialog_tester<addon_manager>
 {
@@ -740,14 +758,18 @@ template<>
 struct dialog_tester<game_load>
 {
 	config cfg;
-	savegame::load_game_metadata data;
+	game_config_view view;
+	// It would be good to have a test directory instead of using the same directory as the player,
+	// however this code will support that - default_saves_dir() will respect --userdata-dir.
+	savegame::load_game_metadata data{savegame::save_index_class::default_saves_dir()};
 	dialog_tester()
 	{
 		/** @todo Would be nice to add real data to the config. */
 	}
 	game_load* create()
 	{
-		return new game_load(cfg, data);
+		view = game_config_view::wrap(cfg);
+		return new game_load(view, data);
 	}
 
 };
@@ -826,43 +848,25 @@ struct dialog_tester<hotkey_bind>
 	}
 };
 
-struct wesnothd_connection_init
-{
-	wesnothd_connection_init(wesnothd_connection& conn)
-	{
-		//Swallow the 'cannot connect' execption so that the connection object doesn't throw while we test the dialog.
-		try
-		{
-			while (true) {
-				conn.poll();
-			}
-		}
-		catch (...)
-		{
-
-		}
-	}
-};
-
 template<>
 struct dialog_tester<mp_lobby>
 {
 	config game_config;
-	wesnothd_connection_ptr connection;
-	wesnothd_connection_init init;
+	wesnothd_connection connection;
 	std::vector<std::string> installed_addons;
 	mp::lobby_info li;
-	dialog_tester() : connection(wesnothd_connection::create("", "")), init(*connection), li(installed_addons)
+	int selected_game;
+	dialog_tester() : connection("", ""), li(installed_addons)
 	{
 	}
 	mp_lobby* create()
 	{
-		return new mp_lobby(game_config, li, *connection);
+		return new mp_lobby(li, connection, selected_game);
 	}
 };
 
 class fake_chat_handler : public events::chat_handler {
-	void add_chat_message(const time_t&,
+	void add_chat_message(const std::time_t&,
 		const std::string&, int, const std::string&,
 		MESSAGE_TYPE) {}
 	void send_chat_message(const std::string&, bool) {}
@@ -874,13 +878,12 @@ struct dialog_tester<lobby_player_info>
 {
 	config c;
 	fake_chat_handler ch;
-	wesnothd_connection_ptr connection;
-	wesnothd_connection_init init;
+	wesnothd_connection connection;
 	mp::user_info ui;
 	std::vector<std::string> installed_addons;
 	mp::lobby_info li;
 	dialog_tester()
-		: connection(wesnothd_connection::create("", "")), init(*connection)
+		: connection("", "")
 		, ui(c), li(installed_addons)
 	{
 	}
@@ -927,7 +930,7 @@ struct dialog_tester<mp_create_game>
 	}
 	mp_create_game* create()
 	{
-		return new mp_create_game(main_config, state, true, nullptr);
+		return new mp_create_game(state, true);
 	}
 };
 
@@ -991,7 +994,7 @@ struct dialog_tester<screenshot_notification>
 template<>
 struct dialog_tester<theme_list>
 {
-	static theme_info make_theme(std::string name)
+	static theme_info make_theme(const std::string& name)
 	{
 		theme_info ti;
 		ti.id = name;
@@ -1040,16 +1043,6 @@ struct dialog_tester<editor_new_map>
 	}
 };
 
-template<>
-struct dialog_tester<editor_set_starting_position>
-{
-	std::vector<map_location> locations;
-	editor_set_starting_position* create()
-	{
-		return new editor_set_starting_position(0, 0, locations);
-	}
-};
-
 template<>
 struct dialog_tester<editor_resize_map>
 {
@@ -1082,15 +1075,6 @@ struct dialog_tester<folder_create>
 	}
 };
 
-template<>
-struct dialog_tester<mp_server_list>
-{
-	static modal_dialog* create()
-	{
-		return unit_test_mp_server_list();
-	}
-};
-
 template<>
 struct dialog_tester<transient_message>
 {
@@ -1106,7 +1090,7 @@ struct dialog_tester<title_screen>
 	std::vector<std::string> args;
 	commandline_options opts;
 	game_launcher game;
-	dialog_tester() : opts(args), game(opts, "unit_tests") {}
+	dialog_tester() : opts(args), game(opts) {}
 	title_screen* create()
 	{
 		return new title_screen(game);
