@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2015 - 2018 by Iris Morelle <shadowm2006@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2015 - 2021
+	by Iris Morelle <shadowm2006@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include <boost/test/unit_test.hpp>
@@ -30,7 +31,7 @@ void dump(const T& v)
 }
 #endif
 
-BOOST_AUTO_TEST_SUITE( filesystem ) // implicit namespace filesystem
+BOOST_AUTO_TEST_SUITE( filesystem ); // implicit namespace filesystem
 
 const std::string& gamedata = game_config::path;
 
@@ -40,7 +41,7 @@ BOOST_AUTO_TEST_CASE( test_fs_game_path_reverse_engineering )
 
 	std::string gamedata_rev = get_wml_location("_main.cfg");
 
-	const size_t strip_len = (maincfg + "/data/").length();
+	const std::size_t strip_len = (maincfg + "/data/").length();
 	BOOST_REQUIRE(gamedata_rev.length() > strip_len);
 	gamedata_rev.resize(gamedata_rev.length() - strip_len);
 
@@ -132,7 +133,7 @@ BOOST_AUTO_TEST_CASE( test_fs_enum )
 	// FIXME: get_files_in_dir with mode == FILE_NAME_ONLY will fail to reorder
 	//        entries because the sorting code looks for forward slashes.
 	//        This affects both the BFS-based and legacy implementations.
-	get_files_in_dir(path, &files, &dirs, ENTIRE_FILE_PATH, NO_FILTER, DO_REORDER);
+	get_files_in_dir(path, &files, &dirs, name_mode::ENTIRE_FILE_PATH, filter_mode::NO_FILTER, reorder_mode::DO_REORDER);
 
 	BOOST_CHECK( files == expected_filepaths );
 	BOOST_CHECK( dirs  == expected_dirpaths  );
@@ -157,7 +158,7 @@ BOOST_AUTO_TEST_CASE( test_fs_binary_path )
 	BOOST_CHECK_EQUAL( get_binary_file_location("sounds", "explosion.ogg"),
 	                   gamedata + "/data/core/sounds/explosion.ogg" );
 
-	BOOST_CHECK_EQUAL( get_independent_image_path("wesnoth-icon.png"),
+	BOOST_CHECK_EQUAL( get_independent_binary_file_path("images", "wesnoth-icon.png"),
 	                   "data/core/images/wesnoth-icon.png" );
 
 	// Inexistent paths are resolved empty.
@@ -167,7 +168,7 @@ BOOST_AUTO_TEST_CASE( test_fs_binary_path )
 	BOOST_CHECK( get_binary_file_location("images", "bunnies_and_ponies_and_rainbows_oh_em_gee.psd").empty() );
 	BOOST_CHECK( get_binary_file_location("music", "this_track_does_not_exist.aiff").empty() );
 	BOOST_CHECK( get_binary_file_location("sounds", "rude_noises.aiff").empty() );
-	BOOST_CHECK( get_independent_image_path("dopefish.txt").empty() );
+	BOOST_CHECK( get_independent_binary_file_path("images", "dopefish.txt").empty() );
 }
 
 BOOST_AUTO_TEST_CASE( test_fs_wml_path )
