@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2011 - 2018 by Karol Kozub <karol.alt@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2011 - 2021
+	by Karol Kozub <karol.alt@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-test"
@@ -18,6 +19,7 @@
 #include <sstream>
 
 #include "game_config.hpp"
+#include "game_config_view.hpp"
 #include "config_cache.hpp"
 #include "config.hpp"
 #include "color_range.hpp"
@@ -29,7 +31,7 @@
 using namespace image;
 
 namespace {
-/// Sets up the environment for every test
+/** Sets up the environment for every test */
 class environment_setup
 {
 public:
@@ -40,7 +42,6 @@ public:
 		, paths_manager_()
 	{
 		set_up_color_info();
-		set_up_team_colors();
 		set_up_image_paths();
 	}
 
@@ -61,21 +62,7 @@ private:
 						  "2E419B,FFFFFF,0F0F0F,0000FF",
 						  "Blue"));
 
-		game_config::add_color_info(cfg);
-	}
-
-	/** Sets up team color mapping
-	 *
-	 * This is required by TC modification
-	 */
-	static void set_up_team_colors()
-	{
-		std::vector<std::string> tc;
-
-		tc.push_back("red");
-		tc.push_back("blue");
-
-		image::set_team_colors(&tc);
+		game_config::add_color_info(game_config_view::wrap(cfg));
 	}
 
 	/** Sets up the paths later used to load images
@@ -86,11 +73,12 @@ private:
 	void set_up_image_paths()
 	{
 		config cfg;
-
+		game_config_view v = game_config_view::wrap(cfg);
 		cfg.add_child("binary_path",
 			      create_path_config("data/core"));
 
-		paths_manager_.set_paths(cfg);
+
+		paths_manager_.set_paths(v);
 	}
 
 	static config create_color_range(const std::string& id,
@@ -162,7 +150,7 @@ BOOST_AUTO_TEST_CASE(test_modificaiton_queue_order)
 	queue.pop();
 }
 
-/// Tests if the TC modification is correctly decoded
+/** Tests if the TC modification is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_tc_modification_decoding)
 {
 	environment_setup env_setup;
@@ -184,7 +172,7 @@ BOOST_AUTO_TEST_CASE(test_tc_modification_decoding)
 	BOOST_CHECK(expected == mod->map());
 }
 
-/// Tests if the TC modification with invalid arguments is ignored
+/** Tests if the TC modification with invalid arguments is ignored */
 BOOST_AUTO_TEST_CASE(test_tc_modification_decoding_invalid_args)
 {
 	environment_setup env_setup;
@@ -194,7 +182,7 @@ BOOST_AUTO_TEST_CASE(test_tc_modification_decoding_invalid_args)
 	BOOST_REQUIRE_EQUAL(queue.size(), 0);
 }
 
-/// Tests if the RC modification is correctly decoded
+/** Tests if the RC modification is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_rc_modification_decoding)
 {
 	environment_setup env_setup;
@@ -215,7 +203,7 @@ BOOST_AUTO_TEST_CASE(test_rc_modification_decoding)
 	BOOST_CHECK(expected == mod->map());
 }
 
-/// Tests if the RC modification with invalid arguments is ignored
+/** Tests if the RC modification with invalid arguments is ignored */
 BOOST_AUTO_TEST_CASE(test_rc_modification_decoding_invalid_args)
 {
 	environment_setup env_setup;
@@ -225,7 +213,7 @@ BOOST_AUTO_TEST_CASE(test_rc_modification_decoding_invalid_args)
 	BOOST_REQUIRE_EQUAL(queue.size(), 0);
 }
 
-/// Tests if the PAL modification is correctly decoded
+/** Tests if the PAL modification is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_pal_modification_decoding)
 {
 	environment_setup env_setup;
@@ -244,7 +232,7 @@ BOOST_AUTO_TEST_CASE(test_pal_modification_decoding)
 	const std::vector<color_t>& new_palette = game_config::tc_info("FFFFFF,FF00FF");
 	color_range_map expected;
 
-	for(size_t i = 0; i < old_palette.size() && i < new_palette.size(); ++i) {
+	for(std::size_t i = 0; i < old_palette.size() && i < new_palette.size(); ++i) {
 	environment_setup env_setup;
 
 		expected[old_palette[i]] = new_palette[i];
@@ -253,7 +241,7 @@ BOOST_AUTO_TEST_CASE(test_pal_modification_decoding)
 	BOOST_CHECK(expected == mod->map());
 }
 
-/// Tests if the PAL modification with invalid arguments is ignored
+/** Tests if the PAL modification with invalid arguments is ignored */
 BOOST_AUTO_TEST_CASE(test_pal_modification_decoding_invalid_args)
 {
 	environment_setup env_setup;
@@ -264,7 +252,7 @@ BOOST_AUTO_TEST_CASE(test_pal_modification_decoding_invalid_args)
 	BOOST_REQUIRE_EQUAL(queue.size(), 0);
 }
 
-/// Tests if the FL modification is correctly decoded without arguments
+/** Tests if the FL modification is correctly decoded without arguments */
 BOOST_AUTO_TEST_CASE(test_fl_modification_decoding_default)
 {
 	environment_setup env_setup;
@@ -282,7 +270,7 @@ BOOST_AUTO_TEST_CASE(test_fl_modification_decoding_default)
 	BOOST_CHECK(!mod->get_vert());
 }
 
-/// Tests if the FL modification is correctly decoded with the horiz argument
+/** Tests if the FL modification is correctly decoded with the horiz argument */
 BOOST_AUTO_TEST_CASE(test_fl_modification_decoding_horiz)
 {
 	environment_setup env_setup;
@@ -300,7 +288,7 @@ BOOST_AUTO_TEST_CASE(test_fl_modification_decoding_horiz)
 	BOOST_CHECK(!mod->get_vert());
 }
 
-/// Tests if the FL modification is correctly decoded with the vert argument
+/** Tests if the FL modification is correctly decoded with the vert argument */
 BOOST_AUTO_TEST_CASE(test_fl_modification_decoding_vert)
 {
 	environment_setup env_setup;
@@ -318,7 +306,7 @@ BOOST_AUTO_TEST_CASE(test_fl_modification_decoding_vert)
 	BOOST_CHECK(mod->get_vert());
 }
 
-/// Tests if the FL modification is correctly decoded with both horiz and vert
+/** Tests if the FL modification is correctly decoded with both horiz and vert */
 BOOST_AUTO_TEST_CASE(test_fl_modification_decoding_horiz_and_vert)
 {
 	environment_setup env_setup;
@@ -336,7 +324,7 @@ BOOST_AUTO_TEST_CASE(test_fl_modification_decoding_horiz_and_vert)
 	BOOST_CHECK(mod->get_vert());
 }
 
-/// Tests if the GS modification is correctly decoded
+/** Tests if the GS modification is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_gs_modification_decoding)
 {
 	environment_setup env_setup;
@@ -351,7 +339,7 @@ BOOST_AUTO_TEST_CASE(test_gs_modification_decoding)
 	BOOST_CHECK(mod != nullptr);
 }
 
-/// Tests if the CROP modification without arguments is ignored
+/** Tests if the CROP modification without arguments is ignored */
 BOOST_AUTO_TEST_CASE(test_crop_modification_decoding_no_args)
 {
 	environment_setup env_setup;
@@ -361,7 +349,7 @@ BOOST_AUTO_TEST_CASE(test_crop_modification_decoding_no_args)
 	BOOST_REQUIRE_EQUAL(queue.size(), 0);
 }
 
-/// Tests if the CROP modification is correctly decoded when given one argument
+/** Tests if the CROP modification is correctly decoded when given one argument */
 BOOST_AUTO_TEST_CASE(test_crop_modification_decoding_1_arg)
 {
 	environment_setup env_setup;
@@ -383,7 +371,7 @@ BOOST_AUTO_TEST_CASE(test_crop_modification_decoding_1_arg)
 	BOOST_CHECK_EQUAL(slice.h, 0);
 }
 
-/// Tests if the CROP modification is correctly decoded when given two args
+/** Tests if the CROP modification is correctly decoded when given two args */
 BOOST_AUTO_TEST_CASE(test_crop_modification_decoding_2_args)
 {
 	environment_setup env_setup;
@@ -405,7 +393,7 @@ BOOST_AUTO_TEST_CASE(test_crop_modification_decoding_2_args)
 	BOOST_CHECK_EQUAL(slice.h, 0);
 }
 
-/// Tests if the CROP modification is correctly decoded when given three args
+/** Tests if the CROP modification is correctly decoded when given three args */
 BOOST_AUTO_TEST_CASE(test_crop_modification_decoding_3_args)
 {
 	environment_setup env_setup;
@@ -427,7 +415,7 @@ BOOST_AUTO_TEST_CASE(test_crop_modification_decoding_3_args)
 	BOOST_CHECK_EQUAL(slice.h, 0);
 }
 
-/// Tests if the CROP modification is correctly decoded when given four args
+/** Tests if the CROP modification is correctly decoded when given four args */
 BOOST_AUTO_TEST_CASE(test_crop_modification_decoding_4_args)
 {
 	environment_setup env_setup;
@@ -493,7 +481,7 @@ BOOST_AUTO_TEST_CASE(test_blit_modification_decoding_3_args)
 	BOOST_CHECK_EQUAL(mod->get_y(), 2);
 }
 
-/// Tests if the BLIT modification with invalid arguments is ignored
+/** Tests if the BLIT modification with invalid arguments is ignored */
 BOOST_AUTO_TEST_CASE(test_blit_modification_decoding_invalid_args)
 {
 	environment_setup env_setup;
@@ -552,7 +540,7 @@ BOOST_AUTO_TEST_CASE(test_mask_modification_decoding_3_args)
 	BOOST_CHECK_EQUAL(mod->get_y(), 4);
 }
 
-/// Tests if the MASK modification with invalid arguments is ignored
+/** Tests if the MASK modification with invalid arguments is ignored */
 BOOST_AUTO_TEST_CASE(test_mask_modification_decoding_invalid_args)
 {
 	environment_setup env_setup;
@@ -566,7 +554,7 @@ BOOST_AUTO_TEST_CASE(test_mask_modification_decoding_invalid_args)
 	BOOST_CHECK_EQUAL(queue.size(), 0);
 }
 
-/// Tests if the L modification without arguments is ignored
+/** Tests if the L modification without arguments is ignored */
 BOOST_AUTO_TEST_CASE(test_l_modification_decoding_no_args)
 {
 	environment_setup env_setup;
@@ -596,7 +584,7 @@ BOOST_AUTO_TEST_CASE(test_l_modification_decoding_1_arg)
 	BOOST_CHECK(mod->get_surface());
 }
 
-/// Tests if the SCALE modification without arguments is ignored
+/** Tests if the SCALE modification without arguments is ignored */
 BOOST_AUTO_TEST_CASE(test_scale_modification_decoding_no_args)
 {
 	environment_setup env_setup;
@@ -606,7 +594,7 @@ BOOST_AUTO_TEST_CASE(test_scale_modification_decoding_no_args)
 	BOOST_CHECK_EQUAL(queue.size(), 0);
 }
 
-/// Tests if the SCALE modification with one argument is correctly decoded
+/** Tests if the SCALE modification with one argument is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_scale_modification_decoding_1_arg)
 {
 	environment_setup env_setup;
@@ -624,7 +612,7 @@ BOOST_AUTO_TEST_CASE(test_scale_modification_decoding_1_arg)
 	BOOST_CHECK_EQUAL(mod->get_h(), 0);
 }
 
-/// Tests if the SCALE modification with two arguments is correctly decoded
+/** Tests if the SCALE modification with two arguments is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_scale_modification_decoding_2_args)
 {
 	environment_setup env_setup;
@@ -642,7 +630,7 @@ BOOST_AUTO_TEST_CASE(test_scale_modification_decoding_2_args)
 	BOOST_CHECK_EQUAL(mod->get_h(), 5);
 }
 
-/// Tests if the O modification with a percent argument is correctly decoded
+/** Tests if the O modification with a percent argument is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_o_modification_decoding_percent_args)
 {
 	environment_setup env_setup;
@@ -660,7 +648,7 @@ BOOST_AUTO_TEST_CASE(test_o_modification_decoding_percent_args)
 	BOOST_CHECK(mod->get_opacity() < 0.46f);
 }
 
-/// Tests if the O modification with a fraction argument is correctly decoded
+/** Tests if the O modification with a fraction argument is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_o_modification_decoding_fraction_args)
 {
 	environment_setup env_setup;
@@ -678,7 +666,7 @@ BOOST_AUTO_TEST_CASE(test_o_modification_decoding_fraction_args)
 	BOOST_CHECK(mod->get_opacity() < 0.35f);
 }
 
-/// Tests if the BL modification without arguments is correctly decoded
+/** Tests if the BL modification without arguments is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_bl_modification_decoding_no_args)
 {
 	environment_setup env_setup;
@@ -695,7 +683,7 @@ BOOST_AUTO_TEST_CASE(test_bl_modification_decoding_no_args)
 	BOOST_CHECK_EQUAL(mod->get_depth(), 0);
 }
 
-/// Tests if the BL modification with one argument is correctly decoded
+/** Tests if the BL modification with one argument is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_bl_modification_decoding)
 {
 	environment_setup env_setup;
@@ -712,7 +700,7 @@ BOOST_AUTO_TEST_CASE(test_bl_modification_decoding)
 	BOOST_CHECK_EQUAL(mod->get_depth(), 2);
 }
 
-/// Tests if the R, G and B modifications without args are correctly decoded
+/** Tests if the R, G and B modifications without args are correctly decoded */
 BOOST_AUTO_TEST_CASE(test_rgb_modification_decoding_no_args)
 {
 	environment_setup env_setup;
@@ -737,7 +725,7 @@ BOOST_AUTO_TEST_CASE(test_rgb_modification_decoding_no_args)
 	}
 }
 
-/// Tests if the R modification with one argument is correctly decoded
+/** Tests if the R modification with one argument is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_r_modification_decoding)
 {
 	environment_setup env_setup;
@@ -756,7 +744,7 @@ BOOST_AUTO_TEST_CASE(test_r_modification_decoding)
 	BOOST_CHECK_EQUAL(mod->get_b(), 0);
 }
 
-/// Tests if the G modification with one argument is correctly decoded
+/** Tests if the G modification with one argument is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_g_modification_decoding)
 {
 	environment_setup env_setup;
@@ -775,7 +763,7 @@ BOOST_AUTO_TEST_CASE(test_g_modification_decoding)
 	BOOST_CHECK_EQUAL(mod->get_b(), 0);
 }
 
-/// Tests if the B modification with one argument is correctly decoded
+/** Tests if the B modification with one argument is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_b_modification_decoding)
 {
 	environment_setup env_setup;
@@ -794,7 +782,7 @@ BOOST_AUTO_TEST_CASE(test_b_modification_decoding)
 	BOOST_CHECK_EQUAL(mod->get_b(), 312);
 }
 
-/// Tests if the BG modification without arguments is correctly decoded
+/** Tests if the BG modification without arguments is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_bg_modification_decoding_no_args)
 {
 	environment_setup env_setup;
@@ -814,7 +802,7 @@ BOOST_AUTO_TEST_CASE(test_bg_modification_decoding_no_args)
 	BOOST_CHECK_EQUAL(mod->get_color().a, SDL_ALPHA_OPAQUE);
 }
 
-/// Tests if the BG modification with one argument is correctly decoded
+/** Tests if the BG modification with one argument is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_bg_modification_decoding_1_arg)
 {
 	environment_setup env_setup;
@@ -834,7 +822,7 @@ BOOST_AUTO_TEST_CASE(test_bg_modification_decoding_1_arg)
 	BOOST_CHECK_EQUAL(mod->get_color().a, SDL_ALPHA_OPAQUE);
 }
 
-/// Tests if the BG modification with two arguments is correctly decoded
+/** Tests if the BG modification with two arguments is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_bg_modification_decoding_2_args)
 {
 	environment_setup env_setup;
@@ -854,7 +842,7 @@ BOOST_AUTO_TEST_CASE(test_bg_modification_decoding_2_args)
 	BOOST_CHECK_EQUAL(mod->get_color().a, SDL_ALPHA_OPAQUE);
 }
 
-/// Tests if the BG modification with three arguments is correctly decoded
+/** Tests if the BG modification with three arguments is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_bg_modification_decoding_3_args)
 {
 	environment_setup env_setup;
@@ -874,7 +862,7 @@ BOOST_AUTO_TEST_CASE(test_bg_modification_decoding_3_args)
 	BOOST_CHECK_EQUAL(mod->get_color().a, SDL_ALPHA_OPAQUE);
 }
 
-/// Tests if the BG modification with four arguments is correctly decoded
+/** Tests if the BG modification with four arguments is correctly decoded */
 BOOST_AUTO_TEST_CASE(test_bg_modification_decoding_4_args)
 {
 	environment_setup env_setup;
