@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2009 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2009 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-test"
@@ -22,7 +23,6 @@
 #include <boost/mpl/copy.hpp>
 #include <boost/mpl/back_inserter.hpp>
 #include <boost/mpl/contains.hpp>
-#include <boost/test/test_case_template.hpp>
 
 #include <iostream>
 
@@ -156,8 +156,7 @@ BOOST_AUTO_TEST_CASE(test_lexical_cast_long_long)
 }
 
 typedef boost::mpl::vector<
-	  bool
-	, unsigned char
+	  unsigned char
 	, unsigned short
 	, unsigned int
 	, unsigned long> test_lexical_cast_unsigned_types;
@@ -196,6 +195,22 @@ BOOST_AUTO_TEST_CASE(test_lexical_cast_unsigned_long_long)
 			std::string(value)), const char*, validate);
 }
 
+BOOST_AUTO_TEST_CASE(test_lexical_cast_bool)
+{
+	result = "specialized - To bool - From (const) char*";
+
+	const char* value = "test";
+	BOOST_CHECK_EXCEPTION(lexical_cast<bool>(
+			value), const char*, validate);
+	BOOST_CHECK_EXCEPTION(lexical_cast<bool>(
+			const_cast<char*>(value)), const char*, validate);
+
+	result = "specialized - To bool - From std::string";
+
+	BOOST_CHECK_EXCEPTION(lexical_cast<bool>(
+			std::string(value)), const char*, validate);
+}
+
 typedef boost::mpl::vector<
 	  float
 	, double
