@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2013 - 2018 by Andrius Silinskas <silinskas.andrius@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2013 - 2021
+	by Andrius Silinskas <silinskas.andrius@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-test"
@@ -53,18 +54,17 @@ struct mp_connect_fixture {
 		hotkey_manager(),
 		config_manager()
 	{
-
-		config_manager.reset(new game_config_manager(cmdline_opts, false));
+		config_manager.reset(new game_config_manager(cmdline_opts));
 		config_manager->init_game_config(game_config_manager::NO_FORCE_RELOAD);
 
 		state.reset(new saved_game());
 		state->classification().campaign_type = game_classification::CAMPAIGN_TYPE::MULTIPLAYER;
-		config_manager->load_game_config_for_game(state->classification());
+		state->classification().era_id = "era_default";
+		config_manager->load_game_config_for_game(state->classification(), state->get_scenario_id());
 
-		state->mp_settings().mp_era = "era_default";
 		state->mp_settings().name = "multiplayer_The_Freelands";
 		state->mp_settings().use_map_settings = true;
-		state->mp_settings().saved_game = false;
+		state->mp_settings().saved_game = mp_game_settings::SAVED_GAME_MODE::NONE;
 
 		state->set_scenario(config_manager->
 			game_config().find_child("multiplayer", "id", state->mp_settings().name));
@@ -115,7 +115,7 @@ BOOST_AUTO_TEST_CASE( flg_map_settings )
 {
 	// Set up side_engine and its dependencies.
 	state->mp_settings().use_map_settings = true;
-	state->mp_settings().saved_game = false;
+	state->mp_settings().saved_game = mp_game_settings::SAVED_GAME_MODE::NONE;
 	std::unique_ptr<test_connect_engine>
 		connect_engine(create_test_connect_engine());
 	ng::side_engine_ptr side_engine;
@@ -325,7 +325,7 @@ BOOST_AUTO_TEST_CASE( flg_no_map_settings )
 {
 	// Set up side_engine and its dependencies.
 	state->mp_settings().use_map_settings = false;
-	state->mp_settings().saved_game = false;
+	state->mp_settings().saved_game = mp_game_settings::SAVED_GAME_MODE::NONE;
 	const std::unique_ptr<test_connect_engine>
 		connect_engine(create_test_connect_engine());
 	ng::side_engine_ptr side_engine;
