@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #include "theme.hpp"
 
 #include "desktop/battery_info.hpp"
@@ -25,7 +24,7 @@
 #include "sdl/rect.hpp"
 #include "serialization/string_utils.hpp"
 #include "wml_exception.hpp"
-
+#include "game_config_view.hpp"
 #include <sstream>
 #include <utility>
 
@@ -36,18 +35,16 @@ static lg::log_domain log_display("display");
 
 namespace
 {
-const int XDim = 1024;
-const int YDim = 768;
 
-const size_t DefaultFontSize = font::SIZE_NORMAL;
+const std::size_t DefaultFontSize = font::SIZE_NORMAL;
 const color_t DefaultFontRGB {200, 200, 200};
 
 _rect ref_rect {0, 0, 0, 0};
 }
 
-static size_t compute(std::string expr, size_t ref1, size_t ref2 = 0)
+static std::size_t compute(std::string expr, std::size_t ref1, std::size_t ref2 = 0)
 {
-	size_t ref = 0;
+	std::size_t ref = 0;
 	if(expr[0] == '=') {
 		ref = ref1;
 		expr = expr.substr(1);
@@ -283,10 +280,12 @@ theme::object::object()
 	, last_screen_(sdl::empty_rect)
 	, xanchor_(object::FIXED)
 	, yanchor_(object::FIXED)
+	, spec_width_(0)
+	, spec_height_(0)
 {
 }
 
-theme::object::object(const config& cfg)
+theme::object::object(std::size_t sw, std::size_t sh, const config& cfg)
 	: location_modified_(false)
 	, id_(cfg["id"])
 	, loc_(read_sdl_rect(cfg))
@@ -294,6 +293,8 @@ theme::object::object(const config& cfg)
 	, last_screen_(sdl::empty_rect)
 	, xanchor_(read_anchor(cfg["xanchor"]))
 	, yanchor_(read_anchor(cfg["yanchor"]))
+	, spec_width_(sw)
+	, spec_height_(sh)
 {
 }
 
@@ -328,15 +329,15 @@ SDL_Rect& theme::object::location(const SDL_Rect& screen) const
 			break;
 		case TOP_ANCHORED:
 			relative_loc_.x = loc_.x;
-			relative_loc_.w = screen.w - std::min<size_t>(XDim - loc_.w, screen.w);
+			relative_loc_.w = loc_.w + screen.w - std::min<std::size_t>(spec_width_, loc_.w + screen.w);
 			break;
 		case BOTTOM_ANCHORED:
-			relative_loc_.x = screen.w - std::min<size_t>(XDim - loc_.x, screen.w);
+			relative_loc_.x = loc_.x + screen.w - std::min<std::size_t>(spec_width_, loc_.x + screen.w);
 			relative_loc_.w = loc_.w;
 			break;
 		case PROPORTIONAL:
-			relative_loc_.x = (loc_.x * screen.w) / XDim;
-			relative_loc_.w = (loc_.w * screen.w) / XDim;
+			relative_loc_.x = (loc_.x * screen.w) / spec_width_;
+			relative_loc_.w = (loc_.w * screen.w) / spec_width_;
 			break;
 		default:
 			assert(false);
@@ -349,23 +350,30 @@ SDL_Rect& theme::object::location(const SDL_Rect& screen) const
 			break;
 		case TOP_ANCHORED:
 			relative_loc_.y = loc_.y;
-			relative_loc_.h = screen.h - std::min<size_t>(YDim - loc_.h, screen.h);
+			relative_loc_.h = loc_.h + screen.h - std::min<std::size_t>(spec_height_, loc_.h + screen.h);
 			break;
 		case BOTTOM_ANCHORED:
-			relative_loc_.y = screen.h - std::min<size_t>(YDim - loc_.y, screen.h);
+			relative_loc_.y = loc_.y + screen.h - std::min<std::size_t>(spec_height_, loc_.y + screen.h);
 			relative_loc_.h = loc_.h;
 			break;
 		case PROPORTIONAL:
-			relative_loc_.y = (loc_.y * screen.h) / YDim;
-			relative_loc_.h = (loc_.h * screen.h) / YDim;
+			relative_loc_.y = (loc_.y * screen.h) / spec_height_;
+			relative_loc_.h = (loc_.h * screen.h) / spec_height_;
 			break;
 		default:
 			assert(false);
 	}
 
-	relative_loc_.x = std::min<int>(relative_loc_.x, screen.w);
+	relative_loc_.w = std::min<int>(relative_loc_.w, screen.w);
+	if(relative_loc_.x > screen.w) {
+		relative_loc_.x = std::min<int>(relative_loc_.x, screen.w - relative_loc_.w);
+	}
 	relative_loc_.w = std::min<int>(relative_loc_.w, screen.w - relative_loc_.x);
-	relative_loc_.y = std::min<int>(relative_loc_.y, screen.h);
+
+	relative_loc_.h = std::min<int>(relative_loc_.h, screen.h);
+	if(relative_loc_.y > screen.h) {
+		relative_loc_.y = std::min<int>(relative_loc_.y, screen.h - relative_loc_.h);
+	}
 	relative_loc_.h = std::min<int>(relative_loc_.h, screen.h - relative_loc_.y);
 
 	return relative_loc_;
@@ -422,9 +430,9 @@ theme::label::label()
 {
 }
 
-theme::label::label(const config& cfg)
-	: object(cfg)
-	, text_(cfg["prefix"].str() + cfg["text"].str() + cfg["postfix"].str())
+theme::label::label(std::size_t sw, std::size_t sh, const config& cfg)
+	: object(sw, sh, cfg)
+	, text_(cfg["prefix"].str() + cfg["prefix_literal"].str() + cfg["text"].str() + cfg["postfix_literal"].str() + cfg["postfix"].str())
 	, icon_(cfg["icon"])
 	, font_(cfg["font_size"])
 	, font_rgb_set_(false)
@@ -439,8 +447,8 @@ theme::label::label(const config& cfg)
 	}
 }
 
-theme::status_item::status_item(const config& cfg)
-	: object(cfg)
+theme::status_item::status_item(std::size_t sw, std::size_t sh, const config& cfg)
+	: object(sw, sh, cfg)
 	, prefix_(cfg["prefix"].str() + cfg["prefix_literal"].str())
 	, postfix_(cfg["postfix_literal"].str() + cfg["postfix"].str())
 	, label_()
@@ -452,7 +460,7 @@ theme::status_item::status_item(const config& cfg)
 		font_ = DefaultFontSize;
 
 	if(const config& label_child = cfg.child("label")) {
-		label_ = label(label_child);
+		label_ = label(sw, sh, label_child);
 	}
 
 	if(cfg.has_attribute("font_rgb")) {
@@ -461,8 +469,8 @@ theme::status_item::status_item(const config& cfg)
 	}
 }
 
-theme::panel::panel(const config& cfg)
-	: object(cfg)
+theme::panel::panel(std::size_t sw, std::size_t sh, const config& cfg)
+	: object(sw, sh, cfg)
 	, image_(cfg["image"])
 {
 }
@@ -476,8 +484,8 @@ theme::slider::slider()
 	, black_line_(false)
 {
 }
-theme::slider::slider(const config& cfg)
-	: object(cfg)
+theme::slider::slider(std::size_t sw, std::size_t sh, const config& cfg)
+	: object(sw, sh, cfg)
 	, title_(cfg["title"].str() + cfg["title_literal"].str())
 	, tooltip_(cfg["tooltip"])
 	, image_(cfg["image"])
@@ -498,8 +506,8 @@ theme::menu::menu()
 {
 }
 
-theme::menu::menu(const config& cfg)
-	: object(cfg)
+theme::menu::menu(std::size_t sw, std::size_t sh, const config& cfg)
+	: object(sw, sh, cfg)
 	, button_(cfg["button"].to_bool(true))
 	, context_(cfg["is_context_menu"].to_bool(false))
 	, title_(cfg["title"].str() + cfg["title_literal"].str())
@@ -534,8 +542,8 @@ theme::action::action()
 {
 }
 
-theme::action::action(const config& cfg)
-	: object(cfg)
+theme::action::action(std::size_t sw, std::size_t sh, const config& cfg)
+	: object(sw, sh, cfg)
 	, context_(cfg["is_context_menu"].to_bool())
 	, auto_tooltip_(cfg["auto_tooltip"].to_bool(false))
 	, tooltip_name_prepend_(cfg["tooltip_name_prepend"].to_bool(false))
@@ -548,7 +556,7 @@ theme::action::action(const config& cfg)
 {
 }
 
-const std::string theme::action::tooltip(size_t index) const
+const std::string theme::action::tooltip(std::size_t index) const
 {
 	std::stringstream result;
 	if(auto_tooltip_ && tooltip_.empty() && items_.size() > index) {
@@ -584,32 +592,14 @@ theme::theme(const config& cfg, const SDL_Rect& screen)
 	, palette_()
 	, border_()
 	, screen_dimensions_(screen)
+	, cur_spec_width_(0)
+	, cur_spec_height_(0)
 {
 	do_resolve_rects(expand_partialresolution(cfg), cfg_);
 	set_resolution(screen);
 }
 
-theme& theme::operator=(theme&& other)
-{
-	theme_reset_event_ = other.theme_reset_event_;
-	cur_theme = std::move(other.cur_theme);
-	cfg_ = std::move(other.cfg_);
-	panels_ = std::move(other.panels_);
-	labels_ = std::move(other.labels_);
-	menus_ = std::move(other.menus_);
-	actions_ = std::move(other.actions_);
-	sliders_ = std::move(other.sliders_);
-	context_ = other.context_;
-	action_context_ = other.action_context_;
-	status_ = std::move(other.status_);
-	main_map_ = other.main_map_;
-	mini_map_ = other.mini_map_;
-	unit_image_ = other.unit_image_;
-	palette_ = other.palette_;
-	border_ = other.border_;
-
-	return *this;
-}
+theme& theme::operator=(theme&& other) = default;
 
 bool theme::set_resolution(const SDL_Rect& screen)
 {
@@ -643,6 +633,8 @@ bool theme::set_resolution(const SDL_Rect& screen)
 		}
 		return false;
 	}
+	cur_spec_width_ = (*current)["width"];
+	cur_spec_height_ = (*current)["height"];
 
 	std::map<std::string, std::string> title_stash_menus;
 	std::vector<theme::menu>::iterator m;
@@ -665,7 +657,7 @@ bool theme::set_resolution(const SDL_Rect& screen)
 	actions_.clear();
 	sliders_.clear();
 
-	add_object(*current);
+	add_object(cur_spec_width_, cur_spec_height_, *current);
 
 	for(m = menus_.begin(); m != menus_.end(); ++m) {
 		if(title_stash_menus.find(m->get_id()) != title_stash_menus.end())
@@ -682,45 +674,45 @@ bool theme::set_resolution(const SDL_Rect& screen)
 	return result;
 }
 
-void theme::add_object(const config& cfg)
+void theme::add_object(std::size_t sw, std::size_t sh, const config& cfg)
 {
 	if(const config& c = cfg.child("main_map")) {
-		main_map_ = object(c);
+		main_map_ = object(sw, sh, c);
 	}
 
 	if(const config& c = cfg.child("mini_map")) {
-		mini_map_ = object(c);
+		mini_map_ = object(sw, sh, c);
 	}
 
 	if(const config& c = cfg.child("palette")) {
-		palette_ = object(c);
+		palette_ = object(sw, sh, c);
 	}
 
 	if(const config& status_cfg = cfg.child("status")) {
 		for(const config::any_child& i : status_cfg.all_children_range()) {
-			status_[i.key].reset(new status_item(i.cfg));
+			status_[i.key].reset(new status_item(sw, sh, i.cfg));
 		}
 		if(const config& unit_image_cfg = status_cfg.child("unit_image")) {
-			unit_image_ = object(unit_image_cfg);
+			unit_image_ = object(sw, sh, unit_image_cfg);
 		} else {
 			unit_image_ = object();
 		}
 	}
 
 	for(const config& p : cfg.child_range("panel")) {
-		panel new_panel(p);
+		panel new_panel(sw, sh, p);
 		set_object_location(new_panel, p["rect"], p["ref"]);
 		panels_.push_back(new_panel);
 	}
 
 	for(const config& lb : cfg.child_range("label")) {
-		label new_label(lb);
+		label new_label(sw, sh, lb);
 		set_object_location(new_label, lb["rect"], lb["ref"]);
 		labels_.push_back(new_label);
 	}
 
 	for(const config& m : cfg.child_range("menu")) {
-		menu new_menu(m);
+		menu new_menu(sw, sh, m);
 		DBG_DP << "adding menu: " << (new_menu.is_context() ? "is context" : "not context") << "\n";
 		if(new_menu.is_context())
 			context_ = new_menu;
@@ -733,7 +725,7 @@ void theme::add_object(const config& cfg)
 	}
 
 	for(const config& a : cfg.child_range("action")) {
-		action new_action(a);
+		action new_action(sw, sh, a);
 		DBG_DP << "adding action: " << (new_action.is_context() ? "is context" : "not context") << "\n";
 		if(new_action.is_context())
 			action_context_ = new_action;
@@ -746,7 +738,7 @@ void theme::add_object(const config& cfg)
 	}
 
 	for(const config& s : cfg.child_range("slider")) {
-		slider new_slider(s);
+		slider new_slider(sw, sh, s);
 		DBG_DP << "adding slider\n";
 		set_object_location(new_slider, s["rect"], s["ref"]);
 		sliders_.push_back(new_slider);
@@ -850,7 +842,7 @@ void theme::modify(const config& cfg)
 
 	// Add new theme objects.
 	for(const config& c : cfg.child_range("add")) {
-		add_object(c);
+		add_object(cur_spec_width_, cur_spec_height_, c);
 	}
 
 	// Remove existent theme objects.
@@ -925,7 +917,7 @@ const theme::status_item* theme::get_status_item(const std::string& key) const
 typedef std::map<std::string, config> known_themes_map;
 known_themes_map theme::known_themes;
 
-void theme::set_known_themes(const config* cfg)
+void theme::set_known_themes(const game_config_view* cfg)
 {
 	known_themes.clear();
 	if(!cfg)
@@ -1001,7 +993,7 @@ theme::object* theme::refresh_title2(const std::string& id, const std::string& t
 	if(!cfg[title_tag].empty())
 		new_title = cfg[title_tag].str();
 
-	return refresh_title(id, new_title);
+	return refresh_title(id, new_title + cfg["title_literal"].str());
 }
 
 void theme::modify_label(const std::string& id, const std::string& text)
