@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -26,7 +27,9 @@
 #include <memory>
 #include <SDL2/SDL_rect.h>
 
-struct _rect { size_t x1,y1,x2,y2; };
+class game_config_view;
+
+struct _rect { std::size_t x1,y1,x2,y2; };
 
 struct theme_info
 {
@@ -42,7 +45,7 @@ class theme
 	{
 	public:
 		object();
-		object(const config& cfg);
+		object(std::size_t sw, std::size_t sh, const config& cfg);
 		virtual ~object() { }
 
 		virtual SDL_Rect& location(const SDL_Rect& screen) const;
@@ -72,6 +75,7 @@ class theme
 		mutable SDL_Rect last_screen_;
 
 		ANCHORING xanchor_, yanchor_;
+		std::size_t spec_width_, spec_height_;
 
 		static ANCHORING read_anchor(const std::string& str);
 	};
@@ -96,7 +100,7 @@ public:
 	{
 	public:
 		label();
-		explicit label(const config& cfg);
+		explicit label(std::size_t sw, std::size_t sh, const config& cfg);
 
 		using object::location;
 
@@ -106,12 +110,12 @@ public:
 
 		bool empty() const { return text_.empty() && icon_.empty(); }
 
-		size_t font_size() const { return font_; }
+		std::size_t font_size() const { return font_; }
 		color_t font_rgb() const { return font_rgb_; }
 		bool font_rgb_set() const { return font_rgb_set_; }
 	private:
 		std::string text_, icon_;
-		size_t font_;
+		std::size_t font_;
 		bool font_rgb_set_;
 		color_t font_rgb_;
 	};
@@ -120,7 +124,7 @@ public:
 	{
 	public:
 
-		explicit status_item(const config& cfg);
+		explicit status_item(std::size_t sw, std::size_t sh, const config& cfg);
 
 		using object::location;
 
@@ -130,14 +134,14 @@ public:
 		// If the item has a label associated with it, Show where the label is
 		const label* get_label() const { return label_.empty() ? nullptr : &label_; }
 
-		size_t font_size() const { return font_; }
+		std::size_t font_size() const { return font_; }
 		color_t font_rgb() const { return font_rgb_; }
 		bool font_rgb_set() const { return font_rgb_set_; }
 
 	private:
 		std::string prefix_, postfix_;
 		label label_;
-		size_t font_;
+		std::size_t font_;
 		bool font_rgb_set_;
 		color_t font_rgb_;
 	};
@@ -145,7 +149,7 @@ public:
 	class panel : public object
 	{
 	public:
-		explicit panel(const config& cfg);
+		explicit panel(std::size_t sw, std::size_t sh, const config& cfg);
 
 		using object::location;
 
@@ -159,7 +163,7 @@ public:
 	{
 	public:
 		action();
-		explicit action(const config& cfg);
+		explicit action(std::size_t sw, std::size_t sh, const config& cfg);
 
 		using object::location;
 
@@ -167,7 +171,7 @@ public:
 
 		const std::string& title() const { return title_; }
 
-		const std::string tooltip(size_t index) const;
+		const std::string tooltip(std::size_t index) const;
 
 		const std::string& type() const { return type_; }
 
@@ -188,7 +192,7 @@ public:
 	{
 	public:
 		slider();
-		explicit slider(const config& cfg);
+		explicit slider(std::size_t sw, std::size_t sh, const config& cfg);
 
 		using object::location;
 
@@ -212,7 +216,7 @@ public:
 	{
 	public:
 		menu();
-		explicit menu(const config& cfg);
+		explicit menu(std::size_t sw, std::size_t sh, const config& cfg);
 
 		using object::location;
 
@@ -274,7 +278,7 @@ public:
 	const SDL_Rect& palette_location(const SDL_Rect& screen) const
 		{ return palette_.location(screen); }
 
-    static void set_known_themes(const config* cfg);
+    static void set_known_themes(const game_config_view* cfg);
     static std::vector<theme_info> get_known_themes();
 
 	const border_t& border() const { return border_; }
@@ -283,7 +287,7 @@ public:
 
 private:
 	theme::object& find_element(const std::string& id);
-	void add_object(const config& cfg);
+	void add_object(std::size_t sw, std::size_t sh, const config& cfg);
 	void remove_object(const std::string& id);
 	void set_object_location(theme::object& element, std::string rect_str, std::string ref_id);
 
@@ -310,4 +314,5 @@ private:
 	border_t border_;
 
 	SDL_Rect screen_dimensions_;
+	std::size_t cur_spec_width_, cur_spec_height_;
 };
