@@ -1,24 +1,23 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #pragma once
 
 #include "tstring.hpp"
-#include "utils/general.hpp"
 
+#include <algorithm>
 #include <vector>
 
 class config;
@@ -27,9 +26,9 @@ class config;
 // This is a color delta, so do not replace with color_t!
 struct tod_color {
 	explicit tod_color(int red = 0, int green = 0, int blue = 0)
-		: r(utils::clamp(red, -510, 510))
-		, g(utils::clamp(green, -510, 510))
-		, b(utils::clamp(blue, -510, 510))
+		: r(std::clamp(red, -510, 510))
+		, g(std::clamp(green, -510, 510))
+		, b(std::clamp(blue, -510, 510))
 	{}
 	bool operator==(const tod_color& o) const {
 		return r == o.r && g == o.g && b == o.b;
