@@ -1,16 +1,17 @@
 /*
-   Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
-   Copyright (C) 2005 - 2018 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2004 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -42,14 +43,11 @@ const char UNTRANSLATABLE_PART = 0x02;
 const char TEXTDOMAIN_SEPARATOR = 0x03;
 const char ID_TRANSLATABLE_PART = 0x04;
 const char PLURAL_PART = 0x05;
-
-std::vector<std::string> id_to_textdomain;
-std::map<std::string, unsigned int> textdomain_to_id;
 }
 
-size_t t_string_base::hash_value() const
+std::size_t t_string_base::hash_value() const
 {
-	size_t seed = 0;
+	std::size_t seed = 0;
 	boost::hash_combine(seed, value_);
 	boost::hash_combine(seed, translatable_);
 	boost::hash_combine(seed, last_untranslatable_);
@@ -115,7 +113,7 @@ void t_string_base::walker::update()
 			end_ = string_.size();
 		}
 
-		id = string_[begin_ + 1] + string_[begin_ + 2] * 256;
+		id = static_cast<unsigned char>(string_[begin_ + 1]) + static_cast<unsigned char>(string_[begin_ + 2]) * 256;
 		if(id >= id_to_textdomain.size()) {
 			ERR_CF << "Error: invalid string: " << string_ << std::endl;
 			begin_ = string_.size();
@@ -275,8 +273,8 @@ t_string_base::t_string_base(const std::string& string, const std::string& textd
 		id = idi->second;
 	}
 
-	value_ += char(id & 0xff);
-	value_ += char(id >> 8);
+	value_ += static_cast<char>(id & 0xff);
+	value_ += static_cast<char>(id >> 8);
 	value_ += string;
 }
 
@@ -304,8 +302,8 @@ t_string_base::t_string_base(const std::string& sing, const std::string& pl, int
 		id = idi->second;
 	}
 
-	value_ += char(id & 0xff);
-	value_ += char(id >> 8);
+	value_ += static_cast<char>(id & 0xff);
+	value_ += static_cast<char>(id >> 8);
 	value_ += sing;
 	value_ += PLURAL_PART;
 
