@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2006 - 2018 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "units/animation.hpp"
@@ -375,7 +376,7 @@ unit_animation::unit_animation(const config& cfg,const std::string& frame_string
 }
 
 int unit_animation::matches(const display& disp, const map_location& loc, const map_location& second_loc,
-		const unit* my_unit, const std::string& event, const int value, hit_type hit, const_attack_ptr attack,
+		unit_const_ptr my_unit, const std::string& event, const int value, hit_type hit, const_attack_ptr attack,
 		const_attack_ptr second_attack, int value2) const
 {
 	int result = base_score_;
@@ -636,6 +637,7 @@ void unit_animation::add_anims( std::vector<unit_animation> & animations, const
 
 	add_simple_anim(animations, cfg, "resistance_anim", "resistance");
 	add_simple_anim(animations, cfg, "leading_anim", "leading");
+	add_simple_anim(animations, cfg, "teaching_anim", "teaching");
 	add_simple_anim(animations, cfg, "recruit_anim", "recruited");
 	add_simple_anim(animations, cfg, "recruiting_anim", "recruiting");
 	add_simple_anim(animations, cfg, "idle_anim", "idling", display::LAYER_UNIT_DEFAULT, false);
@@ -650,7 +652,7 @@ void unit_animation::add_anims( std::vector<unit_animation> & animations, const
 		// Add cycles to all frames within a standing animation block
 		for(config::const_all_children_iterator ci : ab.children) {
 			std::string sub_frame_name = ci->key;
-			size_t pos = sub_frame_name.find("_frame");
+			std::size_t pos = sub_frame_name.find("_frame");
 			if(pos != std::string::npos) {
 				anim[sub_frame_name.substr(0, pos) + "_cycles"] = true;
 			}
@@ -675,7 +677,7 @@ void unit_animation::add_anims( std::vector<unit_animation> & animations, const
 
 		for(config::const_all_children_iterator ci : ab.children) {
 			std::string sub_frame_name = ci->key;
-			size_t pos = sub_frame_name.find("_frame");
+			std::size_t pos = sub_frame_name.find("_frame");
 			if(pos != std::string::npos) {
 				anim[sub_frame_name.substr(0, pos) + "_cycles"] = true;
 			}
@@ -1197,7 +1199,7 @@ std::ostream& operator<<(std::ostream& outstream, const unit_animation& u_animat
 
 	if(u_animation.primary_attack_filter_.size() > 0) {
 		outstream << "[filter_attack]\n";
-		for(const config cfg : u_animation.primary_attack_filter_) {
+		for(const config& cfg : u_animation.primary_attack_filter_) {
 			outstream << cfg.debug();
 		}
 
@@ -1206,31 +1208,31 @@ std::ostream& operator<<(std::ostream& outstream, const unit_animation& u_animat
 
 	if(u_animation.secondary_attack_filter_.size() > 0) {
 		outstream << "[filter_second_attack]\n";
-		for(const config cfg : u_animation.secondary_attack_filter_) {
+		for(const config& cfg : u_animation.secondary_attack_filter_) {
 			outstream << cfg.debug();
 		}
 
 		outstream << "[/filter_second_attack]\n";
 	}
 
-	for(size_t i = 0; i < u_animation.unit_anim_.get_frames_count(); i++) {
+	for(std::size_t i = 0; i < u_animation.unit_anim_.get_frames_count(); i++) {
 		outstream << "\t[frame]\n";
-		for(const std::string frame_string : u_animation.unit_anim_.get_frame(i).debug_strings()) {
+		for(const std::string& frame_string : u_animation.unit_anim_.get_frame(i).debug_strings()) {
 			outstream << "\t\t" << frame_string <<"\n";
 		}
 		outstream << "\t[/frame]\n";
 	}
 
 	for(std::pair<std::string, unit_animation::particle> p : u_animation.sub_anims_) {
-		for(size_t i = 0; i < p.second.get_frames_count(); i++) {
+		for(std::size_t i = 0; i < p.second.get_frames_count(); i++) {
 			std::string sub_frame_name = p.first;
-			size_t pos = sub_frame_name.find("_frame");
+			std::size_t pos = sub_frame_name.find("_frame");
 			if(pos != std::string::npos) sub_frame_name = sub_frame_name.substr(0, pos);
 
 			outstream << "\t" << sub_frame_name << "_start_time=" << p.second.get_begin_time() << '\n';
 			outstream << "\t[" << p.first << "]\n";
 
-			for(const std::string frame_string : p.second.get_frame(i).debug_strings()) {
+			for(const std::string& frame_string : p.second.get_frame(i).debug_strings()) {
 				outstream << "\t\t" << frame_string << '\n';
 			}
 
@@ -1291,7 +1293,7 @@ void unit_animation::particle::start_animation(int start_time)
 	last_frame_begin_time_ = get_begin_time() -1;
 }
 
-void unit_animator::add_animation(const unit* animated_unit
+void unit_animator::add_animation(unit_const_ptr animated_unit
 		, const std::string& event
 		, const map_location &src
 		, const map_location &dst
@@ -1309,12 +1311,12 @@ void unit_animator::add_animation(const unit* animated_unit
 	display* disp = display::get_singleton();
 
 	anim_elem tmp;
-	tmp.my_unit = unit_const_ptr(animated_unit);
+	tmp.my_unit = std::move(animated_unit);
 	tmp.text = text;
 	tmp.text_color = text_color;
 	tmp.src = src;
 	tmp.with_bars= with_bars;
-	tmp.animation = animated_unit->anim_comp().choose_animation(*disp, src, event, dst, value, hit_type, attack, second_attack, value2);
+	tmp.animation = tmp.my_unit->anim_comp().choose_animation(*disp, src, event, dst, value, hit_type, attack, second_attack, value2);
 
 	if(!tmp.animation) return;
 
@@ -1322,7 +1324,7 @@ void unit_animator::add_animation(const unit* animated_unit
 	animated_units_.push_back(std::move(tmp));
 }
 
-void unit_animator::add_animation(const unit* animated_unit
+void unit_animator::add_animation(unit_const_ptr animated_unit
 	, const unit_animation* anim
 	, const map_location &src
 	, bool with_bars
@@ -1332,7 +1334,7 @@ void unit_animator::add_animation(const unit* animated_unit
 	if(!animated_unit) return;
 
 	anim_elem tmp;
-	tmp.my_unit = unit_const_ptr(animated_unit);
+	tmp.my_unit = std::move(animated_unit);
 	tmp.text = text;
 	tmp.text_color = text_color;
 	tmp.src = src;
@@ -1345,7 +1347,21 @@ void unit_animator::add_animation(const unit* animated_unit
 	animated_units_.push_back(std::move(tmp));
 }
 
-void unit_animator::replace_anim_if_invalid(const unit* animated_unit
+bool unit_animator::has_animation(unit_const_ptr animated_unit
+		, const std::string& event
+		, const map_location &src
+		, const map_location &dst
+		, const int value
+		, const unit_animation::hit_type hit_type
+		, const_attack_ptr attack
+		, const_attack_ptr second_attack
+		, int value2) const
+{
+	display* disp = display::get_singleton();
+	return (animated_unit && animated_unit->anim_comp().choose_animation(*disp, src, event, dst, value, hit_type, attack, second_attack, value2));
+}
+
+void unit_animator::replace_anim_if_invalid(unit_const_ptr animated_unit
 	, const std::string& event
 	, const map_location &src
 	, const map_location & dst
@@ -1367,7 +1383,7 @@ void unit_animator::replace_anim_if_invalid(const unit* animated_unit
 			*disp, src, dst, animated_unit, event, value, hit_type, attack, second_attack, value2) > unit_animation::MATCH_FAIL)
 	{
 		anim_elem tmp;
-		tmp.my_unit = unit_const_ptr(animated_unit);
+		tmp.my_unit = animated_unit;
 		tmp.text = text;
 		tmp.text_color = text_color;
 		tmp.src = src;
