@@ -1,38 +1,66 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "units/drawer.hpp"
 
+#include "color.hpp"
 #include "display.hpp"
 #include "display_context.hpp"
 #include "formatter.hpp"
 #include "game_display.hpp"
-#include "preferences/game.hpp"
 #include "halo.hpp"
-#include "map/map.hpp"
 #include "map/location.hpp"
-#include "color.hpp"
+#include "map/map.hpp"
+#include "picture.hpp"
+#include "preferences/game.hpp"
 #include "sdl/surface.hpp"
 #include "team.hpp"
-#include "units/unit.hpp"
 #include "units/animation.hpp"
 #include "units/animation_component.hpp"
 #include "units/frame.hpp"
+#include "units/types.hpp"
+#include "units/unit.hpp"
 
 // Map of different energy bar surfaces and their dimensions.
 static std::map<surface, SDL_Rect> energy_bar_rects;
 
+namespace
+{
+/**
+ * Wrapper which will assemble the image path (including IPF for the color from get_orb_color) for a given orb.
+ * Returns nullptr if the preferences have been configured to hide this orb.
+ */
+std::unique_ptr<image::locator> get_orb_image(orb_status os)
+{
+	if(os == orb_status::disengaged) {
+		if(orb_status_helper::prefs_show_orb(os)) {
+			auto partial_color = orb_status_helper::get_orb_color(orb_status::partial);
+			auto moved_color = orb_status_helper::get_orb_color(orb_status::moved);
+			return std::make_unique<image::locator>(game_config::images::orb_two_color + "~RC(ellipse_red>"
+				+ moved_color + ")~RC(magenta>" + partial_color + ")");
+		}
+		os = orb_status::partial;
+	}
+
+	if(!orb_status_helper::prefs_show_orb(os))
+		return nullptr;
+	auto color = orb_status_helper::get_orb_color(os);
+	return std::make_unique<image::locator>(game_config::images::orb + "~RC(magenta>" + color + ")");
+}
+}
+
 unit_drawer::unit_drawer(display & thedisp) :
 	disp(thedisp),
 	dc(disp.get_disp_context()),
@@ -54,6 +82,11 @@ unit_drawer::unit_drawer(display & thedisp) :
 	if(const game_display* game_display = dynamic_cast<class game_display*>(&disp)) {
 		units_that_can_reach_goal = game_display->units_that_can_reach_goal();
 	}
+
+	// This used to be checked in the drawing code, where it simply triggered skipping some logic.
+	// However, I think it's obsolete, and that the initialization of viewing_team_ref would already
+	// be undefined behavior in the situation where this assert fails.
+	assert(disp.team_valid());
 }
 
 void unit_drawer::redraw_unit (const unit & u) const
@@ -68,8 +101,6 @@ void unit_drawer::redraw_unit (const unit & u) const
 	map_location::DIRECTION facing = u.facing();
 	int hitpoints = u.hitpoints();
 	int max_hitpoints = u.max_hitpoints();
-	int movement_left = u.movement_left();
-	int total_movement = u.total_movement();
 
 	bool can_recruit = u.can_recruit();
 	bool can_advance = u.can_advance();
@@ -87,8 +118,7 @@ void unit_drawer::redraw_unit (const unit & u) const
 	const bool is_highlighted_enemy = units_that_can_reach_goal.count(loc) > 0;
 	const bool is_selected_hex = (loc == sel_hex || is_highlighted_enemy);
 
-	if ( hidden || is_blindfolded || !u.is_visible_to_team(viewing_team_ref, dc, show_everything) )
-	{
+	if(hidden || is_blindfolded || !u.is_visible_to_team(viewing_team_ref, show_everything)) {
 		ac.clear_haloes();
 		if(ac.anim_) {
 			ac.anim_->update_last_draw_time();
@@ -113,8 +143,7 @@ void unit_drawer::redraw_unit (const unit & u) const
 	// instead use -1.0 (as in "negative depth", it will be ignored by rendering)
 	params.submerge= is_flying ? -1.0 : terrain_info.unit_submerge();
 
-	if (u.invisible(loc, dc) &&
-			params.highlight_ratio > 0.6) {
+	if(u.invisible(loc) && params.highlight_ratio > 0.6) {
 		params.highlight_ratio = 0.6;
 	}
 	if (is_selected_hex && params.highlight_ratio == 1.0) {
@@ -231,8 +260,6 @@ void unit_drawer::redraw_unit (const unit & u) const
 			xsrc, ysrc +adjusted_params.y-ellipse_floating, ellipse_front);
 	}
 	if(draw_bars) {
-		const image::locator* orb_img = nullptr;
-
 		const auto& type_cfg = u.type().get_cfg();
 		const auto& cfg_offset_x = type_cfg["bar_offset_x"];
 		const auto& cfg_offset_y = type_cfg["bar_offset_y"];
@@ -248,53 +275,31 @@ void unit_drawer::redraw_unit (const unit & u) const
 			yoff = cfg_offset_y.to_int();
 		}
 
-		/*static*/ const image::locator partmoved_orb(game_config::images::orb + "~RC(magenta>" +
-						preferences::partial_color() + ")"  );
-		/*static*/ const image::locator moved_orb(game_config::images::orb + "~RC(magenta>" +
-						preferences::moved_color() + ")"  );
-		/*static*/ const image::locator ally_orb(game_config::images::orb + "~RC(magenta>" +
-						preferences::allied_color() + ")"  );
-		/*static*/ const image::locator enemy_orb(game_config::images::orb + "~RC(magenta>" +
-						preferences::enemy_color() + ")"  );
-		/*static*/ const image::locator unmoved_orb(game_config::images::orb + "~RC(magenta>" +
-						preferences::unmoved_color() + ")"  );
-
 		const std::string* energy_file = &game_config::images::energy;
 
-		if(size_t(side) != viewing_team+1) {
-			if(disp.team_valid() &&
-			   viewing_team_ref.is_enemy(side)) {
-				if (preferences::show_enemy_orb() && !u.incapacitated())
-					orb_img = &enemy_orb;
-				else
-					orb_img = nullptr;
-			} else {
-				if (preferences::show_allied_orb())
-					orb_img = &ally_orb;
-				else orb_img = nullptr;
-			}
+		using namespace orb_status_helper;
+		std::unique_ptr<image::locator> orb_img = nullptr;
+		if(viewing_team_ref.is_enemy(side)) {
+			if(!u.incapacitated())
+				orb_img = get_orb_image(orb_status::enemy);
+		} else if(static_cast<std::size_t>(side) != playing_team + 1) {
+			// We're looking at either the player's own unit or an ally's unit, but either way it
+			// doesn't belong to the playing_team and isn't expected to move until after its next
+			// turn refresh.
+			auto os = orb_status::moved;
+			if(static_cast<std::size_t>(side) != viewing_team + 1)
+				os = orb_status::allied;
+			orb_img = get_orb_image(os);
 		} else {
-			if (preferences::show_moved_orb())
-				orb_img = &moved_orb;
-			else orb_img = nullptr;
-
-			if(playing_team == viewing_team && !u.user_end_turn()) {
-				if (movement_left == total_movement) {
-					if (preferences::show_unmoved_orb())
-						orb_img = &unmoved_orb;
-					else orb_img = nullptr;
-				} else if ( dc.unit_can_move(u) ) {
-					if (preferences::show_partial_orb())
-						orb_img = &partmoved_orb;
-					else orb_img = nullptr;
-				}
-			}
+			// We're looking at either the player's own unit, or an ally's unit, during the unit's
+			// owner's turn.
+			auto os = dc.unit_orb_status(u);
+			orb_img = get_orb_image(os);
 		}
 
-		if (orb_img != nullptr) {
-			surface orb(image::get_image(*orb_img,image::SCALED_TO_ZOOM));
-			disp.drawing_buffer_add(display::LAYER_UNIT_BAR,
-				loc, xsrc + xoff, ysrc + yoff + adjusted_params.y, orb);
+		if(orb_img != nullptr) {
+			surface orb(image::get_image(*orb_img, image::SCALED_TO_ZOOM));
+			disp.drawing_buffer_add(display::LAYER_UNIT_BAR, loc, xsrc + xoff, ysrc + yoff + adjusted_params.y, orb);
 		}
 
 		double unit_energy = 0.0;
@@ -343,9 +348,10 @@ void unit_drawer::redraw_unit (const unit & u) const
 	const t_translation::terrain_code terrain_dst = map.get_terrain(dst);
 	const terrain_type& terrain_dst_info = map.get_terrain_info(terrain_dst);
 
-	int height_adjust_unit = static_cast<int>((terrain_info.unit_height_adjust() * (1.0 - adjusted_params.offset) +
-											  terrain_dst_info.unit_height_adjust() * adjusted_params.offset) *
-											  zoom_factor);
+	// height_adjust_unit is not scaled by zoom_factor here otherwise it results in a squared offset that results in #5974
+	// It appears the tiles and units are scaled together somewhere else
+	int height_adjust_unit = static_cast<int>(terrain_info.unit_height_adjust() * (1.0 - adjusted_params.offset) +
+											  terrain_dst_info.unit_height_adjust() * adjusted_params.offset);
 	if (is_flying && height_adjust_unit < 0) {
 		height_adjust_unit = 0;
 	}
@@ -357,12 +363,12 @@ void unit_drawer::redraw_unit (const unit & u) const
 }
 
 void unit_drawer::draw_bar(const std::string& image, int xpos, int ypos,
-		const map_location& loc, size_t height, double filled,
+		const map_location& loc, std::size_t height, double filled,
 		const color_t& col, fixed_t alpha) const
 {
 
 	filled = std::min<double>(std::max<double>(filled,0.0),1.0);
-	height = static_cast<size_t>(height*zoom_factor);
+	height = static_cast<std::size_t>(height*zoom_factor);
 
 	surface surf(image::get_image(image,image::SCALED_TO_HEX));
 
@@ -393,7 +399,7 @@ void unit_drawer::draw_bar(const std::string& image, int xpos, int ypos,
 		bar_loc = scaled_bar_loc;
 	}
 
-	if(height > static_cast<size_t>(bar_loc.h)) {
+	if(height > static_cast<std::size_t>(bar_loc.h)) {
 		height = bar_loc.h;
 	}
 
@@ -404,7 +410,7 @@ void unit_drawer::draw_bar(const std::string& image, int xpos, int ypos,
 	//	}
 	//}
 
-	const size_t skip_rows = bar_loc.h - height;
+	const std::size_t skip_rows = bar_loc.h - height;
 
 	SDL_Rect top {0, 0, surf->w, bar_loc.y};
 	SDL_Rect bot = sdl::create_rect(0, bar_loc.y + skip_rows, surf->w, 0);
@@ -413,7 +419,7 @@ void unit_drawer::draw_bar(const std::string& image, int xpos, int ypos,
 	disp.drawing_buffer_add(display::LAYER_UNIT_BAR, loc, xpos, ypos, surf, top);
 	disp.drawing_buffer_add(display::LAYER_UNIT_BAR, loc, xpos, ypos + top.h, surf, bot);
 
-	size_t unfilled = static_cast<size_t>(height * (1.0 - filled));
+	std::size_t unfilled = static_cast<std::size_t>(height * (1.0 - filled));
 
 	if(unfilled < height && alpha >= ftofxp(0.3)) {
 		const uint8_t r_alpha = std::min<unsigned>(unsigned(fxpmult(alpha,255)),255);
