@@ -1,21 +1,24 @@
 /*
-   Copyright (C) 2014 - 2018 by Chris Beck <render787@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2014 - 2021
+	by Chris Beck <render787@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "units/filter.hpp"
 
 #include "log.hpp"
 
+#include "display.hpp"
+#include "display_context.hpp"
 #include "config.hpp"
 #include "game_data.hpp"
 #include "map/map.hpp"
@@ -35,13 +38,14 @@
 #include "formula/string_utils.hpp"
 #include "resources.hpp"
 
-#include <boost/optional.hpp>
-
 static lg::log_domain log_config("config");
 #define ERR_CF LOG_STREAM(err, log_config)
 #define WRN_CF LOG_STREAM(warn, log_config)
 #define DBG_CF LOG_STREAM(debug, log_config)
 
+static lg::log_domain log_wml("wml");
+#define ERR_WML LOG_STREAM(err, log_wml)
+
 using namespace unit_filter_impl;
 
 unit_filter::unit_filter(vconfig cfg)
@@ -103,7 +107,7 @@ struct unit_filter_xy : public unit_filter_base
 			return false;
 		}
 		else if(x == "recall" && y == "recall") {
-			return !args.fc->get_disp_context().map().on_board(args.loc);
+			return !args.context().get_disp_context().map().on_board(args.loc);
 		}
 		else {
 			return args.loc.matches_range(x, y);
@@ -124,9 +128,8 @@ struct unit_filter_adjacent : public unit_filter_base
 
 	virtual bool matches(const unit_filter_args& args) const override
 	{
-		const unit_map& units = args.fc->get_disp_context().units();
-		adjacent_loc_array_t adjacent;
-		get_adjacent_tiles(args.loc, adjacent.data());
+		const unit_map& units = args.context().get_disp_context().units();
+		const auto adjacent = get_adjacent_tiles(args.loc);
 		int match_count=0;
 
 		config::attribute_value i_adjacent = cfg_["adjacent"];
@@ -142,7 +145,7 @@ struct unit_filter_adjacent : public unit_filter_base
 				continue;
 			}
 			auto is_enemy = cfg_["is_enemy"];
-			if (!is_enemy.empty() && is_enemy.to_bool() != args.fc->get_disp_context().get_team(args.u.side()).is_enemy(unit_itor->side())) {
+			if (!is_enemy.empty() && is_enemy.to_bool() != args.context().get_disp_context().get_team(args.u.side()).is_enemy(unit_itor->side())) {
 				continue;
 			}
 			++match_count;
@@ -197,7 +200,10 @@ struct unit_filter_attribute_literal : public unit_filter_base
 	F f_;
 };
 
-class contains_dollar_visitor : public boost::static_visitor<bool>
+class contains_dollar_visitor
+#ifdef USING_BOOST_VARIANT
+	: public boost::static_visitor<bool>
+#endif
 {
 public:
 	contains_dollar_visitor() {}
@@ -229,10 +235,10 @@ bool unit_filter_compound::matches(const unit_filter_args& args) const
 	bool res;
 
 	if(args.loc.valid()) {
-		scoped_xy_unit auto_store("this_unit", args.u.get_location(), args.fc->get_disp_context().units());
+		scoped_xy_unit auto_store("this_unit", args.u.get_location(), args.context().get_disp_context().units());
 		if (args.u2) {
 			const map_location& loc2 = args.u2->get_location();
-			scoped_xy_unit u2_auto_store("other_unit", loc2, args.fc->get_disp_context().units());
+			scoped_xy_unit u2_auto_store("other_unit", loc2, args.context().get_disp_context().units());
 			res = filter_impl(args);
 		} else {
 			res = filter_impl(args);
@@ -288,6 +294,26 @@ void unit_filter_compound::create_attribute(const config::attribute_value v, C c
 	}
 }
 
+namespace {
+
+	struct ability_match
+	{
+		std::string tag_name;
+		const config* cfg;
+	};
+
+	void get_ability_children_id(std::vector<ability_match>& id_result,
+	                           const config& parent, const std::string& id) {
+		for (const config::any_child &sp : parent.all_children_range())
+		{
+			if(sp.cfg["id"] == id) {
+				ability_match special = { sp.key, &sp.cfg };
+				id_result.push_back(special);
+			}
+		}
+	}
+}
+
 void unit_filter_compound::fill(vconfig cfg)
 	{
 		const config& literal = cfg.get_config();
@@ -308,14 +334,6 @@ void unit_filter_compound::fill(vconfig cfg)
 			}
 		);
 
-		create_attribute(literal["speaker"],
-			[](const config::attribute_value& c) { return c.str(); },
-			[](const std::string& speaker, const unit_filter_args& args)
-			{
-				return speaker == args.u.id();
-			}
-		);
-
 		create_attribute(literal["type"],
 			[](const config::attribute_value& c) { return utils::split(c.str()); },
 			[](const std::vector<std::string>& types, const unit_filter_args& args)
@@ -356,7 +374,7 @@ void unit_filter_compound::fill(vconfig cfg)
 			[](const std::vector<std::string>& types, const unit_filter_args& args)
 			{
 				// If this unit is a variation itself then search in the base unit's variations.
-				const unit_type* const type = args.u.variation().empty() ? &args.u.type() : unit_types.find(args.u.type().base_id());
+				const unit_type* const type = args.u.variation().empty() ? &args.u.type() : unit_types.find(args.u.type().parent_id());
 				assert(type);
 
 				for(const std::string& variation_id : types) {
@@ -394,6 +412,42 @@ void unit_filter_compound::fill(vconfig cfg)
 			}
 		);
 
+		create_attribute(literal["ability_id_active"],
+			[](const config::attribute_value& c) { return utils::split(c.str()); },
+			[](const std::vector<std::string>& abilities, const unit_filter_args& args)
+			{
+				assert(display::get_singleton());
+				const unit_map& units = display::get_singleton()->get_units();
+				for(const std::string& ability : abilities) {
+					std::vector<ability_match> ability_id_matches_self;
+					get_ability_children_id(ability_id_matches_self, args.u.abilities(), ability);
+					for(const ability_match& entry : ability_id_matches_self) {
+						if (args.u.get_self_ability_bool(*entry.cfg, entry.tag_name, args.loc)) {
+							return true;
+						}
+					}
+
+					const auto adjacent = get_adjacent_tiles(args.loc);
+					for(unsigned i = 0; i < adjacent.size(); ++i) {
+						const unit_map::const_iterator it = units.find(adjacent[i]);
+						if (it == units.end() || it->incapacitated())
+							continue;
+						if (&*it == (args.u.shared_from_this()).get())
+							continue;
+
+						std::vector<ability_match> ability_id_matches_adj;
+						get_ability_children_id(ability_id_matches_adj, it->abilities(), ability);
+						for(const ability_match& entry : ability_id_matches_adj) {
+							if (args.u.get_adj_ability_bool(*entry.cfg, entry.tag_name,i, args.loc, *it)) {
+								return true;
+							}
+						}
+					}
+				}
+				return false;
+			}
+		);
+
 		create_attribute(literal["ability_type_active"],
 			[](const config::attribute_value& c) { return utils::split(c.str()); },
 			[](const std::vector<std::string>& abilities, const unit_filter_args& args)
@@ -442,6 +496,21 @@ void unit_filter_compound::fill(vconfig cfg)
 			}
 		);
 
+		create_attribute(literal["upkeep"],
+			[](const config::attribute_value& c) -> unit::upkeep_t
+			{
+				try {
+					return c.apply_visitor(unit::upkeep_parser_visitor());
+				} catch(std::invalid_argument&) {
+					return unit::upkeep_full();
+				}
+			},
+			[](unit::upkeep_t upkeep, const unit_filter_args& args)
+			{
+				return args.u.upkeep() == utils::visit(unit::upkeep_value_visitor{args.u}, upkeep);
+			}
+		);
+
 		create_attribute(literal["side"],
 			[](const config::attribute_value& c)
 			{
@@ -496,6 +565,14 @@ void unit_filter_compound::fill(vconfig cfg)
 			}
 		);
 
+		create_attribute(literal["alignment"],
+			[](const config::attribute_value& c) { return c.str(); },
+			[](const std::string& alignment, const unit_filter_args& args)
+			{
+				return args.u.alignment().to_string() == alignment;
+			}
+		);
+
 		create_attribute(literal["ai_special"],
 			[](const config::attribute_value& c) { return c.str(); },
 			[](const std::string& ai_special, const unit_filter_args& args)
@@ -504,6 +581,19 @@ void unit_filter_compound::fill(vconfig cfg)
 			}
 		);
 
+		create_attribute(literal["usage"],
+			[](const config::attribute_value& c) { return utils::split(c.str()); },
+			[](const std::vector<std::string>& usages, const unit_filter_args& args)
+			{
+				for(const std::string& usage : usages) {
+					if(args.u.usage() == usage) {
+						return true;
+					}
+				}
+				return false;
+			}
+		);
+
 		create_attribute(literal["canrecruit"],
 			[](const config::attribute_value& c) { return c.to_bool(); },
 			[](bool canrecruit, const unit_filter_args& args)
@@ -542,7 +632,7 @@ void unit_filter_compound::fill(vconfig cfg)
 			[](const config::attribute_value& c) { return utils::parse_ranges(c.str()); },
 			[](const std::vector<std::pair<int,int>>& ranges, const unit_filter_args& args)
 			{
-				int actual_defense = args.u.defense_modifier(args.fc->get_disp_context().map().get_terrain(args.loc));
+				int actual_defense = args.u.defense_modifier(args.context().get_disp_context().map().get_terrain(args.loc));
 				for(auto def : ranges) {
 					if(def.first <= actual_defense && actual_defense <= def.second) {
 						return true;
@@ -556,7 +646,7 @@ void unit_filter_compound::fill(vconfig cfg)
 			[](const config::attribute_value& c) { return utils::parse_ranges(c.str()); },
 			[](const std::vector<std::pair<int,int>>& ranges, const unit_filter_args& args)
 			{
-				int actual_cost = args.u.movement_cost(args.fc->get_disp_context().map().get_terrain(args.loc));
+				int actual_cost = args.u.movement_cost(args.context().get_disp_context().map().get_terrain(args.loc));
 				for(auto cost : ranges) {
 					if(cost.first <= actual_cost && actual_cost <= cost.second) {
 						return true;
@@ -570,7 +660,7 @@ void unit_filter_compound::fill(vconfig cfg)
 			[](const config::attribute_value& c) { return utils::parse_ranges(c.str()); },
 			[](const std::vector<std::pair<int,int>>& ranges, const unit_filter_args& args)
 			{
-				int actual_cost = args.u.vision_cost(args.fc->get_disp_context().map().get_terrain(args.loc));
+				int actual_cost = args.u.vision_cost(args.context().get_disp_context().map().get_terrain(args.loc));
 				for(auto cost : ranges) {
 					if(cost.first <= actual_cost && actual_cost <= cost.second) {
 						return true;
@@ -584,7 +674,7 @@ void unit_filter_compound::fill(vconfig cfg)
 			[](const config::attribute_value& c) { return utils::parse_ranges(c.str()); },
 			[](const std::vector<std::pair<int,int>>& ranges, const unit_filter_args& args)
 			{
-				int actual_cost = args.u.jamming_cost(args.fc->get_disp_context().map().get_terrain(args.loc));
+				int actual_cost = args.u.jamming_cost(args.context().get_disp_context().map().get_terrain(args.loc));
 				for(auto cost : ranges) {
 					if(cost.first <= actual_cost && actual_cost <= cost.second) {
 						return true;
@@ -598,7 +688,7 @@ void unit_filter_compound::fill(vconfig cfg)
 			[](const config::attribute_value& c) { return c.str(); },
 			[](const std::string& lua_function, const unit_filter_args& args)
 			{
-				if (game_lua_kernel * lk = args.fc->get_lua_kernel()) {
+				if (game_lua_kernel * lk = args.context().get_lua_kernel()) {
 					return lk->run_filter(lua_function.c_str(), args.u);
 				}
 				return true;
@@ -626,7 +716,8 @@ void unit_filter_compound::fill(vconfig cfg)
 					}
 					return true;
 				} catch(const wfl::formula_error& e) {
-					lg::wml_error() << "Formula error in unit filter: " << e.type << " at " << e.filename << ':' << e.line << ")\n";
+					lg::log_to_chat() << "Formula error in unit filter: " << e.type << " at " << e.filename << ':' << e.line << ")\n";
+					ERR_WML << "Formula error in unit filter: " << e.type << " at " << e.filename << ':' << e.line << ")";
 					// Formulae with syntax errors match nothing
 					return false;
 				}
@@ -638,7 +729,7 @@ void unit_filter_compound::fill(vconfig cfg)
 			[](const std::string& find_in, const unit_filter_args& args)
 			{
 				// Allow filtering by searching a stored variable of units
-				if (const game_data * gd = args.fc->get_game_data()) {
+				if (const game_data * gd = args.context().get_game_data()) {
 					try
 					{
 						for (const config& c : gd->get_variable_access_read(find_in).as_array())
@@ -665,7 +756,7 @@ void unit_filter_compound::fill(vconfig cfg)
 		for(auto child : cfg.all_ordered()) {
 			CONDITIONAL_TYPE cond;
 			if(cond.parse(child.first)) {
-				cond_children_.emplace_back(std::piecewise_construct_t(), std::make_tuple(cond), std::make_tuple(child.second));
+				cond_children_.emplace_back(std::piecewise_construct_t(), std::tuple(cond), std::tuple(child.second));
 			}
 			else if (child.first == "filter_wml") {
 				create_child(child.second, [](const vconfig& c, const unit_filter_args& args) {
@@ -691,8 +782,9 @@ void unit_filter_compound::fill(vconfig cfg)
 					viewers.insert(sides.begin(), sides.end());
 
 					for (const int viewer : viewers) {
-						bool fogged = args.fc->get_disp_context().get_team(viewer).fogged(args.loc);
-						bool hiding = args.u.invisible(args.loc, args.fc->get_disp_context()) && args.fc->get_disp_context().get_team(viewer).is_enemy(args.u.side());
+						bool fogged = args.context().get_disp_context().get_team(viewer).fogged(args.loc);
+						// Check is_enemy() before invisible() to prevent infinite recursion in [abilities][hides][filter_self][filter_vision]
+						bool hiding = args.context().get_disp_context().get_team(viewer).is_enemy(args.u.side()) && args.u.invisible(args.loc);
 						bool unit_hidden = fogged || hiding;
 						if (c["visible"].to_bool(true) != unit_hidden) {
 							return true;
