@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2006 - 2018 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "units/frame.hpp"
@@ -72,21 +73,15 @@ frame_builder::frame_builder(const config& cfg,const std::string& frame_string)
 	, primary_frame_(boost::logic::indeterminate)
 	, drawing_layer_(cfg[frame_string + "layer"])
 {
-	if(!cfg.has_attribute(frame_string + "auto_vflip")) {
-		auto_vflip_ = boost::logic::indeterminate;
-	} else {
+	if(cfg.has_attribute(frame_string + "auto_vflip")) {
 		auto_vflip_ = cfg[frame_string + "auto_vflip"].to_bool();
 	}
 
-	if(!cfg.has_attribute(frame_string + "auto_hflip")) {
-		auto_hflip_ = boost::logic::indeterminate;
-	} else {
+	if(cfg.has_attribute(frame_string + "auto_hflip")) {
 		auto_hflip_ = cfg[frame_string + "auto_hflip"].to_bool();
 	}
 
-	if(!cfg.has_attribute(frame_string + "primary")) {
-		primary_frame_ = boost::logic::indeterminate;
-	} else {
+	if(cfg.has_attribute(frame_string + "primary")) {
 		primary_frame_ = cfg[frame_string + "primary"].to_bool();
 	}
 
@@ -460,15 +455,15 @@ std::vector<std::string> frame_parsed_parameters::debug_strings() const
 	}
 
 	if(!boost::indeterminate(auto_vflip_)) {
-		v.emplace_back("auto_vflip=" + utils::bool_string(auto_vflip_));
+		v.emplace_back("auto_vflip=" + utils::bool_string(static_cast<bool>(auto_vflip_)));
 	}
 
 	if(!boost::indeterminate(auto_hflip_)) {
-		v.emplace_back("auto_hflip=" + utils::bool_string(auto_hflip_));
+		v.emplace_back("auto_hflip=" + utils::bool_string(static_cast<bool>(auto_hflip_)));
 	}
 
 	if(!boost::indeterminate(primary_frame_)) {
-		v.emplace_back("primary_frame=" + utils::bool_string(primary_frame_));
+		v.emplace_back("primary_frame=" + utils::bool_string(static_cast<bool>(primary_frame_)));
 	}
 
 	if(!drawing_layer_.get_original().empty()) {
@@ -528,6 +523,7 @@ void unit_frame::redraw(const int frame_time, bool on_start_time, bool in_scope_
 
 	const int x = static_cast<int>(tmp_offset * xdst + (1.0 - tmp_offset) * xsrc) + d2;
 	const int y = static_cast<int>(tmp_offset * ydst + (1.0 - tmp_offset) * ysrc) + d2;
+	const double disp_zoom = display::get_singleton()->get_zoom_factor();
 
 	if(image != nullptr) {
 		bool facing_west = (
@@ -542,19 +538,19 @@ void unit_frame::redraw(const int frame_time, bool on_start_time, bool in_scope_
 		if(!current_data.auto_hflip) { facing_west = false; }
 		if(!current_data.auto_vflip) { facing_north = true; }
 
-		int my_x = x + current_data.x - image->w / 2;
-		int my_y = y + current_data.y - image->h / 2;
+		int my_x = x + current_data.x * disp_zoom - image->w / 2;
+		int my_y = y + current_data.y * disp_zoom - image->h / 2;
 
 		if(facing_west) {
-			my_x -= current_data.directional_x;
+			my_x -= current_data.directional_x * disp_zoom;
 		} else {
-			my_x += current_data.directional_x;
+			my_x += current_data.directional_x * disp_zoom;
 		}
 
 		if(facing_north) {
-			my_y += current_data.directional_y;
+			my_y += current_data.directional_y * disp_zoom;
 		} else {
-			my_y -= current_data.directional_y;
+			my_y -= current_data.directional_y * disp_zoom;
 		}
 
 		display::get_singleton()->render_image(my_x, my_y,
@@ -608,16 +604,16 @@ void unit_frame::redraw(const int frame_time, bool on_start_time, bool in_scope_
 
 	if(direction != map_location::SOUTH_WEST && direction != map_location::NORTH_WEST) {
 		halo_id = halo_man.add(
-			static_cast<int>(x + current_data.halo_x * display::get_singleton()->get_zoom_factor()),
-			static_cast<int>(y + current_data.halo_y * display::get_singleton()->get_zoom_factor()),
+			static_cast<int>(x + current_data.halo_x * disp_zoom),
+			static_cast<int>(y + current_data.halo_y * disp_zoom),
 			current_data.halo  + current_data.halo_mod,
 			map_location(-1, -1),
 			orientation
 		);
 	} else {
 		halo_id = halo_man.add(
-			static_cast<int>(x - current_data.halo_x * display::get_singleton()->get_zoom_factor()),
-			static_cast<int>(y + current_data.halo_y * display::get_singleton()->get_zoom_factor()),
+			static_cast<int>(x - current_data.halo_x * disp_zoom),
+			static_cast<int>(y + current_data.halo_y * disp_zoom),
 			current_data.halo  + current_data.halo_mod,
 			map_location(-1, -1),
 			orientation
@@ -682,10 +678,6 @@ std::set<map_location> unit_frame::get_overlaped_hex(const int frame_time, const
 	} else {
 		int w = 0, h = 0;
 
-#ifdef _OPENMP
-#pragma omp critical(frame_surface) // with the way surfaces work it's hard to lock the refcount within sdl_utils
-#endif //_OPENMP
-
 		{
 			surface image;
 			if(!image_loc.is_void() && !image_loc.get_filename().empty()) { // invalid diag image, or not diagonal
@@ -772,7 +764,7 @@ const frame_parameters unit_frame::merge_parameters(int current_time, const fram
 	}
 
 	// Convert the tribool to bool
-	const bool primary = result.primary_frame == true || boost::logic::indeterminate(result.primary_frame);
+	const bool primary = static_cast<bool>(result.primary_frame) || boost::logic::indeterminate(result.primary_frame);
 
 	/** The engine provides a default image to use for the unit when none is available */
 	result.image = current_val.image.is_void() || current_val.image.get_filename().empty()
@@ -835,8 +827,8 @@ const frame_parameters unit_frame::merge_parameters(int current_time, const fram
 	}
 
 	/** The engine provides a blend color for poisoned units */
-	result.blend_ratio = current_val.blend_ratio?current_val.blend_ratio:animation_val.blend_ratio;
-	if(primary && engine_val.blend_ratio) {
+	result.blend_ratio = current_val.blend_ratio != 0 ? current_val.blend_ratio:animation_val.blend_ratio;
+	if(primary && engine_val.blend_ratio != 0) {
 		result.blend_ratio = std::min(result.blend_ratio + engine_val.blend_ratio, 1.0);
 	}
 
@@ -854,8 +846,8 @@ const frame_parameters unit_frame::merge_parameters(int current_time, const fram
 	}
 
 	/** The engine provides a submerge for units in water */
-	result.submerge = current_val.submerge ? current_val.submerge : animation_val.submerge;
-	if(primary && engine_val.submerge && !result.submerge) {
+	result.submerge = current_val.submerge != 0 ? current_val.submerge : animation_val.submerge;
+	if(primary && engine_val.submerge != 0 && result.submerge == 0) {
 		result.submerge = engine_val.submerge;
 	}
 
