@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #pragma once
 
 #include "units/map.hpp"
@@ -40,4 +39,14 @@ int number_of_possible_advances(const unit &unit);
  */
 std::string resistance_color(const int resistance);
 
+/**
+ * @return the tooltip text showing a unit's level and what it can advance to.
+ */
+std::string unit_level_tooltip(const unit &u);
+
+/**
+ * @return the tooltip text showing a unit type's level and what it can
+ * advance to.
+ */
+std::string unit_level_tooltip(const unit_type &u);
 }
