@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -23,6 +24,7 @@
 #include "game_config.hpp"
 #include "game_errors.hpp" //thrown sometimes
 //#include "gettext.hpp"
+#include "language.hpp" // for string_table
 #include "log.hpp"
 #include "units/abilities.hpp"
 #include "units/animation.hpp"
@@ -35,6 +37,7 @@
 
 #include <boost/range/algorithm_ext/erase.hpp>
 
+#include <array>
 #include <locale>
 
 static lg::log_domain log_config("config");
@@ -45,17 +48,17 @@ static lg::log_domain log_config("config");
 
 static lg::log_domain log_unit("unit");
 #define DBG_UT LOG_STREAM(debug, log_unit)
+#define LOG_UT LOG_STREAM(info, log_unit)
 #define ERR_UT LOG_STREAM(err, log_unit)
 
 /* ** unit_type ** */
 
 unit_type::unit_type(const unit_type& o)
 	: cfg_(o.cfg_)
-	, unit_cfg_()
-	, built_unit_cfg_(false) // Not copied; will be re-created if needed.
 	, id_(o.id_)
 	, debug_id_(o.debug_id_)
-	, base_id_(o.base_id_)
+	, parent_id_(o.parent_id_)
+	, base_unit_id_(o.base_unit_id_)
 	, type_name_(o.type_name_)
 	, description_(o.description_)
 	, hitpoints_(o.hitpoints_)
@@ -87,7 +90,6 @@ unit_type::unit_type(const unit_type& o)
 	, do_not_list_(o.do_not_list_)
 	, advances_to_(o.advances_to_)
 	, experience_needed_(o.experience_needed_)
-	, in_advancefrom_(o.in_advancefrom_)
 	, alignment_(o.alignment_)
 	, movement_type_(o.movement_type_)
 	, possible_traits_(o.possible_traits_)
@@ -99,14 +101,15 @@ unit_type::unit_type(const unit_type& o)
 	gender_types_[1].reset(gender_types_[1] != nullptr ? new unit_type(*o.gender_types_[1]) : nullptr);
 }
 
-unit_type::unit_type(const config& cfg, const std::string& parent_id)
-	: cfg_(cfg)
-	, unit_cfg_()
-	, built_unit_cfg_(false)
-	, id_(cfg_.has_attribute("id") ? cfg_["id"].str() : parent_id)
+unit_type::unit_type(defaut_ctor_t, const config& cfg, const std::string & parent_id)
+	: cfg_(nullptr)
+	, built_cfg_()
+	, has_cfg_build_()
+	, id_(cfg.has_attribute("id") ? cfg["id"].str() : parent_id)
 	, debug_id_()
-	, base_id_(!parent_id.empty() ? parent_id : id_)
-	, type_name_(cfg_["name"].t_str())
+	, parent_id_(!parent_id.empty() ? parent_id : id_)
+	, base_unit_id_()
+	, type_name_()
 	, description_()
 	, hitpoints_(0)
 	, hp_bar_scaling_(0.0)
@@ -120,25 +123,24 @@ unit_type::unit_type(const config& cfg, const std::string& parent_id)
 	, cost_(0)
 	, usage_()
 	, undead_variation_()
-	, image_(cfg_["image"].str())
+	, image_()
 	, icon_()
 	, small_profile_()
 	, profile_()
-	, flag_rgb_(cfg_["flag_rgb"].str())
+	, flag_rgb_()
 	, num_traits_(0)
 	, gender_types_()
 	, variations_()
-	, default_variation_(cfg_["variation"])
-	, variation_name_(cfg_["variation_name"].t_str())
+	, default_variation_()
+	, variation_name_()
 	, race_(&unit_race::null_race)
 	, abilities_()
 	, adv_abilities_()
 	, zoc_(false)
 	, hide_help_(false)
-	, do_not_list_(cfg_["do_not_list"].to_bool(false))
+	, do_not_list_()
 	, advances_to_()
 	, experience_needed_(0)
-	, in_advancefrom_(false)
 	, alignment_(unit_type::ALIGNMENT::NEUTRAL)
 	, movement_type_()
 	, possible_traits_()
@@ -146,9 +148,26 @@ unit_type::unit_type(const config& cfg, const std::string& parent_id)
 	, animations_()
 	, build_status_(NOT_BUILT)
 {
+	if(const config& base_unit = cfg.child("base_unit")) {
+		base_unit_id_ = base_unit["id"].str();
+		LOG_UT << "type '" <<  id_ << "' has base unit '" << base_unit_id_ << "'\n";
+	}
 	check_id(id_);
-	check_id(base_id_);
+	check_id(parent_id_);
 }
+unit_type::unit_type(const config& cfg, const std::string & parent_id)
+	: unit_type(defaut_ctor_t(), cfg, parent_id)
+{
+	cfg_ = &cfg;
+
+}
+
+unit_type::unit_type(config&& cfg, const std::string & parent_id)
+	: unit_type(defaut_ctor_t(), cfg, parent_id)
+{
+	built_cfg_ = std::make_unique<config>(std::move(cfg));
+}
+
 
 unit_type::~unit_type()
 {
@@ -173,7 +192,7 @@ unit_type::ability_metadata::ability_metadata(const config& cfg)
  * Load data into an empty unit_type (build to FULL).
  */
 void unit_type::build_full(
-		const movement_type_map& mv_types, const race_map& races, const config::const_child_itors& traits)
+		const movement_type_map& mv_types, const race_map& races, const config_array_view& traits)
 {
 	// Don't build twice.
 	if(FULL <= build_status_) {
@@ -195,12 +214,12 @@ void unit_type::build_full(
 		}
 	}
 
-	zoc_ = cfg_["zoc"].to_bool(level_ > 0);
+	zoc_ = get_cfg()["zoc"].to_bool(level_ > 0);
 
-	game_config::add_color_info(cfg_);
+	game_config::add_color_info(game_config_view::wrap(get_cfg()));
 
-	hp_bar_scaling_ = cfg_["hp_bar_scaling"].to_double(game_config::hp_bar_scaling);
-	xp_bar_scaling_ = cfg_["xp_bar_scaling"].to_double(game_config::xp_bar_scaling);
+	hp_bar_scaling_ = get_cfg()["hp_bar_scaling"].to_double(game_config::hp_bar_scaling);
+	xp_bar_scaling_ = get_cfg()["xp_bar_scaling"].to_double(game_config::xp_bar_scaling);
 
 	// Propagate the build to the variations.
 	for(variations_map::value_type& variation : variations_) {
@@ -216,7 +235,7 @@ void unit_type::build_full(
  * Partially load data into an empty unit_type (build to HELP_INDEXED).
  */
 void unit_type::build_help_index(
-		const movement_type_map& mv_types, const race_map& races, const config::const_child_itors& traits)
+		const movement_type_map& mv_types, const race_map& races, const config_array_view& traits)
 {
 	// Don't build twice.
 	if(HELP_INDEXED <= build_status_) {
@@ -224,29 +243,36 @@ void unit_type::build_help_index(
 	}
 
 	// Make sure we are built to the preceding build level.
-	build_created(mv_types, races, traits);
+	build_created();
 
-	type_name_ = cfg_["name"];
-	description_ = cfg_["description"];
-	hitpoints_ = cfg_["hitpoints"].to_int(1);
-	level_ = cfg_["level"];
-	recall_cost_ = cfg_["recall_cost"].to_int(-1);
-	movement_ = cfg_["movement"].to_int(1);
-	vision_ = cfg_["vision"].to_int(-1);
-	jamming_ = cfg_["jamming"].to_int(0);
-	max_attacks_ = cfg_["attacks"].to_int(1);
-	cost_ = cfg_["cost"].to_int(1);
-	usage_ = cfg_["usage"].str();
-	undead_variation_ = cfg_["undead_variation"].str();
-	image_ = cfg_["image"].str();
-	icon_ = cfg_["image_icon"].str();
-	small_profile_ = cfg_["small_profile"].str();
-	profile_ = cfg_["profile"].str();
+	const config& cfg = get_cfg();
+
+	type_name_ = cfg["name"];
+	description_ = cfg["description"];
+	hitpoints_ = cfg["hitpoints"].to_int(1);
+	level_ = cfg["level"];
+	recall_cost_ = cfg["recall_cost"].to_int(-1);
+	movement_ = cfg["movement"].to_int(1);
+	vision_ = cfg["vision"].to_int(-1);
+	jamming_ = cfg["jamming"].to_int(0);
+	max_attacks_ = cfg["attacks"].to_int(1);
+	usage_ = cfg["usage"].str();
+	undead_variation_ = cfg["undead_variation"].str();
+	default_variation_ = cfg["variation"].str();
+	icon_ = cfg["image_icon"].str();
+	small_profile_ = cfg["small_profile"].str();
+	profile_ = cfg["profile"].str();
+	flag_rgb_ = cfg["flag_rgb"].str();
+	do_not_list_ = cfg["do_not_list"].to_bool(false);
+
+	for(const config& sn : cfg.child_range("special_note")) {
+		special_notes_.push_back(sn["note"]);
+	}
 
 	adjust_profile(profile_);
 
 	alignment_ = unit_type::ALIGNMENT::NEUTRAL;
-	alignment_.parse(cfg_["alignment"].str());
+	alignment_.parse(cfg["alignment"].str());
 
 	for(int i = 0; i < 2; ++i) {
 		if(gender_types_[i]) {
@@ -254,7 +280,11 @@ void unit_type::build_help_index(
 		}
 	}
 
-	const race_map::const_iterator race_it = races.find(cfg_["race"]);
+	for(auto& pair : variations_) {
+		pair.second.build_help_index(mv_types, races, traits);
+	}
+
+	const race_map::const_iterator race_it = races.find(cfg["race"]);
 	if(race_it != races.end()) {
 		race_ = &race_it->second;
 	} else {
@@ -262,9 +292,9 @@ void unit_type::build_help_index(
 	}
 
 	// if num_traits is not defined, we use the num_traits from race
-	num_traits_ = cfg_["num_traits"].to_int(race_->num_traits());
+	num_traits_ = cfg["num_traits"].to_int(race_->num_traits());
 
-	for(const std::string& g : utils::split(cfg_["gender"])) {
+	for(const std::string& g : utils::split(cfg["gender"])) {
 		genders_.push_back(string_gender(g));
 	}
 
@@ -273,13 +303,13 @@ void unit_type::build_help_index(
 		genders_.push_back(unit_race::MALE);
 	}
 
-	if(const config& abil_cfg = cfg_.child("abilities")) {
+	if(const config& abil_cfg = cfg.child("abilities")) {
 		for(const config::any_child& ab : abil_cfg.all_children_range()) {
 			abilities_.emplace_back(ab.cfg);
 		}
 	}
 
-	for(const config& adv : cfg_.child_range("advancement")) {
+	for(const config& adv : cfg.child_range("advancement")) {
 		for(const config& effect : adv.child_range("effect")) {
 			const config& abil_cfg = effect.child("abilities");
 
@@ -294,7 +324,7 @@ void unit_type::build_help_index(
 	}
 
 	// Set the movement type.
-	const std::string move_type = cfg_["movement_type"];
+	const std::string move_type = cfg["movement_type"];
 	const movement_type_map::const_iterator find_it = mv_types.find(move_type);
 
 	if(find_it != mv_types.end()) {
@@ -305,7 +335,7 @@ void unit_type::build_help_index(
 	}
 
 	// Override parts of the movement type with what is in our config.
-	movement_type_.merge(cfg_);
+	movement_type_.merge(cfg);
 
 	for(const config& t : traits) {
 		possible_traits_.add_child("trait", t);
@@ -316,7 +346,7 @@ void unit_type::build_help_index(
 			possible_traits_.clear();
 		}
 
-		if(cfg_["ignore_race_traits"].to_bool()) {
+		if(cfg["ignore_race_traits"].to_bool()) {
 			possible_traits_.clear();
 		} else {
 			for(const config& t : race_->additional_traits()) {
@@ -331,29 +361,11 @@ void unit_type::build_help_index(
 	}
 
 	// Insert any traits that are just for this unit type
-	for(const config& trait : cfg_.child_range("trait")) {
+	for(const config& trait : cfg.child_range("trait")) {
 		possible_traits_.add_child("trait", trait);
 	}
 
-	for(const config& var_cfg : cfg_.child_range("variation")) {
-		const std::string& var_id = var_cfg["variation_id"].empty()
-			? var_cfg["variation_name"]
-			: var_cfg["variation_id"];
-
-		variations_map::iterator ut;
-		bool success;
-		std::tie(ut, success) = variations_.emplace(var_id, unit_type(var_cfg, id_));
-
-		if(success) {
-			ut->second.debug_id_ = debug_id_ + " [" + var_id + "]";
-			ut->second.base_id_ = base_id_; // In case this is not id_.
-			ut->second.build_help_index(mv_types, races, traits);
-		} else {
-			ERR_CF << "Skipping duplicate unit variation ID: " << var_id << "\n";
-		}
-	}
-
-	hide_help_ = cfg_["hide_help"].to_bool();
+	hide_help_ = cfg["hide_help"].to_bool();
 
 	build_status_ = HELP_INDEXED;
 }
@@ -363,45 +375,43 @@ void unit_type::build_help_index(
  * This creates the gender-specific types (if needed) and also defines how much
  * experience is needed to advance as well as what this advances to.
  */
-void unit_type::build_created(
-		const movement_type_map& mv_types, const race_map& races, const config::const_child_itors& traits)
+void unit_type::build_created()
 {
 	// Don't build twice.
 	if(CREATED <= build_status_) {
 		return;
 	}
 
-	// There is no preceding build level (other than being constructed).
-
-	// These should still be nullptr from the constructor.
-	assert(!gender_types_[0]);
-	assert(!gender_types_[1]);
-
-	if(const config& male_cfg = cfg_.child("male")) {
-		gender_types_[0].reset(new unit_type(male_cfg, id_));
-		gender_types_[0]->debug_id_ = debug_id_ + " (male)";
-	}
-
-	if(const config& female_cfg = cfg_.child("female")) {
-		gender_types_[1].reset(new unit_type(female_cfg, id_));
-		gender_types_[1]->debug_id_ = debug_id_ + " (female)";
-	}
 
 	for(unsigned i = 0; i < gender_types_.size(); ++i) {
 		if(gender_types_[i]) {
-			gender_types_[i]->build_created(mv_types, races, traits);
+			gender_types_[i]->build_created();
 		}
 	}
 
-	const std::string& advances_to_val = cfg_["advances_to"];
+	for(auto& pair : variations_) {
+		pair.second.build_created();
+	}
+
+
+	const config& cfg = get_cfg();
+
+	const std::string& advances_to_val = cfg["advances_to"];
 	if(advances_to_val != "null" && !advances_to_val.empty()) {
 		advances_to_ = utils::split(advances_to_val);
 	}
 
+
+	type_name_ = cfg["name"].t_str();
+	variation_name_ = cfg["variation_name"].t_str();
+
 	DBG_UT << "unit_type '" << log_id() << "' advances to : " << advances_to_val << "\n";
 
-	experience_needed_ = cfg_["experience"].to_int(500);
+	experience_needed_ = cfg["experience"].to_int(500);
+	cost_ = cfg["cost"].to_int(1);
 
+	//needed by the editor.
+	image_ = cfg["image"].str();
 	build_status_ = CREATED;
 }
 
@@ -411,7 +421,7 @@ void unit_type::build_created(
 void unit_type::build(BUILD_STATUS status,
 		const movement_type_map& movement_types,
 		const race_map& races,
-		const config::const_child_itors& traits)
+		const config_array_view& traits)
 {
 	DBG_UT << "Building unit type " << log_id() << ", level " << status << '\n';
 
@@ -422,7 +432,7 @@ void unit_type::build(BUILD_STATUS status,
 
 	case CREATED:
 		// Build the basic data.
-		build_created(movement_types, races, traits);
+		build_created();
 		return;
 
 	case VARIATIONS: // Implemented as part of HELP_INDEXED
@@ -456,7 +466,7 @@ const unit_type& unit_type::get_gender_unit_type(std::string gender) const
 
 const unit_type& unit_type::get_gender_unit_type(unit_race::GENDER gender) const
 {
-	const size_t i = gender;
+	const std::size_t i = gender;
 	if(i < gender_types_.size() && gender_types_[i] != nullptr) {
 		return *gender_types_[i];
 	}
@@ -483,10 +493,51 @@ t_string unit_type::unit_description() const
 	}
 }
 
+std::vector<t_string> unit_type::special_notes() const {
+	return combine_special_notes(special_notes_, abilities_cfg(), attacks(), movement_type());
+}
+
+static void append_special_note(std::vector<t_string>& notes, const t_string& new_note) {
+	if(new_note.empty()) return;
+	std::string_view note_plain = new_note.c_str();
+	utils::trim(note_plain);
+	if(note_plain.empty()) return;
+	auto iter = std::find(notes.begin(), notes.end(), new_note);
+	if(iter != notes.end()) return;
+	notes.push_back(new_note);
+}
+
+std::vector<t_string> combine_special_notes(const std::vector<t_string> direct, const config& abilities, const_attack_itors attacks, const movetype& mt)
+{
+	std::vector<t_string> notes;
+	for(const auto& note : direct) {
+		append_special_note(notes, note);
+	}
+	for(const config::any_child ability : abilities.all_children_range()) {
+		if(ability.cfg.has_attribute("special_note")) {
+			append_special_note(notes, ability.cfg["special_note"].t_str());
+		}
+	}
+	for(const auto& attack : attacks) {
+		for(const config::any_child ability : attack.specials().all_children_range()) {
+			if(ability.cfg.has_attribute("special_note")) {
+				append_special_note(notes, ability.cfg["special_note"].t_str());
+			}
+		}
+		if(auto attack_type_note = string_table.find("special_note_damage_type_" + attack.type()); attack_type_note != string_table.end()) {
+			append_special_note(notes, attack_type_note->second);
+		}
+	}
+	for(const auto& move_note : mt.special_notes()) {
+		append_special_note(notes, move_note);
+	}
+	return notes;
+}
+
 const std::vector<unit_animation>& unit_type::animations() const
 {
 	if(animations_.empty()) {
-		unit_animation::fill_initial_animations(animations_, cfg_);
+		unit_animation::fill_initial_animations(animations_, get_cfg());
 	}
 
 	return animations_;
@@ -498,7 +549,7 @@ const_attack_itors unit_type::attacks() const
 		return make_attack_itors(attacks_cache_);
 	}
 
-	for(const config& att : cfg_.child_range("attack")) {
+	for(const config& att : get_cfg().child_range("attack")) {
 		attacks_cache_.emplace_back(new attack_type(att));
 	}
 
@@ -540,21 +591,9 @@ int unit_type::experience_needed(bool with_acceleration) const
 	return experience_needed_;
 }
 
-#if 0
-const char* unit_type::alignment_description(unit_type::ALIGNMENT align, unit_race::GENDER gender)
-{
-	static const char* aligns[] { N_("lawful"), N_("neutral"), N_("chaotic"), N_("liminal") };
-	static const char* aligns_female[] { N_("female^lawful"), N_("female^neutral"), N_("female^chaotic"),
-N_("female^liminal") };
-	const char** tlist = (gender == unit_race::MALE ? aligns : aligns_female);
-
-	return (translation::sgettext(tlist[align]));
-}
-#endif
-
 bool unit_type::has_ability_by_id(const std::string& ability) const
 {
-	if(const config& abil = cfg_.child("abilities")) {
+	if(const config& abil = get_cfg().child("abilities")) {
 		for(const config::any_child& ab : abil.all_children_range()) {
 			if(ab.cfg["id"] == ability) {
 				return true;
@@ -569,7 +608,7 @@ std::vector<std::string> unit_type::get_ability_list() const
 {
 	std::vector<std::string> res;
 
-	const config& abilities = cfg_.child("abilities");
+	const config& abilities = get_cfg().child("abilities");
 	if(!abilities) {
 		return res;
 	}
@@ -590,67 +629,6 @@ bool unit_type::hide_help() const
 	return hide_help_ || unit_types.hide_help(id_, race_->id());
 }
 
-void unit_type::add_advancement(const unit_type& to_unit, int xp)
-{
-	const std::string& to_id = to_unit.id_;
-
-	// Add extra advancement path to this unit type
-	LOG_CONFIG << "adding advancement from " << log_id() << " to " << to_unit.log_id() << "\n";
-	if(std::find(advances_to_.begin(), advances_to_.end(), to_id) == advances_to_.end()) {
-		advances_to_.push_back(to_id);
-	} else {
-		LOG_CONFIG << "advancement from " << log_id() << " to " << to_unit.log_id() << " already known, ignoring.\n";
-		return;
-	}
-
-	if(xp > 0) {
-		// xp is 0 in case experience= wasn't given.
-		if(!in_advancefrom_) {
-			// This function is called for and only for an [advancefrom] tag in a unit_type referencing this unit_type.
-			in_advancefrom_ = true;
-			experience_needed_ = xp;
-
-			DBG_UT << "Changing experience_needed from " << experience_needed_ << " to " << xp
-				   << " due to (first) [advancefrom] of " << to_unit.log_id() << "\n";
-		} else if(experience_needed_ > xp) {
-			experience_needed_ = xp;
-			DBG_UT << "Lowering experience_needed from " << experience_needed_ << " to " << xp
-				   << " due to (multiple, lower) [advancefrom] of " << to_unit.log_id() << "\n";
-		} else {
-			DBG_UT << "Ignoring experience_needed change from " << experience_needed_ << " to " << xp
-				   << " due to (multiple, higher) [advancefrom] of " << to_unit.log_id() << "\n";
-		}
-	}
-
-	// Add advancements to gendered subtypes, if supported by to_unit
-	for(int gender = 0; gender <= 1; ++gender) {
-		if(!gender_types_[gender]) {
-			continue;
-		}
-
-		if(!to_unit.gender_types_[gender]) {
-			WRN_CF << to_unit.log_id() << " does not support gender " << gender << std::endl;
-			continue;
-		}
-
-		LOG_CONFIG << "gendered advancement " << gender << ": ";
-		gender_types_[gender]->add_advancement(*(to_unit.gender_types_[gender]), xp);
-	}
-
-	if(cfg_.has_child("variation")) {
-		// Make sure the variations are created.
-		unit_types.build_unit_type(*this, VARIATIONS);
-
-		// Add advancements to variation subtypes.
-		// Since these are still a rare and special-purpose feature,
-		// we assume that the unit designer knows what they're doing,
-		// and don't block advancements that would remove a variation.
-		for(auto& v : variations_) {
-			LOG_CONFIG << "variation advancement: ";
-			v.second.add_advancement(to_unit, xp);
-		}
-	}
-}
 
 static void advancement_tree_internal(const std::string& id, std::set<std::string>& tree)
 {
@@ -795,58 +773,12 @@ bool unit_type::show_variations_in_help() const
 	return false;
 }
 
-/**
- * Generates (and returns) a trimmed config suitable for use with units.
- */
-const config& unit_type::build_unit_cfg() const
-{
-	// We start with all attributes.
-	assert(unit_cfg_.empty());
-	unit_cfg_.append_attributes(cfg_);
-
-	// Remove "pure" unit_type attributes (attributes that do not get directly
-	// copied to units; some do get copied, but under different keys).
-	static char const* unit_type_attrs[] {
-		"attacks",
-		"base_ids",
-		"die_sound",
-		"experience",
-		"flies",
-		"healed_sound",
-		"hide_help",
-		"hitpoints",
-		"id",
-		"ignore_race_traits",
-		"inherit", "movement",
-		"movement_type",
-		"name",
-		"num_traits",
-		"variation_id",
-		"variation_name",
-		"recall_cost",
-		"cost",
-		"level",
-		"gender",
-		"flag_rgb",
-		"alignment",
-		"advances_to",
-		"do_not_list"
-	};
-
-	for(const char* attr : unit_type_attrs) {
-		unit_cfg_.remove_attribute(attr);
-	}
-
-	built_unit_cfg_ = true;
-	return unit_cfg_;
-}
-
 int unit_type::resistance_against(const std::string& damage_name, bool attacker) const
 {
 	int resistance = movement_type_.resistance_against(damage_name);
 	unit_ability_list resistance_abilities;
 
-	if(const config& abilities = cfg_.child("abilities")) {
+	if(const config& abilities = get_cfg().child("abilities")) {
 		for(const config& cfg : abilities.child_range("resistance")) {
 			if(!cfg["affect_self"].to_bool(true)) {
 				continue;
@@ -856,7 +788,7 @@ int unit_type::resistance_against(const std::string& damage_name, bool attacker)
 				continue;
 			}
 
-			resistance_abilities.push_back(unit_ability(&cfg, map_location::null_location()));
+			resistance_abilities.emplace_back(&cfg, map_location::null_location(), map_location::null_location());
 		}
 	}
 
@@ -942,7 +874,7 @@ unit_type_data::unit_type_data()
 	, hide_help_all_(false)
 	, hide_help_type_()
 	, hide_help_race_()
-	, unit_cfg_(nullptr)
+	, units_cfg_()
 	, build_status_(unit_type::NOT_BUILT)
 {
 }
@@ -972,110 +904,6 @@ void throw_base_unit_recursion_error(const std::vector<std::string>& base_tree,
 	throw config::error(ss.str());
 }
 
-/**
- * Locates the config for the unit type with id= @a key within @a all_types.
- * Throws a config::error if the unit type cannot be found.
- */
-config& find_unit_type_config(const std::string& key, config& all_types)
-{
-	config& cfg = all_types.find_child("unit_type", "id", key);
-	if(cfg) {
-		return cfg;
-	}
-
-	// Bad WML!
-	ERR_CF << "unit type not found: " << key << std::endl;
-	ERR_CF << all_types << std::endl;
-	throw config::error("unit type not found: " + key);
-}
-
-/**
- * Modifies the provided config by merging all base units into it.
- * The @a base_tree parameter is used solely for detecting and reporting
- * cycles of base units; it is no longer needed to prevent infinite loops.
- */
-void apply_base_unit(config& ut_cfg, config& all_types, std::vector<std::string>& base_tree)
-{
-	// Get a list of base units to apply.
-	std::vector<std::string> base_ids;
-	for(config& base : ut_cfg.child_range("base_unit")) {
-		base_ids.push_back(base["id"]);
-	}
-
-	// Nothing to do.
-	if(base_ids.empty()) {
-		return;
-	}
-
-	// Store the base ids for the help system.
-	ut_cfg["base_ids"] = utils::join(base_ids);
-
-	// Clear the base units (otherwise they could interfere with the merge).
-	// This has the side-effect of breaking cycles, hence base_tree is
-	// merely for error detection, not error recovery.
-	ut_cfg.clear_children("base_unit");
-
-	// Merge the base units, in order.
-	for(const std::string& base_id : base_ids) {
-		// Detect recursion so the WML author is made aware of an error.
-		if(std::find(base_tree.begin(), base_tree.end(), base_id) != base_tree.end()) {
-			throw_base_unit_recursion_error(base_tree, base_id);
-		}
-
-		// Find the base unit.
-		config& base_cfg = find_unit_type_config(base_id, all_types);
-
-		// Make sure the base unit has had its base units accounted for.
-		base_tree.push_back(base_id);
-
-		apply_base_unit(base_cfg, all_types, base_tree);
-
-		base_tree.pop_back();
-
-		// Merge the base unit "under" our config.
-		ut_cfg.inherit_from(base_cfg);
-	}
-}
-
-/**
- * Handles inheritance for configs of [male], [female], and [variation].
- * Also removes gendered children, as those serve no purpose.
- * @a default_inherit is the default value for inherit=.
- */
-void fill_unit_sub_type(config& var_cfg, const config& parent, bool default_inherit)
-{
-	if(var_cfg["inherit"].to_bool(default_inherit)) {
-		var_cfg.inherit_from(parent);
-	}
-
-	var_cfg.clear_children("male");
-	var_cfg.clear_children("female");
-}
-
-/**
- * Processes [variation] tags of @a ut_cfg, handling inheritance and
- * child clearing.
- */
-void handle_variations(config& ut_cfg)
-{
-	// Most unit types do not have variations.
-	if(!ut_cfg.has_child("variation")) {
-		return;
-	}
-
-	// Pull the variations out of the base unit type.
-	config variations;
-	variations.splice_children(ut_cfg, "variation");
-
-	// Handle each variation's inheritance.
-	for(config& var_cfg : variations.child_range("variation")) {
-		fill_unit_sub_type(var_cfg, ut_cfg, false);
-	}
-
-	// Restore the variations.
-	ut_cfg.splice_children(variations, "variation");
-}
-
 /**
  * Insert a new value into a movetype, possibly calculating the value based on
  * the existing values in the target movetype.
@@ -1152,17 +980,125 @@ void patch_movetype(movetype& mt,
 }
 } // unnamed namespace
 
+/**
+ * Modifies the provided config by merging all base units into it.
+ * The @a base_tree parameter is used for detecting and reporting
+ * cycles of base units and in particular to prevent infinite loops.
+ */
+
+void unit_type_data::apply_base_unit(unit_type& type, std::vector<std::string>& base_tree)
+{
+	// Nothing to do.
+	if(type.base_unit_id_.empty()) {
+		return;
+	}
+
+	// Detect recursion so the WML author is made aware of an error.
+	if(std::find(base_tree.begin(), base_tree.end(), type.base_unit_id_) != base_tree.end()) {
+		throw_base_unit_recursion_error(base_tree, type.base_unit_id_);
+	}
+
+	// Find the base unit.
+	const unit_type_map::iterator itor = types_.find(type.base_unit_id_);
+	if(itor != types_.end()) {
+
+		unit_type& base_type = itor->second;
+
+		// Make sure the base unit has had its base units accounted for.
+		base_tree.push_back(type.base_unit_id_);
+
+		apply_base_unit(base_type, base_tree);
+
+		base_tree.pop_back();
+
+		// Merge the base unit "under" our config.
+		type.writable_cfg().inherit_from(base_type.get_cfg());
+	}
+	else {
+		ERR_CF << "[base_unit]: unit type not found: " << type.base_unit_id_ << std::endl;
+		throw config::error("unit type not found: " + type.base_unit_id_);
+	}
+}
+
+/**
+ * Handles inheritance for configs of [male], [female], and [variation].
+ * Also removes gendered children, as those serve no purpose.
+ * @a default_inherit is the default value for inherit=.
+ */
+std::unique_ptr<unit_type> unit_type::create_sub_type(const config& var_cfg, bool default_inherit)
+{
+	config var_copy =  var_cfg;
+	if(var_cfg["inherit"].to_bool(default_inherit)) {
+		var_copy.inherit_from(get_cfg());
+	}
+
+	var_copy.clear_children("male");
+	var_copy.clear_children("female");
+
+	return std::make_unique<unit_type>(std::move(var_copy), parent_id());
+}
+
+/**
+ * Processes [variation] tags of @a ut_cfg, handling inheritance and
+ * child clearing.
+ */
+void unit_type::fill_variations()
+{
+	// Most unit types do not have variations.
+	if(!get_cfg().has_child("variation")) {
+		return;
+	}
+
+	// Handle each variation's inheritance.
+	for(const config& var_cfg : get_cfg().child_range("variation")) {
+
+		std::unique_ptr<unit_type> var = create_sub_type(var_cfg, false);
+
+		var->built_cfg_->remove_children("variation", [](const config&){return true;});
+		var->variation_id_ = var_cfg["variation_id"].str();
+		var->debug_id_ = debug_id_ + " [" + var->variation_id_ + "]";
+
+		variations_map::iterator ut;
+		bool success;
+		std::tie(ut, success) = variations_.emplace(var_cfg["variation_id"].str(), std::move(*var));
+		if(!success) {
+			ERR_CF << "Skipping duplicate unit variation ID: " << var_cfg["variation_id"] << "\n";
+		}
+	}
+
+
+}
+
+
+void unit_type::fill_variations_and_gender()
+{
+	// Complete the gender-specific children of the config.
+	if(const config& male_cfg = get_cfg().child("male")) {
+		gender_types_[0] = create_sub_type(male_cfg, true);
+		gender_types_[0]->fill_variations();
+	}
+
+	if(const config& female_cfg = get_cfg().child("female")) {
+		gender_types_[1] = create_sub_type(female_cfg, true);
+		gender_types_[1]->fill_variations();
+	}
+
+	// Complete the variation-defining children of the config.
+	fill_variations();
+
+	gui2::dialogs::loading_screen::progress();
+}
 /**
  * Resets all data based on the provided config.
  * This includes some processing of the config, such as expanding base units.
  * A pointer to the config is stored, so the config must be persistent.
  */
-void unit_type_data::set_config(config& cfg)
+void unit_type_data::set_config(const game_config_view& cfg)
 {
-	DBG_UT << "unit_type_data::set_config, name: " << cfg["name"] << "\n";
+	LOG_UT << "unit_type_data::set_config, nunits: " << cfg.child_range("unit_type").size() << "\n";
 
 	clear();
-	unit_cfg_ = &cfg;
+	units_cfg_ = cfg;
 
 	for(const config& mt : cfg.child_range("movetype")) {
 		movement_types_.emplace(mt["name"].str(), movetype(mt));
@@ -1274,51 +1210,32 @@ void unit_type_data::set_config(config& cfg)
 	}
 	DBG_CF << "End of movetype patching" << std::endl;
 
-	// Apply base units.
-	for(config& ut : cfg.child_range("unit_type")) {
-		if(ut.has_child("base_unit")) {
-			// Derived units must specify a new id.
-			// (An error message will be emitted later if id is empty.)
-			const std::string id = ut["id"];
-			if(!id.empty()) {
-				std::vector<std::string> base_tree(1, id);
-				apply_base_unit(ut, cfg, base_tree);
-
-				gui2::dialogs::loading_screen::progress();
-			}
-		}
-	}
-
-	// Handle inheritance and recording of unit types.
-	for(config& ut : cfg.child_range("unit_type")) {
-		std::string id = ut["id"];
-
+	for(const config& ut : cfg.child_range("unit_type")) {
 		// Every type is required to have an id.
+		std::string id = ut["id"].str();
 		if(id.empty()) {
 			ERR_CF << "[unit_type] with empty id=, ignoring:\n" << ut.debug();
 			continue;
 		}
 
-		// Complete the gender-specific children of the config.
-		if(config& male_cfg = ut.child("male")) {
-			fill_unit_sub_type(male_cfg, ut, true);
-			handle_variations(male_cfg);
-		}
-
-		if(config& female_cfg = ut.child("female")) {
-			fill_unit_sub_type(female_cfg, ut, true);
-			handle_variations(female_cfg);
-		}
-
-		// Complete the variation-defining children of the config.
-		handle_variations(ut);
-
-		// Record this unit type.
 		if(types_.emplace(id, unit_type(ut)).second) {
 			LOG_CONFIG << "added " << id << " to unit_type list (unit_type_data.unit_types)\n";
 		} else {
 			ERR_CF << "Multiple [unit_type]s with id=" << id << " encountered." << std::endl;
 		}
+	}
+
+	// Apply base units.
+	for(auto& type : types_) {
+		std::vector<std::string> base_tree(1, type.second.id());
+		apply_base_unit(type.second, base_tree);
+
+		gui2::dialogs::loading_screen::progress();
+	}
+
+	//handle [male], [female], [variation]
+	for(auto& type : types_) {
+		type.second.fill_variations_and_gender();
 
 		gui2::dialogs::loading_screen::progress();
 	}
@@ -1331,6 +1248,12 @@ void unit_type_data::set_config(config& cfg)
 		hide_help_all_ = hide_help["all"].to_bool();
 		read_hide_help(hide_help);
 	}
+	DBG_UT << "Finished creatign unti types\n";
+}
+
+void unit_type_data::build_unit_type(const unit_type & ut, unit_type::BUILD_STATUS status) const
+{
+	ut.build(status, movement_types_, races_, units_cfg().child_range("trait"));
 }
 
 /**
@@ -1347,11 +1270,7 @@ const unit_type* unit_type_data::find(const std::string& key, unit_type::BUILD_S
 
 	// This might happen if units of another era are requested (for example for savegames)
 	if(itor == types_.end()) {
-#if 0
-		for(unit_type_map::const_iterator ut = types_.begin(); ut != types_.end(); ut++) {
-			DBG_UT << "Known unit_types: key = '" << ut->first << "', id = '" << ut->second.log_id() << "'\n";
-		}
-#endif
+		DBG_CF << "unable to find " << key << " in unit_type list (unit_type_data.unit_types)\n";
 		return nullptr;
 	}
 
@@ -1389,23 +1308,12 @@ void unit_type_data::build_all(unit_type::BUILD_STATUS status)
 		return;
 	}
 
-	assert(unit_cfg_ != nullptr);
-
 	for(const auto& type : types_) {
 		build_unit_type(type.second, status);
 
 		gui2::dialogs::loading_screen::progress();
 	}
 
-	// Handle [advancefrom] (once) after building to (at least) the CREATED level.
-	// (Currently, this could be simply a test for build_status_ == NOT_BUILT,
-	// but to guard against future changes, use a more thorough test.)
-	if(build_status_ < unit_type::CREATED && unit_type::CREATED <= status) {
-		for(auto& type : types_) {
-			add_advancement(type.second);
-		}
-	}
-
 	build_status_ = status;
 }
 
@@ -1457,36 +1365,93 @@ bool unit_type_data::hide_help(const std::string& type, const std::string& race)
 	return res;
 }
 
-void unit_type_data::add_advancement(unit_type& to_unit) const
-{
-	const config& cfg = to_unit.get_cfg();
-
-	for(const config& af : cfg.child_range("advancefrom")) {
-		const std::string& from = af["unit"];
-		int xp = af["experience"];
-
-		unit_type_data::unit_type_map::iterator from_unit = types_.find(from);
-
-		if(from_unit == types_.end()) {
-			std::ostringstream msg;
-			msg << "unit type '" << from << "' not found when resolving [advancefrom] tag for '" << to_unit.log_id()
-				<< "'";
-			throw config::error(msg.str());
-		}
-
-		// Fix up advance_from references
-		from_unit->second.add_advancement(to_unit, xp);
-
-		DBG_UT << "Added advancement ([advancefrom]) from " << from << " to " << to_unit.log_id() << "\n";
-	}
-}
-
 const unit_race* unit_type_data::find_race(const std::string& key) const
 {
 	race_map::const_iterator i = races_.find(key);
 	return i != races_.end() ? &i->second : nullptr;
 }
 
+void unit_type::apply_scenario_fix(const config& cfg)
+{
+	build_created();
+	if(auto p_setxp = cfg.get("set_experience")) {
+		experience_needed_ = p_setxp->to_int();
+	}
+	if(auto attr = cfg.get("set_advances_to")) {
+		advances_to_ = utils::split(attr->str());
+	}
+	if(auto attr = cfg.get("set_cost")) {
+		cost_ = attr->to_int(1);
+	}
+	if(auto attr = cfg.get("add_advancement")) {
+		for(const auto& str : utils::split(attr->str())) {
+			advances_to_.push_back(str);
+		}
+	}
+	if(auto attr = cfg.get("remove_advancement")) {
+		for(const auto& str : utils::split(attr->str())) {
+			boost::remove_erase(advances_to_, str);
+		}
+	}
+
+	// apply recursively to subtypes.
+	for(int gender = 0; gender <= 1; ++gender) {
+		if(!gender_types_[gender]) {
+			continue;
+		}
+		gender_types_[gender]->apply_scenario_fix(cfg);
+	}
+
+	if(get_cfg().has_child("variation")) {
+		// Make sure the variations are created.
+		unit_types.build_unit_type(*this, VARIATIONS);
+		for(auto& v : variations_) {
+			v.second.apply_scenario_fix(cfg);
+		}
+	}
+}
+
+void unit_type_data::apply_scenario_fix(const config& cfg)
+{
+	unit_type_map::iterator itor = types_.find(cfg["type"].str());
+	// This might happen if units of another era are requested (for example for savegames)
+	if(itor != types_.end()) {
+		itor->second.apply_scenario_fix(cfg);
+	}
+	else {
+		// should we give an error message?
+	}
+}
+
+void unit_type::remove_scenario_fixes()
+{
+	advances_to_.clear();
+	const std::string& advances_to_val = get_cfg()["advances_to"];
+	if(advances_to_val != "null" && !advances_to_val.empty()) {
+		advances_to_ = utils::split(advances_to_val);
+	}
+	experience_needed_ = get_cfg()["experience"].to_int(500);
+	cost_ = get_cfg()["cost"].to_int(1);
+
+	// apply recursively to subtypes.
+	for(int gender = 0; gender <= 1; ++gender) {
+		if(!gender_types_[gender]) {
+			continue;
+		}
+		gender_types_[gender]->remove_scenario_fixes();
+	}
+	for(auto& v : variations_) {
+		v.second.remove_scenario_fixes();
+	}
+}
+
+void unit_type_data::remove_scenario_fixes()
+{
+	for(auto& pair : types_) {
+		pair.second.remove_scenario_fixes();
+	}
+}
+
 void unit_type::check_id(std::string& id)
 {
 	assert(!id.empty());
@@ -1498,7 +1463,7 @@ void unit_type::check_id(std::string& id)
 
 	bool gave_warning = false;
 
-	for(size_t pos = 0; pos < id.size(); ++pos) {
+	for(std::size_t pos = 0; pos < id.size(); ++pos) {
 		const char c = id[pos];
 		const bool valid = std::isalnum(c, std::locale::classic()) || c == '_' || c == ' ';
 
