@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #include "units/udisplay.hpp"
 
 #include "fake_unit_manager.hpp"
@@ -31,16 +30,21 @@
 #include "units/animation_component.hpp"
 #include "units/filter.hpp"
 #include "units/map.hpp"
+#include "utils/scope_exit.hpp"
 
 #define LOG_DP LOG_STREAM(info, display)
 
 
+namespace unit_display
+{
+namespace
+{
 /**
  * Returns a string whose first line is @a number, centered over a second line,
  * which consists of @a text.
  * If the number is 0, the first line is suppressed.
  */
-static std::string number_and_text(int number, const std::string & text)
+std::string number_and_text(int number, const std::string& text)
 {
 	// Simple case.
 	if ( number == 0 )
@@ -65,8 +69,7 @@ static std::string number_and_text(int number, const std::string & text)
  * @param temp_unit  The unit to animate (historically, a temporary unit).
  * @param disp       The game display. Assumed neither locked nor faked.
  */
-static void teleport_unit_between(const map_location& a, const map_location& b,
-                                  unit& temp_unit, display& disp)
+void teleport_unit_between(const map_location& a, const map_location& b, unit& temp_unit, display& disp)
 {
 	if ( disp.fogged(a) && disp.fogged(b) ) {
 		return;
@@ -74,8 +77,8 @@ static void teleport_unit_between(const map_location& a, const map_location& b,
 	const display_context& dc = disp.get_disp_context();
 	const team& viewing_team = dc.get_team(disp.viewing_side());
 
-	const bool a_visible = temp_unit.is_visible_to_team(a, viewing_team, dc, false);
-	const bool b_visible = temp_unit.is_visible_to_team(b, viewing_team, dc, false);
+	const bool a_visible = temp_unit.is_visible_to_team(a, viewing_team, false);
+	const bool b_visible = temp_unit.is_visible_to_team(b, viewing_team, false);
 
 	temp_unit.set_location(a);
 	if ( a_visible ) { // teleport
@@ -86,7 +89,7 @@ static void teleport_unit_between(const map_location& a, const map_location& b,
 		else
 			disp.scroll_to_tile(a, game_display::ONSCREEN, true, false);
 		unit_animator animator;
-		animator.add_animation(&temp_unit,"pre_teleport",a);
+		animator.add_animation(temp_unit.shared_from_this(),"pre_teleport",a);
 		animator.start_animations();
 		animator.wait_for_end();
 	}
@@ -100,7 +103,7 @@ static void teleport_unit_between(const map_location& a, const map_location& b,
 		else
 			disp.scroll_to_tile(b, game_display::ONSCREEN, true, false);
 		unit_animator animator;
-		animator.add_animation(&temp_unit,"post_teleport",b);
+		animator.add_animation(temp_unit.shared_from_this(),"post_teleport",b);
 		animator.start_animations();
 		animator.wait_for_end();
 	}
@@ -126,10 +129,13 @@ static void teleport_unit_between(const map_location& a, const map_location& b,
  * @returns  The animation potential until this animation will finish.
  *           INT_MIN indicates that no animation is pending.
  */
-static int move_unit_between(const map_location& a, const map_location& b,
-                             unit_ptr temp_unit, unsigned int step_num,
-                             unsigned int step_left, unit_animator & animator,
-                             display& disp)
+int move_unit_between(const map_location& a,
+		const map_location& b,
+		unit_ptr temp_unit,
+		unsigned int step_num,
+		unsigned int step_left,
+		unit_animator& animator,
+		display& disp)
 {
 	if ( disp.fogged(a) && disp.fogged(b) ) {
 		return INT_MIN;
@@ -138,7 +144,7 @@ static int move_unit_between(const map_location& a, const map_location& b,
 	temp_unit->set_location(a);
 	disp.invalidate(a);
 	temp_unit->set_facing(a.get_relative_dir(b));
-	animator.replace_anim_if_invalid(temp_unit.get(),"movement",a,b,step_num,
+	animator.replace_anim_if_invalid(temp_unit,"movement",a,b,step_num,
 			false,"",{0,0,0},unit_animation::hit_type::INVALID,nullptr,nullptr,step_left);
 	animator.start_animations();
 	animator.pause_animation();
@@ -158,9 +164,14 @@ static int move_unit_between(const map_location& a, const map_location& b,
 	return target_time;
 }
 
-namespace unit_display
+bool do_not_show_anims(display* disp)
 {
 
+	return !disp || disp->video().update_locked() || disp->video().faked();
+}
+
+} // end anon namespace
+
 /**
  * The path must remain unchanged for the life of this object.
  */
@@ -274,14 +285,14 @@ void unit_mover::start(unit_ptr u)
 	disp_->invalidate(path_[0]);
 
 	// If the unit can be seen here by the viewing side:
-	if ( !is_enemy_ || !temp_unit_ptr_->invisible(path_[0], disp_->get_disp_context()) ) {
+	if(!is_enemy_ || !temp_unit_ptr_->invisible(path_[0])) {
 		// Scroll to the path, but only if it fully fits on screen.
 		// If it does not fit we might be able to do a better scroll later.
 		disp_->scroll_to_tiles(path_, game_display::ONSCREEN, true, true, 0.0, false);
 	}
 
 	// extra immobile movement animation for take-off
-	animator_.add_animation(temp_unit_ptr_.get(), "pre_movement", path_[0], path_[1]);
+	animator_.add_animation(temp_unit_ptr_.get_unit_ptr(), "pre_movement", path_[0], path_[1]);
 	animator_.start_animations();
 	animator_.wait_for_end();
 	animator_.clear();
@@ -305,7 +316,7 @@ void unit_mover::start(unit_ptr u)
  * wait (another call to proceed_to() or finish() will implicitly wait). The
  * unit must remain valid until the wait is finished.
  */
-void unit_mover::proceed_to(unit_ptr u, size_t path_index, bool update, bool wait)
+void unit_mover::proceed_to(unit_ptr u, std::size_t path_index, bool update, bool wait)
 {
 	// Nothing to do here if animations cannot be shown.
 	if ( !can_draw_ || !animate_ )
@@ -332,9 +343,9 @@ void unit_mover::proceed_to(unit_ptr u, size_t path_index, bool update, bool wai
 		// When that is the case, and the unit is invisible at path_[current_], we shouldn't
 		// scroll to that hex.
 		std::vector<map_location> locs;
-		if (!temp_unit_ptr_->invisible(path_[current_], disp_->get_disp_context()))
+		if (!temp_unit_ptr_->invisible(path_[current_]))
 			locs.push_back(path_[current_]);
-		if (!temp_unit_ptr_->invisible(path_[current_+1], disp_->get_disp_context()))
+		if (!temp_unit_ptr_->invisible(path_[current_+1]))
 			locs.push_back(path_[current_+1]);
 		// If the unit can be seen by the viewing side while making this step:
 		if ( !is_enemy_ || !locs.empty() )
@@ -394,18 +405,18 @@ void unit_mover::wait_for_anims()
 		animator_.wait_until(wait_until_);
 		// debug code, see unit_frame::redraw()
 		// std::cout << "   end\n";
-		/// @todo For wesnoth 1.14+: check if efficient for redrawing?
-		/// Check with large animated units too make sure artifacts are
-		/// not left on screen after unit movement in particular.
+		// TODO: For wesnoth 1.14+: check if efficient for redrawing?
+		// Check with large animated units too make sure artifacts are
+		// not left on screen after unit movement in particular.
 		if ( disp_ ) { // Should always be true if we get here.
 			// Invalidate the hexes around the move that prompted this wait.
-			adjacent_loc_array_t arr;
-			get_adjacent_tiles(path_[current_-1], arr.data());
-			for ( unsigned i = 0; i < arr.size(); ++i )
-				disp_->invalidate(arr[i]);
-			get_adjacent_tiles(path_[current_], arr.data());
-			for ( unsigned i = 0; i < arr.size(); ++i )
-				disp_->invalidate(arr[i]);
+			for(const map_location& adj : get_adjacent_tiles(path_[current_ - 1])) {
+				disp_->invalidate(adj);
+			}
+
+			for(const map_location& adj : get_adjacent_tiles(path_[current_])) {
+				disp_->invalidate(adj);
+			}
 		}
 	}
 
@@ -450,7 +461,7 @@ void unit_mover::finish(unit_ptr u, map_location::DIRECTION dir)
 		temp_unit_ptr_->set_facing(final_dir);
 
 		// Animation
-		animator_.add_animation(temp_unit_ptr_.get(), "post_movement", end_loc);
+		animator_.add_animation(temp_unit_ptr_.get_unit_ptr(), "post_movement", end_loc);
 		animator_.start_animations();
 		animator_.wait_for_end();
 		animator_.clear();
@@ -459,8 +470,7 @@ void unit_mover::finish(unit_ptr u, map_location::DIRECTION dir)
 		u->set_hidden(was_hidden_);
 		temp_unit_ptr_->set_hidden(true);
 
-		events::mouse_handler* mousehandler = events::mouse_handler::get_singleton();
-		if ( mousehandler ) {
+		if(events::mouse_handler* mousehandler = events::mouse_handler::get_singleton()) {
 			mousehandler->invalidate_reachmap();
 		}
 	}
@@ -490,6 +500,7 @@ void unit_mover::finish(unit_ptr u, map_location::DIRECTION dir)
  *                 (correct unit facing, path hexes redrawing).
  * @param dir      Unit will be set facing this direction after move.
  *                 If nothing passed, direction will be set based on path.
+ * @param force_scroll
  */
 /* Note: Hide the unit in its current location,
  * but don't actually remove it until the move is done,
@@ -511,28 +522,30 @@ void move_unit(const std::vector<map_location>& path, unit_ptr u,
 void reset_helpers(const unit *attacker,const unit *defender);
 
 void unit_draw_weapon(const map_location& loc, unit& attacker,
-		const_attack_ptr attack,const_attack_ptr secondary_attack, const map_location& defender_loc,unit* defender)
+		const_attack_ptr attack,const_attack_ptr secondary_attack, const map_location& defender_loc, unit_ptr defender)
 {
 	display* disp = display::get_singleton();
-	if(!disp ||disp->video().update_locked() || disp->video().faked() || disp->fogged(loc) || preferences::show_combat() == false) {
+	if(do_not_show_anims(disp) || disp->fogged(loc) || !preferences::show_combat()) {
 		return;
 	}
 	unit_animator animator;
 	attacker.set_facing(loc.get_relative_dir(defender_loc));
 	defender->set_facing(defender_loc.get_relative_dir(loc));
-	animator.add_animation(&attacker,"draw_weapon",loc,defender_loc,0,true,"",{0,0,0},unit_animation::hit_type::HIT,attack,secondary_attack,0);
-	animator.add_animation(defender,"draw_weapon",defender_loc,loc,0,true,"",{0,0,0},unit_animation::hit_type::MISS,secondary_attack,attack,0);
+	animator.add_animation(attacker.shared_from_this(),"draw_weapon",loc,defender_loc,0,true,"",{0,0,0},unit_animation::hit_type::HIT,attack,secondary_attack,0);
+	if(defender) {
+		animator.add_animation(defender,"draw_weapon",defender_loc,loc,0,true,"",{0,0,0},unit_animation::hit_type::MISS,secondary_attack,attack,0);
+	}
 	animator.start_animations();
 	animator.wait_for_end();
 
 }
 
 
-void unit_sheath_weapon(const map_location& primary_loc, unit* primary_unit,
-		const_attack_ptr primary_attack,const_attack_ptr secondary_attack, const map_location& secondary_loc,unit* secondary_unit)
+void unit_sheath_weapon(const map_location& primary_loc, unit_ptr primary_unit,
+		const_attack_ptr primary_attack,const_attack_ptr secondary_attack, const map_location& secondary_loc,unit_ptr secondary_unit)
 {
 	display* disp = display::get_singleton();
-	if(!disp ||disp->video().update_locked() || disp->video().faked() || disp->fogged(primary_loc) || preferences::show_combat() == false) {
+	if(do_not_show_anims(disp) || disp->fogged(primary_loc) || !preferences::show_combat()) {
 		return;
 	}
 	unit_animator animator;
@@ -553,30 +566,32 @@ void unit_sheath_weapon(const map_location& primary_loc, unit* primary_unit,
 	if(secondary_unit) {
 		secondary_unit->anim_comp().set_standing();
 	}
-	reset_helpers(primary_unit,secondary_unit);
+	reset_helpers(primary_unit.get(),secondary_unit.get());
 
 }
 
 
 void unit_die(const map_location& loc, unit& loser,
-		const_attack_ptr attack,const_attack_ptr secondary_attack, const map_location& winner_loc,unit* winner)
+		const_attack_ptr attack,const_attack_ptr secondary_attack, const map_location& winner_loc, unit_ptr winner)
 {
 	display* disp = display::get_singleton();
-	if(!disp ||disp->video().update_locked() || disp->video().faked() || disp->fogged(loc) || preferences::show_combat() == false) {
+	if(do_not_show_anims(disp) || disp->fogged(loc) || !preferences::show_combat()) {
 		return;
 	}
 	unit_animator animator;
 	// hide the hp/xp bars of the loser (useless and prevent bars around an erased unit)
-	animator.add_animation(&loser,"death",loc,winner_loc,0,false,"",{0,0,0},unit_animation::hit_type::KILL,attack,secondary_attack,0);
+	animator.add_animation(loser.shared_from_this(),"death",loc,winner_loc,0,false,"",{0,0,0},unit_animation::hit_type::KILL,attack,secondary_attack,0);
 	// but show the bars of the winner (avoid blinking and show its xp gain)
-	animator.add_animation(winner,"victory",winner_loc,loc,0,true,"",{0,0,0},
+	if(winner) {
+		animator.add_animation(winner,"victory",winner_loc,loc,0,true,"",{0,0,0},
 			unit_animation::hit_type::KILL,secondary_attack,attack,0);
+	}
 	animator.start_animations();
 	animator.wait_for_end();
 
-	reset_helpers(winner,&loser);
-	events::mouse_handler* mousehandler = events::mouse_handler::get_singleton();
-	if (mousehandler) {
+	reset_helpers(winner.get(), &loser);
+
+	if(events::mouse_handler* mousehandler = events::mouse_handler::get_singleton()) {
 		mousehandler->invalidate_reachmap();
 	}
 }
@@ -585,10 +600,10 @@ void unit_die(const map_location& loc, unit& loser,
 void unit_attack(display * disp, game_board & board,
                  const map_location& a, const map_location& b, int damage,
                  const attack_type& attack, const_attack_ptr secondary_attack,
-                 int swing,const std::string& hit_text,int drain_amount,const std::string& att_text, const std::vector<std::string>* extra_hit_sounds)
+                 int swing,const std::string& hit_text,int drain_amount,const std::string& att_text, const std::vector<std::string>* extra_hit_sounds,
+                 bool attacking)
 {
-	if(!disp ||disp->video().update_locked() || disp->video().faked() ||
-			(disp->fogged(a) && disp->fogged(b)) || preferences::show_combat() == false) {
+	if(do_not_show_anims(disp) || (disp->fogged(a) && disp->fogged(b)) || !preferences::show_combat()) {
 		return;
 	}
 	//const unit_map& units = disp->get_units();
@@ -607,17 +622,19 @@ void unit_attack(display * disp, game_board & board,
 	assert(def.valid());
 	unit &defender = *def;
 	int def_hitpoints = defender.hitpoints();
+	const_attack_ptr weapon = attack.shared_from_this();
+	auto ctx = weapon->specials_context(attacker.shared_from_this(), defender.shared_from_this(), a, b, attacking, secondary_attack);
+	std::optional<decltype(ctx)> opp_ctx;
+
+	if(secondary_attack) {
+		opp_ctx.emplace(secondary_attack->specials_context(defender.shared_from_this(), attacker.shared_from_this(), b, a, !attacking, weapon));
+	}
 
 	att->set_facing(a.get_relative_dir(b));
 	def->set_facing(b.get_relative_dir(a));
 	defender.set_facing(b.get_relative_dir(a));
 
-
-	unit_animator animator;
-	unit_ability_list leaders = attacker.get_abilities("leadership");
-	unit_ability_list helpers = defender.get_abilities("resistance");
-
-	std::string text   = number_and_text(damage, hit_text);
+	std::string text = number_and_text(damage, hit_text);
 	std::string text_2 = number_and_text(std::abs(drain_amount), att_text);
 
 	unit_animation::hit_type hit_type;
@@ -628,37 +645,99 @@ void unit_attack(display * disp, game_board & board,
 	}else {
 		hit_type = unit_animation::hit_type::MISS;
 	}
-	animator.add_animation(&attacker, "attack", att->get_location(),
-		def->get_location(), damage, true,  text_2,
-		(drain_amount >= 0) ? color_t(0,255,0) : color_t(255,0,0),
-		hit_type, attack.shared_from_this(), secondary_attack, swing);
+
+	unit_animator animator;
+
+	animator.add_animation(attacker.shared_from_this(), "attack", att->get_location(), def->get_location(), damage, true, text_2,
+		(drain_amount >= 0) ? color_t(0, 255, 0) : color_t(255, 0, 0), hit_type, weapon,
+		secondary_attack, swing);
 
 	// note that we take an anim from the real unit, we'll use it later
-	const unit_animation *defender_anim = def->anim_comp().choose_animation(*disp,
-		def->get_location(), "defend", att->get_location(), damage,
-		hit_type, attack.shared_from_this(), secondary_attack, swing);
-	animator.add_animation(&defender, defender_anim, def->get_location(),
-		true,  text , {255,0,0});
+	const unit_animation* defender_anim = def->anim_comp().choose_animation(*disp, def->get_location(), "defend",
+		att->get_location(), damage, hit_type, weapon, secondary_attack, swing);
 
-	for (const unit_ability & ability : leaders) {
-		if(ability.second == a) continue;
-		if(ability.second == b) continue;
-		unit_map::const_iterator leader = board.units().find(ability.second);
+	animator.add_animation(defender.shared_from_this(), defender_anim, def->get_location(), true, text, {255, 0, 0});
+
+	unit_ability_list leadership_list = attacker.get_abilities_weapons("leadership", weapon, secondary_attack);
+	unit_ability_list resistance_list = defender.get_abilities_weapons("resistance", secondary_attack, weapon);
+	for(const unit_ability& ability : leadership_list) {
+		if(ability.teacher_loc == a) {
+			continue;
+		}
+
+		if(ability.teacher_loc == b) {
+			continue;
+		}
+
+		unit_map::const_iterator leader = board.units().find(ability.teacher_loc);
 		assert(leader.valid());
-		leader->set_facing(ability.second.get_relative_dir(a));
-		animator.add_animation(&*leader, "leading", ability.second,
+		leader->set_facing(ability.teacher_loc.get_relative_dir(a));
+		animator.add_animation(leader.get_shared_ptr(), "leading", ability.teacher_loc,
 			att->get_location(), damage, true,  "", {0,0,0},
-			hit_type, attack.shared_from_this(), secondary_attack, swing);
+			hit_type, weapon, secondary_attack, swing);
 	}
-	for (const unit_ability & ability : helpers) {
-		if(ability.second == a) continue;
-		if(ability.second == b) continue;
-		unit_map::const_iterator helper = board.units().find(ability.second);
+
+	for(const unit_ability& ability : resistance_list) {
+		if(ability.teacher_loc == a) {
+			continue;
+		}
+
+		if(ability.teacher_loc == b) {
+			continue;
+		}
+
+		unit_map::const_iterator helper = board.units().find(ability.teacher_loc);
 		assert(helper.valid());
-		helper->set_facing(ability.second.get_relative_dir(b));
-		animator.add_animation(&*helper, "resistance", ability.second,
+		helper->set_facing(ability.teacher_loc.get_relative_dir(b));
+		animator.add_animation(helper.get_shared_ptr(), "resistance", ability.teacher_loc,
 			def->get_location(), damage, true,  "", {0,0,0},
-			hit_type, attack.shared_from_this(), secondary_attack, swing);
+			hit_type, weapon, secondary_attack, swing);
+	}
+
+	unit_ability_list abilities = att->get_location();
+	for(auto& special : attacker.checking_tags()) {
+		abilities.append(weapon->get_weapon_ability(special));
+	}
+
+	for(const unit_ability& ability : abilities) {
+		if(ability.teacher_loc == a) {
+			continue;
+		}
+
+		if(ability.teacher_loc == b) {
+			continue;
+		}
+
+		bool leading_playable = false;
+		bool helping_playable = false;
+		for(const unit_ability& leader_list : leadership_list) {
+			if(ability.teacher_loc == leader_list.teacher_loc) {
+				leading_playable = true;
+				break;
+			}
+		}
+
+		for(const unit_ability& helper_list : resistance_list) {
+			if(ability.teacher_loc == helper_list.teacher_loc) {
+				helping_playable = true;
+				break;
+			}
+		}
+
+		unit_map::const_iterator leader = board.units().find(ability.teacher_loc);
+		assert(leader.valid());
+		leader->set_facing(ability.teacher_loc.get_relative_dir(a));
+		if(animator.has_animation(leader.get_shared_ptr(), "leading", ability.teacher_loc,
+			att->get_location(), damage, hit_type, weapon, secondary_attack, swing) && leading_playable){
+				continue;
+		}
+		if(animator.has_animation(leader.get_shared_ptr(), "resistance", ability.teacher_loc,
+			def->get_location(), damage, hit_type, weapon, secondary_attack, swing) && helping_playable){
+				continue;
+		}
+		animator.add_animation(leader.get_shared_ptr(), "teaching", ability.teacher_loc,
+			att->get_location(), damage, true,  "", {0,0,0},
+			hit_type, weapon, secondary_attack, swing);
 	}
 
 
@@ -695,18 +774,24 @@ void reset_helpers(const unit *attacker,const unit *defender)
 	display* disp = display::get_singleton();
 	const unit_map& units = disp->get_units();
 	if(attacker) {
-		unit_ability_list leaders = attacker->get_abilities("leadership");
-		for (const unit_ability & ability : leaders) {
-			unit_map::const_iterator leader = units.find(ability.second);
+		unit_ability_list attacker_abilities = attacker->get_abilities("leadership");
+		for(auto& special : attacker->checking_tags()) {
+			attacker_abilities.append(attacker->get_abilities(special));
+		}
+		for(const unit_ability& ability : attacker_abilities) {
+			unit_map::const_iterator leader = units.find(ability.teacher_loc);
 			assert(leader != units.end());
 			leader->anim_comp().set_standing();
 		}
 	}
 
 	if(defender) {
-		unit_ability_list helpers = defender->get_abilities("resistance");
-		for (const unit_ability & ability : helpers) {
-			unit_map::const_iterator helper = units.find(ability.second);
+		unit_ability_list defender_abilities = defender->get_abilities("resistance");
+		for(auto& special : defender->checking_tags()) {
+			defender_abilities.append(defender->get_abilities(special));
+		}
+		for(const unit_ability& ability : defender_abilities) {
+			unit_map::const_iterator helper = units.find(ability.teacher_loc);
 			assert(helper != units.end());
 			helper->anim_comp().set_standing();
 		}
@@ -716,40 +801,47 @@ void reset_helpers(const unit *attacker,const unit *defender)
 void unit_recruited(const map_location& loc,const map_location& leader_loc)
 {
 	game_display* disp = game_display::get_singleton();
+	if(do_not_show_anims(disp) || (disp->fogged(loc) && disp->fogged(leader_loc))) {
+		return;
+	}
+
 	const display_context& dc = disp->get_disp_context();
 	const team& viewing_team = dc.get_team(disp->viewing_side());
-	if(!disp || disp->video().update_locked() || disp->video().faked() ||
-		(disp->fogged(loc) && disp->fogged(leader_loc))) return;
 
 	unit_map::const_iterator u = disp->get_units().find(loc);
 	if(u == disp->get_units().end()) return;
-	const bool unit_visible = u->is_visible_to_team(viewing_team, dc, false);
+	const bool unit_visible = u->is_visible_to_team(viewing_team, false);
 
 	unit_map::const_iterator leader = disp->get_units().find(leader_loc); // may be null_location
-	const bool leader_visible = (leader != disp->get_units().end()) && leader->is_visible_to_team(viewing_team, dc, false);
-
-	u->set_hidden(true);
+	const bool leader_visible = (leader != disp->get_units().end()) && leader->is_visible_to_team(viewing_team, false);
 
 	unit_animator animator;
-	if (leader_visible && unit_visible) {
-		disp->scroll_to_tiles(loc,leader_loc,game_display::ONSCREEN,true,0.0,false);
-	} else if (leader_visible) {
-		disp->scroll_to_tile(leader_loc,game_display::ONSCREEN,true,false);
-	} else if (unit_visible) {
-		disp->scroll_to_tile(loc,game_display::ONSCREEN,true,false);
-	} else {
-		return;
-	}
-	if (leader != disp->get_units().end()) {
-		leader->set_facing(leader_loc.get_relative_dir(loc));
-		if (leader_visible) {
-			animator.add_animation(&*leader, "recruiting", leader_loc, loc, 0, true);
-		}
-	}
 
-	disp->draw();
-	u->set_hidden(false);
-	animator.add_animation(&*u, "recruited", loc, leader_loc);
+	{
+		ON_SCOPE_EXIT(u) {
+			u->set_hidden(false);
+		};
+		u->set_hidden(true);
+
+		if (leader_visible && unit_visible) {
+			disp->scroll_to_tiles(loc,leader_loc,game_display::ONSCREEN,true,0.0,false);
+		} else if (leader_visible) {
+			disp->scroll_to_tile(leader_loc,game_display::ONSCREEN,true,false);
+		} else if (unit_visible) {
+			disp->scroll_to_tile(loc,game_display::ONSCREEN,true,false);
+		} else {
+			return;
+		}
+		if (leader != disp->get_units().end()) {
+			leader->set_facing(leader_loc.get_relative_dir(loc));
+			if (leader_visible) {
+				animator.add_animation(leader.get_shared_ptr(), "recruiting", leader_loc, loc, 0, true);
+			}
+		}
+
+		disp->draw();
+	}
+	animator.add_animation(u.get_shared_ptr(), "recruited", loc, leader_loc);
 	animator.start_animations();
 	animator.wait_for_end();
 	animator.set_all_standing();
@@ -760,8 +852,14 @@ void unit_healing(unit &healed, const std::vector<unit *> &healers, int healing,
                   const std::string & extra_text)
 {
 	game_display* disp = game_display::get_singleton();
-	const map_location &healed_loc = healed.get_location();
-	if(!disp || disp->video().update_locked() || disp->video().faked() || disp->fogged(healed_loc)) return;
+	const map_location& healed_loc = healed.get_location();
+	const bool some_healer_is_unfogged =
+		(healers.end() != std::find_if_not(healers.begin(), healers.end(),
+			[&](unit* h) { return disp->fogged(h->get_location()); }));
+
+	if(do_not_show_anims(disp) || (disp->fogged(healed_loc) && !some_healer_is_unfogged)) {
+		return;
+	}
 
 	// This is all the pretty stuff.
 	disp->scroll_to_tile(healed_loc, game_display::ONSCREEN,true,false);
@@ -770,22 +868,22 @@ void unit_healing(unit &healed, const std::vector<unit *> &healers, int healing,
 
 	for (unit *h : healers) {
 		h->set_facing(h->get_location().get_relative_dir(healed_loc));
-		animator.add_animation(h, "healing", h->get_location(),
+		animator.add_animation(h->shared_from_this(), "healing", h->get_location(),
 			healed_loc, healing);
 	}
 
 	if (healing < 0) {
-		animator.add_animation(&healed, "poisoned", healed_loc,
+		animator.add_animation(healed.shared_from_this(), "poisoned", healed_loc,
 		                       map_location::null_location(), -healing, false,
 		                       number_and_text(-healing, extra_text),
 		                       {255,0,0});
 	} else if ( healing > 0 ) {
-		animator.add_animation(&healed, "healed", healed_loc,
+		animator.add_animation(healed.shared_from_this(), "healed", healed_loc,
 		                       map_location::null_location(), healing, false,
 		                       number_and_text(healing, extra_text),
 		                       {0,255,0});
 	} else {
-		animator.add_animation(&healed, "healed", healed_loc,
+		animator.add_animation(healed.shared_from_this(), "healed", healed_loc,
 		                       map_location::null_location(), 0, false,
 		                       extra_text, {0,255,0});
 	}
