@@ -1,54 +1,66 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
+#include "movetype.hpp"
+
 #include "units/id.hpp"
 #include "units/ptr.hpp"
-#include "units/types.hpp"
+#include "units/attack_type.hpp"
+#include "units/race.hpp"
+#include "units/alignment.hpp"
+#include "utils/variant.hpp"
 
-#include "utils/type_trait_aliases.hpp"
+#include <boost/dynamic_bitset_fwd.hpp>
 
 #include <bitset>
-#include <boost/dynamic_bitset_fwd.hpp>
-#include <boost/ptr_container/ptr_vector.hpp>
-#include <boost/variant.hpp>
+#include <optional>
 
 class display;
-class display_context;
 class team;
 class unit_animation_component;
 class unit_formula_manager;
 class vconfig;
 struct color_t;
 
-namespace unit_detail
+/** Data typedef for unit_ability_list. */
+struct unit_ability
 {
-	template<typename T>
-	const T& get_or_default(const std::unique_ptr<T>& v)
+	unit_ability(const config* ability_cfg, map_location student_loc, map_location teacher_loc)
+		: student_loc(student_loc)
+		, teacher_loc(teacher_loc)
+		, ability_cfg(ability_cfg)
 	{
-		if(v) {
-			return *v;
-		} else {
-			static const T def;
-			return def;
-		}
 	}
-}
 
-// Data typedef for unit_ability_list.
-using unit_ability = std::pair<const config*, map_location>;
+	/**
+	 * Used by the formula in the ability.
+	 * The REAL location of the student (not the 'we are assuming the student is at this position' location)
+	 * once unit_ability_list can contain abilities from different 'students', as it contains abilities from
+	 * a unit aswell from its opponents (abilities with apply_to= opponent)
+	 */
+	map_location student_loc;
+	/**
+	 * The location of the teacher, that is the unit who owns the ability tags
+	 * (different from student because of [affect_adjacent])
+	 */
+	map_location teacher_loc;
+	/** The contents of the ability tag, never nullptr. */
+	const config* ability_cfg;
+};
 
 class unit_ability_list
 {
@@ -85,9 +97,18 @@ public:
 	const unit_ability& back()  const  { return cfgs_.back();  }
 
 	iterator erase(const iterator& erase_it)  { return cfgs_.erase(erase_it); }
-	void push_back(const unit_ability& ability)  { cfgs_.push_back(ability); }
+
+	template<typename... T>
+	void emplace_back(T&&... args) { cfgs_.emplace_back(args...); }
 
 	const map_location& loc() const { return loc_; }
+
+	/** Appens the abilities from @a other to @a this, ignores other.loc() */
+	void append(const unit_ability_list& other)
+	{
+		std::copy( other.begin(), other.end(), std::back_inserter(cfgs_ ));
+	}
+
 private:
 	// Data
 	std::vector<unit_ability> cfgs_;
@@ -97,7 +118,7 @@ private:
 /**
  * This class represents a *single* unit of a specific type.
  */
-class unit
+class unit : public std::enable_shared_from_this<unit>
 {
 public:
 	/**
@@ -108,21 +129,67 @@ public:
 
 	/** The path to the leader crown overlay. */
 	static const std::string& leader_crown();
+
 private:
 	void init(const config& cfg, bool use_traits = false, const vconfig* vcfg = nullptr);
 
-	void init(const unit_type& t, int side, bool real_unit, unit_race::GENDER gender = unit_race::NUM_GENDERS);
+	void init(const unit_type& t, int side, bool real_unit, unit_race::GENDER gender = unit_race::NUM_GENDERS, const std::string& variation = "");
 
 	// Copy constructor
 	unit(const unit& u);
 
-	unit();
+	struct unit_ctor_t {};
+
+public:
+	//private default ctor, butusing constructor to allow calling make_shared<unit> in create().
+	unit(unit_ctor_t);
+	unit() = delete;
+
+private:
+	enum UNIT_ATTRIBUTE
+	{
+		UA_MAX_HP,
+		UA_MAX_MP,
+		UA_MAX_AP,
+		UA_MAX_XP,
+		UA_IS_HEALTHY,
+		UA_IS_FEARLESS,
+		UA_LEVEL,
+		UA_MOVEMENT_TYPE,
+		UA_ZOC,
+		UA_ADVANCE_TO,
+		UA_ADVANCEMENTS,
+		UA_ALIGNMENT,
+		UA_UNDEAD_VARIATION,
+		//note that UA_ATTACKS only tracks added/deleted attacks, not modified attacks.
+		UA_ATTACKS,
+		UA_NOTES,
+		UA_PROFILE,
+		UA_SMALL_PROFILE,
+		UA_ABILITIES,
+		UA_UPKEEP,
+		UA_COUNT
+	};
+
+	void set_attr_changed(UNIT_ATTRIBUTE attr)
+	{
+		changed_attributes_[int(attr)] = true;
+	}
+
+	bool get_attacks_changed() const;
+
+	bool get_attr_changed(UNIT_ATTRIBUTE attr) const
+	{
+		return changed_attributes_[int(attr)];
+	}
+
+	void clear_changed_attributes();
 
 public:
 	/** Initializes a unit from a config */
 	static unit_ptr create(const config& cfg, bool use_traits = false, const vconfig* vcfg = nullptr)
 	{
-		unit_ptr res(new unit());
+		unit_ptr res = std::make_shared<unit>(unit_ctor_t());
 		res->init(cfg, use_traits, vcfg);
 		return res;
 	}
@@ -132,16 +199,17 @@ public:
 	 *
 	 * Only real_unit-s should have random traits, name and gender (to prevent OOS caused by RNG calls)
 	 */
-	static unit_ptr create(const unit_type& t, int side, bool real_unit, unit_race::GENDER gender = unit_race::NUM_GENDERS)
+	static unit_ptr create(const unit_type& t, int side, bool real_unit, unit_race::GENDER gender = unit_race::NUM_GENDERS, const std::string& variation = "")
 	{
-		unit_ptr res(new unit());
-		res->init(t, side, real_unit, gender);
+		unit_ptr res = std::make_shared<unit>(unit_ctor_t());
+		res->init(t, side, real_unit, gender, variation);
+
 		return res;
 	}
 
 	unit_ptr clone() const
 	{
-		return unit_ptr(new unit(*this));
+		return std::shared_ptr<unit>(new unit(*this));
 	}
 
 	virtual ~unit();
@@ -158,12 +226,13 @@ public:
 	/** Advances this unit to another type */
 	void advance_to(const unit_type& t, bool use_traits = false);
 
+	using advances_to_t = std::vector<std::string>;
 	/**
 	 * Gets the possible types this unit can advance to on level-up.
 	 *
 	 * @returns                   A list of type IDs this unit may advance to.
 	 */
-	const std::vector<std::string>& advances_to() const
+	const advances_to_t& advances_to() const
 	{
 		return advances_to_;
 	}
@@ -241,9 +310,8 @@ public:
 	 */
 	std::vector<std::pair<std::string, std::string>> amla_icons() const;
 
-	using advancements_list= boost::ptr_vector<config>;
 	/** The raw, unparsed data for modification advancements. */
-	const advancements_list& modification_advancements() const
+	const std::vector<config>& modification_advancements() const
 	{
 		return advancements_;
 	}
@@ -284,12 +352,9 @@ public:
 	 * The id of this unit's type.
 	 *
 	 * If you are dealing with creating units (e.g. recruitment), this is not what you want, as a
-	 * variation can change this; use type().base_id() instead.
+	 * variation can change this; use type().parent_id() instead.
 	 */
-	const std::string& type_id() const
-	{
-		return type_->id();
-	}
+	const std::string& type_id() const;
 
 	/** Gets the translatable name of this unit's type. */
 	const t_string& type_name() const
@@ -315,7 +380,7 @@ public:
 	}
 
 	/** This unit's unique internal ID. This should *not* be used for user-facing operations. */
-	size_t underlying_id() const
+	std::size_t underlying_id() const
 	{
 		return underlying_id_.value;
 	}
@@ -378,6 +443,15 @@ public:
 		return description_;
 	}
 
+	/** A detailed description of this unit. */
+	void set_unit_description(const t_string& new_desc)
+	{
+		description_ = new_desc;
+	}
+
+	/** The unit's special notes. */
+	std::vector<t_string> unit_special_notes() const;
+
 	/** The gender of this unit. */
 	unit_race::GENDER gender() const
 	{
@@ -389,14 +463,15 @@ public:
 	 *
 	 * This affects the time of day during which this unit's attacks do the most damage.
 	 */
-	unit_type::ALIGNMENT alignment() const
+	UNIT_ALIGNMENT alignment() const
 	{
 		return alignment_;
 	}
 
 	/** Sets the alignment of this unit. */
-	void set_alignment(unit_type::ALIGNMENT alignment)
+	void set_alignment(UNIT_ALIGNMENT alignment)
 	{
+		set_attr_changed(UA_ALIGNMENT);
 		alignment_ = alignment;
 	}
 
@@ -423,6 +498,12 @@ public:
 		return max_hit_points_;
 	}
 
+	void set_max_hitpoints(int value)
+	{
+		set_attr_changed(UA_MAX_HP);
+		max_hit_points_ = value;
+	}
+
 	/** Sets the current hitpoint amount. */
 	void set_hitpoints(int hp)
 	{
@@ -441,6 +522,12 @@ public:
 		return max_experience_;
 	}
 
+	void set_max_experience(int value)
+	{
+		set_attr_changed(UA_MAX_XP);
+		max_experience_ = value;
+	}
+
 	/** The number of experience points this unit needs to level up, or 0 if current XP > max XP. */
 	unsigned int experience_to_advance() const
 	{
@@ -468,6 +555,7 @@ public:
 	/** Sets the current level of this unit. */
 	void set_level(int level)
 	{
+		set_attr_changed(UA_LEVEL);
 		level_ = level;
 	}
 
@@ -478,6 +566,11 @@ public:
 	}
 
 	/** The ID of the undead variation (ie, dwarf, swimmer) of this unit. */
+	void set_undead_variation(const std::string& value)
+	{
+		set_attr_changed(UA_UNDEAD_VARIATION);
+		undead_variation_ = value;
+	}
 	const std::string& undead_variation() const
 	{
 		return undead_variation_;
@@ -491,6 +584,11 @@ public:
 	 */
 	std::string small_profile() const;
 
+	void set_small_profile(const std::string& value)
+	{
+		set_attr_changed(UA_SMALL_PROFILE);
+		small_profile_ = value;
+	}
 	/**
 	 * An optional profile image displays when this unit is 'speaking' via [message].
 	 *
@@ -499,6 +597,8 @@ public:
 	 */
 	std::string big_profile() const;
 
+	void set_big_profile(const std::string& value);
+
 	/** Whether this unit can recruit other units - ie, are they a leader unit. */
 	bool can_recruit() const
 	{
@@ -545,6 +645,12 @@ public:
 		return filter_recall_;
 	}
 
+	/** Sets the filter constraints upon which units this unit may recall, if able. */
+	void set_recall_filter(const config& filter)
+	{
+		filter_recall_ = filter;
+	}
+
 	/**
 	 * Gets this unit's role.
 	 *
@@ -570,13 +676,13 @@ public:
 	 */
 	std::string usage() const
 	{
-		return unit_detail::get_or_default(usage_);
+		return usage_.value_or("");
 	}
 
 	/** Sets this unit's usage. */
 	void set_usage(const std::string& usage)
 	{
-		usage_.reset(new std::string(usage));
+		usage_ = usage;
 	}
 
 	/**
@@ -598,7 +704,9 @@ public:
 
 	/**
 	 * Gets whether this unit is currently hidden on the map.
-	 * @todo document hiddenness
+	 *
+	 * Hidden units are not drawn on the main map or the minimap. They are
+	 * an implementation detail. For the [hides] ability, see invisible().
 	 */
 	bool get_hidden() const
 	{
@@ -704,7 +812,7 @@ public:
 
 	/**
 	 * Heal the unit
-	 * @amount The number of hitpoints to gain
+	 * @param amount The number of hitpoints to gain
 	 */
 	void heal(int amount);
 
@@ -740,14 +848,14 @@ public:
 	 * Built-in status effects known to the engine
 	 */
 	enum state_t {
-		STATE_SLOWED = 0, ///< The unit is slowed - it moves slower and does less damage
-		STATE_POISONED,   ///< The unit is poisoned - it loses health each turn
-		STATE_PETRIFIED,  ///< The unit is petrified - it cannot move or be attacked
-		STATE_UNCOVERED,  ///< The unit is uncovered - it was hiding but has been spotted
-		STATE_NOT_MOVED,  ///< The unit has not moved @todo Explain better
-		STATE_UNHEALABLE, ///< The unit cannot be healed
-		STATE_GUARDIAN,   ///< The unit is a guardian - it won't move unless a target is sighted
-		STATE_UNKNOWN = -1///< A status effect not known to the engine
+		STATE_SLOWED = 0, /** The unit is slowed - it moves slower and does less damage */
+		STATE_POISONED,   /** The unit is poisoned - it loses health each turn */
+		STATE_PETRIFIED,  /** The unit is petrified - it cannot move or be attacked */
+		STATE_UNCOVERED,  /** The unit is uncovered - it was hiding but has been spotted */
+		STATE_NOT_MOVED,  /** The unit has not moved @todo Explain better */
+		STATE_UNHEALABLE, /** The unit cannot be healed */
+		STATE_GUARDIAN,   /** The unit is a guardian - it won't move unless a target is sighted */
+		STATE_UNKNOWN = -1/** A status effect not known to the engine */
 	};
 
 	/**
@@ -824,6 +932,7 @@ public:
 	template<typename... Args>
 	attack_ptr add_attack(attack_itors::iterator position, Args&&... args)
 	{
+		set_attr_changed(UA_ATTACKS);
 		return *attacks_.emplace(position.base(), new attack_type(std::forward<Args>(args)...));
 	}
 
@@ -844,13 +953,14 @@ public:
 	 *
 	 * @param attack              The attack to consider.
 	 * @param attacker            Whether this unit should be considered the attacker.
-	 * @param loc                 TODO: what does this do?
+	 * @param loc                 The unit's location (to resolve [resistance] abilities)
+	 * @param weapon              The weapon to check for any abilities or weapon specials
 	 *
 	 * @returns                   The expected damage.
 	 */
-	int damage_from(const attack_type& attack, bool attacker, const map_location& loc) const
+	int damage_from(const attack_type& attack, bool attacker, const map_location& loc, const_attack_ptr weapon = nullptr) const
 	{
-		return resistance_against(attack, attacker, loc);
+		return resistance_against(attack, attacker, loc, weapon);
 	}
 
 	/** The maximum number of attacks this unit may perform per turn, usually 1. */
@@ -859,6 +969,12 @@ public:
 		return max_attacks_;
 	}
 
+	void set_max_attacks(int value)
+	{
+		set_attr_changed(UA_MAX_AP);
+		max_attacks_ = value;
+	}
+
 	/**
 	 * Gets the remaining number of attacks this unit can perform this turn.
 	 *
@@ -901,18 +1017,21 @@ public:
 	 * @param damage_name The damage type
 	 * @param attacker True if this unit is on the offensive (to resolve [resistance] abilities)
 	 * @param loc The unit's location (to resolve [resistance] abilities)
+	 * @param weapon The weapon to check for any abilities or weapon specials
+	 * @param opp_weapon The opponent's weapon to check for any abilities or weapon specials
 	 */
-	int resistance_against(const std::string& damage_name, bool attacker, const map_location& loc) const;
+	int resistance_against(const std::string& damage_name, bool attacker, const map_location& loc, const_attack_ptr weapon = nullptr, const_attack_ptr opp_weapon = nullptr) const;
 
 	/**
 	 * The unit's resistance against a given attack
 	 * @param atk The attack
 	 * @param attacker True if this unit is on the offensive (to resolve [resistance] abilities)
 	 * @param loc The unit's location (to resolve [resistance] abilities)
+	 * @param weapon The weapon to check for any abilities or weapon specials
 	 */
-	int resistance_against(const attack_type& atk, bool attacker, const map_location& loc) const
+	int resistance_against(const attack_type& atk, bool attacker, const map_location& loc, const_attack_ptr weapon = nullptr) const
 	{
-		return resistance_against(atk.type(), attacker, loc);
+		return resistance_against(atk.type(), attacker, loc , weapon, atk.shared_from_this());
 	}
 
 	/** Gets resistances without any abilities applied. */
@@ -986,7 +1105,7 @@ public:
 	 * Gets the amount of gold this unit costs a side per turn.
 	 *
 	 * This fetches an actual numeric gold value:
-	 * - If @rec can_recruit is true, no upkeep is paid (0 is returned).
+	 * - If can_recruit is true, no upkeep is paid (0 is returned).
 	 * - If a special upkeep flag is set, the associated gold amount is returned (see @ref upkeep_value_visitor).
 	 * - If a numeric value is already set, it is returned directly.
 	 *
@@ -1004,8 +1123,13 @@ public:
 		static std::string type() { static std::string v = "loyal"; return v; }
 	};
 
+	using upkeep_t = utils::variant<upkeep_full, upkeep_loyal, int>;
+
 	/** Visitor helper class to fetch the appropriate upkeep value. */
-	class upkeep_value_visitor : public boost::static_visitor<int>
+	class upkeep_value_visitor
+#ifdef USING_BOOST_VARIANT
+		: public boost::static_visitor<int>
+#endif
 	{
 	public:
 		explicit upkeep_value_visitor(const unit& unit) : u_(unit) {}
@@ -1032,10 +1156,13 @@ public:
 	};
 
 	/** Visitor helper struct to fetch the upkeep type flag if applicable, or the the value otherwise. */
-	struct upkeep_type_visitor : public boost::static_visitor<std::string>
+	struct upkeep_type_visitor
+#ifdef USING_BOOST_VARIANT
+		: public boost::static_visitor<std::string>
+#endif
 	{
 		template<typename T>
-		utils::enable_if_t<!std::is_same<int, T>::value, std::string>
+		std::enable_if_t<!std::is_same_v<int, T>, std::string>
 		operator()(T&) const
 		{
 			// Any special upkeep type should have an associated @ref type getter in its helper struct.
@@ -1048,7 +1175,43 @@ public:
 		}
 	};
 
-	using upkeep_t = boost::variant<upkeep_full, upkeep_loyal, int>;
+	/** Visitor helper class to parse the upkeep value from a config. */
+	class upkeep_parser_visitor
+#ifdef USING_BOOST_VARIANT
+		: public boost::static_visitor<upkeep_t>
+#endif
+	{
+	public:
+		template<typename N>
+		std::enable_if_t<std::is_arithmetic_v<N>, upkeep_t>
+		operator()(N n) const
+		{
+			if(n == 0) return upkeep_loyal();
+			if(n < 0) throw std::invalid_argument(std::to_string(n));
+			return static_cast<int>(n);
+		}
+
+		template<typename B>
+		std::enable_if_t<std::is_convertible_v<B, bool> && !std::is_arithmetic_v<B>, upkeep_t>
+		operator()(B b) const
+		{
+			throw std::invalid_argument(b.str());
+		}
+
+		upkeep_t operator()(utils::monostate) const
+		{
+			return upkeep_full();
+		}
+
+		upkeep_t operator()(const std::string& s) const
+		{
+			if(s == "loyal" || s == "free")
+				return upkeep_loyal();
+			if(s == "full")
+				return upkeep_full();
+			throw std::invalid_argument(s);
+		}
+	};
 
 	/**
 	 * Gets the raw variant controlling the upkeep value.
@@ -1094,6 +1257,12 @@ public:
 		return max_movement_;
 	}
 
+	void set_total_movement(int value)
+	{
+		set_attr_changed(UA_MAX_MP);
+		max_movement_ = value;
+	}
+
 	/**
 	 * Gets how far a unit can move, considering the `incapacitated` flag.
 	 *
@@ -1169,6 +1338,7 @@ public:
 	/** Sets the raw zone-of-control flag. */
 	void set_emit_zoc(bool val)
 	{
+		set_attr_changed(UA_ZOC);
 		emit_zoc_ = val;
 	}
 
@@ -1188,7 +1358,7 @@ public:
 		loc_ = loc;
 	}
 
-	/** The current directin this unit is facing within its hex. */
+	/** The current direction this unit is facing within its hex. */
 	map_location::DIRECTION facing() const
 	{
 		return facing_;
@@ -1313,7 +1483,7 @@ public:
 	 * @param id The ID of the modification to count
 	 * @return The total number of modifications of that type and ID.
 	 */
-	size_t modification_count(const std::string& type, const std::string& id) const;
+	std::size_t modification_count(const std::string& type, const std::string& id) const;
 
 	/**
 	 * Add a new modification to the unit.
@@ -1374,7 +1544,7 @@ public:
 	/** Get the unit's halo image. */
 	std::string image_halo() const
 	{
-		return unit_detail::get_or_default(halo_);
+		return halo_.value_or("");
 	}
 
 	/** Set the unit's halo image. */
@@ -1383,14 +1553,14 @@ public:
 	/** Get the unit's ellipse image. */
 	std::string image_ellipse() const
 	{
-		return unit_detail::get_or_default(ellipse_);
+		return ellipse_.value_or("");
 	}
 
 	/** Set the unit's ellipse image. */
 	void set_image_ellipse(const std::string& ellipse)
 	{
 		appearance_changed_ = true;
-		ellipse_.reset(new std::string(ellipse));
+		ellipse_ = ellipse;
 	}
 
 	/**
@@ -1427,6 +1597,7 @@ public:
 	 *                            The maximum_hitpoints are considered as base.
 	 */
 	color_t hp_color() const;
+	static color_t hp_color_max();
 
 	/**
 	 * Color for this unit's hitpoints.
@@ -1440,6 +1611,7 @@ public:
 	 * Color for this unit's XP. See also @ref hp_color
 	 */
 	color_t xp_color() const;
+	static color_t xp_color(int xp_to_advance, bool can_advance, bool has_amla);
 
 	/**
 	 * @}
@@ -1457,9 +1629,8 @@ public:
 	 * @param tag_name            The name of the ability to check for.
 	 * @param loc                 The location around which to check for affected units. This may or
 	 *                            may not be the location of this unit.
-	 * @param dc                  A display_context object, used in calculations.
 	 */
-	bool get_ability_bool(const std::string& tag_name, const map_location& loc, const display_context& dc) const;
+	bool get_ability_bool(const std::string& tag_name, const map_location& loc) const;
 
 	/**
 	 * Checks whether this unit currently possesses or is affected by a given ability.
@@ -1470,13 +1641,49 @@ public:
 	 * This overload uses the location of this unit for calculations.
 	 *
 	 * @param tag_name            The name of the ability to check for.
-	 * @param dc                  A display_context object, used in calculations.
 	 */
-	bool get_ability_bool(const std::string& tag_name, const display_context& dc) const
+	bool get_ability_bool(const std::string& tag_name) const
 	{
-		return get_ability_bool(tag_name, loc_, dc);
+		return get_ability_bool(tag_name, loc_);
 	}
 
+	/** Checks whether this unit currently possesses a given ability used like weapon
+	 * @return True if the ability @a tag_name is active.
+	 * @param special the const config to one of abilities @a tag_name checked.
+	 * @param tag_name name of ability type checked.
+	 * @param loc location of the unit checked.
+	 */
+	bool get_self_ability_bool(const config& special, const std::string& tag_name, const map_location& loc) const;
+	/** Checks whether this unit currently possesses a given ability of leadership type
+	 * @return True if the ability @a tag_name is active.
+	 * @param special the const config to one of abilities @a tag_name checked.
+	 * @param tag_name name of ability type checked.
+	 * @param loc location of the unit checked.
+	 * @param weapon the attack used by unit checked in this function.
+	 * @param opp_weapon the attack used by opponent to unit checked.
+	 */
+	bool get_self_ability_bool_weapon(const config& special, const std::string& tag_name, const map_location& loc, const_attack_ptr weapon = nullptr, const_attack_ptr opp_weapon = nullptr) const;
+	/** Checks whether this unit is affected by a given ability  used like weapon
+	 * @return True if the ability @a tag_name is active.
+	 * @param special the const config to one of abilities @a tag_name checked.
+	 * @param tag_name name of ability type checked.
+	 * @param loc location of the unit checked.
+	 * @param from unit adjacent to @a this is checked in case of [affect_adjacent] abilities.
+	 * @param dir direction to research a unit adjacent to @a this.
+	 */
+	bool get_adj_ability_bool(const config& special, const std::string& tag_name, int dir, const map_location& loc, const unit& from) const;
+	/** Checks whether this unit is affected by a given ability of leadership type
+	 * @return True if the ability @a tag_name is active.
+	 * @param special the const config to one of abilities @a tag_name checked.
+	 * @param tag_name name of ability type checked.
+	 * @param loc location of the unit checked.
+	 * @param from unit adjacent to @a this is checked in case of [affect_adjacent] abilities.
+	 * @param dir direction to research a unit adjacent to @a this.
+	 * @param weapon the attack used by unit checked in this function.
+	 * @param opp_weapon the attack used by opponent to unit checked.
+	 */
+	bool get_adj_ability_bool_weapon(const config& special, const std::string& tag_name, int dir, const map_location& loc, const unit& from, const_attack_ptr weapon=nullptr, const_attack_ptr opp_weapon = nullptr) const;
+
 	/**
 	 * Gets the unit's active abilities of a particular type if it were on a specified location.
 	 * @param tag_name The type of ability to check for
@@ -1495,6 +1702,17 @@ public:
 		return get_abilities(tag_name, loc_);
 	}
 
+	unit_ability_list get_abilities_weapons(const std::string& tag_name, const map_location& loc, const_attack_ptr weapon = nullptr, const_attack_ptr opp_weapon = nullptr) const;
+
+	unit_ability_list get_abilities_weapons(const std::string& tag_name, const_attack_ptr weapon = nullptr, const_attack_ptr opp_weapon = nullptr) const
+	{
+		return get_abilities_weapons(tag_name, loc_, weapon, opp_weapon);
+	}
+
+	const config &abilities() const { return abilities_; }
+
+	const std::set<std::string>& checking_tags() const { return checking_tags_; };
+
 	/**
 	 * Gets the names and descriptions of this unit's abilities. Location-independent variant
 	 * with all abilities shown as active.
@@ -1542,7 +1760,10 @@ public:
 	 */
 	void remove_ability_by_id(const std::string& ability);
 
+
 private:
+
+	const std::set<std::string> checking_tags_{"damage", "chance_to_hit", "berserk", "swarm", "drains", "heal_on_hit", "plague", "slow", "petrifies", "firststrike", "poison"};
 	/**
 	 * Check if an ability is active.
 	 * @param ability The type (tag name) of the ability
@@ -1558,6 +1779,7 @@ private:
 	 * @param cfg an ability WML structure
 	 * @param loc The location on which to resolve the ability
 	 * @param from The "other unit" for filter matching
+	 * @param dir The direction the unit is facing
 	 */
 	bool ability_affects_adjacent(const std::string& ability, const config& cfg, int dir, const map_location& loc, const unit& from) const;
 
@@ -1569,6 +1791,12 @@ private:
 	 */
 	bool ability_affects_self(const std::string& ability, const config& cfg, const map_location& loc) const;
 
+	/**
+	 * filters the weapons that condition the use of abilities for combat ([resistance],[leadership] or abilities used like specials
+	 * (deprecated in two last cases)
+	 */
+	bool ability_affects_weapon(const config& cfg, const_attack_ptr weapon, bool is_opp) const;
+
 public:
 	/** Get the unit formula manager. */
 	unit_formula_manager& formula_manager() const
@@ -1580,18 +1808,19 @@ public:
 	void generate_name();
 
 	// Only see_all = true use caching
-	bool invisible(const map_location& loc, const display_context& dc, bool see_all = true) const;
+	bool invisible(const map_location& loc, bool see_all = true) const;
 
-	bool is_visible_to_team(const team& team, const display_context& dc, bool const see_all = true) const;
-	/// Return true if the unit would be visible to team if its location were loc.
-	bool is_visible_to_team(const map_location& loc, const team& team, const display_context& dc, bool const see_all = true) const;
+	bool is_visible_to_team(const team& team, bool const see_all = true) const;
+	/** Return true if the unit would be visible to team if its location were loc. */
+	bool is_visible_to_team(const map_location& loc, const team& team, bool const see_all = true) const;
 
 	/**
 	 * Serializes the current unit metadata values.
 	 *
 	 * @param cfg                 The config to write to.
+	 * @param write_all           set this to false to not write unchanged attributes.
 	 */
-	void write(config& cfg) const;
+	void write(config& cfg, bool write_all = true) const;
 
 	/**
 	 * Mark this unit as clone so it can be inserted to unit_map.
@@ -1600,21 +1829,11 @@ public:
 	 */
 	unit& mark_clone(bool is_temporary);
 
-	/** @} */
-
-	long ref_count() const
-	{
-		return ref_count_;
-	}
-
-	friend void intrusive_ptr_add_ref(const unit*);
-	friend void intrusive_ptr_release(const unit*);
 
 	void set_appearance_changed(bool value) { appearance_changed_ = value; }
 	bool appearance_changed() const { return appearance_changed_; }
 
 protected:
-	mutable long ref_count_; // used by intrusive_ptr
 
 private:
 	map_location loc_;
@@ -1647,7 +1866,7 @@ private:
 	int recall_cost_;
 	bool canrecruit_;
 	std::vector<std::string> recruit_list_;
-	unit_type::ALIGNMENT alignment_;
+	UNIT_ALIGNMENT alignment_;
 
 	std::string flag_rgb_;
 	std::string image_mods_;
@@ -1676,7 +1895,7 @@ private:
 
 	std::set<std::string> states_;
 
-	static const size_t num_bool_states = 7;
+	static const std::size_t num_bool_states = 7;
 
 	std::bitset<num_bool_states> known_boolean_states_;
 	static std::map<std::string, state_t> known_boolean_state_names_;
@@ -1719,13 +1938,14 @@ private:
 	config modifications_;
 	config abilities_;
 
-	advancements_list advancements_;
+	std::vector<config> advancements_;
 
 	t_string description_;
+	std::vector<t_string> special_notes_;
 
-	std::unique_ptr<std::string> usage_;
-	std::unique_ptr<std::string> halo_;
-	std::unique_ptr<std::string> ellipse_;
+	std::optional<std::string> usage_;
+	std::optional<std::string> halo_;
+	std::optional<std::string> ellipse_;
 
 	bool random_traits_;
 	bool generate_name_;
@@ -1737,6 +1957,7 @@ private:
 
 	//Used to check whether the moving units during a move needs to be updated
 	mutable bool appearance_changed_ = true;
+	std::bitset<UA_COUNT> changed_attributes_;
 
 	void parse_upkeep(const config::attribute_value& upkeep);
 	void write_upkeep(config::attribute_value& upkeep) const;
