@@ -1,38 +1,36 @@
 /*
-   Copyright (C) 2003 - 2018 the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
-#include "global.hpp"
-
 #include <algorithm>
 #include <cctype>
+#include <functional>
 
+namespace utils
+{
 inline bool chars_equal_insensitive(char a, char b) { return tolower(a) == tolower(b); }
 inline bool chars_less_insensitive(char a, char b) { return tolower(a) < tolower(b); }
 
-namespace utils {
-
-#ifdef HAVE_CXX17
-using std::clamp;
-#else
-// NOTE: remove once we have C++17 support and can use std::clamp
-template<typename T>
-CONSTEXPR const T& clamp(const T& value, const T& min, const T& max)
-{
-	return std::max<T>(std::min<T>(value, max), min);
-}
-#endif
+/**
+ * Equivalent to as @c std::is_same_v except it uses the decayed form of V.
+ *
+ * @tparam T1    The first type to compare.
+ * @tparam T2    The second type to compare. This will be passed through @c std::decay .
+ */
+template<typename T1, typename T2>
+inline constexpr bool decayed_is_same = std::is_same_v<T1, std::decay_t<T2>>;
 
 namespace detail
 {
