@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -22,35 +23,35 @@
 #include "serialization/unicode_cast.hpp"
 #include "random.hpp"
 
-static void add_prefixes(const ucs4::string& str, size_t length, markov_prefix_map& res)
+static void add_prefixes(const std::u32string& str, std::size_t length, markov_prefix_map& res)
 {
-	for(size_t i = 0; i <= str.size(); ++i) {
-		const size_t start = i > length ? i - length : 0;
-		const ucs4::string key(str.begin() + start, str.begin() + i);
-		const ucs4::char_t c = i != str.size() ? str[i] : 0;
+	for(std::size_t i = 0; i <= str.size(); ++i) {
+		const std::size_t start = i > length ? i - length : 0;
+		const std::u32string key(str.begin() + start, str.begin() + i);
+		const char32_t c = i != str.size() ? str[i] : 0;
 		res[key].push_back(c);
 	}
 }
 
-static markov_prefix_map markov_prefixes(const std::vector<std::string>& items, size_t length)
+static markov_prefix_map markov_prefixes(const std::vector<std::string>& items, std::size_t length)
 {
 	markov_prefix_map res;
 
 	for(std::vector<std::string>::const_iterator i = items.begin(); i != items.end(); ++i) {
-		add_prefixes(unicode_cast<ucs4::string>(*i),length,res);
+		add_prefixes(unicode_cast<std::u32string>(*i),length,res);
 	}
 
 	return res;
 }
 
-static ucs4::string markov_generate_name(const markov_prefix_map& prefixes,
-	size_t chain_size, size_t max_len)
+static std::u32string markov_generate_name(const markov_prefix_map& prefixes,
+	std::size_t chain_size, std::size_t max_len)
 {
 	if(prefixes.empty() || chain_size == 0) {
-		return ucs4::string();
+		return std::u32string();
 	}
 
-	ucs4::string prefix, res;
+	std::u32string prefix, res;
 
 	// Since this function is called in the name description in a MP game it
 	// uses the local locale. The locale between players can be different and
@@ -63,7 +64,7 @@ static ucs4::string markov_generate_name(const markov_prefix_map& prefixes,
 	// traits to be different. To avoid that problem we call get_random()
 	// the maximum number of times and store the result in a lookup table.
 	std::vector<int> random(max_len);
-	size_t j = 0;
+	std::size_t j = 0;
 	for(; j < max_len; ++j) {
 		random[j] = randomness::generator->next_random();
 	}
@@ -75,7 +76,7 @@ static ucs4::string markov_generate_name(const markov_prefix_map& prefixes,
 			return res;
 		}
 
-		const ucs4::char_t c = i->second[random[j++]%i->second.size()];
+		const char32_t c = i->second[random[j++]%i->second.size()];
 		if(c == 0) {
 			return res;
 		}
@@ -98,16 +99,16 @@ static ucs4::string markov_generate_name(const markov_prefix_map& prefixes,
 	// name has end-of-string as a possible next character in the
 	// markov prefix map. If no valid ending is found, use the
 	// originally generated name.
-	ucs4::string originalRes = res;
+	std::u32string originalRes = res;
 	while(!res.empty()) {
 		const int prefixLen = chain_size < res.size() ? chain_size : res.size();
-		prefix = ucs4::string(res.end() - prefixLen, res.end());
+		prefix = std::u32string(res.end() - prefixLen, res.end());
 
 		const markov_prefix_map::const_iterator i = prefixes.find(prefix);
 		if (i == prefixes.end() || i->second.empty()) {
 			return res;
 		}
-		if (std::find(i->second.begin(), i->second.end(), static_cast<ucs4::char_t>(0))
+		if (std::find(i->second.begin(), i->second.end(), static_cast<char32_t>(0))
 				!= i->second.end()) {
 			// This ending is valid.
 			return res;
@@ -123,7 +124,7 @@ static ucs4::string markov_generate_name(const markov_prefix_map& prefixes,
 	return originalRes;
 }
 
-markov_generator::markov_generator(const std::vector<std::string>& items, size_t chain_size, size_t max_len)
+markov_generator::markov_generator(const std::vector<std::string>& items, std::size_t chain_size, std::size_t max_len)
 	: prefixes_(markov_prefixes(items, chain_size))
 	, chain_size_(chain_size)
 	, max_len_(max_len)
@@ -132,6 +133,6 @@ markov_generator::markov_generator(const std::vector<std::string>& items, size_t
 
 std::string markov_generator::generate() const
 {
-	ucs4::string name = markov_generate_name(prefixes_, chain_size_, max_len_);
-	return unicode_cast<utf8::string>(name);
+	std::u32string name = markov_generate_name(prefixes_, chain_size_, max_len_);
+	return unicode_cast<std::string>(name);
 }
