@@ -1,21 +1,20 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
-#include "global.hpp"
-
 #include <string>
 #include <map>
 #include <exception>
@@ -23,7 +22,7 @@
 class name_generator_invalid_exception : public std::exception {
 public:
 	name_generator_invalid_exception(const char* errMessage):errMessage_(errMessage) {}
-	const char* what() const NOEXCEPT { return errMessage_; }
+	const char* what() const noexcept { return errMessage_; }
 
 private:
 	const char* errMessage_;
