@@ -1,23 +1,24 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2018 by Philippe Plantier <ayin@anathas.org>
+	Copyright (C) 2005 - 2021
+	by Philippe Plantier <ayin@anathas.org>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
-
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
 
 #include "config.hpp"
 #include "map/location.hpp"
+#include "variable_info.hpp"
 
 #include <utility>
 
@@ -28,6 +29,7 @@ class config_variable_set : public variable_set {
 public:
 	config_variable_set(const config& cfg) : cfg_(cfg) {}
 	virtual config::attribute_value get_variable_const(const std::string &id) const;
+	virtual variable_access_const get_variable_access_read(const std::string& varname) const;
 };
 
 /**
@@ -43,41 +45,39 @@ class vconfig
 {
 private:
 
-/*
- * Starting with the rc of gcc 4.6 the code failed to compile due to the
- * missing default constructor for vconfig. Not entirely sure whether it's a
- * bug in gcc or not. For now make the code conditional.
- */
-#if __GNUC__ == 4 && __GNUC_MINOR__ == 6 && defined(__GXX_EXPERIMENTAL_CXX0X__)
-	template<class T1, class T2>
-	friend class std::pair;
-#endif
-
 	vconfig();
 	vconfig(const config & cfg, const std::shared_ptr<const config> & cache);
+	vconfig(const config& cfg, const std::shared_ptr<const config> & cache, const variable_set& variables);
 public:
-	/// Constructor from a config.
-	/// Equivalent to vconfig(cfg, false).
-	/// Do not use if the vconfig will persist after @a cfg is destroyed!
-	explicit vconfig(const config &cfg) : cache_(), cfg_(&cfg) {}
-	explicit vconfig(config &&cfg) : cache_(new config(std::move(cfg))), cfg_(cache_.get()) { }
-	vconfig(const config &cfg, bool manage_memory);
+	/**
+	 * Constructor from a config.
+	 * Equivalent to vconfig(cfg, false).
+	 * Do not use if the vconfig will persist after @a cfg is destroyed!
+	 */
+	explicit vconfig(const config &cfg);
+	explicit vconfig(config &&cfg);
+	// Construct a vconfig referencing a non-default set of variables.
+	// Note that the vconfig does NOT take ownership of these variables,
+	// so you need to make sure that their scope encloses the vconfig's scope!
+	vconfig(const config& cfg, const variable_set& variables);
+	vconfig(const config &cfg, bool manage_memory, const variable_set* variables = nullptr);
 	~vconfig();
 
 	static vconfig empty_vconfig(); // Valid to dereference. Contains nothing
 	static vconfig unconstructed_vconfig(); // Must not be dereferenced
 
-	/// A vconfig evaluates to true iff it can be dereferenced.
+	/** A vconfig evaluates to true iff it can be dereferenced. */
 	explicit operator bool() const	{ return !null(); }
 
 	bool null() const { assert(cfg_); return cfg_ == &default_empty_config; }
-	const vconfig& make_safe() const; //!< instruct the vconfig to make a private copy of its underlying data.
+	/** instruct the vconfig to make a private copy of its underlying data. */
+	const vconfig& make_safe() const;
 	const config& get_config() const { return *cfg_; }
 	config get_parsed_config() const;
 
 	typedef std::vector<vconfig> child_list;
 	child_list get_children(const std::string& key) const;
-	size_t count_children(const std::string& key) const;
+	std::size_t count_children(const std::string& key) const;
 	vconfig child(const std::string& key) const;
 	bool has_child(const std::string& key) const;
 
@@ -109,13 +109,13 @@ public:
 		typedef const pointer_proxy pointer;
 		typedef const config::attribute reference;
 		typedef config::const_attribute_iterator Itor;
-		explicit attribute_iterator(const Itor &i): i_(i) {}
+		explicit attribute_iterator(const Itor &i, const variable_set& vars): i_(i), variables_(&vars) {}
 
 		attribute_iterator &operator++() { ++i_; return *this; }
-		attribute_iterator operator++(int) { return attribute_iterator(i_++); }
+		attribute_iterator operator++(int) { return attribute_iterator(i_++, *variables_); }
 
 		attribute_iterator &operator--() { --i_; return *this; }
-		attribute_iterator operator--(int) { return attribute_iterator(i_--); }
+		attribute_iterator operator--(int) { return attribute_iterator(i_--, *variables_); }
 
 		reference operator*() const;
 		pointer operator->() const;
@@ -125,11 +125,12 @@ public:
 
 	private:
 		Itor i_;
+		const variable_set* variables_;
 	};
 
 	boost::iterator_range<attribute_iterator> attribute_range() {
 		config::const_attr_itors range = cfg_->attribute_range();
-		return boost::make_iterator_range(attribute_iterator(range.begin()), attribute_iterator(range.end()));
+		return boost::make_iterator_range(attribute_iterator(range.begin(), *variables_), attribute_iterator(range.end(), *variables_));
 	}
 
 	struct all_children_iterator
@@ -142,8 +143,8 @@ public:
 		typedef const pointer_proxy pointer;
 		typedef const value_type reference;
 		typedef config::const_all_children_iterator Itor;
-		explicit all_children_iterator(const Itor &i);
-		all_children_iterator(const Itor &i, const std::shared_ptr<const config> & cache);
+		explicit all_children_iterator(const Itor &i, const variable_set& vars);
+		all_children_iterator(const Itor &i, const variable_set& vars, const std::shared_ptr<const config> & cache);
 
 		all_children_iterator& operator++();
 		all_children_iterator  operator++(int);
@@ -176,6 +177,7 @@ public:
 		*/
 		int inner_index_;
 		std::shared_ptr<const config> cache_;
+		const variable_set* variables_;
 	};
 
 	struct recursion_error : public config::error {
@@ -190,14 +192,17 @@ public:
 	}
 
 private:
-	/// Returns true if *this has made a copy of its config.
+	/** Returns true if *this has made a copy of its config. */
 	bool memory_managed() const { return static_cast<bool>(cache_); }
 
-	/// Keeps a copy of our config alive when we manage our own memory.
-	/// If this is not null, then cfg_ points to *cache_ or a child of *cache_.
+	/**
+	 * Keeps a copy of our config alive when we manage our own memory.
+	 * If this is not null, then cfg_ points to *cache_ or a child of *cache_.
+	 */
 	mutable std::shared_ptr<const config> cache_;
-	/// Used to access our config (original or copy, as appropriate).
+	/** Used to access our config (original or copy, as appropriate). */
 	mutable const config* cfg_;
+	const variable_set* variables_;
 	static const config default_empty_config;
 };
 
