@@ -1,17 +1,17 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2018 by Philippe Plantier <ayin@anathas.org>
+	Copyright (C) 2005 - 2021
+	by Philippe Plantier <ayin@anathas.org>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
-
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -41,11 +41,6 @@ public:
 	static const config& get_child_at(const config& cfg, const std::string& key, int index)
 	{
 		assert(index >= 0);
-
-		if(!config::valid_tag(key)) {
-			throw invalid_variablename_exception();
-		}
-
 		// cfg.child_or_empty does not support index parameter
 		if(const config& child = cfg.child(key, index)) {
 			return child;
@@ -68,11 +63,6 @@ public:
 	static config& get_child_at(config& cfg, const std::string& key, int index)
 	{
 		assert(index >= 0);
-
-		if (!config::valid_tag(key)) {
-			throw invalid_variablename_exception();
-		}
-
 		// the 'create_if_not_existent' logic.
 		while(static_cast<int>(cfg.child_count(key)) <= index) {
 			cfg.add_child(key);
