@@ -1,22 +1,23 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #include "video.hpp"
 
 #include "display.hpp"
 #include "floating_label.hpp"
-#include "font/sdl_ttf.hpp"
+#include "font/sdl_ttf_compat.hpp"
 #include "picture.hpp"
 #include "log.hpp"
 #include "preferences/general.hpp"
@@ -25,6 +26,11 @@
 #include "sdl/utils.hpp"
 #include "sdl/window.hpp"
 
+#ifdef TARGET_OS_OSX
+#include "desktop/apple_video.hpp"
+#include "game_version.hpp"
+#endif
+
 #include <cassert>
 #include <vector>
 
@@ -32,14 +38,14 @@ static lg::log_domain log_display("display");
 #define LOG_DP LOG_STREAM(info, log_display)
 #define ERR_DP LOG_STREAM(err, log_display)
 
-#define MAGIC_DPI_SCALE_NUMBER 96
-
 CVideo* CVideo::singleton_ = nullptr;
 
 namespace
 {
 surface frameBuffer = nullptr;
 bool fake_interactive = false;
+
+const unsigned MAGIC_DPI_SCALE_NUMBER = 96;
 }
 
 namespace video2
@@ -119,12 +125,6 @@ void CVideo::initSDL()
 
 CVideo::~CVideo()
 {
-	if(sdl_get_version() >= version_info(2, 0, 6)) {
-		// Because SDL will free the framebuffer,
-		// ensure that we won't attempt to free it.
-		frameBuffer.clear_without_free();
-	}
-
 	LOG_DP << "calling SDL_Quit()\n";
 	SDL_Quit();
 	assert(singleton_);
@@ -132,31 +132,6 @@ CVideo::~CVideo()
 	LOG_DP << "called SDL_Quit()\n";
 }
 
-std::string CVideo::video_settings_report()
-{
-	if (singleton_ == nullptr) return "No video initialized.\n";
-	if (singleton_->non_interactive())
-		return "Initialized but non-interactive.\n";
-	sdl::window* win = singleton_->get_window();
-	if (!win) return "Interactive but no SDL window.\n";
-	std::ostringstream o;
-	o << "Current pixel resolution: "
-	  << singleton_->get_width() << "x" << singleton_->get_height()
-	  << '\n'
-	  << "Refresh rate: " << singleton_->current_refresh_rate()
-	  << '\n';
-	float hdpi, vdpi;
-	int returncode = SDL_GetDisplayDPI(win->get_display_index(),
-					   nullptr, &hdpi, &vdpi);
-	if (returncode != 0) {
-		o << "SDL not supplying dots per inch.\n";
-	} else {
-		o << "SDL reports: " << hdpi << "x" << vdpi
-		  << " dots per inch.\n";
-	}
-	return o.str();
-}
-
 bool CVideo::non_interactive() const
 {
 	return fake_interactive ? false : (window == nullptr);
@@ -199,20 +174,12 @@ void CVideo::make_fake()
 	fake_screen_ = true;
 	refresh_rate_ = 1;
 
-#if SDL_VERSION_ATLEAST(2, 0, 6)
 	frameBuffer = SDL_CreateRGBSurfaceWithFormat(0, 16, 16, 24, SDL_PIXELFORMAT_BGR888);
-#else
-	frameBuffer = SDL_CreateRGBSurface(0, 16, 16, 24, 0xFF0000, 0xFF00, 0xFF, 0);
-#endif
 }
 
 void CVideo::make_test_fake(const unsigned width, const unsigned height)
 {
-#if SDL_VERSION_ATLEAST(2, 0, 6)
 	frameBuffer = SDL_CreateRGBSurfaceWithFormat(0, width, height, 32, SDL_PIXELFORMAT_BGR888);
-#else
-	frameBuffer = SDL_CreateRGBSurface(0, width, height, 32, 0xFF0000, 0xFF00, 0xFF, 0);
-#endif
 
 	fake_interactive = true;
 	refresh_rate_ = 1;
@@ -225,13 +192,6 @@ void CVideo::update_framebuffer()
 	}
 
 	surface fb = SDL_GetWindowSurface(*window);
-
-	if(frameBuffer && sdl_get_version() >= version_info(2, 0, 6)) {
-		// Because SDL has already freed the old framebuffer,
-		// ensure that we won't attempt to free it.
-		frameBuffer.clear_without_free();
-	}
-
 	frameBuffer = fb;
 }
 
@@ -250,7 +210,9 @@ void CVideo::init_window()
 
 	// Add any more default flags here
 	window_flags |= SDL_WINDOW_RESIZABLE;
+#ifdef __APPLE__
 	window_flags |= SDL_WINDOW_ALLOW_HIGHDPI;
+#endif
 
 	if(preferences::fullscreen()) {
 		window_flags |= SDL_WINDOW_FULLSCREEN_DESKTOP;
@@ -317,8 +279,7 @@ SDL_Rect CVideo::screen_area(bool as_pixels) const
 
 	// Then convert the dimensions into screen coordinates, if applicable.
 	if(!as_pixels) {
-		float scale_x, scale_y;
-		std::tie(scale_x, scale_y) = get_dpi_scale_factor();
+		auto [scale_x, scale_y] = get_dpi_scale_factor();
 
 		size.x /= scale_x;
 		size.y /= scale_y;
@@ -395,6 +356,25 @@ sdl::window* CVideo::get_window()
 	return window.get();
 }
 
+std::string CVideo::current_driver()
+{
+	const char* const drvname = SDL_GetCurrentVideoDriver();
+	return drvname ? drvname : "<not initialized>";
+}
+
+std::vector<std::string> CVideo::enumerate_drivers()
+{
+	std::vector<std::string> res;
+	int num_drivers = SDL_GetNumVideoDrivers();
+
+	for(int n = 0; n < num_drivers; ++n) {
+		const char* drvname = SDL_GetVideoDriver(n);
+		res.emplace_back(drvname ? drvname : "<invalid driver>");
+	}
+
+	return res;
+}
+
 bool CVideo::window_has_flags(uint32_t flags) const
 {
 	if(!window) {
@@ -404,21 +384,43 @@ bool CVideo::window_has_flags(uint32_t flags) const
 	return (window->get_flags() & flags) != 0;
 }
 
+std::pair<float, float> CVideo::get_dpi() const
+{
+	float hdpi, vdpi;
+	if(window && SDL_GetDisplayDPI(window->get_display_index(), nullptr, &hdpi, &vdpi) == 0) {
+#ifdef TARGET_OS_OSX
+		// SDL 2.0.12 changes SDL_GetDisplayDPI. Function now returns DPI
+		// multiplied by screen's scale factor. This part of code reverts
+		// this multiplication.
+		//
+		// For more info see issue: https://github.com/wesnoth/wesnoth/issues/5019
+		SDL_version sdl_version;
+		SDL_GetVersion(&sdl_version);
+
+		const version_info sdl_version_info(sdl_version.major, sdl_version.minor, sdl_version.patch);
+		const version_info version_to_compare(2, 0, 12);
+
+		if (sdl_version_info >= version_to_compare) {
+			float scale_factor = desktop::apple::get_scale_factor(window->get_display_index());
+			hdpi /= scale_factor;
+			vdpi /= scale_factor;
+		}
+#endif
+		return { hdpi, vdpi };
+	}
+	// SDL doesn't know the screen dpi, there's a configuration issue, or we
+	// don't have a window yet.
+	return { 0.0f, 0.0f };
+}
+
 std::pair<float, float> CVideo::get_dpi_scale_factor() const
 {
-	std::pair<float, float> result{1.0f, 1.0f};
-
-	if(!window) {
-		return result;
+	auto dpi = get_dpi();
+	if(dpi.first != 0.0f && dpi.second != 0.0f) {
+		return { dpi.first / MAGIC_DPI_SCALE_NUMBER, dpi.second / MAGIC_DPI_SCALE_NUMBER };
 	}
-
-	float hdpi, vdpi;
-	SDL_GetDisplayDPI(window->get_display_index(), nullptr, &hdpi, &vdpi);
-
-	result.first = hdpi / MAGIC_DPI_SCALE_NUMBER;
-	result.second = vdpi / MAGIC_DPI_SCALE_NUMBER;
-
-	return result;
+	// Assume a scale factor of 1.0 if the screen dpi is currently unknown.
+	return { 1.0f, 1.0f };
 }
 
 std::vector<point> CVideo::get_available_resolutions(const bool include_current)
@@ -441,8 +443,7 @@ std::vector<point> CVideo::get_available_resolutions(const bool include_current)
 
 #if 0
 	// DPI scale factor.
-	float scale_h, scale_v;
-	std::tie(scale_h, scale_v) = get_dpi_scale_factor();
+	auto [scale_h, scale_v] = get_dpi_scale_factor();
 #endif
 
 	// The maximum size to which this window can be set. For some reason this won't
@@ -503,7 +504,7 @@ int CVideo::set_help_string(const std::string& str)
 	int size = font::SIZE_LARGE;
 
 	while(size > 0) {
-		if(font::line_width(str, size) > get_width()) {
+		if(font::pango_line_width(str, size) > get_width()) {
 			size--;
 		} else {
 			break;
