@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -40,13 +41,16 @@ public:
 
 	~CVideo();
 
+	static bool setup_completed()
+	{
+		return singleton_ != nullptr;
+	}
+
 	static CVideo& get_singleton()
 	{
 		return *singleton_;
 	}
 
-	static std::string video_settings_report();
-
 	/***** ***** ***** ***** Unit test-related functions ***** ***** ****** *****/
 
 	void make_fake();
@@ -74,6 +78,15 @@ public:
 	/** Returns a pointer to the underlying SDL window. */
 	sdl::window* get_window();
 
+	bool has_window()
+	{
+		return get_window() != nullptr;
+	}
+
+	static std::string current_driver();
+
+	static std::vector<std::string> enumerate_drivers();
+
 private:
 	enum MODE_EVENT { TO_RES, TO_FULLSCREEN, TO_WINDOWED, TO_MAXIMIZED_WINDOW };
 
@@ -122,6 +135,9 @@ public:
 	/** Returns the window renderer height in pixels or in screen coordinates. */
 	int get_height(bool as_pixels = true) const;
 
+	/** The current game screen dpi. */
+	std::pair<float, float> get_dpi() const;
+
 	/** The current scale factor on High-DPI screens. */
 	std::pair<float, float> get_dpi_scale_factor() const;
 
