@@ -1,16 +1,17 @@
 /*
- Copyright (C) 2010 - 2018 by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
- Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2010 - 2021
+	by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
- See the COPYING file for more details.
- */
+	See the COPYING file for more details.
+*/
 
 /**
  * @file
@@ -92,7 +93,7 @@ void action::show()
 	do_show();
 }
 
-action::action(size_t team_index, bool hidden)
+action::action(std::size_t team_index, bool hidden)
 	: team_index_(team_index)
 	, hidden_(hidden)
 {
@@ -114,7 +115,7 @@ action::~action()
 {
 }
 
-size_t action::get_unit_id() const
+std::size_t action::get_unit_id() const
 {
 	unit_ptr ret = get_unit();
 	return ret ? ret->underlying_id() : 0;
