@@ -1,16 +1,17 @@
 /*
- Copyright (C) 2010 - 2018 by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
- Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2010 - 2021
+	by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
- See the COPYING file for more details.
- */
+	See the COPYING file for more details.
+*/
 
 /**
  * @file
@@ -32,7 +33,7 @@ class visitor;
 class action : public std::enable_shared_from_this<action>
 {
 public:
-	action(size_t team_index, bool hidden);
+	action(std::size_t team_index, bool hidden);
 	action(const config&, bool hidden); // For deserialization
 	virtual ~action();
 
@@ -75,12 +76,12 @@ public:
 	 * Returns the id of the unit targeted by this action.
 	 * @retval 0 no unit is targeted.
 	 */
-	virtual size_t get_unit_id() const;
+	virtual std::size_t get_unit_id() const;
 
 	/** @return pointer to the fake unit used only for visuals */
 	virtual fake_unit_ptr get_fake_unit() = 0;
 	/** Returns the index of the team that owns this action */
-	size_t team_index() const { return team_index_; }
+	std::size_t team_index() const { return team_index_; }
 	/** Returns the number of the side that owns this action, i.e. the team index + 1. */
 	int side_number() const
 	{
@@ -138,7 +139,7 @@ private:
 	virtual void do_hide() {}
 	virtual void do_show() {}
 
-	size_t team_index_;
+	std::size_t team_index_;
 	bool hidden_;
 };
 
