@@ -1,16 +1,17 @@
 /*
- Copyright (C) 2010 - 2018 by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
- Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2010 - 2021
+	by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
- See the COPYING file for more details.
- */
+	See the COPYING file for more details.
+*/
 
 /**
  * @file
@@ -51,7 +52,7 @@
 #include "units/animation_component.hpp"
 #include "units/udisplay.hpp"
 
-#include "utils/functional.hpp"
+#include <functional>
 
 #include <sstream>
 
@@ -99,7 +100,7 @@ manager::~manager()
 #if 0
 static void print_to_chat(const std::string& title, const std::string& message)
 {
-	display::get_singleton()->add_chat_message(time(nullptr), title, 0, message,
+	display::get_singleton()->add_chat_message(std::time(nullptr), title, 0, message,
 			events::chat_handler::MESSAGE_PRIVATE, false);
 }
 #endif
@@ -163,7 +164,7 @@ bool manager::can_modify_game_state() const
 bool manager::can_activate() const
 {
 	//any more than one reference means a lock on whiteboard state was requested
-	if(!activation_state_lock_.unique())
+	if(activation_state_lock_.use_count() != 1)
 		return false;
 
 	return can_modify_game_state();
@@ -202,7 +203,7 @@ void manager::set_active(bool active)
 void manager::set_invert_behavior(bool invert)
 {
 	//any more than one reference means a lock on whiteboard state was requested
-	if(!activation_state_lock_.unique())
+	if(activation_state_lock_.use_count() != 1)
 		return;
 
 	bool block_whiteboard_activation = false;
@@ -368,7 +369,7 @@ static void hide_all_plans()
 }
 
 /* private */
-void manager::update_plan_hiding(size_t team_index)
+void manager::update_plan_hiding(std::size_t team_index)
 {
 	//We don't control the "viewing" side ... we're probably an observer
 	if(!resources::gameboard->teams().at(team_index).is_local_human())
@@ -392,7 +393,7 @@ void manager::update_plan_hiding(size_t team_index)
 void manager::update_plan_hiding()
 	{update_plan_hiding(viewer_team());}
 
-void manager::on_viewer_change(size_t team_index)
+void manager::on_viewer_change(std::size_t team_index)
 {
 	if(!wait_for_side_init_)
 		update_plan_hiding(team_index);
@@ -417,8 +418,8 @@ void manager::on_change_controller(int side, const team& t)
 			hide_all_plans(); // give up knowledge of everyone's plans, in case we became an observer
 
 		//tell them our plans -- they may not have received them up to this point
-		size_t num_teams = resources::gameboard->teams().size();
-		for(size_t i=0; i<num_teams; ++i)
+		std::size_t num_teams = resources::gameboard->teams().size();
+		for(std::size_t i=0; i<num_teams; ++i)
 		{
 			team& local_team = resources::gameboard->teams().at(i);
 			if(local_team.is_local_human() && !local_team.is_enemy(side))
@@ -459,9 +460,9 @@ void manager::validate_viewer_actions()
 static void draw_numbers(const map_location& hex, side_actions::numbers_t numbers)
 {
 	std::vector<int>& numbers_to_draw = numbers.numbers_to_draw;
-	std::vector<size_t>& team_numbers = numbers.team_numbers;
+	std::vector<std::size_t>& team_numbers = numbers.team_numbers;
 	int& main_number = numbers.main_number;
-	std::set<size_t>& secondary_numbers = numbers.secondary_numbers;
+	std::set<std::size_t>& secondary_numbers = numbers.secondary_numbers;
 
 	const double x_offset_base = 0.0;
 	const double y_offset_base = 0.2;
@@ -472,22 +473,16 @@ static void draw_numbers(const map_location& hex, side_actions::numbers_t number
 	const double y_origin = 0.5 - numbers_to_draw.size() * (y_offset_base / 2);
 	double x_offset = 0, y_offset = 0;
 
-	size_t size = numbers_to_draw.size();
-	for(size_t i=0; i<size; ++i)
+	std::size_t size = numbers_to_draw.size();
+	for(std::size_t i=0; i<size; ++i)
 	{
 		int number = numbers_to_draw[i];
 
 		std::string number_text = std::to_string(number);
-		size_t font_size;
-		if (static_cast<int>(i) == main_number) {
-			font_size = 19;
-		}
-		else if (secondary_numbers.find(i)!=secondary_numbers.end()) {
-			font_size = 17;
-		}
-		else {
-			font_size = 15;
-		}
+		std::size_t font_size;
+		if (static_cast<int>(i) == main_number) font_size = 19;
+		else if (secondary_numbers.find(i)!=secondary_numbers.end()) font_size = 17;
+		else font_size = 15;
 
 		color_t color = team::get_side_color(static_cast<int>(team_numbers[i]+1));
 		const double x_in_hex = x_origin + x_offset;
@@ -515,12 +510,12 @@ namespace
 			action->accept(*this);
 		}
 
-		const std::set<size_t>& get_units_owning_moves() {
+		const std::set<std::size_t>& get_units_owning_moves() {
 			return move_owners_;
 		}
 
 		virtual void visit(move_ptr move) {
-			if(size_t id = move->get_unit_id()) {
+			if(std::size_t id = move->get_unit_id()) {
 				move_owners_.insert(id);
 			}
 		}
@@ -528,7 +523,7 @@ namespace
 		virtual void visit(attack_ptr attack) {
 			//also add attacks if they have an associated move
 			if(attack->get_route().steps.size() >= 2) {
-				if(size_t id = attack->get_unit_id()) {
+				if(std::size_t id = attack->get_unit_id()) {
 					move_owners_.insert(id);
 				}
 			}
@@ -538,18 +533,18 @@ namespace
 		virtual void visit(suppose_dead_ptr){}
 
 	private:
-		std::set<size_t> move_owners_;
+		std::set<std::size_t> move_owners_;
 	};
 }
 
 void manager::pre_draw()
 {
-	if (can_modify_game_state() && has_actions() && unit_map_lock_.unique()) {
+	if (can_modify_game_state() && has_actions() && unit_map_lock_.use_count() == 1) {
 		move_owners_finder move_finder;
 		for_each_action(std::ref(move_finder));
 		units_owning_moves_ = move_finder.get_units_owning_moves();
 
-		for (size_t unit_id : units_owning_moves_) {
+		for (std::size_t unit_id : units_owning_moves_) {
 			unit_map::iterator unit_iter = resources::gameboard->units().find(unit_id);
 			if(unit_iter.valid()) {
 				ghost_owner_unit(&*unit_iter);
@@ -560,7 +555,7 @@ void manager::pre_draw()
 
 void manager::post_draw()
 {
-	for (size_t unit_id : units_owning_moves_)
+	for (std::size_t unit_id : units_owning_moves_)
 	{
 		unit_map::iterator unit_iter = resources::gameboard->units().find(unit_id);
 		if (unit_iter.valid()) {
@@ -630,8 +625,8 @@ void manager::on_gamestate_change()
 
 void manager::send_network_data()
 {
-	size_t size = net_buffer_.size();
-	for(size_t team_index=0; team_index<size; ++team_index)
+	std::size_t size = net_buffer_.size();
+	for(std::size_t team_index=0; team_index<size; ++team_index)
 	{
 		config& buf_cfg = net_buffer_[team_index];
 
@@ -647,7 +642,7 @@ void manager::send_network_data()
 
 		resources::controller->send_to_wesnothd(packet, "whiteboard");
 
-		size_t count = wb_cfg.child_count("net_cmd");
+		std::size_t count = wb_cfg.child_count("net_cmd");
 		LOG_WB << "Side " << (team_index+1) << " sent wb data (" << count << " cmds).\n";
 	}
 }
@@ -656,7 +651,7 @@ void manager::process_network_data(const config& cfg)
 {
 	if(const config& wb_cfg = cfg.child("whiteboard"))
 	{
-		size_t count = wb_cfg.child_count("net_cmd");
+		std::size_t count = wb_cfg.child_count("net_cmd");
 		LOG_WB << "Received wb data (" << count << ").\n";
 
 		team& team_from = resources::gameboard->get_team(wb_cfg["side"]);
@@ -665,7 +660,7 @@ void manager::process_network_data(const config& cfg)
 	}
 }
 
-void manager::queue_net_cmd(size_t team_index, const side_actions::net_cmd& cmd)
+void manager::queue_net_cmd(std::size_t team_index, const side_actions::net_cmd& cmd)
 {
 	assert(team_index < net_buffer_.size());
 	net_buffer_[team_index].add_child("net_cmd",cmd);
@@ -708,7 +703,7 @@ void manager::create_temp_move()
 	route_.reset(new pathfind::marked_route(route));
 	//NOTE: route_->steps.back() = dst, and route_->steps.front() = src
 
-	size_t turn = 0;
+	std::size_t turn = 0;
 	std::vector<map_location>::iterator prev_itor = route.steps.begin();
 	std::vector<map_location>::iterator curr_itor = prev_itor;
 	std::vector<map_location>::iterator end_itor  = route.steps.end();
@@ -793,19 +788,19 @@ void manager::save_temp_move()
 		side_actions& sa = *viewer_actions();
 		unit* u = future_visible_unit(route_->steps.front());
 		assert(u);
-		size_t first_turn = sa.get_turn_num_of(*u);
+		std::size_t first_turn = sa.get_turn_num_of(*u);
 
 		validate_viewer_actions();
 
 		assert(move_arrows_.size() == fake_units_.size());
-		size_t size = move_arrows_.size();
-		for(size_t i=0; i<size; ++i)
+		std::size_t size = move_arrows_.size();
+		for(std::size_t i=0; i<size; ++i)
 		{
 			arrow_ptr move_arrow = move_arrows_[i];
 			if(!arrow::valid_path(move_arrow->get_path()))
 				continue;
 
-			size_t turn = first_turn + i;
+			std::size_t turn = first_turn + i;
 
 			//@todo Using a marked_route here is wrong, since right now it's not marked
 			//either switch over to a plain route for planned moves, or mark it correctly
@@ -897,7 +892,7 @@ bool manager::save_recruit(const std::string& name, int side_num, const map_loca
 				recruiter = find_recruiter(side_num-1,recruit_hex);
 			} // end planned unit map scope
 			assert(recruiter);
-			size_t turn = sa.get_turn_num_of(*recruiter);
+			std::size_t turn = sa.get_turn_num_of(*recruiter);
 			sa.queue_recruit(turn,name,recruit_hex);
 			created_planned_recruit = true;
 
@@ -921,7 +916,7 @@ bool manager::save_recall(const unit& unit, int side_num, const map_location& re
 		else
 		{
 			side_actions& sa = *viewer_actions();
-			size_t turn = sa.num_turns();
+			std::size_t turn = sa.num_turns();
 			if(turn > 0)
 				--turn;
 			sa.queue_recall(turn,unit,recall_hex);
@@ -1045,7 +1040,7 @@ void manager::contextual_delete()
 		unit const* selected_unit = future_visible_unit(resources::controller->get_mouse_handler_base().get_selected_hex(), viewer_side());
 		if(selected_unit && (it = viewer_actions()->find_last_action_of(*selected_unit)) != viewer_actions()->end()) {
 			viewer_actions()->remove_action(it);
-			///@todo Shouldn't we probably deselect the unit at this point?
+			// TODO: Shouldn't we probably deselect the unit at this point?
 		} else if(highlighter_ && (action = highlighter_->get_delete_target()) && (it = viewer_actions()->get_position_of(action)) != viewer_actions()->end()) {
 			viewer_actions()->remove_action(it);
 			validate_viewer_actions();
@@ -1133,7 +1128,7 @@ void manager::options_dlg()
 		allies.push_back(&t);
 
 		t_vars["player"] = t.current_player();
-		size_t t_index = t.side()-1;
+		std::size_t t_index = t.side()-1;
 		if(team_plans_hidden_[t_index])
 			options.emplace_back(VGETTEXT("Show plans for $player", t_vars));
 		else
@@ -1162,7 +1157,7 @@ void manager::options_dlg()
 	default:
 		if(selection > 1)
 		{
-			size_t t_index = allies[selection-2]->side()-1;
+			std::size_t t_index = allies[selection-2]->side()-1;
 			//toggle ...
 			bool hidden = team_plans_hidden_[t_index];
 			team_plans_hidden_[t_index] = !hidden;
@@ -1179,7 +1174,7 @@ void manager::set_planned_unit_map()
 		return;
 	}
 	//any more than one reference means a lock on unit map was requested
-	if(!unit_map_lock_.unique()) {
+	if(unit_map_lock_.use_count() != 1) {
 		LOG_WB << "Not building planned unit map: unit map locked.\n";
 		return;
 	}
@@ -1188,7 +1183,7 @@ void manager::set_planned_unit_map()
 		return;
 	}
 
-	log_scope2("whiteboard", "Building planned unit map");
+	log_scope2(log_whiteboard, "Building planned unit map");
 	mapbuilder_.reset(new mapbuilder(resources::gameboard->units()));
 	mapbuilder_->build_map();
 
@@ -1203,7 +1198,7 @@ void manager::set_real_unit_map()
 		assert(!wait_for_side_init_);
 		if(mapbuilder_)
 		{
-			log_scope2("whiteboard", "Restoring regular unit map.");
+			log_scope2(log_whiteboard, "Restoring regular unit map.");
 			mapbuilder_.reset();
 		}
 		planned_unit_map_active_ = false;
