@@ -1,16 +1,17 @@
 /*
- Copyright (C) 2010 - 2018 by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
- Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2010 - 2021
+	by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
- See the COPYING file for more details.
- */
+	See the COPYING file for more details.
+*/
 
 /**
  * @file
@@ -90,7 +91,7 @@ public:
 	void on_mouseover_change(const map_location& hex);
 	void on_deselect_hex(){ erase_temp_move();}
 	void on_gamestate_change();
-	void on_viewer_change(size_t team_index);
+	void on_viewer_change(std::size_t team_index);
 	void on_change_controller(int side, const team& t);
 	void on_kill_unit();
 	/** Handles various cleanup right before removing an action from the queue */
@@ -103,7 +104,7 @@ public:
 	/** Called by turn_info::process_network_data() when network data needs to be processed */
 	void process_network_data(const config&);
 	/** Adds a side_actions::net_cmd to net_buffer_[team_index], whereupon it will (later) be sent to all allies */
-	void queue_net_cmd(size_t team_index, const side_actions::net_cmd&);
+	void queue_net_cmd(std::size_t team_index, const side_actions::net_cmd&);
 
 	/** Whether the current side has actions in the first turn of its planned actions queue */
 	static bool current_side_has_actions();
@@ -196,7 +197,7 @@ private:
 
 	void validate_actions_if_needed();
 	/** Called by all of the save_***() methods after they have added their action to the queue */
-	void update_plan_hiding(size_t viewing_team);
+	void update_plan_hiding(std::size_t viewing_team);
 	void update_plan_hiding(); //same as above, but uses wb::viewer_team() as default argument
 
 	/** Tracks whether the whiteboard is active. */
@@ -230,20 +231,20 @@ private:
 
 	std::vector<arrow_ptr> move_arrows_;
 	std::vector<fake_unit_ptr> fake_units_;
-	size_t temp_move_unit_underlying_id_;
+	std::size_t temp_move_unit_underlying_id_;
 
 	const std::unique_ptr<CKey> key_poller_;
 
 	std::vector<map_location> hidden_unit_hexes_;
 
-	///net_buffer_[i] = whiteboard network data to be sent "from" teams[i].
+	/** net_buffer_[i] = whiteboard network data to be sent "from" teams[i]. */
 	std::vector<config> net_buffer_;
 
-	///team_plans_hidden_[i] = whether or not to hide actions from teams[i].
+	/** team_plans_hidden_[i] = whether or not to hide actions from teams[i]. */
 	boost::dynamic_bitset<> team_plans_hidden_;
 
-	///used to keep track of units owning planned moves for visual ghosting/unghosting
-	std::set<size_t> units_owning_moves_;
+	/** used to keep track of units owning planned moves for visual ghosting/unghosting */
+	std::set<std::size_t> units_owning_moves_;
 };
 
 /** Applies the planned unit map for the duration of the struct's life.
