@@ -1,16 +1,17 @@
 /*
- Copyright (C) 2010 - 2018 by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
- Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2010 - 2021
+	by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
- See the COPYING file for more details.
- */
+	See the COPYING file for more details.
+*/
 
 /**
  * @file
@@ -28,8 +29,7 @@
 #include "resources.hpp"
 #include "units/unit.hpp"
 #include "units/map.hpp"
-
-#include <boost/range/adaptor/reversed.hpp>
+#include "utils/ranges.hpp"
 
 namespace wb
 {
@@ -68,8 +68,8 @@ void mapbuilder::pre_build()
 		//Remove any unit the current side cannot see to avoid their detection by planning
 		//Units will be restored to the unit map by destruction of removers_
 
-		if(!on_current_side && !u.is_visible_to_team(resources::gameboard->teams()[viewer_team()], *resources::gameboard, false)) {
-			removers_.push_back(new temporary_unit_remover(resources::gameboard->units(), u.get_location()));
+		if(!on_current_side && !u.is_visible_to_team(resources::gameboard->teams()[viewer_team()], false)) {
+			removers_.emplace_back(new temporary_unit_remover(resources::gameboard->units(), u.get_location()));
 
 			//Don't do anything else to the removed unit!
 			continue;
@@ -78,7 +78,7 @@ void mapbuilder::pre_build()
 		//Reset movement points, to be restored by destruction of resetters_
 
 		//restore movement points only to units not on the current side
-		resetters_.push_back(new unit_movement_resetter(u,!on_current_side));
+		resetters_.emplace_back(new unit_movement_resetter(u,!on_current_side));
 		//make sure current side's units are not reset to full moves on first turn
 		if(on_current_side) {
 			acted_this_turn_.insert(&u);
@@ -94,7 +94,7 @@ void mapbuilder::build_map()
 	}
 
 	bool stop = false;
-	for(size_t turn=0; !stop; ++turn) {
+	for(std::size_t turn=0; !stop; ++turn) {
 		stop = true;
 		for (team &side : resources::gameboard->teams()) {
 			side_actions &actions = *side.get_side_actions();
@@ -174,13 +174,13 @@ void mapbuilder::process(side_actions &sa, side_actions::iterator action_it, boo
 	applied_actions_this_turn_.push_back(action);
 }
 
-void mapbuilder::post_visit_team(size_t turn)
+void mapbuilder::post_visit_team(std::size_t turn)
 {
 	std::set<unit const*> seen;
 
 	// Go backwards through the actions of this turn to identify
 	// which ones are moves that end a turn.
-	for(action_ptr action : boost::adaptors::reverse(applied_actions_this_turn_)) {
+	for(action_ptr action : utils::reversed_view(applied_actions_this_turn_)) {
 		move_ptr move = std::dynamic_pointer_cast<class move>(action);
 		if(move) {
 			move->set_turn_number(0);
@@ -200,7 +200,7 @@ void mapbuilder::post_visit_team(size_t turn)
 void mapbuilder::restore_normal_map()
 {
 	//applied_actions_ contain only the actions that we applied to the unit map
-	for(action_ptr act : boost::adaptors::reverse(applied_actions_)) {
+	for(action_ptr act : utils::reversed_view(applied_actions_)) {
 		act->remove_temp_modifier(unit_map_);
 	}
 }
