@@ -1,16 +1,17 @@
 /*
- Copyright (C) 2011 - 2018 by Tommy Schmitz
- Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2011 - 2021
+	by Tommy Schmitz
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
- See the COPYING file for more details.
- */
+	See the COPYING file for more details.
+*/
 
 /**
  * @file
@@ -29,7 +30,7 @@ namespace wb {
 class suppose_dead: public action
 {
 public:
-	suppose_dead(size_t team_index, bool hidden, unit& curr_unit, const map_location& loc);
+	suppose_dead(std::size_t team_index, bool hidden, unit& curr_unit, const map_location& loc);
 	suppose_dead(const config&, bool hidden); // For deserialization
 	virtual ~suppose_dead();
 
@@ -74,7 +75,7 @@ protected:
 		return std::static_pointer_cast<suppose_dead>(action::shared_from_this());
 	}
 
-	size_t unit_underlying_id_;
+	std::size_t unit_underlying_id_;
 	std::string unit_id_;
 	map_location loc_;
 
