@@ -1,16 +1,17 @@
 /*
- Copyright (C) 2010 - 2018 by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
- Part of the Battle for Wesnoth Project https://www.wesnoth.org
+	Copyright (C) 2010 - 2021
+	by Gabriel Morin <gabrielmorin (at) gmail (dot) com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
- See the COPYING file for more details.
- */
+	See the COPYING file for more details.
+*/
 
 /**
  * @file
@@ -37,7 +38,7 @@
 
 namespace wb {
 
-size_t viewer_team()
+std::size_t viewer_team()
 {
 	return display::get_singleton()->viewing_team();
 }
@@ -76,7 +77,7 @@ unit_const_ptr find_backup_leader(const unit & leader)
 	return unit_const_ptr();
 }
 
-unit* find_recruiter(size_t team_index, const map_location& hex)
+unit* find_recruiter(std::size_t team_index, const map_location& hex)
 {
 	if ( !resources::gameboard->map().is_castle(hex) )
 		return nullptr;
@@ -139,7 +140,7 @@ int path_cost(const std::vector<map_location>& path, const unit& u)
 
 	int result = 0;
 	const gamemap& map = resources::gameboard->map();
-	for(const map_location& loc : std::make_pair(path.begin()+1,path.end())) {
+	for(const map_location& loc : std::pair(path.begin()+1,path.end())) {
 		result += u.movement_cost(map[loc]);
 	}
 	return result;
@@ -185,7 +186,7 @@ bool team_has_visible_plan(team &t)
 void for_each_action(std::function<void(action*)> function, team_filter team_filter)
 {
 	bool end = false;
-	for(size_t turn=0; !end; ++turn) {
+	for(std::size_t turn=0; !end; ++turn) {
 		end = true;
 		for(team &side : resources::gameboard->teams()) {
 			side_actions &actions = *side.get_side_actions();
@@ -202,7 +203,7 @@ void for_each_action(std::function<void(action*)> function, team_filter team_fil
 action_ptr find_action_at(map_location hex, team_filter team_filter)
 {
 	action_ptr result;
-	size_t result_turn = std::numeric_limits<size_t>::max();
+	std::size_t result_turn = std::numeric_limits<std::size_t>::max();
 
 	for(team &side : resources::gameboard->teams()) {
 		side_actions &actions = *side.get_side_actions();
@@ -212,7 +213,7 @@ action_ptr find_action_at(map_location hex, team_filter team_filter)
 				continue;
 			}
 
-			size_t chall_turn = actions.get_turn(chall);
+			std::size_t chall_turn = actions.get_turn(chall);
 			if(chall_turn < result_turn) {
 				result = *chall;
 				result_turn = chall_turn;
