@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -39,18 +40,18 @@ public:
 		virtual void init() {}
 
 		virtual SDL_Rect item_size(const std::string& item) const;
-		virtual void draw_row_bg(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
-		virtual void draw_row(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+		virtual void draw_row_bg(menu& menu_ref, const std::size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+		virtual void draw_row(menu& menu_ref, const std::size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
 		void scale_images(int max_width, int max_height);
 		surface get_item_image(const image::locator &i_locator) const;
-		size_t get_font_size() const;
-		size_t get_cell_padding() const;
-		size_t get_thickness() const;
+		std::size_t get_font_size() const;
+		std::size_t get_cell_padding() const;
+		std::size_t get_thickness() const;
 
 	protected:
-		size_t font_size_;
-		size_t cell_padding_;
-		size_t thickness_;  //additional cell padding for style use only
+		std::size_t font_size_;
+		std::size_t cell_padding_;
+		std::size_t thickness_;  //additional cell padding for style use only
 
 		int normal_rgb_, selected_rgb_, heading_rgb_;
 		double normal_alpha_, selected_alpha_, heading_alpha_;
@@ -67,8 +68,8 @@ public:
 		virtual ~imgsel_style();
 
 		virtual SDL_Rect item_size(const std::string& item) const;
-		virtual void draw_row_bg(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
-		virtual void draw_row(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+		virtual void draw_row_bg(menu& menu_ref, const std::size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+		virtual void draw_row(menu& menu_ref, const std::size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
 
 		virtual void init() { load_images(); }
 		bool load_images();
@@ -99,7 +100,6 @@ public:
 	friend class style;
 	friend class imgsel_style;
 	static style &default_style;
-	static style simple_style;
 	static imgsel_style bluebg_style;
 
 	struct item
@@ -107,13 +107,13 @@ public:
 		item() : fields(), help(), id(0)
 		{}
 
-		item(const std::vector<std::string>& fields, size_t id)
+		item(const std::vector<std::string>& fields, std::size_t id)
 			: fields(fields), help(), id(id)
 		{}
 
 		std::vector<std::string> fields;
 		std::vector<std::string> help;
-		size_t id;
+		std::size_t id;
 	};
 
 	class sorter
@@ -132,8 +132,6 @@ public:
 
 		basic_sorter& set_alpha_sort(int column);
 		basic_sorter& set_numeric_sort(int column);
-		basic_sorter& set_xp_sort(int column);
-		basic_sorter& set_level_sort(int level_column, int xp_column);
 		basic_sorter& set_id_sort(int column);
 		basic_sorter& set_redirect_sort(int column, int to);
 		basic_sorter& set_position_sort(int column, const std::vector<int>& pos);
@@ -142,10 +140,9 @@ public:
 		virtual bool less(int column, const item& row1, const item& row2) const;
 
 	private:
-		std::set<int> alpha_sort_, numeric_sort_, id_sort_, xp_sort_, level_sort_;
+		std::set<int> alpha_sort_, numeric_sort_, id_sort_;
 		std::map<int,int> redirect_sort_;
 		std::map<int,std::vector<int>> pos_sort_;
-		int xp_col_; //used by level sort
 	};
 
 	menu(CVideo& video, const std::vector<std::string>& items,
@@ -157,34 +154,38 @@ public:
 
 	int selection() const;
 
-	void move_selection(size_t id);
-	void move_selection_keeping_viewport(size_t id);
+	void move_selection(std::size_t id);
+	void move_selection_keeping_viewport(std::size_t id);
 	void reset_selection();
 
 	// allows user to change_item while running (dangerous)
 	void change_item(int pos1,int pos2,const std::string& str);
 
-	virtual void erase_item(size_t index);
+	virtual void erase_item(std::size_t index);
 
 	void set_heading(const std::vector<std::string>& heading);
 
-	/// Set new items to show and redraw/recalculate everything. If
-	/// strip_spaces is false, spaces will remain at the item edges. If
-	/// keep_viewport is true, the menu tries to keep the selection at
-	/// the same position as it were before the items were set.
+	/**
+	 * Set new items to show and redraw/recalculate everything. If
+	 * strip_spaces is false, spaces will remain at the item edges. If
+	 * keep_viewport is true, the menu tries to keep the selection at
+	 * the same position as it were before the items were set.
+	 */
 	virtual void set_items(const std::vector<std::string>& items, bool strip_spaces=true,
 				   bool keep_viewport=false);
 
-	/// Set a new max height for this menu. Note that this does not take
-	/// effect immediately, only after certain operations that clear
-	/// everything, such as set_items().
+	/**
+	 * Set a new max height for this menu. Note that this does not take
+	 * effect immediately, only after certain operations that clear
+	 * everything, such as set_items().
+	 */
 	void set_max_height(const int new_max_height);
 	void set_max_width(const int new_max_width);
 
 	int get_max_height() const { return max_height_; }
 	int get_max_width() const { return max_width_; }
 
-	size_t number_of_items() const { return items_.size(); }
+	std::size_t number_of_items() const { return items_.size(); }
 
 	int process();
 
@@ -209,7 +210,7 @@ protected:
 
 	bool requires_event_focus(const SDL_Event *event=nullptr) const;
 	const std::vector<int>& column_widths() const;
-	virtual void draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+	virtual void draw_row(const std::size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
 
 	style *style_;
 	bool silent_;
@@ -221,14 +222,14 @@ protected:
 
 	int hit_heading(int x, int y) const;
 
-	void invalidate_row(size_t id);
-	void invalidate_row_pos(size_t pos);
+	void invalidate_row(std::size_t id);
+	void invalidate_row_pos(std::size_t pos);
 	void invalidate_heading();
 
 private:
-	size_t max_items_onscreen() const;
+	std::size_t max_items_onscreen() const;
 
-	size_t heading_height() const;
+	std::size_t heading_height() const;
 
 	int max_height_, max_width_;
 	mutable int max_items_, item_height_;
@@ -237,7 +238,7 @@ private:
 	void key_press(SDL_Keycode key);
 
 	std::vector<item> items_;
-	std::vector<size_t> item_pos_;
+	std::vector<std::size_t> item_pos_;
 
 	std::vector<std::string> heading_;
 	mutable int heading_height_;
@@ -250,11 +251,11 @@ private:
 
 	mutable std::vector<int> column_widths_;
 
-	size_t selected_;
+	std::size_t selected_;
 	bool click_selects_;
 	bool out_;
 	bool previous_button_;
-	//std::set<size_t> undrawn_items_;
+	//std::set<std::size_t> undrawn_items_;
 
 	bool show_result_;
 
@@ -269,9 +270,9 @@ private:
 	mutable std::map<int,SDL_Rect> itemRects_;
 
 	SDL_Rect get_item_rect(int item) const;
-	SDL_Rect get_item_rect_internal(size_t pos) const;
-	size_t get_item_height_internal(const std::vector<std::string>& item) const;
-	size_t get_item_height(int item) const;
+	SDL_Rect get_item_rect_internal(std::size_t pos) const;
+	std::size_t get_item_height_internal(const std::vector<std::string>& item) const;
+	std::size_t get_item_height(int item) const;
 	int items_start() const;
 
 	int items_end() const;
@@ -279,8 +280,10 @@ private:
 
 	void update_scrollbar_grip_height();
 
-	///variable which determines whether a numeric keypress should
-	///select an item on the dialog
+	/**
+	 * variable which determines whether a numeric keypress should
+	 * select an item on the dialog
+	 */
 	bool num_selects_;
 	// These two variables are used to get the correct double click
 	// behavior so that a click that causes one double click won't be
@@ -296,8 +299,10 @@ private:
 	bool sortreversed_;
 	int highlight_heading_;
 
-	/// Set new items to show. If strip_spaces is false, spaces will
-	/// remain at the item edges.
+	/**
+	 * Set new items to show. If strip_spaces is false, spaces will
+	 * remain at the item edges.
+	 */
 	void fill_items(const std::vector<std::string>& items, bool strip_spaces);
 
 	void do_sort();
@@ -306,10 +311,10 @@ private:
 
 	void update_size();
 	enum SELECTION_MOVE_VIEWPORT { MOVE_VIEWPORT, NO_MOVE_VIEWPORT };
-	void set_selection_pos(size_t pos, bool silent=false, SELECTION_MOVE_VIEWPORT move_viewport=MOVE_VIEWPORT);
-	void move_selection_to(size_t id, bool silent=false, SELECTION_MOVE_VIEWPORT move_viewport=MOVE_VIEWPORT);
-	void move_selection_up(size_t dep);
-	void move_selection_down(size_t dep);
+	void set_selection_pos(std::size_t pos, bool silent=false, SELECTION_MOVE_VIEWPORT move_viewport=MOVE_VIEWPORT);
+	void move_selection_to(std::size_t id, bool silent=false, SELECTION_MOVE_VIEWPORT move_viewport=MOVE_VIEWPORT);
+	void move_selection_up(std::size_t dep);
+	void move_selection_down(std::size_t dep);
 
 	std::set<int> invalid_;
 };
