@@ -1,16 +1,17 @@
 /*
-   wesnoth menu styles Copyright (C) 2006 - 2018 by Patrick Parker <patrick_x99@hotmail.com>
-   wesnoth menu Copyright (C) 2003-5 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2006 - 2021
+	by Patrick Parker <patrick_x99@hotmail.com>
+	Copyright (C) 2003 - 2005 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -29,7 +30,6 @@ namespace gui {
 menu::imgsel_style menu::bluebg_style("dialogs/selection", true,
 										   0x000000, 0x000000, 0x333333,
 										   0.35, 0.0, 0.3);
-menu::style menu::simple_style;
 
 menu::style &menu::default_style = menu::bluebg_style;
 
@@ -53,9 +53,9 @@ menu::imgsel_style::imgsel_style(const std::string &img_base, bool has_bg,
 menu::imgsel_style::~imgsel_style()
 {}
 
-size_t menu::style::get_font_size() const { return font_size_; }
-size_t menu::style::get_cell_padding() const { return cell_padding_; }
-size_t menu::style::get_thickness() const { return thickness_; }
+std::size_t menu::style::get_font_size() const { return font_size_; }
+std::size_t menu::style::get_cell_padding() const { return cell_padding_; }
+std::size_t menu::style::get_thickness() const { return thickness_; }
 
 void menu::style::scale_images(int max_width, int max_height)
 {
@@ -137,7 +137,7 @@ bool menu::imgsel_style::load_images()
 	return (!load_failed_);
 }
 
-void menu::imgsel_style::draw_row_bg(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
+void menu::imgsel_style::draw_row_bg(menu& menu_ref, const std::size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 {
 	if(type == SELECTED_ROW && has_background_ && !load_failed_) {
 		if(bg_cache_.width != rect.w || bg_cache_.height != rect.h)
@@ -156,7 +156,7 @@ void menu::imgsel_style::draw_row_bg(menu& menu_ref, const size_t row_index, con
 	}
 }
 
-void menu::imgsel_style::draw_row(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
+void menu::imgsel_style::draw_row(menu& menu_ref, const std::size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 {
 	if(!load_failed_) {
 		//draw item inside
@@ -234,7 +234,7 @@ SDL_Rect menu::imgsel_style::item_size(const std::string& item) const
 	SDL_Rect bounds = style::item_size(item);
 
 	bounds.w += 2 * thickness_;
-	bounds.h += 2 * thickness_;
+	bounds.h += 2 * thickness_ + 4;
 
 	return bounds;
 }
