@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2004 - 2018 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2004 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "widgets/scrollarea.hpp"
@@ -33,7 +32,7 @@ scrollarea::scrollarea(CVideo &video, const bool auto_join)
 
 bool scrollarea::has_scrollbar() const
 {
-	return shown_size_ < full_size_ && scrollbar_.is_valid_height(location().h);
+	return shown_size_ < full_size_;
 }
 
 sdl_handler_vector scrollarea::handler_members()
