@@ -1,20 +1,19 @@
 /*
-   Copyright (C) 2003 by David White <dave@whitevine.net>
-                 2004 - 2015 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2004 - 2021
+	by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+	Copyright (C) 2003 by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
-/** @file */
-
 #pragma once
 
 #include "button.hpp"
@@ -36,14 +35,11 @@ public:
 	//- @param callback  a callback interface for warning that the grip has been moved
 	scrollbar(CVideo &video);
 
-	virtual void hide(bool value = true);
-
 	/**
 	 * Determine where the scrollbar is.
 	 *
 	 * @return  the position.
-	 * @retval  returns 0 if the scrollbar is at the top,
-	 * @retval  returns (full_size - shown_size) if it is at the bottom.
+	 * @retval  returns 0 if the scrollbar is at the top or (full_size - shown_size) if it is at the bottom.
 	 */
 	unsigned get_position() const;
 
@@ -67,9 +63,6 @@ public:
 	/** Set scroll rate. */
 	void set_scroll_rate(unsigned r);
 
-	/** Return true if the scrollbar has a valid size. */
-	bool is_valid_height(int height) const;
-
 	/** Scrolls down one step */
 	void scroll_down();
 
@@ -77,19 +70,13 @@ public:
 	void scroll_up();
 
 protected:
-	virtual sdl_handler_vector handler_members();
-	virtual void update_location(const SDL_Rect& rect);
 	virtual void handle_event(const SDL_Event& event);
-	virtual void process_event();
 	virtual void draw_contents();
 
 private:
 	SDL_Rect grip_area() const;
-	SDL_Rect groove_area() const;
 	surface mid_scaled_, groove_scaled_;
 
-	button uparrow_, downarrow_;
-
 	enum STATE { UNINIT, NORMAL, ACTIVE, DRAGGED };
 	STATE state_;
 
