@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -18,7 +19,7 @@
 
 #include "cursor.hpp"
 #include "desktop/clipboard.hpp"
-#include "font/sdl_ttf.hpp"
+#include "font/sdl_ttf_compat.hpp"
 #include "log.hpp"
 #include "sdl/rect.hpp"
 #include "serialization/string_utils.hpp"
@@ -30,8 +31,8 @@ static lg::log_domain log_display("display");
 
 namespace gui {
 
-textbox::textbox(CVideo &video, int width, const std::string& text, bool editable, size_t max_size, int font_size, double alpha, double alpha_focus, const bool auto_join)
-	   : scrollarea(video, auto_join), max_size_(max_size), font_size_(font_size), text_(unicode_cast<ucs4::string>(text)),
+textbox::textbox(CVideo &video, int width, const std::string& text, bool editable, std::size_t max_size, int font_size, double alpha, double alpha_focus, const bool auto_join)
+	   : scrollarea(video, auto_join), max_size_(max_size), font_size_(font_size), text_(unicode_cast<std::u32string>(text)),
 	     cursor_(text_.size()), selstart_(-1), selend_(-1),
 	     grabmouse_(false), text_pos_(0), editable_(editable),
 	     show_cursor_(true), show_cursor_at_(0), text_image_(nullptr),
@@ -41,7 +42,7 @@ textbox::textbox(CVideo &video, int width, const std::string& text, bool editabl
 		,listening_(false)
 {
 	// static const SDL_Rect area = video.screen_area();
-	// const int height = font::draw_text(nullptr,area,font_size,font::NORMAL_COLOR,"ABCD",0,0).h;
+	// const int height = font::pango_draw_text(nullptr,area,font_size,font::NORMAL_COLOR,"ABCD",0,0).h;
 	set_measurements(width, font::get_max_height(font_size_));
 	set_scroll_rate(font::get_max_height(font_size_) / 2);
 	update_text_cache(true);
@@ -72,14 +73,14 @@ void textbox::set_inner_location(const SDL_Rect& rect)
 
 const std::string textbox::text() const
 {
-	const std::string &ret = unicode_cast<utf8::string>(text_);
+	const std::string &ret = unicode_cast<std::string>(text_);
 	return ret;
 }
 
 // set_text does not respect max_size_
 void textbox::set_text(const std::string& text, const color_t& color)
 {
-	text_ = unicode_cast<ucs4::string>(text);
+	text_ = unicode_cast<std::u32string>(text);
 	cursor_ = text_.size();
 	text_pos_ = 0;
 	selstart_ = -1;
@@ -101,10 +102,10 @@ void textbox::append_text(const std::string& text, bool auto_scroll, const color
 		return;
 	}
 	const bool is_at_bottom = get_position() == get_max_position();
-	const ucs4::string& wtext = unicode_cast<ucs4::string>(text);
+	const std::u32string& wtext = unicode_cast<std::u32string>(text);
 
 	surface new_text = add_text_line(wtext, color);
-	surface new_surface(std::max<size_t>(text_image_->w,new_text->w),text_image_->h+new_text->h);
+	surface new_surface(std::max<std::size_t>(text_image_->w,new_text->w),text_image_->h+new_text->h);
 
 	adjust_surface_alpha(new_text, SDL_ALPHA_TRANSPARENT);
 	adjust_surface_alpha(text_image_, SDL_ALPHA_TRANSPARENT);
@@ -148,8 +149,8 @@ void textbox::set_selection(const int selstart, const int selend)
 	if (!editable_) {
 		return;
 	}
-	if (selstart < 0 || selend < 0 || size_t(selstart) > text_.size() ||
-		size_t(selend) > text_.size()) {
+	if (selstart < 0 || selend < 0 || std::size_t(selstart) > text_.size() ||
+		std::size_t(selend) > text_.size()) {
 		WRN_DP << "out-of-boundary selection" << std::endl;
 		return;
 	}
@@ -163,7 +164,7 @@ void textbox::set_cursor_pos(const int cursor_pos)
 	if (!editable_) {
 		return;
 	}
-	if (cursor_pos < 0 || size_t(cursor_pos) > text_.size()) {
+	if (cursor_pos < 0 || std::size_t(cursor_pos) > text_.size()) {
 		WRN_DP << "out-of-boundary selection" << std::endl;
 		return;
 	}
@@ -208,8 +209,8 @@ void textbox::draw_contents()
 
 	if(text_image_ != nullptr) {
 		src.y = yscroll_;
-		src.w = std::min<size_t>(loc.w,text_image_->w);
-		src.h = std::min<size_t>(loc.h,text_image_->h);
+		src.w = std::min<std::size_t>(loc.w,text_image_->w);
+		src.h = std::min<std::size_t>(loc.h,text_image_->h);
 		src.x = text_pos_;
 		SDL_Rect dest = video().screen_area();
 		dest.x = loc.x;
@@ -225,8 +226,8 @@ void textbox::draw_contents()
 			const int endy = char_y_[end];
 
 			while(starty <= endy) {
-				const size_t right = starty == endy ? endx : text_image_->w;
-				if(right <= size_t(startx)) {
+				const std::size_t right = starty == endy ? endx : text_image_->w;
+				if(right <= std::size_t(startx)) {
 					break;
 				}
 
@@ -300,7 +301,7 @@ void textbox::scroll(unsigned int pos)
 	set_dirty(true);
 }
 
-surface textbox::add_text_line(const ucs4::string& text, const color_t& color)
+surface textbox::add_text_line(const std::u32string& text, const color_t& color)
 {
 	line_height_ = font::get_max_height(font_size_);
 
@@ -317,24 +318,24 @@ surface textbox::add_text_line(const ucs4::string& text, const color_t& color)
 	// some more complex scripts (that is, RTL languages). This part of the work should
 	// actually be done by the font-rendering system.
 	std::string visible_string;
-	ucs4::string wrapped_text;
+	std::u32string wrapped_text;
 
-	ucs4::string::const_iterator backup_itor = text.end();
+	std::u32string::const_iterator backup_itor = text.end();
 
-	ucs4::string::const_iterator itor = text.begin();
+	std::u32string::const_iterator itor = text.begin();
 	while(itor != text.end()) {
 		//If this is a space, save copies of the current state so we can roll back
 		if(char(*itor) == ' ') {
 			backup_itor = itor;
 		}
-		visible_string.append(unicode_cast<utf8::string>(*itor));
+		visible_string.append(unicode_cast<std::string>(*itor));
 
 		if(char(*itor) == '\n') {
 			backup_itor = text.end();
 			visible_string = "";
 		}
 
-		int w = font::line_width(visible_string, font_size_);
+		int w = font::pango_line_width(visible_string, font_size_);
 
 		if(wrap_ && w >= inner_location().w) {
 			if(backup_itor != text.end()) {
@@ -346,12 +347,12 @@ surface textbox::add_text_line(const ucs4::string& text, const color_t& color)
 					wrapped_text.erase(wrapped_text.end()-backup, wrapped_text.end());
 				}
 			} else {
-				if (visible_string == std::string("").append(unicode_cast<utf8::string>(*itor))) {
+				if (visible_string == std::string("").append(unicode_cast<std::string>(*itor))) {
 					break;	//breaks infinite loop where when running with a fake display, we word wrap a single character infinitely.
 				}
 			}
 			backup_itor = text.end();
-			wrapped_text.push_back(ucs4::char_t('\n'));
+			wrapped_text.push_back(char32_t('\n'));
 			char_x_.push_back(0);
 			char_y_.push_back(char_y_.back() + line_height_);
 			visible_string = "";
@@ -363,8 +364,8 @@ surface textbox::add_text_line(const ucs4::string& text, const color_t& color)
 		}
 	}
 
-	const std::string s = unicode_cast<utf8::string>(wrapped_text);
-	const surface res(font::get_rendered_text(s, font_size_, color));
+	const std::string s = unicode_cast<std::string>(wrapped_text);
+	const surface res(font::pango_render_text(s, font_size_, color));
 
 	return res;
 }
@@ -404,7 +405,7 @@ void textbox::erase_selection()
 	if(!is_selection())
 		return;
 
-	ucs4::string::iterator itor = text_.begin() + std::min(selstart_, selend_);
+	std::u32string::iterator itor = text_.begin() + std::min(selstart_, selend_);
 	text_.erase(itor, itor + std::abs(selend_ - selstart_));
 	cursor_ = std::min(selstart_, selend_);
 	selstart_ = selend_ = -1;
@@ -457,8 +458,8 @@ void textbox::handle_event(const SDL_Event& event)
 bool textbox::handle_text_input(const SDL_Event& event)
 {
 	bool changed = false;
-	utf8::string str = event.text.text;
-	ucs4::string s = unicode_cast<ucs4::string>(str);
+	std::string str = event.text.text;
+	std::u32string s = unicode_cast<std::u32string>(str);
 
 	DBG_G << "Char: " << str << "\n";
 
@@ -572,7 +573,7 @@ bool textbox::handle_key_down(const SDL_Event &event)
 				//cut off anything after the first newline
 				str.erase(std::find_if(str.begin(),str.end(),utils::isnewline),str.end());
 
-				ucs4::string s = unicode_cast<ucs4::string>(str);
+				std::u32string s = unicode_cast<std::u32string>(str);
 
 				if(text_.size() < max_size_) {
 					if(s.size() + text_.size() > max_size_) {
@@ -590,11 +591,11 @@ bool textbox::handle_key_down(const SDL_Event &event)
 			{
 				if(is_selection())
 				{
-					const size_t beg = std::min<size_t>(size_t(selstart_),size_t(selend_));
-					const size_t end = std::max<size_t>(size_t(selstart_),size_t(selend_));
+					const std::size_t beg = std::min<std::size_t>(std::size_t(selstart_),std::size_t(selend_));
+					const std::size_t end = std::max<std::size_t>(std::size_t(selstart_),std::size_t(selend_));
 
-					ucs4::string ws(text_.begin() + beg, text_.begin() + end);
-					std::string s = unicode_cast<utf8::string>(ws);
+					std::u32string ws(text_.begin() + beg, text_.begin() + end);
+					std::string s = unicode_cast<std::string>(ws);
 					desktop::clipboard::copy_to_clipboard(s, false);
 				}
 			}
@@ -607,8 +608,8 @@ bool textbox::handle_key_down(const SDL_Event &event)
 					const size_t beg = std::min<size_t>(size_t(selstart_),size_t(selend_));
 					const size_t end = std::max<size_t>(size_t(selstart_),size_t(selend_));
 
-					ucs4::string ws(text_.begin() + beg, text_.begin() + end);
-					std::string s = unicode_cast<utf8::string>(ws);
+					std::u32string ws(text_.begin() + beg, text_.begin() + end);
+					std::string s = unicode_cast<std::string>(ws);
 					desktop::clipboard::copy_to_clipboard(s, false);
 					erase_selection();
 				}
@@ -645,7 +646,7 @@ void textbox::handle_event(const SDL_Event& event, bool was_forwarded)
 
 	//Sanity check: verify that selection start and end are within text
 	//boundaries
-	if(is_selection() && !(size_t(selstart_) <= text_.size() && size_t(selend_) <= text_.size())) {
+	if(is_selection() && !(std::size_t(selstart_) <= text_.size() && std::size_t(selend_) <= text_.size())) {
 		WRN_DP << "out-of-boundary selection" << std::endl;
 		selstart_ = selend_ = -1;
 	}
