@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2003 - 2018 by David White <dave@whitevine.net>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2003 - 2021
+	by David White <dave@whitevine.net>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 #pragma once
@@ -25,7 +26,7 @@ namespace gui {
 class textbox : public scrollarea
 {
 public:
-	textbox(CVideo &video, int width, const std::string& text="", bool editable=true, size_t max_size = 256, int font_size = font::SIZE_PLUS, double alpha = 0.4, double alpha_focus = 0.2, const bool auto_join = true);
+	textbox(CVideo &video, int width, const std::string& text="", bool editable=true, std::size_t max_size = 256, int font_size = font::SIZE_PLUS, double alpha = 0.4, double alpha_focus = 0.2, const bool auto_join = true);
 	virtual ~textbox();
 
 	const std::string text() const;
@@ -55,13 +56,13 @@ protected:
 	virtual void scroll(unsigned int pos);
 
 private:
-	virtual void handle_text_changed(const ucs4::string&) {}
+	virtual void handle_text_changed(const std::u32string&) {}
 
-	size_t max_size_;
+	std::size_t max_size_;
 
 	int font_size_;
 
-	ucs4::string text_;
+	std::u32string text_;
 
 	// mutable unsigned int firstOnScreen_;
 	int cursor_;
@@ -85,7 +86,7 @@ private:
 
 	bool wrap_;
 
-	size_t line_height_, yscroll_;
+	std::size_t line_height_, yscroll_;
 
 	double alpha_;
 	double alpha_focus_;
@@ -106,7 +107,7 @@ private:
 
 	void draw_cursor(int pos) const;
 	void update_text_cache(bool reset = false, const color_t& color =font::NORMAL_COLOR);
-	surface add_text_line(const ucs4::string& text, const color_t& color =font::NORMAL_COLOR);
+	surface add_text_line(const std::u32string& text, const color_t& color =font::NORMAL_COLOR);
 	bool is_selection();
 	void erase_selection();
 
