@@ -1,15 +1,16 @@
 /*
-   Copyright (C) 2007 - 2018 by Mark de Wever <koraq@xs4all.nl>
-   Part of the Battle for Wesnoth Project https://www.wesnoth.org/
+	Copyright (C) 2007 - 2021
+	by Mark de Wever <koraq@xs4all.nl>
+	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY.
 
-   See the COPYING file for more details.
+	See the COPYING file for more details.
 */
 
 /**
@@ -42,7 +43,7 @@
 		if(!(cond)) {                                                     \
 			throw_wml_exception(#cond, __FILE__, __LINE__, __func__, message);  \
 		}                                                                 \
-	} while(0)
+	} while(false)
 
 #define VALIDATE_WITH_DEV_MESSAGE(cond, message, dev_message)             \
 	do {                                                                  \
@@ -54,12 +55,12 @@
 					, message                                             \
 					, dev_message);                                       \
 		}                                                                 \
-	} while(0)
+	} while(false)
 
 #define FAIL(message)                                                     \
 	do {                                                                  \
 		throw_wml_exception(nullptr, __FILE__, __LINE__, __func__, message);       \
-	} while(0)
+	} while(false)
 
 #define FAIL_WITH_DEV_MESSAGE(message, dev_message)                       \
 	do {                                                                  \
@@ -69,7 +70,7 @@
 				, __func__                                                \
 				, message                                                 \
 				, dev_message);                                           \
-	} while(0)
+	} while(false)
 
 /**
  *  Helper function, don't call this directly.
@@ -79,8 +80,9 @@
  *  @param line         The line at which the test failed.
  *  @param function     The function in which the test failed.
  *  @param message      The translated message to show the user.
+ *  @param dev_message  Any additional information that might be useful to a developer.
  */
-NORETURN void throw_wml_exception(
+[[noreturn]] void throw_wml_exception(
 		  const char* cond
 		, const char* file
 		, int line
@@ -98,7 +100,7 @@ struct wml_exception
 	{
 	}
 
-	~wml_exception() NOEXCEPT {}
+	~wml_exception() noexcept {}
 
 	/**
 	 *  The message for the user explaining what went wrong. This message can
