@@ -18,6 +18,13 @@
 --     PRIMARY KEY (USER_ID, GROUP_ID)
 -- ) ENGINE=InnoDB;
 
+-- a minimal topics table, if not using a phpbb3 installation
+-- create table topics
+-- (
+--     TOPIC_ID MEDIUMINT(8) UNSIGNED NOT NULL AUTO_INCREMENT,
+--     PRIMARY KEY (TOPIC_ID)
+-- ) ENGINE=InnoDB;
+
 -- table which the forum inserts bans into, which wesnothd checks during login
 -- create table ban
 -- (
@@ -74,6 +81,7 @@ create table game_info
     PUBLIC           BIT(1) NOT NULL,
     PRIMARY KEY (INSTANCE_UUID, GAME_ID)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
+CREATE INDEX START_TIME_IDX ON game_info(START_TIME);
 
 -- information about the players in a particular game present in game_info
 -- this is accurate at the start of the game, but is not currently updated if a side changes owners, someone disconnects, etc
@@ -81,9 +89,9 @@ create table game_info
 -- SIDE_NUMBER: the side controlled by USER_ID
 -- IS_HOST: if USER_ID is the game's host
 -- FACTION: the faction being played by this side
--- STATUS: the status of the side, currently only updated at game end
 -- CLIENT_VERSION: the version of the wesnoth client used to connect
 -- CLIENT_SOURCE: where the wesnoth client was downloaded from - SourceForge, Steam, etc
+-- USER_NAME: the username logged in with
 create table game_player_info
 (
     INSTANCE_UUID  CHAR(36) NOT NULL,
@@ -97,10 +105,12 @@ create table game_player_info
     USER_NAME      VARCHAR(255) NOT NULL DEFAULT '',
     PRIMARY KEY (INSTANCE_UUID, GAME_ID, SIDE_NUMBER)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
+CREATE INDEX USER_ID_IDX ON game_player_info(USER_ID);
 
 -- information about the scenario/era/modifications for the game
 -- TYPE: one of era/scenario/modification
 -- ID: the id of the content
+-- NAME: the content's user-visible name
 -- SOURCE: the id of the add-on that the particular content came from
 -- VERSION: the version of the source add-on
 create table game_content_info
@@ -109,7 +119,50 @@ create table game_content_info
     GAME_ID           INT UNSIGNED NOT NULL,
     TYPE              VARCHAR(255) NOT NULL,
     ID                VARCHAR(255) NOT NULL,
+    NAME              VARCHAR(255),
     SOURCE            VARCHAR(255) NOT NULL,
     VERSION           VARCHAR(255) NOT NULL,
     PRIMARY KEY (INSTANCE_UUID, GAME_ID, TYPE, ID)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
+
+-- information about an uploaded addon
+-- INSTANCE_VERSION: the version of the addons server instance
+-- ADDON_ID: the ID of the addon (folder name)
+-- ADDON_NAME: the name of the addon
+-- TYPE: the type of the addon
+-- VERSION: the version of the addon
+-- FORUM_AUTH: whether forum authentication is to be used when uploading
+-- UPLOADED_ON: when the addon was uploaded
+-- FEEDBACK_TOPIC: the forum topic ID where feedback for the addon can be posted, 0 if not set
+create table addon_info
+(
+    INSTANCE_VERSION VARCHAR(255) NOT NULL,
+    ADDON_ID         VARCHAR(255) NOT NULL,
+    ADDON_NAME       VARCHAR(255) NOT NULL,
+    TYPE             VARCHAR(255) NOT NULL,
+    VERSION          VARCHAR(255) NOT NULL,
+    FORUM_AUTH       BIT(1) NOT NULL,
+    UPLOADED_ON      TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+    FEEDBACK_TOPIC   INT UNSIGNED NOT NULL,
+    PRIMARY KEY (INSTANCE_VERSION, ADDON_ID, VERSION)
+) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
+
+-- history of user sessions
+-- LOGIN_ID: auto generated ID to use as a primary key
+-- USER_NAME: the username logged in with
+-- IP: the IP address the login originated from
+-- LOGIN_TIME: when the user logged in
+-- LOGOUT_TIME: when the user logged out
+-- VERSION: the version the user logged in with
+create table connection_history
+(
+    LOGIN_ID    BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
+    USER_NAME   VARCHAR(255) NOT NULL,
+    IP          VARCHAR(255) NOT NULL,
+    LOGIN_TIME  TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
+    LOGOUT_TIME TIMESTAMP NULL DEFAULT NULL,
+    VERSION     VARCHAR(255),
+    PRIMARY KEY (LOGIN_ID)
+) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
+CREATE INDEX CONNECTION_IP_IDX ON connection_history(IP);
+CREATE INDEX CONNECTION_USERNAME_IDX ON connection_history(USER_NAME);
